define("slmui/form/Slider", [
	"../../dojo/_base/declare", // declare
	"idx/form/HorizontalSlider",
	"dijit/form/HorizontalRule",
	"dijit/form/HorizontalRuleLabels",
	"dijit/registry",
	"dojo/dom-construct",
	"dojo/_base/array",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-style",
	"dojo/query",
	'dojo/has'
], function(declare, HorizontalSlider, HorizontalRule, HorizontalRuleLabels, registry, domConstruct, array, windowModule, domAttr, domStyle, query, has){

	// module:
	//		slmui/form/Slider


	var Slider = declare(HorizontalSlider, {


		// values of each item (Array)
		values: null,
		// short text of each item (Array)
		labels: null,
		// long text of each item (Array)
		descriptions: null,
		// index of currently selected item
		selectedValue: 0,
		// array of item's widgets
		items: null,
		// widget name
		name: "",
		// state attributes
		readOnly: false,
		disabled: false,
		error: "",
		required: false,
		// if items exist in srcNodeRef
		empty: false,


		//to show no increment\decrement buttons
		showButtons: false,

		_initSliderFields: function(){
			// summary:
			//			Sets the input data to inherited idx.form.HorizontalSlider

			if(!this.empty){
				this.discreteValues = this.values.length;
				if(this.selectedValue){
					this.value = this.selectedValue;
				}
				this.minimum = 1;
				this.maximum = this.values.length;
			}else{
				this.discreteValues = 1;
			}
		},

		postMixInProperties: function(){
			// summary:
			//			Init of the slider
			this.inherited(arguments);

			if(!this.values){
				this._parseSource();
			}else{
				this.empty = false;
			}

			this._initSliderFields();

            if(!this.empty){
                this.baseClass = this.baseClass + " slmuiSlider";
            } else {
                this.baseClass = this.baseClass + " slmuiSliderEmpty";
            }
		},

		buildRendering: function(){
			//empty the input HTML needed for init
			domConstruct.empty(this.srcNodeRef);

			this.inherited(arguments);
		},

		_parseSource: function(){
			// summary:
			//			Parses the HTML source if widget was not created programmatically.
			var srcNodeName = (has('ie') == 8) ? this.srcNodeRef.nodeName.toLowerCase() : this.srcNodeRef.localName;
			var component = (srcNodeName=="div")?"div":"option";
			var boxes = query("> " + component, this.srcNodeRef);
			if(boxes.length == 0){
				this.empty = true;
				return;
			}
			var _this = this;
			this.values = boxes.map(function(node){
				return domAttr.get(node, "value");
			});
			this.labels = boxes.map(function(node){
				return domAttr.get(node, "label");
			});
			this.descriptions = boxes.map(function(node){
				return node.innerHTML;
			});
			boxes.forEach(function(node, index){
				if(domAttr.get(node, "selected") == "true"){
					_this.selectedValue = index + 1;
				}
			})
		},

		_createTickMarks: function(){
			//summary:
			//			Creates and instantiate top and bottom tick marks for Slider.
			var topTickMarksNode = domConstruct.create("div", {}, this.containerNode, "first");
			var sliderRules = new _MyHorizontalRuleWithToolTips({
				container: "topDecoration",
				tips: this.labels,
				count: this.values.length,
				style: "height: 5px;"
			}, topTickMarksNode);


			var bottomTickMarksNode = domConstruct.create("div", {}, this.containerNode, "first");
			var sliderRules2 = new _MyHorizontalRuleWithToolTips({
				container: "bottomDecoration",
				count: this.values.length,
				tips: this.labels
			}, bottomTickMarksNode);
		},

		_creteLabels: function(){
			// summary:
			//			Creates and instantiates the labels for Slider.

			//we need only the first and the last label
			var labels = [this.labels[0], this.labels[this.labels.length - 1]];

			var labelsNode = domConstruct.create("div", {}, this.containerNode, "last");
			var sliderLabels = new _MyHorizontalRuleLabelsWithAdditionalClassesForExtremeItems({
				container: "bottomDecoration",
				labels: labels,
				style: "margin-bottom: 15px;"
			}, labelsNode);
		},

		postCreate: function(){
			// summary:
			//			Creates tick marks and label for Slide and removes the label node.
			// tags:
			//			extension
			this.inherited(arguments);

			if(!this.empty){
				this._createTickMarks();
				this._creteLabels();
			}
			//We need to remove label node of the Slider.
			//Currently an xcitec.layout.Chapter is responsible for label node
			domConstruct.destroy(this.labelWrap);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//			Sets the tick mark for the slider handle and for the value node.
			// tags:
			//			extension
			this.inherited(arguments);
			if(!this.empty){
				domAttr.set(this.sliderHandle, "title", this.labels[value - 1]);
				this.valueNode.value = this.values[value - 1];
			}
			//when slider have no predefined selected value and user just click (or set focus) on slider without moving
		            if(value==0){
		                this.valueNode.value = this.values[0];
		            }
		},

		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Event handler for mousewheel where supported
			if(this.focused){
				this.inherited(arguments);
			}
		}
	});

	var _MyHorizontalRuleLabelsWithAdditionalClassesForExtremeItems = declare(HorizontalRuleLabels, {

		// summary:
		//			Extended horizontal rule label. See method desc.

		_genHTML: function(pos, ndx){
			// summary:
			//			Adds the addtional styles and CSS classes for extreme left and right items.
			// tags:
			//			extension
			var label = this.labels[ndx];

			var positionPrefix = this._positionPrefix;
			var labelPrefix = this._labelPrefix;

			var additionalLabelClass;
			var additionalPositionClass;
			if(pos == 0){
				additionalLabelClass = "ruleLabelLeft";
				additionalPositionClass = "ruleLabelPositionLeft";
			}else if(pos == 100){
				additionalLabelClass = "ruleLabelRight";
				additionalPositionClass = "ruleLabelPositionRight";
			}
			if(additionalLabelClass && additionalPositionClass){
				positionPrefix = '<div class="dijitRuleLabelContainer dijitRuleLabelContainerH ' + additionalPositionClass + '" style="left:';
				labelPrefix = '"><div class="dijitRuleLabel dijitRuleLabelH ' + additionalLabelClass + '">';
			}
			return positionPrefix + this._calcPosition(pos) + this._positionSuffix + this.labelStyle +
				this._genDirectionHTML(label) +
				labelPrefix + label + this._suffix;
		}
	});

	var _MyHorizontalRuleWithToolTips = declare(HorizontalRule, {

		// summary:
		//			Extended horizontal rules. See the method desc.

		_positionPrefix: '<div class="dijitRuleMark dijitRuleMarkH" style="margin-top: 5px; left:',

		// array of tool tips for tick marks
		tips: [],

		_genHTML: function(pos, index){
			// summary:
			//			Puts the tick marks into the container that will show tool tips.
			// tags:
			//			extension
			var pref = '';
			var post = '';
			var tooltip = this.tips[(index)];
			var margin = (100 / (this.tips.length - 1) - 2).toFixed(1);
			if(pos != 100){
				pref = '<div data-dojo-attach-point="tickMarkNode" style="height: 15px;width: 4%;z-index: 123;	margin-right: ' + margin + '%;margin-left: -2%;	float: left;" title="' +
					tooltip
					+ '">';
				post = '</div>';
			}else{
				pref = '<div data-dojo-attach-point="tickMarkNode" style="height: 15px;width: 4%;z-index: 123;	margin-right: 0;margin-left: -4%;	float: left;" title="' +
					tooltip
					+ '">';
				post = '</div>';
			}
			return pref + this._positionPrefix + pos + this._positionSuffix + this.ruleStyle + this._suffix + post;
		}
	});


	return Slider;
})
;