define("xcitec/form/AnswerBox", [
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/query",
  	"dojo/dom",
	'dojo/has',
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/_StateWidget"
], function (dojo, array, declare, windowModule, domAttr, query, dom, has, _Templated, _Widget, registry, _StateWidget) {

	var AnswerBox = declare( [_Widget, _Templated], {
		templateString: '<div ${nameAttrSetting} class="answerBoxWrapper" tabindex="0">' +
				'<input type="hidden" value="" data-dojo-attach-point="valueNode"/>' +
				'<table class="answerBox">' +
				'<tbody data-dojo-attach-point="itemsNode"></tbody>' +
				'</table>' +
				'</div>',
	
		// values of each item (Array)
		values: null,
		// short text of each item (Array)
		labels: null,
		// long text of each item (Array)
		descriptions: null,
		// tooltips of each item
		tooltips: null,
		// index of currently selected item
		selectedValue: 0,
		// array of item's widgets
		items: null,
		// widget name
		name: "",
		// state attributes
		readOnly: false,
		disabled: false,
		error: "",
		required: false,
		// if items exist in srcNodeRef
		empty: false,

		onchange: function(){
			// onchange function for setting by user
		},

		_parseSource: function(){
			var srcNodeName = (has('ie') == 8) ? this.srcNodeRef.nodeName.toLowerCase() : this.srcNodeRef.localName;
			var component = (srcNodeName=="div")?"div":"option";
			var boxes = query(">" + component, this.srcNodeRef);
			if(boxes.length == 0){
				this.empty = true;
			}
			var _this = this;
			this.values = boxes.map(function(node){
				return domAttr.get(node, "value");
			});
			this.labels = boxes.map(function(node){
				return domAttr.get(node, "label");
			});
			this.descriptions = boxes.map(function(node){
				return node.innerHTML;
			});

			this.tooltips = boxes.map(function(node){
				return (domAttr.get(node, "showTooltip") != "false") ? true : false;
			});

			boxes.forEach(function(node, index){
				if(domAttr.get(node, "selected") == "true"){
					_this.selectedValue = index + 1;
				}
			})
		},

		postMixInProperties: function(){
			this.values = [];
			this.labels = [];
			this.descriptions = [];
			this.tooltips = [];
			this.items = [];
			this.nameAttrSetting = this.name ? ("name='" + this.name + "'") : "";
			this.inherited(arguments);
		},

		postCreate: function(){
			this._parseSource();
			var _this = this;

			this.items = [];

			if(this.empty){
				this.createAnswerItem("", "", "", this, false, true, false, false);
				domAttr.set(_this.valueNode, "value", "");
			} else{
				// create actual items
				if(this.readOnly){
					// if readOnly - create only selected item, or just empty box if nothing selected
					if(this.selectedValue != 0){
						this.createAnswerItem(this.labels[this.selectedValue - 1], this.descriptions[this.selectedValue - 1],
								this.values[this.selectedValue - 1], this, false, true, false, this.tooltips[this.selectedValue - 1]);
						if(!_this.values[_this.selectedValue - 1]){
							domAttr.set(_this.valueNode, "value", "");
						} else{
							domAttr.set(_this.valueNode, "value", _this.values[_this.selectedValue - 1]);
						}
					} else{
						this.createAnswerItem("", "", "", this, false, true, false, false);
						domAttr.set(_this.valueNode, "value", "");
					}
					// if disabled - create only selected item, or just empty box if nothing selected
				} else{
					array.forEach(this.descriptions, function(node, index){
						if(_this.selectedValue == (index + 1)){
							_this.createAnswerItem(_this.labels[index], _this.descriptions[index], _this.values[index],
								 _this, true, false, _this.disabled, _this.tooltips[index]);
							if(!_this.values[index]){
								domAttr.set(_this.valueNode, "value", "");
							} else{
								domAttr.set(_this.valueNode, "value", _this.values[index]);
							}
						} else{
							_this.createAnswerItem(_this.labels[index], _this.descriptions[index], _this.values[index],
								_this, false, false, _this.disabled, _this.tooltips[index]);
						}
					});
				}
			}
			domAttr.set(this.valueNode, "name", this.name);
			dom.setSelectable(this.domNode, false);
			this.inherited(arguments);
		},

		createAnswerItem: function(label, description, value, parent, selected, readOnly, disabled, showTooltip){
			if(value == null){
				value = "";
			}
			var boxItem = new _AnswerBoxItem({ label: label, description: description,
				value: value, parent: parent, selected: selected,
				readOnly: readOnly, disabled: disabled, showTooltip: showTooltip});
			this.items.push(boxItem);
			this.itemsNode.appendChild(boxItem.domNode);
		},

		// makes all items unselected except active
		setSelectedAttr: function(selWidget){
			if(selWidget.selected){
				domAttr.set(this.valueNode, "value", selWidget.value);
			} else{
				domAttr.set(this.valueNode, "value", "");
			}
			array.forEach(this.items, function(widget){
				if(selWidget != widget){
					widget.selected = false;
				}
				widget.setSelected();
			});
		},

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		}
	});

	var _AnswerBoxItem = declare( _StateWidget, {
		templateString: '<tr class="${baseClass}" data-dojo-attach-event="onclick: _onClick">' +
				'<td class="rate"><div class="section" data-dojo-attach-point="labelNode"></div></td>' +
				'<td class="description"><div class="section">${description}<span class="description_line"></span></div></td>' +
				'</tr>',

		baseClass: "answerBox",

		// reference to parent container
		parent: null,

		// short text
		label: "",
		// long text
		description: "",

		// value for sending to server
		value: "",

		// state attributes
		selected: false,
		readOnly: false,
		disabled: false,

		showTooltip: true,

		buildRendering: function(){
			this.inherited(arguments);

			if(this.showTooltip){
				domAttr.set(this.domNode, 'title', this.description);
			}
		},

		_onClick: function(){
            this.parent.domNode.focus();
			if(this.readOnly || this.disabled){
				return;
			}
			this.selected = !this.selected;
			if(this.parent){
				this.parent.setSelectedAttr(this);
				this.parent.onchange();
			}
		},
	
		// makes item visually selected by applying appropriate styles
		setSelected : function(){
			this._setStateClass();
		},
	
		_setLabelAttr: function(/*String*/ label){
			this.label = label;
			if(this.label){
				this.labelNode.appendChild(windowModule.doc.createTextNode(this.label));
			}
		}
	});
	return AnswerBox;
});
