define("xcitec/form/CheckBoxGroup", [
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/on",
	"dojo/query",
  	"dojo/dom",
	'dojo/has',
	"dijit/form/_FormValueWidget",
	"dijit/registry",
	"xcitec/form/MultiSelectUtils",
  "../xcitecUtil",
  "./CheckBox"
], function (dojo, array, declare, lang, domAttr, on, query, dom, has, _FormValueWidget, registry, MultiSelectUtils, xcitecUtil, CheckBox) {

	var CheckBoxGroup = declare( [_FormValueWidget, MultiSelectUtils], {
		baseClass: 'checkBoxGroup',
	  tabIndex: "-1",
	  required:false,
	  templateString: '<div class="dijit dijitReset dijitInline" tabIndex="-1" data-dojo-attach-point="checkBoxButton, focusNode">' +
	                  '<input class="dijitReset" data-dojo-attach-point="textbox"' +
	                  '${!nameAttrSetting} type="hidden" autocomplete="off">' +
	                  '<div class="required"></div>' +
	                  '</div>',
	
	  // Array of selected Checkboxes
	  //selWidgets:null,
	
	  postMixInProperties: function() {
	    this.inherited(arguments);
	    if (this.required) {
	        this.baseClass = this.baseClass + " dijitXcitecChekboxGroupRequired";
	    }
	    this.selWidgets = [];
	  },

	  postCreate: function() {
	    this._parseSource();
	    this._createCheckBoxes(this.options);
	    this._updateHiddenInput();
      	dom.setSelectable(this.domNode, false);
	    this.inherited(arguments);
	  },

	  _onChange: function() {
	    if (this.readOnly || this.disabled) {
	      return;
	    }
	    // user defined function
	    this.onchange();
	  },

	  onchange: function() {
	    // for wicket use in template
	  },

	  _parseSource: function() {
	    // summary:
	    //		Loads our options and sets up our dropdown correctly.  We
	    //		don't want any content, so we don't call any inherit chain
	    //		function.
	    var opts = this.options;
	    if (!opts) {
			var nodeType = has('ie') == 8 ? this.srcNodeRef.nodeName : this.srcNodeRef.localName;
		  var component = (nodeType.toLowerCase()=="div")?"div":"option";
	      opts = this.options = this.srcNodeRef ? query("> "+component,
	                                                         this.srcNodeRef).map(function(node) {
	        return {
	          value: node.getAttribute("value"),
	          caption: node.getAttribute("caption"),
	          captionId: node.getAttribute("captionId"),
	          label: String(node.innerHTML),
	          checked: xcitecUtil.str2bool(domAttr.get(node, "checked")),
	          disabled: xcitecUtil.str2bool(domAttr.get(node, "disabled"))
	        };
	      }, this) : [];
	    }
	  },

	  selWidget: function(widget) {
	    if (this.isNodeInList(this.selWidgets, widget)) {
	      this.selWidgets = this.delFromList(this.selWidgets, widget);
	    } else {
	      this.selWidgets = this.addToList(this.selWidgets, widget);
	    }
	    this._updateHiddenInput();
	  },

	  _createCheckBoxes: function(otpions) {
	    var self = this;

	    array.forEach(otpions, function(node) {

	      // create CheckBoxes according set of options
	      var button = new CheckBox({
	        checked: node.checked,
	        disabled: self.disabled || node.disabled,
	        readOnly: self.readOnly,
	        caption: node.caption,
			innerText: node.label,
	        captionId: node.captionId,
	        value: node.value,
	        style: "width:100%;",
	        parent: self
	      });
	      self.checkBoxButton.appendChild(button.domNode);
	      button.domNode.onchange = "";
	      on(button.domNode, "click", lang.hitch(button, self.changeState));
	    });
	  },

	  changeState: function() {
	    var widget = registry.byNode(this.domNode);
	    widget.focus();
	    widget.parent._onChange();
	  },

	  _updateHiddenInput: function() {
	    var _this = this;
	    var temp = "";
	    var counter = 0;
	    array.forEach(_this.selWidgets, function(widget) {
	      if (counter == 0) {
	        temp = widget.value;
	      } else {
	        temp = temp + "," + widget.value;
	      }
	      counter++;
	    });
	    this.textbox.value = temp;
	  },

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
	});
	return CheckBoxGroup;
});
