define("xcitec/grid/_HGridBase", [
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/dnd/Source",
	"dojo/dom-geometry",
	"dojo/has",
	"dojo/on",
	"dojo/query",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/grid/Avatar",
  "../xcitecUtil"
], function (dojo, domStyle, declare, lang, windowModule, array, domConstruct, domClass, Manager, domAttr, Source, domGeometry, has, on, query, topic, _Container,
             _Templated, _Widget, registry, Avatar, xcitecUtil) {

	/***
	 *
	 * Grid base widget
	 *
	 */
	var _HGridBase = declare( [_Widget, _Templated], {
		rows: null,
		headWidget: null,
	//  strange but defined here looks like static ????     
	//    columnWidths: [],
		showContextMenu: false,
		// enables grid resizing
		gridResize: true,
	
		parentId: "",
	
		// is multiselect available
		allowMultiSelect: false,
		// the message for empty grid
		emptyGridMessage: "",
		// entity class for icon on the grid link toolbar
		entityClass: "",
		// using of vertical scroll - temporary field, later will be replaced by fitToContainer
		vertScroll: false,
		// number of shown rows if vertScroll is true
		shownRows: 5,
		_defaultShownRows: 5,
		_rowHeight: 19,
		_headerHeight: 21,
		_footerHeight: 20,
		// enables column reorder
		columnReorder: false,
	
		isPortletContent: false,
	
		postMixInProperties: function(){
			this.columnWidths = []; // defined here because as attribute looks like static
			if(this.isPortletContent){
				this.vertScroll = true;
			}
			this.inherited(arguments);
		},
	
		buildRendering: function(){
			// get header, body and footer domNodes for xcitec.grid._HGridHeader, xcitec.grid._HGridRows and xitec.grid._HGridFooter
			this.headerNode = query("thead > tr", this.srcNodeRef)[0];
			this.rowsNode = query("tbody", this.srcNodeRef)[0];
	
			this.gridControl = registry.byId(this.parentId);
	
			// grid actions
			this.gridActions = registry.byId(this.id + ".actions");
	
			this.inherited(arguments);
	
			// set Entity Icon if defined
			if(this.entityClass){
				var image = domConstruct.create('span', {}, this.footer, "last");
				domClass.add(image, 'iconDataTable');
				domClass.add(image, xcitecUtil.createEntityIconClass(this.entityClass));
			}
	
			// disable browser context menu
			if(!this.showContextMenu){
				on(this.domNode, "contextmenu", function(e){
					e.preventDefault(), e.stopPropagation();
				});
			}
		},
	
		_fillContent: function(/*DomNode*/ source){
			// parse content and create grid
		},
	
		postCreate: function(){
			if(this.isPortletContent) {
				domStyle.set(this.headWidget.domNode, "display", "none");
				domStyle.set(this.footer, "display", "none");
				domStyle.set(query(".dataTableHeadBg",this.domNode)[0], "display", "none");
				domStyle.set(this.dataTableBody, {
					top: "0",
					bottom: "0"
				});
				domStyle.set(this.dataTable, "border", "0");
			}
	        this.inherited(arguments);

			if(this.vertScroll && this.shownRows > this._defaultShownRows && this.rows && this.rows.allRows.length > this._defaultShownRows){
				domStyle.set(this.domNode, 'height', (this._headerHeight + this._footerHeight + this._rowHeight * this.shownRows) + 'px');
			}
		},

		startup: function(){
			this.inherited(arguments);
			var linkToolbar = registry.byId(this.id + '.linkButtons');
			if(linkToolbar && this.entityClass){
				domClass.add(linkToolbar.domNode, 'dataTableButtonAbsolute');
			}
		},
	
		updateOrder: function(){
			var colOrder = "";
			var _this = this;
			var allCells = this.headWidget.allCells;
			if(this.hasFrozenColumn){
				allCells = this.headFrozenWidget.allCells.concat(this.headWidget.allCells);
			}
			array.forEach(allCells, function(cell){
				if(cell.columnId){
					colOrder = colOrder + cell.columnId + ",";
				}
			});
			colOrder = colOrder.substr(0, colOrder.length - 1);
			if(this.gridControl && this.gridControl.columnOrder){
				if(colOrder == ""){
					domAttr.set(this.gridControl.columnOrder, "value", "#DELETED#");
				} else{
					domAttr.set(this.gridControl.columnOrder, "value", colOrder);
				}
			}
			this.updateColumnWidthsInput();
		},
	
		updateColumnWidthsInput: function(){
			if(this.gridControl.columnWidthsNode){
				var _this = this;
				var allCells = this.headWidget.allCells;
				if(this.hasFrozenColumn){
					var frozenCells = this.headFrozenWidget.allCells.slice();
					frozenCells.pop();
					allCells = frozenCells.concat(this.headWidget.allCells);
				}
				array.forEach(allCells, function(cell, index){
					_this.columnWidths[index] = cell.cellWidth;
				});
				this.columnWidths.pop();
				this.gridControl.columnWidthsNode.value = this.columnWidths.toString();
			}
		},
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		},
	
		destroy: function(){
			var actions = registry.byId(this.id + ".actions");
			if(actions){
				actions.destroyRecursive();
			}
			var linkButtons = registry.byId(this.id + ".linkButtons");
			if(linkButtons){
				linkButtons.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
	return _HGridBase;
});
