define("xcitec/grid/_HGridHeaderCell", [
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/dnd/Source",
	"dojo/dom-geometry",
	"dojo/has",
	"dojo/on",
	"dojo/touch",
	"dojo/query",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "../Menu",
  "../MenuItem"
], function (dojo, domStyle, declare, lang, windowModule, array, domConstruct, domClass, Manager, domAttr, Source, domGeometry, has, on, touch, query, topic, _Container,
             _Templated, _Widget, registry, Menu, MenuItem) {
  return 	declare(_Widget, {

		startPos: 0,
		endPos: 0,
		blockOnClick: false,
		attached: false,
		columnId: "",
		presentmentType: "",
		showDefaultMenu: false,
		showXcitecMenu: true,
		onClickAction: null,
		onClickAttr: null,
		dragging: false,
		deleteIcon: null,
		cellWidth: 0,
		fixedPos: false,
		_addedEvents: [],

		buildRendering: function(){

			this.domNode = this.srcNodeRef;
			var outerDiv = document.createElement("div");
			domClass.add(outerDiv, "wrap");
			var innerDiv = document.createElement("div");
			domClass.add(innerDiv, "inner_head");
			var columnSorting = domAttr.get(this.domNode, "sort"); // column sorting (ASC or DESC)
			var canDeleteColumn = domAttr.get(this.domNode, "canDelete");
			var columnWidth = domAttr.get(this.domNode, "columnWidth"); // width of column if set
			this.onClickAttr = domAttr.get(this.domNode, "onclick");
			this.columnId = domAttr.get(this.domNode, "columnId");
			this.presentmentType = domAttr.get(this.domNode, "presentmentType");
			innerDiv.innerHTML = this.domNode.innerHTML;
			this.domNode.innerHTML = "";
			// if "sort" attribute is set - create div for sorting
			if(columnSorting){
				var sortDiv = document.createElement("div");
				switch(columnSorting){
					case "ASC":
						domClass.add(sortDiv, "sort_arrow arrow_up");
						break;
					case "DESC":
						domClass.add(sortDiv, "sort_arrow arrow_down");
						break;
					case "asc":
						domClass.add(sortDiv, "sort_arrow arrow_up");
						break;
					case "desc":
						domClass.add(sortDiv, "sort_arrow arrow_down");
						break;
					case "none":
						this.showXcitecMenu = false;
						break;
					default:
						break;

				}
				on(this.domNode, "mouseover", function(){
					if((columnSorting == "ASC") || (columnSorting == "asc")){
						domClass.remove(sortDiv, "arrow_up");
						domClass.add(sortDiv, "arrow_down_active");
					} else if((columnSorting == "DESC") || (columnSorting == "desc")){
						domClass.remove(sortDiv, "arrow_down");
						domClass.add(sortDiv, "arrow_up_active");
					}
				});
				on(this.domNode, "mouseout", function(){
					if((columnSorting == "ASC") || (columnSorting == "asc")){
						domClass.remove(sortDiv, "arrow_down_active");
						domClass.add(sortDiv, "arrow_up");
					} else if((columnSorting == "DESC") || (columnSorting == "desc")){
						domClass.remove(sortDiv, "arrow_up_active");
						domClass.add(sortDiv, "arrow_down");
					}
				});
				outerDiv.appendChild(sortDiv);
				this.domNode.appendChild(outerDiv);
			}
			var columnWidths = this.cellContainer.parentTable.columnWidths;
			if(this.cellContainer.isFrozenHeader){
				columnWidths = this.cellContainer.parentTable.columnFrozenWidths;
			}
			// if "columnWidth" attribute is set - assign width for proper column and save it in global array
			if(columnWidth){
				domStyle.set(this.domNode, "width", columnWidth + "px");
				columnWidths[this.cellIndex] = Number(columnWidth);
				this.cellWidth = Number(columnWidth);
			} else{
				columnWidths[this.cellIndex] = null;
				this.cellWidth = null;
			}

			if(canDeleteColumn){
				this.deleteIcon = document.createElement("div");
				domClass.add(this.deleteIcon, "close");
				var wrapAll = domConstruct.create("div");
				domClass.add(wrapAll, "wrapper_all");
				wrapAll.appendChild(this.deleteIcon);
				outerDiv.appendChild(innerDiv);
				wrapAll.appendChild(outerDiv);
				this.domNode.appendChild(wrapAll);
				on(this.deleteIcon, "click", lang.hitch(this, this.deleteColumn));
			} else{
				outerDiv.appendChild(innerDiv);
				this.domNode.appendChild(outerDiv);
			}
			if(this.cellIndex > 0){
				domClass.add(this.domNode, "separator");
			}
		},

		postCreate: function(){
			var _this = this;
			if(this.cellContainer.parentTable.gridActions){
				// get xcitec.grid.Action objects
				var acts = this.cellContainer.parentTable.gridActions.getChildren();
				if((acts.length > 0) && (this.showXcitecMenu)){
					var columnMenu = new Menu();
					columnMenu.bindDomNode(this.domNode);
					array.forEach(acts, function(action){
						if((action.getActionId() == "COL_SORT_ASC") || (action.getActionId() == "COL_SORT_DESC")){
							columnMenu.addChild(new MenuItem({label: action.getActionName(), onClick: function(){
								// update activeColumn hidden input with columnId
								if(_this.cellContainer.parentTable.gridControl && _this.cellContainer.parentTable.gridControl.activeColumn){
									domAttr.set(_this.cellContainer.parentTable.gridControl.activeColumn, "value", _this.columnId);
								}
								action.actionEvent();
							} }));
						}
						if(action.getActionId() == "COL_SORT_SET"){
							_this.onClickAction = action;
						}
					});
					columnMenu.startup();
					this.showXcitecMenu = true;
				}
			} else{
				this.showXcitecMenu = false;
			}

			if(!this.onClickAttr){
				this.connect(this.domNode, "onclick", this.onClick);
			}

			this.saveOnClick = this.domNode.onclick;
			this.domNode.onclick = lang.hitch(this, this.vkOnClick);
			// dojo/on doesn't work here for some reason, so replaced with dojo/touch
			touch.press(this.domNode, lang.hitch(this, "_onMouseDown"));
			touch.move(this.domNode, lang.hitch(this, "_onMouseMove"));
			touch.release(this.domNode, lang.hitch(this, "_onMouseUp"));
			if(has("ie")){
				if(this.cellContainer.parentTable.gridResize){
					on(this.domNode, "mouseup", function(){
						_this.cellContainer.parentTable.dataTableHead.scrollLeft = _this.cellContainer.parentTable.dataTableBody.scrollLeft;

					});
				}
			}
			if(this.presentmentType == "number"){
				this.cellContainer.cellToRightAlign = this.cellContainer.cellToRightAlign.concat(this);
			}
		},

		onClick: function(action){
			if(!this.dragging){
				if(this.onClickAction && (!this.isInResizingZone(action))){
					if(this.cellContainer.parentTable.gridControl && this.cellContainer.parentTable.gridControl.activeColumn){
						domAttr.set(this.cellContainer.parentTable.gridControl.activeColumn, "value", this.columnId);
					}
					this.onClickAction.actionEvent();
				}
			}
		},

		vkOnClick: function(evt){
			if(!this.dragging){
				if(has("ff") || has("chrome") || has('trident') == 7){
					if(!this.isInResizingZone(evt)){
						if(this.saveOnClick){
							this.saveOnClick(evt);
						}
					}
				} else{
					if(!this.blockOnClick){
						if(this.saveOnClick){
							this.saveOnClick(evt);
						}
					}
				}
			}
		},

		getCellsWidths: function(){
			var _this = this;
			this.cellContainer.cellsWidth = [];
			array.forEach(this.cellContainer.allCells, function(cell){
				_this.cellContainer.cellsWidth[cell.cellIndex] = domGeometry.getMarginBox(cell.domNode).w;
			});
		},

		// is cursor in the zone where resizing can be
		isInResizingZone: function(e){
			var cellOffset =  Math.round(e.pageX - domGeometry.position(this.domNode).x);
			var cellWidth = domGeometry.getMarginBox(this.domNode).w;
			if(this.cellIndex == 0){
				if(cellOffset > (cellWidth - 4)){
					return true;
				}
			} else if(this.cellIndex == this.cellContainer.allCells.length - 1){
				if(cellOffset < 4){
					return true;
				}
			} else{
				if((cellOffset < 4)){
					return true;
				} else if(cellOffset > (cellWidth - 4)){
					return true;
				}
			}
			return false;
		},

		// change the cursor style from default to resize and back
		changeCursor: function(e){
			var cellOffset =  Math.round(e.pageX - domGeometry.position(this.domNode).x);
			var cellWidth = domGeometry.getMarginBox(this.domNode).w;
			if(this.cellIndex == 0){ // first column
				if(cellOffset > (cellWidth - 4)){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else{
					domStyle.set(this.domNode, "cursor", "default");
				}
			} else if(this.cellIndex == this.cellContainer.allCells.length - 1){ // last column
				if(cellOffset < 4){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else{
					domStyle.set(this.domNode, "cursor", "default");
				}
			} else{ // middle columns
				if((cellOffset < 4)){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else if(cellOffset > (cellWidth - 4)){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else{
					domStyle.set(this.domNode, "cursor", "default");
				}
			}
		},

		_onMouseDown: function(e){
			if(this.cellContainer.parentTable.gridResize){
				var cellOffset =  Math.round(e.pageX - domGeometry.position(this.domNode).x);
				var cellWidth = domGeometry.getMarginBox(this.domNode).w;
				this.cellContainer.startPos =  Math.round(e.pageX);
				if(this.cellContainer.blockHeight == 0){
					this.cellContainer.blockHeight = domGeometry.getMarginBox(this.cellContainer.parentTable.dataTableBody).h;
				}
				if(this.cellContainer.blockTableHeight == 0){
					this.cellContainer.blockTableHeight = domGeometry.getMarginBox(this.cellContainer.parentTable.domNode).h;
				}

				this.getCellsWidths();
				if(this.isInResizingZone(e)){
					e.preventDefault(), e.stopPropagation();
					if(this.cellContainer.isFrozenHeader){
						var _this = this;
						this.cellContainer.parentTable.headWidget.allCells[0].getCellsWidths();
						this.cellContainer.parentTable.headWidget.tableWidth = 2; //frozen delimiter's width
						array.forEach(this.cellContainer.parentTable.headWidget.cellsWidth, function(value){
							if(value){
								_this.cellContainer.parentTable.headWidget.tableWidth = _this.cellContainer.parentTable.headWidget.tableWidth + value;
								if(has("ff")){
									_this.cellContainer.parentTable.headWidget.tableWidth += 2;
								}
							}
						});
						var headerWidth = this.cellContainer.parentTable.vertScroll ?
							this.cellContainer.parentTable.headWidget.tableWidth + 17 :
							this.cellContainer.parentTable.headWidget.tableWidth;
						domStyle.set(this.cellContainer.parentTable.headWidget.headMinWidth, "width", headerWidth + "px");
						domStyle.set(this.cellContainer.parentTable.bodyMinWidth, "width", this.cellContainer.parentTable.headWidget.tableWidth + "px");
					}
					this.blockOnClick = true;
					this.cellContainer.resizingStarted = true;

					if(this.cellIndex == 0){
						this.cellContainer.resizingCell = this;
					} else if(this == this.cellContainer.allCells[this.cellContainer.allCells.length - 1]){
						this.cellContainer.resizingCell = this.cellContainer.allCells[this.cellContainer.allCells.length - 2];
					} else{
						if((cellOffset < 4)){
							var _this = this;
							array.forEach(this.cellContainer.allCells, function(cell, index){
								if(_this == cell){
									_this.cellContainer.resizingCell = _this.cellContainer.allCells[index - 1];
								}
							})
						} else if(cellOffset > (cellWidth - 4)){
							this.cellContainer.resizingCell = this;
						}
					}
					this.cellContainer.originalWidth = this.cellContainer.cellsWidth[this.cellContainer.resizingCell.cellIndex];
					// capturing onmouse events for controlling resize even out of grid
					if(has("ff") || has("chrome") || has('trident') == 7){
						this._addedEvents.push(touch.release(document, lang.hitch(this.cellContainer.resizingCell, this._onMouseUp)));
						this._addedEvents.push(touch.move(document, lang.hitch(this.cellContainer.resizingCell, this._onMouseMove)));
						this.attached = true;
					} else if(has("ie")){
						this.cellContainer.resizingCell.domNode.setCapture();
					}
				} else{
					this.blockOnClick = false;
				}
			}
		},

		// release mouse after resizing
		releaseMouse: function(){
			if(has("ff") || has("chrome") || has('trident') == 7){
				array.forEach(this._addedEvents, function(handle){
					handle.remove();
				});
			} else{
				this.cellContainer.resizingCell.domNode.releaseCapture();
			}
		},

		_onMouseUp: function(){
			if(this.cellContainer.parentTable.source){
				this.dragging = this.cellContainer.parentTable.source.isDragging;
			}
			if(this.cellContainer.parentTable.gridResize){
				if(this.cellContainer.parentTable.source){
					this.dragging = this.cellContainer.parentTable.source.isDragging;
				}
				if(this.cellContainer.resizingStarted){
					this.cellContainer.resizingStarted = false;
					domStyle.set(this.domNode, "cursor", "default");
					this.cellContainer.resizeColumn();
					this.cellContainer.nowResizing = false;
					this.releaseMouse();
					this.cellContainer.tempOffset = -1;
				}
			}
		},

		_onMouseMove: function(e){
			if(this.cellContainer.parentTable.gridResize){
				if(this.cellContainer.resizingStarted){
					this.cellContainer.nowResizing = true;
					this.endPos = Math.round(e.pageX);
					var offset = this.endPos - this.cellContainer.startPos;
					this.cellContainer.resizeHeaderCell(offset, this.endPos);
				} else{
					if(this.cellContainer.parentTable.source){
						this.dragging = this.cellContainer.parentTable.source.isDragging;
					}
					if(!this.dragging){
						this.changeCursor(e);
					} else{
						domStyle.set(this.domNode, "cursor", "default");
					}
				}
			}
		},

		deleteColumn: function(e){
			e.preventDefault(), e.stopPropagation();
			var _this = this;
			var indexToRemove = -1;
			if(this.cellContainer.parentTable.gridControl.columnOrder){
				array.forEach(this.cellContainer.allCells, function(cell, index){
					if(cell.id == _this.id){
						indexToRemove = index;
					}
				});
				if(indexToRemove == 0){
					domClass.remove(this.cellContainer.allCells[1].domNode, "separator");
				}
				var rows = this.cellContainer.parentTable.rows.allRows;
				if(this.cellContainer.isFrozenHeader){
					rows = this.cellContainer.parentTable.frozenRows.allRows;
				}
				array.forEach(rows, function(row){
					var cells = query("td", row.domNode);
					array.forEach(cells, function(cell, index){
						if(index == (indexToRemove + 1)){
							domConstruct.destroy(cell);
						}
					});
					row.allCells = query("td", row.domNode);
					row.allCells = row.allCells.slice(1, row.allCells.length - 1);
				});
				domConstruct.destroy(this.domNode);
				this.updateCellContainer();
				this.cellContainer.parentTable.updateOrder();
				array.forEach(this.cellContainer.allCells, function(cell, index){
					if(index != (_this.cellContainer.allCells.length - 1)){
						domStyle.set(cell.domNode, "width", "auto");
					}
				});
				array.forEach(rows[0].allCells, function(cell){
					domStyle.set(cell, "width", "auto");
				});
				this.getCellsWidths();
			}
		},

		updateCellContainer: function(){
			this.cellContainer.allCells = query("td[widgetId]", this.cellContainer.domNode).map(function(cell){
				return registry.byNode(cell);
			});
		}
	});
});
