require({cache:{
'xcitec/Box':function(){
define([
	"dojo/_base/array",
	"dojo/query",
	"dojo/fx",
	"dojo/_base/window",
	"dojo/dom-construct",
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/text!xcitec/templates/Box.html",
	"dojo/dom-attr",
    "dojo/dom",
	"dojo/_base/lang",
	'dojo/parser',
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/_EnterKeySubmit",
  "./xcitecUtil",
  "./Toolbar",
  "./_base/ImageFactory"
], function (array, query, fx, windowModule, domConstruct, declare, domStyle, template, domAttr, dom, lang, parser, _Container, _Templated, _Widget, registry, _EnterKeySubmit, xcitecUtil, Toolbar, ImageFactory) {

	/*
	 * xcitec.Box control simply takes its' srcNodeRef and places it inside its' content node,
	 * except some elements:
	 * - xcitecRole='layoutButton' and xcitecRole='editButton' controls if exist are moved to
	 * the header of the Box in appropriate places
	 * - if expandAction/collapseAction attributes exists then its' actions are assigned to +/- icons
	 * - xcitecAttachPoint='buttons' node: if exists all its' content is moved to the bottom of the Box
	 *
	 *      Example:
	 *
	 *      <div data-dojo-type="xcitec.Box" caption="Master Data" id="box1" expandAction="..." collapseAction="...">
	
	            // edit and layout buttons block
	            <div xcitecRole="editButton" iconClass="icon_edit" data-dojo-type="xcitec.form.InlineLinkButton" label="Disabled LinkButton" onclick="alert('Edit button clicked!')">Bearbeiten</div>
	            <button xcitecRole="layoutButton" iconClass="icon_settings" data-dojo-type="xcitec.form.InlineDropDownButton"></button>
	
	            // content block
	            <div data-dojo-type="xcitec.layout.ContentArea">
	                .........
	            </div>
	            // bottom buttons block
	            <div xcitecAttachPoint="buttons">
	                ...........
	            </div>
	        </div>
	 */
	var Box = declare( [_Widget, _Templated, _Container, _EnterKeySubmit], {
	    templateString: template,
		// Flag to parser that I'll parse my contents, so it shouldn't.
		stopParser: true,
	
	    // caption of Box
	    caption: "",
	    captionId: "",
	    // comment for Box caption
	    captionComment: "",
	    captionCommentId: "",
	    /* state of Box, possible states are: 'closed', 'static', 'default', 'dialog'
	        closed - collpased Box with + icon
	        default - expanded Box with - icon
	        static - expanded Box with no icons
	     */
	    state: "default",
	    // is Box currently in edit mode
	    editMode: false,
	    editButtonId: "",
	    iconClass: "",

		_hasCollapseAction: false,
		_hasExpandAction: false,

	    // baseClass: "boxCollapsable",
	    baseClass: "box boxCollapsable",
	
	    // bottom buttons
	    buttons: null, //Array
	    buttonsIds: null, //Array
	
	    // property for unickly identify boxes
	    componentType: "box",
	
	    // filterbox has different style of bottom buttons, so we need to know if it is FilterBox
	    filterbox: false,
	
	    disabled: false,
	
	    // box nesting level, 0 by default
	    level: 0,
	
	    isContentLoaded: false,
	
	    formId: "",
	
	    postMixInProperties: function() {
	        if (this.disabled) {
	            this.state = "closed";
	        }
	        if (this.state == 'static') {
	            this.baseClass = "box";
	        } else {
	            this.stateHolder = query("> [xcitecRole$=stateholder]", this.srcNodeRef)[0];
	            if (this.stateHolder) {
	                var state = xcitecUtil.str2bool(domAttr.get(this.stateHolder, "value"));
	                state ? this.state = "default" : this.state = "closed";
	            }
	        }
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        } else {
	            this.iconClass = "boxIcon"
	        }

			this._hasCollapseAction = 'collapseAction' in this.params;
			this._hasExpandAction = 'expandAction' in this.params;
	
	        this.inherited(arguments);
	    },
	
		_fillContent: function(/*DomNode*/ source){
			var dest = this.containerNode;
			if(!(source && dest)) return;
	
			var _this = this;
			var divEditToolbar = null;
			var editButtons = [];
			this.buttons = [];
			this.buttonsIds = [];
	
			while(source.hasChildNodes()){
				var node = source.firstChild;
				source.removeChild(node);
				var role=domAttr.get(node, "xcitecRole")||"";
				switch (role.toLowerCase()){
				    case "layoutbutton":
				        domConstruct.place(node, _this.headerRight, "last"); //@todo add removing on destroy
				        break;
				    case "editbutton":
				        divEditToolbar = new Toolbar();
				        domAttr.set(node, "iconClass", "TOOLBAR:Edit");
				        domConstruct.place(node, divEditToolbar.domNode, "last");
				        editButtons.push(node);  //@todo check removing on destroy
				        _this.editButtonId = node.id;
				        break;
				    case "toolbar":
						_this._isToolbarExists = true;
				        domConstruct.place(node, _this.toolbar, "last");
				        break;
				    case "boxbottombuttons": //xcitec.Toolbar
				        domConstruct.place(node, _this.bottomButtons, "first");
				        _this.buttons.push(node);
				        var innerButtons = query(">", node);
				        array.forEach(innerButtons, function(btn) {
				            _this.buttonsIds.push(btn.id);
				        });
				        break;
				    default:
					    dest.appendChild(node);
				        break;
				}
			}
	
			if (!this.editMode && divEditToolbar && (this.buttons.length == 0)) {
			  domConstruct.place(divEditToolbar.domNode, _this.bottomButtons, "last");
			  this.buttons = editButtons;
			}

			parser.parse(this.header);
		},
	
	    //postCreate: function() {
		buildRendering: function(){
			this.inherited(arguments);
	        this._setState();
	
	        // set nesting level if necessary
	        if (this.level > 0) {
	            domStyle.set(this.headerLeft, "marginLeft", this.level * 50 + "px");
	        }
	
			// set caption
			if(this.caption != ""){
				this.captionNode.appendChild(windowModule.doc.createTextNode(this.caption));
			}
	
	        // set caption comment for Box
	        if (this.captionComment != "") {
	            this.comment.appendChild(windowModule.doc.createTextNode(this.captionComment));
	        }
	        if (this.stateHolder) {
	            domConstruct.place(this.stateHolder, this.domNode, "first");
	        }
	        if (this.state == 'static') {
	            domConstruct.destroy(this.icon);
	        }
            dom.setSelectable(this.headerLeft, false);
	        domConstruct.place(this.bottomButtons, this.containerNode, "last");
	        if (this.buttons.length == 0) {
	            domConstruct.destroy(this.bottomButtons);
	        }
	
		    // if there is no toolbar - remove holder node to avoid blank place
	        if (!this._isToolbarExists) {
	            domConstruct.destroy(this.toolbar);
	        }
	        this._initEnterPressHandling();
	    },
	
	    startup: function() {
	        if (this._started) {
	            return;
	        }
		    domStyle.set(this.animationNode, "display", "");

	        if ((this.state == "default") || (this.state == "static")) {
	            this._loadExternalContent();
	        }
	
		    //@todo need to review this functionality - dijit is not marked as started in result
		    // preventing loading content of closed boxes
	        //this.inherited(arguments);
	    },
	
	    destroy: function() {
	        /*dojo.forEach(this.buttonsIds, function(id) {
	            if (dojo.exists("destroy", dijit.byId(id))) {
	                dijit.byId(id).destroy();
	            }
	        });*/
	        if (this.editButtonId != "") {
	            if (registry.byId(this.editButtonId)) {
	                registry.byId(this.editButtonId).destroy();
	            }
	        }
		    array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}
			});
	        this.inherited(arguments);
	    },
	
	    _onClick: function(e) {
	        if (this.disabled) {
	            return;
	        }
	        if (this.state == 'static') {
	            return;
	        }
	        switch (this.state) {
	            case "default":
	                if (this._hasCollapseAction) {
	                    if (this.stateHolder) {
	                        domAttr.set(this.stateHolder, "value", "false");
	                    }
	                    this.collapseAction();
	                } else {
	                    this._close(e);
	                }
	                break;
	            case "closed":
	                if (this._hasExpandAction) {
	                    if (this.stateHolder) {
	                        domAttr.set(this.stateHolder, "value", "true");
	                    }
	                    this.expandAction();
	                } else {
	                    this._open(e);
	                }
	                break;
	            default:
	                break;
	        }
	    },

		collapseAction: function(){
			// summary:
			//		Method that fires after click on '+' icon
			// tags:
			//		abstract extension
		},

		expandAction: function(){
			// summary:
			//		Method that fires after click on '-' icon
			// tags:
			//		abstract extension
		},
	
	    // executes action 'COLLAPSE'
	    _onCollapse: function() {
	        this.collapseAction();
	    },
	
	    // executes action 'EXPAND'
	    _onExpand: function() {
	        this.expandAction();
	    },
	
	    // opens Box
	    _open: function(e) {
	        this._loadExternalContent();
	        this.state = "default";
	        domStyle.set(this.animationNode, "display", "none");
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "true");
	        }
	        fx.wipeIn({ node: this.animationNode, beforeBegin: lang.hitch(this, "_setState"), onEnd: lang.hitch(this, "_resizeChildren") }).play();
	    },
	
	    // closes Box
	    _close: function(e) {
	        this.state = "closed";
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "false");
	        }
	        fx.wipeOut({ node: this.animationNode, onEnd: lang.hitch(this, "_setState") }).play();
	    },
	
	    // function for updating Box state according to 'state' and 'editMode' attributes
	    _setState: function() {
	        var newStateClasses = this.baseClass.split(" ");
	//        newStateClasses = newStateClasses.concat(this.baseClass);
	
	        function multiply(modifier){
	          newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){ return c+modifier; }));
	        }
	        if (this.state == 'closed'){
	            multiply('Closed');
	        }
	        if (this.editMode) {
	            multiply('Edit');
	        }
	        domAttr.set(this.domNode, 'class', newStateClasses.join(" "));
	    },
	
	    getCaption: function() {
	        return this.caption;
	    },
	
	    getIconClass: function() {
	        return this.iconClass;
	    },
	
	    isDisabled: function() {
	        return this.disabled;
	    },
	
	    _loadExternalContent: function() {
	        if (!this.isContentLoaded) {
				parser.parse(this.containerNode);
	            this.isContentLoaded = true;
	        }
	    },

		_resizeChildren: function() {
			array.forEach(this.getChildren(), function(widget) {
				if('resize' in widget){
					widget.resize();
				}
			});
		},
	
	    getState: function() {
	        return this.state;
	    },
	
	    setFormId: function(id) {
	        this.formId = id;
	    }
	});


	return Box;
});

},
'dojo/request/default':function(){
define([
	'exports',
	'require',
	'../has'
], function(exports, require, has){
	var defId = has('config-requestProvider'),
		platformId;

	if( 1 ){
		platformId = './xhr';
	}else if( 0 ){
		platformId = './node';
	/* TODO:
	}else if( 0 ){
		platformId = './rhino';
   */
	}

	if(!defId){
		defId = platformId;
	}

	exports.getPlatformDefaultId = function(){
		return platformId;
	};

	exports.load = function(id, parentRequire, loaded, config){
		require([id == 'platform' ? platformId : defId], function(provider){
			loaded(provider);
		});
	};
});

},
'dojo/date':function(){
define(["./has", "./_base/lang"], function(has, lang){
// module:
//		dojo/date

var date = {
	// summary:
	//		Date manipulation utilities
};

date.getDaysInMonth = function(/*Date*/dateObject){
	// summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
};

date.isLeapYear = function(/*Date*/dateObject){
	// summary:
	//		Determines if the year of the dateObject is a leap year
	// description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
};

// FIXME: This is not localized
date.getTimezoneName = function(/*Date*/dateObject){
	// summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	// description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string,
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
};

// Utility methods to do arithmetic calculations with Dates

date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	// summary:
	//		Compare two date objects by date, time, or both.
	// description:
	//		Returns 0 if equal, positive if a > b, else negative.
	// date1:
	//		Date object
	// date2:
	//		Date object.  If not specified, the current Date is used.
	// portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}

	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	// summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	// date: Date
	//		Date object to start with
	// interval:
	//		A string representing the interval.  One of the following:
	//		"year", "month", "day", "hour", "minute", "second",
	//		"millisecond", "quarter", "week", "weekday"
	// amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo/cldr/supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	// summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	// date1:
	//		Date object
	// date2:
	//		Date object.  If not specified, the current Date is used.
	// interval:
	//		A string representing the interval.  One of the following:
	//		"year", "month", "day", "hour", "minute", "second",
	//		"millisecond", "quarter", "week", "weekday"
	//
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(date.difference(date1, date2, "day"));
			var weeks = parseInt(date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

// Don't use setObject() because it may overwrite dojo/date/stamp (if that has already been loaded)
 1  && lang.mixin(lang.getObject("dojo.date", true), date);

return date;
});

},
'xcitec/page/LoadingDialog':function(){
define([
	"dojo/_base/declare",
	"dijit/Dialog"
], function (declare, Dialog) {

	//dojo.require("xcitec.Dialog");
	var LoadingDialog = declare( Dialog, {
		templateString: '<div class="dijitDialog xcitecLoadingDialog"><span data-dojo-attach-point="closeButtonNode" class="dijitDialogCloseIcon"></span>' +
			'<div class="loadingDialogContainer"></div>',
		duration: 1,
		draggable: false
	});
	return LoadingDialog;
});
},
'dijit/selection':function(){
define([
	"dojo/_base/array",
	"dojo/dom", // dom.byId
	"dojo/_base/lang",
	"dojo/sniff", // has("ie") has("opera")
	"dojo/_base/window",
	"dijit/focus"
], function(array, dom, lang, has, baseWindow, focus){

	// module:
	//		dijit/selection

	// Note that this class is using feature detection, but doesn't use has() because sometimes on IE the outer window
	// may be running in standards mode (ie, IE9 mode) but an iframe may be in compatibility mode.   So the code path
	// used will vary based on the window.

	var SelectionManager = function(win){
		// summary:
		//		Class for monitoring / changing the selection (typically highlighted text) in a given window
		// win: Window
		//		The window to monitor/adjust the selection on.

		var doc = win.document;

		this.getType = function(){
			// summary:
			//		Get the selection type (like doc.select.type in IE).
			if(doc.getSelection){
				// W3C path
				var stype = "text";

				// Check if the actual selection is a CONTROL (IMG, TABLE, HR, etc...).
				var oSel;
				try{
					oSel = win.getSelection();
				}catch(e){ /*squelch*/ }

				if(oSel && oSel.rangeCount == 1){
					var oRange = oSel.getRangeAt(0);
					if(	(oRange.startContainer == oRange.endContainer) &&
						((oRange.endOffset - oRange.startOffset) == 1) &&
						(oRange.startContainer.nodeType != 3 /* text node*/)
						){
						stype = "control";
					}
				}
				return stype; //String
			}else{
				// IE6-8
				return doc.selection.type.toLowerCase();
			}
		};

		this.getSelectedText = function(){
			// summary:
			//		Return the text (no html tags) included in the current selection or null if no text is selected
			if(doc.getSelection){
				// W3C path
				var selection = win.getSelection();
				return selection ? selection.toString() : ""; //String
			}else{
				// IE6-8
				if(this.getType() == 'control'){
					return null;
				}
				return doc.selection.createRange().text;
			}
		};

		this.getSelectedHtml = function(){
			// summary:
			//		Return the html text of the current selection or null if unavailable
			if(doc.getSelection){
				// W3C path
				var selection = win.getSelection();
				if(selection && selection.rangeCount){
					var i;
					var html = "";
					for(i = 0; i < selection.rangeCount; i++){
						//Handle selections spanning ranges, such as Opera
						var frag = selection.getRangeAt(i).cloneContents();
						var div = doc.createElement("div");
						div.appendChild(frag);
						html += div.innerHTML;
					}
					return html; //String
				}
				return null;
			}else{
				// IE6-8
				if(this.getType() == 'control'){
					return null;
				}
				return doc.selection.createRange().htmlText;
			}
		};

		this.getSelectedElement = function(){
			// summary:
			//		Retrieves the selected element (if any), just in the case that
			//		a single element (object like and image or a table) is
			//		selected.
			if(this.getType() == "control"){
				if(doc.getSelection){
					// W3C path
					var selection = win.getSelection();
					return selection.anchorNode.childNodes[ selection.anchorOffset ];
				}else{
					// IE6-8
					var range = doc.selection.createRange();
					if(range && range.item){
						return doc.selection.createRange().item(0);
					}
				}
			}
			return null;
		};

		this.getParentElement = function(){
			// summary:
			//		Get the parent element of the current selection
			if(this.getType() == "control"){
				var p = this.getSelectedElement();
				if(p){ return p.parentNode; }
			}else{
				if(doc.getSelection){
					var selection = doc.getSelection();
					if(selection){
						var node = selection.anchorNode;
						while(node && (node.nodeType != 1)){ // not an element
							node = node.parentNode;
						}
						return node;
					}
				}else{
					var r = doc.selection.createRange();
					r.collapse(true);
					return r.parentElement();
				}
			}
			return null;
		};

		this.hasAncestorElement = function(/*String*/ tagName /* ... */){
			// summary:
			//		Check whether current selection has a  parent element which is
			//		of type tagName (or one of the other specified tagName)
			// tagName: String
			//		The tag name to determine if it has an ancestor of.
			return this.getAncestorElement.apply(this, arguments) != null; //Boolean
		};

		this.getAncestorElement = function(/*String*/ tagName /* ... */){
			// summary:
			//		Return the parent element of the current selection which is of
			//		type tagName (or one of the other specified tagName)
			// tagName: String
			//		The tag name to determine if it has an ancestor of.
			var node = this.getSelectedElement() || this.getParentElement();
			return this.getParentOfType(node, arguments); //DOMNode
		};

		this.isTag = function(/*DomNode*/ node, /*String[]*/ tags){
			// summary:
			//		Function to determine if a node is one of an array of tags.
			// node:
			//		The node to inspect.
			// tags:
			//		An array of tag name strings to check to see if the node matches.
			if(node && node.tagName){
				var _nlc = node.tagName.toLowerCase();
				for(var i=0; i<tags.length; i++){
					var _tlc = String(tags[i]).toLowerCase();
					if(_nlc == _tlc){
						return _tlc; // String
					}
				}
			}
			return "";
		};

		this.getParentOfType = function(/*DomNode*/ node, /*String[]*/ tags){
			// summary:
			//		Function to locate a parent node that matches one of a set of tags
			// node:
			//		The node to inspect.
			// tags:
			//		An array of tag name strings to check to see if the node matches.
			while(node){
				if(this.isTag(node, tags).length){
					return node; // DOMNode
				}
				node = node.parentNode;
			}
			return null;
		};

		this.collapse = function(/*Boolean*/ beginning){
			// summary:
			//		Function to collapse (clear), the current selection
			// beginning: Boolean
			//		Indicates whether to collapse the cursor to the beginning of the selection or end.
			if(doc.getSelection){
				// W3C path
				var selection = win.getSelection();
				if(selection.removeAllRanges){ // Mozilla
					if(beginning){
						selection.collapseToStart();
					}else{
						selection.collapseToEnd();
					}
				}else{ // Safari
					// pulled from WebCore/ecma/kjs_window.cpp, line 2536
					selection.collapse(beginning);
				}
			}else{
				// IE6-8
				var range = doc.selection.createRange();
				range.collapse(beginning);
				range.select();
			}
		};

		this.remove = function(){
			// summary:
			//		Function to delete the currently selected content from the document.
			var sel = doc.selection;
			if(doc.getSelection){
				// W3C path
				sel = win.getSelection();
				sel.deleteFromDocument();
				return sel; //Selection
			}else{
				// IE6-8
				if(sel.type.toLowerCase() != "none"){
					sel.clear();
				}
				return sel; //Selection
			}
		};

		this.selectElementChildren = function(/*DomNode*/ element, /*Boolean?*/ nochangefocus){
			// summary:
			//		clear previous selection and select the content of the node
			//		(excluding the node itself)
			// element: DOMNode
			//		The element you wish to select the children content of.
			// nochangefocus: Boolean
			//		Indicates if the focus should change or not.

			var range;
			element = dom.byId(element);
			if(doc.getSelection){
				// W3C
				var selection = win.getSelection();
				if(has("opera")){
					//Opera's selectAllChildren doesn't seem to work right
					//against <body> nodes and possibly others ... so
					//we use the W3C range API
					if(selection.rangeCount){
						range = selection.getRangeAt(0);
					}else{
						range = doc.createRange();
					}
					range.setStart(element, 0);
					range.setEnd(element,(element.nodeType == 3) ? element.length : element.childNodes.length);
					selection.addRange(range);
				}else{
					selection.selectAllChildren(element);
				}
			}else{
				// IE6-8
				range = element.ownerDocument.body.createTextRange();
				range.moveToElementText(element);
				if(!nochangefocus){
					try{
						range.select(); // IE throws an exception here if the widget is hidden.  See #5439
					}catch(e){ /* squelch */}
				}
			}
		};

		this.selectElement = function(/*DomNode*/ element, /*Boolean?*/ nochangefocus){
			// summary:
			//		clear previous selection and select element (including all its children)
			// element: DOMNode
			//		The element to select.
			// nochangefocus: Boolean
			//		Boolean indicating if the focus should be changed.  IE only.
			var range;
			element = dom.byId(element);	// TODO: remove for 2.0 or sooner, spec listed above doesn't allow for string
			if(doc.getSelection){
				// W3C path
				var selection = doc.getSelection();
				range = doc.createRange();
				if(selection.removeAllRanges){ // Mozilla
					// FIXME: does this work on Safari?
					if(has("opera")){
						//Opera works if you use the current range on
						//the selection if present.
						if(selection.getRangeAt(0)){
							range = selection.getRangeAt(0);
						}
					}
					range.selectNode(element);
					selection.removeAllRanges();
					selection.addRange(range);
				}
			}else{
				// IE6-8
				try{
					var tg = element.tagName ? element.tagName.toLowerCase() : "";
					if(tg === "img" || tg === "table"){
						range = baseWindow.body(doc).createControlRange();
					}else{
						range = baseWindow.body(doc).createRange();
					}
					range.addElement(element);
					if(!nochangefocus){
						range.select();
					}
				}catch(e){
					this.selectElementChildren(element, nochangefocus);
				}
			}
		};

		this.inSelection = function(node){
			// summary:
			//		This function determines if 'node' is
			//		in the current selection.
			// tags:
			//		public
			if(node){
				var newRange;
				var range;

				if(doc.getSelection){
					// WC3
					var sel = win.getSelection();
					if(sel && sel.rangeCount > 0){
						range = sel.getRangeAt(0);
					}
					if(range && range.compareBoundaryPoints && doc.createRange){
						try{
							newRange = doc.createRange();
							newRange.setStart(node, 0);
							if(range.compareBoundaryPoints(range.START_TO_END, newRange) === 1){
								return true;
							}
						}catch(e){ /* squelch */}
					}
				}else{
					// IE6-8, so we can't use the range object as the pseudo
					// range doesn't implement the boundary checking, we have to
					// use IE specific crud.
					range = doc.selection.createRange();
					try{
						newRange = node.ownerDocument.body.createTextRange();
						newRange.moveToElementText(node);
					}catch(e2){/* squelch */}
					if(range && newRange){
						// We can finally compare similar to W3C
						if(range.compareEndPoints("EndToStart", newRange) === 1){
							return true;
						}
					}
				}
			}
			return false; // Boolean
		},

		this.getBookmark = function(){
			// summary:
			//		Retrieves a bookmark that can be used with moveToBookmark to reselect the currently selected range.

			// TODO: merge additional code from Editor._getBookmark into this method

			var bm, rg, tg, sel = doc.selection, cf = focus.curNode;

			if(doc.getSelection){
				// W3C Range API for selections.
				sel = win.getSelection();
				if(sel){
					if(sel.isCollapsed){
						tg = cf? cf.tagName : "";
						if(tg){
							// Create a fake rangelike item to restore selections.
							tg = tg.toLowerCase();
							if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
								sel = {
									start: cf.selectionStart,
									end: cf.selectionEnd,
									node: cf,
									pRange: true
								};
								return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
							}
						}
						bm = {isCollapsed:true};
						if(sel.rangeCount){
							bm.mark = sel.getRangeAt(0).cloneRange();
						}
					}else{
						rg = sel.getRangeAt(0);
						bm = {isCollapsed: false, mark: rg.cloneRange()};
					}
				}
			}else if(sel){
				// If the current focus was a input of some sort and no selection, don't bother saving
				// a native bookmark.  This is because it causes issues with dialog/page selection restore.
				// So, we need to create pseudo bookmarks to work with.
				tg = cf ? cf.tagName : "";
				tg = tg.toLowerCase();
				if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
					if(sel.type && sel.type.toLowerCase() == "none"){
						return {
							isCollapsed: true,
							mark: null
						}
					}else{
						rg = sel.createRange();
						return {
							isCollapsed: rg.text && rg.text.length?false:true,
							mark: {
								range: rg,
								pRange: true
							}
						};
					}
				}
				bm = {};

				//'IE' way for selections.
				try{
					// createRange() throws exception when dojo in iframe
					// and nothing selected, see #9632
					rg = sel.createRange();
					bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
				}catch(e){
					bm.isCollapsed = true;
					return bm;
				}
				if(sel.type.toUpperCase() == 'CONTROL'){
					if(rg.length){
						bm.mark=[];
						var i=0,len=rg.length;
						while(i<len){
							bm.mark.push(rg.item(i++));
						}
					}else{
						bm.isCollapsed = true;
						bm.mark = null;
					}
				}else{
					bm.mark = rg.getBookmark();
				}
			}else{
				console.warn("No idea how to store the current selection for this browser!");
			}
			return bm; // Object
		};

		this.moveToBookmark = function(/*Object*/ bookmark){
			// summary:
			//		Moves current selection to a bookmark.
			// bookmark:
			//		This should be a returned object from getBookmark().

			// TODO: merge additional code from Editor._moveToBookmark into this method

			var mark = bookmark.mark;
			if(mark){
				if(doc.getSelection){
					// W3C Range API (FF, WebKit, Opera, etc)
					var sel = win.getSelection();
					if(sel && sel.removeAllRanges){
						if(mark.pRange){
							var n = mark.node;
							n.selectionStart = mark.start;
							n.selectionEnd = mark.end;
						}else{
							sel.removeAllRanges();
							sel.addRange(mark);
						}
					}else{
						console.warn("No idea how to restore selection for this browser!");
					}
				}else if(doc.selection && mark){
					//'IE' way.
					var rg;
					if(mark.pRange){
						rg = mark.range;
					}else if(lang.isArray(mark)){
						rg = doc.body.createControlRange();
						//rg.addElement does not have call/apply method, so can not call it directly
						//rg is not available in "range.addElement(item)", so can't use that either
						array.forEach(mark, function(n){
							rg.addElement(n);
						});
					}else{
						rg = doc.body.createTextRange();
						rg.moveToBookmark(mark);
					}
					rg.select();
				}
			}
		};

		this.isCollapsed = function(){
			// summary:
			//		Returns true if there is no text selected
			return this.getBookmark().isCollapsed;
		};
	};

	// singleton on the main window
	var selection = new SelectionManager(window);

	// hook for editor to use class
	selection.SelectionManager = SelectionManager;

	return selection;
});

},
'slmui/layout/MulticolumnLayout':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/dom-construct',
	'dojo/dom-style',
	'dojo/query',
	'dijit/_TemplatedMixin',
	'dijit/layout/_LayoutWidget',
	'dojo/text!./templates/Multicolumnlayout.html'
], function(declare, array, domConstruct, domStyle, query, _TemplatedMixin, _LayoutWidget, template){

	return declare([_LayoutWidget, _TemplatedMixin], {

		templateString: template,

		baseClass: 'MulticolumnLayout',

		// shownColumnsAmount: Number
		//		Amount of columns that will be shown without scroll.
		shownColumnsAmount: 4,

		startup: function(){
            var columns = this.getChildren().length;
            this.shownColumnsAmount = (columns < this.shownColumnsAmount) ? columns : this.shownColumnsAmount;

			array.forEach(this.getChildren(), function(child){
				this._setupChild(child);
			}, this);

			this.inherited(arguments);
		},

		layout: function() {
			this.inherited(arguments);

			array.forEach(this.getChildren(), function(child){
				var childWidth = (this._contentBox.w - (10 * this.shownColumnsAmount - 10)) / this.shownColumnsAmount,
					childIndex = this.getIndexOfChild(child);
				if(childIndex % 2 == 0){
					childWidth = Math.floor(childWidth);
				}else{
					childWidth = Math.ceil(childWidth);
				}
				child.resize({w: childWidth});
			}, this);
		},

		_setupChild: function(/*dijit/_WidgetBase*/ child){
			var td = domConstruct.create('td'),
		    childIndex = this.getIndexOfChild(child);
			domConstruct.place(child.domNode, td);
			domConstruct.place(td, this.containerNode, childIndex);
		}
	});
});
},
'xcitec/TabBarItem':function(){
define([
         "dojo/dom-class",
         "dojo/_base/declare",
         "dojo/dom-attr",
         "dojo/dom",
         "dojo/query",
         "dojo/keys",
         "dojo/text!./templates/TabBarItem.html",
         "./_StateWidget",
         "./_base/ImageFactory",
         "dijit/_WidgetsInTemplateMixin",
         "dijit/registry",
         "./Label"
       ], function (domClass, declare, domAttr, dom, query, keys, template, _StateWidget, ImageFactory, _WidgetsInTemplateMixin, registry) {

  return declare([_StateWidget, _WidgetsInTemplateMixin], {
    baseClass:"TabBarItem",
    widgetsInTemplate: true,
    templateString: template,

    //dijitDisabled or iconBlockedView

    //attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap), {
    //  label: {node: "containerNode", type: "innerHTML"}
    //}),

    // label: String
    //  Text to display in button.
    label: '',

    // labelId: String
    //  id Of Label for MultyLanguage support.
    labelId: '',

    // iconClass: String
    //		Class to apply to div to make it display an icon
    iconClass: "",

    // entityClass: String
    //		Entity Class represented by this Tab Item (used to define entity spicific icon)
    entityClass: "",

    // selected: Boolean
    //        define that item ia active
    selected: false,

    // pos: Number
    //		Identifier in TabBar
    pos: 0,

    // tabBar: Object
    //		Reference to TabBar container
    tabBar: null,

    //--- BEGIN --- functions of The _Widget Lifecycle  (in order of call) --

    postMixInProperties: function(){
      if (this.selected) {
        this.baseClass += " TabBarItemActive";
      } else {
        this.baseClass += " TabBarItemInactive";
      }
      if (this.entityClass) {
        this.iconClass = ImageFactory.createIconCss("SGOMINI:"+ this.entityClass);
      }
      this.inherited(arguments);
    },

    // method over-ride dijit._Templated to get label value defined inside
    // !importantme to call over-rided method.
    buildRendering: function(){
      var source = this.srcNodeRef;
      if(source && !("label" in this.params)){
        this.set('label', source.innerHTML);
      }
      this.inherited(arguments);
    },

    postCreate: function(){
      dom.setSelectable(this.domNode, false);
      if(this.selected){
        domAttr.set(this.domNode, 'tabindex', '0');
      }
    },

    //--- END --- functions of The _Widget Lifecycle  (in order of call) --

    _onHover: function(){
      domClass.add(this.domNode, 'TabBarItemHover');
    },

    _onUnhover: function(){
      domClass.remove(this.domNode, 'TabBarItemHover');
    },

    _onClick: function(evt){
      this.onClick(evt);
      evt.preventDefault(), evt.stopPropagation();
    },

    _onCloseButtonHover: function(){
      domClass.add(this.closeButton, 'TabBarCloseButtonHover');
    },

    _onCloseButtonUnhover: function(){
      domClass.remove(this.closeButton, 'TabBarCloseButtonHover');
    },

    _onCloseButtonClick: function(evt){
      this.onClose();
      evt.preventDefault(), evt.stopPropagation();
    },

    focus: function(){
      domClass.add(this.domNode, 'dijitMenuItemHover');
    },

    _onBlur: function(){
      domClass.remove(this.domNode, 'TabBarItemHover');
      domClass.remove(this.closeButton, 'TabBarCloseButtonHover');
      this.inherited(arguments);
    },

    // User defined functions to handle clicks
    onClick: function(/*Event*/ evt){
      // summary: User defined function to handle clicks
      console.log("onClick");
    },

    onClose: function(){
      // summary: User defined function to handle closes
      console.log("onClose");
    },

    _onKeyPress: function(/*Event*/ e){
      switch(e.keyCode){
      case keys.RIGHT_ARROW:
        if(this.domNode.nextElementSibling){
          registry.byNode(this.domNode.nextElementSibling).setFocus();
        }else{
          registry.byNode(query('> :first-child', this.domNode.parentNode)[0]).setFocus();
          this.tabBar.scrollToPos(0);
        }
        break;
      case keys.LEFT_ARROW:
        if(this.domNode.previousElementSibling){
          registry.byNode(this.domNode.previousElementSibling).setFocus();
        }else{
          registry.byNode(query('> :last-child', this.domNode.parentNode)[0]).setFocus();
          this.tabBar.scrollToPos(this.tabBar.tabbarVisibleWidth - this.tabBar.currentTabWidth * this.tabBar.tabCount);
        }
        break;
      }
    },

    _onFocus: function(/*Event*/ e){
      var currentWidth = this.tabBar.currentTabWidth * this.pos + this.tabBar.scrollLeftPos;
      if(currentWidth > this.tabBar.tabbarVisibleWidth){
        this.tabBar.scrollToPos(this.tabBar.tabbarVisibleWidth - this.tabBar.currentTabWidth * this.pos);
      }
      this.inherited(arguments);
    },

    _dummy: function(evt) {
      //dummy function
    }
  });
});

},
'xcitec/main':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["require","dojo/_base/lang","dojo/_base/kernel"], 
	   function(dRequire,dLang,dKernel) {
  /**
   * @name xcitec.main
   * @namespace This serves as the main module for the "xcitec" package when using "packages" in the "dojo config".
   */
	var iMain = dLang.getObject("xcitec", true);
	
	return iMain;
});
},
'dijit/Viewport':function(){
define([
	"dojo/Evented",
	"dojo/on",
	"dojo/domReady",
	"dojo/sniff",	// has("ie"), has("ios")
	"dojo/window" // getBox()
], function(Evented, on, domReady, has, winUtils){

	// module:
	//		dijit/Viewport

	/*=====
	return {
		// summary:
		//		Utility singleton to watch for viewport resizes, avoiding duplicate notifications
		//		which can lead to infinite loops.
		// description:
		//		Usage: Viewport.on("resize", myCallback).
		//
		//		myCallback() is called without arguments in case it's _WidgetBase.resize(),
		//		which would interpret the argument as the size to make the widget.
	};
	=====*/

	var Viewport = new Evented();

	var focusedNode;

	domReady(function(){
		var oldBox = winUtils.getBox();
		Viewport._rlh = on(window, "resize", function(){
			var newBox = winUtils.getBox();
			if(oldBox.h == newBox.h && oldBox.w == newBox.w){ return; }
			oldBox = newBox;
			Viewport.emit("resize");
		});

		// Also catch zoom changes on IE8, since they don't naturally generate resize events
		if(has("ie") == 8){
			var deviceXDPI = screen.deviceXDPI;
			setInterval(function(){
				if(screen.deviceXDPI != deviceXDPI){
					deviceXDPI = screen.deviceXDPI;
					Viewport.emit("resize");
				}
			}, 500);
		}

		// On iOS, keep track of the focused node so we can guess when the keyboard is/isn't being displayed.
		if(has("ios")){
			on(document, "focusin", function(evt){
				focusedNode = evt.target;
			});
			on(document, "focusout", function(evt){
				focusedNode = null;
			});
		}
	});

	Viewport.getEffectiveBox = function(/*Document*/ doc){
		// summary:
		//		Get the size of the viewport, or on mobile devices, the part of the viewport not obscured by the
		//		virtual keyboard.

		var box = winUtils.getBox(doc);

		// Account for iOS virtual keyboard, if it's being shown.  Unfortunately no direct way to check or measure.
		var tag = focusedNode && focusedNode.tagName && focusedNode.tagName.toLowerCase();
		if(has("ios") && focusedNode && !focusedNode.readOnly && (tag == "textarea" || (tag == "input" &&
			/^(color|email|number|password|search|tel|text|url)$/.test(focusedNode.type)))){

			// Box represents the size of the viewport.  Some of the viewport is likely covered by the keyboard.
			// Estimate height of visible viewport assuming viewport goes to bottom of screen, but is covered by keyboard.
			box.h *= (orientation == 0 || orientation == 180 ? 0.66 : 0.40);

			// Above measurement will be inaccurate if viewport was scrolled up so far that it ends before the bottom
			// of the screen.   In this case, keyboard isn't covering as much of the viewport as we thought.
			// We know the visible size is at least the distance from the top of the viewport to the focused node.
			var rect = focusedNode.getBoundingClientRect();
			box.h = Math.max(box.h, rect.top + rect.height);
		}

		return box;
	};

	return Viewport;
});

},
'xcitec/layout/_LayoutWidget':function(){

define([ "dijit/registry",
         "dojo/_base/declare",
        "dojo/_base/array",
        "dojo/_base/lang",
        "dojo/dom-attr",
        "dojo/dom-construct",
        "dojo/query",
        "dijit/layout/_LayoutWidget",
        "dijit/_TemplatedMixin"],
       function(registry, declare, array, lang, domAttr, domConstruct, query,_LayoutWidget, _Templated){
    return declare([_LayoutWidget, _Templated], {
      templateString: '<div class="dijitReset" data-dojo-attach-point="containerNode"></div>',

      onlyWidgetsAllowed: true,

      // caption: String
      //  Chapter caption.
      caption: '',

      // captionId: String
      //  id Of Label for MultyLanguage support.
      captionId: '',

      // showDelimiter: Boolean
      // show delimiter after Chapter
      showDelimiter: true,

      //@Override - dijit._Templated
      //     we need add wrappers around contained nodes
      _fillContent: function(/*DomNode*/ source){
        // summary:
        //		relocate source contents to templated container node
        //		this.containerNode must be able to receive children, or exceptions will be thrown
        var dest = this.containerNode;
        if(source && dest){
          this._processChildren(source);
        }
      },

      _appendChildItem: function(/*DomNode*/ child){
        var dest = this.containerNode;
        //append delimiter node if necessary
        if(dest.hasChildNodes() && this.showDelimiter){
          var hr = this.buildDelimiterNode();
          if(hr){
            dest.appendChild(hr);
          }
        }
        dest.appendChild(child);
      },

      _processChildren: function(/*DomNode*/ source){
        var children = query(">", source);
        array.forEach(children, function(child){
          //ignore not an HTML node element child
          if(child.nodeType != 1) return;

          //if onlyWidgetsAllowed ignore not an Widget child
          var isItWidget = domAttr.has(child, "data-dojo-type") && domAttr.get(child, "data-dojo-type");
          if(this.onlyWidgetsAllowed && !isItWidget) return;

          if(isItWidget){
            //create widget container wrapper - _LabelControlHolder
            //var labelNode  = query("label[for='"+child.id+"']", source)[0];
            var labelNode = null;
            if(child.id) labelNode = query("label[for='" + child.id + "']", source)[0];
            this._processChildWidget(child, labelNode);
          }else{
            this._appendChildItem(child);
          }
        }, this);
      },

      _processChildWidget: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
        // separate method to allow overload in Chapter
        var child = this.buildControlHolderNode(controlNode, labelNode);
        this._appendChildItem(child);
      },

      buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
        // abstract method - have to be overloaded in LayoutWidgets
        return controlNode;
      },

		resize: function(){
			// summary:
			//		@Override - dijit.layout._LayoutWidget
			// description:
			//		xcitec layout widgets don't need size calculation
			this.layout();
		},

      ajaxReplaceControl: function(/*WidgetDomNode*/ controlNode, targetNode){
        var parent = registry.getEnclosingWidget(targetNode.parentNode);
        if(parent && parent.isLabelControlHolder){
          var prevSibling = parent.domNode.previousSibling,
            nextSibling = parent.domNode.nextSibling,
			  container = parent.domNode.parentNode;
          if(parent.destroyRecursive){
            parent.destroyRecursive();
          } else if(parent.destroy){
            parent.destroy();
          }
          var controlHolderNode = this.buildControlHolderNode(controlNode);
          if(prevSibling){
            domConstruct.place(controlHolderNode, prevSibling, "after");
          }else if(nextSibling){
            domConstruct.place(controlHolderNode, nextSibling, "before");
          }else{
            domConstruct.place(controlHolderNode, container);
          }
          dojo.parser.parse(controlHolderNode);
          var controlHolderWidget = registry.byNode(controlHolderNode);
          if(lang.isFunction(controlHolderWidget.startup) && !controlHolderWidget._started){
            controlHolderWidget.startup();
          }
        }
      },

      buildDelimiterNode: function(){
        // abstract method - have to be overloaded in LayoutWidgets if delimiter is needed.
        return null;
      },

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
    })
});

},
'xcitec/form/TextBox':function(){

define(["dojo/_base/declare",
        "dojo/_base/window",
        "dojo/has",
        "dojo/keys",
        "dijit/form/ValidationTextBox",
        "dijit/registry",
        "dojo/text!./templates/TextBox.html",
        "dijit/_WidgetsInTemplateMixin",
        "../TrailingLabel"],
    function(declare, window, has, keys, ValidationTextBox, registry, template, _WidgetsInTemplateMixin){
  return declare([ValidationTextBox, _WidgetsInTemplateMixin], {

    templateString: template,
    baseClass: "dijitXcitecTextBox",

    // error: String
    //		Error message if control vaue in invalid state.
    error: "",
    warning: false,
    trailingText: "",
    trailingTextId:"",
    widgetsInTemplate: false,

    postMixInProperties: function() {
        this.inherited(arguments);
        if (this.trailingText) {
            this.widgetsInTemplate = true;
        }
        if (this.readOnly || this.disabled) {
            this.required = false;
        }
        if (this.error) {
            this.state = "Error";
        }
        if (this.warning) {
            this.baseClass = this.baseClass +" dijitXcitecTextBoxWarning";
        }
        if (this.required) {
            this.baseClass = this.baseClass + " dijitXcitecRequired";
        }

    },

    postCreate: function() {
        this.inherited(arguments);
        if (has("moz") || has("opera")) {
            this.connect(this.textbox, "onkeypress", this._onEnter);
        } else {
            this.connect(this.textbox, "onkeydown", this._onEnter);
        }
    },

    _onEnter: function(e) {
        switch (e.keyCode) {
            case keys.ENTER:
                this.doActionOnEnter(e);
                return;
        }
    },

    doActionOnEnter: function(e) {
        var submitButton = registry.byId("submitButton");
        if (submitButton) {
            submitButton.onClick();
            e.preventDefault(),
            e.stopPropagation();
        }
    },

    /*_setRequiredAttr: function(*//*Boolean*//* value) {
     this.required = false;
     },*/

    validate: function(/*Boolean*/ isFocused) {
        // summary:
        //		Called by oninit, onblur, and onkeypress.
        // description:
        //		Show missing or invalid messages if appropriate, and highlight textbox field.
        // tags:
        //		protected
        // Overrides 'dijit.form.ValidationTextBox.validate'
        return true;
    },

    getErrorMessage: function(/*Boolean*/ isFocused) {
        // summary: return an error message to show if appropriate
        return ""; // String
    },

    _setTrailingTextAttr: function(/*String*/ text){
        this.trailingText = text;
        if(this.trailingText){
            this.trailingTextWidget.domNode.appendChild(window.doc.createTextNode(this.trailingText));
        }
    }
  });
});

},
'xcitec/AttachmentContainer':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "./Label"
], function (array, declare, domClass, domConstruct, query, _Templated, _Widget, Label) {

	var AttachmentContainer = declare( [_Widget, _Templated], {
	  templateString: '<div data-dojo-attach-point="containerNode">'+
	                  '<div class="required"></div>' +
	                  '</div>',
	
	  attachments: null,
	  required:false,
	
	  noAttachments: "noAttachments",
	
	  postMixInProperties: function() {
	    var _this = this;
	    this.attachments = query("> ", this.srcNodeRef);
	    array.forEach(this.attachments, function(attachment, index) {
	      if (index != (_this.attachments.length - 1)) {
	        var divider = domConstruct.create("div");
	        domClass.add(divider, "ContentAreaHR");
	        domConstruct.place(divider, attachment, "after");
	      }
	    });
	    if (this.required) {
	        this.baseClass = this.baseClass + "dijitXcitecAttachmentRequired";
	    }
	  },
	
	  postCreate: function() {
	    var noAttachments;
	    if (this.noAttachments == "noAttachments") {
	      if (this.attachments.length == 0) {
	         noAttachments = new Label({label: "No attachments available"});
	         dojo.addClass(this.domNode, "noAttachmentField");
	        this.containerNode.appendChild(noAttachments.domNode);
	      }
	    } else if (this.noAttachments != "#" && this.noAttachments != "noAttachments") {
	      noAttachments = new Label({label: this.noAttachments});
	         dojo.addClass(this.domNode, "noAttachmentField");
	      this.containerNode.appendChild(noAttachments.domNode);
	    }
	    this.inherited(arguments);
	  }
	});
	
	return AttachmentContainer;
});

},
'dijit/Dialog':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/Deferred", // Deferred
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/i18n", // i18n.getLocalization
	"dojo/keys",
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/sniff", // has("ie") has("opera") has("dijit-legacy-requires")
	"dojo/window", // winUtils.getBox, winUtils.get
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/TimedMoveable", // TimedMoveable
	"./focus",
	"./_base/manager", // manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_CssStateMixin",
	"./form/_FormMixin",
	"./_DialogMixin",
	"./DialogUnderlay",
	"./layout/ContentPane",
	"dojo/text!./templates/Dialog.html",
	"dojo/i18n!./nls/common"
], function(require, array, aspect, declare, Deferred,
			dom, domClass, domGeometry, domStyle, fx, i18n, keys, lang, on, ready, has, winUtils,
			Moveable, TimedMoveable, focus, manager, _Widget, _TemplatedMixin, _CssStateMixin, _FormMixin, _DialogMixin,
			DialogUnderlay, ContentPane, template){

	// module:
	//		dijit/Dialog

	var _DialogBase = declare("dijit._DialogBase" + (has("dojo-bidi") ? "_NoBidi" : ""), [_TemplatedMixin, _FormMixin, _DialogMixin, _CssStateMixin], {
		templateString: template,

		baseClass: "dijitDialog",

		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		// Map widget attributes to DOMNode attributes.
		_setTitleAttr: { node: "titleNode", type: "innerHTML" },

		// open: [readonly] Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: manager.defaultDuration,

		// refocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_firstFocusItem: null,

		// _lastFocusItem: [private readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the movable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,
		_setDraggableAttr: function(/*Boolean*/ val){
			// Avoid _WidgetBase behavior of copying draggable attribute to this.domNode,
			// as that prevents text select on modern browsers (#14452)
			this._set("draggable", val);
		},

		// maxRatio: Number
		//		Maximum size to allow the dialog to expand to, relative to viewport size
		maxRatio: 0.9,

		// closable: Boolean
		//		Dialog show [x] icon to close itself, and ESC key will close the dialog.
		closable: true,
		_setClosableAttr: function(val){
			this.closeButtonNode.style.display = val ? "" : "none";
			this._set("closable", val);
		},

		postMixInProperties: function(){
			var _nlsResources = i18n.getLocalization("dijit", "common");
			lang.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			domStyle.set(this.domNode, {
				display: "none",
				position: "absolute"
			});
			this.ownerDocumentBody.appendChild(this.domNode);

			this.inherited(arguments);

			aspect.after(this, "onExecute", lang.hitch(this, "hide"), true);
			aspect.after(this, "onCancel", lang.hitch(this, "hide"), true);

			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overridden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._size();
			this._position();

			if(this.autofocus && DialogLevelManager.isTop(this)){
				this._getFocusItems(this.domNode);
				focus.focus(this._firstFocusItem);
			}

			this.inherited(arguments);
		},

		focus: function(){
			this._getFocusItems(this.domNode);
			focus.focus(this._firstFocusItem);
		},

		_endDrag: function(){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport,
			//		and also adjust position to be fully within the viewport, so user doesn't lose access to handle
			var nodePosition = domGeometry.position(this.domNode),
				viewport = winUtils.getBox(this.ownerDocument);
			nodePosition.y = Math.min(Math.max(nodePosition.y, 0), (viewport.h - nodePosition.h));
			nodePosition.x = Math.min(Math.max(nodePosition.x, 0), (viewport.w - nodePosition.w));
			this._relativePosition = nodePosition;
			this._position();
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = new ((has("ie") == 6) ? TimedMoveable // prevent overload, see #5285
					: Moveable)(node, { handle: this.titleBar });
				aspect.after(this._moveable, "onMoveStop", lang.hitch(this, "_endDrag"), true);
			}else{
				domClass.add(node, "dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": array.map(this["class"].split(/\s/),function(s){
					return s + "_underlay";
				}).join(" "),
				_onKeyDown: lang.hitch(this, "_onKey"),
				ownerDocument: this.ownerDocument
			};
		},

		_size: function(){
			// summary:
			//		If necessary, shrink dialog contents so dialog fits in viewport.
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the domGeometry.position(this.domNode) call below.
			if(this._singleChild){
				if(typeof this._singleChildOriginalStyle != "undefined"){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
					delete this._singleChildOriginalStyle;
				}
			}else{
				domStyle.set(this.containerNode, {
					width: "auto",
					height: "auto"
				});
			}

			var bb = domGeometry.position(this.domNode);

			// Get viewport size but then reduce it by a bit; Dialog should always have some space around it
			// to indicate that it's a popup.  This will also compensate for possible scrollbars on viewport.
			var viewport = winUtils.getBox(this.ownerDocument);
			viewport.w *= this.maxRatio;
			viewport.h *= this.maxRatio;

			if(bb.w >= viewport.w || bb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var containerSize = domGeometry.position(this.containerNode),
					w = Math.min(bb.w, viewport.w) - (bb.w - containerSize.w),
					h = Math.min(bb.h, viewport.h) - (bb.h - containerSize.h);

				if(this._singleChild && this._singleChild.resize){
					if(typeof this._singleChildOriginalStyle == "undefined"){
						this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					}
					this._singleChild.resize({w: w, h: h});
				}else{
					domStyle.set(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"    // workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position the dialog in the viewport.  If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the dialog.  Otherwise, use the Dialog's stored relative offset,
			//		adjusted by the viewport's scroll.
			if(!domClass.contains(this.ownerDocumentBody, "dojoMove")){    // don't do anything if called during auto-scroll
				var node = this.domNode,
					viewport = winUtils.getBox(this.ownerDocument),
					p = this._relativePosition,
					bb = p ? null : domGeometry.position(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
					;
				domStyle.set(node, {
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			if(evt.keyCode == keys.TAB){
				this._getFocusItems(this.domNode);
				var node = evt.target;
				if(this._firstFocusItem == this._lastFocusItem){
					// don't move focus anywhere, but don't allow browser to move focus off of dialog either
					evt.stopPropagation();
					evt.preventDefault();
				}else if(node == this._firstFocusItem && evt.shiftKey){
					// if we are shift-tabbing from first focusable item in dialog, send focus to last item
					focus.focus(this._lastFocusItem);
					evt.stopPropagation();
					evt.preventDefault();
				}else if(node == this._lastFocusItem && !evt.shiftKey){
					// if we are tabbing from last focusable item in dialog, send focus to first item
					focus.focus(this._firstFocusItem);
					evt.stopPropagation();
					evt.preventDefault();
				}
			}else if(this.closable && evt.keyCode == keys.ESCAPE){
				this.onCancel();
				evt.stopPropagation();
				evt.preventDefault();
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			// returns: dojo/promise/Promise
			//		Promise object that resolves when the display animation is complete

			if(this.open){
				return;
			}

			if(!this._started){
				this.startup();
			}

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized = true;
			}

			if(this._fadeOutDeferred){
				// There's a hide() operation in progress, so cancel it, but still call DialogLevelManager.hide()
				// as though the hide() completed, in preparation for the DialogLevelManager.show() call below.
				this._fadeOutDeferred.cancel();
				DialogLevelManager.hide(this);
			}

			// Recenter Dialog if user scrolls browser.  Connecting to document doesn't work on IE, need to use window.
			var win = winUtils.get(this.ownerDocument);
			this._modalconnects.push(on(win, "scroll", lang.hitch(this, "resize")));

			this._modalconnects.push(on(this.domNode, "keydown", lang.hitch(this, "_onKey")));

			domStyle.set(this.domNode, {
				opacity: 0,
				display: ""
			});

			this._set("open", true);
			this._onShow(); // lazy load trigger

			this._size();
			this._position();

			// fade-in Animation object, setup below
			var fadeIn;

			this._fadeInDeferred = new Deferred(lang.hitch(this, function(){
				fadeIn.stop();
				delete this._fadeInDeferred;
			}));

			// If delay is 0, code below will delete this._fadeInDeferred instantly, so grab promise while we can.
			var promise = this._fadeInDeferred.promise;

			fadeIn = fx.fadeIn({
				node: this.domNode,
				duration: this.duration,
				beforeBegin: lang.hitch(this, function(){
					DialogLevelManager.show(this, this.underlayAttrs);
				}),
				onEnd: lang.hitch(this, function(){
					if(this.autofocus && DialogLevelManager.isTop(this)){
						// find focusable items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						focus.focus(this._firstFocusItem);
					}
					this._fadeInDeferred.resolve(true);
					delete this._fadeInDeferred;
				})
			}).play();

			return promise;
		},

		hide: function(){
			// summary:
			//		Hide the dialog
			// returns: dojo/promise/Promise
			//		Promise object that resolves when the display animation is complete

			// If we haven't been initialized yet then we aren't showing and we can just return.
			// Likewise if we are already hidden, or are currently fading out.
			if(!this._alreadyInitialized || !this.open){
				return;
			}
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}

			// fade-in Animation object, setup below
			var fadeOut;

			this._fadeOutDeferred = new Deferred(lang.hitch(this, function(){
				fadeOut.stop();
				delete this._fadeOutDeferred;
			}));

			// fire onHide when the promise resolves.
			this._fadeOutDeferred.then(lang.hitch(this, 'onHide'));

			// If delay is 0, code below will delete this._fadeOutDeferred instantly, so grab promise while we can.
			var promise = this._fadeOutDeferred.promise;

			fadeOut = fx.fadeOut({
				node: this.domNode,
				duration: this.duration,
				onEnd: lang.hitch(this, function(){
					this.domNode.style.display = "none";
					DialogLevelManager.hide(this);
					this._fadeOutDeferred.resolve(true);
					delete this._fadeOutDeferred;
				})
			}).play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this._set("open", false);

			return promise;
		},

		resize: function(){
			// summary:
			//		Called when viewport scrolled or size changed.  Adjust Dialog as necessary to keep it visible.
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				this._size();
				if(!has("touch")){
					// If the user has scrolled the display then reposition the Dialog.  But don't do it for touch
					// devices, because it will counteract when a keyboard pops up and then the browser auto-scrolls
					// the focused node into view.
					this._position();
				}
			}
		},

		destroy: function(){
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}
			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}
			if(this._moveable){
				this._moveable.destroy();
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			DialogLevelManager.hide(this);

			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		_DialogBase = declare("dijit._DialogBase", _DialogBase, {
			_setTitleAttr: function(/*String*/ title){
				this._set("title", title);
				this.titleNode.innerHTML = title;
				this.applyTextDir(this.titleNode);
			},

			_setTextDirAttr: function(textDir){
				if(this._created && this.textDir != textDir){
					this._set("textDir", textDir);
					this.set("title", this.title);
				}
			}
		});
	}

	var Dialog = declare("dijit.Dialog", [ContentPane, _DialogBase], {
		// summary:
		//		A modal dialog Widget.
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.).
		// example:
		// |	<div data-dojo-type="dijit/Dialog" data-dojo-props="href: 'test.html'"></div>
		// example:
		// |	var foo = new Dialog({ title: "test dialog", content: "test content" });
		// |	foo.placeAt(win.body());
		// |	foo.startup();
	});
	Dialog._DialogBase = _DialogBase;	// for monkey patching and dojox/widget/DialogSimple

	var DialogLevelManager = Dialog._DialogLevelManager = {
		// summary:
		//		Controls the various active "levels" on the page, starting with the
		//		stuff initially visible on the page (at z-index 0), and then having an entry for
		//		each Dialog shown.

		_beginZIndex: 950,

		show: function(/*dijit/_WidgetBase*/ dialog, /*Object*/ underlayAttrs){
			// summary:
			//		Call right before fade-in animation for new dialog.
			//		Saves current focus, displays/adjusts underlay for new dialog,
			//		and sets the z-index of the dialog itself.
			//
			//		New dialog will be displayed on top of all currently displayed dialogs.
			//
			//		Caller is responsible for setting focus in new dialog after the fade-in
			//		animation completes.

			// Save current focus
			ds[ds.length - 1].focus = focus.curNode;

			// Set z-index a bit above previous dialog
			var zIndex = ds[ds.length - 1].dialog ? ds[ds.length - 1].zIndex + 2 : Dialog._DialogLevelManager._beginZIndex;
			domStyle.set(dialog.domNode, 'zIndex', zIndex);

			// Display the underlay, or if already displayed then adjust for this new dialog
			DialogUnderlay.show(underlayAttrs, zIndex - 1);

			ds.push({dialog: dialog, underlayAttrs: underlayAttrs, zIndex: zIndex});
		},

		hide: function(/*dijit/_WidgetBase*/ dialog){
			// summary:
			//		Called when the specified dialog is hidden/destroyed, after the fade-out
			//		animation ends, in order to reset page focus, fix the underlay, etc.
			//		If the specified dialog isn't open then does nothing.
			//
			//		Caller is responsible for either setting display:none on the dialog domNode,
			//		or calling dijit/popup.hide(), or removing it from the page DOM.

			if(ds[ds.length - 1].dialog == dialog){
				// Removing the top (or only) dialog in the stack, return focus
				// to previous dialog

				ds.pop();

				var pd = ds[ds.length - 1];	// the new active dialog (or the base page itself)

				// Adjust underlay
				if(ds.length == 1){
					// Returning to original page.  Hide the underlay.
					DialogUnderlay.hide();
				}else{
					// Popping back to previous dialog, adjust underlay.
					DialogUnderlay.show(pd.underlayAttrs, pd.zIndex - 1);
				}

				// Adjust focus.
				// TODO: regardless of setting of dialog.refocus, if the exeucte() method set focus somewhere,
				// don't shift focus back to button.  Note that execute() runs at the start of the fade-out but
				// this code runs later, at the end of the fade-out.  Menu has code like this.
				if(dialog.refocus){
					// If we are returning control to a previous dialog but for some reason
					// that dialog didn't have a focused field, set focus to first focusable item.
					// This situation could happen if two dialogs appeared at nearly the same time,
					// since a dialog doesn't set it's focus until the fade-in is finished.
					var focus = pd.focus;
					if(pd.dialog && (!focus || !dom.isDescendant(focus, pd.dialog.domNode))){
						pd.dialog._getFocusItems(pd.dialog.domNode);
						focus = pd.dialog._firstFocusItem;
					}

					if(focus){
						// Refocus the button that spawned the Dialog.   This will fail in corner cases including
						// page unload on IE, because the dijit/form/Button that launched the Dialog may get destroyed
						// before this code runs.  (#15058)
						try{
							focus.focus();
						}catch(e){
						}
					}
				}
			}else{
				// Removing a dialog out of order (#9944, #10705).
				// Don't need to mess with underlay or z-index or anything.
				var idx = array.indexOf(array.map(ds, function(elem){
					return elem.dialog;
				}), dialog);
				if(idx != -1){
					ds.splice(idx, 1);
				}
			}
		},

		isTop: function(/*dijit/_WidgetBase*/ dialog){
			// summary:
			//		Returns true if specified Dialog is the top in the task
			return ds[ds.length - 1].dialog == dialog;
		}
	};

	// Stack representing the various active "levels" on the page, starting with the
	// stuff initially visible on the page (at z-index 0), and then having an entry for
	// each Dialog shown.
	// Each element in stack has form {
	//		dialog: dialogWidget,
	//		focus: returnFromGetFocus(),
	//		underlayAttrs: attributes to set on underlay (when this widget is active)
	// }
	var ds = Dialog._dialogStack = [
		{dialog: null, focus: null, underlayAttrs: null}    // entry for stuff at z-index: 0
	];

	// If focus was accidentally removed from the dialog, such as if the user clicked a blank
	// area of the screen, or clicked the browser's address bar and then tabbed into the page,
	// then refocus.   Won't do anything if focus was removed because the Dialog was closed, or
	// because a new Dialog popped up on top of the old one, or when focus moves to popups
	focus.watch("curNode", function(attr, oldNode, node){
 		// Note: if no dialogs, ds.length==1 but ds[ds.length-1].dialog is null
		var topDialog = ds[ds.length - 1].dialog;

		// If a node was focused, and there's a Dialog currently showing, and not in the process of fading out...
		// Ignore focus events on other document though because it's likely an Editor inside of the Dialog.
		if(node && topDialog && !topDialog._fadeOutDeferred && node.ownerDocument == topDialog.ownerDocument){
			// If the node that was focused is inside the dialog or in a popup, even a context menu that isn't
			// technically a descendant of the the dialog, don't do anything.
			do{
				if(node == topDialog.domNode || domClass.contains(node, "dijitPopup")){ return; }
			}while(node = node.parentNode);

			// Otherwise, return focus to the dialog.  Use a delay to avoid confusing dijit/focus code's
			// own tracking of focus.
			topDialog.focus();
		}
	});

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/TooltipDialog"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return Dialog;
});

},
'idx/string':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define(["dojo/_base/lang","idx/main","dojo/string"], function (dLang,iMain,dString)
{
    /**
 	 * @name idx.string
 	 * @namespace Provides Javascript utility methods in addition to those provided by Dojo.
 	 */
	var iString = dLang.getObject("string", true, iMain);
	var regexChars = "\\[]()^.+*{}?!=-";
	var regexCache = {};
	var regexCacheCount = 0;
	var regexCacheMax = 50;

	/**
	 * @public 
	 * @function
	 * @name idx.string.unescapedIndexOf
	 * @description Searches the specified text for the first unescaped index of
	 *              a character contained in the specified string of characters.
	 * @param {String} text The text to search.
	 * @param {String} chars The characters for which to search (any of these).
	 * @param {String} escaper The optional character to use for escaping (defaults to "\\").
	 * @param {Number} limit The optional limit to the number of indexes desired in case only
	 *                 the first or first few are needed (use null, undefined, zero, false, or
	 * 	               a negative number to indicate no limit).
	 * @returns {Number[]} The array of indexes at which one of the specified characters
	 *                     is found or an empty array if not found.
	 */
	iString.unescapedIndexesOf = function(/*string*/ text, 
										  /*string*/ chars, 
										  /*string*/ escaper,
										  /*number*/ limit) {
		if (!chars || (chars.length == 0)) {
			throw "One or more characters must be provided to search for unescaped characters: "
				  + " chars=[ " + chars + " ], escaper=[ " + escaper + " ], text=[ " + text + " ]";
		}
		if (!escaper || escaper.length == 0) {
			escaper = "\\";
		}
		if (escaper.length > 1) {
			throw "The escaper must be a single character for unescaped character search: "
				  + " escaper=[ " + escaper + " ], chars=[ " + chars
				  + " ], text=[ " + text + " ]";
		}
		if (chars.indexOf(escaper) >= 0) {
			throw "The escaping character cannot also be a search character for unescaped character search: "
				  + " escaper=[ " +  escaper + " ], separators=[ " + separators
				+ " ] text=[ " + text + " ]";
			
		}
		if ((limit === null) || (limit === undefined) || (limit === 0) || (limit === false)) {
			limit = -1;
		}
		if (limit < 0) limit = 0;
		var result = [];
		var index = 0;
		var escaping = false;
		var current = null;
		for (index = 0; index < text.length; index++) {
			current = text.charAt(index);
			if (escaping) {
				escaping = false;
				continue;
			}
			if (current == escaper) {
				escaping = true;
				continue;
			}
			if (chars.indexOf(current) >= 0) {
				result.push(index);
				if (limit && (result.length == limit)) break;
			}
		}
		return result;
	};

	/**
	 * @public 
	 * @function
	 * @name idx.string.unescapedSplit
	 * @description Similar to iString.parseTokens, this function will split the
	 * 	            the specified text on unescaped separator characters, but will 
	 *              NOT remove any escaping in the original text so that the elements
	 *              in the returned array will still contain any original escaping.
	 * @param {String} text The text to split.
	 * @param {String} separators The optional string containing the separator 
	 *                 characters (defaults to ",")
	 * @param {String} escaper The optional character to use for escaping (defaults to "\\").
	 * @param {Number} limit The optional limit to the number of tokens desired in case only
	 *                 the first or first few are needed (use null, undefined, zero, false, or
	 * 	               a negative number to indicate no limit).
	 * @returns {String[]} Return an array of string parts.
	 */
	iString.unescapedSplit = function(/*string*/  text, 
	                                  /*string*/  separators, 
								      /*string*/  escaper,
								  	/*number*/ limit) {
		if (!separators || (separators.length == 0)) {
			separators = ",";
		}
		if (!escaper || escaper.length == 0) {
			escaper = "\\";
		}
		if (escaper.length > 1) {
			throw "The escaper must be a single character for escaped split: "
				  + " escaper=[ " + escaper + " ], separators=[ " + separators
				  + " ], text=[ " + text + " ]";
		}
		if (separators.indexOf(escaper) >= 0) {
			throw "The escaping character cannot also be a separator for escaped split: "
				  + " escaper=[ " +  escaper + " ], separators=[ " + separators
				+ " ] text=[ " + text + " ]";
			
		}
		if ((limit === null) || (limit === undefined) || (limit === 0) || (limit === false)) {
			limit = -1;
		}
		if (limit < 0) limit = 0;
		var result = [];
		var index = 0;
		var escaping = false;
		var current = null;
		var start = 0;
		for (index = 0; index < text.length; index++) {
			current = text.charAt(index);
			if (escaping) {
				escaping = false;
				continue;
			}
			if (current == escaper) {
				escaping = true;
				continue;
			}
			if (separators.indexOf(current) >= 0) {
				if (start == index) {
					result.push("");
				} else {
					result.push(text.substring(start,index));
				}
				start = index + 1;
				if (limit && (result.length == limit)) break;
			}
		}
		if ((!limit) || (limit && result.length < limit)) {
			if (start == text.length) {
				result.push("");
			} else {
				result.push(text.substring(start));
			}
		}
		return result;						
	};
	
	/**
	 * @public 
	 * @function
	 * @name idx.string.parseTokens
	 * @description Similar to iString.unescapedSplit, this function will split the
	 * 	            the specified text on unescaped separator characters, but will 
	 *              also remove any escaping in the original text when creating the
	 *              tokens unless those escaped characters are included in the optional
	 *              "specialChars" parameter (in which case they remain escaped if 
	 *              they are escaped in the original).
	 * @param {String} text The text to split.
	 * @param {String} separators The optional string containing the separator 
	 *        characters (defaults to ",")
	 * @param {String} escaper The optional character to use for escaping (defaults to "\\").
	 * @param {String} specialChars The optional string of special characters that if escaped
	 *                              in the original text should remain escaped in the tokens.
	 * @returns {String[]} Return an array of string parts.
	 */
	iString.parseTokens = function(/*string*/  text, 
	                               /*string*/  separators, 
								   /*string*/  escaper,
								   /*string*/  specialChars) {
		if (!separators || (separators.length == 0)) {
			separators = ",";
		}
		if (!escaper || escaper.length == 0) {
			escaper = "\\";
		}
		if (escaper.length > 1) {
			throw "The escaper must be a single character for token parsing: "
				  + " escaper=[ " + escaper + " ], separators=[ " + separators
				  + " ], text=[ " + text + " ]";
		}
		if (separators.indexOf(escaper) >= 0) {
			throw "The escaping character cannot also be a separator for token parsing: "
				  + " escaper=[ " +  escaper + " ], separators=[ " + separators
				+ " ] text=[ " + text + " ]";
			
		}
		if (specialChars && specialChars.length == 0) {
			specialChars = null;
		}
		var index = 0;
		var result = [];
		var start = 0;
		var escaping = false;
		var part = "";
		var end = -1;
		var current = null;
		for (index = 0; index < text.length; index++) {
			current = text.charAt(index);
			end = -1;
			if (!escaping && current==escaper) {
				if (specialChars && ((index+1)<text.length) 
				    && (specialChars.indexOf(current)>=0)) {
					// we have an escaped special character
					index++;
					continue;
				}
				
				// take the characters up to and excluding the current
				part = part + text.substring(start, index);
				
				// set the starting position to the next character
				start = index + 1;
				escaping = true;
				continue;
			}
			// check if we are escaping
			if (escaping) {
				escaping = false;
				continue;
			}
			// check if the current character is a separator
			if (separators.indexOf(current) >= 0) {
				end = index;
			}
							
			if (end >= 0) {
				part = part + text.substring(start, end);
				start = end + 1;
				result.push(part);
				part = "";
			}
		}
		
		// get the last part
		if ((end < 0) && (start < text.length)) {
			part = part + text.substring(start);
			result.push(part);
		} else if ((end >= 0)||escaping) {
			result.push(part);
		}
				
		return result;
	};
		
	/**
	 * @public 
	 * @function
	 * @name idx.string.escapedChars
	 * @description Escapes the specified characters in the specified text using the 
	 *              specified escape character.  The escape character is also escaped
	 *              using itself.
	 * @param {String} text The text to be escaped.
	 * @param {String} The string containing the characters to be escaped.
	 * @param {String} escaper The optional character to use for escaping (defaults to "\\").
	 * @returns {String[]} Return the escaped text.
	 */
	iString.escapeChars = function(/*string*/ text, /*string*/ chars, /*string*/ escaper) {
		if (!chars || (chars.length == 0)) {
			chars = "";
		}
		if (!escaper || escaper.length == 0) {
			escaper = "\\";
		}
		if (escaper.length > 1) {
			throw "The escaper must be a single character for escaped split: "
				  + " escaper=[ " + escaper + " ], chars=[ " + chars
				  + " ], text=[ " + text + " ]";
		}
		if (chars.indexOf(escaper) >= 0) {
			throw "The escaping character cannot also be a separator for escaped split: "
				  + " escaper=[ " +  escaper + " ], chars=[ " + chars
				+ " ] text=[ " + text + " ]";
			
		}
		var chars = chars + escaper;
		var regex = regexCache[chars];
		var pattern = "";
		if (! regex) {
			pattern = "([";
			for (index = 0; index < chars.length; index++) {
				if (regexChars.indexOf(chars.charAt(index)) >= 0) {
					pattern = pattern + "\\";
				}
				pattern = pattern + chars.charAt(index);
			}
			pattern = pattern + "])";
			try {
				regex = new RegExp(pattern, "g");
			} catch (e) {
				console.log("Pattern: " + pattern);
				throw e;
			}
			if (regexCacheCount < regexCacheMax) {
				regexCache[chars] = regex;
				regexCacheCount++;
			}
		}
		return text.replace(regex, escaper + "$1");
	};
		
	/**
	 * @public 
	 * @function
	 * @name idx.string.escapedJoin
	 * @description Joins the array into a single string separated by the specified 
	 *              separator and using the specified escaper to escape the separator.
	 * @param {String[]} arr The array of objects to join as a string.
	 * @param {String} separator The single-character string containing the separator character.
	 * @param {String} escaper The optional single character to use for escaping (defaults to "\\")
	 * @returns {String[]} Return an array of string parts.
	 */
	iString.escapedJoin = function(/*array*/ arr, /*string*/ separator, /*string*/ escaper) {
		if (!separator || (separator.length == 0)) {
			separator = ",";
		}
		if (separator.length > 1) {
			throw "Only one separator character can be used in escapedJoin: "
					" separator=[ " + separator + " ], text=[ " + text + " ]";
		}
		if (!escaper || escaper.length == 0) {
			escaper = "\\";
		}
		if (escaper.length > 1) {
			throw "The escaper must be a single character for escaped split: "
				  + " escaper=[ " + escaper + " ], separator=[ " + separator
				  + " ], text=[ " + text + " ]";
		}
		if (separator.indexOf(escaper) >= 0) {
			throw "The escaping character cannot also be a separator for escaped split: "
				  + " escaper=[ " +  escaper + " ], separator=[ " + separator
				+ " ] text=[ " + text + " ]";
			
		}
	
		var index = 0;
		var result = "";
		var part = null;
		var prefix = "";
		
		for (index = 0; index < arr.length; index++) {
			part = arr[index];
			part = iString.escapeChars(part, separator, escaper);
			result = result + prefix + part;
			prefix = separator;
		}
		
		return result;
	};
	
	/**
	 * @public 
	 * @function
	 * @name idx.string.startsWith
	 * @description Checks if the specified text starts with the specified prefix.
	 * @param {String} text The text to look at.
	 * @param {String} prefix The prefix to check for.
	 * @returns {Boolean} Return true if string "text" starts with "prefix"
	 */
	iString.startsWith = function(/*string*/ text, /*string*/ prefix){
		return (dLang.isString(text) && dLang.isString(prefix) && text.indexOf(prefix) === 0); // Boolean
	};
	
	/**
	 * @public 
	 * @function
	 * @name idx.string.endsWith
	 * @description Checks if the specified text ends with the specified suffix.
	 * @param {String} text The text to look at.
	 * @param {String} suffix The suffix to check for.
	 * @returns {Boolean} Return true if string "text" ends with "suffix"
	 */
	iString.endsWith = function(/*string*/ text, /*string*/ suffix){
		return (dLang.isString(text) && dLang.isString(suffix) && text.lastIndexOf(suffix) === text.length - suffix.length); // Boolean
	};

	/**	
	 * @public 
	 * @function
	 * @name idx.string.equalsIgnoreCase
	 * @description Case insensitive check for string equality.
	 * @param {String} s1 The first string.
	 * @param {String} s2 The second string.
	 * @returns {Boolean} Return true if string "s1" equals to "s2" with ignoring case
	 */
	iString.equalsIgnoreCase = function(/*string*/ s1, /*string*/ s2){
		return (dLang.isString(s1) && dLang.isString(s2) && s1.toLowerCase() === s2.toLowerCase()); // Boolean
	};
	
	/**	
	 * @public 
	 * @function
	 * @name idx.string.isNumber
	 * @description Checks if the specified parameter is a number and is finite.
	 * @param {Number} n The value to check.
	 * @returns {Boolean} Return true if 'n' is a Number
	 */
	iString.isNumber = function(/*number*/ n){
		return (typeof n == "number" && isFinite(n)); // Boolean
	};
	
	/**
	 * @public 
	 * @function
	 * @name idx.string.nullTrim
	 * @description Trims the specified string, and if it is empty after trimming, returns null.  
	 *              If the specified parameter is null or undefined, then null is returned.
	 * @param {String} str the string to trim
	 * @returns {String} Trimmed string or null if nothing left
	 */
    iString.nullTrim = function(/*String*/ str) {
            if (! str) return null;
            var result = dString.trim(str);
            return (result.length == 0) ? null : result;
        };
        
     /**
	 * @public 
	 * @function
	 * @name idx.string.propToLabel
	 * @description Converts a property name that is typically separated by camel case into a
	 *              psuedo label (i.e.: one that is not translated but based off the property
	 *              name but formatted nicer).  This method converts dots/periods into slashes. 
	 * @param {String} propName The property name to convert.
	 * @returns {String} The converted psuedo-label.
      *
      */
     iString.propToLabel = function(/*String*/ propName) {
     	if (!propName) return propName;
     	
	    // split the property name at any case switches or underscores
	    propName = dString.trim(propName);
	    var upperProp = propName.toUpperCase();
	    var lowerProp = propName.toLowerCase();
	    var index = 0;
	    var result = "";
	    var prevResult = "";
	    var prevUpper = false;
	    var prevLower = false;

		for (index = 0; index < propName.length; index++) {
	    	var upperChar = upperProp.charAt(index);
	        var lowerChar = lowerProp.charAt(index);
	        var origChar  = propName.charAt(index);

	        var upper = ((upperChar == origChar) && (lowerChar != origChar));
	        var lower = ((lowerChar == origChar) && (upperChar != origChar));

	        // check for spaces or underscores
	        if ((origChar == "_") || (origChar == " ")) {
	        	if (prevResult == " ") continue;
	         	prevResult = " ";
	            prevUpper  = upper;
	            prevLower  = lower;
	            result = result + prevResult;
	            continue;
	        }
	           
	        // check for dot notation
	        if (origChar == ".") {
	        	prevResult = "/";
	        	prevUpper = upper;
	        	prevLower = lower;
	        	result = result + " " + prevResult + " ";
	        	continue;
	        }

	        // check if this is the first character
	        if ((index == 0) || (prevResult == " ")) {
	        	prevResult = upperChar; 
	            prevUpper  = upper;
	            prevLower  = lower;
	            result = result + prevResult;
	            continue;
	        }

	        if ((!upper) && (!lower)) {
	        	if (prevUpper || prevLower) {
	            	result = result + " ";
	            }

	            // the character is not alphabetic, and neither is this one
	            prevUpper = upper;
	            prevLower = lower;
	            prevResult = origChar;

	            result = result + prevResult;
	            continue;
	        }

	        if ((!prevUpper) && (!prevLower)) {
	        	// previous character was non-alpha, but this one is alpha
	        	var prevSlash = (prevResult == "/");
	            prevUpper = upper;
	            prevLower = lower;
	            prevResult = upperChar;
	            if (prevSlash) {
	            	result = result + prevResult;
	            } else {
	            	result = result + " " + prevResult;
	            }
	            continue;
	        }

	        // if we get here then both the previous and current character are
	        // alphabetic characters so we need to detect shifts in case
	        if (upper && prevLower) {
	        	// we have switched cases
	            prevResult = upperChar;
	            prevUpper  = upper;
	            prevLower = lower;
	            result = result + " " + prevResult;
	            continue;
	        }

	        // if we get here then we simply use the lower-case version
	        prevResult = lowerChar;
	        prevUpper  = upper;
	        prevLower  = lower;
	        result = result + prevResult;
	 	}
	    
	    // return the result
	    return result;        
     };
     
     return iString;
});

},
'xcitec/Attachment':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/query",
	"dojo/text!xcitec/templates/Attachment.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "./xcitecUtil",
  "./_base/ImageFactory"
], function (array, declare, domClass, domConstruct, domStyle, query, template, _Templated, _Widget, registry, xcitecUtil, ImageFactory) {

	var Attachment = declare( [_Widget, _Templated], {
		templateString: template,
		// title of attached file
		docTitle: "",
		// filename
		fileName: "",
		// date of uploading file
		insertionDate: "",
		// time of uploading file
		insertionHour: "",
		// person who uploaded file
		person: "",
		iconClass: "",
		// is it new attachment
		newAttachment: false,
	
		uploadedOn: "",
		uploadedAt: "",
		uploadedBy: "",
	
		postMixInProperties: function(){
			if(this.iconClass){
				this.iconClass = ImageFactory.createIconCss(this.iconClass);
			}
			//todo in dojo 1.7 replace with "better solution"
			this.docTitle = xcitecUtil.escapeXml(this.docTitle);
			this.fileName = xcitecUtil.escapeXml(this.fileName);
			this.insertionDate = xcitecUtil.escapeXml(this.insertionDate);
			this.insertionHour = xcitecUtil.escapeXml(this.insertionHour);
			this.person = xcitecUtil.escapeXml(this.person);
	
			this.inherited(arguments);
		},
	
		_fillContent: function(){
			var _this = this;
			if(!this.newAttachment){
				this.buttonsContainer = query("> [xcitecAttachPoint$=buttons]", this.srcNodeRef)[0];
				if(this.buttonsContainer){
					domConstruct.place(this.buttonsContainer, _this.bottomButtons, "replace");
				}else{
					domConstruct.destroy(this.bottomButtons);
				}
			}else{
				// if this is new attachment - destroy unnessesary nodes
				if(this.newAttachment){
					domConstruct.destroy(this.fileProperties);
					domConstruct.destroy(this.downloadLink);
					domConstruct.destroy(this.docTitle);
					domClass.add(this.containerNode, "newAttachment");
					domStyle.set(this.documentIcon, "cursor", "default");
				}
			}
	
			// search for download button and assign it's action to file type icon and link
			//var downloadButton = dojo.byId("downloadButton." + this.id); //doesn't work if Ajax replace
			var downloadButton = query("> [id$=downloadButton."+this.id+"]", this.srcNodeRef)[0];
			if(downloadButton){
				this.onDownload = downloadButton.onclick;
				domConstruct.destroy(downloadButton);
			}
	
			this.inherited(arguments);
		},
	
		_onClick: function(e){
			this.onDownload(e);
		},
	
		// download action
		onDownload: function(e){
	
		},
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			//remove lost downloadButtonWidget
			var downloadButtonWidget = registry.byId("downloadButton." + this.id);
			if (downloadButtonWidget && downloadButtonWidget.destroyRecursive) {
				downloadButtonWidget.destroyRecursive(preserveDom);
			}
	
			// get all direct descendants and destroy them recursively
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
	});
	return Attachment;
});

},
'dojo/dom-construct':function(){
define(["exports", "./_base/kernel", "./sniff", "./_base/window", "./dom", "./dom-attr"],
		function(exports, dojo, has, win, dom, attr){
	// module:
	//		dojo/dom-construct
	// summary:
	//		This module defines the core dojo DOM construction API.

	// TODOC: summary not showing up in output, see https://github.com/csnover/js-doc-parse/issues/42

	// support stuff for toDom()
	var tagWrap = {
			option: ["select"],
			tbody: ["table"],
			thead: ["table"],
			tfoot: ["table"],
			tr: ["table", "tbody"],
			td: ["table", "tbody", "tr"],
			th: ["table", "thead", "tr"],
			legend: ["fieldset"],
			caption: ["table"],
			colgroup: ["table"],
			col: ["table", "colgroup"],
			li: ["ul"]
		},
		reTag = /<\s*([\w\:]+)/,
		masterNode = {}, masterNum = 0,
		masterName = "__" + dojo._scopeName + "ToDomId";

	// generate start/end tag strings to use
	// for the injection for each special tag wrap case.
	for(var param in tagWrap){
		if(tagWrap.hasOwnProperty(param)){
			var tw = tagWrap[param];
			tw.pre = param == "option" ? '<select multiple="multiple">' : "<" + tw.join("><") + ">";
			tw.post = "</" + tw.reverse().join("></") + ">";
			// the last line is destructive: it reverses the array,
			// but we don't care at this point
		}
	}

	var html5domfix;
	if(has("ie") <= 8){
		html5domfix = function(doc){
			doc.__dojo_html5_tested = "yes";
			var div = create('div', {innerHTML: "<nav>a</nav>", style: {visibility: "hidden"}}, doc.body);
			if(div.childNodes.length !== 1){
				('abbr article aside audio canvas details figcaption figure footer header ' +
				'hgroup mark meter nav output progress section summary time video').replace(
					/\b\w+\b/g, function(n){
						doc.createElement(n);
					}
				);
			}
			destroy(div);
		}
	}

	function _insertBefore(/*DomNode*/ node, /*DomNode*/ ref){
		var parent = ref.parentNode;
		if(parent){
			parent.insertBefore(node, ref);
		}
	}

	function _insertAfter(/*DomNode*/ node, /*DomNode*/ ref){
		// summary:
		//		Try to insert node after ref
		var parent = ref.parentNode;
		if(parent){
			if(parent.lastChild == ref){
				parent.appendChild(node);
			}else{
				parent.insertBefore(node, ref.nextSibling);
			}
		}
	}

	exports.toDom = function toDom(frag, doc){
		// summary:
		//		instantiates an HTML fragment returning the corresponding DOM.
		// frag: String
		//		the HTML fragment
		// doc: DocumentNode?
		//		optional document to use when creating DOM nodes, defaults to
		//		dojo/_base/window.doc if not specified.
		// returns:
		//		Document fragment, unless it's a single node in which case it returns the node itself
		// example:
		//		Create a table row:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		var tr = domConstruct.toDom("<tr><td>First!</td></tr>");
		//	|	});

		doc = doc || win.doc;
		var masterId = doc[masterName];
		if(!masterId){
			doc[masterName] = masterId = ++masterNum + "";
			masterNode[masterId] = doc.createElement("div");
		}

		if(has("ie") <= 8){
			if(!doc.__dojo_html5_tested && doc.body){
				html5domfix(doc);
			}
		}

		// make sure the frag is a string.
		frag += "";

		// find the starting tag, and get node wrapper
		var match = frag.match(reTag),
			tag = match ? match[1].toLowerCase() : "",
			master = masterNode[masterId],
			wrap, i, fc, df;
		if(match && tagWrap[tag]){
			wrap = tagWrap[tag];
			master.innerHTML = wrap.pre + frag + wrap.post;
			for(i = wrap.length; i; --i){
				master = master.firstChild;
			}
		}else{
			master.innerHTML = frag;
		}

		// one node shortcut => return the node itself
		if(master.childNodes.length == 1){
			return master.removeChild(master.firstChild); // DOMNode
		}

		// return multiple nodes as a document fragment
		df = doc.createDocumentFragment();
		while((fc = master.firstChild)){ // intentional assignment
			df.appendChild(fc);
		}
		return df; // DocumentFragment
	};

	exports.place = function place(/*DOMNode|String*/ node, /*DOMNode|String*/ refNode, /*String|Number?*/ position){
		// summary:
		//		Attempt to insert node into the DOM, choosing from various positioning options.
		//		Returns the first argument resolved to a DOM node.
		// node: DOMNode|String
		//		id or node reference, or HTML fragment starting with "<" to place relative to refNode
		// refNode: DOMNode|String
		//		id or node reference to use as basis for placement
		// position: String|Number?
		//		string noting the position of node relative to refNode or a
		//		number indicating the location in the childNodes collection of refNode.
		//		Accepted string values are:
		//
		//		- before
		//		- after
		//		- replace
		//		- only
		//		- first
		//		- last
		//
		//		"first" and "last" indicate positions as children of refNode, "replace" replaces refNode,
		//		"only" replaces all children.  position defaults to "last" if not specified
		// returns: DOMNode
		//		Returned values is the first argument resolved to a DOM node.
		//
		//		.place() is also a method of `dojo/NodeList`, allowing `dojo/query` node lookups.
		// example:
		//		Place a node by string id as the last child of another node by string id:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		domConstruct.place("someNode", "anotherNode");
		//	|	});
		// example:
		//		Place a node by string id before another node by string id
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		domConstruct.place("someNode", "anotherNode", "before");
		//	|	});
		// example:
		//		Create a Node, and place it in the body element (last child):
		//	|	require(["dojo/dom-construct", "dojo/_base/window"
		//	|	], function(domConstruct, win){
		//	|		domConstruct.place("<div></div>", win.body());
		//	|	});
		// example:
		//		Put a new LI as the first child of a list by id:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		domConstruct.place("<li></li>", "someUl", "first");
		//	|	});

		refNode = dom.byId(refNode);
		if(typeof node == "string"){ // inline'd type check
			node = /^\s*</.test(node) ? exports.toDom(node, refNode.ownerDocument) : dom.byId(node);
		}
		if(typeof position == "number"){ // inline'd type check
			var cn = refNode.childNodes;
			if(!cn.length || cn.length <= position){
				refNode.appendChild(node);
			}else{
				_insertBefore(node, cn[position < 0 ? 0 : position]);
			}
		}else{
			switch(position){
				case "before":
					_insertBefore(node, refNode);
					break;
				case "after":
					_insertAfter(node, refNode);
					break;
				case "replace":
					refNode.parentNode.replaceChild(node, refNode);
					break;
				case "only":
					exports.empty(refNode);
					refNode.appendChild(node);
					break;
				case "first":
					if(refNode.firstChild){
						_insertBefore(node, refNode.firstChild);
						break;
					}
					// else fallthrough...
				default: // aka: last
					refNode.appendChild(node);
			}
		}
		return node; // DomNode
	};

	var create = exports.create = function create(/*DOMNode|String*/ tag, /*Object*/ attrs, /*DOMNode|String?*/ refNode, /*String?*/ pos){
		// summary:
		//		Create an element, allowing for optional attribute decoration
		//		and placement.
		// description:
		//		A DOM Element creation function. A shorthand method for creating a node or
		//		a fragment, and allowing for a convenient optional attribute setting step,
		//		as well as an optional DOM placement reference.
		//
		//		Attributes are set by passing the optional object through `dojo.setAttr`.
		//		See `dojo.setAttr` for noted caveats and nuances, and API if applicable.
		//
		//		Placement is done via `dojo.place`, assuming the new node to be the action
		//		node, passing along the optional reference node and position.
		// tag: DOMNode|String
		//		A string of the element to create (eg: "div", "a", "p", "li", "script", "br"),
		//		or an existing DOM node to process.
		// attrs: Object
		//		An object-hash of attributes to set on the newly created node.
		//		Can be null, if you don't want to set any attributes/styles.
		//		See: `dojo.setAttr` for a description of available attributes.
		// refNode: DOMNode|String?
		//		Optional reference node. Used by `dojo.place` to place the newly created
		//		node somewhere in the dom relative to refNode. Can be a DomNode reference
		//		or String ID of a node.
		// pos: String?
		//		Optional positional reference. Defaults to "last" by way of `dojo.place`,
		//		though can be set to "first","after","before","last", "replace" or "only"
		//		to further control the placement of the new node relative to the refNode.
		//		'refNode' is required if a 'pos' is specified.
		// example:
		//		Create a DIV:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		var n = domConstruct.create("div");
		//	|	});
		//
		// example:
		//		Create a DIV with content:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		var n = domConstruct.create("div", { innerHTML:"<p>hi</p>" });
		//	|	});
		//
		// example:
		//		Place a new DIV in the BODY, with no attributes set
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		var n = domConstruct.create("div", null, dojo.body());
		//	|	});
		//
		// example:
		//		Create an UL, and populate it with LI's. Place the list as the first-child of a
		//		node with id="someId":
		//	|	require(["dojo/dom-construct", "dojo/_base/array"],
		//	|	function(domConstruct, arrayUtil){
		//	|		var ul = domConstruct.create("ul", null, "someId", "first");
		//	|		var items = ["one", "two", "three", "four"];
		//	|		arrayUtil.forEach(items, function(data){
		//	|			domConstruct.create("li", { innerHTML: data }, ul);
		//	|		});
		//	|	});
		//
		// example:
		//		Create an anchor, with an href. Place in BODY:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		domConstruct.create("a", { href:"foo.html", title:"Goto FOO!" }, dojo.body());
		//	|	});

		var doc = win.doc;
		if(refNode){
			refNode = dom.byId(refNode);
			doc = refNode.ownerDocument;
		}
		if(typeof tag == "string"){ // inline'd type check
			tag = doc.createElement(tag);
		}
		if(attrs){ attr.set(tag, attrs); }
		if(refNode){ exports.place(tag, refNode, pos); }
		return tag; // DomNode
	};

	function _empty(/*DomNode*/ node){
		if(node.canHaveChildren){
			try{
				// fast path
				node.innerHTML = "";
				return;
			}catch(e){
				// innerHTML is readOnly (e.g. TABLE (sub)elements in quirks mode)
				// Fall through (saves bytes)
			}
		}
		// SVG/strict elements don't support innerHTML/canHaveChildren, and OBJECT/APPLET elements in quirks node have canHaveChildren=false
		for(var c; c = node.lastChild;){ // intentional assignment
			_destroy(c, node); // destroy is better than removeChild so TABLE subelements are removed in proper order
		}
	}

	exports.empty = function empty(/*DOMNode|String*/ node){
		// summary:
		//		safely removes all children of the node.
		// node: DOMNode|String
		//		a reference to a DOM node or an id.
		// example:
		//		Destroy node's children byId:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		domConstruct.empty("someId");
		//	|	});

		_empty(dom.byId(node));
	};


	function _destroy(/*DomNode*/ node, /*DomNode*/ parent){
		// in IE quirks, node.canHaveChildren can be false but firstChild can be non-null (OBJECT/APPLET)
		if(node.firstChild){
			_empty(node);
		}
		if(parent){
			// removeNode(false) doesn't leak in IE 6+, but removeChild() and removeNode(true) are known to leak under IE 8- while 9+ is TBD.
			// In IE quirks mode, PARAM nodes as children of OBJECT/APPLET nodes have a removeNode method that does nothing and
			// the parent node has canHaveChildren=false even though removeChild correctly removes the PARAM children.
			// In IE, SVG/strict nodes don't have a removeNode method nor a canHaveChildren boolean.
			has("ie") && parent.canHaveChildren && "removeNode" in node ? node.removeNode(false) : parent.removeChild(node);
		}
	}
	var destroy = exports.destroy = function destroy(/*DOMNode|String*/ node){
		// summary:
		//		Removes a node from its parent, clobbering it and all of its
		//		children.
		//
		// description:
		//		Removes a node from its parent, clobbering it and all of its
		//		children. Function only works with DomNodes, and returns nothing.
		//
		// node: DOMNode|String
		//		A String ID or DomNode reference of the element to be destroyed
		//
		// example:
		//		Destroy a node byId:
		//	|	require(["dojo/dom-construct"], function(domConstruct){
		//	|		domConstruct.destroy("someId");
		//	|	});

		node = dom.byId(node);
		if(!node){ return; }
		_destroy(node, node.parentNode);
	};
});

},
'xcitec/form/MultiSelect':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/form/MultiSelect"
], function (array, declare, domConstruct, query, MultiSelect) {

	var MultiSelect = declare( MultiSelect, {
		// summary:
		//		Widget version of a <select multiple=true> element,
		//		for selecting multiple options.
	
	  getOptions: function(){
	    // summary:
	    //		Access the NodeList of the selected options directly
	    return query("option",this.containerNode); // dojo.NodeList
	  },
	
	  moveSelectedUp: function(){
	    // summary:
	    //		change the order inside.
	    //    move selected items one step up.
	    var prevOpt = null;
	    var opts = this.getOptions();
	    opts.forEach(function(n){
	      if (!n.selected) { prevOpt = n; return; }
	      if (!prevOpt) return;
	      domConstruct.place(n, prevOpt, "before");
	    });
	  },
	
	  moveSelectedDown: function(){
	    // summary:
	    //		change the order inside.
	    //    move selected items one step down.
	    var opts = this.getOptions();
	    // under FF 3.5 standard array functions doesn't work with NodeList
	    // so will create Array from NodeList
	    var tmparr = [];
	    opts.forEach(function(n){ tmparr.push(n); });
	    opts = tmparr;
	
	    // and then apply logic
	    var prevOpt = null;
	    array.forEach( opts.reverse(), function(n){
	      if (!n.selected) { prevOpt = n; return; }
	      if (!prevOpt) return;
	      domConstruct.place(n, prevOpt, "after");
	    });
	  }
	
	
	});
	
	return MultiSelect;
});
},
'idx/form/TriStateCheckBox':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/kernel",
	"dojo/_base/event",
	"dojo/dom-attr",
	"dojo/query",
	"dojo/has",
	"dijit/form/ToggleButton",
	"./_CssStateMixin",
	"./_CompositeMixin",
	"./_ValidationMixin",
		// ====================================================================================================================
	// ------
	// Load _TemplatePlugableMixin and PlatformPluginRegistry if on "mobile" or if on desktop, but using the 
	// platform-plugable API.  Any prior call to PlaformPluginRegistry.setGlobalTargetPlatform() or 
	// PlatformPluginRegistry.setRegistryDefaultPlatform() sets "platform-plugable" property for dojo/has.
	// ------
	"require", 
	"require",
	
	// ------
	// We want to load the desktop template unless we are using the mobile implementation.
	// ------
	"idx/has!#idx_form_TriStateCheckBox-desktop?dojo/text!./templates/TriStateCheckBox.html" 	// desktop widget, load the template
		+ ":#idx_form_TriStateCheckBox-mobile?"													// mobile widget, don't load desktop template
		+ ":#desktop?dojo/text!./templates/TriStateCheckBox.html"								// global desktop platform, load template
		+ ":#mobile?"																			// global mobile platform, don't load
		+ ":dojo/text!./templates/TriStateCheckBox.html", 										// no dojo/has features, load the template
			
	// ------
	// Load the mobile plugin according to build-time/runtime dojo/has features
	// ------
	"idx/has!#idx_form_TriStateCheckBox-mobile?./plugins/phone/TriStateCheckBoxPlugin"			// mobile widget, load the plugin
		+ ":#idx_form_TriStateCheckBox-desktop?"													// desktop widget, don't load plugin
		+ ":#mobile?./plugins/phone/TriStateCheckBoxPlugin"										// global mobile platform, load plugin
		+ ":",																					// no features, don't load plugin

	
	"dojo/NodeList-dom" // NodeList.addClass/removeClass
], function(declare, array, lang, kernel, event, domAttr, query, has, 
			ToggleButton, _CssStateMixin, _CompositeMixin, _ValidationMixin, 
			TemplatePlugableMixin, PlatformPluginRegistry, desktopTemplate, MobilePlugin){
	
	var baseClassName = "idx.form.TriStateCheckBox";
	if (has("mobile") || has("platform-plugable")) {
		baseClassName = baseClassName + "Base";
	}
	if (has("dojo-bidi")) {
		baseClassName = baseClassName + "_";
	}		
		
	var iForm = lang.getObject("idx.oneui.form", true); // for backward compatibility with IDX 1.2

	/**
	 * @name idx.form.TriStateCheckBox
	 * @class idx.form.TriStateCheckBox is implemented according to IBM One UI(tm) <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y6&vsub=*&hsub=*&openpanes=0100110000">Check Boxes Standard</a></b> 
	 * Compared with dijit.form.CheckBox, TriStateCheckBox has a indeterminate state in addtion to checked state and unchecked state.
	 * The state cycle of TriStateCheckBox is configureable, so you can even create a CheckBox with states cycle such as: 'checked' -> 'indeterminate' -> 'checked' -> 'unchecked'
	 * As idx.form.TriStateCheckBox is a composite widget. You can set a label for it directly rather than adding a label tag mannually.
	 * @augments dijit.form.ToggleButton
	 * @augments idx.form._CssStateMixin
	 * @augments idx.form._CompositeMixin
	 * @augments idx.form._ValidationMixin
	 */
	iForm.TriStateCheckBox = declare(baseClassName, [ToggleButton, _CssStateMixin, _CompositeMixin, _ValidationMixin],
	/**@lends idx.form.TriStateCheckBox.prototype*/
	{
		templateString: desktopTemplate,
		
		instantValidate: true,
		
		baseClass: "idxOneuiTriStateCheckBoxWrap",
		
		oneuiBaseClass: "idxOneuiTriStateCheckBox",

		labelAlignment: "horizontal",
		
		/**
		 * States cycle of a TriStateCheckBox. By default it's [false, "mixed", true] which means the state cycles from
		 * 'unchecked' -> 'indeterminate' -> 'checked'.
		 * Note: The value of the 'checked' property of a TriStateCheckBox should be one of these states.
		 * For example, if the 'states' property of a TriStateCheckBox is [false, true], then setting 'checked'
		 * property to 'midex' is invalid.
		 * @type Array|String
		 * @default [false, "mixed", true]
		 */
		states: "",
		
		/**
		 * Alt text used to replace the image to show
		 * current state of TriStateCheckBox in high contrast mode.
		 * @type Object
		 * @default {"False": '&#9633;', "True": '&#8730;', "Mixed": '&#9632;'};
		 * @private
		 */
		_stateLabels: null,
		
		/**
		 * The submit value for each state. By default, "on"/"mixed" will be submitted
		 * if the state of a TriStateCheckBox is checked/indeterminate.
		 * @type Object
		 * @default {"False": false, "True": "on", "Mixed": "mixed"};
		 */
		stateValues: null,
		
		/**
		 * The current state of the TriStateCheckBox
		 * @type Integer
		 * @private
		 */
		_currentState: 0,
		
		/**
		 * The current state type of the TriStateCheckBox.
		 * Could be "False", "True" or "Mixed"
		 * @private
		 */
		_stateType: "False",
		
		/**
		 * Should this widget respond to user input or not.
		 * In markup, this is specified as "readOnly".
		 * Similar to disabled except readOnly form values are submitted.
		 * @type Boolean
		 */
		readOnly: false,
		
		/**
		 * Current check state of the check box. Can be one of element in 'states' property.
		 * @type Boolean|String
		 * @default false
		 */
		checked: "",
		
		constructor: function(){
			// summary:
			//		Runs on widget initialization to setup arrays etc.
			// tags:
			//		private
			this.states = [false, "mixed", true];
			this.checked = false;
			this._stateLabels = {
				"False": '&#9633;',
				"True": '&#8730;',
				"Mixed": '&#9632;'
			};
			this.stateValues = { 
				"False": false,
				"True": "on", 
				"Mixed": "mixed"
			};
		},
		
		_fillContent: function(/*DomNode*/ source){
			// Override Button::_fillContent() since it doesn't make sense for CheckBox,
			// since CheckBox doesn't even have a container
		},
		
		postCreate: function(){
			
			this._event = {
				"input" : "onChange",
				"blur" : "_onBlur",
				"focus" : "_onFocus"
			}
			if(this.instantValidate){
				this.connect(this.focusNode, "onfocus", function(){
					if(this.message == "" || this.mouseFocus){
						this.mouseFocus = false;
						return;
					}
				});
			}
			domAttr.set(this.stateLabelNode, 'innerHTML', this._stateLabels[this._stateType]);
			this.inherited(arguments);
		},
		
		startup: function(){
			this.set("checked", this.params.checked || this.states[this._currentState]);
			domAttr.set(this.stateLabelNode, 'innerHTML', this._stateLabels[this._stateType]);
			this.inherited(arguments);
		},
		
		_isEmpty: function(){
			return !this.get("checked");
		},
		
		_setCheckedAttr: function(/*String|Boolean*/ checked, /*Boolean?*/ priorityChange){
			// summary:
			//		Handler for checked = attribute to constructor, and also calls to
			//		set('checked', val).
			// checked:
			//		true, false or 'mixed'
			// description:
			//		Controls the state of the TriStateCheckBox. Set this.checked, 
			//		this._currentState, value attribute of the <input type="checkbox">
			//		according to the value of 'checked'.
			var stateIndex = array.indexOf(this.states, checked), changed = false;
			if(stateIndex >= 0){
				this._currentState = stateIndex;
				if(checked != this.get("checked")){
					changed = true;
				}
				this._set("checked", checked);
				this._stateType = this._getStateType(checked);
				if(checked == "mixed"){
					domAttr.set(this.focusNode || this.domNode, "checked", true);
				}else{
					domAttr.set(this.focusNode || this.domNode, "checked", checked);
				}
				domAttr.set(this.focusNode, "value", this.stateValues[this._stateType]);
				domAttr.set(this.stateLabelNode, 'innerHTML', this._stateLabels[this._stateType]);
				(this.focusNode || this.domNode).setAttribute("aria-checked", checked);
				if(changed){
					this._handleOnChange(checked, priorityChange);
				}
			}else{
				console.warn("Invalid state!");
			}
		},
		
		setChecked: function(/*String|Boolean*/ checked){
			// summary:
			//		Deprecated.  Use set('checked', true/false) instead.
			kernel.deprecated("setChecked(" + checked + ") is deprecated. Use set('checked'," + checked + ") instead.", "", "2.0");
			this.set('checked', checked);
		},
		
		_setStatesAttr: function(/*Array|String*/ states){
			if(lang.isArray(states)){
				this._set("states", states);
			}else if(lang.isString(states)){
				var map = {
					"true": true,
					"false": false,
					"mixed": "mixed"
				};
				states = states.split(/\s*,\s*/);
				for(var i = 0; i < states.length; i++){
					states[i] = map[states[i]] !== undefined ? map[states[i]] : false;
				}
				this._set("states", states);
			}
		},
		
		_setReadOnlyAttr: function(/*Boolean*/ value){
			this._set("readOnly", value);
			domAttr.set(this.focusNode, "readOnly", value);
		},
		
		_setValueAttr: function(/*String|Boolean*/ newValue, /*Boolean*/ priorityChange){
			// summary:
			//		Handler for value = attribute to constructor, and also calls to
			//		set('value', val).
			// description:
			//		During initialization, just saves as attribute to the <input type=checkbox>.
			//
			//		After initialization,
			//		when passed a boolean or the string 'mixed', controls the state of the
			//		TriStateCheckBox.
			//		If passed a string except 'mixed', changes the value attribute of the
			//		TriStateCheckBox. Sets the state of the TriStateCheckBox to checked.
			if(typeof newValue == "string" && (array.indexOf(this.states, newValue) < 0)){
				if(newValue == ""){
					newValue = "on";
				}
				this.stateValues["True"] = newValue;
				newValue = true;
			}
			if(this._created){
				this._currentState = array.indexOf(this.states, newValue);
				this.set('checked', newValue, priorityChange);
				domAttr.set(this.focusNode, "value", this.stateValues[this._stateType]);
			}
		},
		
		_setValuesAttr: function(/*Array*/ newValues){
			// summary:
			//		Handler for values = attribute to constructor, and also calls to
			//		set('values', val).
			// newValues:
			//		If the length of newValues is 1, it will replace the value of
			//		the TriStateCheckBox in true state. Otherwise, the values of
			//		the TriStateCheckBox in true state and 'mixed' state will be
			//		replaced by the first two values in newValues.
			// description:
			//		Change the value of the TriStateCheckBox in 'mixed' and true states.
			this.stateValues["True"] = newValues[0] ? newValues[0] : this.stateValues["True"];
			this.stateValues["Mixed"] = newValues[1] ? newValues[1] : this.stateValues["Mixed"];
		},
		
		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		Returns value according to current state of the TriStateCheckBox.
			return this.stateValues[this._stateType];
		},
		
		reset: function(){
			this._hasBeenBlurred = false;
			this.set("states", this.params.states || [false, "mixed", true]);
			this.stateValues = this.params.stateValues || {
				"False" : false,
				"True" : "on",
				"Mixed" : "mixed"
			};
			this.set("values", this.params.values || []);
			this.set('checked', this.params.checked || this.states[0]);
		},
		
		_onFocus: function(){
			if(this.id){
				query("label[for='" + this.id + "']").addClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},
		
		_onBlur: function(){
			if(this.id){
				query("label[for='" + this.id + "']").removeClass("dijitFocusedLabel");
			}
			this.mouseFocus = false;
			this.inherited(arguments);
		},
		
		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions - need to check
			//		readOnly and disabled
			if(this.readOnly || this.disabled){
				event.stop(e);
				return false;
			}
			this.click();
			return this.onClick(e); // user click actions
		},
		
		click: function(){
			// summary:
			//		Emulate a click on the check box, but will not trigger the 
			//		onClick method.
			if(this._currentState >= this.states.length - 1){
				this._currentState = 0;
			}else{
				if(this._currentState == -1){
					this.fixState();
				}else{
					this._currentState++;
				}
			}
			var oldState = this._currentState;
			this.set("checked", this.states[this._currentState]);
			this._currentState = oldState;
			domAttr.set(this.stateLabelNode, 'innerHTML', this._stateLabels[this._stateType]);
		},
		
		fixState: function(){
			// summary:
			//		Fix _currentState property if it's out of bound.
			this._currentState = this.states.length - 1;
		},
		
		_getStateType: function(/*String|Boolean*/ state){
			//	summary:
			//		Internal function to return the type of a certain state
			//		false: False
			//		true: True
			//		"mixed": Mixed
			return state ? (state == "mixed" ? "Mixed" : "True") : "False";
		},
		
		_onMouseDown: function(){
			this.mouseFocus = true;
		},
		
		_setDisabledAttr: function(){
			this.inherited(arguments);
			this._refreshState();
		},
		
		_setLabelAlignmentAttr: null,
		_setFieldWidthAttr: null,
		_setLabelWidthAttr: null,
		resize:function(){return false;},
		_setIconClassAttr: null
	});
	
	
	if (has("dojo-bidi")) {
		baseClassName = baseClassName.substring(0, baseClassName.length-1);
		var baseTriStateCheckBox = iForm.TriStateCheckBox;
		iForm.TriStateCheckBox = declare(baseClassName,[baseTriStateCheckBox]);
	}
	
	if ( has("mobile") || has("platform-plugable")) {
	
		var pluginRegistry = PlatformPluginRegistry.register("idx/form/TriStateCheckBox", 
				{	
					desktop: "inherited",	// no plugin for desktop, use inherited methods  
				 	mobile: MobilePlugin	// use the mobile plugin if loaded
				});

		iForm.TriStateCheckBox = declare("idx.form.TriStateCheckBox",[iForm.TriStateCheckBox, TemplatePlugableMixin], {
			/**
		     * Set the template path for the desktop template in case the template was not 
		     * loaded initially, but is later needed due to an instance being constructed 
		     * with "desktop" platform.
	     	 */
			templatePath: require.toUrl("idx/form/templates/TriStateCheckBox.html"), 

			// set the plugin registry
			pluginRegistry: pluginRegistry,
			/**
			 * Stub Plugable method
			 */
			postCreate: function(){
				var promise = this.doWithPlatformPlugin(arguments, "postCreate", "postCreate");
				this.inherited(arguments);
				return promise;
			},	 			
			/**
			 * 
			 * @param {Object} message
			 */
			displayMessage: function(message){
				return this.doWithPlatformPlugin(arguments, "displayMessage", "displayMessage", message);
			},
			/**
			 * 
			 * @param {Object} helpText
			 */
			_setHelpAttr: function(helpText){
				return this.doWithPlatformPlugin(arguments, "_setHelpAttr", "setHelpAttr", helpText);
			},
			/**
			 * 
			 * @param {Object} value
			 */
			_setCheckedAttr: function(value){
				this.inherited(arguments);
				this.focusNode.setAttribute("value", this.stateValues[this._stateType]);
				
			}
		});
	}
	
	return iForm.TriStateCheckBox;
});

},
'dojo/date/locale':function(){
define([
	"../_base/lang",
	"../_base/array",
	"../date",
	/*===== "../_base/declare", =====*/
	"../cldr/supplemental",
	"../i18n",
	"../regexp",
	"../string",
	"../i18n!../cldr/nls/gregorian",
	"module"
], function(lang, array, date, /*===== declare, =====*/ supplemental, i18n, regexp, string, gregorian, module){

// module:
//		dojo/date/locale

var exports = {
	// summary:
	//		This modules defines dojo/date/locale, localization methods for Date.
};
lang.setObject(module.id.replace(/\//g, "."), exports);

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.

// Load the bundles containing localization information for
// names and formats

//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

	// Format a pattern without literals
	function formatPattern(dateObject, bundle, options, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad,
				c = match.charAt(0),
				l = match.length,
				widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							if(!options.fullYear){
								s = String(s); s = s.substr(s.length - 2);
								break;
							}
							// fallthrough
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
				case 'L':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = [
							"months",
							c == 'L' ? "standAlone" : "format",
							widthList[l-3]
						].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = exports._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = exports._getDayOfYear(dateObject); pad = true;
					break;
				case 'e':
				case 'c':
					var d = dateObject.getDay();
					if(l<2){
						s = (d - supplemental.getFirstDayOfWeek(options.locale) + 8) % 7
						break;
					}
					// fallthrough
				case 'E':
					d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = [
							"days",
							c == 'c' ? "standAlone" : "format",
							widthList[l-3]
						].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = dateObject.getHours() < 12 ? 'am' : 'pm';
					s = options[timePeriod] || bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = exports._getZone(dateObject, true, options);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = exports._getZone(dateObject, false, options);
					var tz = [
						(offset<=0 ? "+" : "-"),
						string.pad(Math.floor(Math.abs(offset)/60), 2),
						string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A':
//					console.log(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = string.pad(s, l); }
			return s;
		});
	}

/*=====
var __FormatOptions = exports.__FormatOptions = declare(null, {
	// selector: String
	//		choice of 'time','date' (default: date and time)
	// formatLength: String
	//		choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	// datePattern:String
	//		override pattern with this string
	// timePattern:String
	//		override pattern with this string
	// am: String
	//		override strings for am in times
	// pm: String
	//		override strings for pm in times
	// locale: String
	//		override the locale used to determine formatting rules
	// fullYear: Boolean
	//		(format only) use 4 digit years whenever 2 digit years are called for
	// strict: Boolean
	//		(parse only) strict parsing, off by default
});
=====*/

exports._getZone = function(/*Date*/ dateObject, /*boolean*/ getName, /*__FormatOptions?*/ options){
	// summary:
	//		Returns the zone (or offset) for the given date and options.  This
	//		is broken out into a separate function so that it can be overridden
	//		by timezone-aware code.
	//
	// dateObject:
	//		the date and/or time being formatted.
	//
	// getName:
	//		Whether to return the timezone string (if true), or the offset (if false)
	//
	// options:
	//		The options being used for formatting
	if(getName){
		return date.getTimezoneName(dateObject);
	}else{
		return dateObject.getTimezoneOffset();
	}
};


exports.format = function(/*Date*/ dateObject, /*__FormatOptions?*/ options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo/date/locale.addCustomFormats() method.
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = i18n.normalizeLocale(options.locale),
		formatLength = options.formatLength || 'short',
		bundle = exports._getGregorianBundle(locale),
		str = [],
		sauce = lang.hitch(this, formatPattern, dateObject, bundle, options);
	if(options.selector == "year"){
		return _processPattern(bundle["dateFormatItem-yyyy"] || "yyyy", sauce);
	}
	var pattern;
	if(options.selector != "date"){
		pattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}
	if(options.selector != "time"){
		pattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}

	return str.length == 1 ? str[0] : bundle["dateTimeFormat-"+formatLength].replace(/\'/g,'').replace(/\{(\d+)\}/g,
		function(match, key){ return str[key]; }); // String
};

exports.regexp = function(/*__FormatOptions?*/ options){
	// summary:
	//		Builds the regular needed to parse a localized date

	return exports._parseInfo(options).regexp; // String
};

exports._parseInfo = function(/*__FormatOptions?*/ options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = exports._getGregorianBundle(locale),
		formatLength = options.formatLength || 'short',
		datePattern = options.datePattern || bundle["dateFormat-" + formatLength],
		timePattern = options.timePattern || bundle["timeFormat-" + formatLength],
		pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
			function(match, key){ return [timePattern, datePattern][key]; });
	}

	var tokens = [],
		re = _processPattern(pattern, lang.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

exports.parse = function(/*String*/ value, /*__FormatOptions?*/ options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo/date/locale.addCustomFormats() method.
	//
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//		When two digit years are used, a century is chosen according to a sliding
	//		window of 80 years before and 20 years after present year, for both `yy` and `yyyy` patterns.
	//		year < 100CE requires strict mode.
	//
	// value:
	//		A string representation of a date

	// remove non-printing bidi control chars from input and pattern
	var controlChars = /[\u200E\u200F\u202A\u202E]/g,
		info = exports._parseInfo(options),
		tokens = info.tokens, bundle = info.bundle,
		re = new RegExp("^" + info.regexp.replace(controlChars, "") + "$",
			info.strict ? "" : "i"),
		match = re.exec(value && value.replace(controlChars, ""));

	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'],
		result = [1970,0,1,0,0,0,0], // will get converted to a Date at the end
		amPm = "",
		valid = array.every(match, function(v, i){
		if(!i){return true;}
		var token = tokens[i-1],
			l = token.length,
			c = token.charAt(0);
		switch(c){
			case 'y':
				if(l != 2 && options.strict){
					//interpret year literally, so '5' would be 5 A.D.
					result[0] = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear(),
							century = year.substring(0, 2) * 100,
							cutoff = Math.min(Number(year.substring(2, 4)) + 20, 99);
						result[0] = (v < cutoff) ? century + v : century - 100 + v;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return false;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result[0] = v;
					}
				}
				break;
			case 'M':
			case 'L':
				if(l>2){
					var months = bundle['months-' +
							    (c == 'L' ? 'standAlone' : 'format') +
							    '-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = array.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = array.indexOf(months, v);
					if(v == -1){
//						console.log("dojo/date/locale.parse: Could not parse month name: '" + v + "'.");
						return false;
					}
				}else{
					v--;
				}
				result[1] = v;
				break;
			case 'E':
			case 'e':
			case 'c':
				var days = bundle['days-' +
						  (c == 'c' ? 'standAlone' : 'format') +
						  '-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = array.map(days, function(d){return d.toLowerCase();});
				}
				v = array.indexOf(days, v);
				if(v == -1){
//					console.log("dojo/date/locale.parse: Could not parse weekday name: '" + v + "'.");
					return false;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				result[2] = v;
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.log("dojo/date/locale.parse: Could not parse am/pm part.");
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.log("dojo/date/locale.parse: Illegal hours value");
					return false;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = v;
				break;
			case 'm': //minutes
				result[4] = v;
				break;
			case 's': //seconds
				result[5] = v;
				break;
			case 'S': //milliseconds
				result[6] = v;
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.log("dojo/date/locale.parse: unsupported pattern char=" + token.charAt(0));
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}

	//TODO: implement a getWeekday() method in order to test
	//validity of input strings containing 'EEE' or 'EEEE'...

	var dateObject = new Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // Date
	if(options.strict){
		dateObject.setFullYear(result[0]);
	}

	// Check for overflow.  The Date() constructor normalizes things like April 32nd...
	//TODO: why isn't this done for times as well?
	var allTokens = tokens.join(""),
		dateToken = allTokens.indexOf('d') != -1,
		monthToken = allTokens.indexOf('M') != -1;

	if(!valid ||
		(monthToken && dateObject.getMonth() > result[1]) ||
		(dateToken && dateObject.getDate() > result[2])){
		return null;
	}

	// Check for underflow, due to DST shifts.  See #9366
	// This assumes a 1 hour dst shift correction at midnight
	// We could compare the timezone offset after the shift and add the difference instead.
	if((monthToken && dateObject.getMonth() < result[1]) ||
		(dateToken && dateObject.getDate() < result[2])){
		dateObject = date.add(dateObject, "hour", 1);
	}

	return dateObject; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g),
		literal = pattern.charAt(0) == "'";

	array.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk.replace(/''/g, "'"));
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s,
			c = match.charAt(0),
			l = match.length,
			p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
			case 'L':
				s = (l>2) ? '\\S+?' : '1[0-2]|'+p2+'[1-9]';
				break;
			case 'D':
				s = '[12][0-9][0-9]|3[0-5][0-9]|36[0-6]|'+p2+'[1-9][0-9]|'+p3+'[1-9]';
				break;
			case 'd':
				s = '3[01]|[12]\\d|'+p2+'[1-9]';
				break;
			case 'w':
				s = '[1-4][0-9]|5[0-3]|'+p2+'[1-9]';
				break;
			case 'E':
			case 'e':
			case 'c':
				s = '.+?'; // match anything including spaces until the first pattern delimiter is found such as a comma or space
				break;
			case 'h': //hour (1-12)
				s = '1[0-2]|'+p2+'[1-9]';
				break;
			case 'k': //hour (0-11)
				s = '1[01]|'+p2+'\\d';
				break;
			case 'H': //hour (0-23)
				s = '1\\d|2[0-3]|'+p2+'\\d';
				break;
			case 'K': //hour (1-24)
				s = '1\\d|2[0-4]|'+p2+'[1-9]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					s = am + '|' + pm;
				if(!options.strict){
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					if(s.indexOf('.') != -1){ s += '|' + s.replace(/\./g, ""); }
				}
				s = s.replace(/\./g, "\\.");
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.log("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}

var _customFormats = [];
exports.addCustomFormats = function(/*String*/ packageName, /*String*/ bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo/date/locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

exports._getGregorianBundle = function(/*String*/ locale){
	var gregorian = {};
	array.forEach(_customFormats, function(desc){
		var bundle = i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = lang.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};

exports.addCustomFormats(module.id.replace(/\/date\/locale$/, ".cldr"),"gregorian");

exports.getNames = function(/*String*/ item, /*String*/ type, /*String?*/ context, /*String?*/ locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'abbr' || 'narrow' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// context:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names

	var label,
		lookup = exports._getGregorianBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

exports.isWeekend = function(/*Date?*/ dateObject, /*String?*/ locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = supplemental.getWeekend(locale),
		day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

exports._getDayOfYear = function(/*Date*/ dateObject){
	// summary:
	//		gets the day of the year as represented by dateObject
	return date.difference(new Date(dateObject.getFullYear(), 0, 1, dateObject.getHours()), dateObject) + 1; // Number
};

exports._getWeekOfYear = function(/*Date*/ dateObject, /*Number*/ firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay(),
		adj = (firstDayOfYear - firstDayOfWeek + 7) % 7,
		week = Math.floor((exports._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

return exports;
});

},
'dijit/ToolbarSeparator':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_Widget",
	"./_TemplatedMixin"
], function(declare, dom, _Widget, _TemplatedMixin){

	// module:
	//		dijit/ToolbarSeparator


	return declare("dijit.ToolbarSeparator", [_Widget, _TemplatedMixin], {
		// summary:
		//		A spacer between two `dijit.Toolbar` items

		templateString: '<div class="dijitToolbarSeparator dijitInline" role="presentation"></div>',

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		This widget isn't focusable, so pass along that fact.
			// tags:
			//		protected
			return false;
		}
	});
});

},
'dijit/form/RadioButton':function(){
define([
	"dojo/_base/declare", // declare
	"./CheckBox",
	"./_RadioButtonMixin"
], function(declare, CheckBox, _RadioButtonMixin){

	// module:
	//		dijit/form/RadioButton

	return declare("dijit.form.RadioButton", [CheckBox, _RadioButtonMixin], {
		// summary:
		//		Same as an HTML radio, but with fancy styling.

		baseClass: "dijitRadio"
	});
});

},
'dijit/MenuSeparator':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_WidgetBase",
	"./_TemplatedMixin",
	"./_Contained",
	"dojo/text!./templates/MenuSeparator.html"
], function(declare, dom, _WidgetBase, _TemplatedMixin, _Contained, template){

	// module:
	//		dijit/MenuSeparator

	return declare("dijit.MenuSeparator", [_WidgetBase, _TemplatedMixin, _Contained], {
		// summary:
		//		A line between two menu items

		templateString: template,

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});
});

},
'dijit/form/ToggleButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"./Button",
	"./_ToggleButtonMixin"
], function(declare, kernel, Button, _ToggleButtonMixin){

	// module:
	//		dijit/form/ToggleButton


	return declare("dijit.form.ToggleButton", [Button, _ToggleButtonMixin], {
		// summary:
		//		A templated button widget that can be in two states (checked or not).
		//		Can be base class for things like tabs or checkbox or radio buttons.

		baseClass: "dijitToggleButton",

		setChecked: function(/*Boolean*/ checked){
			// summary:
			//		Deprecated.  Use set('checked', true/false) instead.
			kernel.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
			this.set('checked', checked);
		}
	});
});

},
'xcitec/grid/_HGridHeaderCell':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/dnd/Source",
	"dojo/dom-geometry",
	"dojo/has",
	"dojo/on",
	"dojo/touch",
	"dojo/query",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "../Menu",
  "../MenuItem"
], function (dojo, domStyle, declare, lang, windowModule, array, domConstruct, domClass, Manager, domAttr, Source, domGeometry, has, on, touch, query, topic, _Container,
             _Templated, _Widget, registry, Menu, MenuItem) {
  return 	declare(_Widget, {

		startPos: 0,
		endPos: 0,
		blockOnClick: false,
		attached: false,
		columnId: "",
		presentmentType: "",
		showDefaultMenu: false,
		showXcitecMenu: true,
		onClickAction: null,
		onClickAttr: null,
		dragging: false,
		deleteIcon: null,
		cellWidth: 0,
		fixedPos: false,
		_addedEvents: [],

		buildRendering: function(){

			this.domNode = this.srcNodeRef;
			var outerDiv = document.createElement("div");
			domClass.add(outerDiv, "wrap");
			var innerDiv = document.createElement("div");
			domClass.add(innerDiv, "inner_head");
			var columnSorting = domAttr.get(this.domNode, "sort"); // column sorting (ASC or DESC)
			var canDeleteColumn = domAttr.get(this.domNode, "canDelete");
			var columnWidth = domAttr.get(this.domNode, "columnWidth"); // width of column if set
			this.onClickAttr = domAttr.get(this.domNode, "onclick");
			this.columnId = domAttr.get(this.domNode, "columnId");
			this.presentmentType = domAttr.get(this.domNode, "presentmentType");
			innerDiv.innerHTML = this.domNode.innerHTML;
			this.domNode.innerHTML = "";
			// if "sort" attribute is set - create div for sorting
			if(columnSorting){
				var sortDiv = document.createElement("div");
				switch(columnSorting){
					case "ASC":
						domClass.add(sortDiv, "sort_arrow arrow_up");
						break;
					case "DESC":
						domClass.add(sortDiv, "sort_arrow arrow_down");
						break;
					case "asc":
						domClass.add(sortDiv, "sort_arrow arrow_up");
						break;
					case "desc":
						domClass.add(sortDiv, "sort_arrow arrow_down");
						break;
					case "none":
						this.showXcitecMenu = false;
						break;
					default:
						break;

				}
				on(this.domNode, "mouseover", function(){
					if((columnSorting == "ASC") || (columnSorting == "asc")){
						domClass.remove(sortDiv, "arrow_up");
						domClass.add(sortDiv, "arrow_down_active");
					} else if((columnSorting == "DESC") || (columnSorting == "desc")){
						domClass.remove(sortDiv, "arrow_down");
						domClass.add(sortDiv, "arrow_up_active");
					}
				});
				on(this.domNode, "mouseout", function(){
					if((columnSorting == "ASC") || (columnSorting == "asc")){
						domClass.remove(sortDiv, "arrow_down_active");
						domClass.add(sortDiv, "arrow_up");
					} else if((columnSorting == "DESC") || (columnSorting == "desc")){
						domClass.remove(sortDiv, "arrow_up_active");
						domClass.add(sortDiv, "arrow_down");
					}
				});
				outerDiv.appendChild(sortDiv);
				this.domNode.appendChild(outerDiv);
			}
			var columnWidths = this.cellContainer.parentTable.columnWidths;
			if(this.cellContainer.isFrozenHeader){
				columnWidths = this.cellContainer.parentTable.columnFrozenWidths;
			}
			// if "columnWidth" attribute is set - assign width for proper column and save it in global array
			if(columnWidth){
				domStyle.set(this.domNode, "width", columnWidth + "px");
				columnWidths[this.cellIndex] = Number(columnWidth);
				this.cellWidth = Number(columnWidth);
			} else{
				columnWidths[this.cellIndex] = null;
				this.cellWidth = null;
			}

			if(canDeleteColumn){
				this.deleteIcon = document.createElement("div");
				domClass.add(this.deleteIcon, "close");
				var wrapAll = domConstruct.create("div");
				domClass.add(wrapAll, "wrapper_all");
				wrapAll.appendChild(this.deleteIcon);
				outerDiv.appendChild(innerDiv);
				wrapAll.appendChild(outerDiv);
				this.domNode.appendChild(wrapAll);
				on(this.deleteIcon, "click", lang.hitch(this, this.deleteColumn));
			} else{
				outerDiv.appendChild(innerDiv);
				this.domNode.appendChild(outerDiv);
			}
			if(this.cellIndex > 0){
				domClass.add(this.domNode, "separator");
			}
		},

		postCreate: function(){
			var _this = this;
			if(this.cellContainer.parentTable.gridActions){
				// get xcitec.grid.Action objects
				var acts = this.cellContainer.parentTable.gridActions.getChildren();
				if((acts.length > 0) && (this.showXcitecMenu)){
					var columnMenu = new Menu();
					columnMenu.bindDomNode(this.domNode);
					array.forEach(acts, function(action){
						if((action.getActionId() == "COL_SORT_ASC") || (action.getActionId() == "COL_SORT_DESC")){
							columnMenu.addChild(new MenuItem({label: action.getActionName(), onClick: function(){
								// update activeColumn hidden input with columnId
								if(_this.cellContainer.parentTable.gridControl && _this.cellContainer.parentTable.gridControl.activeColumn){
									domAttr.set(_this.cellContainer.parentTable.gridControl.activeColumn, "value", _this.columnId);
								}
								action.actionEvent();
							} }));
						}
						if(action.getActionId() == "COL_SORT_SET"){
							_this.onClickAction = action;
						}
					});
					columnMenu.startup();
					this.showXcitecMenu = true;
				}
			} else{
				this.showXcitecMenu = false;
			}

			if(!this.onClickAttr){
				this.connect(this.domNode, "onclick", this.onClick);
			}

			this.saveOnClick = this.domNode.onclick;
			this.domNode.onclick = lang.hitch(this, this.vkOnClick);
			// dojo/on doesn't work here for some reason, so replaced with dojo/touch
			touch.press(this.domNode, lang.hitch(this, "_onMouseDown"));
			touch.move(this.domNode, lang.hitch(this, "_onMouseMove"));
			touch.release(this.domNode, lang.hitch(this, "_onMouseUp"));
			if(has("ie")){
				if(this.cellContainer.parentTable.gridResize){
					on(this.domNode, "mouseup", function(){
						_this.cellContainer.parentTable.dataTableHead.scrollLeft = _this.cellContainer.parentTable.dataTableBody.scrollLeft;

					});
				}
			}
			if(this.presentmentType == "number"){
				this.cellContainer.cellToRightAlign = this.cellContainer.cellToRightAlign.concat(this);
			}
		},

		onClick: function(action){
			if(!this.dragging){
				if(this.onClickAction && (!this.isInResizingZone(action))){
					if(this.cellContainer.parentTable.gridControl && this.cellContainer.parentTable.gridControl.activeColumn){
						domAttr.set(this.cellContainer.parentTable.gridControl.activeColumn, "value", this.columnId);
					}
					this.onClickAction.actionEvent();
				}
			}
		},

		vkOnClick: function(evt){
			if(!this.dragging){
				if(has("ff") || has("chrome") || has('trident') == 7){
					if(!this.isInResizingZone(evt)){
						if(this.saveOnClick){
							this.saveOnClick(evt);
						}
					}
				} else{
					if(!this.blockOnClick){
						if(this.saveOnClick){
							this.saveOnClick(evt);
						}
					}
				}
			}
		},

		getCellsWidths: function(){
			var _this = this;
			this.cellContainer.cellsWidth = [];
			array.forEach(this.cellContainer.allCells, function(cell){
				_this.cellContainer.cellsWidth[cell.cellIndex] = domGeometry.getMarginBox(cell.domNode).w;
			});
		},

		// is cursor in the zone where resizing can be
		isInResizingZone: function(e){
			var cellOffset =  Math.round(e.pageX - domGeometry.position(this.domNode).x);
			var cellWidth = domGeometry.getMarginBox(this.domNode).w;
			if(this.cellIndex == 0){
				if(cellOffset > (cellWidth - 4)){
					return true;
				}
			} else if(this.cellIndex == this.cellContainer.allCells.length - 1){
				if(cellOffset < 4){
					return true;
				}
			} else{
				if((cellOffset < 4)){
					return true;
				} else if(cellOffset > (cellWidth - 4)){
					return true;
				}
			}
			return false;
		},

		// change the cursor style from default to resize and back
		changeCursor: function(e){
			var cellOffset =  Math.round(e.pageX - domGeometry.position(this.domNode).x);
			var cellWidth = domGeometry.getMarginBox(this.domNode).w;
			if(this.cellIndex == 0){ // first column
				if(cellOffset > (cellWidth - 4)){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else{
					domStyle.set(this.domNode, "cursor", "default");
				}
			} else if(this.cellIndex == this.cellContainer.allCells.length - 1){ // last column
				if(cellOffset < 4){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else{
					domStyle.set(this.domNode, "cursor", "default");
				}
			} else{ // middle columns
				if((cellOffset < 4)){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else if(cellOffset > (cellWidth - 4)){
					domStyle.set(this.domNode, "cursor", "e-resize");
				} else{
					domStyle.set(this.domNode, "cursor", "default");
				}
			}
		},

		_onMouseDown: function(e){
			if(this.cellContainer.parentTable.gridResize){
				var cellOffset =  Math.round(e.pageX - domGeometry.position(this.domNode).x);
				var cellWidth = domGeometry.getMarginBox(this.domNode).w;
				this.cellContainer.startPos =  Math.round(e.pageX);
				if(this.cellContainer.blockHeight == 0){
					this.cellContainer.blockHeight = domGeometry.getMarginBox(this.cellContainer.parentTable.dataTableBody).h;
				}
				if(this.cellContainer.blockTableHeight == 0){
					this.cellContainer.blockTableHeight = domGeometry.getMarginBox(this.cellContainer.parentTable.domNode).h;
				}

				this.getCellsWidths();
				if(this.isInResizingZone(e)){
					e.preventDefault(), e.stopPropagation();
					if(this.cellContainer.isFrozenHeader){
						var _this = this;
						this.cellContainer.parentTable.headWidget.allCells[0].getCellsWidths();
						this.cellContainer.parentTable.headWidget.tableWidth = 2; //frozen delimiter's width
						array.forEach(this.cellContainer.parentTable.headWidget.cellsWidth, function(value){
							if(value){
								_this.cellContainer.parentTable.headWidget.tableWidth = _this.cellContainer.parentTable.headWidget.tableWidth + value;
								if(has("ff")){
									_this.cellContainer.parentTable.headWidget.tableWidth += 2;
								}
							}
						});
						var headerWidth = this.cellContainer.parentTable.vertScroll ?
							this.cellContainer.parentTable.headWidget.tableWidth + 17 :
							this.cellContainer.parentTable.headWidget.tableWidth;
						domStyle.set(this.cellContainer.parentTable.headWidget.headMinWidth, "width", headerWidth + "px");
						domStyle.set(this.cellContainer.parentTable.bodyMinWidth, "width", this.cellContainer.parentTable.headWidget.tableWidth + "px");
					}
					this.blockOnClick = true;
					this.cellContainer.resizingStarted = true;

					if(this.cellIndex == 0){
						this.cellContainer.resizingCell = this;
					} else if(this == this.cellContainer.allCells[this.cellContainer.allCells.length - 1]){
						this.cellContainer.resizingCell = this.cellContainer.allCells[this.cellContainer.allCells.length - 2];
					} else{
						if((cellOffset < 4)){
							var _this = this;
							array.forEach(this.cellContainer.allCells, function(cell, index){
								if(_this == cell){
									_this.cellContainer.resizingCell = _this.cellContainer.allCells[index - 1];
								}
							})
						} else if(cellOffset > (cellWidth - 4)){
							this.cellContainer.resizingCell = this;
						}
					}
					this.cellContainer.originalWidth = this.cellContainer.cellsWidth[this.cellContainer.resizingCell.cellIndex];
					// capturing onmouse events for controlling resize even out of grid
					if(has("ff") || has("chrome") || has('trident') == 7){
						this._addedEvents.push(touch.release(document, lang.hitch(this.cellContainer.resizingCell, this._onMouseUp)));
						this._addedEvents.push(touch.move(document, lang.hitch(this.cellContainer.resizingCell, this._onMouseMove)));
						this.attached = true;
					} else if(has("ie")){
						this.cellContainer.resizingCell.domNode.setCapture();
					}
				} else{
					this.blockOnClick = false;
				}
			}
		},

		// release mouse after resizing
		releaseMouse: function(){
			if(has("ff") || has("chrome") || has('trident') == 7){
				array.forEach(this._addedEvents, function(handle){
					handle.remove();
				});
			} else{
				this.cellContainer.resizingCell.domNode.releaseCapture();
			}
		},

		_onMouseUp: function(){
			if(this.cellContainer.parentTable.source){
				this.dragging = this.cellContainer.parentTable.source.isDragging;
			}
			if(this.cellContainer.parentTable.gridResize){
				if(this.cellContainer.parentTable.source){
					this.dragging = this.cellContainer.parentTable.source.isDragging;
				}
				if(this.cellContainer.resizingStarted){
					this.cellContainer.resizingStarted = false;
					domStyle.set(this.domNode, "cursor", "default");
					this.cellContainer.resizeColumn();
					this.cellContainer.nowResizing = false;
					this.releaseMouse();
					this.cellContainer.tempOffset = -1;
				}
			}
		},

		_onMouseMove: function(e){
			if(this.cellContainer.parentTable.gridResize){
				if(this.cellContainer.resizingStarted){
					this.cellContainer.nowResizing = true;
					this.endPos = Math.round(e.pageX);
					var offset = this.endPos - this.cellContainer.startPos;
					this.cellContainer.resizeHeaderCell(offset, this.endPos);
				} else{
					if(this.cellContainer.parentTable.source){
						this.dragging = this.cellContainer.parentTable.source.isDragging;
					}
					if(!this.dragging){
						this.changeCursor(e);
					} else{
						domStyle.set(this.domNode, "cursor", "default");
					}
				}
			}
		},

		deleteColumn: function(e){
			e.preventDefault(), e.stopPropagation();
			var _this = this;
			var indexToRemove = -1;
			if(this.cellContainer.parentTable.gridControl.columnOrder){
				array.forEach(this.cellContainer.allCells, function(cell, index){
					if(cell.id == _this.id){
						indexToRemove = index;
					}
				});
				if(indexToRemove == 0){
					domClass.remove(this.cellContainer.allCells[1].domNode, "separator");
				}
				var rows = this.cellContainer.parentTable.rows.allRows;
				if(this.cellContainer.isFrozenHeader){
					rows = this.cellContainer.parentTable.frozenRows.allRows;
				}
				array.forEach(rows, function(row){
					var cells = query("td", row.domNode);
					array.forEach(cells, function(cell, index){
						if(index == (indexToRemove + 1)){
							domConstruct.destroy(cell);
						}
					});
					row.allCells = query("td", row.domNode);
					row.allCells = row.allCells.slice(1, row.allCells.length - 1);
				});
				domConstruct.destroy(this.domNode);
				this.updateCellContainer();
				this.cellContainer.parentTable.updateOrder();
				array.forEach(this.cellContainer.allCells, function(cell, index){
					if(index != (_this.cellContainer.allCells.length - 1)){
						domStyle.set(cell.domNode, "width", "auto");
					}
				});
				array.forEach(rows[0].allCells, function(cell){
					domStyle.set(cell, "width", "auto");
				});
				this.getCellsWidths();
			}
		},

		updateCellContainer: function(){
			this.cellContainer.allCells = query("td[widgetId]", this.cellContainer.domNode).map(function(cell){
				return registry.byNode(cell);
			});
		}
	});
});

},
'dojo/dom-prop':function(){
define(["exports", "./_base/kernel", "./sniff", "./_base/lang", "./dom", "./dom-style", "./dom-construct", "./_base/connect"],
		function(exports, dojo, has, lang, dom, style, ctr, conn){
	// module:
	//		dojo/dom-prop
	// summary:
	//		This module defines the core dojo DOM properties API.

	// TODOC: summary not showing up in output, see https://github.com/csnover/js-doc-parse/issues/42

	// =============================
	// Element properties Functions
	// =============================

	// helper to connect events
	var _evtHdlrMap = {}, _ctr = 0, _attrId = dojo._scopeName + "attrid";

	exports.names = {
		// properties renamed to avoid clashes with reserved words
		"class": "className",
		"for": "htmlFor",
		// properties written as camelCase
		tabindex: "tabIndex",
		readonly: "readOnly",
		colspan: "colSpan",
		frameborder: "frameBorder",
		rowspan: "rowSpan",
		valuetype: "valueType"
	};

	exports.get = function getProp(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Gets a property on an HTML element.
		// description:
		//		Handles normalized getting of properties on DOM nodes.
		//
		// node: DOMNode|String
		//		id or reference to the element to get the property on
		// name: String
		//		the name of the property to get.
		// returns:
		//		the value of the requested property or its default value
		//
		// example:
		//	|	// get the current value of the "foo" property on a node
		//	|	require(["dojo/dom-prop", "dojo/dom"], function(domProp, dom){
		//	|		domProp.get(dom.byId("nodeId"), "foo");
		//	|		// or we can just pass the id:
		//	|		domProp.get("nodeId", "foo");
		//	|	});

		node = dom.byId(node);
		var lc = name.toLowerCase(), propName = exports.names[lc] || name;
		return node[propName];	// Anything
	};

	exports.set = function setProp(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		// summary:
		//		Sets a property on an HTML element.
		// description:
		//		Handles normalized setting of properties on DOM nodes.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to set the property on
		// name: String|Object
		//		the name of the property to set, or a hash object to set
		//		multiple properties at once.
		// value: String?
		//		The value to set for the property
		// returns:
		//		the DOM node
		//
		// example:
		//	|	// use prop() to set the tab index
		//	|	require(["dojo/dom-prop"], function(domProp){
		//	|		domProp.set("nodeId", "tabIndex", 3);
		//	|	});
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	require(["dojo/dom-prop"], function(domProp){
		//	|		domProp.set("formId", {
		//	|			"foo": "bar",
		//	|			"tabIndex": -1,
		//	|			"method": "POST",
		//	|		});
		//	|	});

		node = dom.byId(node);
		var l = arguments.length;
		if(l == 2 && typeof name != "string"){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				exports.set(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(), propName = exports.names[lc] || name;
		if(propName == "style" && typeof value != "string"){ // inline'd type check
			// special case: setting a style
			style.set(node, value);
			return node; // DomNode
		}
		if(propName == "innerHTML"){
			// special case: assigning HTML
			// the hash lists elements with read-only innerHTML on IE
			if(has("ie") && node.tagName.toLowerCase() in {col: 1, colgroup: 1,
						table: 1, tbody: 1, tfoot: 1, thead: 1, tr: 1, title: 1}){
				ctr.empty(node);
				node.appendChild(ctr.toDom(value, node.ownerDocument));
			}else{
				node[propName] = value;
			}
			return node; // DomNode
		}
		if(lang.isFunction(value)){
			// special case: assigning an event handler
			// clobber if we can
			var attrId = node[_attrId];
			if(!attrId){
				attrId = _ctr++;
				node[_attrId] = attrId;
			}
			if(!_evtHdlrMap[attrId]){
				_evtHdlrMap[attrId] = {};
			}
			var h = _evtHdlrMap[attrId][propName];
			if(h){
				//h.remove();
				conn.disconnect(h);
			}else{
				try{
					delete node[propName];
				}catch(e){}
			}
			// ensure that event objects are normalized, etc.
			if(value){
				//_evtHdlrMap[attrId][propName] = on(node, propName, value);
				_evtHdlrMap[attrId][propName] = conn.connect(node, propName, value);
			}else{
				node[propName] = null;
			}
			return node; // DomNode
		}
		node[propName] = value;
		return node;	// DomNode
	};
});

},
'idx/form/_ValidationMixin':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare", 
	"dojo/dom-style", 
	"dojo/i18n", 
	"dijit/_base/wai", 
	"idx/widget/HoverHelpTooltip",
	"dojo/i18n!dijit/form/nls/validate"
], function(declare, domStyle, i18n, wai, HoverHelpTooltip){
	/**
	 * @public
	 * @name idx.form._ValidationMixin
	 * @class Mix-in class to enable form widget perform validation, implemented according to 
	 * IBM One UI(tm) <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y14&vsub=*&hsub=*&openpanes=0000011100">Validation</a></b>.
	 */
	return declare("idx.form._ValidationMixin", null, 
	/**@lends idx.form._ValidationMixin#*/
	{
		/**
		 * Configurable flag of the validation timing, the widget fires validation when widget get input by setting true, 
		 * fire validation when widget get blur by setting false.
		 * @type boolean
		 * @default false
		 */
		instantValidate: false,
		
		// required: Boolean
		//		Indicate whether this widget must have a value
		/**
		 * Indicate whether this widget must have a value
		 * @type boolean
		 * @default false
		 */
		required: false,
		
		/**
		 * The message to display if value is invalid
		 * @type String
		 */
		invalidMessage: "$_unset_$",
		
		/**
		 * The message to display if value is empty and the field is required
		 * @type String
		 */
		missingMessage: null,
		
		/**
		 * The position of the hoverhelpTooltip
		 * @type String[]
		 */
		tooltipPosition: [], // need to override this class-static value in postMixInProperties
		
		postMixInProperties: function(){
			this.tooltipPosition = ["after-centered", "above"]; // set this to a instance-local value
			this.inherited(arguments);
			var messages = i18n.getLocalization("dijit.form", "validate", this.lang);
			this.missingMessage || (this.missingMessage = messages.missingMessage);
			// check the nls message === null
			this.invalidMessage = (this.invalidMessage == "$_unset_$") ? ( (messages && messages.invalidMessage)? messages.invalidMessage :this.invalidMessage ) : this.invalidMessage;
		},
		buildRendering: function(){
			this.inherited(arguments);
			this.messageTooltip = new HoverHelpTooltip({
				connectId: [this.iconNode],
				label: this.message,
				position: this.tooltipPosition,
				forceFocus: false
			});
		},
		postCreate: function(){
			this.inherited(arguments);
			if(this.instantValidate){
				this.connect(this, this._event["focus"], function(){
					//if (this._hasBeenBlurred && (!this._refocusing)) {
					/* Start validation once instantValidate widget got focus. Fix for #9977 */
					if (!this._refocusing) {
						this.validate(true);
					}
				});
				this.connect(this, this._event["input"], function(){
					this.validate(true);
				});
			}else{
				this.connect(this, this._event["focus"], function(){
					if (this.message && this._hasBeenBlurred && (!this._refocusing)) {
						this.displayMessage(this.message);
					}
				})
			}
		},
		refocus: function() {
			this._refocusing = true;
			this.focus();
			this._refocusing = false;
		},
		
		/**
		 * Tests if value is valid.
		 * Can override with the routine in a subclass.
		 * @public
		 * @param {boolean} isFocused
		 * If the widget focused
		 */
		_isValid: function(/*Boolean*/ isFocused){
			return this.isValid(isFocused) && !(this.required && this._isEmpty());
		},
		
		/**
		 * Checks if value is empty.
		 * Should be override with the routine in a subclass
		 * @public
		 */
		_isEmpty: function(){
			// summary:
			// 		Checks for whitespace. Should be overridden.
			return false;
		},
		
		/**
		 * Extension point for user customizing validation rules.
		 * @param {boolean} isFocused
		 */
		isValid: function(isFocused){
			return true;
		},
		
		/**
		 * Return proper error message according to the error type
		 * @param {boolean} isFocused
		 */
		getErrorMessage: function(/*Boolean*/ isFocused){
			return (this.required && this._isEmpty()) ? this.missingMessage : this.invalidMessage;
		},
		
		/**
		 * Perform validation for the widget, called by "input" event if "instantValidate" setting to true,
		 * called by "blur" event if "instantValidate" setting to false.
		 * @param {boolean} isFocused
		 */
		validate: function(/*Boolean*/ isFocused){
			var message, isValid = this.disabled || this._isValid(isFocused);
			
			this.set("state", isValid ? "" : "Error");
			if(this._hasBeenBlurred){
				wai.setWaiState(this.focusNode, "invalid", !isValid);
			}
			if(this.state == "Error"){
				message = this.getErrorMessage(isFocused);
			}
			this._set("message", message);
			this.displayMessage(message);
			return isValid;
		},
		
		/**
		 * Show error message using a hoverHelpTooltip, hide the tooltip if message is empty.
		 * Overridable method to display validation errors/hints.
		 * By default uses a hoverHelpTooltip.
		 * @protected
		 * @param {string} message
		 * Error message
		 * @param {boolean} force
		 * Force displaying message if the value is true, no matter if the widget got focus or not.
		 */
		displayMessage: function(/*String*/ message, /*Boolean*/ force){
			if(message){
				if(!this.messageTooltip){
					this.messageTooltip = new HoverHelpTooltip({
						connectId: [this.iconNode],
						label: message,
						position: this.tooltipPosition,
						forceFocus: false
					});
				}else{
					this.messageTooltip.set("label", message);
				}
				if(this.focused || force){
					var node = domStyle.get(this.iconNode, "visibility") == "hidden" ? this.oneuiBaseNode : this.iconNode;
					this.messageTooltip.open(node);
				}
			}else{
				this.messageTooltip && this.messageTooltip.close();
			}
		},
		
		_onBlur: function(){
			this.inherited(arguments);
			this.displayMessage("");
		}
	});
});


},
'slmui/layout/_ChapterItem':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/_base/lang',
	'dijit/registry',
	'dijit/_Container',
	'xcitec/layout/_ChapterItem',
	'xcitec/xcitecUtil'
], function(declare, array, lang, registry, _Container, _xChapterItem, xcitecUtil){
	return declare([_xChapterItem], {

		// summary:
		//		Deprecated. Will be used util full migration to IDX. Control's wrapper with label for Chapter.
		// tags:
		//		deprecated

		processAdditionalProperties: function(/*DomNode*/ controlNode){
			// summary:
			//		Collect necessary attrs from control.
			// tags:
			//		extension

			var control = registry.byNode(controlNode);
			this.label = control.label;
			this.labelId = control.labelId;
			this.labelPos = control.labelPos;
			this.labelType = control.labelType || this.labelType;
			this.labelRequired = xcitecUtil.str2bool(control.required);
			this.labelDisabled = xcitecUtil.str2bool(control.disabled);
			this.errorMsg = control.error;
			this.controlView = control.controlView || control.controlview || this.controlView;
			this.labelClickEvent = control.onLabelClick;
			this.widthInPercent = control.widthInPercent;
			if(!this.widthInPercent){
				this.widthInPercent = '100';
			}
			this.inner = xcitecUtil.str2bool(control.inner);
			var parentWidget = registry.getEnclosingWidget(control.domNode.parentNode);
			if(this.inner && parentWidget && parentWidget.widthInPercent){
				this.widthInPercent = parentWidget.widthInPercent;
			}
			if (this.controlView) this.processViewMode();
		},

		resize: function(){
			array.forEach(registry.findWidgets(this.domNode), function(child){
				if(lang.isFunction(child.resize)){
					child.resize();
				}
			});
		}
	});
});

},
'xcitec/widgets/HtmlWidget':function(){
define([
	"dojo/_base/declare",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "../_base/ImageFactory"
], function (declare, query, _Templated, _Widget, ImageFactory) {

	var HtmlWidget = declare( _Widget, {
	  baseClass: 'htmlWidget',
	
	    buildRendering: function(){
	        this.inherited(arguments);
	
	        //this.domNode = dojo.create('div');
	        //this.containerNode = this.domNode;
	        //this._fillContent(this.srcNodeRef);
	
	        //var icons = dojo.query("[icon]", this.domNode);
	        //dojo.forEach(icons, function(node) {
	        query("[icon]", this.domNode).forEach( function(node) {
	            var icon = node.getAttribute("icon");
	            if (icon) {
	                node.className = node.className +" "+ ImageFactory.createIconCss(icon);
	            }
	        });
	    },
	
	    _fillContent: function(/*DomNode*/ source){
	        // summary:
	        //		Relocate source contents to templated container node.
	        //		this.containerNode must be able to receive children, or exceptions will be thrown.
	        // tags:
	        //		protected
	
	        var dest = this.containerNode;
	        if(source && dest){
	            while(source.hasChildNodes()){
	                var child = source.firstChild;
	                dest.appendChild(child);
	                var icon = child.getAttribute("icon");
	                if (icon) {
	                    child.className = ImageFactory.createIconCss("FLAG:DE");
	                }
	            }
	        }
	    }
	
	});
	return HtmlWidget;
});

},
'xcitec/form/EntityDropDownSelect':function(){
define([
	"dojo/_base/declare",
	"dojo/text!./templates/EntityDropDownSelect.html",
	"./DropDownSelect",
    "../xcitecUtil",
    "./_XButtonMiddle",
    "./_XButtonSingleRight"
], function (declare, template, DropDownSelect, xcitecUtil) {

	var EntityDropDownSelect = declare( DropDownSelect, {
	
	    templateString: template,
	
	    infoIconClass: "",
	    entityClass: "",
	    xbtnInfoClass: "xcitec/form/_XButtonMiddle",
	
	    _infoButtonActive: true,
	    disabledChooseAttrSetting: "",
	    disabledInfoAttrSetting: "",
	    readOnlyInfoAttrSetting: "",
	
	    postCreate: function() {
	        this.button = this.chooseButton;
	        this.inherited(arguments);
	        this._checkValue();
	    },
	
	    postMixInProperties: function() {
	        this._infoButtonActive = ("onInfo" in this.params);
	        if (this.entityClass) {
	            this.infoIconClass = "iconXBtn " + xcitecUtil.createEntityIconClass(this.entityClass);
	        }
	        else {
	            this.infoIconClass = "iconXBtn";
	        }
	        if (this.disabled) {
	            this.required = false;
	            this.disabledInfoAttrSetting = "disabled";
	            this.disabledChooseAttrSetting = "disabled";
	        }
	        if (this.readOnly) {
	            this.xbtnInfoClass = "xcitec/form/_XButtonSingleRight";
	            this.disabledChooseAttrSetting = "disabled";
	        }
	        this.inherited(arguments);
	    },
	
	    _onClickInfoButton: function() {
	        if (this._isNullOption()) {
	            return;
	        }
	        this.onInfo();
	    },
	
	    _onClickChooseButton: function() {
	        this._onClick();
	    },
	
	    _selectOption:function() {
	        this.inherited(arguments);
	        this._checkValue();
	    },
	
	    appendOption:function() {
	        this.inherited(arguments);
	        this._checkValue();
	    },
	
	    deleteOption:function() {
	        this.inherited(arguments);
	        this._checkValue();
	    },
	
	
	    _checkValue: function() {
	        if (!this._infoButtonActive || this._isNullOption()) {
	            this.infoButton.setReadOnlyStyle(true);
	        } else {
	            if (this.infoButton.readOnly) {
	                this.infoButton.setReadOnlyStyle(false);
	            }
	        }
	    },
	
	    _isNullOption: function() {
	        if(this.options.length == 0){
	          return true;
	        }
	        for (var i = 0; i < this.options.length; i++) {
	            if (this.options[i].hiddenValue == this.valueNode.value) {
	                if (this.options[i].nullOption == true) {
	                    return this.options[i].nullOption;
	                }
	            }
	        }
	        return false;
	    },
	
	    onInfo: function() {
	
	    }
	
	});
	return EntityDropDownSelect;
});

},
'xcitec/grid/HGridFC':function(){
define([
	"dojo",
	"dojo/on",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/query",
	"dojo/dnd/Manager",
	"dojo/text!xcitec/grid/templates/HGridFC.html",
	"dijit/registry",
	"xcitec/grid/_HGridBase",
  "../xcitecUtil",
  "./_HGridHeader",
  "./_HGridBody",
  "./Avatar"
], function (dojo, on, array, declare, dom, domAttr, domClass, domConstruct, domStyle, query, Manager, template, registry, _HGridBase, xcitecUtil, _HGridHeader, _HGridBody, Avatar) {

	/***
	 * As source expected normal table definition:
	 <table>
	 <thead>
	 <tr><td columnId="c1">ColumnName</td><td columnId="c2">ColumnName</td><td columnId="c3">ColumnName</td>...</tr>
	 </thead>
	 <tbody>
	 <tr rowId="r1"><td>Col1Data</td><td>Col2Data</td><td>Col3Data</td>...</tr>
	 <tr rowId="r3"><td>Col1Data</td><td>Col2Data</td><td>Col3Data</td>...</tr>
	 <tr rowId="r3"><td>Col1Data</td><td>Col2Data</td><td>Col3Data</td>...</tr>
	 </tbody>
	 </table>
	 *
	 */
	var HGridFC = declare( _HGridBase, {
		templateString: template,
	
		frozenRows: null,
		hasFrozenColumn: false,
	
		postMixInProperties: function(){
			this.columnFrozenWidths = [];
			this.inherited(arguments);
		},
	
		postCreate: function(){
			//STEP 1: parse header
			// split frozen columns
			var frozenColumns = domConstruct.create("tr");
			var cells = query("th, td", this.headerNode);
			var frozenIndexes = [];
			array.forEach(cells, function(cellNode, index){
				var _isFrozen = domAttr.get(cellNode, "frozen");
				if(_isFrozen && _isFrozen == "true"){
					frozenColumns.appendChild(cellNode);
					frozenIndexes.push(index);
				}
			});
			if(frozenColumns.hasChildNodes()){
				//console.log("there is frozen columns");
				this.hasFrozenColumn = true;
				this.headFrozenWidget = new _HGridHeader({parentTable:this, isFrozenHeader:true}, frozenColumns);
				var hasNullWidthColumns = false;
				array.forEach(this.headFrozenWidget.columnWidths, function(width){
					if(width == undefined){
						hasNullWidthColumns = true;
					}
				});
				if(!hasNullWidthColumns || (hasNullWidthColumns && domStyle.get(this.frozenBodyMinWidth, "width") > 200)){
					domStyle.set(this.nodeFrozenPartContainer, "width", domStyle.get(this.frozenBodyMinWidth, "width") + "px");
				} else{
					domStyle.set(this.nodeFrozenPartContainer, "width", "200px");
				}
				domConstruct.place(this.headFrozenWidget.domNode, this.nodeFrozenPartContainer, "first");
			}
	
			this.headWidget = new _HGridHeader({parentTable:this}, this.headerNode);
	
			if(this.vertScroll){
				domClass.add(this.domNode, "dataTableFixed");
				domClass.add(this.domNode, "minGridHeight");
			}
	
			//this.head.appendChild(this.headWidget.domNode);
			domConstruct.place(this.headWidget.domNode, this.nodeTableContainer, "first");
			this.dataTableHead = this.headWidget.domNode;
	
			//STEP 2: parse body
	
			// if the grid is empty we don't need to create _HGridBody, we only show message
			if(this.emptyGridMessage){
				domClass.add(this.domNode, "minGridHeight");
				var messageDiv = domConstruct.create("div", {innerHTML: this.emptyGridMessage}, this.dataTableBody, "first");
				domClass.add(messageDiv, "message");
				if(!this.hasFrozenColumn){
					domStyle.set(this.nodeFrozenPartContainer, "display", "none");
				}
			} else{
				if(this.hasFrozenColumn){
					var fRows = query('tr', this.rowsNode);
					if(fRows.length != 0){
						var fRowsNode = domConstruct.create("tbody");
						array.forEach(fRows, function(rowNode){
							var fontColor = domAttr.get(rowNode, 'fontColor'),
								fontClass = "";
							if(fontColor){
								fontClass = xcitecUtil.createTypedStyleClass('fontColor', fontColor);
							}
							var fTableRow = domConstruct.create("tr", {className: fontClass});
							var temp = 0;
							array.forEach(frozenIndexes, function(fIndex){
								fTableRow.appendChild(rowNode.cells[fIndex - temp]);
								temp++;
							});
							fRowsNode.appendChild(fTableRow);
						});
						this.frozenRows = _HGridBody({parentTable:this, isFrozenRows: true}, fRowsNode);
						this.frozenBody.appendChild(this.frozenRows.domNode);
					}
				} else{
					domStyle.set(this.nodeFrozenPartContainer, "display", "none");
				}
	
				var cells = query("tr > td", this.rowsNode);
				if(cells.length != 0){
					if(this.gridControl){
						this.rows = new _HGridBody({parentTable:this, selectedRows:this.gridControl.selectedRowsNode}, this.rowsNode);
					} else{
						this.rows = new _HGridBody({parentTable:this}, this.rowsNode);
					}
					this.body.appendChild(this.rows.domNode);
				}
			}
			// scrolling
			var idHead = this.dataTableHead;
			var idBody = this.dataTableBody;
			var idFrozenBody = this.frozenDataTableBody;
	
			on(idBody, "scroll", function(){
				idHead.scrollLeft = idBody.scrollLeft;
				idFrozenBody.scrollTop = idBody.scrollTop;
			});
	
			if(this.columnReorder){
			Manager.manager().makeAvatar = function(){
					return new Avatar(this);
				};
			}
	
			dom.setSelectable(this.domNode, false);
	
			//control fields are defined outside Table so we need to move them to not lost
			//@todo 1.3 remove this behavior to more logic solution (e.g. to GridControl should care)
			if(this.gridControl && this.gridControl.activeColumn){
				domConstruct.place(this.gridControl.activeColumn, this.domNode, "first");
			}
			if(this.gridControl && this.gridControl.columnOrder){
				domConstruct.place(this.gridControl.columnOrder, this.domNode, "first");
			}
			if(this.gridControl && this.gridControl.columnWidthsNode){
				domConstruct.place(this.gridControl.columnWidthsNode, this.domNode, "first");
			}
	
			// assign bottom Table Buttons -> in case that they can be
			this.linkButtonsWidget = registry.byId(this.id + ".linkButtons");
			this.linkButtonsNode = dom.byId(this.id + ".linkButtons");
			if(this.linkButtonsWidget){
				domConstruct.place(this.linkButtonsWidget.domNode, this.footer, "last");
			} else if(this.linkButtonsNode){
				domConstruct.place(this.linkButtonsNode, this.footer, "last");
			}
			if(this.gridActions){
				domConstruct.place(this.gridActions.domNode, this.domNode, "last");
				domStyle.set(this.gridActions.domNode, "display", "none");
			}
	
			if(this.vertScroll){
				domStyle.set(this.dataTableBody, "overflowY", "scroll");
				var heightWithScroll = domStyle.get(idFrozenBody.children[0], "height") + 20;
				domStyle.set(idFrozenBody.children[0], "height", heightWithScroll + "px");
			}
	
			var _this = this;
			if(this.headWidget.cellToRightAlign.length > 0){
				if(this.rows){
					array.forEach(this.headWidget.allCells, function(cell, index){
						array.forEach(_this.headWidget.cellToRightAlign, function (c){
							if(c == cell){
								_this.rows.makeColumnRightAlign(index);
							}
						});
					});
				}
			}
			this.inherited(arguments);
		}
	});
	return HGridFC;
});

},
'xcitec/page/ReportMenu':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dojo/dom",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/_Container"
], function (array, declare, domStyle, geometry, dom, _Templated, _Widget, _Container) {

	var ReportMenu = declare( [_Widget, _Templated, _Container], {
		templateString: '<div class="reportMenu" data-dojo-attach-point="containerNode"></div>',
		// widths of sub item's caption nodes
		// array of sub item's objects
		subItems: null,
		// max width of sub item's caption nodes
		maxWidth: 50,
	
		postMixInProperties: function() {
	        this.subItems = [];
	        this.inherited(arguments);
	    },
	
		postCreate: function(){
			dom.setSelectable(this.domNode, false);
			this.inherited(arguments);
		},
	
		startup: function(){
			this.inherited(arguments);
			this.subItems = [];
			this.maxWidth = 50;
			var _this = this;
			// collect all subItems, get width of caption and adjust max of them to all subitems
			array.forEach(this.getChildren(), function(child){
				array.forEach(child.getChildren(), function(widget){
					_this.subItems.push(widget);
					var _subItemWidth = geometry.getMarginBox(widget.text).w;
					_this.maxWidth = Math.max(_this.maxWidth, _subItemWidth);
				});
			});
			array.forEach(this.subItems, function(item){
				domStyle.set(item.text, "width", _this.maxWidth + 'px');
			});
			array.forEach(this.getChildren(), function(child){
				if(!child.selected){
					domStyle.set(child.animationNode, "display", "none");
				}
			})
		}
	});

	return ReportMenu;
});

},
'dijit/_DialogMixin':function(){
define([
	"dojo/_base/declare", // declare
	"./a11y"	// _getTabNavigable
], function(declare, a11y){

	// module:
	//		dijit/_DialogMixin

	return declare("dijit._DialogMixin", null, {
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//		Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//		Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit/form/DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//		Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//		Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit/form/DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = a11y._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	});
});

},
'dijit/_Widget':function(){
define([
	"dojo/aspect",	// aspect.around
	"dojo/_base/config",	// config.isDebug
	"dojo/_base/connect",	// connect.connect
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/query",
	"dojo/ready",
	"./registry",	// registry.byNode
	"./_WidgetBase",
	"./_OnDijitClickMixin",
	"./_FocusMixin",
	"dojo/uacss",		// browser sniffing (included for back-compat; subclasses may be using)
	"./hccss"		// high contrast mode sniffing (included to set CSS classes on <body>, module ret value unused)
], function(aspect, config, connect, declare, has, kernel, lang, query, ready,
			registry, _WidgetBase, _OnDijitClickMixin, _FocusMixin){


// module:
//		dijit/_Widget


function connectToDomNode(){
	// summary:
	//		If user connects to a widget method === this function, then they will
	//		instead actually be connecting the equivalent event on this.domNode
}

// Trap dojo.connect() calls to connectToDomNode methods, and redirect to _Widget.on()
function aroundAdvice(originalConnect){
	return function(obj, event, scope, method){
		if(obj && typeof event == "string" && obj[event] == connectToDomNode){
			return obj.on(event.substring(2).toLowerCase(), lang.hitch(scope, method));
		}
		return originalConnect.apply(connect, arguments);
	};
}
aspect.around(connect, "connect", aroundAdvice);
if(kernel.connect){
	aspect.around(kernel, "connect", aroundAdvice);
}

var _Widget = declare("dijit._Widget", [_WidgetBase, _OnDijitClickMixin, _FocusMixin], {
	// summary:
	//		Old base class for widgets.   New widgets should extend `dijit/_WidgetBase` instead
	// description:
	//		Old Base class for Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//
	//		- declaratively/programatically specifying widget initialization parameters like
	//			onMouseMove="foo" that call foo when this.domNode gets a mousemove event
	//		- ondijitclick:
	//			Support new data-dojo-attach-event="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//		- focus related functions:
	//			In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//			dijit/_base/focus.js.
	//		- deprecated methods
	//		- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//
	//		- browser sniffing (putting browser class like `dj_ie` on `<html>` node)
	//		- high contrast mode sniffing (add `dijit_a11y` class to `<body>` if machine is in high contrast mode)


	////////////////// DEFERRED CONNECTS ///////////////////

	onClick: connectToDomNode,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: connectToDomNode,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: connectToDomNode,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: connectToDomNode,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: connectToDomNode,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: connectToDomNode,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: connectToDomNode,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: connectToDomNode,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: connectToDomNode,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: connectToDomNode,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: connectToDomNode,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: connectToDomNode,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	constructor: function(params /*===== ,srcNodeRef =====*/){
		// summary:
		//		Create the widget.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
		//		The hash can contain any of the widget's properties, excluding read-only properties.
		// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified:
		//
		//		- use srcNodeRef.innerHTML as my contents
		//		- if this is a behavioral widget then apply behavior to that srcNodeRef
		//		- otherwise, replace srcNodeRef with my generated DOM tree

		// extract parameters like onMouseMove that should connect directly to this.domNode
		this._toConnect = {};
		for(var name in params){
			if(this[name] === connectToDomNode){
				this._toConnect[name.replace(/^on/, "").toLowerCase()] = params[name];
				delete params[name];
			}
		}
	},

	postCreate: function(){
		this.inherited(arguments);

		// perform connection from this.domNode to user specified handlers (ex: onMouseMove)
		for(var name in this._toConnect){
			this.on(name, this._toConnect[name]);
		}
		delete this._toConnect;
	},

	on: function(/*String|Function*/ type, /*Function*/ func){
		if(this[this._onMap(type)] === connectToDomNode){
			// Use connect.connect() rather than on() to get handling for "onmouseenter" on non-IE,
			// normalization of onkeypress/onkeydown to behave like firefox, etc.
			// Also, need to specify context as "this" rather than the default context of the DOMNode
			// Remove in 2.0.
			return connect.connect(this.domNode, type.toLowerCase(), this, func);
		}
		return this.inherited(arguments);
	},

	_setFocusedAttr: function(val){
		// Remove this method in 2.0 (or sooner), just here to set _focused == focused, for back compat
		// (but since it's a private variable we aren't required to keep supporting it).
		this._focused = val;
		this._set("focused", val);
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		kernel.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		This method is deprecated, use get() or set() directly.
		// name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		// value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// tags:
		//		deprecated

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		kernel.deprecated(this.declaredClass+"::getDescendants() is deprecated. Use getChildren() instead.", "", "2.0");
		return this.containerNode ? query('[widgetId]', this.containerNode).map(registry.byNode) : []; // dijit/_WidgetBase[]
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
		//		`dijit/layout/AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
		//		Called when another widget becomes the selected pane in a
		//		`dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
		//		`dijit/layout/AccordionContainer`, etc.
		//
		//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit/layout/StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

// For back-compat, remove in 2.0.
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/_base"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}
return _Widget;
});

},
'dojo/_base/browser':function(){
if(require.has){
	require.has.add("config-selectorEngine", "acme");
}
define([
	"../ready",
	"./kernel",
	"./connect", // until we decide if connect is going back into non-browser environments
	"./unload",
	"./window",
	"./event",
	"./html",
	"./NodeList",
	"../query",
	"./xhr",
	"./fx"], function(dojo){

	// module:
	//		dojo/_base/browser

	/*=====
	return {
		// summary:
		//		This module causes the browser-only base modules to be loaded.
	};
	=====*/

	return dojo;
});

},
'xcitec/StatusText':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-style",
	"xcitec/Text",
  "./_base/ImageFactory"
], function (declare, domClass, domStyle, Text, ImageFactory) {

	var StatusText = declare( Text, {
	    baseClass: "xcitecStatusBox",
	
	    templateString:
	      '<div class="dijitReset ${baseClass} xcitecStatus">' +
	        '<span class="statusBg" data-dojo-attach-point="bgNode"></span>' +
	        '<img class="${iconClass}" src="${_blankGif}" width="1" height="1" alt="" data-dojo-attach-point="imageNode"/>' +
	        '<span class="textContent" data-dojo-attach-point="textNode">${text}</span>' +
	        '<span class="statusPercent" data-dojo-attach-point="percentNode">${percentText}</span>' +
	      '</div>',
	
	    // number of percents
	    percent: "",
	    percentText: "",
	    // color of background
	    statusBg: "",
	
	    iconClass: "",
	
	    timeStatus: "",
	
	    postMixInProperties: function() {
	      if (this.iconClass) {
	          this.iconClass = ImageFactory.createIconCss(this.iconClass);
	      }
	      if (this.percent) this.percentText=this.percent +" %" 
	      this.inherited(arguments);
	    },
	
	    postCreate: function() {
	      if (this.timeStatus) {
	          domClass.add(this.textNode, "xcitecStatus" + this.timeStatus.charAt(0).toUpperCase() + this.timeStatus.substr(1).toLowerCase());
	      }
	
	      if (this.percent) {
	          domStyle.set(this.bgNode, "width", this.percent + "%")
	      } else {
	          domStyle.set(this.bgNode, "width", "100%")
	      }
	
	      switch (this.statusBg) {
	          case "red": {
	              domClass.add(this.bgNode, "statusRed");
	              break;
	          }
	          case "green": {
	              domClass.add(this.bgNode, "statusGreen");
	              break;
	          }
	          case "yellow": {
	              domClass.add(this.bgNode, "statusYellow");
	              break;
	          }
	          case "grey": {
	              domClass.add(this.bgNode, "statusGrey");
	              break;
	          }
	          case "blue": {
	              domClass.add(this.bgNode, "statusBlue");
	              break;
	          }
	          default:
	              break;
	      }
	
	        this.inherited(arguments);
	    }
	});
	return StatusText;
});
},
'xcitec/DividingLabel':function(){
define(["dojo/_base/declare", "./TrailingLabel"], function(declare, TrailingLabel){
  return declare(TrailingLabel, {
    baseClass: "xcitecDividingLabel"
  });
});

},
'xcitec/_base/ImageFamily':function(){
define([
         "dojo/_base/declare"
       ], function (declare) {
  return declare(null, {
    familyId: "DEFAULT",
    familyCSS: "DEFAULT",
    familyPostfix: "",

    createIconCss: function(/*String*/ iconId){
      //has to be overriden
      return this.familyCSS + this.familyPostfix +
             (!iconId ? "" : " " + this.familyCSS + iconId.charAt(0).toUpperCase() + iconId.substr(1) + this.familyPostfix);
    }
  });
});

},
'dijit/form/SimpleTextarea':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/sniff", // has("ie") has("opera")
	"./TextBox"
], function(declare, domClass, has, TextBox){

	// module:
	//		dijit/form/SimpleTextarea

	return declare("dijit.form.SimpleTextarea", TextBox, {
		// summary:
		//		A simple textarea that degrades, and responds to
		//		minimal LayoutContainer usage, and works with dijit/form/Form.
		//		Doesn't automatically size according to input, like Textarea.
		//
		// example:
		//	|	<textarea data-dojo-type="dijit/form/SimpleTextarea" name="foo" value="bar" rows=30 cols=40></textarea>
		//
		// example:
		//	|	new SimpleTextarea({ rows:20, cols:30 }, "foo");

		baseClass: "dijitTextBox dijitTextArea",

		// rows: Number
		//		The number of rows of text.
		rows: "3",

		// rows: Number
		//		The number of characters per line.
		cols: "20",

		templateString: "<textarea ${!nameAttrSetting} data-dojo-attach-point='focusNode,containerNode,textbox' autocomplete='off'></textarea>",

		postMixInProperties: function(){
			// Copy value from srcNodeRef, unless user specified a value explicitly (or there is no srcNodeRef)
			// TODO: parser will handle this in 2.0
			if(!this.value && this.srcNodeRef){
				this.value = this.srcNodeRef.value;
			}
			this.inherited(arguments);
		},

		buildRendering: function(){
			this.inherited(arguments);
			if(has("ie") && this.cols){ // attribute selectors is not supported in IE6
				domClass.add(this.textbox, "dijitTextAreaCols");
			}
		},

		filter: function(/*String*/ value){
			// Override TextBox.filter to deal with newlines... specifically (IIRC) this is for IE which writes newlines
			// as \r\n instead of just \n
			if(value){
				value = value.replace(/\r/g, "");
			}
			return this.inherited(arguments);
		},

		_onInput: function(/*Event?*/ e){
			// Override TextBox._onInput() to enforce maxLength restriction
			if(this.maxLength){
				var maxLength = parseInt(this.maxLength);
				var value = this.textbox.value.replace(/\r/g, '');
				var overflow = value.length - maxLength;
				if(overflow > 0){
					var textarea = this.textbox;
					if(textarea.selectionStart){
						var pos = textarea.selectionStart;
						var cr = 0;
						if(has("opera")){
							cr = (this.textbox.value.substring(0, pos).match(/\r/g) || []).length;
						}
						this.textbox.value = value.substring(0, pos - overflow - cr) + value.substring(pos - cr);
						textarea.setSelectionRange(pos - overflow, pos - overflow);
					}else if(this.ownerDocument.selection){ //IE
						textarea.focus();
						var range = this.ownerDocument.selection.createRange();
						// delete overflow characters
						range.moveStart("character", -overflow);
						range.text = '';
						// show cursor
						range.select();
					}
				}
			}
			this.inherited(arguments);
		}
	});
});

},
'xcitec/page/SearchPanelV1':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dojo/text!xcitec/page/templates/SearchPanelV1.html",
	"xcitec/layout/XLayout"
], function (dojo, declare, domAttr, domConstruct, query, template, XLayout) {

	/*
	 * SearchPanelV1 control holds old GUI template for filters and grid. Its' main goal is to search in the source
	 * such elements as 'tablelayoutbutton', 'xcitectitle', 'toolbar', 'filterlayoutbutton', 'filters', 'buttons',
	 * 'messagepane' and 'grid' and move them to their appropriate places. These elements is marked by 'xcitecRole' attribute.
	 * All that concerns filters is placed in xcitec.Box with 'xcitecRole'=filterBox, all that concerns grid is placed in
	 * xcitecBox with 'xcitecRole=gridBox'.
	 *
	 * Example:
	 *           <div xcitecRole="filterbox" data-dojo-type="xcitec.Box" caption="Filter" captionComment="( [From] later than [01.01.2009] )" id="box1">
	 *              <button xcitecRole="filterlayoutbutton" data-dojo-type="xcitec.form.InlineDropDownButton"
	 *                   caption="Filter layout" id="d1">
	 *                  ...
	 *              </button>
	 *              <div data-dojo-type="xcitec.layout.FormPanel">
	 *                  <div xcitecRole="filters" data-dojo-type="xcitec.layout.ContentArea" label="">
	 *                       ....
	 *                  </div>
	 *              </div>
	 *           </div>
	 */
	var SearchPanelV1 = declare( XLayout, {
	    templateString: template,
	
	    _fillContent: function() {
	
	        /* searching for filterbox */
	        var filterBox = query("[xcitecRole$=filterbox]", this.srcNodeRef)[0];
	        if (filterBox) {
	
	            filterBox.removeAttribute("data-dojo-type");
	
	            /* searching for filterLayoutButton */
	            var filterLayoutButton = query("[xcitecRole$=layoutbutton]", filterBox)[0];
	            if (filterLayoutButton) {
	                domAttr.set(filterLayoutButton, "data-dojo-type", "xcitec/form/DropDownButton");
	                domConstruct.place(filterLayoutButton, this.filterlayoutbutton, "replace");
	            }
	
	            /* searching for filters */
	            var filters = query("[xcitecRole$=filters]", filterBox)[0];
	            if (filters) {
	                domConstruct.place(filters, this.filters, "replace");
	            }
	
	            /* searching for buttons (Search, Reset) */
	            var buttons = query("[xcitecAttachPoint$=buttons]", filterBox)[0];
	            if (buttons) {
	                domConstruct.place(buttons, this.buttons, "replace");
	            }
	        }
	
	        /* searching for gridbox */
	        var gridBox = query("[xcitecRole$=gridbox]", this.srcNodeRef)[0];
	
	        /* searching for tableLayoutButton */
	        if (gridBox) {
	
	            gridBox.removeAttribute("data-dojo-type");
	            
	            var tableLayoutButton = query("[xcitecRole$=layoutbutton]", gridBox)[0];
	            if (tableLayoutButton) {
	                domAttr.set(tableLayoutButton, "data-dojo-type", "xcitec/form/DropDownButton");
	                domConstruct.place(tableLayoutButton, this.tablelayoutbutton, "replace");
	            }
	
	            /* searching for grid */
	            var grid = query("[xcitecRole$=grid]", gridBox)[0];
	            if (filters) {
	                domConstruct.place(grid, this.grid, "replace");
	            }
	
	            /* searching for toolbar */
	            var toolbar = query("[xcitecRole$=toolbar]", gridBox)[0];
	            if (toolbar) {
	                domConstruct.place(toolbar, this.toolbar, "replace");
	            }
	        }
	    }
	});
	return SearchPanelV1;
});

},
'xcitec/page/Form':function(){
define([
	"dojo/dom-attr",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom",
	'dojo/dom-geometry',
	"dojo/json",
	"dojo/query",
	'dojo/window',
	"dijit/_Container",
	"dijit/_WidgetBase",
	"dijit/registry",
	"./LoadingDialog",
  "../xcitecUtil",
	'../Subsegment'
], function (domAttr, array, declare, lang, dom, geometry, JSON, query, win, _Container, _Widget, registry, LoadingDialog, xcitecUtil, Subsegment) {

	/* This widget is used for storing global page information,
	 * for example postition of scroll*/
	var Form = declare( [_Widget, _Container], {
	
		// object with storing parameters
		pageState: { scrollPos: 0 },
		storing: true,
		boxes: null, // Array
		openBoxesCount: 0,
	
		//ajax submit to disable page reload
		ajaxAction: "",
	
		postMixInProperties: function(){
			this.boxes = [];
			var _this = this;
			var nodes = query("[data-dojo-type='xcitec/Box']", this.srcNodeRef);
			array.forEach(nodes, function(n){
				_this.boxes.push(n.id);
			});
			this.inherited(arguments);
			if("onLabelAdminClick" in this.params){
				xcitecUtil.onLabelAdminClick = lang.hitch(this, function(labelId){
					var vlaHolder = dom.byId("adminClickLabelId");
					vlaHolder.value = labelId;
					this.onLabelAdminClick();
				});
			}
		},
	
		postCreate: function(){
			var _this = this;
	//		if(this.storing){
	//			// get parameters on page load
	//			/*dojo.addOnLoad(dojo.hitch(this, function() {
	//			 _this.getPageState();
	//			 }));*/
	//			// override forms' submit method to store parameters
	//			this.domNode._submit = this.domNode.submit;
	//			this.domNode.submit = function(){
	//				_this._applySubmit()
	//			};
	//		}
			this.domNode._submit = this.domNode.submit;
			this.domNode.submit = function(){
				_this._applySubmit();
			};
			this.inherited(arguments);
		},
	
		startup: function(){
			var _this = this;
			array.forEach(this.boxes, function(id){
				if(registry.byId(id).getState() != "closed"){
					_this.openBoxesCount++;
					registry.byId(id).setFormId(_this.id)
				}
			});
			console.log("Form startup!");
			this.inherited(arguments);
		},
	
		// read object and set page state
		getPageState: function(){
			var state = dom.byId("pageState");
			if(state){
				if(domAttr.get(state, "value")){
					var pageState = this.pageState = JSON.parse(state.value);
					if(pageState){
						if(pageState.scrollPos){
							registry.forEach(function(w){
								if(w instanceof Subsegment){
									if(geometry.position(w.domNode).x < pageState.scrollPos + win.getBox().h){
										w.enclosingWidget.parseChildren(w);
									}
								}
							});
							window.scrollTo(0, this.pageState.scrollPos);
						}
					}
				}
			}
		},
	
		// write page state into object (json format)
		_applySubmit: function(){
			if(this.storing){
				// store state
				var state = dom.byId("pageState");
				if(state){
					this.pageState.scrollPos = document.documentElement.scrollTop;
					state.value = JSON.stringify(this.pageState);
				}
			}
			if(typeof xcitec.useAjaxFormPost == "undefined"){ xcitec.useAjaxFormPost = true; }
			if(xcitec.useAjaxFormPost && this.ajaxAction){
	
				this._doAjaxCall();
	//			setTimeout(dojo.hitch(this, "_doAjaxCall"), 100);
			}else{
				this.domNode._submit();
			}
		},
	
		_doAjaxCall: function(){
			wicketSubmitFormById(this.id, this.ajaxAction,
				/*submitButton*/null,
				/*successHandler*/
				null,
				/*failureHandler*/
				function(message){
					alert(message);
				},
				/*precondition*/
				null);
		},
	
		decreaseOpenBoxesCount: function(){
			this.openBoxesCount--;
			if(this.openBoxesCount == 0){
				this.getPageState();
			}
		},
	
	//	_LabelAdminClick: function(){
	//		this.onLabelAdminClick();
	//	},
	
		onLabelAdminClick: function(){
			// user colback to define label click Admin action
		}
	});
	return Form;
});

},
'xcitec/xlayout/QualificationAvatarSingleColumn':function(){
define([
         "dojo",
         "dojo/dom",
         "dojo/_base/declare",
         "dojo/_base/window",
         "dojo/dom-class",
         "dojo/dom-construct",
         "dojo/dom-attr",
         "dojo/dnd/Avatar"
       ], function (dojo, dom, declare, windowModule, domClass, domAttr, domConstruct, Avatar) {
  return declare(Avatar, {
    construct: function(){
      // summary:
      //		constructor function;
      //		it is separate so it can be (dynamically) overwritten in case of need
      this.isA11y = domClass.contains(windowModule.body(),"dijit_a11y");
      var a = domConstruct.create("table", {
            "class": "xcitecDndAvatar",
            style: {
              position: "absolute",
              zIndex:   "1999",
              margin:   "0px"
            }
          }),
          source = this.manager.source, node,
          b = domConstruct.create("tbody", null, a),
          tr = domConstruct.create("tr", null, b),
          td = domConstruct.create("td", null, tr),
          icon = this.isA11y ? domConstruct.create("span", {
            id : "a11yIcon",
            innerHTML : this.manager.copy ? '+' : "<"
          }, td) : null,
          span = domConstruct.create("span", {
            //				innerHTML: source.generateText ? this._generateText() : ""
          }, td),
          k = Math.min(5, this.manager.nodes.length), i = 0;
      domClass.add(span, 'dijitReset dijitInline avatar');
      // we have to set the opacity on IE only after the node is live
      domAttr.set(tr, {
        "class": "dojoDndAvatarHeader",
        style: {opacity: 0.9}
      });
      /*
       for(; i < k; ++i){
       if(source.creator){
       // create an avatar representation of the node
       node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
       }else{
       // or just clone the node and hope it works
       node = this.manager.nodes[i].cloneNode(true);
       if(node.tagName.toLowerCase() == "tr"){
       // insert extra table nodes
       var table = dojo.create("table"),
       tbody = dojo.create("tbody", null, table);
       tbody.appendChild(node);
       node = table;
       }
       }
       node.id = "";
       tr = dojo.create("tr", null, b);
       td = dojo.create("td", null, tr);
       td.appendChild(node);
       dojo.attr(tr, {
       "class": "dojoDndAvatarItem",
       style: {opacity: (9 - i) / 10}
       });
       }
       */


      this.node = a;
    },

    update: function(){
      // summary:
      //		updates the avatar to reflect the current DnD state
      dojo[(this.manager.canDropFlag ? "add" : "remove") + "Class"](this.node, "xcitecDndAvatarCanDrop");
      dojo[(this.manager.canDropFlag ? "remove" : "add") + "Class"](this.node, "xcitecDndAvatarCantDrop");
      if (this.isA11y){
        var icon = dom.byId("a11yIcon");
        var text = '+';   // assume canDrop && copy
        if (this.manager.canDropFlag && !this.manager.copy) {
          text = '< '; // canDrop && move
        }else if (!this.manager.canDropFlag && !this.manager.copy) {
          text = "o"; //!canDrop && move
        }else if(!this.manager.canDropFlag){
          text = 'x';  // !canDrop && copy
        }
        icon.innerHTML=text;
      }
      // replace text
      /*
       dojo.query(("tr.dojoDndAvatarHeader td span" +(this.isA11y ? " span" : "")), this.node).forEach(
       function(node){
       node.innerHTML = this._generateText();
       }, this);
       */
    }
  });
});

},
'dijit/Destroyable':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/declare"
], function(array, aspect, declare){

	// module:
	//		dijit/Destroyable

	return declare("dijit.Destroyable", null, {
		// summary:
		//		Mixin to track handles and release them when instance is destroyed.
		// description:
		//		Call this.own(...) on list of handles (returned from dojo/aspect, dojo/on,
		//		dojo/Stateful::watch, or any class (including widgets) with a destroyRecursive() or destroy() method.
		//		Then call destroy() later to destroy this instance and release the resources.

		destroy: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy this class, releasing any resources registered via own().
			this._destroyed = true;
		},

		own: function(){
			// summary:
			//		Track specified handles and remove/destroy them when this instance is destroyed, unless they were
			//		already removed/destroyed manually.
			// tags:
			//		protected
			// returns:
			//		The array of specified handles, so you can do for example:
			//	|		var handle = this.own(on(...))[0];

			array.forEach(arguments, function(handle){
				var destroyMethodName =
					"destroyRecursive" in handle ? "destroyRecursive" : // remove "destroyRecursive" for 2.0
						"destroy" in handle ? "destroy" :
							"remove";

				// When this.destroy() is called, destroy handle.  Since I'm using aspect.before(),
				// the handle will be destroyed before a subclass's destroy() method starts running, before it calls
				// this.inherited() or even if it doesn't call this.inherited() at all.  If that's an issue, make an
				// onDestroy() method and connect to that instead.
				var odh = aspect.before(this, "destroy", function(preserveDom){
					handle[destroyMethodName](preserveDom);
				});

				// If handle is destroyed manually before this.destroy() is called, remove the listener set directly above.
				var hdh = aspect.after(handle, destroyMethodName, function(){
					odh.remove();
					hdh.remove();
				}, true);
			}, this);

			return arguments;		// handle
		}
	});
});

},
'dojo/request':function(){
define([
	'./request/default!'/*=====,
	'./_base/declare',
	'./promise/Promise' =====*/
], function(request/*=====, declare, Promise =====*/){
	/*=====
	request = function(url, options){
		// summary:
		//		Send a request using the default transport for the current platform.
		// url: String
		//		The URL to request.
		// options: dojo/request.__Options?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.__Promise = declare(Promise, {
		// response: dojo/promise/Promise
		//		A promise resolving to an object representing
		//		the response from the server.
	});
	request.__BaseOptions = declare(null, {
		// query: String|Object?
		//		Query parameters to append to the URL.
		// data: String|Object?
		//		Data to transfer.  This is ignored for GET and DELETE
		//		requests.
		// preventCache: Boolean?
		//		Whether to append a cache-busting parameter to the URL.
		// timeout: Integer?
		//		Milliseconds to wait for the response.  If this time
		//		passes, the then the promise is rejected.
		// handleAs: String?
		//		How to handle the response from the server.  Default is
		//		'text'.  Other values are 'json', 'javascript', and 'xml'.
	});
	request.__MethodOptions = declare(null, {
		// method: String?
		//		The HTTP method to use to make the request.  Must be
		//		uppercase.
	});
	request.__Options = declare([request.__BaseOptions, request.__MethodOptions]);

	request.get = function(url, options){
		// summary:
		//		Send an HTTP GET request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.post = function(url, options){
		// summary:
		//		Send an HTTP POST request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.put = function(url, options){
		// summary:
		//		Send an HTTP POST request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.del = function(url, options){
		// summary:
		//		Send an HTTP DELETE request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	=====*/
	return request;
});

},
'xcitec/form/NumberTextBox':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/config",
	"dojo/text!xcitec/form/templates/NumberTextBox.html",
	"dijit/form/NumberTextBox"
], function (dojo, declare, lang, config, template, NumberTextBox) {

	var NumberTextBox = declare(
			
			NumberTextBox,
			{
				templateString: template,
	            baseClass: "dijitXcitecTextBox",
	
				constructor: function(){
					// default settings for constraints
					this.constraints = {pattern: "#"};
				},
				postMixInProperties: function() {
					userConstraints = config.userConstraints;
					if(userConstraints) {
						// changing constraints if set in djConfig
						this.constraints = {};
						this.constraints = lang.mixin(this.constraints, userConstraints);					
					}			
					this.inherited(arguments);
	
	                if (this.readOnly || this.disabled) {
	                    this.required = false;
	                }
	                if (this.required) {
	                  this.baseClass = this.baseClass + " dijitXcitecRequired";
	                }
				}
			}
	);
	return NumberTextBox;
});
},
'xcitec/FilterBox':function(){
define([
         "dojo/_base/declare",
         "./Box"
       ], function (declare, Box) {
  return declare(Box, {
    filterbox: true
  });
});

},
'dojo/keys':function(){
define(["./_base/kernel", "./sniff"], function(dojo, has){

	// module:
	//		dojo/keys

	return dojo.keys = {
		// summary:
		//		Definitions for common key values.  Client code should test keyCode against these named constants,
		//		as the actual codes can vary by browser.

		BACKSPACE: 8,
		TAB: 9,
		CLEAR: 12,
		ENTER: 13,
		SHIFT: 16,
		CTRL: 17,
		ALT: 18,
		META: has("webkit") ? 91 : 224,		// the apple key on macs
		PAUSE: 19,
		CAPS_LOCK: 20,
		ESCAPE: 27,
		SPACE: 32,
		PAGE_UP: 33,
		PAGE_DOWN: 34,
		END: 35,
		HOME: 36,
		LEFT_ARROW: 37,
		UP_ARROW: 38,
		RIGHT_ARROW: 39,
		DOWN_ARROW: 40,
		INSERT: 45,
		DELETE: 46,
		HELP: 47,
		LEFT_WINDOW: 91,
		RIGHT_WINDOW: 92,
		SELECT: 93,
		NUMPAD_0: 96,
		NUMPAD_1: 97,
		NUMPAD_2: 98,
		NUMPAD_3: 99,
		NUMPAD_4: 100,
		NUMPAD_5: 101,
		NUMPAD_6: 102,
		NUMPAD_7: 103,
		NUMPAD_8: 104,
		NUMPAD_9: 105,
		NUMPAD_MULTIPLY: 106,
		NUMPAD_PLUS: 107,
		NUMPAD_ENTER: 108,
		NUMPAD_MINUS: 109,
		NUMPAD_PERIOD: 110,
		NUMPAD_DIVIDE: 111,
		F1: 112,
		F2: 113,
		F3: 114,
		F4: 115,
		F5: 116,
		F6: 117,
		F7: 118,
		F8: 119,
		F9: 120,
		F10: 121,
		F11: 122,
		F12: 123,
		F13: 124,
		F14: 125,
		F15: 126,
		NUM_LOCK: 144,
		SCROLL_LOCK: 145,
		UP_DPAD: 175,
		DOWN_DPAD: 176,
		LEFT_DPAD: 177,
		RIGHT_DPAD: 178,
		// virtual key mapping
		copyKey: has("mac") && !has("air") ? (has("safari") ? 91 : 224 ) : 17
	};
});

},
'xcitec/layout/Chapter2C':function(){
/*
 slmui.layout.ChapterC2 is a slmui Layout widget for Chapters with wide controls
 allowed next modes of controls rendering:
 mode1 (auto): control take default place label left
 mode2: control take lable+default place, label top
 mode3: control take default+right_part place, label left
 mode3: control take lable+default+right_part place, label top

*/
/*todo: for inner controls populate state from parent? dojo or view? */
define(["dojo/_base/declare",
        "./Chapter",
        "./_Chapter2CItem"], function(declare, Chapter, _Chapter2CItem){
  return declare(Chapter, {
    templateString:
        '<table class="dijitReset fieldsetChapterTable" cellSpacing="0" cellPadding="0">' +
        '<thead class="dijitReset">' +
        '<tr class="dijitReset" style="height:0px;">' +
  //      '<td class="dijitReset" data-dojo-attach-point="columnLabel"  style="width: 25%;"></td>' +
  //      '<td class="dijitReset" data-dojo-attach-point="columnWidget" style="width: 25%;"></td>' +
  //      '<td class="dijitReset" data-dojo-attach-point="columnLabel"  style="width: 18%;"></td>' +
  //      '<td class="dijitReset" data-dojo-attach-point="columnWidget" style="width: 32%;"></td>' +
  //      '<td class="dijitReset fieldsetChapter2CDelimColumn" style="width: 29px;"></td>' +
  //      '<td class="dijitReset" style="width: 25%;"></td>' +
  //      '<td class="dijitReset" style="width: 25%;"></td>' +
        '<td class="dijitReset fieldsetChapter2CLabelColumn" data-dojo-attach-point="columnLabel"></td>' +
        '<td class="dijitReset fieldsetChapter2CControlColumn" data-dojo-attach-point="columnWidget"></td>' +
        '<td class="dijitReset fieldsetChapter2CDelimColumn"></td>' +
        '<td class="dijitReset fieldsetChapter2CLabelColumn"></td>' +
        '<td class="dijitReset fieldsetChapter2CControlColumn"></td>' +
        '</tr>' +
        '<tr class="dijitReset" data-dojo-attach-point="chpaterLabelRowNode">' +
        '<td class="dijitReset fieldsetChapterCaption" colspan="5" data-dojo-attach-point="chpaterLabelContainerNode"></td>' +
        '</tr>' +
        '</thead>' +
        '<tbody class="dijitReset" data-dojo-attach-point="containerNode"></tbody></table>',

    noLabelControlNodeWidth: "50%",

    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
      var controlHolderWidget = new _Chapter2CItem({controlNode:controlNode, labelNode:labelNode});
      return controlHolderWidget.domNode;
    }
  });
});

},
'xcitec/layout/_FiltersItem':function(){
define([
	"dojo/_base/declare",
	"./_LabelControlHolder"
], function (declare, _LabelControlHolder) {

	return declare(_LabelControlHolder, {
	  templateString: '<div class="dijitReset fieldsetFiltersRow" data-dojo-attach-point=""><div class="fieldsetFiltersRowInner" data-dojo-attach-point="controlHolderNode, containerNode"></div></div>'
	});

});
},
'dojo/NodeList-traverse':function(){
define(["./query", "./_base/lang", "./_base/array"], function(dquery, lang, array){

// module:
//		dojo/NodeList-traverse

/*=====
return function(){
	// summary:
	//		Adds chainable methods to dojo/query() / NodeList instances for traversing the DOM
};
=====*/

var NodeList = dquery.NodeList;

lang.extend(NodeList, {
	_buildArrayFromCallback: function(/*Function*/ callback){
		// summary:
		//		builds a new array of possibly differing size based on the input list.
		//		Since the returned array is likely of different size than the input array,
		//		the array's map function cannot be used.
		var ary = [];
		for(var i = 0; i < this.length; i++){
			var items = callback.call(this[i], this[i], ary);
			if(items){
				ary = ary.concat(items);
			}
		}
		return ary;	//Array
	},

	_getUniqueAsNodeList: function(/*Array*/ nodes){
		// summary:
		//		given a list of nodes, make sure only unique
		//		elements are returned as our NodeList object.
		//		Does not call _stash().
		var ary = [];
		//Using for loop for better speed.
		for(var i = 0, node; node = nodes[i]; i++){
			//Should be a faster way to do this. dojo/query has a private
			//_zip function that may be inspirational, but there are pathways
			//in query that force nozip?
			if(node.nodeType == 1 && array.indexOf(ary, node) == -1){
				ary.push(node);
			}
		}
		return this._wrap(ary, null, this._NodeListCtor);	 // dojo/NodeList
	},

	_getUniqueNodeListWithParent: function(/*Array*/ nodes, /*String*/ query){
		// summary:
		//		gets unique element nodes, filters them further
		//		with an optional query and then calls _stash to track parent NodeList.
		var ary = this._getUniqueAsNodeList(nodes);
		ary = (query ? dquery._filterResult(ary, query) : ary);
		return ary._stash(this);  // dojo/NodeList
	},

	_getRelatedUniqueNodes: function(/*String?*/ query, /*Function*/ callback){
		// summary:
		//		cycles over all the nodes and calls a callback
		//		to collect nodes for a possible inclusion in a result.
		//		The callback will get two args: callback(node, ary),
		//		where ary is the array being used to collect the nodes.
		return this._getUniqueNodeListWithParent(this._buildArrayFromCallback(callback), query);  // dojo/NodeList
	},

	children: function(/*String?*/ query){
		// summary:
		//		Returns all immediate child elements for nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the child elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		all immediate child elements for the nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".container").children();
		//	|	});
		//		returns the four divs that are children of the container div.
		//		Running this code:
		//	|	dojo.query(".container").children(".red");
		//		returns the two divs that have the class "red".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			return lang._toArray(node.childNodes);
		}); // dojo/NodeList
	},

	closest: function(/*String*/ query, /*String|DOMNode?*/ root){
		// summary:
		//		Returns closest parent that matches query, including current node in this
		//		dojo/NodeList if it matches the query.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// root:
		//		If specified, query is relative to "root" rather than document body.
		// returns:
		//		the closest parent that matches the query, including the current
		//		node in this dojo/NodeList if it matches the query.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		//	|		<div class="red">Red One</div>
		//	|		Some Text
		//	|		<div class="blue">Blue One</div>
		//	|		<div class="red">Red Two</div>
		//	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".red").closest(".container");
		//	|	});
		//		returns the div with class "container".
		return this._getRelatedUniqueNodes(null, function(node, ary){
			do{
				if(dquery._filterResult([node], query, root).length){
					return node;
				}
			}while(node != root && (node = node.parentNode) && node.nodeType == 1);
			return null; //To make rhino strict checking happy.
		}); // dojo/NodeList
	},

	parent: function(/*String?*/ query){
		// summary:
		//		Returns immediate parent elements for nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the parent elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		immediate parent elements for nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first"><span class="text">Blue One</span></div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue"><span class="text">Blue Two</span></div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".text").parent();
		//	|	});
		//		returns the two divs with class "blue".
		//		Running this code:
		//	|		query(".text").parent(".first");
		//		returns the one div with class "blue" and "first".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			return node.parentNode;
		}); // dojo/NodeList
	},

	parents: function(/*String?*/ query){
		// summary:
		//		Returns all parent elements for nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the child elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		all parent elements for nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first"><span class="text">Blue One</span></div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue"><span class="text">Blue Two</span></div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".text").parents();
		//	|	});
		//		returns the two divs with class "blue", the div with class "container",
		// 	|	the body element and the html element.
		//		Running this code:
		//	|		query(".text").parents(".container");
		//		returns the one div with class "container".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			while(node.parentNode){
				node = node.parentNode;
				pary.push(node);
			}
			return pary;
		}); // dojo/NodeList
	},

	siblings: function(/*String?*/ query){
		// summary:
		//		Returns all sibling elements for nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the sibling elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		all sibling elements for nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".first").siblings();
		//	|	});
		//		returns the two divs with class "red" and the other div
		// 	|	with class "blue" that does not have "first".
		//		Running this code:
		//	|		query(".first").siblings(".red");
		//		returns the two div with class "red".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			var nodes = (node.parentNode && node.parentNode.childNodes);
			for(var i = 0; i < nodes.length; i++){
				if(nodes[i] != node){
					pary.push(nodes[i]);
				}
			}
			return pary;
		}); // dojo/NodeList
	},

	next: function(/*String?*/ query){
		// summary:
		//		Returns the next element for nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the next elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		the next element for nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue last">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".first").next();
		//	|	});
		//		returns the div with class "red" and has innerHTML of "Red Two".
		//		Running this code:
		//	|	dojo.query(".last").next(".red");
		//		does not return any elements.
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var next = node.nextSibling;
			while(next && next.nodeType != 1){
				next = next.nextSibling;
			}
			return next;
		}); // dojo/NodeList
	},

	nextAll: function(/*String?*/ query){
		// summary:
		//		Returns all sibling elements that come after the nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the sibling elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		all sibling elements that come after the nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red next">Red Two</div>
		// 	|		<div class="blue next">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".first").nextAll();
		//	|	});
		//		returns the two divs with class of "next".
		//		Running this code:
		//	|		query(".first").nextAll(".red");
		//		returns the one div with class "red" and innerHTML "Red Two".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			var next = node;
			while((next = next.nextSibling)){
				if(next.nodeType == 1){
					pary.push(next);
				}
			}
			return pary;
		}); // dojo/NodeList
	},

	prev: function(/*String?*/ query){
		// summary:
		//		Returns the previous element for nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the previous elements.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		the previous element for nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".first").prev();
		//	|	});
		//		returns the div with class "red" and has innerHTML of "Red One".
		//		Running this code:
		//	|		query(".first").prev(".blue");
		//		does not return any elements.
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var prev = node.previousSibling;
			while(prev && prev.nodeType != 1){
				prev = prev.previousSibling;
			}
			return prev;
		}); // dojo/NodeList
	},

	prevAll: function(/*String?*/ query){
		// summary:
		//		Returns all sibling elements that come before the nodes in this dojo/NodeList.
		//		Optionally takes a query to filter the sibling elements.
		// description:
		//		The returned nodes will be in reverse DOM order -- the first node in the list will
		//		be the node closest to the original node/NodeList.
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		all sibling elements that come before the nodes in this dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red prev">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue prev">Blue One</div>
		// 	|		<div class="red second">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".second").prevAll();
		//	|	});
		//		returns the two divs with class of "prev".
		//		Running this code:
		//	|		query(".first").prevAll(".red");
		//		returns the one div with class "red prev" and innerHTML "Red One".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			var prev = node;
			while((prev = prev.previousSibling)){
				if(prev.nodeType == 1){
					pary.push(prev);
				}
			}
			return pary;
		}); // dojo/NodeList
	},

	andSelf: function(){
		// summary:
		//		Adds the nodes from the previous dojo/NodeList to the current dojo/NodeList.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red prev">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue prev">Blue One</div>
		// 	|		<div class="red second">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".second").prevAll().andSelf();
		//	|	});
		//		returns the two divs with class of "prev", as well as the div with class "second".
		return this.concat(this._parent);	// dojo/NodeList
	},

	//Alternate methods for the :first/:last/:even/:odd pseudos.
	first: function(){
		// summary:
		//		Returns the first node in this dojo/NodeList as a dojo/NodeList.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// returns:
		//		the first node in this dojo/NodeList
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue last">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".blue").first();
		//	|	});
		//		returns the div with class "blue" and "first".
		return this._wrap(((this[0] && [this[0]]) || []), this); // dojo/NodeList
	},

	last: function(){
		// summary:
		//		Returns the last node in this dojo/NodeList as a dojo/NodeList.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// returns:
		//		the last node in this dojo/NodeList
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue last">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|	query(".blue").last();
		//	|	});
		//		returns the last div with class "blue",
		return this._wrap((this.length ? [this[this.length - 1]] : []), this); // dojo/NodeList
	},

	even: function(){
		// summary:
		//		Returns the even nodes in this dojo/NodeList as a dojo/NodeList.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// returns:
		//		the even nodes in this dojo/NodeList
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="interior red">Red One</div>
		// 	|		<div class="interior blue">Blue One</div>
		// 	|		<div class="interior red">Red Two</div>
		// 	|		<div class="interior blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".interior").even();
		//	|	});
		//		returns the two divs with class "blue"
		return this.filter(function(item, i){
			return i % 2 != 0;
		}); // dojo/NodeList
	},

	odd: function(){
		// summary:
		//		Returns the odd nodes in this dojo/NodeList as a dojo/NodeList.
		// description:
		//		.end() can be used on the returned dojo/NodeList to get back to the
		//		original dojo/NodeList.
		// returns:
		//		the odd nodes in this dojo/NodeList
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="interior red">Red One</div>
		// 	|		<div class="interior blue">Blue One</div>
		// 	|		<div class="interior red">Red Two</div>
		// 	|		<div class="interior blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	require(["dojo/query", "dojo/NodeList-traverse"
		//	|	], function(query){
		//	|		query(".interior").odd();
		//	|	});
		//		returns the two divs with class "red"
		return this.filter(function(item, i){
			return i % 2 == 0;
		}); // dojo/NodeList
	}
});

return NodeList;
});

},
'dijit/form/MultiSelect':function(){
define([
	"dojo/_base/array", // array.indexOf, array.map
	"dojo/_base/declare", // declare
	"dojo/dom-geometry", // domGeometry.setMarginBox
	"dojo/has",
	"dojo/query", // query
	"./_FormValueWidget"
], function(array, declare, domGeometry, has, query, _FormValueWidget){

	// module:
	//		dijit/form/MultiSelect

	var MultiSelect = declare("dijit.form.MultiSelect" + (has("dojo-bidi") ? "_NoBidi" : ""), _FormValueWidget, {
		// summary:
		//		Widget version of a `<select multiple=true>` element,
		//		for selecting multiple options.

		// size: Number
		//		Number of elements to display on a page
		//		NOTE: may be removed in version 2.0, since elements may have variable height;
		//		set the size via style="..." or CSS class names instead.
		size: 7,

		baseClass: "dijitMultiSelect",

		templateString: "<select multiple='true' ${!nameAttrSetting} data-dojo-attach-point='containerNode,focusNode' data-dojo-attach-event='onchange: _onChange'></select>",

		addSelected: function(/*dijit/form/MultiSelect*/ select){
			// summary:
			//		Move the selected nodes of a passed Select widget
			//		instance to this Select widget.
			//
			// example:
			// |	// move all the selected values from "bar" to "foo"
			// |	dijit.byId("foo").addSelected(dijit.byId("bar"));

			select.getSelected().forEach(function(n){
				this.containerNode.appendChild(n);
				// scroll to bottom to see item
				// cannot use scrollIntoView since <option> tags don't support all attributes
				// does not work on IE due to a bug where <select> always shows scrollTop = 0
				this.domNode.scrollTop = this.domNode.offsetHeight; // overshoot will be ignored
				// scrolling the source select is trickier esp. on safari who forgets to change the scrollbar size
				var oldscroll = select.domNode.scrollTop;
				select.domNode.scrollTop = 0;
				select.domNode.scrollTop = oldscroll;
			}, this);
			this._set('value', this.get('value'));
		},

		getSelected: function(){
			// summary:
			//		Access the NodeList of the selected options directly
			return query("option", this.containerNode).filter(function(n){
				return n.selected; // Boolean
			}); // dojo/NodeList
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		Returns an array of the selected options' values.

			// Don't call getSelect.map() because it doesn't return a real array,
			// and that messes up dojo.toJson() calls like in the Form.html test
			return array.map(this.getSelected(), function(n){
				return n.value;
			});
		},

		multiple: true, // for Form

		_setValueAttr: function(/*Array*/ values, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', values) works.
			// description:
			//		Set the value(s) of this Select based on passed values
			query("option", this.containerNode).forEach(function(n){
				n.selected = (array.indexOf(values, n.value) != -1);
			});
			this.inherited(arguments);
		},

		invertSelection: function(/*Boolean?*/ onChange){
			// summary:
			//		Invert the selection
			// onChange: Boolean
			//		If false, onChange is not fired.
			var val = [];
			query("option", this.containerNode).forEach(function(n){
				if(!n.selected){
					val.push(n.value);
				}
			});
			this._setValueAttr(val, !(onChange === false || onChange == null));
		},

		_onChange: function(/*Event*/){
			this._handleOnChange(this.get('value'), true);
		},

		// for layout widgets:
		resize: function(/*Object*/ size){
			if(size){
				domGeometry.setMarginBox(this.domNode, size);
			}
		},

		postCreate: function(){
			this._set('value', this.get('value'));
			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		MultiSelect = declare("dijit.form.MultiSelect", MultiSelect, {
			addSelected: function(/*dijit/form/MultiSelect*/ select){
				select.getSelected().forEach(function(n){
					n.text = this.enforceTextDirWithUcc(this.restoreOriginalText(n), n.text);
				}, this);
				this.inherited(arguments);
			},

			_setTextDirAttr: function(textDir){
				// to insure the code executed only when _BidiSupport loaded, and only
				// when there was a change in textDir
				if((this.textDir != textDir || !this._created) && this.enforceTextDirWithUcc){
					this._set("textDir", textDir);

					query("option", this.containerNode).forEach(function(option){
						// If the value wasn't defined explicitly, it the same object as
						// option.text. Since the option.text will be modified (by wrapping of UCC)
						// we want to save the original option.value for form submission.
						if(!this._created && option.value === option.text){
							option.value = option.text;
						}
						// apply the bidi support
						option.text = this.enforceTextDirWithUcc(option, option.originalText || option.text);
					}, this);
				}
			}
		});
	}

	return MultiSelect;
});

},
'dojo/main':function(){
define([
	"./_base/kernel",	// kernel.isAsync
	"./has",
	"require",
	"./sniff",
	"./_base/lang",
	"./_base/array",
	"./_base/config",
	"./ready",
	"./_base/declare",
	"./_base/connect",
	"./_base/Deferred",
	"./_base/json",
	"./_base/Color",
	"./has!dojo-firebug?./_firebug/firebug",
	"./_base/browser",
	"require"
], function(kernel, has, require, sniff, lang, array, config, ready){
	// module:
	//		dojo/main
	// summary:
	//		This is the package main module for the dojo package; it loads dojo base appropriate for the execution environment.

	// the preferred way to load the dojo firebug console is by setting has("dojo-firebug") true in dojoConfig
	// the isDebug config switch is for backcompat and will work fine in sync loading mode; it works in
	// async mode too, but there's no guarantee when the module is loaded; therefore, if you need a firebug
	// console guaranteed at a particular spot in an app, either set config.has["dojo-firebug"] true before
	// loading dojo.js or explicitly include dojo/_firebug/firebug in a dependency list.
	if(config.isDebug){
		require(["./_firebug/firebug"]);
	}

	// dojoConfig.require is deprecated; use the loader configuration property deps
	 1 || has.add("dojo-config-require", 1);
	if( 1 ){
		var deps= config.require;
		if(deps){
			// config.require may be dot notation
			deps= array.map(lang.isArray(deps) ? deps : [deps], function(item){ return item.replace(/\./g, "/"); });
			if(kernel.isAsync){
				require(deps);
			}else{
				// this is a bit janky; in 1.6- dojo is defined before these requires are applied; but in 1.7+
				// dojo isn't defined until returning from this module; this is only a problem in sync mode
				// since we're in sync mode, we know we've got our loader with its priority ready queue
				ready(1, function(){require(deps);});
			}
		}
	}

	return kernel;
});

},
'xcitec/widgets/WidgetsContainer':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "./WidgetBox"
], function (dojo, array, declare, domAttr, domConstruct, query, _Templated, _Widget, WidgetBox) {

	/* WidgetsContainer is a container for xcitec widgets, it places them in WidgetBoxes and align them in a table*/
	
	var WidgetsContainer = declare( [_Widget, _Templated], {
	    templateString: '<div class="xwidget_wraper">' +
	                      '<table class="xwidget_inner">' +
	                          '<tbody data-dojo-attach-point="containerNode">' +
	  //                                '<tr data-dojo-attach-point="row"></tr>' +
	                          '</tbody>' +
	                      '</table>' +
	                    '</div>',
	
	    // max number of cells in WidgetContainer row
	    maxCells: 3,
	
	    _fillContent: function(/*DomNode*/ source){
	        var _this = this;
	
			// sum of widgets width
			var widthSum=0;
			var tr;
			// number of created td's
			var numTd=0;
	
			// creating WidgetBox and placing widgets into it
			var portlets = query(">", source);
			array.forEach(portlets, function(widget, index) {
	            var td, div = null;
	            if (widthSum >= _this.maxCells) {
	                widthSum = 0;
	            }
	            // if no row exists - create it
	            if (widthSum == 0) {
	                tr = domConstruct.create("tr", {}, _this.containerNode, "last");
	                // new row - no td's
	                numTd = 0;
	            }
	            // get widget size (1, 2 or 3)
	            var widgetSize = domAttr.get(widget, 'widgetWidth');
	            if (!widgetSize) {
	                widgetSize = 1;
	            }
	            widthSum += Number(widgetSize);
	
	            // check if widget has enough place to put itself in the existing row
	            if (widthSum > _this.maxCells) {
	                if (_this.maxCells - numTd > 0) {
	                    td = domConstruct.create("td", {colSpan: _this.maxCells - numTd}, tr, "last");
	                }
	                tr = domConstruct.create("tr", {}, _this.containerNode, "last");
	                numTd = 0;
	                widthSum = Number(widgetSize);
	            }
	
	            // create WidgetBox and place widget node inside it
	            td = domConstruct.create("td", {colSpan: widgetSize}, tr, "last");
	            div = domConstruct.create("div");
	            div.appendChild(widget);
	
				var createParams = {
					caption: domAttr.get(widget, "caption"),
					captionId: domAttr.get(widget, "captionId"),
					captionIcon: domAttr.get(widget, "captionIcon"),
					widgetHidden: domAttr.get(widget, "widgetHidden")
				};
				var widgetHeight = domAttr.get(widget, 'widgetHeight');
				if (widgetHeight){
					createParams.widgetHeight = widgetHeight;
				}
	
	
				var widgetBox = new WidgetBox( createParams, div);
	            td.appendChild(widgetBox.domNode);
	            numTd += Number(widgetSize);
	        });
	        // if there is empty space in row - create additional td to prevent widget change it's native size
	        while (numTd < 3) {
	            domConstruct.create("td", {}, tr, "last");
	            numTd++;
	        }
	    }
	});
	
	return WidgetsContainer;
});

},
'xcitec/layout/DoubleField':function(){
define([
	"dojo",
	"dojo/dom-class",
	"dojo/_base/window",
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/dom-attr",
	"dojo/query",
	"dojo/text!./templates/DoubleField.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
    "dijit/_WidgetsInTemplateMixin",
	"dijit/registry",
	"../Label"
], function (dojo, domClass, windowModule, declare, domStyle, domAttr, query, template, _Templated, _Widget, _WidgetsInTemplateMixin, registry) {

	/*
	  DoubleField layout implemented via CSS styles.
	  for to have right layout with dividingText:
	  - calculate text size
	  - set right width for middle column
	  - NOTE: label size and column width is set in startup - earlier it returns 0!
	
	  for IE7 there is a problem that width is always 0 (because middle column 0 and label width: 100%
	  to avoid it - middle column get size 60 (more then max available label width).
	  then calc value and set correct width.
	  NOTE: strange but now it not needed any more. good works without it.
	  
	*/
	
	var DoubleField = declare( [_Widget, _Templated, _WidgetsInTemplateMixin], {
	    templateString: template,
	    tstrMiddleColumnWidth: "0px",
	    slider: null,
	
	    // layout of fields left_fiels:right_field (in %)
	    // 50:50 - deault
	    // TODO make constants "zip", "street" and "pobox" according to predefined complex modes in specifications   
	    layout: "",
	
	    dividingText: "",
	    dividingTextId: "",
        widgetsInTemplate: false,

        buildRendering: function(){
	      this.inherited(arguments);
	      this.layoutControls();
	    },
	
	    _fillContent: function(/*DomNode*/ source){
	      // Overrides _Templated._fillcContent().
	      //
	      // My inner HTML contains two widgets the button contents and a drop down widget, like
	      // <DoubleField>  <input /> <input /> </DoubleField>
	
	      if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
	        var widgets = query("> [data-dojo-type]", this.srcNodeRef);
	        this.leftControlNode = widgets[0];
	        domStyle.set(this.leftControlNode, "width", "100%");
	        this.leftFieldNode.appendChild(this.leftControlNode);
	        var rightField = widgets[1];
	        domStyle.set(rightField, "width", "100%");
	        this.rightFieldNode.appendChild(rightField);
	      }
	
		    // set dividing text
			if(this.dividingText != ""){
				this.middleLableWidget.domNode.appendChild(windowModule.doc.createTextNode(this.dividingText));
			}
	    },
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        if (this.dividingText) {
	            this.widgetsInTemplate = true;
	//            this.tstrMiddleColumnWidth = "60px";
	        }
	    },
	
	  startup: function(){
	    this.inherited(arguments);
	    if (this.widgetsInTemplate && this.middleLableWidget) {
	
	      var labelWidth = this.middleLableWidget.getLabelWidth() + 20;
	      domStyle.set(this.columnMiddle, "width", ""+ labelWidth +"px");
	    }
	      if ((domAttr.get(this.leftControlNode, "data-dojo-type") == "xcitec/form/Slider")) {
	          this.slider = registry.byNode(query(">", this.leftFieldNode)[0]);
	          this.textBox = registry.byNode(query(">", this.rightFieldNode)[0]);
	          domStyle.set(this.rightFieldNode, "verticalAlign", "top");
	          this.slider.labelWidget = this.textBox;
	      }
	      
	  },
	
	    layoutControls: function(){
	//      if (this.widgetsInTemplate && this.middleLableWidget) {
	//        console.log(this.middleLableWidget.getLabelWidth())
	//      }
	      if (!this.layout) return; //default layout
	      if (this.layout=="default") return; //default layout
	      if (this.layout=="filter") {
	        //filter layout
	        //dojo.style(this.columnLeft, "width", "62px");
	        // correction filter field view see SRM-460
	//        dojo.style(this.columnLeft, "width", "55px");
	//        dojo.style(this.columnRight, "width", "100%");
	        domClass.add(this.columnsDefinition, "xcitecFilterFieldColumns");
	        domClass.add(this.leftControlNode, "xcitecFilterDropDown");
	        return;
	      }
	      var sizes = this.layout.split(":");
	      domStyle.set(this.columnLeft, "width", sizes[0]+"%");
	      domStyle.set(this.columnRight, "width", sizes[1]+"%");
	    },
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			this.containerNode = this.domNode; //force to destroy nested widgets
			this.inherited(arguments);
		}
	
	});
	return DoubleField;
});

},
'xcitec/grid/Action':function(){
define([
	"dojo/_base/declare",
  "../_base/Action"
], function (declare, Action) {
	return declare(Action, {

	});
});
},
'xcitec/form/RadioGroup':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom",
	"dojo/on",
	"dojo/query",
	"dijit/_Container",
	"dijit/form/_FormValueWidget",
	"dijit/registry",
  "../xcitecUtil",
  "./RadioButton"
], function (dojo, array, declare, lang, domAttr, dom, on, query, _Container, _FormValueWidget, registry, xcitecUtil, RadioButton) {

	var RadioGroup = declare( [_FormValueWidget, _Container], {
	    selCount: 0,    // need when multiply selected="true" attributes take place
	    selIndex: -1,
	    tabIndex: "-1",
	    required: false,
	    _buttons: null,
	    templateString: '<div class="dijit dijitReset dijitInline" tabIndex="-1" data-dojo-attach-point="radioNode, focusNode">'+
	                    '<div class="required"></div>' +
	                    '</div>',
	
	    _parseSource: function() {
	        // summary:
	        //		Loads our options and sets up our dropdown correctly.  We
	        //		don't want any content, so we don't call any inherit chain
	        //		function.
	        var opts = this.options;
	        if (!opts) {
	            opts = this.options = this.srcNodeRef ? query(">",
	                    this.srcNodeRef).map(function(node, index) {
	                // actually count for selected="true" attributes
	
	                if (xcitecUtil.str2bool(node.getAttribute("checked"))) {
	                    this.selCount++;
	                    this.selIndex = index;
	                }
	                return {
	                    value: node.getAttribute("value"),
	                    caption: node.getAttribute("caption"),
	                    captionId: node.getAttribute("captionId"),
	                    icon: node.getAttribute("icon"),
	                    label: String(node.innerHTML),
	                    checked: xcitecUtil.str2bool(domAttr.get(node, "checked")),
	                    disabled: xcitecUtil.str2bool(domAttr.get(node, "disabled"))
	                };
	            }, this) : [];
	        }
	    },  
	    
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        if (this.required) {
	            this.baseClass = this.baseClass + "dijitXcitecRadioboxGroupRequired";
	        }
	        this._parseSource();
	    },
	
	    buildRendering: function(){
	        this._buttons = [];
	        this.inherited(arguments);
	        this._createRadioGroup(this.options);
	        dom.setSelectable(this.domNode, false);
	    },
	
	    _createRadioGroup: function(otpions) {
	        var self = this;
	        var ind = 0;
	
	        array.forEach(otpions, function(node, index) {
	            var text = "";
	            if (node.caption) {
	                text = node.caption;
	            } else if (node.label) {
	                text = node.label;
	            }
	
	            // create RadioButtons according set of options
	            var radioButton = new RadioButton({
	                name: self.name,
	                checked: (index == self.selIndex),
	                disabled: self.disabled || node.disabled,
	                readOnly: self.readOnly,
	                caption: text,
	                captionId: node.captionId,
	                icon: node.icon,
	                value: node.value,
	                style: "width:100%;",
	                index: ind,
	                parent: self
	                //, onChange: function(){ alert("test "+ arguments[0] ); }
	            });
	            self._buttons[ind]=radioButton;
	            ind++;
	            self.radioNode.appendChild(radioButton.domNode);
	            radioButton.domNode.onchange = "";
	            on(radioButton.domNode, "click", lang.hitch(radioButton, self.changeState));
	        });
	    },
	
	    changeState: function() {
	        if (this.readOnly || this.disabled) {
	            return;
	        }
	        var widget = registry.byNode(this.domNode);
	        widget.focus();
	        if (widget.index != widget.parent.selIndex) {
	            if (widget.parent.selIndex > -1) {
	                widget.parent._buttons[widget.parent.selIndex].set('checked', false);
	            }
	            widget.parent.selIndex = widget.index;
	            widget.parent._onChange();
	        }
	    },
	
	    _onChange: function() {
	        // user defined function
	        this.onchange();
	    },
	
	    onchange: function() {
	        // for wicket use in template
	    },

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
	});
	return RadioGroup;
});

},
'dojo/data/util/filter':function(){
define(["../../_base/lang"], function(lang){
	// module:
	//		dojo/data/util/filter
	// summary:
	//		TODOC

var filter = {};
lang.setObject("dojo.data.util.filter", filter);

filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	// summary:
	//		Helper function to convert a simple pattern to a regular expression for matching.
	// description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:
	//
	//		- ca*   -> /^ca.*$/
	//		- *ca*  -> /^.*ca.*$/
	//		- *c\*a*  -> /^.*c\*a.*$/
	//		- *c\*a?*  -> /^.*c\*a..*$/
	//
	//		and so on.
	// pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//
	//		- * Means match anything, so ca* means match anything starting with ca
	//		- ? Means match single character.  So, b?b will match to bob and bab, and so on.
	//		- \ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//
	//		To use a \ as a character in the string, it must be escaped.  So in the pattern it should be
	//		represented by \\ to be treated as an ordinary \ character instead of an escape.
	// ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch(c){
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"mi"); //RegExp
	}else{
		return new RegExp(rxp,"m"); //RegExp
	}

};

return filter;
});

},
'xcitec/page/DataTabPanel':function(){
define([
	"dojo/_base/declare",
	"./SubTabPanelV1"
], function (declare, SubTabPanelV1) {

	return declare(SubTabPanelV1, {

	});
});
},
'dojo/dnd/common':function(){
define(["../sniff", "../_base/kernel", "../_base/lang", "../dom"],
	function(has, kernel, lang, dom){

// module:
//		dojo/dnd/common

var exports = lang.getObject("dojo.dnd", true);
/*=====
// TODO: for 2.0, replace line above with this code.
var exports = {
	// summary:
	//		TODOC
};
=====*/

exports.getCopyKeyState = function(evt){
	return evt[has("mac") ? "metaKey" : "ctrlKey"]
};

exports._uniqueId = 0;
exports.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = kernel._scopeName + "Unique" + (++exports._uniqueId);
	}while(dom.byId(id));
	return id;
};

exports._empty = {};

exports.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " a button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

return exports;
});

},
'dijit/tree/ObjectStoreModel':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.indexOf array.some
	"dojo/aspect", // aspect.before, aspect.after
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"dojo/when"
], function(array, aspect, declare, lang, when){

	// module:
	//		dijit/tree/ObjectStoreModel

	return declare("dijit.tree.ObjectStoreModel", null, {
		// summary:
		//		Implements dijit/tree/model connecting dijit/Tree to a dojo/store/api/Store that implements
		//		getChildren().
		//
		//		If getChildren() returns an array with an observe() method, then it will be leveraged to reflect
		//		store updates to the tree.   So, this class will work best when:
		//
		//			1. the store implements dojo/store/Observable
		//			2. getChildren() is implemented as a query to the server (i.e. it calls store.query())
		//
		//		Drag and Drop: To support drag and drop, besides implementing getChildren()
		//		and dojo/store/Observable, the store must support the parent option to put().
		//		And in order to have child elements ordered according to how the user dropped them,
		//		put() must support the before option.

		// store: dojo/store/api/Store
		//		Underlying store
		store: null,

		// labelAttr: String
		//		Get label for tree node from this attribute
		labelAttr: "name",

		// labelType: [const] String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// root: [readonly] Object
		//		Pointer to the root item from the dojo/store/api/Store (read only, not a parameter)
		root: null,

		// query: anything
		//		Specifies datastore query to return the root item for the tree.
		//		Must only return a single item.   Alternately can just pass in pointer
		//		to root item.
		// example:
		//	|	{id:'ROOT'}
		query: null,

		constructor: function(/* Object */ args){
			// summary:
			//		Passed the arguments listed above (store, etc)
			// tags:
			//		private

			lang.mixin(this, args);

			this.childrenCache = {};	// map from id to array of children
		},

		destroy: function(){
			// TODO: should cancel any in-progress processing of getRoot(), getChildren()
			for(var id in this.childrenCache){
				this.childrenCache[id].close && this.childrenCache[id].close();
			}
		},

		// =======================================================================
		// Methods for traversing hierarchy

		getRoot: function(onItem, onError){
			// summary:
			//		Calls onItem with the root item for the tree, possibly a fabricated item.
			//		Calls onError on error.
			if(this.root){
				onItem(this.root);
			}else{
				var res;
				when(res = this.store.query(this.query),
					lang.hitch(this, function(items){
						//console.log("queried root: ", res);
						if(items.length != 1){
							throw new Error("dijit.tree.ObjectStoreModel: root query returned " + items.length +
								" items, but must return exactly one");
						}
						this.root = items[0];
						onItem(this.root);

						// Setup listener to detect if root item changes
						if(res.observe){
							res.observe(lang.hitch(this, function(obj){
								// Presumably removedFrom == insertedInto == 1, and this call indicates item has changed.
								//console.log("root changed: ", obj);
								this.onChange(obj);
							}), true);	// true to listen for updates to obj
						}
					}),
					onError
				);
			}
		},

		mayHaveChildren: function(/*===== item =====*/){
			// summary:
			//		Tells if an item has or might have children.  Implementing logic here
			//		avoids showing +/- expando icon for nodes that we know won't have children.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node).
			//
			//		Application code should override this method based on the data, for example
			//		it could be `return item.leaf == true;`.
			//
			//		Note that mayHaveChildren() must return true for an item if it could possibly
			//		have children in the future, due to drag-an-drop or some other data store update.
			//		Also note that it may return true if it's just too expensive to check during tree
			//		creation whether or not the item has children.
			// item: Object
			//		Item from the dojo/store
			return true;
		},

		getChildren: function(/*Object*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
			// summary:
			//		Calls onComplete() with array of child items of given parent item.
			// parentItem:
			//		Item from the dojo/store

			var id = this.store.getIdentity(parentItem);
			if(this.childrenCache[id]){
				when(this.childrenCache[id], onComplete, onError);
				return;
			}

			var res = this.childrenCache[id] = this.store.getChildren(parentItem);

			// User callback
			when(res, onComplete, onError);

			// Setup listener in case children list changes, or the item(s) in the children list are
			// updated in some way.
			if(res.observe){
				res.observe(lang.hitch(this, function(obj, removedFrom, insertedInto){
					//console.log("observe on children of ", id, ": ", obj, removedFrom, insertedInto);

					// If removedFrom == insertedInto, this call indicates that the item has changed.
					// Even if removedFrom != insertedInto, the item may have changed.
					this.onChange(obj);

					if(removedFrom != insertedInto){
						// Indicates an item was added, removed, or re-parented.  The children[] array (returned from
						// res.then(...)) has already been updated (like a live collection), so just use it.
						when(res, lang.hitch(this, "onChildrenChange", parentItem));
					}
				}), true);	// true means to notify on item changes
			}
		},

		// =======================================================================
		// Inspecting items

		isItem: function(/*===== something =====*/){
			return true;	// Boolean
		},

		getIdentity: function(/* item */ item){
			return this.store.getIdentity(item);	// Object
		},

		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Get the label for an item
			return item[this.labelAttr];	// String
		},

		// =======================================================================
		// Write interface, for DnD

		newItem: function(/* dijit/tree/dndSource.__Item */ args, /*Item*/ parent, /*int?*/ insertIndex, /*Item*/ before){
			// summary:
			//		Creates a new item.   See `dojo/data/api/Write` for details on args.
			//		Used in drag & drop when item from external source dropped onto tree.

			return this.store.put(args, {
				parent: parent,
				before: before
			});
		},

		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem,
					/*Boolean*/ bCopy, /*int?*/ insertIndex, /*Item*/ before){
			// summary:
			//		Move or copy an item from one parent item to another.
			//		Used in drag & drop

			if(!bCopy){
				// In order for DnD moves to work correctly, childItem needs to be orphaned from oldParentItem
				// before being adopted by newParentItem.   That way, the TreeNode is moved rather than
				// an additional TreeNode being created, and the old TreeNode subsequently being deleted.
				// The latter loses information such as selection and opened/closed children TreeNodes.
				// Unfortunately simply calling this.store.put() will send notifications in a random order, based
				// on when the TreeNodes in question originally appeared, and not based on the drag-from
				// TreeNode vs. the drop-onto TreeNode.

				var oldParentChildren = [].concat(this.childrenCache[this.getIdentity(oldParentItem)]), // concat to make copy
					index = array.indexOf(oldParentChildren, childItem);
				oldParentChildren.splice(index, 1);
				this.onChildrenChange(oldParentItem, oldParentChildren);
			}

			return this.store.put(childItem, {
				overwrite: true,
				parent: newParentItem,
				before: before
			});
		},

		// =======================================================================
		// Callbacks

		onChange: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},

		onChildrenChange: function(/*===== parent, newChildrenList =====*/){
			// summary:
			//		Callback to do notifications about new, updated, or deleted items.
			// parent: dojo/data/Item
			// newChildrenList: Object[]
			//		Items from the store
			// tags:
			//		callback
		},

		onDelete: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback when an item has been deleted.
			//		Actually we have no way of knowing this with the new dojo.store API,
			//		so this method is never called (but it's left here since Tree connects
			//		to it).
			// tags:
			//		callback
		}
	});
});

},
'dijit/tree/ForestStoreModel':function(){
define([
	"dojo/_base/array", // array.indexOf array.some
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // global
	"dojo/_base/lang", // lang.hitch
	"./TreeStoreModel"
], function(array, declare, kernel, lang, TreeStoreModel){

// module:
//		dijit/tree/ForestStoreModel

return declare("dijit.tree.ForestStoreModel", TreeStoreModel, {
	// summary:
	//		Interface between a dijit.Tree and a dojo.data store that doesn't have a root item,
	//		a.k.a. a store that has multiple "top level" items.
	//
	// description:
	//		Use this class to wrap a dojo.data store, making all the items matching the specified query
	//		appear as children of a fabricated "root item".  If no query is specified then all the
	//		items returned by fetch() on the underlying store become children of the root item.
	//		This class allows dijit.Tree to assume a single root item, even if the store doesn't have one.
	//
	//		When using this class the developer must override a number of methods according to their app and
	//		data, including:
	//
	//		- onNewRootItem
	//		- onAddToRoot
	//		- onLeaveRoot
	//		- onNewItem
	//		- onSetItem

	// Parameters to constructor

	// rootId: String
	//		ID of fabricated root item
	rootId: "$root$",

	// rootLabel: String
	//		Label of fabricated root item
	rootLabel: "ROOT",

	// query: String
	//		Specifies the set of children of the root item.
	// example:
	//	|	{type:'continent'}
	query: null,

	// End of parameters to constructor

	constructor: function(params){
		// summary:
		//		Sets up variables, etc.
		// tags:
		//		private

		// Make dummy root item
		this.root = {
			store: this,
			root: true,
			id: params.rootId,
			label: params.rootLabel,
			children: params.rootChildren	// optional param
		};
	},

	// =======================================================================
	// Methods for traversing hierarchy

	mayHaveChildren: function(/*dojo/data/Item*/ item){
		// summary:
		//		Tells if an item has or may have children.  Implementing logic here
		//		avoids showing +/- expando icon for nodes that we know don't have children.
		//		(For efficiency reasons we may not want to check if an element actually
		//		has children until user clicks the expando node)
		// tags:
		//		extension
		return item === this.root || this.inherited(arguments);
	},

	getChildren: function(/*dojo/data/Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
		// summary:
		//		Calls onComplete() with array of child items of given parent item, all loaded.
		if(parentItem === this.root){
			if(this.root.children){
				// already loaded, just return
				callback(this.root.children);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						this.root.children = items;
						callback(items);
					}),
					onError: onError
				});
			}
		}else{
			this.inherited(arguments);
		}
	},

	// =======================================================================
	// Inspecting items

	isItem: function(/* anything */ something){
		return (something === this.root) ? true : this.inherited(arguments);
	},

	fetchItemByIdentity: function(/* object */ keywordArgs){
		if(keywordArgs.identity == this.root.id){
			var scope = keywordArgs.scope || kernel.global;
			if(keywordArgs.onItem){
				keywordArgs.onItem.call(scope, this.root);
			}
		}else{
			this.inherited(arguments);
		}
	},

	getIdentity: function(/* item */ item){
		return (item === this.root) ? this.root.id : this.inherited(arguments);
	},

	getLabel: function(/* item */ item){
		return	(item === this.root) ? this.root.label : this.inherited(arguments);
	},

	// =======================================================================
	// Write interface

	newItem: function(/* dijit/tree/dndSource.__Item */ args, /*Item*/ parent, /*int?*/ insertIndex){
		// summary:
		//		Creates a new item.   See dojo/data/api/Write for details on args.
		//		Used in drag & drop when item from external source dropped onto tree.
		if(parent === this.root){
			this.onNewRootItem(args);
			return this.store.newItem(args);
		}else{
			return this.inherited(arguments);
		}
	},

	onNewRootItem: function(/* dijit/tree/dndSource.__Item */ /*===== args =====*/){
		// summary:
		//		User can override this method to modify a new element that's being
		//		added to the root of the tree, for example to add a flag like root=true
	},

	pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
		// summary:
		//		Move or copy an item from one parent item to another.
		//		Used in drag & drop
		if(oldParentItem === this.root){
			if(!bCopy){
				// It's onLeaveRoot()'s responsibility to modify the item so it no longer matches
				// this.query... thus triggering an onChildrenChange() event to notify the Tree
				// that this element is no longer a child of the root node
				this.onLeaveRoot(childItem);
			}
		}
		this.inherited(arguments, [childItem,
			oldParentItem === this.root ? null : oldParentItem,
			newParentItem === this.root ? null : newParentItem,
			bCopy,
			insertIndex
		]);
		if(newParentItem === this.root){
			// It's onAddToRoot()'s responsibility to modify the item so it matches
			// this.query... thus triggering an onChildrenChange() event to notify the Tree
			// that this element is now a child of the root node
			this.onAddToRoot(childItem);
		}
	},

	// =======================================================================
	// Handling for top level children

	onAddToRoot: function(/* item */ item){
		// summary:
		//		Called when item added to root of tree; user must override this method
		//		to modify the item so that it matches the query for top level items
		// example:
		//	|	store.setValue(item, "root", true);
		// tags:
		//		extension
		console.log(this, ": item ", item, " added to root");
	},

	onLeaveRoot: function(/* item */ item){
		// summary:
		//		Called when item removed from root of tree; user must override this method
		//		to modify the item so it doesn't match the query for top level items
		// example:
		//	|	store.unsetAttribute(item, "root");
		// tags:
		//		extension
		console.log(this, ": item ", item, " removed from root");
	},

	// =======================================================================
	// Events from data store

	_requeryTop: function(){
		// reruns the query for the children of the root node,
		// sending out an onSet notification if those children have changed
		var oldChildren = this.root.children || [];
		this.store.fetch({
			query: this.query,
			onComplete: lang.hitch(this, function(newChildren){
				this.root.children = newChildren;

				// If the list of children or the order of children has changed...
				if(oldChildren.length != newChildren.length ||
					array.some(oldChildren, function(item, idx){ return newChildren[idx] != item;})){
					this.onChildrenChange(this.root, newChildren);
				}
			})
		});
	},

	onNewItem: function(/* dojo/data/api/Item */ item, /* Object */ parentInfo){
		// summary:
		//		Handler for when new items appear in the store.  Developers should override this
		//		method to be more efficient based on their app/data.
		// description:
		//		Note that the default implementation requeries the top level items every time
		//		a new item is created, since any new item could be a top level item (even in
		//		addition to being a child of another item, since items can have multiple parents).
		//
		//		If developers can detect which items are possible top level items (based on the item and the
		//		parentInfo parameters), they should override this method to only call _requeryTop() for top
		//		level items.  Often all top level items have parentInfo==null, but
		//		that will depend on which store you use and what your data is like.
		// tags:
		//		extension
		this._requeryTop();

		this.inherited(arguments);
	},

	onDeleteItem: function(/*Object*/ item){
		// summary:
		//		Handler for delete notifications from underlying store

		// check if this was a child of root, and if so send notification that root's children
		// have changed
		if(array.indexOf(this.root.children, item) != -1){
			this._requeryTop();
		}

		this.inherited(arguments);
	},

	onSetItem: function(/* item */ item,
					/* attribute-name-string */ attribute,
					/* Object|Array */ oldValue,
					/* Object|Array */ newValue){
		// summary:
		//		Updates the tree view according to changes to an item in the data store.
		//		Developers should override this method to be more efficient based on their app/data.
		// description:
		//		Handles updates to an item's children by calling onChildrenChange(), and
		//		other updates to an item by calling onChange().
		//
		//		Also, any change to any item re-executes the query for the tree's top-level items,
		//		since this modified item may have started/stopped matching the query for top level items.
		//
		//		If possible, developers should override this function to only call _requeryTop() when
		//		the change to the item has caused it to stop/start being a top level item in the tree.
		// tags:
		//		extension

		this._requeryTop();
		this.inherited(arguments);
	}

});

});

},
'xcitec/form/_XButtonSingleRight':function(){
define([
  "dojo/_base/declare",
	"./_XButton"
], function (declare, _XButton) {
  return declare(_XButton, {
	    baseClass: "dijitXcitecBtn dijitXcitecBtnSingleRight"
	});
});
},
'xcitec/BlankSeparator':function(){
define([
	"dojo/_base/declare",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (declare, _Templated, _Widget) {

	var BlankSeparator = declare( [_Widget, _Templated], {
	       templateString: "<div class='blankSeparator'></div>"
	});
	return BlankSeparator;
});

},
'dijit/Menu':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId dom.isDescendant
	"dojo/dom-attr", // domAttr.get domAttr.set domAttr.has domAttr.remove
	"dojo/dom-geometry", // domStyle.getComputedStyle domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.F10
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie"), has("quirks")
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./popup",
	"./DropDownMenu",
	"dojo/ready"
], function(require, array, declare, dom, domAttr, domGeometry, domStyle, keys, lang, on, has, win, winUtils, pm, DropDownMenu, ready){

	// module:
	//		dijit/Menu

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/MenuItem", "dijit/PopupMenuItem", "dijit/CheckedMenuItem", "dijit/MenuSeparator"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.Menu", DropDownMenu, {
		// summary:
		//		A context menu you can assign to multiple elements

		constructor: function(/*===== params, srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified:
			//
			//		- use srcNodeRef.innerHTML as my contents
			//		- replace srcNodeRef with my generated DOM tree

			this._bindings = [];
		},

		// targetNodeIds: [const] String[]
		//		Array of dom node ids of nodes to attach to.
		//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
		targetNodeIds: [],

		// selector: String?
		//		CSS expression to apply this Menu to descendants of targetNodeIds, rather than to
		//		the nodes specified by targetNodeIds themselves.  Useful for applying a Menu to
		//		a range of rows in a table, tree, etc.
		//
		//		The application must require() an appropriate level of dojo/query to handle the selector.
		selector: "",

		// TODO: in 2.0 remove support for multiple targetNodeIds.   selector gives the same effect.
		// So, change targetNodeIds to a targetNodeId: "", remove bindDomNode()/unBindDomNode(), etc.

		/*=====
		// currentTarget: [readonly] DOMNode
		//		For context menus, set to the current node that the Menu is being displayed for.
		//		Useful so that the menu actions can be tailored according to the node
		currentTarget: null,
		=====*/

		// contextMenuForWindow: [const] Boolean
		//		If true, right clicking anywhere on the window will cause this context menu to open.
		//		If false, must specify targetNodeIds.
		contextMenuForWindow: false,

		// leftClickToOpen: [const] Boolean
		//		If true, menu will open on left click instead of right click, similar to a file menu.
		leftClickToOpen: false,

		// refocus: Boolean
		//		When this menu closes, re-focus the element which had focus before it was opened.
		refocus: true,

		postCreate: function(){
			if(this.contextMenuForWindow){
				this.bindDomNode(this.ownerDocumentBody);
			}else{
				array.forEach(this.targetNodeIds, this.bindDomNode, this);
			}
			this.inherited(arguments);
		},

		// thanks burstlib!
		_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
			// summary:
			//		Returns the window reference of the passed iframe
			// tags:
			//		private
			return winUtils.get(this._iframeContentDocument(iframe_el)) ||
				// Moz. TODO: is this available when defaultView isn't?
				this._iframeContentDocument(iframe_el)['__parent__'] ||
				(iframe_el.name && document.frames[iframe_el.name]) || null;	//	Window
		},

		_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
			// summary:
			//		Returns a reference to the document object inside iframe_el
			// tags:
			//		protected
			return iframe_el.contentDocument // W3
				|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
				|| (iframe_el.name && document.frames[iframe_el.name] && document.frames[iframe_el.name].document)
				|| null;	//	HTMLDocument
		},

		bindDomNode: function(/*String|DomNode*/ node){
			// summary:
			//		Attach menu to given node
			node = dom.byId(node, this.ownerDocument);

			var cn;	// Connect node

			// Support context menus on iframes.  Rather than binding to the iframe itself we need
			// to bind to the <body> node inside the iframe.
			if(node.tagName.toLowerCase() == "iframe"){
				var iframe = node,
					window = this._iframeContentWindow(iframe);
				cn = win.body(window.document);
			}else{
				// To capture these events at the top level, attach to <html>, not <body>.
				// Otherwise right-click context menu just doesn't work.
				cn = (node == win.body(this.ownerDocument) ? this.ownerDocument.documentElement : node);
			}


			// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
			var binding = {
				node: node,
				iframe: iframe
			};

			// Save info about binding in _bindings[], and make node itself record index(+1) into
			// _bindings[] array.  Prefix w/_dijitMenu to avoid setting an attribute that may
			// start with a number, which fails on FF/safari.
			domAttr.set(node, "_dijitMenu" + this.id, this._bindings.push(binding));

			// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
			// loading yet, in which case we need to wait for the onload event first, and then connect
			// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
			// we need to monitor keyboard events in addition to the oncontextmenu event.
			var doConnects = lang.hitch(this, function(cn){
				var selector = this.selector,
					delegatedEvent = selector ?
						function(eventType){
							return on.selector(selector, eventType);
						} :
						function(eventType){
							return eventType;
						},
					self = this;
				return [
					// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
					// rather than shift-F10?
					on(cn, delegatedEvent(this.leftClickToOpen ? "click" : "contextmenu"), function(evt){
						evt.stopPropagation();
						evt.preventDefault();

						if((new Date()).getTime() < this._lastKeyDown + 500){
							// Ignore contextmenu/click events that were already processed in keydown handler below.
							// But still call preventDefault() (above) so system context menu doesn't appear.
							return;
						}

						// Schedule context menu to be opened.
						// Note that this won't work will if the click was generated by the keyboard, while
						// focused on a <button> etc.   In that case evt.pageX and evt.pageY are either (0,0) or
						// wherever the mouse cursor is.  See keydown handler below.
						self._scheduleOpen(this, iframe, {x: evt.pageX, y: evt.pageY}, evt.target);
					}),
					on(cn, delegatedEvent("keydown"), function(evt){
						if(evt.keyCode == 93 ||									// context menu key
							(evt.shiftKey && evt.keyCode == keys.F10) ||		// shift-F10
							(this.leftClickToOpen && evt.keyCode == keys.SPACE)	// space key
						){
							evt.stopPropagation();
							evt.preventDefault();

							// Open the menu around evt.target.  Note that "this" and evt.target
							// are likely different, especially for global context menu, where "this" is <body>.
							self._scheduleOpen(this, iframe, null, evt.target);	// no coords - open near evt.target

							this._lastKeyDown = (new Date()).getTime();
						}
					})
				];
			});
			binding.connects = cn ? doConnects(cn) : [];

			if(iframe){
				// Setup handler to [re]bind to the iframe when the contents are initially loaded,
				// and every time the contents change.
				// Need to do this b/c we are actually binding to the iframe's <body> node.
				// Note: can't use connect.connect(), see #9609.

				binding.onloadHandler = lang.hitch(this, function(){
					// want to remove old connections, but IE throws exceptions when trying to
					// access the <body> node because it's already gone, or at least in a state of limbo

					var window = this._iframeContentWindow(iframe),
						cn = win.body(window.document);
					binding.connects = doConnects(cn);
				});
				if(iframe.addEventListener){
					iframe.addEventListener("load", binding.onloadHandler, false);
				}else{
					iframe.attachEvent("onload", binding.onloadHandler);
				}
			}
		},

		unBindDomNode: function(/*String|DomNode*/ nodeName){
			// summary:
			//		Detach menu from given node

			var node;
			try{
				node = dom.byId(nodeName, this.ownerDocument);
			}catch(e){
				// On IE the dom.byId() call will get an exception if the attach point was
				// the <body> node of an <iframe> that has since been reloaded (and thus the
				// <body> node is in a limbo state of destruction.
				return;
			}

			// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
			var attrName = "_dijitMenu" + this.id;
			if(node && domAttr.has(node, attrName)){
				var bid = domAttr.get(node, attrName) - 1, b = this._bindings[bid], h;
				while((h = b.connects.pop())){
					h.remove();
				}

				// Remove listener for iframe onload events
				var iframe = b.iframe;
				if(iframe){
					if(iframe.removeEventListener){
						iframe.removeEventListener("load", b.onloadHandler, false);
					}else{
						iframe.detachEvent("onload", b.onloadHandler);
					}
				}

				domAttr.remove(node, attrName);
				delete this._bindings[bid];
			}
		},

		_scheduleOpen: function(delegatedTarget, iframe, coords, target){
			// summary:
			//		Set timer to display myself.  Using a timer rather than displaying immediately solves
			//		IE problem: without the delay, focus work in "open" causes the system
			//		context menu to appear in spite of evt.preventDefault().
			// delegatedTarget: Element
			//		The node specified in targetNodeIds or matching selector that the menu is being opened for.
			// iframe: HTMLIframeElement?
			//		Set if target is inside the specified iframe.
			// coords: Object
			//		x/y position to center the menu around.  Undefined if menu was opened via keyboard.
			// target: Element
			//		The actual clicked node, either delegatedTarget or a descendant.

			if(!this._openTimer){
				this._openTimer = this.defer(function(){
					delete this._openTimer;
					this._openMyself({
						target: target,
						delegatedTarget: delegatedTarget,
						iframe: iframe,
						coords: coords
					});
				}, 1);
			}
		},

		_openMyself: function(args){
			// summary:
			//		Internal function for opening myself when the user does a right-click or something similar.
			// args:
			//		This is an Object containing:
			//
			//		- target: The node that is being clicked.
			//		- delegatedTarget: The node from this.targetNodeIds or matching this.selector,
			//		  either the same as target or an ancestor of target.
			//		- iframe: If an `<iframe>` is being clicked, iframe points to that iframe
			//		- coords: Mouse cursor x/y coordinates.  Null when opened via keyboard.
			//		  Put menu at specified position in iframe (if iframe specified) or otherwise in viewport.
			//
			//		_openMyself() formerly took the event object, and since various code references
			//		evt.target (after connecting to _openMyself()), using an Object for parameters
			//		(so that old code still works).

			var target = args.target,
				iframe = args.iframe,
				coords = args.coords,
				byKeyboard = !coords;

			// To be used by MenuItem event handlers to tell which node the menu was opened on
			this.currentTarget = args.delegatedTarget;

			// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
			// then near the node the menu is assigned to.
			if(coords){
				if(iframe){
					// Specified coordinates are on <body> node of an <iframe>, convert to match main document
					var ifc = domGeometry.position(iframe, true),
						window = this._iframeContentWindow(iframe),
						scroll = domGeometry.docScroll(window.document);

					var cs = domStyle.getComputedStyle(iframe),
						tp = domStyle.toPixelValue,
						left = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingLeft)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderLeftWidth) : 0),
						top = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingTop)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderTopWidth) : 0);

					coords.x += ifc.x + left - scroll.x;
					coords.y += ifc.y + top - scroll.y;
				}
			}else{
				coords = domGeometry.position(target, true);
				coords.x += 10;
				coords.y += 10;
			}

			var self = this;
			var prevFocusNode = this._focusManager.get("prevNode");
			var curFocusNode = this._focusManager.get("curNode");
			var savedFocusNode = !curFocusNode || (dom.isDescendant(curFocusNode, this.domNode)) ? prevFocusNode : curFocusNode;

			function closeAndRestoreFocus(){
				// user has clicked on a menu or popup
				if(self.refocus && savedFocusNode){
					savedFocusNode.focus();
				}
				pm.close(self);
			}

			pm.open({
				popup: this,
				x: coords.x,
				y: coords.y,
				onExecute: closeAndRestoreFocus,
				onCancel: closeAndRestoreFocus,
				orient: this.isLeftToRight() ? 'L' : 'R'
			});

			// Focus the menu even when opened by mouse, so that a click on blank area of screen will close it
			this.focus();
			if(!byKeyboard){
				// But then (when opened by mouse), mark Menu as passive, so that the first item isn't highlighted.
				// On IE9+ this needs to be on a delay because the focus is asynchronous.
				this.defer(function(){
					this._cleanUp(true);
				});
			}

			this._onBlur = function(){
				this.inherited('_onBlur', arguments);
				// Usually the parent closes the child widget but if this is a context
				// menu then there is no parent
				pm.close(this);
				// don't try to restore focus; user has clicked another part of the screen
				// and set focus there
			};
		},

		destroy: function(){
			array.forEach(this._bindings, function(b){
				if(b){
					this.unBindDomNode(b.node);
				}
			}, this);
			this.inherited(arguments);
		}
	});
});

},
'xcitec/form/AutoCompleteEntitySelect':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/query",
	"dojo/text!./templates/AutoCompleteEntitySelect.html",
	"./TextBox",
    "../xcitecUtil",
    "../_base/ImageFactory",
    "./_XButtonRight",
    "./_XButtonMiddle",
    "./_XButtonSingleRight"
], function (declare, domClass, domConstruct, domStyle, query, template, TextBox, xcitecUtil, ImageFactory) {

	var AutoCompleteEntitySelect = declare( TextBox,	{
	  templateString: template,
	  iconClass: "",
	  iconClassTest: "",
	  entityClass: "",
	  infoIconClass: "",
	  frontIconClass: "",
	  bgColor: '',
	  savedValue: "",
	  selectedIdHolder: null,
	
	  _infoButtonActive: false,
	
	//    autoCompleteInit: null,
	
	  // template attributes
	  disabledChooseAttrSetting: "",
	  disabledInfoAttrSetting: "",
	  readOnlyInfoAttrSetting: "",
	  xbtnInfoClass: "xcitec/form/_XButtonMiddle",
	  widgetsInTemplate: true,
	
	
	  postMixInProperties: function() {
	    this.inherited(arguments);
	    this._infoButtonActive = ("onInfo" in this.params);
	    if (this.entityClass) {
	      this.infoIconClass = "iconXBtn "+ xcitecUtil.createEntityIconClass(this.entityClass);
	    }
	    else {
	      this.infoIconClass = "iconXBtn";
	    }
	    if (this.disabled) {
	      this.required = false;
	      this.disabledChooseAttrSetting = "disabled";
	      this.disabledInfoAttrSetting = "disabled";
	    }
	    if (!this._infoButtonActive) {
	      this.readOnlyInfoAttrSetting = "readOnly='true'";
	    }
	    if (this.readOnly) {
	      this.required = false;
	      this.xbtnInfoClass = "xcitec/form/_XButtonSingleRight";
	      if (!this._infoButtonActive) this.disabledInfoAttrSetting = "disabled";
	      this.disabledChooseAttrSetting = "disabled";
	    }
	    if (this.iconClass) {
	      this._iconClassDef=this.iconClass;
	      this.iconClass = ImageFactory.createIconCss(this.iconClass);
	    } 
	    if (this.frontIconClass) {
	      this.frontIconClass = ImageFactory.createIconCss(this.frontIconClass);
	    }
	    this.selectedIdHolder = query("[name='" + this.id + ".autoCompleteSelectionResult']", this.srcNodeRef)[0];
	  },
	
	  postCreate: function() {
	    this.inherited(arguments);
	
	    if ("onDelete" in this.params) {
	      domClass.add(this.domNode, "closeEntitySelect");
	    }
	    if (!this.iconClass) {
	      domStyle.set(this.imageNode, "display", "none");
	    } 
	    if (!this.frontIconClass) {
	      domStyle.set(this.frontIconNode, "display", "none");
	    }
	    if (this.textbox.value) {
	        this.valueNode.value = this.textbox.value;
	        this.textNode.value = this.textbox.value;
	        this.savedValue = this.textbox.value;
	    }
	    if (this.required) {
	      domClass.add(this.domNode, "dijitXcitecRequired");
	    }
	    if (this.bgColor) {
	      domClass.add(this.domNode, "entitySelect" + this.bgColor.charAt(0).toUpperCase() + this.bgColor.slice(1));
	    }
	//    dojo.setSelectable(this.domNode, false);
	    if (this.selectedIdHolder) {
	        domConstruct.place(this.selectedIdHolder, this.valueNode, "after");
	    }
	//    if (this.readOnly) {
	//      dojo.style(this.chooseButton.domNode, "display", "none");
	//    }
				//	this.connect(this.clear, "onkeydown", this._onInput);
	    this.connect(this.domNode, "onkeydown", this._onInput1);
	    this.connect(this.domNode, "onkeyup", this._onInput2);
	  },
	
	    startup: function() {
	        this.inherited(arguments);
	        this.autoCompleteInit();
	    },
	
	  _onClickInfoButton: function(){
	    this.onInfo();
	  },
	
	  _onClickChooseButton: function(){
	    this.onChoose();
	  },
	
	  onFirstClick: function(){
	      //have to be ovveriden
	  },
	
	  onSecondClick: function(){
	      //have to be ovveriden
	  },
	
	  onInfo: function(){
	  //have to be ovveriden
	      this.onFirstClick();
	  },
	
	  onChoose: function(){
	  //have to be ovveriden
	    this.onSecondClick();
	  },
	
	  onDelete: function() {
	
	  },
	
	  validate: function() {
	        // overrides dijit.form.ValidationTextBox validate function
	  },
	
	  _onInput1: function(e) {
	      this.valueNode.value = this.textNode.value
	//      if ((e.keyCode == dojo.keys.DELETE) || (e.keyCode == dojo.keys.BACKSPACE)) {
	//          this.onClear();
	//          dojo.stopEvent(e);
	//      }
	  },
	
	  _onInput2: function(e) {
	//      if ((e.keyCode == dojo.keys.DELETE) || (e.keyCode == dojo.keys.BACKSPACE)) {
	//          dojo.stopEvent(e);
	//      }
	  },
	
	  onClear: function() {
	      //have to be ovveriden
	  },
	
	    _onClick: function(){
	        if (this.iconClass) {
	            domStyle.set(this.imageNode, "display", "none");
	        }
	
	    },
	
	    _onBlur: function() {
	        if (this.iconClass && (this.textbox.value != "")) {
	            domStyle.set(this.imageNode, "display", "");
	        }
	    },
	
	    _onFocus: function() {
	        if (this.iconClass) {
	            domStyle.set(this.imageNode, "display", "none");
	        }
	    },
	
	    autoCompleteInit: function() {
	
	    }
	});
	return AutoCompleteEntitySelect;
});

},
'dojo/dom-attr':function(){
define(["exports", "./sniff", "./_base/lang", "./dom", "./dom-style", "./dom-prop"],
		function(exports, has, lang, dom, style, prop){
	// module:
	//		dojo/dom-attr
	// summary:
	//		This module defines the core dojo DOM attributes API.

	// TODOC: summary not showing up in output see https://github.com/csnover/js-doc-parse/issues/42

	// =============================
	// Element attribute Functions
	// =============================

	// This module will be obsolete soon. Use dojo/prop instead.

	// dojo/dom-attr.get() should conform to http://www.w3.org/TR/DOM-Level-2-Core/

	// attribute-related functions (to be obsolete soon)

	var forcePropNames = {
			innerHTML:	1,
			className:	1,
			htmlFor:	has("ie"),
			value:		1
		},
		attrNames = {
			// original attribute names
			classname: "class",
			htmlfor: "for",
			// for IE
			tabindex: "tabIndex",
			readonly: "readOnly"
		};

	function _hasAttr(node, name){
		var attr = node.getAttributeNode && node.getAttributeNode(name);
		return attr && attr.specified; // Boolean
	}

	// There is a difference in the presence of certain properties and their default values
	// between browsers. For example, on IE "disabled" is present on all elements,
	// but it is value is "false"; "tabIndex" of <div> returns 0 by default on IE, yet other browsers
	// can return -1.

	exports.has = function hasAttr(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Returns true if the requested attribute is specified on the
		//		given element, and false otherwise.
		// node: DOMNode|String
		//		id or reference to the element to check
		// name: String
		//		the name of the attribute
		// returns: Boolean
		//		true if the requested attribute is specified on the
		//		given element, and false otherwise

		var lc = name.toLowerCase();
		return forcePropNames[prop.names[lc] || name] || _hasAttr(dom.byId(node), attrNames[lc] || name);	// Boolean
	};

	exports.get = function getAttr(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Gets an attribute on an HTML element.
		// description:
		//		Handles normalized getting of attributes on DOM Nodes.
		// node: DOMNode|String
		//		id or reference to the element to get the attribute on
		// name: String
		//		the name of the attribute to get.
		// returns:
		//		the value of the requested attribute or null if that attribute does not have a specified or
		//		default value;
		//
		// example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	require(["dojo/dom-attr", "dojo/dom"], function(domAttr, dom){
		//	|		domAttr.get(dom.byId("nodeId"), "foo");
		//	|		// or we can just pass the id:
		//	|		domAttr.get("nodeId", "foo");
		//	|	});	
		//	|	

		node = dom.byId(node);
		var lc = name.toLowerCase(),
			propName = prop.names[lc] || name,
			forceProp = forcePropNames[propName],
			value = node[propName];		// should we access this attribute via a property or via getAttribute()?

		if(forceProp && typeof value != "undefined"){
			// node's property
			return value;	// Anything
		}
		if(propName != "href" && (typeof value == "boolean" || lang.isFunction(value))){
			// node's property
			return value;	// Anything
		}
		// node's attribute
		// we need _hasAttr() here to guard against IE returning a default value
		var attrName = attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	};

	exports.set = function setAttr(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		// summary:
		//		Sets an attribute on an HTML element.
		// description:
		//		Handles normalized setting of attributes on DOM Nodes.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to set the attribute on
		// name: String|Object
		//		the name of the attribute to set, or a hash of key-value pairs to set.
		// value: String?
		//		the value to set for the attribute, if the name is a string.
		// returns:
		//		the DOM node
		//
		// example:
		//	|	// use attr() to set the tab index
		//	|	require(["dojo/dom-attr"], function(domAttr){
		//	|		domAttr.set("nodeId", "tabIndex", 3);
		//	|	});
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	require(["dojo/dom-attr"],
		//	|	function(domAttr){
		//	|		domAttr.set("formId", {
		//	|			"foo": "bar",
		//	|			"tabIndex": -1,
		//	|			"method": "POST"
		//	|		}
		//	|	});

		node = dom.byId(node);
		if(arguments.length == 2){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				exports.set(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(),
			propName = prop.names[lc] || name,
			forceProp = forcePropNames[propName];
		if(propName == "style" && typeof value != "string"){ // inline'd type check
			// special case: setting a style
			style.set(node, value);
			return node; // DomNode
		}
		if(forceProp || typeof value == "boolean" || lang.isFunction(value)){
			return prop.set(node, name, value);
		}
		// node's attribute
		node.setAttribute(attrNames[lc] || name, value);
		return node; // DomNode
	};

	exports.remove = function removeAttr(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Removes an attribute from an HTML element.
		// node: DOMNode|String
		//		id or reference to the element to remove the attribute from
		// name: String
		//		the name of the attribute to remove

		dom.byId(node).removeAttribute(attrNames[name.toLowerCase()] || name);
	};

	exports.getNodeProp = function getNodeProp(/*DomNode|String*/ node, /*String*/ name){
		// summary:
		//		Returns an effective value of a property or an attribute.
		// node: DOMNode|String
		//		id or reference to the element to remove the attribute from
		// name: String
		//		the name of the attribute
		// returns:
		//		the value of the attribute

		node = dom.byId(node);
		var lc = name.toLowerCase(), propName = prop.names[lc] || name;
		if((propName in node) && propName != "href"){
			// node's property
			return node[propName];	// Anything
		}
		// node's attribute
		var attrName = attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	};
});

},
'dijit/_base/window':function(){
define([
	"dojo/window", // windowUtils.get
	"../main"	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/window

	/*=====
	return {
		// summary:
		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
	};
	=====*/

	dijit.getDocumentWindow = function(doc){
		return windowUtils.get(doc);
	};
});

},
'dojo/store/Observable':function(){
define(["../_base/kernel", "../_base/lang", "../when", "../_base/array" /*=====, "./api/Store" =====*/
], function(kernel, lang, when, array /*=====, Store =====*/){

// module:
//		dojo/store/Observable

var Observable = function(/*Store*/ store){
	// summary:
	//		The Observable store wrapper takes a store and sets an observe method on query()
	//		results that can be used to monitor results for changes.
	//
	// description:
	//		Observable wraps an existing store so that notifications can be made when a query
	//		is performed.
	//
	// example:
	//		Create a Memory store that returns an observable query, and then log some
	//		information about that query.
	//
	//	|	var store = Observable(new Memory({
	//	|		data: [
	//	|			{id: 1, name: "one", prime: false},
	//	|			{id: 2, name: "two", even: true, prime: true},
	//	|			{id: 3, name: "three", prime: true},
	//	|			{id: 4, name: "four", even: true, prime: false},
	//	|			{id: 5, name: "five", prime: true}
	//	|		]
	//	|	}));
	//	|	var changes = [], results = store.query({ prime: true });
	//	|	var observer = results.observe(function(object, previousIndex, newIndex){
	//	|		changes.push({previousIndex:previousIndex, newIndex:newIndex, object:object});
	//	|	});
	//
	//		See the Observable tests for more information.

	var undef, queryUpdaters = [], revision = 0;
	// a Comet driven store could directly call notify to notify observers when data has
	// changed on the backend
	// create a new instance
	store = lang.delegate(store);
	
	store.notify = function(object, existingId){
		revision++;
		var updaters = queryUpdaters.slice();
		for(var i = 0, l = updaters.length; i < l; i++){
			updaters[i](object, existingId);
		}
	};
	var originalQuery = store.query;
	store.query = function(query, options){
		options = options || {};
		var results = originalQuery.apply(this, arguments);
		if(results && results.forEach){
			var nonPagedOptions = lang.mixin({}, options);
			delete nonPagedOptions.start;
			delete nonPagedOptions.count;

			var queryExecutor = store.queryEngine && store.queryEngine(query, nonPagedOptions);
			var queryRevision = revision;
			var listeners = [], queryUpdater;
			results.observe = function(listener, includeObjectUpdates){
				if(listeners.push(listener) == 1){
					// first listener was added, create the query checker and updater
					queryUpdaters.push(queryUpdater = function(changed, existingId){
						when(results, function(resultsArray){
							var atEnd = resultsArray.length != options.count;
							var i, l, listener;
							if(++queryRevision != revision){
								throw new Error("Query is out of date, you must observe() the query prior to any data modifications");
							}
							var removedObject, removedFrom = -1, insertedInto = -1;
							if(existingId !== undef){
								// remove the old one
								for(i = 0, l = resultsArray.length; i < l; i++){
									var object = resultsArray[i];
									if(store.getIdentity(object) == existingId){
										removedObject = object;
										removedFrom = i;
										if(queryExecutor || !changed){// if it was changed and we don't have a queryExecutor, we shouldn't remove it because updated objects would be eliminated
											resultsArray.splice(i, 1);
										}
										break;
									}
								}
							}
							if(queryExecutor){
								// add the new one
								if(changed &&
										// if a matches function exists, use that (probably more efficient)
										(queryExecutor.matches ? queryExecutor.matches(changed) : queryExecutor([changed]).length)){

									var firstInsertedInto = removedFrom > -1 ? 
										removedFrom : // put back in the original slot so it doesn't move unless it needs to (relying on a stable sort below)
										resultsArray.length;
									resultsArray.splice(firstInsertedInto, 0, changed); // add the new item
									insertedInto = array.indexOf(queryExecutor(resultsArray), changed); // sort it
									// we now need to push the chagne back into the original results array
									resultsArray.splice(firstInsertedInto, 1); // remove the inserted item from the previous index
									
									if((options.start && insertedInto == 0) ||
										(!atEnd && insertedInto == resultsArray.length)){
										// if it is at the end of the page, assume it goes into the prev or next page
										insertedInto = -1;
									}else{
										resultsArray.splice(insertedInto, 0, changed); // and insert into the results array with the correct index
									}
								}
							}else if(changed){
								// we don't have a queryEngine, so we can't provide any information
								// about where it was inserted or moved to. If it is an update, we leave it's position alone, other we at least indicate a new object
								if(existingId !== undef){
									// an update, keep the index the same
									insertedInto = removedFrom;
								}else if(!options.start){
									// a new object
									insertedInto = store.defaultIndex || 0;
									resultsArray.splice(insertedInto, 0, changed);
								}
							}
							if((removedFrom > -1 || insertedInto > -1) &&
									(includeObjectUpdates || !queryExecutor || (removedFrom != insertedInto))){
								var copyListeners = listeners.slice();
								for(i = 0;listener = copyListeners[i]; i++){
									listener(changed || removedObject, removedFrom, insertedInto);
								}
							}
						});
					});
				}
				var handle = {};
				// TODO: Remove cancel in 2.0.
				handle.remove = handle.cancel = function(){
					// remove this listener
					var index = array.indexOf(listeners, listener);
					if(index > -1){ // check to make sure we haven't already called cancel
						listeners.splice(index, 1);
						if(!listeners.length){
							// no more listeners, remove the query updater too
							queryUpdaters.splice(array.indexOf(queryUpdaters, queryUpdater), 1);
						}
					}
				};
				return handle;
			};
		}
		return results;
	};
	var inMethod;
	function whenFinished(method, action){
		var original = store[method];
		if(original){
			store[method] = function(value){
				if(inMethod){
					// if one method calls another (like add() calling put()) we don't want two events
					return original.apply(this, arguments);
				}
				inMethod = true;
				try{
					var results = original.apply(this, arguments);
					when(results, function(results){
						action((typeof results == "object" && results) || value);
					});
					return results;
				}finally{
					inMethod = false;
				}
			};
		}
	}
	// monitor for updates by listening to these methods
	whenFinished("put", function(object){
		store.notify(object, store.getIdentity(object));
	});
	whenFinished("add", function(object){
		store.notify(object);
	});
	whenFinished("remove", function(id){
		store.notify(undefined, id);
	});

	return store;
};

lang.setObject("dojo.store.Observable", Observable);

return Observable;
});

},
'dijit/form/MappedTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("msapp")
	"dojo/dom-construct", // domConstruct.place
	"./ValidationTextBox"
], function(declare, has, domConstruct, ValidationTextBox){

	// module:
	//		dijit/form/MappedTextBox

	return declare("dijit.form.MappedTextBox", ValidationTextBox, {
		// summary:
		//		A dijit/form/ValidationTextBox subclass which provides a base class for widgets that have
		//		a visible formatted display value, and a serializable
		//		value in a hidden input field which is actually sent to the server.
		// description:
		//		The visible display may
		//		be locale-dependent and interactive.  The value sent to the server is stored in a hidden
		//		input field which uses the `name` attribute declared by the original widget.  That value sent
		//		to the server is defined by the dijit/form/MappedTextBox.serialize() method and is typically
		//		locale-neutral.
		// tags:
		//		protected

		postMixInProperties: function(){
			this.inherited(arguments);

			// We want the name attribute to go to the hidden <input>, not the displayed <input>,
			// so override _FormWidget.postMixInProperties() setting of nameAttrSetting for IE.
			this.nameAttrSetting = "";
		},

		// Remap name attribute to be mapped to hidden node created in buildRendering(), rather than this.focusNode
		_setNameAttr: "valueNode",

		serialize: function(val /*=====, options =====*/){
			// summary:
			//		Overridable function used to convert the get('value') result to a canonical
			//		(non-localized) string.  For example, will print dates in ISO format, and
			//		numbers the same way as they are represented in javascript.
			// val: anything
			// options: Object?
			// tags:
			//		protected extension
			return val.toString ? val.toString() : ""; // String
		},

		toString: function(){
			// summary:
			//		Returns widget as a printable string using the widget's value
			// tags:
			//		protected
			var val = this.filter(this.get('value')); // call filter in case value is nonstring and filter has been customized
			return val != null ? (typeof val == "string" ? val : this.serialize(val, this.constraints)) : ""; // String
		},

		validate: function(){
			// Overrides `dijit/form/TextBox.validate`
			this.valueNode.value = this.toString();
			return this.inherited(arguments);
		},

		buildRendering: function(){
			// Overrides `dijit/_TemplatedMixin/buildRendering`

			this.inherited(arguments);

			// Create a hidden <input> node with the serialized value used for submit
			// (as opposed to the displayed value).
			// Passing in name as markup rather than relying on _setNameAttr custom setter above
			// to make query(input[name=...]) work on IE. (see #8660).
			// But not doing that for Windows 8 Store apps because it causes a security exception (see #16452).
			this.valueNode = domConstruct.place("<input type='hidden'" +
				((this.name && !has("msapp")) ? ' name="' + this.name.replace(/"/g, "&quot;") + '"' : "") + "/>",
				this.textbox, "after");
		},

		reset: function(){
			// Overrides `dijit/form/ValidationTextBox.reset` to
			// reset the hidden textbox value to ''
			this.valueNode.value = '';
			this.inherited(arguments);
		}
	});
});

},
'slmui/layout/FilterLayout':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/_base/lang',
	'dojo/dom-attr',
	'dojo/dom-construct',
	'dojo/dom-class',
	'dojo/dom-style',
	'dojo/query',
	'dijit/_TemplatedMixin',
	'dijit/_WidgetBase',
	'dijit/registry',
	'xcitec/form/DropDownSelect',
	'../form/MultiSelect'
], function(declare, array, lang, domAttr, domConstruct, domClass, domStyle, query, _TemplatedMixin, _WidgetBase, registry, xDropDownSelect, MultiSelect){
	return declare([_WidgetBase, _TemplatedMixin], {

		widgetsInTemplate: true,

		buildRendering: function(){
			var domNode = this.domNode = this.srcNodeRef || domConstruct.create('div');

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.

			_TemplatedMixin.prototype._attachTemplateNodes.call(this, domNode);

			// collect nested widgets to process them as own

			var moveRightBtn = query('[data-dojo-attach-event="onClick:_moveItemsRight"]', domNode)[0];
			if(moveRightBtn){
				domAttr.set(moveRightBtn, 'id', this.id + '.moveRight');
			}
			var moveLeftBtn = query('[data-dojo-attach-event="onClick:_moveItemsLeft"]', domNode)[0];
			if(moveLeftBtn){
				domAttr.set(moveLeftBtn, 'id', this.id + '.moveLeft');
			}
			var moveUpBtn = query('[data-dojo-attach-event="onClick:_moveItemsUp"]', domNode)[0];
			if(moveUpBtn){
				domAttr.set(moveUpBtn, 'id', this.id + '.moveUp');
			}
			var moveDownBtn = query('[data-dojo-attach-event="onClick:_moveItemsDown"]', domNode)[0];
			if(moveDownBtn){
				domAttr.set(moveDownBtn, 'id', this.id + '.moveDown');
			}

			this._fillContent();
		},

		_fillContent: function(){
			var selectedItemsValues = '';
			if(this.selectedItemsInput){
				this.selectedItemsInput.setAttribute('autocomplete', 'off');
				selectedItemsValues = this.selectedItemsInput.value;
			}

			var availableItemsWidget = this.availableItemsWidget = new MultiSelect({srcNodeRef: this.listAvailableItemsNode, style: 'width: 200px!important;', isTree: true});
			var selectedItemsWidget = this.selectedItemsWidget = new MultiSelect({srcNodeRef: this.listSelectedItemsNode, style: 'width: 200px!important;', isTree: false});

			function show(/*DomNode*/node){
				domStyle.set(node, 'display', 'block');
			}

			function hide(/*DomNode*/node){
				domStyle.set(node, 'display', 'none');
			}

			show(availableItemsWidget.rootLoadingIndicator);
			show(selectedItemsWidget.rootLoadingIndicator);
			hide(availableItemsWidget.containerNode);
			hide(selectedItemsWidget.containerNode);

			this._elementsLoadingDeferred = availableItemsWidget.expandAll().then(function(){
				availableItemsWidget.collapseAll().then(function(){
					availableItemsWidget.expandNodes();
				}).then(function(){
					hide(availableItemsWidget.rootLoadingIndicator);
					hide(selectedItemsWidget.rootLoadingIndicator);
					show(availableItemsWidget.containerNode);
					show(selectedItemsWidget.containerNode);
				});
			}).then(function(){
					array.forEach(selectedItemsValues.split(','), function(itemid){
						var node = availableItemsWidget.getNodesByItem(itemid);
						if(node.length){
							availableItemsWidget.setSelectedItem(node[0], true, false);
						}
					});
					selectedItemsWidget.addSelected(availableItemsWidget);
					selectedItemsWidget.set('selectedNodes', []);
				});
		},

		postCreate: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "filterLayout");
		},

		startup: function(){
			if(this._started){
				return;
			}

			var _this = this;

			this._elementsLoadingDeferred.then(function(){
				if(_this.sortDropDown){

					var sortName = domAttr.get(_this.sortDropDown, 'name');
					_this.sort = new xDropDownSelect({name: sortName, srcNodeRef: _this.sortDropDown, style: 'width: 100%;'});
					var sortInput = _this.sortDropDownInput;
					array.forEach(_this.selectedItemsWidget.store.query(), function(item){
						if(item.id != 'root' && !item.notSortable){
							_this.sort.appendOption(item.name, item.id);
						}
					});
					if(_this.sortDropDownInput.value){
						var dropDownMenu = _this.sort.getMenuItems();
						array.forEach(dropDownMenu, function (node){
							if(node.getAttribute('hiddenValue') == sortInput.value){
								_this.sort._setSelected(node);
							}
						})
					}
					domConstruct.destroy(_this.sortDropDownInput);
				}
			});

			var moveRightBtn = registry.byId(this.id + ".moveRight");
			if(moveRightBtn){
				moveRightBtn.onClick = lang.hitch(this, this._moveItemsRight);
			}
			var moveLeftBtn = registry.byId(this.id + ".moveLeft");
			if(moveLeftBtn){
				moveLeftBtn.onClick = lang.hitch(this, this._moveItemsLeft);
			}
			var moveUpBtn = registry.byId(this.id + ".moveUp");
			if(moveUpBtn){
				moveUpBtn.onClick = lang.hitch(this, this._moveItemsUp);
			}
			var moveDownBtn = registry.byId(this.id + ".moveDown");
			if(moveDownBtn){
				moveDownBtn.onClick = lang.hitch(this, this._moveItemsDown);
			}
		},

		_setSelectedItems: function(/*String*/ value){
			if(this.selectedItemsInput){
				this.selectedItemsInput.value = value;
			}
		},

		_updateSelectedItems: function(/*Array*/ items){
			var val = array.map(items,
				function(item){
					return item.id
				}).join(",");
			this._setSelectedItems(val);
		},

		_updateDropDown: function(/*Array*/ items, /*String*/ action){
			if(action == "remove"){
				var dropDownMenu = this.sort.getMenuItems();
				array.forEach(dropDownMenu, function(n){
					var ident = false;
					array.forEach(items, function(item){
						if(item.id){
							if(item.id == n.getAttribute("hiddenValue")){
								ident = true;
							}
						} else{
							if(item.name == n.lastChild.innerHTML){
								ident = true;
							}
						}
					}, this);
					if(ident){
						this.sort.deleteOption(n.getAttribute("hiddenValue"))
					}
				}, this);
			} else if(action == "append"){
				array.forEach(items, function(item){
					if(!item.notSortable){
						this.sort.appendOption(item.name, item.id);
					}
				}, this)
			}
		},

		_moveItemsLeft: function(){
			var selectedItems = this.selectedItemsWidget;
			if(this.sortDropDown){
				this._updateDropDown(selectedItems.selectedItems, "remove");
			}
			this.availableItemsWidget.addSelected(selectedItems);
			this._updateSelectedItems(selectedItems.getItems());
		},

		_moveItemsRight: function(){
			var selectedItems = this.selectedItemsWidget;
			selectedItems.addSelected(this.availableItemsWidget);
			this._updateSelectedItems(selectedItems.getItems());
			if(this.sortDropDown){
				this._updateDropDown(selectedItems.selectedItems, "append");
			}
		},

		_moveItemsUp: function(){
			var selectedItems = this.selectedItemsWidget;
			selectedItems.moveSelectedUp();
			this._updateSelectedItems(selectedItems.getItems());
		},
		_moveItemsDown: function(){
			var selectedItems = this.selectedItemsWidget;
			selectedItems.moveSelectedDown();
			this._updateSelectedItems(selectedItems.getItems());
		},

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		}
	});
});
},
'dojo/dnd/Moveable':function(){
define([
	"../_base/array", "../_base/declare", "../_base/lang",
	"../dom", "../dom-class", "../Evented", "../on", "../topic", "../touch", "./common", "./Mover", "../_base/window"
], function(array, declare, lang, dom, domClass, Evented, on, topic, touch, dnd, Mover, win){

// module:
//		dojo/dnd/Moveable


var Moveable = declare("dojo.dnd.Moveable", [Evented], {
	// summary:
	//		an object, which makes a node movable

	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,

	constructor: function(node, params){
		// node: Node
		//		a node (or node's id) to be moved
		// params: Moveable.__MoveableArgs?
		//		optional parameters
		this.node = dom.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dom.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : Mover;
		this.events = [
			on(this.handle, touch.press, lang.hitch(this, "onMouseDown")),
			// cancel text selection and text dragging
			on(this.handle, "dragstart",   lang.hitch(this, "onSelectStart")),
			on(this.handle, "selectstart",   lang.hitch(this, "onSelectStart"))
		];
	},

	// markup methods
	markupFactory: function(params, node, Ctor){
		return new Ctor(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.events = this.node = this.handle = null;
	},

	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown/ontouchstart, creates a Mover for the node
		// e: Event
		//		mouse/touch event
		if(this.skip && dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				on(this.handle, touch.move, lang.hitch(this, "onMouseMove")),
				on(this.handle, touch.release, lang.hitch(this, "onMouseUp"))
			);
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			this.onDragDetected(e);
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove, used only for delayed drags
		// e: Event
		//		mouse/touch event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			this.events.pop().remove();
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dnd.isFormElement(e)){
			e.stopPropagation();
			e.preventDefault();
		}
	},

	// local events
	onDragDetected: function(/*Event*/ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/*Mover*/ mover){
		// summary:
		//		called before every move operation
		topic.publish("/dnd/move/start", mover);
		domClass.add(win.body(), "dojoMove");
		domClass.add(this.node, "dojoMoveItem");
	},
	onMoveStop: function(/*Mover*/ mover){
		// summary:
		//		called after every move operation
		topic.publish("/dnd/move/stop", mover);
		domClass.remove(win.body(), "dojoMove");
		domClass.remove(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/*===== mover, e =====*/){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		// mover: Mover
		// e: Event

		// default implementation does nothing
	},
	onMove: function(mover, leftTop /*=====, e =====*/){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		// mover: Mover
		// leftTop: Object
		// e: Event
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/*===== mover, leftTop =====*/){
		// summary:
		//		called before every incremental move; can be overwritten.
		// mover: Mover
		// leftTop: Object

		// default implementation does nothing
	},
	onMoved: function(/*===== mover, leftTop =====*/){
		// summary:
		//		called after every incremental move; can be overwritten.
		// mover: Mover
		// leftTop: Object

		// default implementation does nothing
	}
});

/*=====
Moveable.__MoveableArgs = declare([], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dnd.Mover
});
=====*/

return Moveable;
});

},
'dojo/store/util/QueryResults':function(){
define(["../../_base/array", "../../_base/lang", "../../when"
], function(array, lang, when){

// module:
//		dojo/store/util/QueryResults

var QueryResults = function(results){
	// summary:
	//		A function that wraps the results of a store query with additional
	//		methods.
	// description:
	//		QueryResults is a basic wrapper that allows for array-like iteration
	//		over any kind of returned data from a query.  While the simplest store
	//		will return a plain array of data, other stores may return deferreds or
	//		promises; this wrapper makes sure that *all* results can be treated
	//		the same.
	//
	//		Additional methods include `forEach`, `filter` and `map`.
	// results: Array|dojo/promise/Promise
	//		The result set as an array, or a promise for an array.
	// returns:
	//		An array-like object that can be used for iterating over.
	// example:
	//		Query a store and iterate over the results.
	//
	//	|	store.query({ prime: true }).forEach(function(item){
	//	|		//	do something
	//	|	});

	if(!results){
		return results;
	}

	var isPromise = !!results.then;
	// if it is a promise it may be frozen
	if(isPromise){
		results = lang.delegate(results);
	}
	function addIterativeMethod(method){
		// Always add the iterative methods so a QueryResults is
		// returned whether the environment is ES3 or ES5
		results[method] = function(){
			var args = arguments;
			var result = when(results, function(results){
				Array.prototype.unshift.call(args, results);
				return QueryResults(array[method].apply(array, args));
			});
			// forEach should only return the result of when()
			// when we're wrapping a promise
			if(method !== "forEach" || isPromise){
				return result;
			}
		};
	}

	addIterativeMethod("forEach");
	addIterativeMethod("filter");
	addIterativeMethod("map");
	if(results.total == null){
		results.total = when(results, function(results){
			return results.length;
		});
	}
	return results; // Object
};

lang.setObject("dojo.store.util.QueryResults", QueryResults);

return QueryResults;

});

},
'dijit/form/CheckBox':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/has",		// has("dijit-legacy-requires")
	"dojo/query", // query
	"dojo/ready",
	"./ToggleButton",
	"./_CheckBoxMixin",
	"dojo/text!./templates/CheckBox.html",
	"dojo/NodeList-dom", // NodeList.addClass/removeClass
	"../a11yclick"	// template uses ondijitclick
], function(require, declare, domAttr, has, query, ready, ToggleButton, _CheckBoxMixin, template){

	// module:
	//		dijit/form/CheckBox

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/RadioButton"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.form.CheckBox", [ToggleButton, _CheckBoxMixin], {
		// summary:
		//		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//
		//		There are two modes:
		//
		//		1. High contrast mode
		//		2. Normal mode
		//
		//		In case 1, the regular html inputs are shown and used by the user.
		//		In case 2, the regular html inputs are invisible but still used by
		//		the user. They are turned quasi-invisible and overlay the background-image.

		templateString: template,

		baseClass: "dijitCheckBox",

		_setValueAttr: function(/*String|Boolean*/ newValue, /*Boolean*/ priorityChange){
			// summary:
			//		Handler for value= attribute to constructor, and also calls to
			//		set('value', val).
			// description:
			//		During initialization, just saves as attribute to the `<input type=checkbox>`.
			//
			//		After initialization,
			//		when passed a boolean, controls whether or not the CheckBox is checked.
			//		If passed a string, changes the value attribute of the CheckBox (the one
			//		specified as "value" when the CheckBox was constructed
			//		(ex: `<input data-dojo-type="dijit/CheckBox" value="chicken">`).
			//
			//		`widget.set('value', string)` will check the checkbox and change the value to the
			//		specified string.
			//
			//		`widget.set('value', boolean)` will change the checked state.

			if(typeof newValue == "string"){
				this.inherited(arguments);
				newValue = true;
			}
			if(this._created){
				this.set('checked', newValue, priorityChange);
			}
		},
		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		If the CheckBox is checked, returns the value attribute.
			//		Otherwise returns false.
			return this.checked && this._get("value");
		},

		// Override behavior from Button, since we don't have an iconNode or valueNode
		_setIconClassAttr: null,
		_setNameAttr: "focusNode",

		postMixInProperties: function(){
			this.inherited(arguments);

			// Need to set initial checked state via node.setAttribute so that form submit works
			// and IE8 radio button tab order is preserved.
			// domAttr.set(node, "checked", bool) doesn't work on IE until node has been attached
			// to <body>, see #8666
			this.checkedAttrSetting = "";
		},

		 _fillContent: function(){
			// Override Button::_fillContent() since it doesn't make sense for CheckBox,
			// since CheckBox doesn't even have a container
		},

		_onFocus: function(){
			if(this.id){
				query("label[for='"+this.id+"']").addClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},

		_onBlur: function(){
			if(this.id){
				query("label[for='"+this.id+"']").removeClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/DropDownButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // hitch
	"dojo/query", // query
	"../registry", // registry.byNode
	"../popup", // dijit.popup2.hide
	"./Button",
	"../_Container",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownButton.html"
], function(declare, lang, query, registry, popup, Button, _Container, _HasDropDown, template){

	// module:
	//		dijit/form/DropDownButton

	return declare("dijit.form.DropDownButton", [Button, _Container, _HasDropDown], {
		// summary:
		//		A button with a drop down
		//
		// example:
		// |	<button data-dojo-type="dijit/form/DropDownButton">
		// |		Hello world
		// |		<div data-dojo-type="dijit/Menu">...</div>
		// |	</button>
		//
		// example:
		// |	var button1 = new DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
		// |	win.body().appendChild(button1);
		//

		baseClass: "dijitDropDownButton",

		templateString: template,

		_fillContent: function(){
			// Overrides Button._fillContent().
			//
			// My inner HTML contains both the button contents and a drop down widget, like
			// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
			// The first node is assumed to be the button content. The widget is the popup.

			if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
				//FIXME: figure out how to filter out the widget and use all remaining nodes as button
				//	content, not just nodes[0]
				var nodes = query("*", this.srcNodeRef);
				this.inherited(arguments, [nodes[0]]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){
				return;
			}

			// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
			// make it invisible, and store a reference to pass to the popup code.
			if(!this.dropDown && this.dropDownContainer){
				var dropDownNode = query("[widgetId]", this.dropDownContainer)[0];
				if(dropDownNode){
					this.dropDown = registry.byNode(dropDownNode);
				}
				delete this.dropDownContainer;
			}
			if(this.dropDown){
				popup.hide(this.dropDown);
			}

			this.inherited(arguments);
		},

		isLoaded: function(){
			// Returns whether or not we are loaded - if our dropdown has an href,
			// then we want to check that.
			var dropDown = this.dropDown;
			return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
		},

		loadDropDown: function(/*Function*/ callback){
			// Default implementation assumes that drop down already exists,
			// but hasn't loaded it's data (ex: ContentPane w/href).
			// App must override if the drop down is lazy-created.
			var dropDown = this.dropDown;
			var handler = dropDown.on("load", lang.hitch(this, function(){
				handler.remove();
				callback();
			}));
			dropDown.refresh();		// tell it to load
		},

		isFocusable: function(){
			// Overridden so that focus is handled by the _HasDropDown mixin, not by
			// the _FormWidget mixin.
			return this.inherited(arguments) && !this._mouseDown;
		}
	});
});

},
'dijit/_base/scroll':function(){
define([
	"dojo/window", // windowUtils.scrollIntoView
	"../main"	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/scroll

	/*=====
	return {
		// summary:
		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
	};
	=====*/

	dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
		// summary:
		//		Scroll the passed node into view, if it is not already.
		//		Deprecated, use `windowUtils.scrollIntoView` instead.

		windowUtils.scrollIntoView(node, pos);
	};
});

},
'dojo/regexp':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang){

// module:
//		dojo/regexp

var regexp = {
	// summary:
	//		Regular expressions and Builder resources
};
lang.setObject("dojo.regexp", regexp);

regexp.escapeString = function(/*String*/str, /*String?*/except){
	// summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	// summary:
	//		Builds a regular expression that groups subexpressions
	// description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return regexp.group(b.join("|"), nonCapture); // String
};

regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

return regexp;
});

},
'xcitec/xlayout/_ContentAreaPaneItemV2SingleColumn':function(){
define([
         "dojo/_base/declare",
         "dojo/dom-attr",
         "dojo/dom-construct",
         "dojo/dom-class",
         "dojo/dom-style",
         "dojo/query",
         "dijit/registry",
         "../layout/_ContentAreaPaneItem"
       ], function (declare, domAttr, domConstruct, domClass, domStyle, query, registry, _ContentAreaPaneItem) {
  return declare(_ContentAreaPaneItem, {

    templateString: '<div class="dijitReset question dojoDndItem questionSingleColumn" data-dojo-attach-point="containerNode, controlHolderNode">' +
                    '<span class="question_wrapper" data-dojo-attach-point="iconsContainerNode">' +
                    '<img src="${_blankGif}" data-dojo-attach-event="onclick:_onChangedNewClick" class="iconMarking" data-dojo-attach-point="iconMarkingChangedNew" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" data-dojo-attach-event="onclick:_onChangedClick" class="iconMarking" data-dojo-attach-point="iconMarkingChanged" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" class="iconMarking" data-dojo-attach-point="iconMarkingKo" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" class="iconMarking" data-dojo-attach-point="iconMarkingCritical" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" class="iconMarking" data-dojo-attach-point="iconMarkingScore" title="" width="1" height="1" alt="" />' +
                    '</span>' +
                    '</div>',
    templateStringV2: '<div class="dijitReset question designMode dojoDndItem questionSingleColumn" data-dojo-attach-point="containerNode, controlHolderNode">' +
                      '<span class="question_wrapper" data-dojo-attach-point="iconsContainerNode">' +
                      '<span class="dijitReset dijitInline questionIcon questionIconDelete" data-dojo-attach-event="onclick: _onDeleteClick"></span>' +
                      '<span class="dijitReset dijitInline questionIcon questionIconSettings" data-dojo-attach-event="onclick: _onSettingsClick"></span>' +
                      '</span>' +
                      '</div>',
    templateStringV3: '<div class="dijitReset question designMode styleElementHolder dojoDndItem" data-dojo-attach-point="containerNode, controlHolderNode">' +
                      '<span class="question_wrapper" data-dojo-attach-point="iconsContainerNode">' +
                      '<span class="dijitReset dijitInline questionIcon questionIconSettings" data-dojo-attach-event="onclick: _onSettingsClick"></span>' +
                      '<span class="dijitReset dijitInline questionIcon questionIconDelete" data-dojo-attach-event="onclick: _onDeleteClick"></span>' +
                      '</span>' +
                      '</div>',

    designMode: false,

    parent: null,
    isStyleElement: false,

    postMixInProperties: function() {
      this.inherited(arguments);
      // if in design mode - use another template
      if (this.designMode) {
        if (domAttr.get(this.controlNode, 'data-dojo-type') == 'xcitec/xlayout/StyleElement') {
          this.templateString = this.templateStringV3;
          this.isStyleElement = true;
        } else {
          this.templateString = this.templateStringV2;
        }
        domAttr.set(this.controlNode, 'designMode', 'true');
      } else {
        // if in view mode - StyleElement empty line must be without text
        if (domAttr.get(this.controlNode, 'elementType') == 'emptyline') {
          domAttr.set(this.controlNode, 'elementType', 'emptylineview');
        }
      }
    },

    startup: function() {
      this.inherited(arguments);
      if (!this.designMode) {
        var question = query(">", this.domNode)[1];
        if (question) {
          if (registry.byNode(question).isChangedSet()) {
            domClass.add(this.iconMarkingChanged, "iconMarkingChanged");
            domStyle.set(this.iconMarkingChanged, "display", "inline");
          }
          if (registry.byNode(question).isChangedNewSet()) {
            domClass.add(this.iconMarkingChangedNew, "iconMarkingChangedNew");
            domStyle.set(this.iconMarkingChangedNew, "display", "inline");
          }
          if (registry.byNode(question).isChangedOldSet()) {
            domClass.add(this.containerNode, "markingChangedOld");
          }
          if(registry.byNode(question).isScoreSet()) {
            domClass.add(this.iconMarkingScore, "iconMarkingScore");
            domStyle.set(this.iconMarkingScore, "display", "inline");
            if (registry.byNode(question).score != "true") {
              domAttr.set(this.iconMarkingScore, "title", registry.byNode(question).score);
            }
          }
          if (registry.byNode(question).isCriticalSet()) {
            domClass.add(this.iconMarkingCritical, "iconMarkingCritical");
            domStyle.set(this.iconMarkingCritical, "display", "inline");
            if (registry.byNode(question).critical != "true") {
              domAttr.set(this.iconMarkingCritical, "title", registry.byNode(question).critical);
            }
          }
          if (registry.byNode(question).isKoSet()) {
            domClass.add(this.iconMarkingKo, "iconMarkingKo");
            domStyle.set(this.iconMarkingKo, "display", "inline");
            if (registry.byNode(question).ko != "true") {
              domAttr.set(this.iconMarkingKo, "title", registry.byNode(question).ko);
            }
          }
        }
      }
      if(this.controlWidget && this.controlWidget.insertActionIcons){
        this.controlWidget.insertActionIcons(this.iconsContainerNode);
      }
    },

    /* actions for buttons (delete and settings) */
    _onDeleteClick: function() {
      var nodes = query(">", this.domNode);
      var question;
      if (this.isStyleElement) {
        question = nodes[1];
      } else {
        question = nodes[0];
      }
      if (question) {
        if (registry.byNode(question).isDeleteSet()) {
          registry.byNode(question).onDelete();
        } else {
          if (this.parent.parent.isDeleteSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onDelete();
          } else {
            domConstruct.destroy(this.domNode);
            this.parent.parent.updateColumnsOrder();
          }
        }
      }
    },

    _onSettingsClick: function() {
      var nodes = query(">", this.domNode);
      var question;
      if (this.isStyleElement) {
        question = nodes[1];
      } else {
        question = nodes[0];
      }
      if (question) {
        if (registry.byNode(question).isSettingsSet()) {
          registry.byNode(question).onSettings();
        } else {
          if (this.parent.parent.isSettingsSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onSettings();
          }
        }
      }
    },

    _onChangedClick: function() {
      var nodes = query(">", this.domNode);
      var question;
      if (this.isStyleElement) {
        question = nodes[1];
      } else {
        question = nodes[0];
      }
      if (question) {
        if (registry.byNode(question).isChangedSet()) {
          registry.byNode(question).onChanged();
        } else {
          if (this.parent.parent.isChangedSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onChanged();
          }
        }
      }
    },

    _onChangedNewClick: function() {
      var nodes = query(">", this.domNode);
      var question;
      if (this.isStyleElement) {
        question = nodes[1];
      } else {
        question = nodes[0];
      }
      if (question) {
        if (registry.byNode(question).isChangedNewSet()) {
          registry.byNode(question).onChangedNew();
        } else {
          if (this.parent.parent.isChangedNewSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onChangedNew();
          }
        }
      }
    }
  });
});
},
'xcitec/WikiText':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dijit/_WidgetBase",
  "./xcitecUtil"
], function (declare, domClass, domConstruct, _Widget, xcitecUtil) {

	var WikiText = declare( _Widget, {
	
		text: "",
	
		buildRendering: function(){
			this.inherited(arguments);
	    domClass.add(this.domNode, "wikiText");  
			if(this.text){
				this.text = xcitecUtil.toHtml(this.text);
				domConstruct.place(this.text, this.domNode, "first");
			}
		}
	});
	return WikiText;
});

},
'xcitec/layout/FormPanel':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
  "dijit/_WidgetsInTemplateMixin",
	"dijit/_WidgetBase",
  "dijit/registry"
], function (array, declare, domClass, domConstruct, query, _Templated, _WidgetsInTemplateMixin, _Widget, registry) {

	var FormPanel = declare( [_Widget, _Templated, _WidgetsInTemplateMixin], {
	
	    // layout widget for views with one of standart layouts and button(s) after it
	
	    baseClass: 'formPanel',
	
	    background: false,
	
	    templateString:"<div class='${baseClass}'><div class='formPanelContent' data-dojo-attach-point='containerNode'></div>" +
                     "<div class='formPanelButtons' data-dojo-attach-point='buttonNode'></div>" +
                     "<div class='formPanel_corner formPanel_tl'></div>" +
                     "<div class='formPanel_corner formPanel_tr'></div>" +
                     "<div class='formPanel_corner formPanel_bl'></div>" +
                     "<div class='formPanel_corner formPanel_br'></div></div>",

    _fillContent: function() {
	        this.inherited(arguments);
	        // find buttons and place them to buttonNode
	        var buttonPanel = query("[xcitecAttachPoint]", this.domNode)[0];
	            if (buttonPanel) {
	            var buttons = query(" >", buttonPanel);
	            var _this = this;
	            array.forEach(buttons, function(button) {
	                domConstruct.place(button, _this.buttonNode, "last");
	            });
	            domConstruct.destroy(buttonPanel);
	        }
	
	        if (this.background) {
	            domClass.add(this.domNode, "panelBgViolet");
	        }
	    },
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		}
	});
	return FormPanel;
});

},
'dojo/dnd/Avatar':function(){
define([
	"../_base/declare",
	"../_base/window",
	"../dom",
	"../dom-attr",
	"../dom-class",
	"../dom-construct",
	"../hccss",
	"../query"
], function(declare, win, dom, domAttr, domClass, domConstruct, has, query){

// module:
//		dojo/dnd/Avatar

return declare("dojo.dnd.Avatar", null, {
	// summary:
	//		Object that represents transferred DnD items visually
	// manager: Object
	//		a DnD manager object

	constructor: function(manager){
		this.manager = manager;
		this.construct();
	},

	// methods
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need

		var a = domConstruct.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = domConstruct.create("tbody", null, a),
			tr = domConstruct.create("tr", null, b),
			td = domConstruct.create("td", null, tr),
			k = Math.min(5, this.manager.nodes.length), i = 0;

		if(has("highcontrast")){
			domConstruct.create("span", {
				id : "a11yIcon",
				innerHTML : this.manager.copy ? '+' : "<"
			}, td)
		}
		domConstruct.create("span", {
			innerHTML: source.generateText ? this._generateText() : ""
		}, td);

		// we have to set the opacity on IE only after the node is live
		domAttr.set(tr, {
			"class": "dojoDndAvatarHeader",
			style: {opacity: 0.9}
		});
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = domConstruct.create("table"),
						tbody = domConstruct.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = domConstruct.create("tr", null, b);
			td = domConstruct.create("td", null, tr);
			td.appendChild(node);
			domAttr.set(tr, {
				"class": "dojoDndAvatarItem",
				style: {opacity: (9 - i) / 10}
			});
		}
		this.node = a;
	},
	destroy: function(){
		// summary:
		//		destructor for the avatar; called to remove all references so it can be garbage-collected
		domConstruct.destroy(this.node);
		this.node = false;
	},
	update: function(){
		// summary:
		//		updates the avatar to reflect the current DnD state
		domClass.toggle(this.node, "dojoDndAvatarCanDrop", this.manager.canDropFlag);
		if(has("highcontrast")){
			var icon = dom.byId("a11yIcon");
			var text = '+';   // assume canDrop && copy
			if (this.manager.canDropFlag && !this.manager.copy){
				text = '< '; // canDrop && move
			}else if (!this.manager.canDropFlag && !this.manager.copy){
				text = "o"; //!canDrop && move
			}else if(!this.manager.canDropFlag){
				text = 'x';  // !canDrop && copy
			}
			icon.innerHTML=text;
		}
		// replace text
		query(("tr.dojoDndAvatarHeader td span" +(has("highcontrast") ? " span" : "")), this.node).forEach(
			function(node){
				node.innerHTML = this.manager.source.generateText ? this._generateText() : "";
			}, this);
	},
	_generateText: function(){
		// summary:
		//		generates a proper text to reflect copying or moving of items
		return this.manager.nodes.length.toString();
	}
});

});

},
'dojo/string':function(){
define([
	"./_base/kernel",	// kernel.global
	"./_base/lang"
], function(kernel, lang){

// module:
//		dojo/string

var string = {
	// summary:
	//		String utilities for Dojo
};
lang.setObject("dojo.string", string);

string.rep = function(/*String*/str, /*Integer*/num){
	// summary:
	//		Efficiently replicate a string `n` times.
	// str:
	//		the string to replicate
	// num:
	//		number of times to replicate the string

	if(num <= 0 || !str){ return ""; }

	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	// text:
	//		the string to pad
	// size:
	//		length to provide padding
	// ch:
	//		character to pad, defaults to '0'
	// end:
	//		adds padding at the end if true, otherwise pads at start
	// example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	// map:
	//		hash to search for substitutions
	// transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	// thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	// example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	// example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	// example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || kernel.global;
	transform = transform ?
		lang.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = lang.getObject(key, false, map);
			if(format){
				value = lang.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

string.trim = String.prototype.trim ?
	lang.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

/*=====
 string.trim = function(str){
	 // summary:
	 //		Trims whitespace from both sides of the string
	 // str: String
	 //		String to be trimmed
	 // returns: String
	 //		Returns the trimmed string
	 // description:
	 //		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	 //		The short yet performant version of this function is dojo/_base/lang.trim(),
	 //		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	 return "";	// String
 };
 =====*/

	return string;
});

},
'dijit/_Contained':function(){
define([
	"dojo/_base/declare", // declare
	"./registry"	// registry.getEnclosingWidget(), registry.byNode()
], function(declare, registry){

	// module:
	//		dijit/_Contained

	return declare("dijit._Contained", null, {
		// summary:
		//		Mixin for widgets that are children of a container widget
		// example:
		//	|	// make a basic custom widget that knows about its parents
		//	|	declare("my.customClass",[dijit._WidgetBase, dijit._Contained],{});

		_getSibling: function(/*String*/ which){
			// summary:
			//		Returns next or previous sibling
			// which:
			//		Either "next" or "previous"
			// tags:
			//		private
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			return node && registry.byNode(node);	// dijit/_WidgetBase
		},

		getPreviousSibling: function(){
			// summary:
			//		Returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous"); // dijit/_WidgetBase
		},

		getNextSibling: function(){
			// summary:
			//		Returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next"); // dijit/_WidgetBase
		},

		getIndexInParent: function(){
			// summary:
			//		Returns the index of this widget within its container parent.
			//		It returns -1 if the parent does not exist, or if the parent
			//		is not a dijit/_Container

			var p = this.getParent();
			if(!p || !p.getIndexOfChild){
				return -1; // int
			}
			return p.getIndexOfChild(this); // int
		}
	});
});

},
'xcitec/layout/MultiLanguageContainer':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "../xcitecUtil"
], function (dojo, domStyle, declare, array, lang, domAttr, domClass, domConstruct, query, _Templated, _Widget, registry, xcitecUtil) {

	var MultiLanguageContainer = declare( [_Widget, _Templated], {
	
	    templateString: '<table class="dijitReset ${baseClass}" cellspacing="0" cellpadding="0"><tbody data-dojo-attach-point="containerNode"></tbody></table>',
	
	    baseClass: "mlContainer",
	    // 'tr' containers with controls (Array)
	    containers: null,
	    // state of inner controls
	    isVisible: false,
	
	    // reference to button widget
	    buttonWidget: null,
	
	    // control states
	    readonly: false,
	    required: false,
	    disabled: false,
	    error: false,
	
		postMixInProperties: function() {
		    this.containers = [];
	        this.inherited(arguments);
	    },
	
	
	    _fillContent: function() {
	        var controls = query(">", this.srcNodeRef);
	        var _this = this;
	        // if no selected element - then the first element becomes selected
	        var selExists = false;
	        array.forEach(controls, function(node) {
	            if (xcitecUtil.str2bool(domAttr.get(node, "selected"))) {
	                selExists = true;
	            }
	        });
	        if (!selExists) {
	            if (controls[0]) {
	                domAttr.set(controls[0], "selected", "true");
	            }
	        }
	        // place controls in table
	        this.containers = [];
	        array.forEach(controls, function(node, index) {
	            domStyle.set(node, "width", "100%");
	            if (_this.readonly) {
	                domAttr.set(node, "readonly", "true");
	            }
	            if (_this.disabled) {
	                domAttr.set(node, "disabled", "true");
	            }
	            var tr = domConstruct.create("tr");
	            var td = domConstruct.create("td", {}, tr, "last");
	//            if (index != (controls.length - 1)) {
	                domClass.add(td, "mlContainerItem");
	//            }
	            td.appendChild(node);
	            // show/hide controls according selected attribute
	            if (index == 0) {
		            domAttr.set(node, "buttonEnabled", "true"); //mark to show active button
	            } else {
	                domStyle.set(tr, "display", "none");
	                _this.containers.push(tr);
	            }
	            if (xcitecUtil.str2bool(domAttr.get(node, "selected"))) {
	                if (domAttr.get(node, "error")) {
	                    _this.error = true;
	                }
	                if (_this.required) {
	                    domAttr.set(node, "required", "true");
	                }
	            }
	            _this.containerNode.appendChild(tr);
	        });
	        this.inherited(arguments);
	    },
	
	    showHidden: function() {
	        if (this.isVisible) {
	            array.forEach(this.containers, function(node) {
	                domStyle.set(node, "display", "none");
	            });
	            this.isVisible = false;
	            if (this.buttonWidget) {
	                this.buttonWidget.setSelectedStyle(false);
	            }
	        } else {
	            array.forEach(this.containers, function(node) {
	                domStyle.set(node, "display", "");
	            });
	            this.isVisible = true;
	            if (this.buttonWidget) {
	                this.buttonWidget.setSelectedStyle(true);
	            }
	        }
	    },
	
	    startup: function() {
	        // overriding native onclick function
	        if (this.error) {
	            var labelWidget = registry.byNode(query("[for='" + this.id + "']")[0]);
	            domClass.add(labelWidget.domNode, labelWidget.baseClass +"Error");        
	        }
	        var widgets = query("[widgetId]", this.domNode);
	        if (widgets[1]) {
	            this.buttonWidget = registry.byNode(widgets[1]);
	            widgets[1].onclick = lang.hitch(this, this.showHidden);
	        }
	        this.inherited(arguments);
	    }
	});
	return MultiLanguageContainer;
});

},
'xcitec/layout/FacetFilter':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/dom-construct',
	'dojo/query',
	'dijit/registry',
	'dijit/layout/_LayoutWidget',
	'dijit/_TemplatedMixin',
	'../form/LinkButton',
	'../wicket/AjaxLazyPane',
	'dojo/text!./templates/FacetFilter.html'
], function(declare, array, domConstruct, query, registry, _LayoutWidget, _TemplatedMixin, LinkButton, AjaxLazyPane, template){
	return declare([_LayoutWidget, _TemplatedMixin], {
		templateString: template,
		baseClass: 'facetFilter',

		layout: function(){
			var children = registry.findWidgets(this.domNode);

			for(var i = 0; i < children.length; i++){
				if(children[i] instanceof AjaxLazyPane){
					var paneChildren = registry.findWidgets(children[i].domNode);
					array.forEach(paneChildren, function(child){
						if(child instanceof LinkButton){
							var wrapper = domConstruct.create('div', null, this.buttonsContainer);
							domConstruct.place(child.domNode, wrapper);
						}
					}, this);
					domConstruct.place(this.buttonsContainer, this.domNode, 'last');
					return;
				}
			}
		}
	});
});
},
'xcitec/form/ListBoxV2':function(){
define([
	"dojo",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-class",
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/cookie",
	"dojo/data/ItemFileWriteStore",
	"dojo/dom-attr",
	"dojo/dom",
	"dojo/dom-geometry",
	"dojo/fx",
	"dojo/_base/array",
	"dojo/query",
	"dojo/text!xcitec/form/templates/ListBoxV2.html",
	"dojo/text!xcitec/form/templates/ListBoxV2Node.html",
	"dojo/topic",
  "dojo/keys",
	"dijit/_Contained",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/form/_FormWidget",
	"dijit/tree/ForestStoreModel",
	"dijit/tree/TreeStoreModel",
	"dijit/registry",
	"xcitec/form/MultiSelectUtils",
  "../_base/ImageFactory"
], function (dojo, lang, windowModule, domClass, declare, domStyle, cookieModule, ItemFileWriteStore, domAttr, dom, geometry, fx, array, queryModule, template,
             ListBoxV2NodeTemplate, topic, keys, _Contained, _Container, _Templated, _Widget, _FormWidget, ForestStoreModel, TreeStoreModel, registry, MultiSelectUtils,
             ImageFactory) {

	var _TreeNode = declare(
		[_Widget, _Templated, _Container, _Contained],
		{
			// summary:
			//		Single node within a tree.   This class is used internally
			//		by Tree and should not be accessed directly.
			// tags:
			//		private
	
			// item: dojo.data.Item
			//		the dojo.data entry this tree represents
			item: null,
	
			// isTreeNode: [protected] Boolean
			//		Indicates that this is a TreeNode.   Used by `dijit.Tree` only,
			//		should not be accessed directly.
			isTreeNode: true,
	
			// label: String
			//		Text of this tree node
			label: "",
	
			// isExpandable: [private] Boolean
			//		This node has children, so show the expando node (+ sign)
			isExpandable: null,
	
			// isExpanded: [readonly] Boolean
			//		This node is currently expanded (ie, opened)
			isExpanded: false,
	
			selected: false,
	
			value: "",
			itemIndex: "",
			exp: false,
	
			// state: [private] String
			//		Dynamic loading-related stuff.
			//		When an empty folder node appears, it is "UNCHECKED" first,
			//		then after dojo.data query it becomes "LOADING" and, finally "LOADED"
			state: "UNCHECKED",
	
			iconClass: "",
			icon: "",
	
			templateString: ListBoxV2NodeTemplate,
	
			postMixInProperties: function(){
				if(this.icon){
					this.iconClass = ImageFactory.createIconCss(this.icon);
				}
			},
	
			postCreate: function(){
				if(this.iconClass){
					domClass.add(this.contentNode, "dijitTreeRowIcon " + this.iconClass);
				}
				// set label, escaping special characters
				this.setLabelNode(this.label);
	
				if(this.selected == "true"){
					this.setSelected(true);
					this.tree.selectedTreeNodes = this.tree.selectedTreeNodes.concat(this);
				}
				//var clazz = (this.itemIndex % 2 == 0) ? "oddRow" : "evenRow";
				//dojo.addClass(this.domNode, clazz);
	
				// set expand icon for leaf
				this._setExpando();
	
				// set icon and label class based on item
				this._updateItemClasses(this.item);
	
				if(this.isExpandable){
					this.labelNode.setAttribute("aria-expanded", this.isExpanded);
					if(this == this.tree.rootNode){
						this.tree.domNode.setAttribute("aria-expanded", this.isExpanded);
					}
				}
				dom.setSelectable(this.domNode, false);
				//dojo.setSelectable(this.contentNode, false);
				this.connect(this.domNode, "onclick", "_onClick");
				this.connect(this.expandoNode, "onclick", "_onExpClick");
	
				this.tree._expandNode(this);
				if(!this.exp[0]){
					this.tree._collapseNode(this);
				}
	
			},
	
	
			_setIndentAttr: function(indent){
				// summary:
				//		Tell this node how many levels it should be indented
				// description:
				//		0 for top level nodes, 1 for their children, 2 for their
				//		grandchildren, etc.
				this.indent = indent;
	
				// Math.max() is to prevent negative padding on hidden root node (when indent == -1)
				// 19 is the width of the expandoIcon (TODO: get this from CSS instead of hardcoding)
				var pixels = (Math.max(indent, 0) * 19) + "px";
	
				domStyle.set(this.domNode, "backgroundPosition", pixels + " 0px");
				domStyle.set(this.rowNode,  geometry.isBodyLtr() ? "paddingLeft" : "paddingRight", pixels);
	
				array.forEach(this.getChildren(), function(child){
					child.set("indent", indent + 1);
				});
				if(indent == 0){
					domStyle.set(this.domNode, "backgroundImage", "none");
				}
			},
	
			markProcessing: function(){
				// summary:
				//		Visually denote that tree is loading data, etc.
				// tags:
				//		private
				this.state = "LOADING";
				this._setExpando(true);
			},
	
			unmarkProcessing: function(){
				// summary:
				//		Clear markup from markProcessing() call
				// tags:
				//		private
				this._setExpando(false);
			},
	
			_updateItemClasses: function(item){
				// summary:
				//		Set appropriate CSS classes for icon and label dom node
				//		(used to allow for item updates to change respective CSS)
				// tags:
				//		private
				var tree = this.tree, model = tree.model;
				if(tree._v10Compat && item === model.root){
					// For back-compat with 1.0, need to use null to specify root item (TODO: remove in 2.0)
					item = null;
				}
	
				/*if(this._iconClass){
				 dojo.removeClass(this.iconNode, this._iconClass);
				 }
				 this._iconClass = tree.getIconClass(item, this.isExpanded);
				 if(this._iconClass){
				 dojo.addClass(this.iconNode, this._iconClass);
				 }
				 dojo.style(this.iconNode, tree.getIconStyle(item, this.isExpanded) || {});*/
	
				if(this._labelClass){
					domClass.remove(this.labelNode, this._labelClass);
				}
				this._labelClass = tree.getLabelClass(item, this.isExpanded);
				if(this._labelClass){
					domClass.add(this.labelNode, this._labelClass);
				}
				domStyle.set(this.labelNode, tree.getLabelStyle(item, this.isExpanded) || {});
			},
	
			_updateLayout: function(){
				// summary:
				//		Set appropriate CSS classes for this.domNode
				// tags:
				//		private
				var parent = this.getParent();
				if(!parent || !parent.rowNode || parent.rowNode.style.display == "none"){
				  /* if we are hiding the root node then make every first level child look like a root node */
				  domClass.add(this.domNode, "dijitTreeIsRoot");
			  } else{
				  domClass.toggle(this.domNode, "dijitTreeIsLast", !this.getNextSibling());
			  }
			},
	
			_setExpando: function(/*Boolean*/ processing){
				// summary:
				//		Set the right image for the expando node
				// tags:
				//		private
	
				// apply the appropriate class to the expando node
				if(this.tree.isTree){
					var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
						"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"];
					var _a11yStates = ["*","-","+","*"];
	
					var idx = processing ? 0 : (this.isExpandable ? (this.isExpanded ? 1 : 2) : 3);
					array.forEach(styles,
						function(s){
							domClass.remove(this.expandoNode, s);
						}, this
					);
					domClass.add(this.expandoNode, styles[idx]);
	
					// provide a non-image based indicator for images-off mode
					this.expandoNodeText.innerHTML = _a11yStates[idx];
				} else{
					this.expandoNode.style.display = "none";
					domStyle.set(this.rowNode, "paddingLeft", "10px");
				}
	
			},
	
			expand: function(){
				// summary:
				//		Show my children
				if(this.isExpanded){
					return;
				}
				// cancel in progress collapse operation
	
				this._wipeOut && this._wipeOut.stop();
	
				this.isExpanded = true;
				this.labelNode.setAttribute("aria-expanded", "true");
				this.containerNode.setAttribute("role", "group");
				domClass.add(this.contentNode, 'dijitTreeContentExpanded');
				this._setExpando();
				this._updateItemClasses(this.item);
				if(this == this.tree.rootNode){
				this.tree.domNode.setAttribute("aria-expanded", "true");
				}
	
				if(!this._wipeIn){
					this._wipeIn = fx.wipeIn({
						node: this.containerNode, duration: 0
					});
				}
				this._wipeIn.play();
			},
	
			collapse: function(){
				// summary:
				//		Collapse this node (if it's expanded)
	
				if(!this.isExpanded){
					return;
				}
	
				// cancel in progress expand operation
				this._wipeIn && this._wipeIn.stop();
	
				this.isExpanded = false;
				this.labelNode.setAttribute("aria-expanded", "false");
				if(this == this.tree.rootNode){
					this.tree.domNode.setAttribute("aria-expanded", "false");
				}
				domClass.remove(this.contentNode, 'dijitTreeContentExpanded');
				this._setExpando();
				this._updateItemClasses(this.item);
	
				if(!this._wipeOut){
					this._wipeOut = fx.wipeOut({
						node: this.containerNode, duration: 0
					});
				}
				this._wipeOut.play();
			},
	
			setLabelNode: function(label){
				// summary:
				//		Sets the label
				this.labelNode.innerHTML = "";
				this.labelNode.appendChild(windowModule.doc.createTextNode(label));
			},
	
			// indent: Integer
			//		Levels from this node to the root node
			indent: 0,
	
			setChildItems: function(/* Object[] */ items){
				// summary:
				//		Sets the child items of this node, removing/adding nodes
				//		from current children to match specified items[] array.
	
				var tree = this.tree,
					model = tree.model;
	
				// Orphan all my existing children.
				// If items contains some of the same items as before then we will reattach them.
				// Don't call this.removeChild() because that will collapse the tree etc.
				array.forEach(this.getChildren(), function(child){
					_Container.prototype.removeChild.call(this, child);
				}, this);
	
				this.state = "LOADED";
	
				if(items && items.length > 0){
					this.isExpandable = true;
	
					// Create _TreeNode widget for each specified tree node, unless one already
					// exists and isn't being used (presumably it's from a DnD move and was recently
					// released
					array.forEach(items, function(item){
						var id = model.getIdentity(item),
							existingNode = tree._itemNodeMap[id],
							node =
								( existingNode && !existingNode.getParent() ) ?
									existingNode :
									this.tree._createTreeNode({
										item: item,
										tree: tree,
										isExpandable: model.mayHaveChildren(item),
										label: tree.getLabel(item),
										indent: this.indent + 1,
										selected: item.selected,
										value: item.value,
										itemIndex: item.itemIndex,
										exp: item.exp,
										icon: item.icon[0]
									});
						if(existingNode){
							existingNode.set('indent', this.indent + 1);
						}
						this.addChild(node);
						tree.allTreeNodes = tree.allTreeNodes.concat(node);
						// note: this won't work if there are two nodes for one item (multi-parented items); will be fixed later
						tree._itemNodeMap[id] = node;
						if(this.tree._state(item)){
							tree._expandNode(node);
						}
					}, this);
	
					// note that updateLayout() needs to be called on each child after
					// _all_ the children exist
					array.forEach(this.getChildren(), function(child, idx){
						child._updateLayout();
					});
				} else{
					this.isExpandable = false;
				}
	
				if(this._setExpando){
					// change expando to/from dot or + icon, as appropriate
					this._setExpando(false);
				}
	
				// On initial tree show, make the selected TreeNode as either the root node of the tree,
				// or the first child, if the root node is hidden
				if(this == tree.rootNode){
					var fc = this.tree.showRoot ? this : this.getChildren()[0];
					if(fc){
						//fc.setSelected(true);
						tree.lastFocused = fc;
					} else{
						// fallback: no nodes in tree so focus on Tree <div> itself
						tree.domNode.setAttribute("tabIndex", "0");
					}
				}
			},
	
			removeChild: function(/* treeNode */ node){
				this.inherited(arguments);
	
				var children = this.getChildren();
				if(children.length == 0){
					this.isExpandable = false;
					this.collapse();
				}
	
				array.forEach(children, function(child){
					child._updateLayout();
				});
			},
	
			makeExpandable: function(){
				//summary:
				//		if this node wasn't already showing the expando node,
				//		turn it into one and call _setExpando()
	
				// TODO: hmm this isn't called from anywhere, maybe should remove it for 2.0
	
				this.isExpandable = true;
				this._setExpando(false);
			},
	
			/*_onLabelFocus: function(evt){
			 // summary:
			 //		Called when this node is focused (possibly programatically)
			 // tags:
			 //		private
			 dojo.addClass(this.labelNode, "dijitTreeLabelFocused");
			 this.tree._onNodeFocus(this);
			 },
	
			 _onLabelBlur: function(evt){
			 // summary:
			 //		Called when focus was moved away from this node, either to
			 //		another TreeNode or away from the Tree entirely.
			 //		Note that we aren't using _onFocus/_onBlur builtin to dijit
			 //		because _onBlur() isn't called when focus is moved to my child TreeNode.
			 // tags:
			 //		private
			 dojo.removeClass(this.labelNode, "dijitTreeLabelFocused");
			 },*/
	
			setSelected: function(/*Boolean*/ selected){
				// summary:
				//		A Tree has a (single) currently selected node.
				//		Mark that this node is/isn't that currently selected node.
				// description:
				//		In particular, setting a node as selected involves setting tabIndex
				//		so that when user tabs to the tree, focus will go to that node (only).
				//var labelNode = this.labelNode;
				//labelNode.setAttribute("tabIndex", selected ? "0" : "-1");
				//dijit.setWaiState(labelNode, "selected", selected);
				//dojo.toggleClass(this.rowNode, "dijitTreeNodeSelected", selected);
				if(selected){
	
					domClass.add(this.rowNode, "selectedRow");
				} else{
					domClass.remove(this.rowNode, "selectedRow");
				}
			},
	
			_onClick: function(evt){
				evt.preventDefault(), evt.stopPropagation();
				this.tree.selectTreeNode(this, evt);
			},
	
			_onExpClick: function(evt){
				evt.preventDefault(), evt.stopPropagation();
				this.tree._onClick(this, evt);
			},
	
	
			_onMouseEnter: function(evt){
				// summary:
				//		Handler for onmouseenter event on a node
				// tags:
				//		private
				//	dojo.addClass(this.rowNode, "dijitTreeNodeHover");
				//	this.tree._onNodeMouseEnter(this, evt);
			},
	
			_onMouseLeave: function(evt){
				// summary:
				//		Handler for onmouseenter event on a node
				// tags:
				//		private
				//	dojo.removeClass(this.rowNode, "dijitTreeNodeHover");
				//	this.tree._onNodeMouseLeave(this, evt);
			}
		});
	
	var ListBoxV2 = declare([_Widget, _Templated, MultiSelectUtils],
		{
			// summary:
			//		This widget displays hierarchical data from a store.
	
			// store: [deprecated] String||dojo.data.Store
			//		Deprecated.  Use "model" parameter instead.
			//		The store to get data to display in the tree.
			store: null,
	
			// model: dijit.Tree.model
			//		Interface to read tree data, get notifications of changes to tree data,
			//		and for handling drop operations (i.e drag and drop onto the tree)
			model: null,
	
			// query: [deprecated] anything
			//		Deprecated.  User should specify query to the model directly instead.
			//		Specifies datastore query to return the root item or top items for the tree.
			query: null,
	
			// label: [deprecated] String
			//		Deprecated.  Use dijit.tree.ForestStoreModel directly instead.
			//		Used in conjunction with query parameter.
			//		If a query is specified (rather than a root node id), and a label is also specified,
			//		then a fake root node is created and displayed, with this label.
			label: "",
	
			// showRoot: [const] Boolean
			//		Should the root node be displayed, or hidden?
			showRoot: true,
	
			// childrenAttr: [deprecated] String[]
			//		Deprecated.   This information should be specified in the model.
			//		One ore more attributes that holds children of a tree node
			childrenAttr: ["children"],
	
			// openOnClick: Boolean
			//		If true, clicking a folder node's label will open it, rather than calling onClick()
			openOnClick: false,
	
			// openOnDblClick: Boolean
			//		If true, double-clicking a folder node's label will open it, rather than calling onDblClick()
			openOnDblClick: false,
	
			templateString: template,
	
			// isExpandable: [private deprecated] Boolean
			//		TODO: this appears to be vestigal, back from when Tree extended TreeNode.  Remove.
			isExpandable: true,
	
			// isTree: [private deprecated] Boolean
			//		TODO: this appears to be vestigal.  Remove.
			isTree: false,
	
			// persist: Boolean
			//		Enables/disables use of cookies for state saving.
			persist: false,
	
			// dndController: [protected] String
			//		Class name to use as as the dnd controller.  Specifying this class enables DnD.
			//		Generally you should specify this as "dijit._tree.dndSource".
			dndController: null,
	
			// parameters to pull off of the tree and pass on to the dndController as its params
			dndParams: ["onDndDrop","itemCreator","onDndCancel","checkAcceptance", "checkItemAcceptance", "dragThreshold", "betweenThreshold"],
	
			//declare the above items so they can be pulled from the tree's markup
	
			// onDndDrop: [protected] Function
			//		Parameter to dndController, see `dijit._tree.dndSource.onDndDrop`.
			//		Generally this doesn't need to be set.
			onDndDrop: null,
	
			// itemCreator: [protected] Function
			//		Parameter to dndController, see `dijit._tree.dndSource.itemCreator`.
			//		Generally this doesn't need to be set.
			itemCreator: null,
	
			// onDndCancel: [protected] Function
			//		Parameter to dndController, see `dijit._tree.dndSource.onDndCancel`.
			//		Generally this doesn't need to be set.
			onDndCancel: null,
	
			baseClass: "xcitecListBoxV2",
	
			error: "",
			disabled: false,
			readOnly: false,
			value: "",
			allTreeNodes: null, //Array
			selectedTreeNodes: null, //Array
			multiselect: "true",
			lastSelected: null,
			required:false,
	
			/*=====
			 checkAcceptance: function(source, nodes){
			 // summary:
			 //		Checks if the Tree itself can accept nodes from this source
			 // source: dijit.tree._dndSource
			 //		The source which provides items
			 // nodes: DOMNode[]
			 //		Array of DOM nodes corresponding to nodes being dropped, dijitTreeRow nodes if
			 //		source is a dijit.Tree.
			 // tags:
			 //		extension
			 return true;	// Boolean
			 },
			 =====*/
			checkAcceptance: null,
	
			/*=====
			 checkItemAcceptance: function(target, source, position){
			 // summary:
			 //		Stub function to be overridden if one wants to check for the ability to drop at the node/item level
			 // description:
			 //		In the base case, this is called to check if target can become a child of source.
			 //		When betweenThreshold is set, position="before" or "after" means that we
			 //		are asking if the source node can be dropped before/after the target node.
			 // target: DOMNode
			 //		The dijitTreeRoot DOM node inside of the TreeNode that we are dropping on to
			 //		Use dijit.getEnclosingWidget(target) to get the TreeNode.
			 // source: dijit._tree.dndSource
			 //		The (set of) nodes we are dropping
			 // position: String
			 //		"over", "before", or "after"
			 // tags:
			 //		extension
			 return true;	// Boolean
			 },
			 =====*/
			checkItemAcceptance: null,
	
			// dragThreshold: Integer
			//		Number of pixels mouse moves before it's considered the start of a drag operation
			dragThreshold: 0,
	
			// betweenThreshold: Integer
			//		Set to a positive value to allow drag and drop "between" nodes.
			//
			//		If during DnD mouse is over a (target) node but less than betweenThreshold
			//		pixels from the bottom edge, dropping the the dragged node will make it
			//		the next sibling of the target node, rather than the child.
			//
			//		Similarly, if mouse is over a target node but less that betweenThreshold
			//		pixels from the top edge, dropping the dragged node will make it
			//		the target node's previous sibling rather than the target node's child.
			betweenThreshold: 0,
	
			_publish: function(/*String*/ topicName, /*Object*/ message){
				// summary:
				//		Publish a message for this widget/topic
				topic.publish(this.id, lang.mixin({tree: this, event: topicName}, message || {}));
			},
	
			postMixInProperties: function(){
				//this.inherited(arguments);
				this.allTreeNodes = [];
				this.selectedTreeNodes = [];
				this.value = this.value.split(",");
				this.value = array.map(this.value, function(value){
					return lang.trim(value);
				});
				if(this.error){
					this.state = "Error";
				}
				if(this.required){
					this.baseClass = this.baseClass + "Required";
				}
				this.tree = this;
	
				this._itemNodeMap = {};
	
				if(!this.cookieName){
					this.cookieName = this.id + "SaveStateCookie";
				}
	
				// TODO: this.inherited(arguments)
			},
	
			_setStateClass: function(){
				// summary:
				//		Update the visual state of the widget by setting the css classes on this.domNode
				//		(or this.stateNode if defined) by combining this.baseClass with
				//		various suffixes that represent the current widget state(s).
				//
				// description:
				//		In the case where a widget has multiple
				//		states, it sets the class based on all possible
				//	 	combinations.  For example, an invalid form widget that is being hovered
				//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
				//
				//		For complex widgets with multiple regions, there can be various hover/active states,
				//		such as "Hover" or "CloseButtonHover" (for tab buttons).
				//		This is controlled by a stateModifier="CloseButton" attribute on the close button node.
				//
				//		The widget may have one or more of the following states, determined
				//		by this.state, this.checked, this.valid, and this.selected:
				//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
				//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
				//			- Selected - ex: currently selected tab will have this.selected==true
				//
				//		In addition, it may have one or more of the following states,
				//		based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
				//			- Disabled	- if the widget is disabled
				//			- Active		- if the mouse (or space/enter key?) is being pressed down
				//			- Focused		- if the widget has focus
				//			- Hover		- if the mouse is over the widget
	
				// Compute new set of classes
				var newStateClasses = this.baseClass.split(" ");
	
				function multiply(modifier){
					newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){
						return c + modifier;
					}), "dijit" + modifier);
				}
	
				if(this.checked){
					multiply("Checked");
				}
				if(this.state){
					multiply(this.state);
				}
				if(this.selected){
					multiply("Selected");
				}
	
				if(this.disabled){
					multiply("Disabled");
				} else if(this.readOnly){
					multiply("ReadOnly");
				} else if(this._active){
					multiply(this.stateModifier + "Active");
				} else{
					if(this._focused){
						multiply("Focused");
					}
					if(this._hovering){
						multiply(this.stateModifier + "Hover");
					}
				}
	
				// Remove old state classes and add new ones.
				// For performance concerns we only write into domNode.className once.
				var tn = this.stateNode || this.domNode,
					classHash = {};	// set of all classes (state and otherwise) for node
	
				array.forEach(tn.className.split(" "), function(c){
					classHash[c] = true;
				});
	
				if("_stateClasses" in this){
					array.forEach(this._stateClasses, function(c){
						delete classHash[c];
					});
				}
	
				array.forEach(newStateClasses, function(c){
					classHash[c] = true;
				});
	
				var newClasses = [];
				for(var c in classHash){
					newClasses.push(c);
				}
				tn.className = newClasses.join(" ");
	
				this._stateClasses = newStateClasses;
			},
	
			postCreate: function(){
				if(!this.disabled){
					domAttr.set(this.textbox, "name",  domAttr.get(this.srcNodeRef, 'name') ? domAttr.get(this.srcNodeRef, 'name') : "");
				}
	
				this._initState();
				this._parseSource();
	
				// Create glue between store and Tree, if not specified directly by user
				if(!this.model){
					this._store2model();
				}
	
				// monitor changes to items
				this.connect(this.model, "onChange", "_onItemChange");
				this.connect(this.model, "onChildrenChange", "_onItemChildrenChange");
				this.connect(this.model, "onDelete", "_onItemDelete");
	
				this._load();
	
				this.inherited(arguments);
	
				if(this.dndController){
					if(lang.isString(this.dndController)){
						this.dndController = lang.getObject(this.dndController);
					}
					var params = {};
					for(var i = 0; i < this.dndParams.length; i++){
						if(this[this.dndParams[i]]){
							params[this.dndParams[i]] = this[this.dndParams[i]];
						}
					}
					this.dndController = new this.dndController(this, params);
				}
				this._setStateClass();
	
				this.textbox.value = "";
				var temp = "";
				var counter = 0;
				array.forEach(this.value, function(val){
					if(counter == 0){
						temp = val;
					} else{
						temp = temp + "," + val;
					}
					counter++;
				});
				this.textbox.value = temp;
	
				domStyle.set(this.domNode, "width", "auto");
	
				var _this = this;
				array.forEach(this.rootNode.getChildren(), function(n){
					_this._collapseNode(n)
				});
				array.forEach(this.allTreeNodes, function(node){
					if((node.indent == 0) && (node.getChildren().length == 0) && (_this.isTree)){
						node.expandoNode.style.display = "none";
						node.labelNode.style.paddingLeft = "23px";
					}
					if(node.selected[0]){
						_this.lastSelected = node;
					}
				})
			},
	
			_parseSource: function(){
				this.values = queryModule(" > div", this.srcNodeRef).map(function(node){
					return domAttr.get(node, "value");
				});
				this.labels = queryModule(" > div", this.srcNodeRef).map(function(node){
					return String(node.innerHTML);
				});
				this.parents = queryModule(" > div", this.srcNodeRef).map(function(node){
					return domAttr.get(node, "parent");
				});
				this.expands = queryModule(" > div", this.srcNodeRef).map(function(node){
					return domAttr.get(node, "expanded");
				});
				this.icons = queryModule(" > div", this.srcNodeRef).map(function(node){
					return domAttr.get(node, "icon");
				})
			},
	
			_store2model: function(){
				var jsonObject = { "identifier": "value",
					"label": "name",
					"items": []};
	
				var _this = this;
				var ind = 0;
				array.forEach(this.labels, function(label, index){
					var itemType = 'parent';
					if(_this.parents[index]){
						itemType = 'child';
					}
					var itemChildren = [];
					array.forEach(_this.parents, function(parent, index2){
						if(parent == _this.values[index]){
							itemChildren = itemChildren.concat({"_reference": _this.values[index2]});
						}
					});
					var isSelected = false;
					array.forEach(_this.value, function(val){
						if(val == _this.values[index]){
							isSelected = true;
						}
					});
					var isExpanded = false;
					if(_this.expands[index] == "true"){
						isExpanded = true;
					}
	
					jsonObject.items[index] = {name:label, value:_this.values[index], type: itemType,
						children: itemChildren, selected: isSelected, itemIndex: ind, exp: isExpanded, icon: _this.icons[index]};
					ind++;
				});
				var storeObject = new ItemFileWriteStore({data:jsonObject});
				this._v10Compat = true;
	
				var modelParams = {
					id: this.id + "_ForestStoreModel",
					store: storeObject,
					query: {type:'parent'},
					childrenAttrs: this.childrenAttr
				};
	
				// Only override the model's mayHaveChildren() method if the user has specified an override
				if(this.params.mayHaveChildren){
					modelParams.mayHaveChildren = lang.hitch(this, "mayHaveChildren");
				}
	
				if(this.params.getItemChildren){
					modelParams.getChildren = lang.hitch(this, function(item, onComplete, onError){
						this.getItemChildren((this._v10Compat && item === this.model.root) ? null : item, onComplete, onError);
					});
				}
				this.model = new ForestStoreModel(modelParams);
	
				// For backwards compatibility, the visibility of the root node is controlled by
				// whether or not the user has specified a label
				this.showRoot = false;
			},
	
			_load: function(){
				// summary:
				//		Initial load of the tree.
				//		Load root node (possibly hidden) and it's children.
				this.model.getRoot(
					lang.hitch(this, function(item){
						var rn = this.rootNode = this.tree._createTreeNode({
							item: item,
							tree: this,
							isExpandable: true,
							label: this.label || this.getLabel(item),
							indent: this.showRoot ? 0 : -1
						});
						if(!this.showRoot){
							rn.rowNode.style.display = "none";
						}
						//this.domNode.appendChild(rn.domNode);
						this.attach.appendChild(rn.domNode);
						this._itemNodeMap[this.model.getIdentity(item)] = rn;
	
						rn._updateLayout();		// sets "dijitTreeIsRoot" CSS classname
	
						// load top level children
						this._expandNode(rn);
					}),
					function(err){
						console.error(this, ": error loading root: ", err);
					}
				);
			},
	
			////////////// Data store related functions //////////////////////
			// These just get passed to the model; they are here for back-compat
	
			mayHaveChildren: function(/*dojo.data.Item*/ item){
				// summary:
				//		Deprecated.   This should be specified on the model itself.
				//
				//		Overridable function to tell if an item has or may have children.
				//		Controls whether or not +/- expando icon is shown.
				//		(For efficiency reasons we may not want to check if an element actually
				//		has children until user clicks the expando node)
				// tags:
				//		deprecated
			},
	
			getItemChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete){
				// summary:
				//		Deprecated.   This should be specified on the model itself.
				//
				// 		Overridable function that return array of child items of given parent item,
				//		or if parentItem==null then return top items in tree
				// tags:
				//		deprecated
			},
	
			///////////////////////////////////////////////////////
			// Functions for converting an item to a TreeNode
			getLabel: function(/*dojo.data.Item*/ item){
				// summary:
				//		Overridable function to get the label for a tree node (given the item)
				// tags:
				//		extension
				return this.model.getLabel(item);	// String
			},
	
			getIconClass: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
				// summary:
				//		Overridable function to return CSS class name to display icon
				// tags:
				//		extension
				//return (!item || this.model.mayHaveChildren(item)) ? (opened ? "dijitFolderOpened" : "dijitFolderClosed") : "dijitLeaf"
			},
	
			getLabelClass: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
				// summary:
				//		Overridable function to return CSS class name to display label
				// tags:
				//		extension
			},
	
			getIconStyle: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
				// summary:
				//		Overridable function to return CSS styles to display icon
				// returns:
				//		Object suitable for input to dojo.style() like {backgroundImage: "url(...)"}
				// tags:
				//		extension
			},
	
			getLabelStyle: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
				// summary:
				//		Overridable function to return CSS styles to display label
				// returns:
				//		Object suitable for input to dojo.style() like {color: "red", background: "green"}
				// tags:
				//		extension
			},
	
			/////////// Keyboard and Mouse handlers ////////////////////
	
			_onKeyPress: function(/*Event*/ e){
				// summary:
				//		Translates keypress events into commands for the controller
				if(e.altKey){
					return;
				}
				var dk = keys;
				var treeNode = registry.getEnclosingWidget(e.target);
				if(!treeNode){
					return;
				}
	
				var key = e.charOrCode;
				if(typeof key == "string"){  // handle printables (letter navigation)
					// Check for key navigation.
					if(!e.altKey && !e.ctrlKey && !e.shiftKey && !e.metaKey){
						this._onLetterKeyNav({ node: treeNode, key: key.toLowerCase() });
						e.preventDefault(), e.stopPropagation();
					}
				} else{  // handle non-printables (arrow keys)
					var map = this._keyHandlerMap;
					if(!map){
						// setup table mapping keys to events
						map = {};
						map[dk.ENTER] = "_onEnterKey";
						map[this.isLeftToRight() ? dk.LEFT_ARROW : dk.RIGHT_ARROW] = "_onLeftArrow";
						map[this.isLeftToRight() ? dk.RIGHT_ARROW : dk.LEFT_ARROW] = "_onRightArrow";
						map[dk.UP_ARROW] = "_onUpArrow";
						map[dk.DOWN_ARROW] = "_onDownArrow";
						map[dk.HOME] = "_onHomeKey";
						map[dk.END] = "_onEndKey";
						this._keyHandlerMap = map;
					}
					if(this._keyHandlerMap[key]){
						this[this._keyHandlerMap[key]]({ node: treeNode, item: treeNode.item });
						e.preventDefault(), e.stopPropagation();
					}
				}
			},
	
			_onEnterKey: function(/*Object*/ message){
				this._publish("execute", { item: message.item, node: message.node});
				this.onClick(message.item, message.node);
			},
	
			_onDownArrow: function(/*Object*/ message){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary: down arrow pressed; get next visible node, set focus there
				var node = this._getNextNode(message.node);
				if(node && node.isTreeNode){
					this.focusNode(node);
				}
			},
	
			_onUpArrow: function(/*Object*/ message){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		Up arrow pressed; move to previous visible node
	
				var node = message.node;
	
				// if younger siblings
				var previousSibling = node.getPreviousSibling();
				if(previousSibling){
					node = previousSibling;
					// if the previous node is expanded, dive in deep
					while(node.isExpandable && node.isExpanded && node.hasChildren()){
						// move to the last child
						var children = node.getChildren();
						node = children[children.length - 1];
					}
				} else{
					// if this is the first child, return the parent
					// unless the parent is the root of a tree with a hidden root
					var parent = node.getParent();
					if(!(!this.showRoot && parent === this.rootNode)){
						node = parent;
					}
				}
	
				if(node && node.isTreeNode){
					this.focusNode(node);
				}
			},
	
			_onRightArrow: function(/*Object*/ message){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		Right arrow pressed; go to child node
				var node = message.node;
	
				// if not expanded, expand, else move to 1st child
				if(node.isExpandable && !node.isExpanded){
					this._expandNode(node);
				} else if(node.hasChildren()){
					node = node.getChildren()[0];
					if(node && node.isTreeNode){
						this.focusNode(node);
					}
				}
			},
	
			_onLeftArrow: function(/*Object*/ message){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		Left arrow pressed.
				//		If not collapsed, collapse, else move to parent.
	
				var node = message.node;
	
				if(node.isExpandable && node.isExpanded){
					this._collapseNode(node);
				} else{
					var parent = node.getParent();
					if(parent && parent.isTreeNode && !(!this.showRoot && parent === this.rootNode)){
						this.focusNode(parent);
					}
				}
			},
	
			_onHomeKey: function(){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		Home key pressed; get first visible node, and set focus there
				var node = this._getRootOrFirstNode();
				if(node){
					this.focusNode(node);
				}
			},
	
			_onEndKey: function(/*Object*/ message){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		End key pressed; go to last visible node.
	
				var node = this.rootNode;
				while(node.isExpanded){
					var c = node.getChildren();
					node = c[c.length - 1];
				}
	
				if(node && node.isTreeNode){
					this.focusNode(node);
				}
			},
	
			_onLetterKeyNav: function(message){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		Letter key pressed; search for node starting with first char = key
				var node = message.node, startNode = node,
					key = message.key;
				do{
					node = this._getNextNode(node);
					//check for last node, jump to first node if necessary
					if(!node){
						node = this._getRootOrFirstNode();
					}
				} while(node !== startNode && (node.label.charAt(0).toLowerCase() != key));
				if(node && node.isTreeNode){
					// no need to set focus if back where we started
					if(node !== startNode){
						this.focusNode(node);
					}
				}
			},

			selectTreeNode: function(nodeWidget, evt){
				if(this.disabled || this.readOnly){
					return;
				}
				if(this.multiselect == "true" ){
					this._multiSelect(nodeWidget, evt);
				} else{
					if(!this.isNodeInList(this.selectedTreeNodes, nodeWidget)){
						array.forEach(this.selectedTreeNodes, function(value){
							value.setSelected(false);
						});
						this.selectedTreeNodes = this.clearList(this.selectedTreeNodes);
						this.textbox.value = "";
						nodeWidget.setSelected(true);
						this.selectedTreeNodes = this.addToList(this.selectedTreeNodes, nodeWidget);
						this.textbox.value = nodeWidget.value;
					} else{
						if(evt.ctrlKey){
							nodeWidget.setSelected(false);
							this.selectedTreeNodes = this.clearList(this.selectedTreeNodes);
							this.textbox.value = "";
						} else{
							array.forEach(this.selectedTreeNodes, function(value){
								value.setSelected(false);
							});
							this.selectedTreeNodes = this.clearList(this.selectedTreeNodes);
							this.textbox.value = "";
							nodeWidget.setSelected(true);
							this.selectedTreeNodes = this.addToList(this.selectedTreeNodes, nodeWidget);
							this.textbox.value = nodeWidget.value;
						}
					}
				}
			},
			_multiSelect: function(/* Widget */ node, /* Event */ evt){
				var index = node.itemIndex; // index of selected row
				//var sendValue = node.value ? node.value : node.itemLabel;
				if(this.selectedTreeNodes.length >= 1){ // if there are selected rows
					if(evt.ctrlKey){
						if(this.isNodeInList(this.selectedTreeNodes, node)){
							this.selectedTreeNodes = this.delFromList(this.selectedTreeNodes, node);
							node.setSelected(false);
							var temp = "";
							var counter = 0;
							array.forEach(this.selectedTreeNodes, function(widget){
								var sendValue = widget.value ? widget.value : widget.itemLabel;
								if(counter == 0){
									temp = widget.value;
								} else{
									temp = temp + "," + widget.value;
								}
								counter++;
							});
							this.textbox.value = temp;
						} else{
							this.selectedTreeNodes = this.addToList(this.selectedTreeNodes, node);
							node.setSelected(true);
							this.lastSelected = node;
							this.addToInput(node.value);
						}
					} else if(evt.shiftKey){
						var _this = this;
						if(index >= this.lastSelected.itemIndex){
							array.forEach(_this.allTreeNodes, function(value){
								if((Number(value.itemIndex) >= Number(_this.lastSelected.itemIndex)) && (Number(value.itemIndex) <= Number(node.itemIndex))){
									if(!(_this.isNodeInList(_this.selectedTreeNodes, value))){
										_this.selectedTreeNodes = _this.addToList(_this.selectedTreeNodes, value);
										_this.addToInput(value.value);
										value.setSelected(true);
									}
								}
							});
						} else{
							array.forEach(_this.allTreeNodes, function(value){
								if((Number(value.itemIndex) >= Number(node.itemIndex)) && (Number(value.itemIndex) <= Number(_this.lastSelected.itemIndex))){
									if(!(_this.isNodeInList(_this.selectedTreeNodes, value))){
										_this.selectedTreeNodes = _this.addToList(_this.selectedTreeNodes, value);
										_this.addToInput(value.value);
										value.setSelected(true);
									}
								}
							});
						}
					} else{
						array.forEach(this.selectedTreeNodes, function(node){
							node.setSelected(false);
						});
						this.selectedTreeNodes = this.clearList(this.selectedTreeNodes);
						this.textbox.value = "";
						node.setSelected(true);
						this.lastSelected = node;
						this.selectedTreeNodes = this.addToList(this.selectedTreeNodes, node);
						this.addToInput(node.value);
					}
				} else{ // if there aren't selected rows
					if(!evt.shiftKey){
						this.selectedTreeNodes = this.addToList(this.selectedTreeNodes, node);
						this.addToInput(node.value);
						node.setSelected(true);
						this.lastSelected = node;
					}
				}
			},
	
			addToInput: function(addValue){
				var temp = this.textbox.value;
				if(temp == ""){
					temp = addValue;
				} else{
					temp = temp + "," + addValue;
				}
				this.textbox.value = temp;
			},
	
			_onClick: function(nodeWidget, /*Event*/ e){
				if(this.disabled || this.readOnly){
					return;
				}
				if((nodeWidget.isExpandable)){
					if(nodeWidget.isExpandable){
						this._onExpandoClick({node:nodeWidget});
					}
				}
				e.preventDefault(), e.stopPropagation();
				/*if (this.disabled || this.readOnly) {
				 return;
				 }
				 // summary:
				 //		Translates click events into commands for the controller to process
				 var domElement = e.target;
	
				 // find node
				 var nodeWidget = dijit.getEnclosingWidget(domElement);
	
				 if(!nodeWidget || !nodeWidget.isTreeNode){
				 return;
				 }
	
				 if (this.multiselect == "false") {
				 // this._multiSelect(node, evt);
				 } else {
				 if (!this.isNodeInList(this.selectedTreeNodes, nodeWidget)) {
				 //var sendValue = node.value ? node.value : node.itemLabel;
				 dojo.forEach(this.selectedTreeNodes, function(value) {
				 value.setSelected(false);
				 });
				 this.selectedTreeNodes = this.clearList(this.selectedTreeNodes);
				 //this.textbox.value= "";
				 nodeWidget.setSelected(true);
				 this.selectedTreeNodes = this.addToList(this.selectedTreeNodes, nodeWidget);
				 //this.textbox.value= sendValue;
				 } else {
				 if (e.ctrlKey) {
				 nodeWidget.setSelected(false);
				 this.selectedTreeNodes = this.clearList(this.selectedTreeNodes);
				 //this.textbox.value = "";
				 }
				 }
				 }
				 dojo.forEach(this.allTreeNodes, function(node) {
				 dojo.removeClass(node.rowNode, "selectedRow");
				 });
	
	
				 if( (this.openOnClick && nodeWidget.isExpandable) ||
				 (domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText) ){
				 // expando node was clicked, or label of a folder node was clicked; open it
				 if(nodeWidget.isExpandable){
				 this._onExpandoClick({node:nodeWidget});
				 }
				 }else{
				 this._publish("execute", { item: nodeWidget.item, node: nodeWidget} );
				 this.onClick(nodeWidget.item, nodeWidget);
				 this.focusNode(nodeWidget);
				 }
				 dojo.stopEvent(e);*/
			},
			_onDblClick: function(/*Event*/ e){
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		Translates double-click events into commands for the controller to process
				var domElement = e.target;
	
				// find node
				var nodeWidget = registry.getEnclosingWidget(domElement);
				if(!nodeWidget || !nodeWidget.isTreeNode){
					return;
				}
	
				if((this.openOnDblClick && nodeWidget.isExpandable) ||
					(domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText)){
					// expando node was clicked, or label of a folder node was clicked; open it
					if(nodeWidget.isExpandable){
						this._onExpandoClick({node:nodeWidget});
					}
				} else{
					this._publish("execute", { item: nodeWidget.item, node: nodeWidget});
					this.onDblClick(nodeWidget.item, nodeWidget);
					this.focusNode(nodeWidget);
				}
				e.preventDefault(), e.stopPropagation();
			},
	
			_onExpandoClick: function(/*Object*/ message){
	
				if(this.disabled || this.readOnly){
					return;
				}
				// summary:
				//		User clicked the +/- icon; expand or collapse my children.
				var node = message.node;
	
				// If we are collapsing, we might be hiding the currently focused node.
				// Also, clicking the expando node might have erased focus from the current node.
				// For simplicity's sake just focus on the node with the expando.
				//this.focusNode(node);
	
				if(node.isExpanded){
					this._collapseNode(node);
				} else{
					this._expandNode(node);
				}
			},
	
			onClick: function(/* dojo.data */ item, /*TreeNode*/ node){
				// summary:
				//		Callback when a tree node is clicked
				// tags:
				//		callback
			},
			onDblClick: function(/* dojo.data */ item, /*TreeNode*/ node){
				// summary:
				//		Callback when a tree node is double-clicked
				// tags:
				//		callback
			},
			onOpen: function(/* dojo.data */ item, /*TreeNode*/ node){
				// summary:
				//		Callback when a node is opened
				// tags:
				//		callback
			},
			onClose: function(/* dojo.data */ item, /*TreeNode*/ node){
				// summary:
				//		Callback when a node is closed
				// tags:
				//		callback
			},
	
			_getNextNode: function(node){
				// summary:
				//		Get next visible node
	
				if(node.isExpandable && node.isExpanded && node.hasChildren()){
					// if this is an expanded node, get the first child
					return node.getChildren()[0];		// _TreeNode
				} else{
					// find a parent node with a sibling
					while(node && node.isTreeNode){
						var returnNode = node.getNextSibling();
						if(returnNode){
							return returnNode;		// _TreeNode
						}
						node = node.getParent();
					}
					return null;
				}
			},
	
			_getRootOrFirstNode: function(){
				// summary:
				//		Get first visible node
				return this.showRoot ? this.rootNode : this.rootNode.getChildren()[0];
			},
	
			_collapseNode: function(/*_TreeNode*/ node){
				// summary:
				//		Called when the user has requested to collapse the node
	
				if(node.isExpandable){
					if(node.state == "LOADING"){
						// ignore clicks while we are in the process of loading data
						return;
					}
	
					node.collapse();
					this.onClose(node.item, node);
	
					if(node.item){
						this._state(node.item, false);
						this._saveState();
					}
				}
			},
	
			_expandNode: function(/*_TreeNode*/ node){
				// summary:
				//		Called when the user has requested to expand the node
	
				if(!node.isExpandable){
					return;
				}
	
				var model = this.model,
					item = node.item;
	
				switch(node.state){
					case "LOADING":
						// ignore clicks while we are in the process of loading data
						return;
	
					case "UNCHECKED":
						// need to load all the children, and then expand
						node.markProcessing();
						var _this = this;
						model.getChildren(item, function(items){
								node.unmarkProcessing();
								node.setChildItems(items);
								_this._expandNode(node);
							},
							function(err){
								console.error(_this, ": error loading root children: ", err);
							});
						break;
	
					default:
						// data is already loaded; just proceed
						node.expand();
						this.onOpen(node.item, node);
	
						if(item){
							this._state(item, true);
							this._saveState();
						}
				}
			},
	
			////////////////// Miscellaneous functions ////////////////
	
			focusNode: function(/* _tree.Node */ node){
	
				// summary:
				//		Focus on the specified node (which must be visible)
				// tags:
				//		protected
	
				// set focus so that the label will be voiced using screen readers
				node.labelNode.focus();
			},
	
			_onNodeFocus: function(/*Widget*/ node){
				if(this.disabled || this.readOnly){
					return;
				}
				array.forEach(this.allTreeNodes, function(node){
					// dojo.removeClass(node.domNode, "selectedRow");
				});
	
				// summary:
				//		Called when a TreeNode gets focus, either by user clicking
				//		it, or programatically by arrow key handling code.
				// description:
				//		It marks that the current node is the selected one, and the previously
				//		selected node no longer is.
	
				/*if (node){
				 if(node != this.lastFocused){
				 // mark that the previously selected node is no longer the selected one
				 this.lastFocused.setSelected(false);
				 }
	
				 // mark that the new node is the currently selected one
				 node.setSelected(true);
				 this.lastFocused = node;
				 }*/
			},
	
			_onNodeMouseEnter: function(/*Widget*/ node){
				// summary:
				//		Called when mouse is over a node (onmouseenter event)
			},
	
			_onNodeMouseLeave: function(/*Widget*/ node){
				// summary:
				//		Called when mouse is over a node (onmouseenter event)
			},
	
			//////////////// Events from the model //////////////////////////
	
			_onItemChange: function(/*Item*/ item){
				// summary:
				//		Processes notification of a change to an item's scalar values like label
				var model = this.model,
					identity = model.getIdentity(item),
					node = this._itemNodeMap[identity];
	
				if(node){
					node.setLabelNode(this.getLabel(item));
					node._updateItemClasses(item);
				}
			},
	
			_onItemChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList){
				// summary:
				//		Processes notification of a change to an item's children
				var model = this.model,
					identity = model.getIdentity(parent),
					parentNode = this._itemNodeMap[identity];
	
				if(parentNode){
					parentNode.setChildItems(newChildrenList);
				}
			},

			_onItemDelete: function(/*Item*/ item){
				// summary:
				//		Processes notification of a deletion of an item
				var model = this.model,
					identity = model.getIdentity(item),
					node = this._itemNodeMap[identity];

				if(node){
					var parent = node.getParent();
					if(parent){
						// if node has not already been orphaned from a _onSetItem(parent, "children", ..) call...
						parent.removeChild(node);
					}
					node.destroy();
					delete this._itemNodeMap[identity];
				}
			},
	
			/////////////// Miscellaneous funcs
	
			_initState: function(){
				// summary:
				//		Load in which nodes should be opened automatically
				if(this.persist){
					var cookie = cookieModule(this.cookieName);
					this._openedItemIds = {};
					if(cookie){
						array.forEach(cookie.split(','), function(item){
							this._openedItemIds[item] = true;
						}, this);
					}
				}
			},
			_state: function(item, expanded){
				// summary:
				//		Query or set expanded state for an item,
				if(!this.persist){
					return false;
				}
				var id = this.model.getIdentity(item);
				if(arguments.length === 1){
					return this._openedItemIds[id];
				}
				if(expanded){
					this._openedItemIds[id] = true;
				} else{
					delete this._openedItemIds[id];
				}
			},
			_saveState: function(){
				// summary:
				//		Create and save a cookie with the currently expanded nodes identifiers
				if(!this.persist){
					return;
				}
				var ary = [];
				for(var id in this._openedItemIds){
					ary.push(id);
				}
				cookieModule(this.cookieName, ary.join(","), {expires:365});
			},
	
			destroy: function(){
				if(this.rootNode){
					this.rootNode.destroyRecursive();
				}
				if(this.dndController && !lang.isString(this.dndController)){
					this.dndController.destroy();
				}
				this.rootNode = null;
				this.inherited(arguments);
			},
	
			destroyRecursive: function(){
				// A tree is treated as a leaf, not as a node with children (like a grid),
				// but defining destroyRecursive for back-compat.
				this.destroy();
			},
	
			_createTreeNode: function(/*Object*/ args){
				// summary:
				//		creates a TreeNode
				// description:
				//		Developers can override this method to define their own TreeNode class;
				//		However it will probably be removed in a future release in favor of a way
				//		of just specifying a widget for the label, rather than one that contains
				//		the children too.
				return new _TreeNode(args);
			}
		});
	
	// For back-compat.  TODO: remove in 2.0
	
	return ListBoxV2;
});

},
'xcitec/form/FlatButton':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dijit/form/Button",
  "../_base/ImageFactory"
], function (declare, lang, windowModule, Button, ImageFactory) {

	var FlatButton = declare( Button,	{
		baseClass: "dijitButton flatButton",
	
	    caption: "",
	    readOnly: false,
	
	    postMixInProperties: function() {
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        }
	        this.inherited(arguments);
	    },
	
	    create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
	        // Dijit button use label as button text, Xcitec label using for layout labels
	        // - if label in params remove it
	        // - if caption in params set "label" to call behavior of Dijit Button
	        if ("label" in params)   { delete params["label"]; };
	        if ("caption" in params) { params["label"]=""; };
	        this.inherited(arguments);
	    },
	
	
	    _setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for attr('label', ...) to work.
			// description:
			//		Set the label (text) of the button; takes an HTML string.
	        this.label = content;
	        if (lang.trim(this.label) == ""){
				this.containerNode.innerHTML = "";
				this.containerNode.appendChild(windowModule.doc.createTextNode(this.caption));
	        } else {
	            this.inherited(arguments);
	        }
		},
	
	    _onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions
			if(this.disabled || this.readOnly){
				return false;
			}
			//TODO this._clicked(); // widget click actions
			return this.onClick(e); // user click actions
		}
	});
	return FlatButton;
});

},
'dijit/_Container':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.place
	"dojo/_base/kernel" // kernel.deprecated
], function(array, declare, domConstruct, kernel){

	// module:
	//		dijit/_Container

	return declare("dijit._Container", null, {
		// summary:
		//		Mixin for widgets that contain HTML and/or a set of widget children.

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// All widgets with descendants must set containerNode.
				// NB: this code doesn't quite work right because for TabContainer it runs before
				// _TemplatedMixin::buildRendering(), and thus
				// sets this.containerNode to this.domNode, later to be overridden by the assignment in the template.
				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit/_WidgetBase*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			// I want to just call domConstruct.place(widget.domNode, this.containerNode, insertIndex), but the counting
			// is thrown off by text nodes and comment nodes that show up when constructed by markup.
			// In the future consider stripping those nodes on construction, either in the parser or this widget code.
			var refNode = this.containerNode;
			if(insertIndex > 0){
				// Old-school way to get nth child; dojo.query would be easier but _Container was weened from dojo.query
				// in #10087 to minimize download size.   Not sure if that's still and issue with new smaller dojo/query.
				refNode = refNode.firstChild;
				while(insertIndex > 0){
					if(refNode.nodeType == 1){ insertIndex--; }
					refNode = refNode.nextSibling;
				}
				if(refNode){
					insertIndex = "before";
				}else{
					// to support addChild(child, n-1) where there are n children (should add child at end)
					refNode = this.containerNode;
					insertIndex = "last";
				}
			}

			domConstruct.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget|int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove (ie, removeChild(5) removes the sixth widget).

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.
			return this.getChildren().length > 0;	// Boolean
		},

		_getSiblingOfChild: function(/*dijit/_WidgetBase*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//		private
			kernel.deprecated(this.declaredClass+"::_getSiblingOfChild() is deprecated. Use _KeyNavMixin::_getNext() instead.", "", "2.0");
			var children = this.getChildren(),
				idx = array.indexOf(children, child);	// int
			return children[idx + dir];
		},

		getIndexOfChild: function(/*dijit/_WidgetBase*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return array.indexOf(this.getChildren(), child);	// int
		}
	});
});

},
'xcitec/page/ReportMenuSubItem':function(){
define([
         "dojo/dom-attr",
         "dojo/_base/declare",
         "dojo/_base/lang",
         "dojo/_base/window",
         "dojo/dom-construct",
         "dojo/dom-style",
         "dojo/fx",
         "dojo/query",
         "dijit/_Container",
         "../_StateWidget",
         "../xcitecUtil"
       ], function (domAttr, declare, lang, windowModule, domConstruct, domStyle, fx, query, _Container, _StateWidget, xcitecUtil) {

  return declare(_StateWidget, {
    templateString: '<table class="${baseClass}">' +
                    '<tr><td class="reportMenuSubItemLeft">' +
                    '<div class="reportMenuSubItemLeftInner">' +
                    '<span class="reportMenuSubItemHead" data-dojo-attach-event="onclick: _onExpandoClick"></span>' +
                    '<div class="reportMenuSubItemTitle" data-dojo-attach-point="text" data-dojo-attach-event="ondijitclick:_onClick"></div>' +
                    '</div>' +
                    '</td><td class="reportMenuSubItemRight">' +
                    '<div class="reportMenuSubItemQuestion" data-dojo-attach-point="descriptionNode"></div>' +
                    '<div class="reportMenuSubItemAnswer"><div data-dojo-attach-point="containerNode" class="reportMenuSubItemWrap"><div class="${iconClass}" data-dojo-attach-point="reportImage"></div></div></div>' +
                    '</td></tr></table>',

    baseClass: "reportMenuSubItem",
    caption: "",
    description: "",
    selected: false,
    disabled: false,
    iconClass: "",
    leaf: true,

    postMixInProperties: function(){
      this.stateHolder = query(">*[xcitecRole$=subrepstateholder]", this.srcNodeRef)[0];
      if(this.iconClass == ""){
        domConstruct.destroy(this.reportImage);
      } else{
        this.iconClass = "pic " + this.iconClass;
      }
      if(this.stateHolder){
        this.selected = xcitecUtil.str2bool(domAttr.get(this.stateHolder, "value"));
      }
      this.inherited(arguments);
    },

    _fillContent: function(){
      if(this.caption != ""){
        this.text.appendChild(windowModule.doc.createTextNode(this.caption));
      }
      if(this.description != ""){
        this.descriptionNode.appendChild(windowModule.doc.createTextNode(this.description));
      }
      this.inherited(arguments);
    },

    postCreate: function(){
      this._setStateClass();
      if(this.stateHolder){
        domConstruct.place(this.stateHolder, this.domNode, "first");
      }
      this.inherited(arguments);
    },

    _onExpandoClick: function(){
      if(this.disabled){
        return;
      }
      this.selected ? this.collapse() : this.expand();
    },

    _onClick: function(){
      if(this.disabled){
        return;
      }
      this.onClick();
    },

    onClick: function(){
      // client Event
    },

    expand: function(){
      this.selected = true;
      if(this.stateHolder){
        domAttr.set(this.stateHolder, "value", "true");
      }
      domStyle.set(this.containerNode, "display", "none");
      fx.wipeIn({ node: this.containerNode, beforeBegin: lang.hitch(this, "_setStateClass") }).play();
    },

    collapse: function(){
      this.selected = false;
      if(this.stateHolder){
        domAttr.set(this.stateHolder, "value", "false");
      }
      fx.wipeOut({ node: this.containerNode, onEnd: lang.hitch(this, "_setStateClass") }).play();
    },

    hasInnerCombo: function(){
      return false;
    }
  });
});

},
'dijit/_base/wai':function(){
define([
	"dojo/dom-attr", // domAttr.attr
	"dojo/_base/lang", // lang.mixin
	"../main",	// export symbols to dijit
	"../hccss"			// not using this module directly, but loading it sets CSS flag on <html>
], function(domAttr, lang, dijit){

	// module:
	//		dijit/_base/wai

	var exports = {
		// summary:
		//		Deprecated methods for setting/getting wai roles and states.
		//		New code should call setAttribute()/getAttribute() directly.
		//
		//		Also loads hccss to apply dj_a11y class to root node if machine is in high-contrast mode.

		hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
			// summary:
			//		Determines if an element has a particular role.
			// returns:
			//		True if elem has the specific role attribute and false if not.
			//		For backwards compatibility if role parameter not provided,
			//		returns true if has a role
			var waiRole = this.getWaiRole(elem);
			return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
		},

		getWaiRole: function(/*Element*/ elem){
			// summary:
			//		Gets the role for an element (which should be a wai role).
			// returns:
			//		The role of elem or an empty string if elem
			//		does not have a role.
			 return lang.trim((domAttr.get(elem, "role") || "").replace("wairole:",""));
		},

		setWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Sets the role on an element.
			// description:
			//		Replace existing role attribute with new role.

			domAttr.set(elem, "role", role);
		},

		removeWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Removes the specified role from an element.
			//		Removes role attribute if no specific role provided (for backwards compat.)

			var roleValue = domAttr.get(elem, "role");
			if(!roleValue){ return; }
			if(role){
				var t = lang.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
				domAttr.set(elem, "role", t);
			}else{
				elem.removeAttribute("role");
			}
		},

		hasWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Determines if an element has a given state.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		true if elem has a value for the given state and
			//		false if it does not.

			return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
		},

		getWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Gets the value of a state on an element.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		The value of the requested state on elem
			//		or an empty string if elem has no value for state.

			return elem.getAttribute("aria-"+state) || "";
		},

		setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
			// summary:
			//		Sets a state on an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.setAttribute("aria-"+state, value);
		},

		removeWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Removes a state from an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.removeAttribute("aria-"+state);
		}
	};

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dojo/dnd/Mover':function(){
define([
	"../_base/array", "../_base/declare", "../_base/lang", "../sniff", "../_base/window",
	"../dom", "../dom-geometry", "../dom-style", "../Evented", "../on", "../touch", "./common", "./autoscroll"
], function(array, declare, lang, has, win, dom, domGeom, domStyle, Evented, on, touch, dnd, autoscroll){

// module:
//		dojo/dnd/Mover

return declare("dojo.dnd.Mover", [Evented], {
	// summary:
	//		an object which makes a node follow the mouse, or touch-drag on touch devices.
	//		Used as a default mover, and as a base class for custom movers.

	constructor: function(node, e, host){
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dom.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = (this.host = host), d = node.ownerDocument;

		function stopEvent(e){
			e.preventDefault();
			e.stopPropagation();
		}

		this.events = [
			// At the start of a drag, onFirstMove is called, and then the following
			// listener is disconnected.
			on(d, touch.move, lang.hitch(this, "onFirstMove")),

			// These are called continually during the drag
			on(d, touch.move, lang.hitch(this, "onMouseMove")),

			// And these are called at the end of the drag
			on(d, touch.release,  lang.hitch(this, "onMouseUp")),

			// cancel text selection and text dragging
			on(d, "dragstart",   stopEvent),
			on(d.body, "selectstart", stopEvent)
		];

		// Tell autoscroll that a drag is starting
		autoscroll.autoScrollStart(d);

		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove
		// e: Event
		//		mouse/touch event
		autoscroll.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY}, e);
		e.preventDefault();
		e.stopPropagation();
	},
	onMouseUp: function(e){
		if(has("webkit") && has("mac") && this.mouseButton == 2 ?
				e.button == 0 : this.mouseButton == e.button){ // TODO Should condition be met for touch devices, too?
			this.destroy();
		}
		e.preventDefault();
		e.stopPropagation();
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once.
		//		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = domGeom.getMarginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing domGeom.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style.
				var b = win.doc.body;
				var bs = domStyle.getComputedStyle(b);
				var bm = domGeom.getMarginBox(b, bs);
				var bc = domGeom.getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}

		// Disconnect touch.move that call this function
		this.events.shift().remove();
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

});

},
'xcitec/xlayout/QuestionV2SingleColumn':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-construct",
	"./QuestionV2",
	"./_QuestionV2ItemSingleColumn"
], function (declare, domConstruct, QuestionV2, _QuestionV2ItemSingleColumn) {

	/* xcitec.xlayout.QuestionV2SingleColumn is a layout control which is used to show questions in qualification */
	
	return declare( QuestionV2, {
	
		buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
			var controlHolderWidget = new _QuestionV2ItemSingleColumn({
				controlNode: controlNode,
				labelNode: labelNode,
				designMode: this.designMode
			});
			return controlHolderWidget.domNode;
		},
	
		insertActionIcons: function(/* DomNode */ iconsNode){
			var firstChildWidget = this.getChildren()[0];
			if(firstChildWidget && firstChildWidget.controlHolderNode){
				domConstruct.place(iconsNode, firstChildWidget.controlHolderNode, "first")
			}
		}

	});
});

},
'xcitec/xcitecDialog':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/dom-attr",
	"dojo/query",
	"dojo/dom",
	"dojo/text!xcitec/templates/xcitecDialog.html",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dijit/registry",
	"xcitec/Dialog",
  "./xcitecUtil"
], function (dojo, domStyle, domAttr, query, dom, template, declare, array, lang, domClass, domConstruct, registry, Dialog, xcitecUtil) {

	var xcitecDialog = declare( Dialog, {
	    templateString: template,
	
	    buttonIds: null, //Array
	    dialogContainer: false,
	    dialogDeletion: false,
	
		postMixInProperties: function() {
	        this.buttonIds = [];
	        this.inherited(arguments);
		},
	
	    postCreate:function() {
	        var actBtnsContainer = query("[xcitecAttachPoint$=buttons]", this.domNode)[0];
	        if (actBtnsContainer) {
	            var children = query(">", actBtnsContainer);
	            var _this = this;
	            array.forEach(children, function(c, index) {
	                _this.buttonIds.push(domAttr.get(c, "id"));
	                domConstruct.place(c, _this.buttonsBarNode, "last");
	            });
	        } else {
	            domStyle.set(this.buttonsBarNode, "display", "none");
	        }
	
	        if (this.dialogContainer) {
	            domClass.add(this.dialogNode, "dialogContainer");
	        }
	        if (this.dialogDeletion) {
	            domClass.add(this.dialogNode, "dialogDeletion");
	        }
	
	        this.inherited(arguments);
	    },
	
	    destroy: function() {
	        array.forEach(this.buttonIds, function(id) {
	            if (lang.exists("destroy", registry.byId(id))) {
	                registry.byId(id).destroy();
	            }
	        });
	        this.inherited(arguments);
	    }
	});
	
	function showDialog(containerId) {
	    var container = dom.byId(containerId);
	    if (container) {
	        var dialogNode = query(">", container)[0];
	        if (dialogNode) {
	            var dialogId = domAttr.get(dialogNode, "id");
	            dialogNode.removeAttribute("showOnLoad");
		        dijit._curFocus = null;
	            if (dialogId) {
	                dojo.parser.parse(container);
	                var dialogWidget = registry.byId(dialogId);
	                if (dialogWidget) {
	                    dialogWidget.show();
	                }
	            }
	        }
	    }
	}
	
	function hideDialog(dialogId) {
	    var dialogWidget = registry.byId(dialogId);
	    if (dialogWidget) {
	        dialogWidget.xcitecHide();
	//        dialogWidget.hide();
	        dialogWidget.destroyRecursive();
	    }
	}
	
	xcitecUtil.showDialog = showDialog;
	xcitecUtil.hideDialog = hideDialog;
	return xcitecDialog;
});

},
'xcitec/grid/_HGridBody':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/dnd/Source",
	"dojo/dom-geometry",
	"dojo/has",
	"dojo/on",
	"dojo/touch",
	"dojo/query",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "../xcitecUtil"
], function (dojo, domStyle, declare, lang, windowModule, array, domConstruct, domClass, Manager, domAttr, Source, domGeometry, has, on, touch, query, topic, _Container, _Templated, _Widget, xcitecUtil) {
  return 	declare(_Widget, {

		parentTable: null, // reference to xcitec.grid.HGridTest
		selectedRows: null, // selected rows
		lastSelected: null,
		allRows: null, //array of rows' objects
		showContextMenu: false,
		isFrozenRows: false,

		postMixInProperties: function(){
			this.allRows = [];
			this.inherited(arguments);
		},

		buildRendering: function(){
			this.domNode = this.srcNodeRef;
			// all widgets with descendants must set containerNode
			// todo check may be use _Container instead
			this.containerNode = this.domNode;

			var rows = query("tr", this.srcNodeRef);

			// if there are preselected rows - get number of selected row
			if(this.selectedRows){
				domConstruct.place(this.selectedRows, this.parentTable.domNode, "first");
				if(this.parentTable.allowMultiSelect){
					this.selectedRowsArray = this.selectedRows.value.split(",");
				} else{
					this.selectedRowsArray = this.selectedRows.value;
				}
			}

			var rowCounter = 0;
			var _this = this;
			var list = [];
			array.forEach(rows, function(rowNode){
				var rowObject = _this._buildRow(rowNode, rowCounter);
				rowCounter++;
				var tdLeft = domConstruct.create("td", {}, rowObject.domNode, "first");
				var tdRight = domConstruct.create("td", {}, rowObject.domNode, "last");
				domClass.add(tdLeft, "selectedLine");
				domClass.add(tdRight, "selectedLine");

				_this.allRows = _this.allRows.concat(rowObject);
				if(_this.selectedRowsArray){
					if(_this.parentTable.allowMultiSelect){
						array.forEach(_this.selectedRowsArray, function(rowIndex){
							if(domAttr.get(rowNode, "rowId") == rowIndex){
								_this._setSelectedRow(rowObject, true);
								list = list.concat(rowObject);
							}
						});
					} else{
						if(domAttr.get(rowNode, "rowId") == _this.selectedRowsArray){
							list = list.concat(rowObject);
							_this._setSelectedRow(rowObject, true);
						}
					}
				}
			});
			if(rowCounter < 5){
				domClass.add(this.parentTable.domNode, "minGridHeight");
			} else{
				if(!this.parentTable.vertScroll){
					domClass.add(this.parentTable.domNode, "dataTable_static");
				}
			}
			this._clearList();
			this.selectedRowsArray = list;
		},

		postCreate: function(){
			var index = 0;
			var columnWidths = this.parentTable.columnWidths;
			if(this.isFrozenRows){
				columnWidths = this.parentTable.columnFrozenWidths;
			}
			array.forEach(this.allRows[0].allCells, function(node, index){
				if(columnWidths[index]){
					domAttr.set(node, "width", columnWidths[index]);
				} else{
					columnWidths[index] = 0;
				}
			});
			if(has('ie') == 9){ // SRM-11209: refresh nodeTableContainer to prevent its height surge
				var nodeTableContainerHeight = this.parentTable.nodeTableContainer.clientHeight,
					that = this;
				touch.move(this.domNode, function(){
					if(that.parentTable.nodeTableContainer.clientHeight != nodeTableContainerHeight){
						that.parentTable.nodeTableContainer.style.display = 'none';
						that.parentTable.nodeTableContainer.style.display = 'block';
					}
				});
			}
		},

		_buildRow: function(/*row node*/ row, /*number*/ index){
			var rowObject = {
				rowIndex: index,
				domNode: null,
				allCells: null, //array of row's cells
				openIcon: null
			};
			rowObject.allCells = [];
			rowObject.domNode = row;
			var rowCells = query("td", row);
			var _this = this;
			var cellIndex = 0;
			array.forEach(rowCells, function(cellNode){
				var innerDiv = document.createElement("DIV");
				var multiline = domAttr.get(cellNode, "multiline");
				domClass.add(innerDiv, "wrap");
				innerDiv.innerHTML = cellNode.innerHTML;
				cellNode.innerHTML = "";
				if(multiline){
					rowObject.openIcon = document.createElement("div");
					domClass.add(rowObject.openIcon, "multilines");
					var wrapAll = domConstruct.create("div");
					domClass.add(wrapAll, "wrapper_all");
					wrapAll.appendChild(rowObject.openIcon);
					wrapAll.appendChild(innerDiv);
					cellNode.appendChild(wrapAll);
				} else{
					cellNode.appendChild(innerDiv);
				}
				rowObject.allCells[cellIndex] = cellNode;
				cellIndex++;
			});
			var clazz = (rowObject.rowIndex % 2 == 0) ? "oddRow" : "evenRow";
			domClass.add(rowObject.domNode, clazz);

			var fontColor = domAttr.get(rowObject.domNode, 'fontColor');
			if(fontColor){
				domClass.add(rowObject.domNode, xcitecUtil.createTypedStyleClass('fontColor', fontColor));
			}

			on(rowObject.domNode, "click", lang.hitch(rowObject, function(evt){
				_this._selectRow(this, evt);
			}));
			return rowObject;
		},

		_setSelectedRow: function(row, /* boolean */ isSelect){
			if(isSelect){
				domClass.add(row.domNode, "selectedRow selected_overRow");
			} else{
				domClass.remove(row.domNode, "selectedRow selected_overRow");
			}
		},

		// for marking row as selected
		_selectRow: function(node, evt){
			if(!this.isFrozenRows){
				if(this.parentTable.allowMultiSelect){
					this._processMultiSelect(node, evt);
				} else{
					if(!this._isRowInList(this.selectedRowsArray, node)){
						var _this = this;
						array.forEach(this.selectedRowsArray, function(value){
							_this._setSelectedRow(value, false);
						});
						if(_this.selectedRows){
							_this.selectedRows.value = "";
						}
						this._clearList();
						this._setSelectedRow(node, true);
						this._addToList(node);
					}
				}
			}
		},

		_processMultiSelect: function(node, evt){
			var _this = this;
			var index = node.rowIndex; // index of selected row
			if(this.selectedRowsArray.length >= 1){ // if there are selected rows
				if(evt.ctrlKey){
					if(this._isRowInList(this.selectedRowsArray, node)){
						this._delFromList(node);
						this._setSelectedRow(node, false);

					} else{
						this._addToList(node);
						this._setSelectedRow(node, true);
						this.lastSelected = node;
					}
				} else if(evt.shiftKey){
					if(index >= this.lastSelected.rowIndex){
						array.forEach(_this.allRows, function(value){
							if((Number(value.rowIndex) >= Number(_this.lastSelected.rowIndex)) && (Number(value.rowIndex) <= Number(node.rowIndex))){
								if(!(_this._isRowInList(_this.selectedRowsArray, value))){
									_this._addToList(value);
									_this._setSelectedRow(value, true);
								}
							}
						});
					} else{
						array.forEach(_this.allRows, function(value){
							if((Number(value.rowIndex) >= Number(node.rowIndex)) && (Number(value.rowIndex) <= Number(_this.lastSelected.rowIndex))){
								if(!(_this._isRowInList(_this.selectedRowsArray, value))){
									_this._addToList(value);
									_this._setSelectedRow(value, true);
								}
							}
						});
					}
				} else{
					array.forEach(this.selectedRowsArray, function(node){
						_this._setSelectedRow(node, false);
					});
					this._clearList();
					if(this.selectedRows){
						this.selectedRows.value = "";
					}
					this._setSelectedRow(node, true);
					this.lastSelected = node;
					this._addToList(node);
				}
			} else{ // if there aren't selected rows
				if(!evt.shiftKey){
					this._addToList(node);
					this._setSelectedRow(node, true);
					this.lastSelected = node;
				}
			}
		},

		resizeRowCell: function(row, cellIndex, width){
			domStyle.set(row.allCells[cellIndex], "width", width + "px");
		},

		js2push: function(arr){
			var i;
			for(i = 1; i < arguments.length; i++){
				arr[arr.length] = arguments[i];
			}
			return arr;
		},

		js2splice: function(arr, index, howMany){
			var i;
			var returnArr = new Array();
			var postArr = new Array();
			for(i = 0; i < arr.length; i++){
				if(i < index) this.js2push(returnArr, arr[i]);
				if(i >= (index + howMany)) this.js2push(postArr, arr[i]);
			}
			for(i = 3; i < arguments.length; i++)
				this.js2push(returnArr, arguments[i]);
			for(i = 0; i < postArr.length; i++)
				this.js2push(returnArr, postArr[i]);
			return returnArr;
		},

		_isRowInList: function(list, node){
			var isInList = false;
			array.forEach(list, function(value){
				if(value == node){
					isInList = true;
				}
			});
			return isInList;
		},

		_delFromList: function(row){
			var _this = this;
			array.forEach(this.selectedRowsArray, function(node){
				if(node == row){
					_this.selectedRowsArray = _this.js2splice(_this.selectedRowsArray, array.indexOf(_this.selectedRowsArray, node), 1);
				}
			});
			// update hidden input
			var temp = "";
			var counter = 0;
			array.forEach(this.selectedRowsArray, function(node){
				if(counter == 0){
					temp = domAttr.get(node.domNode, "rowId");
				} else{
					temp = temp + "," + domAttr.get(node.domNode, "rowId");
				}
				counter++;
			});
			if(this.selectedRows){
				this.selectedRows.value = temp;
			}
		},

		_addToList: function(row){
			this.selectedRowsArray = this.selectedRowsArray.concat(row);
			if(this.selectedRows){
				var temp = this.selectedRows.value;
				if(temp == ""){
					temp = domAttr.get(row.domNode, "rowId");
				} else{
					temp = temp + "," + domAttr.get(row.domNode, "rowId");
				}
				this.selectedRows.value = temp;
			}
		},

		_clearList: function(){
			if(this.selectedRowsArray){
				this.selectedRowsArray = this.js2splice(this.selectedRowsArray, 0);
			}
		},

		makeColumnRightAlign: function(columnIndex){
			array.forEach(this.allRows, function(row){
				domClass.add(row.allCells[columnIndex], "alignRight");
			})
		}
	});
});

},
'xcitec/layout/_Chapter2CItem':function(){
/*
 slmui.layout._Chapter2CItem is internal holder for Label/Control
 */
define(["dojo/_base/declare",
        "./_ChapterItem"], function(declare, _ChapterItem){
  return declare(_ChapterItem, {
    templateString:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td class="dijitReset fieldsetChapterLabelCell" data-dojo-attach-point="labelHolderColumnNode"' +
        '><div class="dijitReset fieldsetChapterLabelHolder" data-dojo-attach-point="labelHolderNode"></div>' +
        '</td><td class="dijitReset fieldsetChapterControlCell"><div data-dojo-attach-point="controlHolderNode"></div></td>' +
        '<td class="dijitReset"></td>' +
        '<td class="dijitReset fieldsetChapterLabelCell"></td>' +
          //      '<td class="dijitReset fieldsetChapterLabelCell"><LABEL class="xcitecElipsesLabel  xcitecLabel" unselectable="on">State Attr</LABEL></td>' +
        '<td class="dijitReset fieldsetChapterControlCell"></td>' +
        '</tr>',

    templateStringMode2:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td colspan="2" class="dijitReset fieldsetChapterControlCell" ' +
        '><div data-dojo-attach-point="labelHolderNode,controlHolderNode"></div></td>' +
        '<td class="dijitReset"></td>' +
        '<td class="dijitReset fieldsetChapterLabelCell"></td>' +
        '<td class="dijitReset fieldsetChapterControlCell"></td>' +
        '</tr>',

    templateStringWide:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td class="dijitReset fieldsetChapterLabelCell" data-dojo-attach-point="labelHolderColumnNode"' +
        '><div class="dijitReset fieldsetChapterLabelHolder" data-dojo-attach-point="labelHolderNode"></div>' +
        '</td><td colspan="4" class="dijitReset fieldsetChapterControlCell"><div data-dojo-attach-point="controlHolderNode"></div></td>' +
        '</tr>',

    templateStringWide2:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td colspan="5" class="dijitReset fieldsetChapterControlCell" ' +
        '><div data-dojo-attach-point="labelHolderNode,controlHolderNode"></div></td>' +
        '</tr>',

    templateStringInvert:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td class="dijitReset invertFieldsetChapterControlCell"><div data-dojo-attach-point="controlHolderNode"></div></td>' +
        '<td colspan="4" class="dijitReset invertFieldsetChapterLabelCell" data-dojo-attach-point="labelHolderColumnNode" style="font-weight:bold;"' +
        '><div class="dijitReset fieldsetChapterLabelHolder" data-dojo-attach-point="labelHolderNode"></div></td>' +
        '</tr>',

    processViewMode: function() {
      switch (this.controlView.toUpperCase()) {
      case "MODE2":
        this.templateString = this.templateStringMode2;
        this.defaultLabelPos = "top";
        break;

      case "MODE3": case "WIDE":
        this.templateString = this.templateStringWide;
        break;

      case "MODE4": case "WIDE2":
        this.templateString = this.templateStringWide2;
        this.defaultLabelPos = "top";
        break;
      case "MODE5": case "INVERT":
        this.templateString = this.templateStringInvert;
        this.widthInPercent = "";
        break;

      default:
        break;
      }
    }
  });
});

},
'xcitec/Toolbar':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dijit/Toolbar",
	"dijit/_Container"
], function (array, declare, domClass, Toolbar, _Container) {

	var Toolbar = declare( [Toolbar, _Container],  {
	  disabled: false,
	  readOnly: false,
	  alignButtons: "LEFT",
	
		buildRendering: function(){
			this.inherited(arguments);
			switch(this.alignButtons.toUpperCase()){
				case "RIGHT":
					domClass.add(this.domNode, "dijitToolbarAlignRight");
					break;
				default:
					domClass.add(this.domNode, "dijitToolbarAlignLeft");
					break;
			}
		},
	
	  startup: function(){
	    if(this._started){ return; }
	    if (this.disabled || this.readOnly) {
	      array.forEach(this.getChildren(), function(child) {
	         child.set("disabled", true);
	      }, this);
	    }
	    this.inherited(arguments);
	  }
	});
	return Toolbar;
});

},
'dijit/Calendar':function(){
define([
	"dojo/_base/array", // array.map
	"dojo/date",
	"dojo/date/locale",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.get
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove domClass.toggle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie")
	"./CalendarLite",
	"./_Widget",
	"./_CssStateMixin",
	"./_TemplatedMixin",
	"./form/DropDownButton"
], function(array, date, local, declare, domAttr, domClass, kernel, keys, lang, on, has, CalendarLite, _Widget, _CssStateMixin, _TemplatedMixin, DropDownButton){

	// module:
	//		dijit/Calendar

	// _Widget for deprecated methods like setAttribute()
	var Calendar = declare("dijit.Calendar", [CalendarLite, _Widget, _CssStateMixin], {
		// summary:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//
		// description:
		//		See CalendarLite for general description.   Calendar extends CalendarLite, adding:
		//
		//		- month drop down list
		//		- keyboard navigation
		//		- CSS classes for hover/mousepress on date, month, and year nodes
		//		- support of deprecated methods (will be removed in 2.0)

		// Set node classes for various mouse events, see dijit._CssStateMixin for more details
		cssStateNodes: {
			"decrementMonth": "dijitCalendarArrow",
			"incrementMonth": "dijitCalendarArrow",
			"previousYearLabelNode": "dijitCalendarPreviousYear",
			"nextYearLabelNode": "dijitCalendarNextYear"
		},

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.   Use set('value', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Calendar:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return new Calendar._MonthDropDownButton({
				id: this.id + "_mddb",
				tabIndex: -1,
				onMonthSelect: lang.hitch(this, "_onMonthSelect"),
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Events specific to Calendar, not used in CalendarLite
			this.own(
				on(this.domNode, "keydown", lang.hitch(this, "_onKeyDown")),
				on(this.dateRowsNode, "mouseover", lang.hitch(this, "_onDayMouseOver")),
				on(this.dateRowsNode, "mouseout", lang.hitch(this, "_onDayMouseOut")),
				on(this.dateRowsNode, "mousedown", lang.hitch(this, "_onDayMouseDown")),
				on(this.dateRowsNode, "mouseup", lang.hitch(this, "_onDayMouseUp"))
			);
		},

		_onMonthSelect: function(/*Number*/ newMonth){
			// summary:
			//		Handler for when user selects a month from the drop down list
			// tags:
			//		protected

			// move to selected month, bounding by the number of days in the month
			// (ex: jan 31 --> feb 28, not feb 31)
			var date = new this.dateClassObj(this.currentFocus);
			date.setDate(1);
			date.setMonth(newMonth);
			var daysInMonth = this.dateModule.getDaysInMonth(date);
			var currentDate = this.currentFocus.getDate();
			date.setDate(Math.min(currentDate, daysInMonth));
			this._setCurrentFocusAttr(date);
		},

		_onDayMouseOver: function(/*Event*/ evt){
			// summary:
			//		Handler for mouse over events on days, sets hovered style
			// tags:
			//		protected

			// event can occur on <td> or the <span> inside the td,
			// set node to the <td>.
			var node =
				domClass.contains(evt.target, "dijitCalendarDateLabel") ?
					evt.target.parentNode :
					evt.target;

			if(node && (
				(node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate"))
					|| node == this.previousYearLabelNode || node == this.nextYearLabelNode
				)){
				domClass.add(node, "dijitCalendarHoveredDate");
				this._currentNode = node;
			}
		},

		_onDayMouseOut: function(/*Event*/ evt){
			// summary:
			//		Handler for mouse out events on days, clears hovered style
			// tags:
			//		protected

			if(!this._currentNode){
				return;
			}

			// if mouse out occurs moving from <td> to <span> inside <td>, ignore it
			if(evt.relatedTarget && evt.relatedTarget.parentNode == this._currentNode){
				return;
			}
			var cls = "dijitCalendarHoveredDate";
			if(domClass.contains(this._currentNode, "dijitCalendarActiveDate")){
				cls += " dijitCalendarActiveDate";
			}
			domClass.remove(this._currentNode, cls);
			this._currentNode = null;
		},

		_onDayMouseDown: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate")){
				domClass.add(node, "dijitCalendarActiveDate");
				this._currentNode = node;
			}
		},

		_onDayMouseUp: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				domClass.remove(node, "dijitCalendarActiveDate");
			}
		},

		handleKey: function(/*Event*/ evt){
			// summary:
			//		Provides keyboard navigation of calendar.
			// description:
			//		Called from _onKeyDown() to handle keydown on a stand alone Calendar,
			//		and also from `dijit/form/_DateTimeTextBox` to pass a keydown event
			//		from the `dijit/form/DateTextBox` to be handled in this widget
			// returns:
			//		False if the key was recognized as a navigation key,
			//		to indicate that the event was handled by Calendar and shouldn't be propagated
			// tags:
			//		protected
			var increment = -1,
				interval,
				newValue = this.currentFocus;
			switch(evt.keyCode){
				case keys.RIGHT_ARROW:
					increment = 1;
				//fallthrough...
				case keys.LEFT_ARROW:
					interval = "day";
					if(!this.isLeftToRight()){
						increment *= -1;
					}
					break;
				case keys.DOWN_ARROW:
					increment = 1;
				//fallthrough...
				case keys.UP_ARROW:
					interval = "week";
					break;
				case keys.PAGE_DOWN:
					increment = 1;
				//fallthrough...
				case keys.PAGE_UP:
					interval = evt.ctrlKey || evt.altKey ? "year" : "month";
					break;
				case keys.END:
					// go to the next month
					newValue = this.dateModule.add(newValue, "month", 1);
					// subtract a day from the result when we're done
					interval = "day";
				//fallthrough...
				case keys.HOME:
					newValue = new this.dateClassObj(newValue);
					newValue.setDate(1);
					break;
				default:
					return true;
			}

			if(interval){
				newValue = this.dateModule.add(newValue, interval, increment);
			}

			this._setCurrentFocusAttr(newValue);

			return false;
		},

		_onKeyDown: function(/*Event*/ evt){
			// summary:
			//		For handling keydown events on a stand alone calendar
			if(!this.handleKey(evt)){
				evt.stopPropagation();
				evt.preventDefault();
			}
		},

		onValueSelected: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Deprecated.   Notification that a date cell was selected.  It may be the same as the previous value.
			// description:
			//		Formerly used by `dijit/form/_DateTimeTextBox` (and thus `dijit/form/DateTextBox`)
			//		to get notification when the user has clicked a date.  Now onExecute() (above) is used.
			// tags:
			//		protected
		},

		onChange: function(value){
			this.onValueSelected(value);	// remove in 2.0
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension

			/*=====
			 return ""; // String
			 =====*/
		}
	});

	Calendar._MonthDropDownButton = declare("dijit.Calendar._MonthDropDownButton", DropDownButton, {
		// summary:
		//		DropDownButton for the current month.    Displays name of current month
		//		and a list of month names in the drop down

		onMonthSelect: function(){
		},

		postCreate: function(){
			this.inherited(arguments);
			this.dropDown = new Calendar._MonthDropDown({
				id: this.id + "_mdd", //do not change this id because it is referenced in the template
				onChange: this.onMonthSelect
			});
		},
		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month);
			this.dropDown.set("months", monthNames);

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.containerNode.innerHTML =
				(has("ie") == 6 ? "" : "<div class='dijitSpacer'>" + this.dropDown.domNode.innerHTML + "</div>") +
					"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" + monthNames[month.getMonth()] + "</div>";
		}
	});

	Calendar._MonthDropDown = declare("dijit.Calendar._MonthDropDown", [_Widget, _TemplatedMixin], {
		// summary:
		//		The list-of-months drop down from the MonthDropDownButton

		// months: String[]
		//		List of names of months, possibly w/some undefined entries for Hebrew leap months
		//		(ex: ["January", "February", undefined, "April", ...])
		months: [],

		templateString: "<div class='dijitCalendarMonthMenu dijitMenu' " +
			"data-dojo-attach-event='onclick:_onClick,onmouseover:_onMenuHover,onmouseout:_onMenuHover'></div>",

		_setMonthsAttr: function(/*String[]*/ months){
			this.domNode.innerHTML = array.map(months,function(month, idx){
				return month ? "<div class='dijitCalendarMonthLabel' month='" + idx + "'>" + month + "</div>" : "";
			}).join("");
		},

		_onClick: function(/*Event*/ evt){
			this.onChange(domAttr.get(evt.target, "month"));
		},

		onChange: function(/*Number*/ /*===== month =====*/){
			// summary:
			//		Callback when month is selected from drop down
		},

		_onMenuHover: function(evt){
			domClass.toggle(evt.target, "dijitCalendarMonthLabelHover", evt.type == "mouseover");
		}
	});

	return Calendar;
});

},
'dojo/topic':function(){
define(["./Evented"], function(Evented){

	// module:
	//		dojo/topic

	var hub = new Evented;
	return {
		// summary:
		//		Pubsub hub.
		// example:
		//		| 	topic.subscribe("some/topic", function(event){
		//		|	... do something with event
		//		|	});
		//		|	topic.publish("some/topic", {name:"some event", ...});

		publish: function(topic, event){
			// summary:
			//		Publishes a message to a topic on the pub/sub hub. All arguments after
			//		the first will be passed to the subscribers, so any number of arguments
			//		can be provided (not just event).
			// topic: String
			//		The name of the topic to publish to
			// event: Object
			//		An event to distribute to the topic listeners
			return hub.emit.apply(hub, arguments);
		},

		subscribe: function(topic, listener){
			// summary:
			//		Subscribes to a topic on the pub/sub hub
			// topic: String
			//		The topic to subscribe to
			// listener: Function
			//		A function to call when a message is published to the given topic
			return hub.on.apply(hub, arguments);
		}
	};
});

},
'idx/widget/HoverHelpTooltip':function(){
/*
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(["dojo/_base/declare", "dojo/_base/fx", // fx.fadeIn fx.fadeOut
 "dojo/keys", // keys
 "dojo/_base/array", // array.forEach array.indexOf array.map
 "dojo/dom", // dom.byId
 "dojo/on",
 "dojo/aspect",
 "dojo/when", 
 "dojo/Deferred",
 "dojo/dom-attr", // domAttr.set
 "dojo/_base/lang", // lang.hitch lang.isArrayLike
 "dojo/_base/sniff", // has("ie")
 "dijit/popup",
 "dijit/focus", "dojo/_base/event", // event.stop
 "dojo/dom-geometry", // domGeometry.getMarginBox domGeometry.position
 "dojo/dom-construct",
 "dojo/dom-class",
 "dijit/registry",
 "dijit/place", "dijit/a11y", // _getTabNavigable
 "dojo/dom-style", // domStyle.set, domStyle.get
 "dojo/_base/window", // win.body
 "dijit/_base/manager", // manager.defaultDuration
 "dijit/_Widget", "dijit/_TemplatedMixin", "dijit/Tooltip", 
 "dojo/has!dojo-bidi?../bidi/widget/HoverHelpTooltip",
 "dojo/text!./templates/HoverHelpTooltip.html", "dijit/dijit", 
 "dojo/i18n!./nls/Dialog", "dojo/i18n!./nls/HoverHelpTooltip"
], function(declare, fx, keys, array, dom, on, aspect, when, Deferred, domAttr, lang, has, popup, dijitfocus, event, domGeometry, domConstruct, domClass, registry, place, 
 a11y, domStyle, win, manager, _Widget, _TemplatedMixin, Tooltip, bidiExtension, template, dijit, dialogNls, hoverHelpTooltipNls){
	var oneuiRoot = lang.getObject("idx.oneui", true); // for backward compatibility with IDX 1.2
	
    /**
     * @name idx.widget.HoverHelpTooltip
     * @class HoverHelpTooltip provides pop-up information that displays when users hover the mouse pointer over an help indicator.
     * HoverHelpTooltip is implemented following the standard and specified IBM One UI(tm)
     * <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y20&vsub=*&hsub=*&openpanes=1111111111">Hover Help</a></b>
     * @augments dijit.Tooltip
     * @example
     &lt;span data-dojo-type="idx.widget.HoverHelpTooltip" data-dojo-props='
     connectId:["anchor"],
     forceFocus: true,
     showLearnMore:true,
     learnMoreLinkValue:"http://www.ibm.com"'
     style="width: 300px"&gt;
     Passwords must be between 5 and 20 characters. There must be a combination of alphanumeric characters, starting with a letter and at least one number.&lt;br /&gt;&lt;br /&gt;
     &lt;/span&gt;
     * @see dijit.Tooltip
     **/
    var HoverHelpTooltip = declare("idx.widget.HoverHelpTooltip", Tooltip, {
        /** @lends idx.widget.HoverHelpTooltip.prototype */
        showDelay: 500,
        hideDelay: 800,
        /**
         * Whether to show Learn more link
         * @type Boolean
         */
        showLearnMore: false,
        /**
         * Learn more link value
         * @type String
         */
        learnMoreLinkValue: "#updateme",
        
        showCloseIcon: true,
        /**
         * Focus HoverHelpTooltip once it shown.
         * @type Boolean
         */
        forceFocus: false,
		
		textDir: "auto",

        _onHover: function(/*Event*/e){
            // summary:
            //		Despite the name of this method, it actually handles both hover and focus
            //		events on the target node, setting a timer to show the HoverHelpTooltip.
            // tags:
            //		private
            if (!HoverHelpTooltip._showTimer) {
                var target = e.target;
                HoverHelpTooltip._showTimer = setTimeout(lang.hitch(this, function(){
                    this.open(target)
                }), this.showDelay);
            }
            if (HoverHelpTooltip._hideTimer) {
                clearTimeout(HoverHelpTooltip._hideTimer);
                delete HoverHelpTooltip._hideTimer;
            }
        },
        
        _onUnHover: function(/*Event*/ /*===== e =====*/){
            // summary:
            //		Despite the name of this method, it actually handles both mouseleave and blur
            //		events on the target node, hiding the HoverHelpTooltip.
            // tags:
            //		private
            
            // keep a HoverHelpTooltip open if the associated element still has focus (even though the
            // mouse moved away)
            if (HoverHelpTooltip._showTimer) {
                clearTimeout(HoverHelpTooltip._showTimer);
                delete HoverHelpTooltip._showTimer;
            }
            if (!HoverHelpTooltip._hideTimer) {
                HoverHelpTooltip._hideTimer = setTimeout(lang.hitch(this, function(){
                    this.close()
                }), this.hideDelay);
            }
        },
        
        /**
         * Display the HoverHelpTooltip
         * @private
         */
        open: function(/*DomNode*/target){
            // summary:
            //		
            // tags:
            //		private
            if (HoverHelpTooltip._showTimer) {
                clearTimeout(HoverHelpTooltip._showTimer);
                delete HoverHelpTooltip._showTimer;
            }
			// Fix for Defect 12350: 
			if(!dom.byId(target)){return;}
			
			var ariaLabel = domAttr.get(this.domNode, "aria-label");
            HoverHelpTooltip.show(
				this.content || this.label || this.domNode.innerHTML, 
				target, this.position, !this.isLeftToRight(), 
				this.textDir, this.showLearnMore, this.learnMoreLinkValue, 
				this.showCloseIcon, this.forceFocus, ariaLabel);
            this._connectNode = target;
            this.onShow(target, this.position);
        },
		
        close: function(force){
            // summary:
            //		Hide the tooltip or cancel timer for show of tooltip
            // tags:
            //		private
            if (this._connectNode) {
                // if tooltip is currently shown
            	
            	// CHANGE FROM BARRY -- CREATED A DEFERRED OBJECT THAT GETS RESOLVED WHEN THE
            	// ANIMATION FOR HIDING COMPLETES AND RETURN IT FORM THIS METHOD SO THE CALLER
            	// CAN OPT TO WAIT FOR ANIMATION TO COMPLETE BEFORE TAKING A NEXT STEP
            	// -- I NEEDED THIS AT ONE POINT, BUT AFTER FIXING AN ISSUE WITH INTERFERENCE OF
            	// MULTIPLE TOOLTIPS IN RACE CONDITIONS IT WAS NO LONGER NEEDED.  I LEFT IT HERE
            	// SINCE IT CANNOT HURT TO GIVE THE CALLER THIS OPTION OF USING dojo/promise API
                var anim = HoverHelpTooltip.hide(this._connectNode, force);
                delete this._connectNode;
                this.onHide();
                if (anim) {
                   // create the deferred object
              	   var d = new Deferred();
              	   var handles = [];
              	   
              	   // define the function to resolve the deferred
              	   var resolve = function() {
              		   // mark the deferred as resolved (fulfill the promise)
              		   if (d) d.resolve();
              		   
              		   // for each aspect handle, remove it
              		   array.forEach(handles, function(handle) {
              			   if (handle) handle.remove();
              		   });
              	   };
              	   
              	   // use aspect.after to attach to the "onEnd" and "onStop" methods and
              	   // fire the "resolve" callback when one of these fires
              	   handles.push(aspect.after(anim, "onEnd", resolve));
              	   handles.push(aspect.after(anim, "onStop", resolve));
              	   
              	   // double-check if the animation already stopped and our deferred missed the event
              	   if ((anim.status() == "stopped") && (!d.isResolved())) {
              		   resolve();
              	   }
              	   
              	   // return the dojo/Deferred "promise"
              	   return d;
                 } else {
                   // if no animation then return null for dojo/when API
              	   return null;
                 }             	
            }
            if (HoverHelpTooltip._showTimer) {
                // if tooltip is scheduled to be shown (after a brief delay)
                clearTimeout(HoverHelpTooltip._showTimer);
                delete HoverHelpTooltip._showTimer;
            }
        },
        _setConnectIdAttr: function(/*String|String[]*/newId){
            // summary:
            //		Connect to specified node(s)
            
            // Remove connections to old nodes (if there are any)
            array.forEach(this._connections || [], function(nested){
                array.forEach(nested, lang.hitch(this, "disconnect"));
            }, this);
            
            // Make array of id's to connect to, excluding entries for nodes that don't exist yet, see startup()
            this._connectIds = array.filter(lang.isArrayLike(newId) ? newId : (newId ? [newId] : []), function(id){
                return dom.byId(id);
            });
            
            // Make connections
            this._connections = array.map(this._connectIds, function(id){
                var node = dom.byId(id);
                return [
                    // TODO - NOTE FROM BARRY: consider "onclick" triggering focus but "onmouseenter" NOT triggering focus
					this.connect(node, "onmouseenter", "_onHover"),
					this.connect(node, "onmouseleave", "_onUnHover"), 
					this.connect(node, "onclick", "_onHover"),
					this.connect(node, "onkeypress", "_onConnectIdKey")
				];
            }, this);
            
            this._set("connectId", newId);
        },
        _onConnectIdKey: function(/*Event*/evt){
            // summary:
            //		Handler for keyboard events
            // description:
            // tags:
            //		private
            var node = evt.target;
            
            if (evt.charOrCode == keys.ENTER || evt.charOrCode == keys.SPACE || evt.charOrCode == " " || evt.charOrCode == keys.F1) {
                // Use setTimeout to avoid crash on IE, see #10396.
                HoverHelpTooltip._showTimer = setTimeout(lang.hitch(this, function(){
                    this.open(node)
                }), this.showDelay);
                
                event.stop(evt);
            }
        }
        
    });
    
    var baseClassName = has("dojo-bidi")? "idx.widget._MasterHoverHelpTooltip_" : "idx.widget._MasterHoverHelpTooltip";
    var MasterHoverHelpTooltipBase = declare(baseClassName, [_Widget, _TemplatedMixin], {
		/**
		 * Milliseconds to fade in/fade out
		 * @type Integer
		 */
        duration: manager.defaultDuration,
        
        templateString: template,
		
        learnMoreLabel: "",
        
        /**
         * draggable: Boolean
         *		Toggles the moveable aspect of the HoverHelpTooltip. If true, HoverHelpTooltip
         *		can be dragged by it's grippy bar. If false it will remain positioned
         *		relative to the attached node
         *		@type boolean
         **/
        draggable: true,
        
        _firstFocusItem: null,
		
        _lastFocusItem: null,
        
        postMixInProperties: function(){
            this.learnMoreLabel = hoverHelpTooltipNls.learnMoreLabel;
			this.buttonClose = dialogNls.closeButtonLabel;
        },
        postCreate: function(){
            win.body().appendChild(this.domNode);
            
            //this.bgIframe = new BackgroundIframe(this.domNode);
            
            // Setup fade-in and fade-out functions.
            this.fadeIn = fx.fadeIn({
                node: this.domNode,
                duration: this.duration,
                onEnd: lang.hitch(this, "_onShow")
            });
            this.fadeOut = fx.fadeOut({
                node: this.domNode,
                duration: this.duration,
                onEnd: lang.hitch(this, "_onHide")
            });
            this.connect(this.domNode, "onkeypress", "_onKey");
            this.connect(this.domNode, "onmouseenter", lang.hitch(this, function(e){
				if(HoverHelpTooltip._hideTimer) {
	                clearTimeout(HoverHelpTooltip._hideTimer);
	                delete HoverHelpTooltip._hideTimer;
	            }

				// check if another tooltip is already deck
				if (!this._onDeck) {
					// CHANGE FROM BARRY - ONLY RESTORE A DYING TOOLTIP IF ANOTHER IS NOT ON DECK
					// no tooltip on deck so refocus this one and keep showing it
					
					if (this.forceFocus && this.showLearnMore) this.focus(); // NOTE FROM BARRY: consider removing this unless clicked
					this._keepShowing = true;
					
					// restore the previous state that was cleared out in forceHide
					if (this._prevState) {
						lang.mixin(this, this._prevState);
						this._prevState = null;
					}
					this.fadeOut.stop();
					this.fadeIn.play();
				} else {
					// CHANGE FROM BARRY -- IGNORE THE MOUSE LEAVE EVENT AS THE TOOLTIP CLOSES SINCE ANOTHER IS ON DECK
					// another tooltip is already deck so signal that the mouse leave event on the
					// current tooltip should not trigger a close
					this._ignoreMouseLeave = true;
				}
            }));
			this.connect(this.domNode, "onmouseleave", lang.hitch(this, function(e){
				this._keepShowing = false;
				if (this._ignoreMouseLeave) {
					// mouseenter occurred on previous tooltip and mouseleave is now firing
					// ignore this mouseleave event since the mouseenter was not for this tooltip
					// and we don't want to close the current tooltip
					delete this._ignoreMouseLeave;
					return;
				}
				HoverHelpTooltip._hideTimer = setTimeout(lang.hitch(this, function(){this.hide(this.aroundNode)}), this.hideDelay);
			}));
        },
        show: function(innerHTML, aroundNode, position, rtl, textDir, showLearnMore, 
			learnMoreLinkValue, showCloseIcon, forceFocus, ariaLabel){
			this._lastFocusNode = aroundNode;
            if (showLearnMore) {
                this.learnMoreNode.style.display = "inline";
                this.learnMoreNode.href = learnMoreLinkValue;
            }
            else {
                this.learnMoreNode.style.display = "none";
            }
            if (showCloseIcon || showCloseIcon == null) 
                this.closeButtonNode.style.display = "inline";
            else {
                this.closeButtonNode.style.display = "none";
            }
            //in case connectorNode was hidden on a previous call to hide
            this.connectorNode.hidden = false;
            
            if (this.aroundNode && this.aroundNode === aroundNode && this.containerNode.innerHTML == innerHTML) {
                return;
            }
            
            // reset width; it may have been set by orient() on a previous HoverHelpTooltip show()
            this.domNode.width = "auto";
            
            if (this.fadeOut.status() == "playing") {
                // previous HoverHelpTooltip is being hidden; wait until the hide completes then show new one
                this._onDeck = arguments;
                return;
            }
			
			
			//this._attachPopParent();
			//domStyle.set(this._popupWrapper, "display", "");
            
            this.containerNode.innerHTML = innerHTML;
			if(ariaLabel)domAttr.set(this.domNode, "aria-label", ariaLabel);
            
            this.set("textDir", textDir);
            this.containerNode.align = rtl ? "right" : "left"; //fix the text alignment
            var pos = place.around(this.domNode, aroundNode, position && position.length ? position : HoverHelpTooltip.defaultPosition, !rtl, lang.hitch(this, "orient"));
            
            // Position the HoverHelpTooltip connector for middle alignment.
            // This could not have been done in orient() since the HoverHelpTooltip wasn't positioned at that time.
            var aroundNodeCoords = pos.aroundNodePos;
            if (pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M') {
                this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
                this.connectorNode.style.left = "";
            }
            else if (pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M') {
                this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
            }
			
			////Set order in popup stack
			//this._addToPopupStack(); 
			
            // show it
            domStyle.set(this.domNode, "opacity", 0);
			domClass.add(this.domNode, "dijitPopup");
			//this._popupWrapper.appendChild(this.domNode);
            this.fadeIn.play();
            this.isShowingNow = true;
			this.aroundNode = aroundNode;
			// Add WAI-ARIA attribute to the hover tooltip container
			var sourceId = domAttr.get(aroundNode, "id")
			if(typeof sourceId == "string"){
				dijit.setWaiState(this.containerNode, "labelledby", sourceId);
			}
			if (forceFocus) {
                this.focus();
            }
			return;
        },
		
		/*_addToPopupStack: function(){
			var stack = popup._stack;
			domStyle.set(this.domNode, "zIndex", popup._beginZIndex + stack.length);
			popup._stack.push({
				widget: this,
				handlers: []
			});
		},
        _attachPopParent: function(){
			//This is for focusManager to find popup parent of HHT
			if(!this._popupWrapper){
				this._popupWrapper = domConstruct.create("div", {
					style: {"display": "none"}
				}, this.ownerDocumentBody);
			}
			if(this.aroundNode && this.aroundNode.id){
				domAttr.set(this._popupWrapper, "dijitPopupParent", this.aroundNode.id);
			}else{
				this._popupWrapper.dijitPopupParent = this.aroundNode;
			}
		},
        _removeFromPopupStack: function(){
			popup._stack.pop();
		},*/
        orient: function(/*DomNode*/node, /*String*/ aroundCorner, /*String*/ HoverHelpTooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
            // summary:
            //		Private function to set CSS for HoverHelpTooltip node based on which position it's in.
            //		This is called by the dijit popup code.   It will also reduce the HoverHelpTooltip's
            //		width to whatever width is available
            // tags:
            //		protected
            this.connectorNode.style.top = ""; //reset to default
            //Adjust the spaceAvailable width, without changing the spaceAvailable object
            var HoverHelpTooltipSpaceAvaliableWidth = spaceAvailable.w - this.connectorNode.offsetWidth;
            
            node.className = "idxOneuiHoverHelpTooltip " +
            {
                "MR-ML": "idxOneuiHoverHelpTooltipRight",
                "ML-MR": "idxOneuiHoverHelpTooltipLeft",
                "TM-BM": "idxOneuiHoverHelpTooltipAbove",
                "BM-TM": "idxOneuiHoverHelpTooltipBelow",
                "BL-TL": "idxOneuiHoverHelpTooltipBelow idxOneuiHoverHelpTooltipABLeft",
                "TL-BL": "idxOneuiHoverHelpTooltipAbove idxOneuiHoverHelpTooltipABLeft",
                "BR-TR": "idxOneuiHoverHelpTooltipBelow idxOneuiHoverHelpTooltipABRight",
                "TR-BR": "idxOneuiHoverHelpTooltipAbove idxOneuiHoverHelpTooltipABRight",
                "BR-BL": "idxOneuiHoverHelpTooltipRight",
                "BL-BR": "idxOneuiHoverHelpTooltipLeft",
                "TR-TL": "idxOneuiHoverHelpTooltipRight"
            }[aroundCorner + "-" + HoverHelpTooltipCorner];
            
            // reduce HoverHelpTooltip's width to the amount of width available, so that it doesn't overflow screen
            this.domNode.style.width = "auto";
            var size = domGeometry.getContentBox(this.domNode);
            
            var width = Math.min((Math.max(HoverHelpTooltipSpaceAvaliableWidth, 1)), size.w);
            var widthWasReduced = width < size.w;
            
            this.domNode.style.width = width + "px";
            
            //Adjust width for HoverHelpTooltips that have a really long word or a nowrap setting
            if (widthWasReduced) {
                this.containerNode.style.overflow = "auto"; //temp change to overflow to detect if our HoverHelpTooltip needs to be wider to support the content
                var scrollWidth = this.containerNode.scrollWidth;
                this.containerNode.style.overflow = "visible"; //change it back
                if (scrollWidth > width) {
                    scrollWidth = scrollWidth + domStyle.get(this.domNode, "paddingLeft") + domStyle.get(this.domNode, "paddingRight");
                    this.domNode.style.width = scrollWidth + "px";
                }
            }
            
            // Reposition the HoverHelpTooltip connector.
            if (HoverHelpTooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B') {
                var mb = domGeometry.getMarginBox(node);
                var HoverHelpTooltipConnectorHeight = this.connectorNode.offsetHeight;
                if (mb.h > spaceAvailable.h) {
                    // The HoverHelpTooltip starts at the top of the page and will extend past the aroundNode
                    var aroundNodePlacement = spaceAvailable.h - ((aroundNodeCoords.h + HoverHelpTooltipConnectorHeight) >> 1);
                    this.connectorNode.style.top = aroundNodePlacement + "px";
                    this.connectorNode.style.bottom = "";
                }
                else {
                    // Align center of connector with center of aroundNode, except don't let bottom
                    // of connector extend below bottom of HoverHelpTooltip content, or top of connector
                    // extend past top of HoverHelpTooltip content
                    this.connectorNode.style.bottom = Math.min(Math.max(aroundNodeCoords.h / 2 - HoverHelpTooltipConnectorHeight / 2, 0), mb.h - HoverHelpTooltipConnectorHeight) +
                    "px";
                    this.connectorNode.style.top = "";
                }
            }
            else {
                // reset the HoverHelpTooltip back to the defaults
                this.connectorNode.style.top = "";
                this.connectorNode.style.bottom = "";
            }
            
            return Math.max(0, size.w - HoverHelpTooltipSpaceAvaliableWidth);
        },
        
        focus: function(){
			if(this._focus){return;}
            this._getFocusItems(this.outerContainerNode);
            this._focus = true;
            dijitfocus.focus(this._firstFocusItem);
        },
        
        _onShow: function(){
            // summary:
            //		Called at end of fade-in operation
            // tags:
            //		protected
            if (has("ie")) {
                // the arrow won't show up on a node w/an opacity filter
                this.domNode.style.filter = "";
            }
            domAttr.set(this.containerNode, "tabindex", "0");
            domAttr.set(this.learnMoreNode, "tabindex", "0");
            domAttr.set(this.closeButtonNode, "tabindex", "0");
        },
        
        hide: function(aroundNode, force){
            // summary:
            //		Hide the HoverHelpTooltip
            if(this._keepShowing){
            	this._keepShowing = false; 
            	if (!force) return;
            }
            if (this._onDeck && this._onDeck[1] == aroundNode) {
                // this hide request is for a show() that hasn't even started yet;
                // just cancel the pending show()
                this._onDeck = null;
            }
            else if(this.aroundNode === aroundNode){
                    // this hide request is for the currently displayed HoverHelpTooltip
                return this._forceHide();
            }
        },
        hideOnClickClose: function(){
            // summary:
            //		Hide the HoverHelpTooltip
            // this hide request is for the currently displayed HoverHelpTooltip    
            this._forceHide(true);
        },
        
        // CHANGE FROM BARRY: added "refocus" parameter so that refocus only happens on hiding the tooltip
        // if the user initiated the hide via "hideOnClickClose" or "ESC" key.  this prevents toltips that
        // are disappearing because their associated field blurred focus from refocusing the field that was
        // just blurred.  programmatic hiding/showing of the tooltip makes this problem evident
        _forceHide: function(refocus){
        	// CHANGE FROM BARRY
        	// only refocus if flag is set and the node is defined
            if (refocus && this._lastFocusNode) {
            	// check if the defined node has an enclosing widget with a "refocus" method
            	var currentWrapWidget = registry.getEnclosingWidget(this._lastFocusNode);
            	
            	// if the widget is found and has a refocus method then call it, else focus the node
            	if (currentWrapWidget && lang.isFunction(currentWrapWidget.refocus)) currentWrapWidget.refocus();
            	else dijitfocus.focus(this._lastFocusNode);
            }
            
            // CHANGE FROM BARRY
            // We save the state of this instance in case the "_forceHide" is cancelled by a "mouseenter"
            // event.  We need to restore the state after cancellingt he fadeOut and fading the tooltip 
            // back in.
            if ((this.aroundNode)||(this._lastFocusNode)) {
            	this._prevState = {
            			_lastFocusNode: this._lastFocusNode,
            			_firstFocusItem: this._firstFocusItem,
            			_lastFocusItem: this._lastFocusItem,
            			_focus: this._focus,
            			isShowingNow: this.isShowingNow,
            			aroundNode: this.aroundNode
            	};
            }
			this._lastFocusNode = null;
			this._firstFocusItem = null;
			this._lastFocusItem = null;
            this._focus = false;
            this.fadeIn.stop();
            this.isShowingNow = false;
			this.aroundNode = null; // moved this to here, similar to dijit.Tooltip
			//this._removeFromPopupStack();
            return this.fadeOut.play();
        },
        _getFocusItems: function(){
            // summary:
            //		Finds focusable items in tooltip,
            //		and sets this._firstFocusItem and this._lastFocusItem
            // tags:
            //		protected
			
			if(this._firstFocusItem){
				this._firstFocusItem = this.closeButtonNode;
				return;
			}
			var elems = a11y._getTabNavigable(this.containerNode),
				endFocusableNode = domStyle.get(this.learnMoreNode, "display") == "none" ? this.closeButtonNode : this.learnMoreNode;
			this._firstFocusItem = elems.lowest || elems.first || endFocusableNode;
			this._lastFocusItem = elems.last || elems.highest || endFocusableNode;
        },
        _onKey: function(/*Event*/evt){
            // summary:
            //		Handler for keyboard events
            // description:
            // tags:
            //		private
            
            var node = evt.target;
            if (evt.charOrCode === keys.TAB) {
                this._getFocusItems(this.outerContainerNode);
            }
            var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
            if (evt.charOrCode == keys.ESCAPE) {
                // Use setTimeout to avoid crash on IE, see #10396.
                setTimeout(lang.hitch(this, "hideOnClickClose"), 0);
                event.stop(evt);
            }
            else 
                if (node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB) {
                    if (!singleFocusItem) {
                        dijitfocus.focus(this._lastFocusItem); // send focus to last item in dialog
                    }
                    event.stop(evt);
                }
                else 
                    if (node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey) {
                        if (!singleFocusItem) {
                            dijitfocus.focus(this._firstFocusItem); // send focus to first item in dialog
                        }
                        event.stop(evt);
                    }
                    else 
                        if (evt.charOrCode === keys.TAB) {
                            // we want the browser's default tab handling to move focus
                            // but we don't want the tab to propagate upwards
                            evt.stopPropagation();
                        }
        },
        _onHide: function(){
            // summary:
            //		Called at end of fade-out operation
            // tags:
            //		protected
            this._prevState = null;
            this.domNode.style.cssText = ""; // to position offscreen again
            this.containerNode.innerHTML = "";
			//domStyle.set(this._popupWrapper, "display", "none");
            domAttr.remove(this.containerNode, "tabindex");
            domAttr.remove(this.learnMoreNode, "tabindex");
            domAttr.remove(this.closeButtonNode, "tabindex");      
			domAttr.remove(this.domNode, "aria-label");      
            if (this._onDeck) {
            	var args = this._onDeck;
            	this._onDeck = null;
                // a show request has been queued up; do it now
                this.show.apply(this, args);
            }
        },
        onBlur: function(){
            this._forceHide();
        }
    }); //end declare
    //    var MasterHoverHelpTooltip = Tooltip._MasterTooltip;
    HoverHelpTooltip._MasterHoverHelpTooltip = MasterHoverHelpTooltip = has("dojo-bidi")? declare("idx.widget._MasterHoverHelpTooltip",[MasterHoverHelpTooltipBase,bidiExtension]) : MasterHoverHelpTooltipBase; // for monkey patching
    // summary:
    //		Static method to display HoverHelpTooltip w/specified contents in specified position.
    //		See description of idx.widget.HoverHelpTooltip.defaultPosition for details on position parameter.
    //		If position is not specified then idx.widget.HoverHelpTooltip.defaultPosition is used.
    // innerHTML: String
    //		Contents of the HoverHelpTooltip
    // aroundNode: dijit.__Rectangle
    //		Specifies that HoverHelpTooltip should be next to this node / area
    // position: String[]?
    //		List of positions to try to position HoverHelpTooltip (ex: ["right", "above"])
    // rtl: Boolean?
    //		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
    //		means "rtl"; specifies GUI direction, not text direction.
    // textDir: String?
    //		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.	
    HoverHelpTooltip.show = idx.widget.showHoverHelpTooltip = function(innerHTML, aroundNode, position, rtl, textDir, showLearnMore, learnMoreLinkValue, showCloseIcon, forceFocus, ariaLabel){
    
        if (!HoverHelpTooltip._masterTT) {
            idx.widget._masterTT = HoverHelpTooltip._masterTT = new MasterHoverHelpTooltip();
        }
        return HoverHelpTooltip._masterTT.show(innerHTML, aroundNode, position, rtl, textDir, showLearnMore, learnMoreLinkValue, showCloseIcon, forceFocus, ariaLabel);
    };
    
    // summary:
    //		Static method to hide the HoverHelpTooltip displayed via showHoverHelpTooltip()
    HoverHelpTooltip.hide = idx.widget.hideHoverHelpTooltip = function(aroundNode){
    
        return HoverHelpTooltip._masterTT && HoverHelpTooltip._masterTT.hide(aroundNode);
    };
    
    HoverHelpTooltip.defaultPosition = ["after-centered", "before-centered", "below", "above"];
    
    // for IDX 1.2 compatibility
	oneuiRoot.HoverHelpTooltip = HoverHelpTooltip;

    return HoverHelpTooltip;
    
});

},
'dijit/_base/manager':function(){
define([
	"dojo/_base/array",
	"dojo/_base/config", // defaultDuration
	"dojo/_base/lang",
	"../registry",
	"../main"	// for setting exports to dijit namespace
], function(array, config, lang, registry, dijit){

	// module:
	//		dijit/_base/manager

	var exports = {
		// summary:
		//		Deprecated.  Shim to methods on registry, plus a few other declarations.
		//		New code should access dijit/registry directly when possible.
	};

	array.forEach(["byId", "getUniqueId", "findWidgets", "_destroyAll", "byNode", "getEnclosingWidget"], function(name){
		exports[name] = registry[name];
	});

	 lang.mixin(exports, {
		 // defaultDuration: Integer
		 //		The default fx.animation speed (in ms) to use for all Dijit
		 //		transitional fx.animations, unless otherwise specified
		 //		on a per-instance basis. Defaults to 200, overrided by
		 //		`djConfig.defaultDuration`
		 defaultDuration: config["defaultDuration"] || 200
	 });

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'xcitec/form/_XButton':function(){
define([
           "dojo/_base/declare",
           "dojo/dom",
           "dijit/form/_FormWidget",
           "../xcitecUtil"
         ], function (declare, dom, _FormWidget, xcitecUtil) {

    //dojo.require("dijit._Widget");
    //dojo.require("dijit._Templated");
    return declare( _FormWidget, {
      templateString: "<div " +
                      "  data-dojo-attach-event='onclick:_onClick' " +
                      "  class='dijitReset dijitInline ${baseClass}'" +
                      "  data-dojo-attach-point='titleNode,focusNode' " +
                      "><img class='${iconClass}' src='${_blankGif}' width='1' height='1' alt=''/>" +
                      "<span class='${iconTextClass}'>${iconText}</span>"+
                      "</div>",
      baseClass: "",
      iconTextClass: "dijitXcitecBtnTextNone",
      baseIconClass: "iconXBtn",
      iconClass: "",
      iconText: "",
      iconType: "",
      tabIndex: "-1", //to avoid moving by TAB
      readOnly: false,

      postMixInProperties: function() {
        this.inherited(arguments);
        if (this.iconText) {
          this.iconTextClass="dijitXcitecBtnText";
          this.iconClass="iconXBtnNone";
        }
        if (this.iconClass) {

        } else if (this.iconType) {
          //if (!this.iconClass && this.iconType) {
          this.iconClass = this.baseIconClass +" "+ xcitecUtil.createTypedStyleClass(this.baseIconClass,this.iconType);
        } else {
          this.iconClass = this.baseIconClass +"None"
        }
      },

      postCreate: function(){
        this.inherited(arguments);
        dom.setSelectable(this.domNode, false);
      },

      _setReadOnlyAttr: function(/*Boolean*/ value){
        this.setReadOnlyStyle( value );
      },

      setSelectedStyle: function(/*boolean*/ value){
        //	sets selected style for button
        this.selected = value;
        this._setStateClass();
      },

      setReadOnlyStyle: function(/*boolean*/ value){
        //	sets selected style for button
        this.readOnly = value;
        this._setStateClass();
      },

      _onClick: function() {
        this.onDoAction();
      },

      onDoAction: function() {
        //callcack function - have to be defined by user callcack function
        //console.log("onDoAction1");
      }
    });
});

},
'dojo/data/ItemFileReadStore':function(){
define(["../_base/kernel", "../_base/lang", "../_base/declare", "../_base/array", "../_base/xhr",
	"../Evented", "./util/filter", "./util/simpleFetch", "../date/stamp"
], function(kernel, lang, declare, array, xhr, Evented, filterUtil, simpleFetch, dateStamp){

// module:
//		dojo/data/ItemFileReadStore

var ItemFileReadStore = declare("dojo.data.ItemFileReadStore", [Evented],{
	// summary:
	//		The ItemFileReadStore implements the dojo/data/api/Read API and reads
	//		data from JSON files that have contents in this format --
	// |	{ items: [
	// |		{ name:'Kermit', color:'green', age:12, friends:['Gonzo', {_reference:{name:'Fozzie Bear'}}]},
	// |		{ name:'Fozzie Bear', wears:['hat', 'tie']},
	// |		{ name:'Miss Piggy', pets:'Foo-Foo'}
	// |	]}
	//		Note that it can also contain an 'identifier' property that specified which attribute on the items
	//		in the array of items that acts as the unique identifier for that item.

	constructor: function(/* Object */ keywordParameters){
		// summary:
		//		constructor
		// keywordParameters:
		//		{url: String} {data: jsonObject} {typeMap: object}
		//		The structure of the typeMap object is as follows:
		// |	{
		// |		type0: function || object,
		// |		type1: function || object,
		// |		...
		// |		typeN: function || object
		// |	}
		//		Where if it is a function, it is assumed to be an object constructor that takes the
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		// |	{
		// |		type: function, //constructor.
		// |		deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		// |	}

		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = [];
		this._loadFinished = false;
		this._jsonFileUrl = keywordParameters.url;
		this._ccUrl = keywordParameters.url;
		this.url = keywordParameters.url;
		this._jsonData = keywordParameters.data;
		this.data = null;
		this._datatypeMap = keywordParameters.typeMap || {};
		if(!this._datatypeMap['Date']){
			//If no default mapping for dates, then set this as default.
			//We use the dojo/date/stamp here because the ISO format is the 'dojo way'
			//of generically representing dates.
			this._datatypeMap['Date'] = {
				type: Date,
				deserialize: function(value){
					return dateStamp.fromISOString(value);
				}
			};
		}
		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true};
		this._itemsByIdentity = null;
		this._storeRefPropName = "_S"; // Default name for the store reference to attach to every item.
		this._itemNumPropName = "_0"; // Default Item Id for isItem to attach to every item.
		this._rootItemPropName = "_RI"; // Default Item Id for isItem to attach to every item.
		this._reverseRefMap = "_RRM"; // Default attribute for constructing a reverse reference map for use with reference integrity
		this._loadInProgress = false; //Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];
		if(keywordParameters.urlPreventCache !== undefined){
			this.urlPreventCache = keywordParameters.urlPreventCache?true:false;
		}
		if(keywordParameters.hierarchical !== undefined){
			this.hierarchical = keywordParameters.hierarchical?true:false;
		}
		if(keywordParameters.clearOnClose){
			this.clearOnClose = true;
		}
		if("failOk" in keywordParameters){
			this.failOk = keywordParameters.failOk?true:false;
		}
	},

	url: "",	// use "" rather than undefined for the benefit of the parser (#3539)

	//Internal var, crossCheckUrl.  Used so that setting either url or _jsonFileUrl, can still trigger a reload
	//when clearOnClose and close is used.
	_ccUrl: "",

	data: null,	// define this so that the parser can populate it

	typeMap: null, //Define so parser can populate.

	// clearOnClose: Boolean
	//		Parameter to allow users to specify if a close call should force a reload or not.
	//		By default, it retains the old behavior of not clearing if close is called.  But
	//		if set true, the store will be reset to default state.  Note that by doing this,
	//		all item handles will become invalid and a new fetch must be issued.
	clearOnClose: false,

	// urlPreventCache: Boolean
	//		Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.
	//		Note this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.
	//		Added for tracker: #6072
	urlPreventCache: false,

	// failOk: Boolean
	//		Parameter for specifying that it is OK for the xhrGet call to fail silently.
	failOk: false,

	// hierarchical: Boolean
	//		Parameter to indicate to process data from the url as hierarchical
	//		(data items can contain other data items in js form).  Default is true
	//		for backwards compatibility.  False means only root items are processed
	//		as items, all child objects outside of type-mapped objects and those in
	//		specific reference format, are left straight JS data objects.
	hierarchical: true,

	_assertIsItem: function(/* dojo/data/api/Item */ item){
		// summary:
		//		This function tests whether the item passed in is indeed an item in the store.
		// item:
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){
			throw new Error(this.declaredClass + ": Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		// summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		// attribute:
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){
			throw new Error(this.declaredClass + ": Invalid attribute argument.");
		}
	},

	getValue: function(	/* dojo/data/api/Item */ item,
						   /* attribute-name-string */ attribute,
						   /* value? */ defaultValue){
		// summary:
		//		See dojo/data/api/Read.getValue()
		var values = this.getValues(item, attribute);
		return (values.length > 0)?values[0]:defaultValue; // mixed
	},

	getValues: function(/* dojo/data/api/Item */ item,
						/* attribute-name-string */ attribute){
		// summary:
		//		See dojo/data/api/Read.getValues()

		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		// Clone it before returning.  refs: #10474
		return (item[attribute] || []).slice(0); // Array
	},

	getAttributes: function(/* dojo/data/api/Item */ item){
		// summary:
		//		See dojo/data/api/Read.getAttributes()
		this._assertIsItem(item);
		var attributes = [];
		for(var key in item){
			// Save off only the real item attributes, not the special id marks for O(1) isItem.
			if((key !== this._storeRefPropName) && (key !== this._itemNumPropName) && (key !== this._rootItemPropName) && (key !== this._reverseRefMap)){
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* dojo/data/api/Item */ item,
							   /* attribute-name-string */ attribute){
		// summary:
		//		See dojo/data/api/Read.hasAttribute()
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		return (attribute in item);
	},

	containsValue: function(/* dojo/data/api/Item */ item,
							/* attribute-name-string */ attribute,
							/* anything */ value){
		// summary:
		//		See dojo/data/api/Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = filterUtil.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* dojo/data/api/Item */ item,
								 /* attribute-name-string */ attribute,
								 /* anything */ value,
								 /* RegExp?*/ regexp){
		// summary:
		//		Internal function for looking at the values contained by the item.
		// description:
		//		Internal function for looking at the values contained by the item.  This
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		// item:
		//		The data item to examine for attribute values.
		// attribute:
		//		The attribute to inspect.
		// value:
		//		The value to match.
		// regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		return array.some(this.getValues(item, attribute), function(possibleValue){
			if(possibleValue !== null && !lang.isObject(possibleValue) && regexp){
				if(possibleValue.toString().match(regexp)){
					return true; // Boolean
				}
			}else if(value === possibleValue){
				return true; // Boolean
			}
		});
	},

	isItem: function(/* anything */ something){
		// summary:
		//		See dojo/data/api/Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			if(this._arrayOfAllItems[something[this._itemNumPropName]] === something){
				return true;
			}
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		// summary:
		//		See dojo/data/api/Read.isItemLoaded()
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		// summary:
		//		See dojo/data/api/Read.loadItem()
		this._assertIsItem(keywordArgs.item);
	},

	getFeatures: function(){
		// summary:
		//		See dojo/data/api/Read.getFeatures()
		return this._features; //Object
	},

	getLabel: function(/* dojo/data/api/Item */ item){
		// summary:
		//		See dojo/data/api/Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* dojo/data/api/Item */ item){
		// summary:
		//		See dojo/data/api/Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	filter: function(/* Object */ requestArgs, /* item[] */ arrayOfItems, /* Function */ findCallback){
		// summary:
		//		This method handles the basic filtering needs for ItemFile* based stores.
		var items = [],
			i, key;

		if(requestArgs.query){
			var value,
				ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false;

			//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
			//same value for each item examined.  Much more efficient.
			var regexpList = {};
			for(key in requestArgs.query){
				value = requestArgs.query[key];
				if(typeof value === "string"){
					regexpList[key] = filterUtil.patternToRegExp(value, ignoreCase);
				}else if(value instanceof RegExp){
					regexpList[key] = value;
				}
			}
			for(i = 0; i < arrayOfItems.length; ++i){
				var match = true;
				var candidateItem = arrayOfItems[i];
				if(candidateItem === null){
					match = false;
				}else{
					for(key in requestArgs.query){
						value = requestArgs.query[key];
						if(!this._containsValue(candidateItem, key, value, regexpList[key])){
							match = false;
						}
					}
				}
				if(match){
					items.push(candidateItem);
				}
			}
			findCallback(items, requestArgs);
		}else{
			// We want a copy to pass back in case the parent wishes to sort the array.
			// We shouldn't allow resort of the internal list, so that multiple callers
			// can get lists and sort without affecting each other.  We also need to
			// filter out any null values that have been left as a result of deleteItem()
			// calls in ItemFileWriteStore.
			for(i = 0; i < arrayOfItems.length; ++i){
				var item = arrayOfItems[i];
				if(item !== null){
					items.push(item);
				}
			}
			findCallback(items, requestArgs);
		}
	},

	_fetchItems: function(	/* Object */ keywordArgs,
							  /* Function */ findCallback,
							  /* Function */ errorCallback){
		// summary:
		//		See dojo/data/util.simpleFetch.fetch()
		var self = this;

		if(this._loadFinished){
			this.filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions), findCallback);
		}else{
			//Do a check on the JsonFileUrl and crosscheck it.
			//If it doesn't match the cross-check, it needs to be updated
			//This allows for either url or _jsonFileUrl to he changed to
			//reset the store load location.  Done this way for backwards
			//compatibility.  People use _jsonFileUrl (even though officially
			//private.
			if(this._jsonFileUrl !== this._ccUrl){
				kernel.deprecated(this.declaredClass + ": ",
					"To change the url, set the url property of the store," +
						" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}

			//See if there was any forced reset of data.
			if(this.data != null){
				this._jsonData = this.data;
				this.data = null;
			}

			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: lang.hitch(self, "filter"), findCallback: lang.hitch(self, findCallback)});
				}else{
					this._loadInProgress = true;
					var getArgs = {
						url: self._jsonFileUrl,
						handleAs: "json-comment-optional",
						preventCache: this.urlPreventCache,
						failOk: this.failOk
					};
					var getHandler = xhr.get(getArgs);
					getHandler.addCallback(function(data){
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;

							self.filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions), findCallback);
							self._handleQueuedFetches();
						}catch(e){
							self._loadFinished = true;
							self._loadInProgress = false;
							errorCallback(e, keywordArgs);
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						errorCallback(error, keywordArgs);
					});

					//Wire up the cancel to abort of the request
					//This call cancel on the deferred if it hasn't been called
					//yet and then will chain to the simple abort of the
					//simpleFetch keywordArgs
					var oldAbort = null;
					if(keywordArgs.abort){
						oldAbort = keywordArgs.abort;
					}
					keywordArgs.abort = function(){
						var df = getHandler;
						if(df && df.fired === -1){
							df.cancel();
							df = null;
						}
						if(oldAbort){
							oldAbort.call(keywordArgs);
						}
					};
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._getItemsFromLoadedData(this._jsonData);
					this._jsonData = null;
					self.filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions), findCallback);
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error(this.declaredClass + ": No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		}
	},

	_handleQueuedFetches: function(){
		// summary:
		//		Internal function to execute delayed request in the store.
		
		//Execute any deferred fetches now.
		if(this._queuedFetches.length > 0){
			for(var i = 0; i < this._queuedFetches.length; i++){
				var fData = this._queuedFetches[i],
					delayedQuery = fData.args,
					delayedFilter = fData.filter,
					delayedFindCallback = fData.findCallback;
				if(delayedFilter){
					delayedFilter(delayedQuery, this._getItemsArray(delayedQuery.queryOptions), delayedFindCallback);
				}else{
					this.fetchItemByIdentity(delayedQuery);
				}
			}
			this._queuedFetches = [];
		}
	},

	_getItemsArray: function(/*object?*/queryOptions){
		// summary:
		//		Internal function to determine which list of items to search over.
		// queryOptions: The query options parameter, if any.
		if(queryOptions && queryOptions.deep){
			return this._arrayOfAllItems;
		}
		return this._arrayOfTopLevelItems;
	},

	close: function(/*dojo/data/api/Request|Object?*/ request){
		// summary:
		//		See dojo/data/api/Read.close()
		if(this.clearOnClose &&
			this._loadFinished &&
			!this._loadInProgress){
			//Reset all internalsback to default state.  This will force a reload
			//on next fetch.  This also checks that the data or url param was set
			//so that the store knows it can get data.  Without one of those being set,
			//the next fetch will trigger an error.

			if(((this._jsonFileUrl == "" || this._jsonFileUrl == null) &&
				(this.url == "" || this.url == null)
				) && this.data == null){
				console.debug(this.declaredClass + ": WARNING!  Data reload " +
					" information has not been provided." +
					"  Please set 'url' or 'data' to the appropriate value before" +
					" the next fetch");
			}
			this._arrayOfAllItems = [];
			this._arrayOfTopLevelItems = [];
			this._loadFinished = false;
			this._itemsByIdentity = null;
			this._loadInProgress = false;
			this._queuedFetches = [];
		}
	},

	_getItemsFromLoadedData: function(/* Object */ dataObject){
		// summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		// description:
		//		Function to parse the loaded data into item format and build the internal items array.
		// dataObject:
		//		The JS data object containing the raw data to convery into item format.
		// returns: Array
		//		Array of items in store item format.

		// First, we define a couple little utility functions...
		var addingArrays = false,
			self = this;

		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'1802-05-14'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			return (aValue !== null) &&
				(typeof aValue === "object") &&
				(!lang.isArray(aValue) || addingArrays) &&
				(!lang.isFunction(aValue)) &&
				(aValue.constructor == Object || lang.isArray(aValue)) &&
				(typeof aValue._reference === "undefined") &&
				(typeof aValue._type === "undefined") &&
				(typeof aValue._value === "undefined") &&
				self.hierarchical;
		}

		function addItemAndSubItemsToArrayOfAllItems(/* dojo/data/api/Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(lang.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		this._labelAttr = dataObject.label;

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i,
			item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject.items;

		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			if(lang.isArray(item)){
				addingArrays = true;
			}
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items,
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		//
		// We also store the attribute names so we can validate our store
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {},
			key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if(key !== this._rootItemPropName){
					var value = item[key];
					if(value !== null){
						if(!lang.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}
		while(allAttributeNames[this._reverseRefMap]){
			this._reverseRefMap += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is
		// the name of an attribute that holds the identity of each item.
		// If this data file specified an identifier attribute, then build a
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = dataObject.identifier;
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!Object.hasOwnProperty.call(this._itemsByIdentity, identity)){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error(this.declaredClass + ":  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error(this.declaredClass + ":  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] }
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'1918-07-18'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date(1918, 6, 18)) }
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j){
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(("_type" in value) && ("_value" in value)){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){
								throw new Error("dojo.data.ItemFileReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(lang.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(lang.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojo.data.ItemFileReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(!lang.isObject(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._getItemByIdentity(referenceDescription);
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k],
										found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){
											found = false;
										}
									}
									if(found){
										arrayOfValues[j] = candidateItem;
									}
								}
							}
							if(this.referenceIntegrity){
								var refItem = arrayOfValues[j];
								if(this.isItem(refItem)){
									this._addReferenceToMap(refItem, item, key);
								}
							}
						}else if(this.isItem(value)){
							//It's a child item (not one referenced through _reference).
							//We need to treat this as a referenced item, so it can be cleaned up
							//in a write store easily.
							if(this.referenceIntegrity){
								this._addReferenceToMap(value, item, key);
							}
						}
					}
				}
			}
		}
	},

	_addReferenceToMap: function(/*item*/ refItem, /*item*/ parentItem, /*string*/ attribute){
		// summary:
		//		Method to add an reference map entry for an item and attribute.
		// description:
		//		Method to add an reference map entry for an item and attribute.
		// refItem:
		//		The item that is referenced.
		// parentItem:
		//		The item that holds the new reference to refItem.
		// attribute:
		//		The attribute on parentItem that contains the new reference.

		//Stub function, does nothing.  Real processing is in ItemFileWriteStore.
	},

	getIdentity: function(/* dojo/data/api/Item */ item){
		// summary:
		//		See dojo/data/api/Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			return item[this._itemNumPropName]; // Number
		}else{
			var arrayOfValues = item[identifier];
			if(arrayOfValues){
				return arrayOfValues[0]; // Object|String
			}
		}
		return null; // null
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		// summary:
		//		See dojo/data/api/Identity.fetchItemByIdentity()

		// Hasn't loaded yet, we have to trigger the load.
		var item,
			scope;
		if(!this._loadFinished){
			var self = this;
			//Do a check on the JsonFileUrl and crosscheck it.
			//If it doesn't match the cross-check, it needs to be updated
			//This allows for either url or _jsonFileUrl to he changed to
			//reset the store load location.  Done this way for backwards
			//compatibility.  People use _jsonFileUrl (even though officially
			//private.
			if(this._jsonFileUrl !== this._ccUrl){
				kernel.deprecated(this.declaredClass + ": ",
					"To change the url, set the url property of the store," +
						" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}

			//See if there was any forced reset of data.
			if(this.data != null && this._jsonData == null){
				this._jsonData = this.data;
				this.data = null;
			}

			if(this._jsonFileUrl){

				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs});
				}else{
					this._loadInProgress = true;
					var getArgs = {
						url: self._jsonFileUrl,
						handleAs: "json-comment-optional",
						preventCache: this.urlPreventCache,
						failOk: this.failOk
					};
					var getHandler = xhr.get(getArgs);
					getHandler.addCallback(function(data){
						var scope = keywordArgs.scope?keywordArgs.scope:kernel.global;
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							item = self._getItemByIdentity(keywordArgs.identity);
							if(keywordArgs.onItem){
								keywordArgs.onItem.call(scope, item);
							}
							self._handleQueuedFetches();
						}catch(error){
							self._loadInProgress = false;
							if(keywordArgs.onError){
								keywordArgs.onError.call(scope, error);
							}
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						if(keywordArgs.onError){
							var scope = keywordArgs.scope?keywordArgs.scope:kernel.global;
							keywordArgs.onError.call(scope, error);
						}
					});
				}

			}else if(this._jsonData){
				// Passed in data, no need to xhr.
				self._getItemsFromLoadedData(self._jsonData);
				self._jsonData = null;
				self._loadFinished = true;
				item = self._getItemByIdentity(keywordArgs.identity);
				if(keywordArgs.onItem){
					scope = keywordArgs.scope?keywordArgs.scope:kernel.global;
					keywordArgs.onItem.call(scope, item);
				}
			}
		}else{
			// Already loaded.  We can just look it up and call back.
			item = this._getItemByIdentity(keywordArgs.identity);
			if(keywordArgs.onItem){
				scope = keywordArgs.scope?keywordArgs.scope:kernel.global;
				keywordArgs.onItem.call(scope, item);
			}
		}
	},

	_getItemByIdentity: function(/* Object */ identity){
		// summary:
		//		Internal function to look an item up by its identity map.
		var item = null;
		if(this._itemsByIdentity){
			// If this map is defined, we need to just try to get it.  If it fails
			// the item does not exist.
			if(Object.hasOwnProperty.call(this._itemsByIdentity, identity)){
				item = this._itemsByIdentity[identity];
			}
		}else if (Object.hasOwnProperty.call(this._arrayOfAllItems, identity)){
			item = this._arrayOfAllItems[identity];
		}
		if(item === undefined){
			item = null;
		}
		return item; // Object
	},

	getIdentityAttributes: function(/* dojo/data/api/Item */ item){
		// summary:
		//		See dojo/data/api/Identity.getIdentityAttributes()

		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			// If (identifier === Number) it means getIdentity() just returns
			// an integer item-number for each item.  The dojo/data/api/Identity
			// spec says we need to return null if the identity is not composed
			// of attributes
			return null; // null
		}else{
			return [identifier]; // Array
		}
	},

	_forceLoad: function(){
		// summary:
		//		Internal function to force a load of the store if it hasn't occurred yet.  This is required
		//		for specific functions to work properly.
		var self = this;
		//Do a check on the JsonFileUrl and crosscheck it.
		//If it doesn't match the cross-check, it needs to be updated
		//This allows for either url or _jsonFileUrl to he changed to
		//reset the store load location.  Done this way for backwards
		//compatibility.  People use _jsonFileUrl (even though officially
		//private.
		if(this._jsonFileUrl !== this._ccUrl){
			kernel.deprecated(this.declaredClass + ": ",
				"To change the url, set the url property of the store," +
					" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
			this._ccUrl = this._jsonFileUrl;
			this.url = this._jsonFileUrl;
		}else if(this.url !== this._ccUrl){
			this._jsonFileUrl = this.url;
			this._ccUrl = this.url;
		}

		//See if there was any forced reset of data.
		if(this.data != null){
			this._jsonData = this.data;
			this.data = null;
		}

		if(this._jsonFileUrl){
			var getArgs = {
				url: this._jsonFileUrl,
				handleAs: "json-comment-optional",
				preventCache: this.urlPreventCache,
				failOk: this.failOk,
				sync: true
			};
			var getHandler = xhr.get(getArgs);
			getHandler.addCallback(function(data){
				try{
					//Check to be sure there wasn't another load going on concurrently
					//So we don't clobber data that comes in on it.  If there is a load going on
					//then do not save this data.  It will potentially clobber current data.
					//We mainly wanted to sync/wait here.
					//TODO:  Revisit the loading scheme of this store to improve multi-initial
					//request handling.
					if(self._loadInProgress !== true && !self._loadFinished){
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
					}else if(self._loadInProgress){
						//Okay, we hit an error state we can't recover from.  A forced load occurred
						//while an async load was occurring.  Since we cannot block at this point, the best
						//that can be managed is to throw an error.
						throw new Error(this.declaredClass + ":  Unable to perform a synchronous load, an async load is in progress.");
					}
				}catch(e){
					console.log(e);
					throw e;
				}
			});
			getHandler.addErrback(function(error){
				throw error;
			});
		}else if(this._jsonData){
			self._getItemsFromLoadedData(self._jsonData);
			self._jsonData = null;
			self._loadFinished = true;
		}
	}
});
//Mix in the simple fetch implementation to this class.
lang.extend(ItemFileReadStore,simpleFetch);

return ItemFileReadStore;

});

},
'xcitec/layout/FilterField':function(){
define([
         "dojo/_base/declare",
         "./DoubleField"
       ], function (declare, DoubleField) {
  return declare(DoubleField, {
    layout: "filter"
  });
});

},
'xcitec/grid/LinkToolbar':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, array, declare, domAttr, domConstruct, _Container, _Templated, _Widget) {

	var LinkToolbar = declare( [_Widget, _Templated, _Container], {
	    templateString: '<div class="dataTableButton"><ul data-dojo-attach-point="containerNode"></ul></div>',
	
	    startup: function() {
	        var children = this.getChildren();
	        var _this = this;
	        array.forEach(children, function(child) {
	            domAttr.set(child.domNode, 'class', domAttr.get(child.domNode, 'class').replace(/linkButton/gi, "linkToolbarButton"));
	            child.baseClass = "linkToolbarButton";
	            var li = domConstruct.create("li");
	            li.appendChild(child.domNode);
	            _this.containerNode.appendChild(li);
	        });
	    }
	});
	return LinkToolbar;
});

},
'xcitec/grid/GridControl':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/Label"
], function (dojo, declare, lang, domAttr, domClass, domConstruct, query, _Widget, registry, Label) {

	 
	
	var GridControl = declare( _Widget, {
	  caption: "",
	  captionId: "",
	  labelWidget: null,
	  vertScroll: false,
	  gridId: null,
	  required:false,
	
	  postMixInProperties: function() {
	    this.inherited(arguments);
	    var grid = query("table", this.srcNodeRef)[0];
	    if (grid) {
	        if (this.vertScroll) {
	            domAttr.set(grid, "vertScroll", "true");
	        }
	        domAttr.set(grid, "parentId", this.id);
	    }
	    this.gridId = domAttr.get(grid, "id");
	
	    // preselected rows
	    var srn = this.gridId + ".selRows";
	    this.selectedRowsNode = query("[id='" + srn + "']", this.srcNodeRef)[0];
	
	    // column widths
	    var cwn = this.gridId + ".columnWidths";
	    this.columnWidthsNode = query("[id='" + cwn + "']", this.srcNodeRef)[0];
	
	    // active column
	    var ac = this.gridId + ".activeColumn";
	    this.activeColumn = query("[id='" + ac + "']", this.srcNodeRef)[0];
	
	    // grid column order
	    var co = this.gridId + ".columnOrder";
	    this.columnOrder = query("[id='" + co + "']", this.srcNodeRef)[0];
	    
	    if (this.required) {
	        this.baseClass = this.baseClass + "dijitXcitecGridRequired";
	    }
	  },
	
	  buildRendering: function(){
	    this.inherited(arguments);
	    domClass.add(this.domNode, "XcitecGridControl");
	    if (this.caption) {
	      this.labelWidget = new Label({
	        label: this.caption,
	        labelId: this.captionId
	        //htmlFor: dojo.attr(this.controlNode,"widgetId"),
	        //labelType: this.labelType,
	        //required: this.labelRequired,
	        //disabled: this.labelDisabled
	      });
	      domConstruct.place(this.labelWidget.domNode, this.domNode,"first");
	    }
	    this.domNode.onsubmit = lang.hitch(this, this.onSubmit);
	  },
	
	  onSubmit: function(){
	      return true;
	  },
	    
	  destroy: function(){
	    if (this.labelWidget) this.labelWidget.destroy();
	    if (this.gridId) {
	      registry.byId(this.gridId).destroyRecursive();
	    }
	    this.inherited(arguments);
	  },

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
			domAttr.remove(this.domNode, 'disabled');
		}
	});
	return GridControl;
});

},
'xcitec/grid/_HGridBase':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/dnd/Source",
	"dojo/dom-geometry",
	"dojo/has",
	"dojo/on",
	"dojo/query",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/grid/Avatar",
  "../xcitecUtil"
], function (dojo, domStyle, declare, lang, windowModule, array, domConstruct, domClass, Manager, domAttr, Source, domGeometry, has, on, query, topic, _Container,
             _Templated, _Widget, registry, Avatar, xcitecUtil) {

	/***
	 *
	 * Grid base widget
	 *
	 */
	var _HGridBase = declare( [_Widget, _Templated], {
		rows: null,
		headWidget: null,
	//  strange but defined here looks like static ????     
	//    columnWidths: [],
		showContextMenu: false,
		// enables grid resizing
		gridResize: true,
	
		parentId: "",
	
		// is multiselect available
		allowMultiSelect: false,
		// the message for empty grid
		emptyGridMessage: "",
		// entity class for icon on the grid link toolbar
		entityClass: "",
		// using of vertical scroll - temporary field, later will be replaced by fitToContainer
		vertScroll: false,
		// number of shown rows if vertScroll is true
		shownRows: 5,
		_defaultShownRows: 5,
		_rowHeight: 19,
		_headerHeight: 21,
		_footerHeight: 20,
		// enables column reorder
		columnReorder: false,
	
		isPortletContent: false,
	
		postMixInProperties: function(){
			this.columnWidths = []; // defined here because as attribute looks like static
			if(this.isPortletContent){
				this.vertScroll = true;
			}
			this.inherited(arguments);
		},
	
		buildRendering: function(){
			// get header, body and footer domNodes for xcitec.grid._HGridHeader, xcitec.grid._HGridRows and xitec.grid._HGridFooter
			this.headerNode = query("thead > tr", this.srcNodeRef)[0];
			this.rowsNode = query("tbody", this.srcNodeRef)[0];
	
			this.gridControl = registry.byId(this.parentId);
	
			// grid actions
			this.gridActions = registry.byId(this.id + ".actions");
	
			this.inherited(arguments);
	
			// set Entity Icon if defined
			if(this.entityClass){
				var image = domConstruct.create('span', {}, this.footer, "last");
				domClass.add(image, 'iconDataTable');
				domClass.add(image, xcitecUtil.createEntityIconClass(this.entityClass));
			}
	
			// disable browser context menu
			if(!this.showContextMenu){
				on(this.domNode, "contextmenu", function(e){
					e.preventDefault(), e.stopPropagation();
				});
			}
		},
	
		_fillContent: function(/*DomNode*/ source){
			// parse content and create grid
		},
	
		postCreate: function(){
			if(this.isPortletContent) {
				domStyle.set(this.headWidget.domNode, "display", "none");
				domStyle.set(this.footer, "display", "none");
				domStyle.set(query(".dataTableHeadBg",this.domNode)[0], "display", "none");
				domStyle.set(this.dataTableBody, {
					top: "0",
					bottom: "0"
				});
				domStyle.set(this.dataTable, "border", "0");
			}
	        this.inherited(arguments);

			if(this.vertScroll && this.shownRows > this._defaultShownRows && this.rows && this.rows.allRows.length > this._defaultShownRows){
				domStyle.set(this.domNode, 'height', (this._headerHeight + this._footerHeight + this._rowHeight * this.shownRows) + 'px');
			}
		},

		startup: function(){
			this.inherited(arguments);
			var linkToolbar = registry.byId(this.id + '.linkButtons');
			if(linkToolbar && this.entityClass){
				domClass.add(linkToolbar.domNode, 'dataTableButtonAbsolute');
			}
		},
	
		updateOrder: function(){
			var colOrder = "";
			var _this = this;
			var allCells = this.headWidget.allCells;
			if(this.hasFrozenColumn){
				allCells = this.headFrozenWidget.allCells.concat(this.headWidget.allCells);
			}
			array.forEach(allCells, function(cell){
				if(cell.columnId){
					colOrder = colOrder + cell.columnId + ",";
				}
			});
			colOrder = colOrder.substr(0, colOrder.length - 1);
			if(this.gridControl && this.gridControl.columnOrder){
				if(colOrder == ""){
					domAttr.set(this.gridControl.columnOrder, "value", "#DELETED#");
				} else{
					domAttr.set(this.gridControl.columnOrder, "value", colOrder);
				}
			}
			this.updateColumnWidthsInput();
		},
	
		updateColumnWidthsInput: function(){
			if(this.gridControl.columnWidthsNode){
				var _this = this;
				var allCells = this.headWidget.allCells;
				if(this.hasFrozenColumn){
					var frozenCells = this.headFrozenWidget.allCells.slice();
					frozenCells.pop();
					allCells = frozenCells.concat(this.headWidget.allCells);
				}
				array.forEach(allCells, function(cell, index){
					_this.columnWidths[index] = cell.cellWidth;
				});
				this.columnWidths.pop();
				this.gridControl.columnWidthsNode.value = this.columnWidths.toString();
			}
		},
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		},
	
		destroy: function(){
			var actions = registry.byId(this.id + ".actions");
			if(actions){
				actions.destroyRecursive();
			}
			var linkButtons = registry.byId(this.id + ".linkButtons");
			if(linkButtons){
				linkButtons.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
	return _HGridBase;
});

},
'dijit/WidgetSet':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.global
	"./registry"	// to add functions to dijit.registry
], function(array, declare, kernel, registry){

	// module:
	//		dijit/WidgetSet

	var WidgetSet = declare("dijit.WidgetSet", null, {
		// summary:
		//		A set of widgets indexed by id.
		//		Deprecated, will be removed in 2.0.
		//
		// example:
		//		Create a small list of widgets:
		//		|	require(["dijit/WidgetSet", "dijit/registry"],
		//		|		function(WidgetSet, registry){
		//		|		var ws = new WidgetSet();
		//		|		ws.add(registry.byId("one"));
		//		|		ws.add(registry.byId("two"));
		//		|		// destroy both:
		//		|		ws.forEach(function(w){ w.destroy(); });
		//		|	});

		constructor: function(){
			this._hash = {};
			this.length = 0;
		},

		add: function(/*dijit/_WidgetBase*/ widget){
			// summary:
			//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
			//
			// widget: dijit/_WidgetBase
			//		Any dijit/_WidgetBase subclass.
			if(this._hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			this._hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
			//		removes the reference.
			if(this._hash[id]){
				delete this._hash[id];
				this.length--;
			}
		},

		forEach: function(/*Function*/ func, /* Object? */thisObj){
			// summary:
			//		Call specified function for each widget in this set.
			//
			// func:
			//		A callback function to run for each item. Is passed the widget, the index
			//		in the iteration, and the full hash, similar to `array.forEach`.
			//
			// thisObj:
			//		An optional scope parameter
			//
			// example:
			//		Using the default `dijit.registry` instance:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.forEach(function(widget){
			//		|			console.log(widget.declaredClass);
			//		|		});
			//		|	});
			//
			// returns:
			//		Returns self, in order to allow for further chaining.

			thisObj = thisObj || kernel.global;
			var i = 0, id;
			for(id in this._hash){
				func.call(thisObj, this._hash[id], i++, this._hash);
			}
			return this;	// dijit/WidgetSet
		},

		filter: function(/*Function*/ filter, /* Object? */thisObj){
			// summary:
			//		Filter down this WidgetSet to a smaller new WidgetSet
			//		Works the same as `array.filter` and `NodeList.filter`
			//
			// filter:
			//		Callback function to test truthiness. Is passed the widget
			//		reference and the pseudo-index in the object.
			//
			// thisObj: Object?
			//		Option scope to use for the filter function.
			//
			// example:
			//		Arbitrary: select the odd widgets in this list
			//		|	
			//		|		
			//		|	
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.filter(function(w, i){
			//		|			return i % 2 == 0;
			//		|		}).forEach(function(w){ /* odd ones */ });
			//		|	});

			thisObj = thisObj || kernel.global;
			var res = new WidgetSet(), i = 0, id;
			for(id in this._hash){
				var w = this._hash[id];
				if(filter.call(thisObj, w, i++, this._hash)){
					res.add(w);
				}
			}
			return res; // dijit/WidgetSet
		},

		byId: function(/*String*/ id){
			// summary:
			//		Find a widget in this list by it's id.
			// example:
			//		Test if an id is in a particular WidgetSet
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		var ws = new WidgetSet();
			//		|		ws.add(registry.byId("bar"));
			//		|		var t = ws.byId("bar") // returns a widget
			//		|		var x = ws.byId("foo"); // returns undefined
			//		|	});

			return this._hash[id];	// dijit/_WidgetBase
		},

		byClass: function(/*String*/ cls){
			// summary:
			//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
			//
			// cls: String
			//		The Class to scan for. Full dot-notated string.
			//
			// example:
			//		Find all `dijit.TitlePane`s in a page:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });
			//		|	});

			var res = new WidgetSet(), id, widget;
			for(id in this._hash){
				widget = this._hash[id];
				if(widget.declaredClass == cls){
					res.add(widget);
				}
			 }
			 return res; // dijit/WidgetSet
		},

		toArray: function(){
			// summary:
			//		Convert this WidgetSet into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		array.map(registry.toArray(), function(w){ return w.domNode; });
			//		|	});


			var ar = [];
			for(var id in this._hash){
				ar.push(this._hash[id]);
			}
			return ar;	// dijit/_WidgetBase[]
		},

		map: function(/* Function */func, /* Object? */thisObj){
			// summary:
			//		Create a new Array from this WidgetSet, following the same rules as `array.map`
			// example:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		var nodes = registry.map(function(w){ return w.domNode; });
			//		|	});
			//
			// returns:
			//		A new array of the returned values.
			return array.map(this.toArray(), func, thisObj); // Array
		},

		every: function(func, thisObj){
			// summary:
			//		A synthetic clone of `array.every` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first false return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || kernel.global;
			var x = 0, i;
			for(i in this._hash){
				if(!func.call(thisObj, this._hash[i], x++, this._hash)){
					return false; // Boolean
				}
			}
			return true; // Boolean
		},

		some: function(func, thisObj){
			// summary:
			//		A synthetic clone of `array.some` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first true return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || kernel.global;
			var x = 0, i;
			for(i in this._hash){
				if(func.call(thisObj, this._hash[i], x++, this._hash)){
					return true; // Boolean
				}
			}
			return false; // Boolean
		}

	});

	// Add in 1.x compatibility methods to dijit/registry.
	// These functions won't show up in the API doc but since they are deprecated anyway,
	// that's probably for the best.
	array.forEach(["forEach", "filter", "byClass", "map", "every", "some"], function(func){
		registry[func] = WidgetSet.prototype[func];
	});


	return WidgetSet;
});

},
'dijit/form/RangeBoundTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/i18n", // i18n.getLocalization
	"./MappedTextBox"
], function(declare, i18n, MappedTextBox){

	// module:
	//		dijit/form/RangeBoundTextBox


	var RangeBoundTextBox = declare("dijit.form.RangeBoundTextBox", MappedTextBox, {
		// summary:
		//		Base class for textbox form widgets which defines a range of valid values.

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		/*=====
		// constraints: RangeBoundTextBox.__Constraints
		constraints: {},
		======*/

		rangeCheck: function(/*Number*/ primitive, /*dijit/form/RangeBoundTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the range of the numeric input value.
			// tags:
			//		protected
			return	("min" in constraints? (this.compare(primitive,constraints.min) >= 0) : true) &&
				("max" in constraints? (this.compare(primitive,constraints.max) <= 0) : true); // Boolean
		},

		isInRange: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if the value is in the min/max range specified in constraints
			// tags:
			//		protected
			return this.rangeCheck(this.get('value'), this.constraints);
		},

		_isDefinitelyOutOfRange: function(){
			// summary:
			//		Returns true if the value is out of range and will remain
			//		out of range even if the user types more characters
			var val = this.get('value');
			if(val == null){ return false; } // not yet valid enough to compare to
			var outOfRange = false;
			if("min" in this.constraints){
				var min = this.constraints.min;
				outOfRange = this.compare(val, ((typeof min == "number") && min >= 0 && val != 0) ? 0 : min) < 0;
			}
			if(!outOfRange && ("max" in this.constraints)){
				var max = this.constraints.max;
				outOfRange = this.compare(val, ((typeof max != "number") || max > 0) ? max : 0) > 0;
			}
			return outOfRange;
		},

		_isValidSubset: function(){
			// summary:
			//		Overrides `dijit/form/ValidationTextBox._isValidSubset()`.
			//		Returns true if the input is syntactically valid, and either within
			//		range or could be made in range by more typing.
			return this.inherited(arguments) && !this._isDefinitelyOutOfRange();
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/ValidationTextBox.isValid() to check that the value is also in range.
			return this.inherited(arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused)); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/ValidationTextBox.getErrorMessage() to print "out of range" message if appropriate
			var v = this.get('value');
			if(v != null /* and !undefined */ && v !== '' && (typeof v != "number" || !isNaN(v)) && !this.isInRange(isFocused)){ // don't check isInRange w/o a real value
				return this.rangeMessage; // String
			}
			return this.inherited(arguments);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.rangeMessage){
				this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		}
	});
	/*=====
	RangeBoundTextBox.__Constraints = declare(null, {
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		// max: Number
		//		Maximum signed value.  Default is +Infinity
	});
	=====*/
	return RangeBoundTextBox;
});

},
'dojo/dom-class':function(){
define(["./_base/lang", "./_base/array", "./dom"], function(lang, array, dom){
	// module:
	//		dojo/dom-class

	var className = "className";

	/* Part I of classList-based implementation is preserved here for posterity
	var classList = "classList";
	has.add("dom-classList", function(){
		return classList in document.createElement("p");
	});
	*/

	// =============================
	// (CSS) Class Functions
	// =============================

	var cls, // exports object
		spaces = /\s+/, a1 = [""];

	function str2array(s){
		if(typeof s == "string" || s instanceof String){
			if(s && !spaces.test(s)){
				a1[0] = s;
				return a1;
			}
			var a = s.split(spaces);
			if(a.length && !a[0]){
				a.shift();
			}
			if(a.length && !a[a.length - 1]){
				a.pop();
			}
			return a;
		}
		// assumed to be an array
		if(!s){
			return [];
		}
		return array.filter(s, function(x){ return x; });
	}

	/* Part II of classList-based implementation is preserved here for posterity
	if(has("dom-classList")){
		// new classList version
		cls = {
			contains: function containsClass(node, classStr){
				var clslst = classStr && dom.byId(node)[classList];
				return clslst && clslst.contains(classStr); // Boolean
			},

			add: function addClass(node, classStr){
				node = dom.byId(node);
				classStr = str2array(classStr);
				for(var i = 0, len = classStr.length; i < len; ++i){
					node[classList].add(classStr[i]);
				}
			},

			remove: function removeClass(node, classStr){
				node = dom.byId(node);
				if(classStr === undefined){
					node[className] = "";
				}else{
					classStr = str2array(classStr);
					for(var i = 0, len = classStr.length; i < len; ++i){
						node[classList].remove(classStr[i]);
					}
				}
			},

			replace: function replaceClass(node, addClassStr, removeClassStr){
				node = dom.byId(node);
				if(removeClassStr === undefined){
					node[className] = "";
				}else{
					removeClassStr = str2array(removeClassStr);
					for(var i = 0, len = removeClassStr.length; i < len; ++i){
						node[classList].remove(removeClassStr[i]);
					}
				}
				addClassStr = str2array(addClassStr);
				for(i = 0, len = addClassStr.length; i < len; ++i){
					node[classList].add(addClassStr[i]);
				}
			},

			toggle: function toggleClass(node, classStr, condition){
				node = dom.byId(node);
				if(condition === undefined){
					classStr = str2array(classStr);
					for(var i = 0, len = classStr.length; i < len; ++i){
						node[classList].toggle(classStr[i]);
					}
				}else{
					cls[condition ? "add" : "remove"](node, classStr);
				}
				return condition;   // Boolean
			}
		}
	}
	*/

	// regular DOM version
	var fakeNode = {};  // for effective replacement
	cls = {
		// summary:
		//		This module defines the core dojo DOM class API.

		contains: function containsClass(/*DomNode|String*/ node, /*String*/ classStr){
			// summary:
			//		Returns whether or not the specified classes are a portion of the
			//		class list currently applied to the node.
			// node: String|DOMNode
			//		String ID or DomNode reference to check the class for.
			// classStr: String
			//		A string class name to look for.
			// example:
			//		Do something if a node with id="someNode" has class="aSillyClassName" present
			//	|	if(dojo.hasClass("someNode","aSillyClassName")){ ... }

			return ((" " + dom.byId(node)[className] + " ").indexOf(" " + classStr + " ") >= 0); // Boolean
		},

		add: function addClass(/*DomNode|String*/ node, /*String|Array*/ classStr){
			// summary:
			//		Adds the specified classes to the end of the class list on the
			//		passed node. Will not re-apply duplicate classes.
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to add a class string too
			//
			// classStr: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			//
			// example:
			//		Add a class to some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.add("someNode", "anewClass");
			//	|	});
			//
			// example:
			//		Add two classes at once:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.add("someNode", "firstClass secondClass");
			//	|	});
			//
			// example:
			//		Add two classes at once (using array):
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.add("someNode", ["firstClass", "secondClass"]);
			//	|	});
			//
			// example:
			//		Available in `dojo/NodeList` for multiple additions
			//	|	require(["dojo/query"], function(query){
			//	|		query("ul > li").addClass("firstLevel");
			//	|	});

			node = dom.byId(node);
			classStr = str2array(classStr);
			var cls = node[className], oldLen;
			cls = cls ? " " + cls + " " : " ";
			oldLen = cls.length;
			for(var i = 0, len = classStr.length, c; i < len; ++i){
				c = classStr[i];
				if(c && cls.indexOf(" " + c + " ") < 0){
					cls += c + " ";
				}
			}
			if(oldLen < cls.length){
				node[className] = cls.substr(1, cls.length - 2);
			}
		},

		remove: function removeClass(/*DomNode|String*/ node, /*String|Array?*/ classStr){
			// summary:
			//		Removes the specified classes from node. No `contains()`
			//		check is required.
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to remove the class from.
			//
			// classStr: String|Array
			//		An optional String class name to remove, or several space-separated
			//		class names, or an array of class names. If omitted, all class names
			//		will be deleted.
			//
			// example:
			//		Remove a class from some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode", "firstClass");
			//	|	});
			//
			// example:
			//		Remove two classes from some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode", "firstClass secondClass");
			//	|	});
			//
			// example:
			//		Remove two classes from some node (using array):
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode", ["firstClass", "secondClass"]);
			//	|	});
			//
			// example:
			//		Remove all classes from some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode");
			//	|	});
			//
			// example:
			//		Available in `dojo/NodeList` for multiple removal
			//	|	require(["dojo/query"], function(query){
			//	|		query("ul > li").removeClass("foo");
			//	|	});

			node = dom.byId(node);
			var cls;
			if(classStr !== undefined){
				classStr = str2array(classStr);
				cls = " " + node[className] + " ";
				for(var i = 0, len = classStr.length; i < len; ++i){
					cls = cls.replace(" " + classStr[i] + " ", " ");
				}
				cls = lang.trim(cls);
			}else{
				cls = "";
			}
			if(node[className] != cls){ node[className] = cls; }
		},

		replace: function replaceClass(/*DomNode|String*/ node, /*String|Array*/ addClassStr, /*String|Array?*/ removeClassStr){
			// summary:
			//		Replaces one or more classes on a node if not present.
			//		Operates more quickly than calling dojo.removeClass and dojo.addClass
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to remove the class from.
			//
			// addClassStr: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			//
			// removeClassStr: String|Array?
			//		A String class name to remove, or several space-separated class names,
			//		or an array of class names.
			//
			// example:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.replace("someNode", "add1 add2", "remove1 remove2");
			//	|	});
			//
			// example:
			//	Replace all classes with addMe
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.replace("someNode", "addMe");
			//	|	});
			//
			// example:
			//	Available in `dojo/NodeList` for multiple toggles
			//	|	require(["dojo/query"], function(query){
			//	|		query(".findMe").replaceClass("addMe", "removeMe");
			//	|	});

			node = dom.byId(node);
			fakeNode[className] = node[className];
			cls.remove(fakeNode, removeClassStr);
			cls.add(fakeNode, addClassStr);
			if(node[className] !== fakeNode[className]){
				node[className] = fakeNode[className];
			}
		},

		toggle: function toggleClass(/*DomNode|String*/ node, /*String|Array*/ classStr, /*Boolean?*/ condition){
			// summary:
			//		Adds a class to node if not present, or removes if present.
			//		Pass a boolean condition if you want to explicitly add or remove.
			//		Returns the condition that was specified directly or indirectly.
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to toggle a class string
			//
			// classStr: String|Array
			//		A String class name to toggle, or several space-separated class names,
			//		or an array of class names.
			//
			// condition:
			//		If passed, true means to add the class, false means to remove.
			//		Otherwise dojo.hasClass(node, classStr) is used to detect the class presence.
			//
			// example:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.toggle("someNode", "hovered");
			//	|	});
			//
			// example:
			//		Forcefully add a class
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.toggle("someNode", "hovered", true);
			//	|	});
			//
			// example:
			//		Available in `dojo/NodeList` for multiple toggles
			//	|	require(["dojo/query"], function(query){
			//	|		query(".toggleMe").toggleClass("toggleMe");
			//	|	});

			node = dom.byId(node);
			if(condition === undefined){
				classStr = str2array(classStr);
				for(var i = 0, len = classStr.length, c; i < len; ++i){
					c = classStr[i];
					cls[cls.contains(node, c) ? "remove" : "add"](node, c);
				}
			}else{
				cls[condition ? "add" : "remove"](node, classStr);
			}
			return condition;   // Boolean
		}
	};

	return cls;
});

},
'idx/form/HorizontalSlider':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/has",
	"dojo/on",
	"dojo/dom-style",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dijit/_base/wai",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/form/HorizontalSlider",
	"idx/typematic",
	"./_CssStateMixin",
	"./_ValidationMixin",
	"./_CompositeMixin",
	"require", 
	"require",
	
	"idx/has!#idx_form_HorizontalSlider-desktop?dojo/text!./templates/HorizontalSlider.html"  // desktop widget, load the template
		+ ":#idx_form_HorizontalSlider-mobile?"										// mobile widget, don't load desktop template
		+ ":#desktop?dojo/text!./templates/HorizontalSlider.html"						// global desktop platform, load template
		+ ":#mobile?"														// global mobile platform, don't load
		+ ":dojo/text!./templates/HorizontalSlider.html", 							// no dojo/has features, load the template
		
	"idx/has!#idx_form_HorizontalSlider-mobile?./plugins/mobile/HorizontalSliderPlugin"		// mobile widget, load the plugin
		+ ":#idx_form_HorizontalSlider-desktop?"										// desktop widget, don't load plugin
		+ ":#mobile?./plugins/mobile/HorizontalSliderPlugin"							// global mobile platform, load plugin
		+ ":"																// no features, don't load plugin
		
], function(declare, lang, has, on, domStyle, domAttr, domClass, domConstruct, wai, 
			_WidgetBase, _TemplatedMixin, HorizontalSlider, typematic, _CssStateMixin, _ValidationMixin, _CompositeMixin,
			_TemplatePlugableMixin, PlatformPluginRegistry, desktopTemplate, MobilePlugin){
	var iForm = lang.getObject("idx.oneui.form", true); // for backward compatibility with IDX 1.2

	/**
	* @name idx.form.HorizontalSlider
	* @class A form widget that allows one to select a value with a horizontally draggable handle
	* @augments dijit.form.HorizontalSlider
	* @augments idx.form._ValidationMixin
	* @augments idx.form._CompositeMixin
	* @augments idx.form._CssStateMixin
	*/ 
	var HorizontalSlider = declare("idx.form.HorizontalSlider_base", [HorizontalSlider, _ValidationMixin, _CompositeMixin, _CssStateMixin], {
	/**@lends idx.form.HorizontalSlider*/ 
		// summary:
		//		A form widget that allows one to select a value with a horizontally draggable handle
		
		/**
		* Fire validation when widget get input by set true, 
		* fire validation when widget get blur by set false
		* @type Boolean
		*/
		templateString: desktopTemplate,
		// instantValidate: Boolean
		//		Fire validation when widget get input by set true, 
		//		fire validation when widget get blur by set false
		instantValidate: true,
		/**
		* base class of dijit widget
		*/
		oneuiBaseClass: "dijitSlider",
		/**
		* base class of this oneui widget
		*/
		baseClass:"idxSliderWrapH",
		cssStateNodes: {
			incrementButton: "dijitSliderIncrementButton",
			decrementButton: "dijitSliderDecrementButton",
			focusNode: "dijitSliderThumb"
		},
		/** @ignore */
		postCreate: function(){
			this._event = {
				"input" : "_setValueAttr",
				"blur" 	: "_onBlur",
				"focus" : "_onFocus"
			}
			
			if(this.showButtons){
				this.own(
					typematic.addTouchListener(this.decrementButton, this, "_typematicCallback", 25, 500),
					typematic.addTouchListener(this.incrementButton, this, "_typematicCallback", 25, 500)
				);
				this.showButtons = false;
				this.inherited(arguments);
				this.showButtons = true;
			}else{
				this.inherited(arguments);
			}
			
			this._resize();
		},
		/** @ignore */
		startup: function(){
			this.inherited(arguments);
			var width = domStyle.get(this.domNode, "width");
			domStyle.set(this.oneuiBaseNode, "width", width + "px");
		},
		/**
		* use set("label", labelValue) to set the label of the HorizontalSlider
		*/
		_setLabelAttr: function(/*String*/ label){
			this.inherited(arguments);
			wai.setWaiState(this.focusNode, "labelledby", this.id + "_label");
			domAttr.remove(this.compLabelNode, "for");
		}
	});
	if(has("mobile") || has("platform-plugable")){
	
		var pluginRegistry = PlatformPluginRegistry.register("idx/form/HorizontalSlider", {	
			desktop: "inherited",	// no plugin for desktop, use inherited methods  
			mobile: MobilePlugin	// use the mobile plugin if loaded
		});
		
		HorizontalSlider = declare([HorizontalSlider,_TemplatePlugableMixin], {
			/**
		     * Set the template path for the desktop template in case the template was not 
		     * loaded initially, but is later needed due to an instance being constructed 
		     * with "desktop" platform.
	     	 */
			templatePath: require.toUrl("idx/form/templates/HorizontalSlider.html"),  
			// set the plugin registry
			pluginRegistry: pluginRegistry
		});
	}
	return iForm.HorizontalSlider = declare("idx.form.HorizontalSlider", HorizontalSlider);
});

},
'dojo/touch':function(){
define(["./_base/kernel", "./aspect", "./dom", "./dom-class", "./_base/lang", "./on", "./has", "./mouse", "./domReady", "./_base/window"],
function(dojo, aspect, dom, domClass, lang, on, has, mouse, domReady, win){

	// module:
	//		dojo/touch

	var hasTouch = has("touch");

	var ios4 = has("ios") < 5;
	
	var msPointer = navigator.pointerEnabled || navigator.msPointerEnabled,
		pointer = (function () {
			var pointer = {};
			for (var type in { down: 1, move: 1, up: 1, cancel: 1, over: 1, out: 1 }) {
				pointer[type] = !navigator.pointerEnabled ?
					"MSPointer" + type.charAt(0).toUpperCase() + type.slice(1) :
					"pointer" + type;
			}
			return pointer;
		})();

	// Click generation variables
	var clicksInited, clickTracker, clickTarget, clickX, clickY, clickDx, clickDy, clickTime;

	// Time of most recent touchstart, touchmove, or touchend event
	var lastTouch;

	function dualEvent(mouseType, touchType, msPointerType){
		// Returns synthetic event that listens for both the specified mouse event and specified touch event.
		// But ignore fake mouse events that were generated due to the user touching the screen.
		if(msPointer && msPointerType){
			// IE10+: MSPointer* events are designed to handle both mouse and touch in a uniform way,
			// so just use that regardless of hasTouch.
			return function(node, listener){
				return on(node, msPointerType, listener);
			}
		}else if(hasTouch){
			return function(node, listener){
				var handle1 = on(node, touchType, function(evt){
						listener.call(this, evt);

						// On slow mobile browsers (see https://bugs.dojotoolkit.org/ticket/17634),
						// a handler for a touch event may take >1s to run.  That time shouldn't
						// be included in the calculation for lastTouch.
						lastTouch = (new Date()).getTime();
					}),
					handle2 = on(node, mouseType, function(evt){
						if(!lastTouch || (new Date()).getTime() > lastTouch + 1000){
							listener.call(this, evt);
						}
					});
				return {
					remove: function(){
						handle1.remove();
						handle2.remove();
					}
				};
			};
		}else{
			// Avoid creating listeners for touch events on performance sensitive older browsers like IE6
			return function(node, listener){
				return on(node, mouseType, listener);
			}
		}
	}

	function marked(/*DOMNode*/ node){
		// Test if a node or its ancestor has been marked with the dojoClick property to indicate special processing,
		do{
			if(node.dojoClick !== undefined){ return node.dojoClick; }
		}while(node = node.parentNode);
	}
	
	function doClicks(e, moveType, endType){
		// summary:
		//		Setup touch listeners to generate synthetic clicks immediately (rather than waiting for the browser
		//		to generate clicks after the double-tap delay) and consistently (regardless of whether event.preventDefault()
		//		was called in an event listener. Synthetic clicks are generated only if a node or one of its ancestors has
		//      its dojoClick property set to truthy. If a node receives synthetic clicks because one of its ancestors has its
		//      dojoClick property set to truthy, you can disable synthetic clicks on this node by setting its own dojoClick property
		//      to falsy.
		
		clickTracker  = !e.target.disabled && marked(e.target); // click threshold = true, number or x/y object
		if(clickTracker){
			clickTarget = e.target;
			clickX = e.changedTouches ? e.changedTouches[0].pageX : e.clientX;
			clickY = e.changedTouches ? e.changedTouches[0].pageY : e.clientY;
			clickDx = (typeof clickTracker == "object" ? clickTracker.x : (typeof clickTracker == "number" ? clickTracker : 0)) || 4;
			clickDy = (typeof clickTracker == "object" ? clickTracker.y : (typeof clickTracker == "number" ? clickTracker : 0)) || 4;

			// add move/end handlers only the first time a node with dojoClick is seen,
			// so we don't add too much overhead when dojoClick is never set.
			if(!clicksInited){
				clicksInited = true;

				win.doc.addEventListener(moveType, function(e){
					clickTracker = clickTracker &&
						(e.changedTouches ? e.changedTouches[0].target : e.target) == clickTarget &&
						Math.abs((e.changedTouches ? e.changedTouches[0].pageX : e.clientX) - clickX) <= clickDx &&
						Math.abs((e.changedTouches ? e.changedTouches[0].pageY : e.clientY) - clickY) <= clickDy;
				}, true);

				win.doc.addEventListener(endType, function(e){
					if(clickTracker){
						clickTime = (new Date()).getTime();
						var target = e.target;
						if(target.tagName === "LABEL"){
							// when clicking on a label, forward click to its associated input if any
							target = dom.byId(target.getAttribute("for")) || target;
						}
						//some attributes can be on the Touch object, not on the Event:
						//http://www.w3.org/TR/touch-events/#touch-interface
						var src = (e.changedTouches) ? e.changedTouches[0] : e;
						//create the synthetic event.
						//http://www.w3.org/TR/DOM-Level-3-Events/#widl-MouseEvent-initMouseEvent
						var clickEvt = document.createEvent("MouseEvents");
						clickEvt._dojo_click = true;
						clickEvt.initMouseEvent("click",
							true, //bubbles
							true, //cancelable
							e.view,
							e.detail,
							src.screenX,
							src.screenY,
							src.clientX,
							src.clientY,
							e.ctrlKey,
							e.altKey,
							e.shiftKey,
							e.metaKey,
							0, //button
							null //related target
						);
						setTimeout(function(){
							on.emit(target, "click", clickEvt);
						}, 0);
					}
				}, true);

				function stopNativeEvents(type){
					win.doc.addEventListener(type, function(e){
						// Stop native events when we emitted our own click event.  Note that the native click may occur
						// on a different node than the synthetic click event was generated on.  For example,
						// click on a menu item, causing the menu to disappear, and then (~300ms later) the browser
						// sends a click event to the node that was *underneath* the menu.  So stop all native events
						// sent shortly after ours, similar to what is done in dualEvent.
						// The INPUT.dijitOffScreen test is for offscreen inputs used in dijit/form/Button, on which
						// we call click() explicitly, we don't want to stop this event.
						if(!e._dojo_click &&
								(new Date()).getTime() <= clickTime + 1000 &&
								!(e.target.tagName == "INPUT" && domClass.contains(e.target, "dijitOffScreen"))){
							e.stopPropagation();
							e.stopImmediatePropagation && e.stopImmediatePropagation();
							if(type == "click" && (e.target.tagName != "INPUT" || e.target.type == "radio" || e.target.type == "checkbox")
								&& e.target.tagName != "TEXTAREA" && e.target.tagName != "AUDIO" && e.target.tagName != "VIDEO"){
								 // preventDefault() breaks textual <input>s on android, keyboard doesn't popup,
								 // but it is still needed for checkboxes and radio buttons, otherwise in some cases
								 // the checked state becomes inconsistent with the widget's state
								e.preventDefault();
							}
						}
					}, true);
				}

				stopNativeEvents("click");

				// We also stop mousedown/up since these would be sent well after with our "fast" click (300ms),
				// which can confuse some dijit widgets.
				stopNativeEvents("mousedown");
				stopNativeEvents("mouseup");
			}
		}
	}

	var hoveredNode;

	if(hasTouch){
		if(msPointer){
			 // MSPointer (IE10+) already has support for over and out, so we just need to init click support
			domReady(function(){
				win.doc.addEventListener(pointer.down, function(evt){
					doClicks(evt, pointer.move, pointer.up);
				}, true);
			});		
		}else{
			domReady(function(){
				// Keep track of currently hovered node
				hoveredNode = win.body();	// currently hovered node

				win.doc.addEventListener("touchstart", function(evt){
					lastTouch = (new Date()).getTime();

					// Precede touchstart event with touch.over event.  DnD depends on this.
					// Use addEventListener(cb, true) to run cb before any touchstart handlers on node run,
					// and to ensure this code runs even if the listener on the node does event.stop().
					var oldNode = hoveredNode;
					hoveredNode = evt.target;
					on.emit(oldNode, "dojotouchout", {
						relatedTarget: hoveredNode,
						bubbles: true
					});
					on.emit(hoveredNode, "dojotouchover", {
						relatedTarget: oldNode,
						bubbles: true
					});
				
					doClicks(evt, "touchmove", "touchend"); // init click generation
				}, true);

				function copyEventProps(evt){
					// Make copy of event object and also set bubbles:true.  Used when calling on.emit().
					var props = lang.delegate(evt, {
						bubbles: true
					});

					if(has("ios") >= 6){
						// On iOS6 "touches" became a non-enumerable property, which 
						// is not hit by for...in.  Ditto for the other properties below.
						props.touches = evt.touches;
						props.altKey = evt.altKey;
						props.changedTouches = evt.changedTouches;
						props.ctrlKey = evt.ctrlKey;
						props.metaKey = evt.metaKey;
						props.shiftKey = evt.shiftKey;
						props.targetTouches = evt.targetTouches;
					}

					return props;
				}
				
				on(win.doc, "touchmove", function(evt){
					lastTouch = (new Date()).getTime();

					var newNode = win.doc.elementFromPoint(
						evt.pageX - (ios4 ? 0 : win.global.pageXOffset), // iOS 4 expects page coords
						evt.pageY - (ios4 ? 0 : win.global.pageYOffset)
					);

					if(newNode){
						// Fire synthetic touchover and touchout events on nodes since the browser won't do it natively.
						if(hoveredNode !== newNode){
							// touch out on the old node
							on.emit(hoveredNode, "dojotouchout", {
								relatedTarget: newNode,
								bubbles: true
							});

							// touchover on the new node
							on.emit(newNode, "dojotouchover", {
								relatedTarget: hoveredNode,
								bubbles: true
							});

							hoveredNode = newNode;
						}

						// Unlike a listener on "touchmove", on(node, "dojotouchmove", listener) fires when the finger
						// drags over the specified node, regardless of which node the touch started on.
						if(!on.emit(newNode, "dojotouchmove", copyEventProps(evt))){
							// emit returns false when synthetic event "dojotouchmove" is cancelled, so we prevent the
							// default behavior of the underlying native event "touchmove".
							evt.preventDefault();
						}
					}
				});

				// Fire a dojotouchend event on the node where the finger was before it was removed from the screen.
				// This is different than the native touchend, which fires on the node where the drag started.
				on(win.doc, "touchend", function(evt){
					lastTouch = (new Date()).getTime();
					var node = win.doc.elementFromPoint(
						evt.pageX - (ios4 ? 0 : win.global.pageXOffset), // iOS 4 expects page coords
						evt.pageY - (ios4 ? 0 : win.global.pageYOffset)
					) || win.body(); // if out of the screen

					on.emit(node, "dojotouchend", copyEventProps(evt));
				});
			});
		}
	}

	//device neutral events - touch.press|move|release|cancel/over/out
	var touch = {
		press: dualEvent("mousedown", "touchstart", pointer.down),
		move: dualEvent("mousemove", "dojotouchmove", pointer.move),
		release: dualEvent("mouseup", "dojotouchend", pointer.up),
		cancel: dualEvent(mouse.leave, "touchcancel", hasTouch ? pointer.cancel : null),
		over: dualEvent("mouseover", "dojotouchover", pointer.over),
		out: dualEvent("mouseout", "dojotouchout", pointer.out),
		enter: mouse._eventHandler(dualEvent("mouseover","dojotouchover", pointer.over)),
		leave: mouse._eventHandler(dualEvent("mouseout", "dojotouchout", pointer.out))
	};

	/*=====
	touch = {
		// summary:
		//		This module provides unified touch event handlers by exporting
		//		press, move, release and cancel which can also run well on desktop.
		//		Based on http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html
		//      Also, if the dojoClick property is set to truthy on a DOM node, dojo/touch generates
		//      click events immediately for this node and its descendants (except for descendants that
		//      have a dojoClick property set to falsy), to avoid the delay before native browser click events,
		//      and regardless of whether evt.preventDefault() was called in a touch.press event listener.
		//
		// example:
		//		Used with dojo/on
		//		|	define(["dojo/on", "dojo/touch"], function(on, touch){
		//		|		on(node, touch.press, function(e){});
		//		|		on(node, touch.move, function(e){});
		//		|		on(node, touch.release, function(e){});
		//		|		on(node, touch.cancel, function(e){});
		// example:
		//		Used with touch.* directly
		//		|	touch.press(node, function(e){});
		//		|	touch.move(node, function(e){});
		//		|	touch.release(node, function(e){});
		//		|	touch.cancel(node, function(e){});
		// example:
		//		Have dojo/touch generate clicks without delay, with a default move threshold of 4 pixels
		//		|	node.dojoClick = true;
		// example:
		//		Have dojo/touch generate clicks without delay, with a move threshold of 10 pixels horizontally and vertically
		//		|	node.dojoClick = 10;
		// example:
		//		Have dojo/touch generate clicks without delay, with a move threshold of 50 pixels horizontally and 10 pixels vertically
		//		|	node.dojoClick = {x:50, y:5};
		// example:
		//    Disable clicks without delay generated by dojo/touch on a node that has an ancestor with property dojoClick set to truthy
		//    |  node.dojoClick = false;		

		press: function(node, listener){
			// summary:
			//		Register a listener to 'touchstart'|'mousedown' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		move: function(node, listener){
			// summary:
			//		Register a listener that fires when the mouse cursor or a finger is dragged over the given node.
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		release: function(node, listener){
			// summary:
			//		Register a listener to releasing the mouse button while the cursor is over the given node
			//		(i.e. "mouseup") or for removing the finger from the screen while touching the given node.
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		cancel: function(node, listener){
			// summary:
			//		Register a listener to 'touchcancel'|'mouseleave' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		over: function(node, listener){
			// summary:
			//		Register a listener to 'mouseover' or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		out: function(node, listener){
			// summary:
			//		Register a listener to 'mouseout' or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		enter: function(node, listener){
			// summary:
			//		Register a listener to mouse.enter or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		leave: function(node, listener){
			// summary:
			//		Register a listener to mouse.leave or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		}
	};
	=====*/

	 1  && (dojo.touch = touch);

	return touch;
});

},
'dojo/cache':function(){
define(["./_base/kernel", "./text"], function(dojo){
	// module:
	//		dojo/cache

	// dojo.cache is defined in dojo/text
	return dojo.cache;
});

},
'xcitec/Image':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-style",
	"dojo/on",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "./_base/ImageFactory"
], function (dojo, declare, domAttr, domStyle, on, _Templated, _Widget, ImageFactory) {

	var Image = declare( [_Widget, _Templated], {
	
		templateString: '<img class="${imageClass}" src="${_blankGif}" data-dojo-attach-point="imageNode"/>',
	
		src: "",
		width: "",
		height: "",
		imageClass: "",
		borderAttr: true,
		url: "",
		webSiteScriptUrl: "",
		
	
		postMixInProperties: function(){
			if(this.imageClass){
				this.imageClass = ImageFactory.createIconCss(this.imageClass);
			}
			this.inherited(arguments);
		},
	
		_fillContent: function(){
			var _this = this;
			if(this.border){
				domStyle.set(this.domNode, "border", "solid 1px gray");
			}
			if(this.src){
				domAttr.set(this.domNode, "src", this.src);
			}
			if(this.width){
				domAttr.set(this.domNode, "width", this.width + "px");
			}
			if(this.height){
				domAttr.set(this.domNode, "height", this.height + "px");
			}
			if(this.url){
				domStyle.set(this.imageNode, "cursor", "pointer");
				domAttr.set(this.domNode, "onclick", this.webSiteScriptUrl);
			}
			this.inherited(arguments);
		}
	});
	return Image;
});

},
'xcitec/grid/_HGridHeader':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/dnd/Source",
	"dojo/dom-geometry",
	"dojo/has",
	"dojo/on",
	"dojo/query",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "./_HGridHeaderCell"
], function (dojo, domStyle, declare, lang, windowModule, array, domConstruct, domClass, Manager, domAttr, Source, domGeometry, has, on, query, topic,
             _Container, _Templated, _Widget, registry, _HGridHeaderCell) {
  return 	declare([_Widget, _Templated], {
		templateString: '<div class="dataTableHead">' +
			'<table width="100%" class="dataTableMinWidth"><tr><td>' +
			'<table width="100%" data-dojo-attach-point="tableHead" class="head_content">' +
			'<tbody><tr data-dojo-attach-point="containerNode">' +
			'</tr></tbody>' +
			'</table>' +
			'<img src="${_blankGif}" class="lineMinWidth" height="1" alt="" data-dojo-attach-point="headMinWidth"/>' +
			'</td></tr></table>' +
			'</div>',
		resizingCell: null,
		resizingStarted: false,
		cellsWidth: null, //array of cells' widths
		allCells: null, //array of cells' objects
		parentTable:null,
		minCellWidth: 30,
		newWidth: 0,
		originalWidth: 0,
		tableWidth: 0,
		nowResizing: false,
		minColumnWidth: 30,
		blockHeight: 0,
		blockTableHeight: 0,
		tempOffset: -1,
		cellToRightAlign: null, //array of cells' objects with right align
		showLastSeparator: false,
		// DnD related attribute
		source: null,
		isFrozenHeader: false,

		postMixInProperties: function(){
			this.allCells = [];
			this.cellsWidth = [];
			this.cellToRightAlign = [];

			this.inherited(arguments);
		},

		_fillContent: function(/*DomNode*/ source){
			if(source){
				var _this = this;
				var cells = query("th, td", source);
				var cellsCounter = 0;

				array.forEach(cells, function(cellNode){
					_this.containerNode.appendChild(cellNode);
					var fixedPos = false;
					if(domAttr.get(cellNode, "fixedPos")){
						fixedPos = true;
					}
					var headerCell = new _HGridHeaderCell({cellIndex: cellsCounter, cellContainer: _this, fixedPos: fixedPos}, cellNode);
					domClass.add(headerCell.domNode, "dojoDndItem");
					domStyle.set(headerCell.domNode, {'display': 'table-cell', 'padding': '0px'});  // defaulting to values from dojo 1.6 to make it work
					_this.allCells[cellsCounter] = headerCell;
					cellsCounter++;
				});

				//add empty cell at left and right side (offsets)
				domConstruct.create("td", {"class": "selectedLine"}, this.containerNode, "first");
				//this.tdHeadRight = dojo.create("td", {"class": "selectedLine"}, this.containerNode);
			}
		},

		resizeHeaderCell: function(offset){
			var currCellWidth = this.cellsWidth[this.resizingCell.cellIndex];
			this.newWidth = Math.max(this.originalWidth + offset, this.minCellWidth);
			if(currCellWidth == this.newWidth){
				return;
			}
			var _this = this;

			if(this.isFrozenHeader){
				var currHeadWidth = 0;
				array.forEach(this.cellsWidth, function(width, index){
					if(width){
						if(index == _this.resizingCell.cellIndex){
							currHeadWidth += _this.newWidth;
							return;
						}
						currHeadWidth += width;
					}
				});
				if((this.parentTable.domNode.clientWidth - 102) <= currHeadWidth){
					this.newWidth -= currHeadWidth - (this.parentTable.domNode.clientWidth - 102);
				}
			}

			this.tableWidth = 0;
			domStyle.set(this.resizingCell.domNode, "width", _this.newWidth + "px");
			this.cellsWidth[this.resizingCell.cellIndex] = _this.newWidth;
			if(offset <= 0){
				if(this.resizingCell == this.allCells[this.allCells.length - 2]){
					if(!domClass.contains(this.allCells[this.allCells.length - 1].domNode, "separator")){
						domClass.add(this.allCells[this.allCells.length - 1].domNode, "separator");
					}
				}
			}
			/*dojo.forEach(this.allCells, function(cell) {
			 dojo.style(cell.domNode, "width", _this.cellsWidth[cell.cellIndex] + "px");
			 });*/
			array.forEach(this.cellsWidth, function(value){
				if(value){
					_this.tableWidth = _this.tableWidth + value;
				}
			});
			if(this.isFrozenHeader){
				domStyle.set(this.parentTable.nodeFrozenPartContainer, "width", this.tableWidth + 2 + "px");
			}
			if(this.parentTable.vertScroll && !this.isFrozenHeader){
				domAttr.set(this.tableHead, "width", this.tableWidth + 17 + 2);
			} else{
				domAttr.set(this.tableHead, "width", this.tableWidth + 2);
			}
		},

		resizeColumn: function(){
			var _this = this;
			if(this.nowResizing){
				var rows = this.parentTable.rows;
				var body = this.parentTable.body;
				var bodyMinWidth = this.parentTable.bodyMinWidth;
				if(this.isFrozenHeader){
					rows = this.parentTable.frozenRows;
					body = this.parentTable.frozenBody;
					bodyMinWidth = this.parentTable.frozenBodyMinWidth;
				}
				if(rows){
					var columnWidths = this.parentTable.columnWidths;
					if(this.isFrozenHeader){
						columnWidths = this.parentTable.columnFrozenWidths;
					}
					array.forEach(this.allCells, function(cell, index){
						if(cell == _this.resizingCell){
							rows.resizeRowCell(rows.allRows[0], index, _this.newWidth);
							columnWidths[index] = _this.newWidth;
							cell.cellWidth = _this.newWidth;
							_this.parentTable.updateColumnWidthsInput();
						}
					});
				}
				domAttr.set(body, "width", this.tableWidth + 2);
				domAttr.set(this.headMinWidth, "width", this.tableWidth);
				domAttr.set(bodyMinWidth, "width", this.tableWidth);
				// control separator in the last column
				if(!this.parentTable.vertScroll && !this.isFrozenHeader){
					if((domAttr.get(this.parentTable.body, "width") - domGeometry.getMarginBox(this.parentTable.dataTableBody).w) < 0){
						domClass.add(this.allCells[this.allCells.length - 1].domNode, "separator");
					} else{
						domClass.remove(this.allCells[this.allCells.length - 1].domNode, "separator");
					}
				}
			}
		},

		postCreate: function(){
			var minTableWidth = 0; // min table width (set to <img>)
			var _this = this;
			var colsWithWidthCounter = 0;
			var columnWidths = this.parentTable.columnWidths;
			var bodyMinWidth = this.parentTable.bodyMinWidth;
			var body = this.parentTable.body;
			if(this.isFrozenHeader){
				columnWidths = this.parentTable.columnFrozenWidths;
				bodyMinWidth = this.parentTable.frozenBodyMinWidth;
				body = this.parentTable.frozenBody;
			}
			array.forEach(columnWidths, function(colWidth){
				if(colWidth){
					minTableWidth += Number(colWidth);
					colsWithWidthCounter++;
				} else{
					minTableWidth += Number(_this.minColumnWidth);
				}
			});
			this.columnWidths=columnWidths;
			if(minTableWidth == 0){
				minTableWidth = this.allCells.length * this.minCellWidth;
			}
			minTableWidth += 4;
			this.tableWidth = minTableWidth;
			domAttr.set(this.headMinWidth, "width", minTableWidth);
			domAttr.set(bodyMinWidth, "width", minTableWidth);
			if(colsWithWidthCounter == this.allCells.length){
				domAttr.set(this.tableHead, "width", minTableWidth);
				domAttr.set(body, "width", minTableWidth);
				this.parentTable.showLastSeparator = true;
			}

			this.tdHeadRight = domConstruct.create("td", {"class": "selectedLine"}, this.containerNode);
			var headerCell = new _HGridHeaderCell({cellIndex: this.allCells.length, cellContainer: this}, this.tdHeadRight);
			if(!this.parentTable.showLastSeparator){
				domClass.remove(headerCell.domNode, "separator");
			}
			this.allCells[this.allCells.length] = headerCell;
			this.cellsWidth[headerCell.cellIndex] = domGeometry.getMarginBox(headerCell.domNode).w;

			if(this.parentTable.vertScroll && !this.isFrozenHeader){
				domConstruct.create("td", {width: "17px"}, this.containerNode, "last");
				domAttr.set(this.headMinWidth, "width", minTableWidth + 17);
			}
			if(this.parentTable.vertScroll || this.isFrozenHeader){
				domClass.add(headerCell.domNode, "separator");
			}
			if(this.parentTable.columnReorder){
				this.prepareDragging();
			}
		},

		prepareDragging: function(){
			var node = this.containerNode;
			var _this = this;
			this.source = new Source(node, {horizontal: true, accept: [this.id], singular: true});

			// trick to disable copying of draggable items
			this.source.copyState = function(){
				return false;
			};

			topic.subscribe("/dnd/start", function(source, nodes, copy){
				if(domAttr.get(nodes[0], "fixedPos")){
					topic.publish('/dnd/cancel');

					// replacement for dojo.dnd.manager().stopDrag()
					var manager = Manager;
					domClass.remove(windowModule.body(), ["dojoDndCopy", "dojoDndMove"]);
					array.forEach(manager.events, dojo.disconnect);
					manager.events = [];
					if(manager.avatar){
						manager.avatar.destroy();
					}
					manager.avatar = null;
					manager.source = manager.target = null;
					manager.nodes = [];
				}
			});

			on(this.source, "MouseUp", function(e){
				var firstFixedPos = null,
					lastFixedPos = null,
					endFixedPosX = 0;
				array.forEach(_this.allCells, function(cell){
					if(cell.fixedPos){
						if(!firstFixedPos){
							firstFixedPos = cell;
						} else{
							lastFixedPos = cell;
						}
					}
				});
				if(firstFixedPos){
					endFixedPosX = dojo.coords(firstFixedPos.domNode).x + dojo.coords(firstFixedPos.domNode).w;
				}
				if(lastFixedPos){
					endFixedPosX = dojo.coords(lastFixedPos.domNode).x + dojo.coords(lastFixedPos.domNode).w;
				}
				if(e.clientX < endFixedPosX && _this.source.isDragging){
					topic.publish('/dnd/cancel');
				Manager.stopDrag();
				}
			});
			on(this.source, "Drop", function(){
				var ar = [];
				var index = 0;
				array.forEach(_this.source.getAllNodes(), function(node){
					ar[index] = registry.byNode(node);
					index++;
				});
				index = 0;
				array.forEach(ar, function(node){
					if(index == 0){
						domClass.remove(node.domNode, "separator");
					} else if(!domClass.contains(node.domNode, "separator")){
						domClass.add(node.domNode, "separator");
					}
					index++;
				});
				var oldInd = array.indexOf(_this.allCells, registry.byNode(_this.source.getSelectedNodes()[0]));
				var temp = _this.allCells[_this.allCells.length - 1];
				_this.allCells = ar;
				_this.allCells = _this.allCells.concat(temp);
				var newInd = array.indexOf(ar, registry.byNode(_this.source.getSelectedNodes()[0]));
				if(!_this.parentTable.emptyGridMessage){
					var rows = _this.parentTable.rows;
					if(_this.isFrozenHeader){
						rows = _this.parentTable.frozenRows;
					}
					array.forEach(rows.allRows, function(row){
						var im = query("div.hierarchicalImages", row.allCells[0]);
						if(newInd == 0){
							domConstruct.place(row.allCells[oldInd], row.allCells[newInd], "before");
							if(im[0]){
								domConstruct.place(im[0], query(" >", row.allCells[oldInd])[0], "first");
							}
						} else if(newInd == (ar.length - 1)){
							if(oldInd == 0){
								if(im[0]){
									domConstruct.place(im[0], query(" >", row.allCells[oldInd + 1])[0], "first");
								}
							}
							domConstruct.place(row.allCells[oldInd], row.allCells[newInd], "after");
						} else{
							if((newInd - oldInd) > 0){
								if(im[0] && (oldInd == 0)){
									domConstruct.place(im[0], query(" >", row.allCells[oldInd + 1])[0], "first");
								}
								domConstruct.place(row.allCells[oldInd], row.allCells[newInd], "after");
							} else{
								domConstruct.place(row.allCells[oldInd], row.allCells[newInd], "before");
							}
						}
						row.allCells = query(" >", row.domNode);
						row.allCells = row.allCells.slice(1, row.allCells.length - 1);
						_this.allCells[0].getCellsWidths();
					});
				}
				var cellInd = 0;
				array.forEach(_this.allCells, function(cell){
					cell.cellIndex = cellInd;
					cellInd++;
				});
				_this.parentTable.updateOrder();
			});
		}
	});
});

},
'xcitec/_base/Scroll':function(){
define([
	"dojo",
	"dojo/_base/lang",
  "dojo/_base/fx",
    "dojo/_base/array",
  "dojo/dom-geometry",
    "dojo/dom-attr",
	"dojo/has",
    "dijit/registry",
  "./Line",
    "xcitec/Subsegment"
], function (dojo, lang, baseFx, array, geometry, domAttr, has, registry, Line, Subsegment) {
  return {
    smoothScroll : function(/* Object */args){
      // summary:
      //      Returns an animation that will smooth-scroll to a node (specified in etup())
      // description:
      //      This implementation support either horizental or vertical scroll, as well as
      //		both. In addition, element in iframe can be scrolled to correctly.
      // offset: {x: int, y: int} this will be added to the target position
      // duration: Duration of the animation in milliseconds.
      // win: a node or window object to scroll

      if(!args.target){
        if(args.node == null){
          var parent = this._getParsedParent(args.component);
          if(parent instanceof Subsegment){
              parent.enclosingWidget.parseChildren(parent).then(function(){
                  args.node = registry.byId(args.component.id).domNode;
              });
          }
        }

        args.target = dojo.coords(args.node, true);
      }

      var isWindow = dojo[(has("ie") ? "isObject" : "isFunction")](args["win"].scrollTo);

      var _anim = (isWindow) ?
          (function(val){
            args.win.scrollTo(val[0], val[1]);
          }) :
          (function(val){
            args.win.scrollLeft = val[0];
            args.win.scrollTop = val[1];
          });

      var anim = new baseFx.Animation(lang.mixin({
        beforeBegin: function(){
          if(this.curve){
            delete this.curve;
          }
          var current = isWindow ? geometry.docScroll() : {x: args.win.scrollLeft, y: args.win.scrollTop};
          anim.curve = new Line([current.x,current.y], [args.target.x,args.target.y]);
        },
        onAnimate: _anim
      }, args));
      return anim; // dojo.Animation
    },

    _getParsedParent : function (component){
        var parent = component.parentElement;
        var widget = registry.byId(parent.id);
        return widget ? widget : this._getParsedParent(parent);
    }
  };
});
},
'dijit/DialogUnderlay':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"dojo/aspect", // aspect.after
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/on",
	"dojo/window", // winUtils.getBox, winUtils.get
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe",
	"./Viewport",
	"./main" // for back-compat, exporting dijit._underlay (remove in 2.0)
], function(declare, lang, aspect, domAttr, domStyle, on,
			winUtils, _Widget, _TemplatedMixin, BackgroundIframe, Viewport, dijit){

	// module:
	//		dijit/DialogUnderlay

	var DialogUnderlay = declare("dijit.DialogUnderlay", [_Widget, _TemplatedMixin], {
		// summary:
		//		A component used to block input behind a `dijit/Dialog`.
		//
		//		Normally this class should not be instantiated directly, but rather shown and hidden via
		//		DialogUnderlay.show() and DialogUnderlay.hide().  And usually the module is not accessed directly
		//		at all, since the underlay is shown and hidden by Dialog.DialogLevelManager.
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' tabIndex='-1' data-dojo-attach-point='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		// This will get overwritten as soon as show() is call, but leave an empty array in case hide() or destroy()
		// is called first.   The array is shared between instances but that's OK because we never write into it.
		_modalConnects: [],

		_setDialogIdAttr: function(id){
			domAttr.set(this.node, "id", id + "_underlay");
			this._set("dialogId", id);
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
			this._set("class", clazz);
		},

		postCreate: function(){
			// Append the underlay to the body
			this.ownerDocumentBody.appendChild(this.domNode);

			this.own(on(this.domNode, "keydown", lang.hitch(this, "_onKeyDown")));

			this.inherited(arguments);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = winUtils.getBox(this.ownerDocument);
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.open = true;
			this.layout();
			this.bgIframe = new BackgroundIframe(this.domNode);

			var win = winUtils.get(this.ownerDocument);
			this._modalConnects = [
				Viewport.on("resize", lang.hitch(this, "layout")),
				on(win, "scroll", lang.hitch(this, "layout"))
			];

		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay

			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
			while(this._modalConnects.length){ (this._modalConnects.pop()).remove(); }
			this.open = false;
		},

		destroy: function(){
			while(this._modalConnects.length){ (this._modalConnects.pop()).remove(); }
			this.inherited(arguments);
		},

		_onKeyDown: function(){
			// summary:
			//		Extension point so Dialog can monitor keyboard events on the underlay.
		}
	});

	DialogUnderlay.show = function(/*Object*/ attrs, /*Number*/ zIndex){
		// summary:
		//		Display the underlay with the given attributes set.  If the underlay is already displayed,
		//		then adjust it's attributes as specified.
		// attrs:
		//		The parameters to create DialogUnderlay with.
		// zIndex:
		//		zIndex of the underlay

		var underlay = DialogUnderlay._singleton;
		if(!underlay || underlay._destroyed){
			underlay = dijit._underlay = DialogUnderlay._singleton = new DialogUnderlay(attrs);
		}else{
			if(attrs){ underlay.set(attrs); }
		}
		domStyle.set(underlay.domNode, 'zIndex', zIndex);
		if(!underlay.open){
			underlay.show();
		}
	};

	DialogUnderlay.hide = function(){
		// summary:
		//		Hide the underlay.

		// Guard code in case the underlay widget has already been destroyed
		// because we are being called during page unload (when all widgets are destroyed)
		var underlay = DialogUnderlay._singleton;
		if(underlay && !underlay._destroyed){
			underlay.hide();
		}
	};

	return DialogUnderlay;
});

},
'slmui/layout/Chapter':function(){
define([
	'dojo/_base/declare',
	'./_ChapterBase',
	'./_ChapterItem',
	'dojo/text!./templates/Chapter.html'
], function(declare, _ChapterBase, _ChapterItem, template){
	return declare([_ChapterBase], {

		// summary:
		//		Chapter layout.

		templateString: template,

		buildControlHolder: function(controlNode){
			// summary:
			//		Deprecated. Build control's wrapper and label.
			// tags:
			//		deprecated protected extension
			return new _ChapterItem({controlNode: controlNode});
		}
	});
});
},
'dijit/form/_ToggleButtonMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr" // domAttr.set
], function(declare, domAttr){

	// module:
	//		dijit/form/_ToggleButtonMixin

	return declare("dijit.form._ToggleButtonMixin", null, {
		// summary:
		//		A mixin to provide functionality to allow a button that can be in two states (checked or not).

		// checked: Boolean
		//		Corresponds to the native HTML `<input>` element's attribute.
		//		In markup, specified as "checked='checked'" or just "checked".
		//		True if the button is depressed, or the checkbox is checked,
		//		or the radio button is selected, etc.
		checked: false,

		// aria-pressed for toggle buttons, and aria-checked for checkboxes
		_aria_attr: "aria-pressed",

		_onClick: function(/*Event*/ evt){
			var original = this.checked;
			this._set('checked', !original); // partially set the toggled value, assuming the toggle will work, so it can be overridden in the onclick handler
			var ret = this.inherited(arguments); // the user could reset the value here
			this.set('checked', ret ? this.checked : original); // officially set the toggled or user value, or reset it back
			return ret;
		},

		_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
			this._set("checked", value);
			var node = this.focusNode || this.domNode;
			if(this._created){ // IE is not ready to handle checked attribute (affects tab order)
				// needlessly setting "checked" upsets IE's tab order
				if(domAttr.get(node, "checked") != !!value){
					domAttr.set(node, "checked", !!value); // "mixed" -> true
				}
			}
			node.setAttribute(this._aria_attr, String(value)); // aria values should be strings
			this._handleOnChange(value, priorityChange);
		},

		postCreate: function(){ // use postCreate instead of startup so users forgetting to call startup are OK
			this.inherited(arguments);
			var node = this.focusNode || this.domNode;
			if(this.checked){
				// need this here instead of on the template so IE8 tab order works
				node.setAttribute('checked', 'checked');
			}

			// Update our reset value if it hasn't yet been set (because this.set()
			// is only called when there *is* a value)
			if(this._resetValue === undefined){
				this._lastValueReported = this._resetValue = this.checked;
			}
		},

		reset: function(){
			// summary:
			//		Reset the widget's value to what it was at initialization time

			this._hasBeenBlurred = false;

			// set checked state to original setting
			this.set('checked', this.params.checked || false);
		}
	});
});

},
'xcitec/_base/Actions':function(){
define([
	"dojo/_base/declare",
	"dijit/_Container",
	"dijit/_WidgetBase"
], function (declare, _Container, _Widget) {
	return declare([_Widget, _Container], {
	});
});

},
'xcitec/form/UrlTextBox':function(){
define([
         "dojo/_base/declare",
         "./TextBoxButton"
       ], function (declare, TextBoxButton) {
  return declare(TextBoxButton, {
    actionIconType: "Link",
	webSiteScriptUrl: "",

    _isValueValid: function(/*String */ value) {
      return value ? true : false;
    },

    postCreate: function() {
      // if value is not valid action button is not allowed
      // set right value for buttonEnabled
      this.buttonEnabled = this._isValueValid(this.get('value'));
      this.inherited(arguments);
    },

    _onInput: function() {
      this.inherited(arguments);
      if (this._isValueValid(this.get('value'))) {
        this.button.setReadOnlyStyle(false);
        this.buttonEnabled = true;
      } else {
        this.button.setReadOnlyStyle(true);
        this.buttonEnabled = false;
      }
    },

    _doExecuteAction: function(/*String*/ value) {
      if (!this.buttonEnabled) return;
	  eval(this.webSiteScriptUrl);
    }
  });
});

},
'dojo/store/util/SimpleQueryEngine':function(){
define(["../../_base/array" /*=====, "../api/Store" =====*/], function(arrayUtil /*=====, Store =====*/){

// module:
//		dojo/store/util/SimpleQueryEngine

return function(query, options){
	// summary:
	//		Simple query engine that matches using filter functions, named filter
	//		functions or objects by name-value on a query object hash
	//
	// description:
	//		The SimpleQueryEngine provides a way of getting a QueryResults through
	//		the use of a simple object hash as a filter.  The hash will be used to
	//		match properties on data objects with the corresponding value given. In
	//		other words, only exact matches will be returned.
	//
	//		This function can be used as a template for more complex query engines;
	//		for example, an engine can be created that accepts an object hash that
	//		contains filtering functions, or a string that gets evaluated, etc.
	//
	//		When creating a new dojo.store, simply set the store's queryEngine
	//		field as a reference to this function.
	//
	// query: Object
	//		An object hash with fields that may match fields of items in the store.
	//		Values in the hash will be compared by normal == operator, but regular expressions
	//		or any object that provides a test() method are also supported and can be
	//		used to match strings by more complex expressions
	//		(and then the regex's or object's test() method will be used to match values).
	//
	// options: dojo/store/api/Store.QueryOptions?
	//		An object that contains optional information such as sort, start, and count.
	//
	// returns: Function
	//		A function that caches the passed query under the field "matches".  See any
	//		of the "query" methods on dojo.stores.
	//
	// example:
	//		Define a store with a reference to this engine, and set up a query method.
	//
	//	|	var myStore = function(options){
	//	|		//	...more properties here
	//	|		this.queryEngine = SimpleQueryEngine;
	//	|		//	define our query method
	//	|		this.query = function(query, options){
	//	|			return QueryResults(this.queryEngine(query, options)(this.data));
	//	|		};
	//	|	};

	// create our matching query function
	switch(typeof query){
		default:
			throw new Error("Can not query with a " + typeof query);
		case "object": case "undefined":
			var queryObject = query;
			query = function(object){
				for(var key in queryObject){
					var required = queryObject[key];
					if(required && required.test){
						// an object can provide a test method, which makes it work with regex
						if(!required.test(object[key], object)){
							return false;
						}
					}else if(required != object[key]){
						return false;
					}
				}
				return true;
			};
			break;
		case "string":
			// named query
			if(!this[query]){
				throw new Error("No filter function " + query + " was found in store");
			}
			query = this[query];
			// fall through
		case "function":
			// fall through
	}
	function execute(array){
		// execute the whole query, first we filter
		var results = arrayUtil.filter(array, query);
		// next we sort
		var sortSet = options && options.sort;
		if(sortSet){
			results.sort(typeof sortSet == "function" ? sortSet : function(a, b){
				for(var sort, i=0; sort = sortSet[i]; i++){
					var aValue = a[sort.attribute];
					var bValue = b[sort.attribute];
					// valueOf enables proper comparison of dates
					aValue = aValue != null ? aValue.valueOf() : aValue;
					bValue = bValue != null ? bValue.valueOf() : bValue;
					if (aValue != bValue){
						return !!sort.descending == (aValue == null || aValue > bValue) ? -1 : 1;
					}
				}
				return 0;
			});
		}
		// now we paginate
		if(options && (options.start || options.count)){
			var total = results.length;
			results = results.slice(options.start || 0, (options.start || 0) + (options.count || Infinity));
			results.total = total;
		}
		return results;
	}
	execute.matches = query;
	return execute;
};

});

},
'dijit/_TemplatedMixin':function(){
define([
	"dojo/cache",	// dojo.cache
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.destroy, domConstruct.toDom
	"dojo/_base/lang", // lang.getObject
	"dojo/on",
	"dojo/sniff", // has("ie")
	"dojo/string", // string.substitute string.trim
	"./_AttachMixin"
], function(cache, declare, domConstruct, lang, on, has, string, _AttachMixin){

	// module:
	//		dijit/_TemplatedMixin

	var _TemplatedMixin = declare("dijit._TemplatedMixin", _AttachMixin, {
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template.
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with require([... "dojo/text!..."], ...) instead
		templatePath: null,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template nodes poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

/*=====
		// _rendered: Boolean
		//		Not normally use, but this flag can be set by the app if the server has already rendered the template,
		//		i.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to
		//		just function like _AttachMixin.
		_rendered: false,
=====*/

		// Set _AttachMixin.searchContainerNode to true for back-compat for widgets that have data-dojo-attach-point's
		// and events inside this.containerNode.   Remove for 2.0.
		searchContainerNode: true,

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = lang.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			if(!this._rendered){
				if(!this.templateString){
					this.templateString = cache(this.templatePath, {sanitize: true});
				}

				// Lookup cached version of template, and download to cache if it
				// isn't there already.  Returns either a DomNode or a string, depending on
				// whether or not the template contains ${foo} replacement parameters.
				var cached = _TemplatedMixin.getCachedTemplate(this.templateString, this._skipNodeCache, this.ownerDocument);

				var node;
				if(lang.isString(cached)){
					node = domConstruct.toDom(this._stringRepl(cached), this.ownerDocument);
					if(node.nodeType != 1){
						// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
						throw new Error("Invalid template: " + cached);
					}
				}else{
					// if it's a node, all we have to do is clone it
					node = cached.cloneNode(true);
				}

				this.domNode = node;
			}

			// Call down to _WidgetBase.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to _setBaseClassAttr
			this.inherited(arguments);

			if(!this._rendered){
				this._fillContent(this.srcNodeRef);
			}

			this._rendered = true;
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		}

	});

	// key is templateString; object is either string or DOM tree
	_TemplatedMixin._templateCache = {};

	_TemplatedMixin.getCachedTemplate = function(templateString, alwaysUseString, doc){
		// summary:
		//		Static method to get a template based on the templatePath or
		//		templateString key
		// templateString: String
		//		The template
		// alwaysUseString: Boolean
		//		Don't cache the DOM tree for this template, even if it doesn't have any variables
		// doc: Document?
		//		The target document.   Defaults to document global if unspecified.
		// returns: Mixed
		//		Either string (if there are ${} variables that need to be replaced) or just
		//		a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = _TemplatedMixin._templateCache;
		var key = templateString;
		var cached = tmplts[key];
		if(cached){
			try{
				// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the
				// current document, then use the current cached value
				if(!cached.ownerDocument || cached.ownerDocument == (doc || document)){
					// string or node of the same document
					return cached;
				}
			}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
			domConstruct.destroy(cached);
		}

		templateString = string.trim(templateString);

		if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			var node = domConstruct.toDom(templateString, doc);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + templateString);
			}
			return (tmplts[key] = node); //Node
		}
	};

	if(has("ie")){
		on(window, "unload", function(){
			var cache = _TemplatedMixin._templateCache;
			for(var key in cache){
				var value = cache[key];
				if(typeof value == "object"){ // value is either a string or a DOM node template
					domConstruct.destroy(value);
				}
				delete cache[key];
			}
		});
	}

	return _TemplatedMixin;
});

},
'xcitec/layout/ContentArea':function(){
/*
  slmui.layout.ContentArea is a slmui Layout widget for Content Area
  Ecpected body contect widgets of slmui.layout.Chapter.
  NOTE: all other content items can provide unexpected behaviour

  Example slmui.layout.ContentArea:
  <div data-dojo-type="xcitec/layout/ContentArea" >
    <div data-dojo-type="xcitec/layout/ContentAreaPane">
      <div data-dojo-type="xcitec/layout/Chapter" >...</div>
      <div data-dojo-type="xcitec/layout/Chapter" >...</div>
      ...
    </div>
    <div data-dojo-type="xcitec/layout/ContentAreaPane">
      <div data-dojo-type="xcitec/layout/Chapter" >...</div>
      <div data-dojo-type="xcitec/layout/Chapter" >...</div>
      ...
    </div>
  </div>

  Example slmui.layout.ContentAreaType2:
  <div data-dojo-type="xcitec/layout/ContentAreaType2" >
    <div data-dojo-type="xcitec/layout/Chapter" contentAreaPane="1">...</div>
    <div data-dojo-type="xcitec/layout/Chapter" contentAreaPane="1">...</div>
    ...
    <div data-dojo-type="xcitec/layout/Chapter" contentAreaPane="2">...</div>
    ...
  </div>
 */
define(["dojo/_base/declare",
        "dojo/_base/array",
        "dojo/dom-style",
        "./_LayoutWidget",
        "./_ContentAreaColumn"],
       function(declare, array, domStyle, _LayoutWidget, _ContentAreaColumn){

  return declare(_LayoutWidget, {

    baseClass: "ContentArea",

    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
      var controlContainer = new _ContentAreaColumn({controlNode:controlNode, labelNode:labelNode});
      return controlContainer.domNode;
    },

    startup: function(){
      if(this._started){ return; }
      this.inherited(arguments);
    },

    resize: function () {
      //method added during migration to Dojo 1.9
      // description:
      //  Method is called when the widget is getting visible
      //  to notify widget that its size has been changed
      var children = this.getChildren();
      if (children.length>1) {
        var xWidth = 100/children.length;
        var leftColumn = null;
        var rightColumn = null;
        array.forEach( children, function(child){
          domStyle.set(child.domNode, "width", ""+ xWidth+"%");
          if (!leftColumn) leftColumn=child;
          rightColumn = child;
          child.setColumnType("Middle");
        });
        leftColumn.setColumnType("Left");
        rightColumn.setColumnType("Right");
      }
    }
  });
});

},
'xcitec/form/_FormWidget':function(){
define([
	"dojo/_base/declare"
], function (declare) {

	var _FormWidget = declare( null, {
	   fitToContainer: false,

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
	});
	
	return _FormWidget;
});
},
'xcitec/TrailingLabel':function(){
define(["dojo/_base/declare", "./Label"], function(declare, Label){
  return declare(Label, {

    baseClass: "xcitecTrailingLabel",
    maxLabelWidth: 100
  });
})

},
'xcitec/page/MenuV2':function(){
define([
	"dojo/_base/declare",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (declare, _Templated, _Widget) {

	var MenuV2 = declare( [_Widget, _Templated], {
	  templateString: "<ul data-dojo-attach-point='containerNode' role='menu'></ul>",
	  level: -1
	});
	return MenuV2;
});

},
'xcitec/xlayout/QualificationLayout':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dnd/Source",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"xcitec/layout/ContentArea",
	"xcitec/xlayout/ContentAreaPaneV2",
	"xcitec/xlayout/QualificationLayoutBase"
], function (dojo, array, declare, Source, domAttr, domClass, domConstruct, query, ContentArea, ContentAreaPaneV2, QualificationLayoutBase) {

	/* xcitec.xlayout.QualificationLayout is a layout control which is used for placing
	   questions in qualification in appropritae way. Can be in design and view mode. Allow
	   drag&drop between columns and in column. Has a hidden input field to store the order
	   of elements in a columns
	    Example:
	        <div data-dojo-type="xcitec.xlayout.QualificationLayout" designMode="true" id="ql1">
	            <input id="ql1.order" name="ql1.order" type="hidden" value="">
	            <div data-dojo-type="xcitec.xlayout.StyleElement" elementType='headline' caption='Personal data'></div>
	            <div data-dojo-type="xcitec.xlayout.QuestionV2" labelId="supplier0.supplierid">
	                <select label="Salutation" data-dojo-type="xcitec.form.DropDownSelect" widthInPercent="50">
	                    <option selected="true" value="a">Mr.</option>
	                    <option selected="false" value="b">Ms.</option>
	                </select>
	            </div>
	            <div data-dojo-type="xcitec.xlayout.QuestionV2" labelId="supplier0.supplierid">
	                <input type="text" data-dojo-type="xcitec.form.TextBox"
	                       name="supplier0.supplierid.suppliercode0"
	                       label="First name"
	                       labelId="supplier0.supplierid.suppliercode0"
	                       value="">
	            </div>
	        </div>
	*/
	
	var QualificationLayout = declare( QualificationLayoutBase, {
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        this.firstColumnControls = [];
	        this.secondColumnControls = [];
	    },
	
	    _fillContent: function() {
	
	        this._startupWidgets = this._startupWidgets || [];
	
	        var children = query(">", this.srcNodeRef);
	        var _this = this;
	
	        // create helper nodes
	        this.helperNodeLeft = domConstruct.create("div");
	        domClass.add(this.helperNodeLeft, "dojoDndItem");
	        this.helperNodeRight = domConstruct.create("div");
	        domClass.add(this.helperNodeRight, "dojoDndItem");
	      
	        // collecting nodes
	        var currentColumnArray = [];
	        array.forEach(children, function(child) {
	            if (domClass.contains(child, "delimeter")) {
	                _this.delimeterId = domAttr.get(child, "id");
	                _this.firstColumnControls = currentColumnArray;
	                currentColumnArray = [];
	                _this.hasDelimeter = true;
	                domConstruct.destroy(child);
	            } else {
	                currentColumnArray.push(child);
	            }
	        });
	        if (this.hasDelimeter) {
	            this.secondColumnControls = currentColumnArray;
	        } else {
	            this.firstColumnControls = currentColumnArray;
	        }
	
	        // add helper nodes to anable DnD when no items in one column
	        this.firstColumnControls.push(this.helperNodeLeft);
	        this.secondColumnControls.push(this.helperNodeRight);
	
	        // create markup for each column
	        var contentAreaPane1 = domConstruct.create("div", {'data-dojo-type': 'xcitec/xlayout/ContentAreaPaneV2', designMode: this.designMode});
	        array.forEach(this.firstColumnControls, function(node) {
	            contentAreaPane1.appendChild(node);
	        });
	        var contentAreaPane2 = domConstruct.create("div", {'data-dojo-type': 'xcitec/xlayout/ContentAreaPaneV2', designMode: this.designMode});
	        array.forEach(this.secondColumnControls, function(node) {
	            contentAreaPane2.appendChild(node);
	        });
	
	        // create container for columns
	        var container = domConstruct.create("div");
	        container.appendChild(contentAreaPane1);
	        container.appendChild(contentAreaPane2);
	        var contentArea = new ContentArea({}, container);
	
	        // actually crerate column's objects
	        this.pane1 = new ContentAreaPaneV2({columnOrder: this.firstColumnOrder, parent: this, idHolder: this.clicked }, contentAreaPane1);
	        this.pane2 = new ContentAreaPaneV2({columnOrder: this.secondColumnOrder, parent: this, idHolder: this.clicked }, contentAreaPane2);
	        this._startupWidgets.push(contentArea);
	        this._startupWidgets.push(this.pane1);
	        this._startupWidgets.push(this.pane2);
	
	        this.containerNode.appendChild(contentArea.domNode);
	        this.inherited(arguments);
	    },
	
	    // update order of elements in each column
	    updateColumnsOrder: function() {
	        this.pane1.updateOrder();
	        this.pane2.updateOrder();
	        this.updateHiddenField(this.pane1.getColumnOrder(), this.pane2.getColumnOrder());
	    },
	
	    // update value of hidden input
	    updateHiddenField: function(firstColumn, secondColumn) {
	        var _this = this;
		    if(this.order){
			    this.order.value = '';
			    array.forEach(firstColumn, function(id) {
				    _this.order.value = _this.order.value + id + ",";
			    });
			    if (secondColumn.length != 0) {
				    if (this.delimeterId) {
					    this.order.value += this.delimeterId + ",";
				    } else {
					    this.order.value += 'separator,';
				    }
				    array.forEach(secondColumn, function(id) {
					    _this.order.value = _this.order.value + id + ",";
				    });
			    }
			    this.order.value = this.order.value.substring(0, this.order.value.length - 1);
		    }
	    },
	
	    startup: function() {
	        this.inherited(arguments);
	        if (this.designMode) {
		        if(this.order){
			        domConstruct.place(this.order, this.domNode, 'first');
		        }
	            if (this.clicked) {
	                domConstruct.place(this.clicked, this.domNode, 'first');
	            }
	            // makes the panes accept only appropriate elements
	            this.pane1.setSourceForDnd([this.pane1.id, this.pane2.id]);
	            this.pane2.setSourceForDnd([this.pane1.id, this.pane2.id]);
	            this.updateColumnsOrder();
	            this.resizeHelperNodes();
	        }
	    }
	});
	return QualificationLayout;
});

},
'dijit/form/HorizontalSlider':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dnd/move",
	"dojo/_base/fx", // fx.animateProperty
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.DOWN_ARROW keys.END keys.HOME keys.LEFT_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.RIGHT_ARROW keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("ie") has("mozilla")
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/Mover", // Mover Mover.prototype.destroy.apply
	"dojo/query", // query
	"dojo/mouse", // mouse.wheel
	"dojo/on",
	"../_base/manager", // defaultDuration
	"../focus", // focus.focus()
	"../typematic",
	"./Button",
	"./_FormValueWidget",
	"../_Container",
	"dojo/text!./templates/HorizontalSlider.html"
], function(array, declare, move, fx, domGeometry, domStyle, keys, lang, has, Moveable, Mover, query, mouse, on,
			manager, focus, typematic, Button, _FormValueWidget, _Container, template){

	// module:
	//		dijit/form/HorizontalSlider

	var _SliderMover = declare("dijit.form._SliderMover", Mover, {
		onMouseMove: function(e){
			var widget = this.widget;
			var abspos = widget._abspos;
			if(!abspos){
				abspos = widget._abspos = domGeometry.position(widget.sliderBarContainer, true);
				widget._setPixelValue_ = lang.hitch(widget, "_setPixelValue");
				widget._isReversed_ = widget._isReversed();
			}
			var pixelValue = e[widget._mousePixelCoord] - abspos[widget._startingPixelCoord];
			widget._setPixelValue_(widget._isReversed_ ? (abspos[widget._pixelCount] - pixelValue) : pixelValue, abspos[widget._pixelCount], false);
		},

		destroy: function(e){
			Mover.prototype.destroy.apply(this, arguments);
			var widget = this.widget;
			widget._abspos = null;
			widget._setValueAttr(widget.value, true);
		}
	});

	var HorizontalSlider = declare("dijit.form.HorizontalSlider", [_FormValueWidget, _Container], {
		// summary:
		//		A form widget that allows one to select a value with a horizontally draggable handle

		templateString: template,

		// Overrides FormValueWidget.value to indicate numeric value
		value: 0,

		// showButtons: [const] Boolean
		//		Show increment/decrement buttons at the ends of the slider?
		showButtons: true,

		// minimum: [const] Integer
		//		The minimum value the slider can be set to.
		minimum: 0,

		// maximum: [const] Integer
		//		The maximum value the slider can be set to.
		maximum: 100,

		// discreteValues: Integer
		//		If specified, indicates that the slider handle has only 'discreteValues' possible positions,
		//		and that after dragging the handle, it will snap to the nearest possible position.
		//		Thus, the slider has only 'discreteValues' possible values.
		//
		//		For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has
		//		three possible positions, representing values 10, 20, or 30.
		//
		//		If discreteValues is not specified or if it's value is higher than the number of pixels
		//		in the slider bar, then the slider handle can be moved freely, and the slider's value will be
		//		computed/reported based on pixel position (in this case it will likely be fractional,
		//		such as 123.456789).
		discreteValues: Infinity,

		// pageIncrement: Integer
		//		If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)
		//		that the slider handle is moved via pageup/pagedown keys.
		//		If discreteValues is not specified, it indicates the number of pixels.
		pageIncrement: 2,

		// clickSelect: Boolean
		//		If clicking the slider bar changes the value or not
		clickSelect: true,

		// slideDuration: Number
		//		The time in ms to take to animate the slider handle from 0% to 100%,
		//		when clicking the slider bar to make the handle move.
		slideDuration: manager.defaultDuration,

		// Map widget attributes to DOMNode attributes.
		_setIdAttr: "", // Override _FormWidget which sends id to focusNode
		_setNameAttr: "valueNode", // Override default behavior to send to focusNode

		baseClass: "dijitSlider",

		// Apply CSS classes to up/down arrows and handle per mouse state
		cssStateNodes: {
			incrementButton: "dijitSliderIncrementButton",
			decrementButton: "dijitSliderDecrementButton",
			focusNode: "dijitSliderThumb"
		},

		_mousePixelCoord: "pageX",
		_pixelCount: "w",
		_startingPixelCoord: "x",
		_handleOffsetCoord: "left",
		_progressPixelSize: "width",

		_onKeyUp: function(/*Event*/ e){
			if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){
				return;
			}
			this._setValueAttr(this.value, true);
		},

		_onKeyDown: function(/*Event*/ e){
			if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){
				return;
			}
			switch(e.keyCode){
				case keys.HOME:
					this._setValueAttr(this.minimum, false);
					break;
				case keys.END:
					this._setValueAttr(this.maximum, false);
					break;
				// this._descending === false: if ascending vertical (min on top)
				// (this._descending || this.isLeftToRight()): if left-to-right horizontal or descending vertical
				case ((this._descending || this.isLeftToRight()) ? keys.RIGHT_ARROW : keys.LEFT_ARROW):
				case (this._descending === false ? keys.DOWN_ARROW : keys.UP_ARROW):
				case (this._descending === false ? keys.PAGE_DOWN : keys.PAGE_UP):
					this.increment(e);
					break;
				case ((this._descending || this.isLeftToRight()) ? keys.LEFT_ARROW : keys.RIGHT_ARROW):
				case (this._descending === false ? keys.UP_ARROW : keys.DOWN_ARROW):
				case (this._descending === false ? keys.PAGE_UP : keys.PAGE_DOWN):
					this.decrement(e);
					break;
				default:
					return;
			}
			e.stopPropagation();
			e.preventDefault();
		},

		_onHandleClick: function(e){
			if(this.disabled || this.readOnly){
				return;
			}
			if(!has("ie")){
				// make sure you get focus when dragging the handle
				// (but don't do on IE because it causes a flicker on mouse up (due to blur then focus)
				focus.focus(this.sliderHandle);
			}
			e.stopPropagation();
			e.preventDefault();
		},

		_isReversed: function(){
			// summary:
			//		Returns true if direction is from right to left
			// tags:
			//		protected extension
			return !this.isLeftToRight();
		},

		_onBarClick: function(e){
			if(this.disabled || this.readOnly || !this.clickSelect){
				return;
			}
			focus.focus(this.sliderHandle);
			e.stopPropagation();
			e.preventDefault();
			var abspos = domGeometry.position(this.sliderBarContainer, true);
			var pixelValue = e[this._mousePixelCoord] - abspos[this._startingPixelCoord];
			this._setPixelValue(this._isReversed() ? (abspos[this._pixelCount] - pixelValue) : pixelValue, abspos[this._pixelCount], true);
			this._movable.onMouseDown(e);
		},

		_setPixelValue: function(/*Number*/ pixelValue, /*Number*/ maxPixels, /*Boolean?*/ priorityChange){
			if(this.disabled || this.readOnly){
				return;
			}
			var count = this.discreteValues;
			if(count <= 1 || count == Infinity){
				count = maxPixels;
			}
			count--;
			var pixelsPerValue = maxPixels / count;
			var wholeIncrements = Math.round(pixelValue / pixelsPerValue);
			this._setValueAttr(Math.max(Math.min((this.maximum - this.minimum) * wholeIncrements / count + this.minimum, this.maximum), this.minimum), priorityChange);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			this._set("value", value);
			this.valueNode.value = value;
			this.focusNode.setAttribute("aria-valuenow", value);
			this.inherited(arguments);
			var percent = (value - this.minimum) / (this.maximum - this.minimum);
			var progressBar = (this._descending === false) ? this.remainingBar : this.progressBar;
			var remainingBar = (this._descending === false) ? this.progressBar : this.remainingBar;
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			if(priorityChange && this.slideDuration > 0 && progressBar.style[this._progressPixelSize]){
				// animate the slider
				var _this = this;
				var props = {};
				var start = parseFloat(progressBar.style[this._progressPixelSize]);
				var duration = this.slideDuration * (percent - start / 100);
				if(duration == 0){
					return;
				}
				if(duration < 0){
					duration = 0 - duration;
				}
				props[this._progressPixelSize] = { start: start, end: percent * 100, units: "%" };
				this._inProgressAnim = fx.animateProperty({ node: progressBar, duration: duration,
					onAnimate: function(v){
						remainingBar.style[_this._progressPixelSize] = (100 - parseFloat(v[_this._progressPixelSize])) + "%";
					},
					onEnd: function(){
						delete _this._inProgressAnim;
					},
					properties: props
				});
				this._inProgressAnim.play();
			}else{
				progressBar.style[this._progressPixelSize] = (percent * 100) + "%";
				remainingBar.style[this._progressPixelSize] = ((1 - percent) * 100) + "%";
			}
		},

		_bumpValue: function(signedChange, /*Boolean?*/ priorityChange){
			if(this.disabled || this.readOnly){
				return;
			}
			var s = domStyle.getComputedStyle(this.sliderBarContainer);
			var c = domGeometry.getContentBox(this.sliderBarContainer, s);
			var count = this.discreteValues;
			if(count <= 1 || count == Infinity){
				count = c[this._pixelCount];
			}
			count--;
			var value = (this.value - this.minimum) * count / (this.maximum - this.minimum) + signedChange;
			if(value < 0){
				value = 0;
			}
			if(value > count){
				value = count;
			}
			value = value * (this.maximum - this.minimum) / count + this.minimum;
			this._setValueAttr(value, priorityChange);
		},

		_onClkBumper: function(val){
			if(this.disabled || this.readOnly || !this.clickSelect){
				return;
			}
			this._setValueAttr(val, true);
		},

		_onClkIncBumper: function(){
			this._onClkBumper(this._descending === false ? this.minimum : this.maximum);
		},

		_onClkDecBumper: function(){
			this._onClkBumper(this._descending === false ? this.maximum : this.minimum);
		},

		decrement: function(/*Event*/ e){
			// summary:
			//		Decrement slider
			// tags:
			//		private
			this._bumpValue(e.keyCode == keys.PAGE_DOWN ? -this.pageIncrement : -1);
		},

		increment: function(/*Event*/ e){
			// summary:
			//		Increment slider
			// tags:
			//		private
			this._bumpValue(e.keyCode == keys.PAGE_UP ? this.pageIncrement : 1);
		},

		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Event handler for mousewheel where supported
			evt.stopPropagation();
			evt.preventDefault();
			this._bumpValue(evt.wheelDelta < 0 ? -1 : 1, true); // negative scroll acts like a decrement
		},

		startup: function(){
			if(this._started){
				return;
			}

			array.forEach(this.getChildren(), function(child){
				if(this[child.container] != this.containerNode){
					this[child.container].appendChild(child.domNode);
				}
			}, this);

			this.inherited(arguments);
		},

		_typematicCallback: function(/*Number*/ count, /*Object*/ button, /*Event*/ e){
			if(count == -1){
				this._setValueAttr(this.value, true);
			}else{
				this[(button == (this._descending ? this.incrementButton : this.decrementButton)) ? "decrement" : "increment"](e);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			if(this.showButtons){
				this.incrementButton.style.display = "";
				this.decrementButton.style.display = "";
			}

			// find any associated label element and add to slider focusnode.
			var label = query('label[for="' + this.id + '"]');
			if(label.length){
				if(!label[0].id){
					label[0].id = this.id + "_label";
				}
				this.focusNode.setAttribute("aria-labelledby", label[0].id);
			}

			this.focusNode.setAttribute("aria-valuemin", this.minimum);
			this.focusNode.setAttribute("aria-valuemax", this.maximum);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(this.showButtons){
				this.own(
					typematic.addMouseListener(this.decrementButton, this, "_typematicCallback", 25, 500),
					typematic.addMouseListener(this.incrementButton, this, "_typematicCallback", 25, 500)
				);
			}
			this.own(
				on(this.domNode, mouse.wheel, lang.hitch(this, "_mouseWheeled"))
			);

			// define a custom constructor for a SliderMover that points back to me
			var mover = declare(_SliderMover, {
				widget: this
			});
			this._movable = new Moveable(this.sliderHandle, {mover: mover});

			this._layoutHackIE7();
		},

		destroy: function(){
			this._movable.destroy();
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			this.inherited(arguments);
		}
	});

	HorizontalSlider._Mover = _SliderMover;	// for monkey patching

	return HorizontalSlider;
});

},
'xcitec/Menu':function(){
define([
	"dojo/_base/declare",
	"dijit/Menu"
], function (declare, Menu) {

	var Menu = declare( Menu, {
	    
	})
	return Menu;
});
},
'xcitec/widgets/ThumbnailWidget':function(){
define([
	"dojo/_base/declare",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (declare, _Templated, _Widget) {

	var ThumbnailWidget = declare( [_Widget, _Templated], {
	  templateString: '<div class="${baseClass}" data-dojo-attach-point="containerNode"></div>',
	  baseClass: 'thumbnailWidget',
	
	  postMixInProperties: function() {
	    
	  }
	});
	return ThumbnailWidget;
});

},
'dojo/dnd/Manager':function(){
define([
	"../_base/array",  "../_base/declare", "../_base/lang", "../_base/window",
	"../dom-class", "../Evented", "../has", "../keys", "../on", "../topic", "../touch",
	"./common", "./autoscroll", "./Avatar"
], function(array, declare, lang, win, domClass, Evented, has, keys, on, topic, touch,
	dnd, autoscroll, Avatar){

// module:
//		dojo/dnd/Manager

var Manager = declare("dojo.dnd.Manager", [Evented], {
	// summary:
	//		the manager of DnD operations (usually a singleton)
	constructor: function(){
		this.avatar  = null;
		this.source = null;
		this.nodes = [];
		this.copy  = true;
		this.target = null;
		this.canDropFlag = false;
		this.events = [];
	},

	// avatar's offset from the mouse
	OFFSET_X: has("touch") ? 0 : 16,
	OFFSET_Y: has("touch") ? -64 : 16,

	// methods
	overSource: function(source){
		// summary:
		//		called when a source detected a mouse-over condition
		// source: Object
		//		the reporter
		if(this.avatar){
			this.target = (source && source.targetState != "Disabled") ? source : null;
			this.canDropFlag = Boolean(this.target);
			this.avatar.update();
		}
		topic.publish("/dnd/source/over", source);
	},
	outSource: function(source){
		// summary:
		//		called when a source detected a mouse-out condition
		// source: Object
		//		the reporter
		if(this.avatar){
			if(this.target == source){
				this.target = null;
				this.canDropFlag = false;
				this.avatar.update();
				topic.publish("/dnd/source/over", null);
			}
		}else{
			topic.publish("/dnd/source/over", null);
		}
	},
	startDrag: function(source, nodes, copy){
		// summary:
		//		called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		// Tell autoscroll that a drag is starting
		autoscroll.autoScrollStart(win.doc);

		this.source = source;
		this.nodes  = nodes;
		this.copy   = Boolean(copy); // normalizing to true boolean
		this.avatar = this.makeAvatar();
		win.body().appendChild(this.avatar.node);
		topic.publish("/dnd/start", source, nodes, this.copy);

		function stopEvent(e){
			e.preventDefault();
			e.stopPropagation();
		}

		this.events = [
			on(win.doc, touch.move, lang.hitch(this, "onMouseMove")),
			on(win.doc, touch.release,   lang.hitch(this, "onMouseUp")),
			on(win.doc, "keydown",   lang.hitch(this, "onKeyDown")),
			on(win.doc, "keyup",     lang.hitch(this, "onKeyUp")),

			// cancel text selection and text dragging
			on(win.doc, "dragstart",   stopEvent),
			on(win.body(), "selectstart", stopEvent)
		];
		var c = "dojoDnd" + (copy ? "Copy" : "Move");
		domClass.add(win.body(), c);
	},
	canDrop: function(flag){
		// summary:
		//		called to notify if the current target can accept items
		var canDropFlag = Boolean(this.target && flag);
		if(this.canDropFlag != canDropFlag){
			this.canDropFlag = canDropFlag;
			this.avatar.update();
		}
	},
	stopDrag: function(){
		// summary:
		//		stop the DnD in progress
		domClass.remove(win.body(), ["dojoDndCopy", "dojoDndMove"]);
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.events = [];
		this.avatar.destroy();
		this.avatar = null;
		this.source = this.target = null;
		this.nodes = [];
	},
	makeAvatar: function(){
		// summary:
		//		makes the avatar; it is separate to be overwritten dynamically, if needed
		return new Avatar(this);
	},
	updateAvatar: function(){
		// summary:
		//		updates the avatar; it is separate to be overwritten dynamically, if needed
		this.avatar.update();
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		var a = this.avatar;
		if(a){
			autoscroll.autoScrollNodes(e);
			//autoscroll.autoScroll(e);
			var s = a.node.style;
			s.left = (e.pageX + this.OFFSET_X) + "px";
			s.top  = (e.pageY + this.OFFSET_Y) + "px";
			var copy = Boolean(this.source.copyState(dnd.getCopyKeyState(e)));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
		if(has("touch")){
			// Prevent page from scrolling so that user can drag instead.
			e.preventDefault();
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.avatar){
			if(this.target && this.canDropFlag){
				var copy = Boolean(this.source.copyState(dnd.getCopyKeyState(e)));
				topic.publish("/dnd/drop/before", this.source, this.nodes, copy, this.target, e);
				topic.publish("/dnd/drop", this.source, this.nodes, copy, this.target, e);
			}else{
				topic.publish("/dnd/cancel");
			}
			this.stopDrag();
		}
	},

	// keyboard event processors
	onKeyDown: function(e){
		// summary:
		//		event processor for onkeydown:
		//		watching for CTRL for copy/move status, watching for ESCAPE to cancel the drag
		// e: Event
		//		keyboard event
		if(this.avatar){
			switch(e.keyCode){
				case keys.CTRL:
					var copy = Boolean(this.source.copyState(true));
					if(this.copy != copy){
						this._setCopyStatus(copy);
					}
					break;
				case keys.ESCAPE:
					topic.publish("/dnd/cancel");
					this.stopDrag();
					break;
			}
		}
	},
	onKeyUp: function(e){
		// summary:
		//		event processor for onkeyup, watching for CTRL for copy/move status
		// e: Event
		//		keyboard event
		if(this.avatar && e.keyCode == keys.CTRL){
			var copy = Boolean(this.source.copyState(false));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},

	// utilities
	_setCopyStatus: function(copy){
		// summary:
		//		changes the copy status
		// copy: Boolean
		//		the copy status
		this.copy = copy;
		this.source._markDndStatus(this.copy);
		this.updateAvatar();
		domClass.replace(win.body(),
			"dojoDnd" + (this.copy ? "Copy" : "Move"),
			"dojoDnd" + (this.copy ? "Move" : "Copy"));
	}
});

// dnd._manager:
//		The manager singleton variable. Can be overwritten if needed.
dnd._manager = null;

Manager.manager = dnd.manager = function(){
	// summary:
	//		Returns the current DnD manager.  Creates one if it is not created yet.
	if(!dnd._manager){
		dnd._manager = new Manager();
	}
	return dnd._manager;	// Object
};

// TODO: for 2.0, store _manager and manager in Manager only.   Don't access dnd or dojo.dnd.

return Manager;
});

},
'xcitec/_base/Action':function(){
define([
         "dojo/_base/declare",
         "dijit/_Container",
         "dijit/_WidgetBase"
       ], function (declare, _Container, _Widget) {
  return declare(_Widget, {
    actionId: "",
    actionName: "",
    actionState: "",

    getActionId: function(){
      return this.actionId;
    },

    getActionName: function(){
      return this.actionName;
    },

    getActionState: function(){
      return this.actionState;
    },

    actionEvent: function(){
    }
  });
});

},
'xcitec/grid/HGrid':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/dom",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dnd/Manager",
	"dojo/on",
	"dojo/query",
	"dojo/text!xcitec/grid/templates/HGrid.html",
	"dijit/registry",
	"xcitec/grid/_HGridBase",
  "./_HGridHeader",
  "./_HGridBody",
  "./Avatar"
], function (dojo, declare, array, dom, domClass, domConstruct, domStyle, Manager, on, query, template, registry, _HGridBase, _HGridHeader, _HGridBody, Avatar) {

	/***
	 * As source expected normal table definition:
	 <table>
	 <thead>
	 <tr><td columnId="c1">ColumnName</td><td columnId="c2">ColumnName</td><td columnId="c3">ColumnName</td>...</tr>
	 </thead>
	 <tbody>
	 <tr rowId="r1"><td>Col1Data</td><td>Col2Data</td><td>Col3Data</td>...</tr>
	 <tr rowId="r3"><td>Col1Data</td><td>Col2Data</td><td>Col3Data</td>...</tr>
	 <tr rowId="r3"><td>Col1Data</td><td>Col2Data</td><td>Col3Data</td>...</tr>
	 </tbody>
	 </table>
	 *
	 */
	var HGrid = declare( _HGridBase, {
		templateString: template,
	
		postCreate: function(){
			this.headWidget = new _HGridHeader({parentTable:this}, this.headerNode);
	
			if(this.vertScroll){
				domClass.add(this.domNode, "dataTableFixed");
				domClass.add(this.domNode, "minGridHeight");
			}
	
			domConstruct.place(this.headWidget.domNode, this.nodeTableContainer, "first");
			this.dataTableHead = this.headWidget.domNode;
	
			// if the grid is empty we don't need to create _HGridRows, we only show message
			if(this.emptyGridMessage || !this.rowsNode){
				domClass.add(this.domNode, "minGridHeight");
				var messageDiv = domConstruct.create("div", {innerHTML: this.emptyGridMessage}, this.dataTableBody, "first");
				domClass.add(messageDiv, "message");
			} else{
				var rows = query("tr", this.rowsNode);
				if(rows.length != 0){
					if(this.gridControl){
						this.rows = new _HGridBody({parentTable:this, selectedRows:this.gridControl.selectedRowsNode}, this.rowsNode);
					} else{
						this.rows = new _HGridBody({parentTable:this}, this.rowsNode);
					}
					this.body.appendChild(this.rows.domNode);
				} else{
					domClass.add(this.domNode, "minGridHeight");
				}
			}
			// scrolling
			var idHead = this.dataTableHead;
			var idBody = this.dataTableBody;
	
			on(idBody, "scroll", function(){
				idHead.scrollLeft = idBody.scrollLeft;
			});
	
			if(this.columnReorder){
			Manager.manager().makeAvatar = function(){
					return new Avatar(this);
				};
			}
	
			dom.setSelectable(this.domNode, false);
	
			if(this.gridControl && this.gridControl.activeColumn){
				domConstruct.place(this.gridControl.activeColumn, this.domNode, "first");
			}
			if(this.gridControl && this.gridControl.columnOrder){
				domConstruct.place(this.gridControl.columnOrder, this.domNode, "first");
			}
	
			if(this.gridControl && this.gridControl.columnWidthsNode){
				domConstruct.place(this.gridControl.columnWidthsNode, this.domNode, "first");
			}
	
			this.linkButtonsWidget = registry.byId(this.id + ".linkButtons");
			this.linkButtonsNode = dom.byId(this.id + ".linkButtons");
			if(this.linkButtonsWidget){
				domConstruct.place(this.linkButtonsWidget.domNode, this.footer, "last");
			} else if(this.linkButtonsNode){
				domConstruct.place(this.linkButtonsNode, this.footer, "last");
			}
			if(this.gridActions){
				domConstruct.place(this.gridActions.domNode, this.domNode, "last");
				domStyle.set(this.gridActions.domNode, "display", "none");
			}
	
			if(this.vertScroll){
				domStyle.set(this.dataTableBody, "overflowY", "scroll");
			}
	
			var _this = this;
			if(this.headWidget.cellToRightAlign.length > 0){
				if(this.rows){
					array.forEach(this.headWidget.allCells, function(cell, index){
						array.forEach(_this.headWidget.cellToRightAlign, function (c){
							if(c == cell){
								_this.rows.makeColumnRightAlign(index);
							}
						});
					});
				}
			}
			this.inherited(arguments);
		}
	});
	return HGrid;
});
},
'dijit/_KeyNavMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/keys", // keys.END keys.HOME, keys.LEFT_ARROW etc.
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dijit/registry",
	"dijit/_FocusMixin"        // to make _onBlur() work
], function(array, declare, domAttr, keys, lang, on, registry, _FocusMixin){

	// module:
	//		dijit/_KeyNavMixin

	return declare("dijit._KeyNavMixin", _FocusMixin, {
		// summary:
		//		A mixin to allow arrow key and letter key navigation of child or descendant widgets.
		//		It can be used by dijit/_Container based widgets with a flat list of children,
		//		or more complex widgets like dijit/Tree.
		//
		//		To use this mixin, the subclass must:
		//
		//			- Implement  _getNext(), _getFirst(), _getLast(), _onLeftArrow(), _onRightArrow()
		//			  _onDownArrow(), _onUpArrow() methods to handle home/end/left/right/up/down keystrokes.
		//			  Next and previous in this context refer to a linear ordering of the descendants used
		//			  by letter key search.
		//			- Set all descendants' initial tabIndex to "-1"; both initial descendants and any
		//			  descendants added later, by for example addChild()
		//			- Define childSelector to a function or string that identifies focusable descendant widgets
		//
		//		Also, child widgets must implement a focus() method.

		/*=====
		 // focusedChild: [protected readonly] Widget
		 //		The currently focused child widget, or null if there isn't one
		 focusedChild: null,

		 // _keyNavCodes: Object
		 //		Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.
		 //		Usually not used directly, as subclasses can instead override _onLeftArrow() etc.
		 _keyNavCodes: {},
		 =====*/

		// tabIndex: String
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		// childSelector: [protected abstract] Function||String
		//		Selector (passed to on.selector()) used to identify what to treat as a child widget.   Used to monitor
		//		focus events and set this.focusedChild.   Must be set by implementing class.   If this is a string
		//		(ex: "> *") then the implementing class must require dojo/query.
		childSelector: null,

		postCreate: function(){
			this.inherited(arguments);

			// Set tabIndex on this.domNode.  Will be automatic after #7381 is fixed.
			domAttr.set(this.domNode, "tabIndex", this.tabIndex);

			if(!this._keyNavCodes){
				var keyCodes = this._keyNavCodes = {};
				keyCodes[keys.HOME] = lang.hitch(this, "focusFirstChild");
				keyCodes[keys.END] = lang.hitch(this, "focusLastChild");
				keyCodes[this.isLeftToRight() ? keys.LEFT_ARROW : keys.RIGHT_ARROW] = lang.hitch(this, "_onLeftArrow");
				keyCodes[this.isLeftToRight() ? keys.RIGHT_ARROW : keys.LEFT_ARROW] = lang.hitch(this, "_onRightArrow");
				keyCodes[keys.UP_ARROW] = lang.hitch(this, "_onUpArrow");
				keyCodes[keys.DOWN_ARROW] = lang.hitch(this, "_onDownArrow");
			}

			var self = this,
				childSelector = typeof this.childSelector == "string"
					? this.childSelector
					: lang.hitch(this, "childSelector");
			this.own(
				on(this.domNode, "keypress", lang.hitch(this, "_onContainerKeypress")),
				on(this.domNode, "keydown", lang.hitch(this, "_onContainerKeydown")),
				on(this.domNode, "focus", lang.hitch(this, "_onContainerFocus")),
				on(this.containerNode, on.selector(childSelector, "focusin"), function(evt){
					self._onChildFocus(registry.getEnclosingWidget(this), evt);
				})
			);
		},

		_onLeftArrow: function(){
			// summary:
			//		Called on left arrow key, or right arrow key if widget is in RTL mode.
			//		Should go back to the previous child in horizontal container widgets like Toolbar.
			// tags:
			//		extension
		},

		_onRightArrow: function(){
			// summary:
			//		Called on right arrow key, or left arrow key if widget is in RTL mode.
			//		Should go to the next child in horizontal container widgets like Toolbar.
			// tags:
			//		extension
		},

		_onUpArrow: function(){
			// summary:
			//		Called on up arrow key. Should go to the previous child in vertical container widgets like Menu.
			// tags:
			//		extension
		},

		_onDownArrow: function(){
			// summary:
			//		Called on down arrow key. Should go to the next child in vertical container widgets like Menu.
			// tags:
			//		extension
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused.

			// Leverage _getNextFocusableChild() to skip disabled children
			return this._getNextFocusableChild(null, 1);	// dijit/_WidgetBase
		},

		_getLastFocusableChild: function(){
			// summary:
			//		Returns last child that can be focused.

			// Leverage _getNextFocusableChild() to skip disabled children
			return this._getNextFocusableChild(null, -1);	// dijit/_WidgetBase
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected

			this.focusChild(this._getFirstFocusableChild());
		},

		focusLastChild: function(){
			// summary:
			//		Focus the last focusable child in the container.
			// tags:
			//		protected

			this.focusChild(this._getLastFocusableChild());
		},

		focusChild: function(/*dijit/_WidgetBase*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus specified child widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected

			if(!widget){
				return;
			}

			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);	// used to be used by _MenuBase
			}
			widget.set("tabIndex", this.tabIndex);	// for IE focus outline to appear, must set tabIndex before focus
			widget.focus(last ? "end" : "start");

			// Don't set focusedChild here, because the focus event should trigger a call to _onChildFocus(), which will
			// set it.   More importantly, _onChildFocus(), which may be executed asynchronously (after this function
			// returns) needs to know the old focusedChild to set its tabIndex to -1.
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container itself gets focus.
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child.
			//
			//		TODO for 2.0 (or earlier): Instead of having the container tabbable, always maintain a single child
			//		widget as tabbable, Requires code in startup(), addChild(), and removeChild().
			//		That would avoid various issues like #17347.
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively).
			// Also, _onFocus() would fire when focus went directly to a child widget due to mouse click.

			// Ignore spurious focus events:
			//	1. focus on a child widget bubbles on FF
			//	2. on IE, clicking the scrollbar of a select dropdown moves focus from the focused child item to me
			if(evt.target !== this.domNode || this.focusedChild){
				return;
			}

			this.focus();
		},

		_onFocus: function(){
			// When the container gets focus by being tabbed into, or a descendant gets focus by being clicked,
			// set the container's tabIndex to -1 (don't remove as that breaks Safari 4) so that tab or shift-tab
			// will go to the fields after/before the container, rather than the container itself
			domAttr.set(this.domNode, "tabIndex", "-1");

			this.inherited(arguments);
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)

			// TODO: for 2.0 consider changing this to blur whenever the container blurs, to be truthful that there is
			// no focused child at that time.

			domAttr.set(this.domNode, "tabIndex", this.tabIndex);
			if(this.focusedChild){
				this.focusedChild.set("tabIndex", "-1");
				this.lastFocusedChild = this.focusedChild;
				this._set("focusedChild", null);
			}
			this.inherited(arguments);
		},

		_onChildFocus: function(/*dijit/_WidgetBase*/ child){
			// summary:
			//		Called when a child widget gets focus, either by user clicking
			//		it, or programatically by arrow key handling code.
			// description:
			//		It marks that the current node is the selected one, and the previously
			//		selected node no longer is.

			if(child && child != this.focusedChild){
				if(this.focusedChild && !this.focusedChild._destroyed){
					// mark that the previously focusable node is no longer focusable
					this.focusedChild.set("tabIndex", "-1");
				}

				// mark that the new node is the currently selected one
				child.set("tabIndex", this.tabIndex);
				this.lastFocused = child;		// back-compat for Tree, remove for 2.0
				this._set("focusedChild", child);
			}
		},

		_searchString: "",
		// multiCharSearchDuration: Number
		//		If multiple characters are typed where each keystroke happens within
		//		multiCharSearchDuration of the previous keystroke,
		//		search for nodes matching all the keystrokes.
		//
		//		For example, typing "ab" will search for entries starting with
		//		"ab" unless the delay between "a" and "b" is greater than multiCharSearchDuration.
		multiCharSearchDuration: 1000,

		onKeyboardSearch: function(/*dijit/_WidgetBase*/ item, /*Event*/ evt, /*String*/ searchString, /*Number*/ numMatches){
			// summary:
			//		When a key is pressed that matches a child item,
			//		this method is called so that a widget can take appropriate action is necessary.
			// tags:
			//		protected
			if(item){
				this.focusChild(item);
			}
		},

		_keyboardSearchCompare: function(/*dijit/_WidgetBase*/ item, /*String*/ searchString){
			// summary:
			//		Compares the searchString to the widget's text label, returning:
			//
			//			* -1: a high priority match  and stop searching
			//		 	* 0: not a match
			//		 	* 1: a match but keep looking for a higher priority match
			// tags:
			//		private

			var element = item.domNode,
				text = item.label || (element.focusNode ? element.focusNode.label : '') || element.innerText || element.textContent || "",
				currentString = text.replace(/^\s+/, '').substr(0, searchString.length).toLowerCase();

			return (!!searchString.length && currentString == searchString) ? -1 : 0; // stop searching after first match by default
		},

		_onContainerKeydown: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then it's handled here.
			// tags:
			//		private

			var func = this._keyNavCodes[evt.keyCode];
			if(func){
				func(evt, this.focusedChild);
				evt.stopPropagation();
				evt.preventDefault();
				this._searchString = ''; // so a DOWN_ARROW b doesn't search for ab
			}else if(evt.keyCode == keys.SPACE && this._searchTimer && !(evt.ctrlKey || evt.altKey || evt.metaKey)){
				evt.stopImmediatePropagation(); // stop a11yclick and _HasDropdown from seeing SPACE if we're doing keyboard searching
				evt.preventDefault(); // stop IE from scrolling, and most browsers (except FF) from sending keypress
				this._keyboardSearch(evt, ' ');
			}
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a printable key is pressed, it's handled here, searching by letter.
			// tags:
			//		private

			// Ignore:
			// 		- duplicate events on firefox (ex: arrow key that will be handled by keydown handler)
			//		- control sequences like CMD-Q.
			//		- the SPACE key (only occurs on FF)
			//
			// Note: if there's no search in progress, then SPACE should be ignored.   If there is a search
			// in progress, then SPACE is handled in _onContainerKeyDown.
			if(evt.charCode <= keys.SPACE || evt.ctrlKey || evt.altKey || evt.metaKey){
				return;
			}

			evt.preventDefault();
			evt.stopPropagation();

			this._keyboardSearch(evt, String.fromCharCode(evt.charCode).toLowerCase());
		},

		_keyboardSearch: function(/*Event*/ evt, /*String*/ keyChar){
			// summary:
			//		Perform a search of the widget's options based on the user's keyboard activity
			// description:
			//		Called on keypress (and sometimes keydown), searches through this widget's children
			//		looking for items that match the user's typed search string.  Multiple characters
			//		typed within 1 sec of each other are combined for multicharacter searching.
			// tags:
			//		private
			var
				matchedItem = null,
				searchString,
				numMatches = 0,
				search = lang.hitch(this, function(){
					if(this._searchTimer){
						this._searchTimer.remove();
					}
					this._searchString += keyChar;
					var allSameLetter = /^(.)\1*$/.test(this._searchString);
					var searchLen = allSameLetter ? 1 : this._searchString.length;
					searchString = this._searchString.substr(0, searchLen);
					// commented out code block to search again if the multichar search fails after a smaller timeout
					//this._searchTimer = this.defer(function(){ // this is the "failure" timeout
					//	this._typingSlowly = true; // if the search fails, then treat as a full timeout
					//	this._searchTimer = this.defer(function(){ // this is the "success" timeout
					//		this._searchTimer = null;
					//		this._searchString = '';
					//	}, this.multiCharSearchDuration >> 1);
					//}, this.multiCharSearchDuration >> 1);
					this._searchTimer = this.defer(function(){ // this is the "success" timeout
						this._searchTimer = null;
						this._searchString = '';
					}, this.multiCharSearchDuration);
					var currentItem = this.focusedChild || null;
					if(searchLen == 1 || !currentItem){
						currentItem = this._getNextFocusableChild(currentItem, 1); // skip current
						if(!currentItem){
							return;
						} // no items
					}
					var stop = currentItem;
					do{
						var rc = this._keyboardSearchCompare(currentItem, searchString);
						if(!!rc && numMatches++ == 0){
							matchedItem = currentItem;
						}
						if(rc == -1){ // priority match
							numMatches = -1;
							break;
						}
						currentItem = this._getNextFocusableChild(currentItem, 1);
					}while(currentItem != stop);
					// commented out code block to search again if the multichar search fails after a smaller timeout
					//if(!numMatches && (this._typingSlowly || searchLen == 1)){
					//	this._searchString = '';
					//	if(searchLen > 1){
					//		// if no matches and they're typing slowly, then go back to first letter searching
					//		search();
					//	}
					//}
				});

			search();
			// commented out code block to search again if the multichar search fails after a smaller timeout
			//this._typingSlowly = false;
			this.onKeyboardSearch(matchedItem, evt, searchString, numMatches);
		},

		_onChildBlur: function(/*dijit/_WidgetBase*/ /*===== widget =====*/){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			//		Used to be used by MenuBase.js (remove for 2.0)
			// tags:
			//		protected
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable descendant, compared to "child".
			//		Implements and extends _KeyNavMixin._getNextFocusableChild() for a _Container.
			// child: Widget
			//		The current widget
			// dir: Integer
			//		- 1 = after
			//		- -1 = before
			// tags:
			//		abstract extension

			var wrappedValue = child;
			do{
				if(!child){
					child = this[dir > 0 ? "_getFirst" : "_getLast"]();
					if(!child){ break; }
				}else{
					child = this._getNext(child, dir);
				}
				if(child != null && child != wrappedValue && child.isFocusable()){
					return child;	// dijit/_WidgetBase
				}
			}while(child != wrappedValue);
			// no focusable child found
			return null;	// dijit/_WidgetBase
		},

		_getFirst: function(){
			// summary:
			//		Returns the first child.
			// tags:
			//		abstract extension

			return null;	// dijit/_WidgetBase
		},

		_getLast: function(){
			// summary:
			//		Returns the last descendant.
			// tags:
			//		abstract extension

			return null;	// dijit/_WidgetBase
		},

		_getNext: function(child, dir){
			// summary:
			//		Returns the next descendant, compared to "child".
			// child: Widget
			//		The current widget
			// dir: Integer
			//		- 1 = after
			//		- -1 = before
			// tags:
			//		abstract extension

			if(child){
				child = child.domNode;
				while(child){
					child = child[dir < 0 ? "previousSibling" : "nextSibling"];
					if(child  && "getAttribute" in child){
						var w = registry.byNode(child);
						if(w){
							return w; // dijit/_WidgetBase
						}
					}
				}
			}
			return null;	// dijit/_WidgetBase
		}
	});
});

},
'xcitec/ComboContainer':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"dijit/_Container",
  "./form/LinkButton"
], function (dojo, array, declare, lang, domConstruct, query, _Templated, _Widget, registry, _Container, LinkButton) {

	var ComboContainer = declare( [_Widget, _Templated, _Container], {
	
	    templateString: 
	    '<div data-dojo-attach-point="containerNode" class="comboContainer">' +
	      '<div class="comboContainerAction">' +
	        '<div class="linksWrapp" style="float:left" data-dojo-attach-point="linksWrap">' +
	          '<span class="linksBracket">[</span>' +
	          '<div data-dojo-attach-point="links" class="links"></div>' +
	          '<span class="linksBracket">]</span>' +
	        '</div>' +
	        '<div class="linksWrapp" style="float:right" data-dojo-attach-point="linksWrap2">' +
	          '<span class="linksBracket">[</span>' +
	          '<div data-dojo-attach-point="links2" class="links"></div>' +
	          '<span class="linksBracket">]</span>' +
	        '</div>' +
	        '<div class="linksWrapp linksMarks" style="float:right" data-dojo-attach-point="linksWrap3">' +
	          '<span class="linksBracket">[</span>' +
	          '<div data-dojo-attach-point="links3" class="links"></div>' +
	          '<span class="linksBracket">]</span>' +
	        '</div>' +
	      '</div>' +
	    '</div>',
	
	    // arrays of nodes and widgets to be open/collpase
	    comboElementsNodes: null,
	    comboElementsWidgets: null,
	    // array of eva criterion widgets levels
	    levels: null,
	
		postMixInProperties: function() {
	        this.comboElementsNodes = [];
	        this.comboElementsWidgets = [];
			this.levels = [];
	        this.inherited(arguments);
		},
	
	    startup: function() {
	        // search for actions and create buttons
	        this.actionsContainer = registry.byId(this.id + ".actions");
	        this.changesContainer = registry.byId(this.id + ".changes");
	        this.rightActionsContainer = registry.byId(this.id + ".rightActions");
	        var _this = this;
	        if (this.actionsContainer) {
	            var actions = this.actionsContainer.getChildren();
	            var link = null;
	            array.forEach(actions, function(action) {
	                if (action.getActionId() == "COLLAPSE_ALL") {
	                    link = new LinkButton({caption: action.getActionName(), onClick: lang.hitch(_this, "collapseAll")});
	                } else if (action.getActionId() == "OPEN_ALL") {
	                    link = new LinkButton({caption: action.getActionName(), onClick: lang.hitch(_this, "openAll")});
	                } else if (action.getActionId() == "OPEN_STRUCTURE") {
	                    link = new LinkButton({caption: action.getActionName(), onClick: lang.hitch(_this, "openStructure")});
	                } 
	                _this.links.appendChild(link.domNode);
	            });
	            
	        }else { domConstruct.destroy(this.linksWrap) 
	          }; 
	          
	          
	          if (this.changesContainer) {
	            var changes = this.changesContainer.getChildren();
	            array.forEach(changes, function(action) {
	//                var link = null;
	//                if (action.getActionId() == "NO_CHANGES") {
	//                    link = new xcitec.form.LinkButton({caption: action.getActionName(), onClick: dojo.hitch(_this, "noChanges")});
	//                } else if (action.getActionId() == "LAST_CHANGES") {
	//                    link = new xcitec.form.LinkButton({caption: action.getActionName(), onClick: dojo.hitch(_this, "lastChanges")});
	//                } else if (action.getActionId() == "ALL_CHANGES") {
	//                    link = new xcitec.form.LinkButton({caption: action.getActionName(), onClick: dojo.hitch(_this, "allChanges")});
	//                }
	                var link = new LinkButton({caption: action.getActionName(), onClick: action.actionEvent});
	                _this.links2.appendChild(link.domNode);
	            });
	            
	        }else { domConstruct.destroy(this.linksWrap2) 
	          };
	          
	           if (this.rightActionsContainer) {
	            var rightActions = this.rightActionsContainer.getChildren();
	            array.forEach(rightActions, function(action) {
	                var link = new LinkButton({
		                caption: action.getActionName(),
		                checked: (action.getActionState()=="checked"),
		                onClick: action.actionEvent});
	                _this.links3.appendChild(link.domNode);
	            });
	
	        }else { domConstruct.destroy(this.linksWrap3) 
	          }  
	          
	          
	          
	          
	                  
	        this.inherited(arguments);
	    },
	
	    collapseAll: function() {
	        this.initComboElements();
	        array.forEach(this.comboElementsWidgets, function(widget) {
	            if (lang.exists('getState', widget)) {
	                if (widget.getState() == 'Open') {
	                    if (("level" in widget)) {
	                        if (widget.level >= 0) {
	                            widget.collapse();
	                        } else if (("evaType" in widget) && (widget.evaType == "criterion")) {
	                            widget.collapse();
	                        }
	                    } else {
	                        widget.collapse();
	                    }
	                }
	            } else if (widget.selected) {
	                widget.collapse();
	            }
	        })
	    },
	
	    openAll: function() {
	        this.initComboElements();
	        array.forEach(this.comboElementsWidgets, function(widget) {
	            if (lang.exists('getState', widget)) {
	                if (("evaType" in widget) && (widget.dependencyChapter)) {
	                    if (widget.getState() == 'Open') {
	                        widget.collapse();
	                    }
	                } else if (widget.getState() == 'Close') {
	                    widget.expand();
	                }
	            } else if (!widget.selected && !widget.disabled) {
	                widget.expand();
	            }
	        })
	    },
	
	    openStructure: function() {
	        this.initComboElements();
	        array.forEach(this.comboElementsWidgets, function(widget) {
	            if (widget.hasInnerCombo()) {
	                if (("evaType" in widget) && (widget.dependency)) {
	                    if (widget.getState() == 'Open') {
	                        widget.collapse();
	                    }
	                } else if (widget.getState() == 'Close') {
	                    widget.expand();
	                }
	            } else {
	                if (lang.exists('getState', widget)) {
	                    if ((widget.level > 0)) {
	                        if (widget.getState() == 'Open') {
	                            widget.collapse();
	                        }
	                    } else if (("evaType" in widget) && (widget.evaType == "criterion")) {
	                        if (widget.getState() == 'Open') {
	                            widget.collapse();
	                        }
	                    }
	                } else if (widget.selected && !widget.disabled && widget.leaf) {
	                    widget.collapse();
	                } else if (!widget.selected &&!widget.disabled && !widget.leaf) {
	                    widget.expand();
	                }
	            }
	        });
	    },
	    
	    noChanges: function() {
	      
	    },
	    
	    lastChanges: function() {
	      
	    },
	    
	    allChanges: function() {
	     
	    },
	    
	    allEvaluator: function() {
	    
	    },
	    
	    onlyFinshed: function() {
	    
	    },
	    
	    onlyInProgress: function() {
	    
	    },
	    
	    onlyNotStarted: function() {
	    
	    },

		resize: function() {
			array.forEach(this.getChildren(), function(widget) {
				if('resize' in widget){
					widget.resize();
				}
			});
		},
	
	    // initialize widgets to open/collapse
	    initComboElements: function() {
	        var _this = this;
	        this.comboElementsWidgets = [];
	        this.comboElementsNodes = query("[widgetId]", this.domNode);
	        array.forEach(this.comboElementsNodes, function(node) {
	            if (lang.exists("expand", registry.byNode(node))) {
	                _this.comboElementsWidgets.push(registry.byNode(node));
	            }
	        });
	    }
	});
	return ComboContainer;
});

},
'dijit/form/_TextBoxMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/has",
	"dojo/keys", // keys.ALT keys.CAPS_LOCK keys.CTRL keys.META keys.SHIFT
	"dojo/_base/lang", // lang.mixin
	"dojo/on", // on
	"../main"    // for exporting dijit._setSelectionRange, dijit.selectInputText
], function(array, declare, dom, has, keys, lang, on, dijit){

	// module:
	//		dijit/form/_TextBoxMixin

	var _TextBoxMixin = declare("dijit.form._TextBoxMixin" + (has("dojo-bidi") ? "_NoBidi" : ""), null, {
		// summary:
		//		A mixin for textbox form input widgets

		// trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		// uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		// lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		// propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		// selectOnClick: [const] Boolean
		//		If true, all text will be selected when focused with mouse
		selectOnClick: false,

		// placeHolder: String
		//		Defines a hint to help users fill out the input field (as defined in HTML 5).
		//		This should only contain plain text (no html markup).
		placeHolder: "",

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works as we like.
			// description:
			//		For `dijit/form/TextBox` this basically returns the value of the `<input>`.
			//
			//		For `dijit/form/MappedTextBox` subclasses, which have both
			//		a "displayed value" and a separate "submit value",
			//		This treats the "displayed value" as the master value, computing the
			//		submit value from it via this.parse().
			return this.parse(this.get('displayedValue'), this.constraints);
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so set('value', ...) works.
			//
			// description:
			//		Sets the value of the widget to "value" which can be of
			//		any type as determined by the widget.
			//
			// value:
			//		The visual element value is also set to a corresponding,
			//		but not necessarily the same, value.
			//
			// formattedValue:
			//		If specified, used to set the visual element value,
			//		otherwise a computed visual value is used.
			//
			// priorityChange:
			//		If true, an onChange event is fired immediately instead of
			//		waiting for the next blur event.

			var filteredValue;
			if(value !== undefined){
				// TODO: this is calling filter() on both the display value and the actual value.
				// I added a comment to the filter() definition about this, but it should be changed.
				filteredValue = this.filter(value);
				if(typeof formattedValue != "string"){
					if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
						formattedValue = this.filter(this.format(filteredValue, this.constraints));
					}else{
						formattedValue = '';
					}
				}
			}
			if(formattedValue != null /* and !undefined */ && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
				this.textbox.value = formattedValue;
				this._set("displayedValue", this.get("displayedValue"));
			}

			this.inherited(arguments, [filteredValue, priorityChange]);
		},

		// displayedValue: String
		//		For subclasses like ComboBox where the displayed value
		//		(ex: Kentucky) and the serialized value (ex: KY) are different,
		//		this represents the displayed value.
		//
		//		Setting 'displayedValue' through set('displayedValue', ...)
		//		updates 'value', and vice-versa.  Otherwise 'value' is updated
		//		from 'displayedValue' periodically, like onBlur etc.
		//
		//		TODO: move declaration to MappedTextBox?
		//		Problem is that ComboBox references displayedValue,
		//		for benefit of FilteringSelect.
		displayedValue: "",

		_getDisplayedValueAttr: function(){
			// summary:
			//		Hook so get('displayedValue') works.
			// description:
			//		Returns the displayed value (what the user sees on the screen),
			//		after filtering (ie, trimming spaces etc.).
			//
			//		For some subclasses of TextBox (like ComboBox), the displayed value
			//		is different from the serialized value that's actually
			//		sent to the server (see `dijit/form/ValidationTextBox.serialize()`)

			// TODO: maybe we should update this.displayedValue on every keystroke so that we don't need
			// this method
			// TODO: this isn't really the displayed value when the user is typing
			return this.filter(this.textbox.value);
		},

		_setDisplayedValueAttr: function(/*String*/ value){
			// summary:
			//		Hook so set('displayedValue', ...) works.
			// description:
			//		Sets the value of the visual element to the string "value".
			//		The widget value is also set to a corresponding,
			//		but not necessarily the same, value.

			if(value == null /* or undefined */){
				value = ''
			}
			else if(typeof value != "string"){
				value = String(value)
			}

			this.textbox.value = value;

			// sets the serialized value to something corresponding to specified displayedValue
			// (if possible), and also updates the textbox.value, for example converting "123"
			// to "123.00"
			this._setValueAttr(this.get('value'), undefined);

			this._set("displayedValue", this.get('displayedValue'));
		},

		format: function(value /*=====, constraints =====*/){
			// summary:
			//		Replaceable function to convert a value to a properly formatted string.
			// value: String
			// constraints: Object
			// tags:
			//		protected extension
			return value == null /* or undefined */ ? "" : (value.toString ? value.toString() : value);
		},

		parse: function(value /*=====, constraints =====*/){
			// summary:
			//		Replaceable function to convert a formatted string to a value
			// value: String
			// constraints: Object
			// tags:
			//		protected extension

			return value;	// String
		},

		_refreshState: function(){
			// summary:
			//		After the user types some characters, etc., this method is
			//		called to check the field for validity etc.  The base method
			//		in `dijit/form/TextBox` does nothing, but subclasses override.
			// tags:
			//		protected
		},

		 onInput: function(/*===== event =====*/){
			 // summary:
			 //		Connect to this function to receive notifications of various user data-input events.
			 //		Return false to cancel the event and prevent it from being processed.
			 // event:
			 //		keydown | keypress | cut | paste | input
			 // tags:
			 //		callback
		 },

		__skipInputEvent: false,
		_onInput: function(/*Event*/ evt){
			// summary:
			//		Called AFTER the input event has happened

			this._processInput(evt);

			if(this.intermediateChanges){
				// allow the key to post to the widget input box
				this.defer(function(){
					this._handleOnChange(this.get('value'), false);
				});
			}
		},

		_processInput: function(/*Event*/ evt){
			// summary:
			//		Default action handler for user input events

			this._refreshState();

			// In case someone is watch()'ing for changes to displayedValue
			this._set("displayedValue", this.get("displayedValue"));
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values should be the same

			this.inherited(arguments);

			// normalize input events to reduce spurious event processing
			//	onkeydown: do not forward modifier keys
			//		       set charOrCode to numeric keycode
			//	onkeypress: do not forward numeric charOrCode keys (already sent through onkeydown)
			//	onpaste & oncut: set charOrCode to 229 (IME)
			//	oninput: if primary event not already processed, set charOrCode to 229 (IME), else do not forward
			function handleEvent(e){
				var charOrCode;
				if(e.type == "keydown"){
					charOrCode = e.keyCode;
					switch(charOrCode){ // ignore state keys
						case keys.SHIFT:
						case keys.ALT:
						case keys.CTRL:
						case keys.META:
						case keys.CAPS_LOCK:
						case keys.NUM_LOCK:
						case keys.SCROLL_LOCK:
							return;
					}
					if(!e.ctrlKey && !e.metaKey && !e.altKey){ // no modifiers
						switch(charOrCode){ // ignore location keys
							case keys.NUMPAD_0:
							case keys.NUMPAD_1:
							case keys.NUMPAD_2:
							case keys.NUMPAD_3:
							case keys.NUMPAD_4:
							case keys.NUMPAD_5:
							case keys.NUMPAD_6:
							case keys.NUMPAD_7:
							case keys.NUMPAD_8:
							case keys.NUMPAD_9:
							case keys.NUMPAD_MULTIPLY:
							case keys.NUMPAD_PLUS:
							case keys.NUMPAD_ENTER:
							case keys.NUMPAD_MINUS:
							case keys.NUMPAD_PERIOD:
							case keys.NUMPAD_DIVIDE:
								return;
						}
						if((charOrCode >= 65 && charOrCode <= 90) || (charOrCode >= 48 && charOrCode <= 57) || charOrCode == keys.SPACE){
							return; // keypress will handle simple non-modified printable keys
						}
						var named = false;
						for(var i in keys){
							if(keys[i] === e.keyCode){
								named = true;
								break;
							}
						}
						if(!named){
							return;
						} // only allow named ones through
					}
				}
				charOrCode = e.charCode >= 32 ? String.fromCharCode(e.charCode) : e.charCode;
				if(!charOrCode){
					charOrCode = (e.keyCode >= 65 && e.keyCode <= 90) || (e.keyCode >= 48 && e.keyCode <= 57) || e.keyCode == keys.SPACE ? String.fromCharCode(e.keyCode) : e.keyCode;
				}
				if(!charOrCode){
					charOrCode = 229; // IME
				}
				if(e.type == "keypress"){
					if(typeof charOrCode != "string"){
						return;
					}
					if((charOrCode >= 'a' && charOrCode <= 'z') || (charOrCode >= 'A' && charOrCode <= 'Z') || (charOrCode >= '0' && charOrCode <= '9') || (charOrCode === ' ')){
						if(e.ctrlKey || e.metaKey || e.altKey){
							return;
						} // can only be stopped reliably in keydown
					}
				}
				if(e.type == "input"){
					if(this.__skipInputEvent){ // duplicate event
						this.__skipInputEvent = false;
						return;
					}
				}else{
					this.__skipInputEvent = true;
				}
				// create fake event to set charOrCode and to know if preventDefault() was called
				var faux = { faux: true }, attr;
				for(attr in e){
					if(!/^(layer[XY]|returnValue|keyLocation)$/.test(attr)){ // prevent WebKit warnings
						var v = e[attr];
						if(typeof v != "function" && typeof v != "undefined"){
							faux[attr] = v;
						}
					}
				}
				lang.mixin(faux, {
					charOrCode: charOrCode,
					_wasConsumed: false,
					preventDefault: function(){
						faux._wasConsumed = true;
						e.preventDefault();
					},
					stopPropagation: function(){
						e.stopPropagation();
					}
				});
				// give web page author a chance to consume the event
				//console.log(faux.type + ', charOrCode = (' + (typeof charOrCode) + ') ' + charOrCode + ', ctrl ' + !!faux.ctrlKey + ', alt ' + !!faux.altKey + ', meta ' + !!faux.metaKey + ', shift ' + !!faux.shiftKey);
				if(this.onInput(faux) === false){ // return false means stop
					faux.preventDefault();
					faux.stopPropagation();
				}
				if(faux._wasConsumed){
					return;
				} // if preventDefault was called
				this.defer(function(){
					this._onInput(faux);
				}); // widget notification after key has posted
			}
			this.own(
				on(this.textbox, "keydown, keypress, paste, cut, input, compositionend", lang.hitch(this, handleEvent)),

				// Allow keypress to bubble to this.domNode, so that TextBox.on("keypress", ...) works,
				// but prevent it from further propagating, so that typing into a TextBox inside a Toolbar doesn't
				// trigger the Toolbar's letter key navigation.
				on(this.domNode, "keypress", function(e){ e.stopPropagation(); })
			);
		},

		_blankValue: '', // if the textbox is blank, what value should be reported
		filter: function(val){
			// summary:
			//		Auto-corrections (such as trimming) that are applied to textbox
			//		value on blur or form submit.
			// description:
			//		For MappedTextBox subclasses, this is called twice
			//
			//		- once with the display value
			//		- once the value as set/returned by set('value', ...)
			//
			//		and get('value'), ex: a Number for NumberTextBox.
			//
			//		In the latter case it does corrections like converting null to NaN.  In
			//		the former case the NumberTextBox.filter() method calls this.inherited()
			//		to execute standard trimming code in TextBox.filter().
			//
			//		TODO: break this into two methods in 2.0
			//
			// tags:
			//		protected extension
			if(val === null){
				return this._blankValue;
			}
			if(typeof val != "string"){
				return val;
			}
			if(this.trim){
				val = lang.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0, 1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		_setBlurValue: function(){
			// Format the displayed value, for example (for NumberTextBox) convert 1.4 to 1.400,
			// or (for CurrencyTextBox) 2.50 to $2.50

			this._setValueAttr(this.get('value'), true);
		},

		_onBlur: function(e){
			if(this.disabled){
				return;
			}
			this._setBlurValue();
			this.inherited(arguments);
		},

		_isTextSelected: function(){
			return this.textbox.selectionStart != this.textbox.selectionEnd;
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){
				return;
			}

			// Select all text on focus via click if nothing already selected.
			// Since mouse-up will clear the selection, need to defer selection until after mouse-up.
			// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
			if(this.selectOnClick && by == "mouse"){
				// Use on.once() to only select all text on first click only; otherwise users would have no way to clear
				// the selection.
				this._selectOnClickHandle = on.once(this.domNode, "mouseup, touchend", lang.hitch(this, function(evt){
					// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
					// and if not, then select all the text
					if(!this._isTextSelected()){
						_TextBoxMixin.selectInputText(this.textbox);
					}
				}));
				this.own(this._selectOnClickHandle);

				// in case the mouseup never comes
				this.defer(function(){
					if(this._selectOnClickHandle){
						this._selectOnClickHandle.remove();
						this._selectOnClickHandle = null;
					}
				}, 500); // if mouseup not received soon, then treat it as some gesture
			}
			// call this.inherited() before refreshState(), since this.inherited() will possibly scroll the viewport
			// (to scroll the TextBox into view), which will affect how _refreshState() positions the tooltip
			this.inherited(arguments);

			this._refreshState();
		},

		reset: function(){
			// Overrides `dijit/_FormWidget/reset()`.
			// Additionally resets the displayed textbox value to ''
			this.textbox.value = '';
			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		_TextBoxMixin = declare("dijit.form._TextBoxMixin", _TextBoxMixin, {
			_setValueAttr: function(){
				this.inherited(arguments);
				this.applyTextDir(this.focusNode);
			},
			_setDisplayedValueAttr: function(){
				this.inherited(arguments);
				this.applyTextDir(this.focusNode);
			},
			_onInput: function(){
				this.applyTextDir(this.focusNode);
				this.inherited(arguments);
			}
		});
	}

	_TextBoxMixin._setSelectionRange = dijit._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
		if(element.setSelectionRange){
			element.setSelectionRange(start, stop);
		}
	};

	_TextBoxMixin.selectInputText = dijit.selectInputText = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
		// summary:
		//		Select text in the input element argument, from start (default 0), to stop (default end).

		// TODO: use functions in _editor/selection.js?
		element = dom.byId(element);
		if(isNaN(start)){
			start = 0;
		}
		if(isNaN(stop)){
			stop = element.value ? element.value.length : 0;
		}
		try{
			element.focus();
			_TextBoxMixin._setSelectionRange(element, start, stop);
		}catch(e){ /* squelch random errors (esp. on IE) from unexpected focus changes or DOM nodes being hidden */
		}
	};

	return _TextBoxMixin;
});

},
'xcitec/_base/Date':function(){
define([], function () {
/* function for parsing String to Date object according to specific format if available */

Date.parseFormat = function(CurDate, Mask, DefaultTo){

	// Check the input parameters
	if(typeof CurDate != "string" || CurDate == ""){
		return null;
	}
	if(typeof Mask != "string" || Mask == ""){
		return null;
	}
	if(typeof DefaultTo != 0 && DefaultTo != 1 && DefaultTo != 2){
		DefaultTo = 0;
	}

	// Set Mask Characters
	var MaskChars = "DMYhHmsltTz";
	// SetRegEx chars (these need to be escaped in the Mask)
	var RegExChars = "^$.*+?=!:|\\/()[]{}-";
	// Set a reference object for month names
	var MonthRef = {jan:0,feb:1,mar:2,apr:3,may:4,jun:5,jul:6,aug:7,sep:8,oct:9,nov:10,dec:11};
	// Begin the RegEx
	var RegEx = "";
	// Tack a temporary space at the end of the mask to ensure that the last character isn't a mask character
	Mask += " ";

	// Default the positions of the date fragments
	// 0 = year, 1 = month, 2 = day, 3 = hour, 4 = minute, 5 = second, 6 = millisecond, 7 = ampmind, 8 = TimeZone
	var DF = [null,null,null,null,null,null,null, null];

	// Parse the Mask
	var CurChar;
	var MaskPart = "";
	var MaskPartCnt = 1;
	// Loop over the mask, character by character
	for(var Cnt = 0; Cnt < Mask.length; Cnt++){
		// Get the character
		CurChar = Mask.charAt(Cnt);
		// Determine if the character is a mask element
		if(( MaskChars.indexOf(CurChar) == -1 ) || ( MaskPart != "" && CurChar != MaskPart.charAt(MaskPart.length - 1) )){
			// Determine if we need to parse a MaskPart or not
			if(MaskPart != ""){
				// Set the position of the mask part
				switch(MaskPart){
					case "YY":
					case "YYYY":
						DF[0] = MaskPartCnt;
						break;
					case "M":
					case "MM":
					case "MMM":
					case "MMMM":
						DF[1] = MaskPartCnt;
						break;
					case "D":
					case "DD":
					case "DDD":
					case "DDDD":
						DF[2] = MaskPartCnt;
						break;
					case "h":
					case "hh":
					case "H":
					case "HH":
						DF[3] = MaskPartCnt;
						break;
					case "m":
					case "mm":
						DF[4] = MaskPartCnt;
						break;
					case "s":
					case "ss":
						DF[5] = MaskPartCnt;
						break;
					case "l":
						DF[6] = MaskPartCnt;
						break;
					case "t":
					case "T":
					case "tt":
					case "TT":
						DF[7] = MaskPartCnt;
						break;
					case "z":
						DF[8] = MaskPartCnt;
						break;
				}

				// Convert the mask part to a regex fragment
				switch(MaskPart){
					case "h":
						RegEx += "(1[0-2]|[1-9])";
						break;
					case "hh":
						RegEx += "(1[0-2]|0[1-9])";
						break;
					case "H":
						RegEx += "(2[0-4]|1[0-9]|[0-9])";
						break;
					case "HH":
						RegEx += "(2[0-4]|1[0-9]|0[0-9])";
						break;
					case "s":
					case "m":
						RegEx += "([0-5]?[0-9])";
						break;
					case "ss":
					case "mm":
						RegEx += "([0-5]?[0-9])";
						break;
					case "l":
						RegEx += "([0-9]+)";
						break;
					case "t":
					case "T":
						RegEx += "(a|p)";
						break;
					case "tt":
					case "TT":
						RegEx += "(am|pm)";
						break;
					case "D":
						RegEx += "((?:3[01])|(?:[12][0-9])|(?:0[1-9])|[1-9])";
						break;
					case "DD":
						RegEx += "((?:3[01])|(?:[12][0-9])|(?:0[1-9])|[1-9])";
						break;
					case "DDD":
						RegEx += "(sun|mon|tue|wed|thu|fri|sat)";
						break;
					case "DDDD":
						RegEx += "(sunday|monday|tuesday|wednesday|thursday|friday|saturday)";
						break;
					case "M":
						RegEx += "((?:1[012])|(?:0[1-9])|[1-9])";
						break;
					case "MM":
						RegEx += "((?:1[012])|(?:0[1-9])|[1-9])";
						break;
					case "MMM":
						RegEx += "(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)";
						break;
					case "MMMM":
						RegEx += "(january|february|march|april|may|june|july|august|september|october|november|december)";
						break;
					case "YY":
						RegEx += "([0-9]{2})";
						break;
					case "YYYY":
						RegEx += "((?:1[6-9]|[2-9][0-9])[0-9]{2})";
						break;
					case "z":
						RegEx += "(Z|UT|[\+\-](?:1[012]|[0]?[0-9])(?::?[0-5]?[0-9]))";
						break;
				}
				// Reset the MaskPart to nothing
				MaskPart = "";
				// Increment the count of discovered mask parts
				MaskPartCnt++;
			}
			// If the char is a mask char, start a new mask part, otherwise, dump it to the output
			if(MaskChars.indexOf(CurChar) > -1){
				MaskPart = CurChar;
			}else{
				// The character isn't a mask character
				if(RegExChars.indexOf(CurChar) >= 0){
					RegEx += "\\";
				}
				RegEx += CurChar;
			}
		}else{
			// Add the current mask character to the MaskPart
			MaskPart += CurChar;
		}
	}
	// Remove the temporary space from the end of the formatted date
	RegEx = RegEx.substring(0, RegEx.length - 1);

	// Try to parse the input
	var ParsedDF;
	if(ParsedDF = new RegExp("^" + RegEx + "$", "i").exec(CurDate)){
	}
	// If the date couldn't be parsed, return null
	if(!ParsedDF){
		return null
	}
	// Replace the date fragment positions with the actual dates

	for(var Cnt = 0; Cnt < DF.length; Cnt++){
		if(DF[Cnt] != null){
			DF[Cnt] = ParsedDF[DF[Cnt]];
		}
	}

	// Set Month, if present
	if(DF[1] != null){
		var CurDF = MonthRef[DF[1].substring(0, 3).toLowerCase()];
		if(CurDF != null){
			DF[1] = CurDF;
		}else{
			DF[1] = DF[1] - 1;
		}
	}
	// Set AM/PM, if present (Hours must be present and less than 12 for it to matter)
	if(DF[7] != null && DF[3] != null && DF[3] < 13){
		var CurAP = DF[7].substring(0, 1).toLowerCase();
		if(CurAP == "p"){
			DF[3] = DF[3] + 12;
		}
	}

	// Set defaults is current time, set the values
	if(DefaultTo == 0){
		// No changes needed for this default
	}else if(DefaultTo == 1){
		// Set the Current Date Parts
		var NowDate = new Date();
		if(DF[0] == null){
			DF[0] = NowDate.getFullYear()
		}
		if(DF[1] == null){
			DF[1] = NowDate.getMonth()
		}
		if(DF[2] == null){
			DF[2] = NowDate.getDate()
		}
		if(DF[3] == null){
			DF[3] = NowDate.getHours()
		}
		if(DF[4] == null){
			DF[4] = NowDate.getMinutes()
		}
		if(DF[5] == null){
			DF[5] = NowDate.getSeconds()
		}
		if(DF[6] == null){
			DF[6] = NowDate.getMilliseconds()
		}
	}else if(DefaultTo == 2){
		var NowDate = new Date();
		if(DF[0] == null){
			DF[0] = NowDate.getUTCFullYear()
		}
		if(DF[1] == null){
			DF[1] = NowDate.getUTCMonth()
		}
		if(DF[2] == null){
			DF[2] = NowDate.getUTCDate()
		}
		if(DF[3] == null){
			DF[3] = NowDate.getUTCHours()
		}
		if(DF[4] == null){
			DF[4] = NowDate.getUTCMinutes()
		}
		if(DF[5] == null){
			DF[5] = NowDate.getUTCSeconds()
		}
		if(DF[6] == null){
			DF[6] = NowDate.getUTCMilliseconds()
		}
	}

	// If there's no timezone info the data is local time
	if(DF[8] == null){
		return new Date(DF[0], DF[1], DF[2], DF[3], DF[4], DF[5], DF[6]);
	}else{
		// If Timezone indicator is "Z" or "UT", it's UTC, otherwise it's an offset and needs to be figured out
		if(DF[8] == "Z" || DF[8] == "UT"){
			return new Date(Date.UTC(DF[0], DF[1], DF[2], DF[3], DF[4], DF[5], DF[6]));
		}else{
			// Regex value to split the parts
			var ParsedTZ = new RegExp("^([\+\-])(1[012]|[0]?[0-9])(?::?)([0-5]?[0-9])$").exec(DF[8]);
			// Get current Timezone information
			var CurTZ = new Date().getTimezoneOffset();
			var CurTZh = ParsedTZ[1] + ParsedTZ[2] - ((CurTZ >= 0 ? "-" : "+") + Math.floor(Math.abs(CurTZ) / 60));
			var CurTZm = ParsedTZ[1] + ParsedTZ[3] - ((CurTZ >= 0 ? "-" : "+") + (Math.abs(CurTZ) % 60));
			// Return the date
			return new Date(DF[0], DF[1], DF[2], DF[3] - CurTZh, DF[4] - CurTZm, DF[5], DF[6]);
		}
	}
	// If we've reached here we couldn't deal with the input, return null
	return null;

};
  return Date;
});

},
'idx/form/_FocusManager':function(){
define([
	"dijit/focus",
	"dojo/_base/window",
	"dojo/window",
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.extend	
	"dijit/registry"
], function(focus, win, winUtils, dom, domAttr, domClass, declare, lang, registry){
	focus._onTouchNode = function(/*DomNode*/ node, /*String*/ by){
		var srcNode = node;
		if(this._clearActiveWidgetsTimer){
			clearTimeout(this._clearActiveWidgetsTimer);
			delete this._clearActiveWidgetsTimer;
		}
		
		// if the click occurred on the scrollbar of a dropdown, treat it as a click on the dropdown,
		// even though the scrollbar is technically on the popup wrapper (see dojo ticket #10631)
		if(domClass.contains(node, "dijitPopup")){
			node = node.firstChild;
		}
		
		var newStack=[];
		try{
			while(node){
				var popupParent = domAttr.get(node, "dijitPopupParent");
				if(typeof node.dijitPopupParent == "object"){
					node = node.dijitPopupParent;
				}else if(popupParent){
					node = registry.byId(popupParent) ? 
						registry.byId(popupParent).domNode :
						dom.byId(popupParent);
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					if(node === win.body()){
						break;
					}
					node=winUtils.get(node.ownerDocument).frameElement;
				}else{
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && registry.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						if(!widget._isValidFocusNode || widget._isValidFocusNode(srcNode)){
							newStack.unshift(id);
						}
					}
					node=node.parentNode;
				}
			}
		}catch(e){}
		this._setStack(newStack, by);
	}
	return focus;
});

},
'dijit/a11y':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/dom",			// dom.byId
	"dojo/dom-attr", // domAttr.attr domAttr.has
	"dojo/dom-style", // domStyle.style
	"dojo/_base/lang", // lang.mixin()
	"dojo/sniff", // has("ie")  1 
	"./main"	// for exporting methods to dijit namespace
], function(array, dom, domAttr, domStyle, lang, has, dijit){

	// module:
	//		dijit/a11y

	var undefined;

	var a11y = {
		// summary:
		//		Accessibility utility functions (keyboard, tab stops, etc.)

		_isElementShown: function(/*Element*/ elem){
			var s = domStyle.get(elem);
			return (s.visibility != "hidden")
				&& (s.visibility != "collapsed")
				&& (s.display != "none")
				&& (domAttr.get(elem, "type") != "hidden");
		},

		hasDefaultTabStop: function(/*Element*/ elem){
			// summary:
			//		Tests if element is tab-navigable even without an explicit tabIndex setting

			// No explicit tabIndex setting, need to investigate node type
			switch(elem.nodeName.toLowerCase()){
				case "a":
					// An <a> w/out a tabindex is only navigable if it has an href
					return domAttr.has(elem, "href");
				case "area":
				case "button":
				case "input":
				case "object":
				case "select":
				case "textarea":
					// These are navigable by default
					return true;
				case "iframe":
					// If it's an editor <iframe> then it's tab navigable.
					var body;
					try{
						// non-IE
						var contentDocument = elem.contentDocument;
						if("designMode" in contentDocument && contentDocument.designMode == "on"){
							return true;
						}
						body = contentDocument.body;
					}catch(e1){
						// contentWindow.document isn't accessible within IE7/8
						// if the iframe.src points to a foreign url and this
						// page contains an element, that could get focus
						try{
							body = elem.contentWindow.document.body;
						}catch(e2){
							return false;
						}
					}
					return body && (body.contentEditable == 'true' ||
						(body.firstChild && body.firstChild.contentEditable == 'true'));
				default:
					return elem.contentEditable == 'true';
			}
		},

		effectiveTabIndex: function(/*Element*/ elem){
			// summary:
			//		Returns effective tabIndex of an element, either a number, or undefined if element isn't focusable.

			if(domAttr.get(elem, "disabled")){
				return undefined;
			}else if(domAttr.has(elem, "tabIndex")){
				// Explicit tab index setting
				return +domAttr.get(elem, "tabIndex");// + to convert string --> number
			}else{
				// No explicit tabIndex setting, so depends on node type
				return a11y.hasDefaultTabStop(elem) ? 0 : undefined;
			}
		},

		isTabNavigable: function(/*Element*/ elem){
			// summary:
			//		Tests if an element is tab-navigable

			return a11y.effectiveTabIndex(elem) >= 0;
		},

		isFocusable: function(/*Element*/ elem){
			// summary:
			//		Tests if an element is focusable by tabbing to it, or clicking it with the mouse.

			return a11y.effectiveTabIndex(elem) >= -1;
		},

		_getTabNavigable: function(/*DOMNode*/ root){
			// summary:
			//		Finds descendants of the specified root node.
			// description:
			//		Finds the following descendants of the specified root node:
			//
			//		- the first tab-navigable element in document order
			//		  without a tabIndex or with tabIndex="0"
			//		- the last tab-navigable element in document order
			//		  without a tabIndex or with tabIndex="0"
			//		- the first element in document order with the lowest
			//		  positive tabIndex value
			//		- the last element in document order with the highest
			//		  positive tabIndex value
			var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};

			function radioName(node){
				// If this element is part of a radio button group, return the name for that group.
				return node && node.tagName.toLowerCase() == "input" &&
					node.type && node.type.toLowerCase() == "radio" &&
					node.name && node.name.toLowerCase();
			}

			var shown = a11y._isElementShown, effectiveTabIndex = a11y.effectiveTabIndex;
			var walkTree = function(/*DOMNode*/ parent){
				for(var child = parent.firstChild; child; child = child.nextSibling){
					// Skip text elements, hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
					// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
					if(child.nodeType != 1 || (has("ie") <= 9 && child.scopeName !== "HTML") || !shown(child)){
						continue;
					}

					var tabindex = effectiveTabIndex(child);
					if(tabindex >= 0){
						if(tabindex == 0){
							if(!first){
								first = child;
							}
							last = child;
						}else if(tabindex > 0){
							if(!lowest || tabindex < lowestTabindex){
								lowestTabindex = tabindex;
								lowest = child;
							}
							if(!highest || tabindex >= highestTabindex){
								highestTabindex = tabindex;
								highest = child;
							}
						}
						var rn = radioName(child);
						if(domAttr.get(child, "checked") && rn){
							radioSelected[rn] = child;
						}
					}
					if(child.nodeName.toUpperCase() != 'SELECT'){
						walkTree(child);
					}
				}
			};
			if(shown(root)){
				walkTree(root);
			}
			function rs(node){
				// substitute checked radio button for unchecked one, if there is a checked one with the same name.
				return radioSelected[radioName(node)] || node;
			}

			return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
		},

		getFirstInTabbingOrder: function(/*String|DOMNode*/ root, /*Document?*/ doc){
			// summary:
			//		Finds the descendant of the specified root node
			//		that is first in the tabbing order
			var elems = a11y._getTabNavigable(dom.byId(root, doc));
			return elems.lowest ? elems.lowest : elems.first; // DomNode
		},

		getLastInTabbingOrder: function(/*String|DOMNode*/ root, /*Document?*/ doc){
			// summary:
			//		Finds the descendant of the specified root node
			//		that is last in the tabbing order
			var elems = a11y._getTabNavigable(dom.byId(root, doc));
			return elems.last ? elems.last : elems.highest; // DomNode
		}
	};

	 1  && lang.mixin(dijit, a11y);

	return a11y;
});

},
'xcitec/xlayout/ContentAreaPaneV2':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dnd/Source",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/on",
	"xcitec/layout/ContentArea",
  "../xcitecUtil",
  "../layout/ContentAreaPane",
  "./QualificationAvatar",
  "./_ContentAreaPaneItemV2"
], function (array, declare, Source, Manager, domAttr, on, ContentArea, xcitecUtil, ContentAreaPane, QualificationAvatar, _ContentAreaPaneItemV2) {

  return declare( ContentAreaPane, {
	    baseClass: 'ContentAreaPane questionContentArea',
	
	    // is in design mode
	    designMode: false,
	    // show delimiter?
	    showDelimiter: false,
	    // order of elements in column
	    columnOrder: null,
	    // parent element
	    parent: null,
	
	    idHolder: null,
	    
	    postMixInProperties: function() {
	       this.inherited(arguments);
	       this.designMode = xcitecUtil.str2bool(domAttr.get(this.srcNodeRef, 'designMode'));
	    },
	
	    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode) {
	        var controlContainer = new _ContentAreaPaneItemV2({ controlNode:controlNode, labelNode:labelNode,
	                                                                          designMode: this.designMode, parent: this });
	        if (this.designMode) {
	            domAttr.set(controlContainer.domNode, 'dndType', this.id);
	        }
	        return controlContainer.domNode;
	    },
	
	    // updating order of elements in column
	    updateOrder: function() {
	        var elements = this.getChildren();
	        var _this = this;
	        this.columnOrder = [];
	        array.forEach(elements, function(element) {
	            _this.columnOrder.push(domAttr.get(element.getChildren()[0].domNode, 'id'));
	        });
	    },
	
	    // getter for element's order
	    getColumnOrder: function() {
	        return this.columnOrder;
	    },
	
	    // create source for drag&drop
	    setSourceForDnd: function(/*Array*/ sourceIds) {
	        this.source = new Source(this.domNode, {accept: sourceIds});
	        var _this = this;
	        // connect function for updating order of elements in columns to 'onDrop' event
	        on(this.source, "Drop", function() {
	            _this.parent.updateColumnsOrder();
	            _this.parent.resizeHelperNodes();
	        });
	        // connect function for updating initial state
	        on(this.source, "DndStart", function() {
	            _this.parent.updateColumnsOrder();
	            _this.parent.resizeHelperNodes();
	        });
	        Manager.makeAvatar = function() {
	            return new QualificationAvatar(this);
	        };
	    }
	});
});

},
'dijit/form/TextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie") has("mozilla")
	"./_FormValueWidget",
	"./_TextBoxMixin",
	"dojo/text!./templates/TextBox.html",
	"../main"	// to export dijit._setSelectionRange, remove in 2.0
], function(declare, domConstruct, domStyle, kernel, lang, on, has,
			_FormValueWidget, _TextBoxMixin, template, dijit){

	// module:
	//		dijit/form/TextBox

	var TextBox = declare("dijit.form.TextBox" + (has("dojo-bidi") ? "_NoBidi" : ""), [_FormValueWidget, _TextBoxMixin], {
		// summary:
		//		A base class for textbox form inputs

		templateString: template,
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" data-dojo-attach-point="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: has("ie") ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == this.constructor.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(has("ie") < 9){
				// IE INPUT tag fontFamily has to be set directly using STYLE
				// the defer gives IE a chance to render the TextBox and to deal with font inheritance
				this.defer(function(){
					try{
						var s = domStyle.getComputedStyle(this.domNode); // can throw an exception if widget is immediately destroyed
						if(s){
							var ff = s.fontFamily;
							if(ff){
								var inputs = this.domNode.getElementsByTagName("INPUT");
								if(inputs){
									for(var i=0; i < inputs.length; i++){
										inputs[i].style.fontFamily = ff;
									}
								}
							}
						}
					}catch(e){/*when used in a Dialog, and this is called before the dialog is
					 shown, s.fontFamily would trigger "Invalid Argument" error.*/}
				});
			}
		},

		_setPlaceHolderAttr: function(v){
			this._set("placeHolder", v);
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				// dijitInputField class gives placeHolder same padding as the input field
				// parent node already has dijitInputField class but it doesn't affect this <span>
				// since it's position: absolute.
				this._phspan = domConstruct.create('span',{className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
				this.own(
					on(this._phspan, "mousedown", function(evt){ evt.preventDefault(); }),
					on(this._phspan, "touchend, pointerup, MSPointerUp", lang.hitch(this, function(){
						// If the user clicks placeholder rather than the <input>, need programmatic focus.  Normally this
						// is done in _FormWidgetMixin._onFocus() but after [30663] it's done on a delay, which is ineffective.
						this.focus();
					}))
				);
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(this._phspan.ownerDocument.createTextNode(v));
			this._updatePlaceHolder();
		},

		_onInput: function(/*Event*/ evt){
			// summary:
			//		Called AFTER the input event has happened
			//		See if the placeHolder text should be removed or added while editing.
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display = (this.placeHolder && !this.textbox.value) ? "" : "none";
			}
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.  Use get('displayedValue') instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use get('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		setDisplayedValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();

			if(has("mozilla")){
				if(this.selectOnClick){
					// clear selection so that the next mouse click doesn't reselect
					this.textbox.selectionStart = this.textbox.selectionEnd = undefined;
				}
			}
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();
		}
	});

	if(has("ie") < 9){
		TextBox.prototype._isTextSelected = function(){
			var range = this.ownerDocument.selection.createRange();
			var parent = range.parentElement();
			return parent == this.textbox && range.text.length > 0;
		};

		// Overrides definition of _setSelectionRange from _TextBoxMixin (TODO: move to _TextBoxMixin.js?)
		dijit._setSelectionRange = _TextBoxMixin._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
			if(element.createTextRange){
				var r = element.createTextRange();
				r.collapse(true);
				r.moveStart("character", -99999); // move to 0
				r.moveStart("character", start); // delta from 0 is the correct position
				r.moveEnd("character", stop-start);
				r.select();
			}
		}
	}

	if(has("dojo-bidi")){
		TextBox = declare("dijit.form.TextBox", TextBox, {
			_setPlaceHolderAttr: function(v){
				this.inherited(arguments);
				this.applyTextDir(this._phspan);
			}
		});
	}

	return TextBox;
});

},
'xcitec/page/ObjectDivider':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "../Title"
], function (declare, domClass, _Templated, _Widget, Title) {

	var ObjectDivider = declare( [_Widget, _Templated], {
	    templateString: '<div>' +
	                        '<div class="shadow"></div>' +
	                        '<div class="title_wrap" data-dojo-attach-point="separator, containerNode"></div>' +
	                    '</div>',
	
	    caption: "Object information",
	    entityClass: "rating",
	
	    postCreate: function() {
	        var separator = new Title( {text: this.caption, entityClass: this.entityClass} );
	        domClass.add(separator.domNode, 'iconObjectInformation');
	        this.separator.appendChild(separator.domNode);
	        this.inherited(arguments);
	    }
	});
	return ObjectDivider;
});

},
'dijit/form/_RadioButtonMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/lang", // lang.hitch
	"dojo/query", // query
	"../registry"    // registry.getEnclosingWidget
], function(array, declare, domAttr, lang, query, registry){

	// module:
	//		dijit/form/_RadioButtonMixin

	return declare("dijit.form._RadioButtonMixin", null, {
		// summary:
		//		Mixin to provide widget functionality for an HTML radio button

		// type: [private] String
		//		type attribute on `<input>` node.
		//		Users should not change this value.
		type: "radio",

		_getRelatedWidgets: function(){
			// Private function needed to help iterate over all radio buttons in a group.
			var ary = [];
			query("input[type=radio]", this.focusNode.form || this.ownerDocument).forEach(// can't use name= since query doesn't support [] in the name
				lang.hitch(this, function(inputNode){
					if(inputNode.name == this.name && inputNode.form == this.focusNode.form){
						var widget = registry.getEnclosingWidget(inputNode);
						if(widget){
							ary.push(widget);
						}
					}
				})
			);
			return ary;
		},

		_setCheckedAttr: function(/*Boolean*/ value){
			// If I am being checked then have to deselect currently checked radio button
			this.inherited(arguments);
			if(!this._created){
				return;
			}
			if(value){
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					if(widget != this && widget.checked){
						widget.set('checked', false);
					}
				}));
			}
		},

		_getSubmitValue: function(/*String*/ value){
			return value == null ? "on" : value;
		},

		_onClick: function(/*Event*/ e){
			if(this.checked || this.disabled){ // nothing to do
				e.stopPropagation();
				e.preventDefault();
				return false;
			}
			if(this.readOnly){ // ignored by some browsers so we have to resync the DOM elements with widget values
				e.stopPropagation();
				e.preventDefault();
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					domAttr.set(this.focusNode || this.domNode, 'checked', widget.checked);
				}));
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'xcitec/AboutDialog':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/on",
	"dojo/text!xcitec/templates/AboutDialog.html",
	"dijit/Dialog"
], function (declare, lang, on, template, Dialog) {

	var AboutDialog = declare( Dialog, {
		templateString: template,
		draggable: false,
	
		licenseInfo: "Licensed Materials - Property of IBM Corp. &copy; Copyright IBM Corporation&trade; and its licensors&reg; 2011-2013.  All Rights Reserved. IBM, the IBM logo and IBM Emptoris Supplier Lifecycle Management are trademarks of IBM Corporation in the United States, other countries, or both. Java and all Java-based trademarks and logos are trademarks or registered trademarks of Oracle and/or its affiliates. Other company, product or service names may be trademarks or service marks of others.",
		version: "",
		fullVersion: "",
	
		postCreate: function(){
			this.inherited(arguments);
			var underlay = null;
			if(dijit._underlay === undefined){
				dijit._underlay = new dijit.DialogUnderlay();
			}
			underlay = dijit._underlay.domNode;
			on(underlay, "click", lang.hitch(this, function(e){
				this._onClick();
			}));
		},

		show: function(){
			// summary:
			//	@Override - dijit/Dialog.
			var promise = this.inherited(arguments);
			this._modalconnects.splice(0, 1)[0].remove(); //fix to scroll dialog (dialog position not fixed)
			return promise;
		},
	
		_onClick: function(e){
			this.hide(true);
		},
	
		destroy: function() {
			if(dijit._underlay) {
				dijit._underlay.destroy();
			}
			this.inherited(arguments);
		}
	});
	return AboutDialog;
});
},
'slmui/main':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["require","dojo/_base/lang","dojo/_base/kernel"], 
	   function(dRequire,dLang,dKernel) {
  /**
   * @name xcitec.main
   * @namespace This serves as the main module for the "slmui" package when using "packages" in the "dojo config".
   */
	var iMain = dLang.getObject("slmui", true);
	
	return iMain;
});
},
'slmui/form/RadioButtonSet':function(){
define([
	"dojo/_base/declare", // declare
	"xcitec/form/RadioButton",
	"dijit/form/_FormValueWidget",
	"dijit/_Container",
	"dijit/registry",
	"dojo/dom-construct",
	"dojo/_base/array",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/query",
	"dojo/on",
	"dojo/_base/lang",
	"dojo/dom",
	"dojo/dom-class",
	"dojo/text!slmui/form/templates/RadioButton.html",
	"dojo/text!slmui/form/templates/RadioButtonSet.html",
	"dijit/Viewport",
	"dojo/has",
	"dojo/touch",
	"dojo/dom-geometry",
	"dojo/dom-style"
], function(declare, RadioButton, _FormValueWidget, _Container, registry, domConstruct, array, windowModule, domAttr, query, on, lang, dom, domClass, templateRadioButton, templateRadioButtonSet, Viewport, has, touch, domGeom, style){

	// module:
	//		slmui/form/RadioButtonSet

	var RadioGroup = declare([_FormValueWidget, _Container], {

		// summary:
		//			A vertical radio button group with colored containers for radio buttons.

		// values of each item (Array)
		values: null,
		// short text of each item (Array)
		labels: null,
		// long text of each item (Array)
		descriptions: null,
		// index of currently selected item
		selectedValue: 0,
		// array of item's widgets
//		items: null,
		// widget name
		name: "",
		// state attributes
		readOnly: false,
		disabled: false,
		error: "",
//		required: false,
		// if items exist in srcNodeRef
		empty: false,

		selIndex: -1,
		tabIndex: "-1",

		_buttons: null,
		templateString: templateRadioButtonSet,

		postMixInProperties: function(){
			// summary:
			//			 Parse source and init of the RadioButtonSet
			// tags:
			//			extension
			this.inherited(arguments);
			this._parseSource();
			this._initOptions();
		},

		_initOptions: function(){
			// summary:
			//			Prepare init data
			var opts = [];

			this.selIndex = this.selectedValue;
			for(var i = 0; i < this.values.length; i++){
				var selected;
				if(this.selIndex - 1 == i){
					selected = true;
				}else{
					selected = false;
				}
				opts.push({
					values: this.values[i],
					caption: this.labels[i],
					label: this.descriptions[i],
					checked: selected,
					disabled: this.disabled,
					icon: "",
					captionId: ""
				});
			}

			this.options = opts;
		},

		buildRendering: function(){
			// summary:
			//			Building the DOM model.
			// tags:
			//			extension
			this._buttons = [];
			domConstruct.empty(this.srcNodeRef);
			this.inherited(arguments);
			this._createRadioGroup(this.options);
			dom.setSelectable(this.domNode, false);
		},

		_parseSource: function(){
			// summary:
			//			Parses the HTML source.
			// tags:
			//			protected
			var srcNodeName = (has('ie') == 8) ? this.srcNodeRef.nodeName.toLowerCase() : this.srcNodeRef.localName;
			var component = (srcNodeName=="div")?"div":"option";
			var boxes = query("> " + component, this.srcNodeRef);
			if(boxes.length == 0){
				this.empty = true;
			}
			var _this = this;
			this.values = boxes.map(function(node){
				return domAttr.get(node, "value");
			});
			this.labels = boxes.map(function(node){
				return domAttr.get(node, "label");
			});
			this.descriptions = boxes.map(function(node){
				return node.innerHTML;
			});
			boxes.forEach(function(node, index){
				if(domAttr.get(node, "selected") == "true"){
					_this.selectedValue = index + 1;
				}
			})
		},

		_createRadioGroup: function(otpions){
			// summary:
			//			Creates radiobuttons and puts into DOM model.
			// tags:
			//			private
			var self = this;
			var ind = 0;

			array.forEach(otpions, function(node, index){
				var text = "";
				text = node.label;

				// create RadioButtons according set of options
				var radioButton = new BlockRadioButton({
					name: self.name,
					checked: node.checked,
					disabled: self.disabled,
					readOnly: self.readOnly,
					caption: text,
					captionId: node.captionId,
					icon: node.icon,
					value: node.values,
					index: ind,
					parent: self,
					title: node.caption
				});
				if(index == (self.values.length - 1)){
					domClass.add(radioButton.domNode, "rightRadioButton");
				}
				if(index == 0){
					domClass.add(radioButton.domNode, "leftRadioButton");
				}
				self._buttons[ind] = radioButton;
				ind++;
				self.radioNode.appendChild(radioButton.domNode);
				radioButton.domNode.onchange = "";
				on(radioButton.domNode, "click", lang.hitch(radioButton, self.changeState));
			});


			var leftLabel = this.labels[0];
			var rightLabel = this.labels[this.labels.length - 1];
			var leftTickMark = domConstruct.create("div", {innerHTML: leftLabel, style: "width: 50% !important; overflow: hidden; float: left; text-overflow: ellipsis;"});
			this.tickMarks.appendChild(leftTickMark);
			if(otpions.length >= 2){
				this.tickMarks.appendChild(domConstruct.create("div", {innerHTML: rightLabel, style: "overflow: hidden; text-overflow: ellipsis; text-align: right;"}));
			}
			this.tickMarks.style.width = this.radioNode.scrollWidth + 'px';
		},

		startup: function(){
			// summary:
			// 			Start listening the viewport resize event
			// tags:
			//			extension
			this.inherited(arguments);
			this.own(Viewport.on("resize", lang.hitch(this, "resize")));
			this.resize();
		},

		resize: function(){

			if(this.options.length < 1){return;}
			//summary:
			//			Recalculates the width of the buttons to set the equal distance between all the buttons.

//			this.radioNode.style.overflowX='hidden';
			var minMargin = 10/2;
			var maxMargin = 30/2;

			var leftButtonNode = query(".leftRadioButton", this.domNode)[0];
			var rightButtonNode = query(".rightRadioButton", this.domNode)[0];
			var middleButtons = query("div.dijitXcitecRadio:not(.leftRadioButton):not(.rightRadioButton)", this.domNode)
			var n = this.options.length;

			var totalWidth = this.domNode.parentNode.offsetWidth;

            var radioButtonWidth = 34;
			var visibleWidth = radioButtonWidth * n + minMargin * (n-1);

            var halfPiece = minMargin;
            if(visibleWidth < totalWidth) {
                halfPiece = (totalWidth - radioButtonWidth * n - minMargin - 3 ) / (2 * n - 2);
                if (halfPiece > maxMargin) {
                    halfPiece = maxMargin;
                }
            }

			leftButtonNode.style.marginRight = halfPiece + 'px';
			rightButtonNode.style.marginLeft = halfPiece + 'px';

			for (var i = 0; i<middleButtons.length; i++) {
				middleButtons[i].style.marginRight = halfPiece + 'px';
				middleButtons[i].style.marginLeft = halfPiece + 'px';
			}

//			this.radioNode.style.overflowX='auto';
			var scrollingWidth = this.radioNode.scrollWidth;

			if(scrollingWidth == 0) scrollingWidth = visibleWidth;

			this.tickMarks.style.width = scrollingWidth + 'px';
		},

		postCreate: function() {
			this.inherited(arguments);

			if(has('ie') == 9){ // SRM-11209: refresh nodeTableContainer to prevent its height surge
				var controlHeight = this.overflowContainer.offsetHeight;
				var that = this;
				touch.move(this.domNode.parentNode.parentNode.parentNode, function(){
					if(that.overflowContainer.offsetHeight != controlHeight){
						that.overflowContainer.style.display = 'none';
						that.overflowContainer.style.display = 'block';
					}
				});
			}
		},

		changeState: function(){
			// summary:
			//			changes the state of radiobutton set, if new radio button was selected
			if(this.readOnly || this.disabled){
				return;
			}
			var widget = registry.byNode(this.domNode);
			widget.focus();
			if(widget.index != widget.parent.selIndex){
				if(widget.parent.selIndex > -1){
					var prev = widget.parent._buttons[widget.parent.selIndex];
					prev.set('checked', false);
				}
				widget.parent.selIndex = widget.index;
				widget.parent._onChange();
			}
		},

		_onChange: function(e){
			// summary:
			// 			user defined function
			this.onchange();
		},

		onchange: function(){
			// summary:
			// 			for wicket use in template
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// tags:
			//		extension protected
			this._set('disabled', value);
		}
	});

	var BlockRadioButton = declare(RadioButton, {

		// summary:
		//			Extended xcitec/form/RadioButoon. Now it has an colored block as a background.
		templateString: templateRadioButton,

		_setCheckedAttr: function(/*Boolean*/ value){

			// summary:
			//			Colors the checked radio button container
			//			and uncolors the unchecked one
			// tags:
			//			extension protected
			this.inherited(arguments);
			if(value){
				domClass.add(this.borderNode, "checkedRadioButtonContainer");
			}else{
				domClass.remove(this.borderNode, "checkedRadioButtonContainer");
			}
		}
	});


	return RadioGroup;

});
},
'dijit/form/_FormWidgetMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-style", // domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/mouse", // mouse.isLeft
	"dojo/on",
	"dojo/sniff", // has("webkit")
	"dojo/window", // winUtils.scrollIntoView
	"../a11y"    // a11y.hasDefaultTabStop
], function(array, declare, domAttr, domStyle, lang, mouse, on, has, winUtils, a11y){

	// module:
	//		dijit/form/_FormWidgetMixin

	return declare("dijit.form._FormWidgetMixin", null, {
		// summary:
		//		Mixin for widgets corresponding to native HTML elements such as `<checkbox>` or `<button>`,
		//		which can be children of a `<form>` node or a `dijit/form/Form` widget.
		//
		// description:
		//		Represents a single HTML element.
		//		All these widgets should have these attributes just like native HTML input elements.
		//		You can set them during widget construction or afterwards, via `dijit/_WidgetBase.set()`.
		//
		//		They also share some common methods.

		// name: [const] String
		//		Name used when submitting form; same as "name" attribute or plain HTML elements
		name: "",

		// alt: String
		//		Corresponds to the native HTML `<input>` element's attribute.
		alt: "",

		// value: String
		//		Corresponds to the native HTML `<input>` element's attribute.
		value: "",

		// type: [const] String
		//		Corresponds to the native HTML `<input>` element's attribute.
		type: "text",

		// type: String
		//		Apply aria-label in markup to the widget's focusNode
		"aria-label": "focusNode",

		// tabIndex: String
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",
		_setTabIndexAttr: "focusNode", // force copy even when tabIndex default value, needed since Button is <span>

		// disabled: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "disabled='disabled'", or just "disabled".
		disabled: false,

		// intermediateChanges: Boolean
		//		Fires onChange for each value change or only on demand
		intermediateChanges: false,

		// scrollOnFocus: Boolean
		//		On focus, should this widget scroll into view?
		scrollOnFocus: true,

		// Override _WidgetBase mapping id to this.domNode, needs to be on focusNode so <label> etc.
		// works with screen reader
		_setIdAttr: "focusNode",

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set("disabled", value);
			domAttr.set(this.focusNode, 'disabled', value);
			if(this.valueNode){
				domAttr.set(this.valueNode, 'disabled', value);
			}
			this.focusNode.setAttribute("aria-disabled", value ? "true" : "false");

			if(value){
				// reset these, because after the domNode is disabled, we can no longer receive
				// mouse related events, see #4200
				this._set("hovering", false);
				this._set("active", false);

				// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
				var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex :
					("_setTabIndexAttr" in this) ? this._setTabIndexAttr : "focusNode";
				array.forEach(lang.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
					var node = this[attachPointName];
					// complex code because tabIndex=-1 on a <div> doesn't work on FF
					if(has("webkit") || a11y.hasDefaultTabStop(node)){    // see #11064 about webkit bug
						node.setAttribute('tabIndex', "-1");
					}else{
						node.removeAttribute('tabIndex');
					}
				}, this);
			}else{
				if(this.tabIndex != ""){
					this.set('tabIndex', this.tabIndex);
				}
			}
		},

		_onFocus: function(/*String*/ by){
			// If user clicks on the widget, even if the mouse is released outside of it,
			// this widget's focusNode should get focus (to mimic native browser behavior).
			// Browsers often need help to make sure the focus via mouse actually gets to the focusNode.
			// TODO: consider removing all of this for 2.0 or sooner, see #16622 etc.
			if(by == "mouse" && this.isFocusable()){
				// IE exhibits strange scrolling behavior when refocusing a node so only do it when !focused.
				var focusHandle = this.own(on(this.focusNode, "focus", function(){
					mouseUpHandle.remove();
					focusHandle.remove();
				}))[0];
				// Set a global event to handle mouseup, so it fires properly
				// even if the cursor leaves this.domNode before the mouse up event.
				var event = has("pointer-events") ? "pointerup" : has("MSPointer") ? "MSPointerUp" :
					has("touch-events") ? "touchend, mouseup" :		// seems like overkill but see #16622, #16725
					"mouseup";
				var mouseUpHandle = this.own(on(this.ownerDocumentBody, event, lang.hitch(this, function(evt){
					mouseUpHandle.remove();
					focusHandle.remove();
					// if here, then the mousedown did not focus the focusNode as the default action
					if(this.focused){
						if(evt.type == "touchend"){
							this.defer("focus"); // native focus hasn't occurred yet
						}else{
							this.focus(); // native focus already occurred on mousedown
						}
					}
				})))[0];
			}
			if(this.scrollOnFocus){
				this.defer(function(){
					winUtils.scrollIntoView(this.domNode);
				}); // without defer, the input caret position can change on mouse click
			}
			this.inherited(arguments);
		},

		isFocusable: function(){
			// summary:
			//		Tells if this widget is focusable or not.  Used internally by dijit.
			// tags:
			//		protected
			return !this.disabled && this.focusNode && (domStyle.get(this.domNode, "display") != "none");
		},

		focus: function(){
			// summary:
			//		Put focus on this widget
			if(!this.disabled && this.focusNode.focus){
				try{
					this.focusNode.focus();
				}catch(e){
				}
				/*squelch errors from hidden nodes*/
			}
		},

		compare: function(/*anything*/ val1, /*anything*/ val2){
			// summary:
			//		Compare 2 values (as returned by get('value') for this widget).
			// tags:
			//		protected
			if(typeof val1 == "number" && typeof val2 == "number"){
				return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
			}else if(val1 > val2){
				return 1;
			}else if(val1 < val2){
				return -1;
			}else{
				return 0;
			}
		},

		onChange: function(/*===== newValue =====*/){
			// summary:
			//		Callback when this widget's value is changed.
			// tags:
			//		callback
		},

		// _onChangeActive: [private] Boolean
		//		Indicates that changes to the value should call onChange() callback.
		//		This is false during widget initialization, to avoid calling onChange()
		//		when the initial value is set.
		_onChangeActive: false,

		_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Called when the value of the widget is set.  Calls onChange() if appropriate
			// newValue:
			//		the new value
			// priorityChange:
			//		For a slider, for example, dragging the slider is priorityChange==false,
			//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
			//		onChange is only called form priorityChange=true events.
			// tags:
			//		private
			if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
				// this block executes not for a change, but during initialization,
				// and is used to store away the original value (or for ToggleButton, the original checked state)
				this._resetValue = this._lastValueReported = newValue;
			}
			this._pendingOnChange = this._pendingOnChange
				|| (typeof newValue != typeof this._lastValueReported)
				|| (this.compare(newValue, this._lastValueReported) != 0);
			if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
				this._lastValueReported = newValue;
				this._pendingOnChange = false;
				if(this._onChangeActive){
					if(this._onChangeHandle){
						this._onChangeHandle.remove();
					}
					// defer allows hidden value processing to run and
					// also the onChange handler can safely adjust focus, etc
					this._onChangeHandle = this.defer(
						function(){
							this._onChangeHandle = null;
							this.onChange(newValue);
						}); // try to collapse multiple onChange's fired faster than can be processed
				}
			}
		},

		create: function(){
			// Overrides _Widget.create()
			this.inherited(arguments);
			this._onChangeActive = true;
		},

		destroy: function(){
			if(this._onChangeHandle){ // destroy called before last onChange has fired
				this._onChangeHandle.remove();
				this.onChange(this._lastValueReported);
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/_base/place':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/lang", // lang.isArray, lang.mixin
	"dojo/window", // windowUtils.getBox
	"../place",
	"../main"	// export to dijit namespace
], function(array, lang, windowUtils, place, dijit){

	// module:
	//		dijit/_base/place


	var exports = {
		// summary:
		//		Deprecated back compatibility module, new code should use dijit/place directly instead of using this module.
	};

	exports.getViewport = function(){
		// summary:
		//		Deprecated method to return the dimensions and scroll position of the viewable area of a browser window.
		//		New code should use windowUtils.getBox()

		return windowUtils.getBox();
	};

	exports.placeOnScreen = place.at;

	exports.placeOnScreenAroundElement = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
		//		for the "around" argument and finds a proper processor to place a node.
		//		Deprecated, new code should use dijit/place.around() instead.

		// Convert old style {"BL": "TL", "BR": "TR"} type argument
		// to style needed by dijit.place code:
		//		[
		//			{aroundCorner: "BL", corner: "TL" },
		//			{aroundCorner: "BR", corner: "TR" }
		//		]
		var positions;
		if(lang.isArray(aroundCorners)){
			positions = aroundCorners;
		}else{
			positions = [];
			for(var key in aroundCorners){
				positions.push({aroundCorner: key, corner: aroundCorners[key]});
			}
		}

		return place.around(node, aroundNode, positions, true, layoutNode);
	};

	exports.placeOnScreenAroundNode = exports.placeOnScreenAroundElement;
	/*=====
	exports.placeOnScreenAroundNode = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Position node adjacent or kitty-corner to aroundNode
		//		such that it's fully visible in viewport.
		//		Deprecated, new code should use dijit/place.around() instead.
	};
	=====*/

	exports.placeOnScreenAroundRectangle = exports.placeOnScreenAroundElement;
	/*=====
	exports.placeOnScreenAroundRectangle = function(node, aroundRect, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except that the "around"
		//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
		//		instead of a dom node.
		//		Deprecated, new code should use dijit/place.around() instead.
	};
	=====*/

	exports.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
		// summary:
		//		Deprecated method, unneeded when using dijit/place directly.
		//		Transforms the passed array of preferred positions into a format suitable for
		//		passing as the aroundCorners argument to dijit/place.placeOnScreenAroundElement.
		// position: String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//		- before: places drop down to the left of the target node/widget, or to the right in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- after: places drop down to the right of the target node/widget, or to the left in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- above: drop down goes above target node
		//		- below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		// leftToRight: Boolean
		//		Whether the popup will be displaying in leftToRight mode.

		var align = {};
		array.forEach(position, function(pos){
			var ltr = leftToRight;
			switch(pos){
				case "after":
					align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
					break;
				case "before":
					align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
					break;
				case "below-alt":
					ltr = !ltr;
					// fall through
				case "below":
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "BL" : "BR"] = ltr ? "TL" : "TR";
					align[ltr ? "BR" : "BL"] = ltr ? "TR" : "TL";
					break;
				case "above-alt":
					ltr = !ltr;
					// fall through
				case "above":
				default:
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "TL" : "TR"] = ltr ? "BL" : "BR";
					align[ltr ? "TR" : "TL"] = ltr ? "BR" : "BL";
					break;
			}
		});
		return align;
	};

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'idx/main':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["require","dojo/_base/lang","dojo/_base/kernel","dojo/_base/window","dojo/dom-class"], 
	   function(dRequire,dLang,dKernel,dWindow,dDomClass) {
  /**
   * @name idx.main
   * @namespace This serves as the main module for the "idx" package when using "packages" in the "dojo config".
   *            This module's only action is to apply a CSS class to the body tag that identifies the Dojo version
   *            being utilized on the page. 
   */
	var iMain = dLang.getObject("idx", true);
	
	var majorVersion = dKernel.version.major;
	var minorVersion = dKernel.version.minor;
	
	var applyExtarClass = function(){
		// apply dojo version class, "idx_dojo_1.x"
		var bodyNode = dWindow.body();
		var versionClass = "idx_dojo_" + dKernel.version.major + "_" + dKernel.version.minor;
		dDomClass.add(bodyNode, versionClass);	
		
		// apply bidi class for hebrew
		if(dojo.locale == "he"){
			dDomClass.add(bodyNode, "idx_i18n_il");
		}
	}
	
	
	if ((majorVersion < 2) && (minorVersion < 7)) {
		// for dojo 1.6 we need to use "addOnLoad" to ensure the body exists first
		dojo.addOnLoad(applyExtarClass);
	} else {
		// for dojo 1.7 or later we rely on the "dojo/domReady!" dependency
		dRequire(["dojo/domReady!"],applyExtarClass);
	}
	
	return iMain;
});
},
'xcitec/xlayout/QualificationLayoutSingleColumn':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"xcitec/xlayout/ContentAreaPaneV2SingleColumn",
	"xcitec/xlayout/QualificationLayoutBase"
], function (array, declare, domClass, domConstruct, query, ContentAreaPaneV2SingleColumn, QualificationLayoutBase) {

	/* xcitec.xlayout.QualificationLayout is a layout control which is used for placing
	   questions in qualification in appropritae way. Can be in design and view mode. Allow
	   drag&drop between columns and in column. Has a hidden input field to store the order
	   of elements in a columns
	    Example:
	        <div data-dojo-type="xcitec.xlayout.QualificationLayout" designMode="true" id="ql1">
	            <input id="ql1.order" name="ql1.order" type="hidden" value="">
	            <div data-dojo-type="xcitec.xlayout.StyleElement" elementType='headline' caption='Personal data'></div>
	            <div data-dojo-type="xcitec.xlayout.QuestionV2" labelId="supplier0.supplierid">
	                <select label="Salutation" data-dojo-type="xcitec.form.DropDownSelect" widthInPercent="50">
	                    <option selected="true" value="a">Mr.</option>
	                    <option selected="false" value="b">Ms.</option>
	                </select>
	            </div>
	            <div data-dojo-type="xcitec.xlayout.QuestionV2" labelId="supplier0.supplierid">
	                <input type="text" data-dojo-type="xcitec.form.TextBox"
	                       name="supplier0.supplierid.suppliercode0"
	                       label="First name"
	                       labelId="supplier0.supplierid.suppliercode0"
	                       value="">
	            </div>
	        </div>
	*/
	
	var QualificationLayoutSingleColumn = declare( QualificationLayoutBase, { 
	    
	    isSingleColumn: true,
	
	    _fillContent: function() {
	        this._startupWidgets = this._startupWidgets || [];
	
	        var children = query(">", this.srcNodeRef);
	
	        // create markup for each column
	        var contentAreaPane1 = domConstruct.create("div", {'data-dojo-type': 'xcitec/xlayout/ContentAreaPaneV2SingleColumn', designMode: this.designMode});
	        array.forEach(children, function(node) {
	            if (domClass.contains(node, "delimeter")) {
	                domConstruct.destroy(node);
	            } else {
	                contentAreaPane1.appendChild(node);
	            }
	        });
	
	        // actually crerate column's objects
	        this.pane1 = new ContentAreaPaneV2SingleColumn({columnOrder: this.firstColumnOrder, parent: this, idHolder: this.clicked }, contentAreaPane1);
	        this._startupWidgets.push(this.pane1);
	
	        this.containerNode.appendChild(this.pane1.domNode);
	        this.inherited(arguments);  
	    },
	
	    // update order of elements in each column
	    updateColumnsOrder: function() {
	        this.pane1.updateOrder();
	        this.updateHiddenField(this.pane1.getColumnOrder());
	    },
	
	    resizeHelperNodes: function() {
	        // do nothig, we have no helper nodes here
	    },
	
	    // update value of hidden input
	    updateHiddenField: function(firstColumn) {
	        var _this = this;
	        this.order.value = '';
	        array.forEach(firstColumn, function(id) {
	            _this.order.value = _this.order.value + id + ",";
	        });
	        this.order.value = this.order.value.substring(0, this.order.value.length - 1);
	    },
	
	    startup: function() {
	        this.inherited(arguments);
	        if (this.designMode) {
	            domConstruct.place(this.order, this.domNode, 'first');
	            if (this.clicked) {
	                domConstruct.place(this.clicked, this.domNode, 'first');
	            }
	            // makes the panes accept only appropriate elements
	            this.pane1.setSourceForDnd([this.pane1.id]);
	            this.updateColumnsOrder();
	        }
	    }
	});
	return QualificationLayoutSingleColumn;
});

},
'slmui/layout/_XcitecCompatibleLayoutMixin':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/lang',
	'dojo/_base/array',
	'dojo/query',
	'dojo/dom-construct',
	'dojo/parser',
	'dijit/registry'
], function(declare, lang, array, query, domConstruct, parser, registry){
	return declare(null, {

		// summary:
		//		Mixin for backward compatibility with Xcitec widgets

		buildRendering: function(){
			this.inherited(arguments);
			this._processInnerControls();
		},

		startup: function(){
			this.inherited(arguments);


			array.forEach(registry.findWidgets(this.containerNode), function(widget, index){
				if(!widget.hasOwnLabelProcessing){
					this.addChild(this.buildControlHolder(widget.domNode), index);
				}
			}, this);
		},

		_processInnerControls: function(){
			query('[inner]', this.containerNode).forEach(function(node){
				var parent = node.parentNode;
				if(parent && parent != this.containerNode){
					domConstruct.place(node, parent, 'after');
				}
			}, this);
		},

		ajaxReplaceControl: function(/*WidgetDomNode*/ controlNode, targetNode){
			var parent = registry.getEnclosingWidget(targetNode.parentNode);
			if(parent && parent.isLabelControlHolder){
				var prevSibling = parent.domNode.previousSibling,
					nextSibling = parent.domNode.nextSibling;
				if(parent.destroyRecursive){
					parent.destroyRecursive();
				} else if(parent.destroy){
					parent.destroy();
				}
				var tmpContainer = domConstruct.create('div');
				domConstruct.place(controlNode, tmpContainer);
				parser.parse(tmpContainer);
				var control = registry.findWidgets(tmpContainer)[0];
				var controlHolder = this.buildControlHolder(control.domNode);
				if(prevSibling){
					domConstruct.place(controlHolder.domNode, prevSibling, "after");
				}else if(nextSibling){
					domConstruct.place(controlHolder.domNode, nextSibling, "before");
				}else{
					domConstruct.place(controlHolder.domNode, this.containerNode);
				}
				if(lang.isFunction(controlHolder.startup) && !controlHolder._started){
					controlHolder.startup();
				}
			}
			this.layout();
		},

		buildControlHolder: function(controlNode){
			// summary:
			//		Deprecated. Will be used util full migration to IDX.
			// tags:
			//		deprecated protected extension
		}
	});
});
},
'xcitec/form/Upload':function(){
define([
	"dojo/dom-style",
	"dojo/dom-construct",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-class",
	"dojo/_base/window",
	"dojo/_base/declare",
	"dojo/text!./templates/UploadDialog.html",
    "dijit/ProgressBar",
	"dijit/registry",
  "dijit/_WidgetsInTemplateMixin",
	"./Button",
	"./EntitySelect",
	"../xcitecDialog"
], function (domStyle, domConstruct, array, lang, domClass, windowModule, declare, template, ProgressBar, registry, _WidgetsInTemplateMixin, Button, EntitySelect, xcitecDialog) {

	var Upload = declare( EntitySelect, {
	    // instance of upload dialog
	    myDialog: null,
	
	    // url for
	    callbackUrl: "",
	
	    postCreate: function(){
	        this.inherited(arguments);
	
			//hide infobutton
		    if(this.infoButton){
		      domStyle.set(this.infoButton.domNode, "display", "none");
		    }
	
	        // create dialog and attach it to document body
	        this.myDialog = new UploadDialog({ title: "Upload file dialog", parent: this });
	        domConstruct.place(this.myDialog.domNode, windowModule.body(), "last");
	    },
		
		_createTextNode: function(){
            this.valueNode.value = this.textbox.value;
            this.textNode.appendChild(windowModule.doc.createTextNode(this.textbox.value.split("|")[0]));
        },
	
	    onChoose: function(){
	        if(this.myDialog){
	            // show dialog, clear filt input and set progress bar position to "0"
	            this.myDialog.show();
	            this.myDialog.bar.update({ progress: "0" });
	            this.myDialog.file.value = "";
	        }
	    },
	
		destroy: function() {
			if(this.myDialog) {
				this.myDialog.destroy();
			}
			this.inherited(arguments);
		}
	});
	
	var UploadDialog = declare([xcitecDialog, _WidgetsInTemplateMixin], {
	    templateString: template,
	    // paretnt widget
	    parent: null,
	    widgetsInTemplate: true,
	
	    postCreate: function() {
	        this.inherited(arguments);
	        var _this = this;
	        var button = new Button({caption: "Cancel", onClick: lang.hitch(_this, "_hide")});
	        domStyle.set(this.buttonsBarNode, "display", "");
	        this.buttonsBarNode.appendChild(button.domNode);
	    },
	
		_hide: function(){
			domClass.remove(this.bytesRead, "uploadDialogErrorLabel");
			this.bytesRead.innerHTML = '';
			this.bytesRead.style.visibility = 'hidden';
	        this.xcitecHide();
		},
	
	    URLDecode: function(encodedString) {
	        // Create a regular expression to search all +s in the string
	        var lsRegExp = /\+/g;
	        // Return the decoded string
	        return unescape(String(encodedString).replace(lsRegExp, " "));
	    },
	
	    shiftFunction: function() {
	        window.setTimeout(lang.hitch(this, "ajaxFunction"), 300);
	    },
	
	    ajaxFunction: function() {
	        if (this.file.value == "") {
	            return;
	        }
	        var _this = this;
	        var xhrArgs = {
	            url: "../servlet/FileUploadServlet",
	            handleAs: "xml",
	            preventCache: true,
	            load: function(data) {
	                _this.processStateChange(data);
	            },
	            error: function(error) {
	                console.log(error);
	            }
	        };
	
	        //Call the asynchronous xhrGet
	        dojo.xhrGet(xhrArgs);
	    },
	
	    processStateChange: function(xml) {
	        // No need to iterate since there will only be one set of lines
	        var isNotFinished = xml.getElementsByTagName("finished")[0];
	        var myBytesRead = xml.getElementsByTagName("bytes_read")[0];
	        var myContentLength = xml.getElementsByTagName("content_length")[0];
	        var myPercent = xml.getElementsByTagName("percent_complete")[0];
			var errorMessage = xml.getElementsByTagName('error_message')[0];
	
			if(errorMessage){
				this.bytesRead.style.visibility = "visible";
				domClass.add(this.bytesRead, "uploadDialogErrorLabel");
				this.bytesRead.innerHTML = errorMessage.firstChild.data;
			} else if ((isNotFinished == null) && (myPercent == null)) { // Check to see if it's even started yet
	            this.bytesRead.style.visibility = "visible";
	            this.bytesRead.innerHTML = "Initializing Upload...";
	            registry.byNode(this.bar.domNode).update({progress: "0"});
	
	            // Sleep then call the function again
	            window.setTimeout(lang.hitch(this, "ajaxFunction"), 100);
	        }
	        else {
	            document.getElementById("progressBarTable").style.visibility = "visible";
	            this.bytesRead.style.visibility = "visible";
	
	            myBytesRead = myBytesRead.firstChild.data;
	            myContentLength = myContentLength.firstChild.data;
	
	            // It's started, get the status of the upload
	            if (myPercent != null) {
	                myPercent = myPercent.firstChild.data;
	                registry.byNode(this.bar.domNode).update({progress: myPercent});
	                this.bytesRead.innerHTML = myBytesRead + " of " +
	                        myContentLength + " bytes read";
	
	                // Sleep then call the function again
	                window.setTimeout(lang.hitch(this, "ajaxFunction"), 100);
	            }
	            else {
	                var origFN = xml.getElementsByTagName("original_file_name")[0];
	                var serverFN = xml.getElementsByTagName("server_file_name")[0];
	                if (origFN) {
	                    this.parent.valueNode.value = this.URLDecode(decodeURI(origFN.firstChild.data));
	                    this.parent.textNode.appendChild(windowModule.doc.createTextNode(this.URLDecode(decodeURI(origFN.firstChild.data))));
	                }
	                if (serverFN) {
	                    this.parent.valueNode.value = this.parent.valueNode.value + "|" + this.URLDecode(decodeURI(serverFN.firstChild.data));
	                }
	                registry.byNode(this.bar.domNode).update({progress: "100%"});
	                this._hide();
	                if (this.parent.callbackUrl != "") {
	                    eval(this.parent.callbackUrl);
	                }
	            }
	        }
	    },
	
		destroy: function(){
			array.forEach(registry.findWidgets(this.buttonsBarNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}
			});
			this.inherited(arguments);
		}
	});
	return Upload;
});

},
'dojo/dnd/Selector':function(){
define([
	"../_base/array", "../_base/declare", "../_base/kernel", "../_base/lang",
	"../dom", "../dom-construct", "../mouse", "../_base/NodeList", "../on", "../touch", "./common", "./Container"
], function(array, declare, kernel, lang, dom, domConstruct, mouse, NodeList, on, touch, dnd, Container){

// module:
//		dojo/dnd/Selector

/*
	Container item states:
		""			- an item is not selected
		"Selected"	- an item is selected
		"Anchor"	- an item is selected, and is an anchor for a "shift" selection
*/

/*=====
var __SelectorArgs = declare([Container.__ContainerArgs], {
	// singular: Boolean
	//		allows selection of only one element, if true
	singular: false,

	// autoSync: Boolean
	//		autosynchronizes the source with its list of DnD nodes,
	autoSync: false
});
=====*/

var Selector = declare("dojo.dnd.Selector", Container, {
	// summary:
	//		a Selector object, which knows how to select its children

	/*=====
	// selection: Set<String>
	//		The set of id's that are currently selected, such that this.selection[id] == 1
	//		if the node w/that id is selected.  Can iterate over selected node's id's like:
	//	|		for(var id in this.selection)
	selection: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		constructor of the Selector
		// node: Node||String
		//		node or node's id to build the selector on
		// params: __SelectorArgs?
		//		a dictionary of parameters
		if(!params){ params = {}; }
		this.singular = params.singular;
		this.autoSync = params.autoSync;
		// class-specific variables
		this.selection = {};
		this.anchor = null;
		this.simpleSelection = false;
		// set up events
		this.events.push(
			on(this.node, touch.press, lang.hitch(this, "onMouseDown")),
			on(this.node, touch.release, lang.hitch(this, "onMouseUp"))
		);
	},

	// object attributes (for markup)
	singular: false,	// is singular property

	// methods
	getSelectedNodes: function(){
		// summary:
		//		returns a list (an array) of selected nodes
		var t = new NodeList();
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			t.push(dom.byId(i));
		}
		return t;	// NodeList
	},
	selectNone: function(){
		// summary:
		//		unselects all items
		return this._removeSelection()._removeAnchor();	// self
	},
	selectAll: function(){
		// summary:
		//		selects all items
		this.forInItems(function(data, id){
			this._addItemClass(dom.byId(id), "Selected");
			this.selection[id] = 1;
		}, this);
		return this._removeAnchor();	// self
	},
	deleteSelectedNodes: function(){
		// summary:
		//		deletes all selected items
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var n = dom.byId(i);
			this.delItem(i);
			domConstruct.destroy(n);
		}
		this.anchor = null;
		this.selection = {};
		return this;	// self
	},
	forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over selected items;
		//		see `dojo/dnd/Container.forInItems()` for details
		o = o || kernel.global;
		var s = this.selection, e = dnd._empty;
		for(var i in s){
			if(i in e){ continue; }
			f.call(o, this.getItem(i), i, this);
		}
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map

		Selector.superclass.sync.call(this);

		// fix the anchor
		if(this.anchor){
			if(!this.getItem(this.anchor.id)){
				this.anchor = null;
			}
		}

		// fix the selection
		var t = [], e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			if(!this.getItem(i)){
				t.push(i);
			}
		}
		array.forEach(t, function(i){
			delete this.selection[i];
		}, this);

		return this;	// self
	},
	insertNodes: function(addSelected, data, before, anchor){
		// summary:
		//		inserts new data items (see `dojo/dnd/Container.insertNodes()` method for details)
		// addSelected: Boolean
		//		all new nodes will be added to selected items, if true, no selection change otherwise
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		var oldCreator = this._normalizedCreator;
		this._normalizedCreator = function(item, hint){
			var t = oldCreator.call(this, item, hint);
			if(addSelected){
				if(!this.anchor){
					this.anchor = t.node;
					this._removeItemClass(t.node, "Selected");
					this._addItemClass(this.anchor, "Anchor");
				}else if(this.anchor != t.node){
					this._removeItemClass(t.node, "Anchor");
					this._addItemClass(t.node, "Selected");
				}
				this.selection[t.node.id] = 1;
			}else{
				this._removeItemClass(t.node, "Selected");
				this._removeItemClass(t.node, "Anchor");
			}
			return t;
		};
		Selector.superclass.insertNodes.call(this, data, before, anchor);
		this._normalizedCreator = oldCreator;
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		Selector.superclass.destroy.call(this);
		this.selection = this.anchor = null;
	},

	// mouse events
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(this.autoSync){ this.sync(); }
		if(!this.current){ return; }
		if(!this.singular && !dnd.getCopyKeyState(e) && !e.shiftKey && (this.current.id in this.selection)){
			this.simpleSelection = true;
			if(mouse.isLeft(e)){
				// Accept the left button and stop the event.   Stopping the event prevents text selection while
				// dragging.   However, don't stop the event on mobile because that prevents a click event,
				// and also prevents scroll (see #15838).
				// For IE we don't stop event when multiple buttons are pressed.
				e.stopPropagation();
				e.preventDefault();
			}
			return;
		}
		if(!this.singular && e.shiftKey){
			if(!dnd.getCopyKeyState(e)){
				this._removeSelection();
			}
			var c = this.getAllNodes();
			if(c.length){
				if(!this.anchor){
					this.anchor = c[0];
					this._addItemClass(this.anchor, "Anchor");
				}
				this.selection[this.anchor.id] = 1;
				if(this.anchor != this.current){
					var i = 0, node;
					for(; i < c.length; ++i){
						node = c[i];
						if(node == this.anchor || node == this.current){ break; }
					}
					for(++i; i < c.length; ++i){
						node = c[i];
						if(node == this.anchor || node == this.current){ break; }
						this._addItemClass(node, "Selected");
						this.selection[node.id] = 1;
					}
					this._addItemClass(this.current, "Selected");
					this.selection[this.current.id] = 1;
				}
			}
		}else{
			if(this.singular){
				if(this.anchor == this.current){
					if(dnd.getCopyKeyState(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");
					this.selection[this.current.id] = 1;
				}
			}else{
				if(dnd.getCopyKeyState(e)){
					if(this.anchor == this.current){
						delete this.selection[this.anchor.id];
						this._removeAnchor();
					}else{
						if(this.current.id in this.selection){
							this._removeItemClass(this.current, "Selected");
							delete this.selection[this.current.id];
						}else{
							if(this.anchor){
								this._removeItemClass(this.anchor, "Anchor");
								this._addItemClass(this.anchor, "Selected");
							}
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[this.current.id] = 1;
						}
					}
				}else{
					if(!(this.current.id in this.selection)){
						this.selectNone();
						this.anchor = this.current;
						this._addItemClass(this.current, "Anchor");
						this.selection[this.current.id] = 1;
					}
				}
			}
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onMouseUp: function(/*===== e =====*/){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(!this.simpleSelection){ return; }
		this.simpleSelection = false;
		this.selectNone();
		if(this.current){
			this.anchor = this.current;
			this._addItemClass(this.anchor, "Anchor");
			this.selection[this.current.id] = 1;
		}
	},
	onMouseMove: function(/*===== e =====*/){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		this.simpleSelection = false;
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		this.onmousemoveEvent = on(this.node, touch.move, lang.hitch(this, "onMouseMove"));
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		if(this.onmousemoveEvent){
			this.onmousemoveEvent.remove();
			delete this.onmousemoveEvent;
		}
	},
	_removeSelection: function(){
		// summary:
		//		unselects all items
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var node = dom.byId(i);
			if(node){ this._removeItemClass(node, "Selected"); }
		}
		this.selection = {};
		return this;	// self
	},
	_removeAnchor: function(){
		if(this.anchor){
			this._removeItemClass(this.anchor, "Anchor");
			this.anchor = null;
		}
		return this;	// self
	}
});

return Selector;

});

},
'xcitec/xlayout/_QuestionV2ItemSingleColumn':function(){
define([
         "dojo/_base/declare",
         "./_QuestionV2Item"
       ], function (declare, _QuestionV2Item) {
  return declare(_QuestionV2Item, {

    controlView: "MODE2",
    defaultLabelPos: "top"
  });
});
},
'slmui/layout/_ChapterAdvancedItem':function(){
define([
	'dojo/_base/declare',
	'xcitec/layout/_Chapter2CItem',
	'./_ChapterItem'
], function(declare, _xChapter2CItem, _ChapterItem){
	return declare([_ChapterItem, _xChapter2CItem], {
		// summary:
		//		Deprecated. Will be used util full migration to IDX. Control's wrapper with label for advanced Chapter.
		// tags:
		//		deprecated
	});
});

},
'xcitec/formx/EVAComboField':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dom",
	"dojo/fx",
	"dojo/query",
	"dojo/text!xcitec/formx/templates/EVAComboField.html",
	"dijit/registry",
	"xcitec/_StateWidget"
], function (dojo, declare, array, lang, domAttr, domClass, domConstruct, domStyle, dom, fx, query, template, registry, _StateWidget) {

	var EVAComboField = declare( _StateWidget, {
	    baseClass: "evacriterionHeader",
	    state: "Open",
	    commentExists: false,
	
	    templateString: template,
	
	    postMixInProperties: function() {
	        // search for controls marked with xcitecRole attribute
	        this.xcitecRoles = query(" > [xcitecRole]", this.srcNodeRef);
	        this.inherited(arguments);    
	    },
	
	    _fillContent: function() {
	        var _this = this;
	
	        // move controls in appropriate places if they exist
	        array.forEach(_this.xcitecRoles, function(node) {
	            switch (domAttr.get(node, "xcitecRole").toLowerCase()) {
	                case "result":
	                    domConstruct.place(node, _this.criteriaResult, "last");
	                    break;
	                case "evaluator":
	                    domConstruct.place(node, _this.evaluator, "last");
	                    break;
	                default:
	                    break;
	            }
	        });
	
	        if (!this.commentExists) {
	            domConstruct.destroy(this.commentIcon);
	        }
	        this.inherited(arguments);
	    },
	
	    postCreate: function() {
	        this.inherited(arguments);
	        if (this.state == "Close") {
	            domClass.add(this.comboBody, "evacriterionBodyClose")      
	        }
	        dom.setSelectable(this.domNode, false);
	    },
	                                                                 
	    _onClick: function() {
	        switch (this.state) {
	            case "Close":
	                this.expand();
	                break;
	            case "Open":
	                this.collapse();
	                break;
	            default:
	                break;
	        }
	    },
	
	    expand: function() {
	        this.state = "Open";
	        domStyle.set(this.comboBody, "display", "none");
	        if (domClass.contains(this.comboBody, "evacriterionBodyClose")) {
	            domClass.remove(this.comboBody, "evacriterionBodyClose");
	        }
	        fx.wipeIn({ node: this.comboBody, beforeBegin: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    collapse: function() {
	        this.state = "Close";
	        fx.wipeOut({ node: this.comboBody, onEnd: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    // for ComboContainer: this widget has no inner combo 
	    hasInnerCombo: function() {
	        return false;
	    },
	
	    getState: function() {
	        return this.state;
	    },
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		}
	});
	
	return EVAComboField;
});
},
'xcitec/form/MultiSelectUtils':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare"
], function (array, declare) {

	var MultiSelectUtils = declare( null, {
	
	    js2push: function(arr) {
	        var i;
	        for (i = 1; i < arguments.length; i++) {
	            arr[arr.length] = arguments[i];
	        }
	        return arr;
	    },
	
	    js2splice: function(arr, index, howMany) {
	        var i;
	        var returnArr = new Array();
	        var postArr = new Array();
	        for (i = 0; i < arr.length; i++) {
	            if (i < index) this.js2push(returnArr, arr[i]);
	            if (i >= (index + howMany)) this.js2push(postArr, arr[i]);
	        }
	        for (i = 3; i < arguments.length; i++)
	            this.js2push(returnArr, arguments[i]);
	        for (i = 0; i < postArr.length; i++)
	            this.js2push(returnArr, postArr[i]);
	        return returnArr;
	    },
	
	    isNodeInList: function(/* Array */ list, /* DomNode */ node) {
	        var isInList = false;
	        array.forEach(list, function(value) {
	            if (value == node) {
	                isInList = true;
	            }
	        });
	        return isInList;
	    },
	
	    delFromList: function(/* Array */ list, /* DomNode */ node) {
	        var _this = this;
	        array.forEach(list, function(value) {
	            if (value == node) {
	               list = _this.js2splice(list, array.indexOf(list, value), 1);
	            }
	        });
	        return list;
	    },
	
	    addToList: function(/* Array */ list, /* DomNode */ node) {
	        list = list.concat(node);
	        return list;
	    },
	
	    clearList: function(/* Array */ list) {
	        if (list) {
	            list = this.js2splice(list, 0);
	        }
	        return list;
	    }
	});
	return MultiSelectUtils;
});
},
'xcitec/form/_XButtonMiddle':function(){
define([
  "dojo/_base/declare",
	"./_XButton"
], function (declare, _XButton) {
  return declare(_XButton, {
	    baseClass: "dijitXcitecBtn dijitXcitecBtnMiddle"
	});
});


},
'xcitec/xcitecUtil':function(){
define([
         "dojo/_base/lang",
         "./_base/Date",
         "./_base/wiky"
       ], function (lang, Date, Wiky) {
  /*=====
   xcitecUtil = {
   // summary:
   //     Utilities for slmui
   };
   =====*/

  function str2obj (/*String*/ value, /*String*/ type){
    // summary:
    //		Convert given string value to given type
    //NOTE: copy from dojo.parser.str2obj
    var d = dojo;
    switch(type){
    case "string":
      return value;
    case "number":
      return value.length ? Number(value) : NaN;
    case "boolean":
      // for checked/disabled value might be "" or "checked".	 interpret as true.
      return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
    case "function":
      if(d.isFunction(value)){
        // IE gives us a function, even when we say something like onClick="foo"
        // (in which case it gives us an invalid function "function(){ foo }").
        //	Therefore, convert to string
        value=value.toString();
        value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
      }
      try{
        if(value === "" || value.search(/[^\w\.]+/i) != -1){
          // The user has specified some text for a function like "return x+5"
          return new Function(value);
        }else{
          // The user has specified the name of a function like "myOnClick"
          // or a single word function "return"
          return d.getObject(value, false) || new Function(value);
        }
      }catch(e){ return new Function(); }
    case "array":
      return value ? value.split(/\s*,\s*/) : [];
    case "date":
      switch(value){
      case "": return new Date("");	// the NaN of dates
      case "now": return new Date();	// current date
      default: return d.date.stamp.fromISOString(value);
      }
    case "url":
      return d.baseUrl + value;
    default:
      return d.fromJson(value);
    }
  }

  return {
    str2obj : str2obj,

    str2bool: function(/*String*/ value){
      //if (value==null) return false;
      //return xcitecUtil.str2obj( value, "boolean");
      return (value != null) && str2obj(value, "boolean");
      // for "selected" IE8 returns empty string if attribut "selected" is not defined.
      // fix was wrong under FF it returns empty string if just attribut "selected" is defined
      //return value && (value!=null) && xcitecUtil.str2obj( value, "boolean");
    },

    escapeXml: function(/*String*/ str, /*Boolean?*/ noSingleQuotes){
      // summary:
      //		Adds escape sequences for special characters in XML: &<>"'
      //		Optionally skips escapes for single quotes
      //
      if(!str) return str;
      str = str.replace(/&/gm, "&amp;")
          .replace(/</gm, "&lt;")
          .replace(/>/gm, "&gt;")
          .replace(/"/gm, "&quot;")
          .replace(/'/gm, "&#39;");
      //	if(!noSingleQuotes){
      //		str = str.replace(/'/gm, "&#39;");
      //	}
      return str; // string
    },

    createTypedStyleClass: function(/*String*/ baseStyleClass, /*String*/ styleType){
      return baseStyleClass + styleType.charAt(0).toUpperCase() + styleType.substr(1).replace(/-.*/, "");
    },

    createEntityIconClass: function(/*String*/ entityClass){
      if(!entityClass) return entityClass;
      return "sboIcon" + entityClass.charAt(0).toUpperCase() + entityClass.substr(1);
    },

    parseDate: function(Input, Mask, Default){
      return Date.parseFormat(Input, Mask.toUpperCase(), Default);
    },

    toHtml: function(text){
      return Wiky.toHtml(text);
    },

    processLabelAdminClick: function(labelId){
      if(typeof this.onLabelAdminClick == "function"){
        return this.onLabelAdminClick(labelId);
      }
    },

	isValidEmail: function(value){
		// user name RE per rfc5322
		var usernameRE = "([!#-'*+\\-\\/-9=?A-Z^-~]+[.])*[!#-'*+\\-\\/-9=?A-Z^-~]+";

		// Domain name labels can not end with a dash.
		var domainLabelRE = "(?:[\\da-zA-Z](?:[-\\da-zA-Z]{0,61}[\\da-zA-Z])?)";
		var domainNameRE = "(?:[a-zA-Z](?:[-\\da-zA-Z]{0,6}[\\da-zA-Z])?)";


		// decimal-dotted IP address RE.
		var dottedDecimalRE =
			// Each number is between 0-255.  Zero padding is not allowed.
			"((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])";

		// dotted hex IP address RE.  Each number is between 0x0-0xff.  Zero padding is allowed, e.g. 0x00.
		var dottedHexRE = "(0[xX]0*[\\da-fA-F]?[\\da-fA-F]\\.){3}0[xX]0*[\\da-fA-F]?[\\da-fA-F]";

		// dotted octal IP address RE.  Each number is between 0000-0377.
		// Zero padding is allowed, but each number must have at least 4 characters.
		var dottedOctalRE = "(0+[0-3][0-7][0-7]\\.){3}0+[0-3][0-7][0-7]";

		// decimal IP address RE.  A decimal number between 0-4294967295.
		var decimalRE =  "(0|[1-9]\\d{0,8}|[1-3]\\d{9}|4[01]\\d{8}|42[0-8]\\d{7}|429[0-3]\\d{6}|" +
			"4294[0-8]\\d{5}|42949[0-5]\\d{4}|429496[0-6]\\d{3}|4294967[01]\\d{2}|42949672[0-8]\\d|429496729[0-5])";

		// hexadecimal IP address RE.
		// A hexadecimal number between 0x0-0xFFFFFFFF. Case insensitive.  Zero padding is allowed.
		var hexRE = "0[xX]0*[\\da-fA-F]{1,8}";

		// IPv6 address RE.
		// The format is written as eight groups of four hexadecimal digits, x:x:x:x:x:x:x:x,
		// where x is between 0000-ffff. Zero padding is optional. Case insensitive.
		var ipv6RE = "([\\da-fA-F]{1,4}\\:){7}[\\da-fA-F]{1,4}";

		// IPv6/IPv4 Hybrid address RE.
		// The format is written as six groups of four hexadecimal digits,
		// followed by the 4 dotted decimal IPv4 format. x:x:x:x:x:x:d.d.d.d
		var hybridRE = "([\\da-fA-F]{1,4}\\:){6}" +
			"((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])";

		// Build IP Address RE
		var a = [];
		a.push(dottedDecimalRE);
		a.push(dottedHexRE);
		a.push(dottedOctalRE);
		a.push(decimalRE);
		a.push(hexRE);
		a.push(ipv6RE);
		a.push(hybridRE);

		var ipAddressRE = "(" + a.join("|") + ")";

		// build host RE
		var hostNameRE = "(((?:" + domainLabelRE + "\\.)+" + domainNameRE + "\\.?))|" + ipAddressRE;

		// build emailAddress RE
		var emailAddressRE = usernameRE + "@" + hostNameRE;

		var re = new RegExp("^" + emailAddressRE + "$", "i");
		return re.test(value);
	}
  };
});

},
'xcitec/page/UserInfoBox':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/query",
	'dijit/registry',
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "../xcitecUtil"
], function (dojo, declare, domAttr, query, registry, _Container, _Templated, _Widget, xcitecUtil) {

	var UserInfoBox = declare( [_Widget, _Templated, _Container], {
		templateString: "<div class='user'>" +
			"<div class='user_nav'><div class='user_nav_items' data-dojo-attach-point='containerNode'></div></div>" +
			"</div>",
		caption: "",
		captionId: "",
		widgetsInTemplate: true,
		isContainer: true,
		highlighted: false,
	
		_fillContent: function(/*DomNode*/ source){
			var actions = query('>', this.srcNodeRef);
	
			if(actions.length > 0){
				var _this = this;
				actions.forEach(function(action){
					if(xcitecUtil.str2bool(domAttr.get(action, 'userContainer'))){
						domAttr.set(action, 'caption', _this.caption);
					}
	
					var linkActions = query("[href]", action);
					if(linkActions.length > 0){
						linkActions.forEach(function(act){
							var hrefAttr = domAttr.get(act, "href");
							var target = domAttr.get(act, "target");
							if(target){
								domAttr.set(act, 'onclick', 'window.open("' + hrefAttr + '", "' + target + '")');
							}else{
								domAttr.set(act, 'onclick', 'window.location.href="' + hrefAttr + '"');
							}
						});
					}
				});
			}
			this.inherited(arguments);
		},

		startup: function(){
			this.inherited(arguments);
			if(this.highlighted){
				var userDropdown = registry.findWidgets(this.containerNode)[0];
				if(userDropdown){
					dojo.addClass(userDropdown.domNode, 'highlighted');
				}
			}
		}
	});
	return UserInfoBox;
});

},
'xcitec/xlayout/_ContentAreaPaneItemV2':function(){
define([
         "dojo/_base/declare",
         "dojo/dom-attr",
         "dojo/dom-class",
         "dojo/dom-construct",
         "dojo/dom-style",
         "dojo/query",
         "dijit/registry",
         "../layout/_ContentAreaPaneItem"
       ], function (declare, domAttr, domClass, domConstruct, domStyle, query, registry, _ContentAreaPaneItem) {
  return declare(_ContentAreaPaneItem, {

    templateString: '<div class="dijitReset question dojoDndItem" data-dojo-attach-point="containerNode, controlHolderNode">' +
                    '<span class="question_wrapper">' +
                    '<img src="${_blankGif}" data-dojo-attach-event="onclick:_onChangedNewClick" class="iconMarking" data-dojo-attach-point="iconMarkingChangedNew" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" data-dojo-attach-event="onclick:_onChangedClick" class="iconMarking" data-dojo-attach-point="iconMarkingChanged" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" class="iconMarking" data-dojo-attach-point="iconMarkingKo" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" class="iconMarking" data-dojo-attach-point="iconMarkingCritical" width="1" height="1" alt="" />' +
                    '<img src="${_blankGif}" class="iconMarking" data-dojo-attach-point="iconMarkingScore" title="" width="1" height="1" alt="" />' +
                    '</span>' +
                    '</div>',
    templateStringV2: '<div class="dijitReset question designMode dojoDndItem" data-dojo-attach-point="containerNode, controlHolderNode">' +
                      '<span class="question_wrapper">' +
                      '<span class="dijitReset dijitInline questionIcon questionIconDelete" data-dojo-attach-event="onclick: _onDeleteClick"></span>' +
                      '<span class="dijitReset dijitInline questionIcon questionIconSettings" data-dojo-attach-event="onclick: _onSettingsClick"></span>' +
                      '</span>' +
                      '</div>',
    templateStringV3: '<div class="dijitReset question designMode styleElementHolder dojoDndItem" data-dojo-attach-point="containerNode, controlHolderNode">' +
                      '<span class="question_wrapper">' +
                      '<span class="dijitReset dijitInline questionIcon questionIconSettings" data-dojo-attach-event="onclick: _onSettingsClick"></span>' +
                      '<span class="dijitReset dijitInline questionIcon questionIconDelete" data-dojo-attach-event="onclick: _onDeleteClick"></span>' +
                      '</span>' +
                      '</div>',

    designMode: false,

    parent: null,

    postMixInProperties: function() {
      this.inherited(arguments);
      // if in design mode - use another template
      if (this.designMode) {
        if (domAttr.get(this.controlNode, 'data-dojo-type') == 'xcitec/xlayout/StyleElement') {
          this.templateString = this.templateStringV3;
        } else {
          this.templateString = this.templateStringV2;
        }
        domAttr.set(this.controlNode, 'designMode', 'true');
      } else {
        // if in view mode - StyleElement empty line must be without text
        if (domAttr.get(this.controlNode, 'elementType') == 'emptyline') {
          domAttr.set(this.controlNode, 'elementType', 'emptylineview');
        }
      }
    },

    startup: function() {
      this.inherited(arguments);
      if (!this.designMode) {
        var question = query(">", this.domNode)[1];
        if (question) {
          if(registry.byNode(question).isScoreSet()) {
            domClass.add(this.iconMarkingScore, "iconMarkingScore");
            domStyle.set(this.iconMarkingScore, "display", "inline");
            if (registry.byNode(question).score != "true") {
              domAttr.set(this.iconMarkingScore, "title", registry.byNode(question).score);
            }
          }
          if (registry.byNode(question).isChangedSet()) {
            domClass.add(this.iconMarkingChanged, "iconMarkingChanged");
            domStyle.set(this.iconMarkingChanged, "display", "inline");
          }
          if (registry.byNode(question).isChangedNewSet()) {
            domClass.add(this.iconMarkingChangedNew, "iconMarkingChangedNew");
            domStyle.set(this.iconMarkingChangedNew, "display", "inline");
          }
          if (registry.byNode(question).isChangedOldSet()) {
            domClass.add(this.containerNode, "markingChangedOld");
          }

          if (registry.byNode(question).isCriticalSet()) {
            domClass.add(this.iconMarkingCritical, "iconMarkingCritical");
            domStyle.set(this.iconMarkingCritical, "display", "inline");
            if (registry.byNode(question).critical != "true") {
              domAttr.set(this.iconMarkingCritical, "title", registry.byNode(question).critical);
            }
          }
          if (registry.byNode(question).isKoSet()) {
            domClass.add(this.iconMarkingKo, "iconMarkingKo");
            domStyle.set(this.iconMarkingKo, "display", "inline");
            if (registry.byNode(question).ko != "true") {
              domAttr.set(this.iconMarkingKo, "title", registry.byNode(question).ko);
            }
          }
        }
      }
    },

    /* actions for buttons (delete and settings) */
    _onDeleteClick: function() {
      var question = query(">", this.domNode)[1];
      if (question) {
        if (registry.byNode(question).isDeleteSet()) {
          registry.byNode(question).onDelete();
        } else {
          if (this.parent.parent.isDeleteSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onDelete();
          } else {
            domConstruct.destroy(this.domNode);
            this.parent.parent.updateColumnsOrder();
          }
        }
      }
    },

    _onSettingsClick: function() {
      var question = query(">", this.domNode)[1];
      if (question) {
        if (registry.byNode(question).isSettingsSet()) {
          registry.byNode(question).onSettings();
        } else {
          if (this.parent.parent.isSettingsSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onSettings();
          }
        }
      }
    },

    _onChangedClick: function() {
      var question = query(">", this.domNode)[1];
      if (question) {
        if (registry.byNode(question).isChangedSet()) {
          registry.byNode(question).onChanged();
        } else {
          if (this.parent.parent.isChangedSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onChanged();
          }
        }
      }
    },

    _onChangedNewClick: function() {
      var question = query(">", this.domNode)[1];
      if (question) {
        if (registry.byNode(question).isChangedNewSet()) {
          registry.byNode(question).onChangedNew();
        } else {
          if (this.parent.parent.isChangedNewSet()) {
            this.parent.idHolder.value = registry.byNode(question).id;
            this.parent.parent.onChangedNew();
          }
        }
      }
    }
  });
});
},
'xcitec/grid/Actions':function(){
define([
	"dojo/_base/declare",
	"../_base/Actions"
], function (declare, Actions) {
	return declare( Actions, {
	
	});
});

},
'dojo/_base/Color':function(){
define(["./kernel", "./lang", "./array", "./config"], function(dojo, lang, ArrayUtil, config){

	var Color = dojo.Color = function(/*Array|String|Object*/ color){
		// summary:
		//		Takes a named string, hex string, array of rgb or rgba values,
		//		an object with r, g, b, and a properties, or another `Color` object
		//		and creates a new Color instance to work from.
		//
		// example:
		//		Work with a Color instance:
		//	|	require(["dojo/_base/color"], function(Color){
		//	|		var c = new Color();
		//	|		c.setColor([0,0,0]); // black
		//	|		var hex = c.toHex(); // #000000
		//	|	});
		//
		// example:
		//		Work with a node's color:
		//	| 
		//	|	require(["dojo/_base/color", "dojo/dom-style"], function(Color, domStyle){
		//	|		var color = domStyle("someNode", "backgroundColor");
		//	|		var n = new Color(color);
		//	|		// adjust the color some
		//	|		n.r *= .5;
		//	|		console.log(n.toString()); // rgb(128, 255, 255);
		//	|	});
		if(color){ this.setColor(color); }
	};

	// FIXME:
	// there's got to be a more space-efficient way to encode or discover
	// these!! Use hex?
	Color.named = {
		// summary:
		//		Dictionary list of all CSS named colors, by name. Values are 3-item arrays with corresponding RG and B values.
		"black":  [0,0,0],
		"silver": [192,192,192],
		"gray":	  [128,128,128],
		"white":  [255,255,255],
		"maroon": [128,0,0],
		"red":	  [255,0,0],
		"purple": [128,0,128],
		"fuchsia":[255,0,255],
		"green":  [0,128,0],
		"lime":	  [0,255,0],
		"olive":  [128,128,0],
		"yellow": [255,255,0],
		"navy":	  [0,0,128],
		"blue":	  [0,0,255],
		"teal":	  [0,128,128],
		"aqua":	  [0,255,255],
		"transparent": config.transparentColor || [0,0,0,0]
	};

	lang.extend(Color, {
		r: 255, g: 255, b: 255, a: 1,
		_set: function(r, g, b, a){
			var t = this; t.r = r; t.g = g; t.b = b; t.a = a;
		},
		setColor: function(/*Array|String|Object*/ color){
			// summary:
			//		Takes a named string, hex string, array of rgb or rgba values,
			//		an object with r, g, b, and a properties, or another `Color` object
			//		and sets this color instance to that value.
			//
			// example:
			//	|	require(["dojo/_base/color"], function(Color){
			//	|		var c = new Color(); // no color
			//	|		c.setColor("#ededed"); // greyish
			//	|	});
			if(lang.isString(color)){
				Color.fromString(color, this);
			}else if(lang.isArray(color)){
				Color.fromArray(color, this);
			}else{
				this._set(color.r, color.g, color.b, color.a);
				if(!(color instanceof Color)){ this.sanitize(); }
			}
			return this;	// Color
		},
		sanitize: function(){
			// summary:
			//		Ensures the object has correct attributes
			// description:
			//		the default implementation does nothing, include dojo.colors to
			//		augment it with real checks
			return this;	// Color
		},
		toRgb: function(){
			// summary:
			//		Returns 3 component array of rgb values
			// example:
			//	|	require(["dojo/_base/color"], function(Color){
			//	|		var c = new Color("#000000");
			//	|		console.log(c.toRgb()); // [0,0,0]
			//	|	});
			var t = this;
			return [t.r, t.g, t.b]; // Array
		},
		toRgba: function(){
			// summary:
			//		Returns a 4 component array of rgba values from the color
			//		represented by this object.
			var t = this;
			return [t.r, t.g, t.b, t.a];	// Array
		},
		toHex: function(){
			// summary:
			//		Returns a CSS color string in hexadecimal representation
			// example:
			//	|	require(["dojo/_base/color"], function(Color){
			//	|		console.log(new Color([0,0,0]).toHex()); // #000000
			//	|	});
			var arr = ArrayUtil.map(["r", "g", "b"], function(x){
				var s = this[x].toString(16);
				return s.length < 2 ? "0" + s : s;
			}, this);
			return "#" + arr.join("");	// String
		},
		toCss: function(/*Boolean?*/ includeAlpha){
			// summary:
			//		Returns a css color string in rgb(a) representation
			// example:
			//	|	require(["dojo/_base/color"], function(Color){
			//	|		var c = new Color("#FFF").toCss();
			//	|		console.log(c); // rgb('255','255','255')
			//	|	});
			var t = this, rgb = t.r + ", " + t.g + ", " + t.b;
			return (includeAlpha ? "rgba(" + rgb + ", " + t.a : "rgb(" + rgb) + ")";	// String
		},
		toString: function(){
			// summary:
			//		Returns a visual representation of the color
			return this.toCss(true); // String
		}
	});

	Color.blendColors = dojo.blendColors = function(
		/*Color*/ start,
		/*Color*/ end,
		/*Number*/ weight,
		/*Color?*/ obj
	){
		// summary:
		//		Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,
		//		can reuse a previously allocated Color object for the result
		var t = obj || new Color();
		ArrayUtil.forEach(["r", "g", "b", "a"], function(x){
			t[x] = start[x] + (end[x] - start[x]) * weight;
			if(x != "a"){ t[x] = Math.round(t[x]); }
		});
		return t.sanitize();	// Color
	};

	Color.fromRgb = dojo.colorFromRgb = function(/*String*/ color, /*Color?*/ obj){
		// summary:
		//		Returns a `Color` instance from a string of the form
		//		"rgb(...)" or "rgba(...)". Optionally accepts a `Color`
		//		object to update with the parsed value and return instead of
		//		creating a new object.
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		var m = color.toLowerCase().match(/^rgba?\(([\s\.,0-9]+)\)/);
		return m && Color.fromArray(m[1].split(/\s*,\s*/), obj);	// Color
	};

	Color.fromHex = dojo.colorFromHex = function(/*String*/ color, /*Color?*/ obj){
		// summary:
		//		Converts a hex string with a '#' prefix to a color object.
		//		Supports 12-bit #rgb shorthand. Optionally accepts a
		//		`Color` object to update with the parsed value.
		//
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		//
		// example:
		//	|	require(["dojo/_base/color"], function(Color){
		//	|		var thing = new Color().fromHex("#ededed"); // grey, longhand
		//	|		var thing2 = new Color().fromHex("#000"); // black, shorthand
		//	|	});
		var t = obj || new Color(),
			bits = (color.length == 4) ? 4 : 8,
			mask = (1 << bits) - 1;
		color = Number("0x" + color.substr(1));
		if(isNaN(color)){
			return null; // Color
		}
		ArrayUtil.forEach(["b", "g", "r"], function(x){
			var c = color & mask;
			color >>= bits;
			t[x] = bits == 4 ? 17 * c : c;
		});
		t.a = 1;
		return t;	// Color
	};

	Color.fromArray = dojo.colorFromArray = function(/*Array*/ a, /*Color?*/ obj){
		// summary:
		//		Builds a `Color` from a 3 or 4 element array, mapping each
		//		element in sequence to the rgb(a) values of the color.
		// example:
		//		|	require(["dojo/_base/color"], function(Color){
		//		|		var myColor = new Color().fromArray([237,237,237,0.5]); // grey, 50% alpha
		//		|	});
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		var t = obj || new Color();
		t._set(Number(a[0]), Number(a[1]), Number(a[2]), Number(a[3]));
		if(isNaN(t.a)){ t.a = 1; }
		return t.sanitize();	// Color
	};

	Color.fromString = dojo.colorFromString = function(/*String*/ str, /*Color?*/ obj){
		// summary:
		//		Parses `str` for a color value. Accepts hex, rgb, and rgba
		//		style color values.
		// description:
		//		Acceptable input values for str may include arrays of any form
		//		accepted by dojo.colorFromArray, hex strings such as "#aaaaaa", or
		//		rgb or rgba strings such as "rgb(133, 200, 16)" or "rgba(10, 10,
		//		10, 50)"
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		var a = Color.named[str];
		return a && Color.fromArray(a, obj) || Color.fromRgb(str, obj) || Color.fromHex(str, obj);	// Color
	};

	return Color;
});

},
'dojo/fx':function(){
define([
	"./_base/lang",
	"./Evented",
	"./_base/kernel",
	"./_base/array",
	"./aspect",
	"./_base/fx",
	"./dom",
	"./dom-style",
	"./dom-geometry",
	"./ready",
	"require" // for context sensitive loading of Toggler
], function(lang, Evented, dojo, arrayUtil, aspect, baseFx, dom, domStyle, geom, ready, require){

	// module:
	//		dojo/fx
	
	// For back-compat, remove in 2.0.
	if(!dojo.isAsync){
		ready(0, function(){
			var requires = ["./fx/Toggler"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	var coreFx = dojo.fx = {
		// summary:
		//		Effects library on top of Base animations
	};

	var _baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		arrayUtil.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	_chain.prototype = new Evented();
	lang.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			this._onAnimateCtx.remove();
			this._onEndCtx.remove();
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = aspect.after(this._current, "onAnimate", lang.hitch(this, "_onAnimate"), true);
				this._onEndCtx = aspect.after(this._current, "onEnd", lang.hitch(this, "_onEnd"), true);
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = aspect.after(this._current, "beforeBegin", lang.hitch(this, function(){
					this._fire("beforeBegin");
				}), true),
				onBegin = aspect.after(this._current, "onBegin", lang.hitch(this, function(arg){
					this._fire("onBegin", arguments);
				}), true),
				onPlay = aspect.after(this._current, "onPlay", lang.hitch(this, function(arg){
					this._fire("onPlay", arguments);
					beforeBegin.remove();
					onBegin.remove();
					onPlay.remove();
				}));
			if(this._onAnimateCtx){
				this._onAnimateCtx.remove();
			}
			this._onAnimateCtx = aspect.after(this._current, "onAnimate", lang.hitch(this, "_onAnimate"), true);
			if(this._onEndCtx){
				this._onEndCtx.remove();
			}
			this._onEndCtx = aspect.after(this._current, "onEnd", lang.hitch(this, "_onEnd"), true);
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = aspect.after(this._current, "onPause", lang.hitch(this, function(arg){
						this._fire("onPause", arguments);
						e.remove();
					}), true);
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			arrayUtil.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = aspect.after(this._current, "onStop", lang.hitch(this, function(arg){
						this._fire("onStop", arguments);
						e.remove();
					}), true);
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ this._onAnimateCtx.remove(); }
			if(this._onEndCtx){ this._onEndCtx.remove(); }
		}
	});
	lang.extend(_chain, _baseObj);

	coreFx.chain = function(/*dojo/_base/fx.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo/_base/fx.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo/_base/fx.Animation` which will play all passed
		//		`dojo/_base/fx.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	require(["dojo/fx"], function(fx){
		//	|		fx.chain([
		//	|			fx.fadeIn({ node:node }),
		//	|			fx.fadeOut({ node:otherNode })
		//	|		]).play();
		//	|	});
		//
		return new _chain(animations); // dojo/_base/fx.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		arrayUtil.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(aspect.after(a, "onEnd", lang.hitch(this, "_onEnd"), true));
		}, this);

		this._pseudoAnimation = new baseFx.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		arrayUtil.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(aspect.after(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); },
				true));
			}
		);
	};
	lang.extend(_combine, {
		_doAction: function(action, args){
			arrayUtil.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			arrayUtil.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			arrayUtil.forEach(this._connects, function(handle){
				handle.remove();
			});
		}
	});
	lang.extend(_combine, _baseObj);

	coreFx.combine = function(/*dojo/_base/fx.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo/_base/fx.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo/_base/fx.Animation`s to run in parallel,
		//		providing a new `dojo/_base/fx.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	require(["dojo/fx"], function(fx){
		//	|		fx.combine([
		//	|			fx.fadeIn({ node:node }),
		//	|			fx.fadeOut({ node:otherNode })
		//	|		]).play();
		//	|	});
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	require(["dojo/fx"], function(fx){
		//	|		var anim = fx.combine([
		//	|			fx.fadeIn({ node: n, duration:700 }),
		//	|			fx.fadeOut({ node: otherNode, duration: 300 })
		//	|		]);
		//	|		aspect.after(anim, "onEnd", function(){
		//	|			// overall animation is done.
		//	|		}, true);
		//	|		anim.play(); // play the animation
		//	|	});
		//
		return new _combine(animations); // dojo/_base/fx.Animation
	};

	coreFx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo/_base/fx.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	require(["dojo/fx"], function(fx){
		//	|		fx.wipeIn({
		//	|			node:"someId"
		//	|		}).play()
		//	|	});

		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = domStyle.get(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		var fini = function(){
			s.height = "auto";
			s.overflow = o;
		};
		aspect.after(anim, "onStop", fini, true);
		aspect.after(anim, "onEnd", fini, true);

		return anim; // dojo/_base/fx.Animation
	};

	coreFx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo/_base/fx.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	require(["dojo/fx"], function(fx){
		//	|		fx.wipeOut({ node:"someId" }).play()
		//	|	});

		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		aspect.after(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		}, true);
		var fini = function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		};
		aspect.after(anim, "onStop", fini, true);
		aspect.after(anim, "onEnd", fini, true);

		return anim; // dojo/_base/fx.Animation
	};

	coreFx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo/_base/fx.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = dom.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = domStyle.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = geom.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		aspect.after(anim, "beforeBegin", init, true);

		return anim; // dojo/_base/fx.Animation
	};

	return coreFx;
});

},
'dijit/_base/sniff':function(){
define([ "dojo/uacss" ], function(){

	// module:
	//		dijit/_base/sniff

	/*=====
	return {
		// summary:
		//		Deprecated, back compatibility module, new code should require dojo/uacss directly instead of this module.
	};
	=====*/
});

},
'xcitec/layout/_ChapterItem':function(){

/*
 slmui.layout._ChapterItem is internal holder for Label/Control
 */
define([ "dijit/registry",
         "dojo/_base/declare",
        "dojo/dom-class",
        "dojo/dom-attr",
        "dojo/dom-style",
        "dojo/dom-construct",
        "./_LabelControlHolder",
       "../xcitecUtil"],
       function(registry, declare, domClass, domAttr, domStyle, domConstruct, _LabelControlHolder, xcitecUtil){

  return declare(_LabelControlHolder, {
    templateString:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td class="dijitReset fieldsetChapterLabelCell" data-dojo-attach-point="labelHolderColumnNode"' +
        '><div class="dijitReset fieldsetChapterLabelHolder" data-dojo-attach-point="labelHolderNode"></div>' +
        '</td><td class="dijitReset fieldsetChapterControlCell"><div data-dojo-attach-point="controlHolderNode"></div></td></tr>',

    templateStringMode2:
        '<tr class="dijitReset fieldsetChapterRow"' +
        '><td colspan="2" class="dijitReset fieldsetChapterControlCell" ' +
        '><div data-dojo-attach-point="labelHolderNode,controlHolderNode"></div></td></tr>',

    defaultLabelPos: "left",
    inner: false,

    _addLabelNode: function() {
      //this.labelHolderNode.appendChild(this.labelNode);
      switch(this.getLabelPos().toUpperCase()){
      case "NONE":
        break;
      case "TOP":
        domConstruct.place(this.labelNode, this.controlNode, "before");
        break;
      default:
        domConstruct.place(this.labelNode, this.labelHolderNode, "first");
      }
    },

    _addControlNode: function() {
      this.inherited(arguments);
      if(this.inner && this.widthInPercent != '100' && this.widthInPercent != ''){
        var paddingRight = 100 - this.widthInPercent;
        this.controlHolderNode.style.paddingRight = paddingRight + "%";
        domStyle.set(this.controlNode, "width", "100%");
      }
    },

    processAdditionalProperties: function(/*DomNode*/ controlNode) {
      this.inherited(arguments);
      this.inner = xcitecUtil.str2bool(domAttr.get(this.controlNode, "inner"));
      var parentWidget = registry.getEnclosingWidget(this.controlNode.parentNode);
      if(this.inner && parentWidget && parentWidget.widthInPercent){
        this.widthInPercent = parentWidget.widthInPercent;
      }
      if (this.controlView) this.processViewMode();
    },

    processViewMode: function() {
      switch (this.controlView.toUpperCase()) {
      case "MODE2":
      case "MODE3": case "WIDE":
      case "MODE4": case "WIDE2":
        this.templateString = this.templateStringMode2;
        this.defaultLabelPos = "top";
        //this.labelPos = "top";
        break;

      default:
        break;
      }
    },

    postCreate: function() {
      if (this.inner) {
        if (this.labelHolderColumnNode) domClass.add(this.labelHolderColumnNode, "innerControl");
        domClass.add(this.controlHolderNode, "innerControl");
      }
    }
  })
});

},
'xcitec/form/DateTextBox':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/aspect",
	"dojo/date/locale",
  "dojo/date",
  "dojo/dom-geometry",
	"dojo/dom-construct",
	"dojo/topic",
  "dojo/keys",
  "dijit/registry",
  "dijit/popup",
	"./TextBoxButton",
  "./_Calendar",
  "../xcitecUtil"
], function (declare, langModule, aspect, locale, date, geometry, domConstruct, topic, keys, registry, popup, TextBoxButton, _Calendar, xcitecUtil) {

	var DateTextBox = declare( TextBoxButton, {
	//    popupClass: "dijit.Calendar",
	    popupClass: _Calendar,
	    // format to display the date
	    dateFormat: "yyyy-MM-dd",
	    // format to send date on server and receive from server
	    defaultDateFormat: 'yyyy-MM-dd',
	    // date to show when calendar picker is open
	    defaultDate: "",
	
	    actionIconType: "Calendar",
	    hiddenInput: null,
        widgetsInTemplate: true,
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	
	        this.constraints = {datePattern:this.dateFormat};
	        this.buttonEnabled = true;
	
	        if (this.disabled) {
	            this.buttonEnabled = false;
	            this.disabledAttrSetting = "disabled";
	        }
	        if (this.readOnly) {
	            // kave: in readOnly action is allowed to
	            this.buttonEnabled = false;
	            //this.xbtnClass = "xcitec.form._XButtonSingleRight";
	        }
	    },
	
	    postCreate: function(){
	        this.inherited(arguments);
	        // create hidden input to send value on server
	        this.hiddenInput = domConstruct.create("input", {type: 'hidden'}, this.domNode, 'first');
	        if (this.name) {
	            this.hiddenInput.name = this.name
	        }
	        this.textbox.removeAttribute('name');
	
	        // convert string date into date object
		    var curValue = this.get("value");
		    this.hiddenInput.value = curValue;
	        var date = xcitecUtil.parseDate(curValue, this.defaultDateFormat.toUpperCase(), 0);
	        if (date) {
	            //this.hiddenInput.value = curValue;
	            this.set('value', locale.format(date, {datePattern: this.dateFormat, selector: "date"}))
	        }
	        this.connect(this.focusNode, 'onkeypress', this._onKeyPress);
	    },
	
	    _onClick: function() {
	        if (this.readOnly || this.disabled) {
	            return;
	        }
	        var button = this.button;
	        if(this._opened){
	            this._close();
	        } else {
	            this._open();
	            aspect.after(this._picker, "_onDayClick", function(){
	                button.setSelectedStyle(false);
	            }, true);
	            button.setSelectedStyle(true);
	            //this.textbox.focus();
	        }
	    },
	
	    _open: function() {
	        // summary:
	        //		opens the TimePicker, and sets the onValueSelected value
	        if (this.disabled || this.readOnly || !this.popupClass) {
	            return;
	        }
	
	        if(this._picker) {
				this._picker.destroy();
			}

       var PopupProto = langModule.isString(this.popupClass) ? langModule.getObject(this.popupClass, false) : this.popupClass,
            textBox = this;
	        var curVal = this.hiddenInput.value;
	        var value = xcitecUtil.parseDate( curVal||this.defaultDate, textBox.defaultDateFormat.toUpperCase(), 0) || new Date();
	
	        this._picker = new PopupProto({
	            //value: value,
	            currentFocus: value,
	            lang: textBox.lang,
	            constraints: textBox.constraints,
		        onChange: function(value) {
		            textBox._close();
		            textBox.hiddenInput.value = locale.format(value, {datePattern: textBox.defaultDateFormat, selector: "date"})
		            textBox.set("value", locale.format(value, {datePattern: textBox.dateFormat, selector: "date"}));
			        textBox.focus();
		        },
	            isDisabledDate: function(/*Date*/ date) {
	                // summary:
	                // 	disables dates outside of the min/max of the _DateTimeTextBox
	                var compare = date.compare;
	                var constraints = textBox.constraints;
	                return constraints && (constraints.min && (compare(constraints.min, date, "date") > 0) ||
	                        (constraints.max && compare(constraints.max, date, "date") < 0));
	            }
	        });
		    // to avoid IE bug date selected after open Calender popup
		    // (see SRM-6452 Calender dosn't get closed after selecting a date
		    this._picker.set('value', value, false);
	
	        if (!this._opened) {
            popup.open({
	                parent: this,
	                popup: this._picker,
	                around: this.domNode,
	                onCancel: langModule.hitch(this, this._close),
	//	            onExecute: function(){
	//		            //self.closeDropDown(true);
	//	            },
	//	            onCancel: function(){
	//		            //self.closeDropDown(true);
	//	            },
	                onClose: function(){
	                    textBox._opened = false;
	                }
	            });
	            this._opened = true;
	            this.button.setSelectedStyle(true);
	        }

          geometry.setMarginBox(this._picker.domNode, { w:this.domNode.offsetWidth });
	    },
	
	    _close: function() {
	        if (this._opened) {
            popup.close(this._picker);
	            this._opened = false;
	          this.button.setSelectedStyle(false);
	        }
	    },
	
	    _onBlur: function() {
	        // summary:
	        //		Called magically when focus has shifted away from this widget and it's dropdown
	        this._close();
	        if (this._picker) {
	            // teardown so that constraints will be rebuilt next time (redundant reference: #6002)
	            this._picker.destroy();
	            delete this._picker;
	        }
	        this.button.setSelectedStyle(false);
	        this.inherited(arguments);
	        // don't focus on <input>.  the user has explicitly focused on something else.
	    },
	
	    _onInput: function() {
	        this.inherited(arguments);
	        var convertedValue = xcitecUtil.parseDate(this.textbox.value, this.dateFormat.toUpperCase(), 0);
	        if (convertedValue) {
	            this.hiddenInput.value = locale.format(convertedValue, {datePattern: this.defaultDateFormat, selector: "date"})
	        } else {
	            this.hiddenInput.value = this.textbox.value;
	        }
	    },
	
	    _onKeyPress: function(/*Event*/ e) {
	        // summary:
	        //		Handler for keypress events
	
	        var p = this._picker, dk = keys;
	        // Handle the key in the picker, if it has a handler.  If the handler
	        // returns false, then don't handle any other keys.
	        if (p && this._opened && p.handleKey) {
	            if (p.handleKey(e) === false) {
	                return;
	            }
	        }
	        if (this._opened && e.charOrCode == dk.ESCAPE && !e.shiftKey && !e.ctrlKey && !e.altKey) {
	            this._close();
	            e.preventDefault(), e.stopPropagation();
	        } else if (!this._opened && e.charOrCode == dk.DOWN_ARROW) {
	            this._open();
	            e.preventDefault(), e.stopPropagation();
	        } else if (e.charOrCode === dk.TAB) {
	            this._tabbingAway = true;
	        } else if (this._opened && (e.keyChar || e.charOrCode === dk.BACKSPACE || e.charOrCode == dk.DELETE)) {
	            // Replace the element - but do it after a delay to allow for
	            // filtering to occur
	            setTimeout(langModule.hitch(this, function() {
                registry.placeOnScreenAroundElement(p.domNode.parentNode, this.domNode, {'BL':'TL', 'TL':'BL'}, p.orient ? langModule.hitch(p, "orient") : null);
	            }), 1);
	        }
	    }
	});

	return DateTextBox;
});

},
'dojo/data/util/sorter':function(){
define(["../../_base/lang"], function(lang){
	// module:
	//		dojo/data/util/sorter
	// summary:
	//		TODOC

var sorter = {};
lang.setObject("dojo.data.util.sorter", sorter);

sorter.basicComparator = function(	/*anything*/ a,
													/*anything*/ b){
	// summary:
	//		Basic comparison function that compares if an item is greater or less than another item
	// description:
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		'null' values (null, undefined) are treated as larger values so that they're pushed to the end of the list.
	//		And compared to each other, null is equivalent to undefined.

	//null is a problematic compare, so if null, we set to undefined.
	//Makes the check logic simple, compact, and consistent
	//And (null == undefined) === true, so the check later against null
	//works for undefined and is less bytes.
	var r = -1;
	if(a === null){
		a = undefined;
	}
	if(b === null){
		b = undefined;
	}
	if(a == b){
		r = 0;
	}else if(a > b || a == null){
		r = 1;
	}
	return r; //int {-1,0,1}
};

sorter.createSortFunction = function(	/* attributes[] */sortSpec, /*dojo/data/api/Read*/ store){
	// summary:
	//		Helper function to generate the sorting function based off the list of sort attributes.
	// description:
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	// sortSpec:
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	// |	{
	// |		attribute: "attributeName-string" || attribute,
	// |		descending: true|false;   // Default is false.
	// |	}
	// store:
	//		The datastore object to look up item values from.

	var sortFunctions=[];

	function createSortFunction(attr, dir, comp, s){
		//Passing in comp and s (comparator and store), makes this
		//function much faster.
		return function(itemA, itemB){
			var a = s.getValue(itemA, attr);
			var b = s.getValue(itemB, attr);
			return dir * comp(a,b); //int
		};
	}
	var sortAttribute;
	var map = store.comparatorMap;
	var bc = sorter.basicComparator;
	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		var attr = sortAttribute.attribute;
		if(attr){
			var dir = (sortAttribute.descending) ? -1 : 1;
			var comp = bc;
			if(map){
				if(typeof attr !== "string" && ("toString" in attr)){
					 attr = attr.toString();
				}
				comp = map[attr] || bc;
			}
			sortFunctions.push(createSortFunction(attr,
				dir, comp, store));
		}
	}
	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int
	}; // Function
};

return sorter;
});

},
'dojo/window':function(){
define(["./_base/lang", "./sniff", "./_base/window", "./dom", "./dom-geometry", "./dom-style", "./dom-construct"],
	function(lang, has, baseWindow, dom, geom, style, domConstruct){

	// feature detection
	/* not needed but included here for future reference
	has.add("rtl-innerVerticalScrollBar-on-left", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'hidden', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', width:'64px', height:'64px'}
			}, body, "last"),
			center = domConstruct.create('center', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			inner = domConstruct.create('div', {
				style: {overflow:'visible', display:'inline' }
			}, center, "last");
		inner.innerHTML="&nbsp;";
		var midPoint = Math.max(inner.offsetLeft, geom.position(inner).x);
		var ret = midPoint >= 32;
		center.removeChild(inner);
		scrollable.removeChild(center);
		body.removeChild(scrollable);
		return ret;
	});
	*/
	has.add("rtl-adjust-position-for-verticalScrollBar", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = domConstruct.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			ret = geom.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
		return ret;
	});

	has.add("position-fixed-support", function(win, doc){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		var	body = baseWindow.body(doc),
			outer = domConstruct.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = domConstruct.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last"),
			ret = geom.position(inner).x != geom.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
		return ret;
	});

	// module:
	//		dojo/window

	var window = {
		// summary:
		//		TODOC

		getBox: function(/*Document?*/ doc){
			// summary:
			//		Returns the dimensions and scroll position of the viewable area of a browser window

			doc = doc || baseWindow.doc;

			var
				scrollRoot = (doc.compatMode == 'BackCompat') ? baseWindow.body(doc) : doc.documentElement,
				// get scroll position
				scroll = geom.docScroll(doc), // scrollRoot.scrollTop/Left should work
				w, h;

			if(has("touch")){ // if(scrollbars not supported)
				var uiWindow = window.get(doc);   // use UI window, not dojo.global window
				// on mobile, scrollRoot.clientHeight <= uiWindow.innerHeight <= scrollRoot.offsetHeight, return uiWindow.innerHeight
				w = uiWindow.innerWidth || scrollRoot.clientWidth; // || scrollRoot.clientXXX probably never evaluated
				h = uiWindow.innerHeight || scrollRoot.clientHeight;
			}else{
				// on desktops, scrollRoot.clientHeight <= scrollRoot.offsetHeight <= uiWindow.innerHeight, return scrollRoot.clientHeight
				// uiWindow.innerWidth/Height includes the scrollbar and cannot be used
				w = scrollRoot.clientWidth;
				h = scrollRoot.clientHeight;
			}
			return {
				l: scroll.x,
				t: scroll.y,
				w: w,
				h: h
			};
		},

		get: function(/*Document*/ doc){
			// summary:
			//		Get window object associated with document doc.
			// doc:
			//		The document to get the associated window for.

			// In some IE versions (at least 6.0), document.parentWindow does not return a
			// reference to the real window object (maybe a copy), so we must fix it as well
			// We use IE specific execScript to attach the real window reference to
			// document._parentWindow for later use
			if(has("ie") && window !== document.parentWindow){
				/*
				In IE 6, only the variable "window" can be used to connect events (others
				may be only copies).
				*/
				doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
				//to prevent memory leak, unset it after use
				//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
				var win = doc._parentWindow;
				doc._parentWindow = null;
				return win;	//	Window
			}

			return doc.parentWindow || doc.defaultView;	//	Window
		},

		scrollIntoView: function(/*DomNode*/ node, /*Object?*/ pos){
			// summary:
			//		Scroll the passed node into view using minimal movement, if it is not already.

			// Don't rely on node.scrollIntoView working just because the function is there since
			// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
			// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
			// and when there's a fixed position scrollable element

			try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
				node = dom.byId(node);
				var	doc = node.ownerDocument || baseWindow.doc,	// TODO: why baseWindow.doc?  Isn't node.ownerDocument always defined?
					body = baseWindow.body(doc),
					html = doc.documentElement || body.parentNode,
					isIE = has("ie"),
					isWK = has("webkit");
				// if an untested browser, then use the native method
				if(node == body || node == html){ return; }
				if(!(has("mozilla") || isIE || isWK || has("opera") || has("trident")) && ("scrollIntoView" in node)){
					node.scrollIntoView(false); // short-circuit to native if possible
					return;
				}
				var	backCompat = doc.compatMode == 'BackCompat',
					rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
					rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
					scrollRoot = (isWK || backCompat) ? body : html,
					nodePos = pos || geom.position(node),
					el = node.parentNode,
					isFixed = function(el){
						return (isIE <= 6 || (isIE == 7 && backCompat))
							? false
							: (has("position-fixed-support") && (style.get(el, 'position').toLowerCase() == "fixed"));
					},
					self = this,
					scrollElementBy = function(el, x, y){
						if(el.tagName == "BODY" || el.tagName == "HTML"){
							self.get(el.ownerDocument).scrollBy(x, y);
						}else{
							x && (el.scrollLeft += x);
							y && (el.scrollTop += y);
						}
					};
				if(isFixed(node)){ return; } // nothing to do
				while(el){
					if(el == body){ el = scrollRoot; }
					var	elPos = geom.position(el),
						fixedPos = isFixed(el),
						rtl = style.getComputedStyle(el).direction.toLowerCase() == "rtl";

					if(el == scrollRoot){
						elPos.w = rootWidth; elPos.h = rootHeight;
						if(scrollRoot == html && (isIE || has("trident")) && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
						if(elPos.x < 0 || !isIE || isIE >= 9 || has("trident")){ elPos.x = 0; } // older IE can have values > 0
						if(elPos.y < 0 || !isIE || isIE >= 9 || has("trident")){ elPos.y = 0; }
					}else{
						var pb = geom.getPadBorderExtents(el);
						elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
						var clientSize = el.clientWidth,
							scrollBarSize = elPos.w - clientSize;
						if(clientSize > 0 && scrollBarSize > 0){
							if(rtl && has("rtl-adjust-position-for-verticalScrollBar")){
								elPos.x += scrollBarSize;
							}
							elPos.w = clientSize;
						}
						clientSize = el.clientHeight;
						scrollBarSize = elPos.h - clientSize;
						if(clientSize > 0 && scrollBarSize > 0){
							elPos.h = clientSize;
						}
					}
					if(fixedPos){ // bounded by viewport, not parents
						if(elPos.y < 0){
							elPos.h += elPos.y; elPos.y = 0;
						}
						if(elPos.x < 0){
							elPos.w += elPos.x; elPos.x = 0;
						}
						if(elPos.y + elPos.h > rootHeight){
							elPos.h = rootHeight - elPos.y;
						}
						if(elPos.x + elPos.w > rootWidth){
							elPos.w = rootWidth - elPos.x;
						}
					}
					// calculate overflow in all 4 directions
					var	l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
						t = nodePos.y - elPos.y, // beyond top: < 0
						r = l + nodePos.w - elPos.w, // beyond right: > 0
						bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
					var s, old;
					if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
						s = Math[l < 0? "max" : "min"](l, r);
						if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9 || has("trident"))){ s = -s; }
						old = el.scrollLeft;
						scrollElementBy(el, s, 0);
						s = el.scrollLeft - old;
						nodePos.x -= s;
					}
					if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
						s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
						old = el.scrollTop;
						scrollElementBy(el, 0, s);
						s = el.scrollTop - old;
						nodePos.y -= s;
					}
					el = (el != scrollRoot) && !fixedPos && el.parentNode;
				}
			}catch(error){
				console.error('scrollIntoView: ' + error);
				node.scrollIntoView(false);
			}
		}
	};

	 1  && lang.setObject("dojo.window", window);

	return window;
});

},
'xcitec/layout/ExtendedLayout':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"./Chapter",
  "./ContentArea",
  "./ContentAreaPane",
  "./Chapter2C"
], function (dojo, array, declare, domAttr, domConstruct, query, _Templated, _Widget, Chapter, ContentArea, ContentAreaPane, Chapter2C) {

	/*
	  Layout which let define both long and short controls
	 */
	var ExtendedLayout = declare( [_Widget, _Templated], {
	  templateString: '<div data-dojo-attach-point="attachNode, containerNode"></div>',
	
	  /* arrays to keep controls according to column
	   in which they should be placed */
	  firstColumnControls: null,
	  secondColumnControls: null,
	  // is it time to build ContentArea layout
	  flushControls: false,
	
	  postMixInProperties: function() {
	    this.firstColumnControls = [];
	    this.secondColumnControls = [];
	    this.inherited(arguments);
	  },
	
	  _fillContent: function() {
	    this._startupWidgets = this._startupWidgets || [];
	    var firstColumn = true;
	    var children = query(">", this.srcNodeRef);
	    var _this = this;
	
	    array.forEach(children, function(child, index) {
	      /* if control has 'controlView' attribute
	       then we build Chapter2Columns layout,
	       else just add node to appropriate column
	       of ContentArea layout
	        */
	      if (domAttr.get(child, "controlView")) {
	        /* if there were control already
	        then put them in ContentArea layout */
	        if (_this.flushControls) {
	          _this.attachNode.appendChild(_this.createContentArea().domNode);
	          _this.flushControls = false;
	          firstColumn = true
	        }
	        // create long control
	        div = domConstruct.create("div");
	        div.appendChild(child);
	        var chapter2c = new Chapter2C({}, div);
	        _this.attachNode.appendChild(chapter2c.domNode);
	        _this._startupWidgets.push(chapter2c);
	        _this.flushControls = false;
	      } else {
	        if (firstColumn) {
	          _this.firstColumnControls.push(child);
	          firstColumn = false;
	        } else {
	          _this.secondColumnControls.push(child);
	          firstColumn = true
	        }
	        _this.flushControls = true;
	      }
	      // if it is last control - put it in ContentArea layout
	      if ((children.length - 1) == index) {
	        if (_this.firstColumnControls.length > 0) {
	          _this.attachNode.appendChild(_this.createContentArea().domNode);
	        }
	      }
	    });
	    this.inherited(arguments);
	  },
	
	  /*
	      Creates ContentArae layout with controls inside it
	   */
	  createContentArea: function() {
	    var container = domConstruct.create("div");
	    var contentAreaPane1 = domConstruct.create("div", { 'data-dojo-type': 'xcitec/layout/ContentAreaPane' });
	    container.appendChild(contentAreaPane1);
	
	    if (this.firstColumnControls.length > 0) {
	      var div = domConstruct.create("div");
	      array.forEach(this.firstColumnControls, function(node) {
	        div.appendChild(node)
	      }
	          );
	      var firstColumnChapter = new Chapter({}, div);
	      contentAreaPane1.appendChild(firstColumnChapter.domNode);
	
	    }
	    var contentAreaPane2 = domConstruct.create("div", { 'data-dojo-type': 'xcitec/layout/ContentAreaPane' });
	    container.appendChild(contentAreaPane2);
	
	    if (this.secondColumnControls.length > 0) {
	      div = domConstruct.create("div");
	      array.forEach(this.secondColumnControls, function(node) {
	        div.appendChild(node)
	      }
	          );
	      var secondColumnChapter = new Chapter({}, div);
	      contentAreaPane2.appendChild(secondColumnChapter.domNode);
	
	    }
	    var contentArea = new ContentArea({}, container);
	
	    if (this.firstColumnControls.length > 0) {
	      var pane1 = new ContentAreaPane({showDelimiter: false }, contentAreaPane1);
	      this._startupWidgets.push(pane1);
	    }
	    if (this.secondColumnControls.length > 0) {
	      var pane2 = new ContentAreaPane({showDelimiter: false }, contentAreaPane2);
	      this._startupWidgets.push(pane2);
	    }
	    
	    this._startupWidgets.push(contentArea);
	    this.firstColumnControls = [];
	    this.secondColumnControls = [];
	    return contentArea;
	  }
	});
	return ExtendedLayout;
});

},
'dijit/_base/popup':function(){
define([
	"dojo/dom-class", // domClass.contains
	"dojo/_base/window",
	"../popup",
	"../BackgroundIframe"	// just loading for back-compat, in case client code is referencing it
], function(domClass, win, popup){

// module:
//		dijit/_base/popup

/*=====
return {
	// summary:
	//		Deprecated.   Old module for popups, new code should use dijit/popup directly.
};
=====*/


// Hack support for old API passing in node instead of a widget (to various methods)
var origCreateWrapper = popup._createWrapper;
popup._createWrapper = function(widget){
	if(!widget.declaredClass){
		// make fake widget to pass to new API
		widget = {
			_popupWrapper: (widget.parentNode && domClass.contains(widget.parentNode, "dijitPopup")) ?
				widget.parentNode : null,
			domNode: widget,
			destroy: function(){},
			ownerDocument: widget.ownerDocument,
			ownerDocumentBody: win.body(widget.ownerDocument)
		};
	}
	return origCreateWrapper.call(this, widget);
};

// Support old format of orient parameter
var origOpen = popup.open;
popup.open = function(/*__OpenArgs*/ args){
	// Convert old hash structure (ex: {"BL": "TL", ...}) of orient to format compatible w/new popup.open() API.
	// Don't do conversion for:
	//		- null parameter (that means to use the default positioning)
	//		- "R" or "L" strings used to indicate positioning for context menus (when there is no around node)
	//		- new format, ex: ["below", "above"]
	//		- return value from deprecated dijit.getPopupAroundAlignment() method,
	//			ex: ["below", "above"]
	if(args.orient && typeof args.orient != "string" && !("length" in args.orient)){
		var ary = [];
		for(var key in args.orient){
			ary.push({aroundCorner: key, corner: args.orient[key]});
		}
		args.orient = ary;
	}

	return origOpen.call(this, args);
};

return popup;
});

},
'xcitec/formx/RISIndicator':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom",
	"dojo/fx",
	"dojo/query",
	"dojo/text!xcitec/formx/templates/RISIndicator.html",
	"xcitec/_StateWidget",
  "../xcitecUtil",
  "../_base/ImageFactory"
], function (dojo, domStyle, array, declare, lang, windowModule, domAttr, domClass, domConstruct, dom, fx, query, template, _StateWidget, xcitecUtil, ImageFactory) {

	var RISIndicator = declare( _StateWidget, {
	    templateString: template,
	
	    baseClass: "stiComboExpendableHead",
	
	    caption: "",
	    captionId: "",
	
	    xcitecRoles: null, // Array
	
	    // STICriterion states: "Open" or "Close", "Open" by default
	    state: "Open",
	
	    iconClass: "",
	
	    commentExists: false,
	
	    bgColor: '',
	
	    postMixInProperties: function() {
	        // search for controls marked with xcitecRole attribute
	        this.xcitecRoles = query(" > [xcitecRole]", this.srcNodeRef);
	        // set iconClass
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        }
	        this.inherited(arguments);
	    },
	
	    _fillContent: function() {
	        var _this = this;
	
	        // move controls in appropriate places if they exist
	        array.forEach(_this.xcitecRoles, function(node) {
	            switch (domAttr.get(node, "xcitecRole").toLowerCase()) {
	                case "result":
	                    domConstruct.place(node, _this.resultHolderNode, "last");
	                    break;
	                case "body1":
	                    domConstruct.place(node, _this.body1HolderNode, "last");
	                    break;
	                case "body2":
	                    domConstruct.place(node, _this.body2HolderNode, "last");
	                    break;
	
	                case "weight":
	                    domConstruct.place(node, _this.weight, "last");
	                    break;
	                case "ratinglabel":
	                    domConstruct.place(node, _this.ratinglabel, "last");
	                    break;
	                case "comment":
	                    domConstruct.place(node, _this.body2HolderNode, "last");
	                    break;
	                case "na":
	                    domConstruct.place(node, _this.na, "last");
	                    break;
	                case "expandostate":
	                    _this.stateHolder = node;
	                    var state = xcitecUtil.str2bool(domAttr.get(_this.stateHolder, "value"));
	                    state ? _this.state = "Open" : _this.state = "Close";
	                    break;
	                default:
	                    break;
	            }
	        });
	        if (!xcitecUtil.str2bool(this.commentExists)) {
	            domConstruct.destroy(this.commentIcon);
	        }
	        this._setStateClass();
	
	        // destroy icon node if no iconClass provided
	        if (!this.iconClass) {
	            domConstruct.destroy(this.statusIcon);
	        }
	        if (this.state == "Close") {
	            domClass.add(this.comboBody, "stiComboExpendableCloseBody")
	        }
	        if (this.stateHolder) {
	            domConstruct.place(this.stateHolder, this.domNode, "first");
	        }
	        if (this.bgColor) {
	            domClass.add(this.domNode, "risIndicator" + this.bgColor.charAt(0).toUpperCase() + this.bgColor.slice(1));
	        }
	        dom.setSelectable(this.stateNode, false);
	
		    // set caption
			if(this.caption != ""){
				this.captionNode.appendChild(windowModule.doc.createTextNode(this.caption));
			}
	
	        this.inherited(arguments);
	
		    this.containerNode=this.domNode;
	    },
	
	    _onClick: function() {
	        switch (this.state) {
	            case "Close":
	                this.expand();
	                break;
	            case "Open":
	                this.collapse();
	                break;
	            default:
	                break;
	        }
	    },
	
	    expand: function() {
	        this.state = "Open";
	        domStyle.set(this.comboBody, "display", "none");
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "true");
	        }
	        if (domClass.contains(this.comboBody, "stiComboExpendableCloseBody")) {
	            domClass.remove(this.comboBody, "stiComboExpendableCloseBody");
	        }
	        fx.wipeIn({ node: this.comboBody, beforeBegin: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    collapse: function() {
	        this.state = "Close";
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "false");
	        }
	        fx.wipeOut({ node: this.comboBody, onEnd: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    getState: function() {
	        return this.state;
	    }
	});
	return RISIndicator;
});

},
'xcitec/PopupMenuItem':function(){
define([
	"dojo/_base/declare",
	"dijit/PopupMenuItem"
], function (declare, PopupMenuItem) {

	var PopupMenuItem = declare( PopupMenuItem, {
	
	});
	return PopupMenuItem;
});
},
'xcitec/widgets/SyncSupplierWidget':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/text!xcitec/widgets/templates/SyncSupplierWidget.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, declare, domAttr, template, _Templated, _Widget) {

	var SyncSupplierWidget = declare( [_Widget, _Templated], {
	    templateString: template,
	    postCreate: function() {
	        domAttr.set(this.SynchronizedSupplierFrame, "src", require.toUrl("external/SynchronizedSupplier.html").toString());
	    }
	});
	return SyncSupplierWidget;
});

},
'dijit/registry':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/sniff", // has("ie")
	"dojo/_base/window", // win.body
	"./main"	// dijit._scopeName
], function(array, has, win, dijit){

	// module:
	//		dijit/registry

	var _widgetTypeCtr = {}, hash = {};

	var registry =  {
		// summary:
		//		Registry of existing widget on page, plus some utility methods.

		// length: Number
		//		Number of registered widgets
		length: 0,

		add: function(widget){
			// summary:
			//		Add a widget to the registry. If a duplicate ID is detected, a error is thrown.
			// widget: dijit/_WidgetBase
			//		Any dijit/_WidgetBase subclass.
			if(hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from the registry. Does not destroy the widget; simply
			//		removes the reference.
			if(hash[id]){
				delete hash[id];
				this.length--;
			}
		},

		byId: function(/*String|Widget*/ id){
			// summary:
			//		Find a widget by it's id.
			//		If passed a widget then just returns the widget.
			return typeof id == "string" ? hash[id] : id;	// dijit/_WidgetBase
		},

		byNode: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget corresponding to the given DOMNode
			return hash[node.getAttribute("widgetId")]; // dijit/_WidgetBase
		},

		toArray: function(){
			// summary:
			//		Convert registry into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	array.map(registry.toArray(), function(w){ return w.domNode; });

			var ar = [];
			for(var id in hash){
				ar.push(hash[id]);
			}
			return ar;	// dijit/_WidgetBase[]
		},

		getUniqueId: function(/*String*/widgetType){
			// summary:
			//		Generates a unique id for a given widgetType

			var id;
			do{
				id = widgetType + "_" +
					(widgetType in _widgetTypeCtr ?
						++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
			}while(hash[id]);
			return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
		},

		findWidgets: function(root, skipNode){
			// summary:
			//		Search subtree under root returning widgets found.
			//		Doesn't search for nested widgets (ie, widgets inside other widgets).
			// root: DOMNode
			//		Node to search under.
			// skipNode: DOMNode
			//		If specified, don't search beneath this node (usually containerNode).

			var outAry = [];

			function getChildrenHelper(root){
				for(var node = root.firstChild; node; node = node.nextSibling){
					if(node.nodeType == 1){
						var widgetId = node.getAttribute("widgetId");
						if(widgetId){
							var widget = hash[widgetId];
							if(widget){	// may be null on page w/multiple dojo's loaded
								outAry.push(widget);
							}
						}else if(node !== skipNode){
							getChildrenHelper(node);
						}
					}
				}
			}

			getChildrenHelper(root);
			return outAry;
		},

		_destroyAll: function(){
			// summary:
			//		Code to destroy all widgets and do other cleanup on page unload

			// Clean up focus manager lingering references to widgets and nodes
			dijit._curFocus = null;
			dijit._prevFocus = null;
			dijit._activeStack = [];

			// Destroy all the widgets, top down
			array.forEach(registry.findWidgets(win.body()), function(widget){
				// Avoid double destroy of widgets like Menu that are attached to <body>
				// even though they are logically children of other widgets.
				if(!widget._destroyed){
					if(widget.destroyRecursive){
						widget.destroyRecursive();
					}else if(widget.destroy){
						widget.destroy();
					}
				}
			});
		},

		getEnclosingWidget: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
			//		the node is not contained within the DOM tree of any widget
			while(node){
				var id = node.nodeType == 1 && node.getAttribute("widgetId");
				if(id){
					return hash[id];
				}
				node = node.parentNode;
			}
			return null;
		},

		// In case someone needs to access hash.
		// Actually, this is accessed from WidgetSet back-compatibility code
		_hash: hash
	};

	dijit.registry = registry;

	return registry;
});

},
'xcitec/ReportBox':function(){
define([
         "dojo/dom-construct",
         "dojo/has",
         "dojo/_base/declare",
         "dijit/_Container",
         "dijit/_TemplatedMixin",
         "dijit/_WidgetBase",
         "./_EnterKeySubmit",
         "./Text"
       ], function ( domConstruct, has, declare, _Container, _Templated, _Widget, _EnterKeySubmit, Text) {
  return declare([_Widget, _Templated, _Container, _EnterKeySubmit], {
    templateString: '<div class="boxReport" data-dojo-attach-point="containerNode"></div>',
    // url of html to show in iframe
    src: "",

    // text to show if no content provided in src attribute
    emptyContentText: "",

    postCreate: function() {
      this.inherited(arguments);

      if (this.src) {
        // create iframe in box
        var iframe = domConstruct.create("iframe", {src: this.src, frameborder: 'no', width: '100%', scrolling: 'no', marginWidth: '0', marginHeight: '0'}, this.containerNode, "first");

        // connect onload or onreadystatechange events depending on browser
        if (has("ie") && has('ie') != 11) {
          iframe.onreadystatechange = function(){
            iframe.height = iframe.contentWindow.document.body.scrollHeight;
            iframe.width = iframe.contentWindow.document.body.scrollWidth;
          };
        } else {
          iframe.onload = function(){
            iframe.height = iframe.contentWindow.document.body.scrollHeight;
            iframe.width = iframe.contentWindow.document.body.scrollWidth;
          };
        }
      } else {
        if (this.emptyContentText) {
          var text = new Text({ text: this.emptyContentText });
          domConstruct.place(text.domNode, this.containerNode, "first");
        }
      }
    }
  });
});

},
'xcitec/SubsegmentAvatar':function(){
define([
    "dojo/_base/declare",
    "dojo/dnd/Avatar",
    "dojo/_base/window",
	'dojo/dom-attr',
    "dojo/dom-class",
    "dojo/dom-construct"
], function(declare, Avatar, windowModule, domAttr, domClass, domConstruct){
  return declare(Avatar, {

		construct: function(){
			// summary:
			//		constructor function;
			//		it is separate so it can be (dynamically) overwritten in case of need
			this.isA11y = domClass.contains(windowModule.body(),"dijit_a11y");
			var a = domConstruct.create("table", {
					"class": "xcitecDndAvatar",
					style: {
						position: "absolute",
						zIndex:   "1999",
						margin:   "0px"
					}
				}),
				source = this.manager.source, node,
				b = domConstruct.create("tbody", null, a),
				tr = domConstruct.create("tr", null, b),
				td = domConstruct.create("td", null, tr),
				icon = this.isA11y ? domConstruct.create("span", {
							id : "a11yIcon",
							innerHTML : this.manager.copy ? '+' : "<"
						}, td) : null,
				span = domConstruct.create("span", {
	//				innerHTML: source.generateText ? this._generateText() : ""
				}, td),
				k = Math.min(5, this.manager.nodes.length), i = 0;
	        domClass.add(span, 'dijitReset dijitInline avatar');
			// we have to set the opacity on IE only after the node is live
			domAttr.set(tr, {
				"class": "dojoDndAvatarHeader",
				style: {opacity: 0.9}
			});
			this.node = a;
		},

	    	update: function(){
			// summary:
			//		updates the avatar to reflect the current DnD state
			dojo[(this.manager.canDropFlag ? "add" : "remove") + "Class"](this.node, "xcitecDndAvatarCanDrop");
			dojo[(this.manager.canDropFlag ? "remove" : "add") + "Class"](this.node, "xcitecDndAvatarCantDrop");
			if (this.isA11y){
				var icon = dom.byId("a11yIcon");
				var text = '+';   // assume canDrop && copy
				if (this.manager.canDropFlag && !this.manager.copy) {
					text = '< '; // canDrop && move
				}else if (!this.manager.canDropFlag && !this.manager.copy) {
					text = "o"; //!canDrop && move
				}else if(!this.manager.canDropFlag){
					text = 'x';  // !canDrop && copy
				}
				icon.innerHTML=text;
			}
		}
	});
});
},
'xcitec/Line':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dijit/_WidgetBase"
], function (declare, domClass, domConstruct, _Widget) {

	var Line = declare( _Widget, {
	  buildRendering: function(){
	    this.domNode = domConstruct.create('div');
	    domClass.add(this.domNode, "ContentAreaHR");
	  }
	});
	
	return Line;
});

},
'dijit/_base/focus':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/dom", // dom.isDescendant
	"dojo/_base/lang", // lang.isArray
	"dojo/topic", // publish
	"dojo/_base/window", // win.doc win.doc.selection win.global win.global.getSelection win.withGlobal
	"../focus",
	"../selection",
	"../main"	// for exporting symbols to dijit
], function(array, dom, lang, topic, win, focus, selection, dijit){

	// module:
	//		dijit/_base/focus

	var exports = {
		// summary:
		//		Deprecated module to monitor currently focused node and stack of currently focused widgets.
		//		New code should access dijit/focus directly.

		// _curFocus: DomNode
		//		Currently focused item on screen
		_curFocus: null,

		// _prevFocus: DomNode
		//		Previously focused item on screen
		_prevFocus: null,

		isCollapsed: function(){
			// summary:
			//		Returns true if there is no text selected
			return dijit.getBookmark().isCollapsed;
		},

		getBookmark: function(){
			// summary:
			//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
			var sel = win.global == window ? selection : new selection.SelectionManager(win.global);
			return sel.getBookmark();
		},

		moveToBookmark: function(/*Object*/ bookmark){
			// summary:
			//		Moves current selection to a bookmark
			// bookmark:
			//		This should be a returned object from dijit.getBookmark()

			var sel = win.global == window ? selection : new selection.SelectionManager(win.global);
			return sel.moveToBookmark(bookmark);
		},

		getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
			// summary:
			//		Called as getFocus(), this returns an Object showing the current focus
			//		and selected text.
			//
			//		Called as getFocus(widget), where widget is a (widget representing) a button
			//		that was just pressed, it returns where focus was before that button
			//		was pressed.   (Pressing the button may have either shifted focus to the button,
			//		or removed focus altogether.)   In this case the selected text is not returned,
			//		since it can't be accurately determined.
			//
			// menu: dijit/_WidgetBase|{domNode: DomNode} structure
			//		The button that was just pressed.  If focus has disappeared or moved
			//		to this button, returns the previous focus.  In this case the bookmark
			//		information is already lost, and null is returned.
			//
			// openedForWindow:
			//		iframe in which menu was opened
			//
			// returns:
			//		A handle to restore focus/selection, to be passed to `dijit.focus`
			var node = !focus.curNode || (menu && dom.isDescendant(focus.curNode, menu.domNode)) ? dijit._prevFocus : focus.curNode;
			return {
				node: node,
				bookmark: node && (node == focus.curNode) && win.withGlobal(openedForWindow || win.global, dijit.getBookmark),
				openedForWindow: openedForWindow
			}; // Object
		},

		// _activeStack: dijit/_WidgetBase[]
		//		List of currently active widgets (focused widget and it's ancestors)
		_activeStack: [],

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the `<iframe>` itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle to pass to unregisterIframe()
			return focus.registerIframe(iframe);
		},

		unregisterIframe: function(/*Object*/ handle){
			// summary:
			//		Unregisters listeners on the specified iframe created by registerIframe.
			//		After calling be sure to delete or null out the handle itself.
			// handle:
			//		Handle returned by registerIframe()

			handle && handle.remove();
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle to pass to unregisterWin()

			return focus.registerWin(targetWindow, effectiveNode);
		},

		unregisterWin: function(/*Handle*/ handle){
			// summary:
			//		Unregisters listeners on the specified window (either the main
			//		window or an iframe's window) according to handle returned from registerWin().
			//		After calling be sure to delete or null out the handle itself.

			handle && handle.remove();
		}
	};

	// Override focus singleton's focus function so that dijit.focus()
	// has backwards compatible behavior of restoring selection (although
	// probably no one is using that).
	focus.focus = function(/*Object|DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			focus._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && win.withGlobal(openedForWindow || win.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				win.withGlobal(openedForWindow || win.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	};

	// For back compatibility, monitor changes to focused node and active widget stack,
	// publishing events and copying changes from focus manager variables into dijit (top level) variables
	focus.watch("curNode", function(name, oldVal, newVal){
		dijit._curFocus = newVal;
		dijit._prevFocus = oldVal;
		if(newVal){
			topic.publish("focusNode", newVal);	// publish
		}
	});
	focus.watch("activeStack", function(name, oldVal, newVal){
		dijit._activeStack = newVal;
	});

	focus.on("widget-blur", function(widget, by){
		topic.publish("widgetBlur", widget, by);	// publish
	});
	focus.on("widget-focus", function(widget, by){
		topic.publish("widgetFocus", widget, by);	// publish
	});

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'xcitec/layout/Chapter2Columns':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"./Chapter",
  "./ContentArea",
  "./ContentAreaPane"
], function (array, declare, domClass, domConstruct, query, _Templated, _Widget, Chapter, ContentArea, ContentAreaPane) {

	var Chapter2Columns = declare( [_Widget, _Templated], {
	    templateString: '<div data-dojo-attach-point="containerNode"></div>',
	
	    // nodes to be placed in the first column (Array)
	    firstColumnControls: null,
	
	    // nodes to be placed in the second column (Array)
	    secondColumnControls: null,
	
	    // order of elements in each column (Array)
	    firstColumnOrder: null,
	    secondColumnOrder: null,
	
	    // references to columns
	    pane1: null,
	    pane2: null,
	
	    background: "",
	
	    hasDelimeter: false,
	
		postMixInProperties: function(){
			this.firstColumnControls = [];
			this.secondColumnControls = [];
			this.firstColumnOrder = [];
			this.secondColumnOrder = [];
			this.inherited(arguments);
		},
	
	    _fillContent: function() {
	
	        if (this.background != "") {
	            switch (this.background.toLowerCase()) {
	                case "grey":
	                    domClass.add(this.domNode, "chapterBgGrey");
	                    break;
	                case "blue":
	                    domClass.add(this.domNode, "chapterBgBlue");
	                    break;
	                case "red":
	                    domClass.add(this.domNode, "chapterBgRed");
	                    break;
	                default:
	                    domClass.add(this.domNode, "chapterBgGrey");
	                    break;
	            }
	        }
	        
	        this._startupWidgets = this._startupWidgets || [];
	
	        var children = query(">", this.srcNodeRef);
	        var _this = this;
	        // collecting nodes
	        var currentColumnArray = [];
	        array.forEach(children, function(child) {
	            if (domClass.contains(child, "delimeter")) {
	                _this.firstColumnControls = currentColumnArray;
	                currentColumnArray = [];
	                _this.hasDelimeter = true;
	                domConstruct.destroy(child);
	            } else {
	                currentColumnArray.push(child);
	            }
	        });
	        if (this.hasDelimeter) {
	            this.secondColumnControls = currentColumnArray;
	        } else {
	            this.firstColumnControls = currentColumnArray;
	        }
	
	        // create markup for each column
	        var contentAreaPane1 = domConstruct.create("div", { 'data-dojo-type': 'xcitec/layout/ContentAreaPane' });
	        array.forEach(this.firstColumnControls, function(node) {
	            var div = domConstruct.create("div");
	            div.appendChild(node);
	            var chapter = new Chapter({}, div);
	            contentAreaPane1.appendChild(chapter.domNode);
	        });
	        
	        var contentAreaPane2 = domConstruct.create("div", { 'data-dojo-type': 'xcitec/layout/ContentAreaPane' });
	        array.forEach(this.secondColumnControls, function(node) {
	            var div = domConstruct.create("div");
	            div.appendChild(node);
	            var chapter = new Chapter({}, div);
	            contentAreaPane2.appendChild(chapter.domNode);
	        });
	
	        // create container for columns
	        var container = domConstruct.create("div");
	        container.appendChild(contentAreaPane1);
	        container.appendChild(contentAreaPane2);
	        var contentArea = new ContentArea({}, container);
	
	        // actually crerate column's objects
	        this.pane1 = new ContentAreaPane({columnOrder: this.firstColumnOrder, showDelimiter: false }, contentAreaPane1);
	        this.pane2 = new ContentAreaPane({columnOrder: this.secondColumnOrder, showDelimiter: false }, contentAreaPane2);
	        this._startupWidgets.push(contentArea);
	        this._startupWidgets.push(this.pane1);
	        this._startupWidgets.push(this.pane2);
	
	        this.containerNode.appendChild(contentArea.domNode);
	        this.inherited(arguments);
	    }
	});
	return Chapter2Columns;
});

},
'dijit/form/_FormMixin':function(){
define([
	"dojo/_base/array", // array.every array.filter array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/on",
	"dojo/window" // winUtils.scrollIntoView
], function(array, declare, kernel, lang, on, winUtils){

	// module:
	//		dijit/form/_FormMixin

	return declare("dijit.form._FormMixin", null, {
		// summary:
		//		Mixin for containers of form widgets (i.e. widgets that represent a single value
		//		and can be children of a `<form>` node or `dijit/form/Form` widget)
		// description:
		//		Can extract all the form widgets
		//		values and combine them into a single javascript object, or alternately
		//		take such an object and set the values for all the contained
		//		form widgets

	/*=====
		// value: Object
		//		Name/value hash for each child widget with a name and value.
		//		Child widgets without names are not part of the hash.
		//
		//		If there are multiple child widgets w/the same name, value is an array,
		//		unless they are radio buttons in which case value is a scalar (since only
		//		one radio button can be checked at a time).
		//
		//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
		//
		//		Example:
		//	|	{ name: "John Smith", interests: ["sports", "movies"] }
	=====*/

		// state: [readonly] String
		//		Will be "Error" if one or more of the child widgets has an invalid value,
		//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
		//		which indicates that the form is ready to be submitted.
		state: "",

		// TODO:
		//	* Repeater
		//	* better handling for arrays.  Often form elements have names with [] like
		//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])


		_getDescendantFormWidgets: function(/*dijit/_WidgetBase[]?*/ children){
			// summary:
			//		Returns all form widget descendants, searching through non-form child widgets like BorderContainer
			var res = [];
			array.forEach(children || this.getChildren(), function(child){
				if("value" in child){
					res.push(child);
				}else{
					res = res.concat(this._getDescendantFormWidgets(child.getChildren()));
				}
			}, this);
			return res;
		},

		reset: function(){
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features:
			//
			//		1. it will highlight any sub-widgets that are not valid
			//		2. it will call focus() on the first invalid sub-widget
			var didFocus = false;
			return array.every(array.map(this._getDescendantFormWidgets(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					winUtils.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
				return valid;
			}), function(item){ return item; });
		},

		setValues: function(val){
			kernel.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = lang.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				values = [].concat(values);
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					array.forEach(widgets, function(w){
						w.set('value', array.indexOf(values, w._get('value')) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					array.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 *	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			array.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							array.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						array.forEach(element.options, function(option){
							option.selected = array.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						array.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
			});
			*/

			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			kernel.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			//		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the defer(...) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							lang.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = lang.getObject(name, false, obj);
							if(value === undefined){
								lang.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=lang.getObject(name, false, obj);
						if(!ary){
							ary=[];
							lang.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=lang.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(lang.isArray(prev)){
							prev.push(value);
						}else{
							lang.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						lang.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also domForm.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			array.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					}else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					}else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					}else{
						// can not set value when there is no name
					}
				}else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				}else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

		isValid: function(){
			// summary:
			//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(/*Boolean*/ /*===== isValid =====*/){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = array.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return array.indexOf(states, "Error") >= 0 ? "Error" :
				array.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Deprecated method.   Applications no longer need to call this.   Remove for 2.0.
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			// TODO: rename for 2.0

			this._descendants = this._getDescendantFormWidgets();

			// To get notifications from children they need to be started.   Children didn't used to need to be started,
			// so for back-compat, start them here
			array.forEach(this._descendants, function(child){
				if(!child._started){ child.startup(); }
			});

			if(!inStartup){
				this._onChildChange();
			}
		},

		_onChildChange: function(/*String*/ attr){
			// summary:
			//		Called when child's value or disabled state changes

			// The unit tests expect state update to be synchronous, so update it immediately.
			if(!attr || attr == "state" || attr == "disabled"){
				this._set("state", this._getState());
			}

			// Use defer() to collapse value changes in multiple children into a single
			// update to my value.   Multiple updates will occur on:
			//	1. Form.set()
			//	2. Form.reset()
			//	3. user selecting a radio button (which will de-select another radio button,
			//		 causing two onChange events)
			if(!attr || attr == "value" || attr == "disabled" || attr == "checked"){
				if(this._onChangeDelayTimer){
					this._onChangeDelayTimer.remove();
				}
				this._onChangeDelayTimer = this.defer(function(){
					delete this._onChangeDelayTimer;
					this._set("value", this.get("value"));
				}, 10);
			}
		},

		startup: function(){
			this.inherited(arguments);

			// Set initial this.value and this.state.   Don't emit watch() notifications.
			this._descendants = this._getDescendantFormWidgets();
			this.value = this.get("value");
			this.state = this._getState();

			// Initialize value and valid/invalid state tracking.
			var self = this;
			this.own(
				on(
					this.containerNode,
					"attrmodified-state, attrmodified-disabled, attrmodified-value, attrmodified-checked",
					function(evt){
						if(evt.target == self.domNode){
							return;	// ignore events that I fire on myself because my children changed
						}
						self._onChildChange(evt.type.replace("attrmodified-", ""));
					}
				)
			);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.inherited(arguments);
		}

	});
});

},
'xcitec/Dialog':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom",
	"dojo/dom-class",
	"dojo/dom-style",
  "dojo/dom-geometry",
  "dojo/ready",
  "dojo/window",
	"dijit/Dialog",
	"dijit/_WidgetBase",
	"dijit/a11y"
], function (dojo, declare, lang, windowModule, dom, domClass, domStyle, geometry, ready, dWindow, Dialog, _Widget, a11y) {

	var Dialog = declare( Dialog, {
		//@Override
		//      we don't need to refocusing
		refocus: false,
	
		//@Override
		//      we don't need to autofocusing
		//autofocus: false,
	
		// showOnLoad: Boolean
		// 		True if Dialog is should be displayed on screen after dom.
	    showOnLoad: false,
	
		attributeMap: lang.delegate(_Widget.prototype.attributeMap, {
			title: { node: "titleBar", type: "attribute" },
			"aria-describedby":""
		}),
	
		_fillContent: function(){
			//set title
		    if(this.title != ""){
				this.titleNode.innerHTML = "";
				this.titleNode.appendChild(windowModule.doc.createTextNode(this.title));
			}
			this.inherited(arguments);
		},
	
		//@Override
		//      process own new attributes
	    postCreate: function() {
	        this.inherited(arguments);
	        domClass.add(this.domNode, "xcitecDialog");
	//	    var viewport = dojo.window.getBox();
	//	    var w = Math.floor(viewport.w * 0.75);
	//	    dojo.style(this.domNode, {width: w + "px"});
	
	        var xcitec_form = dom.byId("xcitec_form");
	        if (xcitec_form) {
	            xcitec_form.appendChild(this.domNode);
	        }
	        if (this.showOnLoad) {
                ready(lang.hitch(this, function() {
	                this.show();
	            }));
	        }
	    },
	
		//@Override
		//      we don't need single children container behavior
	    _checkIfSingleChild: function() {
	        //overide default behavior
	    },
	
		//@Override
		//      we don't need scrollig inside dialog
		_size: function() {
		     this.inherited(arguments);
		     domStyle.set(this.containerNode, { height: "auto" });
		 },
	
		//@Override
		//      we don't need move dialog when scrolling window
		layout: function() {
		     // summary:
		     //		Position the Dialog and the underlay
		     // tags:
		     //		private
		     if (this.domNode.style.display != "none") {
		         if (dijit._underlay) {    // avoid race condition during show()
		             dijit._underlay.layout();
		         }
				//this._position();
		     }
		 },
	
		//@Override
		//      should avoid showing dialog outside viewport
	    _position: function(){
	        // summary:
	        //		Position modal dialog in the viewport. If no relative offset
	        //		in the viewport has been determined (by dragging, for instance),
	        //		center the node. Otherwise, use the Dialog's stored relative offset,
	        //		and position the node to top: left: values based on the viewport.
	        // tags:
	        //		private
	        if(!domClass.contains(windowModule.body(),"dojoMove")){
		        var node = this.domNode,
					viewport = dWindow.getBox(),
					p = this._relativePosition,
					bb = p ? null : geometry.position(node),
	                l = Math.max(0, Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2))),
	                t = Math.max(0, Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2)))
	            ;
	            domStyle.set(node, {
	                left: l + "px",
	                top: t + "px"
	            });
	        }
	    },
	
	    //@Override
	    //      calculate and remember list of navigable items
	    //      can be an issue when _firstFocusItem are changed
	    //      (at least so mentioned in sijit.Dialog)
	    //      todo review dijit._getTabNavigable
	    _getFocusItems: function(){
	        if(this._navElems){ return; }
	        var elems = a11y._getTabNavigable(this.containerNode);
	        this._navElems = elems;
	        this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
	        this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
	
	        //this.inherited(arguments);
		},
	
	    //@Override
	    //      we need to clear stored state of navigable elements
	    show: function(){
	        if(this._navelems){ delete this._navElems; }
	        var promise = this.inherited(arguments);
			this._modalconnects.splice(0, 1)[0].remove(); //fix to scroll dialog (dialog position not fixed)
			return promise;
	    },
	
	    //@Override
		//      we should avoid standard behavior for hidding dialog
	    hide: function(/*Boolean?*/ hidding) {
	        // we need to overide default behavior - we don't need hide by ESC
	        if (hidding) {
	            this.inherited(arguments);
	        }
	        //do nothing we are reloading page or waitng AJAX response!!!
	    },
	
	    xcitecHide: function() {
			// summary:
			//		Hide the dialog
	        this.hide(true);
	    }
	
	});
	
	return Dialog;
});

},
'xcitec/xlayout/QualificationAvatar':function(){
define([
         "dojo/_base/declare",
         "dojo/_base/window",
         "dojo/dom",
         "dojo/dnd/Avatar",
         "dojo/dom-attr",
         "dojo/dom-class",
         "dojo/dom-construct"
       ], function (declare, windowModule, dom, Avatar, domAttr, domClass, domConstruct) {
  return declare(Avatar, {
    construct: function(){

      // summary:
      //		constructor function;
      //		it is separate so it can be (dynamically) overwritten in case of need
      this.isA11y = domClass.contains(windowModule.body(),"dijit_a11y");
      var a = domConstruct.create("table", {
            "class": "xcitecDndAvatar",
            style: {
              position: "absolute",
              zIndex:   "1999",
              margin:   "0px"
            }
          }),
          source = this.manager.source, node,
          b = domConstruct.create("tbody", null, a),
          tr = domConstruct.create("tr", null, b),
          td = domConstruct.create("td", null, tr),
          icon = this.isA11y ? domConstruct.create("span", {
            id : "a11yIcon",
            innerHTML : this.manager.copy ? '+' : "<"
          }, td) : null,
          span = domConstruct.create("span", {
            //				innerHTML: source.generateText ? this._generateText() : ""
          }, td),
          k = Math.min(5, this.manager.nodes.length), i = 0;
      domClass.add(span, 'dijitReset dijitInline avatar');
      // we have to set the opacity on IE only after the node is live
      domAttr.set(tr, {
        "class": "dojoDndAvatarHeader",
        style: {opacity: 0.9}
      });
      /*
       for(; i < k; ++i){
       if(source.creator){
       // create an avatar representation of the node
       node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
       }else{
       // or just clone the node and hope it works
       node = this.manager.nodes[i].cloneNode(true);
       if(node.tagName.toLowerCase() == "tr"){
       // insert extra table nodes
       var table = dojo.create("table"),
       tbody = dojo.create("tbody", null, table);
       tbody.appendChild(node);
       node = table;
       }
       }
       node.id = "";
       tr = dojo.create("tr", null, b);
       td = dojo.create("td", null, tr);
       td.appendChild(node);
       dojo.attr(tr, {
       "class": "dojoDndAvatarItem",
       style: {opacity: (9 - i) / 10}
       });
       }
       */


      this.node = a;
    },

    update: function(){
      // summary:
      //		updates the avatar to reflect the current DnD state
      dojo[(this.manager.canDropFlag ? "add" : "remove") + "Class"](this.node, "xcitecDndAvatarCanDrop");
      dojo[(this.manager.canDropFlag ? "remove" : "add") + "Class"](this.node, "xcitecDndAvatarCantDrop");
      if (this.isA11y){
        var icon = dom.byId("a11yIcon");
        var text = '+';   // assume canDrop && copy
        if (this.manager.canDropFlag && !this.manager.copy) {
          text = '< '; // canDrop && move
        }else if (!this.manager.canDropFlag && !this.manager.copy) {
          text = "o"; //!canDrop && move
        }else if(!this.manager.canDropFlag){
          text = 'x';  // !canDrop && copy
        }
        icon.innerHTML=text;
      }
      // replace text
      /*
       dojo.query(("tr.dojoDndAvatarHeader td span" +(this.isA11y ? " span" : "")), this.node).forEach(
       function(node){
       node.innerHTML = this._generateText();
       }, this);
       */


    }
  });
});

},
'dijit/BackgroundIframe':function(){
define([
	"require",			// require.toUrl
	"./main",	// to export dijit.BackgroundIframe
	"dojo/_base/config",
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang", // lang.extend lang.hitch
	"dojo/on",
	"dojo/sniff" // has("ie"), has("mozilla"), has("quirks")
], function(require, dijit, config, domConstruct, domStyle, lang, on, has){

	// module:
	//		dijit/BackgroundIFrame

	// Flag for whether to create background iframe behind popups like Menus and Dialog.
	// A background iframe is useful to prevent problems with popups appearing behind applets/pdf files,
	// and is also useful on older versions of IE (IE6 and IE7) to prevent the "bleed through select" problem.
	// TODO: For 2.0, make this false by default.  Also, possibly move definition to has.js so that this module can be
	// conditionally required via  dojo/has!bgIfame?dijit/BackgroundIframe
	has.add("config-bgIframe", !has("touch"));

	// TODO: remove _frames, it isn't being used much, since popups never release their
	// iframes (see [22236])
	var _frames = new function(){
		// summary:
		//		cache of iframes

		var queue = [];

		this.pop = function(){
			var iframe;
			if(queue.length){
				iframe = queue.pop();
				iframe.style.display="";
			}else{
				// transparency needed for DialogUnderlay and for tooltips on IE (to see screen near connector)
				if(has("ie") < 9){
					var burl = config["dojoBlankHtmlUrl"] || require.toUrl("dojo/resources/blank.html") || "javascript:\"\"";
					var html="<iframe src='" + burl + "' role='presentation'"
						+ " style='position: absolute; left: 0px; top: 0px;"
						+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
					iframe = document.createElement(html);
				}else{
					iframe = domConstruct.create("iframe");
					iframe.src = 'javascript:""';
					iframe.className = "dijitBackgroundIframe";
					iframe.setAttribute("role", "presentation");
					domStyle.set(iframe, "opacity", 0.1);
				}
				iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			}
			return iframe;
		};

		this.push = function(iframe){
			iframe.style.display="none";
			queue.push(iframe);
		}
	}();


	dijit.BackgroundIframe = function(/*DomNode*/ node){
		// summary:
		//		For IE/FF z-index shenanigans. id attribute is required.
		//
		// description:
		//		new dijit.BackgroundIframe(node).
		//
		//		Makes a background iframe as a child of node, that fills
		//		area (and position) of node

		if(!node.id){ throw new Error("no id"); }
		if(has("config-bgIframe")){
			var iframe = (this.iframe = _frames.pop());
			node.appendChild(iframe);
			if(has("ie")<7 || has("quirks")){
				this.resize(node);
				this._conn = on(node, 'resize', lang.hitch(this, "resize", node));
			}else{
				domStyle.set(iframe, {
					width: '100%',
					height: '100%'
				});
			}
		}
	};

	lang.extend(dijit.BackgroundIframe, {
		resize: function(node){
			// summary:
			//		Resize the iframe so it's the same size as node.
			//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
			if(this.iframe){
				domStyle.set(this.iframe, {
					width: node.offsetWidth + 'px',
					height: node.offsetHeight + 'px'
				});
			}
		},
		destroy: function(){
			// summary:
			//		destroy the iframe
			if(this._conn){
				this._conn.remove();
				this._conn = null;
			}
			if(this.iframe){
				_frames.push(this.iframe);
				delete this.iframe;
			}
		}
	});

	return dijit.BackgroundIframe;
});

},
'dijit/_MenuBase':function(){
define([
	"dojo/_base/array", // array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant domClass.replace
	"dojo/dom-attr",
	"dojo/dom-class", // domClass.replace
	"dojo/_base/lang", // lang.hitch
	"dojo/mouse", // mouse.enter, mouse.leave
	"dojo/on",
	"dojo/window",
	"./a11yclick",
	"./registry",
	"./_Widget",
	"./_CssStateMixin",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(array, declare, dom, domAttr, domClass, lang, mouse, on, winUtils, a11yclick,
			registry, _Widget, _CssStateMixin, _KeyNavContainer, _TemplatedMixin){

	// module:
	//		dijit/_MenuBase

	return declare("dijit._MenuBase", [_Widget, _TemplatedMixin, _KeyNavContainer, _CssStateMixin], {
		// summary:
		//		Abstract base class for Menu and MenuBar.
		//		Subclass should implement _onUpArrow(), _onDownArrow(), _onLeftArrow(), and _onRightArrow().

		// selected: dijit/MenuItem
		//		Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.
		//		If a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if
		//		this Menu is in passive mode (i.e. hasn't been clicked yet), will be null, because
		//		"selected" is not merely "hovered".
		selected: null,
		_setSelectedAttr: function(item){
			if(this.selected != item){
				if(this.selected){
					this.selected._setSelected(false);
					this._onChildDeselect(this.selected);
				}
				if(item){
					item._setSelected(true);
				}
				this._set("selected", item);
			}
		},

		// activated: [readonly] Boolean
		//		This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,
		//		so mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the
		//		menu is not active (ie no dropdown) until an item is clicked.
		activated: false,
		_setActivatedAttr: function(val){
			domClass.toggle(this.domNode, "dijitMenuActive", val);
			domClass.toggle(this.domNode, "dijitMenuPassive", !val);
			this._set("activated", val);
		},

		// parentMenu: [readonly] Widget
		//		pointer to menu that displayed me
		parentMenu: null,

		// popupDelay: Integer
		//		After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering
		//		(without clicking) another MenuItem causes that MenuItem's popup to automatically open.
		popupDelay: 500,

		// passivePopupDelay: Integer
		//		For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause
		//		the popup to open.  Default is Infinity, meaning you need to click the menu to open it.
		passivePopupDelay: Infinity,

		// autoFocus: Boolean
		//		A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar
		//		or DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.
		autoFocus: false,

		childSelector: function(/*DOMNode*/ node){
			// summary:
			//		Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children
			//		like MenuSeparator.  If subclass overrides to a string (ex: "> *"), the subclass must require dojo/query.
			// tags:
			//		protected

			var widget = registry.byNode(node);
			return node.parentNode == this.containerNode && widget && widget.focus;
		},

		postCreate: function(){
			var self = this,
				matches = typeof this.childSelector == "string" ? this.childSelector : lang.hitch(this, "childSelector");
			this.own(
				on(this.containerNode, on.selector(matches, mouse.enter), function(){
					self.onItemHover(registry.byNode(this));
				}),
				on(this.containerNode, on.selector(matches, mouse.leave), function(){
					self.onItemUnhover(registry.byNode(this));
				}),
				on(this.containerNode, on.selector(matches, a11yclick), function(evt){
					self.onItemClick(registry.byNode(this), evt);
					evt.stopPropagation();
					evt.preventDefault();
				})
			);
			this.inherited(arguments);
		},

		onKeyboardSearch: function(/*MenuItem*/ item, /*Event*/ evt, /*String*/ searchString, /*Number*/ numMatches){
			// summary:
			//		Attach point for notification about when a menu item has been searched for
			//		via the keyboard search mechanism.
			// tags:
			//		protected
			this.inherited(arguments);
			if(!!item && (numMatches == -1 || (!!item.popup && numMatches == 1))){
				this.onItemClick(item, evt);
			}
		},

		_keyboardSearchCompare: function(/*dijit/_WidgetBase*/ item, /*String*/ searchString){
			// summary:
			//		Compares the searchString to the widget's text label, returning:
			//		-1: a high priority match and stop searching
			//		 0: no match
			//		 1: a match but keep looking for a higher priority match
			// tags:
			//		private
			if(!!item.shortcutKey){
				// accessKey matches have priority
				return searchString == item.shortcutKey.toLowerCase() ? -1 : 0;
			}
			return this.inherited(arguments) ? 1 : 0; // change return value of -1 to 1 so that searching continues
		},

		onExecute: function(){
			// summary:
			//		Attach point for notification about when a menu item has been executed.
			//		This is an internal mechanism used for Menus to signal to their parent to
			//		close them, because they are about to execute the onClick handler.  In
			//		general developers should not attach to or override this method.
			// tags:
			//		protected
		},

		onCancel: function(/*Boolean*/ /*===== closeAll =====*/){
			// summary:
			//		Attach point for notification about when the user cancels the current menu
			//		This is an internal mechanism used for Menus to signal to their parent to
			//		close them.  In general developers should not attach to or override this method.
			// tags:
			//		protected
		},

		_moveToPopup: function(/*Event*/ evt){
			// summary:
			//		This handles the right arrow key (left arrow key on RTL systems),
			//		which will either open a submenu, or move to the next item in the
			//		ancestor MenuBar
			// tags:
			//		private

			if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
				this.onItemClick(this.focusedChild, evt);
			}else{
				var topMenu = this._getTopMenu();
				if(topMenu && topMenu._isMenuBar){
					topMenu.focusNext();
				}
			}
		},

		_onPopupHover: function(/*Event*/ /*===== evt =====*/){
			// summary:
			//		This handler is called when the mouse moves over the popup.
			// tags:
			//		private

			// if the mouse hovers over a menu popup that is in pending-close state,
			// then stop the close operation.
			// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)

			// highlight the parent menu item pointing to this popup (in case user temporarily moused over another MenuItem)
			this.set("selected", this.currentPopupItem);

			// cancel the pending close (if there is one) (in case user temporarily moused over another MenuItem)
			this._stopPendingCloseTimer();
		},

		onItemHover: function(/*MenuItem*/ item){
			// summary:
			//		Called when cursor is over a MenuItem.
			// tags:
			//		protected

			// Don't do anything unless user has "activated" the menu by:
			//		1) clicking it
			//		2) opening it from a parent menu (which automatically activates it)

			if(this.activated){
				this.set("selected", item);
				if(item.popup && !item.disabled && !this.hover_timer){
					this.hover_timer = this.defer(function(){
						this._openItemPopup(item);
					}, this.popupDelay);
				}
			}else if(this.passivePopupDelay < Infinity){
				if(this.passive_hover_timer){
					this.passive_hover_timer.remove();
				}
				this.passive_hover_timer = this.defer(function(){
					this.onItemClick(item, {type: "click"});
				}, this.passivePopupDelay);
			}

			this._hoveredChild = item;

			item._set("hovering", true);
		},

		_onChildDeselect: function(item){
			// summary:
			//		Called when a child MenuItem becomes deselected.   Setup timer to close its popup.

			this._stopPopupTimer();

			// Setup timer to close all popups that are open and descendants of this menu.
			// Will be canceled if user quickly moves the mouse over the popup.
			if(this.currentPopupItem == item){
				this._stopPendingCloseTimer();
				this._pendingClose_timer = this.defer(function(){
					this._pendingClose_timer = null;
					this.currentPopupItem = null;
					item._closePopup(); // this calls onClose
				}, this.popupDelay);
			}
		},

		onItemUnhover: function(/*MenuItem*/ item){
			// summary:
			//		Callback fires when mouse exits a MenuItem
			// tags:
			//		protected

			if(this._hoveredChild == item){
				this._hoveredChild = null;
			}

			if(this.passive_hover_timer){
				this.passive_hover_timer.remove();
				this.passive_hover_timer = null;
			}

			item._set("hovering", false);
		},

		_stopPopupTimer: function(){
			// summary:
			//		Cancels the popup timer because the user has stop hovering
			//		on the MenuItem, etc.
			// tags:
			//		private

			if(this.hover_timer){
				this.hover_timer = this.hover_timer.remove();
			}
		},

		_stopPendingCloseTimer: function(){
			// summary:
			//		Cancels the pending-close timer because the close has been preempted
			// tags:
			//		private
			if(this._pendingClose_timer){
				this._pendingClose_timer = this._pendingClose_timer.remove();
			}
		},

		_getTopMenu: function(){
			// summary:
			//		Returns the top menu in this chain of Menus
			// tags:
			//		private
			for(var top = this; top.parentMenu; top = top.parentMenu){}
			return top;
		},

		onItemClick: function(/*dijit/_WidgetBase*/ item, /*Event*/ evt){
			// summary:
			//		Handle clicks on an item.
			// tags:
			//		private

			if(this.passive_hover_timer){
				this.passive_hover_timer.remove();
			}

			this.focusChild(item);

			if(item.disabled){
				return false;
			}

			if(item.popup){
				this.set("selected", item);
				this.set("activated", true);
				var byKeyboard = /^key/.test(evt._origType || evt.type) ||
					(evt.clientX == 0 && evt.clientY == 0);	// detects accessKey like ALT+SHIFT+F, where type is "click"
				this._openItemPopup(item, byKeyboard);
			}else{
				// before calling user defined handler, close hierarchy of menus
				// and restore focus to place it was when menu was opened
				this.onExecute();

				// user defined handler for click
				item._onClick ? item._onClick(evt) : item.onClick(evt);
			}
		},

		_openItemPopup: function(/*dijit/MenuItem*/ from_item, /*Boolean*/ focus){
			// summary:
			//		Open the popup to the side of/underneath the current menu item, and optionally focus first item
			// tags:
			//		protected

			if(from_item == this.currentPopupItem){
				// Specified popup is already being shown, so just return
				return;
			}
			if(this.currentPopupItem){
				// If another popup is currently shown, then close it
				this._stopPendingCloseTimer();
				this.currentPopupItem._closePopup();
			}
			this._stopPopupTimer();

			var popup = from_item.popup;
			popup.parentMenu = this;

			// detect mouseover of the popup to handle lazy mouse movements that temporarily focus other menu items\c
			this.own(this._mouseoverHandle = on.once(popup.domNode, "mouseover", lang.hitch(this, "_onPopupHover")));

			var self = this;
			from_item._openPopup({
				parent: this,
				orient: this._orient || ["after", "before"],
				onCancel: function(){ // called when the child menu is canceled
					if(focus){
						// put focus back on my node before focused node is hidden
						self.focusChild(from_item);
					}

					// close the submenu (be sure this is done _after_ focus is moved)
					self._cleanUp();
				},
				onExecute: lang.hitch(this, "_cleanUp", true),
				onClose: function(){
					// Remove handler created by onItemHover
					if(self._mouseoverHandle){
						self._mouseoverHandle.remove();
						delete self._mouseoverHandle;
					}
				}
			}, focus);

			this.currentPopupItem = from_item;

			// TODO: focusing a popup should clear tabIndex on Menu (and it's child MenuItems), so that neither
			// TAB nor SHIFT-TAB returns to the menu.  Only ESC or ENTER should return to the menu.
		},

		onOpen: function(/*Event*/ /*===== e =====*/){
			// summary:
			//		Callback when this menu is opened.
			//		This is called by the popup manager as notification that the menu
			//		was opened.
			// tags:
			//		private

			this.isShowingNow = true;
			this.set("activated", true);
		},

		onClose: function(){
			// summary:
			//		Callback when this menu is closed.
			//		This is called by the popup manager as notification that the menu
			//		was closed.
			// tags:
			//		private

			this.set("activated", false);
			this.set("selected", null);
			this.isShowingNow = false;
			this.parentMenu = null;
		},

		_closeChild: function(){
			// summary:
			//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
			// tags:
			//		private
			this._stopPopupTimer();

			if(this.currentPopupItem){
				// If focus is on a descendant MenuItem then move focus to me,
				// because IE doesn't like it when you display:none a node with focus,
				// and also so keyboard users don't lose control.
				// Likely, immediately after a user defined onClick handler will move focus somewhere
				// else, like a Dialog.
				if(this.focused){
					domAttr.set(this.selected.focusNode, "tabIndex", this.tabIndex);
					this.selected.focusNode.focus();
				}

				// Close all popups that are open and descendants of this menu
				this.currentPopupItem._closePopup();
				this.currentPopupItem = null;
			}
		},

		_onItemFocus: function(/*MenuItem*/ item){
			// summary:
			//		Called when child of this Menu gets focus from:
			//
			//		1. clicking it
			//		2. tabbing into it
			//		3. being opened by a parent menu.
			//
			//		This is not called just from mouse hover.

			if(this._hoveredChild && this._hoveredChild != item){
				this.onItemUnhover(this._hoveredChild);	// any previous mouse movement is trumped by focus selection
			}
			this.set("selected", item);
		},

		_onBlur: function(){
			// summary:
			//		Called when focus is moved away from this Menu and it's submenus.
			// tags:
			//		protected

			this._cleanUp(true);
			this.inherited(arguments);
		},

		_cleanUp: function(/*Boolean*/ clearSelectedItem){
			// summary:
			//		Called when the user is done with this menu.  Closes hierarchy of menus.
			// tags:
			//		private

			this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
			if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
				this.set("activated", false);
			}

			if(clearSelectedItem){
				this.set("selected", null);
			}
		}
	});
});

},
'xcitec/page/MenuItemV2':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/topic",
	"xcitec/page/_MenuItemBase"
], function (declare, domClass, domConstruct, topic, _MenuItemBase) {

	var MenuItemV2 = declare( _MenuItemBase, {
	
		templateString: "<li role='menuitem'><span class='${baseClass}' data-dojo-attach-event='onclick: _onClick, onmouseenter: _onMouse, onmouseleave: _onMouse, onkeypress: _onKeyPress' data-dojo-attach-point='stateNode,containerNode' tabindex='0'></span></li>",
		baseClass: "menuItem",
		url: "",
	
		postCreate: function(){
			if(this.url){
				this.onClick = this.urlClick;
			}
			if(this.iconClass){
				domClass.add(this.stateNode, this.iconClass);
			}
			this.inherited(arguments);
			if(this.highlighted){
				var strongTag = domConstruct.create("strong", {innerHTML: "&nbsp;"}, this.stateNode, "last");
				domClass.add(strongTag, "caution");
			}
		},
	
		urlClick: function(/*Event*/ e){
			window.open(this.url);
		},
	
		_onClick: function(/*Event*/ e){
			if(!this.disabled){
				e.preventDefault(), e.stopPropagation();
				this.onClick(e);
			}
		}
	});
	return MenuItemV2;
});
},
'xcitec/formx/FilterLayout':function(){
define([
         "dojo",
         "dojo/_base/array",
         "dojo/_base/declare",
         "dojo/_base/lang",
         "dojo/dom-attr",
         "dojo/dom-construct",
         "dojo/query",
         "dijit/_TemplatedMixin",
         "dijit/_WidgetBase",
         "dijit/registry",
         "../form/Button",
         "../form/CheckBox",
         "../form/DropDownSelect",
         "../form/MultiSelect",
         "../xcitecUtil"
       ], function (dojo, array, declare, lang, domAttr, domConstruct, query, _Templated, _Widget, registry, Button, CheckBox, DropDownSelect, MultiSelect, xcitecUtil) {

  return declare([_Widget, _Templated], {
    // module:
    //		xcitec/formx/FilterLayout
    // summary:
    //		Special component using for Column/Filter Configuration dialog.
    //		Component allow user to configure options and send them to server
    //		Expect next items marked with dojoAttachPoints:
    //		selectedItemsInput: input filed contains list of selected items
    //		sortDropDownInput: input filed which contains value for sort
    //		listAvailableItemsNode: select with available items
    //		listSelectedItemsNode: select with selected items
    //		sortDropDown: dropdown field for choose sorting item
    //		resetButtonHolderNode: node which contains reset button

    storage: {items:[], fields:{}},

    widgetsInTemplate: true,
    buildRendering: function(){
      var node = this.srcNodeRef || domConstruct.create('div');
      this.domNode = node;
      // recurse through the node, looking for, and attaching to, our
      // attachment points and events, which should be defined on the template node.

      //this._attachTemplateNodes(node);
      this._attachPoints = [];
      _Templated.prototype._attachTemplateNodes.call(this, node);

      // collect nested widgets to process them as own
      // var list = dojo.query("[dojoType]",node);
      this.b1 = query("[data-dojo-attach-event='onClick:_moveItemsRight']", node)[0];
      if(this.b1){
        domAttr.set(this.b1, "id", this.id + ".moveRight");
      }
      this.b2 = query("[data-dojo-attach-event='onClick:_moveItemsLeft']", node)[0];
      if(this.b2){
        domAttr.set(this.b2, "id", this.id + ".moveLeft");
      }
      this.b3 = query("[data-dojo-attach-event='onClick:_moveItemsUp']", node)[0];
      if(this.b3){
        domAttr.set(this.b3, "id", this.id + ".moveUp");
      }
      this.b4 = query("[data-dojo-attach-event='onClick:_moveItemsDown']", node)[0];
      if(this.b4){
        domAttr.set(this.b4, "id", this.id + ".moveDown");
      }

      this._fillContent();
    },

    _fillContent: function(){
      var selectedItems = "";
      if(this.selectedItemsInput){
        this.selectedItemsInput.setAttribute("autocomplete", "off");
        selectedItems = this.selectedItemsInput.value;
      }

      var srcItemsNode = this.listAvailableItemsNode;
      if(srcItemsNode){
        // create storage
        var storage = this.storage;
        query("option", srcItemsNode).forEach(function(n){
          var item = {};
          item.value = n.value;
          item.name = n.innerHTML;
          item.domNode = n;
          storage.fields[item.value] = item;
          storage.items.push(item);
        });

        // collect seteced items
        var tmpSelect = this.listSelectedItemsNode;
        array.forEach(selectedItems.split(","), function(itemid){
          var item = storage.fields[itemid];
          if(item){
            tmpSelect.appendChild(item.domNode);
          }
        });
      }

      this.availableItemsWidget = new MultiSelect({srcNodeRef: this.listAvailableItemsNode, style: "width: 210px!important;", multiply: true, size: 10});
      this.selectedItemsWidget = new MultiSelect({srcNodeRef: this.listSelectedItemsNode, style: "width: 210px!important;", multiply: true, size: 10});
      //    var arr = dojo.parser.instantiate( [
      //      this.listAvailableItemsNode,
      //      this.listSelectedItemsNode
      //    ], {dojoType:"xcitec.form.MultiSelect"});
      //    this.availableItemsWidget = arr[0];
      //    this.selectedItemsWidget = arr[1];
      //    this._supportingWidgets.push(this.availableItemsWidget);
      //    this._supportingWidgets.push(this.selectedItemsWidget);
    },

    startup: function(){
      if(this._started){
        return;
      }
      //startup manually created widgets
      //    this.selectedItemsWidget.startup();
      //    this.availableItemsWidget.startup();

      // fill DropDownSelect with initial options
      if(this.sortDropDown){

        var sortName = domAttr.get(this.sortDropDown, "name");
        this.sort = new DropDownSelect({name: sortName, srcNodeRef: this.sortDropDown, style: "width: 100%;"});
        var sortInput = this.sortDropDownInput;
        var _this = this;
        array.forEach(query("option", this.selectedItemsWidget.domNode), function(n){
          if(!xcitecUtil.str2bool(domAttr.get(n, "notSortable"))){
            _this.sort.appendOption(n.innerHTML, n.value);
          }
        });
        if(this.sortDropDownInput.value){
          var dropDownMenu = _this.sort.getMenuItems();
          //sort.valueNode.value = this.sortDropDownInput.value;
          array.forEach(dropDownMenu, function (node){
            if(node.getAttribute("hiddenValue") == sortInput.value){
              _this.sort._setSelected(node);
            }
          })
        }
        domConstruct.destroy(this.sortDropDownInput);
      }
      var b1widget = registry.byId(this.id + ".moveRight");
      if(b1widget){
        b1widget.onClick = lang.hitch(this, this._moveItemsRight);
      }
      var b2widget = registry.byId(this.id + ".moveLeft");
      if(b2widget){
        b2widget.onClick = lang.hitch(this, this._moveItemsLeft);
      }
      var b3widget = registry.byId(this.id + ".moveUp");
      if(b3widget){
        b3widget.onClick = lang.hitch(this, this._moveItemsUp);
      }
      var b4widget = registry.byId(this.id + ".moveDown");
      if(b4widget){
        b4widget.onClick = lang.hitch(this, this._moveItemsDown);
      }
    },

    _setSelectedItems: function(/*String*/ value){
      if(this.selectedItemsInput){
        this.selectedItemsInput.value = value;
      }
    },

    _updateSelectedItems: function(/*NodeList*/ options){
      var val = options.map(
          function(n){
            return n.value
          }).join(",");
      this._setSelectedItems(val);
    },

    _updateDropDown: function(/*NodeList*/ options, /*String*/ action){
      var _this = this;
      if(action == "remove"){
        var dropDownMenu = this.sort.getMenuItems();
        array.forEach(dropDownMenu, function(n){
          var ident = false;
          array.forEach(options, function(node){
            if(node.value){
              if(node.value == n.getAttribute("hiddenValue")){
                ident = true;
              }
            } else{
              if(node.innerHTML == n.lastChild.innerHTML){
                ident = true;
              }
            }
          });
          if(!ident){
            _this.sort.deleteOption(n.getAttribute("hiddenValue"))
          }
        });
      } else if(action == "append"){
        array.forEach(options, function(node){
          if(!xcitecUtil.str2bool(domAttr.get(node, "notSortable"))){
            _this.sort.appendOption(node.innerHTML, node.value);
          }
        })
      }
    },

    _moveItemsLeft: function(){
      var dijitSelItems = this.selectedItemsWidget;
      var	tmpUnselected = this._tmpUnselect(dijitSelItems);
      this.availableItemsWidget.addSelected(dijitSelItems);
      array.forEach(tmpUnselected, function(option){
        option.selected = true;
      });
      this._updateSelectedItems(dijitSelItems.getOptions());
      if(this.sortDropDown){
        this._updateDropDown(dijitSelItems.getOptions(), "remove");
      }
    },
    _moveItemsRight: function(){
      var dijitSelItems = this.selectedItemsWidget;
      dijitSelItems.addSelected(this.availableItemsWidget);
      this._updateSelectedItems(dijitSelItems.getOptions());
      if(this.sortDropDown){
        this._updateDropDown(dijitSelItems.getOptions(), "append");
      }
    },

    _moveItemsUp: function(){
      var dijitSelItems = this.selectedItemsWidget;
      var tmpUnselected = this._tmpUnselect(dijitSelItems);
      if(!this._checkPrevFixedPos(dijitSelItems)){
        dijitSelItems.moveSelectedUp();
      }
      array.forEach(tmpUnselected, function(option){
        option.selected = true;
      });
      this._updateSelectedItems(dijitSelItems.getOptions());
    },
    _moveItemsDown: function(){
      var dijitSelItems = this.selectedItemsWidget;
      var tmpUnselected = this._tmpUnselect(dijitSelItems);
      dijitSelItems.moveSelectedDown();
      array.forEach(tmpUnselected, function(option){
        option.selected = true;
      });
      this._updateSelectedItems(dijitSelItems.getOptions());
    },

    _checkPrevFixedPos: function(/*MultiSelect*/ dijitSelItems){
      var prevOpt = null,
          items = dijitSelItems.getOptions();
      array.forEach(dijitSelItems.getSelected(), function(option){
        if(items[option.index - 1] && xcitecUtil.str2bool(domAttr.get(items[option.index - 1], "fixedPos"))){
          prevOpt = items[option.index - 1];
        }
      });
      return prevOpt;
    },

    _tmpUnselect: function(/*MultiSelect*/ dijitSelItems){
      var tmpUnselected = [];
      array.forEach(dijitSelItems.getSelected(), function(option){
        if(xcitecUtil.str2bool(domAttr.get(option, "fixedPos"))){
          option.selected = false;
          tmpUnselected.push(option);
        }
      });
      return tmpUnselected;
    },

    destroyDescendants: function(/*Boolean?*/ preserveDom){
      array.forEach(registry.findWidgets(this.domNode), function(widget){
        if(widget.destroyRecursive){
          widget.destroyRecursive(preserveDom);
        }
      });
      this.inherited(arguments);
    }
  });
});

},
'dijit/tree/_dndSelector':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/_base/connect", // connect.isCopyKey
	"dojo/_base/declare", // declare
	"dojo/_base/kernel",	// global
	"dojo/_base/lang", // lang.hitch
	"dojo/dom", // isDescendant
	"dojo/mouse", // mouse.isLeft
	"dojo/on",
	"dojo/touch",
	"../a11yclick",
	"./_dndContainer"
], function(array, connect, declare, kernel, lang, dom, mouse, on, touch, a11yclick, _dndContainer){

	// module:
	//		dijit/tree/_dndSelector

	return declare("dijit.tree._dndSelector", _dndContainer, {
		// summary:
		//		This is a base class for `dijit/tree/dndSource`, and isn't meant to be used directly.
		//		It's based on `dojo/dnd/Selector`.
		// tags:
		//		protected

		/*=====
		// selection: Object
		//		(id to DomNode) map for every TreeNode that's currently selected.
		//		The DOMNode is the TreeNode.rowNode.
		selection: {},
		=====*/

		constructor: function(){
			// summary:
			//		Initialization
			// tags:
			//		private

			this.selection={};
			this.anchor = null;

			this.events.push(
				// listeners setup here but no longer used (left for backwards compatibility
				on(this.tree.domNode, touch.press, lang.hitch(this,"onMouseDown")),
				on(this.tree.domNode, touch.release, lang.hitch(this,"onMouseUp")),

				// listeners used in this module
				on(this.tree.domNode, touch.move, lang.hitch(this,"onMouseMove")),
				on(this.tree.domNode, a11yclick.press, lang.hitch(this,"onClickPress")),
				on(this.tree.domNode, a11yclick.release, lang.hitch(this,"onClickRelease"))
			);
		},

		// singular: Boolean
		//		Allows selection of only one element, if true.
		//		Tree hasn't been tested in singular=true mode, unclear if it works.
		singular: false,

		// methods
		getSelectedTreeNodes: function(){
			// summary:
			//		Returns a list of selected node(s).
			//		Used by dndSource on the start of a drag.
			// tags:
			//		protected
			var nodes=[], sel = this.selection;
			for(var i in sel){
				nodes.push(sel[i]);
			}
			return nodes;
		},

		selectNone: function(){
			// summary:
			//		Unselects all items
			// tags:
			//		private

			this.setSelection([]);
			return this;	// self
		},

		destroy: function(){
			// summary:
			//		Prepares the object to be garbage-collected
			this.inherited(arguments);
			this.selection = this.anchor = null;
		},
		addTreeNode: function(/*dijit/Tree._TreeNode*/ node, /*Boolean?*/isAnchor){
			// summary:
			//		add node to current selection
			// node: Node
			//		node to add
			// isAnchor: Boolean
			//		Whether the node should become anchor.

			this.setSelection(this.getSelectedTreeNodes().concat( [node] ));
			if(isAnchor){ this.anchor = node; }
			return node;
		},
		removeTreeNode: function(/*dijit/Tree._TreeNode*/ node){
			// summary:
			//		remove node and it's descendants from current selection
			// node: Node
			//		node to remove
			var newSelection = array.filter(this.getSelectedTreeNodes(), function(selectedNode){
				return !dom.isDescendant(selectedNode.domNode, node.domNode); // also matches when selectedNode == node
			});
			this.setSelection(newSelection);
			return node;
		},
		isTreeNodeSelected: function(/*dijit/Tree._TreeNode*/ node){
			// summary:
			//		return true if node is currently selected
			// node: Node
			//		the node to check whether it's in the current selection

			return node.id && !!this.selection[node.id];
		},
		setSelection: function(/*dijit/Tree._TreeNode[]*/ newSelection){
			// summary:
			//		set the list of selected nodes to be exactly newSelection. All changes to the
			//		selection should be passed through this function, which ensures that derived
			//		attributes are kept up to date. Anchor will be deleted if it has been removed
			//		from the selection, but no new anchor will be added by this function.
			// newSelection: Node[]
			//		list of tree nodes to make selected
			var oldSelection = this.getSelectedTreeNodes();
			array.forEach(this._setDifference(oldSelection, newSelection), lang.hitch(this, function(node){
				node.setSelected(false);
				if(this.anchor == node){
					delete this.anchor;
				}
				delete this.selection[node.id];
			}));
			array.forEach(this._setDifference(newSelection, oldSelection), lang.hitch(this, function(node){
				node.setSelected(true);
				this.selection[node.id] = node;
			}));
			this._updateSelectionProperties();
		},
		_setDifference: function(xs,ys){
			// summary:
			//		Returns a copy of xs which lacks any objects
			//		occurring in ys. Checks for membership by
			//		modifying and then reading the object, so it will
			//		not properly handle sets of numbers or strings.

			array.forEach(ys, function(y){ y.__exclude__ = true; });
			var ret = array.filter(xs, function(x){ return !x.__exclude__; });

			// clean up after ourselves.
			array.forEach(ys, function(y){ delete y['__exclude__'] });
			return ret;
		},
		_updateSelectionProperties: function(){
			// summary:
			//		Update the following tree properties from the current selection:
			//		path[s], selectedItem[s], selectedNode[s]

			var selected = this.getSelectedTreeNodes();
			var paths = [], nodes = [], selects = [];
			array.forEach(selected, function(node){
				var ary = node.getTreePath(), model = this.tree.model;
				nodes.push(node);
				paths.push(ary);
				ary = array.map(ary, function(item){
					return model.getIdentity(item);
				}, this);
				selects.push(ary.join("/"))
			}, this);
			var items = array.map(nodes,function(node){ return node.item; });
			this.tree._set("paths", paths);
			this.tree._set("path", paths[0] || []);
			this.tree._set("selectedNodes", nodes);
			this.tree._set("selectedNode", nodes[0] || null);
			this.tree._set("selectedItems", items);
			this.tree._set("selectedItem", items[0] || null);
		},

		// selection related events
		onClickPress: function(e){
			// summary:
			//		Event processor for onmousedown/ontouchstart/onkeydown corresponding to a click event
			// e: Event
			//		onmousedown/ontouchstart/onkeydown event
			// tags:
			//		protected

			// ignore mouse or touch on expando node
			if(this.current && this.current.isExpandable && this.tree.isExpandoNode(e.target, this.current)){ return; }

			if(e.type == "mousedown" && mouse.isLeft(e)){
				// Prevent text selection while dragging on desktop, see #16328.   But don't call preventDefault()
				// for mobile because it will break things completely, see #15838.  Also, don't preventDefault() on
				// MSPointerDown or pointerdown events, because that stops the mousedown event from being generated,
				// see #17709.
				// TODO: remove this completely in 2.0.  It shouldn't be needed since dojo/dnd/Manager already
				// calls preventDefault() for the "selectstart" event.  It can also be achieved via CSS:
				// http://stackoverflow.com/questions/826782/css-rule-to-disable-text-selection-highlighting
				e.preventDefault();
			}

			var treeNode = e.type == "keydown" ? this.tree.focusedChild : this.current;

			if(!treeNode){
				// Click must be on the Tree but not on a TreeNode, happens especially when Tree is stretched to fill
				// a pane of a BorderContainer, etc.
				return;
			}

			var copy = connect.isCopyKey(e), id = treeNode.id;

			// if shift key is not pressed, and the node is already in the selection,
			// delay deselection until onmouseup so in the case of DND, deselection
			// will be canceled by onmousemove.
			if(!this.singular && !e.shiftKey && this.selection[id]){
				this._doDeselect = true;
				return;
			}else{
				this._doDeselect = false;
			}
			this.userSelect(treeNode, copy, e.shiftKey);
		},

		onClickRelease: function(e){
			// summary:
			//		Event processor for onmouseup/ontouchend/onkeyup corresponding to a click event
			// e: Event
			//		onmouseup/ontouchend/onkeyup event
			// tags:
			//		protected

			// _doDeselect is the flag to indicate that the user wants to either ctrl+click on
			// an already selected item (to deselect the item), or click on a not-yet selected item
			// (which should remove all current selection, and add the clicked item). This can not
			// be done in onMouseDown, because the user may start a drag after mousedown. By moving
			// the deselection logic here, the user can drag an already selected item.
			if(!this._doDeselect){ return; }
			this._doDeselect = false;
			this.userSelect(e.type == "keyup" ? this.tree.focusedChild : this.current, connect.isCopyKey(e), e.shiftKey);
		},
		onMouseMove: function(/*===== e =====*/){
			// summary:
			//		event processor for onmousemove/ontouchmove
			// e: Event
			//		onmousemove/ontouchmove event
			this._doDeselect = false;
		},

		// mouse/touch events that are no longer used
		onMouseDown: function(){
			// summary:
			//		Event processor for onmousedown/ontouchstart
			// e: Event
			//		onmousedown/ontouchstart event
			// tags:
			//		protected
		},
		onMouseUp: function(){
			// summary:
			//		Event processor for onmouseup/ontouchend
			// e: Event
			//		onmouseup/ontouchend event
			// tags:
			//		protected
		},

		_compareNodes: function(n1, n2){
			if(n1 === n2){
				return 0;
			}

			if('sourceIndex' in document.documentElement){ //IE
				//TODO: does not yet work if n1 and/or n2 is a text node
				return n1.sourceIndex - n2.sourceIndex;
			}else if('compareDocumentPosition' in document.documentElement){ //FF, Opera
				return n1.compareDocumentPosition(n2) & 2 ? 1: -1;
			}else if(document.createRange){ //Webkit
				var r1 = doc.createRange();
				r1.setStartBefore(n1);

				var r2 = doc.createRange();
				r2.setStartBefore(n2);

				return r1.compareBoundaryPoints(r1.END_TO_END, r2);
			}else{
				throw Error("dijit.tree._compareNodes don't know how to compare two different nodes in this browser");
			}
		},

		userSelect: function(node, multi, range){
			// summary:
			//		Add or remove the given node from selection, responding
			//		to a user action such as a click or keypress.
			// multi: Boolean
			//		Indicates whether this is meant to be a multi-select action (e.g. ctrl-click)
			// range: Boolean
			//		Indicates whether this is meant to be a ranged action (e.g. shift-click)
			// tags:
			//		protected

			if(this.singular){
				if(this.anchor == node && multi){
					this.selectNone();
				}else{
					this.setSelection([node]);
					this.anchor = node;
				}
			}else{
				if(range && this.anchor){
					var cr = this._compareNodes(this.anchor.rowNode, node.rowNode),
					begin, end, anchor = this.anchor;

					if(cr < 0){ //current is after anchor
						begin = anchor;
						end = node;
					}else{ //current is before anchor
						begin = node;
						end = anchor;
					}
					var nodes = [];
					//add everything betweeen begin and end inclusively
					while(begin != end){
						nodes.push(begin);
						begin = this.tree._getNext(begin);
					}
					nodes.push(end);

					this.setSelection(nodes);
				}else{
					if( this.selection[ node.id ] && multi ){
						this.removeTreeNode( node );
					}else if(multi){
						this.addTreeNode(node, true);
					}else{
						this.setSelection([node]);
						this.anchor = node;
					}
				}
			}
		},

		getItem: function(/*String*/ key){
			// summary:
			//		Returns the dojo/dnd/Container._Item (representing a dragged node) by it's key (id).
			//		Called by dojo/dnd/Source.checkAcceptance().
			// tags:
			//		protected

			var widget = this.selection[key];
			return {
				data: widget,
				type: ["treeNode"]
			}; // dojo/dnd/Container._Item
		},

		forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
			// summary:
			//		Iterates over selected items;
			//		see `dojo/dnd/Container.forInItems()` for details
			o = o || kernel.global;
			for(var id in this.selection){
				// console.log("selected item id: " + id);
				f.call(o, this.getItem(id), id, this);
			}
		}
	});
});

},
'xcitec/formx/EVAComboFieldContainer':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, declare, dom, _Templated, _Widget) {

	var EVAComboFieldContainer = declare( [_Widget, _Templated], {
	    caption: "",
	    templateString: "<div data-dojo-attach-point='containerNode'>${caption}</div>",
	
	    postCreate: function() {
	        this.inherited(arguments);
	        dom.setSelectable(this.domNode, false);
	    }
	});
	
	return EVAComboFieldContainer;
});

},
'dijit/_OnDijitClickMixin':function(){
define([
	"dojo/on",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/_base/declare", // declare
	"dojo/has", // has("dom-addeventlistener")
	"./a11yclick"
], function(on, array, keys, declare, has, a11yclick){

	// module:
	//		dijit/_OnDijitClickMixin

	var ret = declare("dijit._OnDijitClickMixin", null, {
		// summary:
		//		Deprecated.   New code should access the dijit/a11yclick event directly, ex:
		//		|	this.own(on(node, a11yclick, function(){ ... }));
		//
		//		Mixing in this class will make _WidgetBase.connect(node, "ondijitclick", ...) work.
		//		It also used to be necessary to make templates with ondijitclick work, but now you can just require
		//		dijit/a11yclick.

		connect: function(obj, event, method){
			// override _WidgetBase.connect() to make this.connect(node, "ondijitclick", ...) work
			return this.inherited(arguments, [obj, event == "ondijitclick" ? a11yclick : event, method]);
		}
	});

	ret.a11yclick = a11yclick;	// back compat

	return ret;
});

},
'xcitec/widgets/GoToWidget':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/query",
	"dojo/text!xcitec/widgets/templates/GoToWidget.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"../_base/Scroll",
  "../xcitecUtil",
  "../form/InlineLinkButton"
], function (array, declare, domClass, domStyle, query, template, _Templated, _Widget, registry, Scroll, xcitecUtil, InlineLinkButton) {

	/* GoToWidget is a special widget which aim is to scroll page to the appropriate Box */
	
	var GoToWidget = declare( [_Widget, _Templated], {
	    templateString: template,
	//    baseClass: 'list',
	    baseClass: 'xwidget_goto',
	
	    // box nodes and widgets (Array)
	    boxNodes: null,
	    boxWidgets: null,
	    // array of created InlineLinkButtons
	    links: null,
	    // true if there is an image in box title
	    isImageExist: false,
	
		postMixInProperties: function() {
	        this.boxNodes = [];
	        this.boxWidgets = [];
	        this.links = [];
	        this.inherited(arguments);
	    },
	
	    startup: function() {
	        this.boxWidgets = [];
	        var _this = this;
	
	        // seraching for boxes
	        this.boxNodes = query("[widgetId]");
	        array.forEach(this.boxNodes, function(node) {
	            // find all boxes
	            if (("componentType" in registry.byNode(node)) && (registry.byNode(node).componentType == "box")) {
	                // if this is Cockpit - don't include it in the final array
	                if (!(query("#" + _this.id, node).length > 0)) {
	                    _this.boxWidgets.push(registry.byNode(node));
	                }
	            }
	        });
	
	        // creating widget items according to founded boxes
	        this.links = [];
	        array.forEach(this.boxWidgets, function(widget, index) {
	            //fix: after ajax reload of box -> goto can't find box node. (see SRM-5717)
	            var link = new InlineLinkButton({ text: widget.getCaption(), iconClass: widget.getIconClass(), onClick: function() {Scroll.smoothScroll({ node: registry.byId(widget.id).domNode, win: window, duration: 300 }).play()}, disabled: widget.isDisabled() });
	            if(index < 8){
		            _this.leftCol.appendChild(link.domNode);
	            } else{
		            _this.rightCol.appendChild(link.domNode);
	            }
	            _this.links.push(link.domNode);
	        });
			if(this.links.length<=8){
				domStyle.set(_this.leftCol, "width", "auto");
			}
	
	        // check if there are images and add margin-left if needed
	        array.forEach(this.links, function(link) {
	            if (domStyle.get(query(" > img", link)[0], "display") != 'none') {
	                _this.isImageExist = true;
	            }
	        });
	        if (this.isImageExist) {
	            array.forEach(this.links, function(link) {
	                if (domStyle.get(query(" > img", link)[0], "display") == 'none') {
	                    domClass.add(link, 'noImage')
	                }
	            });
	        }
	        this.inherited(arguments);
	    }
	});
	return GoToWidget;
});

},
'xcitec/form/DropDownButton':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
  	"dojo/dom",
	"dojo/text!./templates/DropDownButton.html",
	"dijit/form/Button",
	"dijit/form/DropDownButton",
	"dijit/form/_FormWidget"
], function (declare, lang, windowModule, dom, template, Button, DropDownButton, _FormWidget) {

	//require dijit.form.DropDownButton which declared in dijit.form.Button   
	var DropDownButton = declare( DropDownButton, {
		templateString: template,
		baseClass: "dijitXcitecButton dijitXcitecDropDownButton",
	
		caption: "",
		captionId: "",
		readOnly: false,
	
		attributeMap: lang.delegate(_FormWidget.prototype.attributeMap, {
			iconClass: { node: "iconNode", type: "class" }
		}),
	
		create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
			// Dijit button use label as button text, Xcitec label using for layout labels
			// - if label in params remove it
			// - if caption in params set "label" to call behavior of Dijit Button
			if("label" in params){
				delete params["label"];
			}
			this.inherited(arguments);
		},
	
		_fillContent: function(/*DomNode*/ source){
			this.containerNode.appendChild(windowModule.doc.createTextNode(this.caption));
			this.dropDownContainer = this.srcNodeRef;
		},
	
		postCreate: function(){
			if(this.disabled){
				// remove the tabIndex, especially for FF for don't get focus by TABing
				this.focusNode.removeAttribute('tabIndex');
			}
			this.inherited(arguments);
      	dom.setSelectable(this.domNode, false);
		},
	
		startup: function(){
			if(this._started){
				return;
			}
			this.inherited(arguments);
	
			if(!this.disabled && this.dropDown){
				this.set("disabled", this.dropDown.hasChildren && !this.dropDown.hasChildren());
			}
		},
	
		_setCaptionAttr: function(/*String*/ content){
			// summary:
			//		Hook for attr('caption', ...) to work.
			// description:
			//		Set the label (text) of the button; takes an HTML string.
			//this._setLabelAttr(content);
		},
	
		_openDropDown: function(){
			this.inherited(arguments);
			this.checked = this._opened;
			this._setStateClass();
		},
	
		_closeDropDown: function(/*Boolean*/ focus){
			this.inherited(arguments);
			this.checked = this._opened;
			this._setStateClass();
		}
	
	});
	
	return DropDownButton;
});

},
'xcitec/form/DropDownSelect':function(){
define([
	"dojo/_base/window",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/dom-attr",
	"dojo/_base/array",
	"dojo/query",
	"dojo/_base/lang",
	"dojo/dom-construct",
	"dojo/_base/declare",
	"dojo/text!./templates/DropDownSelect.html",
	"dojo/dom",
	"dojo/dom-geometry",
	"dojo/keys",
    "dijit/form/_FormValueWidget",
    "dijit/popup",
    "../xcitecUtil",
    "../_base/ImageFactory",
	"./_DropDownMenu",
    "dijit/_WidgetsInTemplateMixin",
    "./_XButtonRight"
], function (windowModule, domClass, domStyle, domAttr, array, query, lang, domConstruct, declare, template, dom, geometry, keys, _FormValueWidget,
             popup, xcitecUtil, ImageFactory, _DropDownMenu, _WidgetsInTemplateMixin) {

	return declare([_FormValueWidget, _WidgetsInTemplateMixin],
	    {
	        widgetsInTemplate: true,
	        options: null,
	        _opened: false,
	        boldItem: null,
	        duplicateItem: false,
	        baseClass: "dijitXcitecTextBox xcitecDropDownSelect",
	        required: false,
	
	        templateString: template,
	
	        // error: String
	        //		Error message if control vaue in invalid state.
	        error: "",
	
	        _fillContent: function() {
	            this.inherited(arguments);
	            this.dropdown = new _DropDownMenu({
	                onChange: lang.hitch(this, this._selectOption)
	            });
	        },
	
	        // returns a list of DOM nodes of menu items
	        getMenuItems: function() {
	            return query(">", this.dropdown.domNode);
	        },
	
	        // dynamically adding option
	        appendOption: function(/*String*/ label, /*String*/ value, /*String*/ iconClass) {
	            var s = this.duplicateItem;
	            query("li", this.dropdown.domNode).forEach(function(node) {
	                if ((node.getAttribute("hiddenValue") == value) && (node.lastChild.innerHTML == label))
	                    s = true;
	            })
	            if (s) {
	                this.duplicateItem = false;
	                return;
	            }
	
	            var item = windowModule.doc.createElement("li");
	
	            if (iconClass) {
	                iconClass = ImageFactory.createIconCss(iconClass);
	                var image = windowModule.doc.createElement("img");
	                domAttr.set(image, "src", this._blankGif);
	                domAttr.set(image, "class", "xcitecDropDownMenuIcon");
	                domClass.add(image, iconClass);
	                item.appendChild(image);
	            }
	            if (label == '') {
	                var text = windowModule.doc.createElement("span");
	                text.innerHTML = "&nbsp";
	                item.appendChild(text);
	            } else if (label) {
	                var text = windowModule.doc.createElement("span");
	                text.innerHTML = label;
	                item.appendChild(text);
	            }
	            if (value == '') {
	                domAttr.set(item, "hiddenValue", "");
	                var optsCount = this.dropdown.getOptionsCount();
	                domAttr.set(item, "id", this.dropdown.domNode.id + optsCount);
	                this.dropdown.setOptionsCount(optsCount + 1);
	            } else if (!value) {
	                domAttr.set(item, "hiddenValue", "");
	                var optsCount = this.dropdown.getOptionsCount();
	                domAttr.set(item, "id", this.dropdown.domNode.id + optsCount);
	                this.dropdown.setOptionsCount(optsCount + 1);
	            } else {
	                domAttr.set(item, "hiddenValue", value);
	                var optsCount = this.dropdown.getOptionsCount();
	                domAttr.set(item, "id", this.dropdown.domNode.id + optsCount);
	                this.dropdown.setOptionsCount(optsCount + 1);
	            }
	            item.className = "dijitReset dijitMenuItem";
	
	            // if adding in an empty DropDownSelect
	            if (!this.dropdown.domNode.firstChild) {
	                this.textbox.innerHTML = item.innerHTML;
	                this.valueNode.value = value;
	                this.boldItem = item;
	                domClass.add(this.boldItem, "dojoxDropDownSelectSelectedOption");
	                this.valueNode.value = domAttr.get(item, "hiddenValue");
	            }
	
	            this.dropdown.domNode.appendChild(item);
	        },
	
	        // dynamically removing option
	        deleteOption: function(/*String*/ value) {
	            var items = query("li", this.dropdown.domNode);
	            var itemsCount = items.length;
	            var optionToRemove = null;
	            if (value == '') {
	                value = "&nbsp;";
	            }
	
	            // searching for the option to remove
	            array.forEach(items, function(item) {
	                // searching by value
	                if (domAttr.get(item, "hiddenValue") == value) {
	                    optionToRemove = item;
	                }
	            })
	            if (!optionToRemove) {
	                // searching by label
	                array.forEach(items, function(item) {
	                    if (item.lastChild.innerHTML == value) {
	                        optionToRemove = item;
	                    }
	                })
	            }
	
	            // actually removing option
	            if (itemsCount == 1) {
	                // if this option is the last
	                this.textbox.innerHTML = "";
	                domConstruct.destroy(optionToRemove);
	                this.valueNode.value = "";
	                return;
	            }
	            if (optionToRemove == this.boldItem) {
	                // if this option is currently active
	                domConstruct.destroy(optionToRemove);
	                this.textbox.innerHTML = this.dropdown.domNode.firstChild.innerHTML;
	                this.boldItem = this.dropdown.domNode.firstChild;
	                domClass.add(this.boldItem, "dojoxDropDownSelectSelectedOption");
	                this.valueNode.value = domAttr.get(this.boldItem, "hiddenValue");
	                return;
	            }
	            domConstruct.destroy(optionToRemove);
	        },
	
	        _setSelected: function(/*DomNode*/ node) {
	            this.textbox.innerHTML = node.innerHTML;
	            if (this.boldItem) {
	                domClass.remove(this.boldItem, "dojoxDropDownSelectSelectedOption");
	                this.boldItem = node
	                domClass.add(this.boldItem, "dojoxDropDownSelectSelectedOption");
	                this.valueNode.value = domAttr.get(this.boldItem, "hiddenValue");
	            }
	        },
	
	        _hideMenu: function() {
	            if (this._opened) {
	                popup.close(this.dropdown);
	                this._opened = false;
	            }
	        },
	
	        _onKeyPress: function(/*Event*/ evt) {
	            // summary:
	            //		Handles keyboard events
	            var key = evt.charOrCode;
	            var dd = this.dropdown;
	            var dk = keys;
	            if (this._opened) {
	                dd.handleKey(key);
	                curr = dd.currentNode;
	            }
	            switch (key) {
	                case dk.PAGE_DOWN:
	                case dk.DOWN_ARROW:
	                    if (!this._opened) {
	                        this._onClick();
	                        this.button.setSelectedStyle(true);
	                    }
	                    break;
	
	                case dk.ENTER:
	                    // prevent submitting form if user presses enter. Also
	                    // prevent accepting the value if either Next or Previous
	                    // are selected
	                    if (this._opened) {
	                        if (curr) {
	                            this.textbox.innerHTML = curr.innerHTML;
	                            if (this.boldItem) {
	                                domClass.remove(this.boldItem, "dojoxDropDownSelectSelectedOption");
	                            }
	                            this.boldItem = curr;
	                            domClass.add(curr, "dojoxDropDownSelectSelectedOption");
	                            this.valueNode.value = domAttr.get(curr, "hiddenValue");
	                        }
	                        this._hideMenu();
	                        this.button.setSelectedStyle(false);
	                        evt.preventDefault(), evt.stopPropagation();
	                        this.onChange();
	                    }
	                    break;
	
	                case dk.ESCAPE:
	                    if (this._opened) {
	                        evt.preventDefault(), evt.stopPropagation();
	                        this._hideMenu();
	                        this.button.setSelectedStyle(false);
	                    }
	                    break;
	            }
	        },
	
	        _onBlur: function() {
	            // summary:
	            //		Called magically when focus has shifted away from this widget and it's drop down
	            this._hideMenu();
	            this.button.setSelectedStyle(false);
	            this.inherited(arguments);
	        },
	
	        _selectOption: function(evt) {
	
	            if (this.boldItem) {
	                domClass.remove(this.boldItem, "dojoxDropDownSelectSelectedOption");
	            }
	            domClass.add(evt, "dojoxDropDownSelectSelectedOption");
	
	            this.boldItem = evt;
	            this.textbox.innerHTML = evt.innerHTML;
	            this.valueNode.value = domAttr.get(evt, "hiddenValue");
	            this._hideMenu();
	            this.button.setSelectedStyle(false);
	            this.focus();
	            this.onChange();
	        },
	
	        postMixInProperties: function() {
	            this.inherited(arguments);
	            if (this.readOnly || this.disabled) {
	                this.required = false;
	            }
	            if (this.error) {
	                this.state = "Error";
	            }
	            if (this.required) {
	                this.baseClass = this.baseClass + " dijitXcitecRequired";
	            }
	        },
	
	        postCreate: function() {
	            //dojo.style(this.textbox, "outline", "none");
	            this._parseSource();
	            this._createDropDown();
	            dom.setSelectable(this.domNode, false);
	        },
	
	        _createDropDown: function() {
	            this.dropdown.createMenu(this.options);
	            this.boldItem = this.dropdown.getCurrentOptionNode();
	
	            if (this.boldItem) {
	                domClass.add(this.boldItem, "dojoxDropDownSelectSelectedOption");
	            }
	            var si = this.srcNodeRef.selectedIndex;
	            if (this.options.length == 0) {
	                return;
	            }
	            //this.value = this.options[si != -1 ? si : 0].hiddeValue;
	            this.value = domAttr.get(this.boldItem, "hiddenValue");
	            this.textbox.innerHTML = this.boldItem.innerHTML;
	            this.valueNode.value = this.value;
	            /*if (this.options[si].iconClass) {
	             var image = dojo.doc.createElement("img");
	             dojo.attr(image, "src", this._blankGif);
	             dojo.attr(image, "class", "dijitMenuItemIcon");
	             dojo.addClass(image, this.options[si].iconClass);
	             this.textbox.appendChild(image);
	             }
	             var text = dojo.doc.createElement("span");
	             text.innerHTML = this.options[si].label;
	             this.textbox.appendChild(text);
	             this.valueNode.value = this.options[si].hiddenValue;*/
	        },
	
	        _parseSource: function() {
	            // summary:
	            //		Loads our options and sets up our dropdown correctly.  We
	            //		don't want any content, so we don't call any inherit chain
	            //		function.
	            var opts = this.options;
	            if (!opts) {
	                opts = this.options = this.srcNodeRef ? query(">",
	                    this.srcNodeRef).map(function(node) {
	                        return {
	                            nullOption: node.getAttribute("nullOption") == null ? false : true,
	                            hiddenValue: node.getAttribute("value"),
	                            iconClass: node.getAttribute("iconClass"),
	                            label: String(node.innerHTML),
	                            selected: xcitecUtil.str2bool(domAttr.get(node, "selected")),
	                            disabled: xcitecUtil.str2bool(domAttr.get(node, "disabled"))
	                        };
	                    }, this) : [];
	            }
	        },
	
	        _onClick: function() {
	            if (this.disabled || this.readOnly) {
	                return;
	            }
	            if (this.dropdown.getOptionsCount() == 0) {
	                return;
	            }
	            this.focus();
	            if (this._opened) {
	                this._hideMenu();
	                this.button.setSelectedStyle(false);
	            } else {
	                // forces full population of results, if they click
	                // on the arrow it means they want to see more options
	                this.focus();
	                this._showMenu();
	                this._opened = true;
	                this.button.setSelectedStyle(true);
	            }
	        },
	
	        _showMenu: function() {
	            this._hideMenu();
	            domStyle.set(this.dropdown.domNode, {width: "", height: ""});
	
	            var best = this._open();
	            // #3212:
	            //		only set auto scroll bars if necessary prevents issues with
	            //		scroll bars appearing when they shouldn't when node is made
	            //		wider (fractional pixels cause this)
	            var popupbox = geometry.getMarginBox(this.dropdown.domNode);
	            this.dropdown.domNode.style.overflow =
	                ((best.h == popupbox.h) && (best.w == popupbox.w)) ? "hidden" : "auto";   
	            // #4134:
	            //		borrow TextArea scrollbar test so content isn't covered by
	            //		scrollbar and horizontal scrollbar doesn't appear
	            var newwidth = best.w;
	            if (best.h < this.dropdown.domNode.scrollHeight) {
	                newwidth += 16;
	            }
              	geometry.setMarginBox(this.dropdown.domNode, {
	                h: best.h,
	                w: Math.max(newwidth, this.domNode.offsetWidth) 
	            });   
	
	        },
	
	        _open: function() {
	            var drop = this.dropdown;
	            return popup.open({
	                popup: drop,
	                around: this.domNode,
					maxHeight: drop.domNode.scrollHeight,
					parent: this
	            });
	            this._opened = true;
	        },
	
		    destroy: function() {
			    if(this.dropdown) {
				    this.dropdown.destroy();
			    }
			    this.inherited(arguments);
		    }
	    }
	);
});

},
'dojo/dom-geometry':function(){
define(["./sniff", "./_base/window","./dom", "./dom-style"],
		function(has, win, dom, style){
	// module:
	//		dojo/dom-geometry

	// the result object
	var geom = {
		// summary:
		//		This module defines the core dojo DOM geometry API.
	};

	// Box functions will assume this model.
	// On IE/Opera, BORDER_BOX will be set if the primary document is in quirks mode.
	// Can be set to change behavior of box setters.

	// can be either:
	//	"border-box"
	//	"content-box" (default)
	geom.boxModel = "content-box";

	// We punt per-node box mode testing completely.
	// If anybody cares, we can provide an additional (optional) unit
	// that overrides existing code to include per-node box sensitivity.

	// Opera documentation claims that Opera 9 uses border-box in BackCompat mode.
	// but experiments (Opera 9.10.8679 on Windows Vista) indicate that it actually continues to use content-box.
	// IIRC, earlier versions of Opera did in fact use border-box.
	// Opera guys, this is really confusing. Opera being broken in quirks mode is not our fault.

	if(has("ie") /*|| has("opera")*/){
		// client code may have to adjust if compatMode varies across iframes
		geom.boxModel = document.compatMode == "BackCompat" ? "border-box" : "content-box";
	}

	geom.getPadExtents = function getPadExtents(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		Returns object with special values specifically useful for node
		//		fitting.
		// description:
		//		Returns an object with `w`, `h`, `l`, `t` properties:
		//	|		l/t/r/b = left/top/right/bottom padding (respectively)
		//	|		w = the total of the left and right padding
		//	|		h = the total of the top and bottom padding
		//		If 'node' has position, l/t forms the origin for child nodes.
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), px = style.toPixelValue,
			l = px(node, s.paddingLeft), t = px(node, s.paddingTop), r = px(node, s.paddingRight), b = px(node, s.paddingBottom);
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	var none = "none";

	geom.getBorderExtents = function getBorderExtents(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		returns an object with properties useful for noting the border
		//		dimensions.
		// description:
		//		- l/t/r/b = the sum of left/top/right/bottom border (respectively)
		//		- w = the sum of the left and right border
		//		- h = the sum of the top and bottom border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var px = style.toPixelValue, s = computedStyle || style.getComputedStyle(node),
			l = s.borderLeftStyle != none ? px(node, s.borderLeftWidth) : 0,
			t = s.borderTopStyle != none ? px(node, s.borderTopWidth) : 0,
			r = s.borderRightStyle != none ? px(node, s.borderRightWidth) : 0,
			b = s.borderBottomStyle != none ? px(node, s.borderBottomWidth) : 0;
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	geom.getPadBorderExtents = function getPadBorderExtents(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		Returns object with properties useful for box fitting with
		//		regards to padding.
		// description:
		//		- l/t/r/b = the sum of left/top/right/bottom padding and left/top/right/bottom border (respectively)
		//		- w = the sum of the left and right padding and border
		//		- h = the sum of the top and bottom padding and border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node),
			p = geom.getPadExtents(node, s),
			b = geom.getBorderExtents(node, s);
		return {
			l: p.l + b.l,
			t: p.t + b.t,
			r: p.r + b.r,
			b: p.b + b.b,
			w: p.w + b.w,
			h: p.h + b.h
		};
	};

	geom.getMarginExtents = function getMarginExtents(node, computedStyle){
		// summary:
		//		returns object with properties useful for box fitting with
		//		regards to box margins (i.e., the outer-box).
		//
		//		- l/t = marginLeft, marginTop, respectively
		//		- w = total width, margin inclusive
		//		- h = total height, margin inclusive
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), px = style.toPixelValue,
			l = px(node, s.marginLeft), t = px(node, s.marginTop), r = px(node, s.marginRight), b = px(node, s.marginBottom);
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	// Box getters work in any box context because offsetWidth/clientWidth
	// are invariant wrt box context
	//
	// They do *not* work for display: inline objects that have padding styles
	// because the user agent ignores padding (it's bogus styling in any case)
	//
	// Be careful with IMGs because they are inline or block depending on
	// browser and browser mode.

	// Although it would be easier to read, there are not separate versions of
	// _getMarginBox for each browser because:
	// 1. the branching is not expensive
	// 2. factoring the shared code wastes cycles (function call overhead)
	// 3. duplicating the shared code wastes bytes

	geom.getMarginBox = function getMarginBox(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		returns an object that encodes the width, height, left and top
		//		positions of the node's margin box.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), me = geom.getMarginExtents(node, s),
			l = node.offsetLeft - me.l, t = node.offsetTop - me.t, p = node.parentNode, px = style.toPixelValue, pcs;
		if(has("mozilla")){
			// Mozilla:
			// If offsetParent has a computed overflow != visible, the offsetLeft is decreased
			// by the parent's border.
			// We don't want to compute the parent's style, so instead we examine node's
			// computed left/top which is more stable.
			var sl = parseFloat(s.left), st = parseFloat(s.top);
			if(!isNaN(sl) && !isNaN(st)){
				l = sl;
				t = st;
			}else{
				// If child's computed left/top are not parseable as a number (e.g. "auto"), we
				// have no choice but to examine the parent's computed style.
				if(p && p.style){
					pcs = style.getComputedStyle(p);
					if(pcs.overflow != "visible"){
						l += pcs.borderLeftStyle != none ? px(node, pcs.borderLeftWidth) : 0;
						t += pcs.borderTopStyle != none ? px(node, pcs.borderTopWidth) : 0;
					}
				}
			}
		}else if(has("opera") || (has("ie") == 8 && !has("quirks"))){
			// On Opera and IE 8, offsetLeft/Top includes the parent's border
			if(p){
				pcs = style.getComputedStyle(p);
				l -= pcs.borderLeftStyle != none ? px(node, pcs.borderLeftWidth) : 0;
				t -= pcs.borderTopStyle != none ? px(node, pcs.borderTopWidth) : 0;
			}
		}
		return {l: l, t: t, w: node.offsetWidth + me.w, h: node.offsetHeight + me.h};
	};

	geom.getContentBox = function getContentBox(node, computedStyle){
		// summary:
		//		Returns an object that encodes the width, height, left and top
		//		positions of the node's content box, irrespective of the
		//		current box model.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		// clientWidth/Height are important since the automatically account for scrollbars
		// fallback to offsetWidth/Height for special cases (see #3378)
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), w = node.clientWidth, h,
			pe = geom.getPadExtents(node, s), be = geom.getBorderExtents(node, s);
		if(!w){
			w = node.offsetWidth;
			h = node.offsetHeight;
		}else{
			h = node.clientHeight;
			be.w = be.h = 0;
		}
		// On Opera, offsetLeft includes the parent's border
		if(has("opera")){
			pe.l += be.l;
			pe.t += be.t;
		}
		return {l: pe.l, t: pe.t, w: w - pe.w - be.w, h: h - pe.h - be.h};
	};

	// Box setters depend on box context because interpretation of width/height styles
	// vary wrt box context.
	//
	// The value of boxModel is used to determine box context.
	// boxModel can be set directly to change behavior.
	//
	// Beware of display: inline objects that have padding styles
	// because the user agent ignores padding (it's a bogus setup anyway)
	//
	// Be careful with IMGs because they are inline or block depending on
	// browser and browser mode.
	//
	// Elements other than DIV may have special quirks, like built-in
	// margins or padding, or values not detectable via computedStyle.
	// In particular, margins on TABLE do not seems to appear
	// at all in computedStyle on Mozilla.

	function setBox(/*DomNode*/ node, /*Number?*/ l, /*Number?*/ t, /*Number?*/ w, /*Number?*/ h, /*String?*/ u){
		// summary:
		//		sets width/height/left/top in the current (native) box-model
		//		dimensions. Uses the unit passed in u.
		// node:
		//		DOM Node reference. Id string not supported for performance
		//		reasons.
		// l:
		//		left offset from parent.
		// t:
		//		top offset from parent.
		// w:
		//		width in current box model.
		// h:
		//		width in current box model.
		// u:
		//		unit measure to use for other measures. Defaults to "px".
		u = u || "px";
		var s = node.style;
		if(!isNaN(l)){
			s.left = l + u;
		}
		if(!isNaN(t)){
			s.top = t + u;
		}
		if(w >= 0){
			s.width = w + u;
		}
		if(h >= 0){
			s.height = h + u;
		}
	}

	function isButtonTag(/*DomNode*/ node){
		// summary:
		//		True if the node is BUTTON or INPUT.type="button".
		return node.tagName.toLowerCase() == "button" ||
			node.tagName.toLowerCase() == "input" && (node.getAttribute("type") || "").toLowerCase() == "button"; // boolean
	}

	function usesBorderBox(/*DomNode*/ node){
		// summary:
		//		True if the node uses border-box layout.

		// We could test the computed style of node to see if a particular box
		// has been specified, but there are details and we choose not to bother.

		// TABLE and BUTTON (and INPUT type=button) are always border-box by default.
		// If you have assigned a different box to either one via CSS then
		// box functions will break.

		return geom.boxModel == "border-box" || node.tagName.toLowerCase() == "table" || isButtonTag(node); // boolean
	}

	geom.setContentSize = function setContentSize(/*DomNode*/ node, /*Object*/ box, /*Object*/ computedStyle){
		// summary:
		//		Sets the size of the node's contents, irrespective of margins,
		//		padding, or borders.
		// node: DOMNode
		// box: Object
		//		hash with optional "w", and "h" properties for "width", and "height"
		//		respectively. All specified properties should have numeric values in whole pixels.
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var w = box.w, h = box.h;
		if(usesBorderBox(node)){
			var pb = geom.getPadBorderExtents(node, computedStyle);
			if(w >= 0){
				w += pb.w;
			}
			if(h >= 0){
				h += pb.h;
			}
		}
		setBox(node, NaN, NaN, w, h);
	};

	var nilExtents = {l: 0, t: 0, w: 0, h: 0};

	geom.setMarginBox = function setMarginBox(/*DomNode*/ node, /*Object*/ box, /*Object*/ computedStyle){
		// summary:
		//		sets the size of the node's margin box and placement
		//		(left/top), irrespective of box model. Think of it as a
		//		passthrough to setBox that handles box-model vagaries for
		//		you.
		// node: DOMNode
		// box: Object
		//		hash with optional "l", "t", "w", and "h" properties for "left", "right", "width", and "height"
		//		respectively. All specified properties should have numeric values in whole pixels.
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), w = box.w, h = box.h,
		// Some elements have special padding, margin, and box-model settings.
		// To use box functions you may need to set padding, margin explicitly.
		// Controlling box-model is harder, in a pinch you might set dojo/dom-geometry.boxModel.
			pb = usesBorderBox(node) ? nilExtents : geom.getPadBorderExtents(node, s),
			mb = geom.getMarginExtents(node, s);
		if(has("webkit")){
			// on Safari (3.1.2), button nodes with no explicit size have a default margin
			// setting an explicit size eliminates the margin.
			// We have to swizzle the width to get correct margin reading.
			if(isButtonTag(node)){
				var ns = node.style;
				if(w >= 0 && !ns.width){
					ns.width = "4px";
				}
				if(h >= 0 && !ns.height){
					ns.height = "4px";
				}
			}
		}
		if(w >= 0){
			w = Math.max(w - pb.w - mb.w, 0);
		}
		if(h >= 0){
			h = Math.max(h - pb.h - mb.h, 0);
		}
		setBox(node, box.l, box.t, w, h);
	};

	// =============================
	// Positioning
	// =============================

	geom.isBodyLtr = function isBodyLtr(/*Document?*/ doc){
		// summary:
		//		Returns true if the current language is left-to-right, and false otherwise.
		// doc: Document?
		//		Optional document to query.   If unspecified, use win.doc.
		// returns: Boolean

		doc = doc || win.doc;
		return (win.body(doc).dir || doc.documentElement.dir || "ltr").toLowerCase() == "ltr"; // Boolean
	};

	geom.docScroll = function docScroll(/*Document?*/ doc){
		// summary:
		//		Returns an object with {node, x, y} with corresponding offsets.
		// doc: Document?
		//		Optional document to query.   If unspecified, use win.doc.
		// returns: Object

		doc = doc || win.doc;
		var node = win.doc.parentWindow || win.doc.defaultView;   // use UI window, not dojo.global window.   TODO: use dojo/window::get() except for circular dependency problem
		return "pageXOffset" in node ? {x: node.pageXOffset, y: node.pageYOffset } :
			(node = has("quirks") ? win.body(doc) : doc.documentElement) &&
				{x: geom.fixIeBiDiScrollLeft(node.scrollLeft || 0, doc), y: node.scrollTop || 0 };
	};

	if(has("ie")){
		geom.getIeDocumentElementOffset = function getIeDocumentElementOffset(/*Document?*/ doc){
			// summary:
			//		returns the offset in x and y from the document body to the
			//		visual edge of the page for IE
			// doc: Document?
			//		Optional document to query.   If unspecified, use win.doc.
			// description:
			//		The following values in IE contain an offset:
			//	|		event.clientX
			//	|		event.clientY
			//	|		node.getBoundingClientRect().left
			//	|		node.getBoundingClientRect().top
			//		But other position related values do not contain this offset,
			//		such as node.offsetLeft, node.offsetTop, node.style.left and
			//		node.style.top. The offset is always (2, 2) in LTR direction.
			//		When the body is in RTL direction, the offset counts the width
			//		of left scroll bar's width.  This function computes the actual
			//		offset.

			//NOTE: assumes we're being called in an IE browser

			doc = doc || win.doc;
			var de = doc.documentElement; // only deal with HTML element here, position() handles body/quirks

			if(has("ie") < 8){
				var r = de.getBoundingClientRect(), // works well for IE6+
					l = r.left, t = r.top;
				if(has("ie") < 7){
					l += de.clientLeft;	// scrollbar size in strict/RTL, or,
					t += de.clientTop;	// HTML border size in strict
				}
				return {
					x: l < 0 ? 0 : l, // FRAME element border size can lead to inaccurate negative values
					y: t < 0 ? 0 : t
				};
			}else{
				return {
					x: 0,
					y: 0
				};
			}
		};
	}

	geom.fixIeBiDiScrollLeft = function fixIeBiDiScrollLeft(/*Integer*/ scrollLeft, /*Document?*/ doc){
		// summary:
		//		In RTL direction, scrollLeft should be a negative value, but IE
		//		returns a positive one. All codes using documentElement.scrollLeft
		//		must call this function to fix this error, otherwise the position
		//		will offset to right when there is a horizontal scrollbar.
		// scrollLeft: Number
		// doc: Document?
		//		Optional document to query.   If unspecified, use win.doc.
		// returns: Number

		// In RTL direction, scrollLeft should be a negative value, but IE
		// returns a positive one. All codes using documentElement.scrollLeft
		// must call this function to fix this error, otherwise the position
		// will offset to right when there is a horizontal scrollbar.

		doc = doc || win.doc;
		var ie = has("ie");
		if(ie && !geom.isBodyLtr(doc)){
			var qk = has("quirks"),
				de = qk ? win.body(doc) : doc.documentElement,
				pwin = win.global;	// TODO: use winUtils.get(doc) after resolving circular dependency b/w dom-geometry.js and dojo/window.js
			if(ie == 6 && !qk && pwin.frameElement && de.scrollHeight > de.clientHeight){
				scrollLeft += de.clientLeft; // workaround ie6+strict+rtl+iframe+vertical-scrollbar bug where clientWidth is too small by clientLeft pixels
			}
			return (ie < 8 || qk) ? (scrollLeft + de.clientWidth - de.scrollWidth) : -scrollLeft; // Integer
		}
		return scrollLeft; // Integer
	};

	geom.position = function(/*DomNode*/ node, /*Boolean?*/ includeScroll){
		// summary:
		//		Gets the position and size of the passed element relative to
		//		the viewport (if includeScroll==false), or relative to the
		//		document root (if includeScroll==true).
		//
		// description:
		//		Returns an object of the form:
		//		`{ x: 100, y: 300, w: 20, h: 15 }`.
		//		If includeScroll==true, the x and y values will include any
		//		document offsets that may affect the position relative to the
		//		viewport.
		//		Uses the border-box model (inclusive of border and padding but
		//		not margin).  Does not act as a setter.
		// node: DOMNode|String
		// includeScroll: Boolean?
		// returns: Object

		node = dom.byId(node);
		var	db = win.body(node.ownerDocument),
			ret = node.getBoundingClientRect();
		ret = {x: ret.left, y: ret.top, w: ret.right - ret.left, h: ret.bottom - ret.top};

		if(has("ie") < 9){
			// On IE<9 there's a 2px offset that we need to adjust for, see dojo.getIeDocumentElementOffset()
			var offset = geom.getIeDocumentElementOffset(node.ownerDocument);

			// fixes the position in IE, quirks mode
			ret.x -= offset.x + (has("quirks") ? db.clientLeft + db.offsetLeft : 0);
			ret.y -= offset.y + (has("quirks") ? db.clientTop + db.offsetTop : 0);
		}

		// account for document scrolling
		// if offsetParent is used, ret value already includes scroll position
		// so we may have to actually remove that value if !includeScroll
		if(includeScroll){
			var scroll = geom.docScroll(node.ownerDocument);
			ret.x += scroll.x;
			ret.y += scroll.y;
		}

		return ret; // Object
	};

	// random "private" functions wildly used throughout the toolkit

	geom.getMarginSize = function getMarginSize(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		returns an object that encodes the width and height of
		//		the node's margin box
		// node: DOMNode|String
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo/dom-style.getComputedStyle to get one. It is a better way, calling
		//		dojo/dom-style.getComputedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var me = geom.getMarginExtents(node, computedStyle || style.getComputedStyle(node));
		var size = node.getBoundingClientRect();
		return {
			w: (size.right - size.left) + me.w,
			h: (size.bottom - size.top) + me.h
		};
	};

	geom.normalizeEvent = function(event){
		// summary:
		//		Normalizes the geometry of a DOM event, normalizing the pageX, pageY,
		//		offsetX, offsetY, layerX, and layerX properties
		// event: Object
		if(!("layerX" in event)){
			event.layerX = event.offsetX;
			event.layerY = event.offsetY;
		}
		if(!has("dom-addeventlistener")){
			// old IE version
			// FIXME: scroll position query is duped from dojo/_base/html to
			// avoid dependency on that entire module. Now that HTML is in
			// Base, we should convert back to something similar there.
			var se = event.target;
			var doc = (se && se.ownerDocument) || document;
			// DO NOT replace the following to use dojo/_base/window.body(), in IE, document.documentElement should be used
			// here rather than document.body
			var docBody = has("quirks") ? doc.body : doc.documentElement;
			var offset = geom.getIeDocumentElementOffset(doc);
			event.pageX = event.clientX + geom.fixIeBiDiScrollLeft(docBody.scrollLeft || 0, doc) - offset.x;
			event.pageY = event.clientY + (docBody.scrollTop || 0) - offset.y;
		}
	};

	// TODO: evaluate separate getters/setters for position and sizes?

	return geom;
});

},
'dijit/form/_FormValueMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/keys", // keys.ESCAPE
	"dojo/_base/lang",
	"dojo/on",
	"dojo/sniff", // has("ie"), has("quirks")
	"./_FormWidgetMixin"
], function(declare, domAttr, keys, lang, on, has, _FormWidgetMixin){

	// module:
	//		dijit/form/_FormValueMixin

	return declare("dijit.form._FormValueMixin", _FormWidgetMixin, {
		// summary:
		//		Mixin for widgets corresponding to native HTML elements such as `<input>` or `<select>`
		//		that have user changeable values.
		// description:
		//		Each _FormValueMixin represents a single input value, and has a (possibly hidden) `<input>` element,
		//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
		//		works as expected.

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,

		_setReadOnlyAttr: function(/*Boolean*/ value){
			domAttr.set(this.focusNode, 'readOnly', value);
			this._set("readOnly", value);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Update our reset value if it hasn't yet been set (because this.set()
			// is only called when there *is* a value)
			if(this._resetValue === undefined){
				this._lastValueReported = this._resetValue = this.value;
			}
		},

		_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the widget.
			//		If the value has changed, then fire onChange event, unless priorityChange
			//		is specified as null (or false?)
			this._handleOnChange(newValue, priorityChange);
		},

		_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Called when the value of the widget has changed.  Saves the new value in this.value,
			//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
			this._set("value", newValue);
			this.inherited(arguments);
		},

		undo: function(){
			// summary:
			//		Restore the value to the last value passed to onChange
			this._setValueAttr(this._lastValueReported, false);
		},

		reset: function(){
			// summary:
			//		Reset the widget's value to what it was at initialization time
			this._hasBeenBlurred = false;
			this._setValueAttr(this._resetValue, true);
		}
	});
});

},
'xcitec/xlayout/StyleElement':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"../Image",
	"../Label",
	"../Text",
	"../WikiText",
  "../BoldLabel"
], function (dojo, domStyle, declare, domAttr, domClass, domConstruct, _Templated, _Widget, Image, Label, Text, WikiText, BoldLabel) {

	/* xcitec.xlayout.StyleElement is a control which is used in qualification to
	   show style elements in design and view node. There are 5 style elements: headline,
	   emptyline, image, divider and text. Type of style element is set through 'elementType'
	   attribute*/
	
	var StyleElement = declare( [_Widget, _Templated], {
	    templateString: "<div class='styleElement' data-dojo-attach-point='containerNode'></div>",
	
	    // type of style element
	    elementType: "",
	
	    caption: "",
	
	    // onDelete attribute
	    onDeleteAttr: null,
	    onSettingsAttr: null,
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        this.onDeleteAttr = 'onDelete' in this.params;
	        this.onSettingsAttr = 'onSettings' in this.params;
	    },
	
	    postCreate: function() {
	        this.inherited(arguments);
	        var element = null;
	        var _this = this;
	
	        // create actual element according to elementType attribute
	        switch (this.elementType) {
	            case 'headline':
	                element = new BoldLabel({label: this.caption, allowWrap: true});
	                domConstruct.place(element.domNode, _this.containerNode, 'first');
	                break;
	            case 'emptyline':
	                element = new Label({label: "empty line", style: 'font-style:Italic'});
	                domConstruct.place(element.domNode, _this.containerNode, 'first');
	                break;
	            case 'emptylineview':
	                element = new Label({label: ""});
	                domConstruct.place(element.domNode, _this.containerNode, 'first');
	                break;
	            case 'image':
	                element = new Image({src: "../html/themes/xcitec/images/logo.gif", width: '149', height:'49'});
	                domStyle.set(this.domNode, 'height', '60px');
	                domConstruct.place(element.domNode, _this.containerNode, 'first');
	                break;
	            case 'divider':
	                element = domConstruct.create("div");
	                domClass.add(element, "ContentAreaHR");
	                domConstruct.place(element, _this.containerNode, 'first');
	                break;
	            case 'text':
	//                element = new xcitec.Text({text: this.caption});
	                element = new WikiText({text: this.caption});
	                //dojo.style(this.domNode, 'height', '40px');
	                domConstruct.place(element.domNode, _this.containerNode, 'first');
	                break;
	            default:
	                break;
	        }
	        domStyle.set(this.domNode, 'width', 'auto');
	    },
	
	    // actions for delete and settings buttons
	    onDelete: function() {
	
	    },
	
	    onSettings: function() {
	
	    },
	
	    // getter for onDelete attribute
	    isDeleteSet: function() {
	        return this.onDeleteAttr;
	    },
	
	    isSettingsSet: function() {
	        return this.onSettingsAttr;
	    },
	
	    isScoreSet: function() {
	        return false;
	    },
	
	    isCriticalSet: function() {
	        return false;
	    },
	    
	    isChangedSet: function() {
	        return false;
	    },
	    
	    isChangedNewSet: function() {
	        return false;
	    },
	    
	    isChangedOldSet: function() {
	        return false;
	    },
	
	    isKoSet: function() {
	        return false;
	    }
	});
	return StyleElement;
});

},
'dojo/date/stamp':function(){
define(["../_base/lang", "../_base/array"], function(lang, array){

// module:
//		dojo/date/stamp

var stamp = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.date.stamp", stamp);

// Methods to convert dates to or from a wire (string) format using well-known conventions

stamp.fromISOString = function(/*String*/ formattedString, /*Number?*/ defaultTime){
	// summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	// description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//		- dates only
	//			- yyyy
	//			- yyyy-MM
	//			- yyyy-MM-dd
	//		- times only, with an optional time zone appended
	//			- THH:mm
	//			- THH:mm:ss
	//			- THH:mm:ss.SSS
	//		- and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	//		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
  	// formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	// defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!stamp._isoRegExp){
		stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			array.forEach(array.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
var __Options = {
	// selector: String
	//		"date" or "time" for partial formatting of the Date object.
	//		Both date and time will be formatted by default.
	// zulu: Boolean
	//		if true, UTC/GMT is used for a timezone
	// milliseconds: Boolean
	//		if true, output milliseconds
};
=====*/

stamp.toISOString = function(/*Date*/ dateObject, /*__Options?*/ options){
	// summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	// description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	// dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

return stamp;
});

},
'xcitec/page/ListTabPanel':function(){
define([
	"dojo/_base/declare",
	"./BoxPanelV2"
], function (declare, BoxPanelV2) {

	return declare(BoxPanelV2, {

	});
});
},
'dijit/TitlePane':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.set or get domAttr.remove
	"dojo/dom-class", // domClass.replace
	"dojo/dom-geometry", // domGeometry.setMarginBox domGeometry.getMarginBox
	"dojo/fx", // fxUtils.wipeIn fxUtils.wipeOut
	"dojo/has",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER
	"./_CssStateMixin",
	"./_TemplatedMixin",
	"./layout/ContentPane",
	"dojo/text!./templates/TitlePane.html",
	"./_base/manager",    // defaultDuration
	"./a11yclick"	// template uses ondijitclick
], function(array, declare, dom, domAttr, domClass, domGeometry, fxUtils, has, kernel, keys,
			_CssStateMixin, _TemplatedMixin, ContentPane, template, manager){

	// module:
	//		dijit/TitlePane

	var TitlePane = declare("dijit.TitlePane", [ContentPane, _TemplatedMixin, _CssStateMixin], {
		// summary:
		//		A pane with a title on top, that can be expanded or collapsed.
		//
		// description:
		//		An accessible container with a title Heading, and a content
		//		section that slides open and closed. TitlePane is an extension to
		//		`dijit/layout/ContentPane`, providing all the useful content-control aspects from it.
		//
		// example:
		//	|	// load a TitlePane from remote file:
		//	|	var foo = new dijit.TitlePane({ href: "foobar.html", title:"Title" });
		//	|	foo.startup();
		//
		// example:
		//	|	<!-- markup href example: -->
		//	|	<div data-dojo-type="dijit/TitlePane" data-dojo-props="href: 'foobar.html', title: 'Title'"></div>
		//
		// example:
		//	|	<!-- markup with inline data -->
		//	|	<div data-dojo-type="dijit/TitlePane" title="Title">
		//	|		<p>I am content</p>
		//	|	</div>

		// title: String
		//		Title of the pane
		title: "",
		_setTitleAttr: { node: "titleNode", type: "innerHTML" }, // override default where title becomes a hover tooltip

		// open: Boolean
		//		Whether pane is opened or closed.
		open: true,

		// toggleable: Boolean
		//		Whether pane can be opened or closed by clicking the title bar.
		toggleable: true,

		// tabIndex: String
		//		Tabindex setting for the title (so users can tab to the title then
		//		use space/enter to open/close the title pane)
		tabIndex: "0",

		// duration: Integer
		//		Time in milliseconds to fade in/fade out
		duration: manager.defaultDuration,

		// baseClass: [protected] String
		//		The root className to be placed on this widget's domNode.
		baseClass: "dijitTitlePane",

		templateString: template,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for TitlePane, since TitlePane
		//		is never a child of a layout container, nor should TitlePane try to control
		//		the size of an inner widget.
		doLayout: false,

		// Tooltip is defined in _WidgetBase but we need to handle the mapping to DOM here
		_setTooltipAttr: {node: "focusNode", type: "attribute", attribute: "title"}, // focusNode spans the entire width, titleNode doesn't

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.titleNode, false);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Hover and focus effect on title bar, except for non-toggleable TitlePanes
			// This should really be controlled from _setToggleableAttr() but _CssStateMixin
			// doesn't provide a way to disconnect a previous _trackMouseState() call
			if(this.toggleable){
				this._trackMouseState(this.titleBarNode, this.baseClass + "Title");
			}

			// setup open/close animations
			var hideNode = this.hideNode, wipeNode = this.wipeNode;
			this._wipeIn = fxUtils.wipeIn({
				node: wipeNode,
				duration: this.duration,
				beforeBegin: function(){
					hideNode.style.display = "";
				}
			});
			this._wipeOut = fxUtils.wipeOut({
				node: wipeNode,
				duration: this.duration,
				onEnd: function(){
					hideNode.style.display = "none";
				}
			});
		},

		_setOpenAttr: function(/*Boolean*/ open, /*Boolean*/ animate){
			// summary:
			//		Hook to make set("open", boolean) control the open/closed state of the pane.
			// open: Boolean
			//		True if you want to open the pane, false if you want to close it.

			array.forEach([this._wipeIn, this._wipeOut], function(animation){
				if(animation && animation.status() == "playing"){
					animation.stop();
				}
			});

			if(animate){
				var anim = this[open ? "_wipeIn" : "_wipeOut"];
				anim.play();
			}else{
				this.hideNode.style.display = this.wipeNode.style.display = open ? "" : "none";
			}

			// load content (if this is the first time we are opening the TitlePane
			// and content is specified as an href, or href was set when hidden)
			if(this._started){
				if(open){
					this._onShow();
				}else{
					this.onHide();
				}
			}

			this.containerNode.setAttribute("aria-hidden", open ? "false" : "true");
			this.focusNode.setAttribute("aria-pressed", open ? "true" : "false");

			this._set("open", open);

			this._setCss();
		},

		_setToggleableAttr: function(/*Boolean*/ canToggle){
			// summary:
			//		Hook to make set("toggleable", boolean) work.
			// canToggle: Boolean
			//		True to allow user to open/close pane by clicking title bar.

			this.focusNode.setAttribute("role", canToggle ? "button" : "heading");
			if(canToggle){
				this.focusNode.setAttribute("aria-controls", this.id + "_pane");
				this.focusNode.setAttribute("tabIndex", this.tabIndex);
				this.focusNode.setAttribute("aria-pressed", this.open);
			}else{
				domAttr.remove(this.focusNode, "aria-controls");
				domAttr.remove(this.focusNode, "tabIndex");
				domAttr.remove(this.focusNode, "aria-pressed");
			}

			this._set("toggleable", canToggle);

			this._setCss();
		},

		_setContentAttr: function(/*String|DomNode|Nodelist*/ content){
			// summary:
			//		Hook to make set("content", ...) work.
			//		Typically called when an href is loaded.  Our job is to make the animation smooth.

			if(!this.open || !this._wipeOut || this._wipeOut.status() == "playing"){
				// we are currently *closing* the pane (or the pane is closed), so just let that continue
				this.inherited(arguments);
			}else{
				if(this._wipeIn && this._wipeIn.status() == "playing"){
					this._wipeIn.stop();
				}

				// freeze container at current height so that adding new content doesn't make it jump
				domGeometry.setMarginBox(this.wipeNode, { h: domGeometry.getMarginBox(this.wipeNode).h });

				// add the new content (erasing the old content, if any)
				this.inherited(arguments);

				// call _wipeIn.play() to animate from current height to new height
				if(this._wipeIn){
					this._wipeIn.play();
				}else{
					this.hideNode.style.display = "";
				}
			}
		},

		toggle: function(){
			// summary:
			//		Switches between opened and closed state
			// tags:
			//		private

			this._setOpenAttr(!this.open, true);
		},

		_setCss: function(){
			// summary:
			//		Set the open/close css state for the TitlePane
			// tags:
			//		private

			var node = this.titleBarNode || this.focusNode;
			var oldCls = this._titleBarClass;
			this._titleBarClass = this.baseClass + "Title" + (this.toggleable ? "" : "Fixed") + (this.open ? "Open" : "Closed");
			domClass.replace(node, this._titleBarClass, oldCls || "");

			// Back compat, remove for 2.0
			domClass.replace(node, this._titleBarClass.replace("TitlePaneTitle", ""), (oldCls || "").replace("TitlePaneTitle", ""));

			this.arrowNodeInner.innerHTML = this.open ? "-" : "+";
		},

		_onTitleKey: function(/*Event*/ e){
			// summary:
			//		Handler for when user hits a key
			// tags:
			//		private

			if(e.keyCode == keys.DOWN_ARROW && this.open){
				this.containerNode.focus();
				e.preventDefault();
			}
		},

		_onTitleClick: function(){
			// summary:
			//		Handler when user clicks the title bar
			// tags:
			//		private
			if(this.toggleable){
				this.toggle();
			}
		},

		setTitle: function(/*String*/ title){
			// summary:
			//		Deprecated.  Use set('title', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.TitlePane.setTitle() is deprecated.  Use set('title', ...) instead.", "", "2.0");
			this.set("title", title);
		}
	});

	if(has("dojo-bidi")){
		TitlePane.extend({
			_setTitleAttr: function(/*String*/ title){
				// Override default where title becomes a hover tooltip
				this._set("title", title);
				this.titleNode.innerHTML = title;
				this.applyTextDir(this.titleNode);
			},

			_setTooltipAttr: function(/*String*/ tooltip){
				this._set("tooltip", tooltip);
				if(this.textDir){
					tooltip = this.enforceTextDirWithUcc(null, tooltip);
				}
				domAttr.set(this.focusNode, "title", tooltip);			// focusNode spans the entire width, titleNode doesn't
			},

			_setTextDirAttr: function(textDir){
				if(this._created && this.textDir != textDir){
					this._set("textDir", textDir);
					this.set("title", this.title);
					this.set("tooltip", this.tooltip);
				}
			}
		});
	}

	return TitlePane;
});

},
'dojo/dnd/Container':function(){
define([
	"../_base/array",
	"../_base/declare",
	"../_base/kernel",
	"../_base/lang",
	"../_base/window",
	"../dom",
	"../dom-class",
	"../dom-construct",
	"../Evented",
	"../has",
	"../on",
	"../query",
	"../touch",
	"./common"
], function(
	array, declare, kernel, lang, win,
	dom, domClass, domConstruct, Evented, has, on, query, touch, dnd){

// module:
//		dojo/dnd/Container

/*
	Container states:
		""		- normal state
		"Over"	- mouse over a container
	Container item states:
		""		- normal state
		"Over"	- mouse over a container item
*/



var Container = declare("dojo.dnd.Container", Evented, {
	// summary:
	//		a Container object, which knows when mouse hovers over it,
	//		and over which element it hovers

	// object attributes (for markup)
	skipForm: false,
	// allowNested: Boolean
	//		Indicates whether to allow dnd item nodes to be nested within other elements.
	//		By default this is false, indicating that only direct children of the container can
	//		be draggable dnd item nodes
	allowNested: false,
	/*=====
	// current: DomNode
	//		The DOM node the mouse is currently hovered over
	current: null,

	// map: Hash<String, Container.Item>
	//		Map from an item's id (which is also the DOMNode's id) to
	//		the dojo/dnd/Container.Item itself.
	map: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		a constructor of the Container
		// node: Node
		//		node or node's id to build the container on
		// params: Container.__ContainerArgs
		//		a dictionary of parameters
		this.node = dom.byId(node);
		if(!params){ params = {}; }
		this.creator = params.creator || null;
		this.skipForm = params.skipForm;
		this.parent = params.dropParent && dom.byId(params.dropParent);

		// class-specific variables
		this.map = {};
		this.current = null;

		// states
		this.containerState = "";
		domClass.add(this.node, "dojoDndContainer");

		// mark up children
		if(!(params && params._skipStartup)){
			this.startup();
		}

		// set up events
		this.events = [
			on(this.node, touch.over, lang.hitch(this, "onMouseOver")),
			on(this.node, touch.out,  lang.hitch(this, "onMouseOut")),
			// cancel text selection and text dragging
			on(this.node, "dragstart",   lang.hitch(this, "onSelectStart")),
			on(this.node, "selectstart", lang.hitch(this, "onSelectStart"))
		];
	},

	// object attributes (for markup)
	creator: function(){
		// summary:
		//		creator function, dummy at the moment
	},

	// abstract access to the map
	getItem: function(/*String*/ key){
		// summary:
		//		returns a data item by its key (id)
		return this.map[key];	// Container.Item
	},
	setItem: function(/*String*/ key, /*Container.Item*/ data){
		// summary:
		//		associates a data item with its key (id)
		this.map[key] = data;
	},
	delItem: function(/*String*/ key){
		// summary:
		//		removes a data item from the map by its key (id)
		delete this.map[key];
	},
	forInItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over a data map skipping members that
		//		are present in the empty object (IE and/or 3rd-party libraries).
		o = o || kernel.global;
		var m = this.map, e = dnd._empty;
		for(var i in m){
			if(i in e){ continue; }
			f.call(o, m[i], i, this);
		}
		return o;	// Object
	},
	clearItems: function(){
		// summary:
		//		removes all data items from the map
		this.map = {};
	},

	// methods
	getAllNodes: function(){
		// summary:
		//		returns a list (an array) of all valid child nodes
		return query((this.allowNested ? "" : "> ") + ".dojoDndItem", this.parent);	// NodeList
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		var map = {};
		this.getAllNodes().forEach(function(node){
			if(node.id){
				var item = this.getItem(node.id);
				if(item){
					map[node.id] = item;
					return;
				}
			}else{
				node.id = dnd.getUniqueId();
			}
			var type = node.getAttribute("dndType"),
				data = node.getAttribute("dndData");
			map[node.id] = {
				data: data || node.innerHTML,
				type: type ? type.split(/\s*,\s*/) : ["text"]
			};
		}, this);
		this.map = map;
		return this;	// self
	},
	insertNodes: function(data, before, anchor){
		// summary:
		//		inserts an array of new nodes before/after an anchor node
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		if(!this.parent.firstChild){
			anchor = null;
		}else if(before){
			if(!anchor){
				anchor = this.parent.firstChild;
			}
		}else{
			if(anchor){
				anchor = anchor.nextSibling;
			}
		}
		var i, t;
		if(anchor){
			for(i = 0; i < data.length; ++i){
				t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				anchor.parentNode.insertBefore(t.node, anchor);
			}
		}else{
			for(i = 0; i < data.length; ++i){
				t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.appendChild(t.node);
			}
		}
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares this object to be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.clearItems();
		this.node = this.parent = this.current = null;
	},

	// markup methods
	markupFactory: function(params, node, Ctor){
		params._skipStartup = true;
		return new Ctor(node, params);
	},
	startup: function(){
		// summary:
		//		collects valid child items and populate the map

		// set up the real parent node
		if(!this.parent){
			// use the standard algorithm, if not assigned
			this.parent = this.node;
			if(this.parent.tagName.toLowerCase() == "table"){
				var c = this.parent.getElementsByTagName("tbody");
				if(c && c.length){ this.parent = c[0]; }
			}
		}
		this.defaultCreator = dnd._defaultCreator(this.parent);

		// process specially marked children
		this.sync();
	},

	// mouse events
	onMouseOver: function(e){
		// summary:
		//		event processor for onmouseover or touch, to mark that element as the current element
		// e: Event
		//		mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ this._addItemClass(n, "Over"); }
		this.current = n;
	},
	onMouseOut: function(e){
		// summary:
		//		event processor for onmouseout
		// e: Event
		//		mouse event
		for(var n = e.relatedTarget; n;){
			if(n == this.node){ return; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(this.current){
			this._removeItemClass(this.current, "Over");
			this.current = null;
		}
		this._changeState("Container", "");
		this.onOutEvent();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skipForm || !dnd.isFormElement(e)){
			e.stopPropagation();
			e.preventDefault();
		}
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
	},
	_changeState: function(type, newState){
		// summary:
		//		changes a named state to new state value
		// type: String
		//		a name of the state to change
		// newState: String
		//		new state
		var prefix = "dojoDnd" + type;
		var state  = type.toLowerCase() + "State";
		//domClass.replace(this.node, prefix + newState, prefix + this[state]);
		domClass.replace(this.node, prefix + newState, prefix + this[state]);
		this[state] = newState;
	},
	_addItemClass: function(node, type){
		// summary:
		//		adds a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		domClass.add(node, "dojoDndItem" + type);
	},
	_removeItemClass: function(node, type){
		// summary:
		//		removes a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		domClass.remove(node, "dojoDndItem" + type);
	},
	_getChildByEvent: function(e){
		// summary:
		//		gets a child, which is under the mouse at the moment, or null
		// e: Event
		//		a mouse event
		var node = e.target;
		if(node){
			for(var parent = node.parentNode; parent; node = parent, parent = node.parentNode){
				if((parent == this.parent || this.allowNested) && domClass.contains(node, "dojoDndItem")){ return node; }
			}
		}
		return null;
	},
	_normalizedCreator: function(/*Container.Item*/ item, /*String*/ hint){
		// summary:
		//		adds all necessary data to the output of the user-supplied creator function
		var t = (this.creator || this.defaultCreator).call(this, item, hint);
		if(!lang.isArray(t.type)){ t.type = ["text"]; }
		if(!t.node.id){ t.node.id = dnd.getUniqueId(); }
		domClass.add(t.node, "dojoDndItem");
		return t;
	}
});

dnd._createNode = function(tag){
	// summary:
	//		returns a function, which creates an element of given tag
	//		(SPAN by default) and sets its innerHTML to given text
	// tag: String
	//		a tag name or empty for SPAN
	if(!tag){ return dnd._createSpan; }
	return function(text){	// Function
		return domConstruct.create(tag, {innerHTML: text});	// Node
	};
};

dnd._createTrTd = function(text){
	// summary:
	//		creates a TR/TD structure with given text as an innerHTML of TD
	// text: String
	//		a text for TD
	var tr = domConstruct.create("tr");
	domConstruct.create("td", {innerHTML: text}, tr);
	return tr;	// Node
};

dnd._createSpan = function(text){
	// summary:
	//		creates a SPAN element with given text as its innerHTML
	// text: String
	//		a text for SPAN
	return domConstruct.create("span", {innerHTML: text});	// Node
};

// dnd._defaultCreatorNodes: Object
//		a dictionary that maps container tag names to child tag names
dnd._defaultCreatorNodes = {ul: "li", ol: "li", div: "div", p: "div"};

dnd._defaultCreator = function(node){
	// summary:
	//		takes a parent node, and returns an appropriate creator function
	// node: Node
	//		a container node
	var tag = node.tagName.toLowerCase();
	var c = tag == "tbody" || tag == "thead" ? dnd._createTrTd :
			dnd._createNode(dnd._defaultCreatorNodes[tag]);
	return function(item, hint){	// Function
		var isObj = item && lang.isObject(item), data, type, n;
		if(isObj && item.tagName && item.nodeType && item.getAttribute){
			// process a DOM node
			data = item.getAttribute("dndData") || item.innerHTML;
			type = item.getAttribute("dndType");
			type = type ? type.split(/\s*,\s*/) : ["text"];
			n = item;	// this node is going to be moved rather than copied
		}else{
			// process a DnD item object or a string
			data = (isObj && item.data) ? item.data : item;
			type = (isObj && item.type) ? item.type : ["text"];
			n = (hint == "avatar" ? dnd._createSpan : c)(String(data));
		}
		if(!n.id){
			n.id = dnd.getUniqueId();
		}
		return {node: n, data: data, type: type};
	};
};

/*=====
Container.__ContainerArgs = declare([], {
	creator: function(){
		// summary:
		//		a creator function, which takes a data item, and returns an object like that:
		//		{node: newNode, data: usedData, type: arrayOfStrings}
	},

	// skipForm: Boolean
	//		don't start the drag operation, if clicked on form elements
	skipForm: false,

	// dropParent: Node||String
	//		node or node's id to use as the parent node for dropped items
	//		(must be underneath the 'node' parameter in the DOM)
	dropParent: null,

	// _skipStartup: Boolean
	//		skip startup(), which collects children, for deferred initialization
	//		(this is used in the markup mode)
	_skipStartup: false
});

Container.Item = function(){
	// summary:
	//		Represents (one of) the source node(s) being dragged.
	//		Contains (at least) the "type" and "data" attributes.
	// type: String[]
	//		Type(s) of this item, by default this is ["text"]
	// data: Object
	//		Logical representation of the object being dragged.
	//		If the drag object's type is "text" then data is a String,
	//		if it's another type then data could be a different Object,
	//		perhaps a name/value hash.

	this.type = type;
	this.data = data;
};
=====*/

return Container;
});

},
'xcitec/xlayout/QualificationLayoutBase':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom",
	"dojo/dom-attr",
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, declare, dom, domAttr, domStyle, geometry, _Templated, _Widget) {

	/*
	    Base class for QLF layouts, do not instantiate it directly
	 */
	var QualificationLayoutBase = declare( [_Widget, _Templated], {
	    templateString: '<div data-dojo-attach-point="containerNode"></div>',
	
	    // nodes to be placed in the first column (Array)
	    firstColumnControls: null,
	
	    // nodes to be placed in the second column (Array)
	    secondColumnControls: null,
	
	    // is in design mode
	    designMode: false,
	
	    // order of elements in each column (Array)
	    firstColumnOrder: null,
	    secondColumnOrder: null,
	
	    // references to columns
	    pane1: null,
	    pane2: null,
	
	    delimeterId: "",
	
	    // helper nodes for DnD
	    helperNodeLeft : null,
	    helperNodeRight: null,
	
	    hasDelimeter: false,
	
	    onDeleteAttr: null,
	    onSettingsAttr: null,
	    /*onChangedAttr: null, */
	
	    postMixInProperties: function() {
		    this.firstColumnControls = [];
		    this.secondColumnControls = [];
		    this.firstColumnOrder = [];
		    this.secondColumnOrder = [];
	        this.inherited(arguments);
	        this.order = dom.byId(this.id + ".order");
	        this.clicked = dom.byId(this.id + ".clicked");
	        this.onDeleteAttr = 'onDelete' in this.params;
	        this.onSettingsAttr = 'onSettings' in this.params;
	        /*this.onChangedAttr = dojo.attr(this.srcNodeRef, 'onChanged'); */
	    },
	
	    resizeHelperNodes: function() {
	        if (this.pane2.getColumnOrder().length == 0 && this.pane1.getColumnOrder().length != 0) {
	          domStyle.set(this.helperNodeLeft, "height", "0px");
	            domStyle.set(this.helperNodeRight, "height", geometry.getMarginBox(this.pane1.domNode).h + "px");
	        } else if (this.pane1.getColumnOrder().length == 0 && this.pane2.getColumnOrder().length != 0) {
	          domStyle.set(this.helperNodeRight, "height", "0px");
	            domStyle.set(this.helperNodeLeft, "height", geometry.getMarginBox(this.pane2.domNode).h + "px");
	        } else {
	            domStyle.set(this.helperNodeLeft, "height", "0px");
	            domStyle.set(this.helperNodeRight, "height", "0px");
	        }
	    },
	
	    isDeleteSet: function() {
	        return this.onDeleteAttr;
	    },
	
	    isSettingsSet: function() {
	        return this.onSettingsAttr;
	    },
	
	    onDelete: function() {
	
	    },
	
	    onSettings: function() {
	
	    }
	});
	return QualificationLayoutBase;
});

},
'xcitec/widgets/WidgetBox':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/text!xcitec/widgets/templates/WidgetBox.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "../_base/ImageFactory"
], function (dojo, declare, lang, windowModule, domClass, domStyle, template, _Templated, _Widget, ImageFactory) {

	var WidgetBox = declare( [_Widget, _Templated], {
	    templateString: template,
	
	    caption: "",
	    captionId: "",
		captionIcon: "",
	  iconClass: "",
	//	widgetHeight: "auto",
		widgetHeight: "2",
		widgetHidden: "",
	
		buildRendering: function() {
			this.inherited(arguments);
	        this.text.innerHTML = this.caption;
	        this.saveOnClick = this.onClick;
	        this.onClick = lang.hitch(this, "_onClick");
	
	  		var height = Number(this.widgetHeight);
			if(height){
				// protlet Height =
				// 96px - sigle height; 12px - gap between portlets
				// 77px - height of content 1-height portlet
				//dojo.style(this.domNode, { height: ""+ (height*(96+12)-12) +"px" });
				//dojo.style(this.containerNode, { height: ""+ (77 + (height-1)*(96+12)) +"px" });
	
				// 81px - 1x height; 9px - gap between portlets
				// 62px - height of content 1-height portlet;
				domStyle.set(this.containerNode, { height: ""+ (62 + (height-1)*(81+9)) +"px" });
				//dojo.marginBox(this.domNode, {h: (height*(96+12)-12)});
			}
			if(this.widgetHidden){
				domStyle.set(this.containerNode, { lineHeight: ""+ (62 + (height-1)*(81+9)) +"px" });
				domClass.add(this.containerNode, "xwidget_content_hidden");
				this.containerNode.appendChild(windowModule.doc.createTextNode(this.widgetHidden));
			}
	
	    },
	    
	    postMixInProperties: function(){
	      if (this.captionIcon) {
	        this.iconClass = ImageFactory.createIconCss("SGOMINI:"+ this.captionIcon);
	      }
			  this.inherited(arguments);
			},
	
	    _onClick: function(e) {
	        // block function for onclick
	    },
	
	    _onIconClick: function() {
	        this.saveOnClick();
	    }
	});
	
	return WidgetBox;
});

},
'dijit/place':function(){
define([
	"dojo/_base/array", // array.forEach array.map array.some
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/window", // win.body
	"./Viewport", // getEffectiveBox
	"./main"	// dijit (defining dijit.place to match API doc)
], function(array, domGeometry, domStyle, kernel, win, Viewport, dijit){

	// module:
	//		dijit/place


	function _place(/*DomNode*/ node, choices, layoutNode, aroundNodeCoords){
		// summary:
		//		Given a list of spots to put node, put it at the first spot where it fits,
		//		of if it doesn't fit anywhere then the place with the least overflow
		// choices: Array
		//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
		//		Above example says to put the top-left corner of the node at (10,20)
		// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
		//		for things like tooltip, they are displayed differently (and have different dimensions)
		//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
		//		It also passes in the available size for the popup, which is useful for tooltips to
		//		tell them that their width is limited to a certain amount.	 layoutNode() may return a value expressing
		//		how much the popup had to be modified to fit into the available space.	 This is used to determine
		//		what the best placement is.
		// aroundNodeCoords: Object
		//		Size of aroundNode, ex: {w: 200, h: 50}

		// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
		// viewport over document
		var view = Viewport.getEffectiveBox(node.ownerDocument);

		// This won't work if the node is inside a <div style="position: relative">,
		// so reattach it to <body>.	 (Otherwise, the positioning will be wrong
		// and also it might get cutoff.)
		if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
			win.body(node.ownerDocument).appendChild(node);
		}

		var best = null;
		array.some(choices, function(choice){
			var corner = choice.corner;
			var pos = choice.pos;
			var overflow = 0;

			// calculate amount of space available given specified position of node
			var spaceAvailable = {
				w: {
					'L': view.l + view.w - pos.x,
					'R': pos.x - view.l,
					'M': view.w
				}[corner.charAt(1)],
				h: {
					'T': view.t + view.h - pos.y,
					'B': pos.y - view.t,
					'M': view.h
				}[corner.charAt(0)]
			};

			// Clear left/right position settings set earlier so they don't interfere with calculations,
			// specifically when layoutNode() (a.k.a. Tooltip.orient()) measures natural width of Tooltip
			var s = node.style;
			s.left = s.right = "auto";

			// configure node to be displayed in given position relative to button
			// (need to do this in order to get an accurate size for the node, because
			// a tooltip's size changes based on position, due to triangle)
			if(layoutNode){
				var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
				overflow = typeof res == "undefined" ? 0 : res;
			}

			// get node's size
			var style = node.style;
			var oldDisplay = style.display;
			var oldVis = style.visibility;
			if(style.display == "none"){
				style.visibility = "hidden";
				style.display = "";
			}
			var bb = domGeometry.position(node);
			style.display = oldDisplay;
			style.visibility = oldVis;

			// coordinates and size of node with specified corner placed at pos,
			// and clipped by viewport
			var
				startXpos = {
					'L': pos.x,
					'R': pos.x - bb.w,
					'M': Math.max(view.l, Math.min(view.l + view.w, pos.x + (bb.w >> 1)) - bb.w) // M orientation is more flexible
				}[corner.charAt(1)],
				startYpos = {
					'T': pos.y,
					'B': pos.y - bb.h,
					'M': Math.max(view.t, Math.min(view.t + view.h, pos.y + (bb.h >> 1)) - bb.h)
				}[corner.charAt(0)],
				startX = Math.max(view.l, startXpos),
				startY = Math.max(view.t, startYpos),
				endX = Math.min(view.l + view.w, startXpos + bb.w),
				endY = Math.min(view.t + view.h, startYpos + bb.h),
				width = endX - startX,
				height = endY - startY;

			overflow += (bb.w - width) + (bb.h - height);

			if(best == null || overflow < best.overflow){
				best = {
					corner: corner,
					aroundCorner: choice.aroundCorner,
					x: startX,
					y: startY,
					w: width,
					h: height,
					overflow: overflow,
					spaceAvailable: spaceAvailable
				};
			}

			return !overflow;
		});

		// In case the best position is not the last one we checked, need to call
		// layoutNode() again.
		if(best.overflow && layoutNode){
			layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
		}

		// And then position the node.  Do this last, after the layoutNode() above
		// has sized the node, due to browser quirks when the viewport is scrolled
		// (specifically that a Tooltip will shrink to fit as though the window was
		// scrolled to the left).

		var top = best.y,
			side = best.x,
			body = win.body(node.ownerDocument);

		if(/relative|absolute/.test(domStyle.get(body, "position"))){
			// compensate for margin on <body>, see #16148
			top -= domStyle.get(body, "marginTop");
			side -= domStyle.get(body, "marginLeft");
		}

		var s = node.style;
		s.top = top + "px";
		s.left = side + "px";
		s.right = "auto";	// needed for FF or else tooltip goes to far left

		return best;
	}

	var reverse = {
		// Map from corner to kitty-corner
		"TL": "BR",
		"TR": "BL",
		"BL": "TR",
		"BR": "TL"
	};

	var place = {
		// summary:
		//		Code to place a DOMNode relative to another DOMNode.
		//		Load using require(["dijit/place"], function(place){ ... }).

		at: function(node, pos, corners, padding, layoutNode){
			// summary:
			//		Positions node kitty-corner to the rectangle centered at (pos.x, pos.y) with width and height of
			//		padding.x * 2 and padding.y * 2, or zero if padding not specified.  Picks first corner in corners[]
			//		where node is fully visible, or the corner where it's most visible.
			//
			//		Node is assumed to be absolutely or relatively positioned.
			// node: DOMNode
			//		The node to position
			// pos: dijit/place.__Position
			//		Object like {x: 10, y: 20}
			// corners: String[]
			//		Array of Strings representing order to try corners of the node in, like ["TR", "BL"].
			//		Possible values are:
			//
			//		- "BL" - bottom left
			//		- "BR" - bottom right
			//		- "TL" - top left
			//		- "TR" - top right
			// padding: dijit/place.__Position?
			//		Optional param to set padding, to put some buffer around the element you want to position.
			//		Defaults to zero.
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.  This adjusts the popup based on orientation.
			// example:
			//		Try to place node's top right corner at (10,20).
			//		If that makes node go (partially) off screen, then try placing
			//		bottom left corner at (10,20).
			//	|	place(node, {x: 10, y: 20}, ["TR", "BL"])
			var choices = array.map(corners, function(corner){
				var c = {
					corner: corner,
					aroundCorner: reverse[corner],	// so TooltipDialog.orient() gets aroundCorner argument set
					pos: {x: pos.x,y: pos.y}
				};
				if(padding){
					c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
					c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
				}
				return c;
			});

			return _place(node, choices, layoutNode);
		},

		around: function(
			/*DomNode*/		node,
			/*DomNode|dijit/place.__Rectangle*/ anchor,
			/*String[]*/	positions,
			/*Boolean*/		leftToRight,
			/*Function?*/	layoutNode){

			// summary:
			//		Position node adjacent or kitty-corner to anchor
			//		such that it's fully visible in viewport.
			// description:
			//		Place node such that corner of node touches a corner of
			//		aroundNode, and that node is fully visible.
			// anchor:
			//		Either a DOMNode or a rectangle (object with x, y, width, height).
			// positions:
			//		Ordered list of positions to try matching up.
			//
			//		- before: places drop down to the left of the anchor node/widget, or to the right in the case
			//			of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//			with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//		- after: places drop down to the right of the anchor node/widget, or to the left in the case
			//			of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//			with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//		- before-centered: centers drop down to the left of the anchor node/widget, or to the right
			//			in the case of RTL scripts like Hebrew and Arabic
			//		- after-centered: centers drop down to the right of the anchor node/widget, or to the left
			//			in the case of RTL scripts like Hebrew and Arabic
			//		- above-centered: drop down is centered above anchor node
			//		- above: drop down goes above anchor node, left sides aligned
			//		- above-alt: drop down goes above anchor node, right sides aligned
			//		- below-centered: drop down is centered above anchor node
			//		- below: drop down goes below anchor node
			//		- below-alt: drop down goes below anchor node, right sides aligned
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
			// leftToRight:
			//		True if widget is LTR, false if widget is RTL.   Affects the behavior of "above" and "below"
			//		positions slightly.
			// example:
			//	|	placeAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
			//		This will try to position node such that node's top-left corner is at the same position
			//		as the bottom left corner of the aroundNode (ie, put node below
			//		aroundNode, with left edges aligned).	If that fails it will try to put
			//		the bottom-right corner of node where the top right corner of aroundNode is
			//		(ie, put node above aroundNode, with right edges aligned)
			//

			// If around is a DOMNode (or DOMNode id), convert to coordinates.
			var aroundNodePos;
			if(typeof anchor == "string" || "offsetWidth" in anchor || "ownerSVGElement" in anchor){
				aroundNodePos = domGeometry.position(anchor, true);

				// For above and below dropdowns, subtract width of border so that popup and aroundNode borders
				// overlap, preventing a double-border effect.  Unfortunately, difficult to measure the border
				// width of either anchor or popup because in both cases the border may be on an inner node.
				if(/^(above|below)/.test(positions[0])){
					var anchorBorder = domGeometry.getBorderExtents(anchor),
						anchorChildBorder = anchor.firstChild ? domGeometry.getBorderExtents(anchor.firstChild) : {t:0,l:0,b:0,r:0},
						nodeBorder =  domGeometry.getBorderExtents(node),
						nodeChildBorder = node.firstChild ? domGeometry.getBorderExtents(node.firstChild) : {t:0,l:0,b:0,r:0};
					aroundNodePos.y += Math.min(anchorBorder.t + anchorChildBorder.t, nodeBorder.t + nodeChildBorder.t);
					aroundNodePos.h -=  Math.min(anchorBorder.t + anchorChildBorder.t, nodeBorder.t+ nodeChildBorder.t) +
						Math.min(anchorBorder.b + anchorChildBorder.b, nodeBorder.b + nodeChildBorder.b);
				}
			}else{
				aroundNodePos = anchor;
			}

			// Compute position and size of visible part of anchor (it may be partially hidden by ancestor nodes w/scrollbars)
			if(anchor.parentNode){
				// ignore nodes between position:relative and position:absolute
				var sawPosAbsolute = domStyle.getComputedStyle(anchor).position == "absolute";
				var parent = anchor.parentNode;
				while(parent && parent.nodeType == 1 && parent.nodeName != "BODY"){  //ignoring the body will help performance
					var parentPos = domGeometry.position(parent, true),
						pcs = domStyle.getComputedStyle(parent);
					if(/relative|absolute/.test(pcs.position)){
						sawPosAbsolute = false;
					}
					if(!sawPosAbsolute && /hidden|auto|scroll/.test(pcs.overflow)){
						var bottomYCoord = Math.min(aroundNodePos.y + aroundNodePos.h, parentPos.y + parentPos.h);
						var rightXCoord = Math.min(aroundNodePos.x + aroundNodePos.w, parentPos.x + parentPos.w);
						aroundNodePos.x = Math.max(aroundNodePos.x, parentPos.x);
						aroundNodePos.y = Math.max(aroundNodePos.y, parentPos.y);
						aroundNodePos.h = bottomYCoord - aroundNodePos.y;
						aroundNodePos.w = rightXCoord - aroundNodePos.x;
					}
					if(pcs.position == "absolute"){
						sawPosAbsolute = true;
					}
					parent = parent.parentNode;
				}
			}			

			var x = aroundNodePos.x,
				y = aroundNodePos.y,
				width = "w" in aroundNodePos ? aroundNodePos.w : (aroundNodePos.w = aroundNodePos.width),
				height = "h" in aroundNodePos ? aroundNodePos.h : (kernel.deprecated("place.around: dijit/place.__Rectangle: { x:"+x+", y:"+y+", height:"+aroundNodePos.height+", width:"+width+" } has been deprecated.  Please use { x:"+x+", y:"+y+", h:"+aroundNodePos.height+", w:"+width+" }", "", "2.0"), aroundNodePos.h = aroundNodePos.height);

			// Convert positions arguments into choices argument for _place()
			var choices = [];
			function push(aroundCorner, corner){
				choices.push({
					aroundCorner: aroundCorner,
					corner: corner,
					pos: {
						x: {
							'L': x,
							'R': x + width,
							'M': x + (width >> 1)
						}[aroundCorner.charAt(1)],
						y: {
							'T': y,
							'B': y + height,
							'M': y + (height >> 1)
						}[aroundCorner.charAt(0)]
					}
				})
			}
			array.forEach(positions, function(pos){
				var ltr =  leftToRight;
				switch(pos){
					case "above-centered":
						push("TM", "BM");
						break;
					case "below-centered":
						push("BM", "TM");
						break;
					case "after-centered":
						ltr = !ltr;
						// fall through
					case "before-centered":
						push(ltr ? "ML" : "MR", ltr ? "MR" : "ML");
						break;
					case "after":
						ltr = !ltr;
						// fall through
					case "before":
						push(ltr ? "TL" : "TR", ltr ? "TR" : "TL");
						push(ltr ? "BL" : "BR", ltr ? "BR" : "BL");
						break;
					case "below-alt":
						ltr = !ltr;
						// fall through
					case "below":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "BL" : "BR", ltr ? "TL" : "TR");
						push(ltr ? "BR" : "BL", ltr ? "TR" : "TL");
						break;
					case "above-alt":
						ltr = !ltr;
						// fall through
					case "above":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "TL" : "TR", ltr ? "BL" : "BR");
						push(ltr ? "TR" : "TL", ltr ? "BR" : "BL");
						break;
					default:
						// To assist dijit/_base/place, accept arguments of type {aroundCorner: "BL", corner: "TL"}.
						// Not meant to be used directly.  Remove for 2.0.
						push(pos.aroundCorner, pos.corner);
				}
			});

			var position = _place(node, choices, layoutNode, {w: width, h: height});
			position.aroundNodePos = aroundNodePos;

			return position;
		}
	};

	/*=====
	place.__Position = {
		// x: Integer
		//		horizontal coordinate in pixels, relative to document body
		// y: Integer
		//		vertical coordinate in pixels, relative to document body
	};
	place.__Rectangle = {
		// x: Integer
		//		horizontal offset in pixels, relative to document body
		// y: Integer
		//		vertical offset in pixels, relative to document body
		// w: Integer
		//		width in pixels.   Can also be specified as "width" for backwards-compatibility.
		// h: Integer
		//		height in pixels.   Can also be specified as "height" for backwards-compatibility.
	};
	=====*/

	return dijit.place = place;	// setting dijit.place for back-compat, remove for 2.0
});

},
'dijit/layout/_LayoutWidget':function(){
define([
	"dojo/_base/lang", // lang.mixin
	"../_Widget",
	"../_Container",
	"../_Contained",
	"../Viewport",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style" // domStyle.getComputedStyle
], function(lang, _Widget, _Container, _Contained, Viewport,
	declare, domClass, domGeometry, domStyle){

	// module:
	//		dijit/layout/_LayoutWidget


	return declare("dijit.layout._LayoutWidget", [_Widget, _Container, _Contained], {
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under <body>.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent();
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when viewport size changes so that I can re-layout.
				this.own(Viewport.on("resize", lang.hitch(this, "resize")));
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		####Change size mode:
			//
			//		When changeSize is specified, changes the marginBox of this widget
			//		and forces it to re-layout its contents accordingly.
			//		changeSize may specify height, width, or both.
			//
			//		If resultSize is specified it indicates the size the widget will
			//		become after changeSize has been applied.
			//
			//		####Notification mode:
			//
			//		When changeSize is null, indicates that the caller has already changed
			//		the size of the widget, or perhaps it changed because the browser
			//		window was resized.  Tells widget to re-layout its contents accordingly.
			//
			//		If resultSize is also specified it indicates the size the widget has
			//		become.
			//
			//		In either mode, this method also:
			//
			//		1. Sets this._borderBox and this._contentBox to the new size of
			//			the widget.  Queries the current domNode size if necessary.
			//		2. Calls layout() to resize contents (and maybe adjust child widgets).
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				domGeometry.setMarginBox(node, changeSize);
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = lang.mixin(domGeometry.getMarginBox(node), mb);	// just use domGeometry.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling domGeometry.getContentBox() since that may fail if size was recently set)
			var cs = domStyle.getComputedStyle(node);
			var me = domGeometry.getMarginExtents(node, cs);
			var be = domGeometry.getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = domGeometry.getPadExtents(node, cs);
			this._contentBox = {
				l: domStyle.toPixelValue(node, cs.paddingLeft),
				t: domStyle.toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit/_WidgetBase*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			domClass.add(child.domNode, cls);
		},

		addChild: function(/*dijit/_WidgetBase*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit/_WidgetBase*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			domClass.remove(child.domNode, cls);

			this.inherited(arguments);
		}
	});
});

},
'dojo/cldr/supplemental':function(){
define(["../_base/lang", "../i18n"], function(lang, i18n){

// module:
//		dojo/cldr/supplemental


var supplemental = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.cldr.supplemental", supplemental);

supplemental.getFirstDayOfWeek = function(/*String?*/locale){
	// summary:
	//		Returns a zero-based index for first day of the week
	// description:
	//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
	//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		bd:5,mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,iq:6,ir:6,jo:6,kw:6,
		ly:6,ma:6,om:6,qa:6,sa:6,sd:6,sy:6,ye:6,
		ag:0,ar:0,as:0,au:0,br:0,bs:0,bt:0,bw:0,by:0,bz:0,ca:0,cn:0,
		co:0,dm:0,'do':0,et:0,gt:0,gu:0,hk:0,hn:0,id:0,ie:0,il:0,'in':0,
		jm:0,jp:0,ke:0,kh:0,kr:0,la:0,mh:0,mm:0,mo:0,mt:0,mx:0,mz:0,
		ni:0,np:0,nz:0,pa:0,pe:0,ph:0,pk:0,pr:0,py:0,sg:0,sv:0,th:0,
		tn:0,tt:0,tw:0,um:0,us:0,ve:0,vi:0,ws:0,za:0,zw:0
	};

	var country = supplemental._region(locale);
	var dow = firstDay[country];
	return (dow === undefined) ? 1 : dow; /*Number*/
};

supplemental._region = function(/*String?*/locale){
	locale = i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
		region = {
			aa:"et", ab:"ge", af:"za", ak:"gh", am:"et", ar:"eg", as:"in", av:"ru", ay:"bo", az:"az", ba:"ru",
			be:"by", bg:"bg", bi:"vu", bm:"ml", bn:"bd", bo:"cn", br:"fr", bs:"ba", ca:"es", ce:"ru", ch:"gu",
			co:"fr", cr:"ca", cs:"cz", cv:"ru", cy:"gb", da:"dk", de:"de", dv:"mv", dz:"bt", ee:"gh", el:"gr",
			en:"us", es:"es", et:"ee", eu:"es", fa:"ir", ff:"sn", fi:"fi", fj:"fj", fo:"fo", fr:"fr", fy:"nl",
			ga:"ie", gd:"gb", gl:"es", gn:"py", gu:"in", gv:"gb", ha:"ng", he:"il", hi:"in", ho:"pg", hr:"hr",
			ht:"ht", hu:"hu", hy:"am", ia:"fr", id:"id", ig:"ng", ii:"cn", ik:"us", "in":"id", is:"is", it:"it",
			iu:"ca", iw:"il", ja:"jp", ji:"ua", jv:"id", jw:"id", ka:"ge", kg:"cd", ki:"ke", kj:"na", kk:"kz",
			kl:"gl", km:"kh", kn:"in", ko:"kr", ks:"in", ku:"tr", kv:"ru", kw:"gb", ky:"kg", la:"va", lb:"lu",
			lg:"ug", li:"nl", ln:"cd", lo:"la", lt:"lt", lu:"cd", lv:"lv", mg:"mg", mh:"mh", mi:"nz", mk:"mk",
			ml:"in", mn:"mn", mo:"ro", mr:"in", ms:"my", mt:"mt", my:"mm", na:"nr", nb:"no", nd:"zw", ne:"np",
			ng:"na", nl:"nl", nn:"no", no:"no", nr:"za", nv:"us", ny:"mw", oc:"fr", om:"et", or:"in", os:"ge",
			pa:"in", pl:"pl", ps:"af", pt:"br", qu:"pe", rm:"ch", rn:"bi", ro:"ro", ru:"ru", rw:"rw", sa:"in",
			sd:"in", se:"no", sg:"cf", si:"lk", sk:"sk", sl:"si", sm:"ws", sn:"zw", so:"so", sq:"al", sr:"rs",
			ss:"za", st:"za", su:"id", sv:"se", sw:"tz", ta:"in", te:"in", tg:"tj", th:"th", ti:"et", tk:"tm",
			tl:"ph", tn:"za", to:"to", tr:"tr", ts:"za", tt:"ru", ty:"pf", ug:"cn", uk:"ua", ur:"pk", uz:"uz",
			ve:"za", vi:"vn", wa:"be", wo:"sn", xh:"za", yi:"il", yo:"ng", za:"cn", zh:"cn", zu:"za",
			ace:"id", ady:"ru", agq:"cm", alt:"ru", amo:"ng", asa:"tz", ast:"es", awa:"in", bal:"pk",
			ban:"id", bas:"cm", bax:"cm", bbc:"id", bem:"zm", bez:"tz", bfq:"in", bft:"pk", bfy:"in",
			bhb:"in", bho:"in", bik:"ph", bin:"ng", bjj:"in", bku:"ph", bqv:"ci", bra:"in", brx:"in",
			bss:"cm", btv:"pk", bua:"ru", buc:"yt", bug:"id", bya:"id", byn:"er", cch:"ng", ccp:"in",
			ceb:"ph", cgg:"ug", chk:"fm", chm:"ru", chp:"ca", chr:"us", cja:"kh", cjm:"vn", ckb:"iq",
			crk:"ca", csb:"pl", dar:"ru", dav:"ke", den:"ca", dgr:"ca", dje:"ne", doi:"in", dsb:"de",
			dua:"cm", dyo:"sn", dyu:"bf", ebu:"ke", efi:"ng", ewo:"cm", fan:"gq", fil:"ph", fon:"bj",
			fur:"it", gaa:"gh", gag:"md", gbm:"in", gcr:"gf", gez:"et", gil:"ki", gon:"in", gor:"id",
			grt:"in", gsw:"ch", guz:"ke", gwi:"ca", haw:"us", hil:"ph", hne:"in", hnn:"ph", hoc:"in",
			hoj:"in", ibb:"ng", ilo:"ph", inh:"ru", jgo:"cm", jmc:"tz", kaa:"uz", kab:"dz", kaj:"ng",
			kam:"ke", kbd:"ru", kcg:"ng", kde:"tz", kdt:"th", kea:"cv", ken:"cm", kfo:"ci", kfr:"in",
			kha:"in", khb:"cn", khq:"ml", kht:"in", kkj:"cm", kln:"ke", kmb:"ao", koi:"ru", kok:"in",
			kos:"fm", kpe:"lr", krc:"ru", kri:"sl", krl:"ru", kru:"in", ksb:"tz", ksf:"cm", ksh:"de",
			kum:"ru", lag:"tz", lah:"pk", lbe:"ru", lcp:"cn", lep:"in", lez:"ru", lif:"np", lis:"cn",
			lki:"ir", lmn:"in", lol:"cd", lua:"cd", luo:"ke", luy:"ke", lwl:"th", mad:"id", mag:"in",
			mai:"in", mak:"id", man:"gn", mas:"ke", mdf:"ru", mdh:"ph", mdr:"id", men:"sl", mer:"ke",
			mfe:"mu", mgh:"mz", mgo:"cm", min:"id", mni:"in", mnk:"gm", mnw:"mm", mos:"bf", mua:"cm",
			mwr:"in", myv:"ru", nap:"it", naq:"na", nds:"de", "new":"np", niu:"nu", nmg:"cm", nnh:"cm",
			nod:"th", nso:"za", nus:"sd", nym:"tz", nyn:"ug", pag:"ph", pam:"ph", pap:"bq", pau:"pw",
			pon:"fm", prd:"ir", raj:"in", rcf:"re", rej:"id", rjs:"np", rkt:"in", rof:"tz", rwk:"tz",
			saf:"gh", sah:"ru", saq:"ke", sas:"id", sat:"in", saz:"in", sbp:"tz", scn:"it", sco:"gb",
			sdh:"ir", seh:"mz", ses:"ml", shi:"ma", shn:"mm", sid:"et", sma:"se", smj:"se", smn:"fi",
			sms:"fi", snk:"ml", srn:"sr", srr:"sn", ssy:"er", suk:"tz", sus:"gn", swb:"yt", swc:"cd",
			syl:"bd", syr:"sy", tbw:"ph", tcy:"in", tdd:"cn", tem:"sl", teo:"ug", tet:"tl", tig:"er",
			tiv:"ng", tkl:"tk", tmh:"ne", tpi:"pg", trv:"tw", tsg:"ph", tts:"th", tum:"mw", tvl:"tv",
			twq:"ne", tyv:"ru", tzm:"ma", udm:"ru", uli:"fm", umb:"ao", unr:"in", unx:"in", vai:"lr",
			vun:"tz", wae:"ch", wal:"et", war:"ph", xog:"ug", xsr:"np", yao:"mz", yap:"fm", yav:"cm", zza:"tr"
		}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
};

supplemental.getWeekend = function(/*String?*/locale){
	// summary:
	//		Returns a hash containing the start and end days of the weekend
	// description:
	//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
	//		or by default in the user's locale.
	//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
			'in':0,
			af:4,dz:4,ir:4,om:4,sa:4,ye:4,
			ae:5,bh:5,eg:5,il:5,iq:5,jo:5,kw:5,ly:5,ma:5,qa:5,sd:5,sy:5,tn:5
		},

		weekendEnd = {/*default is 0=Sunday*/
			af:5,dz:5,ir:5,om:5,sa:5,ye:5,
			ae:6,bh:5,eg:6,il:6,iq:6,jo:6,kw:6,ly:6,ma:6,qa:6,sd:6,sy:6,tn:6
		},

		country = supplemental._region(locale),
		start = weekendStart[country],
		end = weekendEnd[country];

	if(start === undefined){start=6;}
	if(end === undefined){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

return supplemental;
});

},
'idx/typematic':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect", // connect._keyPress
	"dojo/_base/lang", // lang.mixin, lang.hitch
	"dojo/on",
	"dojo/touch",
	"dojo/sniff", // has("ie")
	"dijit/typematic"        // setting dijit.typematic global
], function(declare, array, connect, lang, on, touch, has, typematic){

	// module:
	//		idx/typematic
	var idx_typematic = {};

	lang.mixin(idx_typematic, typematic, {
		addTouchListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			var handles = [
				on(node, touch.press, lang.hitch(this, function(evt){
					evt.preventDefault();
					typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
				})),
				on(node, touch.release, lang.hitch(this, function(evt){
					if(this._obj){
						evt.preventDefault();
					}
					typematic.stop();
				})),
				on(node, "mouseout", lang.hitch(this, function(evt){
					if(this._obj){
						evt.preventDefault();
					}
					typematic.stop();
				})),
				on(node, "dblclick", lang.hitch(this, function(evt){
					evt.preventDefault();
					if(has("ie") < 9){
						typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
						setTimeout(lang.hitch(this, typematic.stop), 50);
					}
				}))
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		}

	});

	return idx_typematic;
});

},
'dojo/number':function(){
define([/*===== "./_base/declare", =====*/ "./_base/lang", "./i18n", "./i18n!./cldr/nls/number", "./string", "./regexp"],
	function(/*===== declare, =====*/ lang, i18n, nlsNumber, dstring, dregexp){

// module:
//		dojo/number

var number = {
	// summary:
	//		localized formatting and parsing routines for Number
};
lang.setObject("dojo.number", number);

/*=====
number.__FormatOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	// round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means do not round.
	// locale: String?
	//		override the locale used to determine formatting rules
	// fractional: Boolean?
	//		If false, show no decimal places, overriding places and pattern settings.
});
=====*/

number.format = function(/*Number*/ value, /*number.__FormatOptions?*/ options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		[Common Locale Data Repository](http://unicode.org/cldr) as well as the appropriate symbols and
	//		delimiters.
	//		If value is Infinity, -Infinity, or is not a valid JavaScript number, return null.
	// value:
	//		the number to be formatted

	options = lang.mixin({}, options || {});
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value) || Math.abs(value) == Infinity){ return null; } // null
	return number._applyPattern(value, pattern, options); // String
};

//number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

number._applyPattern = function(/*Number*/ value, /*String*/ pattern, /*number.__FormatOptions?*/ options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	// options: number.__FormatOptions?
	//		_applyPattern is usually called via `dojo/number.format()` which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group,
		decimal = options.customs.decimal,
		patternList = pattern.split(';'),
		positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}

	//TODO: support @ sig figs?
	var numberPatternRE = number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	if(options.fractional === false){ options.places = 0; }
	return pattern.replace(numberPatternRE,
		number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places, round: options.round}));
};

number.round = function(/*Number*/ value, /*Number?*/ places, /*Number?*/ increment){
	// summary:
	//		Rounds to the nearest value with the given number of decimal places, away from zero
	// description:
	//		Rounds to the nearest value with the given number of decimal places, away from zero if equal.
	//		Similar to Number.toFixed(), but compensates for browser quirks. Rounding can be done by
	//		fractional increments also, such as the nearest quarter.
	//		NOTE: Subject to floating point errors.  See dojox/math/round for experimental workaround.
	// value:
	//		The number to round
	// places:
	//		The number of decimal places where rounding takes place.  Defaults to 0 for whole rounding.
	//		Must be non-negative.
	// increment:
	//		Rounds next place to nearest value of increment/10.  10 by default.
	// example:
	// |	>>> number.round(-0.5)
	// |	-1
	// |	>>> number.round(162.295, 2)
	// |	162.29  // note floating point error.  Should be 162.3
	// |	>>> number.round(10.71, 0, 2.5)
	// |	10.75
	var factor = 10 / (increment || 10);
	return (factor * +value).toFixed(places) / factor; // Number
};

if((0.9).toFixed() == 0){
	// (isIE) toFixed() bug workaround: Rounding fails on IE when most significant digit
	// is just after the rounding place and is >=5
	var round = number.round;
	number.round = function(v, p, m){
		var d = Math.pow(10, -p || 0), a = Math.abs(v);
		if(!v || a >= d){
			d = 0;
		}else{
			a /= d;
			if(a < 0.5 || a >= 0.95){
				d = 0;
			}
		}
		return round(v, p, m) + (v > 0 ? d : -d);
	};

	// Use "doc hint" so the doc parser ignores this new definition of round(), and uses the one above.
	/*===== number.round = round; =====*/
}

/*=====
number.__FormatAbsoluteOptions = declare(null, {
	// decimal: String?
	//		the decimal separator
	// group: String?
	//		the group separator
	// places: Number|String?
	//		number of decimal places.  the range "n,m" will format to m places.
	// round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
});
=====*/

number._formatAbsolute = function(/*Number*/ value, /*String*/ pattern, /*number.__FormatAbsoluteOptions?*/ options){
	// summary:
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. `#,##0.00`)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split("."),
		comma = typeof options.places == "string" && options.places.indexOf(","),
		maxPlaces = options.places;
	if(comma){
		maxPlaces = options.places.substring(comma + 1);
	}else if(!(maxPlaces >= 0)){
		maxPlaces = (patternParts[1] || []).length;
	}
	if(!(options.round < 0)){
		value = number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split("."),
		fractional = valueParts[1] || "";
	if(patternParts[1] || options.places){
		if(comma){
			options.places = options.places.substring(0, comma);
		}
		// Pad fractional with trailing zeros
		var pad = options.places !== undefined ? options.places : (patternParts[1] && patternParts[1].lastIndexOf("0") + 1);
		if(pad > fractional.length){
			valueParts[1] = dstring.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		if(maxPlaces < fractional.length){
			valueParts[1] = fractional.substr(0, maxPlaces);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dstring.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(','),
		groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
number.__RegexpOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// locale: String?
	//		override the locale used to determine formatting rules
	// strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	// places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  Defined by pattern or Infinity if pattern not provided.
});
=====*/
number.regexp = function(/*number.__RegexpOptions?*/ options){
	// summary:
	//		Builds the regular needed to parse a number
	// description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return number._parseInfo(options).regexp; // String
};

number._parseInfo = function(/*Object?*/ options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale),
		pattern = options.pattern || bundle[(options.type || "decimal") + "Format"],
//TODO: memoize?
		group = bundle.group,
		decimal = bundle.decimal,
		factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dregexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dregexp.escapeString(pattern, '.')+")";
		return pattern.replace(number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false
				},

				parts = format.split('.'),
				places = options.places;

			// special condition for percent (factor != 1)
			// allow decimal places even if not specified in pattern
			if(parts.length == 1 && factor != 1){
			    parts[1] = "###";
			}
			if(parts.length == 1 || places === 0){
				flags.fractional = false;
			}else{
				if(places === undefined){ places = options.pattern ? parts[1].lastIndexOf('0') + 1 : Infinity; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length > 1){
				flags.groupSize = groups.pop().length;
				if(groups.length > 1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/([\s\xa0]*)(\u00a4{1,3})([\s\xa0]*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1],
				symbol = dregexp.escapeString(options[prop] || options.currency || "");
			before = before ? "[\\s\\xa0]" : "";
			after = after ? "[\\s\\xa0]" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
};

/*=====
number.__ParseOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// locale: String?
	//		override the locale used to determine formatting rules
	// strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	// fractional: Boolean|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by pattern
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
});
=====*/
number.parse = function(/*String*/ expression, /*number.__ParseOptions?*/ options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale
	//		and follow the syntax described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
    	//		Note that literal characters in patterns are not supported.
	// expression:
	//		A string representation of a Number
	var info = number._parseInfo(options),
		results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return absoluteMatch * info.factor; //Number
};

/*=====
number.__RealNumberRegexpFlags = declare(null, {
	// places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	// decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	// fractional: Boolean|Array?
	//		Whether decimal places are used.  Can be true, false, or [true,
	//		false].  Default is [true, false] which means optional.
	// exponent: Boolean|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	// eSigned: Boolean|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
});
=====*/

number._realNumberRegexp = function(/*__RealNumberRegexpFlags?*/ flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation

	// assign default values to missing parameters
	flags = flags || {};
	//TODO: use mixin instead?
	if(!("places" in flags)){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(!("fractional" in flags) || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(!("exponent" in flags)){ flags.exponent = [true, false]; }
	if(!("eSigned" in flags)){ flags.eSigned = [true, false]; }

	var integerRE = number._integerRegexp(flags),
		decimalRE = dregexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){
					re = "(?:" + re + "\\d+)?";
				}else{
					re += "\\d{" + flags.places + "}";
				}
			}
			return re;
		},
		true
	);

	var exponentRE = dregexp.buildGroupRE(flags.exponent,
		function(q){
			if(q){ return "([eE]" + number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return "";
		}
	);

	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
number.__IntegerRegexpFlags = declare(null, {
	// signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or `[true,false]`.
	//		Default is `[true, false]`, (i.e. will match if it is signed
	//		or unsigned).
	// separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. `[",", ""]`,
	//		makes ',' optional.
	// groupSize: Number?
	//		group size between separators
	// groupSize2: Number?
	//		second grouping, where separators 2..n have a different interval than the first separator (for India)
});
=====*/

number._integerRegexp = function(/*number.__IntegerRegexpFlags?*/ flags){
	// summary:
	//		Builds a regular expression that matches an integer

	// assign default values to missing parameters
	flags = flags || {};
	if(!("signed" in flags)){ flags.signed = [true, false]; }
	if(!("separator" in flags)){
		flags.separator = "";
	}else if(!("groupSize" in flags)){
		flags.groupSize = 3;
	}

	var signRE = dregexp.buildGroupRE(flags.signed,
		function(q){ return q ? "[-+]" : ""; },
		true
	);

	var numberRE = dregexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:\\d+)";
			}

			sep = dregexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			//TODO: should we continue to enforce that numbers with separators begin with 1-9?  See #6933
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	return signRE + numberRE; // String
};

return number;
});

},
'xcitec/formx/TableLayout':function(){
define([
	"dojo/_base/declare",
	"./FilterLayout"
], function (declare, FilterLayout) {
	return declare(FilterLayout, {
  });
});

},
'dijit/layout/_ContentPaneResizeMixin':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.contains domClass.toggle
	"dojo/dom-geometry", // domGeometry.contentBox domGeometry.marginBox
	"dojo/dom-style",
	"dojo/_base/lang", // lang.mixin
	"dojo/query", // query
	"dojo/sniff", // has("ie")
	"../registry", // registry.byId
	"../Viewport",
	"./utils" // marginBox2contextBox
], function(array, declare, domClass, domGeometry, domStyle, lang, query, has,
			registry, Viewport, layoutUtils){

	// module:
	//		dijit/layout/_ContentPaneResizeMixin

	return declare("dijit.layout._ContentPaneResizeMixin", null, {
		// summary:
		//		Resize() functionality of ContentPane.   If there's a single layout widget
		//		child then it will call resize() with the same dimensions as the ContentPane.
		//		Otherwise just calls resize on each child.
		//
		//		Also implements basic startup() functionality, where starting the parent
		//		will start the children

		// doLayout: Boolean
		//		- false - don't adjust size of children
		//		- true - if there is a single visible child widget, set it's size to however big the ContentPane is
		doLayout: true,

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget will call resize() on it's child widgets
		//		when they become visible.
		isLayoutContainer: true,

		startup: function(){
			// summary:
			//		See `dijit/layout/_LayoutWidget.startup()` for description.
			//		Although ContentPane doesn't extend _LayoutWidget, it does implement
			//		the same API.

			if(this._started){
				return;
			}

			var parent = this.getParent();
			this._childOfLayoutWidget = parent && parent.isLayoutContainer;

			// I need to call resize() on my child/children (when I become visible), unless
			// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
			this._needLayout = !this._childOfLayoutWidget;

			this.inherited(arguments);

			if(this._isShown()){
				this._onShow();
			}

			if(!this._childOfLayoutWidget){
				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when viewport size changes so that I can re-layout.
				// This is more for subclasses of ContentPane than ContentPane itself, although it
				// could be useful for a ContentPane if it has a single child widget inheriting ContentPane's size.
				this.own(Viewport.on("resize", lang.hitch(this, "resize")));
			}
		},

		_checkIfSingleChild: function(){
			// summary:
			//		Test if we have exactly one visible widget as a child,
			//		and if so assume that we are a container for that widget,
			//		and should propagate startup() and resize() calls to it.
			//		Skips over things like data stores since they aren't visible.

			var candidateWidgets = [],
				otherVisibleNodes = false;

			query("> *", this.containerNode).some(function(node){
				var widget = registry.byNode(node);
				if(widget && widget.resize){
					candidateWidgets.push(widget);
				}else if(!/script|link|style/i.test(node.nodeName) && node.offsetHeight){
					otherVisibleNodes = true;
				}
			});

			this._singleChild = candidateWidgets.length == 1 && !otherVisibleNodes ?
				candidateWidgets[0] : null;

			// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
			domClass.toggle(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		See `dijit/layout/_LayoutWidget.resize()` for description.
			//		Although ContentPane doesn't extend _LayoutWidget, it does implement
			//		the same API.

			this._resizeCalled = true;

			this._scheduleLayout(changeSize, resultSize);
		},

		_scheduleLayout: function(changeSize, resultSize){
			// summary:
			//		Resize myself, and call resize() on each of my child layout widgets, either now
			//		(if I'm currently visible) or when I become visible
			if(this._isShown()){
				this._layout(changeSize, resultSize);
			}else{
				this._needLayout = true;
				this._changeSize = changeSize;
				this._resultSize = resultSize;
			}
		},

		_layout: function(changeSize, resultSize){
			// summary:
			//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
			//		Also, since I am an isLayoutContainer widget, each of my children expects me to
			//		call resize() or layout() on it.
			//
			//		Should be called on initialization and also whenever we get new content
			//		(from an href, or from set('content', ...))... but deferred until
			//		the ContentPane is visible

			delete this._needLayout;

			// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
			// never called directly, so resize() is our trigger to do the initial href download (see [20099]).
			// However, don't load href for closed TitlePanes.
			if(!this._wasShown && this.open !== false){
				this._onShow();
			}

			// Set margin box size, unless it wasn't specified, in which case use current size.
			if(changeSize){
				domGeometry.setMarginBox(this.domNode, changeSize);
			}

			// Compute content box size of containerNode in case we [later] need to size our single child.
			var cn = this.containerNode;
			if(cn === this.domNode){
				// If changeSize or resultSize was passed to this method and this.containerNode ==
				// this.domNode then we can compute the content-box size without querying the node,
				// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
				var mb = resultSize || {};
				lang.mixin(mb, changeSize || {}); // changeSize overrides resultSize
				if(!("h" in mb) || !("w" in mb)){
					mb = lang.mixin(domGeometry.getMarginBox(cn), mb); // just use domGeometry.setMarginBox() to fill in missing values
				}
				this._contentBox = layoutUtils.marginBox2contentBox(cn, mb);
			}else{
				this._contentBox = domGeometry.getContentBox(cn);
			}

			this._layoutChildren();
		},

		_layoutChildren: function(){
			// Call _checkIfSingleChild() again in case app has manually mucked w/the content
			// of the ContentPane (rather than changing it through the set("content", ...) API.
			if(this.doLayout){
				this._checkIfSingleChild();
			}

			if(this._singleChild && this._singleChild.resize){
				var cb = this._contentBox || domGeometry.getContentBox(this.containerNode);

				// note: if widget has padding this._contentBox will have l and t set,
				// but don't pass them to resize() or it will doubly-offset the child
				this._singleChild.resize({w: cb.w, h: cb.h});
			}else{
				// All my child widgets are independently sized (rather than matching my size),
				// but I still need to call resize() on each child to make it layout.
				var children = this.getChildren(),
					widget,
					i = 0;
				while(widget = children[i++]){
					if(widget.resize){
						widget.resize();
					}
				}
			}
		},

		_isShown: function(){
			// summary:
			//		Returns true if the content is currently shown.
			// description:
			//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
			//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
			//		tree every call, and at least solves the performance problem on page load by deferring loading
			//		hidden ContentPanes until they are first shown

			if(this._childOfLayoutWidget){
				// If we are TitlePane, etc - we return that only *IF* we've been resized
				if(this._resizeCalled && "open" in this){
					return this.open;
				}
				return this._resizeCalled;
			}else if("open" in this){
				return this.open;		// for TitlePane, etc.
			}else{
				var node = this.domNode, parent = this.domNode.parentNode;
				return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !domClass.contains(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
			}
		},

		_onShow: function(){
			// summary:
			//		Called when the ContentPane is made visible
			// description:
			//		For a plain ContentPane, this is called on initialization, from startup().
			//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
			//		called whenever the pane is made visible.
			//
			//		Does layout/resize of child widget(s)

			// Need to keep track of whether ContentPane has been shown (which is different than
			// whether or not it's currently visible).
			this._wasShown = true;

			if(this._needLayout){
				// If a layout has been scheduled for when we become visible, do it now
				this._layout(this._changeSize, this._resultSize);
			}

			this.inherited(arguments);
		}
	});
});

},
'dijit/form/_ButtonMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/has",
	"../registry"        // registry.byNode
], function(declare, dom, has, registry){

	// module:
	//		dijit/form/_ButtonMixin

	var ButtonMixin = declare("dijit.form._ButtonMixin" + (has("dojo-bidi") ? "_NoBidi" : ""), null, {
		// summary:
		//		A mixin to add a thin standard API wrapper to a normal HTML button
		// description:
		//		A label should always be specified (through innerHTML) or the label attribute.
		//
		//		Attach points:
		//
		//		- focusNode (required): this node receives focus
		//		- valueNode (optional): this node's value gets submitted with FORM elements
		//		- containerNode (optional): this node gets the innerHTML assignment for label
		// example:
		// |	<button data-dojo-type="dijit/form/Button" onClick="...">Hello world</button>
		// example:
		// |	var button1 = new Button({label: "hello world", onClick: foo});
		// |	dojo.body().appendChild(button1.domNode);

		// label: HTML String
		//		Content to display in button.
		label: "",

		// type: [const] String
		//		Type of button (submit, reset, button, checkbox, radio)
		type: "button",

		__onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to divert the real click onto the hidden INPUT that has a native default action associated with it
			// type:
			//		private
			e.stopPropagation();
			e.preventDefault();
			if(!this.disabled){
				// cannot use on.emit since button default actions won't occur
				this.valueNode.click(e);
			}
			return false;
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions
			if(this.disabled){
				e.stopPropagation();
				e.preventDefault();
				return false;
			}
			if(this.onClick(e) === false){
				e.preventDefault();
			}
			var cancelled = e.defaultPrevented;

			// Signal Form/Dialog to submit/close.  For 2.0, consider removing this code and instead making the Form/Dialog
			// listen for bubbled click events where evt.target.type == "submit" && !evt.defaultPrevented.
			if(!cancelled && this.type == "submit" && !(this.valueNode || this.focusNode).form){
				for(var node = this.domNode; node.parentNode; node = node.parentNode){
					var widget = registry.byNode(node);
					if(widget && typeof widget._onSubmit == "function"){
						widget._onSubmit(e);
						e.preventDefault(); // action has already occurred
						cancelled = true;
						break;
					}
				}
			}

			return !cancelled;
		},

		postCreate: function(){
			this.inherited(arguments);
			dom.setSelectable(this.focusNode, false);
		},

		onClick: function(/*Event*/ /*===== e =====*/){
			// summary:
			//		Callback for when button is clicked.
			//		If type="submit", return true to perform submit, or false to cancel it.
			// type:
			//		callback
			return true;		// Boolean
		},

		_setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for set('label', ...) to work.
			// description:
			//		Set the label (text) of the button; takes an HTML string.
			this._set("label", content);
			var labelNode = this.containerNode || this.focusNode;
			labelNode.innerHTML = content;
		}
	});

	if(has("dojo-bidi")){
		ButtonMixin = declare("dijit.form._ButtonMixin", ButtonMixin, {
			_setLabelAttr: function(){
				this.inherited(arguments);
				var labelNode = this.containerNode || this.focusNode;
				this.applyTextDir(labelNode);
			}
		});
	}

	return ButtonMixin;
});

},
'xcitec/page/MainFrameV1':function(){
define([
	"dojo/_base/declare",
	"dojo/text!xcitec/page/templates/MainFrameV1.html",
	"xcitec/layout/XLayout"
], function (declare, template, XLayout) {

	/*
	 * MainFrameV1 control holds old GUI template. Its' main goal is to search in the source
	 * such elements as 'userinfobox', 'menu', 'tabbar' and 'content' and move them to their
	 * appropriate places. These elements is marked by 'xcitecRole' attribute.
	 *
	 * Example:
	 *      <div xcitecRole="userinfobox" data-dojo-type="xcitec.UserInfoBox">
	 *              ......
	 *      </div>
	 *      <div xcitecRole="menu" data-dojo-type="xcitec.MainMenu" label="Suppliers" labelId="mainMenu.supplier">
	 *              ......
	 *      </div>
	 */
	var MainFrameV1 = declare( XLayout, {
	    templateString: template
	});
	return MainFrameV1;
});
},
'dijit/tree/TreeStoreModel':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.indexOf array.some
	"dojo/aspect", // aspect.after
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.hitch
], function(array, aspect, declare, lang){

	// module:
	//		dijit/tree/TreeStoreModel

	return declare("dijit.tree.TreeStoreModel", null, {
		// summary:
		//		Implements dijit/Tree/model connecting to a dojo.data store with a single
		//		root item.  Any methods passed into the constructor will override
		//		the ones defined here.

		// store: dojo/data/api/Read
		//		Underlying store
		store: null,

		// childrenAttrs: String[]
		//		One or more attribute names (attributes in the dojo.data item) that specify that item's children
		childrenAttrs: ["children"],

		// newItemIdAttr: String
		//		Name of attribute in the Object passed to newItem() that specifies the id.
		//
		//		If newItemIdAttr is set then it's used when newItem() is called to see if an
		//		item with the same id already exists, and if so just links to the old item
		//		(so that the old item ends up with two parents).
		//
		//		Setting this to null or "" will make every drop create a new item.
		newItemIdAttr: "id",

		// labelAttr: String
		//		If specified, get label for tree node from this attribute, rather
		//		than by calling store.getLabel()
		labelAttr: "",

		// root: [readonly] dojo/data/Item
		//		Pointer to the root item (read only, not a parameter)
		root: null,

		// query: anything
		//		Specifies datastore query to return the root item for the tree.
		//		Must only return a single item.   Alternately can just pass in pointer
		//		to root item.
		// example:
		//	|	{id:'ROOT'}
		query: null,

		// deferItemLoadingUntilExpand: Boolean
		//		Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes
		//		until they are expanded. This allows for lazying loading where only one
		//		loadItem (and generally one network call, consequently) per expansion
		//		(rather than one for each child).
		//		This relies on partial loading of the children items; each children item of a
		//		fully loaded item should contain the label and info about having children.
		deferItemLoadingUntilExpand: false,

		constructor: function(/* Object */ args){
			// summary:
			//		Passed the arguments listed above (store, etc)
			// tags:
			//		private

			lang.mixin(this, args);

			this.connects = [];

			var store = this.store;
			if(!store.getFeatures()['dojo.data.api.Identity']){
				throw new Error("dijit.tree.TreeStoreModel: store must support dojo.data.Identity");
			}

			// if the store supports Notification, subscribe to the notification events
			if(store.getFeatures()['dojo.data.api.Notification']){
				this.connects = this.connects.concat([
					aspect.after(store, "onNew", lang.hitch(this, "onNewItem"), true),
					aspect.after(store, "onDelete", lang.hitch(this, "onDeleteItem"), true),
					aspect.after(store, "onSet", lang.hitch(this, "onSetItem"), true)
				]);
			}
		},

		destroy: function(){
			var h;
			while(h = this.connects.pop()){ h.remove(); }
			// TODO: should cancel any in-progress processing of getRoot(), getChildren()
		},

		// =======================================================================
		// Methods for traversing hierarchy

		getRoot: function(onItem, onError){
			// summary:
			//		Calls onItem with the root item for the tree, possibly a fabricated item.
			//		Calls onError on error.
			if(this.root){
				onItem(this.root);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						if(items.length != 1){
							throw new Error("dijit.tree.TreeStoreModel: root query returned " + items.length +
								" items, but must return exactly one");
						}
						this.root = items[0];
						onItem(this.root);
					}),
					onError: onError
				});
			}
		},

		mayHaveChildren: function(/*dojo/data/Item*/ item){
			// summary:
			//		Tells if an item has or may have children.  Implementing logic here
			//		avoids showing +/- expando icon for nodes that we know don't have children.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node)
			return array.some(this.childrenAttrs, function(attr){
				return this.store.hasAttribute(item, attr);
			}, this);
		},

		getChildren: function(/*dojo/data/Item*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
			// summary:
			//		Calls onComplete() with array of child items of given parent item, all loaded.

			var store = this.store;
			if(!store.isItemLoaded(parentItem)){
				// The parent is not loaded yet, we must be in deferItemLoadingUntilExpand
				// mode, so we will load it and just return the children (without loading each
				// child item)
				var getChildren = lang.hitch(this, arguments.callee);
				store.loadItem({
					item: parentItem,
					onItem: function(parentItem){
						getChildren(parentItem, onComplete, onError);
					},
					onError: onError
				});
				return;
			}
			// get children of specified item
			var childItems = [];
			for(var i=0; i<this.childrenAttrs.length; i++){
				var vals = store.getValues(parentItem, this.childrenAttrs[i]);
				childItems = childItems.concat(vals);
			}

			// count how many items need to be loaded
			var _waitCount = 0;
			if(!this.deferItemLoadingUntilExpand){
				array.forEach(childItems, function(item){ if(!store.isItemLoaded(item)){ _waitCount++; } });
			}

			if(_waitCount == 0){
				// all items are already loaded (or we aren't loading them).  proceed...
				onComplete(childItems);
			}else{
				// still waiting for some or all of the items to load
				array.forEach(childItems, function(item, idx){
					if(!store.isItemLoaded(item)){
						store.loadItem({
							item: item,
							onItem: function(item){
								childItems[idx] = item;
								if(--_waitCount == 0){
									// all nodes have been loaded, send them to the tree
									onComplete(childItems);
								}
							},
							onError: onError
						});
					}
				});
			}
		},

		// =======================================================================
		// Inspecting items

		isItem: function(/* anything */ something){
			return this.store.isItem(something);	// Boolean
		},

		fetchItemByIdentity: function(/* object */ keywordArgs){
			// summary:
			//		Given the identity of an item, this method returns the item that has
			//		that identity through the onItem callback.  Conforming implementations
			//		should return null if there is no item with the given identity.
			//		Implementations of fetchItemByIdentity() may sometimes return an item
			//		from a local cache and may sometimes fetch an item from a remote server.
			// tags:
			//		extension
			this.store.fetchItemByIdentity(keywordArgs);
		},

		getIdentity: function(/* item */ item){
			return this.store.getIdentity(item);	// Object
		},

		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Get the label for an item
			if(this.labelAttr){
				return this.store.getValue(item,this.labelAttr);	// String
			}else{
				return this.store.getLabel(item);	// String
			}
		},

		// =======================================================================
		// Write interface

		newItem: function(/* dijit/tree/dndSource.__Item */ args, /*dojo/data/api/Item*/ parent, /*int?*/ insertIndex){
			// summary:
			//		Creates a new item.   See `dojo/data/api/Write` for details on args.
			//		Used in drag & drop when item from external source dropped onto tree.
			// description:
			//		Developers will need to override this method if new items get added
			//		to parents with multiple children attributes, in order to define which
			//		children attribute points to the new item.

			var pInfo = {parent: parent, attribute: this.childrenAttrs[0]}, LnewItem;

			if(this.newItemIdAttr && args[this.newItemIdAttr]){
				// Maybe there's already a corresponding item in the store; if so, reuse it.
				this.fetchItemByIdentity({identity: args[this.newItemIdAttr], scope: this, onItem: function(item){
					if(item){
						// There's already a matching item in store, use it
						this.pasteItem(item, null, parent, true, insertIndex);
					}else{
						// Create new item in the tree, based on the drag source.
						LnewItem=this.store.newItem(args, pInfo);
						if(LnewItem && (insertIndex!=undefined)){
							// Move new item to desired position
							this.pasteItem(LnewItem, parent, parent, false, insertIndex);
						}
					}
				}});
			}else{
				// [as far as we know] there is no id so we must assume this is a new item
				LnewItem=this.store.newItem(args, pInfo);
				if(LnewItem && (insertIndex!=undefined)){
					// Move new item to desired position
					this.pasteItem(LnewItem, parent, parent, false, insertIndex);
				}
			}
		},

		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
			// summary:
			//		Move or copy an item from one parent item to another.
			//		Used in drag & drop
			var store = this.store,
				parentAttr = this.childrenAttrs[0];	// name of "children" attr in parent item

			// remove child from source item, and record the attribute that child occurred in
			if(oldParentItem){
				array.forEach(this.childrenAttrs, function(attr){
					if(store.containsValue(oldParentItem, attr, childItem)){
						if(!bCopy){
							var values = array.filter(store.getValues(oldParentItem, attr), function(x){
								return x != childItem;
							});
							store.setValues(oldParentItem, attr, values);
						}
						parentAttr = attr;
					}
				});
			}

			// modify target item's children attribute to include this item
			if(newParentItem){
				if(typeof insertIndex == "number"){
					// call slice() to avoid modifying the original array, confusing the data store
					var childItems = store.getValues(newParentItem, parentAttr).slice();
					childItems.splice(insertIndex, 0, childItem);
					store.setValues(newParentItem, parentAttr, childItems);
				}else{
					store.setValues(newParentItem, parentAttr,
						store.getValues(newParentItem, parentAttr).concat(childItem));
				}
			}
		},

		// =======================================================================
		// Callbacks

		onChange: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},

		onChildrenChange: function(/*===== parent, newChildrenList =====*/){
			// summary:
			//		Callback to do notifications about new, updated, or deleted items.
			// parent: dojo/data/Item
			// newChildrenList: dojo/data/Item[]
			// tags:
			//		callback
		},

		onDelete: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback when an item has been deleted.
			// description:
			//		Note that there will also be an onChildrenChange() callback for the parent
			//		of this item.
			// tags:
			//		callback
		},

		// =======================================================================
		// Events from data store

		onNewItem: function(/* dojo/data/Item */ item, /* Object */ parentInfo){
			// summary:
			//		Handler for when new items appear in the store, either from a drop operation
			//		or some other way.   Updates the tree view (if necessary).
			// description:
			//		If the new item is a child of an existing item,
			//		calls onChildrenChange() with the new list of children
			//		for that existing item.
			//
			// tags:
			//		extension

			// We only care about the new item if it has a parent that corresponds to a TreeNode
			// we are currently displaying
			if(!parentInfo){
				return;
			}

			// Call onChildrenChange() on parent (ie, existing) item with new list of children
			// In the common case, the new list of children is simply parentInfo.newValue or
			// [ parentInfo.newValue ], although if items in the store has multiple
			// child attributes (see `childrenAttr`), then it's a superset of parentInfo.newValue,
			// so call getChildren() to be sure to get right answer.
			this.getChildren(parentInfo.item, lang.hitch(this, function(children){
				this.onChildrenChange(parentInfo.item, children);
			}));
		},

		onDeleteItem: function(/*Object*/ item){
			// summary:
			//		Handler for delete notifications from underlying store
			this.onDelete(item);
		},

		onSetItem: function(item, attribute /*===== , oldValue, newValue =====*/){
			// summary:
			//		Updates the tree view according to changes in the data store.
			// description:
			//		Handles updates to an item's children by calling onChildrenChange(), and
			//		other updates to an item by calling onChange().
			//
			//		See `onNewItem` for more details on handling updates to an item's children.
			// item: Item
			// attribute: attribute-name-string
			// oldValue: Object|Array
			// newValue: Object|Array
			// tags:
			//		extension

			if(array.indexOf(this.childrenAttrs, attribute) != -1){
				// item's children list changed
				this.getChildren(item, lang.hitch(this, function(children){
					// See comments in onNewItem() about calling getChildren()
					this.onChildrenChange(item, children);
				}));
			}else{
				// item's label/icon/etc. changed.
				this.onChange(item);
			}
		}
	});
});

},
'xcitec/page/MainFrameV2':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/query",
  "dojo/keys",
	"dojo/text!xcitec/page/templates/MainFrameV2.html",
	"dijit/_Container",
	"dijit/registry",
	"xcitec/layout/XLayout"
], function (dojo, declare, domAttr, query, keys, template, _Container, registry, XLayout) {

	/*
	 * MainFrameV2 control holds new GUI template. Its' main goal is to search in the source
	 * such elements as 'userinfobox', 'menu', 'tabbar' and 'content' and move them to their
	 * appropriate places. These elements is marked by 'xcitecRole' attribute.
	 *
	 * Example:
	 *      <div xcitecRole="userinfobox" data-dojo-type="xcitec.UserInfoBox">
	 *              ......
	 *      </div>
	 *      <div xcitecRole="menu" data-dojo-type="xcitec.MainMenuV2" label="Suppliers" labelId="mainMenu.supplier">
	 *              ......
	 *      </div>
	 */
	var MainFrameV2 = declare( [XLayout, _Container], {
	    templateString: template,
	
		buildRendering: function() {
			this.inherited(arguments);
	
			var footer = query('[xcitecAttachPoint$=footer]')[0];
			if(footer){
	                           domAttr.set(footer, 'role', 'contentinfo');
	                }
		},
	
		_onKeyPress: function(/*Event*/ e){
			if(e.keyCode == keys.UP_ARROW && e.ctrlKey){
				registry.byNode(query('[tabindex="0"]', registry.byNode(query('.TabBar')[0]).containerNode)[0]).setFocus();
			}
		}
	});
	
	return MainFrameV2;
});
},
'dojo/dnd/TimedMoveable':function(){
define(["../_base/declare", "./Moveable" /*=====, "./Mover" =====*/], function(declare, Moveable /*=====, Mover =====*/){
	// module:
	//		dojo/dnd/TimedMoveable

	/*=====
	var __TimedMoveableArgs = declare([Moveable.__MoveableArgs], {
		// timeout: Number
		//		delay move by this number of ms,
		//		accumulating position changes during the timeout
		timeout: 0
	});
	=====*/

	// precalculate long expressions
	var oldOnMove = Moveable.prototype.onMove;

	return declare("dojo.dnd.TimedMoveable", Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.

		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps

		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: __TimedMoveableArgs
			//		object with additional parameters.

			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},

		onMoveStop: function(/*Mover*/ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer);
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop);
			}
			Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/*Mover*/ mover, /*Object*/ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
});

},
'xcitec/SubsegmentContainer':function(){
define([
	"dojo",
	"dojo/dom-construct",
	"dojo/on",
	"dojo/dom",
	"dojo/query",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/has",
	"dojo/dom-geometry",
	"dojo/_base/array",
  "dojo/dnd/Avatar",
	"dojo/dnd/Manager",
    "dojo/Deferred",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "./SubsegmentAvatar",
  "./Source"
], function (dojo, domConstruct, on, dom, query, domAttr, domClass, lang, declare, has, domGeometry, array, Avatar, Manager, Deferred, _Container, _Templated, _Widget, registry, SubsegmentAvatar, Source) {

	var SubsegmentContainer = declare( [_Widget, _Templated, _Container], {
	    templateString: '<div class="subsegmentContainer" data-dojo-attach-point="containerNode"></div>',
	
	    // array of all subsegments in container
	    subsegmentWidgets: null,
	    // json hierarchy of subsegments
	    hierarchy: null,
	    // used for keep zero level of widgets
	    level: 0,
	    // last widget in hierarchy of specified object
	    lastChild: null,
	    // all children (not only direct) of the widget (Array)
	    allChildren: null,
	
	    // movingNode and targetNode
	    movingNode: null,
	    targetNode: null,
	
	    // handler of connection myMove function to onMouseMove
	    mouseMoveHandle: null,
	
	    // is subsegments in designMode
	    designMode: false,
	
	    onDeleteAttr: null,
	    onSettingsAttr: null,
	
	    postMixInProperties: function() {
	        // set designMode attribute to allow subsegments show right buttons (delete and settings)
	        if (this.designMode) {
	            var subs = query(">", this.srcNodeRef);
	            array.forEach(subs, function(sub) {
	                domAttr.set(sub, 'designMode', 'true');
	            });
	        }
	        // collect parents and ids of subsegments
	        this.parents = query(" > div", this.srcNodeRef).map(function(node) {
	            return domAttr.get(node, "parent");
	        });
	        this.ids = query(" > div", this.srcNodeRef).map(function(node) {
	            return domAttr.get(node, "id");
	        });
	
	        this.onDeleteAttr = 'onDelete' in this.params;
	        this.onSettingsAttr = 'onSettings' in this.params;
	        this.clicked = dom.byId(this.id + ".clicked");
	        this.order = dom.byId(this.id + ".order");
		    this.subsegmentWidgets = [];
		    this.allChildren = [];
	        this.inherited(arguments);
	    },
	
	    startup: function() {
	        this.inherited(arguments);
	        var _this = this;
	
	        // set proper level for each subsegment according to it place in hierarchy
	        var subsegments = this.getChildren();
	        array.forEach(subsegments, function(subsegment) {
	            if (subsegment.getParent() != "") {
	                var parentId = subsegment.getParent();
	                var parentWidget = registry.byId(parentId);
	                subsegment.setNestingLevel(parentWidget.getNestingLevel() + 1);
	            }
	            _this.subsegmentWidgets.push(subsegment);
	        });

			// provide children array for every subsegment widget
			array.forEach(this.subsegmentWidgets, function(widget) {
				if (widget.getParent() != "") {
					registry.byId(widget.getParent()).addChild(widget);
				}
				widget.setEnclosingWidget(_this);
			});

	        if (this.designMode) {

	            this.updateHierarchy();
	            this.updateHiddenInput();
	            this.source = new Source(this.domNode, {accept: this.id});
	            Manager.manager().makeAvatar = function() {
	                return new SubsegmentAvatar(this);
	            };
	            on(this.source, "Drop", lang.hitch(this, "myDrop"));
	            on(this.source, "DndStart", lang.hitch(this, "myStart"));
	            on(this.source, "DndDrop", lang.hitch(this, "myDndDrop"));
	            on(this.source, "DndCancel", lang.hitch(this, "myDndDrop"));
	        }
	    },
	
		destroy: function(/*Boolean*/ preserveDom){
			if(this.source && this.source.destroy){
				this.source.destroy();
			}
			this.inherited(arguments);
		},
	
		updateHiddenInput: function() {
	        var _this = this;
	        if (this.order) {
		        this.order.value = "";
	            array.forEach(this.subsegmentWidgets, function(widget) {
	                if (widget.getParent() == "") {
	                    _this.order.value = _this.order.value + widget.id + ",";
	                } else {
	                    _this.order.value = _this.order.value + widget.id + "#" + widget.getParent() + ",";
	                }
	            });
	            this.order.value = this.order.value.substring(0, this.order.value.length - 1);
	        }
	    },
	
	    myStart: function() {
	        this.movingNode = this.source.getSelectedNodes()[0];
	        this.mouseMoveHandle = on(this.source, "MouseMove", lang.hitch(this, "myMove"));
	    },
	
	    myDndDrop: function() {
	        if (this.mouseMoveHandle) {
	            this.mouseMoveHandle.remove();
	        }
	        array.forEach(this.subsegmentWidgets, function(sub) {
	            if (domClass.contains(sub.domNode, "dojoDndItemXcitecOver")) {
	                domClass.remove(sub.domNode, "dojoDndItemXcitecOver");
	            }
	        });
	    },
	
	    myMove: function() {
	        this.targetNode = this.source.current;
			var manager = Manager.manager();
	        if (manager.canDropFlag) {
	            if (this.targetNode) {
	                if (this.movingNode) {
	                    if (this.isDestNodeChild(registry.byNode(this.movingNode), registry.byNode(this.targetNode))) {
                          manager.canDropFlag = false;
	                    } else {
                          manager.canDropFlag = true;
	                    }
	                }
	            } else {
                  manager.canDropFlag = false;
	            }
              manager.avatar.update();
	        }
	    },
	
	    // function for additional processing of widget drop
	    myDrop: function() {
	        // check if movingWidget and destinationWidget are available
	        if (this.source.getSelectedNodes()[0]) {
	            var movingWidget = registry.byNode(this.source.getSelectedNodes()[0]);
	        }
	        if (this.source.targetAnchor) {
	            var destinationWidget = registry.byNode(this.source.targetAnchor);
	        }
	        if (!destinationWidget) {
	            return;
	        }
	        // main processing logic
	        if (movingWidget) {
	            // check if movingWidget not really the parent of destinationWidget, if true - no moving
	            if (this.isDestNodeChild(movingWidget, destinationWidget)) {
	                var prevWidget = this.getPreviousWidget(movingWidget);
	                if (prevWidget) {
	                    domConstruct.place(movingWidget.domNode, prevWidget.domNode, "after");
	                } else {
	                    domConstruct.place(movingWidget.domNode, this.domNode, "first");
	                }
	                return;
	            }
	            // do I need this?
	            // additional level control
	            this.level = 0;
	            this.getActualLevel(destinationWidget);
	            this.lastChild = null;
	
	            // is drop occured over the widget or no
	            if (domClass.contains(destinationWidget.domNode, 'dojoDndItemXcitecOver')) {
	                this.processOverEvent(destinationWidget, movingWidget);
	            } else {
	                // set level for moveNode
	                movingWidget.setNestingLevel(destinationWidget.getNestingLevel());
	                // if moveNode has parent - remove this moveNode from parent's children array
	                if (movingWidget.getParent() != "") {
	                    registry.byId(movingWidget.getParent()).removeChild(movingWidget);
	                }
	                // if destNode has parent - set this parent for moveNode, else - assign 0 level and no parent
	                if (destinationWidget.getParent() != "") {
	                    movingWidget.setParent(destinationWidget.getParent());
	                } else {
	                    movingWidget.setParent("");
	                    movingWidget.setNestingLevel(0);
	                }
	
	                // process drop then it was after destWidget
	                if (domClass.contains(destinationWidget.domNode, "dojoDndItemAfter")) {
	                    this.processAfterEvent(destinationWidget, movingWidget);
	                }
	
	                // process drop then it was before destWidget
	                if (domClass.contains(destinationWidget.domNode, "dojoDndItemBefore")) {
	                    if (destinationWidget.getParent() != "") {
	                        if (!registry.byId(destinationWidget.getParent()).hasChild(movingWidget)) {
	                            registry.byId(destinationWidget.getParent()).addChild(movingWidget);
	                        }
	                    }
	                }
	            }
	
	            // additional processing of child widgets of moving widget
	            if (this.hasChildren(movingWidget)) {
	                this.moveNodeWithChildren(movingWidget);
	            }
	            // and finally update json hierarchy
	            this.updateHierarchy();
	            this.updateHiddenInput();
	        }
	    },
	
	    processOverEvent: function(/* Widget */ destinationWidget, /* Widget */ movingWidget) {
	        // if destNode has children then place moveNode after the last one
	        var destNode = null;
	        if (movingWidget.getParent() == destinationWidget.id) {
	            domConstruct.place(movingWidget.domNode, destinationWidget.domNode, 'after');
	        } else {
	            if (this.hasChildren(destinationWidget)) {
	                destNode = this.getLastChild(destinationWidget).domNode;
	                if (destNode == movingWidget.domNode) {
	                    destNode = this.getPreviousWidget(movingWidget).domNode;
	                }
	                domConstruct.place(movingWidget.domNode, destNode, 'after');
	            } else {
	                domConstruct.place(movingWidget.domNode, destinationWidget.domNode, 'after');
	            }
	        }
	        // set level for moveNode one bigger than his parent
	        movingWidget.setNestingLevel(destinationWidget.getNestingLevel() + 1);
	        // remove child from children array of old parent (if it exists)
	        if (movingWidget.getParent() != "") {
	            registry.byId(movingWidget.getParent()).removeChild(movingWidget);
	        }
	        // set new parent for mvoeNode and add it to children array of parent
	        movingWidget.setParent(destinationWidget.id);
	        destinationWidget.addChild(movingWidget);
	    },
	
	    processAfterEvent: function(/* Widget */ destinationWidget, /* Widget */ movingWidget) {
	        // add child to destNode paren's children array if parent exists
	        var destNode = null;
	        if (destinationWidget.getParent() != "") {
	            if (!registry.byId(destinationWidget.getParent()).hasChild(movingWidget)) {
	                registry.byId(destinationWidget.getParent()).addChild(movingWidget);
	            }
	        }
	        if (this.hasChildren(destinationWidget)) {
	            destNode = this.getLastChild(destinationWidget).domNode;
	            if (destNode == movingWidget.domNode) {
	                destNode = this.getPreviousWidget(movingWidget).domNode;
	            }
	            domConstruct.place(movingWidget.domNode, destNode, 'after');
	        }
	    },
	
	    getPreviousWidget: function(/* Widget */ widget) {
	        return this.subsegmentWidgets[array.indexOf(this.subsegmentWidgets, widget) - 1];
	    },
	
	    // return true if destNode is really the child of moveNode
	    isDestNodeChild: function(/* DomNode */ possibleParent, /* DomNode */ possibleChild) {
	        this.allChildren = [];
	        var isChild = false;
	        this.collectAllNodeChildren(possibleParent);
	        if (array.indexOf(this.allChildren, possibleChild.id) >= 0) {
	            isChild = true
	        }
	        return isChild;
	    },
	
	    // function for collecting all children ids of passed widget
	    collectAllNodeChildren: function(/* Widget */ widget) {
	        var _this = this;
	        array.forEach(widget.getNestedSegments(), function(child) {
	            _this.allChildren.push(child.id);
	            if (_this.hasChildren(child)) {
	                _this.collectAllNodeChildren(child);
	            }
	        })
	    },
	
	    getActualLevel: function(/* Widget */ widget) {
	        var _this = this;
	        if (widget.getParent().length != 0) {
	            _this.level = _this.level + 1;
	            _this.getActualLevel(registry.byId(widget.getParent()));
	        }
	    },
	
	    // returns true if passed node has inner nodes, false otherwise
	    hasChildren: function(/* Widget */ widget) {
	        var has = false;
	        if (widget.getNestedSegments().length > 0 ) {
	            has = true
	        }
	        return has;
	    },
	
	    // returns last direct child of passed node
	    getLastChild: function(/* Widget */ widget) {
	        var _this = this;
	        array.forEach(this.hierarchy.items, function(item) {
	            if (item.id == widget.id) {
	                if (item.children.length > 0) {
	                    _this.lastChild = registry.byId(item.children[item.children.length - 1]._reference);
	                    if (_this.hasChildren(_this.lastChild)) {
	                        _this.getLastChild(_this.lastChild)
	                    }
	                }
	            }
	        });
	        return this.lastChild;
	    },
	
	    // moves all the children of passed node with this node in new place
	    moveNodeWithChildren: function(/* Widget */ widget) {
	        var innerNodes = widget.getNestedSegments();
	        var _this = this;
	        innerNodes = innerNodes.reverse();
	        array.forEach(innerNodes, function(inner) {
	            domConstruct.place(inner.domNode, widget.domNode, "after");
	            inner.setNestingLevel(widget.getNestingLevel() + 1);
	            if(_this.hasChildren(inner)){
	                _this.moveNodeWithChildren(inner);
	            }
	        });
	    },
	
	    // updates json object with new hierarchy of subsegments
	    updateHierarchy: function() {
	        this.subsegmentWidgets = this.getChildren();
	        var parents, ids;
	        if (this.parents && this.ids) {
	            parents = this.parents;
	            ids = this.ids;
	            this.parents = this.ids = null;
	        } else {
	            parents = array.map(this.subsegmentWidgets, function(node) {
	                return node.getParent();
	            });
	            ids = array.map(this.subsegmentWidgets, function(node) {
	                return node.id;
	            });
	        }
	        var _this = this;
	        this.hierarchy = { "identifier": "id",
	            "items": []};
	
	        var ind = 0;
	        array.forEach(parents, function(parent, index) {
	            var itemType = 'parent';
	            if (parents[index]) {
	                itemType = 'child';
	            }
	            var itemChildren = [];
	            array.forEach(parents, function(parent, index2) {
	                if (parent == ids[index]) {
	                    itemChildren = itemChildren.concat({"_reference": ids[index2]});
	                }
	            });
	
	            _this.hierarchy.items[index] = { id: ids[index], type: itemType,
	                children: itemChildren, itemIndex: ind };
	            ind++;
	        });
	        this.updateHiddenInput();
	    },
	
	    isDeleteSet: function() {
	        return this.onDeleteAttr;
	    },
	
	    isSettingsSet: function() {
	        return this.onSettingsAttr;
	    },
	
	    onDelete: function() {
	
	    },
	
	    onSettings: function() {
	
	    },
	
	    getClicked: function() {
	        return this.clicked;
	    },

		resize: function(){
			array.forEach(this.subsegmentWidgets, function(widget) {
				if(widget.state=="Open"){
					widget._checkPosition();
				}
			});
		},

        parseChildren: function(targetWidget) {
            var deferred = new Deferred();
            if(array.some(this.subsegmentWidgets, function(widget){
                widget._loadExternalContent();
                if(widget == targetWidget){
                    return true;
                }
            })){
                deferred.resolve();
            } else if(!targetWidget){
                deferred.resolve();
            }

            return deferred.promise;
        }
	});
	return SubsegmentContainer;
});

},
'xcitec/TabBar':function(){
define([
	"dojo/dom-class",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-geometry",
	"dojo/fx",
	"dojo/_base/declare",
	"dojo/query",
	"dojo/text!./templates/TabBar.html",
	"dijit/Menu",
	"dijit/MenuSeparator",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "dijit/popup",
  "./MenuItem",
  "dijit/_FocusMixin"
], function (domClass, array, lang, window, geometry, fx, declare, query, TabBarTemplate, Menu, MenuSeparator, _Container, _Templated, _Widget, popup, MenuItem, _FocusMixin) {

	/* TabBar control
	  Open topics:
	  - Done: resctiction scroll to Right & Left 
	  - Done: scroll to Right & Left when partly shown 
	  - Dine: scrolling behaviour on resize
	  - Done: recalculate tabbarVisibleWidth when scrolling enabled
	  - Done: disabling scroll buttons
	  
	  Tab Bar:
	  Attruibutes:
	   labelCloseAll   - (String) text to display CloseAll item in DropDown list 
	   labelIdCloseAll - (String) label Id for ML support for CloseAll item in DropDown list (not implemented) 
	  Events: 
	   onCloseAll - (function) fires when user choose CloseAll item in DropDown list.
	   
	  Tab (Tab Bar Item):
	  Attruibutes:
	   label - (String) Text to display tab title (can be provided as body of html tag).
	   labelId - (String) label Id for ML support for tab title
	   selected - (boolen) define that item ia active
	  Events: 
	   onClick - (function) fires when user choose Tab (click on tab).
	   onClose - (function) fires when user press close button of Tab.
	*/
	
	var TabBar = declare( [_Widget, _Templated, _Container, _FocusMixin], {
	    templateString: TabBarTemplate,
	
	    baseClass: "TabBar",
	    
	    // labelCloseAll: String
	    //  Text to display CloseAll item in DropDown list.
	  	labelCloseAll: '',
	
	    // labelId: String
	    //  label Id for ML support for CloseAll item in DropDown list
	  	labelIdCloseAll: '',
	    
	    // minTabWidth: number
	    //  minimal width of Tab.
	    minTabWidth: 75, 
	    //minTabWidth: 150, 
	
	    // maxTabWidth: number
	    //  maximal width of Tab.
	    maxTabWidth: 185,
	    
	    tabCount: 0,
	    currentTabWidth: 0,
	    scrollLeftPos: 0, 
	
	    //--- BEGIN --- functions of The _Widget Lifecycle  (in order of call) --
	    postCreate: function(){
	        this.inherited(arguments);
	
	        // extra listeners
	        //this.connect(this.domNode, !dojo.isMozilla ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
	        //this.connect(dojo.global, 'onresize', '_onResize');
	    },
	
	    startup: function(){
	        if(this._started){ return; }
			var tabs = this.getChildren();
			var _this = this;
			array.forEach(tabs, function(tab, index){
				tab.pos = index + 1;
				tab.tabBar = _this;
			});
	        this.tabCount = tabs.length;
	        // extra listeners
	        this._onResize();
	        this.connect(window.global, 'onresize', '_onResize');
	        this.inherited(arguments);
	    },
			
	  	destroyDescendants: function(){
	  		if(this.dropDown){
	  			this.dropDown.destroyRecursive();
	  			delete this.dropDown;
	  		}
	  		this.inherited(arguments);
	  	},
	
	    //--- END --- functions of The _Widget Lifecycle  (in order of call) --
	
	    _onResize: function(){
	      var cb = geometry.getContentBox(this.domNode);
	      if (this._lastTabbarWidth!= cb.w) {
	        this._lastTabbarWidth = cb.w;
	        this.layoutTabs();
	      }
	    },
			
	    layoutTabs: function() {
	        if (this.tabCount==0) {
	          // if no child hide downArrowNode and do nothing; 
	          this.rightWrapNode.style.display = "none";
	          return; 
	        }
	        
	        // remove scroll state to recalculate available TabBar Viewport
	        if (domClass.contains(this.domNode, 'TabBar-Scroll')) {
	  		    domClass.remove(this.domNode, 'TabBar-Scroll-Right');
	  		    domClass.remove(this.domNode, 'TabBar-Scroll-Left');
	  		    domClass.remove(this.domNode, 'TabBar-Scroll');
	        }
	
	        var cb = geometry.getContentBox(this.domNode);
	        var mbri = geometry.getMarginBox(this.rightWrapNode)
	        var tbWidth = cb.w - mbri.w;
	        var childWidth = this.minTabWidth;
	        if (tbWidth < (childWidth * this.tabCount )) {
	          // minimal SUM(size of tabs)>then available Viewport
	          // have to enable scroll bars and recalc available Viewport
	  		    domClass.add(this.domNode, 'TabBar-Scroll');
	          mbri = geometry.getMarginBox(this.rightWrapNode)
	          var mbli = geometry.getMarginBox(this.scrollArrowLeftNode);
	          tbWidth = cb.w - mbri.w - mbli.w;
	        } else {
	          // tabs fit to available Viewport
	          // have to calulate tabs width
	          childWidth = Math.max( Math.min( tbWidth/this.tabCount, this.maxTabWidth), this.minTabWidth);
	          //childWidth = tbWidth / this.tabCount;
	          //if (childWidth>this.maxTabWidth) childWidth=this.maxTabWidth;  
	          //if (childWidth<this.minTabWidth) childWidth=this.minTabWidth;
	        }
	        this.tabbarVisibleWidth = tbWidth;
	        
	        //set calculated width to Tabs
	        if (this.currentTabWidth != childWidth) {
	          this.currentTabWidth = childWidth;
	          array.forEach(this.getChildren(), function(child) {
              geometry.setContentSize( child.domNode, { w: childWidth } );
	          }, this);
	        }
	
	        //invalidate tab scroll
	        this.scrollTabs(0);
	    },
	    
	    scrollToPos: function( left, anim ){
	      if (anim) {
	        fx.slideTo({ node: this.containerNode, left:left, unit:"px" }).play(); 
	      } else {
	        this.containerNode.style.left = left+"px";
	      }
	      this.scrollLeftPos = left;
	    },
	
	    scrollTabs: function( shift ){
	      var viewportTabs   = this.tabCount * this.currentTabWidth;
	      var viewportTabBar = this.tabbarVisibleWidth;
	      if (shift==0) {
	        // usualy on resize to invalidate scroll position.
	        var newPos = -(viewportTabs - viewportTabBar);
	        if (newPos>=0) {
	          // there is more space then necessary for showing Tabs
	          // have to reset scroll position;
	          this.scrollToPos(0);
	        } else if (newPos>this.scrollLeftPos) {
	          // there is free space at ther right -> have to shift scroll
	          this.scrollToPos( newPos );
	        } 
	      } else if (shift<0) {
	        // pressed scroll to Left
	        var indexLeftUnvisibleTab = -Math.floor( this.scrollLeftPos / this.currentTabWidth ); //minus because this.scrollLeftPos<0  
	        if (indexLeftUnvisibleTab>0) {
	          var newPos = - (indexLeftUnvisibleTab-1) * this.currentTabWidth;
	          this.scrollToPos( newPos, true );
	        } 
	      } else if (shift>0) {
	        // pressed scroll to Right
	        //dojo.addClass(this.scrollArrowRight, 'TabBarButtonRightActive');
	        var xVisibleWidth = this.tabbarVisibleWidth - this.scrollLeftPos; //minus because this.scrollLeftPos<0
	        var indexRightVisibleTab = Math.floor( xVisibleWidth / this.currentTabWidth );
	        if (indexRightVisibleTab<this.tabCount) {
	          var newPos = (indexRightVisibleTab+1) * this.currentTabWidth - this.tabbarVisibleWidth;
	          this.scrollToPos( -newPos, true );
	        } 
	      }
	      // update state of ScrollButtons if necessary
	      if (domClass.contains(this.domNode, 'TabBar-Scroll')) {
	        if (this.scrollLeftPos<0) 
	             domClass.add(this.domNode, 'TabBar-Scroll-Left'); 
	        else domClass.remove(this.domNode, 'TabBar-Scroll-Left');
	        if (viewportTabBar<(this.scrollLeftPos + viewportTabs)) 
	             domClass.add(this.domNode, 'TabBar-Scroll-Right'); 
	        else domClass.remove(this.domNode, 'TabBar-Scroll-Right');
	      }
	    },
	    
	    _onScrollArrowLeftHover:   function() { domClass.add(this.scrollArrowLeftNode, 'TabBarButtonLeftHover');    },
	    _onScrollArrowLeftUnhover: function() { domClass.remove(this.scrollArrowLeftNode, 'TabBarButtonLeftHover'); },
	    _onScrollArrowLeftClick:   function() { this.scrollTabs(-1); },
	
	    _onScrollArrowRightHover:   function() { domClass.add(this.scrollArrowRight, 'TabBarButtonRightHover');    },
	    _onScrollArrowRightUnhover: function() { domClass.remove(this.scrollArrowRight, 'TabBarButtonRightHover'); },
	    _onScrollArrowRightClick:   function() { this.scrollTabs(1); },
	
	    _onDownArrowHover:   function() { domClass.add(this.downArrowNode, 'TabBarListButtonHover');    },
	    _onDownArrowUnhover: function() { domClass.remove(this.downArrowNode, 'TabBarListButtonHover'); },
	    _onDownArrowClick:   function() {
	      this._createPopupList();
	  		if(!this._opened){
	        this._openDropDown();
	  		}else{
	  			this._closeDropDown();
	  		}
	  	},
	
	    _createPopupList: function() {
	      if(!this.dropDown){
					var popupId = this.id + "_popup";
					this.dropDown = new Menu({
					  id:popupId
					});
					
	        var pw = this.dropDown;
	        array.forEach(this.getChildren(), function(child) {
	           pw.addChild(new MenuItem({
	            label: child.label,
	            iconClass: child.iconClass, 
	            onClick: lang.hitch(child, "_onClick")
	           }));
	        }, this);
	        
	        pw.addChild(new MenuSeparator());
	        pw.addChild(new MenuItem({
	           label: this.labelCloseAll,
	           onClick: lang.hitch(this, "_onCloseAll")
	        }));
	        popup.moveOffScreen(this.dropDown);
				}
	    },
	  	
	  	_onBlur: function(){
	  		// summary: called magically when focus has shifted away from this widget and it's dropdown
	  		this._closeDropDown();
	  		// don't focus on button.  the user has explicitly focused on something else.
	  		this.inherited(arguments);
	  	},
	
	  	_openDropDown: function(){
	  		var dropDown = this.dropDown;
	  		var oldWidth=dropDown.domNode.style.width;
	  		var self = this;
	
	  		popup.open({
	  			parent: self,
	  			popup: dropDown,
	  			//around: this.popupStateNode,
	  			around: self.domNode,
	  			orient:
	  				// TODO: add user-defined positioning option, like in Tooltip.js
	  				//this.isLeftToRight() ? {'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'}: {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
	  				//this.isLeftToRight() ? {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}:{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'},
            this.isLeftToRight() ? ['below-alt', 'above-alt', 'below', 'above'] : ['below', 'above', 'below-alt', 'above-alt'],
	  			onExecute: function(){
	  				self._closeDropDown(true);
	  			},
	  			onCancel: function(){
	  				self._closeDropDown(true);
	  			},
	  			onClose: function(){
	  				dropDown.domNode.style.width = oldWidth;
	  				self.popupStateNode.removeAttribute("popupActive");
	  				self._opened = false;
	  			}
	  		});
	  		this.popupStateNode.setAttribute("popupActive", "true");
	  		this._opened=true;
	  		if(dropDown.focus){
	  			dropDown.focus();
	  		}
	  		domClass.add(this.downArrowNode, 'TabBarListButtonActive');
	  		// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
	  	},
	  	
	  	_closeDropDown: function(/*Boolean*/ focus){
	  		if(this._opened){
	  			popup.close(this.dropDown);
	  			//if(focus){ this.focus(); }
	  			this._opened = false;
	  			domClass.remove(this.downArrowNode, 'TabBarListButtonActive');
	  		}
	  	},
	  	
	  	_onCloseAll: function(evt) {
	  	  this.onCloseAll();
	  		evt.preventDefault(), evt.stopPropagation();
	  	},
	    
	  	onCloseAll: function(){
	  		// summary: User defined function to handle closes
	      console.log("TabBar: onCloseAll"); 
	  	},
	    
	    log: function( str ) {
	      //if (!this._logCounter) this._logCounter=0;
	      //this._logCounter++;
	      //console.log( "{"+ this._logCounter +"} "+ str) 
	      // dummy function
	    }
	});
	return TabBar;
});

},
'xcitec/form/SliderLabels':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/number",
	"dojo/query",
	"dijit/form/HorizontalRuleLabels"
], function (dojo, array, declare, domAttr, number, query, HorizontalRuleLabels) {

	var SliderLabels = declare( HorizontalRuleLabels, {
	    labelText: null, // Array
	    selected: 0,
	    sendValues: null, // Array
	    container: "bottomDecoration",
	    style: "height:1em;font-size:100%;",
	
	    getLabels: function(){
			// summary:
			//		Overridable function to return array of labels to use for this slider.
			//		Can specify a getLabels() method instead of a labels[] array, or min/max attributes.
			// tags:
			//		protected extension
	
			// if the labels array was not specified directly, then see if <li> children were
			var labels = this.labels;
	        var labelText = this.labelText;
			if(!labels.length){
				// for markup creation, labels are specified as child elements
				labels = query("> div", this.srcNodeRef).map(function(node){
					return String(node.innerHTML);
				});
	            /*var lis = dojo.query("> li", this.srcNodeRef);
	            var index = 0;
	            dojo.forEach(lis, function(li) {
	
	                labelText[labels[index]] = dojo.attr(li, "label");
	                index++;
	            });*/
	            labelText = query("> div", this.srcNodeRef).map(function(node){
	                return domAttr.get(node, "label");
					//String(node.innerHTML);
				});
	            var sel = query("> div", this.srcNodeRef);
	            var index = 0;
	            var _this = this;
	            array.forEach(sel, function(node) {
	                if (domAttr.get(node, "selected") == "true") {
	                    _this.selected = index;
	                }
	                index++;
	            });
	
	            this.sendValues = query("> div", this.srcNodeRef).map(function(node){
	                return domAttr.get(node, "value");
				});
	
	            this.labelText = labelText;
			}
			this.srcNodeRef.innerHTML = '';
			// if the labels were not specified directly and not as <li> children, then calculate numeric labels
			if(!labels.length && this.count > 1){
				var start = this.minimum;
				var inc = (this.maximum - start) / (this.count-1);
				for (var i=0; i < this.count; i++){
					labels.push((i<this.numericMargin||i>=(this.count-this.numericMargin))? '' : number.format(start, this.constraints));
					start += inc;
				}
			}
			return labels;
		},
	
	    getLabelsText: function(){
			// summary:
			//		Overridable function to return array of labels to use for this slider.
			//		Can specify a getLabels() method instead of a labels[] array, or min/max attributes.
			// tags:
			//		protected extension
	
			// if the labels array was not specified directly, then see if <li> children were
			//var labels = this.labels;
	        var labelText = this.labelText;
	
			if(!labelText.length){
				// for markup creation, labels are specified as child elements
				/*labelText = dojo.query("> li", this.srcNodeRef).map(function(node){
					return dojo.attr(node, "label");
				});*/
	            var labelTexts = query("> div", this.srcNodeRef);
			}
	
			return labelText;
		},
	
	    postMixInProperties: function(){
		    this.labelText = [];
		    this.sendValues = [];
			this.inherited(arguments);
			this.labels = this.getLabels();
			this.count = this.labels.length;
		}
	});
	return SliderLabels;
});
},
'dijit/a11yclick':function(){
define([
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/mouse",
	"dojo/on",
	"dojo/touch" // touch support for click is now there
], function(keys, mouse, on, touch){

	// module:
	//		dijit/a11yclick

	/*=====
	return {
		// summary:
		//		Custom press, release, and click synthetic events
		//		which trigger on a left mouse click, touch, or space/enter keyup.

		click: function(node, listener){
			// summary:
			//		Logical click operation for mouse, touch, or keyboard (space/enter key)
		},
		press: function(node, listener){
			// summary:
			//		Mousedown (left button), touchstart, or keydown (space or enter) corresponding to logical click operation.
		},
		release: function(node, listener){
			// summary:
			//		Mouseup (left button), touchend, or keyup (space or enter) corresponding to logical click operation.
		},
		move: function(node, listener){
			// summary:
			//		Mouse cursor or a finger is dragged over the given node.
		}
	};
	=====*/

	function clickKey(/*Event*/ e){
		// Test if this keyboard event should be tracked as the start (if keydown) or end (if keyup) of a click event.
		// Only track for nodes marked to be tracked, and not for buttons or inputs,
		// since buttons handle keyboard click natively, and text inputs should not
		// prevent typing spaces or newlines.
		if((e.keyCode === keys.ENTER || e.keyCode === keys.SPACE) && !/input|button|textarea/i.test(e.target.nodeName)){

			// Test if a node or its ancestor has been marked with the dojoClick property to indicate special processing
			for(var node = e.target; node; node = node.parentNode){
				if(node.dojoClick){ return true; }
			}
		}
	}

	var lastKeyDownNode;

	on(document, "keydown", function(e){
		//console.log("a11yclick: onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
		if(clickKey(e)){
			// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
			lastKeyDownNode = e.target;

			// Prevent viewport scrolling on space key in IE<9.
			// (Reproducible on test_Button.html on any of the first dijit/form/Button examples)
			e.preventDefault();
		}else{
			lastKeyDownNode = null;
		}
	});

	on(document, "keyup", function(e){
		//console.log("a11yclick: onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
		if(clickKey(e) && e.target == lastKeyDownNode){	// === breaks greasemonkey
			//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
			lastKeyDownNode = null;

			on.emit(e.target, "click", {
				cancelable: true,
				bubbles: true,
				ctrlKey: e.ctrlKey,
				shiftKey: e.shiftKey,
				metaKey: e.metaKey,
				altKey: e.altKey,
				_origType: e.type
			});
		}
	});

	// I want to return a hash of the synthetic events, but for backwards compatibility the main return value
	// needs to be the click event.   Change for 2.0.

	var click = function(node, listener){
		// Set flag on node so that keydown/keyup above emits click event
		node.dojoClick = true;

		return on(node, "click", listener);
	};
	click.click = click;	// forward compatibility with 2.0

	click.press =  function(node, listener){
		var touchListener = on(node, touch.press, function(evt){
			if(evt.type == "mousedown" && !mouse.isLeft(evt)){
				// Ignore right click
				return;
			}
			listener(evt);
		}), keyListener = on(node, "keydown", function(evt){
			if(evt.keyCode === keys.ENTER || evt.keyCode === keys.SPACE){
				listener(evt);
			}
		});
		return {
			remove: function(){
				touchListener.remove();
				keyListener.remove();
			}
		};
	};

	click.release =  function(node, listener){
		var touchListener = on(node, touch.release, function(evt){
			if(evt.type == "mouseup" && !mouse.isLeft(evt)){
				// Ignore right click
				return;
			}
			listener(evt);
		}), keyListener = on(node, "keyup", function(evt){
			if(evt.keyCode === keys.ENTER || evt.keyCode === keys.SPACE){
				listener(evt);
			}
		});
		return {
			remove: function(){
				touchListener.remove();
				keyListener.remove();
			}
		};
	};

	click.move = touch.move;	// just for convenience

	return click;
});

},
'idx/form/_CssStateMixin':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
"dojo", 
"dijit/dijit", 
"dijit/_WidgetBase", 
"dojo/_base/array", 
"dojo/has", 
"dojo/ready", 
"dojo/on", 
"dojo/_base/window", 
"dijit/registry",
"dojo/dom", 
"dojo/dom-class"], 
function(dojo, dijit, widgetBase, array, has, ready, on, win, registry, dom, domClass){

var CssStateMixin =  dojo.declare("idx.form._CssStateMixin", [], {
	
	cssStateNodes: {},
	hovering: false,
	active: false,

	// stateNode
	//		The original dijit domNode (inner field widget)
	
	// oneuiBaseClass
	//		The original dijit baseClass (inner field widget)

	_applyAttributes: function(){
		widgetBase.prototype._applyAttributes.apply(this, arguments);

		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		dojo.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active", "required"], function(attr){
			this.watch(attr, dojo.hitch(this, "_setStateClass"));
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		this._trackMouseState(this.stateNode, this.oneuiBaseClass);
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets hovering and active properties depending on mouse state,
		//	which triggers _setStateClass() to set appropriate CSS classes for this.domNode.
		if(!this.disabled){
			switch(event.type){
				case "mouseover":
					this._set("hovering", true);
					this._set("active", this._mouseDown);
					break;
				case "mouseout":
					this._set("hovering", false);
					this._set("active", false);
					break;
				case "mousedown":
				case "touchstart":
					this._set("active", true);
					break;
				case "mouseup":
				case "touchend":
					this._set("active", false);
					break;
			}
		}
	},
	
	_setStateClass: function(){
		// Compute new set of classes
		var newStateClasses = this._getModifiedClasses(this.oneuiBaseClass);
		this._applyStateClass(this.stateNode, newStateClasses);
		newStateClasses = this._getModifiedClasses(this.baseClass);
		this._applyStateClass(this.domNode, newStateClasses);
	},
	
	_getModifiedClasses: function(/*String*/className){
		var clazz = className.split(" ");
		function multiply(modifier){
			clazz = clazz.concat(dojo.map(clazz, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		var checkedState = this.checked == "mixed" ? "Mixed" : (this.checked ? "Checked" : "");
		if(this.checked){
			multiply(checkedState);
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}
		if(this.required){
			multiply("Required");
		}
		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this.active){
				multiply("Active");
			}else if(this.hovering){
				multiply("Hover");
			}
		}

		if(this.focused){
			multiply("Focused");
		}
		return clazz;
	},
	
	_applyStateClass: function(/*DomNode*/ node, /*Array*/classes){
		// Compute new set of classes
		// Remove old state classes and add new ones.
		// For performance concerns we only write into stateNode.className and domNode.className once.
		var classHash = {};	// set of all classes (state and otherwise) for node
		
		dojo.forEach(node.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in node){
			dojo.forEach(node._stateClasses, function(c){ delete classHash[c]; });
		}

		dojo.forEach(classes, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		node.className = newClasses.join(" ");
		node._stateClasses = classes;
	},
	
	_subnodeCssMouseEvent: function(node, clazz, evt){
		// summary:
		//		Handler for hover/active mouse event on widget's subnode
		if(this.disabled || this.readOnly){
			return;
		}
		var classSet = clazz.split(/\s+/);
		function hover(isHovering){
			array.forEach(classSet, function(clazz){
				domClass.toggle(node, clazz+"Hover", isHovering);
			})
		}
		function active(isActive){
			array.forEach(classSet, function(clazz){
				domClass.toggle(node, clazz+"Active", isActive);
			})
		}
		function focused(isFocused){
			array.forEach(classSet, function(clazz){
				domClass.toggle(node, clazz+"Focused", isFocused);
			})
		}
		switch(evt.type){
			case "mouseover":
				hover(true);
				break;
			case "mouseout":
				hover(false);
				active(false);
				break;
			case "mousedown":
			case "touchstart":
				active(true);
				break;
			case "mouseup":
			case "touchend":
				active(false);
				break;
			case "focus":
			case "focusin":
				focused(true);
				break;
			case "blur":
			case "focusout":
				focused(false);
				break;
		}
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//
		//		- fooActive: if the user is currently pressing down the mouse button while over the node
		//		- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//		- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow)

		// Flag for listener code below to call this._cssMouseEvent() or this._subnodeCssMouseEvent()
		// when node is hovered/active
		node._cssState = clazz;
	}
});
ready(function(){
	// Document level listener to catch hover etc. events on widget root nodes and subnodes.
	// Note that when the mouse is moved quickly, a single onmouseenter event could signal that multiple widgets
	// have been hovered or unhovered (try test_Accordion.html)
	function handler(evt){
		// Poor man's event propagation.  Don't propagate event to ancestors of evt.relatedTarget,
		// to avoid processing mouseout events moving from a widget's domNode to a descendant node;
		// such events shouldn't be interpreted as a mouseleave on the widget.
		if(!dom.isDescendant(evt.relatedTarget, evt.target)){
			for(var node = evt.target; node && node != evt.relatedTarget; node = node.parentNode){
				// Process any nodes with _cssState property.   They are generally widget root nodes,
				// but could also be sub-nodes within a widget
				if(node._cssState){
					var widget = registry.getEnclosingWidget(node);
					if(widget){
						if(node == widget.oneuiBaseNode){
							widget._cssMouseEvent(evt);
						}else{
							if(node == widget.domNode){
								// event on the widget's root node
								widget._cssMouseEvent(evt);
							}else{
								// event on widget's sub-node
								widget._subnodeCssMouseEvent(node, node._cssState, evt);
							}
						}
					}
				}
			}
		}
	}
	function ieHandler(evt){
		evt.target = evt.srcElement;
		handler(evt);
	}

	// Use addEventListener() (and attachEvent() on IE) to catch the relevant events even if other handlers
	// (on individual nodes) call evt.stopPropagation() or event.stopEvent().
	// Currently typematic.js is doing that, not sure why.
	// Don't monitor mouseover/mouseout on mobile because iOS generates "phantom" mouseover/mouseout events when
	// drag-scrolling, at the point in the viewport where the drag originated.   Test the Tree in api viewer.
	var body = win.body(),
		types = (has("touch") ? [] : ["mouseover", "mouseout"]).concat(["mousedown", "touchstart", "mouseup", "touchend"]);
	array.forEach(types, function(type){
		if(body.addEventListener){
			body.addEventListener(type, handler, true);	// W3C
		}else{
			body.attachEvent("on"+type, ieHandler);	// IE
		}
	});

	// Track focus events on widget sub-nodes that have been registered via _trackMouseState().
	// However, don't track focus events on the widget root nodes, because focus is tracked via the
	// focus manager (and it's not really tracking focus, but rather tracking that focus is on one of the widget's
	// nodes or a subwidget's node or a popup node, etc.)
	// Remove for 2.0 (if focus CSS needed, just use :focus pseudo-selector).
	on(body, "focusin, focusout", function(evt){
		var node = evt.target;
		if(node._cssState && !node.getAttribute("widgetId")){
			var widget = registry.getEnclosingWidget(node);
			widget._subnodeCssMouseEvent(node, node._cssState, evt);
		}
	});
});
return CssStateMixin;
});
},
'dijit/form/HorizontalRule':function(){
define([
	"dojo/_base/declare", // declare
	"../_Widget",
	"../_TemplatedMixin"
], function(declare, _Widget, _TemplatedMixin){

	// module:
	//		dijit/form/HorizontalRule

	return declare("dijit.form.HorizontalRule", [_Widget, _TemplatedMixin], {
		// summary:
		//		Hash marks for `dijit/form/HorizontalSlider`

		templateString: '<div class="dijitRuleContainer dijitRuleContainerH"></div>',

		// count: Integer
		//		Number of hash marks to generate
		count: 3,

		// container: String
		//		For HorizontalSlider, this is either "topDecoration" or "bottomDecoration",
		//		and indicates whether this rule goes above or below the slider.
		container: "containerNode",

		// ruleStyle: String
		//		CSS style to apply to individual hash marks
		ruleStyle: "",

		_positionPrefix: '<div class="dijitRuleMark dijitRuleMarkH" style="left:',
		_positionSuffix: '%;',
		_suffix: '"></div>',

		_genHTML: function(pos){
			return this._positionPrefix + pos + this._positionSuffix + this.ruleStyle + this._suffix;
		},

		// _isHorizontal: [protected extension] Boolean
		//		VerticalRule will override this...
		_isHorizontal: true,

		buildRendering: function(){
			this.inherited(arguments);

			var innerHTML;
			if(this.count == 1){
				innerHTML = this._genHTML(50, 0);
			}else{
				var i;
				var interval = 100 / (this.count - 1);
				if(!this._isHorizontal || this.isLeftToRight()){
					innerHTML = this._genHTML(0, 0);
					for(i = 1; i < this.count - 1; i++){
						innerHTML += this._genHTML(interval * i, i);
					}
					innerHTML += this._genHTML(100, this.count - 1);
				}else{
					innerHTML = this._genHTML(100, 0);
					for(i = 1; i < this.count - 1; i++){
						innerHTML += this._genHTML(100 - interval * i, i);
					}
					innerHTML += this._genHTML(0, this.count - 1);
				}
			}
			this.domNode.innerHTML = innerHTML;
		}
	});
});

},
'idx/form/_CompositeMixin':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare", 
	"dojo/_base/lang",
	"dojo/aspect",
	"dojo/dom-attr",
	"dojo/dom",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/i18n", 
	"dojo/query", 
	"dojo/dom-class", 
	"dojo/dom-style",
	"dojo/on",
	"dijit/_base/wai", 
	"idx/widget/HoverHelpTooltip",
	"../util",
	"../string",
	"./_FocusManager"
], function(declare, lang, aspect, domAttr, dom, domConstruct, domGeometry, i18n, query, domClass, domStyle, on, wai, HoverHelpTooltip, iUtil, iString, _focusManager) {
	/**
	 * @public
	 * @name idx.form._CompositeMixin
	 * @class Mix-in class to provide customized label, hint, unit, and field layout, implemented according to 
	 * IBM One UI(tm) <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y29&vsub=*&hsub=*&openpanes=0000010000">Text Areas & Fields</a></b>.
	 * It takes the assumption that a composite widget will follow the dom structure below
	 * <br>
	 &lt;div class="idxComposite"&gt;
		&lt;div class="idxLabel"&gt;
			&lt;span&gt;*&lt;/span&gt;
				&lt;label dojotAttachPoint="compLabelNode"&gt;Label Text&lt;/label&gt;
		&lt;/div&gt;
		&lt;div&gt;other dom structure...&lt;/div&gt;&lt;div dojoAttachPoint="compUnitNode"&gt;unit text&lt;/div&gt;
		&lt;div dojoAttachPoint="compHintNode" class="idxHint dijitHidden"&gt;hint text&lt;/div&gt;
	 &lt;/div>
	 * <br>
	 * @aguments idx.form._FocusManager
	 */

	
	return declare("idx.form._CompositeMixin", null, 
	/**@lends idx.form._CompositeMixin#*/
	{
		/**
		 * Layout of the label and the field, "horizontal" or "vertical", implemented according to 
		 * IBM One UI(tm) <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y16&vsub=*&hsub=*&openpanes=0000011100">Field & Label Alignment</a></b>
		 * @type String
		 * @default "horizontal"
		 */
		labelAlignment: "horizontal",
		
		/**
		 * Label text
		 * @type String
		 */
		label: "",
		
		/**
		 * Width from the left of label to the left of corresponding field, this parameter works in the composite widget layout of "horizontal".
		 * @type String | Number
		 */
		labelWidth: "",
		
		/**
		 * Width of the field with a hidden validation icon
		 * @type String | Number
		 */
		fieldWidth: "",
		
		/**
		 * For input widgets only. The position of the hint text: "inside" / "outside", inner the field input or not.
		 * @type String
		 * @default "inside"
		 */
		hintPosition: "inside",
		
		/**
		 * For input widgets only. The hint text.
		 * @type String
		 */
		hint: "",
		
		/**
		 * Indicates that it's a required field or not. A required field will have a red asterisk.
		 * implemented according to 
	 	 * IBM One UI(tm) <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y17&vsub=*&hsub=*&openpanes=0000011100">Required Fields</a></b>.
		 * @type boolean
		 * @default false
		 */
		required: false,
		
		/**
		 * The text of unit for the numerical value input widget.
		 * @type String
		 */
		unit: "",
		
		/**
		 * Focus manager for all composite widget
		 * @type idx.form.FocusManager
		 * @private
		 */
		_focusManager: _focusManager,
		
		
		/**
		 * Help message popup from help icon follows label.
		 * @type String
		 */
		help:"",
		
		_errorIconWidth: 27,
		
		postMixInProperties: function(){
			this.tooltipPosition = ["after-centered", "above"];
			this.inherited(arguments);
		},
		/**
		 * Handles resizing form widgets.
		 */
		resize: function() {
			
			if(this._holdResize()){
				return;
			}
			domStyle.set(this.domNode, {visibility: "hidden"});
			// if percentage style width then clear the label and field widths so the parent container
			// has the opportunity to resize
			if (iUtil.isPercentage(this._styleWidth)) {
				domStyle.set(this.labelWrap, {width: ""});
				domStyle.set(this.oneuiBaseNode, {width: ""});
			}
			
			// schedule a resize
			if (this._resizeTimeout) {
				clearTimeout(this._resizeTimeout);
				delete this._resizeTimeout;
			}
			this._resizeTimeout = setTimeout(lang.hitch(this, this._resize), 250);
		},
		/**
		 * Check if resize action should be hold, by widget visibility and applied width
		 */
		_holdResize: function(widgetInvisible){
			if(!this.domNode){return true;}
			if(!(this.labelWidth || this.fieldWidth || this._styleWidth)){
				domStyle.set(this.labelWrap, {width: ""});
				domStyle.set(this.oneuiBaseNode, {width: ""});				
				return true;
			}
			var swFixed = (this._styleWidth && !iUtil.isPercentage(this._styleWidth)),
				fwFixed = (this.fieldWidth && !iUtil.isPercentage(this.fieldWidth)),
				lwFixed = (this.labelWidth && !iUtil.isPercentage(this.labelWidth)),
				widthFixed = swFixed || (fwFixed&&lwFixed),				
				widgetInvisible = domGeometry.getContentBox(this.domNode).w <= 0;
				
			return (widthFixed && this._resized) || // If widget width is fixed, and widget has been resized of fit, then skip.
				(!widthFixed) && widgetInvisible; // If widget is invisible with percentage width, then skip.
		},
		/**
		 * Handles deferring the resize until the widget is started.  This function returns true
		 * if the resize should be deferred.
		 */
		_deferResize: function() {
			// check if the widget has no fixed width -- there is no point in resizing until the widget's 
			// DOM node is properly placed in the DOM since percentage width cannot be computed  before that
			if (!this._started) {
				// if we have not yet created a resize callback on "startup", then use aspect to do that
				if (!this._resizeHandle) {
					this._resizeHandle = aspect.after(this, "startup", lang.hitch(this, this._resize));
				}
				return true; // defer until startup
			}
			else {
				// if we are started and previously created a resizeHandle, then remove it
				if (this._resizeHandle) {
					this._resizeHandle.remove();
					delete this._resizeHandle;
				}
				
				return false;
			}	
		},
		
		_resize: function(){
			if (this._resizeTimeout) { 
				clearTimeout(this._resizeTimeout); 
				delete this._resizeTimeout; 
			}
			var deferring = this._deferResize();
			if (deferring) {
				return;
			}
			if(this._holdResize()){
				return;
			}
			
			var labelWidth = this.labelWidth,
				fieldWidth = this.fieldWidth,
				styleWidth = this._styleWidth;

			var	realWidth = domGeometry.getContentBox(this.domNode).w,
				realWidth = ((realWidth <= 0) && styleWidth)? iUtil.normalizedLength(styleWidth,this.domNode) : realWidth;

			if (!styleWidth) {
				if (iUtil.isPercentage(labelWidth)) domStyle.set(this.labelWrap, "width", "");
				if (iUtil.isPercentage(fieldWidth)) domStyle.set(this.oneuiBaseNode, "width", "");				
			}
			
			if(this.label && this.labelAlignment == "horizontal"){
				if(labelWidth){
					if(iUtil.isPercentage(labelWidth)){
						labelWidth = Math.floor(realWidth * parseInt(labelWidth)/100) - 10;
					}
					domStyle.set(this.labelWrap, "width", iUtil.normalizedLength(labelWidth,this.domNode) + "px");
				} else {
					domStyle.set(this.labelWrap, "width", "");
				}
				if(fieldWidth){
					var isFieldWidthPercentage = iUtil.isPercentage(fieldWidth);
					if(isFieldWidthPercentage){
						fieldWidth = Math.floor(realWidth * parseInt(fieldWidth)/100);
					}
					domStyle.set(this.oneuiBaseNode, "width", 
						iUtil.normalizedLength(fieldWidth,this.domNode) - (isFieldWidthPercentage ? (this._errorIconWidth + 2) : 0) + "px");
				} else {
					domStyle.set(this.oneuiBaseNode, "width", "");
				}
				if (styleWidth && !fieldWidth) {
					var compLabelWidth = domGeometry.getMarginBox(this.labelWrap).w;
					var compFieldWidth = realWidth - this._errorIconWidth - compLabelWidth - 2;
					if (compFieldWidth > 0) {
						domStyle.set(this.oneuiBaseNode, "width", compFieldWidth + "px");
					}
				}
			}else{
				if(styleWidth){
					domStyle.set(this.oneuiBaseNode, "width", realWidth - this._errorIconWidth - 2 + "px");
				}else if(fieldWidth && !iUtil.isPercentage(fieldWidth)){
					domStyle.set(this.oneuiBaseNode, "width", iUtil.normalizedLength(fieldWidth,this.domNode) + "px");
				} else {
					domStyle.set(this.oneuiBaseNode, "width", "");
				}
			}
			this._resizeHint();
			domStyle.set(this.domNode, {visibility: ""});
			this._resized = true;
		},
		_resizeHint: function(){
			if(this.hint && (this.hintPosition == "outside") && this._created){
				var inputWidth = domStyle.get(this.fieldWrap || this.oneuiBaseNode, "width")/*valication icon placeholder*/;
				domStyle.set(this.compHintNode, "width", inputWidth? inputWidth + "px" : "");
			}
		},
		_setStyleAttr: function(style){
			// If there's no label share the widget width with field,
			// field would occupy whole widget width if "width" is specified in given style.
			this.inherited(arguments);
			this._styleWidth = iUtil.getValidCSSWidth(style);
			this._created && this._resize();
		},
				/**
		 * Set the width of label, the width is from the start of label to the start of the field.
		 * @public
		 * @param {string | number} width 
		 * Unit of "pt","em","px" will be normalized to "px", and "px" by default for numeral value.
		 */
		_setLabelWidthAttr: function(/*String | Integer*/width){
			domStyle.set(this.labelWrap, "width", typeof width === "number" ? width + "px" : width);
			this._set("labelWidth", width);
			this._created && this._resize();
		},
		
		/**
		 * Set the width of field with a hidden validation icon.
		 * @public
		 * @param {string | number} width 
		 * Unit of "pt","em","px" will be normalized to "px", and "px" by default for numeral value.
		 */
		_setFieldWidthAttr: function(/*String | Integer*/width){
			if(!iUtil.isPercentage(width)){
				domStyle.set(this.oneuiBaseNode, "width", typeof width === "number" ? width + "px" : width);
			}
			this._set("fieldWidth", width);
			this._created && this._resize();
		},
		
		/**
		 * Set the alignment for the label and field,  update the style of the label node to make 
		 * it be at the right place.
		 * @public
		 * @param {string} alignment
		 * The alignment of the label and field. Can be "vertical" or "horizontal".
		 * If "vertical" is used, the label is put above the TextBox.
		 * If "horizontal" is used, the label is put on the left of the TextBox (on
		 * the right of the TextBox if RTL language is used).
		 */
		_setLabelAlignmentAttr: function(/*String*/ alignment){
			var h = alignment == "horizontal";
			domClass.toggle(this.labelWrap, "dijitInline", h);
			query(".idxCompContainer", this.domNode).toggleClass("dijitInline", h);
			this._set("labelAlignment", alignment);
			this._resize();
		},
		
		_setupHelpListener: function() {
			if (this._helpListener) return;
			var blurHandler = function(e) {
				if (this.widget.helpTooltip) {
					this.widget.helpTooltip.close(true);
				}
				if (this.handle) {
					this.handle.remove();
					delete this.handle;
					this.widget._blurHandler = null;
				}
			};
			var keyHandler = function(e) {
				var charOrCode = e.keyCode;
				if(e.type == "keydown"){
					if(e.ctrlKey && e.shiftKey && (charOrCode == 191) && this.helpTooltip && this._help) {
						this.helpTooltip.open(this.helpNode);
				
						if (e.target && !this._blurHandler) {
							var scope = {
									handle: null,
									widget: this
							};
							this._blurHandler = scope.handle = on(e.target, "blur", lang.hitch(scope, blurHandler));
						}
					}
				}
			};
			var node = (this.oneuiBaseNode) ? this.oneuiBaseNode : this.focusNode;
			if (node) {
				this._keyListener = this.own(on(node, "keydown", lang.hitch(this, keyHandler)));
			}
		},
		
		_setHelpAttr: function(/*String*/ helpText){
			this.help = helpText; // set the help to what the caller provided
			this._help = helpText = iString.nullTrim(helpText); // set the internal value to the trimmed version
			if (helpText) {
				if (!this.helpNode) {
					this.helpNode = domConstruct.toDom("<div class='dijitInline idxTextBoxHoverHelp'><span class='idxHelpIconAltText'>?</span></div>");
					domConstruct.place(this.helpNode, this.compLabelNode, "after");
					this.helpTooltip = new HoverHelpTooltip({
						connectId: [this.helpNode],
						label: helpText,
						position: ['above', 'below'],
						forceFocus: false,
						textDir: this.textDir
					});
					this._setupHelpListener();
					
				}
				else {
					this._setupHelpListener();
					this.helpTooltip.set("label", helpText);
					domClass.remove(this.helpNode, "dijitHidden");
				}
			}
			else {
				if (this.helpNode) {
					this.helpTooltip.set("label", "");
					domClass.add(this.helpNode, "dijitHidden");
				}
			}
		},
		
		/**
		 * Set the label text. Update the content of the label node.
		 * @public
		 * @param {string} label
		 * The text will be displayed as the content of the label. If text is null or
		 * empty string, nothing would be displayed.
		 */
		_setLabelAttr: function(/*String*/ label){
			this.compLabelNode.innerHTML = label;
			var islabelEmpty = /^\s*$/.test(label);
			domAttr[islabelEmpty?"remove":"set"](this.compLabelNode, "for", this.id);
			domClass.toggle(this.labelWrap, "dijitHidden", islabelEmpty);
			this._set("label", label);
		},
		
		/**
		 * Set this field as a required field or not. If this field is required,
		 * a red asterisk will be shown at the start of label.
		 * @public
		 * @param {boolean} required
		 */
		_setRequiredAttr: function(/*Boolean*/ required){
			wai.setWaiState(this.focusNode, "required", required + "");
			this._set("required", required);
			this._refreshState();
		},
		
		_refreshState: function(){
			if(!this._created){return;}
			if(this.disabled){
				this._set("state", "");
			}else if(this.required && this._isEmpty(this.value) && (this.state == "")){
				this._set("state", "Incomplete");
			}
		},
		_setValueAttr: function(){
			// summary:
			//		Hook so set('value', ...) works.
			this.inherited(arguments);
			this.validate(this.focused);
			// Workaround for idt ValidationTextBox defect of #12517
			if(this.focusNode && (!this._hasBeenBlurred)){
				domAttr.remove(this.focusNode, "aria-invalid");
			}
		},
		
		/**
		 * Set position of the hint text. If position is "outside", update the content
		 * of the hint node. If position is "inside" and the value of the TextBox is
		 * null, set the value of the TextBox to the hintText
		 * @protected
		 * @param {string} position
		 * The position of the label. Can be "outside" or "inside".
		 * If "outside" is used, the hint text is put below the TextBox.
		 * If "inside" is used and the TextBox has a value, display the value in the TextBox. Once
		 * the value of the TextBox is null, display the hint text inside the TextBox in a specified
		 * color (e.g: gray).
		 */
		_setHintPositionAttr: function(/*String*/ position){
			if(!this.compHintNode){ return; }
			this._set("hintPosition", position);
			domClass.toggle(this.compHintNode, "dijitVisible", position != "inside");
			this.set("hint", this.hint);
			this._resizeHint();
		},
		
		/**
		 * Set the hint text
		 * @public
		 * @param {string} hint
		 * The text will be displayed inside or below the TextBox per the "position" attribute.
		 */
		_setHintAttr: function(/*String*/ hint){
			this.set("placeHolder", this.hintPosition == "inside" ? hint : "");
			if(!this.compHintNode){ return; }
			this.compHintNode.innerHTML = this.hintPosition == "inside" ? "" : hint;
			
			if(this.hintPosition == "outside"){
				domAttr.set(this.compHintNode, "id", this.id + "_hint_outside");
			}
			if(this._placeholder === undefined || this._placeholder === false){
				wai.setWaiState(this.focusNode, "describedby", this.id + "_hint_" + this.hintPosition);
			}
			this._set("hint", hint);
			this._resizeHint();
		},
		
		_setPlaceHolderAttr: function(v){
			this._set("placeHolder", v);
			if(v === null || v === undefined){
				v = "";
			}
			if(this.focusNode.placeholder !== undefined && this._placeholder !== false){
				domAttr.set(this.focusNode, "placeholder", v);
				this._placeholder = v;
			}else{
				if(!this._phspan){
					this._attachPoints.push('_phspan');
					this._phspan = domConstruct.create('span',{
						className:'dijitPlaceHolder dijitInputField',
						id: this.id + "_hint_inside"
					},this.focusNode,'after');
				}
				this._phspan.innerHTML = "";
				this._phspan.appendChild(document.createTextNode(v));
				this._phspan.style.display=(this.placeHolder&&!this.focused&&!this.textbox.value)?"":"none";
			}
		},
		
		/**
		 * Set the text of unit
		 * @public
		 * @param {string} unit
		 * The unit will be displayed on the right of the input box(on the left of the input
		 * box if RTL language is used).
		 */
		_setUnitAttr: function(/*String*/ unit){
			if(!this.compUnitNode){ return; }
			this.compUnitNode.innerHTML = unit;
			domClass.toggle(this.compUnitNode, "dijitHidden", /^\s*$/.test(unit));
			this._set("unit", unit);
		},
		_isValidFocusNode: function(mousedownNode){
			return dom.isDescendant(mousedownNode, this.oneuiBaseNode) ||
				!dom.isDescendant(mousedownNode, this.domNode);
		},
		
		/**
		 * Reset the value and state of the composite widget.
		 * @public
		 */
		reset: function(){
			this.set("state", this.required ? "Incomplete" : "");
			this.message = "";
			this.inherited(arguments);
		},
		/**
		 * Validate value, directly get focus and show error if turn out to be invalid.
		 * @public
		 */
		validateAndFocus: function(){
			if(this.validate && !this.disabled){
				var hasBeenBlurred = this._hasBeenBlurred;
				this._hasBeenBlurred = true;
				var valid = this.validate();
				if(!valid){
					this.focus();
				}
				this._hasBeenBlurred = hasBeenBlurred
				return valid;
			}
			return true;
		}
	});
});

},
'xcitec/Label':function(){
/*
todo: still there is a problem with max width for IE6 when label inside template
      getting label width in postCreate and startup return 0.

todo: general problem if widget in template calculation of sizes is impossible.
     if label is not visible - size is always 0!
 */

define(["dojo/_base/declare",
        "dojo/dom-construct",
        "dojo/dom-class",
        "dojo/_base/lang",
        "dojo/dom",
        "dojo/dom-style",
        "dojo/dom-geometry",
        "dojo/has",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "./xcitecUtil", "dijit/_OnDijitClickMixin"], function(declare, domConstruct, domClass, lang, dom, domStyle, geometry, has, _Widget, _Templated, xcitecUtil, _OnDijitClickMixin){
  return declare([_Widget, _OnDijitClickMixin], {

    useFFhack: true,
    baseClass: "xcitecLabel",
    baseClassExtended: "",
    additionalClasses: "",
    labelType: "",
    labelId: "",
    label: "",
    htmlFor: "",
    color: "",

    inline: false,

    // marking that label is an anchor (has onLabelClick event defined)
    hasAnchor: false,

    // marking that long label can be wrapped and takes more then one line
    allowWrap: false,

    // label "owner" control is marked as required
    required: false,
    // label "owner" control is marked as disabled
    disabled: false,
    // label "owner" control is marked as error
    error: false,

    //requiredPrefix: "*",
    requiredPrefix: "",

    maxLabelWidth: 0,

    postMixInProperties: function(){
      this.inherited(arguments);

		if(this.label){
			this.label = xcitecUtil.escapeXml(this.label);
		}

      if(this.labelType){
        this.baseClass = xcitecUtil.createTypedStyleClass(this.baseClass, this.labelType);
      }

      if(this.allowWrap){
              this.label = this.label.replace(/&lt;br&gt;/gm, "<br>");
              this.useFFhack = false;
        this.additionalClasses += " xcitecLabelMultiline";
      }

      if((has("ff") || has("ie") == 8) && this.inline){
        this.additionalClasses += " xcitecElipsesLabelInLine";
      }

      if(this.hasAnchor){
        this.baseClassExtended += " " + this.baseClass + "Anchor";
      }
      if(this.disabled){
        this.baseClassExtended += " " + this.baseClass + "Disabled";
      }else if(this.required){
        this.baseClassExtended += " " + this.baseClass + "Required";
      }
      if(this.error){
        this.baseClassExtended += " " + this.baseClass + "Error";
      }

    },

    buildRendering: function(){
      var source = this.srcNodeRef;
      if(source){
        if(!this.htmlFor) this.htmlFor = source.htmlFor;
        if(!("label" in this.params)){
          this.label = source.innerHTML;
        }
      }

		if(this.required && this.requiredPrefix && this.label){
			this.label = this.requiredPrefix + this.label;
		}

      this.domNode = domConstruct.create("label");
      this.domNode.innerHTML = this.label;

      var classValue = "xcitecElipsesLabel " + this.additionalClasses + " " + this.baseClass + this.baseClassExtended;
      domClass.add(this.domNode, classValue);
      //this.inherited(arguments);

      if(has("ff") && has("ff") < 4 && this.useFFhack){
        var description = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "label");
        description.setAttribute("align", "end");
        // XUL description doesn't process html escape characters!!!
        description.setAttribute("value", lang.trim(this.domNode.textContent));
        //description.setAttribute("tooltiptext","Vk TEST");
        this.domNode.innerHTML = '';
        this.domNode.appendChild(description);
      }
      if(this.htmlFor){
        this.domNode.htmlFor = this.htmlFor;
      }
		if(this.color && this.color != ""){
			this.domNode.style.color = this.color;
		}
      dom.setSelectable(this.domNode, false);
      this.connect(this.domNode, "ondijitclick", this._onLabelClick);
      this.connect(this.domNode, "onmouseover", this._onLabelOver);
      this.connect(this.domNode, "onmouseout", this._onLabelOut);
    },

    getLabelWidth: function(){
      var cbox = geometry.getContentBox(this.domNode);
      var labelWidth = cbox.w;
      if(this.maxLabelWidth > 0 && labelWidth > this.maxLabelWidth) labelWidth = this.maxLabelWidth;
      return labelWidth;
    },

    _isAdminClickAllowed: function(){
      return this.labelId && lang.isFunction(xcitecUtil.onLabelAdminClick);
    },

    _onLabelClick: function(evt){
      if(this._isAdminClickAllowed() && evt.ctrlKey && evt.altKey){
        //console.log(this.declaredClass + ": _onLabelClick: labelId: " + this.labelId);
        xcitecUtil.processLabelAdminClick(this.labelId);
        evt.preventDefault(), evt.stopPropagation()
      }else if(this.hasAnchor){
        this.onLabelClick();
        evt.preventDefault(), evt.stopPropagation()
      }else if(this.htmlFor && dom.byId(this.htmlFor)){
        dom.byId(this.htmlFor).focus();
        evt.preventDefault(), evt.stopPropagation()
      }
    },

    _onLabelOver: function(evt){
      if(this._isAdminClickAllowed() && evt.ctrlKey && evt.altKey){
        domClass.add(this.domNode, 'xcitecLabelAdminHover');
        this._adminHovered = true;
      }
    },

    _onLabelOut: function(evt){
      if(this._adminHovered){
        domClass.remove(this.domNode, 'xcitecLabelAdminHover');
      }
    },

    onLabelClick: function(){
      // user defined Event
    },

    _setLabelAttr: function(/*String*/ value){
      this.label = value;
      if(this.required && this.requiredPrefix && this.label){
        this.label = this.requiredPrefix + this.label;
      }
    },

	  _setDisabledAttr: function(/*Boolean*/ value){
		  this._set('disabled', value);
	  },

    logLabelWidth: function(/*String*/ value){
      var labelWidth = geometry.getContentBox(this.domNode).w;
      //console.log("["+ this.id +"] "+ value +" label width: "+ labelWidth );
      //console.log("["+ this.id +"] "+ value +" max-width : " + dojo.style(this.domNode, "max-width"));
    }

  });
});



},
'dijit/hccss':function(){
define(["dojo/dom-class", "dojo/hccss", "dojo/domReady", "dojo/_base/window"], function(domClass, has, domReady, win){

	// module:
	//		dijit/hccss

	/*=====
	return function(){
		// summary:
		//		Test if computer is in high contrast mode, and sets `dijit_a11y` flag on `<body>` if it is.
		//		Deprecated, use ``dojo/hccss`` instead.
	};
	=====*/

	domReady(function(){
		if(has("highcontrast")){
			domClass.add(win.body(), "dijit_a11y");
		}
	});

	return has;
});

},
'xcitec/BoldLabel':function(){
define(["dojo/_base/declare", "./Label"], function(declare, Label){
  return declare(Label, {
    additionalClasses: "xcitecBoldLabel"
  });
});

},
'dijit/form/HorizontalRuleLabels':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/number", // number.format
	"dojo/query", // query
	"dojo/_base/lang", // lang
	"./HorizontalRule"
], function(declare, has, number, query, lang, HorizontalRule){

	// module:
	//		dijit/form/HorizontalRuleLabels

	var HorizontalRuleLabels = declare("dijit.form.HorizontalRuleLabels", HorizontalRule, {
		// summary:
		//		Labels for `dijit/form/HorizontalSlider`

		templateString: '<div class="dijitRuleContainer dijitRuleContainerH dijitRuleLabelsContainer dijitRuleLabelsContainerH"></div>',

		// labelStyle: String
		//		CSS style to apply to individual text labels
		labelStyle: "",

		// labels: String[]?
		//		Array of text labels to render - evenly spaced from left-to-right or bottom-to-top.
		//		Alternately, minimum and maximum can be specified, to get numeric labels.
		labels: [],

		// numericMargin: Integer
		//		Number of generated numeric labels that should be rendered as '' on the ends when labels[] are not specified
		numericMargin: 0,

		// numericMinimum: Integer
		//		Leftmost label value for generated numeric labels when labels[] are not specified
		minimum: 0,

		// numericMaximum: Integer
		//		Rightmost label value for generated numeric labels when labels[] are not specified
		maximum: 1,

		// constraints: Object
		//		pattern, places, lang, et al (see dojo.number) for generated numeric labels when labels[] are not specified
		constraints: {pattern: "#%"},

		_positionPrefix: '<div class="dijitRuleLabelContainer dijitRuleLabelContainerH" style="left:',
		_labelPrefix: '"><div class="dijitRuleLabel dijitRuleLabelH">',
		_suffix: '</div></div>',

		_calcPosition: function(pos){
			// summary:
			//		Returns the value to be used in HTML for the label as part of the left: attribute
			// tags:
			//		protected extension
			return pos;
		},

		_genHTML: function(pos, ndx){
			var label = this.labels[ndx];
			return this._positionPrefix + this._calcPosition(pos) + this._positionSuffix + this.labelStyle +
				this._genDirectionHTML(label) +
				this._labelPrefix + label + this._suffix;
		},

		_genDirectionHTML: function(label){
			// extension point for bidi code
			return "";
		},

		getLabels: function(){
			// summary:
			//		Overridable function to return array of labels to use for this slider.
			//		Can specify a getLabels() method instead of a labels[] array, or min/max attributes.
			// tags:
			//		protected extension

			// if the labels array was not specified directly, then see if <li> children were
			var labels = this.labels;
			if(!labels.length && this.srcNodeRef){
				// for markup creation, labels are specified as child elements
				labels = query("> li", this.srcNodeRef).map(function(node){
					return String(node.innerHTML);
				});
			}
			// if the labels were not specified directly and not as <li> children, then calculate numeric labels
			if(!labels.length && this.count > 1){
				var start = this.minimum;
				var inc = (this.maximum - start) / (this.count - 1);
				for(var i = 0; i < this.count; i++){
					labels.push((i < this.numericMargin || i >= (this.count - this.numericMargin)) ? '' : number.format(start, this.constraints));
					start += inc;
				}
			}
			return labels;
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.labels = this.getLabels();
			this.count = this.labels.length;
		}
	});

	if(has("dojo-bidi")){
		HorizontalRuleLabels.extend({
			_setTextDirAttr: function(textDir){
				if(this.textDir != textDir){
					this._set("textDir", textDir);
					query(".dijitRuleLabelContainer", this.domNode).forEach(
						lang.hitch(this, function(labelNode){
							labelNode.style.direction = this.getTextDir(labelNode.innerText || labelNode.textContent || "");
						})
					);
				}
			},

			_genDirectionHTML: function(label){
				return (this.textDir ? ("direction:" + this.getTextDir(label) + ";") : "")
			}
		});
	}

	return HorizontalRuleLabels;
});

},
'xcitec/formx/EVACriterionSingleColumn':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/text!xcitec/formx/templates/EVACriterionSingleColumn.html",
	"xcitec/formx/EVACriterion"
], function (declare, domClass, template, EVACriterion) {

	var EVACriterionSingleColumn = declare( EVACriterion, {
	  templateString: template,
	  
	  isSingleColumn: true,
	  
	  _fillContent: function() {
	    if (this.isSingleColumn) {
	      domClass.add(this.containerNode, "evacriterionSingleColumn");
	      domClass.add(this.description, "evacriterionSingleColumnDescription");
	    }        
	    this.inherited(arguments);  
	  }
	});  
	return EVACriterionSingleColumn;
});
},
'xcitec/page/PopupMenuItemV2':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/fx",
	"dojo/query",
	"dojo/topic",
	"xcitec/page/_MenuItemBase"
], function (declare, domConstruct, domStyle, fx, query, topic, _MenuItemBase) {

	var PopupMenuItemV2 = declare( _MenuItemBase, {
		templateString: "<li data-dojo-attach-point='containerNode' role='menuitem'><span class='${baseClass}' data-dojo-attach-event='onclick: _onClick, onmouseenter: _onMouse, onmouseleave: _onMouse, onkeypress: _onKeyPress' data-dojo-attach-point='stateNode' tabindex='0'></span></li>",
		isPopup: true,
		baseClass: "popupMenuItem",
	
	
		postCreate: function(){
			if(!this.selected){
				domStyle.set(query(">", this.domNode)[1], "display", "none");
			}
			if(this.stateHolder){
				domConstruct.place(this.stateHolder, this.domNode, "last");
			}
			this.inherited(arguments);
		},
	
	
		// hide/open menu
		_onClick: function(e){
			e.preventDefault(), e.stopPropagation();
			this._toggleSubMenu();
		}
	});
	return PopupMenuItemV2;
});
},
'dijit/popup':function(){
define([
	"dojo/_base/array", // array.forEach array.some
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy
	"dojo/dom-geometry", // domGeometry.isBodyLtr
	"dojo/dom-style", // domStyle.set
	"dojo/has", // has("config-bgIframe")
	"dojo/keys",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"./place",
	"./BackgroundIframe",
	"./Viewport",
	"./main"    // dijit (defining dijit.popup to match API doc)
], function(array, aspect, declare, dom, domAttr, domConstruct, domGeometry, domStyle, has, keys, lang, on,
			place, BackgroundIframe, Viewport, dijit){

	// module:
	//		dijit/popup

	/*=====
	 var __OpenArgs = {
		 // popup: Widget
		 //		widget to display
		 // parent: Widget
		 //		the button etc. that is displaying this popup
		 // around: DomNode
		 //		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
		 // x: Integer
		 //		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		 // y: Integer
		 //		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		 // orient: Object|String
		 //		When the around parameter is specified, orient should be a list of positions to try, ex:
		 //	|	[ "below", "above" ]
		 //		For backwards compatibility it can also be an (ordered) hash of tuples of the form
		 //		(around-node-corner, popup-node-corner), ex:
		 //	|	{ "BL": "TL", "TL": "BL" }
		 //		where BL means "bottom left" and "TL" means "top left", etc.
		 //
		 //		dijit/popup.open() tries to position the popup according to each specified position, in order,
		 //		until the popup appears fully within the viewport.
		 //
		 //		The default value is ["below", "above"]
		 //
		 //		When an (x,y) position is specified rather than an around node, orient is either
		 //		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
		 //		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
		 //		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
		 //		and the top-right corner.
		 // onCancel: Function
		 //		callback when user has canceled the popup by:
		 //
		 //		1. hitting ESC or
		 //		2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		 //		   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
		 // onClose: Function
		 //		callback whenever this popup is closed
		 // onExecute: Function
		 //		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		 // padding: place.__Position
		 //		adding a buffer around the opening position. This is only useful when around is not set.
		 // maxHeight: Integer
		 //		The max height for the popup.  Any popup taller than this will have scrollbars.
		 //		Set to Infinity for no max height.  Default is to limit height to available space in viewport,
		 //		above or below the aroundNode or specified x/y position.
	 };
	 =====*/

	function destroyWrapper(){
		// summary:
		//		Function to destroy wrapper when popup widget is destroyed.
		//		Left in this scope to avoid memory leak on IE8 on refresh page, see #15206.
		if(this._popupWrapper){
			domConstruct.destroy(this._popupWrapper);
			delete this._popupWrapper;
		}
	}

	var PopupManager = declare(null, {
		// summary:
		//		Used to show drop downs (ex: the select list of a ComboBox)
		//		or popups (ex: right-click context menus).

		// _stack: dijit/_WidgetBase[]
		//		Stack of currently popped up widgets.
		//		(someone opened _stack[0], and then it opened _stack[1], etc.)
		_stack: [],

		// _beginZIndex: Number
		//		Z-index of the first popup.   (If first popup opens other
		//		popups they get a higher z-index.)
		_beginZIndex: 1000,

		_idGen: 1,

		_repositionAll: function(){
			// summary:
			//		If screen has been scrolled, reposition all the popups in the stack.
			//		Then set timer to check again later.

			if(this._firstAroundNode){	// guard for when clearTimeout() on IE doesn't work
				var oldPos = this._firstAroundPosition,
					newPos = domGeometry.position(this._firstAroundNode, true),
					dx = newPos.x - oldPos.x,
					dy = newPos.y - oldPos.y;

				if(dx || dy){
					this._firstAroundPosition = newPos;
					for(var i = 0; i < this._stack.length; i++){
						var style = this._stack[i].wrapper.style;
						style.top = (parseInt(style.top, 10) + dy) + "px";
						if(style.right == "auto"){
							style.left = (parseInt(style.left, 10) + dx) + "px";
						}else{
							style.right = (parseInt(style.right, 10) - dx) + "px";
						}
					}
				}

				this._aroundMoveListener = setTimeout(lang.hitch(this, "_repositionAll"), dx || dy ? 10 : 50);
			}
		},

		_createWrapper: function(/*Widget*/ widget){
			// summary:
			//		Initialization for widgets that will be used as popups.
			//		Puts widget inside a wrapper DIV (if not already in one),
			//		and returns pointer to that wrapper DIV.

			var wrapper = widget._popupWrapper,
				node = widget.domNode;

			if(!wrapper){
				// Create wrapper <div> for when this widget [in the future] will be used as a popup.
				// This is done early because of IE bugs where creating/moving DOM nodes causes focus
				// to go wonky, see tests/robot/Toolbar.html to reproduce
				wrapper = domConstruct.create("div", {
					"class": "dijitPopup",
					style: { display: "none"},
					role: "region",
					"aria-label": widget["aria-label"] || widget.label || widget.name || widget.id
				}, widget.ownerDocumentBody);
				wrapper.appendChild(node);

				var s = node.style;
				s.display = "";
				s.visibility = "";
				s.position = "";
				s.top = "0px";

				widget._popupWrapper = wrapper;
				aspect.after(widget, "destroy", destroyWrapper, true);
			}

			return wrapper;
		},

		moveOffScreen: function(/*Widget*/ widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			// Besides setting visibility:hidden, move it out of the viewport, see #5776, #10111, #13604
			var ltr = domGeometry.isBodyLtr(widget.ownerDocument),
				style = {
					visibility: "hidden",
					top: "-9999px",
					display: ""
				};
			style[ltr ? "left" : "right"] = "-9999px";
			style[ltr ? "right" : "left"] = "auto";
			domStyle.set(wrapper, style);

			return wrapper;
		},

		hide: function(/*Widget*/ widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			//		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, {
				display: "none",
				height: "auto",		// Open may have limited the height to fit in the viewport
				overflow: "visible",
				border: ""			// Open() may have moved border from popup to wrapper.
			});

			// Open() may have moved border from popup to wrapper.  Move it back.
			var node = widget.domNode;
			if("_originalStyle" in node){
				node.style.cssText = node._originalStyle;
			}
		},

		getTopPopup: function(){
			// summary:
			//		Compute the closest ancestor popup that's *not* a child of another popup.
			//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
			var stack = this._stack;
			for(var pi = stack.length - 1; pi > 0 && stack[pi].parent === stack[pi - 1].widget; pi--){
				/* do nothing, just trying to get right value for pi */
			}
			return stack[pi];
		},

		open: function(/*__OpenArgs*/ args){
			// summary:
			//		Popup the widget at the specified position
			//
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			//
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit/popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

			var stack = this._stack,
				widget = args.popup,
				node = widget.domNode,
				orient = args.orient || ["below", "below-alt", "above", "above-alt"],
				ltr = args.parent ? args.parent.isLeftToRight() : domGeometry.isBodyLtr(widget.ownerDocument),
				around = args.around,
				id = (args.around && args.around.id) ? (args.around.id + "_dropdown") : ("popup_" + this._idGen++);

			// If we are opening a new popup that isn't a child of a currently opened popup, then
			// close currently opened popup(s).   This should happen automatically when the old popups
			// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
			while(stack.length && (!args.parent || !dom.isDescendant(args.parent.domNode, stack[stack.length - 1].widget.domNode))){
				this.close(stack[stack.length - 1].widget);
			}

			// Get pointer to popup wrapper, and create wrapper if it doesn't exist.  Remove display:none (but keep
			// off screen) so we can do sizing calculations.
			var wrapper = this.moveOffScreen(widget);

			if(widget.startup && !widget._started){
				widget.startup(); // this has to be done after being added to the DOM
			}

			// Limit height to space available in viewport either above or below aroundNode (whichever side has more
			// room), adding scrollbar if necessary. Can't add scrollbar to widget because it may be a <table> (ex:
			// dijit/Menu), so add to wrapper, and then move popup's border to wrapper so scroll bar inside border.
			var maxHeight, popupSize = domGeometry.position(node);
			if("maxHeight" in args && args.maxHeight != -1){
				maxHeight = args.maxHeight || Infinity;	// map 0 --> infinity for back-compat of _HasDropDown.maxHeight
			}else{
				var viewport = Viewport.getEffectiveBox(this.ownerDocument),
					aroundPos = around ? domGeometry.position(around, false) : {y: args.y - (args.padding||0), h: (args.padding||0) * 2};
				maxHeight = Math.floor(Math.max(aroundPos.y, viewport.h - (aroundPos.y + aroundPos.h)));
			}
			if(popupSize.h > maxHeight){
				// Get style of popup's border.  Unfortunately domStyle.get(node, "border") doesn't work on FF or IE,
				// and domStyle.get(node, "borderColor") etc. doesn't work on FF, so need to use fully qualified names.
				var cs = domStyle.getComputedStyle(node),
					borderStyle = cs.borderLeftWidth + " " + cs.borderLeftStyle + " " + cs.borderLeftColor;
				domStyle.set(wrapper, {
					overflowY: "scroll",
					height: maxHeight + "px",
					border: borderStyle	// so scrollbar is inside border
				});
				node._originalStyle = node.style.cssText;
				node.style.border = "none";
			}

			domAttr.set(wrapper, {
				id: id,
				style: {
					zIndex: this._beginZIndex + stack.length
				},
				"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] + "Popup",
				dijitPopupParent: args.parent ? args.parent.id : ""
			});

			if(stack.length == 0 && around){
				// First element on stack. Save position of aroundNode and setup listener for changes to that position.
				this._firstAroundNode = around;
				this._firstAroundPosition = domGeometry.position(around, true);
				this._aroundMoveListener = setTimeout(lang.hitch(this, "_repositionAll"), 50);
			}

			if(has("config-bgIframe") && !widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _WidgetBase.destroyRendering()
				widget.bgIframe = new BackgroundIframe(wrapper);
			}

			// position the wrapper node and make it visible
			var layoutFunc = widget.orient ? lang.hitch(widget, "orient") : null,
				best = around ?
					place.around(wrapper, around, orient, ltr, layoutFunc) :
					place.at(wrapper, args, orient == 'R' ? ['TR', 'BR', 'TL', 'BL'] : ['TL', 'BL', 'TR', 'BR'], args.padding,
						layoutFunc);

			wrapper.style.visibility = "visible";
			node.style.visibility = "visible";	// counteract effects from _HasDropDown

			var handlers = [];

			// provide default escape and tab key handling
			// (this will work for any widget, not just menu)
			handlers.push(on(wrapper, "keydown", lang.hitch(this, function(evt){
				if(evt.keyCode == keys.ESCAPE && args.onCancel){
					evt.stopPropagation();
					evt.preventDefault();
					args.onCancel();
				}else if(evt.keyCode == keys.TAB){
					evt.stopPropagation();
					evt.preventDefault();
					var topPopup = this.getTopPopup();
					if(topPopup && topPopup.onCancel){
						topPopup.onCancel();
					}
				}
			})));

			// watch for cancel/execute events on the popup and notify the caller
			// (for a menu, "execute" means clicking an item)
			if(widget.onCancel && args.onCancel){
				handlers.push(widget.on("cancel", args.onCancel));
			}

			handlers.push(widget.on(widget.onExecute ? "execute" : "change", lang.hitch(this, function(){
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onExecute){
					topPopup.onExecute();
				}
			})));

			stack.push({
				widget: widget,
				wrapper: wrapper,
				parent: args.parent,
				onExecute: args.onExecute,
				onCancel: args.onCancel,
				onClose: args.onClose,
				handlers: handlers
			});

			if(widget.onOpen){
				// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
				widget.onOpen(best);
			}

			return best;
		},

		close: function(/*Widget?*/ popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.

			var stack = this._stack;

			// Basically work backwards from the top of the stack closing popups
			// until we hit the specified popup, but IIRC there was some issue where closing
			// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
			// closing C might close B indirectly and then the while() condition will run where stack==[A]...
			// so the while condition is constructed defensively.
			while((popup && array.some(stack, function(elem){
				return elem.widget == popup;
			})) ||
				(!popup && stack.length)){
				var top = stack.pop(),
					widget = top.widget,
					onClose = top.onClose;

				if(widget.onClose){
					// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here).
					// Actually, StackContainer also calls onClose(), but to mean that the pane is being deleted
					// (i.e. that the TabContainer's tab's [x] icon was clicked)
					widget.onClose();
				}

				var h;
				while(h = top.handlers.pop()){
					h.remove();
				}

				// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
				if(widget && widget.domNode){
					this.hide(widget);
				}

				if(onClose){
					onClose();
				}
			}

			if(stack.length == 0 && this._aroundMoveListener){
				clearTimeout(this._aroundMoveListener);
				this._firstAroundNode = this._firstAroundPosition = this._aroundMoveListener = null;
			}
		}
	});

	return (dijit.popup = new PopupManager());
});

},
'dojo/data/ItemFileWriteStore':function(){
define(["../_base/lang", "../_base/declare", "../_base/array", "../_base/json", "../_base/kernel",
	"./ItemFileReadStore", "../date/stamp"
], function(lang, declare, arrayUtil, jsonUtil, kernel, ItemFileReadStore, dateStamp){

// module:
//		dojo/data/ItemFileWriteStore

return declare("dojo.data.ItemFileWriteStore", ItemFileReadStore, {
	// summary:
	//		TODOC

	constructor: function(/* object */ keywordParameters){
		// keywordParameters:
		//		The structure of the typeMap object is as follows:
		// |	{
		// |		type0: function || object,
		// |		type1: function || object,
		// |		...
		// |		typeN: function || object
		// |	}
		//		Where if it is a function, it is assumed to be an object constructor that takes the
		//		value of _value as the initialization parameters.  It is serialized assuming object.toString()
		//		serialization.  If it is an object, then it is assumed
		//		to be an object of general form:
		// |	{
		// |		type: function, //constructor.
		// |		deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		// |		serialize:	function(object) //The function that converts the object back into the proper file format form.
		// |	}

		// ItemFileWriteStore extends ItemFileReadStore to implement these additional dojo.data APIs
		this._features['dojo.data.api.Write'] = true;
		this._features['dojo.data.api.Notification'] = true;

		// For keeping track of changes so that we can implement isDirty and revert
		this._pending = {
			_newItems:{},
			_modifiedItems:{},
			_deletedItems:{}
		};

		if(!this._datatypeMap['Date'].serialize){
			this._datatypeMap['Date'].serialize = function(obj){
				return dateStamp.toISOString(obj, {zulu:true});
			};
		}
		//Disable only if explicitly set to false.
		if(keywordParameters && (keywordParameters.referenceIntegrity === false)){
			this.referenceIntegrity = false;
		}

		// this._saveInProgress is set to true, briefly, from when save() is first called to when it completes
		this._saveInProgress = false;
	},

	referenceIntegrity: true, //Flag that defaultly enabled reference integrity tracking.  This way it can also be disabled pogrammatially or declaratively.

	_assert: function(/* boolean */ condition){
		if(!condition){
			throw new Error("assertion failed in ItemFileWriteStore");
		}
	},

	_getIdentifierAttribute: function(){
		// this._assert((identifierAttribute === Number) || (dojo.isString(identifierAttribute)));
		return this.getFeatures()['dojo.data.api.Identity'];
	},


/* dojo/data/api/Write */

	newItem: function(/* Object? */ keywordArgs, /* Object? */ parentInfo){
		// summary:
		//		See dojo/data/api/Write.newItem()

		this._assert(!this._saveInProgress);

		if(!this._loadFinished){
			// We need to do this here so that we'll be able to find out what
			// identifierAttribute was specified in the data file.
			this._forceLoad();
		}

		if(typeof keywordArgs != "object" && typeof keywordArgs != "undefined"){
			throw new Error("newItem() was passed something other than an object");
		}
		var newIdentity = null;
		var identifierAttribute = this._getIdentifierAttribute();
		if(identifierAttribute === Number){
			newIdentity = this._arrayOfAllItems.length;
		}else{
			newIdentity = keywordArgs[identifierAttribute];
			if(typeof newIdentity === "undefined"){
				throw new Error("newItem() was not passed an identity for the new item");
			}
			if(lang.isArray(newIdentity)){
				throw new Error("newItem() was not passed an single-valued identity");
			}
		}

		// make sure this identity is not already in use by another item, if identifiers were
		// defined in the file.  Otherwise it would be the item count,
		// which should always be unique in this case.
		if(this._itemsByIdentity){
			this._assert(typeof this._itemsByIdentity[newIdentity] === "undefined");
		}
		this._assert(typeof this._pending._newItems[newIdentity] === "undefined");
		this._assert(typeof this._pending._deletedItems[newIdentity] === "undefined");

		var newItem = {};
		newItem[this._storeRefPropName] = this;
		newItem[this._itemNumPropName] = this._arrayOfAllItems.length;
		if(this._itemsByIdentity){
			this._itemsByIdentity[newIdentity] = newItem;
			//We have to set the identifier now, otherwise we can't look it
			//up at calls to setValueorValues in parentInfo handling.
			newItem[identifierAttribute] = [newIdentity];
		}
		this._arrayOfAllItems.push(newItem);

		//We need to construct some data for the onNew call too...
		var pInfo = null;

		// Now we need to check to see where we want to assign this thingm if any.
		if(parentInfo && parentInfo.parent && parentInfo.attribute){
			pInfo = {
				item: parentInfo.parent,
				attribute: parentInfo.attribute,
				oldValue: undefined
			};

			//See if it is multi-valued or not and handle appropriately
			//Generally, all attributes are multi-valued for this store
			//So, we only need to append if there are already values present.
			var values = this.getValues(parentInfo.parent, parentInfo.attribute);
			if(values && values.length > 0){
				var tempValues = values.slice(0, values.length);
				if(values.length === 1){
					pInfo.oldValue = values[0];
				}else{
					pInfo.oldValue = values.slice(0, values.length);
				}
				tempValues.push(newItem);
				this._setValueOrValues(parentInfo.parent, parentInfo.attribute, tempValues, false);
				pInfo.newValue = this.getValues(parentInfo.parent, parentInfo.attribute);
			}else{
				this._setValueOrValues(parentInfo.parent, parentInfo.attribute, newItem, false);
				pInfo.newValue = newItem;
			}
		}else{
			//Toplevel item, add to both top list as well as all list.
			newItem[this._rootItemPropName]=true;
			this._arrayOfTopLevelItems.push(newItem);
		}

		this._pending._newItems[newIdentity] = newItem;

		//Clone over the properties to the new item
		for(var key in keywordArgs){
			if(key === this._storeRefPropName || key === this._itemNumPropName){
				// Bummer, the user is trying to do something like
				// newItem({_S:"foo"}).  Unfortunately, our superclass,
				// ItemFileReadStore, is already using _S in each of our items
				// to hold private info.  To avoid a naming collision, we
				// need to move all our private info to some other property
				// of all the items/objects.  So, we need to iterate over all
				// the items and do something like:
				//	  item.__S = item._S;
				//	  item._S = undefined;
				// But first we have to make sure the new "__S" variable is
				// not in use, which means we have to iterate over all the
				// items checking for that.
				throw new Error("encountered bug in ItemFileWriteStore.newItem");
			}
			var value = keywordArgs[key];
			if(!lang.isArray(value)){
				value = [value];
			}
			newItem[key] = value;
			if(this.referenceIntegrity){
				for(var i = 0; i < value.length; i++){
					var val = value[i];
					if(this.isItem(val)){
						this._addReferenceToMap(val, newItem, key);
					}
				}
			}
		}
		this.onNew(newItem, pInfo); // dojo/data/api/Notification call
		return newItem; // item
	},

	_removeArrayElement: function(/* Array */ array, /* anything */ element){
		var index = arrayUtil.indexOf(array, element);
		if(index != -1){
			array.splice(index, 1);
			return true;
		}
		return false;
	},

	deleteItem: function(/* dojo/data/api/Item */ item){
		// summary:
		//		See dojo/data/api/Write.deleteItem()
		this._assert(!this._saveInProgress);
		this._assertIsItem(item);

		// Remove this item from the _arrayOfAllItems, but leave a null value in place
		// of the item, so as not to change the length of the array, so that in newItem()
		// we can still safely do: newIdentity = this._arrayOfAllItems.length;
		var indexInArrayOfAllItems = item[this._itemNumPropName];
		var identity = this.getIdentity(item);

		//If we have reference integrity on, we need to do reference cleanup for the deleted item
		if(this.referenceIntegrity){
			//First scan all the attributes of this items for references and clean them up in the map
			//As this item is going away, no need to track its references anymore.

			//Get the attributes list before we generate the backup so it
			//doesn't pollute the attributes list.
			var attributes = this.getAttributes(item);

			//Backup the map, we'll have to restore it potentially, in a revert.
			if(item[this._reverseRefMap]){
				item["backup_" + this._reverseRefMap] = lang.clone(item[this._reverseRefMap]);
			}

			//TODO:  This causes a reversion problem.  This list won't be restored on revert since it is
			//attached to the 'value'. item, not ours.  Need to back tese up somehow too.
			//Maybe build a map of the backup of the entries and attach it to the deleted item to be restored
			//later.  Or just record them and call _addReferenceToMap on them in revert.
			arrayUtil.forEach(attributes, function(attribute){
				arrayUtil.forEach(this.getValues(item, attribute), function(value){
					if(this.isItem(value)){
						//We have to back up all the references we had to others so they can be restored on a revert.
						if(!item["backupRefs_" + this._reverseRefMap]){
							item["backupRefs_" + this._reverseRefMap] = [];
						}
						item["backupRefs_" + this._reverseRefMap].push({id: this.getIdentity(value), attr: attribute});
						this._removeReferenceFromMap(value, item, attribute);
					}
				}, this);
			}, this);

			//Next, see if we have references to this item, if we do, we have to clean them up too.
			var references = item[this._reverseRefMap];
			if(references){
				//Look through all the items noted as references to clean them up.
				for(var itemId in references){
					var containingItem = null;
					if(this._itemsByIdentity){
						containingItem = this._itemsByIdentity[itemId];
					}else{
						containingItem = this._arrayOfAllItems[itemId];
					}
					//We have a reference to a containing item, now we have to process the
					//attributes and clear all references to the item being deleted.
					if(containingItem){
						for(var attribute in references[itemId]){
							var oldValues = this.getValues(containingItem, attribute) || [];
							var newValues = arrayUtil.filter(oldValues, function(possibleItem){
								return !(this.isItem(possibleItem) && this.getIdentity(possibleItem) == identity);
							}, this);
							//Remove the note of the reference to the item and set the values on the modified attribute.
							this._removeReferenceFromMap(item, containingItem, attribute);
							if(newValues.length < oldValues.length){
								this._setValueOrValues(containingItem, attribute, newValues, true);
							}
						}
					}
				}
			}
		}

		this._arrayOfAllItems[indexInArrayOfAllItems] = null;

		item[this._storeRefPropName] = null;
		if(this._itemsByIdentity){
			delete this._itemsByIdentity[identity];
		}
		this._pending._deletedItems[identity] = item;

		//Remove from the toplevel items, if necessary...
		if(item[this._rootItemPropName]){
			this._removeArrayElement(this._arrayOfTopLevelItems, item);
		}
		this.onDelete(item); // dojo/data/api/Notification call
		return true;
	},

	setValue: function(/* dojo/data/api/Item */ item, /* attribute-name-string */ attribute, /* almost anything */ value){
		// summary:
		//		See dojo/data/api/Write.set()
		return this._setValueOrValues(item, attribute, value, true); // boolean
	},

	setValues: function(/* dojo/data/api/Item */ item, /* attribute-name-string */ attribute, /* array */ values){
		// summary:
		//		See dojo/data/api/Write.setValues()
		return this._setValueOrValues(item, attribute, values, true); // boolean
	},

	unsetAttribute: function(/* dojo/data/api/Item */ item, /* attribute-name-string */ attribute){
		// summary:
		//		See dojo/data/api/Write.unsetAttribute()
		return this._setValueOrValues(item, attribute, [], true);
	},

	_setValueOrValues: function(/* dojo/data/api/Item */ item, /* attribute-name-string */ attribute, /* anything */ newValueOrValues, /*boolean?*/ callOnSet){
		this._assert(!this._saveInProgress);

		// Check for valid arguments
		this._assertIsItem(item);
		this._assert(lang.isString(attribute));
		this._assert(typeof newValueOrValues !== "undefined");

		// Make sure the user isn't trying to change the item's identity
		var identifierAttribute = this._getIdentifierAttribute();
		if(attribute == identifierAttribute){
			throw new Error("ItemFileWriteStore does not have support for changing the value of an item's identifier.");
		}

		// To implement the Notification API, we need to make a note of what
		// the old attribute value was, so that we can pass that info when
		// we call the onSet method.
		var oldValueOrValues = this._getValueOrValues(item, attribute);

		var identity = this.getIdentity(item);
		if(!this._pending._modifiedItems[identity]){
			// Before we actually change the item, we make a copy of it to
			// record the original state, so that we'll be able to revert if
			// the revert method gets called.  If the item has already been
			// modified then there's no need to do this now, since we already
			// have a record of the original state.
			var copyOfItemState = {};
			for(var key in item){
				if((key === this._storeRefPropName) || (key === this._itemNumPropName) || (key === this._rootItemPropName)){
					copyOfItemState[key] = item[key];
				}else if(key === this._reverseRefMap){
					copyOfItemState[key] = lang.clone(item[key]);
				}else{
					copyOfItemState[key] = item[key].slice(0, item[key].length);
				}
			}
			// Now mark the item as dirty, and save the copy of the original state
			this._pending._modifiedItems[identity] = copyOfItemState;
		}

		// Okay, now we can actually change this attribute on the item
		var success = false;

		if(lang.isArray(newValueOrValues) && newValueOrValues.length === 0){

			// If we were passed an empty array as the value, that counts
			// as "unsetting" the attribute, so we need to remove this
			// attribute from the item.
			success = delete item[attribute];
			newValueOrValues = undefined; // used in the onSet Notification call below

			if(this.referenceIntegrity && oldValueOrValues){
				var oldValues = oldValueOrValues;
				if(!lang.isArray(oldValues)){
					oldValues = [oldValues];
				}
				for(var i = 0; i < oldValues.length; i++){
					var value = oldValues[i];
					if(this.isItem(value)){
						this._removeReferenceFromMap(value, item, attribute);
					}
				}
			}
		}else{
			var newValueArray;
			if(lang.isArray(newValueOrValues)){
				// Unfortunately, it's not safe to just do this:
				//	  newValueArray = newValueOrValues;
				// Instead, we need to copy the array, which slice() does very nicely.
				// This is so that our internal data structure won't
				// get corrupted if the user mucks with the values array *after*
				// calling setValues().
				newValueArray = newValueOrValues.slice(0, newValueOrValues.length);
			}else{
				newValueArray = [newValueOrValues];
			}

			//We need to handle reference integrity if this is on.
			//In the case of set, we need to see if references were added or removed
			//and update the reference tracking map accordingly.
			if(this.referenceIntegrity){
				if(oldValueOrValues){
					var oldValues = oldValueOrValues;
					if(!lang.isArray(oldValues)){
						oldValues = [oldValues];
					}
					//Use an associative map to determine what was added/removed from the list.
					//Should be O(n) performant.  First look at all the old values and make a list of them
					//Then for any item not in the old list, we add it.  If it was already present, we remove it.
					//Then we pass over the map and any references left it it need to be removed (IE, no match in
					//the new values list).
					var map = {};
					arrayUtil.forEach(oldValues, function(possibleItem){
						if(this.isItem(possibleItem)){
							var id = this.getIdentity(possibleItem);
							map[id.toString()] = true;
						}
					}, this);
					arrayUtil.forEach(newValueArray, function(possibleItem){
						if(this.isItem(possibleItem)){
							var id = this.getIdentity(possibleItem);
							if(map[id.toString()]){
								delete map[id.toString()];
							}else{
								this._addReferenceToMap(possibleItem, item, attribute);
							}
						}
					}, this);
					for(var rId in map){
						var removedItem;
						if(this._itemsByIdentity){
							removedItem = this._itemsByIdentity[rId];
						}else{
							removedItem = this._arrayOfAllItems[rId];
						}
						this._removeReferenceFromMap(removedItem, item, attribute);
					}
				}else{
					//Everything is new (no old values) so we have to just
					//insert all the references, if any.
					for(var i = 0; i < newValueArray.length; i++){
						var value = newValueArray[i];
						if(this.isItem(value)){
							this._addReferenceToMap(value, item, attribute);
						}
					}
				}
			}
			item[attribute] = newValueArray;
			success = true;
		}

		// Now we make the dojo/data/api/Notification call
		if(callOnSet){
			this.onSet(item, attribute, oldValueOrValues, newValueOrValues);
		}
		return success; // boolean
	},

	_addReferenceToMap: function(/* dojo/data/api/Item */ refItem, /* dojo/data/api/Item */ parentItem, /* string */ attribute){
		// summary:
		//		Method to add an reference map entry for an item and attribute.
		// description:
		//		Method to add an reference map entry for an item and attribute.
		// refItem:
		//		The item that is referenced.
		// parentItem:
		//		The item that holds the new reference to refItem.
		// attribute:
		//		The attribute on parentItem that contains the new reference.

		var parentId = this.getIdentity(parentItem);
		var references = refItem[this._reverseRefMap];

		if(!references){
			references = refItem[this._reverseRefMap] = {};
		}
		var itemRef = references[parentId];
		if(!itemRef){
			itemRef = references[parentId] = {};
		}
		itemRef[attribute] = true;
	},

	_removeReferenceFromMap: function(/* dojo/data/api/Item */ refItem, /* dojo/data/api/Item */ parentItem, /* string */ attribute){
		// summary:
		//		Method to remove an reference map entry for an item and attribute.
		// description:
		//		Method to remove an reference map entry for an item and attribute.  This will
		//		also perform cleanup on the map such that if there are no more references at all to
		//		the item, its reference object and entry are removed.
		// refItem:
		//		The item that is referenced.
		// parentItem:
		//		The item holding a reference to refItem.
		// attribute:
		//		The attribute on parentItem that contains the reference.
		var identity = this.getIdentity(parentItem);
		var references = refItem[this._reverseRefMap];
		var itemId;
		if(references){
			for(itemId in references){
				if(itemId == identity){
					delete references[itemId][attribute];
					if(this._isEmpty(references[itemId])){
						delete references[itemId];
					}
				}
			}
			if(this._isEmpty(references)){
				delete refItem[this._reverseRefMap];
			}
		}
	},

	_dumpReferenceMap: function(){
		// summary:
		//		Function to dump the reverse reference map of all items in the store for debug purposes.
		// description:
		//		Function to dump the reverse reference map of all items in the store for debug purposes.
		var i;
		for(i = 0; i < this._arrayOfAllItems.length; i++){
			var item = this._arrayOfAllItems[i];
			if(item && item[this._reverseRefMap]){
				console.log("Item: [" + this.getIdentity(item) + "] is referenced by: " + jsonUtil.toJson(item[this._reverseRefMap]));
			}
		}
	},

	_getValueOrValues: function(/* dojo/data/api/Item */ item, /* attribute-name-string */ attribute){
		var valueOrValues = undefined;
		if(this.hasAttribute(item, attribute)){
			var valueArray = this.getValues(item, attribute);
			if(valueArray.length == 1){
				valueOrValues = valueArray[0];
			}else{
				valueOrValues = valueArray;
			}
		}
		return valueOrValues;
	},

	_flatten: function(/* anything */ value){
		if(this.isItem(value)){
			// Given an item, return an serializable object that provides a
			// reference to the item.
			// For example, given kermit:
			//	  var kermit = store.newItem({id:2, name:"Kermit"});
			// we want to return
			//	  {_reference:2}
			return {_reference: this.getIdentity(value)};
		}else{
			if(typeof value === "object"){
				for(var type in this._datatypeMap){
					var typeMap = this._datatypeMap[type];
					if(lang.isObject(typeMap) && !lang.isFunction(typeMap)){
						if(value instanceof typeMap.type){
							if(!typeMap.serialize){
								throw new Error("ItemFileWriteStore:  No serializer defined for type mapping: [" + type + "]");
							}
							return {_type: type, _value: typeMap.serialize(value)};
						}
					}else if(value instanceof typeMap){
						//SImple mapping, therefore, return as a toString serialization.
						return {_type: type, _value: value.toString()};
					}
				}
			}
			return value;
		}
	},

	_getNewFileContentString: function(){
		// summary:
		//		Generate a string that can be saved to a file.
		//		The result should look similar to:
		//		http://trac.dojotoolkit.org/browser/dojo/trunk/tests/data/countries.json
		var serializableStructure = {};

		var identifierAttribute = this._getIdentifierAttribute();
		if(identifierAttribute !== Number){
			serializableStructure.identifier = identifierAttribute;
		}
		if(this._labelAttr){
			serializableStructure.label = this._labelAttr;
		}
		serializableStructure.items = [];
		for(var i = 0; i < this._arrayOfAllItems.length; ++i){
			var item = this._arrayOfAllItems[i];
			if(item !== null){
				var serializableItem = {};
				for(var key in item){
					if(key !== this._storeRefPropName && key !== this._itemNumPropName && key !== this._reverseRefMap && key !== this._rootItemPropName){
						var valueArray = this.getValues(item, key);
						if(valueArray.length == 1){
							serializableItem[key] = this._flatten(valueArray[0]);
						}else{
							var serializableArray = [];
							for(var j = 0; j < valueArray.length; ++j){
								serializableArray.push(this._flatten(valueArray[j]));
								serializableItem[key] = serializableArray;
							}
						}
					}
				}
				serializableStructure.items.push(serializableItem);
			}
		}
		var prettyPrint = true;
		return jsonUtil.toJson(serializableStructure, prettyPrint);
	},

	_isEmpty: function(something){
		// summary:
		//		Function to determine if an array or object has no properties or values.
		// something:
		//		The array or object to examine.
		var empty = true;
		if(lang.isObject(something)){
			var i;
			for(i in something){
				empty = false;
				break;
			}
		}else if(lang.isArray(something)){
			if(something.length > 0){
				empty = false;
			}
		}
		return empty; //boolean
	},

	save: function(/* object */ keywordArgs){
		// summary:
		//		See dojo/data/api/Write.save()
		this._assert(!this._saveInProgress);

		// this._saveInProgress is set to true, briefly, from when save is first called to when it completes
		this._saveInProgress = true;

		var self = this;
		var saveCompleteCallback = function(){
			self._pending = {
				_newItems:{},
				_modifiedItems:{},
				_deletedItems:{}
			};

			self._saveInProgress = false; // must come after this._pending is cleared, but before any callbacks
			if(keywordArgs && keywordArgs.onComplete){
				var scope = keywordArgs.scope || kernel.global;
				keywordArgs.onComplete.call(scope);
			}
		};
		var saveFailedCallback = function(err){
			self._saveInProgress = false;
			if(keywordArgs && keywordArgs.onError){
				var scope = keywordArgs.scope || kernel.global;
				keywordArgs.onError.call(scope, err);
			}
		};

		if(this._saveEverything){
			var newFileContentString = this._getNewFileContentString();
			this._saveEverything(saveCompleteCallback, saveFailedCallback, newFileContentString);
		}
		if(this._saveCustom){
			this._saveCustom(saveCompleteCallback, saveFailedCallback);
		}
		if(!this._saveEverything && !this._saveCustom){
			// Looks like there is no user-defined save-handler function.
			// That's fine, it just means the datastore is acting as a "mock-write"
			// store -- changes get saved in memory but don't get saved to disk.
			saveCompleteCallback();
		}
	},

	revert: function(){
		// summary:
		//		See dojo/data/api/Write.revert()
		this._assert(!this._saveInProgress);

		var identity;
		for(identity in this._pending._modifiedItems){
			// find the original item and the modified item that replaced it
			var copyOfItemState = this._pending._modifiedItems[identity];
			var modifiedItem = null;
			if(this._itemsByIdentity){
				modifiedItem = this._itemsByIdentity[identity];
			}else{
				modifiedItem = this._arrayOfAllItems[identity];
			}

			// Restore the original item into a full-fledged item again, we want to try to
			// keep the same object instance as if we don't it, causes bugs like #9022.
			copyOfItemState[this._storeRefPropName] = this;
			for(var key in modifiedItem){
				delete modifiedItem[key];
			}
			lang.mixin(modifiedItem, copyOfItemState);
		}
		var deletedItem;
		for(identity in this._pending._deletedItems){
			deletedItem = this._pending._deletedItems[identity];
			deletedItem[this._storeRefPropName] = this;
			var index = deletedItem[this._itemNumPropName];

			//Restore the reverse refererence map, if any.
			if(deletedItem["backup_" + this._reverseRefMap]){
				deletedItem[this._reverseRefMap] = deletedItem["backup_" + this._reverseRefMap];
				delete deletedItem["backup_" + this._reverseRefMap];
			}
			this._arrayOfAllItems[index] = deletedItem;
			if(this._itemsByIdentity){
				this._itemsByIdentity[identity] = deletedItem;
			}
			if(deletedItem[this._rootItemPropName]){
				this._arrayOfTopLevelItems.push(deletedItem);
			}
		}
		//We have to pass through it again and restore the reference maps after all the
		//undeletes have occurred.
		for(identity in this._pending._deletedItems){
			deletedItem = this._pending._deletedItems[identity];
			if(deletedItem["backupRefs_" + this._reverseRefMap]){
				arrayUtil.forEach(deletedItem["backupRefs_" + this._reverseRefMap], function(reference){
					var refItem;
					if(this._itemsByIdentity){
						refItem = this._itemsByIdentity[reference.id];
					}else{
						refItem = this._arrayOfAllItems[reference.id];
					}
					this._addReferenceToMap(refItem, deletedItem, reference.attr);
				}, this);
				delete deletedItem["backupRefs_" + this._reverseRefMap];
			}
		}

		for(identity in this._pending._newItems){
			var newItem = this._pending._newItems[identity];
			newItem[this._storeRefPropName] = null;
			// null out the new item, but don't change the array index so
			// so we can keep using _arrayOfAllItems.length.
			this._arrayOfAllItems[newItem[this._itemNumPropName]] = null;
			if(newItem[this._rootItemPropName]){
				this._removeArrayElement(this._arrayOfTopLevelItems, newItem);
			}
			if(this._itemsByIdentity){
				delete this._itemsByIdentity[identity];
			}
		}

		this._pending = {
			_newItems:{},
			_modifiedItems:{},
			_deletedItems:{}
		};
		return true; // boolean
	},

	isDirty: function(/* item? */ item){
		// summary:
		//		See dojo/data/api/Write.isDirty()
		if(item){
			// return true if the item is dirty
			var identity = this.getIdentity(item);
			return new Boolean(this._pending._newItems[identity] ||
				this._pending._modifiedItems[identity] ||
				this._pending._deletedItems[identity]).valueOf(); // boolean
		}else{
			// return true if the store is dirty -- which means return true
			// if there are any new items, dirty items, or modified items
			return !this._isEmpty(this._pending._newItems) ||
				!this._isEmpty(this._pending._modifiedItems) ||
				!this._isEmpty(this._pending._deletedItems); // boolean
		}
	},

/* dojo/data/api/Notification */

	onSet: function(/* dojo/data/api/Item */ item,
					/*attribute-name-string*/ attribute,
					/*object|array*/ oldValue,
					/*object|array*/ newValue){
		// summary:
		//		See dojo/data/api/Notification.onSet()

		// No need to do anything. This method is here just so that the
		// client code can connect observers to it.
	},

	onNew: function(/* dojo/data/api/Item */ newItem, /*object?*/ parentInfo){
		// summary:
		//		See dojo/data/api/Notification.onNew()

		// No need to do anything. This method is here just so that the
		// client code can connect observers to it.
	},

	onDelete: function(/* dojo/data/api/Item */ deletedItem){
		// summary:
		//		See dojo/data/api/Notification.onDelete()

		// No need to do anything. This method is here just so that the
		// client code can connect observers to it.
	},

	close: function(/* object? */ request){
		 // summary:
		 //		Over-ride of base close function of ItemFileReadStore to add in check for store state.
		 // description:
		 //		Over-ride of base close function of ItemFileReadStore to add in check for store state.
		 //		If the store is still dirty (unsaved changes), then an error will be thrown instead of
		 //		clearing the internal state for reload from the url.

		 //Clear if not dirty ... or throw an error
		 if(this.clearOnClose){
			 if(!this.isDirty()){
				 this.inherited(arguments);
			 }else{
				 //Only throw an error if the store was dirty and we were loading from a url (cannot reload from url until state is saved).
				 throw new Error("dojo.data.ItemFileWriteStore: There are unsaved changes present in the store.  Please save or revert the changes before invoking close.");
			 }
		 }
	}
});

});

},
'dijit/_base':function(){
define([
	"./main",
	"./a11y",	// used to be in dijit/_base/manager
	"./WidgetSet",	// used to be in dijit/_base/manager
	"./_base/focus",
	"./_base/manager",
	"./_base/place",
	"./_base/popup",
	"./_base/scroll",
	"./_base/sniff",
	"./_base/typematic",
	"./_base/wai",
	"./_base/window"
], function(dijit){

	// module:
	//		dijit/_base

	/*=====
	return {
		// summary:
		//		Includes all the modules in dijit/_base
	};
	=====*/

	return dijit._base;
});

},
'xcitec/MultiSelectDialog':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dojo/text!xcitec/templates/MultiSelectDialog.html",
	"xcitec/xcitecDialog"
], function (dojo, array, declare, domAttr, domConstruct, query, template, xcitecDialog) {

	/* Component for displaying multi select dialog */
	var MultiSelectDialog = declare( xcitecDialog, {
	
	  templateString: template,
	
	  baseClass: "multiselectDialog",
	
	  postMixInProperties: function() {
	    // search for controls marked with xcitecRole attribute
	    this.xcitecRoles = query(" > [xcitecRole]", this.srcNodeRef);
	    this.inherited(arguments);
	  },
	
	  _fillContent: function() {
	    var _this = this;
	
	    // move controls in appropriate places if they exist
	    array.forEach(_this.xcitecRoles, function(node) {
	      switch (domAttr.get(node, "xcitecRole").toLowerCase()) {
	        case "ms_description":
	          domConstruct.place(node, _this.ms_description, "replace");
	          break;
	        case "ms_filterbox":
	          domConstruct.place(node, _this.ms_filterbox, "replace");
	          break;
	        case "ms_buttons":
	          _this.placeButtons(node);
	          break;
	        case "ms_searchbox":
	          domConstruct.place(node, _this.ms_searchbox, "replace");
	          break;
	        case "ms_selectbox":
	          domConstruct.place(node, _this.ms_selectbox, "replace");
	          break;
	        default:
	          break;
	      }
	    });
	    this.inherited(arguments);
	  },
	
	  placeButtons: function(node) {
	    var buttons = query(">", node);
	    var places = [this.select_all, this.select_with_children, this.select, this.deselect, this.deselect_all]
	    // move buttons in given order to appropriate place
	    array.forEach(buttons, function(button, index) {
	      domConstruct.place(button, places[index], "replace");
	    });
	  }
	});
	return MultiSelectDialog;
});
},
'xcitec/form/EntitySelect':function(){
define([
    "dojo",
    "dojo/_base/declare",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/_base/window",
    "dojo/dom",
	'dojo/html',
    "dojo/text!./templates/EntitySelect.html",
    "dojo/topic",
    "dojo/keys",
    "./TextBox",
    "../xcitecUtil",
    "../_base/ImageFactory",
    "./_XButtonRight",
    "./_XButtonMiddle",
    "./_XButtonSingleRight"
], function (dojo, declare, domClass, domStyle, windowModule, dom, html, template, topic, keys, TextBox, xcitecUtil, ImageFactory) {

	var EntitySelect = declare( TextBox,	{
	  iconClass: "",
	  iconClassTest: "",
	  entityClass: "",
	  infoIconClass: "",
	  frontIconClass: "",
	  bgColor: '',
	  _infoButtonActive: false,
	
	  // template attributes
	  disabledChooseAttrSetting: "",
	  disabledInfoAttrSetting: "",
	  readOnlyInfoAttrSetting: "",
	  xbtnInfoClass: "xcitec/form/_XButtonMiddle",
	  widgetsInTemplate: true,
	  templateString: template,
	
			
	  postMixInProperties: function() {
	    this.inherited(arguments);
	    this._infoButtonActive = ("onInfo" in this.params);
	    if (this.entityClass) {
	      this.infoIconClass = "iconXBtn "+ xcitecUtil.createEntityIconClass(this.entityClass);
	    }
	    else {
	      this.infoIconClass = "iconXBtn";
	    }
	    if (this.disabled) {
	      this.required = false;
	      this.disabledChooseAttrSetting = "disabled";
	      this.disabledInfoAttrSetting = "disabled";
	    }
	    if (!this._infoButtonActive) {
	      this.readOnlyInfoAttrSetting = "readOnly='true'";
	    }
	    if (this.readOnly) {
	      this.required = false;
	      this.xbtnInfoClass = "xcitec/form/_XButtonSingleRight";
	      if (!this._infoButtonActive) this.disabledInfoAttrSetting = "disabled";
	      this.disabledChooseAttrSetting = "disabled";
	    }
	    if (this.iconClass) {
	      this._iconClassDef=this.iconClass;
	      this.iconClass = ImageFactory.createIconCss(this.iconClass);
	    }                 
	    if (this.frontIconClass) {
	      this.frontIconClass = ImageFactory.createIconCss(this.frontIconClass);
	    }
	  },
		  
	  postCreate: function() {
	    this.inherited(arguments);
	
	    if ("onDelete" in this.params) {
	      domClass.add(this.domNode, "closeEntitySelect");
	    }
	    if (!this.iconClass) {
	      domStyle.set(this.imageNode, "display", "none");
	    }
	    if (!this.frontIconClass) {
	      domStyle.set(this.frontIconNode, "display", "none");
	    }
	    if (this.textbox.value) {
	      this._createTextNode();
	    }
	    if (this.required) {
	      domClass.add(this.domNode, "dijitXcitecRequired");
	    }
	    if (this.bgColor) {
	      domClass.add(this.domNode, "entitySelect" + this.bgColor.charAt(0).toUpperCase() + this.bgColor.slice(1));
	    }
      	dom.setSelectable(this.domNode, false);
	//    if (this.readOnly) {
	//      dojo.style(this.chooseButton.domNode, "display", "none");
	//    }
				//	this.connect(this.clear, "onkeydown", this._onInput);
	    this.connect(this.domNode, "onkeydown", this._onInput1);
	    this.connect(this.domNode, "onkeyup", this._onInput2);
	  },
	  
	  _createTextNode: function(){
        this.valueNode.value = this.textbox.value;
        this.textNode.appendChild(windowModule.doc.createTextNode(this.textbox.value));
      },
		
	  _onClickInfoButton: function(){
	    this.onInfo();
	  },
	
	  _onClickChooseButton: function(){
	    this.onChoose();
	  },
		
	  onFirstClick: function(){
	      //have to be ovveriden
	  },
	
	  onSecondClick: function(){
	      //have to be ovveriden
	  },
	
	  onInfo: function(){
	  //have to be ovveriden
	      this.onFirstClick();
	  },
	
	  onChoose: function(){
	  //have to be ovveriden
	    this.onSecondClick();
	  },
	
	  onDelete: function() {
	
	  },
	
	  validate: function() {
	        // overrides dijit.form.ValidationTextBox validate function
	  },
	
	  _onInput1: function(e) {
	      if ((e.keyCode == keys.DELETE) || (e.keyCode == keys.BACKSPACE)) {
	          this.onClear();
	          e.preventDefault(), e.stopPropagation();
	      }
	  },
	
	  _onInput2: function(e) {
	      if ((e.keyCode == keys.DELETE) || (e.keyCode == keys.BACKSPACE)) {
	          e.preventDefault(), e.stopPropagation();
	      }
	  },
	
	  onClear: function() {
	      //have to be ovveriden
	  }
	});
	return EntitySelect;
});

},
'dijit/form/_FormValueWidget':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("ie")
	"./_FormWidget",
	"./_FormValueMixin"
], function(declare, has, _FormWidget, _FormValueMixin){

	// module:
	//		dijit/form/_FormValueWidget

	return declare("dijit.form._FormValueWidget", [_FormWidget, _FormValueMixin], {
		// summary:
		//		Base class for widgets corresponding to native HTML elements such as `<input>` or `<select>`
		//		that have user changeable values.
		// description:
		//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) `<input>` element,
		//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
		//		works as expected.

		// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
		// directly in the template as read by the parser in order to function. IE is known to specifically
		// require the 'name' attribute at element creation time.  See #8484, #8660.

		_layoutHackIE7: function(){
			// summary:
			//		Work around table sizing bugs on IE7 by forcing redraw

			if(has("ie") == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
				var domNode = this.domNode;
				var parent = domNode.parentNode;
				var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
				var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
				var _this = this;
				while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
					(function ping(){
						var disconnectHandle = _this.connect(parent, "onscroll",
							function(){
								_this.disconnect(disconnectHandle); // only call once
								pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
								_this.defer(function(){
									pingNode.style.filter = origFilter;
								}); // restore custom filter, if any
							}
						);
					})();
					parent = parent.parentNode;
				}
			}
		}
	});
});

},
'xcitec/formx/EVACriterion':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom",
	"dojo/fx",
	"dojo/query",
	"dojo/text!xcitec/formx/templates/EVACriterion.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "../xcitecUtil",
  "../_base/ImageFactory"
], function (dojo, domStyle, array, declare, lang, windowModule, domAttr, domClass, domConstruct, dom, fx, query, template, _Templated, _Widget, registry, xcitecUtil, ImageFactory) {

	/*
	 EVACriterion is a layout control which keeps inside it other controls (including EVACriterion
	 itself), places them in appropriate places and could be expanded/collapsed.
	 Inner controls must have appropriate xcitec roles to be placed in their placeholders.
	 These roles are: 'weight', 'rating', 'ratinglabel', 'description', 'comment', 'na', 'evacriterion'
	 <div data-dojo-type="xcitec.formx.EVACriterion" caption="Monopoly position" state="Close>
	 <div xcitecRole="weight" data-dojo-type="xcitec.Text" icon="WEIGHT:criterion">20%</div>
	 <div xcitecRole="ratinglabel" data-dojo-type="xcitec.Label">high (60%)</div>
	 <div xcitecRole="rating" data-dojo-type="xcitec.Label">1 evaluator</div>
	 <div xcitecRole="description" data-dojo-type="xcitec.layout.Chapter">
	 ....
	 </div>
	 <div xcitecRole="comment" data-dojo-type="xcitec.layout.Chapter">
	 ....
	 </div>
	 </div>
	
	Roles:
	
	 evastateholder - input field state holder to submit state open/close
	
	Header part roles:
	 weight      - Weights info - icon+number - headline center
	 ratinglabel - label for rating result 
	 rating      - Info line - headline right
	 evaluation  - component identification of evaluation for dependency 
	 
	
	Body roles:
	 description  - body-left content (usual question description)
	 comment      - body-right content (usual input component for answering)
	 combofield   - body-left content2 - additional info about question (other answers)
	 evacriterion - body content - collection of nested eva criteria/chapters/evacriterion
	
	         |---------------|--------|-------------------------|
	 head -> | title         |[weight]| [ratinglabel]|[rating]  |
	   or    |---------------|----|---|-------------------------|
	 head -> | [evaluation]       |     [ratinglabel]|[rating]  |
	         |--------------------|-----------------------------|
	 body -> | [description]      |           [comment]         |
	         | [combofield]       |                             |
	         |--------------------|-----------------------------|
	         | [evacriterion]                                   |
	         |--------------------------------------------------|
	
	Chapter view:
	         |---------------|--------|-------------------------|
	 head -> | title         |[weight]| [ratinglabel]|[rating]  |
	         |---------------|----|---|-------------------------|
	 body -> | [description]      |                             |
	         |--------------------|-----------------------------|
	         | [evacriterion]                                   |
	         |--------------------------------------------------|
	
	Criterion view:
	         |---------------|--------|-------------------------|
	 head -> | title         |[weight]| [ratinglabel]|[rating]  |
	         |---------------|----|---|-------------------------|
	 body -> | [description]      |           [comment]         |
	         | [combofield]       |                             |
	         |--------------------|-----------------------------|
	
	Criterion with dependency view: (dependency="true")
	         |---------------|--------|-------------------------|
	 head -> | title         |[weight]| [ratinglabel]|[rating]  |
	         |---------------|----|---|-------------------------|
	 body -> | [description]      |                             |
	         |--------------------|-----------------------------|
	         | [evacriterion]                                   |
	         |--------------------------------------------------|
	
	Dependency view: (dependencyChapter="true")
	         |--------------------|-----------------------------|
	 head -> | [evaluation]       |     [ratinglabel]|[rating]  |
	         |--------------------|-----------------------------|
	 body -> | [evacriterion]                                   |
	         |--------------------------------------------------|
	
	 */
	
	var EVACriterion = declare( [_Widget, _Templated], {
	    templateString: template,
	
	    baseClass: "evacriterion",
	    _expandoOpenClass: "evacriterionHeaderOpen",
	
	    caption: "",
	    captionId: "",
	
	    xcitecRoles: null, // Array
	
	    // EVACriterion states: "Open" or "Close", "Open" by default
	    state: "Open",
	
	    iconClass: "",
	
	    isRatingExists: false,
	
	    isSingleColumn: false,
	
	    // nesting level
	    level: 0,
	
	    // error state for EVACriterion
	    error: false,
	
	    evaType: "criterion",
	
	    // is comment icon exists
	    commentExists: false,
	
	    // is attachment icon exists
	    attachmentExists: false,
	
	    isInDependency: false,
	    
	    isBodyNodeExists : false,
	    
	    // dependency
	    dependency: false,
	    dependencyChapter: false,
	
	    // Override _Widget's attributeMap
	    // todo replace in dojo 1.7 with _setXXXAttr()
	    // _setCaptionAttr: { node: "captionNode", type: "innerText" },
	//    attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
	//        caption: { node: "captionNode", type: "innerText" }
	//    }),
	
	    postMixInProperties: function() {
	        // search for controls marked with xcitecRole attribute
	        this.xcitecRoles = query(" > [xcitecRole]", this.srcNodeRef);
	        // set iconClass
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        }
	        this.inherited(arguments);
	    },
	
	    _fillContent: function() {
	        var _this = this;
	
	        //todo with 1.7 replace to attribute-mapping
	        this.captionNode.appendChild(windowModule.doc.createTextNode(this.caption));
	
	        // move controls in appropriate places if they exist
	        array.forEach(_this.xcitecRoles, function(node) {
	            switch (domAttr.get(node, "xcitecRole").toLowerCase()) {
	                case "evacriterion":
	                    domConstruct.place(node, _this.evacriterion, "last");
	                    domAttr.set(node, "level", _this.level + 1);
	                    _this.isInnerComboExists = true;
	                    _this.evaType = "chapter";
	                    if (_this.isInDependency || _this.dependency || _this.dependencyChapter) {
	                        domAttr.set(node, "isInDependency", "true");
	                        domAttr.set(node, "level", _this.level); //all nested items inherit current level
	                    }
	                    break;
	                case "evastateholder":
	                    _this.stateHolder = node;
	                    domConstruct.place(_this.stateHolder, _this.domNode, "first");
	                    var state = xcitecUtil.str2bool(domAttr.get(_this.stateHolder, "value"));
	                    state ? _this.state = "Open" : _this.state = "Close";
	                    break;
	                case "combofield":
	                    domConstruct.place(node, _this.combofieldNode, "last");
	                    break;
	                case "weight":
	                    domConstruct.place(node, _this.weight, "last");
	                    break;
	                case "evaluation":
	                    domConstruct.place(node, _this.evaluation, "last");
	                    domAttr.set(node, "isInDependency", "true");
	                    break;
	                case "rating":
	                    domConstruct.place(node, _this.rating, "last");
	                    domClass.remove(_this.rating, "evacriterionLine");
	                    break;
	                case "ratinglabel":
	                    domConstruct.place(node, _this.ratinglabel, "last");
	                    break;
	                case "description":
	                    domConstruct.place(node, _this.description, "last"); 
	                    _this.isBodyNodeExists = true;
	                    break;
	                case "comment":
	                    domConstruct.place(node, _this.comment, "last");
	                    _this.isBodyNodeExists = true;
	                    break;
	                default:
	                    break;
	            }
	        });           
	
	        if (!xcitecUtil.str2bool(this.commentExists)) {
	            domConstruct.destroy(this.commentIcon);
	        }
	
	        if (!xcitecUtil.str2bool(this.attachmentExists)) {
	            domConstruct.destroy(this.attachmentIcon);
	        }
	
	        if (this.evaType == "chapter") {
	            this.setBoldCaption();
	        }
	
	        //var factor = 7 * (this.level) + (this.isInDependency?5:0);
	        var factor = 7 * (this.level);
	        domStyle.set(this.headerRightcolWrapperNode, "paddingLeft", factor + "px");
	        domStyle.set(this.headerRightcolNode, "marginLeft", -1 * factor + "px");
	        domStyle.set(this.bodyRightcolWrapperNode, "paddingLeft", factor + "px");
	        domStyle.set(this.bodyRightcolNode, "marginLeft", -1 * factor + "px");
	        
	        // classes for this.containerNode
	        var containerNodeClasses = "";
	        if (this.hasInnerCombo()) {
	        }  else  {
	            containerNodeClasses += " evacriterionContainer";
	            domConstruct.destroy(this.evacriterion);
	            domClass.remove(this.lineNode1, "evacriterionLine");
	            domClass.remove(this.lineNode2, "evacriterionLine");
	            domClass.remove(this.rating, "evacriterionLine");
	        }
	        
	        if (!this.isBodyNodeExists) {
	            domConstruct.destroy(this.bodyNode);
	        }
	            
	        if (this.evaType == "scorecard") {
	            this.setBoldCaption();
	            domConstruct.destroy(this.icon);
	            containerNodeClasses = "";
	        }
	
	        if (!this.iconClass) {
	            domConstruct.destroy(this.statusIcon);
	        }
	
	        if (this.error) {
	            containerNodeClasses += " evacriterionContainerError";
	        }
	        if (this.dependency) {
	            containerNodeClasses += " evacriterionContainer";
	        }
	        if (this.dependencyChapter) {
	            containerNodeClasses += " evacriterionDependency";
	        }
	
	        if (this.dependency || this.dependencyChapter) {
	            domClass.remove(this.lineNode1, "evacriterionLine");
	            domClass.remove(this.lineNode2, "evacriterionLine");
	            domClass.remove(this.rating, "evacriterionLine");
	            domClass.remove(this.captionNode, "boldTitle");
	        }
	
	        if (containerNodeClasses.indexOf("evacriterionContainer") != -1) {
	            domStyle.set(this.containerNode, "marginBottom", "5px");
	        }
	        domClass.add(this.containerNode, lang.trim(containerNodeClasses));
	
	        this._updateExpandoState(true);
	        dom.setSelectable(this.comboHeader, false);
	        this.inherited(arguments);
	    },
	
	    _onClick: function() {
	        switch (this.state) {
	            case "Close":
	                this.expand();
	                break;
	            case "Open":
	                this.collapse();
	                break;
	            default:
	                break;
	        }
	    },
	
	    startup: function() {
	        if (this.dependencyChapter) {
	            var childrenNodes = query(">", this.evacriterion);
	            array.forEach(childrenNodes, function(node){
	                domClass.add(registry.byNode(node).comboHeader, "evacriterionDependencyInner");
	            });
	        }
	        this.inherited(arguments);
	    },
	
	    _updateExpandoState : function(firstTime){
	        var tn = this.comboHeader || this.domNode;
	        switch (this.state){
	            case "Close":
	                if(firstTime){
	                    //at first time we need have it visible but hidden - to calculate sizes inside.
	                    domStyle.set(this.comboBody, "height", "0px");
	                }
	                domClass.remove(tn, this._expandoOpenClass);
	                break;
	            case "Open":
	                domClass.add(tn, this._expandoOpenClass);
	                break;
	            default:
	                break;
	        }
	    },
	
	    expand: function() {
	        if (this.evaType == "scorecard") {
	            return;
	        }
	        this.state = "Open";
	        //dojo.style(this.comboBody, "display", "none");
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "true");
	        }
	        fx.wipeIn({ node: this.comboBody, beforeBegin: lang.hitch(this, "_updateExpandoState") }).play();
	    },
	
	    collapse: function() {
	        if (this.evaType == "scorecard") {
	            return;
	        }
	        this.state = "Close";
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "false");
	        }
	        fx.wipeOut({ node: this.comboBody, onEnd: lang.hitch(this, "_updateExpandoState") }).play();
	    },
	
	    hasInnerCombo: function() {
	        return this.isInnerComboExists;
	    }, 
	
	    setBoldCaption: function() {
	        domClass.add(this.captionNode, "boldTitle");
	    },
	
	    getState: function() {
	        return this.state;
	    }
	});
	return EVACriterion;
});

},
'xcitec/_base/FlagImageFamily':function(){
define([
         "dojo/_base/declare",
         "./ImageFamily"
       ], function (declare, ImageFamily) {
  return declare(ImageFamily, {
    familyId: "FLAG",

    createIconCss: function(/*String*/ iconId){
      return "iconFlag" + (!iconId ? "" : " iconFlag" + iconId.toUpperCase());
    }
  });
});

},
'xcitec/Text':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/dom",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "./_base/ImageFactory"
], function (declare, domStyle, dom, _Templated, _Widget, ImageFactory) {

	var Text = declare( [_Widget, _Templated], {
		text: "",
		textId: "",
	
		baseClass: "xcitecTextBox",
		iconClass: "",
		// todo: remove icon attribute in future because it will not be used in view anymore
		icon: "",
		color: "",
		disabled: false,
	
		templateString: '<div class="dijitReset ${baseClass}">' +
				'<img class="${iconClass}" src="${_blankGif}" width="1" height="1" alt="" data-dojo-attach-point="imageNode"/>' +
				'<span class="textContent" data-dojo-attach-point="textNode">${!text}</span>' +
				'</div>',
	
		postMixInProperties: function(){
			if(this.icon){
				this.iconClass = ImageFactory.createIconCss(this.icon);
			}
			if(this.iconClass.charAt(0) == "#"){
				this.iconClass = ImageFactory.createIconCss(this.iconClass.substring(1, this.iconClass.length));
			}
			if (this.disabled) {
	 		    this.baseClass += " xcitecTextBoxDisabled";
			}
			this.text = this.text.toString().replace(/</g, "&lt;").replace(/>/g, "&gt;");
			this.text = this.text.toString().replace(/\n/g, "<br>\n");
			this.inherited(arguments);
		},
	
		buildRendering: function(){
			var source = this.srcNodeRef;
			if(source){
				if(!("text" in this.params)){
					this.set('text', source.innerHTML);
				}
			}
			this.inherited(arguments);
		},
	
		postCreate: function(){
			this.inherited(arguments);
			if(this.color != ""){
				domStyle.set(this.domNode, "color", this.color);
			}
			dom.setSelectable(this.domNode, false);

			if(!this.text && !this.iconClass){
				domStyle.set(this.domNode, "display", "none");
			} else {
				if(!this.text){
					domStyle.set(this.textNode, "display", "none");
				}
				if(!this.iconClass){
					domStyle.set(this.imageNode, "display", "none");
				}
			}
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
	});
	
	return Text;
});

},
'xcitec/page/MainFrame':function(){
define([
	"dojo/_base/declare",
	"./MainFrameV2"
], function (declare, MainFrameV2) {

	return declare(MainFrameV2, {

	});
});
},
'dijit/tree/_dndContainer':function(){
define([
	"dojo/aspect", // aspect.after
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove domClass.replace
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/touch"
], function(aspect, declare, domClass, lang, on, touch){

	// module:
	//		dijit/tree/_dndContainer

	/*=====
	 var __Args = {
	 // summary:
	 //		A dict of parameters for Tree source configuration.
	 // isSource: Boolean?
	 //		Can be used as a DnD source. Defaults to true.
	 // accept: String[]
	 //		List of accepted types (text strings) for a target; defaults to
	 //		["text", "treeNode"]
	 // copyOnly: Boolean?
	 //		Copy items, if true, use a state of Ctrl key otherwise,
	 // dragThreshold: Number
	 //		The move delay in pixels before detecting a drag; 0 by default
	 // betweenThreshold: Integer
	 //		Distance from upper/lower edge of node to allow drop to reorder nodes
	 };
	 =====*/

	return declare("dijit.tree._dndContainer", null, {

		// summary:
		//		This is a base class for `dijit/tree/_dndSelector`, and isn't meant to be used directly.
		//		It's modeled after `dojo/dnd/Container`.
		// tags:
		//		protected

		/*=====
		 // current: TreeNode
		 //		The currently hovered TreeNode.  Not set to anything for keyboard operation.  (TODO: change?)
		 current: null,
		 =====*/

		constructor: function(tree, params){
			// summary:
			//		A constructor of the Container
			// tree: Node
			//		Node or node's id to build the container on
			// params: __Args
			//		A dict of parameters, which gets mixed into the object
			// tags:
			//		private
			this.tree = tree;
			this.node = tree.domNode;	// TODO: rename; it's not a TreeNode but the whole Tree
			lang.mixin(this, params);

			// states
			this.containerState = "";
			domClass.add(this.node, "dojoDndContainer");

			// set up events
			this.events = [
				// Mouse (or touch) enter/leave on Tree itself
				on(this.node, touch.enter, lang.hitch(this, "onOverEvent")),
				on(this.node, touch.leave, lang.hitch(this, "onOutEvent")),

				// switching between TreeNodes
				aspect.after(this.tree, "_onNodeMouseEnter", lang.hitch(this, "onMouseOver"), true),
				aspect.after(this.tree, "_onNodeMouseLeave", lang.hitch(this, "onMouseOut"), true),

				// cancel text selection and text dragging
				on(this.node, "dragstart, selectstart", function(evt){
					evt.preventDefault();
				})
			];
		},

		destroy: function(){
			// summary:
			//		Prepares this object to be garbage-collected

			var h;
			while(h = this.events.pop()){
				h.remove();
			}

			// this.clearItems();
			this.node = this.parent = null;
		},

		// mouse events
		onMouseOver: function(widget /*===== , evt =====*/){
			// summary:
			//		Called when mouse is moved over a TreeNode
			// widget: TreeNode
			// evt: Event
			// tags:
			//		protected
			this.current = widget;
		},

		onMouseOut: function(/*===== widget, evt =====*/){
			// summary:
			//		Called when mouse is moved away from a TreeNode
			// widget: TreeNode
			// evt: Event
			// tags:
			//		protected
			this.current = null;
		},

		_changeState: function(type, newState){
			// summary:
			//		Changes a named state to new state value
			// type: String
			//		A name of the state to change
			// newState: String
			//		new state
			var prefix = "dojoDnd" + type;
			var state = type.toLowerCase() + "State";
			//domClass.replace(this.node, prefix + newState, prefix + this[state]);
			domClass.replace(this.node, prefix + newState, prefix + this[state]);
			this[state] = newState;
		},

		_addItemClass: function(node, type){
			// summary:
			//		Adds a class with prefix "dojoDndItem"
			// node: Node
			//		A node
			// type: String
			//		A variable suffix for a class name
			domClass.add(node, "dojoDndItem" + type);
		},

		_removeItemClass: function(node, type){
			// summary:
			//		Removes a class with prefix "dojoDndItem"
			// node: Node
			//		A node
			// type: String
			//		A variable suffix for a class name
			domClass.remove(node, "dojoDndItem" + type);
		},

		onOverEvent: function(){
			// summary:
			//		This function is called once, when mouse is over our container
			// tags:
			//		protected
			this._changeState("Container", "Over");
		},

		onOutEvent: function(){
			// summary:
			//		This function is called once, when mouse is out of our container
			// tags:
			//		protected
			this._changeState("Container", "");
		}
	});
});

},
'xcitec/layout/Filters':function(){
define([
	"dojo/_base/declare",
	'dojo/_base/array',
	'dojo/dom-attr',
	'dojo/dom-construct',
	'dojo/query',
	"../Label",
	"./_LayoutWidget",
  "./_FiltersItem"
], function (declare, array, domAttr, domConstruct, query, Label, _LayoutWidget, _FiltersItem) {

	/*
	  xcitec.layout.Filters is a Xcitec Layout widget for Filters
	  Ecpected body content widgets and labels with "for".
	  NOTE: all other content items are ignored/lost
	  Example:
	  <div data-dojo-type="xcitec.layout.Filters" >
	    <label for="country">Country</label>
	    <input type="text" id="country" name="country" data-dojo-type="dijit.form.ValidationTextBox" value=""><br>
	    <label for="state">State</label>
	    <input type="text" id="state" name="state" data-dojo-type="dijit.form.ValidationTextBox" value=""><br>
	    <label for="zip_city">ZIP / city</label>
	    <input type="text" id="zip_city" name="zip_city" data-dojo-type="dijit.form.ValidationTextBox" value=""><br>
	    <label for="street">Street / No</label>
	    <input type="text" id="street" name="street" data-dojo-type="dijit.form.ValidationTextBox" value=""><br>
	  </div>
	 */
	return declare( _LayoutWidget, {
		baseClass: "fieldsetFilters",
		columnsQt: 3,
		controls: null,
		hasFacetedFilters: false,
		hasStandardFilters: false,

		postMixInProperties: function(){
			this.inherited(arguments);
			this.controls = [];
		},

		_fillContent: function(/*DomNode*/ source){
			for(var i = 0; i < this.columnsQt; i++){
				var column = domConstruct.create('div', {'class': 'fieldsetFiltersCol'});
				this.domNode.appendChild(column);
			}
			this.inherited(arguments);
		},

		_processChildren: function(/*DomNode*/ source){
			var children = query("> *", source);

			array.forEach(children, function(child){
				//ignore not an HTML node element child
				if(child.nodeType != 1) return;

				//if onlyWidgetsAllowed ignore not an Widget child
				var isWidget = domAttr.has(child, "data-dojo-type") && domAttr.get(child, "data-dojo-type");
				if(this.onlyWidgetsAllowed && !isWidget) return;

				if(isWidget){
					//create widget container wrapper - _LabelControlHolder
					var labelFor,
						labelNode = null;
					if(child.id) {
						labelFor = query("label[for='" + child.id + "']", source);
						if(labelFor.length > 0){
							labelNode = labelFor[0];
						}
					}
					this._processChildWidget(child, labelNode);
				}
			}, this);
			this._layout();
		},

		_layout: function(){
			var columns = query('.fieldsetFiltersCol', this.domNode),
				colIndex = 0;
			if(this.hasFacetedFilters && this.hasStandardFilters){
				var totalCols = columns.length; //this.columnsQt;
				var weight = 0;
				array.filter(this.controls, function(control){
					weight += control.weight;
				});
				var colWeight = Math.round((weight - 1) / totalCols) + 1;

				array.forEach(this.controls, function(control){
					if(colWeight <= 0 && colIndex < columns.length - 1){
						colIndex++;
						totalCols--;
						colWeight = Math.round((weight - 1 ) / totalCols) + 1;
					}
					columns[colIndex].appendChild(control.node);
					colWeight -= control.weight;
					weight -= control.weight;
				});
			}else{
				array.forEach(this.controls, function(control){
					columns[colIndex].appendChild(control.node);
					colIndex++;
					if(colIndex == columns.length){
						colIndex = 0;
					}
				});
			}
		},

		_processChildWidget: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
			var controlInfo = {};
			if(domAttr.get(controlNode, 'data-dojo-type') == 'xcitec/layout/FacetFilter'){
				controlInfo.weight = 3;
				this.hasFacetedFilters = true;
			}else{
				controlInfo.weight = 1;
				this.hasStandardFilters = true;
			}
			controlInfo.node = this.buildControlHolderNode(controlNode, labelNode);

			this.controls.push(controlInfo);
		},



		buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
			var controlContainer = new _FiltersItem({controlNode:controlNode, labelNode:labelNode});
			return controlContainer.domNode;
		}
	});
});
},
'dijit/focus':function(){
define([
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // domAttr.get dom.isDescendant
	"dojo/dom-attr", // domAttr.get dom.isDescendant
	"dojo/dom-class",
	"dojo/dom-construct", // connect to domConstruct.empty, domConstruct.destroy
	"dojo/Evented",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/domReady",
	"dojo/sniff", // has("ie")
	"dojo/Stateful",
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./a11y",	// a11y.isTabNavigable
	"./registry",	// registry.byId
	"./main"		// to set dijit.focus
], function(aspect, declare, dom, domAttr, domClass, domConstruct, Evented, lang, on, domReady, has, Stateful, win, winUtils,
			a11y, registry, dijit){

	// module:
	//		dijit/focus

	// Time of the last focusin event
	var lastFocusin;

	// Time of the last touch/mousedown or focusin event
	var lastTouchOrFocusin;

	var FocusManager = declare([Stateful, Evented], {
		// summary:
		//		Tracks the currently focused node, and which widgets are currently "active".
		//		Access via require(["dijit/focus"], function(focus){ ... }).
		//
		//		A widget is considered active if it or a descendant widget has focus,
		//		or if a non-focusable node of this widget or a descendant was recently clicked.
		//
		//		Call focus.watch("curNode", callback) to track the current focused DOMNode,
		//		or focus.watch("activeStack", callback) to track the currently focused stack of widgets.
		//
		//		Call focus.on("widget-blur", func) or focus.on("widget-focus", ...) to monitor when
		//		when widgets become active/inactive
		//
		//		Finally, focus(node) will focus a node, suppressing errors if the node doesn't exist.

		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit/_WidgetBase[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		constructor: function(){
			// Don't leave curNode/prevNode pointing to bogus elements
			var check = lang.hitch(this, function(node){
				if(dom.isDescendant(this.curNode, node)){
					this.set("curNode", null);
				}
				if(dom.isDescendant(this.prevNode, node)){
					this.set("prevNode", null);
				}
			});
			aspect.before(domConstruct, "empty", check);
			aspect.before(domConstruct, "destroy", check);
		},

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the `<iframe>` itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
			return this.registerWin(iframe.contentWindow, iframe);
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.

			// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

			// Listen for blur and focus events on targetWindow's document.
			var _this = this,
				body = targetWindow.document && targetWindow.document.body;

			if(body){
				// Listen for touches or mousedowns... could also use dojo/touch.press here.
				var event = has("pointer-events") ? "pointerdown" : has("MSPointer") ? "MSPointerDown" :
					has("touch-events") ? "mousedown, touchstart" : "mousedown";
				var mdh = on(targetWindow.document, event, function(evt){

					_this._justMouseDowned = true;
					// Use a 13 ms timeout to work-around Chrome resolving too fast and focusout
					// events not seeing that a mousedown just happened when a popup closes.
					// See https://bugs.dojotoolkit.org/ticket/17668
					setTimeout(function(){ _this._justMouseDowned = false; }, 13);

					// workaround weird IE bug where the click is on an orphaned node
					// (first time clicking a Select/DropDownButton inside a TooltipDialog).
					// actually, strangely this is happening on latest chrome too.
					if(evt && evt.target && evt.target.parentNode == null){
						return;
					}

					_this._onTouchNode(effectiveNode || evt.target, "mouse");
				});

				var fih = on(body, 'focusin', function(evt){
					// When you refocus the browser window, IE gives an event with an empty srcElement
					if(!evt.target.tagName) { return; }

					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// ignore those events
					var tag = evt.target.tagName.toLowerCase();
					if(tag == "#document" || tag == "body"){ return; }

					if(a11y.isFocusable(evt.target)){
						_this._onFocusNode(effectiveNode || evt.target);
					}else{
						// Previous code called _onTouchNode() for any activate event on a non-focusable node.   Can
						// probably just ignore such an event as it will be handled by onmousedown handler above, but
						// leaving the code for now.
						_this._onTouchNode(effectiveNode || evt.target);
					}
				});

				var foh = on(body, 'focusout', function(evt){
					_this._onBlurNode(effectiveNode || evt.target);
				});

				return {
					remove: function(){
						mdh.remove();
						fih.remove();
						foh.remove();
						mdh = fih = foh = null;
						body = null;	// prevent memory leak (apparent circular reference via closure)
					}
				};
			}
		},

		_onBlurNode: function(/*DomNode*/ node){
			// summary:
			//		Called when focus leaves a node.
			//		Usually ignored, _unless_ it *isn't* followed by touching another node,
			//		which indicates that we tabbed off the last field on the page,
			//		in which case every widget is marked inactive

			var now = (new Date()).getTime();

			// IE9+ and chrome have a problem where focusout events come after the corresponding focusin event.
			// For chrome problem see https://bugs.dojotoolkit.org/ticket/17668.
			// IE problem happens when moving focus from the Editor's <iframe> to a normal DOMNode.
			if(now < lastFocusin + 100){
				return;
			}

			// If the blur event isn't followed by a focus event, it means the user clicked on something unfocusable,
			// so clear focus.
			if(this._clearFocusTimer){
				clearTimeout(this._clearFocusTimer);
			}
			this._clearFocusTimer = setTimeout(lang.hitch(this, function(){
				this.set("prevNode", this.curNode);
				this.set("curNode", null);
			}), 0);

			if(this._justMouseDowned){
				// the mouse down caused a new widget to be marked as active; this blur event
				// is coming late, so ignore it.
				return;
			}

			// Unset timer to zero-out widget stack; we'll reset it below if appropriate.
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
			}

			if(now < lastTouchOrFocusin + 100){
				// This blur event is coming late (after the call to _onTouchNode() rather than before.
				// So let _onTouchNode() handle setting the widget stack.
				// See https://bugs.dojotoolkit.org/ticket/17668
				return;
			}

			// If the blur event isn't followed (or preceded) by a focus or touch event then mark all widgets as inactive.
			this._clearActiveWidgetsTimer = setTimeout(lang.hitch(this, function(){
				delete this._clearActiveWidgetsTimer;
				this._setStack([]);
			}), 0);
		},

		_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
			// summary:
			//		Callback when node is focused or touched.
			//		Note that _onFocusNode() calls _onTouchNode().
			// node:
			//		The node that was touched.
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			// Keep track of time of last focusin or touch event.
			lastTouchOrFocusin = (new Date()).getTime();

			if(this._clearActiveWidgetsTimer){
				// forget the recent blur event
				clearTimeout(this._clearActiveWidgetsTimer);
				delete this._clearActiveWidgetsTimer;
			}

			// if the click occurred on the scrollbar of a dropdown, treat it as a click on the dropdown,
			// even though the scrollbar is technically on the popup wrapper (see #10631)
			if(domClass.contains(node, "dijitPopup")){
				node = node.firstChild;
			}

			// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
			var newStack=[];
			try{
				while(node){
					var popupParent = domAttr.get(node, "dijitPopupParent");
					if(popupParent){
						node=registry.byId(popupParent).domNode;
					}else if(node.tagName && node.tagName.toLowerCase() == "body"){
						// is this the root of the document or just the root of an iframe?
						if(node === win.body()){
							// node is the root of the main document
							break;
						}
						// otherwise, find the iframe this node refers to (can't access it via parentNode,
						// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
						node=winUtils.get(node.ownerDocument).frameElement;
					}else{
						// if this node is the root node of a widget, then add widget id to stack,
						// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
						// to support MenuItem)
						var id = node.getAttribute && node.getAttribute("widgetId"),
							widget = id && registry.byId(id);
						if(widget && !(by == "mouse" && widget.get("disabled"))){
							newStack.unshift(id);
						}
						node=node.parentNode;
					}
				}
			}catch(e){ /* squelch */ }

			this._setStack(newStack, by);
		},

		_onFocusNode: function(/*DomNode*/ node){
			// summary:
			//		Callback when node is focused

			if(!node){
				return;
			}

			if(node.nodeType == 9){
				// Ignore focus events on the document itself.  This is here so that
				// (for example) clicking the up/down arrows of a spinner
				// (which don't get focus) won't cause that widget to blur. (FF issue)
				return;
			}

			// Keep track of time of last focusin event.
			lastFocusin = (new Date()).getTime();

			// There was probably a blur event right before this event, but since we have a new focus,
			// forget about the blur
			if(this._clearFocusTimer){
				clearTimeout(this._clearFocusTimer);
				delete this._clearFocusTimer;
			}

			this._onTouchNode(node);

			if(node == this.curNode){ return; }
			this.set("prevNode", this.curNode);
			this.set("curNode", node);
		},

		_setStack: function(/*String[]*/ newStack, /*String*/ by){
			// summary:
			//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
			// newStack:
			//		array of widget id's, starting from the top (outermost) widget
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			var oldStack = this.activeStack, lastOldIdx = oldStack.length - 1, lastNewIdx = newStack.length - 1;

			if(newStack[lastNewIdx] == oldStack[lastOldIdx]){
				// no changes, return now to avoid spurious notifications about changes to activeStack
				return;
			}

			this.set("activeStack", newStack);

			var widget, i;

			// for all elements that have gone out of focus, set focused=false
			for(i = lastOldIdx; i >= 0 && oldStack[i] != newStack[i]; i--){
				widget = registry.byId(oldStack[i]);
				if(widget){
					widget._hasBeenBlurred = true;		// TODO: used by form widgets, should be moved there
					widget.set("focused", false);
					if(widget._focusManager == this){
						widget._onBlur(by);
					}
					this.emit("widget-blur", widget, by);
				}
			}

			// for all element that have come into focus, set focused=true
			for(i++; i <= lastNewIdx; i++){
				widget = registry.byId(newStack[i]);
				if(widget){
					widget.set("focused", true);
					if(widget._focusManager == this){
						widget._onFocus(by);
					}
					this.emit("widget-focus", widget, by);
				}
			}
		},

		focus: function(node){
			// summary:
			//		Focus the specified node, suppressing errors if they occur
			if(node){
				try{ node.focus(); }catch(e){/*quiet*/}
			}
		}
	});

	var singleton = new FocusManager();

	// register top window and all the iframes it contains
	domReady(function(){
		var handle = singleton.registerWin(winUtils.get(document));
		if(has("ie")){
			on(window, "unload", function(){
				if(handle){	// because this gets called twice when doh.robot is running
					handle.remove();
					handle = null;
				}
			});
		}
	});

	// Setup dijit.focus as a pointer to the singleton but also (for backwards compatibility)
	// as a function to set focus.   Remove for 2.0.
	dijit.focus = function(node){
		singleton.focus(node);	// indirection here allows dijit/_base/focus.js to override behavior
	};
	for(var attr in singleton){
		if(!/^_/.test(attr)){
			dijit.focus[attr] = typeof singleton[attr] == "function" ? lang.hitch(singleton, attr) : singleton[attr];
		}
	}
	singleton.watch(function(attr, oldVal, newVal){
		dijit.focus[attr] = newVal;
	});

	return singleton;
});

},
'xcitec/form/Button':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
  "dojo/dom",
	"dojo/text!./templates/Button.html",
	"dijit/form/Button",
	"dijit/form/_FormWidget"
], function (declare, lang, windowModule, dom, template, Button, _FormWidget) {

	/*
	* Xcitec widget Button
	* Requiremets:
	* - button height 20px
	* - width not set -> complete button's caption is visible
	* - width set -> button will have excatly width
	*   (bigger if caption less os smaller if caption longer caption will cut)    
	*/

  return declare( Button, {
	    templateString: template,
	  baseClass: "dijitXcitecButton",
	
	  caption: "",
	  captionId: "",
	  readOnly: false,
	
	  attributeMap: lang.delegate(_FormWidget.prototype.attributeMap, {
	//    caption: { node: "containerNode", type: "innerHTML" },
	    iconClass: { node: "iconNode", type: "class" }
	  }),
	
	  create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
	    // Dijit button use label as button text, Xcitec label using for layout labels
	    // - if label in params remove it
	    // - if caption in params set "label" to call behavior of Dijit Button
	    if ("label" in params)   { delete params["label"]; }
	    //if ("caption" in params) { params["label"]=""; }
	    this.inherited(arguments);
	  },
	
	    _fillContent: function(/*DomNode*/ source){
	        // disable dojo button behavior
	        //todo - implement own component button - without dojo
	        if(this.caption){
	            this.containerNode.appendChild(windowModule.doc.createTextNode(this.caption));
	            return;
	        }
	        this.inherited(arguments);
	    },
	
	    _onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions
			if(this.disabled || this.readOnly){
				return false;
			}
			//this._clicked(); // widget click actions  //todo
			return this.onClick(e); // user click actions
		},

	  postCreate: function(){
	    if (this.disabled) {
	      // remove the tabIndex, especially for FF for don't get focus by TABing
	      this.focusNode.removeAttribute('tabIndex');
	    }
	    this.inherited(arguments);
      	dom.setSelectable(this.domNode, false);
	  },
	
	  _setCaptionAttr: function(/*String*/ content){
	    // summary:
	    //		Hook for attr('caption', ...) to work.
	    // description:
	    //		Set the label (text) of the button; takes an HTML string.
	    //this._setLabelAttr(content);
	  }
	});
});

},
'dojo/_base/declare':function(){
define(["./kernel", "../has", "./lang"], function(dojo, has, lang){
	// module:
	//		dojo/_base/declare

	var mix = lang.mixin, op = Object.prototype, opts = op.toString,
		xtor = new Function, counter = 0, cname = "constructor";

	function err(msg, cls){ throw new Error("declare" + (cls ? " " + cls : "") + ": " + msg); }

	// C3 Method Resolution Order (see http://www.python.org/download/releases/2.3/mro/)
	function c3mro(bases, className){
		var result = [], roots = [{cls: 0, refs: []}], nameMap = {}, clsCount = 1,
			l = bases.length, i = 0, j, lin, base, top, proto, rec, name, refs;

		// build a list of bases naming them if needed
		for(; i < l; ++i){
			base = bases[i];
			if(!base){
				err("mixin #" + i + " is unknown. Did you use dojo.require to pull it in?", className);
			}else if(opts.call(base) != "[object Function]"){
				err("mixin #" + i + " is not a callable constructor.", className);
			}
			lin = base._meta ? base._meta.bases : [base];
			top = 0;
			// add bases to the name map
			for(j = lin.length - 1; j >= 0; --j){
				proto = lin[j].prototype;
				if(!proto.hasOwnProperty("declaredClass")){
					proto.declaredClass = "uniqName_" + (counter++);
				}
				name = proto.declaredClass;
				if(!nameMap.hasOwnProperty(name)){
					nameMap[name] = {count: 0, refs: [], cls: lin[j]};
					++clsCount;
				}
				rec = nameMap[name];
				if(top && top !== rec){
					rec.refs.push(top);
					++top.count;
				}
				top = rec;
			}
			++top.count;
			roots[0].refs.push(top);
		}

		// remove classes without external references recursively
		while(roots.length){
			top = roots.pop();
			result.push(top.cls);
			--clsCount;
			// optimization: follow a single-linked chain
			while(refs = top.refs, refs.length == 1){
				top = refs[0];
				if(!top || --top.count){
					// branch or end of chain => do not end to roots
					top = 0;
					break;
				}
				result.push(top.cls);
				--clsCount;
			}
			if(top){
				// branch
				for(i = 0, l = refs.length; i < l; ++i){
					top = refs[i];
					if(!--top.count){
						roots.push(top);
					}
				}
			}
		}
		if(clsCount){
			err("can't build consistent linearization", className);
		}

		// calculate the superclass offset
		base = bases[0];
		result[0] = base ?
			base._meta && base === result[result.length - base._meta.bases.length] ?
				base._meta.bases.length : 1 : 0;

		return result;
	}

	function inherited(args, a, f){
		var name, chains, bases, caller, meta, base, proto, opf, pos,
			cache = this._inherited = this._inherited || {};

		// crack arguments
		if(typeof args == "string"){
			name = args;
			args = a;
			a = f;
		}
		f = 0;

		caller = args.callee;
		name = name || caller.nom;
		if(!name){
			err("can't deduce a name to call inherited()", this.declaredClass);
		}

		meta = this.constructor._meta;
		bases = meta.bases;

		pos = cache.p;
		if(name != cname){
			// method
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				base = bases[0];
				meta = base._meta;
				if(meta.hidden[name] !== caller){
					// error detection
					chains = meta.chains;
					if(chains && typeof chains[name] == "string"){
						err("calling chained method with inherited: " + name, this.declaredClass);
					}
					// find caller
					do{
						meta = base._meta;
						proto = base.prototype;
						if(meta && (proto[name] === caller && proto.hasOwnProperty(name) || meta.hidden[name] === caller)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
					pos = base ? pos : -1;
				}
			}
			// find next
			base = bases[++pos];
			if(base){
				proto = base.prototype;
				if(base._meta && proto.hasOwnProperty(name)){
					f = proto[name];
				}else{
					opf = op[name];
					do{
						proto = base.prototype;
						f = proto[name];
						if(f && (base._meta ? proto.hasOwnProperty(name) : f !== opf)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
				}
			}
			f = base && f || op[name];
		}else{
			// constructor
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				meta = bases[0]._meta;
				if(meta && meta.ctor !== caller){
					// error detection
					chains = meta.chains;
					if(!chains || chains.constructor !== "manual"){
						err("calling chained constructor with inherited", this.declaredClass);
					}
					// find caller
					while(base = bases[++pos]){ // intentional assignment
						meta = base._meta;
						if(meta && meta.ctor === caller){
							break;
						}
					}
					pos = base ? pos : -1;
				}
			}
			// find next
			while(base = bases[++pos]){	// intentional assignment
				meta = base._meta;
				f = meta ? meta.ctor : base;
				if(f){
					break;
				}
			}
			f = base && f;
		}

		// cache the found super method
		cache.c = f;
		cache.p = pos;

		// now we have the result
		if(f){
			return a === true ? f : f.apply(this, a || args);
		}
		// intentionally no return if a super method was not found
	}

	function getInherited(name, args){
		if(typeof name == "string"){
			return this.__inherited(name, args, true);
		}
		return this.__inherited(name, true);
	}

	function inherited__debug(args, a1, a2){
		var f = this.getInherited(args, a1);
		if(f){ return f.apply(this, a2 || a1 || args); }
		// intentionally no return if a super method was not found
	}

	var inheritedImpl = dojo.config.isDebug ? inherited__debug : inherited;

	// emulation of "instanceof"
	function isInstanceOf(cls){
		var bases = this.constructor._meta.bases;
		for(var i = 0, l = bases.length; i < l; ++i){
			if(bases[i] === cls){
				return true;
			}
		}
		return this instanceof cls;
	}

	function mixOwn(target, source){
		// add props adding metadata for incoming functions skipping a constructor
		for(var name in source){
			if(name != cname && source.hasOwnProperty(name)){
				target[name] = source[name];
			}
		}
		if(has("bug-for-in-skips-shadowed")){
			for(var extraNames= lang._extraNames, i= extraNames.length; i;){
				name = extraNames[--i];
				if(name != cname && source.hasOwnProperty(name)){
					  target[name] = source[name];
				}
			}
		}
	}

	// implementation of safe mixin function
	function safeMixin(target, source){
		// summary:
		//		Mix in properties skipping a constructor and decorating functions
		//		like it is done by declare().
		// target: Object
		//		Target object to accept new properties.
		// source: Object
		//		Source object for new properties.
		// description:
		//		This function is used to mix in properties like lang.mixin does,
		//		but it skips a constructor property and decorates functions like
		//		declare() does.
		//
		//		It is meant to be used with classes and objects produced with
		//		declare. Functions mixed in with dojo.safeMixin can use
		//		this.inherited() like normal methods.
		//
		//		This function is used to implement extend() method of a constructor
		//		produced with declare().
		//
		// example:
		//	|	var A = declare(null, {
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m1");
		//	|		}
		//	|	});
		//	|	B.extend({
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	dojo.safeMixin(x, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m1");
		//	|		},
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m2");
		//	|		}
		//	|	});
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m1
		//	|	// B.m1
		//	|	// X.m1

		var name, t;
		// add props adding metadata for incoming functions skipping a constructor
		for(name in source){
			t = source[name];
			if((t !== op[name] || !(name in op)) && name != cname){
				if(opts.call(t) == "[object Function]"){
					// non-trivial function method => attach its name
					t.nom = name;
				}
				target[name] = t;
			}
		}
		if(has("bug-for-in-skips-shadowed")){
			for(var extraNames= lang._extraNames, i= extraNames.length; i;){
				name = extraNames[--i];
				t = source[name];
				if((t !== op[name] || !(name in op)) && name != cname){
					if(opts.call(t) == "[object Function]"){
						// non-trivial function method => attach its name
						  t.nom = name;
					}
					target[name] = t;
				}
			}
		}
		return target;
	}

	function extend(source){
		declare.safeMixin(this.prototype, source);
		return this;
	}

	function createSubclass(mixins, props){
		return declare([this].concat(mixins), props || {});
	}

	// chained constructor compatible with the legacy declare()
	function chainedConstructor(bases, ctorSpecial){
		return function(){
			var a = arguments, args = a, a0 = a[0], f, i, m,
				l = bases.length, preArgs;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original declare()
			// 1) call two types of the preamble
			if(ctorSpecial && (a0 && a0.preamble || this.preamble)){
				// full blown ritual
				preArgs = new Array(bases.length);
				// prepare parameters
				preArgs[0] = a;
				for(i = 0;;){
					// process the preamble of the 1st argument
					a0 = a[0];
					if(a0){
						f = a0.preamble;
						if(f){
							a = f.apply(this, a) || a;
						}
					}
					// process the preamble of this class
					f = bases[i].prototype;
					f = f.hasOwnProperty("preamble") && f.preamble;
					if(f){
						a = f.apply(this, a) || a;
					}
					// one peculiarity of the preamble:
					// it is called if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
					if(++i == l){
						break;
					}
					preArgs[i] = a;
				}
			}
			// 2) call all non-trivial constructors using prepared arguments
			for(i = l - 1; i >= 0; --i){
				f = bases[i];
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, preArgs ? preArgs[i] : a);
				}
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, args);
			}
		};
	}


	// chained constructor compatible with the legacy declare()
	function singleConstructor(ctor, ctorSpecial){
		return function(){
			var a = arguments, t = a, a0 = a[0], f;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original declare()
			// 1) call two types of the preamble
			if(ctorSpecial){
				// full blown ritual
				if(a0){
					// process the preamble of the 1st argument
					f = a0.preamble;
					if(f){
						t = f.apply(this, t) || t;
					}
				}
				f = this.preamble;
				if(f){
					// process the preamble of this class
					f.apply(this, t);
					// one peculiarity of the preamble:
					// it is called even if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
				}
			}
			// 2) call a constructor
			if(ctor){
				ctor.apply(this, a);
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	// plain vanilla constructor (can use inherited() to call its base constructor)
	function simpleConstructor(bases){
		return function(){
			var a = arguments, i = 0, f, m;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original declare()
			// 1) do not call the preamble
			// 2) call the top constructor (it can use this.inherited())
			for(; f = bases[i]; ++i){ // intentional assignment
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, a);
					break;
				}
			}
			// 3) call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	function chain(name, bases, reversed){
		return function(){
			var b, m, f, i = 0, step = 1;
			if(reversed){
				i = bases.length - 1;
				step = -1;
			}
			for(; b = bases[i]; i += step){ // intentional assignment
				m = b._meta;
				f = (m ? m.hidden : b.prototype)[name];
				if(f){
					f.apply(this, arguments);
				}
			}
		};
	}

	// forceNew(ctor)
	// return a new object that inherits from ctor.prototype but
	// without actually running ctor on the object.
	function forceNew(ctor){
		// create object with correct prototype using a do-nothing
		// constructor
		xtor.prototype = ctor.prototype;
		var t = new xtor;
		xtor.prototype = null;	// clean up
		return t;
	}

	// applyNew(args)
	// just like 'new ctor()' except that the constructor and its arguments come
	// from args, which must be an array or an arguments object
	function applyNew(args){
		// create an object with ctor's prototype but without
		// calling ctor on it.
		var ctor = args.callee, t = forceNew(ctor);
		// execute the real constructor on the new object
		ctor.apply(t, args);
		return t;
	}

	function declare(className, superclass, props){
		// summary:
		//		Create a feature-rich constructor from compact notation.
		// className: String?
		//		The optional name of the constructor (loosely, a "class")
		//		stored in the "declaredClass" property in the created prototype.
		//		It will be used as a global name for a created constructor.
		// superclass: Function|Function[]
		//		May be null, a Function, or an Array of Functions. This argument
		//		specifies a list of bases (the left-most one is the most deepest
		//		base).
		// props: Object
		//		An object whose properties are copied to the created prototype.
		//		Add an instance-initialization function by making it a property
		//		named "constructor".
		// returns: dojo/_base/declare.__DeclareCreatedObject
		//		New constructor function.
		// description:
		//		Create a constructor using a compact notation for inheritance and
		//		prototype extension.
		//
		//		Mixin ancestors provide a type of multiple inheritance.
		//		Prototypes of mixin ancestors are copied to the new class:
		//		changes to mixin prototypes will not affect classes to which
		//		they have been mixed in.
		//
		//		Ancestors can be compound classes created by this version of
		//		declare(). In complex cases all base classes are going to be
		//		linearized according to C3 MRO algorithm
		//		(see http://www.python.org/download/releases/2.3/mro/ for more
		//		details).
		//
		//		"className" is cached in "declaredClass" property of the new class,
		//		if it was supplied. The immediate super class will be cached in
		//		"superclass" property of the new class.
		//
		//		Methods in "props" will be copied and modified: "nom" property
		//		(the declared name of the method) will be added to all copied
		//		functions to help identify them for the internal machinery. Be
		//		very careful, while reusing methods: if you use the same
		//		function under different names, it can produce errors in some
		//		cases.
		//
		//		It is possible to use constructors created "manually" (without
		//		declare()) as bases. They will be called as usual during the
		//		creation of an instance, their methods will be chained, and even
		//		called by "this.inherited()".
		//
		//		Special property "-chains-" governs how to chain methods. It is
		//		a dictionary, which uses method names as keys, and hint strings
		//		as values. If a hint string is "after", this method will be
		//		called after methods of its base classes. If a hint string is
		//		"before", this method will be called before methods of its base
		//		classes.
		//
		//		If "constructor" is not mentioned in "-chains-" property, it will
		//		be chained using the legacy mode: using "after" chaining,
		//		calling preamble() method before each constructor, if available,
		//		and calling postscript() after all constructors were executed.
		//		If the hint is "after", it is chained as a regular method, but
		//		postscript() will be called after the chain of constructors.
		//		"constructor" cannot be chained "before", but it allows
		//		a special hint string: "manual", which means that constructors
		//		are not going to be chained in any way, and programmer will call
		//		them manually using this.inherited(). In the latter case
		//		postscript() will be called after the construction.
		//
		//		All chaining hints are "inherited" from base classes and
		//		potentially can be overridden. Be very careful when overriding
		//		hints! Make sure that all chained methods can work in a proposed
		//		manner of chaining.
		//
		//		Once a method was chained, it is impossible to unchain it. The
		//		only exception is "constructor". You don't need to define a
		//		method in order to supply a chaining hint.
		//
		//		If a method is chained, it cannot use this.inherited() because
		//		all other methods in the hierarchy will be called automatically.
		//
		//		Usually constructors and initializers of any kind are chained
		//		using "after" and destructors of any kind are chained as
		//		"before". Note that chaining assumes that chained methods do not
		//		return any value: any returned value will be discarded.
		//
		// example:
		//	|	declare("my.classes.bar", my.classes.foo, {
		//	|		// properties to be added to the class prototype
		//	|		someValue: 2,
		//	|		// initialization function
		//	|		constructor: function(){
		//	|			this.myComplicatedObject = new ReallyComplicatedObject();
		//	|		},
		//	|		// other functions
		//	|		someMethod: function(){
		//	|			doStuff();
		//	|		}
		//	|	});
		//
		// example:
		//	|	var MyBase = declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass1 = declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass2 = declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyDiamond = declare([MyClass1, MyClass2], {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//
		// example:
		//	|	var F = function(){ console.log("raw constructor"); };
		//	|	F.prototype.method = function(){
		//	|		console.log("raw method");
		//	|	};
		//	|	var A = declare(F, {
		//	|		constructor: function(){
		//	|			console.log("A.constructor");
		//	|		},
		//	|		method: function(){
		//	|			console.log("before calling F.method...");
		//	|			this.inherited(arguments);
		//	|			console.log("...back in A");
		//	|		}
		//	|	});
		//	|	new A().method();
		//	|	// will print:
		//	|	// raw constructor
		//	|	// A.constructor
		//	|	// before calling F.method...
		//	|	// raw method
		//	|	// ...back in A
		//
		// example:
		//	|	var A = declare(null, {
		//	|		"-chains-": {
		//	|			destroy: "before"
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		constructor: function(){
		//	|			console.log("B.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("B.destroy");
		//	|		}
		//	|	});
		//	|	var C = declare(B, {
		//	|		constructor: function(){
		//	|			console.log("C.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("C.destroy");
		//	|		}
		//	|	});
		//	|	new C().destroy();
		//	|	// prints:
		//	|	// B.constructor
		//	|	// C.constructor
		//	|	// C.destroy
		//	|	// B.destroy
		//
		// example:
		//	|	var A = declare(null, {
		//	|		"-chains-": {
		//	|			constructor: "manual"
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		constructor: function(){
		//	|			// ...
		//	|			// call the base constructor with new parameters
		//	|			this.inherited(arguments, [1, 2, 3]);
		//	|			// ...
		//	|		}
		//	|	});
		//
		// example:
		//	|	var A = declare(null, {
		//	|		"-chains-": {
		//	|			m1: "before"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		"-chains-": {
		//	|			m2: "after"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("B.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	x.m1();
		//	|	// prints:
		//	|	// B.m1
		//	|	// A.m1
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m2
		//	|	// B.m2

		// crack parameters
		if(typeof className != "string"){
			props = superclass;
			superclass = className;
			className = "";
		}
		props = props || {};

		var proto, i, t, ctor, name, bases, chains, mixins = 1, parents = superclass;

		// build a prototype
		if(opts.call(superclass) == "[object Array]"){
			// C3 MRO
			bases = c3mro(superclass, className);
			t = bases[0];
			mixins = bases.length - t;
			superclass = bases[mixins];
		}else{
			bases = [0];
			if(superclass){
				if(opts.call(superclass) == "[object Function]"){
					t = superclass._meta;
					bases = bases.concat(t ? t.bases : superclass);
				}else{
					err("base class is not a callable constructor.", className);
				}
			}else if(superclass !== null){
				err("unknown base class. Did you use dojo.require to pull it in?", className);
			}
		}
		if(superclass){
			for(i = mixins - 1;; --i){
				proto = forceNew(superclass);
				if(!i){
					// stop if nothing to add (the last base)
					break;
				}
				// mix in properties
				t = bases[i];
				(t._meta ? mixOwn : mix)(proto, t.prototype);
				// chain in new constructor
				ctor = new Function;
				ctor.superclass = superclass;
				ctor.prototype = proto;
				superclass = proto.constructor = ctor;
			}
		}else{
			proto = {};
		}
		// add all properties
		declare.safeMixin(proto, props);
		// add constructor
		t = props.constructor;
		if(t !== op.constructor){
			t.nom = cname;
			proto.constructor = t;
		}

		// collect chains and flags
		for(i = mixins - 1; i; --i){ // intentional assignment
			t = bases[i]._meta;
			if(t && t.chains){
				chains = mix(chains || {}, t.chains);
			}
		}
		if(proto["-chains-"]){
			chains = mix(chains || {}, proto["-chains-"]);
		}

		// build ctor
		t = !chains || !chains.hasOwnProperty(cname);
		bases[0] = ctor = (chains && chains.constructor === "manual") ? simpleConstructor(bases) :
			(bases.length == 1 ? singleConstructor(props.constructor, t) : chainedConstructor(bases, t));

		// add meta information to the constructor
		ctor._meta  = {bases: bases, hidden: props, chains: chains,
			parents: parents, ctor: props.constructor};
		ctor.superclass = superclass && superclass.prototype;
		ctor.extend = extend;
		ctor.createSubclass = createSubclass;
		ctor.prototype = proto;
		proto.constructor = ctor;

		// add "standard" methods to the prototype
		proto.getInherited = getInherited;
		proto.isInstanceOf = isInstanceOf;
		proto.inherited    = inheritedImpl;
		proto.__inherited  = inherited;

		// add name if specified
		if(className){
			proto.declaredClass = className;
			lang.setObject(className, ctor);
		}

		// build chains and add them to the prototype
		if(chains){
			for(name in chains){
				if(proto[name] && typeof chains[name] == "string" && name != cname){
					t = proto[name] = chain(name, bases, chains[name] === "after");
					t.nom = name;
				}
			}
		}
		// chained methods do not return values
		// no need to chain "invisible" functions

		return ctor;	// Function
	}

	/*=====
	declare.__DeclareCreatedObject = {
		// summary:
		//		dojo/_base/declare() returns a constructor `C`.   `new C()` returns an Object with the following
		//		methods, in addition to the methods and properties specified via the arguments passed to declare().

		inherited: function(name, args, newArgs){
			// summary:
			//		Calls a super method.
			// name: String?
			//		The optional method name. Should be the same as the caller's
			//		name. Usually "name" is specified in complex dynamic cases, when
			//		the calling method was dynamically added, undecorated by
			//		declare(), and it cannot be determined.
			// args: Arguments
			//		The caller supply this argument, which should be the original
			//		"arguments".
			// newArgs: Object?
			//		If "true", the found function will be returned without
			//		executing it.
			//		If Array, it will be used to call a super method. Otherwise
			//		"args" will be used.
			// returns:
			//		Whatever is returned by a super method, or a super method itself,
			//		if "true" was specified as newArgs.
			// description:
			//		This method is used inside method of classes produced with
			//		declare() to call a super method (next in the chain). It is
			//		used for manually controlled chaining. Consider using the regular
			//		chaining, because it is faster. Use "this.inherited()" only in
			//		complex cases.
			//
			//		This method cannot me called from automatically chained
			//		constructors including the case of a special (legacy)
			//		constructor chaining. It cannot be called from chained methods.
			//
			//		If "this.inherited()" cannot find the next-in-chain method, it
			//		does nothing and returns "undefined". The last method in chain
			//		can be a default method implemented in Object, which will be
			//		called last.
			//
			//		If "name" is specified, it is assumed that the method that
			//		received "args" is the parent method for this call. It is looked
			//		up in the chain list and if it is found the next-in-chain method
			//		is called. If it is not found, the first-in-chain method is
			//		called.
			//
			//		If "name" is not specified, it will be derived from the calling
			//		method (using a methoid property "nom").
			//
			// example:
			//	|	var B = declare(A, {
			//	|		method1: function(a, b, c){
			//	|			this.inherited(arguments);
			//	|		},
			//	|		method2: function(a, b){
			//	|			return this.inherited(arguments, [a + b]);
			//	|		}
			//	|	});
			//	|	// next method is not in the chain list because it is added
			//	|	// manually after the class was created.
			//	|	B.prototype.method3 = function(){
			//	|		console.log("This is a dynamically-added method.");
			//	|		this.inherited("method3", arguments);
			//	|	};
			// example:
			//	|	var B = declare(A, {
			//	|		method: function(a, b){
			//	|			var super = this.inherited(arguments, true);
			//	|			// ...
			//	|			if(!super){
			//	|				console.log("there is no super method");
			//	|				return 0;
			//	|			}
			//	|			return super.apply(this, arguments);
			//	|		}
			//	|	});
			return	{};	// Object
		},

		getInherited: function(name, args){
			// summary:
			//		Returns a super method.
			// name: String?
			//		The optional method name. Should be the same as the caller's
			//		name. Usually "name" is specified in complex dynamic cases, when
			//		the calling method was dynamically added, undecorated by
			//		declare(), and it cannot be determined.
			// args: Arguments
			//		The caller supply this argument, which should be the original
			//		"arguments".
			// returns:
			//		Returns a super method (Function) or "undefined".
			// description:
			//		This method is a convenience method for "this.inherited()".
			//		It uses the same algorithm but instead of executing a super
			//		method, it returns it, or "undefined" if not found.
			//
			// example:
			//	|	var B = declare(A, {
			//	|		method: function(a, b){
			//	|			var super = this.getInherited(arguments);
			//	|			// ...
			//	|			if(!super){
			//	|				console.log("there is no super method");
			//	|				return 0;
			//	|			}
			//	|			return super.apply(this, arguments);
			//	|		}
			//	|	});
			return	{};	// Object
		},

		isInstanceOf: function(cls){
			// summary:
			//		Checks the inheritance chain to see if it is inherited from this
			//		class.
			// cls: Function
			//		Class constructor.
			// returns:
			//		"true", if this object is inherited from this class, "false"
			//		otherwise.
			// description:
			//		This method is used with instances of classes produced with
			//		declare() to determine of they support a certain interface or
			//		not. It models "instanceof" operator.
			//
			// example:
			//	|	var A = declare(null, {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|	var B = declare(null, {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|	var C = declare([A, B], {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|	var D = declare(A, {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|
			//	|	var a = new A(), b = new B(), c = new C(), d = new D();
			//	|
			//	|	console.log(a.isInstanceOf(A)); // true
			//	|	console.log(b.isInstanceOf(A)); // false
			//	|	console.log(c.isInstanceOf(A)); // true
			//	|	console.log(d.isInstanceOf(A)); // true
			//	|
			//	|	console.log(a.isInstanceOf(B)); // false
			//	|	console.log(b.isInstanceOf(B)); // true
			//	|	console.log(c.isInstanceOf(B)); // true
			//	|	console.log(d.isInstanceOf(B)); // false
			//	|
			//	|	console.log(a.isInstanceOf(C)); // false
			//	|	console.log(b.isInstanceOf(C)); // false
			//	|	console.log(c.isInstanceOf(C)); // true
			//	|	console.log(d.isInstanceOf(C)); // false
			//	|
			//	|	console.log(a.isInstanceOf(D)); // false
			//	|	console.log(b.isInstanceOf(D)); // false
			//	|	console.log(c.isInstanceOf(D)); // false
			//	|	console.log(d.isInstanceOf(D)); // true
			return	{};	// Object
		},

		extend: function(source){
			// summary:
			//		Adds all properties and methods of source to constructor's
			//		prototype, making them available to all instances created with
			//		constructor. This method is specific to constructors created with
			//		declare().
			// source: Object
			//		Source object which properties are going to be copied to the
			//		constructor's prototype.
			// description:
			//		Adds source properties to the constructor's prototype. It can
			//		override existing properties.
			//
			//		This method is similar to dojo.extend function, but it is specific
			//		to constructors produced by declare(). It is implemented
			//		using dojo.safeMixin, and it skips a constructor property,
			//		and properly decorates copied functions.
			//
			// example:
			//	|	var A = declare(null, {
			//	|		m1: function(){},
			//	|		s1: "Popokatepetl"
			//	|	});
			//	|	A.extend({
			//	|		m1: function(){},
			//	|		m2: function(){},
			//	|		f1: true,
			//	|		d1: 42
			//	|	});
		},
		
		createSubclass: function(mixins, props){
			// summary:
			//		Create a subclass of the declared class from a list of base classes.
			// mixins: Function[]
			//		Specifies a list of bases (the left-most one is the most deepest
			//		base).
			// props: Object?
			//		An optional object whose properties are copied to the created prototype.
			// returns: dojo/_base/declare.__DeclareCreatedObject
			//		New constructor function.
			// description:
			//		Create a constructor using a compact notation for inheritance and
			//		prototype extension.
			//
			//		Mixin ancestors provide a type of multiple inheritance.
			//		Prototypes of mixin ancestors are copied to the new class:
			//		changes to mixin prototypes will not affect classes to which
			//		they have been mixed in.
			//
			// example:
			//	|	var A = declare(null, {
			//	|		m1: function(){},
			//	|		s1: "bar"
			//	|	});
			//	|	var B = declare(null, {
			//	|		m2: function(){},
			//	|		s2: "foo"
			//	|	});
			//	|	var C = declare(null, {
			//	|	});
			//	|	var D1 = A.createSubclass([B, C], {
			//	|		m1: function(){},
			//	|		d1: 42
			//	|	});
			//	|	var d1 = new D1();
			//	|
			//	|	// this is equivalent to:
			//	|	var D2 = declare([A, B, C], {
			//	|		m1: function(){},
			//	|		d1: 42
			//	|	});
			//	|	var d2 = new D2();
		}
	};
	=====*/

	// For back-compat, remove for 2.0
	dojo.safeMixin = declare.safeMixin = safeMixin;
	dojo.declare = declare;

	return declare;
});

},
'xcitec/form/_ListBoxItem':function(){
define([
         "dojo/_base/declare",
         "dojo/_base/array",
         "dojo/query",
         "dojo/dom-class",
         "dojo/dom-construct",
         "dijit/_WidgetBase",
         "../_base/ImageFactory"
       ], function (declare, array, query, domClass, domConstruct, _Widget, ImageFactory) {
  return declare(_Widget, {

    itemIndex: 0,
    itemLabel: "",
    parentNode: null,
    value: null,
    isSelected: false,
    icon: "",

    postMixInProperties: function(){
      if(this.icon){
        this.icon = ImageFactory.createIconCss(this.icon);
      }
    },

    buildRendering: function() {
      var spanIcons = query("> span", this.itemLabel);
      var tr = domConstruct.create("tr", {rowId:this.itemIndex});
      var _this = this;

      if (this.isSelected == 'true') {
        domClass.add(tr, "selectedRow");
        _this.parentNode.selItems = _this.parentNode.selItems.concat(this.value);
        _this.parentNode.selWidgets = _this.parentNode.selWidgets.concat(_this);

      }
      /*
       dojo.forEach(_this.parentNode.value, function(val) {
       if (_this.parentNode.values[_this.itemIndex] == val) {
       dojo.addClass(tr, "selectedRow");
       _this.parentNode.selItems = _this.parentNode.selItems.concat(val);
       _this.parentNode.selWidgets = _this.parentNode.selWidgets.concat(_this);
       }
       });
       */
      /* dojo.forEach(_this.parentNode.selectedRows, function(selRow) {
       console.log(selRow);

       if (selRow == "true") {
       dojo.addClass(tr, "selectedRow");
       _this.parentNode.selItems = _this.parentNode.selItems.concat(_this.parentNode.values[_this.itemIndex]);
       _this.parentNode.selWidgets = _this.parentNode.selWidgets.concat(_this);
       }
       });*/
      var td = domConstruct.create("td", {}, tr, "first");
      var div;
      if (spanIcons.length > 0) {
        div = domConstruct.create("div", {}, td, "first");
        var span = domConstruct.create("span", {}, div, "first");
        domClass.add(span, "branch_wrap");
        array.forEach(spanIcons, function(icon) {
          span.appendChild(icon);
        });
        div.innerHTML += _this.itemLabel.innerHTML;
      } else {
        div = domConstruct.create("div", {innerHTML: _this.itemLabel.innerHTML}, td, "first");
      }
      var clazz = (_this.itemIndex % 2 == 0) ? "oddRow" : "evenRow";
      domClass.add(tr, clazz);
      domClass.add(div, "wrap");
      if(this.icon){
        domClass.add(div, "rowIcon " + this.icon);
      }
      this.domNode = tr;
    },

    postCreate: function() {
      this.connect(this.domNode, "onclick", this._onClick);
    },

    _onClick: function(evt) {
      if (this.parentNode.disabled || this.parentNode.readOnly) {
        return;
      }
      this.parentNode.selectRow(this, evt);
    },

    setSelected: function(isSelect) {
      (isSelect) ? domClass.add(this.domNode, "selectedRow") : domClass.remove(this.domNode, "selectedRow");
    }
  });
});
},
'dojo/text':function(){
define(["./_base/kernel", "require", "./has", "./request"], function(dojo, require, has, request){
	// module:
	//		dojo/text

	var getText;
	if( 1 ){
		getText= function(url, sync, load){
			request(url, {sync:!!sync, headers: { 'X-Requested-With': null } }).then(load);
		};
	}else{
		// Path for node.js and rhino, to load from local file system.
		// TODO: use node.js native methods rather than depending on a require.getText() method to exist.
		if(require.getText){
			getText= require.getText;
		}else{
			console.error("dojo/text plugin failed to load because loader does not support getText");
		}
	}

	var
		theCache = {},

		strip= function(text){
			//Strips <?xml ...?> declarations so that external SVG and XML
			//documents can be added to a document without worry. Also, if the string
			//is an HTML document, only the part inside the body tag is returned.
			if(text){
				text= text.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
				var matches= text.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
				if(matches){
					text= matches[1];
				}
			}else{
				text = "";
			}
			return text;
		},

		notFound = {},

		pending = {};

	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		//		A getter and setter for storing the string content associated with the
		//		module and url arguments.
		// description:
		//		If module is a string that contains slashes, then it is interpretted as a fully
		//		resolved path (typically a result returned by require.toUrl), and url should not be
		//		provided. This is the preferred signature. If module is a string that does not
		//		contain slashes, then url must also be provided and module and url are used to
		//		call `dojo.moduleUrl()` to generate a module URL. This signature is deprecated.
		//		If value is specified, the cache value for the moduleUrl will be set to
		//		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		//		in its internal cache and return that cached value for the URL. To clear
		//		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		//		the URL contents, only modules on the same domain of the page can use this capability.
		//		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		//		If a String with slashes, a fully resolved path; if a String without slashes, the
		//		module name to use for the base part of the URL, similar to module argument
		//		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		//		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		//		The rest of the path to append to the path derived from the module argument. If
		//		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		//		If a String, the value to use in the cache for the module/url combination.
		//		If an Object, it can have two properties: value and sanitize. The value property
		//		should be the value to use in the cache, and sanitize can be set to true or false,
		//		to indicate if XML declarations should be removed from the value and if the HTML
		//		inside a body tag in the value should be extracted as the real value. The value argument
		//		or the value property on the value argument are usually only used by the build system
		//		as it inlines cache content.
		// example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		//		of call is used to avoid an issue with the build system erroneously trying to intern
		//		this example. To get the build system to intern your dojo.cache calls, use the
		//		"dojo.cache" style of call):
		//		| //If template.html contains "<h1>Hello</h1>" that will be
		//		| //the value for the text variable.
		//		| //Note: This is pre-AMD, deprecated syntax
		//		| var text = dojo["cache"]("my.module", "template.html");
		// example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		//		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		//		erroneously trying to intern this example. To get the build system to intern your
		//		dojo.cache calls, use the "dojo.cache" style of call):
		//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		//		| //text variable will contain just "<h1>Hello</h1>".
		//		| //Note: This is pre-AMD, deprecated syntax
		//		| var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		// example:
		//		Same example as previous, but demonstrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		//		| //text variable will contain just "<h1>Hello</h1>".
		//		| //Note: This is pre-AMD, deprecated syntax
		//		| var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//	 * (string string [value]) => (module, url, value)
		//	 * (object [value])        => (module, value), url defaults to ""
		//
		//	 * if module is an object, then it must be convertable to a string
		//	 * (module, url) module + (url ? ("/" + url) : "") must be a legal argument to require.toUrl
		//	 * value may be a string or an object; if an object then may have the properties "value" and/or "sanitize"
		var key;
		if(typeof module=="string"){
			if(/\//.test(module)){
				// module is a version 1.7+ resolved path
				key = module;
				value = url;
			}else{
				// module is a version 1.6- argument to dojo.moduleUrl
				key = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : ""));
			}
		}else{
			key = module + "";
			value = url;
		}
		var
			val = (value != undefined && typeof value != "string") ? value.value : value,
			sanitize = value && value.sanitize;

		if(typeof val == "string"){
			//We have a string, set cache value
			theCache[key] = val;
			return sanitize ? strip(val) : val;
		}else if(val === null){
			//Remove cached value
			delete theCache[key];
			return null;
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in theCache)){
				getText(key, true, function(text){
					theCache[key]= text;
				});
			}
			return sanitize ? strip(theCache[key]) : theCache[key];
		}
	};

	return {
		// summary:
		//		This module implements the dojo/text! plugin and the dojo.cache API.
		// description:
		//		We choose to include our own plugin to leverage functionality already contained in dojo
		//		and thereby reduce the size of the plugin compared to various foreign loader implementations.
		//		Also, this allows foreign AMD loaders to be used without their plugins.
		//
		//		CAUTION: this module is designed to optionally function synchronously to support the dojo v1.x synchronous
		//		loader. This feature is outside the scope of the CommonJS plugins specification.

		// the dojo/text caches it's own resources because of dojo.cache
		dynamic: true,

		normalize: function(id, toAbsMid){
			// id is something like (path may be relative):
			//
			//	 "path/to/text.html"
			//	 "path/to/text.html!strip"
			var parts= id.split("!"),
				url= parts[0];
			return (/^\./.test(url) ? toAbsMid(url) : url) + (parts[1] ? "!" + parts[1] : "");
		},

		load: function(id, require, load){
			// id: String
			//		Path to the resource.
			// require: Function
			//		Object that include the function toUrl with given id returns a valid URL from which to load the text.
			// load: Function
			//		Callback function which will be called, when the loading finished.

			// id is something like (path is always absolute):
			//
			//	 "path/to/text.html"
			//	 "path/to/text.html!strip"
			var
				parts= id.split("!"),
				stripFlag= parts.length>1,
				absMid= parts[0],
				url = require.toUrl(parts[0]),
				requireCacheUrl = "url:" + url,
				text = notFound,
				finish = function(text){
					load(stripFlag ? strip(text) : text);
				};
			if(absMid in theCache){
				text = theCache[absMid];
			}else if(require.cache && requireCacheUrl in require.cache){
				text = require.cache[requireCacheUrl];
			}else if(url in theCache){
				text = theCache[url];
			}
			if(text===notFound){
				if(pending[url]){
					pending[url].push(finish);
				}else{
					var pendingList = pending[url] = [finish];
					getText(url, !require.async, function(text){
						theCache[absMid]= theCache[url]= text;
						for(var i = 0; i<pendingList.length;){
							pendingList[i++](text);
						}
						delete pending[url];
					});
				}
			}else{
				finish(text);
			}
		}
	};

});


},
'xcitec/TabItem':function(){
define([
    "dojo/_base/declare",
    "dojo/dom",
    "dijit/_WidgetsInTemplateMixin",
    "./_StateWidget",
    "./Label"
       ], function (declare, dom, _WidgetsInTemplateMixin, _StateWidget) {

  return declare([_StateWidget, _WidgetsInTemplateMixin],
        {
          baseClass: "TabItem",

          widgetsInTemplate: true,

          label: '',

          labelId: '',

          selected: false,
          disabled: false,
          readOnly: false,

          templateString: '<li class="${baseClass}" data-dojo-attach-event="ondijitclick:_onClick, onmouseenter:_onMouse,onmouseleave:_onMouse">' +
                          '<img src="${_blankGif}" width="1" height="1" alt="" />' +
                          '<span><span data-dojo-type="xcitec/Label" inline="true" class="TabItemLabel" labelId="${labelId}">${label}</span></span>' +
                          '</li>',

          postMixInProperties: function() {
            this.inherited(arguments);
          },

          buildRendering: function(){
            var source = this.srcNodeRef;
            if(source && !("label" in this.params)){
              this.set('label', source.innerHTML);
            }
            this.inherited(arguments);
          },

          postCreate: function() {
            dom.setSelectable(this.domNode, false);
          },

          _onClick: function(/*Event*/ evt){
            // summary:
            //		Internal function to handle click actions
            if(this.disabled || this.readOnly || this.selected){
              return false;
            }
            return this.onClick(evt);
          },

          _onBlur: function() {
            //			this._onUnhover();
          },

          onSelect: function() {
            // user defined event
          },

          onClick: function(/*Event*/ evt) {
            // user defined event
            return this.onSelect(evt);
          }
  });
});

},
'xcitec/form/MultiLanguageTextarea':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"./Textarea",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "./_XButtonRight",
    "./_XButtonSingleRight"
], function (declare, domClass, Textarea, _TemplatedMixin, _WidgetsInTemplateMixin) {

	/*
	 known bug in IE: if there is logn word - scrolls out of container.
	 */
	
	var MultiLanguageTextarea = declare( [Textarea, _TemplatedMixin, _WidgetsInTemplateMixin], {
	    widgetsInTemplate: true,
	
	    xbtnClass: "xcitec/form/_XButtonRight",
	    disabledAttrSetting: "",
	    actionIconType: "Choose",
	    actionIconText: "",
	    buttonEnabled: false,
	
	    templateString:
	            '<div class="dijit dijitReset dijitInlineTable ${baseClass}" '+
	            '    id="widget_${id}" '+
	            '    waiRole="presentation" '+
	            '  ><div style="overflow:hidden;" '+
	            '    ><div class="dijitReset dijitXcitecRight" '+
	            '      ><div data-dojo-type="${!xbtnClass}" iconType="${actionIconType}" iconText="${actionIconText}" '+
	            '        data-dojo-attach-event="onDoAction:_onClick" data-dojo-attach-point="button" '+
	            '        ${disabledAttrSetting} '+
	            '      ></div '+
	            '    ></div'+
	            '    ><div class="dijitReset dijitXcitecMiddle">'+
	            '<table class="dijitReset" style="width:100%; table-layout:fixed;" cellspacing="0" cellpadding="0" ' +
	            '       data-dojo-attach-point="stateNode" '+
	            '  ><tr class="dijitReset"><td class="dijitReset dijitXcitecTextAreaBox" ' +
	            '    ><textarea class="dijitXcitecTextAreaInput" ' +
	            '             ${!nameAttrSetting}  autocomplete="off" ' +
	            '             data-dojo-attach-point="focusNode,containerNode,textbox" ' +
	            '    ></textarea ' +
	            '></td></tr' +
	            '></table>' +
	            '</div>'+
	            '</div><img src="${_blankGif}" alt="" class="required" /></div>',
	
	//            '<table class="dijitReset" cellspacing="0" cellpadding="0" ' +
	//            '       id="widget_${id}" ' +
	//            '       data-dojo-attach-event="onmouseover:_onMouse,onmouseout:_onMouse,onmousedown:_onMouse" ' +
	//            '  ><tr class="dijitReset"><td class="dijitReset dijitXcitecTextAreaBox" ' +
	//            '    ><textarea class="dijitXcitecTextAreaInput" ' +
	//            '             ${nameAttrSetting}  autocomplete="off" ' +
	//            '             data-dojo-attach-point="focusNode,containerNode,textbox" ' +
	//            '    ></textarea ' +
	//            '  ></td ' +
	//            '><td style="vertical-align: top;"><div data-dojo-type="${!xbtnClass}" iconType="${actionIconType}" iconText="${actionIconText}"' +
	//            '  data-dojo-attach-event="onDoAction:_onClick" data-dojo-attach-point="button" ' +
	//            '  ${disabledAttrSetting} ' +
	//            '></div' +
	//            '></td></tr' +
	//            '></table>',
	    rows: "",
	    cols: "",
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
		    this.buttonEnabled = this.buttonEnabled || ("onActionClick" in this.params);
	        if (this.disabled) {
	            this.required = false;
	            this.disabledAttrSetting = "disabled";
	            this.buttonEnabled = false;
	        }
	        if (this.readOnly) {
	            this.required = false;
	            this.xbtnClass = "xcitec/form/_XButtonSingleRight";
	        }
	    },
	
	    postCreate: function() {
	      this.inherited(arguments);
	      //todo same code in TextBoxButton
	      if (!this.buttonEnabled) {
	          // if action for buton is not allowed button has to be deactivated for user
	          // XButton has different view as readOnly and disabled
	          // to feet to style:
	          // - if widget is active -> mark action button as readOnly
	          // - if widget is readOnly -> mark action button as disabled
	          // - if widget is disabled -> action button is not visible in any case
	          if (this.readOnly) {
	              this.button.set("disabled", true);
	          } else {
	              this.button.set("readOnly", true);
	          }
	          this.buttonEnabled = false;
	      }
	        if (this.required) {
	            domClass.add(this.domNode, "dijitXcitecRequired");
	        }
	    },
	
	    _onClick: function() {
	        if (!this.buttonEnabled) {
	            return;
	        }
	        this.onActionClick();
	    },
	
	    onActionClick: function() {
	      //have to be ovveriden
	      console.log("onActionClick is called! this is wrong!!!");
	    }
	});
	
	return MultiLanguageTextarea;
});

},
'xcitec/layout/MultiSelectLayout':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dojo/text!xcitec/layout/templates/MultiSelectLayout.html",
	"dijit/_TemplatedMixin",
	"dijit/layout/_LayoutWidget"
], function (dojo, array, declare, domAttr, domClass, domConstruct, query, template, _Templated, _LayoutWidget) {

	/*
	MultiSelectLayout is a layout control which keeps inside it other controls.
	Inner controls must have appropriate xcitec roles to be placed in their placeholders.
	These roles are: 'searchresult', 'filterbox', 'navigation', 'selectedresult'
	
	 <div data-dojo-type="xcitec.layout.MultiSelectLayout">
	  <div xcitecRole="filterbox" ....>...</div>
	  <div xcitecRole="searchresult" ....>...</div>
	  <div xcitecRole="navigation" ....>...</div>
	  <div xcitecRole="selectedresult" ....>...</div>
	<div>
	
	MultiSelectLayout looks like this:
	
	    |--------------------------|
	    |         filterbox        |
	    |--------------------------|
	    | search  | navi |selected |
	    | result  |gation| result  |
	    |         |      |         |
	    |--------------------------|
	 */
	var MultiSelectLayout = declare( [_LayoutWidget, _Templated], {
	
	  templateString: template,
	
	  postMixInProperties: function() {
	    this.xcitecRoles = query(" > [xcitecRole]", this.srcNodeRef); 
	    this.inherited(arguments);
	  }, 
	
	  _fillContent: function() {
	    var _this = this;
	    array.forEach(_this.xcitecRoles, function(node) {
	      switch (domAttr.get(node, "xcitecRole")) {
	      case "searchresult":
	        domConstruct.place(node, _this.searchResultNode, "last");
	        break;
	      case "filterbox":
	        domConstruct.place(node, _this.filtersNode, "last");
	        break;
	      case "navigation":
	          domConstruct.place(node, _this.navigationNode, "last");
	          domClass.add(node, "multiSelectNavigationInner");
	          break;
	      case "selectedresult":
	        domConstruct.place(node, _this.selectionResultNode, "last");
	        break;
	      default:
	        break;
	      }
	    });
	    this.inherited(arguments);
	  }
	
	});
	return MultiSelectLayout;
});

},
'xcitec/TabContainer':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "dijit/_OnDijitClickMixin"
], function (array, declare, domClass, domConstruct, geometry, _Container, _Templated, _Widget) {

	var TabContainer = declare(
		[_Widget, _Templated, _Container],
		{
			baseClass: "TabContainer",
	    disabled: false,
	    _useOverlay: false,
			
	    templateString: '<ul class="dijitReset ${baseClass}" data-dojo-attach-point="containerNode"></ul>',
	
	    postMixInProperties: function() {
			  if (this.disabled) {
			    this.baseClass += " TabContainerDisabled";
			  }
	      this.inherited(arguments);
	    },
	
	    buildRendering: function(){
	      this.inherited(arguments);
	    },
	
	    postCreate: function() {
	      //dojo.setSelectable(this.domNode, false);
	    },
	
	    startup: function() {
	      //dojo.setSelectable(this.domNode, false);
	      if (this.disabled) {
	        if (this._useOverlay) {
	          this.nodeOverlay = domConstruct.create('div');
	          domClass.add(this.nodeOverlay, "TabContainerOverlay");
	          var mbri = geometry.getContentBox(this.domNode);
	          geometry.setContentSize( this.nodeOverlay, { h: mbri.h+12 } ); //added TabItem margins
	          this.containerNode.appendChild(this.nodeOverlay);
	        } else {
	          array.forEach(this.getChildren(), function(child) {
	             child.set("readOnly", true);
	          }, this);
	        }
	      }
	    },
	
	    onExecute: function() {
			}
		}
	);
	return TabContainer;
});

},
'dojo/data/util/simpleFetch':function(){
define(["../../_base/lang", "../../_base/kernel", "./sorter"],
  function(lang, kernel, sorter){
	// module:
	//		dojo/data/util/simpleFetch
	// summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.

var simpleFetch = {};
lang.setObject("dojo.data.util.simpleFetch", simpleFetch);

simpleFetch.errorHandler = function(/*Object*/ errorData, /*Object*/ requestObject){
	// summary:
	//		The error handler when there is an error fetching items.  This function should not be called
	//		directly and is used by simpleFetch.fetch().
	if(requestObject.onError){
		var scope = requestObject.scope || kernel.global;
		requestObject.onError.call(scope, errorData, requestObject);
	}
};

simpleFetch.fetchHandler = function(/*Array*/ items, /*Object*/ requestObject){
	// summary:
	//		The handler when items are sucessfully fetched.  This function should not be called directly
	//		and is used by simpleFetch.fetch().
	var oldAbortFunction = requestObject.abort || null,
		aborted = false,

		startIndex = requestObject.start?requestObject.start: 0,
		endIndex = (requestObject.count && (requestObject.count !== Infinity))?(startIndex + requestObject.count):items.length;

	requestObject.abort = function(){
		aborted = true;
		if(oldAbortFunction){
			oldAbortFunction.call(requestObject);
		}
	};

	var scope = requestObject.scope || kernel.global;
	if(!requestObject.store){
		requestObject.store = this;
	}
	if(requestObject.onBegin){
		requestObject.onBegin.call(scope, items.length, requestObject);
	}
	if(requestObject.sort){
		items.sort(sorter.createSortFunction(requestObject.sort, this));
	}
	if(requestObject.onItem){
		for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
			var item = items[i];
			if(!aborted){
				requestObject.onItem.call(scope, item, requestObject);
			}
		}
	}
	if(requestObject.onComplete && !aborted){
		var subset = null;
		if(!requestObject.onItem){
			subset = items.slice(startIndex, endIndex);
		}
		requestObject.onComplete.call(scope, subset, requestObject);
	}
};

simpleFetch.fetch = function(/* Object? */ request){
	// summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.
	// description:
	//		The simpleFetch mixin should work well for any datastore that can respond to a _fetchItems()
	//		call by returning an array of all the found items that matched the query.  The simpleFetch mixin
	//		is not designed to work for datastores that respond to a fetch() call by incrementally
	//		loading items, or sequentially loading partial batches of the result
	//		set.  For datastores that mixin simpleFetch, simpleFetch
	//		implements a fetch method that automatically handles eight of the fetch()
	//		arguments -- onBegin, onItem, onComplete, onError, start, count, sort and scope
	//		The class mixing in simpleFetch should not implement fetch(),
	//		but should instead implement a _fetchItems() method.  The _fetchItems()
	//		method takes three arguments, the keywordArgs object that was passed
	//		to fetch(), a callback function to be called when the result array is
	//		available, and an error callback to be called if something goes wrong.
	//		The _fetchItems() method should ignore any keywordArgs parameters for
	//		start, count, onBegin, onItem, onComplete, onError, sort, and scope.
	//		The _fetchItems() method needs to correctly handle any other keywordArgs
	//		parameters, including the query parameter and any optional parameters
	//		(such as includeChildren).  The _fetchItems() method should create an array of
	//		result items and pass it to the fetchHandler along with the original request object --
	//		or, the _fetchItems() method may, if it wants to, create an new request object
	//		with other specifics about the request that are specific to the datastore and pass
	//		that as the request object to the handler.
	//
	//		For more information on this specific function, see dojo/data/api/Read.fetch()
	//
	// request:
	//		The keywordArgs parameter may either be an instance of
	//		conforming to dojo/data/api/Request or may be a simple anonymous object
	//		that may contain any of the following:
	// |	{
	// |		query: query-object or query-string,
	// |		queryOptions: object,
	// |		onBegin: Function,
	// |		onItem: Function,
	// |		onComplete: Function,
	// |		onError: Function,
	// |		scope: object,
	// |		start: int
	// |		count: int
	// |		sort: array
	// |	}
	//		All implementations should accept keywordArgs objects with any of
	//		the 9 standard properties: query, onBegin, onItem, onComplete, onError
	//		scope, sort, start, and count.  Some implementations may accept additional
	//		properties in the keywordArgs object as valid parameters, such as
	//		{includeOutliers:true}.
	//
	//		####The *query* parameter
	//
	//		The query may be optional in some data store implementations.
	//		The dojo/data/api/Read API does not specify the syntax or semantics
	//		of the query itself -- each different data store implementation
	//		may have its own notion of what a query should look like.
	//		However, as of dojo 0.9, 1.0, and 1.1, all the provided datastores in dojo.data
	//		and dojox.data support an object structure query, where the object is a set of
	//		name/value parameters such as { attrFoo: valueBar, attrFoo1: valueBar1}.  Most of the
	//		dijit widgets, such as ComboBox assume this to be the case when working with a datastore
	//		when they dynamically update the query.  Therefore, for maximum compatibility with dijit
	//		widgets the recommended query parameter is a key/value object.  That does not mean that the
	//		the datastore may not take alternative query forms, such as a simple string, a Date, a number,
	//		or a mix of such.  Ultimately, The dojo/data/api/Read API is agnostic about what the query
	//		format.
	//
	//		Further note:  In general for query objects that accept strings as attribute
	//		value matches, the store should also support basic filtering capability, such as *
	//		(match any character) and ? (match single character).  An example query that is a query object
	//		would be like: { attrFoo: "value*"}.  Which generally means match all items where they have
	//		an attribute named attrFoo, with a value that starts with 'value'.
	//
	//		####The *queryOptions* parameter
	//
	//		The queryOptions parameter is an optional parameter used to specify options that may modify
	//		the query in some fashion, such as doing a case insensitive search, or doing a deep search
	//		where all items in a hierarchical representation of data are scanned instead of just the root
	//		items.  It currently defines two options that all datastores should attempt to honor if possible:
	// |	{
	// |		ignoreCase: boolean, // Whether or not the query should match case sensitively or not.  Default behaviour is false.
	// |		deep: boolean	// Whether or not a fetch should do a deep search of items and all child
	// |						// items instead of just root-level items in a datastore.  Default is false.
	// |	}
	//
	//		####The *onBegin* parameter.
	//
	//		function(size, request);
	//		If an onBegin callback function is provided, the callback function
	//		will be called just once, before the first onItem callback is called.
	//		The onBegin callback function will be passed two arguments, the
	//		the total number of items identified and the Request object.  If the total number is
	//		unknown, then size will be -1.  Note that size is not necessarily the size of the
	//		collection of items returned from the query, as the request may have specified to return only a
	//		subset of the total set of items through the use of the start and count parameters.
	//
	//		####The *onItem* parameter.
	//
	//		function(item, request);
	//
	//		If an onItem callback function is provided, the callback function
	//		will be called as each item in the result is received. The callback
	//		function will be passed two arguments: the item itself, and the
	//		Request object.
	//
	//		####The *onComplete* parameter.
	//
	//		function(items, request);
	//
	//		If an onComplete callback function is provided, the callback function
	//		will be called just once, after the last onItem callback is called.
	//		Note that if the onItem callback is not present, then onComplete will be passed
	//		an array containing all items which matched the query and the request object.
	//		If the onItem callback is present, then onComplete is called as:
	//		onComplete(null, request).
	//
	//		####The *onError* parameter.
	//
	//		function(errorData, request);
	//
	//		If an onError callback function is provided, the callback function
	//		will be called if there is any sort of error while attempting to
	//		execute the query.
	//		The onError callback function will be passed two arguments:
	//		an Error object and the Request object.
	//
	//		####The *scope* parameter.
	//
	//		If a scope object is provided, all of the callback functions (onItem,
	//		onComplete, onError, etc) will be invoked in the context of the scope
	//		object.  In the body of the callback function, the value of the "this"
	//		keyword will be the scope object.   If no scope object is provided,
	//		the callback functions will be called in the context of dojo.global().
	//		For example, onItem.call(scope, item, request) vs.
	//		onItem.call(dojo.global(), item, request)
	//
	//		####The *start* parameter.
	//
	//		If a start parameter is specified, this is a indication to the datastore to
	//		only start returning items once the start number of items have been located and
	//		skipped.  When this parameter is paired with 'count', the store should be able
	//		to page across queries with millions of hits by only returning subsets of the
	//		hits for each query
	//
	//		####The *count* parameter.
	//
	//		If a count parameter is specified, this is a indication to the datastore to
	//		only return up to that many items.  This allows a fetch call that may have
	//		millions of item matches to be paired down to something reasonable.
	//
	//		####The *sort* parameter.
	//
	//		If a sort parameter is specified, this is a indication to the datastore to
	//		sort the items in some manner before returning the items.  The array is an array of
	//		javascript objects that must conform to the following format to be applied to the
	//		fetching of items:
	// |	{
	// |		attribute: attribute || attribute-name-string,
	// |		descending: true|false;   // Optional.  Default is false.
	// |	}
	//		Note that when comparing attributes, if an item contains no value for the attribute
	//		(undefined), then it the default ascending sort logic should push it to the bottom
	//		of the list.  In the descending order case, it such items should appear at the top of the list.

	request = request || {};
	if(!request.store){
		request.store = this;
	}

	this._fetchItems(request, lang.hitch(this, "fetchHandler"), lang.hitch(this, "errorHandler"));
	return request;	// Object
};

return simpleFetch;
});

},
'xcitec/xlayout/ContentAreaPaneV2SingleColumn':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dnd/Source",
	"dojo/dnd/Manager",
	"dojo/dom-attr",
	"dojo/on",
  "../xcitecUtil",
  "../layout/ContentAreaPane",
  "../xlayout/_ContentAreaPaneItemV2SingleColumn",
  "../xlayout/QualificationAvatarSingleColumn"
], function (dojo, array, declare, Source, Manager, domAttr, on, xcitecUtil, ContentAreaPane, _ContentAreaPaneItemV2SingleColumn, QualificationAvatarSingleColumn) {

	return declare( ContentAreaPane, {
	    baseClass: 'ContentAreaPane questionContentArea',
	
	    // is in design mode
	    designMode: false,
	    // show delimiter?
	    showDelimiter: false,
	    // order of elements in column
	    columnOrder: null,
	    // parent element
	    parent: null,
	
	    idHolder: null,
	
	    postMixInProperties: function() {
	       this.inherited(arguments);
	       this.designMode = xcitecUtil.str2bool(domAttr.get(this.srcNodeRef, 'designMode'));
	    },
	
	    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode) {
	        var controlContainer = new _ContentAreaPaneItemV2SingleColumn({ controlNode:controlNode, labelNode:labelNode,
	                                                                          designMode: this.designMode, parent: this });
	        if (this.designMode) {
	            domAttr.set(controlContainer.domNode, 'dndType', this.id);
	        }
	        return controlContainer.domNode;
	    },
	
	    // updating order of elements in column
	    updateOrder: function() {
	        var elements = this.getChildren();
	        var _this = this;
	        this.columnOrder = [];
	        array.forEach(elements, function(element) {
	            _this.columnOrder.push(domAttr.get(element.getChildren()[0].domNode, 'id'));
	        });
	    },
	
	    // getter for element's order
	    getColumnOrder: function() {
	        return this.columnOrder;
	    },
	
	    // create source for drag&drop
	    setSourceForDnd: function(/*Array*/ sourceIds) {
	        this.source = new Source(this.domNode, {accept: sourceIds});
	        var _this = this;
	        // connect function for updating order of elements in columns to 'onDrop' event
	        on(this.source, "Drop", function() {
	            _this.parent.updateColumnsOrder();
	            _this.parent.resizeHelperNodes();
	        });
	        Manager.makeAvatar = function() {
	            return new QualificationAvatarSingleColumn(this);
	        };
	    }
	});
});

},
'dijit/_CssStateMixin':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant()
	"dojo/dom-class", // domClass.toggle
	"dojo/has",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/domReady",
	"dojo/touch",
	"dojo/_base/window", // win.body
	"./a11yclick",
	"./registry"
], function(array, declare, dom, domClass, has, lang, on, domReady, touch, win, a11yclick, registry){

	// module:
	//		dijit/_CssStateMixin

	var CssStateMixin = declare("dijit._CssStateMixin", [], {
		// summary:
		//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
		//		state changes, and also higher-level state changes such becoming disabled or selected.
		//
		// description:
		//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
		//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
		//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
		//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
		//
		//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
		//
		//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
		//		within the widget).

		/*=====
		 // cssStateNodes: [protected] Object
		 //		Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that
		 //		need CSS classes applied on mouse hover/press and focus.
		 //
		 //		Each entry in this optional hash is a an attach-point name (like "upArrowButton") mapped to a CSS class name
		 //		(like "dijitUpArrowButton"). Example:
		 //	|		{
		 //	|			"upArrowButton": "dijitUpArrowButton",
		 //	|			"downArrowButton": "dijitDownArrowButton"
		 //	|		}
		 //		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
		 //		is hovered, etc.
		 cssStateNodes: {},
		 =====*/

		// hovering: [readonly] Boolean
		//		True if cursor is over this widget
		hovering: false,

		// active: [readonly] Boolean
		//		True if mouse was pressed while over this widget, and hasn't been released yet
		active: false,

		_applyAttributes: function(){
			// This code would typically be in postCreate(), but putting in _applyAttributes() for
			// performance: so the class changes happen before DOM is inserted into the document.
			// Change back to postCreate() in 2.0.  See #11635.

			this.inherited(arguments);

			// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
			array.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active", "_opened"], function(attr){
				this.watch(attr, lang.hitch(this, "_setStateClass"));
			}, this);

			// Track hover and active mouse events on widget root node, plus possibly on subnodes
			for(var ap in this.cssStateNodes || {}){
				this._trackMouseState(this[ap], this.cssStateNodes[ap]);
			}
			this._trackMouseState(this.domNode, this.baseClass);

			// Set state initially; there's probably no hover/active/focus state but widget might be
			// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
			this._setStateClass();
		},

		_cssMouseEvent: function(/*Event*/ event){
			// summary:
			//		Handler for CSS event on this.domNode. Sets hovering and active properties depending on mouse state,
			//		which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

			if(!this.disabled){
				switch(event.type){
					case "mouseover":
					case "MSPointerOver":
					case "pointerover":
						this._set("hovering", true);
						this._set("active", this._mouseDown);
						break;
					case "mouseout":
					case "MSPointerOut":
					case "pointerout":
						this._set("hovering", false);
						this._set("active", false);
						break;
					case "mousedown":
					case "touchstart":
					case "MSPointerDown":
					case "pointerdown":
					case "keydown":
						this._set("active", true);
						break;
					case "mouseup":
					case "dojotouchend":
					case "MSPointerUp":
					case "pointerup":
					case "keyup":
						this._set("active", false);
						break;
				}
			}
		},

		_setStateClass: function(){
			// summary:
			//		Update the visual state of the widget by setting the css classes on this.domNode
			//		(or this.stateNode if defined) by combining this.baseClass with
			//		various suffixes that represent the current widget state(s).
			//
			// description:
			//		In the case where a widget has multiple
			//		states, it sets the class based on all possible
			//		combinations.  For example, an invalid form widget that is being hovered
			//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
			//
			//		The widget may have one or more of the following states, determined
			//		by this.state, this.checked, this.valid, and this.selected:
			//
			//		- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
			//		- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
			//		- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
			//		- Selected - ex: currently selected tab will have this.selected==true
			//
			//		In addition, it may have one or more of the following states,
			//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
			//
			//		- Disabled	- if the widget is disabled
			//		- Active		- if the mouse (or space/enter key?) is being pressed down
			//		- Focused		- if the widget has focus
			//		- Hover		- if the mouse is over the widget

			// Compute new set of classes
			var newStateClasses = this.baseClass.split(" ");

			function multiply(modifier){
				newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){
					return c + modifier;
				}), "dijit" + modifier);
			}

			if(!this.isLeftToRight()){
				// For RTL mode we need to set an addition class like dijitTextBoxRtl.
				multiply("Rtl");
			}

			var checkedState = this.checked == "mixed" ? "Mixed" : (this.checked ? "Checked" : "");
			if(this.checked){
				multiply(checkedState);
			}
			if(this.state){
				multiply(this.state);
			}
			if(this.selected){
				multiply("Selected");
			}
			if(this._opened){
				multiply("Opened");
			}

			if(this.disabled){
				multiply("Disabled");
			}else if(this.readOnly){
				multiply("ReadOnly");
			}else{
				if(this.active){
					multiply("Active");
				}else if(this.hovering){
					multiply("Hover");
				}
			}

			if(this.focused){
				multiply("Focused");
			}

			// Remove old state classes and add new ones.
			// For performance concerns we only write into domNode.className once.
			var tn = this.stateNode || this.domNode,
				classHash = {};	// set of all classes (state and otherwise) for node

			array.forEach(tn.className.split(" "), function(c){
				classHash[c] = true;
			});

			if("_stateClasses" in this){
				array.forEach(this._stateClasses, function(c){
					delete classHash[c];
				});
			}

			array.forEach(newStateClasses, function(c){
				classHash[c] = true;
			});

			var newClasses = [];
			for(var c in classHash){
				newClasses.push(c);
			}
			tn.className = newClasses.join(" ");

			this._stateClasses = newStateClasses;
		},

		_subnodeCssMouseEvent: function(node, clazz, evt){
			// summary:
			//		Handler for hover/active mouse event on widget's subnode
			if(this.disabled || this.readOnly){
				return;
			}

			function hover(isHovering){
				domClass.toggle(node, clazz + "Hover", isHovering);
			}

			function active(isActive){
				domClass.toggle(node, clazz + "Active", isActive);
			}

			function focused(isFocused){
				domClass.toggle(node, clazz + "Focused", isFocused);
			}

			switch(evt.type){
				case "mouseover":
				case "MSPointerOver":
				case "pointerover":
					hover(true);
					break;
				case "mouseout":
				case "MSPointerOut":
				case "pointerout":
					hover(false);
					active(false);
					break;
				case "mousedown":
				case "touchstart":
				case "MSPointerDown":
				case "pointerdown":
				case "keydown":
					active(true);
					break;
				case "mouseup":
				case "MSPointerUp":
				case "pointerup":
				case "dojotouchend":
				case "keyup":
					active(false);
					break;
				case "focus":
				case "focusin":
					focused(true);
					break;
				case "blur":
				case "focusout":
					focused(false);
					break;
			}
		},

		_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
			// summary:
			//		Track mouse/focus events on specified node and set CSS class on that node to indicate
			//		current state.   Usually not called directly, but via cssStateNodes attribute.
			// description:
			//		Given class=foo, will set the following CSS class on the node
			//
			//		- fooActive: if the user is currently pressing down the mouse button while over the node
			//		- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
			//		- fooFocus: if the node is focused
			//
			//		Note that it won't set any classes if the widget is disabled.
			// node: DomNode
			//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
			//		is handled specially and automatically just by mixing in this class.
			// clazz: String
			//		CSS class name (ex: dijitSliderUpArrow)

			// Flag for listener code below to call this._cssMouseEvent() or this._subnodeCssMouseEvent()
			// when node is hovered/active
			node._cssState = clazz;
		}
	});

	domReady(function(){
		// Document level listener to catch hover etc. events on widget root nodes and subnodes.
		// Note that when the mouse is moved quickly, a single onmouseenter event could signal that multiple widgets
		// have been hovered or unhovered (try test_Accordion.html)

		function pointerHandler(evt, target, relatedTarget){
			// Handler for mouseover, mouseout, a11yclick.press and a11click.release events

			// Poor man's event propagation.  Don't propagate event to ancestors of evt.relatedTarget,
			// to avoid processing mouseout events moving from a widget's domNode to a descendant node;
			// such events shouldn't be interpreted as a mouseleave on the widget.
			if(relatedTarget && dom.isDescendant(relatedTarget, target)){
				return;
			}

			for(var node = target; node && node != relatedTarget; node = node.parentNode){
				// Process any nodes with _cssState property.   They are generally widget root nodes,
				// but could also be sub-nodes within a widget
				if(node._cssState){
					var widget = registry.getEnclosingWidget(node);
					if(widget){
						if(node == widget.domNode){
							// event on the widget's root node
							widget._cssMouseEvent(evt);
						}else{
							// event on widget's sub-node
							widget._subnodeCssMouseEvent(node, node._cssState, evt);
						}
					}
				}
			}
		}

		var body = win.body(), activeNode;

		// Handle pointer related events (i.e. mouse or touch)
		on(body, touch.over, function(evt){
			// Using touch.over rather than mouseover mainly to ignore phantom mouse events on iOS.
			pointerHandler(evt, evt.target, evt.relatedTarget);
		});
		on(body, touch.out, function(evt){
			// Using touch.out rather than mouseout mainly to ignore phantom mouse events on iOS.
			pointerHandler(evt, evt.target, evt.relatedTarget);
		});
		on(body, a11yclick.press, function(evt){
			// Save the a11yclick.press target to reference when the a11yclick.release comes.
			activeNode = evt.target;
			pointerHandler(evt, activeNode)
		});
		on(body, a11yclick.release, function(evt){
			// The release event could come on a separate node than the press event, if for example user slid finger.
			// Reference activeNode to reset the state of the node that got state set in the a11yclick.press handler.
			pointerHandler(evt, activeNode);
			activeNode = null;
		});

		// Track focus events on widget sub-nodes that have been registered via _trackMouseState().
		// However, don't track focus events on the widget root nodes, because focus is tracked via the
		// focus manager (and it's not really tracking focus, but rather tracking that focus is on one of the widget's
		// nodes or a subwidget's node or a popup node, etc.)
		// Remove for 2.0 (if focus CSS needed, just use :focus pseudo-selector).
		on(body, "focusin, focusout", function(evt){
			var node = evt.target;
			if(node._cssState && !node.getAttribute("widgetId")){
				var widget = registry.getEnclosingWidget(node);
				if(widget){
					widget._subnodeCssMouseEvent(node, node._cssState, evt);
				}
			}
		});
	});

	return CssStateMixin;
});

},
'dojo/dnd/Source':function(){
define([
	"../_base/array", "../_base/declare", "../_base/kernel", "../_base/lang",
	"../dom-class", "../dom-geometry", "../mouse", "../ready", "../topic",
	"./common", "./Selector", "./Manager"
], function(array, declare, kernel, lang, domClass, domGeom, mouse, ready, topic,
			dnd, Selector, Manager){

// module:
//		dojo/dnd/Source

/*
	Container property:
		"Horizontal"- if this is the horizontal container
	Source states:
		""			- normal state
		"Moved"		- this source is being moved
		"Copied"	- this source is being copied
	Target states:
		""			- normal state
		"Disabled"	- the target cannot accept an avatar
	Target anchor state:
		""			- item is not selected
		"Before"	- insert point is before the anchor
		"After"		- insert point is after the anchor
*/

/*=====
var __SourceArgs = {
	// summary:
	//		a dict of parameters for DnD Source configuration. Note that any
	//		property on Source elements may be configured, but this is the
	//		short-list
	// isSource: Boolean?
	//		can be used as a DnD source. Defaults to true.
	// accept: Array?
	//		list of accepted types (text strings) for a target; defaults to
	//		["text"]
	// autoSync: Boolean
	//		if true refreshes the node list on every operation; false by default
	// copyOnly: Boolean?
	//		copy items, if true, use a state of Ctrl key otherwise,
	//		see selfCopy and selfAccept for more details
	// delay: Number
	//		the move delay in pixels before detecting a drag; 0 by default
	// horizontal: Boolean?
	//		a horizontal container, if true, vertical otherwise or when omitted
	// selfCopy: Boolean?
	//		copy items by default when dropping on itself,
	//		false by default, works only if copyOnly is true
	// selfAccept: Boolean?
	//		accept its own items when copyOnly is true,
	//		true by default, works only if copyOnly is true
	// withHandles: Boolean?
	//		allows dragging only by handles, false by default
	// generateText: Boolean?
	//		generate text node for drag and drop, true by default
};
=====*/

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dojo/dnd/AutoSource", "dojo/dnd/Target"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

var Source = declare("dojo.dnd.Source", Selector, {
	// summary:
	//		a Source object, which can be used as a DnD source, or a DnD target

	// object attributes (for markup)
	isSource: true,
	horizontal: false,
	copyOnly: false,
	selfCopy: false,
	selfAccept: true,
	skipForm: false,
	withHandles: false,
	autoSync: false,
	delay: 0, // pixels
	accept: ["text"],
	generateText: true,

	constructor: function(/*DOMNode|String*/ node, /*__SourceArgs?*/ params){
		// summary:
		//		a constructor of the Source
		// node:
		//		node or node's id to build the source on
		// params:
		//		any property of this class may be configured via the params
		//		object which is mixed-in to the `dojo/dnd/Source` instance
		lang.mixin(this, lang.mixin({}, params));
		var type = this.accept;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}
		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
		this._lastX = 0;
		this._lastY = 0;
		// states
		this.sourceState  = "";
		if(this.isSource){
			domClass.add(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			domClass.add(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			domClass.add(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			topic.subscribe("/dnd/source/over", lang.hitch(this, "onDndSourceOver")),
			topic.subscribe("/dnd/start",  lang.hitch(this, "onDndStart")),
			topic.subscribe("/dnd/drop",   lang.hitch(this, "onDndDrop")),
			topic.subscribe("/dnd/cancel", lang.hitch(this, "onDndCancel"))
		];
	},

	// methods
	checkAcceptance: function(source, nodes){
		// summary:
		//		checks if the target can accept nodes from this source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		if(this == source){
			return !this.copyOnly || this.selfAccept;
		}
		for(var i = 0; i < nodes.length; ++i){
			var type = source.getItem(nodes[i].id).type;
			// type instanceof Array
			var flag = false;
			for(var j = 0; j < type.length; ++j){
				if(type[j] in this.accept){
					flag = true;
					break;
				}
			}
			if(!flag){
				return false;	// Boolean
			}
		}
		return true;	// Boolean
	},
	copyState: function(keyPressed, self){
		// summary:
		//		Returns true if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		the "copy" key was pressed
		// self: Boolean?
		//		optional flag that means that we are about to drop on itself

		if(keyPressed){ return true; }
		if(arguments.length < 2){
			self = this == Manager.manager().target;
		}
		if(self){
			if(this.copyOnly){
				return this.selfCopy;
			}
		}else{
			return this.copyOnly;
		}
		return false;	// Boolean
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		Source.superclass.destroy.call(this);
		array.forEach(this.topics, function(t){t.remove();});
		this.targetAnchor = null;
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		Source.superclass.onMouseMove.call(this, e);
		var m = Manager.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dnd.getCopyKeyState(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = domGeom.position(this.current, true);
				}
				if(this.horizontal){
					// In LTR mode, the left part of the object means "before", but in RTL mode it means "after".
					before = (e.pageX - this.targetBox.x < this.targetBox.w / 2) == domGeom.isBodyLtr(this.current.ownerDocument);
				}else{
					before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(!this.mouseDown && this._legalMouseDown(e) && (!this.skipForm || !dnd.isFormElement(e))){
			this.mouseDown = true;
			this._lastX = e.pageX;
			this._lastY = e.pageY;
			Source.superclass.onMouseDown.call(this, e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			Source.superclass.onMouseUp.call(this, e);
		}
	},

	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		topic event processor for /dnd/source/over, called when detected a current source
		// source: Object
		//		the source which has the mouse over it
		if(this !== source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = Manager.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		if(this.autoSync){ this.sync(); }
		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.accept && this.checkAcceptance(source, nodes);
		this._changeState("Target", accepted ? "" : "Disabled");
		if(this == source){
			Manager.manager().overSource(this);
		}
		this.isDragging = true;
	},
	onDndDrop: function(source, nodes, copy, target){
		// summary:
		//		topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		// target: Object
		//		the target which accepts items
		if(this == target){
			// this one is for us => move nodes!
			this.onDrop(source, nodes, copy);
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary:
		//		topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},

	// local events
	onDrop: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		if(this != source){
			this.onDropExternal(source, nodes, copy);
		}else{
			this.onDropInternal(nodes, copy);
		}
	},
	onDropExternal: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from an external source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node /*=====, hint =====*/){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node /*=====, hint =====*/){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		this.insertNodes(true, nodes, this.before, this.current);
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	onDropInternal: function(nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from the same target/source
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes within the single source
		if(this.current && this.current.id in this.selection){
			// do nothing
			return;
		}
		if(copy){
			if(this.creator){
				// create new copies of data items
				this._normalizedCreator = function(node, hint){
					return oldCreator.call(this, this.getItem(node.id).data, hint);
				};
			}else{
				// clone nodes
				this._normalizedCreator = function(node/*=====, hint =====*/){
					var t = this.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}
		}else{
			// move nodes
			if(!this.current){
				// do nothing
				return;
			}
			this._normalizedCreator = function(node /*=====, hint =====*/){
				var t = this.getItem(node.id);
				return {node: node, data: t.data, type: t.type};
			};
		}
		this._removeSelection();
		this.insertNodes(true, nodes, this.before, this.current);
		this._normalizedCreator = oldCreator;
	},
	onDraggingOver: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged over this target, and it is not disabled
	},
	onDraggingOut: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged away from this target, and it is not disabled
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		Source.superclass.onOverEvent.call(this);
		Manager.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		Source.superclass.onOutEvent.call(this);
		Manager.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
	},
	_markTargetAnchor: function(before){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary:
		//		removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary:
		//		changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	},
	_legalMouseDown: function(e){
		// summary:
		//		checks if user clicked on "approved" items
		// e: Event
		//		mouse event

		// accept only the left mouse button, or the left finger
		if(e.type != "touchstart" && !mouse.isLeft(e)){ return false; }

		if(!this.withHandles){ return true; }

		// check for handles
		for(var node = e.target; node && node !== this.node; node = node.parentNode){
			if(domClass.contains(node, "dojoDndHandle")){ return true; }
			if(domClass.contains(node, "dojoDndItem") || domClass.contains(node, "dojoDndIgnore")){ break; }
		}
		return false;	// Boolean
	}
});

return Source;

});

},
'xcitec/UserInfoBoxV2':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "./xcitecUtil",
  "./Menu",
  "./MenuItem",
  "./MenuSeparator",
  "./form/InlineDropDownButton"
], function (dojo, declare, lang, domAttr, domClass, domConstruct, query, _Templated, _Widget, xcitecUtil, Menu, MenuItem, MenuSeparator, InlineDropDownButton) {

	var UserInfoBoxV2 = declare( [_Widget, _Templated], {
		stopParser: true, //parser shouldn't parse content - will do it in my own.
		templateString: "<div class='user'>" +
			"<div class='user_nav' data-dojo-attach-point='useractions, containerNode'></div>" +
			"</div>",
		caption: "",
		captionId: "",
	
		_fillContent: function(/*DomNode*/ source){
			// get all predefined actions
			var actions = query(">", this.srcNodeRef);
	
			// create appropriate controls if actions exist
			if(actions.length > 0){
				var _this = this;
				actions.forEach(function(action){
					var innerActions = query(">", action);
					var divContainer = domConstruct.create("div", {}, _this.useractions, "last");
					domClass.add(divContainer, "user_nav_items");
					var caption = domAttr.get(action, "caption");
					caption = caption ? caption : "";
					if(xcitecUtil.str2bool(domAttr.get(action, "userContainer"))){
						caption = _this.caption;
					}
					var iconClass = domAttr.get(action, "iconClass");
					// if there are inner actions - then it's DropDownButton, else - LinkButton
					if(innerActions.length > 0){
						// create xcitec.Menu and attach it to InlineDropDownButton
						var menu = new Menu({});
						innerActions.forEach(function(act){
							var isDisabled = xcitecUtil.str2bool(domAttr.get(act, "disabled"));
							var menuAction = act.onclick;
							var hrefAttr = act.getAttribute("href");
							if(menuAction){
								menu.addChild(new MenuItem({ label: caption, onClick: menuAction, disabled: isDisabled }));
							} else if(hrefAttr){
								menu.addChild(new MenuItem({
									label: caption,onClick: lang.hitch(_this, "hrefClick", hrefAttr, domAttr.get(act, "target")), disabled: isDisabled }));
							} else{
								menu.addChild(new MenuItem({ label: caption, disabled: true }));
							}
							if(xcitecUtil.str2bool(domAttr.get(act, "separate"))){
								menu.addChild(new MenuSeparator());
							}
						});
						var dropDown = new InlineDropDownButton({ caption: caption, dropDown: menu, iconClass: iconClass });
						divContainer.appendChild(dropDown.domNode);
					}
				});
			}
		},
	
		hrefClick: function(destination, target){
			if(target){
				window.open(destination, target);
			} else{
				window.location.href = destination;
			}
		}
	});
	return UserInfoBoxV2;
});

},
'dojo/NodeList-dom':function(){
define(["./_base/kernel", "./query", "./_base/array", "./_base/lang", "./dom-class", "./dom-construct", "./dom-geometry", "./dom-attr", "./dom-style"], function(dojo, query, array, lang, domCls, domCtr, domGeom, domAttr, domStyle){

	// module:
	//		dojo/NodeList-dom.js

	/*=====
	 return function(){
		 // summary:
		 //		Adds DOM related methods to NodeList, and returns NodeList constructor.
	 };
	 =====*/

	var magicGuard = function(a){
		// summary:
		//		the guard function for dojo/dom-attr() and dojo/dom-style()
		return a.length == 1 && (typeof a[0] == "string"); // inline'd type check
	};

	var orphan = function(node){
		// summary:
		//		function to orphan nodes
		var p = node.parentNode;
		if(p){
			p.removeChild(node);
		}
	};
	// FIXME: should we move orphan() to dojo/_base/html?

	var NodeList = query.NodeList,
		awc = NodeList._adaptWithCondition,
		aafe = NodeList._adaptAsForEach,
		aam = NodeList._adaptAsMap;

	function getSet(module){
		return function(node, name, value){
			if(arguments.length == 2){
				return module[typeof name == "string" ? "get" : "set"](node, name);
			}
			// setter
			return module.set(node, name, value);
		};
	}

	lang.extend(NodeList, {
		_normalize: function(/*String||Element||Object||NodeList*/content, /*DOMNode?*/refNode){
			// summary:
			//		normalizes data to an array of items to insert.
			// description:
			//		If content is an object, it can have special properties "template" and
			//		"parse". If "template" is defined, then the template value is run through
			//		dojo/string.substitute (if dojo/string.substitute() has been required elsewhere),
			//		or if templateFunc is a function on the content, that function will be used to
			//		transform the template into a final string to be used for for passing to dojo/dom-construct.toDom().
			//		If content.parse is true, then it is remembered for later, for when the content
			//		nodes are inserted into the DOM. At that point, the nodes will be parsed for widgets
			//		(if dojo/parser has been required elsewhere).

			//Wanted to just use a DocumentFragment, but for the array/NodeList
			//case that meant using cloneNode, but we may not want that.
			//Cloning should only happen if the node operations span
			//multiple refNodes. Also, need a real array, not a NodeList from the
			//DOM since the node movements could change those NodeLists.

			var parse = content.parse === true;

			//Do we have an object that needs to be run through a template?
			if(typeof content.template == "string"){
				var templateFunc = content.templateFunc || (dojo.string && dojo.string.substitute);
				content = templateFunc ? templateFunc(content.template, content) : content;
			}

			var type = (typeof content);
			if(type == "string" || type == "number"){
				content = domCtr.toDom(content, (refNode && refNode.ownerDocument));
				if(content.nodeType == 11){
					//DocumentFragment. It cannot handle cloneNode calls, so pull out the children.
					content = lang._toArray(content.childNodes);
				}else{
					content = [content];
				}
			}else if(!lang.isArrayLike(content)){
				content = [content];
			}else if(!lang.isArray(content)){
				//To get to this point, content is array-like, but
				//not an array, which likely means a DOM NodeList. Convert it now.
				content = lang._toArray(content);
			}

			//Pass around the parse info
			if(parse){
				content._runParse = true;
			}
			return content; //Array
		},

		_cloneNode: function(/*DOMNode*/ node){
			// summary:
			//		private utility to clone a node. Not very interesting in the vanilla
			//		dojo/NodeList case, but delegates could do interesting things like
			//		clone event handlers if that is derivable from the node.
			return node.cloneNode(true);
		},

		_place: function(/*Array*/ary, /*DOMNode*/refNode, /*String*/position, /*Boolean*/useClone){
			// summary:
			//		private utility to handle placing an array of nodes relative to another node.
			// description:
			//		Allows for cloning the nodes in the array, and for
			//		optionally parsing widgets, if ary._runParse is true.

			//Avoid a disallowed operation if trying to do an innerHTML on a non-element node.
			if(refNode.nodeType != 1 && position == "only"){
				return;
			}
			var rNode = refNode, tempNode;

			//Always cycle backwards in case the array is really a
			//DOM NodeList and the DOM operations take it out of the live collection.
			var length = ary.length;
			for(var i = length - 1; i >= 0; i--){
				var node = (useClone ? this._cloneNode(ary[i]) : ary[i]);

				//If need widget parsing, use a temp node, instead of waiting after inserting into
				//real DOM because we need to start widget parsing at one node up from current node,
				//which could cause some already parsed widgets to be parsed again.
				if(ary._runParse && dojo.parser && dojo.parser.parse){
					if(!tempNode){
						tempNode = rNode.ownerDocument.createElement("div");
					}
					tempNode.appendChild(node);
					dojo.parser.parse(tempNode);
					node = tempNode.firstChild;
					while(tempNode.firstChild){
						tempNode.removeChild(tempNode.firstChild);
					}
				}

				if(i == length - 1){
					domCtr.place(node, rNode, position);
				}else{
					rNode.parentNode.insertBefore(node, rNode);
				}
				rNode = node;
			}
		},


		position: aam(domGeom.position),
		/*=====
		position: function(){
			// summary:
			//		Returns border-box objects (x/y/w/h) of all elements in a node list
			//		as an Array (*not* a NodeList). Acts like `dojo/dom-geometry-position`, though
			//		assumes the node passed is each node in this list.

			return dojo.map(this, dojo.position); // Array
		},
		=====*/

		attr: awc(getSet(domAttr), magicGuard),
		/*=====
		attr: function(property, value){
			// summary:
			//		gets or sets the DOM attribute for every element in the
			//		NodeList. See also `dojo/dom-attr`
			// property: String
			//		the attribute to get/set
			// value: String?
			//		optional. The value to set the property to
			// returns:
			//		if no value is passed, the result is an array of attribute values
			//		If a value is passed, the return is this NodeList
			// example:
			//		Make all nodes with a particular class focusable:
			//	|	require(["dojo/query", "dojo/NodeList-dom"], function(query){
			//	|		query(".focusable").attr("tabIndex", -1);
			//	|	});
			// example:
			//		Disable a group of buttons:
			//	|	require(["dojo/query", "dojo/NodeList-dom"], function(query){
			//	|		query("button.group").attr("disabled", true);
			//	|	});
			// example:
			//		innerHTML can be assigned or retrieved as well:
			//	|	// get the innerHTML (as an array) for each list item
			//	|	require(["dojo/query", "dojo/NodeList-dom"], function(query){
			//	|		var ih = query("li.replaceable").attr("innerHTML");
			//	|	});
			return; // dojo/NodeList|Array
		},
		=====*/

		style: awc(getSet(domStyle), magicGuard),
		/*=====
		style: function(property, value){
			// summary:
			//		gets or sets the CSS property for every element in the NodeList
			// property: String
			//		the CSS property to get/set, in JavaScript notation
			//		("lineHieght" instead of "line-height")
			// value: String?
			//		optional. The value to set the property to
			// returns:
			//		if no value is passed, the result is an array of strings.
			//		If a value is passed, the return is this NodeList
			return; // dojo/NodeList
			return; // Array
		},
		=====*/

		addClass: aafe(domCls.add),
		/*=====
		addClass: function(className){
			// summary:
			//		adds the specified class to every node in the list
			// className: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			return; // dojo/NodeList
		},
		=====*/

		removeClass: aafe(domCls.remove),
		/*=====
		removeClass: function(className){
			// summary:
			//		removes the specified class from every node in the list
			// className: String|Array?
			//		An optional String class name to remove, or several space-separated
			//		class names, or an array of class names. If omitted, all class names
			//		will be deleted.
			// returns:
			//		this list
			return; // dojo/NodeList
		},
		=====*/

		toggleClass: aafe(domCls.toggle),
		/*=====
		toggleClass: function(className, condition){
			// summary:
			//		Adds a class to node if not present, or removes if present.
			//		Pass a boolean condition if you want to explicitly add or remove.
			// condition: Boolean?
			//		If passed, true means to add the class, false means to remove.
			// className: String
			//		the CSS class to add
			return; // dojo/NodeList
		},
		=====*/

		replaceClass: aafe(domCls.replace),
		/*=====
		replaceClass: function(addClassStr, removeClassStr){
			// summary:
			//		Replaces one or more classes on a node if not present.
			//		Operates more quickly than calling `removeClass()` and `addClass()`
			// addClassStr: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			// removeClassStr: String|Array?
			//		A String class name to remove, or several space-separated class names,
			//		or an array of class names.
			return; // dojo/NodeList
		 },
		 =====*/

		empty: aafe(domCtr.empty),
		/*=====
		empty: function(){
			// summary:
			//		clears all content from each node in the list. Effectively
			//		equivalent to removing all child nodes from every item in
			//		the list.
			return this.forEach("item.innerHTML='';"); // dojo/NodeList
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		=====*/

		removeAttr: aafe(domAttr.remove),
		/*=====
		 removeAttr: function(name){
			// summary:
			//		Removes an attribute from each node in the list.
			// name: String
			//		the name of the attribute to remove
			return;		// dojo/NodeList
		},
		=====*/

		marginBox: aam(domGeom.getMarginBox),
		/*=====
		marginBox: function(){
			// summary:
			//		Returns margin-box size of nodes
		 	return; // dojo/NodeList
		 },
		 =====*/

		// FIXME: connectPublisher()? connectRunOnce()?

		/*
		destroy: function(){
			// summary:
			//		destroys every item in the list.
			this.forEach(d.destroy);
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		*/

		place: function(/*String||Node*/ queryOrNode, /*String*/ position){
			// summary:
			//		places elements of this node list relative to the first element matched
			//		by queryOrNode. Returns the original NodeList. See: `dojo/dom-construct.place`
			// queryOrNode:
			//		may be a string representing any valid CSS3 selector or a DOM node.
			//		In the selector case, only the first matching element will be used
			//		for relative positioning.
			// position:
			//		can be one of:
			//
			//		-	"last" (default)
			//		-	"first"
			//		-	"before"
			//		-	"after"
			//		-	"only"
			//		-	"replace"
			//
			//		or an offset in the childNodes property
			var item = query(queryOrNode)[0];
			return this.forEach(function(node){ domCtr.place(node, item, position); }); // dojo/NodeList
		},

		orphan: function(/*String?*/ filter){
			// summary:
			//		removes elements in this list that match the filter
			//		from their parents and returns them as a new NodeList.
			// filter:
			//		CSS selector like ".foo" or "div > span"
			// returns:
			//		NodeList containing the orphaned elements
			return (filter ? query._filterResult(this, filter) : this).forEach(orphan); // dojo/NodeList
		},

		adopt: function(/*String||Array||DomNode*/ queryOrListOrNode, /*String?*/ position){
			// summary:
			//		places any/all elements in queryOrListOrNode at a
			//		position relative to the first element in this list.
			//		Returns a dojo/NodeList of the adopted elements.
			// queryOrListOrNode:
			//		a DOM node or a query string or a query result.
			//		Represents the nodes to be adopted relative to the
			//		first element of this NodeList.
			// position:
			//		can be one of:
			//
			//		-	"last" (default)
			//		-	"first"
			//		-	"before"
			//		-	"after"
			//		-	"only"
			//		-	"replace"
			//
			//		or an offset in the childNodes property
			return query(queryOrListOrNode).place(this[0], position)._stash(this);	// dojo/NodeList
		},

		// FIXME: do we need this?
		query: function(/*String*/ queryStr){
			// summary:
			//		Returns a new list whose members match the passed query,
			//		assuming elements of the current NodeList as the root for
			//		each search.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo">
			//	|		<p>
			//	|			bacon is tasty, <span>dontcha think?</span>
			//	|		</p>
			//	|	</div>
			//	|	<div id="bar">
			//	|		<p>great comedians may not be funny <span>in person</span></p>
			//	|	</div>
			//		If we are presented with the following definition for a NodeList:
			//	|	require(["dojo/dom", "dojo/query", "dojo/NodeList-dom"
			//	|	], function(dom, query){
			//	|		var l = new NodeList(dom.byId("foo"), dom.byId("bar"));
			//		it's possible to find all span elements under paragraphs
			//		contained by these elements with this sub-query:
			//	|		var spans = l.query("p span");
			//	|	});

			// FIXME: probably slow
			if(!queryStr){ return this; }
			var ret = new NodeList;
			this.map(function(node){
				// FIXME: why would we ever get undefined here?
				query(queryStr, node).forEach(function(subNode){
					if(subNode !== undefined){
						ret.push(subNode);
					}
				});
			});
			return ret._stash(this);	// dojo/NodeList
		},

		filter: function(/*String|Function*/ filter){
			// summary:
			//		"masks" the built-in javascript filter() method (supported
			//		in Dojo via `dojo.filter`) to support passing a simple
			//		string filter in addition to supporting filtering function
			//		objects.
			// filter:
			//		If a string, a CSS rule like ".thinger" or "div > span".
			// example:
			//		"regular" JS filter syntax as exposed in dojo.filter:
			//	|	require(["dojo/query", "dojo/NodeList-dom"
			//	|	], function(query){
			//	|		query("*").filter(function(item){
			//	|			// highlight every paragraph
			//	|			return (item.nodeName == "p");
			//	|		}).style("backgroundColor", "yellow");
			//	|	});
			// example:
			//	the same filtering using a CSS selector
			//	|	require(["dojo/query", "dojo/NodeList-dom"
			//	|	], function(query){
			//	|		query("*").filter("p").styles("backgroundColor", "yellow");
			//	|	});
			var a = arguments, items = this, start = 0;
			if(typeof filter == "string"){ // inline'd type check
				items = query._filterResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo/NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(array.filter(items, a[start], a[start + 1]), this);	// dojo/NodeList
		},

		/*
		// FIXME: should this be "copyTo" and include parenting info?
		clone: function(){
			// summary:
			//		creates node clones of each element of this list
			//		and returns a new list containing the clones
		},
		*/

		addContent: function(/*String||DomNode||Object||dojo/NodeList*/ content, /*String||Integer?*/ position){
			// summary:
			//		add a node, NodeList or some HTML as a string to every item in the
			//		list.  Returns the original list.
			// description:
			//		a copy of the HTML content is added to each item in the
			//		list, with an optional position argument. If no position
			//		argument is provided, the content is appended to the end of
			//		each item.
			// content:
			//		DOM node, HTML in string format, a NodeList or an Object. If a DOM node or
			//		NodeList, the content will be cloned if the current NodeList has more than one
			//		element. Only the DOM nodes are cloned, no event handlers. If it is an Object,
			//		it should be an object with at "template" String property that has the HTML string
			//		to insert. If dojo.string has already been dojo.required, then dojo.string.substitute
			//		will be used on the "template" to generate the final HTML string. Other allowed
			//		properties on the object are: "parse" if the HTML
			//		string should be parsed for widgets (dojo.require("dojo.parser") to get that
			//		option to work), and "templateFunc" if a template function besides dojo.string.substitute
			//		should be used to transform the "template".
			// position:
			//		can be one of:
			//
			//		-	"last"||"end" (default)
			//		-	"first||"start"
			//		-	"before"
			//		-	"after"
			//		-	"replace" (replaces nodes in this NodeList with new content)
			//		-	"only" (removes other children of the nodes so new content is the only child)
			//
			//		or an offset in the childNodes property
			// example:
			//		appends content to the end if the position is omitted
			//	|	require(["dojo/query", "dojo/NodeList-dom"
			//	|	], function(query){
			//	|		query("h3 > p").addContent("hey there!");
			//	|	});
			// example:
			//		add something to the front of each element that has a
			//		"thinger" property:
			//	|	require(["dojo/query", "dojo/NodeList-dom"
			//	|	], function(query){
			//	|		query("[thinger]").addContent("...", "first");
			//	|	});
			// example:
			//		adds a header before each element of the list
			//	|	require(["dojo/query", "dojo/NodeList-dom"
			//	|	], function(query){
			//	|		query(".note").addContent("<h4>NOTE:</h4>", "before");
			//	|	});
			// example:
			//		add a clone of a DOM node to the end of every element in
			//		the list, removing it from its existing parent.
			//	|	require(["dojo/dom", "dojo/query", "dojo/NodeList-dom"
			//	|	], function(dom, query){
			//	|		query(".note").addContent(dom.byId("foo"));
			//	|	});
			// example:
			//		Append nodes from a templatized string.
			//	|	require(["dojo/string", "dojo/query", "dojo/NodeList-dom"
			//	|	], function(string, query){
			//	|		query(".note").addContent({
			//	|			template: '<b>${id}: </b><span>${name}</span>',
			//	|			id: "user332",
			//	|			name: "Mr. Anderson"
			//	|		});
			//	|	});
			// example:
			//		Append nodes from a templatized string that also has widgets parsed.
			//	|	require(["dojo/string", "dojo/parser", "dojo/query", "dojo/NodeList-dom"
			//	|	], function(string, parser, query){
			//	|		var notes = query(".note").addContent({
			//	|			template: '<button dojoType="dijit/form/Button">${text}</button>',
			//	|			parse: true,
			//	|			text: "Send"
			//	|		});
			//	|	});
			content = this._normalize(content, this[0]);
			for(var i = 0, node; (node = this[i]); i++){
				if(content.length){
					this._place(content, node, position, i > 0);
				}else{
					// if it is an empty array, we empty the target node
					domCtr.empty(node);
				}
			}
			return this; // dojo/NodeList
		}
	});

	return NodeList;
});

},
'xcitec/xlayout/ComplexQuestionV2':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, declare, domAttr, _Templated, _Widget) {

	/* xcitec.xlayout.ComplexQuestionV2 is a control which is used in qualification to
	   show historical answers */
	
	var ComplexQuestionV2 = declare( [_Widget, _Templated], {
	    templateString: "<div class='ComplexQuestionV2' data-dojo-attach-point='containerNode'></div>",
	    
	    designMode: false,
	    
	    score: "",
	    ko: "",
	    critical: "",
	
	    onChangedAttr: null,
	    onChangedNewAttr: null,
	      
	    changedOld: "",
	    
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        this.onChangedAttr = 'onChanged' in this.params;
	        this.onChangedNewAttr = 'onChangedNew' in this.params;
	    },
	      
	    onChanged: function() {
	      
	    },
	    
	    onChangedNew: function() {
	      
	    },
	      
	    isChangedSet: function() {
	        return this.onChangedAttr;  
	    },
	    
	    isChangedNewSet: function() {
	        return this.onChangedNewAttr;  
	    }, 
	         
	    isScoreSet: function() {
	        return ("score" in this.params);
	    },
	
	    isCriticalSet: function() {
	        return ("critical" in this.params);
	    },
	      
	    isChangedOldSet: function() {
	        return ("changedOld" in this.params);
	    },
	    
	    isKoSet: function() {
	        return ("ko" in this.params);
	    }
	});
	return ComplexQuestionV2;
});

},
'dijit/_WidgetBase':function(){
define([
	"require", // require.toUrl
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/config", // config.blankGif
	"dojo/_base/connect", // connect.connect
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.set domAttr.remove
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/dom-construct", // domConstruct.destroy domConstruct.place
	"dojo/dom-geometry", // isBodyLtr
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/has",
	"dojo/_base/kernel",
	"dojo/_base/lang", // mixin(), isArray(), etc.
	"dojo/on",
	"dojo/ready",
	"dojo/Stateful", // Stateful
	"dojo/topic",
	"dojo/_base/window", // win.body()
	"./Destroyable",
	"dojo/has!dojo-bidi?./_BidiMixin",
	"./registry"    // registry.getUniqueId(), registry.findWidgets()
], function(require, array, aspect, config, connect, declare,
			dom, domAttr, domClass, domConstruct, domGeometry, domStyle, has, kernel,
			lang, on, ready, Stateful, topic, win, Destroyable, _BidiMixin, registry){

	// module:
	//		dijit/_WidgetBase

	// Flag to make dijit load modules the app didn't explicitly request, for backwards compatibility
	has.add("dijit-legacy-requires", !kernel.isAsync);

	// Flag to enable support for textdir attribute
	has.add("dojo-bidi", false);


	// For back-compat, remove in 2.0.
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/_base/manager"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	// Nested hash listing attributes for each tag, all strings in lowercase.
	// ex: {"div": {"style": true, "tabindex" true}, "form": { ...
	var tagAttrs = {};

	function getAttrs(obj){
		var ret = {};
		for(var attr in obj){
			ret[attr.toLowerCase()] = true;
		}
		return ret;
	}

	function nonEmptyAttrToDom(attr){
		// summary:
		//		Returns a setter function that copies the attribute to this.domNode,
		//		or removes the attribute from this.domNode, depending on whether the
		//		value is defined or not.
		return function(val){
			domAttr[val ? "set" : "remove"](this.domNode, attr, val);
			this._set(attr, val);
		};
	}

	function isEqual(a, b){
		//	summary:
		//		Function that determines whether two values are identical,
		//		taking into account that NaN is not normally equal to itself
		//		in JS.

		return a === b || (/* a is NaN */ a !== a && /* b is NaN */ b !== b);
	}

	var _WidgetBase = declare("dijit._WidgetBase", [Stateful, Destroyable], {
		// summary:
		//		Future base class for all Dijit widgets.
		// description:
		//		Future base class for all Dijit widgets.
		//		_Widget extends this class adding support for various features needed by desktop.
		//
		//		Provides stubs for widget lifecycle methods for subclasses to extend, like postMixInProperties(), buildRendering(),
		//		postCreate(), startup(), and destroy(), and also public API methods like set(), get(), and watch().
		//
		//		Widgets can provide custom setters/getters for widget attributes, which are called automatically by set(name, value).
		//		For an attribute XXX, define methods _setXXXAttr() and/or _getXXXAttr().
		//
		//		_setXXXAttr can also be a string/hash/array mapping from a widget attribute XXX to the widget's DOMNodes:
		//
		//		- DOM node attribute
		// |		_setFocusAttr: {node: "focusNode", type: "attribute"}
		// |		_setFocusAttr: "focusNode"	(shorthand)
		// |		_setFocusAttr: ""		(shorthand, maps to this.domNode)
		//		Maps this.focus to this.focusNode.focus, or (last example) this.domNode.focus
		//
		//		- DOM node innerHTML
		//	|		_setTitleAttr: { node: "titleNode", type: "innerHTML" }
		//		Maps this.title to this.titleNode.innerHTML
		//
		//		- DOM node innerText
		//	|		_setTitleAttr: { node: "titleNode", type: "innerText" }
		//		Maps this.title to this.titleNode.innerText
		//
		//		- DOM node CSS class
		// |		_setMyClassAttr: { node: "domNode", type: "class" }
		//		Maps this.myClass to this.domNode.className
		//
		//		If the value of _setXXXAttr is an array, then each element in the array matches one of the
		//		formats of the above list.
		//
		//		If the custom setter is null, no action is performed other than saving the new value
		//		in the widget (in this).
		//
		//		If no custom setter is defined for an attribute, then it will be copied
		//		to this.focusNode (if the widget defines a focusNode), or this.domNode otherwise.
		//		That's only done though for attributes that match DOMNode attributes (title,
		//		alt, aria-labelledby, etc.)

		// id: [const] String
		//		A unique, opaque ID string that can be assigned by users or by the
		//		system. If the developer passes an ID which is known not to be
		//		unique, the specified ID is ignored and the system-generated ID is
		//		used instead.
		id: "",
		_setIdAttr: "domNode", // to copy to this.domNode even for auto-generated id's

		// lang: [const] String
		//		Rarely used.  Overrides the default Dojo locale used to render this widget,
		//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
		//		Value must be among the list of locales specified during by the Dojo bootstrap,
		//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
		lang: "",
		// set on domNode even when there's a focus node.	but don't set lang="", since that's invalid.
		_setLangAttr: nonEmptyAttrToDom("lang"),

		// dir: [const] String
		//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
		//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
		//		default direction.
		dir: "",
		// set on domNode even when there's a focus node.	but don't set dir="", since that's invalid.
		_setDirAttr: nonEmptyAttrToDom("dir"), // to set on domNode even when there's a focus node

		// class: String
		//		HTML class attribute
		"class": "",
		_setClassAttr: { node: "domNode", type: "class" },

		// style: String||Object
		//		HTML style attributes as cssText string or name/value hash
		style: "",

		// title: String
		//		HTML title attribute.
		//
		//		For form widgets this specifies a tooltip to display when hovering over
		//		the widget (just like the native HTML title attribute).
		//
		//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
		//		etc., it's used to specify the tab label, accordion pane title, etc.  In this case it's
		//		interpreted as HTML.
		title: "",

		// tooltip: String
		//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
		//		this specifies the tooltip to appear when the mouse is hovered over that text.
		tooltip: "",

		// baseClass: [protected] String
		//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
		//		widget state.
		baseClass: "",

		// srcNodeRef: [readonly] DomNode
		//		pointer to original DOM node
		srcNodeRef: null,

		// domNode: [readonly] DomNode
		//		This is our visible representation of the widget! Other DOM
		//		Nodes may by assigned to other properties, usually through the
		//		template system's data-dojo-attach-point syntax, but the domNode
		//		property is the canonical "top level" node in widget UI.
		domNode: null,

		// containerNode: [readonly] DomNode
		//		Designates where children of the source DOM node will be placed.
		//		"Children" in this case refers to both DOM nodes and widgets.
		//		For example, for myWidget:
		//
		//		|	<div data-dojo-type=myWidget>
		//		|		<b> here's a plain DOM node
		//		|		<span data-dojo-type=subWidget>and a widget</span>
		//		|		<i> and another plain DOM node </i>
		//		|	</div>
		//
		//		containerNode would point to:
		//
		//		|		<b> here's a plain DOM node
		//		|		<span data-dojo-type=subWidget>and a widget</span>
		//		|		<i> and another plain DOM node </i>
		//
		//		In templated widgets, "containerNode" is set via a
		//		data-dojo-attach-point assignment.
		//
		//		containerNode must be defined for any widget that accepts innerHTML
		//		(like ContentPane or BorderContainer or even Button), and conversely
		//		is null for widgets that don't, like TextBox.
		containerNode: null,

		// ownerDocument: [const] Document?
		//		The document this widget belongs to.  If not specified to constructor, will default to
		//		srcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global
		ownerDocument: null,
		_setOwnerDocumentAttr: function(val){
			// this setter is merely to avoid automatically trying to set this.domNode.ownerDocument
			this._set("ownerDocument", val);
		},

		/*=====
		// _started: [readonly] Boolean
		//		startup() has completed.
		_started: false,
		=====*/

		// attributeMap: [protected] Object
		//		Deprecated.	Instead of attributeMap, widget should have a _setXXXAttr attribute
		//		for each XXX attribute to be mapped to the DOM.
		//
		//		attributeMap sets up a "binding" between attributes (aka properties)
		//		of the widget and the widget's DOM.
		//		Changes to widget attributes listed in attributeMap will be
		//		reflected into the DOM.
		//
		//		For example, calling set('title', 'hello')
		//		on a TitlePane will automatically cause the TitlePane's DOM to update
		//		with the new title.
		//
		//		attributeMap is a hash where the key is an attribute of the widget,
		//		and the value reflects a binding to a:
		//
		//		- DOM node attribute
		// |		focus: {node: "focusNode", type: "attribute"}
		//		Maps this.focus to this.focusNode.focus
		//
		//		- DOM node innerHTML
		//	|		title: { node: "titleNode", type: "innerHTML" }
		//		Maps this.title to this.titleNode.innerHTML
		//
		//		- DOM node innerText
		//	|		title: { node: "titleNode", type: "innerText" }
		//		Maps this.title to this.titleNode.innerText
		//
		//		- DOM node CSS class
		// |		myClass: { node: "domNode", type: "class" }
		//		Maps this.myClass to this.domNode.className
		//
		//		If the value is an array, then each element in the array matches one of the
		//		formats of the above list.
		//
		//		There are also some shorthands for backwards compatibility:
		//
		//		- string --> { node: string, type: "attribute" }, for example:
		//
		//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
		//
		//		- "" --> { node: "domNode", type: "attribute" }
		attributeMap: {},

		// _blankGif: [protected] String
		//		Path to a blank 1x1 image.
		//		Used by `<img>` nodes in templates that really get their image via CSS background-image.
		_blankGif: config.blankGif || require.toUrl("dojo/resources/blank.gif"),

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		/*=====
		constructor: function(params, srcNodeRef){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified:
			//
			//		- use srcNodeRef.innerHTML as my contents
			//		- if this is a behavioral widget then apply behavior to that srcNodeRef
			//		- otherwise, replace srcNodeRef with my generated DOM tree
		},
		=====*/

		_introspect: function(){
			// summary:
			//		Collect metadata about this widget (only once per class, not once per instance):
			//
			//			- list of attributes with custom setters, storing in this.constructor._setterAttrs
			//			- generate this.constructor._onMap, mapping names like "mousedown" to functions like onMouseDown

			var ctor = this.constructor;
			if(!ctor._setterAttrs){
				var proto = ctor.prototype,
					attrs = ctor._setterAttrs = [], // attributes with custom setters
					onMap = (ctor._onMap = {});

				// Items in this.attributeMap are like custom setters.  For back-compat, remove for 2.0.
				for(var name in proto.attributeMap){
					attrs.push(name);
				}

				// Loop over widget properties, collecting properties with custom setters and filling in ctor._onMap.
				for(name in proto){
					if(/^on/.test(name)){
						onMap[name.substring(2).toLowerCase()] = name;
					}

					if(/^_set[A-Z](.*)Attr$/.test(name)){
						name = name.charAt(4).toLowerCase() + name.substr(5, name.length - 9);
						if(!proto.attributeMap || !(name in proto.attributeMap)){
							attrs.push(name);
						}
					}
				}

				// Note: this isn't picking up info on properties like aria-label and role, that don't have custom setters
				// but that set() maps to attributes on this.domNode or this.focusNode
			}
		},

		postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
			// summary:
			//		Kicks off widget instantiation.  See create() for details.
			// tags:
			//		private

			// Note that we skip calling this.inherited(), i.e. dojo/Stateful::postscript(), because 1.x widgets don't
			// expect their custom setters to get called until after buildRendering().  Consider changing for 2.0.

			this.create(params, srcNodeRef);
		},

		create: function(params, srcNodeRef){
			// summary:
			//		Kick off the life-cycle of a widget
			// description:
			//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
			//		etc.), some of which of you'll want to override. See http://dojotoolkit.org/reference-guide/dijit/_WidgetBase.html
			//		for a discussion of the widget creation lifecycle.
			//
			//		Of course, adventurous developers could override create entirely, but this should
			//		only be done as a last resort.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified:
			//
			//		- use srcNodeRef.innerHTML as my contents
			//		- if this is a behavioral widget then apply behavior to that srcNodeRef
			//		- otherwise, replace srcNodeRef with my generated DOM tree
			// tags:
			//		private

			// First time widget is instantiated, scan prototype to figure out info about custom setters etc.
			this._introspect();

			// store pointer to original DOM tree
			this.srcNodeRef = dom.byId(srcNodeRef);

			// No longer used, remove for 2.0.
			this._connects = [];
			this._supportingWidgets = [];

			// this is here for back-compat, remove in 2.0 (but check NodeList-instantiate.html test)
			if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){
				this.id = this.srcNodeRef.id;
			}

			// mix in our passed parameters
			if(params){
				this.params = params;
				lang.mixin(this, params);
			}
			this.postMixInProperties();

			// Generate an id for the widget if one wasn't specified, or it was specified as id: undefined.
			// Do this before buildRendering() because it might expect the id to be there.
			if(!this.id){
				this.id = registry.getUniqueId(this.declaredClass.replace(/\./g, "_"));
				if(this.params){
					// if params contains {id: undefined}, prevent _applyAttributes() from processing it
					delete this.params.id;
				}
			}

			// The document and <body> node this widget is associated with
			this.ownerDocument = this.ownerDocument || (this.srcNodeRef ? this.srcNodeRef.ownerDocument : document);
			this.ownerDocumentBody = win.body(this.ownerDocument);

			registry.add(this);

			this.buildRendering();

			var deleteSrcNodeRef;

			if(this.domNode){
				// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
				// Also calls custom setters for all attributes with custom setters.
				this._applyAttributes();

				// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
				// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
				// widget being attached to the DOM since it isn't when a widget is created programmatically like
				// new MyWidget({}).	See #11635.
				var source = this.srcNodeRef;
				if(source && source.parentNode && this.domNode !== source){
					source.parentNode.replaceChild(this.domNode, source);
					deleteSrcNodeRef = true;
				}

				// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
				// assuming that dojo._scopeName even exists in 2.0
				this.domNode.setAttribute("widgetId", this.id);
			}
			this.postCreate();

			// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
			// I think for back-compatibility it isn't deleting srcNodeRef until after postCreate() has run.
			if(deleteSrcNodeRef){
				delete this.srcNodeRef;
			}

			this._created = true;
		},

		_applyAttributes: function(){
			// summary:
			//		Step during widget creation to copy  widget attributes to the
			//		DOM according to attributeMap and _setXXXAttr objects, and also to call
			//		custom _setXXXAttr() methods.
			//
			//		Skips over blank/false attribute values, unless they were explicitly specified
			//		as parameters to the widget, since those are the default anyway,
			//		and setting tabIndex="" is different than not setting tabIndex at all.
			//
			//		For backwards-compatibility reasons attributeMap overrides _setXXXAttr when
			//		_setXXXAttr is a hash/string/array, but _setXXXAttr as a functions override attributeMap.
			// tags:
			//		private

			// Call this.set() for each property that was either specified as parameter to constructor,
			// or is in the list found above.	For correlated properties like value and displayedValue, the one
			// specified as a parameter should take precedence.
			// Particularly important for new DateTextBox({displayedValue: ...}) since DateTextBox's default value is
			// NaN and thus is not ignored like a default value of "".

			// Step 1: Save the current values of the widget properties that were specified as parameters to the constructor.
			// Generally this.foo == this.params.foo, except if postMixInProperties() changed the value of this.foo.
			var params = {};
			for(var key in this.params || {}){
				params[key] = this._get(key);
			}

			// Step 2: Call set() for each property with a non-falsy value that wasn't passed as a parameter to the constructor
			array.forEach(this.constructor._setterAttrs, function(key){
				if(!(key in params)){
					var val = this._get(key);
					if(val){
						this.set(key, val);
					}
				}
			}, this);

			// Step 3: Call set() for each property that was specified as parameter to constructor.
			// Use params hash created above to ignore side effects from step #2 above.
			for(key in params){
				this.set(key, params[key]);
			}
		},

		postMixInProperties: function(){
			// summary:
			//		Called after the parameters to the widget have been read-in,
			//		but before the widget template is instantiated. Especially
			//		useful to set properties that are referenced in the widget
			//		template.
			// tags:
			//		protected
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget, setting this.domNode.
			//		Most widgets will mixin `dijit._TemplatedMixin`, which implements this method.
			// tags:
			//		protected

			if(!this.domNode){
				// Create root node if it wasn't created by _TemplatedMixin
				this.domNode = this.srcNodeRef || this.ownerDocument.createElement("div");
			}

			// baseClass is a single class name or occasionally a space-separated list of names.
			// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
			// TODO: make baseClass custom setter
			if(this.baseClass){
				var classes = this.baseClass.split(" ");
				if(!this.isLeftToRight()){
					classes = classes.concat(array.map(classes, function(name){
						return name + "Rtl";
					}));
				}
				domClass.add(this.domNode, classes);
			}
		},

		postCreate: function(){
			// summary:
			//		Processing after the DOM fragment is created
			// description:
			//		Called after the DOM fragment has been created, but not necessarily
			//		added to the document.  Do not include any operations which rely on
			//		node dimensions or placement.
			// tags:
			//		protected
		},

		startup: function(){
			// summary:
			//		Processing after the DOM fragment is added to the document
			// description:
			//		Called after a widget and its children have been created and added to the page,
			//		and all related widgets have finished their create() cycle, up through postCreate().
			//
			//		Note that startup() may be called while the widget is still hidden, for example if the widget is
			//		inside a hidden dijit/Dialog or an unselected tab of a dijit/layout/TabContainer.
			//		For widgets that need to do layout, it's best to put that layout code inside resize(), and then
			//		extend dijit/layout/_LayoutWidget so that resize() is called when the widget is visible.
			if(this._started){
				return;
			}
			this._started = true;
			array.forEach(this.getChildren(), function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			});
		},

		//////////// DESTROY FUNCTIONS ////////////////////////////////

		destroyRecursive: function(/*Boolean?*/ preserveDom){
			// summary:
			//		Destroy this widget and its descendants
			// description:
			//		This is the generic "destructor" function that all widget users
			//		should call to cleanly discard with a widget. Once a widget is
			//		destroyed, it is removed from the manager object.
			// preserveDom:
			//		If true, this method will leave the original DOM structure
			//		alone of descendant Widgets. Note: This will NOT work with
			//		dijit._TemplatedMixin widgets.

			this._beingDestroyed = true;
			this.destroyDescendants(preserveDom);
			this.destroy(preserveDom);
		},

		destroy: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy this widget, but not its descendants.  Descendants means widgets inside of
			//		this.containerNode.   Will also destroy any resources (including widgets) registered via this.own().
			//
			//		This method will also destroy internal widgets such as those created from a template,
			//		assuming those widgets exist inside of this.domNode but outside of this.containerNode.
			//
			//		For 2.0 it's planned that this method will also destroy descendant widgets, so apps should not
			//		depend on the current ability to destroy a widget without destroying its descendants.   Generally
			//		they should use destroyRecursive() for widgets with children.
			// preserveDom: Boolean
			//		If true, this method will leave the original DOM structure alone.
			//		Note: This will not yet work with _TemplatedMixin widgets

			this._beingDestroyed = true;
			this.uninitialize();

			function destroy(w){
				if(w.destroyRecursive){
					w.destroyRecursive(preserveDom);
				}else if(w.destroy){
					w.destroy(preserveDom);
				}
			}

			// Back-compat, remove for 2.0
			array.forEach(this._connects, lang.hitch(this, "disconnect"));
			array.forEach(this._supportingWidgets, destroy);

			// Destroy supporting widgets, but not child widgets under this.containerNode (for 2.0, destroy child widgets
			// here too).   if() statement is to guard against exception if destroy() called multiple times (see #15815).
			if(this.domNode){
				array.forEach(registry.findWidgets(this.domNode, this.containerNode), destroy);
			}

			this.destroyRendering(preserveDom);
			registry.remove(this.id);
			this._destroyed = true;
		},

		destroyRendering: function(/*Boolean?*/ preserveDom){
			// summary:
			//		Destroys the DOM nodes associated with this widget.
			// preserveDom:
			//		If true, this method will leave the original DOM structure alone
			//		during tear-down. Note: this will not work with _Templated
			//		widgets yet.
			// tags:
			//		protected

			if(this.bgIframe){
				this.bgIframe.destroy(preserveDom);
				delete this.bgIframe;
			}

			if(this.domNode){
				if(preserveDom){
					domAttr.remove(this.domNode, "widgetId");
				}else{
					domConstruct.destroy(this.domNode);
				}
				delete this.domNode;
			}

			if(this.srcNodeRef){
				if(!preserveDom){
					domConstruct.destroy(this.srcNodeRef);
				}
				delete this.srcNodeRef;
			}
		},

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			// summary:
			//		Recursively destroy the children of this widget and their
			//		descendants.
			// preserveDom:
			//		If true, the preserveDom attribute is passed to all descendant
			//		widget's .destroy() method. Not for use with _Templated
			//		widgets.

			// get all direct descendants and destroy them recursively
			array.forEach(this.getChildren(), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
		},

		uninitialize: function(){
			// summary:
			//		Deprecated. Override destroy() instead to implement custom widget tear-down
			//		behavior.
			// tags:
			//		protected
			return false;
		},

		////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

		_setStyleAttr: function(/*String||Object*/ value){
			// summary:
			//		Sets the style attribute of the widget according to value,
			//		which is either a hash like {height: "5px", width: "3px"}
			//		or a plain string
			// description:
			//		Determines which node to set the style on based on style setting
			//		in attributeMap.
			// tags:
			//		protected

			var mapNode = this.domNode;

			// Note: technically we should revert any style setting made in a previous call
			// to his method, but that's difficult to keep track of.

			if(lang.isObject(value)){
				domStyle.set(mapNode, value);
			}else{
				if(mapNode.style.cssText){
					mapNode.style.cssText += "; " + value;
				}else{
					mapNode.style.cssText = value;
				}
			}

			this._set("style", value);
		},

		_attrToDom: function(/*String*/ attr, /*String*/ value, /*Object?*/ commands){
			// summary:
			//		Reflect a widget attribute (title, tabIndex, duration etc.) to
			//		the widget DOM, as specified by commands parameter.
			//		If commands isn't specified then it's looked up from attributeMap.
			//		Note some attributes like "type"
			//		cannot be processed this way as they are not mutable.
			// attr:
			//		Name of member variable (ex: "focusNode" maps to this.focusNode) pointing
			//		to DOMNode inside the widget, or alternately pointing to a subwidget
			// tags:
			//		private

			commands = arguments.length >= 3 ? commands : this.attributeMap[attr];

			array.forEach(lang.isArray(commands) ? commands : [commands], function(command){

				// Get target node and what we are doing to that node
				var mapNode = this[command.node || command || "domNode"];	// DOM node
				var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

				switch(type){
					case "attribute":
						if(lang.isFunction(value)){ // functions execute in the context of the widget
							value = lang.hitch(this, value);
						}

						// Get the name of the DOM node attribute; usually it's the same
						// as the name of the attribute in the widget (attr), but can be overridden.
						// Also maps handler names to lowercase, like onSubmit --> onsubmit
						var attrName = command.attribute ? command.attribute :
							(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

						if(mapNode.tagName){
							// Normal case, mapping to a DOMNode.  Note that modern browsers will have a mapNode.set()
							// method, but for consistency we still call domAttr
							domAttr.set(mapNode, attrName, value);
						}else{
							// mapping to a sub-widget
							mapNode.set(attrName, value);
						}
						break;
					case "innerText":
						mapNode.innerHTML = "";
						mapNode.appendChild(this.ownerDocument.createTextNode(value));
						break;
					case "innerHTML":
						mapNode.innerHTML = value;
						break;
					case "class":
						domClass.replace(mapNode, value, this[attr]);
						break;
				}
			}, this);
		},

		get: function(name){
			// summary:
			//		Get a property from a widget.
			// name:
			//		The property to get.
			// description:
			//		Get a named property from a widget. The property may
			//		potentially be retrieved via a getter method. If no getter is defined, this
			//		just retrieves the object's property.
			//
			//		For example, if the widget has properties `foo` and `bar`
			//		and a method named `_getFooAttr()`, calling:
			//		`myWidget.get("foo")` would be equivalent to calling
			//		`widget._getFooAttr()` and `myWidget.get("bar")`
			//		would be equivalent to the expression
			//		`widget.bar2`
			var names = this._getAttrNames(name);
			return this[names.g] ? this[names.g]() : this._get(name);
		},

		set: function(name, value){
			// summary:
			//		Set a property on a widget
			// name:
			//		The property to set.
			// value:
			//		The value to set in the property.
			// description:
			//		Sets named properties on a widget which may potentially be handled by a
			//		setter in the widget.
			//
			//		For example, if the widget has properties `foo` and `bar`
			//		and a method named `_setFooAttr()`, calling
			//		`myWidget.set("foo", "Howdy!")` would be equivalent to calling
			//		`widget._setFooAttr("Howdy!")` and `myWidget.set("bar", 3)`
			//		would be equivalent to the statement `widget.bar = 3;`
			//
			//		set() may also be called with a hash of name/value pairs, ex:
			//
			//	|	myWidget.set({
			//	|		foo: "Howdy",
			//	|		bar: 3
			//	|	});
			//
			//	This is equivalent to calling `set(foo, "Howdy")` and `set(bar, 3)`

			if(typeof name === "object"){
				for(var x in name){
					this.set(x, name[x]);
				}
				return this;
			}
			var names = this._getAttrNames(name),
				setter = this[names.s];
			if(lang.isFunction(setter)){
				// use the explicit setter
				var result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
			}else{
				// Mapping from widget attribute to DOMNode/subwidget attribute/value/etc.
				// Map according to:
				//		1. attributeMap setting, if one exists (TODO: attributeMap deprecated, remove in 2.0)
				//		2. _setFooAttr: {...} type attribute in the widget (if one exists)
				//		3. apply to focusNode or domNode if standard attribute name, excluding funcs like onClick.
				// Checks if an attribute is a "standard attribute" by whether the DOMNode JS object has a similar
				// attribute name (ex: accept-charset attribute matches jsObject.acceptCharset).
				// Note also that Tree.focusNode() is a function not a DOMNode, so test for that.
				var defaultNode = this.focusNode && !lang.isFunction(this.focusNode) ? "focusNode" : "domNode",
					tag = this[defaultNode] && this[defaultNode].tagName,
					attrsForTag = tag && (tagAttrs[tag] || (tagAttrs[tag] = getAttrs(this[defaultNode]))),
					map = name in this.attributeMap ? this.attributeMap[name] :
						names.s in this ? this[names.s] :
							((attrsForTag && names.l in attrsForTag && typeof value != "function") ||
								/^aria-|^data-|^role$/.test(name)) ? defaultNode : null;
				if(map != null){
					this._attrToDom(name, value, map);
				}
				this._set(name, value);
			}
			return result || this;
		},

		_attrPairNames: {}, // shared between all widgets
		_getAttrNames: function(name){
			// summary:
			//		Helper function for get() and set().
			//		Caches attribute name values so we don't do the string ops every time.
			// tags:
			//		private

			var apn = this._attrPairNames;
			if(apn[name]){
				return apn[name];
			}
			var uc = name.replace(/^[a-z]|-[a-zA-Z]/g, function(c){
				return c.charAt(c.length - 1).toUpperCase();
			});
			return (apn[name] = {
				n: name + "Node",
				s: "_set" + uc + "Attr", // converts dashes to camel case, ex: accept-charset --> _setAcceptCharsetAttr
				g: "_get" + uc + "Attr",
				l: uc.toLowerCase()        // lowercase name w/out dashes, ex: acceptcharset
			});
		},

		_set: function(/*String*/ name, /*anything*/ value){
			// summary:
			//		Helper function to set new value for specified property, and call handlers
			//		registered with watch() if the value has changed.
			var oldValue = this[name];
			this[name] = value;
			if(this._created && !isEqual(oldValue, value)){
				if(this._watchCallbacks){
					this._watchCallbacks(name, oldValue, value);
				}
				this.emit("attrmodified-" + name, {
					detail: {
						prevValue: oldValue,
						newValue: value
					}
				});
			}
		},

		_get: function(/*String*/ name){
			// summary:
			//		Helper function to get value for specified property stored by this._set(),
			//		i.e. for properties with custom setters.  Used mainly by custom getters.
			//
			//		For example, CheckBox._getValueAttr() calls this._get("value").

			// future: return name in this.props ? this.props[name] : this[name];
			return this[name];
		},

		emit: function(/*String*/ type, /*Object?*/ eventObj, /*Array?*/ callbackArgs){
			// summary:
			//		Used by widgets to signal that a synthetic event occurred, ex:
			//	|	myWidget.emit("attrmodified-selectedChildWidget", {}).
			//
			//		Emits an event on this.domNode named type.toLowerCase(), based on eventObj.
			//		Also calls onType() method, if present, and returns value from that method.
			//		By default passes eventObj to callback, but will pass callbackArgs instead, if specified.
			//		Modifies eventObj by adding missing parameters (bubbles, cancelable, widget).
			// tags:
			//		protected

			// Specify fallback values for bubbles, cancelable in case they are not set in eventObj.
			// Also set pointer to widget, although since we can't add a pointer to the widget for native events
			// (see #14729), maybe we shouldn't do it here?
			eventObj = eventObj || {};
			if(eventObj.bubbles === undefined){
				eventObj.bubbles = true;
			}
			if(eventObj.cancelable === undefined){
				eventObj.cancelable = true;
			}
			if(!eventObj.detail){
				eventObj.detail = {};
			}
			eventObj.detail.widget = this;

			var ret, callback = this["on" + type];
			if(callback){
				ret = callback.apply(this, callbackArgs ? callbackArgs : [eventObj]);
			}

			// Emit event, but avoid spurious emit()'s as parent sets properties on child during startup/destroy
			if(this._started && !this._beingDestroyed){
				on.emit(this.domNode, type.toLowerCase(), eventObj);
			}

			return ret;
		},

		on: function(/*String|Function*/ type, /*Function*/ func){
			// summary:
			//		Call specified function when event occurs, ex: myWidget.on("click", function(){ ... }).
			// type:
			//		Name of event (ex: "click") or extension event like touch.press.
			// description:
			//		Call specified function when event `type` occurs, ex: `myWidget.on("click", function(){ ... })`.
			//		Note that the function is not run in any particular scope, so if (for example) you want it to run in the
			//		widget's scope you must do `myWidget.on("click", lang.hitch(myWidget, func))`.

			// For backwards compatibility, if there's an onType() method in the widget then connect to that.
			// Remove in 2.0.
			var widgetMethod = this._onMap(type);
			if(widgetMethod){
				return aspect.after(this, widgetMethod, func, true);
			}

			// Otherwise, just listen for the event on this.domNode.
			return this.own(on(this.domNode, type, func))[0];
		},

		_onMap: function(/*String|Function*/ type){
			// summary:
			//		Maps on() type parameter (ex: "mousemove") to method name (ex: "onMouseMove").
			//		If type is a synthetic event like touch.press then returns undefined.
			var ctor = this.constructor, map = ctor._onMap;
			if(!map){
				map = (ctor._onMap = {});
				for(var attr in ctor.prototype){
					if(/^on/.test(attr)){
						map[attr.replace(/^on/, "").toLowerCase()] = attr;
					}
				}
			}
			return map[typeof type == "string" && type.toLowerCase()];	// String
		},

		toString: function(){
			// summary:
			//		Returns a string that represents the widget.
			// description:
			//		When a widget is cast to a string, this method will be used to generate the
			//		output. Currently, it does not implement any sort of reversible
			//		serialization.
			return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
		},

		getChildren: function(){
			// summary:
			//		Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent
			//		is this widget.   Note that it does not return all descendants, but rather just direct children.
			//		Analogous to [Node.childNodes](https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes),
			//		except containing widgets rather than DOMNodes.
			//
			//		The result intentionally excludes internally created widgets (a.k.a. supporting widgets)
			//		outside of this.containerNode.
			//
			//		Note that the array returned is a simple array.  Application code should not assume
			//		existence of methods like forEach().

			return this.containerNode ? registry.findWidgets(this.containerNode) : []; // dijit/_WidgetBase[]
		},

		getParent: function(){
			// summary:
			//		Returns the parent widget of this widget.

			return registry.getEnclosingWidget(this.domNode.parentNode);
		},

		connect: function(/*Object|null*/ obj, /*String|Function*/ event, /*String|Function*/ method){
			// summary:
			//		Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.
			//
			//		Connects specified obj/event to specified method of this object
			//		and registers for disconnect() on widget destroy.
			//
			//		Provide widget-specific analog to dojo.connect, except with the
			//		implicit use of this widget as the target object.
			//		Events connected with `this.connect` are disconnected upon
			//		destruction.
			// returns:
			//		A handle that can be passed to `disconnect` in order to disconnect before
			//		the widget is destroyed.
			// example:
			//	|	var btn = new Button();
			//	|	// when foo.bar() is called, call the listener we're going to
			//	|	// provide in the scope of btn
			//	|	btn.connect(foo, "bar", function(){
			//	|		console.debug(this.toString());
			//	|	});
			// tags:
			//		protected

			return this.own(connect.connect(obj, event, this, method))[0];	// handle
		},

		disconnect: function(handle){
			// summary:
			//		Deprecated, will be removed in 2.0, use handle.remove() instead.
			//
			//		Disconnects handle created by `connect`.
			// tags:
			//		protected

			handle.remove();
		},

		subscribe: function(t, method){
			// summary:
			//		Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.
			//
			//		Subscribes to the specified topic and calls the specified method
			//		of this object and registers for unsubscribe() on widget destroy.
			//
			//		Provide widget-specific analog to dojo.subscribe, except with the
			//		implicit use of this widget as the target object.
			// t: String
			//		The topic
			// method: Function
			//		The callback
			// example:
			//	|	var btn = new Button();
			//	|	// when /my/topic is published, this button changes its label to
			//	|	// be the parameter of the topic.
			//	|	btn.subscribe("/my/topic", function(v){
			//	|		this.set("label", v);
			//	|	});
			// tags:
			//		protected
			return this.own(topic.subscribe(t, lang.hitch(this, method)))[0];	// handle
		},

		unsubscribe: function(/*Object*/ handle){
			// summary:
			//		Deprecated, will be removed in 2.0, use handle.remove() instead.
			//
			//		Unsubscribes handle created by this.subscribe.
			//		Also removes handle from this widget's list of subscriptions
			// tags:
			//		protected

			handle.remove();
		},

		isLeftToRight: function(){
			// summary:
			//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
			// tags:
			//		protected
			return this.dir ? (this.dir == "ltr") : domGeometry.isBodyLtr(this.ownerDocument); //Boolean
		},

		isFocusable: function(){
			// summary:
			//		Return true if this widget can currently be focused
			//		and false if not
			return this.focus && (domStyle.get(this.domNode, "display") != "none");
		},

		placeAt: function(/* String|DomNode|_Widget */ reference, /* String|Int? */ position){
			// summary:
			//		Place this widget somewhere in the DOM based
			//		on standard domConstruct.place() conventions.
			// description:
			//		A convenience function provided in all _Widgets, providing a simple
			//		shorthand mechanism to put an existing (or newly created) Widget
			//		somewhere in the dom, and allow chaining.
			// reference:
			//		Widget, DOMNode, or id of widget or DOMNode
			// position:
			//		If reference is a widget (or id of widget), and that widget has an ".addChild" method,
			//		it will be called passing this widget instance into that method, supplying the optional
			//		position index passed.  In this case position (if specified) should be an integer.
			//
			//		If reference is a DOMNode (or id matching a DOMNode but not a widget),
			//		the position argument can be a numeric index or a string
			//		"first", "last", "before", or "after", same as dojo/dom-construct::place().
			// returns: dijit/_WidgetBase
			//		Provides a useful return of the newly created dijit._Widget instance so you
			//		can "chain" this function by instantiating, placing, then saving the return value
			//		to a variable.
			// example:
			//	|	// create a Button with no srcNodeRef, and place it in the body:
			//	|	var button = new Button({ label:"click" }).placeAt(win.body());
			//	|	// now, 'button' is still the widget reference to the newly created button
			//	|	button.on("click", function(e){ console.log('click'); }));
			// example:
			//	|	// create a button out of a node with id="src" and append it to id="wrapper":
			//	|	var button = new Button({},"src").placeAt("wrapper");
			// example:
			//	|	// place a new button as the first element of some div
			//	|	var button = new Button({ label:"click" }).placeAt("wrapper","first");
			// example:
			//	|	// create a contentpane and add it to a TabContainer
			//	|	var tc = dijit.byId("myTabs");
			//	|	new ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

			var refWidget = !reference.tagName && registry.byId(reference);
			if(refWidget && refWidget.addChild && (!position || typeof position === "number")){
				// Adding this to refWidget and can use refWidget.addChild() to handle everything.
				refWidget.addChild(this, position);
			}else{
				// "reference" is a plain DOMNode, or we can't use refWidget.addChild().   Use domConstruct.place() and
				// target refWidget.containerNode for nested placement (position==number, "first", "last", "only"), and
				// refWidget.domNode otherwise ("after"/"before"/"replace").  (But not supported officially, see #14946.)
				var ref = refWidget ?
					(refWidget.containerNode && !/after|before|replace/.test(position || "") ?
						refWidget.containerNode : refWidget.domNode) : dom.byId(reference, this.ownerDocument);
				domConstruct.place(this.domNode, ref, position);

				// Start this iff it has a parent widget that's already started.
				// TODO: for 2.0 maybe it should also start the widget when this.getParent() returns null??
				if(!this._started && (this.getParent() || {})._started){
					this.startup();
				}
			}
			return this;
		},

		defer: function(fcn, delay){
			// summary:
			//		Wrapper to setTimeout to avoid deferred functions executing
			//		after the originating widget has been destroyed.
			//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
			// fcn: Function
			//		Function reference.
			// delay: Number?
			//		Delay, defaults to 0.
			// tags:
			//		protected

			var timer = setTimeout(lang.hitch(this,
				function(){
					if(!timer){
						return;
					}
					timer = null;
					if(!this._destroyed){
						lang.hitch(this, fcn)();
					}
				}),
				delay || 0
			);
			return {
				remove: function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
			};
		}
	});

	if(has("dojo-bidi")){
		_WidgetBase.extend(_BidiMixin);
	}

	return _WidgetBase;
});

},
'dojo/cookie':function(){
define(["./_base/kernel", "./regexp"], function(dojo, regexp){

// module:
//		dojo/cookie

/*=====
var __cookieProps = {
	// expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes.
	// path: String?
	//		The path to use for the cookie.
	// domain: String?
	//		The domain to use for the cookie.
	// secure: Boolean?
	//		Whether to only send the cookie on secure connections
};
=====*/


dojo.cookie = function(/*String*/name, /*String?*/ value, /*__cookieProps?*/ props){
	// summary:
	//		Get or set a cookie.
	// description:
	//		If one argument is passed, returns the value of the cookie
	//		For two or more arguments, acts as a setter.
	// name:
	//		Name of the cookie
	// value:
	//		Value for the cookie
	// props:
	//		Properties for the cookie
	// example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	require(["dojo/cookie", "dojo/json"], function(cookie, json){
	//	|		cookie("configObj", json.stringify(config, {expires: 5 }));
	//	|	});
	//
	// example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	require(["dojo/cookie", "dojo/json"], function(cookie, json){
	//	|		config = json.parse(cookie("configObj"));
	//	|	});
	//
	// example:
	//		delete a cookie:
	//	|	require(["dojo/cookie"], function(cookie){
	//	|		cookie("configObj", null, {expires: -1});
	//	|	});
	var c = document.cookie, ret;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + regexp.escapeString(name) + "=([^;]*)"));
		ret = matches ? decodeURIComponent(matches[1]) : undefined; 
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
	return ret; // String|undefined
};

dojo.cookie.isSupported = function(){
	// summary:
	//		Use to determine if the current browser supports cookies or not.
	//
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

return dojo.cookie;
});

},
'xcitec/MenuSeparator':function(){
define([
	"dojo/_base/declare",
	"dijit/MenuSeparator"
], function (declare, MenuSeparator) {

	var MenuSeparator = declare( MenuSeparator, {
	    
	});
	return MenuSeparator;
});
},
'xcitec/form/Rating':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/query",
	"dojo/string",
	"dijit/Tooltip",
	"dijit/form/_FormWidget"
], function (dojo, array, declare, lang, domAttr, domClass, query, string, Tooltip, _FormWidget) {

	var Rating = declare( _FormWidget,{
		// summary:
		//		A widget for rating using stars.
	
		templateString: null,

    // readOnly: Boolean
    //		Should this widget respond to user input?
    //		In markup, this is specified as "readOnly".
    //		Similar to disabled except readOnly form values are submitted.
    readOnly: false,
	
		// numStars: Integer/Float
		//		The number of stars to show, default is 0.
		numStars: 0,
		// value: Integer/Float
		//		The current value of the Rating
		value: "",
	
	    // array of star's labels for tooltip
	    labels: null,
	
	    // array of star's values
	    values: null,
	
	    // domNode of currently hovering star (for purpose of tooltip showing/hiding)
	    currentStar: null,
	
	    // position of tooltip
	//    tooltipPosition: ["before", "below"],
	
		constructor:function(/*Object*/params){
			// Build the templateString. The number of stars is given by this.numStars,
			// which is normally an attribute to the widget node.
			lang.mixin(this, params);
		},
	
		postMixInProperties: function(){
			this.labels = [];
			this.values = [];
			this.inherited(arguments);
		},
	
	    buildRendering: function() {
			var component = (this.srcNodeRef.localName=="div")?"div":"option";
			var stars = query("> " + component, this.srcNodeRef);
	        var _this = this;
	        if (stars.length > 0) {
	            _this.values = stars.map(function(node){
					return domAttr.get(node, "value");
				});
	            _this.labels = stars.map(function(node){
					return domAttr.get(node, "label");
				});
	            stars.forEach(function(node, index) {
	                if (domAttr.get(node, "selected") == "true") {
	                    _this.value = index + 1;
	                }
	            });
	            this.numStars = stars.length;
	        }
	        var tpl = '<div data-dojo-attach-point="domNode" class="xcitecRating dijitInline">' +
						'<input type="hidden" value="0" data-dojo-attach-point="focusNode"/><ul>${stars}</ul>' +
					'</div>';
			// The value-attribute is used to "read" the value for processing in the widget class
			var starTpl = '<li class="xcitecRatingStar dijitInline" data-dojo-attach-event="onclick:onStarClick,onmouseover:_onMouseOver,onmouseout:_onMouseOut" value="${value}"></li>';
			var rendered = "";
			for(var i = 0; i < this.numStars; i++){
				rendered += string.substitute(starTpl, {value:i+1});
			}
			this.templateString = string.substitute(tpl, {stars:rendered});
	        this.inherited(arguments);
	    },
	
		postCreate: function(){
			this.inherited(arguments);
	        var _this = this;
	        domAttr.set(this.focusNode, 'name', this.name);
	        this._renderStars(this.value);
	        var stars = query(".xcitecRatingStar", this.domNode);
	        if (!(this.disabled || this.readOnly)) {
	            stars.forEach(function(node, index) {
	                domAttr.set(node, "title", _this.labels[index])
	            });
	        }
	
	        if (this.value != "") {
	            domAttr.set(this.focusNode, 'value', this.values[this.value - 1]);
	        } else {
	            domAttr.set(this.focusNode, 'value', "");
	        }
		},
	
	    _onMouseOver: function(evt){
	        if (this.readOnly || this.disabled) {
	            return;
	        }
	        var hoverValue = +domAttr.get(evt.target, "value");
	        this.onMouseOver(evt, hoverValue);
	        this._renderStars(hoverValue, true);
		},
	
		_onMouseOut: function(){
	        if (this.readOnly || this.disabled) {
	            return;
	        }
	        this._renderStars(this.value);
		},
	
		_renderStars: function(value, hover){
			// summary: Render the stars depending on the value.
	        var _this = this;
	        var stars = query(".xcitecRatingStar", this.domNode);
			array.forEach(stars, function(star, i){
				if(i + 1 > value){
					domClass.remove(star, "xcitecRatingStarHover");
					domClass.remove(star, "xcitecRatingStarChecked");
	                if (_this.disabled) {
	                    domClass.add(star, "xcitecRatingStarReadOnly");
	                }
				}else{
					domClass.remove(star, "xcitecRatingStar" + (hover ? "Checked" : "Hover"));
					domClass.add(star, "xcitecRatingStar" + (hover ? "Hover" : "Checked"));
	                if (_this.disabled) {
	                    domClass.add(star, "xcitecRatingStarCheckedReadOnly");
	                }
				}
			});
		},
	
		onStarClick:function(/* Event */evt){
			// summary: Connect on this method to get noticed when a star was clicked.
			// example: dojo.connect(widget, "onStarClick", function(event){ ... })
	        if (this.readOnly || this.disabled) {
	            return;
	        }
			var newVal = +domAttr.get(evt.target, "value");
			this.setAttribute("value", newVal == this.value ? 0 : newVal);
			this._renderStars(this.value);
      this.onchange();
		},
	
		onMouseOver: function(){
			// summary: Connect here, the value is passed to this function as the second parameter!
		},

    onchange: function(){
      // onchange function for setting by user
    },
	
		setAttribute: function(/*String*/key, /**/value){
			// summary: When calling setAttribute("value", 4), set the value and render the stars accordingly.
			this.inherited("set", arguments);
			if (key=="value"){
				this._renderStars(this.value);
			}
	        if (this.value != "") {
	            domAttr.set(this.focusNode, 'value', this.values[this.value - 1]);
	        } else {
	            domAttr.set(this.focusNode, 'value', "");
	        }
		}
	});
	return Rating;
});
},
'xcitec/xlayout/QuestionV2':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-class",
	"../layout/Chapter",
  "./_QuestionV2Item"
], function (dojo, declare, domAttr, domClass, Chapter, _QuestionV2Item) {

	/* xcitec.xlayout.QuestionV2 is a layout control which is used to show questions in qualification */
	
	var QuestionV2 = declare( Chapter, {
	    baseClass: "fieldsetChapter questionV2",
	    useLabelAsCaption: false,
	    hasOwnLabelProcessing: false,
	
	   // is onDelete attrinbute set in markup
	    onDeleteAttr: null,
	    onSettingsAttr: null,
	    onChangedAttr: null,
	    onChangedNewAttr: null,
	
	    designMode: false,
	
	    score: "",
	    ko: "",
	    critical: "",
	
	    changedOld: "",
	
	    hidden: false,
	
	    required: false,

        //value to adapt previews labelHolderColumn if current widget has no label
        newRowSpan: 1,
        _labelHolderColumn: null,

        _processChildren: function(/*DomNode*/ source){
            this.inherited(arguments);

            if(this.newRowSpan>1){
                this._labelHolderColumn.rowSpan = this.newRowSpan;
            }
        },

	    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode) {
            var noLabelMode = this._labelHolderColumn && !((domAttr.get(controlNode, "controlView") && domAttr.get(controlNode, "controlView").toUpperCase() == "WIDE") || domAttr.get(controlNode, "label"));
            if (noLabelMode) {
                this.newRowSpan++;
            }

	        var controlHolderWidget = new _QuestionV2Item({ controlNode:controlNode,
	                                                                       labelNode:labelNode, designMode: this.designMode, noLabelMode: noLabelMode});
            if(!noLabelMode){
                if(this.newRowSpan>1){
                    this._labelHolderColumn.rowSpan = this.newRowSpan;
                }
                this.newRowSpan = 1;
                this._labelHolderColumn = controlHolderWidget.labelHolderColumnNode;
            }
	        return controlHolderWidget.domNode;
	    },    
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        this.onDeleteAttr = 'onDelete' in this.params;
	        this.onSettingsAttr = 'onSettings' in this.params;
	        this.onChangedAttr = 'onChanged' in this.params;
	        this.onChangedNewAttr = 'onChangedNew' in this.params;
	        if (this.required) {
	            this.baseClass = this.baseClass + " dijitXcitecQuestionRequired";
	        }
	    },
	
	    postCreate: function(){
	    this.inherited(arguments);
	        if(this.hidden){
	           domClass.add(this.domNode.parentNode, "questionHidden");
	        }
	    },
	
	    
	    
	    /* start declaring actions */
	    onDelete: function() {
	
	    },
	
	    onSettings: function() {
	
	    },
	    
	    onChanged: function() {
	      
	    },
	    
	    onChangedNew: function() {
	      
	    }, 
	    /* end declaring actions */
	
	    // getter for onDelete attribute
	    isDeleteSet: function() {
	        return this.onDeleteAttr;
	    },
	
	    isSettingsSet: function() {
	        return this.onSettingsAttr; 
	    },
	    
	    isChangedSet: function() {
	        return this.onChangedAttr;  
	    },
	    
	    isChangedNewSet: function() {
	        return this.onChangedNewAttr;  
	    }, 
	
	    isScoreSet: function() {
	        return ("score" in this.params);
	    },
	
	    isCriticalSet: function() {
	        return ("critical" in this.params);
	    },
	    
	    isChangedOldSet: function() {
	        return ("changedOld" in this.params);
	    },
	    
	    isKoSet: function() {
	        return ("ko" in this.params);
	    }
	});

	return QuestionV2;
});

},
'xcitec/grid/Avatar':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dnd/Avatar",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dom-geometry"
], function (dojo, declare, Avatar, domAttr, domConstruct, domStyle, geometry) {

	var Avatar = declare( Avatar, {
	    construct: function() {
	        // summary: a constructor function;
	        //	it is separate so it can be (dynamically) overwritten in case of need
	        var a = domConstruct.create("table", {
	            "class": "dojoDndAvatar",
	            style: {
	                position: "absolute",
	                zIndex:   "1999",
	                margin:   "0px"
	            }
	        }),
	                b = domConstruct.create("tbody", null, a),
	                k = Math.min(5, this.manager.nodes.length), i = 0,
	                source = this.manager.source, node;
	        for (; i < k; ++i) {
	            if (source.creator) {
	                // create an avatar representation of the node
	                node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
	            } else {
	                // or just clone the node and hope it works
	                node = this.manager.nodes[i].cloneNode(true);
	                if (node.tagName.toLowerCase() == "tr") {
	                    // insert extra table nodes
	                    var table = domConstruct.create("table"),
	                            tbody = domConstruct.create("tbody", null, table);
	                    tbody.appendChild(node);
	                    node = table;
	                }
	            }
	            node.id = "";
	            tr = domConstruct.create("tr", null, b);
	            td = domConstruct.create("td", null, tr);
	            td.appendChild(node);
	            domAttr.set(tr, {
	                "class": "dojoDndAvatarItem",
	                style: {opacity: (9 - i) / 10}
	            });
	        }
	        this.node = a;
	        // set width to Avatar image according to column width 
	        var cellWidth = geometry.getMarginBox(this.manager.nodes[0]).w;
	        if (cellWidth > 300) {
	            domStyle.set(this.node, "width", "300px");
	        } else {
	            domStyle.set(this.node, "width", cellWidth + "px");
	        }
	    },
	
	    destroy: function() {
	        // summary: a desctructor for the avatar, called to remove all references so it can be garbage-collected
	        domConstruct.destroy(this.node);
	        this.node = false;
	    }
	});
	return Avatar;
});
},
'dojo/_base/fx':function(){
define(["./kernel", "./config", /*===== "./declare", =====*/ "./lang", "../Evented", "./Color", "../aspect", "../sniff", "../dom", "../dom-style"],
	function(dojo, config, /*===== declare, =====*/ lang, Evented, Color, aspect, has, dom, style){
	// module:
	//		dojo/_base/fx
	// notes:
	//		Animation loosely package based on Dan Pupius' work, contributed under CLA; see
	//		http://pupius.co.uk/js/Toolkit.Drawing.js

	var _mixin = lang.mixin;

	// Module export
	var basefx = {
		// summary:
		//		This module defines the base dojo/_base/fx implementation.
	};

	var _Line = basefx._Line = function(/*int*/ start, /*int*/ end){
		// summary:
		//		Object used to generate values from a start value to an end value
		// start: int
		//		Beginning value for range
		// end: int
		//		Ending value for range
		this.start = start;
		this.end = end;
	};

	_Line.prototype.getValue = function(/*float*/ n){
		// summary:
		//		Returns the point on the line
		// n:
		//		a floating point number greater than 0 and less than 1
		return ((this.end - this.start) * n) + this.start; // Decimal
	};

	var Animation = basefx.Animation = function(args){
		// summary:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states.
		// description:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states. Nearly all dojo animation functions
		//		return an instance of this method, usually without calling the
		//		.play() method beforehand. Therefore, you will likely need to
		//		call .play() on instances of `Animation` when one is
		//		returned.
		// args: Object
		//		The 'magic argument', mixing all the properties into this
		//		animation instance.

		_mixin(this, args);
		if(lang.isArray(this.curve)){
			this.curve = new _Line(this.curve[0], this.curve[1]);
		}

	};
	Animation.prototype = new Evented();

	lang.extend(Animation, {
		// duration: Integer
		//		The time in milliseconds the animation will take to run
		duration: 350,

	/*=====
		// curve: _Line|Array
		//		A two element array of start and end values, or a `_Line` instance to be
		//		used in the Animation.
		curve: null,

		// easing: Function?
		//		A Function to adjust the acceleration (or deceleration) of the progress
		//		across a _Line
		easing: null,
	=====*/

		// repeat: Integer?
		//		The number of times to loop the animation
		repeat: 0,

		// rate: Integer?
		//		the time in milliseconds to wait before advancing to next frame
		//		(used as a fps timer: 1000/rate = fps)
		rate: 20 /* 50 fps */,

	/*=====
		// delay: Integer?
		//		The time in milliseconds to wait before starting animation after it
		//		has been .play()'ed
		delay: null,

		// beforeBegin: Event?
		//		Synthetic event fired before a Animation begins playing (synchronous)
		beforeBegin: null,

		// onBegin: Event?
		//		Synthetic event fired as a Animation begins playing (useful?)
		onBegin: null,

		// onAnimate: Event?
		//		Synthetic event fired at each interval of the Animation
		onAnimate: null,

		// onEnd: Event?
		//		Synthetic event fired after the final frame of the Animation
		onEnd: null,

		// onPlay: Event?
		//		Synthetic event fired any time the Animation is play()'ed
		onPlay: null,

		// onPause: Event?
		//		Synthetic event fired when the Animation is paused
		onPause: null,

		// onStop: Event
		//		Synthetic event fires when the Animation is stopped
		onStop: null,

	=====*/

		_percent: 0,
		_startRepeatCount: 0,

		_getStep: function(){
			var _p = this._percent,
				_e = this.easing
			;
			return _e ? _e(_p) : _p;
		},
		_fire: function(/*Event*/ evt, /*Array?*/ args){
			// summary:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			// description:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			//		Fires the callback in the scope of this Animation
			//		instance.
			// evt:
			//		The event to fire.
			// args:
			//		The arguments to pass to the event.
			var a = args||[];
			if(this[evt]){
				if(config.debugAtAllCosts){
					this[evt].apply(this, a);
				}else{
					try{
						this[evt].apply(this, a);
					}catch(e){
						// squelch and log because we shouldn't allow exceptions in
						// synthetic event handlers to cause the internal timer to run
						// amuck, potentially pegging the CPU. I'm not a fan of this
						// squelch, but hopefully logging will make it clear what's
						// going on
						console.error("exception in animation handler for:", evt);
						console.error(e);
					}
				}
			}
			return this; // Animation
		},

		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			// summary:
			//		Start the animation.
			// delay:
			//		How many milliseconds to delay before starting.
			// gotoStart:
			//		If true, starts the animation from the beginning; otherwise,
			//		starts it from its current position.
			// returns: Animation
			//		The instance to allow chaining.

			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(gotoStart){
				_t._stopTimer();
				_t._active = _t._paused = false;
				_t._percent = 0;
			}else if(_t._active && !_t._paused){
				return _t;
			}

			_t._fire("beforeBegin", [_t.node]);

			var de = delay || _t.delay,
				_p = lang.hitch(_t, "_play", gotoStart);

			if(de > 0){
				_t._delayTimer = setTimeout(_p, de);
				return _t;
			}
			_p();
			return _t;	// Animation
		},

		_play: function(gotoStart){
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._startTime = new Date().valueOf();
			if(_t._paused){
				_t._startTime -= _t.duration * _t._percent;
			}

			_t._active = true;
			_t._paused = false;
			var value = _t.curve.getValue(_t._getStep());
			if(!_t._percent){
				if(!_t._startRepeatCount){
					_t._startRepeatCount = _t.repeat;
				}
				_t._fire("onBegin", [value]);
			}

			_t._fire("onPlay", [value]);

			_t._cycle();
			return _t; // Animation
		},

		pause: function(){
			// summary:
			//		Pauses a running animation.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._stopTimer();
			if(!_t._active){ return _t; /*Animation*/ }
			_t._paused = true;
			_t._fire("onPause", [_t.curve.getValue(_t._getStep())]);
			return _t; // Animation
		},

		gotoPercent: function(/*Decimal*/ percent, /*Boolean?*/ andPlay){
			// summary:
			//		Sets the progress of the animation.
			// percent:
			//		A percentage in decimal notation (between and including 0.0 and 1.0).
			// andPlay:
			//		If true, play the animation after setting the progress.
			var _t = this;
			_t._stopTimer();
			_t._active = _t._paused = true;
			_t._percent = percent;
			if(andPlay){ _t.play(); }
			return _t; // Animation
		},

		stop: function(/*boolean?*/ gotoEnd){
			// summary:
			//		Stops a running animation.
			// gotoEnd:
			//		If true, the animation will end.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(!_t._timer){ return _t; /* Animation */ }
			_t._stopTimer();
			if(gotoEnd){
				_t._percent = 1;
			}
			_t._fire("onStop", [_t.curve.getValue(_t._getStep())]);
			_t._active = _t._paused = false;
			return _t; // Animation
		},

		status: function(){
			// summary:
			//		Returns a string token representation of the status of
			//		the animation, one of: "paused", "playing", "stopped"
			if(this._active){
				return this._paused ? "paused" : "playing"; // String
			}
			return "stopped"; // String
		},

		_cycle: function(){
			var _t = this;
			if(_t._active){
				var curr = new Date().valueOf();
				// Allow durations of 0 (instant) by setting step to 1 - see #13798
				var step = _t.duration === 0 ? 1 : (curr - _t._startTime) / (_t.duration);

				if(step >= 1){
					step = 1;
				}
				_t._percent = step;

				// Perform easing
				if(_t.easing){
					step = _t.easing(step);
				}

				_t._fire("onAnimate", [_t.curve.getValue(step)]);

				if(_t._percent < 1){
					_t._startTimer();
				}else{
					_t._active = false;

					if(_t.repeat > 0){
						_t.repeat--;
						_t.play(null, true);
					}else if(_t.repeat == -1){
						_t.play(null, true);
					}else{
						if(_t._startRepeatCount){
							_t.repeat = _t._startRepeatCount;
							_t._startRepeatCount = 0;
						}
					}
					_t._percent = 0;
					_t._fire("onEnd", [_t.node]);
					!_t.repeat && _t._stopTimer();
				}
			}
			return _t; // Animation
		},

		_clearTimer: function(){
			// summary:
			//		Clear the play delay timer
			clearTimeout(this._delayTimer);
			delete this._delayTimer;
		}

	});

	// the local timer, stubbed into all Animation instances
	var ctr = 0,
		timer = null,
		runner = {
			run: function(){}
		};

	lang.extend(Animation, {

		_startTimer: function(){
			if(!this._timer){
				this._timer = aspect.after(runner, "run", lang.hitch(this, "_cycle"), true);
				ctr++;
			}
			if(!timer){
				timer = setInterval(lang.hitch(runner, "run"), this.rate);
			}
		},

		_stopTimer: function(){
			if(this._timer){
				this._timer.remove();
				this._timer = null;
				ctr--;
			}
			if(ctr <= 0){
				clearInterval(timer);
				timer = null;
				ctr = 0;
			}
		}

	});

	var _makeFadeable =
		has("ie") ? function(node){
			// only set the zoom if the "tickle" value would be the same as the
			// default
			var ns = node.style;
			// don't set the width to auto if it didn't already cascade that way.
			// We don't want to f anyones designs
			if(!ns.width.length && style.get(node, "width") == "auto"){
				ns.width = "auto";
			}
		} :
		function(){};

	basefx._fade = function(/*Object*/ args){
		// summary:
		//		Returns an animation that will fade the node defined by
		//		args.node from the start to end values passed (args.start
		//		args.end) (end is mandatory, start is optional)

		args.node = dom.byId(args.node);
		var fArgs = _mixin({ properties: {} }, args),
			props = (fArgs.properties.opacity = {});

		props.start = !("start" in fArgs) ?
			function(){
				return +style.get(fArgs.node, "opacity")||0;
			} : fArgs.start;
		props.end = fArgs.end;

		var anim = basefx.animateProperty(fArgs);
		aspect.after(anim, "beforeBegin", lang.partial(_makeFadeable, fArgs.node), true);

		return anim; // Animation
	};

	/*=====
	var __FadeArgs = declare(null, {
		// node: DOMNode|String
		//		The node referenced in the animation
		// duration: Integer?
		//		Duration of the animation in milliseconds.
		// easing: Function?
		//		An easing function.
	});
	=====*/

	basefx.fadeIn = function(/*__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args' from
		//		its current opacity to fully opaque.
		return basefx._fade(_mixin({ end: 1 }, args)); // Animation
	};

	basefx.fadeOut = function(/*__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args'
		//		from its current opacity to fully transparent.
		return basefx._fade(_mixin({ end: 0 }, args)); // Animation
	};

	basefx._defaultEasing = function(/*Decimal?*/ n){
		// summary:
		//		The default easing function for Animation(s)
		return 0.5 + ((Math.sin((n + 1.5) * Math.PI)) / 2);	// Decimal
	};

	var PropLine = function(properties){
		// PropLine is an internal class which is used to model the values of
		// an a group of CSS properties across an animation lifecycle. In
		// particular, the "getValue" function handles getting interpolated
		// values between start and end for a particular CSS value.
		this._properties = properties;
		for(var p in properties){
			var prop = properties[p];
			if(prop.start instanceof Color){
				// create a reusable temp color object to keep intermediate results
				prop.tempColor = new Color();
			}
		}
	};

	PropLine.prototype.getValue = function(r){
		var ret = {};
		for(var p in this._properties){
			var prop = this._properties[p],
				start = prop.start;
			if(start instanceof Color){
				ret[p] = Color.blendColors(start, prop.end, r, prop.tempColor).toCss();
			}else if(!lang.isArray(start)){
				ret[p] = ((prop.end - start) * r) + start + (p != "opacity" ? prop.units || "px" : 0);
			}
		}
		return ret;
	};

	/*=====
	var __AnimArgs = declare(__FadeArgs, {
		// properties: Object?
		//		A hash map of style properties to Objects describing the transition,
		//		such as the properties of _Line with an additional 'units' property
		properties: {}

		//TODOC: add event callbacks
	});
	=====*/

	basefx.animateProperty = function(/*__AnimArgs*/ args){
		// summary:
		//		Returns an animation that will transition the properties of
		//		node defined in `args` depending how they are defined in
		//		`args.properties`
		//
		// description:
		//		Foundation of most `dojo/_base/fx`
		//		animations. It takes an object of "properties" corresponding to
		//		style properties, and animates them in parallel over a set
		//		duration.
		//
		// example:
		//		A simple animation that changes the width of the specified node.
		//	|	basefx.animateProperty({
		//	|		node: "nodeId",
		//	|		properties: { width: 400 },
		//	|	}).play();
		//		Dojo figures out the start value for the width and converts the
		//		integer specified for the width to the more expressive but
		//		verbose form `{ width: { end: '400', units: 'px' } }` which you
		//		can also specify directly. Defaults to 'px' if omitted.
		//
		// example:
		//		Animate width, height, and padding over 2 seconds... the
		//		pedantic way:
		//	|	basefx.animateProperty({ node: node, duration:2000,
		//	|		properties: {
		//	|			width: { start: '200', end: '400', units:"px" },
		//	|			height: { start:'200', end: '400', units:"px" },
		//	|			paddingTop: { start:'5', end:'50', units:"px" }
		//	|		}
		//	|	}).play();
		//		Note 'paddingTop' is used over 'padding-top'. Multi-name CSS properties
		//		are written using "mixed case", as the hyphen is illegal as an object key.
		//
		// example:
		//		Plug in a different easing function and register a callback for
		//		when the animation ends. Easing functions accept values between
		//		zero and one and return a value on that basis. In this case, an
		//		exponential-in curve.
		//	|	basefx.animateProperty({
		//	|		node: "nodeId",
		//	|		// dojo figures out the start value
		//	|		properties: { width: { end: 400 } },
		//	|		easing: function(n){
		//	|			return (n==0) ? 0 : Math.pow(2, 10 * (n - 1));
		//	|		},
		//	|		onEnd: function(node){
		//	|			// called when the animation finishes. The animation
		//	|			// target is passed to this function
		//	|		}
		//	|	}).play(500); // delay playing half a second
		//
		// example:
		//		Like all `Animation`s, animateProperty returns a handle to the
		//		Animation instance, which fires the events common to Dojo FX. Use `aspect.after`
		//		to access these events outside of the Animation definition:
		//	|	var anim = basefx.animateProperty({
		//	|		node:"someId",
		//	|		properties:{
		//	|			width:400, height:500
		//	|		}
		//	|	});
		//	|	aspect.after(anim, "onEnd", function(){
		//	|		console.log("animation ended");
		//	|	}, true);
		//	|	// play the animation now:
		//	|	anim.play();
		//
		// example:
		//		Each property can be a function whose return value is substituted along.
		//		Additionally, each measurement (eg: start, end) can be a function. The node
		//		reference is passed directly to callbacks.
		//	|	basefx.animateProperty({
		//	|		node:"mine",
		//	|		properties:{
		//	|			height:function(node){
		//	|				// shrink this node by 50%
		//	|				return domGeom.position(node).h / 2
		//	|			},
		//	|			width:{
		//	|				start:function(node){ return 100; },
		//	|				end:function(node){ return 200; }
		//	|			}
		//	|		}
		//	|	}).play();
		//

		var n = args.node = dom.byId(args.node);
		if(!args.easing){ args.easing = dojo._defaultEasing; }

		var anim = new Animation(args);
		aspect.after(anim, "beforeBegin", lang.hitch(anim, function(){
			var pm = {};
			for(var p in this.properties){
				// Make shallow copy of properties into pm because we overwrite
				// some values below. In particular if start/end are functions
				// we don't want to overwrite them or the functions won't be
				// called if the animation is reused.
				if(p == "width" || p == "height"){
					this.node.display = "block";
				}
				var prop = this.properties[p];
				if(lang.isFunction(prop)){
					prop = prop(n);
				}
				prop = pm[p] = _mixin({}, (lang.isObject(prop) ? prop: { end: prop }));

				if(lang.isFunction(prop.start)){
					prop.start = prop.start(n);
				}
				if(lang.isFunction(prop.end)){
					prop.end = prop.end(n);
				}
				var isColor = (p.toLowerCase().indexOf("color") >= 0);
				function getStyle(node, p){
					// domStyle.get(node, "height") can return "auto" or "" on IE; this is more reliable:
					var v = { height: node.offsetHeight, width: node.offsetWidth }[p];
					if(v !== undefined){ return v; }
					v = style.get(node, p);
					return (p == "opacity") ? +v : (isColor ? v : parseFloat(v));
				}
				if(!("end" in prop)){
					prop.end = getStyle(n, p);
				}else if(!("start" in prop)){
					prop.start = getStyle(n, p);
				}

				if(isColor){
					prop.start = new Color(prop.start);
					prop.end = new Color(prop.end);
				}else{
					prop.start = (p == "opacity") ? +prop.start : parseFloat(prop.start);
				}
			}
			this.curve = new PropLine(pm);
		}), true);
		aspect.after(anim, "onAnimate", lang.hitch(style, "set", anim.node), true);
		return anim; // Animation
	};

	basefx.anim = function(	/*DOMNode|String*/	node,
							/*Object*/			properties,
							/*Integer?*/		duration,
							/*Function?*/		easing,
							/*Function?*/		onEnd,
							/*Integer?*/		delay){
		// summary:
		//		A simpler interface to `animateProperty()`, also returns
		//		an instance of `Animation` but begins the animation
		//		immediately, unlike nearly every other Dojo animation API.
		// description:
		//		Simpler (but somewhat less powerful) version
		//		of `animateProperty`.  It uses defaults for many basic properties
		//		and allows for positional parameters to be used in place of the
		//		packed "property bag" which is used for other Dojo animation
		//		methods.
		//
		//		The `Animation` object returned will be already playing, so
		//		calling play() on it again is (usually) a no-op.
		// node:
		//		a DOM node or the id of a node to animate CSS properties on
		// duration:
		//		The number of milliseconds over which the animation
		//		should run. Defaults to the global animation default duration
		//		(350ms).
		// easing:
		//		An easing function over which to calculate acceleration
		//		and deceleration of the animation through its duration.
		//		A default easing algorithm is provided, but you may
		//		plug in any you wish. A large selection of easing algorithms
		//		are available in `dojo/fx/easing`.
		// onEnd:
		//		A function to be called when the animation finishes
		//		running.
		// delay:
		//		The number of milliseconds to delay beginning the
		//		animation by. The default is 0.
		// example:
		//		Fade out a node
		//	|	basefx.anim("id", { opacity: 0 });
		// example:
		//		Fade out a node over a full second
		//	|	basefx.anim("id", { opacity: 0 }, 1000);
		return basefx.animateProperty({ // Animation
			node: node,
			duration: duration || Animation.prototype.duration,
			properties: properties,
			easing: easing,
			onEnd: onEnd
		}).play(delay || 0);
	};


	if( 1 ){
		_mixin(dojo, basefx);
		// Alias to drop come 2.0:
		dojo._Animation = Animation;
	}

	return basefx;
});

},
'dojo/uacss':function(){
define(["./dom-geometry", "./_base/lang", "./domReady", "./sniff", "./_base/window"],
	function(geometry, lang, domReady, has, baseWindow){

	// module:
	//		dojo/uacss

	/*=====
	return {
		// summary:
		//		Applies pre-set CSS classes to the top-level HTML node, based on:
		//
		//		- browser (ex: dj_ie)
		//		- browser version (ex: dj_ie6)
		//		- box model (ex: dj_contentBox)
		//		- text direction (ex: dijitRtl)
		//
		//		In addition, browser, browser version, and box model are
		//		combined with an RTL flag when browser text is RTL. ex: dj_ie-rtl.
		//
		//		Returns the has() method.
	};
	=====*/

	var
		html = baseWindow.doc.documentElement,
		ie = has("ie"),
		opera = has("opera"),
		maj = Math.floor,
		ff = has("ff"),
		boxModel = geometry.boxModel.replace(/-/,''),

		classes = {
			"dj_quirks": has("quirks"),

			// NOTE: Opera not supported by dijit
			"dj_opera": opera,

			"dj_khtml": has("khtml"),

			"dj_webkit": has("webkit"),
			"dj_safari": has("safari"),
			"dj_chrome": has("chrome"),

			"dj_gecko": has("mozilla"),

			"dj_ios": has("ios"),
			"dj_android": has("android")
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = has("quirks");
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = lang.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	domReady(function(){
		if(!geometry.isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ");
			html.className = lang.trim(html.className + " " + rtlClassStr + "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl "));
		}
	});
	return has;
});

},
'dojo/_base/NodeList':function(){
define(["./kernel", "../query", "./array", "./html", "../NodeList-dom"], function(dojo, query, array){
	// module:
	//		dojo/_base/NodeList

	/*=====
	return {
		// summary:
		//		This module extends dojo/NodeList with the legacy connect(), coords(),
		//		blur(), focus(), change(), click(), error(), keydown(), keypress(),
		//		keyup(), load(), mousedown(), mouseenter(), mouseleave(), mousemove(),
		//		mouseout(), mouseover(), mouseup(), and submit() methods.
	};
	=====*/
 
	var NodeList = query.NodeList,
		nlp = NodeList.prototype;

	nlp.connect = NodeList._adaptAsForEach(function(){
		// don't bind early to dojo.connect since we no longer explicitly depend on it
		return dojo.connect.apply(this, arguments);
	});
	/*=====
	nlp.connect = function(methodName, objOrFunc, funcName){
		// summary:
		//		Attach event handlers to every item of the NodeList. Uses dojo.connect()
		//		so event properties are normalized.
		//
		//		Application must manually require() "dojo/_base/connect" before using this method.
		// methodName: String
		//		the name of the method to attach to. For DOM events, this should be
		//		the lower-case name of the event
		// objOrFunc: Object|Function|String
		//		if 2 arguments are passed (methodName, objOrFunc), objOrFunc should
		//		reference a function or be the name of the function in the global
		//		namespace to attach. If 3 arguments are provided
		//		(methodName, objOrFunc, funcName), objOrFunc must be the scope to
		//		locate the bound function in
		// funcName: String?
		//		optional. A string naming the function in objOrFunc to bind to the
		//		event. May also be a function reference.
		// example:
		//		add an onclick handler to every button on the page
		//		|	query("div:nth-child(odd)").connect("onclick", function(e){
		//		|		console.log("clicked!");
		//		|	});
		// example:
		//		attach foo.bar() to every odd div's onmouseover
		//		|	query("div:nth-child(odd)").connect("onmouseover", foo, "bar");

		return null;	// NodeList
	};
	=====*/

	nlp.coords = NodeList._adaptAsMap(dojo.coords);
	/*=====
	nlp.coords = function(){
		// summary:
		//		Deprecated: Use position() for border-box x/y/w/h
		//		or marginBox() for margin-box w/h/l/t.
		//		Returns the box objects of all elements in a node list as
		//		an Array (*not* a NodeList). Acts like `domGeom.coords`, though assumes
		//		the node passed is each node in this list.

		return []; // Array
	};
	=====*/

	NodeList.events = [
		// summary:
		//		list of all DOM events used in NodeList
		"blur", "focus", "change", "click", "error", "keydown", "keypress",
		"keyup", "load", "mousedown", "mouseenter", "mouseleave", "mousemove",
		"mouseout", "mouseover", "mouseup", "submit"
	];

	// FIXME: pseudo-doc the above automatically generated on-event functions

	// syntactic sugar for DOM events
	array.forEach(NodeList.events, function(evt){
			var _oe = "on" + evt;
			nlp[_oe] = function(a, b){
				return this.connect(_oe, a, b);
			};
				// FIXME: should these events trigger publishes?
				/*
				return (a ? this.connect(_oe, a, b) :
							this.forEach(function(n){
								// FIXME:
								//		listeners get buried by
								//		addEventListener and can't be dug back
								//		out to be triggered externally.
								// see:
								//		http://developer.mozilla.org/en/docs/DOM:element

								console.log(n, evt, _oe);

								// FIXME: need synthetic event support!
								var _e = { target: n, faux: true, type: evt };
								// dojo._event_listener._synthesizeEvent({}, { target: n, faux: true, type: evt });
								try{ n[evt](_e); }catch(e){ console.log(e); }
								try{ n[_oe](_e); }catch(e){ console.log(e); }
							})
				);
				*/
		}
	);

	dojo.NodeList = NodeList;
	return NodeList;
});

},
'xcitec/form/_Calendar':function(){
define([
         "dojo/_base/declare",
         "dojo/_base/lang",
         "dojo/aspect",
         "dojo/date/locale",
         "dojo/dom-construct",
         "dojo/topic",
	"dojo/sniff",
         "dijit/Calendar"
       ], function (declare, langModule, aspect, locale, domConstruct, topic, has, Calendar) {
  return declare(Calendar, {
    /* xcitec.form._Calendar are created as workarround for bug undre IE:
     * if user press selected date popup is not closed (same date as selected)
     * _setValueAttr copied from dijit.Calendar and commented dates compare.
     */
    //	_setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
    //		console.log("xcitec.form._Calendar:_setValueAttr: " + value);
    //		this.inherited(arguments);
    //	}

    _setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
      // summary:
      //		Support set("value", ...)
      // description:
      // 		Set the current date and update the UI.  If the date is disabled, the value will
      //		not change, but the display will change to the corresponding month.
      // value:
      //		Either a Date or the number of seconds since 1970.
      // tags:
      //      protected
      if(value){
        // convert from Number to Date, or make copy of Date object so that setHours() call below
        // doesn't affect original value
        value = new this.dateClassObj(value);
      }
      if(this._isValidDate(value)){
        //if(!this._isValidDate(this.value) || this.dateFuncObj.compare(value, this.value)){
        value.setHours(1, 0, 0, 0); // round to nearest day (1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)

        if(!this.isDisabledDate(value, this.lang)){
          this._set("value", value);

          // Set focus cell to the new value.   Arguably this should only happen when there isn't a current
          // focus point.   This will also repopulate the grid, showing the new selected value (and possibly
          // new month/year).
          this.set("currentFocus", value);

          if(priorityChange || typeof priorityChange == "undefined"){
            this.onChange(this.get('value'));
            this.onValueSelected(this.get('value'));	// remove in 2.0
          }
        }
        //}
      }else{
        // clear value, and repopulate grid (to deselect the previously selected day) without changing currentFocus
        this._set("value", null);
        this.set("currentFocus", this.currentFocus);
      }
    },

	  _setCurrentFocusAttr: function(/*Date*/ date, /*Boolean*/ forceFocus){
		  // summary:
		  //		@Override - dijit/Calendar
		  // description:
		  //		If the calendar currently has focus, then focuses specified date,
		  //		changing the currently displayed month/year if necessary.
		  //		If the calendar doesn't have focus, updates currently
		  //		displayed month/year, and sets the cell that will get focus
		  //		when Calendar is focused.
		  // forceFocus:
		  //		If true, will focus() the cell even if calendar itself doesn't have focus

		  var oldFocus = this.currentFocus,
			  oldCell = this._getNodeByDate(oldFocus);
		  date = this._patchDate(date);

		  this._set("currentFocus", date);

		  // If the focus is on a different month than the current calendar month, switch the displayed month.
		  // Also will populate the grid initially, on Calendar creation.
		  if(!this._date2cell || this.dateModule.difference(oldFocus, date, "month") != 0){
			  this._populateGrid();
			  this._populateControls();
			  var selectedDate;
			  if(this._isValidDate(this.value)){
				  selectedDate = this.value;
			  }else{
				  selectedDate = this.currentFocus;
			  }
			  this._markSelectedDates([selectedDate]);
		  }

		  // set tabIndex=0 on new cell, and focus it (but only if Calendar itself is focused)
		  var newCell = this._getNodeByDate(date);
		  newCell.setAttribute("tabIndex", this.tabIndex);
		  if(this.focused || forceFocus){
			  newCell.focus();
		  }

		  // set tabIndex=-1 on old focusable cell
		  if(oldCell && oldCell != newCell){
			  if(has("webkit")){    // see #11064 about webkit bug
				  oldCell.setAttribute("tabIndex", "-1");
			  }else{
				  oldCell.removeAttribute("tabIndex");
			  }
		  }
	  }
  });
});

},
'xcitec/layout/TaskPanel':function(){
define([
	"dojo/_base/declare",
  "./FormPanel"
], function (declare, FormPanel) {

	var TaskPanel = declare(FormPanel, {
    baseClass: "formPanel TaskPanel",
    background: true
	});
	return TaskPanel;
});
},
'xcitec/Title':function(){
define([
         "dojo/_base/declare",
         "./Text",
         "./_base/ImageFactory",
         "./xcitecUtil"
       ], function (declare, Text, ImageFactory, xcitecUtil) {
    return declare(Text, {
      entityClass: "",
      baseClass: "xcitecFormTitle",
      postMixInProperties: function(){
        if(this.entityClass){
          this.iconClass = "sboIconBig " + xcitecUtil.createEntityIconClass(this.entityClass) + "Big";
        }
        if(this.iconClass){
          this.iconClass = ImageFactory.createIconCss(this.iconClass);
        }
        this.inherited(arguments);
      }
    });
});

},
'dijit/form/_FormWidget':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("dijit-legacy-requires"), has("msapp")
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/ready",
	"../_Widget",
	"../_CssStateMixin",
	"../_TemplatedMixin",
	"./_FormWidgetMixin"
], function(declare, has, kernel, ready, _Widget, _CssStateMixin, _TemplatedMixin, _FormWidgetMixin){

	// module:
	//		dijit/form/_FormWidget

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/_FormValueWidget"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.form._FormWidget", [_Widget, _TemplatedMixin, _CssStateMixin, _FormWidgetMixin], {
		// summary:
		//		Base class for widgets corresponding to native HTML elements such as `<checkbox>` or `<button>`,
		//		which can be children of a `<form>` node or a `dijit/form/Form` widget.
		//
		// description:
		//		Represents a single HTML element.
		//		All these widgets should have these attributes just like native HTML input elements.
		//		You can set them during widget construction or afterwards, via `dijit/_WidgetBase.set()`.
		//
		//		They also share some common methods.

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.  Use set('disabled', ...) instead.
			kernel.deprecated("setDisabled(" + disabled + ") is deprecated. Use set('disabled'," + disabled + ") instead.", "", "2.0");
			this.set('disabled', disabled);
		},

		setValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('value', ...) instead.
			kernel.deprecated("dijit.form._FormWidget:setValue(" + value + ") is deprecated.  Use set('value'," + value + ") instead.", "", "2.0");
			this.set('value', value);
		},

		getValue: function(){
			// summary:
			//		Deprecated.  Use get('value') instead.
			kernel.deprecated(this.declaredClass + "::getValue() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},

		postMixInProperties: function(){
			// Setup name=foo string to be referenced from the template (but only if a name has been specified).
			// Unfortunately we can't use _setNameAttr to set the name in IE due to IE limitations, see #8484, #8660.
			// But when IE6 and IE7 are desupported, then we probably don't need this anymore, so should remove it in 2.0.
			// Also, don't do this for Windows 8 Store Apps because it causes a security exception (see #16452).
			// Regarding escaping, see heading "Attribute values" in
			// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
			this.nameAttrSetting = (this.name && !has("msapp")) ? ('name="' + this.name.replace(/"/g, "&quot;") + '"') : '';
			this.inherited(arguments);
		},

		// Override automatic assigning type --> focusNode, it causes exception on IE.
		// Instead, type must be specified as ${type} in the template, as part of the original DOM
		_setTypeAttr: null
	});
});

},
'xcitec/MessagePane':function(){
define([
	"dojo",
	"dojo/_base/declare",
    "dojo/dom",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
  "./xcitecUtil",
  "./form/InlineLinkButton",
  "./_base/Scroll"
], function (dojo, declare, dom, domAttr, domConstruct, query, _Templated, _Widget, registry, xcitecUtil, InlineLinkButton, Scroll) {

	var MessagePane = declare( [_Widget, _Templated], {
	//  baseClass: "MessagePaneError",
	  baseClass: "MessagePane",
	
	  // Message Types: error, warn, info, success
	  msgType: "",
	
	  // Message visibility
	  visible: false,
	
	  templateString :'<div class="${baseClass}" ' +
	                  '><div class="MessagePaneContainer messageRound" data-dojo-attach-point="containerNode"></div ' +
	                  '><div class="mCorner mCornerTL"></div ' +
	                  '><div class="mCorner mCornerBL"></div ' +
	                  '><div class="mCorner mCornerTR"></div ' +
	                  '><div class="mCorner mCornerBR"></div ' +
	                  '></div>',
	
	  postMixInProperties: function() {
	    this.inherited(arguments);
	    if (this.msgType) {
	      //this.baseClass += " "+ this.baseClass + this.msgType.charAt(0).toUpperCase() +  this.msgType.substr(1).replace(/-.*/, "");
	      this.baseClass += " "+ xcitecUtil.createTypedStyleClass(this.baseClass,this.msgType);
	    }
	  },
	
	  postCreate: function(){
	    this.inherited(arguments);
	    query('[relatedComponentId]', this.domNode).forEach(function(node){
			var link = new InlineLinkButton({
				text: domAttr.get(node, 'innerHTML'),
				onClick: function() {
                    var widget = registry.byId(domAttr.get(node, 'relatedComponentId'));
                    var widgetNode = widget ? widget.domNode : null;
                    var component =  widget ? null : dom.byId(domAttr.get(node, 'relatedComponentId'));
                    Scroll.smoothScroll({component: component, node: widgetNode, win: window, duration: 300 }).play()
                }
			});
			domConstruct.place(link.domNode, node, 'replace');
		});
		if (!this.visible) {
	      this.domNode.style.display = "none";
	    }
	  }
	});
	
	return MessagePane;
});

},
'xcitec/grid/HGridV2':function(){
define([
	"dojo/_base/declare",
	"./HGrid"
], function (declare, HGrid) {

	return declare(HGrid, {

	});
});
},
'xcitec/layout/DeletableBlock':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/query",
	"dojo/text!xcitec/layout/templates/DeletableBlock.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (declare, domStyle, query, template, _Templated, _Widget) {

	/*
	    xcitec.layout.DeletableBlock is a layout control which aim is to add delete icon and background
	    for content placed inside it.
	 */
	var DeletableBlock = declare( [_Widget, _Templated], {
	    templateString: template,
		xbtnClass: "ContentAreaEditClose",
	
	    postMixInProperties: function() {
	        this.buttonsContainer = query("> [xcitecAttachPoint$=buttons]", this.srcNodeRef);
	        this.inherited(arguments);
			if (!("onDelete" in this.params)) {
				this.xbtnClass = "ContentAreaEditCloseReadOnly";
			}
	    },
	
	    startup: function() {
	        this.inherited(arguments);
	        domStyle.set(this.domNode, "width", "auto");
	    },
	
	    _onClick: function() {
	        this.onDelete();
	    },
	
	    onDelete: function() {
	        // users' action on press delete icon
	    }
	});
	return DeletableBlock;
});

},
'dojo/query':function(){
define(["./_base/kernel", "./has", "./dom", "./on", "./_base/array", "./_base/lang", "./selector/_loader", "./selector/_loader!default"],
	function(dojo, has, dom, on, array, lang, loader, defaultEngine){

	"use strict";

	has.add("array-extensible", function(){
		// test to see if we can extend an array (not supported in old IE)
		return lang.delegate([], {length: 1}).length == 1 && !has("bug-for-in-skips-shadowed");
	});
	
	var ap = Array.prototype, aps = ap.slice, apc = ap.concat, forEach = array.forEach;

	var tnl = function(/*Array*/ a, /*dojo/NodeList?*/ parent, /*Function?*/ NodeListCtor){
		// summary:
		//		decorate an array to make it look like a `dojo/NodeList`.
		// a:
		//		Array of nodes to decorate.
		// parent:
		//		An optional parent NodeList that generated the current
		//		list of nodes. Used to call _stash() so the parent NodeList
		//		can be accessed via end() later.
		// NodeListCtor:
		//		An optional constructor function to use for any
		//		new NodeList calls. This allows a certain chain of
		//		NodeList calls to use a different object than dojo/NodeList.
		var nodeList = new (NodeListCtor || this._NodeListCtor || nl)(a);
		return parent ? nodeList._stash(parent) : nodeList;
	};

	var loopBody = function(f, a, o){
		a = [0].concat(aps.call(a, 0));
		o = o || dojo.global;
		return function(node){
			a[0] = node;
			return f.apply(o, a);
		};
	};

	// adapters

	var adaptAsForEach = function(f, o){
		// summary:
		//		adapts a single node function to be used in the forEach-type
		//		actions. The initial object is returned from the specialized
		//		function.
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			this.forEach(loopBody(f, arguments, o));
			return this;	// Object
		};
	};

	var adaptAsMap = function(f, o){
		// summary:
		//		adapts a single node function to be used in the map-type
		//		actions. The return is a new array of values, as via `dojo/_base/array.map`
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			return this.map(loopBody(f, arguments, o));
		};
	};

	var adaptAsFilter = function(f, o){
		// summary:
		//		adapts a single node function to be used in the filter-type actions
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			return this.filter(loopBody(f, arguments, o));
		};
	};

	var adaptWithCondition = function(f, g, o){
		// summary:
		//		adapts a single node function to be used in the map-type
		//		actions, behaves like forEach() or map() depending on arguments
		// f: Function
		//		a function to adapt
		// g: Function
		//		a condition function, if true runs as map(), otherwise runs as forEach()
		// o: Object?
		//		an optional context for f and g
		return function(){
			var a = arguments, body = loopBody(f, a, o);
			if(g.call(o || dojo.global, a)){
				return this.map(body);	// self
			}
			this.forEach(body);
			return this;	// self
		};
	};

	var NodeList = function(array){
		// summary:
		//		Array-like object which adds syntactic
		//		sugar for chaining, common iteration operations, animation, and
		//		node manipulation. NodeLists are most often returned as the
		//		result of dojo/query() calls.
		// description:
		//		NodeList instances provide many utilities that reflect
		//		core Dojo APIs for Array iteration and manipulation, DOM
		//		manipulation, and event handling. Instead of needing to dig up
		//		functions in the dojo package, NodeLists generally make the
		//		full power of Dojo available for DOM manipulation tasks in a
		//		simple, chainable way.
		// example:
		//		create a node list from a node
		//		|	require(["dojo/query", "dojo/dom"
		//		|	], function(query, dom){
		//		|		query.NodeList(dom.byId("foo"));
		//		|	});
		// example:
		//		get a NodeList from a CSS query and iterate on it
		//		|	require(["dojo/on", "dojo/dom"
		//		|	], function(on, dom){
		//		|		var l = query(".thinger");
		//		|		l.forEach(function(node, index, nodeList){
		//		|			console.log(index, node.innerHTML);
		//		|		});
		//		|	});
		// example:
		//		use native and Dojo-provided array methods to manipulate a
		//		NodeList without needing to use dojo.* functions explicitly:
		//		|	require(["dojo/query", "dojo/dom-construct", "dojo/dom"
		//		|	], function(query, domConstruct, dom){
		//		|		var l = query(".thinger");
		//		|		// since NodeLists are real arrays, they have a length
		//		|		// property that is both readable and writable and
		//		|		// push/pop/shift/unshift methods
		//		|		console.log(l.length);
		//		|		l.push(domConstruct.create("span"));
		//		|
		//		|		// dojo's normalized array methods work too:
		//		|		console.log( l.indexOf(dom.byId("foo")) );
		//		|		// ...including the special "function as string" shorthand
		//		|		console.log( l.every("item.nodeType == 1") );
		//		|
		//		|		// NodeLists can be [..] indexed, or you can use the at()
		//		|		// function to get specific items wrapped in a new NodeList:
		//		|		var node = l[3]; // the 4th element
		//		|		var newList = l.at(1, 3); // the 2nd and 4th elements
		//		|	});
		// example:
		//		chainability is a key advantage of NodeLists:
		//		|	require(["dojo/query", "dojo/NodeList-dom"
		//		|	], function(query){
		//		|		query(".thinger")
		//		|			.onclick(function(e){ /* ... */ })
		//		|			.at(1, 3, 8) // get a subset
		//		|				.style("padding", "5px")
		//		|				.forEach(console.log);
		//		|	});

		var isNew = this instanceof nl && has("array-extensible");
		if(typeof array == "number"){
			array = Array(array);
		}
		var nodeArray = (array && "length" in array) ? array : arguments;
		if(isNew || !nodeArray.sort){
			// make sure it's a real array before we pass it on to be wrapped 
			var target = isNew ? this : [],
				l = target.length = nodeArray.length;
			for(var i = 0; i < l; i++){
				target[i] = nodeArray[i];
			}
			if(isNew){
				// called with new operator, this means we are going to use this instance and push
				// the nodes on to it. This is usually much faster since the NodeList properties
				//	don't need to be copied (unless the list of nodes is extremely large).
				return target;
			}
			nodeArray = target;
		}
		// called without new operator, use a real array and copy prototype properties,
		// this is slower and exists for back-compat. Should be removed in 2.0.
		lang._mixin(nodeArray, nlp);
		nodeArray._NodeListCtor = function(array){
			// call without new operator to preserve back-compat behavior
			return nl(array);
		};
		return nodeArray;
	};
	
	var nl = NodeList, nlp = nl.prototype = 
		has("array-extensible") ? [] : {};// extend an array if it is extensible

	// expose adapters and the wrapper as private functions

	nl._wrap = nlp._wrap = tnl;
	nl._adaptAsMap = adaptAsMap;
	nl._adaptAsForEach = adaptAsForEach;
	nl._adaptAsFilter  = adaptAsFilter;
	nl._adaptWithCondition = adaptWithCondition;

	// mass assignment

	// add array redirectors
	forEach(["slice", "splice"], function(name){
		var f = ap[name];
		//Use a copy of the this array via this.slice() to allow .end() to work right in the splice case.
		// CANNOT apply ._stash()/end() to splice since it currently modifies
		// the existing this array -- it would break backward compatibility if we copy the array before
		// the splice so that we can use .end(). So only doing the stash option to this._wrap for slice.
		nlp[name] = function(){ return this._wrap(f.apply(this, arguments), name == "slice" ? this : null); };
	});
	// concat should be here but some browsers with native NodeList have problems with it

	// add array.js redirectors
	forEach(["indexOf", "lastIndexOf", "every", "some"], function(name){
		var f = array[name];
		nlp[name] = function(){ return f.apply(dojo, [this].concat(aps.call(arguments, 0))); };
	});

	lang.extend(NodeList, {
		// copy the constructors
		constructor: nl,
		_NodeListCtor: nl,
		toString: function(){
			// Array.prototype.toString can't be applied to objects, so we use join
			return this.join(",");
		},
		_stash: function(parent){
			// summary:
			//		private function to hold to a parent NodeList. end() to return the parent NodeList.
			//
			// example:
			//		How to make a `dojo/NodeList` method that only returns the third node in
			//		the dojo/NodeList but allows access to the original NodeList by using this._stash:
			//	|	require(["dojo/query", "dojo/_base/lang", "dojo/NodeList", "dojo/NodeList-dom"
			//	|	], function(query, lang){
			//	|		lang.extend(NodeList, {
			//	|			third: function(){
			//	|				var newNodeList = NodeList(this[2]);
			//	|				return newNodeList._stash(this);
			//	|			}
			//	|		});
			//	|		// then see how _stash applies a sub-list, to be .end()'ed out of
			//	|		query(".foo")
			//	|			.third()
			//	|				.addClass("thirdFoo")
			//	|			.end()
			//	|			// access to the orig .foo list
			//	|			.removeClass("foo")
			//	|	});
			//
			this._parent = parent;
			return this; // dojo/NodeList
		},

		on: function(eventName, listener){
			// summary:
			//		Listen for events on the nodes in the NodeList. Basic usage is:
			//
			// example:
			//		|	require(["dojo/query"
			//		|	], function(query){
			//		|		query(".my-class").on("click", listener);
			//			This supports event delegation by using selectors as the first argument with the event names as
			//			pseudo selectors. For example:
			//		| 		query("#my-list").on("li:click", listener);
			//			This will listen for click events within `<li>` elements that are inside the `#my-list` element.
			//			Because on supports CSS selector syntax, we can use comma-delimited events as well:
			//		| 		query("#my-list").on("li button:mouseover, li:click", listener);
			//		|	});
			var handles = this.map(function(node){
				return on(node, eventName, listener); // TODO: apply to the NodeList so the same selector engine is used for matches
			});
			handles.remove = function(){
				for(var i = 0; i < handles.length; i++){
					handles[i].remove();
				}
			};
			return handles;
		},

		end: function(){
			// summary:
			//		Ends use of the current `NodeList` by returning the previous NodeList
			//		that generated the current NodeList.
			// description:
			//		Returns the `NodeList` that generated the current `NodeList`. If there
			//		is no parent NodeList, an empty NodeList is returned.
			// example:
			//	|	require(["dojo/query", "dojo/NodeList-dom"
			//	|	], function(query){
			//	|		query("a")
			//	|			.filter(".disabled")
			//	|				// operate on the anchors that only have a disabled class
			//	|				.style("color", "grey")
			//	|			.end()
			//	|			// jump back to the list of anchors
			//	|			.style(...)
			//	|	});
			//
			if(this._parent){
				return this._parent;
			}else{
				//Just return empty list.
				return new this._NodeListCtor(0);
			}
		},

		// http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array#Methods

		// FIXME: handle return values for #3244
		//		http://trac.dojotoolkit.org/ticket/3244

		// FIXME:
		//		need to wrap or implement:
		//			join (perhaps w/ innerHTML/outerHTML overload for toString() of items?)
		//			reduce
		//			reduceRight

		/*=====
		slice: function(begin, end){
			// summary:
			//		Returns a new NodeList, maintaining this one in place
			// description:
			//		This method behaves exactly like the Array.slice method
			//		with the caveat that it returns a `dojo/NodeList` and not a
			//		raw Array. For more details, see Mozilla's [slice
			//		documentation](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/slice)
			// begin: Integer
			//		Can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// end: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			return this._wrap(a.slice.apply(this, arguments));
		},

		splice: function(index, howmany, item){
			// summary:
			//		Returns a new NodeList, manipulating this NodeList based on
			//		the arguments passed, potentially splicing in new elements
			//		at an offset, optionally deleting elements
			// description:
			//		This method behaves exactly like the Array.splice method
			//		with the caveat that it returns a `dojo/NodeList` and not a
			//		raw Array. For more details, see Mozilla's [splice
			//		documentation](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/splice)
			//		For backwards compatibility, calling .end() on the spliced NodeList
			//		does not return the original NodeList -- splice alters the NodeList in place.
			// index: Integer
			//		begin can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// howmany: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			// item: Object...?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList
			return this._wrap(a.splice.apply(this, arguments));	// dojo/NodeList
		},

		indexOf: function(value, fromIndex){
			// summary:
			//		see `dojo/_base/array.indexOf()`. The primary difference is that the acted-on
			//		array is implicitly this NodeList
			// value: Object
			//		The value to search for.
			// fromIndex: Integer?
			//		The location to start searching from. Optional. Defaults to 0.
			// description:
			//		For more details on the behavior of indexOf, see Mozilla's
			//		[indexOf
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/indexOf)
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.indexOf(this, value, fromIndex); // Integer
		},

		lastIndexOf: function(value, fromIndex){
			// summary:
			//		see `dojo/_base/array.lastIndexOf()`. The primary difference is that the
			//		acted-on array is implicitly this NodeList
			// description:
			//		For more details on the behavior of lastIndexOf, see
			//		Mozilla's [lastIndexOf
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/lastIndexOf)
			// value: Object
			//		The value to search for.
			// fromIndex: Integer?
			//		The location to start searching from. Optional. Defaults to 0.
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.lastIndexOf(this, value, fromIndex); // Integer
		},

		every: function(callback, thisObject){
			// summary:
			//		see `dojo/_base/array.every()` and the [Array.every
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/every).
			//		Takes the same structure of arguments and returns as
			//		dojo/_base/array.every() with the caveat that the passed array is
			//		implicitly this NodeList
			// callback: Function
			//		the callback
			// thisObject: Object?
			//		the context
			return d.every(this, callback, thisObject); // Boolean
		},

		some: function(callback, thisObject){
			// summary:
			//		Takes the same structure of arguments and returns as
			//		`dojo/_base/array.some()` with the caveat that the passed array is
			//		implicitly this NodeList.  See `dojo/_base/array.some()` and Mozilla's
			//		[Array.some
			//		documentation](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/some).
			// callback: Function
			//		the callback
			// thisObject: Object?
			//		the context
			return d.some(this, callback, thisObject); // Boolean
		},
		=====*/

		concat: function(item){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		as well as items passed in as parameters
			// description:
			//		This method behaves exactly like the Array.concat method
			//		with the caveat that it returns a `NodeList` and not a
			//		raw Array. For more details, see the [Array.concat
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/concat)
			// item: Object?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList

			//return this._wrap(apc.apply(this, arguments));
			// the line above won't work for the native NodeList, or for Dojo NodeLists either :-(

			// implementation notes:
			// Array.concat() doesn't recognize native NodeLists or Dojo NodeLists
			// as arrays, and so does not inline them into a unioned array, but
			// appends them as single entities. Both the original NodeList and the
			// items passed in as parameters must be converted to raw Arrays
			// and then the concatenation result may be re-_wrap()ed as a Dojo NodeList.

			var t = aps.call(this, 0),
				m = array.map(arguments, function(a){
					return aps.call(a, 0);
				});
			return this._wrap(apc.apply(t, m), this);	// dojo/NodeList
		},

		map: function(/*Function*/ func, /*Function?*/ obj){
			// summary:
			//		see `dojo/_base/array.map()`. The primary difference is that the acted-on
			//		array is implicitly this NodeList and the return is a
			//		NodeList (a subclass of Array)
			return this._wrap(array.map(this, func, obj), this); // dojo/NodeList
		},

		forEach: function(callback, thisObj){
			// summary:
			//		see `dojo/_base/array.forEach()`. The primary difference is that the acted-on
			//		array is implicitly this NodeList. If you want the option to break out
			//		of the forEach loop, use every() or some() instead.
			forEach(this, callback, thisObj);
			// non-standard return to allow easier chaining
			return this; // dojo/NodeList
		},
		filter: function(/*String|Function*/ filter){
			// summary:
			//		"masks" the built-in javascript filter() method (supported
			//		in Dojo via `dojo/_base/array.filter`) to support passing a simple
			//		string filter in addition to supporting filtering function
			//		objects.
			// filter:
			//		If a string, a CSS rule like ".thinger" or "div > span".
			// example:
			//		"regular" JS filter syntax as exposed in `dojo/_base/array.filter`:
			//		|	require(["dojo/query", "dojo/NodeList-dom"
			//		|	], function(query){
			//		|		query("*").filter(function(item){
			//		|			// highlight every paragraph
			//		|			return (item.nodeName == "p");
			//		|		}).style("backgroundColor", "yellow");
			//		|	});
			// example:
			//		the same filtering using a CSS selector
			//		|	require(["dojo/query", "dojo/NodeList-dom"
			//		|	], function(query){
			//		|		query("*").filter("p").styles("backgroundColor", "yellow");
			//		|	});

			var a = arguments, items = this, start = 0;
			if(typeof filter == "string"){ // inline'd type check
				items = query._filterResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo/NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(array.filter(items, a[start], a[start + 1]), this);	// dojo/NodeList
		},
		instantiate: function(/*String|Object*/ declaredClass, /*Object?*/ properties){
			// summary:
			//		Create a new instance of a specified class, using the
			//		specified properties and each node in the NodeList as a
			//		srcNodeRef.
			// example:
			//		Grabs all buttons in the page and converts them to dijit/form/Button's.
			//	|	var buttons = query("button").instantiate(Button, {showLabel: true});
			var c = lang.isFunction(declaredClass) ? declaredClass : lang.getObject(declaredClass);
			properties = properties || {};
			return this.forEach(function(node){
				new c(properties, node);
			});	// dojo/NodeList
		},
		at: function(/*===== index =====*/){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		at the given index or indices.
			//
			// index: Integer...
			//		One or more 0-based indices of items in the current
			//		NodeList. A negative index will start at the end of the
			//		list and go backwards.
			//
			// example:
			//	Shorten the list to the first, second, and third elements
			//	|	require(["dojo/query"
			//	|	], function(query){
			//	|		query("a").at(0, 1, 2).forEach(fn);
			//	|	});
			//
			// example:
			//	Retrieve the first and last elements of a unordered list:
			//	|	require(["dojo/query"
			//	|	], function(query){
			//	|		query("ul > li").at(0, -1).forEach(cb);
			//	|	});
			//
			// example:
			//	Do something for the first element only, but end() out back to
			//	the original list and continue chaining:
			//	|	require(["dojo/query"
			//	|	], function(query){
			//	|		query("a").at(0).onclick(fn).end().forEach(function(n){
			//	|			console.log(n); // all anchors on the page.
			//	|	})
			//	|	});

			var t = new this._NodeListCtor(0);
			forEach(arguments, function(i){
				if(i < 0){ i = this.length + i; }
				if(this[i]){ t.push(this[i]); }
			}, this);
			return t._stash(this); // dojo/NodeList
		}
	});

	function queryForEngine(engine, NodeList){
		var query = function(/*String*/ query, /*String|DOMNode?*/ root){
			// summary:
			//		Returns nodes which match the given CSS selector, searching the
			//		entire document by default but optionally taking a node to scope
			//		the search by. Returns an instance of NodeList.
			if(typeof root == "string"){
				root = dom.byId(root);
				if(!root){
					return new NodeList([]);
				}
			}
			var results = typeof query == "string" ? engine(query, root) : query ? (query.end && query.on) ? query : [query] : [];
			if(results.end && results.on){
				// already wrapped
				return results;
			}
			return new NodeList(results);
		};
		query.matches = engine.match || function(node, selector, root){
			// summary:
			//		Test to see if a node matches a selector
			return query.filter([node], selector, root).length > 0;
		};
		// the engine provides a filtering function, use it to for matching
		query.filter = engine.filter || function(nodes, selector, root){
			// summary:
			//		Filters an array of nodes. Note that this does not guarantee to return a NodeList, just an array.
			return query(selector, root).filter(function(node){
				return array.indexOf(nodes, node) > -1;
			});
		};
		if(typeof engine != "function"){
			var search = engine.search;
			engine = function(selector, root){
				// Slick does it backwards (or everyone else does it backwards, probably the latter)
				return search(root || document, selector);
			};
		}
		return query;
	}
	var query = queryForEngine(defaultEngine, NodeList);
	/*=====
	query = function(selector, context){
		// summary:
		//		This modules provides DOM querying functionality. The module export is a function
		//		that can be used to query for DOM nodes by CSS selector and returns a NodeList
		//		representing the matching nodes.
		// selector: String
		//		A CSS selector to search for.
		// context: String|DomNode?
		//		An optional context to limit the searching scope. Only nodes under `context` will be
		//		scanned.
		// example:
		//		add an onclick handler to every submit button in the document
		//		which causes the form to be sent via Ajax instead:
		//	|	require(["dojo/query", "dojo/request", "dojo/dom-form", "dojo/dom-construct", "dojo/dom-style"
		//	|	], function(query, request, domForm, domConstruct, domStyle){
		//	|		query("input[type='submit']").on("click", function(e){
		//	|			e.preventDefault(); // prevent sending the form
		//	|			var btn = e.target;
		//	|			request.post("http://example.com/", {
		//	|				data: domForm.toObject(btn.form)
		//	|			}).then(function(response){
		//	|				// replace the form with the response
		//	|				domConstruct.create(div, {innerHTML: response}, btn.form, "after");
		//	|				domStyle.set(btn.form, "display", "none");
		//	|			});
		//	|		});
		//	|	});
		//
		// description:
		//		dojo/query is responsible for loading the appropriate query engine and wrapping
		//		its results with a `NodeList`. You can use dojo/query with a specific selector engine
		//		by using it as a plugin. For example, if you installed the sizzle package, you could
		//		use it as the selector engine with:
		//		|	require(["dojo/query!sizzle"], function(query){
		//		|		query("div")...
		//
		//		The id after the ! can be a module id of the selector engine or one of the following values:
		//
		//		- acme: This is the default engine used by Dojo base, and will ensure that the full
		//		Acme engine is always loaded.
		//
		//		- css2: If the browser has a native selector engine, this will be used, otherwise a
		//		very minimal lightweight selector engine will be loaded that can do simple CSS2 selectors
		//		(by #id, .class, tag, and [name=value] attributes, with standard child or descendant (>)
		//		operators) and nothing more.
		//
		//		- css2.1: If the browser has a native selector engine, this will be used, otherwise the
		//		full Acme engine will be loaded.
		//
		//		- css3: If the browser has a native selector engine with support for CSS3 pseudo
		//		selectors (most modern browsers except IE8), this will be used, otherwise the
		//		full Acme engine will be loaded.
		//
		//		- Or the module id of a selector engine can be used to explicitly choose the selector engine
		//
		//		For example, if you are using CSS3 pseudo selectors in module, you can specify that
		//		you will need support them with:
		//		|	require(["dojo/query!css3"], function(query){
		//		|		query('#t > h3:nth-child(odd)')...
		//
		//		You can also choose the selector engine/load configuration by setting the query-selector:
		//		For example:
		//		|	<script data-dojo-config="query-selector:'css3'" src="dojo.js"></script>
		//
		return new NodeList(); // dojo/NodeList
	 };
	 =====*/

	// the query that is returned from this module is slightly different than dojo.query,
	// because dojo.query has to maintain backwards compatibility with returning a
	// true array which has performance problems. The query returned from the module
	// does not use true arrays, but rather inherits from Array, making it much faster to
	// instantiate.
	dojo.query = queryForEngine(defaultEngine, function(array){
		// call it without the new operator to invoke the back-compat behavior that returns a true array
		return NodeList(array);	// dojo/NodeList
	});

	query.load = function(id, parentRequire, loaded){
		// summary:
		//		can be used as AMD plugin to conditionally load new query engine
		// example:
		//	|	require(["dojo/query!custom"], function(qsa){
		//	|		// loaded selector/custom.js as engine
		//	|		qsa("#foobar").forEach(...);
		//	|	});
		loader.load(id, parentRequire, function(engine){
			loaded(queryForEngine(engine, NodeList));
		});
	};

	dojo._filterQueryResult = query._filterResult = function(nodes, selector, root){
		return new NodeList(query.filter(nodes, selector, root));
	};
	dojo.NodeList = query.NodeList = NodeList;
	return query;
});

},
'slmui/form/TriStateCheckBox':function(){
define([
	"../../dojo/_base/declare", // declare
	"dojo/text!./templates/TriStateCheckBox.html",
	"idx/form/TriStateCheckBox",
	"dojo/_base/event",
	"dojo/dom",
	"dijit/_CssStateMixin"
], function(declare, tempalate, TriStateCheckBox, event, dom, _CssStateMixin){

	return declare( [TriStateCheckBox, _CssStateMixin], {

		templateString: tempalate,

		caption: "",


		postMixInProperties: function() {
			this.inherited(arguments);

			this.label=this.caption;
			if(this.required){
				this.baseClass = this.baseClass + " dijitXcitecChekboxRequired";
			}
		},

		postCreate: function(){
			dom.setSelectable(this.compLabelNode, false);
		},

		_onClickCaption: function(e){
			// summary:
			//			prevents the onclick event on checkbox caption
			event.stop(e);
			return false;
		}
	});

});
},
'xcitec/layout/Chapter':function(){
/*
 slmui.layout.Chapter is a slmui Layout widget for Chapters
 Ecpected body contect widgets and labels with "for".
 NOTE: all other content items are ignored/lost
 Example:
 <div data-dojo-type="xcitec/layout/Chapter" >
 <label for="country">Country</label>
 <input type="text" id="country" name="country" data-dojo-type="dijit/form/ValidationTextBox" value=""><br>
 <label for="state">State</label>
 <input type="text" id="state" name="state" data-dojo-type="dijit/form/ValidationTextBox" value=""><br>
 <label for="zip_city">ZIP / city</label>
 <input type="text" id="zip_city" name="zip_city" data-dojo-type="dijit/form/ValidationTextBox" value=""><br>
 <label for="street">Street / No</label>
 <input type="text" id="street" name="street" data-dojo-type="dijit/form/ValidationTextBox" value=""><br>
 </div>

 */
define(["dojo/_base/declare",
        "dojo/_base/array",
        "dojo/dom-attr",
        "dojo/dom-class",
        "dojo/query",
        "dojo/dom-style",
        "dijit/registry",
        "./_LayoutWidget",
        "../Label",
        "./_ChapterItem"], function(declare, array, domAttr, domClass, query, domStyle, registry, _LayoutWidget, Label, _ChapterItem){
  return declare(_LayoutWidget, {

    hasOwnLabelProcessing: false,
    templateString: '<div><div class="fieldsetChapterFrame"><div class="fieldsetChapterFrameH"><div class="fieldsetChapterFrameV"><table class="dijitReset fieldsetChapterTable" cellSpacing="0" cellPadding="0">' +
                    '<thead class="dijitReset">' +
                    '<tr class="dijitReset" style="height:0px;">' +
//                  '<td class="dijitReset fieldsetChapterLabelColumn" data-dojo-attach-point="columnLabel"  style="width: 50%;"></td>' +
//                  '<td class="dijitReset fieldsetChapterControlColumn" data-dojo-attach-point="columnWidget" style="width: 50%;"></td>' +
                    '<td class="dijitReset fieldsetChapterLabelColumn" data-dojo-attach-point="columnLabel"></td>' +
                    '<td class="dijitReset fieldsetChapterControlColumn" data-dojo-attach-point="columnWidget"></td>' +
//                  '<td class="dijitReset fieldsetChapterLabelColumn" data-dojo-attach-point="columnLabel"  style="width: 36%;"></td>' +
//                  '<td class="dijitReset fieldsetChapterControlColumn" data-dojo-attach-point="columnWidget" style="width: 64%;"></td>' +
                    '</tr>' +
                    '<tr class="dijitReset" data-dojo-attach-point="chpaterLabelRowNode">' +
                    '<td class="dijitReset fieldsetChapterCaption" colspan="2" data-dojo-attach-point="chpaterLabelContainerNode"></td>' +
                    '</tr>' +
                    '</thead>' +
                    '<tbody class="dijitReset" data-dojo-attach-point="containerNode"></tbody></table></div></div>' +
                    '<div class="fieldsetChapterCorner fieldsetChapterCornerLT"></div>' +
                    '<div class="fieldsetChapterCorner fieldsetChapterCornerRT"></div>' +
                    '<div class="fieldsetChapterCorner fieldsetChapterCornerLB"></div>' +
                    '<div class="fieldsetChapterCorner fieldsetChapterCornerRB"></div>' +
                    '</div><div class="requiredRound"></div></div>',

    baseClass: "fieldsetChapter",
    noLabelControlNodeWidth: "100%", //possibility to overwrite depending on view mode (Chapter2C)

    // label: String
    //  Text to display in button.
    label: '',

    // labelId: String
    //  id Of Label for MultyLanguage support.
    labelId: '',

    useLabelAsCaption: false,

    frameBox: '',

    postMixInProperties: function(){
      this.inherited(arguments);
      if(this.useLabelAsCaption && !("caption" in this.params)){
        this.caption = this.label;
        this.captionId = this.labelId;
      } else {
        //this.hasOwnLabelProcessing = false;
      }
    },

    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
      var controlHolderWidget = new _ChapterItem({controlNode:controlNode, labelNode:labelNode});
      return controlHolderWidget.domNode;
    },

    _fillContent: function(/*DomNode*/ source) {
      this._fcCreateLabel(source);
      this.inherited(arguments);
      if (this.frameBox) {
        switch (this.frameBox.toUpperCase()) {
        case "GRAY":
          domClass.add(this.domNode, "frameBox frameBoxGray");
          break;
        case "LIGHT-BLUE":
          domClass.add(this.domNode, "frameBox frameBoxLightBlue");
          break;
        case "DARK-BLUE":
          domClass.add(this.domNode, "frameBox frameBoxDarkBlue");
          break;
        }
      }
    },

    postCreate: function() {
      this.inherited(arguments);
      // check if there is label at left side -> if no hide label column
      var children = this.getChildren();
      var bLabel = false;
      array.forEach(children, function(child) {
        if (child.hasLabelAtPos("left")) bLabel = true;
      });
//to keep layout we don't need feature to hide label column
//    if (!bLabel) {
//      domStyle.set(this.columnLabel, "width", "0px");
//      domStyle.set(this.columnWidget, "width", this.noLabelControlNodeWidth);  //required for FF3.0
//    }
    },

    _fcCreateLabel: function(/*DomNode*/ source) {
      if (this.caption) {
        var chpaterLabel = new Label({
                                              label:this.caption,
                                              labelId:this.captionId,
                                              htmlFor:this.id
                                            }, null);
        this.chapterLabelNode = chpaterLabel.domNode;
      }
      if (!this.chapterLabelNode && source) {
        // create chapate label if defined in markup
        var chapterLabelNode = query("label[for='" + this.id + "']", source)[0];
        if (chapterLabelNode) {
          var chpaterLabel = new Label([], chapterLabelNode);
          this.chapterLabelNode = chpaterLabel.domNode;
        }
      }
      if (this.chapterLabelNode) {
        this.chpaterLabelContainerNode.appendChild(this.chapterLabelNode);
      } else {
        domStyle.set(this.chpaterLabelContainerNode, "display", "none");
      }
    },

    _processChildWidget: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
      this.inherited(arguments);
      // in additional process nested widgets of slmui.form.EntitySelect
      if ((domAttr.get(controlNode, "data-dojo-type") == "xcitec/form/EntitySelect") || (domAttr.get(controlNode, "data-dojo-type") == "xcitec/form/AutoCompleteEntitySelect")) {
        this._processChildren(controlNode);
      }
    },

    destroyDescendants: function(/*Boolean?*/ preserveDom){
      array.forEach(registry.findWidgets(this.domNode), function(widget){
        if(widget.destroyRecursive){
          widget.destroyRecursive(preserveDom);
        }
      });
      this.inherited(arguments);
    }

  });
});


},
'xcitec/DialogPane':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/query",
	"dojo/text!xcitec/templates/DialogPane.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, declare, windowModule, query, template, _Templated, _Widget) {

	var DialogPane = declare( [_Widget, _Templated],  {
	
	  templateString: template,
	
	  // Caption of Dialog Pane
	  caption: "",
	  captionId: "",
	
	  _fillContent: function(/*DomNode*/ source){
	    if(!source) return;
	
	    // copy Action Buttons to buttonsBar
	    var actBtnsContainer = query("> [xcitecAttachPoint$=buttons]", source)[0];
	    if (actBtnsContainer) {
	      var destNode = this.buttonsBarNode;
	      var sourceNode = actBtnsContainer;
	      if(sourceNode && destNode){
	        while(sourceNode.hasChildNodes()){
	          destNode.appendChild(sourceNode.firstChild);
	        }
	      }
	      source.removeChild(actBtnsContainer);
	    }
	    var dest = this.containerNode;
	    if(source && dest){
	      while(source.hasChildNodes()){
	        dest.appendChild(source.firstChild);
	      }
	    }
	
		// set caption
		if(this.caption != ""){
			this.captionNode.appendChild(windowModule.doc.createTextNode(this.caption));
		}
	
	  },
	
	
	//  showOnLoad: false,
	//  postCreate: function(){
	//    this.inherited(arguments);
	//    var xcitec_form = dojo.byId("xcitec_form");
	//    if (xcitec_form) {
	//      xcitec_form.appendChild(this.domNode);
	//    }
	//    if (this.showOnLoad) {
	//      dojo.addOnLoad(dojo.hitch(this, function(){ this.show(); }));
	//    }
	//  },
	
	  hide: function(){
	    //do nothing we are reloading page!!!
	  }
	
	});
	
	return DialogPane;
});

},
'xcitec/xlayout/Question':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/query",
	"xcitec/Text",
	"xcitec/layout/_LabelControlHolder",
	"xcitec/layout/_LayoutWidget"
], function (dojo, declare, windowModule, domConstruct, domStyle, query, Text, _LabelControlHolder, _LayoutWidget) {

	var Question = declare( _LayoutWidget, {
	  templateString :
	          '<div class="${baseClass}" data-dojo-attach-point="containerNode">' +
	          '<div class="QItemNA" data-dojo-attach-point="naHolderNode"></div>' +
	          '</div>',
	  baseClass: "XLayoutQuestion",
	
	  lineTop: false,
	  lineBottom: false,
	
	  _fillContent: function(/*DomNode*/ source){
	    if(source){
	//      caption (������ �������� xclRole="title")
	//      lineTop (������ �������� xclRole="line_before")
	//      lineBottom (������ �������� xclRole="line_after")
	
	      //var widgets = dojo.query("> [dojoType]", source);
	      var naNode = query("> [xclRole$=na]", source)[0];
	      if (naNode) {
	        this.naHolderNode.appendChild(naNode);
	      } else {
	        domStyle.set(this.naHolderNode, "display", "none");
	      }
	//      var captionNode = dojo.query("> [xclRole$=title]", source)[0];
	//      if (captionNode){
	//        dojo.addClass(captionNode, 'QTitle');
	//        this.containerNode.appendChild(captionNode);
	//      }
	      if (this.caption) {
	        var captionWidget = new Text ({
	           baseClass: "xcitecTextBox QTitle",
	           text:this.caption,
	           textId:this.captionId
	        }, null);
	        //dojo.place(captionWidget.domNode, this.domNode, "first");
	        this.containerNode.appendChild(captionWidget.domNode);
	      }
	    }
	    this.inherited(arguments);
	    if (this.lineTop) {
	      var lineNode = windowModule.doc.createElement('hr');
	      domConstruct.place(lineNode, this.containerNode, "first");
	    }
	    if (this.lineBottom) {
	      var lineNode = windowModule.doc.createElement('hr');
	      domConstruct.place(lineNode, this.containerNode, "last");
	    }
	  },
	
	  buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
	    var controlContainer = new xcitec.xlayout._QuestionItem({controlNode:controlNode, labelNode:labelNode});
	    return controlContainer.domNode;
	  }
	});
	
	declare("xcitec.xlayout._QuestionItem", _LabelControlHolder, {
	  templateString: '<div class="dijitReset fieldsetQuestionItem" data-dojo-attach-point="containerNode, controlHolderNode"></div>'
	});
	
	return Question;
});
},
'xcitec/page/SubTabPanelV1':function(){
define([
	"dojo/_base/declare",
	"dojo/text!xcitec/page/templates/SubTabPanelV1.html",
	"xcitec/layout/XLayout"
], function (declare, template, XLayout) {

	var SubTabPanelV1 = declare( XLayout, {
	    templateString: template
	});
	return SubTabPanelV1;
});
},
'xcitec/formx/STICriterion':function(){
define([
	"dojo",
	"dojo/dom-style",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom",
	"dojo/fx",
	"dojo/query",
	"dojo/text!xcitec/formx/templates/STICriterion.html",
	"xcitec/_StateWidget",
  "../xcitecUtil",
  "../_base/ImageFactory"
], function (dojo, domStyle, array, declare, lang, windowModule, domAttr, domClass, domConstruct, dom, fx, query, template, _StateWidget, xcitecUtil, ImageFactory) {

	/*
	    STICriterion is a layout control which keeps inside it other controls, places them in appropriate places
	    and could be expanded/collapsed. Inner controls must have appropriate xcitec roles to be placed in their placeholders.
	    These roles are: 'weight', 'rating', 'ratinglabel', 'description', 'comment', 'na'
	     <div data-dojo-type="xcitec.formx.STICriterion" caption="Monopoly position" state="Close>
	        <div xcitecRole="weight" data-dojo-type="xcitec.Text" icon="WEIGHT:criterion">20%</div>
	        <div xcitecRole="ratinglabel" data-dojo-type="xcitec.Label">high (60%)</div>
	        <span xcitecRole="rating" label="Rating with selected attribute"
	              data-dojo-type="xcitec.form.Rating" readonly="true"
	              name="rating1">
	                ...
	        </span>
	        <div xcitecRole="description" data-dojo-type="xcitec.Text">
	                ...
	        </div>
	        <textarea xcitecRole="comment" data-dojo-type="xcitec.form.Textarea" readonly="true">
	                ...
	        </textarea>
	     </div>
	*/
	
	var STICriterion = declare( _StateWidget, {
	    templateString: template,
	
	    baseClass: "stiComboExpendableHead",
	
	    caption: "",
	    captionId: "",
	
	    xcitecRoles: null, // Array
	
	    // STICriterion states: "Open" or "Close", "Open" by default
	    state: "Open",
	
	    iconClass: "",
	
	    commentExists: false,
	
	    postMixInProperties: function() {
	        // search for controls marked with xcitecRole attribute
	        this.xcitecRoles = query(" > [xcitecRole]", this.srcNodeRef);
	        // set iconClass
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        }
	        this.inherited(arguments);
	    },
	
	    _fillContent: function() {
	        var _this = this;
	
	        // move controls in appropriate places if they exist
	        array.forEach(_this.xcitecRoles, function(node) {
	            switch (domAttr.get(node, "xcitecRole").toLowerCase()) {
	                case "weight":
	                    domConstruct.place(node, _this.weight, "last");
	                    break;
	                case "rating":
	                    domConstruct.place(node, _this.rating, "last");
	                    break;
	                case "ratinglabel":
	                    domConstruct.place(node, _this.ratinglabel, "last");
	                    break;
	                case "description":
	                    domConstruct.place(node, _this.description, "last");
	                    break;
	                case "comment":
	                    domConstruct.place(node, _this.comment, "last");
	                    break;
	                case "na":
	                    domConstruct.place(node, _this.na, "last");
	                    break;
	                case "stistateholder":
	                    _this.stateHolder = node;
	                    var state = xcitecUtil.str2bool(domAttr.get(_this.stateHolder, "value"));
	                    state ? _this.state = "Open" : _this.state = "Close";
	                    break;
	                default:
	                    break;
	            }
	        });
	        if (!xcitecUtil.str2bool(this.commentExists)) {
	            domConstruct.destroy(this.commentIcon);
	        }
	        this._setStateClass();
	
	        // destroy icon node if no iconClass provided
	        if (!this.iconClass) {
	            domConstruct.destroy(this.statusIcon);
	        }
	        if (this.state == "Close") {
	            domClass.add(this.comboBody, "stiComboExpendableCloseBody")
	        }
	        if (this.stateHolder) {
	            domConstruct.place(this.stateHolder, this.domNode, "first");
	        }
	        dom.setSelectable(this.stateNode, false);
	
		    // set caption
			if(this.caption != ""){
				this.captionNode.appendChild(windowModule.doc.createTextNode(this.caption));
			}
	        this.inherited(arguments);
	    },
	
	    _onClick: function() {
	        switch (this.state) {
	            case "Close":
	                this.expand();
	                break;
	            case "Open":
	                this.collapse();
	                break;
	            default:
	                break;
	        }
	    },
	
	    expand: function() {
	        this.state = "Open";
	        domStyle.set(this.comboBody, "display", "none");
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "true");
	        }
	        if (domClass.contains(this.comboBody, "stiComboExpendableCloseBody")) {
	            domClass.remove(this.comboBody, "stiComboExpendableCloseBody");
	        }
	        fx.wipeIn({ node: this.comboBody, beforeBegin: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    collapse: function() {
	        this.state = "Close";
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "false");
	        }
	        fx.wipeOut({ node: this.comboBody, onEnd: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    getState: function() {
	        return this.state;
	    }
	});
	return STICriterion;
});

},
'xcitec/xlayout/_QuestionV2Item':function(){
define([
         "dojo/_base/declare",
         "../layout/_ChapterItem"
       ], function (declare, _ChapterItem) {
  return declare(_ChapterItem, {

    templateStringNoLabelMode: '<tr class="dijitReset fieldsetChapterRow"><td class="dijitReset fieldsetChapterControlCell"><div data-dojo-attach-point="controlHolderNode"></div></td></tr>',

    designMode: false,
    noLabelMode: false,

    processAdditionalProperties: function(/*DomNode*/ controlNode) {
      this.inherited(arguments);
      if (this.designMode) {
        this.controlNode.tabIndex = '-1';
      }
      if(this.noLabelMode){
          this.templateString = this.templateStringNoLabelMode;
      }
    }
  });
});

},
'xcitec/_base/ImageFactory':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
  "./ImageFamily",
  "./FlagImageFamily",
  "./LanguageImageFamily"
], function (declare, lang, ImageFamily, FlagImageFamily, LanguageImageFamily) {

	var factory = declare(null, {
		// summary:
		//		A set of widgets indexed by id. A default instance of this class is
		//		available as `dijit.registry`
		//
		constructor: function(){
			this._hash = {};
		},
	
		addImageFamily: function(/*xcitec._base.ImageFamily*/ imageFamily){
			// summary:
			//		Add a ImageFamily to this list.
			//
			// imageFamily: xcitec._base.ImageFamily
			//		Any dijit._Widget subclass.
			this._hash[imageFamily.familyId.toUpperCase()] = imageFamily;
		},
	
		addStandardImageFamily: function(/*Object*/ definitionParams){
			this.addImageFamily(lang.mixin(new ImageFamily(), definitionParams));
		},
	
		removeImageFamily: function(/*String*/ id){
			delete this._hash[id];
		},
	
		createIconCss: function(/*String*/ iconClassName){
			// summary:
			//		resturn a CSS represention for requested Icon.
			//
			// iconClassName: String
			//		name of icon
			//
			// descriptoon:
			//		resturn a CSS for represent requisted Icon by Icon Name.
			//    if Name is matched to FAMILY:ICONID and specified FAMILY is defined
			//    it delegates job to FAMILY class otherewise return source name.
	
			if(!iconClassName){ return null; }
			var valarr = iconClassName.split(":");
			var imageFamilyId = valarr[0].toUpperCase();
			var imageFamily = this._hash[imageFamilyId];
			if(imageFamily){
				return imageFamily.createIconCss(iconClassName.substr(imageFamilyId.length + 1));
			}
			return iconClassName;
		}
	});
	
	var imageFactory = new factory();
  imageFactory.addImageFamily(new FlagImageFamily());
  imageFactory.addImageFamily(new LanguageImageFamily());
	
	imageFactory.addStandardImageFamily({familyId: "TOOLBAR", familyCSS: "toolbarIcon"});
	imageFactory.addStandardImageFamily({familyId: "SGOMINI", familyCSS: "sboIconMini"});
	imageFactory.addStandardImageFamily({familyId: "WEIGHT", familyCSS: "weightIcon"});
	imageFactory.addStandardImageFamily({familyId: "BOX", familyCSS: "boxIcon"});
	imageFactory.addStandardImageFamily({familyId: "TASK", familyCSS: "taskButton"});
	imageFactory.addStandardImageFamily({familyId: "ATTACHMENT", familyCSS: "attachmentIcon"});
	imageFactory.addStandardImageFamily({familyId: "STI", familyCSS: "fulfillmentIcon"});
	imageFactory.addStandardImageFamily({familyId: "EVA", familyCSS: "fulfillmentIcon"});
	imageFactory.addStandardImageFamily({familyId: "PROCESS", familyCSS: "processGraph"});
	imageFactory.addStandardImageFamily({familyId: "RADIO", familyCSS: "dijitXcitecRadioPic"});
	imageFactory.addStandardImageFamily({familyId: "CLACLASS", familyCSS: "classificationIcon"});
	imageFactory.addStandardImageFamily({familyId: "RISCLASS", familyCSS: "riskIcon"});
	imageFactory.addStandardImageFamily({familyId: "STICLASS", familyCSS: "stiIcon"});
	imageFactory.addStandardImageFamily({familyId: "EVACLASS", familyCSS: "evaIcon"});
	
	imageFactory.addStandardImageFamily({familyId: "XGOMINI", familyCSS: "sboIconMini"});
	imageFactory.addStandardImageFamily({familyId: "XGOLOGO", familyCSS: "sboIcon", familyPostfix: "Big"});
	imageFactory.addStandardImageFamily({familyId: "IMG", familyCSS: "img"});
	
	imageFactory.addStandardImageFamily({familyId: "FICON", familyCSS: "frontIcon"});
	
	return imageFactory;
});

},
'xcitec/Subsegment':function(){
define([
	"dojo",
	"dojo/dom-class",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dom",
	"dojo/fx",
	"dojo/on",
	"dojo/query",
	"dojo/parser",
	'dojo/dom-geometry',
	'dojo/window',
	"dojo/text!xcitec/templates/Subsegment.html",
	"dijit/registry",
	"xcitec/_StateWidget",
  "./xcitecUtil"
], function (dojo, domClass, array, declare, lang, windowModule, domAttr, domConstruct, domStyle, dom, fx, on, query, parser, domGeometry, windowD, template, registry, _StateWidget, xcitecUtil) {

	/*
	    Subsegment is a control which keeps inside it other controls and could be expanded/collapsed.
	    Inner controls must have appropriate xcitec roles to be placed in their placeholders.
	    These roles are: 'evaluate', xcitecAttachPoint="buttons".
	    <div data-dojo-type="xcitec.Subsegment" caption="Strategic partner (92,74 pts) for A01, Steel Pipes" editMode="true">
	        <div xcitecRole="evaluate" data-dojo-type="xcitec.form.InlineLinkButton" label="Disabled LinkButton"
	             onclick="alert('Evaluate button clicked!')" iconClass="icon_edit">Evaluate
	        </div>
	        <div data-dojo-type="xcitec.layout.ContentArea">
	            ...
	        </div>
	        <div xcitecAttachPoint="buttons">
	            <div data-dojo-type="xcitec.form.Button" iconClass="TOOLBAR:Save">Save</div>
	            <div data-dojo-type="xcitec.form.Button" iconClass="TOOLBAR:Abort">Cancel</div>
	        </div>
	    </div>
	 */
	
	var Subsegment = declare( _StateWidget, {
	    templateString: template,
	
	    baseClass: "subsegment",

		// Flag to parser that I'll parse my contents, so it shouldn't.
		stopParser: true,
	
	    // Subsegment caption
	    caption: "",
	    captionId: "",
	
	    // Subsegment state, could be "Open" or "Close", "Open" by default
	    state: "Open",
	
	    // is Subsegment in edit mode
	    editMode: false,
	
	    xcitecRoles: null, // Array
	
	    // Subsegment icons
	    score: "",
	    critical: "",
	    ko: "",
	
	    // id of parent subsegment
	    parent: "",
	
	    // level of subsegment in hierarchy
	    nestingLevel: 0,
	
	    // children of this subsegment (Array)
	    children: null,
	
	    // is in design mode
	    designMode: false,
	
	    // is onDelete action provided
	    deleteAttr: null,
	
	    settingsAttr: null,
	
	    // ref to SubsegmentContainer
	    enclosingWidget: null,

		isContentLoaded: false,
	
	    postMixInProperties: function() {
	        // search for controls marked with xcitecRole attribute
	        this.xcitecRoles = query("[xcitecRole]", this.srcNodeRef);
	        // set edit mode style if nedeed
	        if (this.editMode) {
	            this.baseClass = this.baseClass + " subsegmentEdit";
	        }
	        // search for buttons
	        this.buttonsContainer = query("> [xcitecRole$=subsegmentBottomButtons]", this.srcNodeRef);
	        this.deleteAttr = domAttr.get(this.srcNodeRef, "onDelete");
	        this.settingsAttr = domAttr.get(this.srcNodeRef, "onSettings");
	        this.children = [];
	        this.inherited(arguments);
	    },
	
	    _fillContent: function() {
	        var _this = this;
	        var isButtonExists = false;
	
	        // move controls in appropriate places if they exist
	        array.forEach(_this.xcitecRoles, function(node) {
	            switch (domAttr.get(node, "xcitecRole").toLowerCase()) {
	                case "evaluate":
	                    domConstruct.place(node, _this.evaluate, "last");
	                    break;
	                case "substateholder":
	                    _this.stateHolder = node;
	                    var state = xcitecUtil.str2bool(domAttr.get(_this.stateHolder, "value"));
	                    state ? _this.state = "Open" : _this.state = "Close";
	                    break;
	                default:
	                    break;
	            }
	        });
	
	        if (this.buttonsContainer.length > 0) {
	            domConstruct.place(this.buttonsContainer[0], this.bottomButtons, "last");
	            isButtonExists = true;
	        } else {
	            domConstruct.destroy(this.bottomButtons);
	        }
	        this.inherited(arguments);
	        if (isButtonExists) {
	            domConstruct.place(this.bottomButtons, this.containerNode, "last");
	        }
	        if (!this.designMode) {
	            domConstruct.destroy(this.editButtons);
	        }
	        if("score" in this.params) {
	            domClass.add(this.iconMarkingScore, "iconMarkingScore");
	            domStyle.set(this.iconMarkingScore, "display", "inline");
	            if (this.score != "true") {
	                domAttr.set(this.iconMarkingScore, "title", this.score);
	            }
	        }
	        if ("critical" in this.params) {
	            domClass.add(this.iconMarkingCritical, "iconMarkingCritical");
	            domStyle.set(this.iconMarkingCritical, "display", "inline");
	            if (this.critical != "true") {
	                domAttr.set(this.iconMarkingCritical, "title", this.critical);
	            }
	        }
	        if ("ko" in this.params) {
	            domClass.add(this.iconMarkingKo, "iconMarkingKo");
	            domStyle.set(this.iconMarkingKo, "display", "inline");
	            if (this.ko != "true") {
	                domAttr.set(this.iconMarkingKo, "title", this.ko);
	            }
	        }
	
		    // set caption
			if(this.caption != ""){
				this.titleNode.appendChild(windowModule.doc.createTextNode(this.caption));
			}
	    },
	
	    postCreate: function(){
	        this._setStateClass();
	        dom.setSelectable(this.subsegmentHead, false);
	        if(this.stateHolder){
	            domConstruct.place(this.stateHolder, this.domNode, "first");
	        }
	        var _this = this;
	        on(this, "MouseOut", function(){
	            domClass.remove(_this.domNode, "dojoDndItemXcitecOver");
	        });
	        this.inherited(arguments);
	    },

		startup: function() {
			if (this._started) {
				return;
			}
			domStyle.set(this.subsegmentBody, "display", "none");

			this.own(
				on(windowModule.global, "scroll", lang.hitch(this, "_checkPosition")),
				on(windowModule.global, "resize", lang.hitch(this, "_checkPosition"))
			);

			if (this.state == "Open") {
				this._checkPosition();
			}
		},
	
	    _onClick: function(e) {
	        switch (this.state) {
	            case "Open":
	                this.collapse();
	                break;
	            case "Close":
	                this.expand();
	                break;
	            default:
	                break;
	        }
	    },
	
	    expand: function() {
			this._loadExternalContent();
	        this.state = "Open";
	        domStyle.set(this.subsegmentBody, "display", "none");
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "true");
	        }
	        fx.wipeIn({ node: this.subsegmentBody, beforeBegin: lang.hitch(this, "_setStateClass") }).play();
	    },
	
	    collapse: function() {
	        this.state = "Close";
	        if (this.stateHolder) {
	            domAttr.set(this.stateHolder, "value", "false");
	        }
	        fx.wipeOut({ node: this.subsegmentBody, onEnd: lang.hitch(this, "_onCollapse") }).play();
	    },
	
	    getState: function() {
	        return this.state;
	    },
	
	    setNestingLevel: function(nestingLevel) {
	        this.nestingLevel = nestingLevel;
	        this.setShift();
	    },
	
	    getNestingLevel: function() {
	        return this.nestingLevel;
	    },
	
	    getParent: function() {
	        return this.parent;
	    },
	
	    setParent: function(parentId) {
	        this.parent = parentId;
	    },
	
	    setShift: function() {
	        domStyle.set(this.left, "marginLeft", this.getNestingLevel() * 40 + "px");
	    },
	
	    _onDeleteClick: function() {
	        if (this.deleteAttr) {
	            this.onDelete();
	        } else {
	            if (this.enclosingWidget.isDeleteSet()) {
	                this.enclosingWidget.getClicked().value = this.id;
	                this.enclosingWidget.onDelete();
	            } else {
	                if (this.getParent() != "") {
	                    registry.byId(this.getParent()).removeChild(this);
	                }
	                this.removeChildrenNodes(this);
	                this.destroyRecursive();
	                if (this.enclosingWidget) {
	                    this.enclosingWidget.updateHierarchy();
	                }
	            }
	        }
	    },
	
	    // removes all children (including non-direct) from widget
	    removeChildrenNodes: function(widget) {
	        var _this = this;
	        array.forEach(widget.getNestedSegments(), function(child) {
	            _this.removeChildrenNodes(child);
	            child.destroyRecursive();
	        });
	    },
	
	    _onSettingsClick: function() {
	        if (this.onSettingsAttr) {
	            this.onSettings();
	        } else {
	            if (this.enclosingWidget.isSettingsSet()) {
	                this.enclosingWidget.getClicked().value = this.id;
	                this.enclosingWidget.onSettings();
	            }
	        }
	    },
	
	    onDelete: function() {
	
	    },
	
	    onSettings: function() {
	
	    },
	
	    getNestedSegments: function() {
	        return this.children;
	    },
	
	    addChild: function(/* Widget */child) {
	        this.children.push(child);
	    },
	
	    js2splice: function(arr, index, howMany) {
	        var i;
	        var returnArr = new Array();
	        var postArr = new Array();
	        for (i = 0; i < arr.length; i++) {
	            if (i < index) returnArr.push(arr[i]);
	            if (i >= (index + howMany)) postArr.push(arr[i]);
	        }
	        for (i = 3; i < arguments.length; i++)
	            returnArr.push(arguments[i]);
	        for (i = 0; i < postArr.length; i++)
	            returnArr.push(postArr[i]);
	        return returnArr;
	    },
	
	    removeChild: function(/* Widget */ child) {
	        this.children = this.js2splice(this.children, array.indexOf(this.children, child), 1);
	    },
	
	    hasChild: function(/* Widget */ child) {
	        var has = false;
	        (array.indexOf(this.children, child) >= 0) ? has = true : has = false;
	        return has;
	    },
	
	    setEnclosingWidget: function(/* Widget */ enclosingWidget) {
	        this.enclosingWidget = enclosingWidget;
	    },
	
	    getDeleteAttr: function() {
	        return this.deleteAttr;
	    },
	
	    getSettingsAttr: function() {
	        return this.settingsAttr;
	    },

		_loadExternalContent: function() {
			if (!this.isContentLoaded) {
				domStyle.set(this.subsegmentBody, "display", "");
				parser.parse(this.containerNode);
				this.isContentLoaded = true;
			}
		},

		_checkPosition: function(){
			var vs = windowD.getBox();
			var pos = domGeometry.position(this.domNode, true);
			if(pos.y<(vs.t+vs.h)){
				this._loadExternalContent();
			}
		},

		_onCollapse: function(){
			this._setStateClass();
			this.enclosingWidget.resize();
		},
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		}
	});
	return Subsegment;
});

},
'xcitec/form/Slider':function(){
define([
	"dojo",
	"dojo/text!xcitec/form/templates/Slider.html",
	"dojo/dnd/move",
	"dojo/dnd/Moveable",
	"dojo/dnd/Mover",
	"dojo/_base/array",
	"dojo/number",
	"dojo/_base/fx",
	"dojo/dom-style",
	"dojo/has",
	"dojo/query",
	"dojo/_base/declare",
	"dojo/on",
	"dojo/dom-construct",
	"dojo/dom",
	"dojo/_base/lang",
	"dojo/dom-geometry",
	"dojo/topic",
  "dojo/keys",
	"dijit/Tooltip",
	"dijit/_Container",
	"dijit/form/Button",
	"dijit/form/HorizontalRule",
	"dijit/form/_FormWidget",
  "dijit/form/_FormValueWidget",
  "dijit/_base/manager",
  "dijit/focus",
  "dijit/typematic",
	"xcitec/form/SliderLabels"
], function (dojo, template, move, Moveable, Mover, array, number, fx, domStyle, has, query, declare, on, domConstruct, dom, lang, domGeometry, topic, keys, Tooltip,
             _Container, Button, HorizontalRule, _FormWidget, _FormValueWidget, manager, focus, typematic, SliderLabels) {

	var Slider = declare(
		
		[_FormValueWidget, _Container],
	{
		// summary:
		//		A form widget that allows one to select a value with a horizontally draggable handle
	
		templateString: template,
	
		// Overrides FormValueWidget.value to indicate numeric value
		value: 0,
	
		// showButtons: Boolean
		//		Show increment/decrement buttons at the ends of the slider?
		showButtons: true,
	
		// minimum:: Integer
		//		The minimum value the slider can be set to.
		minimum: 0,
	
		// maximum: Integer
		//		The maximum value the slider can be set to.
		maximum: 100,
	
		// discreteValues: Integer
		//		If specified, indicates that the slider handle has only 'discreteValues' possible positions,
		//      and that after dragging the handle, it will snap to the nearest possible position.
		//      Thus, the slider has only 'discreteValues' possible values.
		//
		//		For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has
		//		three possible positions, representing values 10, 20, or 30.
		//
		//		If discreteValues is not specified or if it's value is higher than the number of pixels
		//		in the slider bar, then the slider handle can be moved freely, and the slider's value will be
		//		computed/reported based on pixel position (in this case it will likely be fractional,
		//      such as 123.456789).
		discreteValues: Infinity,
	
		// pageIncrement: Integer
		//		If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)
		//      that the slider handle is moved via pageup/pagedown keys.
	    //      If discreteValues is not specified, it indicates the number of pixels.
		pageIncrement: 1,
	
		// clickSelect: Boolean
		//		If clicking the slider bar changes the value or not
		clickSelect: true,
	
		// slideDuration: Number
		//		The time in ms to take to animate the slider handle from 0% to 100%,
		//		when clicking the slider bar to make the handle move.
		slideDuration: manager.defaultDuration,
	
	    tooltipPosition: ["before", "above"],
	
	    labels: null, // Array
	    values: null, // Array
	    currentValue: null,
	    selectedValue: 0,
	    valuesArray: null, // Array
	    sendValuesArray: null, // Array
	    labelWidget: null,
	    options: "",
	    labWidg: null,
	
		// Flag to _Templated
		widgetsInTemplate: true,
	
		attributeMap: lang.delegate(_FormWidget.prototype.attributeMap, {
			id: ""
		}),
	
		baseClass: "dijitSlider",
	
		_mousePixelCoord: "pageX",
		_pixelCount: "w",
		_startingPixelCoord: "x",
		_startingPixelCount: "l",
		_handleOffsetCoord: "left",
		_progressPixelSize: "width",
	
		postMixInProperties: function(){
			this.labels = [];
			this.values = [];
			this.valuesArray = [];
			this.sendValuesArray = [];
			this.inherited(arguments);
		},
	
		_onKeyPress: function(/*Event*/ e){
			if(this.disabled || this.readOnly || e.altKey || e.ctrlKey){ return; }
			switch(e.charOrCode){
				case keys.HOME:
	                this.currentValue = this.minimum;
					this._setValueAttr(this.minimum, true);
					break;
				case keys.END:
	                this.currentValue = this.maximum;
					this._setValueAttr(this.maximum, true);
					break;
				// this._descending === false: if ascending vertical (min on top)
				// (this._descending || this.isLeftToRight()): if left-to-right horizontal or descending vertical
				case ((this._descending || this.isLeftToRight()) ? keys.RIGHT_ARROW : keys.LEFT_ARROW):
				case (this._descending === false ? keys.DOWN_ARROW : keys.UP_ARROW):
				case (this._descending === false ? keys.PAGE_DOWN : keys.PAGE_UP):
					this.increment(e);
					break;
				case ((this._descending || this.isLeftToRight()) ? keys.LEFT_ARROW : keys.RIGHT_ARROW):
				case (this._descending === false ? keys.UP_ARROW : keys.DOWN_ARROW):
				case (this._descending === false ? keys.PAGE_UP : keys.PAGE_DOWN):
					this.decrement(e);
					break;
				default:
					return;
			}
			e.preventDefault(), e.stopPropagation();
		},
	
	    buildRendering: function() {
	        this.options = query("div", this.srcNodeRef).length;
	        var ol = domConstruct.create("ol");
	        ol.innerHTML = this.srcNodeRef.innerHTML;
	        this.srcNodeRef.innerHTML = '';
	        this.srcNodeRef.appendChild(ol);
	        var src = query("> ol", this.srcNodeRef);
	        this.labWidg = new SliderLabels({}, src[0]);
	        this.inherited(arguments);
	    },
	
		_onHandleClick: function(e){
			if(this.disabled || this.readOnly){ return; }
			if(!has("ie")){
				// make sure you get focus when dragging the handle
				// (but don't do on IE because it causes a flicker on mouse up (due to blur then focus)
				focus.focus(this.sliderHandle);
			}
			e.preventDefault(), e.stopPropagation();
		},
	
		_isReversed: function(){
			// summary:
			//		Returns true if direction is from right to left
			// tags:
			//		protected extension
			return !this.isLeftToRight();
		},
	
		_onBarClick: function(e){
			if(this.disabled || this.readOnly || !this.clickSelect){ return; }
			focus.focus(this.sliderHandle);
			e.preventDefault(), e.stopPropagation();
			var abspos = domGeometry.position(this.sliderBarContainer, true);
			var pixelValue = e[this._mousePixelCoord] - abspos[this._startingPixelCoord];
			this._setPixelValue(this._isReversed() ? (abspos[this._pixelCount] - pixelValue) : pixelValue, abspos[this._pixelCount], true);
			this._movable.onMouseDown(e);
		},
	
		_setPixelValue: function(/*Number*/ pixelValue, /*Number*/ maxPixels, /*Boolean, optional*/ priorityChange){
			if(this.disabled || this.readOnly){ return; }
			pixelValue = pixelValue < 0 ? 0 : maxPixels < pixelValue ? maxPixels : pixelValue;
			var count = this.discreteValues;
			if(count <= 1 || count == Infinity){ count = maxPixels; }
			count--;
			var pixelsPerValue = maxPixels / count;
			var wholeIncrements = Math.round(pixelValue / pixelsPerValue);
			this._setValueAttr((this.maximum-this.minimum)*wholeIncrements/count + this.minimum, priorityChange);
	        this.currentValue = (this.maximum-this.minimum)*wholeIncrements/count + this.minimum;
		},
	
		_setValueAttr: function(/*Number*/ value, /*Boolean, optional*/ priorityChange){
			// summary:
			//		Hook so attr('value', value) works.
	       
			if (this.currentValue != null) {
	            if (this.sendValuesArray[this.currentValue]) {
	                this.valueNode.value = this.value = this.sendValuesArray[this.currentValue];
	            } else {
	                this.valueNode.value = this.value = this.values[this.currentValue];
	            }
	        }
	
	        if (this.labelWidget) {
	            //dojo.attr(this.labelWidget.textbox, "value", this.labels[this.currentValue]);
	            this.labelWidget.textNode.innerHTML = this.labels[this.currentValue];
	            
	        }

			this.focusNode.setAttribute("aria-valuenow", value);
			this.inherited(arguments);
			var percent = (value - this.minimum) / (this.maximum - this.minimum);
			var progressBar = (this._descending === false) ? this.remainingBar : this.progressBar;
			var remainingBar = (this._descending === false) ? this.progressBar : this.remainingBar;
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			if(priorityChange && this.slideDuration > 0 && progressBar.style[this._progressPixelSize]){
				// animate the slider
				var _this = this;
				var props = {};
				var start = parseFloat(progressBar.style[this._progressPixelSize]);
				var duration = this.slideDuration * (percent-start/100);
				if(duration == 0){ return; }
				if(duration < 0){ duration = 0 - duration; }
				props[this._progressPixelSize] = { start: start, end: percent*100, units:"%" };
				this._inProgressAnim = fx.animateProperty({ node: progressBar, duration: duration,
					onAnimate: function(v){ remainingBar.style[_this._progressPixelSize] = (100-parseFloat(v[_this._progressPixelSize])) + "%"; },
					onEnd: function(){ delete _this._inProgressAnim; },
					properties: props
				})
				this._inProgressAnim.play();
			}
			else{
				progressBar.style[this._progressPixelSize] = (percent*100) + "%";
				remainingBar.style[this._progressPixelSize] = ((1-percent)*100) + "%";
			}
		},
	
		_bumpValue: function(signedChange){
			if(this.disabled || this.readOnly){ return; }
			var s = domStyle.getComputedStyle(this.sliderBarContainer);
			var c = domGeometry.getContentBox(this.sliderBarContainer, s);
			var count = this.discreteValues;
			if(count <= 1 || count == Infinity){ count = c[this._pixelCount]; }
			count--;
			var value = (this.currentValue - this.minimum) * count / (this.maximum - this.minimum) + signedChange;
			if(value < 0){ value = 0; }
			if(value > count){ value = count; }
			value = value * (this.maximum - this.minimum) / count + this.minimum;
	        this.currentValue = value;
			this._setValueAttr(value, true);
		},
	
		_onClkBumper: function(val){
			if(this.disabled || this.readOnly || !this.clickSelect){ return; }
	        this.currentValue = val;
			this._setValueAttr(val, true);
		},
	
		_onClkIncBumper: function(){
			this._onClkBumper(this._descending === false ? this.minimum : this.maximum);
		},
	
		_onClkDecBumper: function(){
			this._onClkBumper(this._descending === false ? this.maximum : this.minimum);
		},
	
		decrement: function(e){
			// summary:
			//		Decrement slider by 1 unit
			// tags:
			//		private
			this._bumpValue(e.charOrCode == keys.PAGE_DOWN ? -this.pageIncrement : -1);
		},
	
		increment: function(e){
			// summary:
			//		Increment slider by 1 unit
			// tags:
			//		private
			this._bumpValue(e.charOrCode == keys.PAGE_UP ? this.pageIncrement : 1);
		},
	
		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Event handler for mousewheel where supported
			evt.preventDefault(), evt.stopPropagation();
			// FIXME: this adds mouse wheel support for safari, though stopEvent doesn't prevent
			// it from bleeding to window?!
			var janky = !has("mozilla");
			var scroll = evt[(janky ? "wheelDelta" : "detail")] * (janky ? 1 : -1);
			this[(scroll < 0 ? "decrement" : "increment")](evt);
		},
	
		startup: function(){
			array.forEach(this.getChildren(), function(child){
				if(this[child.container] != this.containerNode){
					this[child.container].appendChild(child.domNode);
				}
			}, this);
	        var child = this.getChildren()[this.getChildren().length - 1];
	        this.labels = child.labelText;
	        this.values = child.labels;
	        this.minimum = 0;
	        this.maximum = this.values.length - 1;
	        for (var i = this.minimum; i <= this.maximum; i++) {
	            this.valuesArray = this.valuesArray.concat(i);
	        }
	        this.selectedValue = child.selected;
	        this.value = this.selectedValue;
	        //this._setValueAttr(this.value);
	
	        /* temporary */
	        var priorityChange = true;
	        this.valueNode.value = this.value;
			this.focusNode.setAttribute("aria-valuenow", this.value);
			this.inherited(arguments);
			var percent = (this.value - this.minimum) / (this.maximum - this.minimum);
			var progressBar = (this._descending === false) ? this.remainingBar : this.progressBar;
			var remainingBar = (this._descending === false) ? this.progressBar : this.remainingBar;
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			if(priorityChange && this.slideDuration > 0 && progressBar.style[this._progressPixelSize]){
				// animate the slider
				var _this = this;
				var props = {};
				var start = parseFloat(progressBar.style[this._progressPixelSize]);
				var duration = this.slideDuration * (percent-start/100);
				if(duration == 0){ return; }
				if(duration < 0){ duration = 0 - duration; }
				props[this._progressPixelSize] = { start: start, end: percent*100, units:"%" };
				this._inProgressAnim = fx.animateProperty({ node: progressBar, duration: duration,
					onAnimate: function(v){ remainingBar.style[_this._progressPixelSize] = (100-parseFloat(v[_this._progressPixelSize])) + "%"; },
					onEnd: function(){ delete _this._inProgressAnim; },
					properties: props
				})
				this._inProgressAnim.play();
			}
			else{
				progressBar.style[this._progressPixelSize] = (percent*100) + "%";
				remainingBar.style[this._progressPixelSize] = ((1-percent)*100) + "%";
			}
	        /* temporary */
	        this.currentValue = this.value;
	        this.sendValuesArray = child.sendValues;
	
	        if (this.currentValue != null) {
	            if (this.sendValuesArray[this.currentValue]) {
	                this.valueNode.value = this.value = this.sendValuesArray[this.currentValue];
	            } else {
	                this.valueNode.value = this.value = this.values[this.currentValue];
	            }
	        }
	        this.intermediateChanges = false;
	        this.discreteValues = this.values.length;
	
	        if (this.labelWidget) {
	            //dojo.attr(this.labelWidget.textbox, "value", this.labels[this.currentValue]);
	            this.labelWidget.textNode.innerHTML = this.labels[this.currentValue]; 
	        }
	        var labs = query(" > ", this.labWidg.domNode);
	        array.forEach(labs, function(lab, index) {
	            if (index == 0) {
	                domStyle.set(lab, "float", "left");
	                domStyle.set(query(" >", lab)[0], "left", "0");
	            } else if (index == (labs.length - 1)) {
	                domStyle.set(lab, "float", "right");
	                domStyle.set(lab, "position", "relative");
	                domStyle.set(lab, "left", "0");
	                domStyle.set(query(" >", lab)[0], "left", "0");
	            }
	        })
		},
	
		_typematicCallback: function(/*Number*/ count, /*Object*/ button, /*Event*/ e){
			if(count == -1){ return; }
			this[(button == (this._descending? this.incrementButton : this.decrementButton))? "decrement" : "increment"](e);
		},
	
		postCreate: function(){
	        if (this.options != "") {
	            var rule = new xcitec.form._SliderRule({container:"bottomDecoration", count: this.options});
	            domStyle.set(rule.domNode, "height", "10px");
	            //this.bottomDecoration.appendChild(rule.domNode);
	            domConstruct.place(rule.domNode, this.bottomDecoration, "first");
	        }
	        this.currentValue = this.value;
	        this.showButtons = false;
			if(this.showButtons){
				this.incrementButton.style.display="";
				this.decrementButton.style.display="";
				this._connects.push(typematic.addMouseListener(
					this.decrementButton, this, "_typematicCallback", 25, 500));
				this._connects.push(typematic.addMouseListener(
					this.incrementButton, this, "_typematicCallback", 25, 500));
			}
			this.connect(this.domNode, !has("mozilla") ? "onmousewheel" : "DOMMouseScroll", "_mouseWheeled");
	
			// define a custom constructor for a SliderMover that points back to me
	        var mover = declare(dijit.form._SliderMover, {
				widget: this
			});
			this._movable = new Moveable(this.sliderHandle, {mover: mover});
			//find any associated label element and add to slider focusnode.
			var label=query('label[for="'+this.id+'"]');
			if(label.length){
				label[0].id = (this.id+"_label");
				this.focusNode.setAttribute("aria-labelledby", label[0].id);
			}
			this.focusNode.setAttribute("aria-valuemin", this.minimum);
			this.focusNode.setAttribute("aria-valuemax", this.maximum);
	
	        on(this.sliderHandle, "mouseover", lang.hitch(this, function() {
	            var tempLabels = this.labels;
	            /*var index = -1;
	            var _this = this;
	            dojo.forEach(tempValues, function(value) {
	                *//*if (value == dojo.byId(_this.id.split(".")[0] + ".valueNode").value) {
	                    index = dojo.indexOf(tempValues, value);
	                }*//*
	                if (value == _this.currentValue) {
	                    index = dojo.indexOf(tempValues, value);
	                }
	            });*/
	            if (this.disabled || this.readOnly) {
	                return;
	            }
	            if ((tempLabels[this.currentValue] != null) && (tempLabels[this.currentValue] != "")) {
	                if (this.currentValue != null) {
                    Tooltip.show(tempLabels[this.currentValue], this.sliderHandle);
	/*
	                    if (this.labelWidget) {
	                        dojo.attr(this.labelWidget.textbox, "value", tempLabels[this.currentValue]);
	                    }
	*/
	                }
	            }
	        }));
	        on(this.sliderHandle, "mouseout", lang.hitch(this, function() {
              Tooltip.hide(this.sliderHandle);
	        }));
	
	        this.slideDuration = 0;
	
	        dom.setSelectable(this.domNode, false);
	
			this.inherited(arguments);
		},
	
		destroy: function(){
			this._movable.destroy();
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			this.inherited(arguments);
		}
	
	});
	
	declare("dijit.form._SliderMover",
		Mover,
	{
		onMouseMove: function(e){
			var widget = this.widget;
			var abspos = widget._abspos;
			if(!abspos){
				abspos = widget._abspos = domGeometry.position(widget.sliderBarContainer, true);
				widget._setPixelValue_ = lang.hitch(widget, "_setPixelValue");
				widget._isReversed_ = widget._isReversed();
			}
			var pixelValue = e[widget._mousePixelCoord] - abspos[widget._startingPixelCoord];
			widget._setPixelValue_(widget._isReversed_ ? (abspos[widget._pixelCount]-pixelValue) : pixelValue, abspos[widget._pixelCount], false);
		},
	
		destroy: function(e){
			domConstruct.destroy.apply(this, arguments);
			var widget = this.widget;
			widget._abspos = null;
			widget._setValueAttr(widget.currentValue, true);
		}
	});
	
	declare("xcitec.form._SliderRule", HorizontalRule, {
	
	});
	return Slider;
});

},
'xcitec/page/PageSeparator':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
  "../Title"
], function (declare, domClass, _Templated, _Widget, Title) {

	var PageSeparator = declare( [_Widget, _Templated], {
	    templateString: '<div>' +
	                        '<div class="shadow"></div>' +
	                        '<div class="title_wrap" data-dojo-attach-point="separator, containerNode"></div>' +
	                    '</div>',
	
	    caption: "360&deg; view",
	    entityClass: "rating",
	
	    postCreate: function() {
	        var separator = new Title( {text: this.caption, entityClass: this.entityClass} );
	        domClass.add(separator.domNode, 'icon360view');
	        this.separator.appendChild(separator.domNode);
	        this.inherited(arguments);
	    }
	});
	return PageSeparator;
});

},
'xcitec/Headline':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-construct",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"xcitec/Label",
  "./BoldLabel"
], function (declare, domConstruct, _Templated, _Widget, Label, BoldLabel) {

	var Headline = declare(  [_Widget, _Templated], {
	
	    templateString :'<div class="headline"/>',
	
	    label: "",
		labelWidget: null,
	
	    postCreate: function() {
	        this.labelWidget = new BoldLabel( {label: this.label} );
	        domConstruct.place(this.labelWidget.domNode, this.domNode, "first");
	        this.inherited(arguments);
	    },
	
		destroy: function(){
			if(this.labelWidget) {
				this.labelWidget.destroy();
			}
			this.inherited(arguments);
		}
	});
	
	
	
	return Headline;
});

},
'dijit/dijit':function(){
define([
	"./main",
	"./_base",
	"dojo/parser",
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./layout/_LayoutWidget",
	"./form/_FormWidget",
	"./form/_FormValueWidget"
], function(dijit){

	// module:
	//		dijit/dijit

	/*=====
	return {
		// summary:
		//		A roll-up for common dijit methods
		//		All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)
		//		And some other stuff that we tend to pull in all the time anyway
	};
	=====*/

	return dijit;
});

},
'xcitec/wicket/AjaxLazyPane':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	'dojo/dom-class',
	'dojo/query',
	"dijit/_Contained",
	"dijit/layout/ContentPane",
	"dijit/registry",
	"xcitec/wicket/_ajax",
  "../xcitecUtil",
  "../Box"
], function (declare, array, domClass, query, _Contained, ContentPane, registry, _ajax, xcitecUtil, Box) {

	var AjaxLazyPane = declare( [ContentPane, _Contained], {
	    baseClass: "xcitecWicketAjaxLazyPane",
	    preventCache: true,
	    preload: false,
	    //loadingMessage: "<img src='ListViewContent/indicator.gif'></img>",
	
	    _load: function(){
	        // summary:
	        //		Load/reload the href specified in this.href
	
	        // display loading message
	        //this._setContent(this.onDownloadStart(), true);
	        //alert("hello");
	
	        var self = this;
	        var successHandler = function(html){
	            try{
	                self._isDownloaded = true;
	                //self._setContent(html, false);
	                self.onDownloadEnd();
	            }catch(err){
	                self._onError('Content', err); // onContentError
	            }
	            delete self._xhrDfd;
	        };
	
	        var failureHandler = function(err){
	            // show error message in the pane
	//            self._onError('Download', err);
				self._setContent(err);
	            delete self._xhrDfd;
	            return err;
	        };
	
	        this._xhrDfd = xcitecUtil.dojoWicketAjaxGet(this.href, successHandler, failureHandler, null)
	
	        // Remove flag saying that a load is needed
	        delete this._hrefChanged;
	    },
	
	    cancel: function(){
	        // summary:
	        //		Cancels an in-flight download of content
	//        if(this._xhrDfd && (this._xhrDfd.fired == -1)){
	//            this._xhrDfd.cancel();
	//        }
	        delete this._xhrDfd; // garbage collect
	
	        this.onLoadDeferred = null;
	    },
	
	    onDownloadEnd: function() {
	        var widget = this.getEnclosingWidget(this.domNode);
	        if (widget instanceof Box) {
	            if (widget.formId != "") {
	                registry.byId(widget.formId).decreaseOpenBoxesCount();
	            }
	        }
	    },
	
	    getEnclosingWidget: function(/* DOMNode */ node){
	        // summary:
	        //		Returns the widget whose DOM tree contains the specified DOMNode, or null if
	    	//		the node is not contained within the DOM tree of any widget
	        node = node.parentNode;
	        while(node){
	            var id = node.getAttribute && node.getAttribute("widgetId");
	            if(id){
	                return registry.byId(id);
	            }
	            node = node.parentNode;
	        }
	        return null;
	    },

		_checkIfSingleChild: function(){
			// summary:
			//		@Override - dijit/layout/_ContentPaneResizeMixin
			// description:
			//		Test if we have exactly one visible widget as a child,
			//		and if so assume that we are a container for that widget,
			//		and should propagate startup() and resize() calls to it.
			//		Skips over things like data stores since they aren't visible.

			var candidateWidgets = [],
				otherVisibleNodes = false;

			query("> *", this.containerNode).some(function(node){
				var widget = registry.byNode(node);
				if(widget && widget.resize){
					candidateWidgets.push(widget);
				}else if(!/script|link|style/i.test(node.nodeName)){
					otherVisibleNodes = true;
				}
			});

			this._singleChild = candidateWidgets.length == 1 && !otherVisibleNodes ?
				candidateWidgets[0] : null;

			// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
			domClass.toggle(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
		},

		resize: function() {
			array.forEach(this.getChildren(), function(widget) {
				if('resize' in widget){
					widget.resize();
				}
			});
		}
	
	});
	
	return AjaxLazyPane;
});

},
'xcitec/form/_DropDownMenu':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/window",
	"dojo/keys",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"../_base/ImageFactory"
], function(declare, windowModule, array, domAttr, domClass, dWindow, keys, _Templated, _Widget, ImageFactory){

	return declare([_Widget, _Templated],
		{
			templateString: "<ul class='dijitReset dijitMenu' data-dojo-attach-event='onmousedown:_onMouseDown,onmouseup:_onMouseUp,onmouseover:_onMouseOver,onmouseout:_onMouseOut' tabIndex='-1'\";'></ul>",

			currentOptionNode: null,

			optionsCount: 0,

			getCurrentOptionNode: function(){
				if(!this.currentOptionNode){
					return this.domNode.firstChild;
				}
				return this.currentOptionNode;
			},

			getOptionsCount: function(){
				return this.optionsCount;
			},

			setOptionsCount: function(optsCount){
				this.optionsCount = optsCount;
			},

			createMenuItem: function(option){
				var menuitem = windowModule.doc.createElement("li");
				var text = windowModule.doc.createElement("span");
				if(option.label == ""){
					text.innerHTML = "&nbsp";
				}else{
					text.innerHTML = option.label;
				}
				if(option.iconClass){
					option.iconClass = ImageFactory.createIconCss(option.iconClass);
					var image = windowModule.doc.createElement("img");
					domAttr.set(image, "src", this._blankGif);
					domAttr.set(image, "class", "xcitecDropDownMenuIcon");
					domClass.add(image, option.iconClass);
					menuitem.appendChild(image);
				}
				menuitem.appendChild(text);
				if(option.selected == true){
					this.currentOptionNode = menuitem;
				}
				return menuitem;
			},

			createMenu: function(options){
				array.forEach(options, function(item, i){
					var menuitem = this.createMenuItem(item);
					menuitem.className = "dijitReset dijitMenuItem";
					domAttr.set(menuitem, "id", this.id + i);
					this.optionsCount += 1;
					domAttr.set(menuitem, "hiddenValue", item.hiddenValue);
					this.domNode.appendChild(menuitem);
				}, this);
			},

			_setText: function(value){
				this.onChange(value);
			},

			onChange: function(/*Object*/ value){
				// summary:
				//		Notifies ComboBox/FilteringSelect that user clicked an option in the drop down menu.
				//		Probably should be called onSelect.
				// tags:
				//		callback
			},

			_onMouseDown: function(/*Event*/ evt){
				evt.preventDefault(), evt.stopPropagation();
			},

			_onMouseUp: function(/*Event*/ evt){
				var tgt = evt.target;
				if(tgt === this.domNode){
					return;
				}
				if(tgt.parentNode.parentNode === this.domNode){
					this._unhoverOptionNode(tgt.parentNode);
					this._setText(tgt.parentNode);
				}else{
					this._unhoverOptionNode(tgt);
					this._setText(tgt);
				}
			},

			_onMouseOver: function(/*Event*/ evt){
				var tgt = evt.target;
				if(tgt === this.domNode){
					return;
				}
				if(tgt.parentNode.parentNode === this.domNode){
					this._hoverOptionNode(tgt.parentNode);
				}else{
					this._hoverOptionNode(tgt);
				}
			},

			_onMouseOut: function(/*Event*/ evt){
				var tgt = evt.target;
				if(tgt === this.domNode){
					if(this._highlighted_option){
						domClass.remove(this._highlighted_option, "dijitMenuItemSelected");
					}
					return;
				}
				if(tgt.parentNode.parentNode === this.domNode){
					this._unhoverOptionNode(tgt.parentNode);
				}else{
					this._unhoverOptionNode(tgt);
				}
			},

			_hoverOptionNode: function(/*DomNode*/ node){
				this._highlighted_option = node;
				this.currentNode = this._highlighted_option;
				domClass.add(node, "dijitMenuItemSelected");
			},

			_unhoverOptionNode: function(/*DomNode*/ node){
				this._highlighted_option = null;
				if(node){
					this.currentNode = node;
					domClass.remove(node, "dijitMenuItemSelected");
				}
			},

			highlightFirstOption: function(){
				//	summary:
				// 		Highlight the first real item in the list (not Previous Choices).
				this._hoverOptionNode(this.domNode.firstChild);
				dWindow.scrollIntoView(this._highlighted_option);
			},

			_highlightNextOption: function(){
				//	summary:
				// 		Highlight the item just below the current selection.
				// 		If nothing selected, highlight first option.

				// because each press of a button clears the menu,
				// the highlighted option sometimes becomes detached from the menu!
				// test to see if the option has a parent to see if this is the case.

				var ns = this._highlighted_option.nextSibling;
				if(!ns){
					return;
				}
				this._unhoverOptionNode(this._highlighted_option);
				this._hoverOptionNode(ns);

				// scrollIntoView is called outside of _focusOptionNode because in IE putting it inside causes the menu to scroll up on mouseover
				dWindow.scrollIntoView(this._highlighted_option);
			},

			_highlightPrevOption: function(){
				//	summary:
				// 		Highlight the item just above the current selection.
				// 		If nothing selected, highlight last option (if
				// 		you select Previous and try to keep scrolling up the list).

				var ps = this._highlighted_option.previousSibling;
				if(!ps){
					return;
				}
				this._unhoverOptionNode(this._highlighted_option);
				this._hoverOptionNode(ps);

				dWindow.scrollIntoView(this._highlighted_option);
			},

			highlightLastOption: function(){
				//	summary:
				// 		Highlight the last real item in the list (not More Choices).
				this._hoverOptionNode(this.domNode.lastChild);
				dWindow.scrollIntoView(this._highlighted_option);
			},

			getHighlightedOption: function(){
				//	summary:
				//		Returns the highlighted option.
				return this._highlighted_option;
			},

			handleKey: function(key){
				switch(key){
					case keys.DOWN_ARROW:
						if(!this._highlighted_option){
							this.highlightFirstOption();
						}else{
							this._highlightNextOption();
						}
						break;
					case keys.UP_ARROW:
						if(!this._highlighted_option){
							this.highlightLastOption();
						}else{
							this._highlightPrevOption();
						}
						break;
					case keys.ENTER:
						this._unhoverOptionNode(this._highlighted_option);
						break;
					case keys.ESCAPE:
						this._unhoverOptionNode(this.currentNode);
						break;
				}
			}
		}
	);
});

},
'dijit/MenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/sniff", // has("ie")
	"dojo/_base/lang", // lang.hitch
	"./_Widget",
	"./_TemplatedMixin",
	"./_Contained",
	"./_CssStateMixin",
	"dojo/text!./templates/MenuItem.html"
], function(declare, dom, domAttr, domClass, kernel, has, lang,
			_Widget, _TemplatedMixin, _Contained, _CssStateMixin, template){

	// module:
	//		dijit/MenuItem

	var MenuItem = declare("dijit.MenuItem" + (has("dojo-bidi") ? "_NoBidi" : ""),
		[_Widget, _TemplatedMixin, _Contained, _CssStateMixin], {
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: template,

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text as HTML
		label: "",
		_setLabelAttr: function(val){
			this._set("label", val);
			var shortcutKey = "";
			var text;
			var ndx = val.search(/{\S}/);
			if(ndx >= 0){
				shortcutKey = val.charAt(ndx + 1);
				var prefix = val.substr(0, ndx);
				var suffix = val.substr(ndx + 3);
				text = prefix + shortcutKey + suffix;
				val = prefix + '<span class="dijitMenuItemShortcutKey">' + shortcutKey + '</span>' + suffix;
			}else{
				text = val;
			}
			this.domNode.setAttribute("aria-label", text + " " + this.accelKey);
			this.containerNode.innerHTML = val;
			this._set('shortcutKey', shortcutKey);
		},

		/*=====
		// shortcutKey: [readonly] String
		//		Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,
		//		also known as [a mnemonic](http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29).
		//		This is denoted in the label by surrounding the single character with {}.
		//		For example, if label="{F}ile", then shortcutKey="F".
		shortcutKey: "",
		=====*/

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "dijitNoIcon",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to
		//		execute the menu item regardless of where the focus is on the page.
		//
		//		Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and
		//		execute those accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this._set('label', source.innerHTML);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			var label = this.id + "_text";
			domAttr.set(this.containerNode, "id", label); // only needed for backward compat
			if(this.accelKeyNode){
				domAttr.set(this.accelKeyNode, "id", this.id + "_accel"); // only needed for backward compat
			}
			dom.setSelectable(this.domNode, false);
		},

		onClick: function(/*Event*/){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(has("ie") == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				this.focusNode.focus();
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected

			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			domClass.toggle(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.

			this.focusNode.setAttribute('aria-disabled', value ? 'true' : 'false');
			this._set("disabled", value);
		},

		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.

			if(this.accelKeyNode){
				this.accelKeyNode.style.display = value ? "" : "none";
				this.accelKeyNode.innerHTML = value;
				//have to use colSpan to make it work in IE
				domAttr.set(this.containerNode, 'colSpan', value ? "1" : "2");
			}
			this._set("accelKey", value);
		}
	});

	if(has("dojo-bidi")){
		MenuItem = declare("dijit.MenuItem", MenuItem, {
			_setLabelAttr: function(val){
				this.inherited(arguments);
				if(this.textDir === "auto"){
					this.applyTextDir(this.textDirNode);
				}
			}
		});
	}

	return MenuItem;
});

},
'dijit/_AttachMixin':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect",	// remove for 2.0
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.getObject
	"dojo/mouse",
	"dojo/on",
	"dojo/touch",
	"./_WidgetBase"
], function(require, array, connect, declare, lang, mouse, on, touch, _WidgetBase){

	// module:
	//		dijit/_AttachMixin

	// Map from string name like "mouseenter" to synthetic event like mouse.enter
	var synthEvents = lang.delegate(touch, {
		"mouseenter": mouse.enter,
		"mouseleave": mouse.leave,
		"keypress": connect._keypress	// remove for 2.0
	});

	// To be lightweight, _AttachMixin doesn't require() dijit/a11yclick.
	// If the subclass has a template using "ondijitclick", it must load dijit/a11yclick itself.
	// In that case, the a11yclick variable below will get set to point to that synthetic event.
	var a11yclick;

	var _AttachMixin = declare("dijit._AttachMixin", null, {
		// summary:
		//		Mixin for widgets to attach to dom nodes and setup events via
		//		convenient data-dojo-attach-point and data-dojo-attach-event DOM attributes.
		//
		//		Superclass of _TemplatedMixin, and can also be used standalone when templates are pre-rendered on the
		//		server.
		//
		//		Does not [yet] handle widgets like ContentPane with this.containerNode set.   It should skip
		//		scanning for data-dojo-attach-point and data-dojo-attach-event inside this.containerNode, but it
		//		doesn't.

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with data-dojo-attach-point=... in the
		//		template, ex: ["containerNode", "labelNode"]
		_attachPoints: [],

		// _attachEvents: [private] Handle[]
		//		List of connections associated with data-dojo-attach-event=... in the
		//		template
		_attachEvents: [],

		// attachScope: [public] Object
		//		Object to which attach points and events will be scoped.  Defaults
		//		to 'this'.
		attachScope: undefined,

		// searchContainerNode: [protected] Boolean
		//		Search descendants of this.containerNode for data-dojo-attach-point and data-dojo-attach-event.
		//		Should generally be left false (the default value) both for performance and to avoid failures when
		//		this.containerNode holds other _AttachMixin instances with their own attach points and events.
 		searchContainerNode: false,
 =====*/

		constructor: function(/*===== params, srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

			this._attachPoints = [];
			this._attachEvents = [];
		},


		buildRendering: function(){
			// summary:
			//		Attach to DOM nodes marked with special attributes.
			// tags:
			//		protected

			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(this.domNode);

			this._beforeFillContent();		// hook for _WidgetsInTemplateMixin
		},

		_beforeFillContent: function(){
		},

		_attachTemplateNodes: function(rootNode){
			// summary:
			//		Iterate through the dom nodes and attach functions and nodes accordingly.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//
			//		- dojoAttachPoint/data-dojo-attach-point
			//		- dojoAttachEvent/data-dojo-attach-event
			// rootNode: DomNode
			//		The node to search for properties. All descendants will be searched.
			// tags:
			//		private

			// DFS to process all nodes except those inside of this.containerNode
			var node = rootNode;
			while(true){
				if(node.nodeType == 1 && (this._processTemplateNode(node, function(n,p){ return n.getAttribute(p); },
						this._attach) || this.searchContainerNode) && node.firstChild){
					node = node.firstChild;
				}else{
					if(node == rootNode){ return; }
					while(!node.nextSibling){
						node = node.parentNode;
						if(node == rootNode){ return; }
					}
					node = node.nextSibling;
				}
			}
		},

		_processTemplateNode: function(/*DOMNode|Widget*/ baseNode, getAttrFunc, attachFunc){
			// summary:
			//		Process data-dojo-attach-point and data-dojo-attach-event for given node or widget.
			//		Returns true if caller should process baseNode's children too.

			var ret = true;

			// Process data-dojo-attach-point
			var _attachScope = this.attachScope || this,
				attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
			if(attachPoint){
				var point, points = attachPoint.split(/\s*,\s*/);
				while((point = points.shift())){
					if(lang.isArray(_attachScope[point])){
						_attachScope[point].push(baseNode);
					}else{
						_attachScope[point] = baseNode;
					}
					ret = (point != "containerNode");
					this._attachPoints.push(point);
				}
			}

			// Process data-dojo-attach-event
			var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");
			if(attachEvent){
				// NOTE: we want to support attributes that have the form
				// "domEvent: nativeEvent; ..."
				var event, events = attachEvent.split(/\s*,\s*/);
				var trim = lang.trim;
				while((event = events.shift())){
					if(event){
						var thisFunc = null;
						if(event.indexOf(":") != -1){
							// oh, if only JS had tuple assignment
							var funcNameArr = event.split(":");
							event = trim(funcNameArr[0]);
							thisFunc = trim(funcNameArr[1]);
						}else{
							event = trim(event);
						}
						if(!thisFunc){
							thisFunc = event;
						}

						this._attachEvents.push(attachFunc(baseNode, event, lang.hitch(_attachScope, thisFunc)));
					}
				}
			}

			return ret;
		},

		_attach: function(node, type, func){
			// summary:
			//		Roughly corresponding to dojo/on, this is the default function for processing a
			//		data-dojo-attach-event.  Meant to attach to DOMNodes, not to widgets.
			// node: DOMNode
			//		The node to setup a listener on.
			// type: String
			//		Event name like "click".
			// getAttrFunc: Function
			//		Function to get the specified property for a given DomNode/Widget.
			// attachFunc: Function?
			//		Attaches an event handler from the specified node/widget to specified function.

			// Map special type names like "mouseenter" to synthetic events.
			// Subclasses are responsible to require() dijit/a11yclick if they want to use it.
			type = type.replace(/^on/, "").toLowerCase();
			if(type == "dijitclick"){
				type = a11yclick || (a11yclick = require("./a11yclick"));
			}else{
				type = synthEvents[type] || type;
			}

			return on(node, type, func);
		},

		_detachTemplateNodes: function() {
			// summary:
			//		Detach and clean up the attachments made in _attachtempalteNodes.

			// Delete all attach points to prevent IE6 memory leaks.
			var _attachScope = this.attachScope || this;
			array.forEach(this._attachPoints, function(point){
				delete _attachScope[point];
			});
			this._attachPoints = [];

			// And same for event handlers
			array.forEach(this._attachEvents, function(handle){ handle.remove(); });
			this._attachEvents = [];
		},

		destroyRendering: function(){
			this._detachTemplateNodes();
			this.inherited(arguments);
		}
	});

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.).
	// Remove for 2.0.   Also, hide from API doc parser.
	lang.extend(_WidgetBase, /*===== {} || =====*/ {
		dojoAttachEvent: "",
		dojoAttachPoint: ""
	});
	
	return _AttachMixin;
});

},
'xcitec/SuiteNav':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"xcitec/Label"
], function (dojo, array, declare, domAttr, domClass, domConstruct, query, _Templated, _Widget, Label) {

	var SuiteNav = declare(  [_Widget, _Templated], {
	    
	    templateString :'<div class="navbar"><div class="navbar-separator"></div></div>',
	
	    postCreate: function() {
	        var linkList = query("a", this.srcNodeRef);
	        var ul = domConstruct.create("ul");
	        this.domNode.appendChild(ul);
	        array.forEach(linkList, function(item){
		        var className = "navbar-link-wrapper";
		        if(domAttr.get(item, "select")) {
			        className = "navbar-link-wrapper-selected";
		        }
	            var li = domConstruct.create("li", { className: className});
	            ul.appendChild(li);
	            li.appendChild(item);
	      domClass.add(item, "navbar-link")
	        });
	    }
	
	});
	
	
	
	return SuiteNav;
});

},
'dojo/ready':function(){
define(["./_base/kernel", "./has", "require", "./domReady", "./_base/lang"], function(dojo, has, require, domReady, lang){
	// module:
	//		dojo/ready
	// note:
	//		This module should be unnecessary in dojo 2.0

	var
		// truthy if DOMContentLoaded or better (e.g., window.onload fired) has been achieved
		isDomReady = 0,

		// The queue of functions waiting to execute as soon as dojo.ready conditions satisfied
		loadQ = [],

		// prevent recursion in onLoad
		onLoadRecursiveGuard = 0,

		handleDomReady = function(){
			isDomReady = 1;
			dojo._postLoad = dojo.config.afterOnLoad = true;
			onEvent();
		},

		onEvent = function(){
			// Called when some state changes:
			//		- dom ready
			//		- dojo/domReady has finished processing everything in its queue
			//		- task added to loadQ
			//		- require() has finished loading all currently requested modules
			//
			// Run the functions queued with dojo.ready if appropriate.


			//guard against recursions into this function
			if(onLoadRecursiveGuard){
				return;
			}
			onLoadRecursiveGuard = 1;

			// Run tasks in queue if require() is finished loading modules, the dom is ready, and there are no
			// pending tasks registered via domReady().
			// The last step is necessary so that a user defined dojo.ready() callback is delayed until after the
			// domReady() calls inside of dojo.	  Failure can be seen on dijit/tests/robot/Dialog_ally.html on IE8
			// because the dijit/focus.js domReady() callback doesn't execute until after the test starts running.
			while(isDomReady && (!domReady || domReady._Q.length == 0) && (require.idle ? require.idle() : true) && loadQ.length){
				var f = loadQ.shift();
				try{
					f();
				}catch(e){
					// force the dojo.js on("error") handler do display the message
					e.info = e.message;
					if(require.signal){
						require.signal("error", e);
					}else{
						throw e;
					}
				}
			}

			onLoadRecursiveGuard = 0;
		};

	// Check if we should run the next queue operation whenever require() finishes loading modules or domReady
	// finishes processing it's queue.
	require.on && require.on("idle", onEvent);
	if(domReady){
		domReady._onQEmpty = onEvent;
	}

	var ready = dojo.ready = dojo.addOnLoad = function(priority, context, callback){
		// summary:
		//		Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.
		//		In most cases, the `domReady` plug-in should suffice and this method should not be needed.
		//
		//		When called in a non-browser environment, just checks that all requested modules have arrived and been
		//		evaluated.
		// priority: Integer?
		//		The order in which to exec this callback relative to other callbacks, defaults to 1000
		// context: Object?|Function
		//		The context in which to run execute callback, or a callback if not using context
		// callback: Function?
		//		The function to execute.
		//
		// example:
		//	Simple DOM and Modules ready syntax
		//	|	require(["dojo/ready"], function(ready){
		//	|		ready(function(){ alert("Dom ready!"); });
		//	|	});
		//
		// example:
		//	Using a priority
		//	|	require(["dojo/ready"], function(ready){
		//	|		ready(2, function(){ alert("low priority ready!"); })
		//	|	});
		//
		// example:
		//	Using context
		//	|	require(["dojo/ready"], function(ready){
		//	|		ready(foo, function(){
		//	|			// in here, this == foo
		//	|		});
		//	|	});
		//
		// example:
		//	Using dojo/hitch style args:
		//	|	require(["dojo/ready"], function(ready){
		//	|		var foo = { dojoReady: function(){ console.warn(this, "dojo dom and modules ready."); } };
		//	|		ready(foo, "dojoReady");
		//	|	});

		var hitchArgs = lang._toArray(arguments);
		if(typeof priority != "number"){
			callback = context;
			context = priority;
			priority = 1000;
		}else{
			hitchArgs.shift();
		}
		callback = callback ?
			lang.hitch.apply(dojo, hitchArgs) :
			function(){
				context();
			};
		callback.priority = priority;
		for(var i = 0; i < loadQ.length && priority >= loadQ[i].priority; i++){}
		loadQ.splice(i, 0, callback);
		onEvent();
	};

	 1 || has.add("dojo-config-addOnLoad", 1);
	if( 1 ){
		var dca = dojo.config.addOnLoad;
		if(dca){
			ready[(lang.isArray(dca) ? "apply" : "call")](dojo, dca);
		}
	}

	if( 0  && dojo.config.parseOnLoad && !dojo.isAsync){
		ready(99, function(){
			if(!dojo.parser){
				dojo.deprecated("Add explicit require(['dojo/parser']);", "", "2.0");
				require(["dojo/parser"]);
			}
		});
	}

	if(domReady){
		domReady(handleDomReady);
	}else{
		handleDomReady();
	}

	return ready;
});

},
'xcitec/ToolbarButton':function(){
define([
         "dojo/_base/declare",
         "dojo/_base/window",
         "dijit/form/Button",
         "./_base/ImageFactory"
       ], function (declare, windowModule, Button, ImageFactory) {
  return declare( Button,	{
    baseClass: "dijitButton",

    caption: "",

    url: "",

    postMixInProperties: function() {
      if (this.iconClass) {
        this.iconClass = ImageFactory.createIconCss(this.iconClass);
      }
      this.inherited(arguments);
    },

    _setLabelAttr: function(/*String*/ content){
      // summary:
      //		Hook for attr('label', ...) to work.
      // description:
      //		Set the label (text) of the button; takes an HTML string.
      this.label = content;

      this.containerNode.innerHTML = "";
      this.containerNode.appendChild(windowModule.doc.createTextNode(this.label || this.caption));
    },


    _setCaptionAttr: function (/*String*/ content) {
      //method added during migration to Dojo 1.9
      // summary:
      //		Hook for attr('caption', ...) to work.
      // description:
      //		Set the caption (text) of the button; takes an HTML string.
      this._setLabelAttr(content);
    },

    postCreate: function(){
      if(this.url) {
        this.onClick = function(evt){
          window.open(this.url);
        };
      }
    }
  });
});

},
'dijit/DropDownMenu':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/keys", // keys
	"dojo/text!./templates/Menu.html",
	"./_OnDijitClickMixin",
	"./_MenuBase"
], function(declare, keys, template, _OnDijitClickMixin, _MenuBase){

	// module:
	//		dijit/DropDownMenu

	return declare("dijit.DropDownMenu", [_MenuBase, _OnDijitClickMixin], {
		// summary:
		//		A menu, without features for context menu (Meaning, drop down menu)

		templateString: template,

		baseClass: "dijitMenu",

		// Arrow key navigation
		_onUpArrow: function(){
			this.focusPrev();
		},
		_onDownArrow: function(){
			this.focusNext();
		},
		_onRightArrow: function(/*Event*/ evt){
			this._moveToPopup(evt);
			evt.stopPropagation();
			evt.preventDefault();
		},
		_onLeftArrow: function(/*Event*/ evt){
			if(this.parentMenu){
				if(this.parentMenu._isMenuBar){
					this.parentMenu.focusPrev();
				}else{
					this.onCancel(false);
				}
			}else{
				evt.stopPropagation();
				evt.preventDefault();
			}
		}
	});
});

},
'dojo/mouse':function(){
define(["./_base/kernel", "./on", "./has", "./dom", "./_base/window"], function(dojo, on, has, dom, win){

	// module:
	//		dojo/mouse

    has.add("dom-quirks", win.doc && win.doc.compatMode == "BackCompat");
	has.add("events-mouseenter", win.doc && "onmouseenter" in win.doc.createElement("div"));
	has.add("events-mousewheel", win.doc && 'onmousewheel' in win.doc);

	var mouseButtons;
	if((has("dom-quirks") && has("ie")) || !has("dom-addeventlistener")){
		mouseButtons = {
			LEFT:   1,
			MIDDLE: 4,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button & button; },
			isLeft:   function(e){ return e.button & 1; },
			isMiddle: function(e){ return e.button & 4; },
			isRight:  function(e){ return e.button & 2; }
		};
	}else{
		mouseButtons = {
			LEFT:   0,
			MIDDLE: 1,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button == button; },
			isLeft:   function(e){ return e.button == 0; },
			isMiddle: function(e){ return e.button == 1; },
			isRight:  function(e){ return e.button == 2; }
		};
	}
	dojo.mouseButtons = mouseButtons;

/*=====
	dojo.mouseButtons = {
		// LEFT: Number
		//		Numeric value of the left mouse button for the platform.
		LEFT:   0,
		// MIDDLE: Number
		//		Numeric value of the middle mouse button for the platform.
		MIDDLE: 1,
		// RIGHT: Number
		//		Numeric value of the right mouse button for the platform.
		RIGHT:  2,

		isButton: function(e, button){
			// summary:
			//		Checks an event object for a pressed button
			// e: Event
			//		Event object to examine
			// button: Number
			//		The button value (example: dojo.mouseButton.LEFT)
			return e.button == button; // Boolean
		},
		isLeft: function(e){
			// summary:
			//		Checks an event object for the pressed left button
			// e: Event
			//		Event object to examine
			return e.button == 0; // Boolean
		},
		isMiddle: function(e){
			// summary:
			//		Checks an event object for the pressed middle button
			// e: Event
			//		Event object to examine
			return e.button == 1; // Boolean
		},
		isRight: function(e){
			// summary:
			//		Checks an event object for the pressed right button
			// e: Event
			//		Event object to examine
			return e.button == 2; // Boolean
		}
	};
=====*/

	function eventHandler(type, selectHandler){
		// emulation of mouseenter/leave with mouseover/out using descendant checking
		var handler = function(node, listener){
			return on(node, type, function(evt){
				if(selectHandler){
					return selectHandler(evt, listener);
				}
				if(!dom.isDescendant(evt.relatedTarget, node)){
					return listener.call(this, evt);
				}
			});
		};
		handler.bubble = function(select){
			return eventHandler(type, function(evt, listener){
				// using a selector, use the select function to determine if the mouse moved inside the selector and was previously outside the selector
				var target = select(evt.target);
				var relatedTarget = evt.relatedTarget;
				if(target && (target != (relatedTarget && relatedTarget.nodeType == 1 && select(relatedTarget)))){
					return listener.call(target, evt);
				} 
			});
		};
		return handler;
	}
	var wheel;
	if(has("events-mousewheel")){
		wheel = 'mousewheel';
	}else{ //firefox
		wheel = function(node, listener){
			return on(node, 'DOMMouseScroll', function(evt){
				evt.wheelDelta = -evt.detail;
				listener.call(this, evt);
			});
		};
	}
	return {
		// summary:
		//		This module provide mouse event handling utility functions and exports
		//		mouseenter and mouseleave event emulation.
		// example:
		//		To use these events, you register a mouseenter like this:
		//		|	define(["dojo/on", dojo/mouse"], function(on, mouse){
		//		|		on(targetNode, mouse.enter, function(event){
		//		|			dojo.addClass(targetNode, "highlighted");
		//		|		});
		//		|		on(targetNode, mouse.leave, function(event){
		//		|			dojo.removeClass(targetNode, "highlighted");
		//		|		});

		_eventHandler: eventHandler,		// for dojo/touch

		// enter: Synthetic Event
		//		This is an extension event for the mouseenter that IE provides, emulating the
		//		behavior on other browsers.
		enter: eventHandler("mouseover"),

		// leave: Synthetic Event
		//		This is an extension event for the mouseleave that IE provides, emulating the
		//		behavior on other browsers.
		leave: eventHandler("mouseout"),

		// wheel: Normalized Mouse Wheel Event
		//		This is an extension event for the mousewheel that non-Mozilla browsers provide,
		//		emulating the behavior on Mozilla based browsers.
		wheel: wheel,

		isLeft: mouseButtons.isLeft,
		/*=====
		isLeft: function(){
			// summary:
			//		Test an event object (from a mousedown event) to see if the left button was pressed.
		},
		=====*/

		isMiddle: mouseButtons.isMiddle,
		/*=====
		 isMiddle: function(){
			 // summary:
			 //		Test an event object (from a mousedown event) to see if the middle button was pressed.
		 },
		 =====*/

		isRight: mouseButtons.isRight
		/*=====
		 , isRight: function(){
			 // summary:
			 //		Test an event object (from a mousedown event) to see if the right button was pressed.
		 }
		 =====*/
	};
});

},
'xcitec/form/CheckBoxGroup':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/on",
	"dojo/query",
  	"dojo/dom",
	'dojo/has',
	"dijit/form/_FormValueWidget",
	"dijit/registry",
	"xcitec/form/MultiSelectUtils",
  "../xcitecUtil",
  "./CheckBox"
], function (dojo, array, declare, lang, domAttr, on, query, dom, has, _FormValueWidget, registry, MultiSelectUtils, xcitecUtil, CheckBox) {

	var CheckBoxGroup = declare( [_FormValueWidget, MultiSelectUtils], {
		baseClass: 'checkBoxGroup',
	  tabIndex: "-1",
	  required:false,
	  templateString: '<div class="dijit dijitReset dijitInline" tabIndex="-1" data-dojo-attach-point="checkBoxButton, focusNode">' +
	                  '<input class="dijitReset" data-dojo-attach-point="textbox"' +
	                  '${!nameAttrSetting} type="hidden" autocomplete="off">' +
	                  '<div class="required"></div>' +
	                  '</div>',
	
	  // Array of selected Checkboxes
	  //selWidgets:null,
	
	  postMixInProperties: function() {
	    this.inherited(arguments);
	    if (this.required) {
	        this.baseClass = this.baseClass + " dijitXcitecChekboxGroupRequired";
	    }
	    this.selWidgets = [];
	  },

	  postCreate: function() {
	    this._parseSource();
	    this._createCheckBoxes(this.options);
	    this._updateHiddenInput();
      	dom.setSelectable(this.domNode, false);
	    this.inherited(arguments);
	  },

	  _onChange: function() {
	    if (this.readOnly || this.disabled) {
	      return;
	    }
	    // user defined function
	    this.onchange();
	  },

	  onchange: function() {
	    // for wicket use in template
	  },

	  _parseSource: function() {
	    // summary:
	    //		Loads our options and sets up our dropdown correctly.  We
	    //		don't want any content, so we don't call any inherit chain
	    //		function.
	    var opts = this.options;
	    if (!opts) {
			var nodeType = has('ie') == 8 ? this.srcNodeRef.nodeName : this.srcNodeRef.localName;
		  var component = (nodeType.toLowerCase()=="div")?"div":"option";
	      opts = this.options = this.srcNodeRef ? query("> "+component,
	                                                         this.srcNodeRef).map(function(node) {
	        return {
	          value: node.getAttribute("value"),
	          caption: node.getAttribute("caption"),
	          captionId: node.getAttribute("captionId"),
	          label: String(node.innerHTML),
	          checked: xcitecUtil.str2bool(domAttr.get(node, "checked")),
	          disabled: xcitecUtil.str2bool(domAttr.get(node, "disabled"))
	        };
	      }, this) : [];
	    }
	  },

	  selWidget: function(widget) {
	    if (this.isNodeInList(this.selWidgets, widget)) {
	      this.selWidgets = this.delFromList(this.selWidgets, widget);
	    } else {
	      this.selWidgets = this.addToList(this.selWidgets, widget);
	    }
	    this._updateHiddenInput();
	  },

	  _createCheckBoxes: function(otpions) {
	    var self = this;

	    array.forEach(otpions, function(node) {

	      // create CheckBoxes according set of options
	      var button = new CheckBox({
	        checked: node.checked,
	        disabled: self.disabled || node.disabled,
	        readOnly: self.readOnly,
	        caption: node.caption,
			innerText: node.label,
	        captionId: node.captionId,
	        value: node.value,
	        style: "width:100%;",
	        parent: self
	      });
	      self.checkBoxButton.appendChild(button.domNode);
	      button.domNode.onchange = "";
	      on(button.domNode, "click", lang.hitch(button, self.changeState));
	    });
	  },

	  changeState: function() {
	    var widget = registry.byNode(this.domNode);
	    widget.focus();
	    widget.parent._onChange();
	  },

	  _updateHiddenInput: function() {
	    var _this = this;
	    var temp = "";
	    var counter = 0;
	    array.forEach(_this.selWidgets, function(widget) {
	      if (counter == 0) {
	        temp = widget.value;
	      } else {
	        temp = temp + "," + widget.value;
	      }
	      counter++;
	    });
	    this.textbox.value = temp;
	  },

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
		}
	});
	return CheckBoxGroup;
});

},
'slmui/form/Slider':function(){
define([
	"../../dojo/_base/declare", // declare
	"idx/form/HorizontalSlider",
	"dijit/form/HorizontalRule",
	"dijit/form/HorizontalRuleLabels",
	"dijit/registry",
	"dojo/dom-construct",
	"dojo/_base/array",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-style",
	"dojo/query",
	'dojo/has'
], function(declare, HorizontalSlider, HorizontalRule, HorizontalRuleLabels, registry, domConstruct, array, windowModule, domAttr, domStyle, query, has){

	// module:
	//		slmui/form/Slider


	var Slider = declare(HorizontalSlider, {


		// values of each item (Array)
		values: null,
		// short text of each item (Array)
		labels: null,
		// long text of each item (Array)
		descriptions: null,
		// index of currently selected item
		selectedValue: 0,
		// array of item's widgets
		items: null,
		// widget name
		name: "",
		// state attributes
		readOnly: false,
		disabled: false,
		error: "",
		required: false,
		// if items exist in srcNodeRef
		empty: false,


		//to show no increment\decrement buttons
		showButtons: false,

		_initSliderFields: function(){
			// summary:
			//			Sets the input data to inherited idx.form.HorizontalSlider

			if(!this.empty){
				this.discreteValues = this.values.length;
				if(this.selectedValue){
					this.value = this.selectedValue;
				}
				this.minimum = 1;
				this.maximum = this.values.length;
			}else{
				this.discreteValues = 1;
			}
		},

		postMixInProperties: function(){
			// summary:
			//			Init of the slider
			this.inherited(arguments);

			if(!this.values){
				this._parseSource();
			}else{
				this.empty = false;
			}

			this._initSliderFields();

            if(!this.empty){
                this.baseClass = this.baseClass + " slmuiSlider";
            } else {
                this.baseClass = this.baseClass + " slmuiSliderEmpty";
            }
		},

		buildRendering: function(){
			//empty the input HTML needed for init
			domConstruct.empty(this.srcNodeRef);

			this.inherited(arguments);
		},

		_parseSource: function(){
			// summary:
			//			Parses the HTML source if widget was not created programmatically.
			var srcNodeName = (has('ie') == 8) ? this.srcNodeRef.nodeName.toLowerCase() : this.srcNodeRef.localName;
			var component = (srcNodeName=="div")?"div":"option";
			var boxes = query("> " + component, this.srcNodeRef);
			if(boxes.length == 0){
				this.empty = true;
				return;
			}
			var _this = this;
			this.values = boxes.map(function(node){
				return domAttr.get(node, "value");
			});
			this.labels = boxes.map(function(node){
				return domAttr.get(node, "label");
			});
			this.descriptions = boxes.map(function(node){
				return node.innerHTML;
			});
			boxes.forEach(function(node, index){
				if(domAttr.get(node, "selected") == "true"){
					_this.selectedValue = index + 1;
				}
			})
		},

		_createTickMarks: function(){
			//summary:
			//			Creates and instantiate top and bottom tick marks for Slider.
			var topTickMarksNode = domConstruct.create("div", {}, this.containerNode, "first");
			var sliderRules = new _MyHorizontalRuleWithToolTips({
				container: "topDecoration",
				tips: this.labels,
				count: this.values.length,
				style: "height: 5px;"
			}, topTickMarksNode);


			var bottomTickMarksNode = domConstruct.create("div", {}, this.containerNode, "first");
			var sliderRules2 = new _MyHorizontalRuleWithToolTips({
				container: "bottomDecoration",
				count: this.values.length,
				tips: this.labels
			}, bottomTickMarksNode);
		},

		_creteLabels: function(){
			// summary:
			//			Creates and instantiates the labels for Slider.

			//we need only the first and the last label
			var labels = [this.labels[0], this.labels[this.labels.length - 1]];

			var labelsNode = domConstruct.create("div", {}, this.containerNode, "last");
			var sliderLabels = new _MyHorizontalRuleLabelsWithAdditionalClassesForExtremeItems({
				container: "bottomDecoration",
				labels: labels,
				style: "margin-bottom: 15px;"
			}, labelsNode);
		},

		postCreate: function(){
			// summary:
			//			Creates tick marks and label for Slide and removes the label node.
			// tags:
			//			extension
			this.inherited(arguments);

			if(!this.empty){
				this._createTickMarks();
				this._creteLabels();
			}
			//We need to remove label node of the Slider.
			//Currently an xcitec.layout.Chapter is responsible for label node
			domConstruct.destroy(this.labelWrap);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//			Sets the tick mark for the slider handle and for the value node.
			// tags:
			//			extension
			this.inherited(arguments);
			if(!this.empty){
				domAttr.set(this.sliderHandle, "title", this.labels[value - 1]);
				this.valueNode.value = this.values[value - 1];
			}
			//when slider have no predefined selected value and user just click (or set focus) on slider without moving
		            if(value==0){
		                this.valueNode.value = this.values[0];
		            }
		},

		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Event handler for mousewheel where supported
			if(this.focused){
				this.inherited(arguments);
			}
		}
	});

	var _MyHorizontalRuleLabelsWithAdditionalClassesForExtremeItems = declare(HorizontalRuleLabels, {

		// summary:
		//			Extended horizontal rule label. See method desc.

		_genHTML: function(pos, ndx){
			// summary:
			//			Adds the addtional styles and CSS classes for extreme left and right items.
			// tags:
			//			extension
			var label = this.labels[ndx];

			var positionPrefix = this._positionPrefix;
			var labelPrefix = this._labelPrefix;

			var additionalLabelClass;
			var additionalPositionClass;
			if(pos == 0){
				additionalLabelClass = "ruleLabelLeft";
				additionalPositionClass = "ruleLabelPositionLeft";
			}else if(pos == 100){
				additionalLabelClass = "ruleLabelRight";
				additionalPositionClass = "ruleLabelPositionRight";
			}
			if(additionalLabelClass && additionalPositionClass){
				positionPrefix = '<div class="dijitRuleLabelContainer dijitRuleLabelContainerH ' + additionalPositionClass + '" style="left:';
				labelPrefix = '"><div class="dijitRuleLabel dijitRuleLabelH ' + additionalLabelClass + '">';
			}
			return positionPrefix + this._calcPosition(pos) + this._positionSuffix + this.labelStyle +
				this._genDirectionHTML(label) +
				labelPrefix + label + this._suffix;
		}
	});

	var _MyHorizontalRuleWithToolTips = declare(HorizontalRule, {

		// summary:
		//			Extended horizontal rules. See the method desc.

		_positionPrefix: '<div class="dijitRuleMark dijitRuleMarkH" style="margin-top: 5px; left:',

		// array of tool tips for tick marks
		tips: [],

		_genHTML: function(pos, index){
			// summary:
			//			Puts the tick marks into the container that will show tool tips.
			// tags:
			//			extension
			var pref = '';
			var post = '';
			var tooltip = this.tips[(index)];
			var margin = (100 / (this.tips.length - 1) - 2).toFixed(1);
			if(pos != 100){
				pref = '<div data-dojo-attach-point="tickMarkNode" style="height: 15px;width: 4%;z-index: 123;	margin-right: ' + margin + '%;margin-left: -2%;	float: left;" title="' +
					tooltip
					+ '">';
				post = '</div>';
			}else{
				pref = '<div data-dojo-attach-point="tickMarkNode" style="height: 15px;width: 4%;z-index: 123;	margin-right: 0;margin-left: -4%;	float: left;" title="' +
					tooltip
					+ '">';
				post = '</div>';
			}
			return pref + this._positionPrefix + pos + this._positionSuffix + this.ruleStyle + this._suffix + post;
		}
	});


	return Slider;
})
;
},
'xcitec/form/AnswerBox':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/query",
  	"dojo/dom",
	'dojo/has',
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/_StateWidget"
], function (dojo, array, declare, windowModule, domAttr, query, dom, has, _Templated, _Widget, registry, _StateWidget) {

	var AnswerBox = declare( [_Widget, _Templated], {
		templateString: '<div ${nameAttrSetting} class="answerBoxWrapper" tabindex="0">' +
				'<input type="hidden" value="" data-dojo-attach-point="valueNode"/>' +
				'<table class="answerBox">' +
				'<tbody data-dojo-attach-point="itemsNode"></tbody>' +
				'</table>' +
				'</div>',
	
		// values of each item (Array)
		values: null,
		// short text of each item (Array)
		labels: null,
		// long text of each item (Array)
		descriptions: null,
		// tooltips of each item
		tooltips: null,
		// index of currently selected item
		selectedValue: 0,
		// array of item's widgets
		items: null,
		// widget name
		name: "",
		// state attributes
		readOnly: false,
		disabled: false,
		error: "",
		required: false,
		// if items exist in srcNodeRef
		empty: false,

		onchange: function(){
			// onchange function for setting by user
		},

		_parseSource: function(){
			var srcNodeName = (has('ie') == 8) ? this.srcNodeRef.nodeName.toLowerCase() : this.srcNodeRef.localName;
			var component = (srcNodeName=="div")?"div":"option";
			var boxes = query(">" + component, this.srcNodeRef);
			if(boxes.length == 0){
				this.empty = true;
			}
			var _this = this;
			this.values = boxes.map(function(node){
				return domAttr.get(node, "value");
			});
			this.labels = boxes.map(function(node){
				return domAttr.get(node, "label");
			});
			this.descriptions = boxes.map(function(node){
				return node.innerHTML;
			});

			this.tooltips = boxes.map(function(node){
				return (domAttr.get(node, "showTooltip") != "false") ? true : false;
			});

			boxes.forEach(function(node, index){
				if(domAttr.get(node, "selected") == "true"){
					_this.selectedValue = index + 1;
				}
			})
		},

		postMixInProperties: function(){
			this.values = [];
			this.labels = [];
			this.descriptions = [];
			this.tooltips = [];
			this.items = [];
			this.nameAttrSetting = this.name ? ("name='" + this.name + "'") : "";
			this.inherited(arguments);
		},

		postCreate: function(){
			this._parseSource();
			var _this = this;

			this.items = [];

			if(this.empty){
				this.createAnswerItem("", "", "", this, false, true, false, false);
				domAttr.set(_this.valueNode, "value", "");
			} else{
				// create actual items
				if(this.readOnly){
					// if readOnly - create only selected item, or just empty box if nothing selected
					if(this.selectedValue != 0){
						this.createAnswerItem(this.labels[this.selectedValue - 1], this.descriptions[this.selectedValue - 1],
								this.values[this.selectedValue - 1], this, false, true, false, this.tooltips[this.selectedValue - 1]);
						if(!_this.values[_this.selectedValue - 1]){
							domAttr.set(_this.valueNode, "value", "");
						} else{
							domAttr.set(_this.valueNode, "value", _this.values[_this.selectedValue - 1]);
						}
					} else{
						this.createAnswerItem("", "", "", this, false, true, false, false);
						domAttr.set(_this.valueNode, "value", "");
					}
					// if disabled - create only selected item, or just empty box if nothing selected
				} else{
					array.forEach(this.descriptions, function(node, index){
						if(_this.selectedValue == (index + 1)){
							_this.createAnswerItem(_this.labels[index], _this.descriptions[index], _this.values[index],
								 _this, true, false, _this.disabled, _this.tooltips[index]);
							if(!_this.values[index]){
								domAttr.set(_this.valueNode, "value", "");
							} else{
								domAttr.set(_this.valueNode, "value", _this.values[index]);
							}
						} else{
							_this.createAnswerItem(_this.labels[index], _this.descriptions[index], _this.values[index],
								_this, false, false, _this.disabled, _this.tooltips[index]);
						}
					});
				}
			}
			domAttr.set(this.valueNode, "name", this.name);
			dom.setSelectable(this.domNode, false);
			this.inherited(arguments);
		},

		createAnswerItem: function(label, description, value, parent, selected, readOnly, disabled, showTooltip){
			if(value == null){
				value = "";
			}
			var boxItem = new _AnswerBoxItem({ label: label, description: description,
				value: value, parent: parent, selected: selected,
				readOnly: readOnly, disabled: disabled, showTooltip: showTooltip});
			this.items.push(boxItem);
			this.itemsNode.appendChild(boxItem.domNode);
		},

		// makes all items unselected except active
		setSelectedAttr: function(selWidget){
			if(selWidget.selected){
				domAttr.set(this.valueNode, "value", selWidget.value);
			} else{
				domAttr.set(this.valueNode, "value", "");
			}
			array.forEach(this.items, function(widget){
				if(selWidget != widget){
					widget.selected = false;
				}
				widget.setSelected();
			});
		},

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
			this.inherited(arguments);
		}
	});

	var _AnswerBoxItem = declare( _StateWidget, {
		templateString: '<tr class="${baseClass}" data-dojo-attach-event="onclick: _onClick">' +
				'<td class="rate"><div class="section" data-dojo-attach-point="labelNode"></div></td>' +
				'<td class="description"><div class="section">${description}<span class="description_line"></span></div></td>' +
				'</tr>',

		baseClass: "answerBox",

		// reference to parent container
		parent: null,

		// short text
		label: "",
		// long text
		description: "",

		// value for sending to server
		value: "",

		// state attributes
		selected: false,
		readOnly: false,
		disabled: false,

		showTooltip: true,

		buildRendering: function(){
			this.inherited(arguments);

			if(this.showTooltip){
				domAttr.set(this.domNode, 'title', this.description);
			}
		},

		_onClick: function(){
            this.parent.domNode.focus();
			if(this.readOnly || this.disabled){
				return;
			}
			this.selected = !this.selected;
			if(this.parent){
				this.parent.setSelectedAttr(this);
				this.parent.onchange();
			}
		},
	
		// makes item visually selected by applying appropriate styles
		setSelected : function(){
			this._setStateClass();
		},
	
		_setLabelAttr: function(/*String*/ label){
			this.label = label;
			if(this.label){
				this.labelNode.appendChild(windowModule.doc.createTextNode(this.label));
			}
		}
	});
	return AnswerBox;
});

},
'xcitec/page/ContentPanel':function(){
define([
	"dojo/_base/declare",
	"./BoxPanelV2"
], function (declare, BoxPanelV2) {

	return declare(BoxPanelV2, {

	});
});
},
'dijit/_base/typematic':function(){
define(["../typematic"], function(){

	/*=====
	return {
		// summary:
		//		Deprecated, for back-compat, just loads top level module
	};
	=====*/

});

},
'xcitec/form/TaskButton':function(){
define([
	"dojo/_base/declare",
	"dojo/text!./templates/TaskButton.html",
	"./Button",
  "../_base/ImageFactory"
], function (declare, template, Button, ImageFactory) {

	//todo 2.0 merge with xcitec.form.Button
	var TaskButton = declare( Button, {
	    templateString: template,
	
	    postMixInProperties: function() {
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        }
	        this.inherited(arguments);
	    }
	});
	
	return TaskButton;
});

},
'xcitec/form/InlineLinkButton':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"./LinkButton",
  "../_base/ImageFactory"
], function (declare, domClass, LinkButton, ImageFactory) {

	//todo 2.0 merge with LinkButton and deprecate InlineLinkButton
	var InlineLinkButton = declare( LinkButton, {
		iconClass: "",
		templateString: "<div>" +
			"<img src='${_blankGif}' width='1' height='1' alt='' data-dojo-attach-point='icon' />" +
			"<span data-dojo-attach-event='onclick:_onLinkClick, onmouseover:_onMouse, onmouseout:_onMouse, onmousedown:_onMouse, onmouseup:_onMouse' data-dojo-attach-point='containerNode'></span></div>",
	
		postMixInProperties: function(){
			if(this.iconClass){
				this.iconClass = ImageFactory.createIconCss(this.iconClass);
			}
			this.inherited(arguments);
		},
	
		postCreate: function(){
			this.inherited(arguments);
			if(this.iconClass != ""){
				domClass.add(this.icon, this.iconClass);
			}
		}
	});
	return InlineLinkButton;
});

},
'dojo/store/Memory':function(){
define(["../_base/declare", "./util/QueryResults", "./util/SimpleQueryEngine" /*=====, "./api/Store" =====*/],
function(declare, QueryResults, SimpleQueryEngine /*=====, Store =====*/){

// module:
//		dojo/store/Memory

// No base class, but for purposes of documentation, the base class is dojo/store/api/Store
var base = null;
/*===== base = Store; =====*/

return declare("dojo.store.Memory", base, {
	// summary:
	//		This is a basic in-memory object store. It implements dojo/store/api/Store.
	constructor: function(options){
		// summary:
		//		Creates a memory object store.
		// options: dojo/store/Memory
		//		This provides any configuration information that will be mixed into the store.
		//		This should generally include the data property to provide the starting set of data.
		for(var i in options){
			this[i] = options[i];
		}
		this.setData(this.data || []);
	},
	// data: Array
	//		The array of all the objects in the memory store
	data:null,

	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",

	// index: Object
	//		An index of data indices into the data array by id
	index:null,

	// queryEngine: Function
	//		Defines the query engine to use for querying the data store
	queryEngine: SimpleQueryEngine,
	get: function(id){
		// summary:
		//		Retrieves an object by its identity
		// id: Number
		//		The identity to use to lookup the object
		// returns: Object
		//		The object in the store that matches the given id.
		return this.data[this.index[id]];
	},
	getIdentity: function(object){
		// summary:
		//		Returns an object's identity
		// object: Object
		//		The object to get the identity from
		// returns: Number
		return object[this.idProperty];
	},
	put: function(object, options){
		// summary:
		//		Stores an object
		// object: Object
		//		The object to store.
		// options: dojo/store/api/Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: Number
		var data = this.data,
			index = this.index,
			idProperty = this.idProperty;
		var id = object[idProperty] = (options && "id" in options) ? options.id : idProperty in object ? object[idProperty] : Math.random();
		if(id in index){
			// object exists
			if(options && options.overwrite === false){
				throw new Error("Object already exists");
			}
			// replace the entry in data
			data[index[id]] = object;
		}else{
			// add the new object
			index[id] = data.push(object) - 1;
		}
		return id;
	},
	add: function(object, options){
		// summary:
		//		Creates an object, throws an error if the object already exists
		// object: Object
		//		The object to store.
		// options: dojo/store/api/Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: Number
		(options = options || {}).overwrite = false;
		// call put with overwrite being false
		return this.put(object, options);
	},
	remove: function(id){
		// summary:
		//		Deletes an object by its identity
		// id: Number
		//		The identity to use to delete the object
		// returns: Boolean
		//		Returns true if an object was removed, falsy (undefined) if no object matched the id
		var index = this.index;
		var data = this.data;
		if(id in index){
			data.splice(index[id], 1);
			// now we have to reindex
			this.setData(data);
			return true;
		}
	},
	query: function(query, options){
		// summary:
		//		Queries the store for objects.
		// query: Object
		//		The query to use for retrieving objects from the store.
		// options: dojo/store/api/Store.QueryOptions?
		//		The optional arguments to apply to the resultset.
		// returns: dojo/store/api/Store.QueryResults
		//		The results of the query, extended with iterative methods.
		//
		// example:
		//		Given the following store:
		//
		// 	|	var store = new Memory({
		// 	|		data: [
		// 	|			{id: 1, name: "one", prime: false },
		//	|			{id: 2, name: "two", even: true, prime: true},
		//	|			{id: 3, name: "three", prime: true},
		//	|			{id: 4, name: "four", even: true, prime: false},
		//	|			{id: 5, name: "five", prime: true}
		//	|		]
		//	|	});
		//
		//	...find all items where "prime" is true:
		//
		//	|	var results = store.query({ prime: true });
		//
		//	...or find all items where "even" is true:
		//
		//	|	var results = store.query({ even: true });
		return QueryResults(this.queryEngine(query, options)(this.data));
	},
	setData: function(data){
		// summary:
		//		Sets the given data as the source for this store, and indexes it
		// data: Object[]
		//		An array of objects to use as the source of data.
		if(data.items){
			// just for convenience with the data format IFRS expects
			this.idProperty = data.identifier || this.idProperty;
			data = this.data = data.items;
		}else{
			this.data = data;
		}
		this.index = {};
		for(var i = 0, l = data.length; i < l; i++){
			this.index[data[i][this.idProperty]] = i;
		}
	}
});

});

},
'dojo/hccss':function(){
define([
	"require",			// require, require.toUrl
	"./_base/config", // config.blankGif
	"./dom-class", // domClass.add
	"./dom-style", // domStyle.getComputedStyle
	"./has",
	"./domReady",
	"./_base/window" // win.body
], function(require, config, domClass, domStyle, has, domReady, win){

	// module:
	//		dojo/hccss

	/*=====
	return function(){
		// summary:
		//		Test if computer is in high contrast mode (i.e. if browser is not displaying background images).
		//		Defines `has("highcontrast")` and sets `dj_a11y` CSS class on `<body>` if machine is in high contrast mode.
		//		Returns `has()` method;
	};
	=====*/

	// Has() test for when background images aren't displayed.  Don't call has("highcontrast") before dojo/domReady!.
	has.add("highcontrast", function(){
		// note: if multiple documents, doesn't matter which one we use
		var div = win.doc.createElement("div");
		div.style.cssText = "border: 1px solid; border-color:red green; position: absolute; height: 5px; top: -999px;" +
			"background-image: url(" + (config.blankGif || require.toUrl("./resources/blank.gif")) + ");";
		win.body().appendChild(div);

		var cs = domStyle.getComputedStyle(div),
			bkImg = cs.backgroundImage,
			hc = (cs.borderTopColor == cs.borderRightColor) ||
				(bkImg && (bkImg == "none" || bkImg == "url(invalid-url:)" ));

		if(has("ie") <= 8){
			div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
		}else{
			win.body().removeChild(div);
		}

		return hc;
	});

	domReady(function(){
		if(has("highcontrast")){
			domClass.add(win.body(), "dj_a11y");
		}
	});

	return has;
});

},
'xcitec/layout/XLayout':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"dijit/registry"
], function (dojo, array, declare, domAttr, domConstruct, query, _Templated, _Widget, registry) {

	/* XLayout is used for collecting controls and blocks of HTML generated by Wicket and marked with
	 * 'xcitecRole' attribute and moving them to appropriate places in currently active template
	 */
	var XLayout = declare( [_Widget, _Templated], {
	
	    /* array of nodes with 'xcitecRole' attribute */
	    xcitecRoles: null,
	
	
	    postMixInProperties: function() {
		    this.xcitecRoles = [];
	        this.findXcitecRoles(this.srcNodeRef);
	
	        this.inherited(arguments);
	    },
	
	
	    /* function for searching appropriate nodes for moving them in the template */
	    findXcitecRoles: function(src) {
	        var _this = this;
	        var currentNodeList = query(" >", src);
	
	        array.forEach(currentNodeList, function(node) {
	            if (domAttr.has(node, "xcitecRole")) {
	                _this.xcitecRoles = _this.xcitecRoles.concat(node);
	            } else if (domAttr.has(node, "data-dojo-type")) {
	                 // do nothing, skip this node
	            } else {
	                _this.findXcitecRoles(node);
	            }
	        });
	    },
	
		_fillContent: function(/*DomNode*/ source){
	        var _this = this;
	
	        // attaching nodes to appropriate dojoAttachPoints
	        array.forEach(_this.xcitecRoles, function(node) {
	            array.forEach(_this._attachPoints, function(point) {
	                if (domAttr.get(node, "xcitecRole") == point) {
	                    try {
	                        domConstruct.place(node, _this[point], "replace");
	                    } catch(e) {}
	                }
	            })
	        });
			// to keep compatible behavior with prev version
			//@todo review XLayout and it's usage
			this.inherited(arguments);
		},
	
		destroyDescendants: function(/*Boolean?*/ preserveDom){
			// get all direct descendants and destroy them recursively
			array.forEach(registry.findWidgets(this.domNode), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
		}
	});
	return XLayout;
});

},
'xcitec/ToolbarDropDownButton':function(){
define([
         "dojo/_base/declare",
         "dojo/_base/window",
	'dojo/parser',
	'dijit/popup',
	'dijit/registry',
         "dijit/_Container",
         "dijit/form/DropDownButton",
         "./_base/ImageFactory"
       ], function (declare, windowModule, parser, popup, registry, _Container, DropDownButton, ImageFactory) {
  return declare(DropDownButton,	{
	stopParser: true,
    postMixInProperties: function() {
      if (this.iconClass) {
        this.iconClass = ImageFactory.createIconCss(this.iconClass);
      }
      this.inherited(arguments);
    },

	_fillContent: function(){
		if(this.srcNodeRef){
			var node = this.srcNodeRef.firstChild;
			while(node.nodeType != 1){
				node = node.nextSibling;
			}
			DropDownButton.superclass._fillContent.call(this, node);

			this.dropDownContainer = this.srcNodeRef;
			this.dropDownContainer.style.display = 'none';
		}
	},

	startup: function(){
		DropDownButton.superclass.startup.apply(this, arguments);
	},

	loadAndOpenDropDown: function(){
		if(!this.dropDown && this.dropDownContainer){
			parser.parse(this.dropDownContainer);
			this.dropDown = registry.findWidgets(this.dropDownContainer)[0];
			delete this.dropDownContainer;
		}
		if(this.dropDown){
			popup.hide(this.dropDown);
		}
		this.inherited(arguments);
	},

    _setLabelAttr: function(/*String*/ content){
      // summary:
      //		Hook for attr('label', ...) to work.
      // description:
      //		Set the label (text) of the button; takes an HTML string.
      this.label = content;

      this.containerNode.innerHTML = "";
      this.containerNode.appendChild(windowModule.doc.createTextNode(this.label));
    }
  });
});

},
'dijit/typematic':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect", // connect._keyPress
	"dojo/_base/lang", // lang.mixin, lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie")
	"./main"        // setting dijit.typematic global
], function(array, connect, lang, on, has, dijit){

	// module:
	//		dijit/typematic

	var typematic = (dijit.typematic = {
		// summary:
		//		These functions are used to repetitively call a user specified callback
		//		method when a specific key or mouse click over a specific DOM node is
		//		held down for a specific amount of time.
		//		Only 1 such event is allowed to occur on the browser page at 1 time.

		_fireEventAndReload: function(){
			this._timer = null;
			this._callback(++this._count, this._node, this._evt);

			// Schedule next event, timer is at most minDelay (default 10ms) to avoid
			// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
			this._currentTimeout = Math.max(
				this._currentTimeout < 0 ? this._initialDelay :
					(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
				this._minDelay);
			this._timer = setTimeout(lang.hitch(this, "_fireEventAndReload"), this._currentTimeout);
		},

		trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number?*/ subsequentDelay, /*Number?*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start a timed, repeating callback sequence.
			//		If already started, the function call is ignored.
			//		This method is not normally called by the user but can be
			//		when the normal listener code is insufficient.
			// evt:
			//		key or mouse event object to pass to the user callback
			// _this:
			//		pointer to the user's widget space.
			// node:
			//		the DOM node object to pass the the callback function
			// callback:
			//		function to call until the sequence is stopped called with 3 parameters:
			// count:
			//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
			// node:
			//		the DOM node object passed in
			// evt:
			//		key or mouse event object
			// obj:
			//		user space object used to uniquely identify each typematic sequence
			// subsequentDelay:
			//		if > 1, the number of milliseconds until the 3->n events occur
			//		or else the fractional time multiplier for the next event's delay, default=0.9
			// initialDelay:
			//		the number of milliseconds until the 2nd event occurs, default=500ms
			// minDelay:
			//		the maximum delay in milliseconds for event to fire, default=10ms
			if(obj != this._obj){
				this.stop();
				this._initialDelay = initialDelay || 500;
				this._subsequentDelay = subsequentDelay || 0.90;
				this._minDelay = minDelay || 10;
				this._obj = obj;
				this._node = node;
				this._currentTimeout = -1;
				this._count = -1;
				this._callback = lang.hitch(_this, callback);
				this._evt = { faux: true };
				for(var attr in evt){
					if(attr != "layerX" && attr != "layerY"){ // prevent WebKit warnings
						var v = evt[attr];
						if(typeof v != "function" && typeof v != "undefined"){
							this._evt[attr] = v
						}
					}
				}
				this._fireEventAndReload();
			}
		},

		stop: function(){
			// summary:
			//		Stop an ongoing timed, repeating callback sequence.
			if(this._timer){
				clearTimeout(this._timer);
				this._timer = null;
			}
			if(this._obj){
				this._callback(-1, this._node, this._evt);
				this._obj = null;
			}
		},

		addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start listening for a specific typematic key.
			//		See also the trigger method for other parameters.
			// keyObject:
			//		an object defining the key to listen for:
			//
			//		- keyCode: the keyCode (number) to listen for, used for non-printable keys
			//		- charCode: the charCode (number) to listen for, used for printable keys
			//		- charOrCode: deprecated, use keyCode or charCode
			//		- ctrlKey: desired ctrl key state to initiate the callback sequence:
			//			- pressed (true)
			//			- released (false)
			//			- either (unspecified)
			//		- altKey: same as ctrlKey but for the alt key
			//		- shiftKey: same as ctrlKey but for the shift key
			// returns:
			//		a connection handle

			// Setup keydown or keypress listener depending on whether keyCode or charCode was specified.
			// If charOrCode is specified use deprecated connect._keypress synthetic event (remove for 2.0)
			var type = "keyCode" in keyObject ? "keydown" : "charCode" in keyObject ? "keypress" : connect._keypress,
				attr = "keyCode" in keyObject ? "keyCode" : "charCode" in keyObject ? "charCode" : "charOrCode";

			var handles = [
				on(node, type, lang.hitch(this, function(evt){
					if(evt[attr] == keyObject[attr] &&
						(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
						(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
						(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
						(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
						evt.stopPropagation();
						evt.preventDefault();
						typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
					}else if(typematic._obj == keyObject){
						typematic.stop();
					}
				})),
				on(node, "keyup", lang.hitch(this, function(){
					if(typematic._obj == keyObject){
						typematic.stop();
					}
				}))
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		},

		addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start listening for a typematic mouse click.
			//		See the trigger method for other parameters.
			// returns:
			//		a connection handle
			var handles = [
				on(node, "mousedown", lang.hitch(this, function(evt){
					evt.preventDefault();
					typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
				})),
				on(node, "mouseup", lang.hitch(this, function(evt){
					if(this._obj){
						evt.preventDefault();
					}
					typematic.stop();
				})),
				on(node, "mouseout", lang.hitch(this, function(evt){
					if(this._obj){
						evt.preventDefault();
					}
					typematic.stop();
				})),
				on(node, "dblclick", lang.hitch(this, function(evt){
					evt.preventDefault();
					if(has("ie") < 9){
						typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
						setTimeout(lang.hitch(this, typematic.stop), 50);
					}
				}))
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		},

		addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start listening for a specific typematic key and mouseclick.
			//		This is a thin wrapper to addKeyListener and addMouseListener.
			//		See the addMouseListener and addKeyListener methods for other parameters.
			// mouseNode:
			//		the DOM node object to listen on for mouse events.
			// keyNode:
			//		the DOM node object to listen on for key events.
			// returns:
			//		a connection handle
			var handles = [
				this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay),
				this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay)
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		}
	});

	return typematic;
});

},
'xcitec/page/LoginFrame':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dojo/text!xcitec/page/templates/LoginFrame.html",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase",
	"xcitec/layout/XLayout"
], function (dojo, array, declare, domAttr, domConstruct, query, template, _Templated, _Widget, XLayout) {

	var LoginFrame = declare( XLayout, {
		helpdesk: true,
		contact: true,
	
		templateString: template,
	
		postMixInProperties: function(){
			this.footer = query("[xcitecAttachPoint$=footer]")[0];
			this.inherited(arguments);
		},
	
		buildRendering: function(){
			this.inherited(arguments);
			if(this.helpdesk){
				this.helpWidget = new xcitec.page._LoginFramePanel({
					title: "Helpdesk",
					baseClass: "helpArea boxGrey",
					url: require.toUrl("external/help.html").toString()
				});
				this.loginContent.appendChild(this.helpWidget.domNode);
			}
			if(this.contact){
				this.contactWidget = new xcitec.page._LoginFramePanel({
					title: "Contact",
					baseClass: "contactArea boxGrey",
					url: require.toUrl("external/contact.html").toString()
				});
				this.loginContent.appendChild(this.contactWidget.domNode);
			}
		},
	
		postCreate: function(){
			var _this = this;
			// place footer in appropriate place and change its' class
			if(this.footer){
				domConstruct.place(this.footer, this.domNode);
				//dojo.place(this.footer, this.domNode, "after");
				domAttr.set(this.footer, "class", "loginFooter");
				domAttr.set(query(" >", this.footer)[0], "class", "loginFooterWrap");
			}
			// processing of message pane
			// @todo 1.3 modified to work without this functionality - remove in 1.3
			array.forEach(_this.xcitecRoles, function(node){
				switch(domAttr.get(node, "xcitecRole").toLowerCase()){
					case "messagepane":
						var message = query("[xcitecRole='message']", node)[0];
						// if exists - move it to innerHTML of main div
						if(message){
							var clazz = domAttr.get(message, "class");
							node.innerHTML = message.innerHTML;
							domAttr.set(node, "class", clazz);
						}
						break;
					default:
						break;
				}
			});
			this.inherited(arguments);
		}
	});
	
	declare("xcitec.page._LoginFramePanel", [_Widget, _Templated], {
		url: "",
		title: "",
	
		templateString:
				'<div><div class="boxWrap boxRound">' +
						'<iframe data-dojo-attach-point="contentFrame" class="boxExternal" frameborder="no" allowtransparency="true" title="${title}"></iframe>' +
						'</div><div class="corner cornerTL"></div><div class="corner cornerBL">' +
						'</div><div class="corner cornerTR"></div><div class="corner cornerBR"></div>' +
						'</div>',
	
		postCreate: function(){
			domAttr.set(this.contentFrame, "src", this.url);
			this.inherited(arguments);
		}
	});
	return LoginFrame;
});

},
'xcitec/_EnterKeySubmit':function(){
define([
	"dojo",
	"dojo/NodeList-traverse",
	"dojo/_base/declare",
	"dojo/has",
	"dojo/topic",
  "dojo/keys",
  "dijit/registry",
  "./form/TextBox",
  "./form/DropDownSelect"
], function (dojo, NodeListTraverse, declare, has, topic, keys, registry, TextBox, DropDownSelect) {

	/*
	    interface which helps to handle onEnter events from TextBox-based controls
	 */
	var _EnterKeySubmit = declare( null, {
	
	    // connect events
	    _initEnterPressHandling: function() {
	        if (has("moz") || has("opera")) {
	            this.connect(this.domNode, "onkeypress", this._onEnter);
	        } else {
	            this.connect(this.domNode, "onkeydown", this._onEnter);
	        }
	    },
	    
	    _onEnter: function(e) {
	        switch (e.keyCode) {
	            case keys.ENTER:
	                if (registry.byId(e.target.id) instanceof TextBox ||
	                    registry.byId(e.target.id) instanceof DropDownSelect) {
	                    this.submitButton = registry.byId("ok." + this.id);
	                    if (this.submitButton) {
	                        this.submitButton.onClick();
	                        e.preventDefault(), e.stopPropagation();
	                    }
	                }
	                return;
	            default:
	                break;
	        }
	    }
	});
	return _EnterKeySubmit;
});

},
'dojo/dnd/move':function(){
define([
	"../_base/declare",
	"../dom-geometry", "../dom-style",
	"./common", "./Mover", "./Moveable"
], function(declare, domGeom, domStyle, dnd, Mover, Moveable){

// module:
//		dojo/dnd/move

/*=====
var __constrainedMoveableArgs = declare([Moveable.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

var constrainedMoveable = declare("dojo.dnd.move.constrainedMoveable", Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/*Mover*/ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = domGeom.getMarginSize(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/*Mover*/ mover, /*Object*/ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		this.onMoving(mover, leftTop);
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	}
});

/*=====
var __boxConstrainedMoveableArgs = declare([__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

var boxConstrainedMoveable = declare("dojo.dnd.move.boxConstrainedMoveable", constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
var __parentConstrainedMoveableArgs = declare( [__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

var parentConstrainedMoveable = declare("dojo.dnd.move.parentConstrainedMoveable", constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode,
				s = domStyle.getComputedStyle(n),
				mb = domGeom.getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = domGeom.getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = domGeom.getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = domGeom.getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});


return {
	// summary:
	//		TODOC
	constrainedMoveable: constrainedMoveable,
	boxConstrainedMoveable: boxConstrainedMoveable,
	parentConstrainedMoveable: parentConstrainedMoveable
};

});

},
'xcitec/form/CheckBox':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/window",
	'dojo/html',
	"dojo/text!xcitec/form/templates/CheckBox.html",
  "dijit/_WidgetsInTemplateMixin",
	"dijit/form/CheckBox",
	"xcitec/Label",
  "xcitec/LabelMultiline",
	"./_FormWidget"
], function (dojo, declare, windowModule, html, template, _WidgetsInTemplateMixin, CheckBox, Label, LabelMultiline, _FormWidget) {

	var CheckBox = declare( [CheckBox, _FormWidget, _WidgetsInTemplateMixin], {
		labelTypeAttrSetting: "xcitec/LabelMultiline",
		baseClass: "dijitXcitecCheckBox",
	
		caption: "",
		innerText: "",
		captionId: "",
		parent: null,
		required:false,
        widgetsInTemplate: false,
	
		templateString: template,
	
		postMixInProperties: function(){
			this.inherited(arguments);
			if(this.fitToContainer){
				this.labelTypeAttrSetting = "xcitec/Label";
			}
			if(this.caption || this.innerText){
				this.widgetsInTemplate = true;
			}
			if(this.required){
				this.baseClass = this.baseClass + " dijitXcitecChekboxRequired";
			}
		},

		_fillContent: function(){
			if(this.caption){
				html.set(this.labelNode.domNode, windowModule.doc.createTextNode(this.caption));
			} else if (this.innerText) {
				this.labelNode.domNode.innerHTML = this.innerText;
			}
		},
	
		postCreate: function(){
			if(this.parent && this.checked){
				this.parent.selWidget(this);
			}
			this.inherited(arguments);
		},
	
		_setLabelAttr: function(/*String*/ content){
			// overriding of Button.js _setLabelAttr method
		},
	
		_onClick: function(e){
			if(this.parent){
				this.parent.selWidget(this);
			}
			this.inherited(arguments);
		}
	});
	return CheckBox;
});

},
'xcitec/form/Textarea':function(){
define([
	"dojo/_base/declare",
	"dijit/form/SimpleTextarea"
], function (declare, SimpleTextarea) {

	/*
	 known bug in IE: if there is logn word - scrolls out of container.
	 */
	
	var Textarea = declare( SimpleTextarea,	{
	  baseClass: "dijitXcitecTextArea",
	  // template is inlined - because it inlined in Dijit
	  //templateString: null,
	  templateString:
	        '<div class="${baseClass}"><table class="dijitReset" cellspacing="0" cellpadding="0" ' +
	        '       style="width:100%; table-layout:fixed;" ' +
	        '       id="widget_${id}" ' +
	        '  ><tr class="dijitReset"><td class="dijitReset dijitXcitecTextAreaBox" ' +
	        '    ><textarea class="dijitXcitecTextAreaInput" ' +
	        '             ${!nameAttrSetting}  autocomplete="off" ' +
	        '             data-dojo-attach-point="focusNode,containerNode,textbox" ' +
	        '    ></textarea ' +
	        '  ></td></tr ' +
	        '></table><img src="${_blankGif}" alt="" class="required"></div>',
	  rows: "",
	  cols: "",
	  // error: String
	  //		Error message if control vaue in invalid state.
	  error: "",
	
	  required: false,
	
	  postMixInProperties: function() {
	    // workaround if value in parameters don't use node value
	    if(!this.value && this.srcNodeRef && ("value" in this.params)){
	      this.srcNodeRef.value = this.value;
	    }
	    if (this.readOnly || this.disabled) {
	        this.required = false;
	    }
	    this.inherited(arguments);
	    if (this.error) {
	      this.state = "Error";
	    }
	    if (this.required) {
	      this.baseClass = this.baseClass + " dijitXcitecRequired";
	    }
	  },
	
	  postCreate: function(){
	    this.inherited(arguments);
	  }
	});
	
	
	/*
	previous version of Textarea - not looks good in Dojo 1.3
	 */
	/*
	dojo.declare("xcitec.form.TextareaV1",
			dijit.form.SimpleTextarea,
			{
	      baseClass: "dijitXcitecTextArea",
	      _templateString:
	            '<div id="${id}" class="dijitInline" ' +
	            '     data-dojo-attach-point="styleNode"' +
	            '     data-dojo-attach-event="onmouseover:_onMouse,onmouseout:_onMouse,onmousedown:_onMouse" ' +
	            '     waiRole="presentation" ' +
	            '  ><div class="dijitXcitecTextAreaDiv" tabindex="-1" ' +
	            '    ><div class="dijitReset dijitTextArea" ' +
	            '          style="overflow:visible; width:auto; display: inline-block; outline: none;" ' +
	            '      ><div data-dojo-attach-point="editNode,focusNode,eventNode" ' +
	            '         data-dojo-attach-event="onpaste:_changing,oncut:_changing" ' +
	            '         waiRole="textbox" waiState="multiline-true" ' +
	            '         contentEditable="true" ' +
	            '         class="dijitXcitecTextAreaEditDiv" ' +
	            '         style="overflow:visible; width:auto; display: inline-block; outline: none;" ' +
	            '      ></div' +
	            '      ><textarea name="${name}" value="${value}" ' +
	            '          data-dojo-attach-point="formValueNode" style="display:none;" autocomplete="off"' +
	            '      ></textarea' +
	            '    ></div' +
	            '  ></div' +
	            '></div>',
	
	      templateString:
	            '<div id="${id}" class="dijitInline" ' +
	            '     data-dojo-attach-point="styleNode"' +
	            '     data-dojo-attach-event="onmouseover:_onMouse,onmouseout:_onMouse,onmousedown:_onMouse" ' +
	            '     waiRole="presentation" ' +
	            '  ><div class="dijitXcitecTextAreaDiv" tabindex="-1" ' +
	            '    ><div class="dijitReset dijitTextArea" ' +
	            '          style="overflow:visible; width:auto; display: inline-block; outline: none;" ' +
	            '      ><textarea ${nameAttrSetting} autocomplete="off" value="${value}" ' +
	            '         data-dojo-attach-point="focusNode,containerNode,textbox" ' +
	            '         class="dijitXcitecTextAreaEditDiv" ' +
	            '         style="overflow:visible; width:100%; display: inline-block; outline: none;" ' +
	            '      ></textarea' +
	            '    ></div' +
	            '  ></div' +
	            '></div>',
	
	      postCreate: function(){
	        this.inherited(arguments);
	        this.domNode.style.overflow = '';
	      }
	    }
	);
	*/
	
	return Textarea;
});
},
'dojo/_base/connect':function(){
define(["./kernel", "../on", "../topic", "../aspect", "./event", "../mouse", "./sniff", "./lang", "../keys"], function(dojo, on, hub, aspect, eventModule, mouse, has, lang){
// module:
//		dojo/_base/connect

has.add("events-keypress-typed", function(){ // keypresses should only occur a printable character is hit
	var testKeyEvent = {charCode: 0};
	try{
		testKeyEvent = document.createEvent("KeyboardEvent");
		(testKeyEvent.initKeyboardEvent || testKeyEvent.initKeyEvent).call(testKeyEvent, "keypress", true, true, null, false, false, false, false, 9, 3);
	}catch(e){}
	return testKeyEvent.charCode == 0 && !has("opera");
});

function connect_(obj, event, context, method, dontFix){
	method = lang.hitch(context, method);
	if(!obj || !(obj.addEventListener || obj.attachEvent)){
		// it is a not a DOM node and we are using the dojo.connect style of treating a
		// method like an event, must go right to aspect
		return aspect.after(obj || dojo.global, event, method, true);
	}
	if(typeof event == "string" && event.substring(0, 2) == "on"){
		event = event.substring(2);
	}
	if(!obj){
		obj = dojo.global;
	}
	if(!dontFix){
		switch(event){
			// dojo.connect has special handling for these event types
			case "keypress":
				event = keypress;
				break;
			case "mouseenter":
				event = mouse.enter;
				break;
			case "mouseleave":
				event = mouse.leave;
				break;
		}
	}
	return on(obj, event, method, dontFix);
}

var _punctMap = {
	106:42,
	111:47,
	186:59,
	187:43,
	188:44,
	189:45,
	190:46,
	191:47,
	192:96,
	219:91,
	220:92,
	221:93,
	222:39,
	229:113
};
var evtCopyKey = has("mac") ? "metaKey" : "ctrlKey";


var _synthesizeEvent = function(evt, props){
	var faux = lang.mixin({}, evt, props);
	setKeyChar(faux);
	// FIXME: would prefer to use lang.hitch: lang.hitch(evt, evt.preventDefault);
	// but it throws an error when preventDefault is invoked on Safari
	// does Event.preventDefault not support "apply" on Safari?
	faux.preventDefault = function(){ evt.preventDefault(); };
	faux.stopPropagation = function(){ evt.stopPropagation(); };
	return faux;
};
function setKeyChar(evt){
	evt.keyChar = evt.charCode ? String.fromCharCode(evt.charCode) : '';
	evt.charOrCode = evt.keyChar || evt.keyCode;
}
var keypress;
if(has("events-keypress-typed")){
	// this emulates Firefox's keypress behavior where every keydown can correspond to a keypress
	var _trySetKeyCode = function(e, code){
		try{
			// squelch errors when keyCode is read-only
			// (e.g. if keyCode is ctrl or shift)
			return (e.keyCode = code);
		}catch(e){
			return 0;
		}
	};
	keypress = function(object, listener){
		var keydownSignal = on(object, "keydown", function(evt){
			// munge key/charCode
			var k=evt.keyCode;
			// These are Windows Virtual Key Codes
			// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/UserInput/VirtualKeyCodes.asp
			var unprintable = (k!=13) && k!=32 && (k!=27||!has("ie")) && (k<48||k>90) && (k<96||k>111) && (k<186||k>192) && (k<219||k>222) && k!=229;
			// synthesize keypress for most unprintables and CTRL-keys
			if(unprintable||evt.ctrlKey){
				var c = unprintable ? 0 : k;
				if(evt.ctrlKey){
					if(k==3 || k==13){
						return listener.call(evt.currentTarget, evt); // IE will post CTRL-BREAK, CTRL-ENTER as keypress natively
					}else if(c>95 && c<106){
						c -= 48; // map CTRL-[numpad 0-9] to ASCII
					}else if((!evt.shiftKey)&&(c>=65&&c<=90)){
						c += 32; // map CTRL-[A-Z] to lowercase
					}else{
						c = _punctMap[c] || c; // map other problematic CTRL combinations to ASCII
					}
				}
				// simulate a keypress event
				var faux = _synthesizeEvent(evt, {type: 'keypress', faux: true, charCode: c});
				listener.call(evt.currentTarget, faux);
				if(has("ie")){
					_trySetKeyCode(evt, faux.keyCode);
				}
			}
		});
		var keypressSignal = on(object, "keypress", function(evt){
			var c = evt.charCode;
			c = c>=32 ? c : 0;
			evt = _synthesizeEvent(evt, {charCode: c, faux: true});
			return listener.call(this, evt);
		});
		return {
			remove: function(){
				keydownSignal.remove();
				keypressSignal.remove();
			}
		};
	};
}else{
	if(has("opera")){
		keypress = function(object, listener){
			return on(object, "keypress", function(evt){
				var c = evt.which;
				if(c==3){
					c=99; // Mozilla maps CTRL-BREAK to CTRL-c
				}
				// can't trap some keys at all, like INSERT and DELETE
				// there is no differentiating info between DELETE and ".", or INSERT and "-"
				c = c<32 && !evt.shiftKey ? 0 : c;
				if(evt.ctrlKey && !evt.shiftKey && c>=65 && c<=90){
					// lowercase CTRL-[A-Z] keys
					c += 32;
				}
				return listener.call(this, _synthesizeEvent(evt, { charCode: c }));
			});
		};
	}else{
		keypress = function(object, listener){
			return on(object, "keypress", function(evt){
				setKeyChar(evt);
				return listener.call(this, evt);
			});
		};
	}
}

var connect = {
	// summary:
	//		This module defines the dojo.connect API.
	//		This modules also provides keyboard event handling helpers.
	//		This module exports an extension event for emulating Firefox's keypress handling.
	//		However, this extension event exists primarily for backwards compatibility and
	//		is not recommended. WebKit and IE uses an alternate keypress handling (only
	//		firing for printable characters, to distinguish from keydown events), and most
	//		consider the WebKit/IE behavior more desirable.

	_keypress:keypress,

	connect:function(obj, event, context, method, dontFix){
		// summary:
		//		`dojo.connect` is a deprecated event handling and delegation method in
		//		Dojo. It allows one function to "listen in" on the execution of
		//		any other, triggering the second whenever the first is called. Many
		//		listeners may be attached to a function, and source functions may
		//		be either regular function calls or DOM events.
		//
		// description:
		//		Connects listeners to actions, so that after event fires, a
		//		listener is called with the same arguments passed to the original
		//		function.
		//
		//		Since `dojo.connect` allows the source of events to be either a
		//		"regular" JavaScript function or a DOM event, it provides a uniform
		//		interface for listening to all the types of events that an
		//		application is likely to deal with though a single, unified
		//		interface. DOM programmers may want to think of it as
		//		"addEventListener for everything and anything".
		//
		//		When setting up a connection, the `event` parameter must be a
		//		string that is the name of the method/event to be listened for. If
		//		`obj` is null, `kernel.global` is assumed, meaning that connections
		//		to global methods are supported but also that you may inadvertently
		//		connect to a global by passing an incorrect object name or invalid
		//		reference.
		//
		//		`dojo.connect` generally is forgiving. If you pass the name of a
		//		function or method that does not yet exist on `obj`, connect will
		//		not fail, but will instead set up a stub method. Similarly, null
		//		arguments may simply be omitted such that fewer than 4 arguments
		//		may be required to set up a connection See the examples for details.
		//
		//		The return value is a handle that is needed to
		//		remove this connection with `dojo.disconnect`.
		//
		// obj: Object?
		//		The source object for the event function.
		//		Defaults to `kernel.global` if null.
		//		If obj is a DOM node, the connection is delegated
		//		to the DOM event manager (unless dontFix is true).
		//
		// event: String
		//		String name of the event function in obj.
		//		I.e. identifies a property `obj[event]`.
		//
		// context: Object|null
		//		The object that method will receive as "this".
		//
		//		If context is null and method is a function, then method
		//		inherits the context of event.
		//
		//		If method is a string then context must be the source
		//		object object for method (context[method]). If context is null,
		//		kernel.global is used.
		//
		// method: String|Function
		//		A function reference, or name of a function in context.
		//		The function identified by method fires after event does.
		//		method receives the same arguments as the event.
		//		See context argument comments for information on method's scope.
		//
		// dontFix: Boolean?
		//		If obj is a DOM node, set dontFix to true to prevent delegation
		//		of this connection to the DOM event manager.
		//
		// example:
		//		When obj.onchange(), do ui.update():
		//	|	dojo.connect(obj, "onchange", ui, "update");
		//	|	dojo.connect(obj, "onchange", ui, ui.update); // same
		//
		// example:
		//		Using return value for disconnect:
		//	|	var link = dojo.connect(obj, "onchange", ui, "update");
		//	|	...
		//	|	dojo.disconnect(link);
		//
		// example:
		//		When onglobalevent executes, watcher.handler is invoked:
		//	|	dojo.connect(null, "onglobalevent", watcher, "handler");
		//
		// example:
		//		When ob.onCustomEvent executes, customEventHandler is invoked:
		//	|	dojo.connect(ob, "onCustomEvent", null, "customEventHandler");
		//	|	dojo.connect(ob, "onCustomEvent", "customEventHandler"); // same
		//
		// example:
		//		When ob.onCustomEvent executes, customEventHandler is invoked
		//		with the same scope (this):
		//	|	dojo.connect(ob, "onCustomEvent", null, customEventHandler);
		//	|	dojo.connect(ob, "onCustomEvent", customEventHandler); // same
		//
		// example:
		//		When globalEvent executes, globalHandler is invoked
		//		with the same scope (this):
		//	|	dojo.connect(null, "globalEvent", null, globalHandler);
		//	|	dojo.connect("globalEvent", globalHandler); // same

		// normalize arguments
		var a=arguments, args=[], i=0;
		// if a[0] is a String, obj was omitted
		args.push(typeof a[0] == "string" ? null : a[i++], a[i++]);
		// if the arg-after-next is a String or Function, context was NOT omitted
		var a1 = a[i+1];
		args.push(typeof a1 == "string" || typeof a1 == "function" ? a[i++] : null, a[i++]);
		// absorb any additional arguments
		for(var l=a.length; i<l; i++){	args.push(a[i]); }
		return connect_.apply(this, args);
	},

	disconnect:function(handle){
		// summary:
		//		Remove a link created by dojo.connect.
		// description:
		//		Removes the connection between event and the method referenced by handle.
		// handle: Handle
		//		the return value of the dojo.connect call that created the connection.

		if(handle){
			handle.remove();
		}
	},

	subscribe:function(topic, context, method){
		// summary:
		//		Attach a listener to a named topic. The listener function is invoked whenever the
		//		named topic is published (see: dojo.publish).
		//		Returns a handle which is needed to unsubscribe this listener.
		// topic: String
		//		The topic to which to subscribe.
		// context: Object?
		//		Scope in which method will be invoked, or null for default scope.
		// method: String|Function
		//		The name of a function in context, or a function reference. This is the function that
		//		is invoked when topic is published.
		// example:
		//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); });
		//	|	dojo.publish("alerts", [ "read this", "hello world" ]);
		return hub.subscribe(topic, lang.hitch(context, method));
	},

	publish:function(topic, args){
		// summary:
		//		Invoke all listener method subscribed to topic.
		// topic: String
		//		The name of the topic to publish.
		// args: Array?
		//		An array of arguments. The arguments will be applied
		//		to each topic subscriber (as first class parameters, via apply).
		// example:
		//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
		//	|	dojo.publish("alerts", [ "read this", "hello world" ]);
		return hub.publish.apply(hub, [topic].concat(args));
	},

	connectPublisher:function(topic, obj, event){
		// summary:
		//		Ensure that every time obj.event() is called, a message is published
		//		on the topic. Returns a handle which can be passed to
		//		dojo.disconnect() to disable subsequent automatic publication on
		//		the topic.
		// topic: String
		//		The name of the topic to publish.
		// obj: Object?
		//		The source object for the event function. Defaults to kernel.global
		//		if null.
		// event: String
		//		The name of the event function in obj.
		//		I.e. identifies a property obj[event].
		// example:
		//	|	dojo.connectPublisher("/ajax/start", dojo, "xhrGet");
		var pf = function(){ connect.publish(topic, arguments); };
		return event ? connect.connect(obj, event, pf) : connect.connect(obj, pf); //Handle
	},

	isCopyKey: function(e){
		// summary:
		//		Checks an event for the copy key (meta on Mac, and ctrl anywhere else)
		// e: Event
		//		Event object to examine
		return e[evtCopyKey];	// Boolean
	}
};

connect.unsubscribe = connect.disconnect;
/*=====
 connect.unsubscribe = function(handle){
	 // summary:
	 //		Remove a topic listener.
	 // handle: Handle
	 //		The handle returned from a call to subscribe.
	 // example:
	 //	|	var alerter = dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
	 //	|	...
	 //	|	dojo.unsubscribe(alerter);
 };
 =====*/

 1  && lang.mixin(dojo, connect);
return connect;

});



},
'xcitec/layout/_LabelControlHolder':function(){
/*
 slmui.layout._LabelControlItem is base class internal holder for Label/Control
 */
define([ "dijit/registry",
         "dojo/_base/declare",
        "dojo/_base/window",
        "dojo/dom-attr",
        "dojo/dom-style",
        "dojo/query",
        "dojo/dom-construct",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "../Label", "../xcitecUtil"], function(registry, declare, windowModule, domAttr, domStyle, query, domConstruct,_Widget, _Templated, Label, xcitecUtil){
  return declare([_Widget, _Templated], {
    // template will contains controlHolderNode
    //templateString: '<div class="dijitReset fieldsetFiltersRow" data-dojo-attach-point="containerNode, controlHolderNode"></div>',

    /* default holder Attributes */
    defaultLabelPos: "top",

    /* controls attributes of label */
    labelType: "",
    label:"",
    labelId:"",
    labelPos:"", /* allowed values none/left/top(/default?) */

    /* additional attributes for label from control */
    labelRequired:false,
    labelDisabled:false,
    errorMsg: "",

    /* control view options attributes */
    controlView: "",

    // width of control in percents
    widthInPercent: '100',

    /* label and control objects DOM/Widget*/
    labelNode: null,
    labelWidget: null,
    controlNode: null,
    controlWidget: null,

    isLabelControlHolder: true,

    postMixInProperties: function(){
      this.inherited(arguments);
      if(this.controlNode) this.processAdditionalProperties(this.controlNode);
    },

    processAdditionalProperties: function(/*DomNode*/ controlNode){
      //collect label attrs
      this.label = domAttr.get(this.controlNode, "label");
      this.labelId = domAttr.get(this.controlNode, "labelId");
      this.labelPos = domAttr.get(this.controlNode, "labelPos");
      this.labelType = domAttr.get(this.controlNode, "labelType") || this.labelType;
      this.labelRequired = xcitecUtil.str2bool(domAttr.get(this.controlNode, "required"));
      this.labelDisabled = xcitecUtil.str2bool(domAttr.get(this.controlNode, "disabled"));
      this.errorMsg = domAttr.get(this.controlNode, "error");
      this.controlView = domAttr.get(this.controlNode, "controlView") || this.controlView;
      this.labelClickEvent = domAttr.get(this.controlNode, "onLabelClick");
      this.widthInPercent = domAttr.get(this.controlNode, "widthInPercent");
      if(this.widthInPercent == null){
        this.widthInPercent = '100';
      }
    },

    _addControlNode: function(){
      if(this.widthInPercent == '100'){
        domStyle.set(this.controlNode, "width", "100%");
        domAttr.set(this.controlNode, "fitToContainer", "true");
      } else if(this.widthInPercent == ""){
        domStyle.set(this.controlNode, "width", "auto");
      } else{
        domStyle.set(this.controlNode, "width", this.widthInPercent + "%");
      }
      this.controlHolderNode.appendChild(this.controlNode);
    },

    _addLabelNode: function(){
      domConstruct.place(this.labelNode, this.controlNode, "before");
    },

    buildRendering: function(){
      this.inherited(arguments);
      if(this.controlNode){
        this._addControlNode();
      }
      if(this.label){
        this.labelNode = windowModule.doc.createElement("label");
        //this.labelNode.innerText = this.label;
        // to avoid script injection - escape label value
        this.labelNode.innerHTML = xcitecUtil.escapeXml(this.label);
      }
      if(this.labelNode){
        //        this._addLabelNode();
      }
    },

    startup: function(){
      if(this._started){
        return;
      }
      if(this.controlHolderNode){
        this.controlNode = query("> [widgetId]", this.controlHolderNode)[0];
      }
      if(this.controlNode){
        this.controlWidget = registry.byNode(this.controlNode);
      }
      if(this.controlWidget && this.controlWidget.hasOwnLabelProcessing){
        this.labelNode = null;
      }

      if(this.labelNode){
        this._addLabelNode();
        var _labelParams = {
          labelId: this.labelId,
          htmlFor: domAttr.get(this.controlNode, "widgetId"),
          labelType: this.labelType,
          allowWrap: this.hasLabelAtPos("TOP"),
          required: this.labelRequired,
          disabled: this.labelDisabled ,
          error: (this.errorMsg ? true : false)
        };
        if(this.labelClickEvent){
          _labelParams.hasAnchor = true;
          _labelParams.onLabelClick = new Function(this.labelClickEvent);
        }
        this.labelWidget = new Label(_labelParams, this.labelNode);
      }

      this.inherited(arguments);
    },

    //@Override - dijit._WidgetBase
    // if containerNode is not set - set it to domNode - to destroy children
    destroyDescendants: function(/*Boolean?*/ preserveDom){
      this.containerNode = this.containerNode || this.domNode;
      this.inherited(arguments);
    },

    /* additional control functions */
    getLabelPos: function(){
      if(!this.label || !this.labelNode) return "none";
      return ((this.labelPos) ? this.labelPos : this.defaultLabelPos);
    },

    hasLabelAtPos: function(/*String*/inPos){
      //if (!this.label || !this.labelNode) return false;
      if(!inPos) return true;
      var curPos = (this.labelPos) ? this.labelPos : this.defaultLabelPos;
      return (curPos.toUpperCase() == inPos.toUpperCase());
    }
  });
});

},
'xcitec/page/_MenuItemBase':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/fx",
	"dojo/query",
	"dojo/topic",
	"dojo/string",
  "dojo/keys",
	"dijit/registry",
	"xcitec/_StateWidget",
  "../xcitecUtil"
], function (dojo, declare, lang, windowModule, domAttr, domClass, fx, query, topic, string, keys, registry, _StateWidget, xcitecUtil) {

	var _MenuItemBase = declare( _StateWidget, {
		level: -1,
		itemIndex: -1,
		iconClass: "",
		label: "",
		labelId: "",
		selected: false,
		highlighted: false,
		isPopup: false,
		_subMenu: null,
	
		postMixInProperties: function(){
			this.stateHolder = query(">*[xcitecRole$=popupstateholder]", this.srcNodeRef)[0];
			if(this.stateHolder){
				this.selected = xcitecUtil.str2bool(domAttr.get(this.stateHolder, "value"));
			}
			var baseClassTemplate = "${0}";
			if(this.level > -1){
				baseClassTemplate += " ${0}L${1}";
				if(this.itemIndex > -1) baseClassTemplate += " ${0}L${1}I${2}";
			}
			this.baseClass = string.substitute(baseClassTemplate, [this.baseClass, this.level, this.itemIndex]);
			this.inherited(arguments);
		},
	
		postCreate: function(){
			this.stateNode.appendChild(windowModule.doc.createTextNode(this.label));
			this._setStateClass();
			domClass.remove(this.domNode, this.baseClass);
		},
	
		startup: function(){
			this.inherited(arguments);
			if(this.isPopup){
				this._subMenu = query('>', this.domNode)[1];
			}
		},
	
		onClick: function(/*Event*/ e){
			//Internal overridable function for when the item is clicked
		},
	
		_showSubMenu: function(){
			this.selected = true;
			if(this.stateHolder){
				domAttr.set(this.stateHolder, 'value', 'true');
			}
			fx.wipeIn({node: this._subMenu, onEnd: lang.hitch(this, '_setStateClass'), duration: 100}).play();
		},
	
		_hideSubMenu: function(){
			this.selected = false;
			if(this.stateHolder){
				domAttr.set(this.stateHolder, 'value', 'false');
			}
			fx.wipeOut({node: this._subMenu, onEnd: lang.hitch(this, '_setStateClass'), duration: 100}).play();
		},
	
		_toggleSubMenu: function(){
			if(this.selected){
				this._hideSubMenu();
			}else{
				this._showSubMenu();
			}
		},
	
		_moveFocusToParent: function(/*Event*/ e){
			e.preventDefault(), e.stopPropagation();
			var parent = query(this.domNode).parents('li')[0];
			if(parent){
				var parentWidget = registry.byNode(parent);
				if(parentWidget.isPopup){
					parentWidget.setFocus();
					return true;
				}
			}
			return false;
		},
	
		_moveFocusToParentNextSibling: function(/*Event*/ e){
			e.preventDefault(), e.stopPropagation();
			var parent = query(this.domNode).parents('li')[0];
			if(parent){
				var parentWidget = registry.byNode(parent);
				if(parentWidget && parentWidget.isPopup){
					while(parent && !parent.nextElementSibling){
						parent = query(parent).parents('li')[0];
					}
					if(parent.nextElementSibling){
						var parentNextSiblingWidget = registry.byNode(parent.nextElementSibling);
						if(parentNextSiblingWidget){
							parentNextSiblingWidget.setFocus();
						}
					}
				}
			}
		},
	
		_moveFocusToNextSibling: function(/*Event*/ e){
			e.preventDefault(), e.stopPropagation();
			if(this.domNode.nextElementSibling){
				var nextWidget = registry.byNode(this.domNode.nextElementSibling);
				if(nextWidget){
					nextWidget.setFocus();
					return true;
				}
			}
			return false;
		},
	
		_moveFocusToPrevSibling: function(/*Event*/ e){
			e.preventDefault(), e.stopPropagation();
			var prevSibling = this.domNode.previousElementSibling;
			if(prevSibling){
				var prevWidget = registry.byNode(prevSibling);
				while(prevWidget && prevWidget.isPopup && prevWidget.selected){
					prevSibling = query('> ul > :last-child', prevSibling)[0];
					prevWidget = registry.byNode(prevSibling);
				}
				prevWidget.setFocus();
				return true;
			}
			return false;
		},
	
		_moveFocusToSubMenuFirstChild: function(/*Event*/ e){
			e.preventDefault(), e.stopPropagation();
			var subMenuFirstChild = query('li', query('>', this.domNode)[1])[0];
			if(subMenuFirstChild){
				var subMenuFirstChildWidget = registry.byNode(subMenuFirstChild);
				if(subMenuFirstChildWidget){
					subMenuFirstChildWidget.setFocus();
				}
			}
		},
	
		_onKeyPress: function(/*Event*/ e){
			switch(e.keyCode){
				case keys.ENTER:
					this._onClick(e);
					break;
				case keys.UP_ARROW:
					if(!this._moveFocusToPrevSibling(e)){
						if(!this._moveFocusToParent(e)){
							registry.byNode(query(this.domNode).parents('div')[0]).setFocus();
						}
					}
					break;
				case keys.DOWN_ARROW:
					if(this.isPopup && this.selected){
						this._moveFocusToSubMenuFirstChild(e);
					}else if(!this._moveFocusToNextSibling(e)){
						this._moveFocusToParentNextSibling(e);
					}
					break;
				case keys.LEFT_ARROW:
					if(this.isPopup && this.selected){
						e.preventDefault(), e.stopPropagation();
						this._hideSubMenu();
					}else{
						this._moveFocusToParent(e);
					}
					break;
				case keys.RIGHT_ARROW:
					if(this.isPopup){
						if(!this.selected){
							e.preventDefault(), e.stopPropagation();
							this._showSubMenu();
						}else{
							this._moveFocusToSubMenuFirstChild(e);
						}
					}
					break;
			}
		}
	});
	
	return _MenuItemBase;
});

},
'xcitec/DataFormDialog':function(){
define([
   "dojo/_base/declare",
   "dojo/window",
   "dojo/dom-style",
   "./xcitecDialog"
], function(declare, window, domStyle, xcitecDialog){
  return declare(xcitecDialog, {
	    postCreate:function() {
		    var viewport = window.getBox();
		    var w = Math.floor(viewport.w * 0.75);
		    domStyle.set(this.domNode, {width: w + "px"});
        this.inherited(arguments);
	    }
	});
});
},
'xcitec/layout/_ContentAreaPaneItem':function(){
/*
 slmui.layout._ContentAreaPaneItem is internal holder for Label/Control
 */
define(["dojo/_base/declare", "./_LabelControlHolder"], function(declare, _LabelControlHolder){
  return declare(_LabelControlHolder, {
    templateString: '<div class="dijitReset" data-dojo-attach-point="containerNode, controlHolderNode"></div>',
    labelType: "Bold"
  });
});

},
'xcitec/form/EmailTextBox':function(){
define([
    "dojo/_base/declare",
    "../xcitecUtil",
    "./UrlTextBox"
], function (declare, xcitecUtil, UrlTextBox) {
    return declare(UrlTextBox, {
        actionIconType: "Mail",

        _isValueValid: function (/*String */ value) {
            return xcitecUtil.isValidEmail(value);
        },

        _doExecuteAction: function (/*String*/ value) {
            if (!this.buttonEnabled) return;
            var mailWindow = window.open("mailto:" + value);
            if (mailWindow) {
                mailWindow.close();
            }
        }
    });
});

},
'dijit/_KeyNavContainer':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys.END keys.HOME
	"dojo/_base/lang", // lang.hitch
	"./registry",
	"./_Container",
	"./_FocusMixin",
	"./_KeyNavMixin"
], function(array, declare, domAttr, kernel, keys, lang, registry, _Container, _FocusMixin, _KeyNavMixin){


	// module:
	//		dijit/_KeyNavContainer

	return declare("dijit._KeyNavContainer", [_FocusMixin, _KeyNavMixin, _Container], {
		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		Provides normalized keyboard and focusing code for Container widgets.
		//		To use this mixin, call connectKeyNavHandlers() in postCreate().
		//		Also, child widgets must implement a focus() method.

		connectKeyNavHandlers: function(/*keys[]*/ prevKeyCodes, /*keys[]*/ nextKeyCodes){
			// summary:
			//		Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,
			//		but the preferred method is to override _onLeftArrow() and _onRightArrow(), or
			//		_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().
			// prevKeyCodes: keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			// TODO: remove for 2.0, and make subclasses override _onLeftArrow, _onRightArrow etc. instead.

			var keyCodes = (this._keyNavCodes = {});
			var prev = lang.hitch(this, "focusPrev");
			var next = lang.hitch(this, "focusNext");
			array.forEach(prevKeyCodes, function(code){
				keyCodes[code] = prev;
			});
			array.forEach(nextKeyCodes, function(code){
				keyCodes[code] = next;
			});
			keyCodes[keys.HOME] = lang.hitch(this, "focusFirstChild");
			keyCodes[keys.END] = lang.hitch(this, "focusLastChild");
		},

		startupKeyNavChildren: function(){
			kernel.deprecated("startupKeyNavChildren() call no longer needed", "", "2.0");
		},

		startup: function(){
			this.inherited(arguments);
			array.forEach(this.getChildren(), lang.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit/_WidgetBase*/ widget, /*int?*/ insertIndex){
			this.inherited(arguments);
			this._startupChild(widget);
		},

		_startupChild: function(/*dijit/_WidgetBase*/ widget){
			// summary:
			//		Setup for each child widget.
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will
			//		leave the container rather than visiting each child.
			//
			//		Note: if you add children by a different method than addChild(), then need to call this manually
			//		or at least make sure the child's tabIndex is -1.
			//
			//		Note: see also _LayoutWidget.setupChild(), which is also called for each child widget.
			// tags:
			//		private

			widget.set("tabIndex", "-1");
		},

		_getFirst: function(){
			// summary:
			//		Returns the first child.
			// tags:
			//		abstract extension
			var children = this.getChildren();
			return children.length ? children[0] : null;
		},

		_getLast: function(){
			// summary:
			//		Returns the last descendant.
			// tags:
			//		abstract extension
			var children = this.getChildren();
			return children.length ? children[children.length - 1] : null;
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, 1));
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, -1), true);
		},

		childSelector: function(/*DOMNode*/ node){
			// Implement _KeyNavMixin.childSelector, to identify focusable child nodes.
			// If we allowed a dojo/query dependency from this module this could more simply be a string "> *"
			// instead of this function.

			var node = registry.byNode(node);
			return node && node.getParent() == this;
		}
	});
});

},
'xcitec/SearchResultBox':function(){
define([
         "dojo/_base/declare",
         "./Box"
       ], function (declare, Box) {
  return declare(Box, {
  });
});

},
'xcitec/page/ClassicVersionFrame':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/text!xcitec/page/templates/ClassicVersionFrame.html",
	"xcitec/layout/XLayout"
], function (dojo, declare, domAttr, template, XLayout) {

	var ClassicVersionFrame = declare( XLayout, {
	    templateString: template,
	    postCreate: function() {
	        domAttr.set(this.ClassicVersionFrame, "src", require.toUrl("external/ClassicVersion.html").toString());
	    }
	});
	return ClassicVersionFrame;
});
},
'dijit/Tooltip':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/dom", // dom.byId
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArrayLike
	"dojo/mouse",
	"dojo/on",
	"dojo/sniff", // has("ie"), has("trident")
	"./_base/manager",	// manager.defaultDuration
	"./place",
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe",
	"dojo/text!./templates/Tooltip.html",
	"./main"		// sets dijit.showTooltip etc. for back-compat
], function(array, declare, fx, dom, domClass, domGeometry, domStyle, lang, mouse, on, has,
			manager, place, _Widget, _TemplatedMixin, BackgroundIframe, template, dijit){

	// module:
	//		dijit/Tooltip


	// TODO: Tooltip should really share more positioning code with TooltipDialog, like:
	//		- the orient() method
	//		- the connector positioning code in show()
	//		- the dijitTooltip[Dialog] class
	//
	// The problem is that Tooltip's implementation supplies it's own <iframe> and interacts directly
	// with dijit/place, rather than going through dijit/popup like TooltipDialog and other popups (ex: Menu).

	var MasterTooltip = declare("dijit._MasterTooltip", [_Widget, _TemplatedMixin], {
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: manager.defaultDuration,

		templateString: template,

		postCreate: function(){
			this.ownerDocumentBody.appendChild(this.domNode);

			this.bgIframe = new BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = fx.fadeIn({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onShow") });
			this.fadeOut = fx.fadeOut({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onHide") });
		},

		show: function(innerHTML, aroundNode, position, rtl, textDir){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)
			// innerHTML: String
			//		Contents of the tooltip
			// aroundNode: DomNode|dijit/place.__Rectangle
			//		Specifies that tooltip should be next to this node / area
			// position: String[]?
			//		List of positions to try to position tooltip (ex: ["right", "above"])
			// rtl: Boolean?
			//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
			//		means "rtl"; specifies GUI direction, not text direction.
			// textDir: String?
			//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.


			if(this.aroundNode && this.aroundNode === aroundNode && this.containerNode.innerHTML == innerHTML){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			if(textDir){
				this.set("textDir", textDir);
			}

			this.containerNode.align = rtl? "right" : "left"; //fix the text alignment

			var pos = place.around(this.domNode, aroundNode,
				position && position.length ? position : Tooltip.defaultPosition, !rtl, lang.hitch(this, "orient"));

			// Position the tooltip connector for middle alignment.
			// This could not have been done in orient() since the tooltip wasn't positioned at that time.
			var aroundNodeCoords = pos.aroundNodePos;
			if(pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M'){
				this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
				this.connectorNode.style.left = "";
			}else if(pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M'){
				this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
			}else{
				// Not *-centered, but just above/below/after/before
				this.connectorNode.style.left = "";
				this.connectorNode.style.top = "";
			}

			// show it
			domStyle.set(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.   It will also reduce the tooltip's
			//		width to whatever width is available
			// tags:
			//		protected

			this.connectorNode.style.top = ""; //reset to default

			var heightAvailable = spaceAvailable.h,
				widthAvailable = spaceAvailable.w;

			node.className = "dijitTooltip " +
				{
					"MR-ML": "dijitTooltipRight",
					"ML-MR": "dijitTooltipLeft",
					"TM-BM": "dijitTooltipAbove",
					"BM-TM": "dijitTooltipBelow",
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];

			// reset width; it may have been set by orient() on a previous tooltip show()
			this.domNode.style.width = "auto";

			// Reduce tooltip's width to the amount of width available, so that it doesn't overflow screen.
			// Note that sometimes widthAvailable is negative, but we guard against setting style.width to a
			// negative number since that causes an exception on IE.
			var size = domGeometry.position(this.domNode);
			if(has("ie") || has("trident")){
				// workaround strange IE bug where setting width to offsetWidth causes words to wrap
				size.w += 2;
			}

			var width = Math.min((Math.max(widthAvailable,1)), size.w);

			domGeometry.setMarginBox(this.domNode, {w: width});

			// Reposition the tooltip connector.
			if(tooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B'){
				var bb = domGeometry.position(node);
				var tooltipConnectorHeight = this.connectorNode.offsetHeight;
				if(bb.h > heightAvailable){
					// The tooltip starts at the top of the page and will extend past the aroundNode
					var aroundNodePlacement = heightAvailable - ((aroundNodeCoords.h + tooltipConnectorHeight) >> 1);
					this.connectorNode.style.top = aroundNodePlacement + "px";
					this.connectorNode.style.bottom = "";
				}else{
					// Align center of connector with center of aroundNode, except don't let bottom
					// of connector extend below bottom of tooltip content, or top of connector
					// extend past top of tooltip content
					this.connectorNode.style.bottom = Math.min(
						Math.max(aroundNodeCoords.h/2 - tooltipConnectorHeight/2, 0),
						bb.h - tooltipConnectorHeight) + "px";
					this.connectorNode.style.top = "";
				}
			}else{
				// reset the tooltip back to the defaults
				this.connectorNode.style.top = "";
				this.connectorNode.style.bottom = "";
			}

			return Math.max(0, size.w - widthAvailable);
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(has("ie")){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip

			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}
	});

	if(has("dojo-bidi")){
		MasterTooltip.extend({
			_setAutoTextDir: function(/*Object*/node){
				// summary:
				//		Resolve "auto" text direction for children nodes
				// tags:
				//		private

				this.applyTextDir(node);
				array.forEach(node.children, function(child){ this._setAutoTextDir(child); }, this);
			},

			_setTextDirAttr: function(/*String*/ textDir){
				// summary:
				//		Setter for textDir.
				// description:
				//		Users shouldn't call this function; they should be calling
				//		set('textDir', value)
				// tags:
				//		private

				this._set("textDir", textDir);

				if (textDir == "auto"){
					this._setAutoTextDir(this.containerNode);
				}else{
					this.containerNode.dir = this.textDir;
				}
			}
		});
	}

	dijit.showTooltip = function(innerHTML, aroundNode, position, rtl, textDir){
		// summary:
		//		Static method to display tooltip w/specified contents in specified position.
		//		See description of dijit/Tooltip.defaultPosition for details on position parameter.
		//		If position is not specified then dijit/Tooltip.defaultPosition is used.
		// innerHTML: String
		//		Contents of the tooltip
		// aroundNode: place.__Rectangle
		//		Specifies that tooltip should be next to this node / area
		// position: String[]?
		//		List of positions to try to position tooltip (ex: ["right", "above"])
		// rtl: Boolean?
		//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
		//		means "rtl"; specifies GUI direction, not text direction.
		// textDir: String?
		//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.

		// After/before don't work, but for back-compat convert them to the working after-centered, before-centered.
		// Possibly remove this in 2.0.   Alternately, get before/after to work.
		if(position){
			position = array.map(position, function(val){
				return {after: "after-centered", before: "before-centered"}[val] || val;
			});
		}

		if(!Tooltip._masterTT){ dijit._masterTT = Tooltip._masterTT = new MasterTooltip(); }
		return Tooltip._masterTT.show(innerHTML, aroundNode, position, rtl, textDir);
	};

	dijit.hideTooltip = function(aroundNode){
		// summary:
		//		Static method to hide the tooltip displayed via showTooltip()
		return Tooltip._masterTT && Tooltip._masterTT.hide(aroundNode);
	};

	var Tooltip = declare("dijit.Tooltip", _Widget, {
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.
		//		Also provides static show() and hide() methods that can be used without instantiating a dijit/Tooltip.

		// label: String
		//		HTML to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String|String[]|DomNode|DomNode[]
		//		Id of domNode(s) to attach the tooltip to.
		//		When user hovers over specified dom node(s), the tooltip will appear.
		connectId: [],

		// position: String[]
		//		See description of `dijit/Tooltip.defaultPosition` for details on position parameter.
		position: [],

		// selector: String?
		//		CSS expression to apply this Tooltip to descendants of connectIds, rather than to
		//		the nodes specified by connectIds themselves.    Useful for applying a Tooltip to
		//		a range of rows in a table, tree, etc.   Use in conjunction with getContent() parameter.
		//		Ex: connectId: myTable, selector: "tr", getContent: function(node){ return ...; }
		//
		//		The application must require() an appropriate level of dojo/query to handle the selector.
		selector: "",

		// TODO: in 2.0 remove support for multiple connectIds.   selector gives the same effect.
		// So, change connectId to a "", remove addTarget()/removeTarget(), etc.

		_setConnectIdAttr: function(/*String|String[]|DomNode|DomNode[]*/ newId){
			// summary:
			//		Connect to specified node(s)

			// Remove connections to old nodes (if there are any)
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, function(handle){ handle.remove(); });
			}, this);

			// Make array of id's to connect to, excluding entries for nodes that don't exist yet, see startup()
			this._connectIds = array.filter(lang.isArrayLike(newId) ? newId : (newId ? [newId] : []),
					function(id){ return dom.byId(id, this.ownerDocument); }, this);

			// Make connections
			this._connections = array.map(this._connectIds, function(id){
				var node = dom.byId(id, this.ownerDocument),
					selector = this.selector,
					delegatedEvent = selector ?
						function(eventType){ return on.selector(selector, eventType); } :
						function(eventType){ return eventType; },
					self = this;
				return [
					on(node, delegatedEvent(mouse.enter), function(){
						self._onHover(this);
					}),
					on(node, delegatedEvent("focusin"), function(){
						self._onHover(this);
					}),
					on(node, delegatedEvent(mouse.leave), lang.hitch(self, "_onUnHover")),
					on(node, delegatedEvent("focusout"), lang.hitch(self, "_onUnHover"))
				];
			}, this);

			this._set("connectId", newId);
		},

		addTarget: function(/*OomNode|String*/ node){
			// summary:
			//		Attach tooltip to specified node if it's not already connected

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node;
			if(array.indexOf(this._connectIds, id) == -1){
				this.set("connectId", this._connectIds.concat(id));
			}
		},

		removeTarget: function(/*DomNode|String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node,	// map from DOMNode back to plain id string
				idx = array.indexOf(this._connectIds, id);
			if(idx >= 0){
				// remove id (modifies original this._connectIds but that's OK in this case)
				this._connectIds.splice(idx, 1);
				this.set("connectId", this._connectIds);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			array.forEach(lang.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		getContent: function(/*DomNode*/ node){
			// summary:
			//		User overridable function that return the text to display in the tooltip.
			// tags:
			//		extension
			return this.label || this.domNode.innerHTML;
		},

		_onHover: function(/*DomNode*/ target){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				this._showTimer = this.defer(function(){ this.open(target); }, this.showDelay);
			}
		},

		_onUnHover: function(){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			if(this._showTimer){
				this._showTimer.remove();
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				this._showTimer.remove();
				delete this._showTimer;
			}

			var content = this.getContent(target);
			if(!content){
				return;
			}
			Tooltip.show(content, target, this.position, !this.isLeftToRight(), this.textDir);

			this._connectNode = target;		// _connectNode means "tooltip currently displayed for this node"
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				Tooltip.hide(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				this._showTimer.remove();
				delete this._showTimer;
			}
		},

		onShow: function(/*===== target, position =====*/){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		destroy: function(){
			this.close();

			// Remove connections manually since they aren't registered to be removed by _WidgetBase
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, function(handle){ handle.remove(); });
			}, this);

			this.inherited(arguments);
		}
	});

	Tooltip._MasterTooltip = MasterTooltip;		// for monkey patching
	Tooltip.show = dijit.showTooltip;		// export function through module return value
	Tooltip.hide = dijit.hideTooltip;		// export function through module return value

	Tooltip.defaultPosition = ["after-centered", "before-centered"];

	/*=====
	lang.mixin(Tooltip, {
		 // defaultPosition: String[]
		 //		This variable controls the position of tooltips, if the position is not specified to
		 //		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the values
		 //		possible for `dijit/place.around()`.   The recommended values are:
		 //
		 //		- before-centered: centers tooltip to the left of the anchor node/widget, or to the right
		 //		  in the case of RTL scripts like Hebrew and Arabic
		 //		- after-centered: centers tooltip to the right of the anchor node/widget, or to the left
		 //		  in the case of RTL scripts like Hebrew and Arabic
		 //		- above-centered: tooltip is centered above anchor node
		 //		- below-centered: tooltip is centered above anchor node
		 //
		 //		The list is positions is tried, in order, until a position is found where the tooltip fits
		 //		within the viewport.
		 //
		 //		Be careful setting this parameter.  A value of "above-centered" may work fine until the user scrolls
		 //		the screen so that there's no room above the target node.   Nodes with drop downs, like
		 //		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
		 //		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
		 //		is only room below (or above) the target node, but not both.
	 });
	=====*/
	return Tooltip;
});

},
'dijit/PopupMenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang",
	"dojo/query", // query
	"./popup",
	"./registry",	// registry.byNode
	"./MenuItem",
	"./hccss"
], function(declare, domStyle, lang, query, pm, registry, MenuItem){

	// module:
	//		dijit/PopupMenuItem

	return declare("dijit.PopupMenuItem", MenuItem, {
		// summary:
		//		An item in a Menu that spawn a drop down (usually a drop down menu)

		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRef.innerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div data-dojo-type="dijit/PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = query("*", this.srcNodeRef);
				this.inherited(arguments, [nodes[0]]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		_openPopup: function(/*Object*/ params, /*Boolean*/ focus){
			// summary:
			//		Open the popup to the side of/underneath this MenuItem, and optionally focus first item
			// tags:
			//		protected

			var popup = this.popup;

			pm.open(lang.delegate(params, {
				popup: this.popup,
				around: this.domNode
			}));

			if(focus && popup.focus){
				popup.focus();
			}
		},

		_closePopup: function(){
			pm.close(this.popup);
			this.popup.parentMenu = null;
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// We didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  Move it to <body>.
			if(!this.popup){
				var node = query("[widgetId]", this.dropDownContainer)[0];
				this.popup = registry.byNode(node);
			}
			this.ownerDocumentBody.appendChild(this.popup.domNode);
			this.popup.domNode.setAttribute("aria-labelledby", this.containerNode.id);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				domStyle.set(this.arrowWrapper, "visibility", "");
			}
			this.focusNode.setAttribute("aria-haspopup", "true");
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive(preserveDom);
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});
});

},
'dojo/selector/acme':function(){
define([
	"../dom", "../sniff", "../_base/array", "../_base/lang", "../_base/window"
], function(dom, has, array, lang, win){

	// module:
	//		dojo/selector/acme

/*
	acme architectural overview:

		acme is a relatively full-featured CSS3 query library. It is
		designed to take any valid CSS3 selector and return the nodes matching
		the selector. To do this quickly, it processes queries in several
		steps, applying caching where profitable.

		The steps (roughly in reverse order of the way they appear in the code):
			1.) check to see if we already have a "query dispatcher"
				- if so, use that with the given parameterization. Skip to step 4.
			2.) attempt to determine which branch to dispatch the query to:
				- JS (optimized DOM iteration)
				- native (FF3.1+, Safari 3.1+, IE 8+)
			3.) tokenize and convert to executable "query dispatcher"
				- this is where the lion's share of the complexity in the
					system lies. In the DOM version, the query dispatcher is
					assembled as a chain of "yes/no" test functions pertaining to
					a section of a simple query statement (".blah:nth-child(odd)"
					but not "div div", which is 2 simple statements). Individual
					statement dispatchers are cached (to prevent re-definition)
					as are entire dispatch chains (to make re-execution of the
					same query fast)
			4.) the resulting query dispatcher is called in the passed scope
					(by default the top-level document)
				- for DOM queries, this results in a recursive, top-down
					evaluation of nodes based on each simple query section
				- for native implementations, this may mean working around spec
					bugs. So be it.
			5.) matched nodes are pruned to ensure they are unique (if necessary)
*/


	////////////////////////////////////////////////////////////////////////
	// Toolkit aliases
	////////////////////////////////////////////////////////////////////////

	// if you are extracting acme for use in your own system, you will
	// need to provide these methods and properties. No other porting should be
	// necessary, save for configuring the system to use a class other than
	// dojo/NodeList as the return instance instantiator
	var trim = 			lang.trim;
	var each = 			array.forEach;

	var getDoc = function(){ return win.doc; };
	// NOTE(alex): the spec is idiotic. CSS queries should ALWAYS be case-sensitive, but nooooooo
	var cssCaseBug = (getDoc().compatMode) == "BackCompat";

	////////////////////////////////////////////////////////////////////////
	// Global utilities
	////////////////////////////////////////////////////////////////////////


	var specials = ">~+";

	// global thunk to determine whether we should treat the current query as
	// case sensitive or not. This switch is flipped by the query evaluator
	// based on the document passed as the context to search.
	var caseSensitive = false;

	// how high?
	var yesman = function(){ return true; };

	////////////////////////////////////////////////////////////////////////
	// Tokenizer
	////////////////////////////////////////////////////////////////////////

	var getQueryParts = function(query){
		// summary:
		//		state machine for query tokenization
		// description:
		//		instead of using a brittle and slow regex-based CSS parser,
		//		acme implements an AST-style query representation. This
		//		representation is only generated once per query. For example,
		//		the same query run multiple times or under different root nodes
		//		does not re-parse the selector expression but instead uses the
		//		cached data structure. The state machine implemented here
		//		terminates on the last " " (space) character and returns an
		//		ordered array of query component structures (or "parts"). Each
		//		part represents an operator or a simple CSS filtering
		//		expression. The structure for parts is documented in the code
		//		below.


		// NOTE:
		//		this code is designed to run fast and compress well. Sacrifices
		//		to readability and maintainability have been made.  Your best
		//		bet when hacking the tokenizer is to put The Donnas on *really*
		//		loud (may we recommend their "Spend The Night" release?) and
		//		just assume you're gonna make mistakes. Keep the unit tests
		//		open and run them frequently. Knowing is half the battle ;-)
		if(specials.indexOf(query.slice(-1)) >= 0){
			// if we end with a ">", "+", or "~", that means we're implicitly
			// searching all children, so make it explicit
			query += " * ";
		}else{
			// if you have not provided a terminator, one will be provided for
			// you...
			query += " ";
		}

		var ts = function(/*Integer*/ s, /*Integer*/ e){
			// trim and slice.

			// take an index to start a string slice from and an end position
			// and return a trimmed copy of that sub-string
			return trim(query.slice(s, e));
		};

		// the overall data graph of the full query, as represented by queryPart objects
		var queryParts = [];


		// state keeping vars
		var inBrackets = -1, inParens = -1, inMatchFor = -1,
			inPseudo = -1, inClass = -1, inId = -1, inTag = -1, currentQuoteChar,
			lc = "", cc = "", pStart;

		// iteration vars
		var x = 0, // index in the query
			ql = query.length,
			currentPart = null, // data structure representing the entire clause
			_cp = null; // the current pseudo or attr matcher

		// several temporary variables are assigned to this structure during a
		// potential sub-expression match:
		//		attr:
		//			a string representing the current full attribute match in a
		//			bracket expression
		//		type:
		//			if there's an operator in a bracket expression, this is
		//			used to keep track of it
		//		value:
		//			the internals of parenthetical expression for a pseudo. for
		//			:nth-child(2n+1), value might be "2n+1"

		var endTag = function(){
			// called when the tokenizer hits the end of a particular tag name.
			// Re-sets state variables for tag matching and sets up the matcher
			// to handle the next type of token (tag or operator).
			if(inTag >= 0){
				var tv = (inTag == x) ? null : ts(inTag, x); // .toLowerCase();
				currentPart[ (specials.indexOf(tv) < 0) ? "tag" : "oper" ] = tv;
				inTag = -1;
			}
		};

		var endId = function(){
			// called when the tokenizer might be at the end of an ID portion of a match
			if(inId >= 0){
				currentPart.id = ts(inId, x).replace(/\\/g, "");
				inId = -1;
			}
		};

		var endClass = function(){
			// called when the tokenizer might be at the end of a class name
			// match. CSS allows for multiple classes, so we augment the
			// current item with another class in its list
			if(inClass >= 0){
				currentPart.classes.push(ts(inClass + 1, x).replace(/\\/g, ""));
				inClass = -1;
			}
		};

		var endAll = function(){
			// at the end of a simple fragment, so wall off the matches
			endId();
			endTag();
			endClass();
		};

		var endPart = function(){
			endAll();
			if(inPseudo >= 0){
				currentPart.pseudos.push({ name: ts(inPseudo + 1, x) });
			}
			// hint to the selector engine to tell it whether or not it
			// needs to do any iteration. Many simple selectors don't, and
			// we can avoid significant construction-time work by advising
			// the system to skip them
			currentPart.loops = (
					currentPart.pseudos.length ||
					currentPart.attrs.length ||
					currentPart.classes.length	);

			currentPart.oquery = currentPart.query = ts(pStart, x); // save the full expression as a string


			// otag/tag are hints to suggest to the system whether or not
			// it's an operator or a tag. We save a copy of otag since the
			// tag name is cast to upper-case in regular HTML matches. The
			// system has a global switch to figure out if the current
			// expression needs to be case sensitive or not and it will use
			// otag or tag accordingly
			currentPart.otag = currentPart.tag = (currentPart["oper"]) ? null : (currentPart.tag || "*");

			if(currentPart.tag){
				// if we're in a case-insensitive HTML doc, we likely want
				// the toUpperCase when matching on element.tagName. If we
				// do it here, we can skip the string op per node
				// comparison
				currentPart.tag = currentPart.tag.toUpperCase();
			}

			// add the part to the list
			if(queryParts.length && (queryParts[queryParts.length-1].oper)){
				// operators are always infix, so we remove them from the
				// list and attach them to the next match. The evaluator is
				// responsible for sorting out how to handle them.
				currentPart.infixOper = queryParts.pop();
				currentPart.query = currentPart.infixOper.query + " " + currentPart.query;
				/*
				console.debug(	"swapping out the infix",
								currentPart.infixOper,
								"and attaching it to",
								currentPart);
				*/
			}
			queryParts.push(currentPart);

			currentPart = null;
		};

		// iterate over the query, character by character, building up a
		// list of query part objects
		for(; lc=cc, cc=query.charAt(x), x < ql; x++){
			//		cc: the current character in the match
			//		lc: the last character (if any)

			// someone is trying to escape something, so don't try to match any
			// fragments. We assume we're inside a literal.
			if(lc == "\\"){ continue; }
			if(!currentPart){ // a part was just ended or none has yet been created
				// NOTE: I hate all this alloc, but it's shorter than writing tons of if's
				pStart = x;
				//	rules describe full CSS sub-expressions, like:
				//		#someId
				//		.className:first-child
				//	but not:
				//		thinger > div.howdy[type=thinger]
				//	the indidual components of the previous query would be
				//	split into 3 parts that would be represented a structure like:
				//		[
				//			{
				//				query: "thinger",
				//				tag: "thinger",
				//			},
				//			{
				//				query: "div.howdy[type=thinger]",
				//				classes: ["howdy"],
				//				infixOper: {
				//					query: ">",
				//					oper: ">",
				//				}
				//			},
				//		]
				currentPart = {
					query: null, // the full text of the part's rule
					pseudos: [], // CSS supports multiple pseud-class matches in a single rule
					attrs: [],	// CSS supports multi-attribute match, so we need an array
					classes: [], // class matches may be additive, e.g.: .thinger.blah.howdy
					tag: null,	// only one tag...
					oper: null, // ...or operator per component. Note that these wind up being exclusive.
					id: null,	// the id component of a rule
					getTag: function(){
						return caseSensitive ? this.otag : this.tag;
					}
				};

				// if we don't have a part, we assume we're going to start at
				// the beginning of a match, which should be a tag name. This
				// might fault a little later on, but we detect that and this
				// iteration will still be fine.
				inTag = x;
			}

			// Skip processing all quoted characters.
			// If we are inside quoted text then currentQuoteChar stores the character that began the quote,
			// thus that character that will end it.
			if(currentQuoteChar){
				if(cc == currentQuoteChar){
					currentQuoteChar = null;
				}
				continue;
			}else if (cc == "'" || cc == '"'){
				currentQuoteChar = cc;
				continue;
			}

			if(inBrackets >= 0){
				// look for a the close first
				if(cc == "]"){ // if we're in a [...] clause and we end, do assignment
					if(!_cp.attr){
						// no attribute match was previously begun, so we
						// assume this is an attribute existence match in the
						// form of [someAttributeName]
						_cp.attr = ts(inBrackets+1, x);
					}else{
						// we had an attribute already, so we know that we're
						// matching some sort of value, as in [attrName=howdy]
						_cp.matchFor = ts((inMatchFor||inBrackets+1), x);
					}
					var cmf = _cp.matchFor;
					if(cmf){
						// try to strip quotes from the matchFor value. We want
						// [attrName=howdy] to match the same
						//	as [attrName = 'howdy' ]
						if(	(cmf.charAt(0) == '"') || (cmf.charAt(0) == "'") ){
							_cp.matchFor = cmf.slice(1, -1);
						}
					}
					// remove backslash escapes from an attribute match, since DOM
					// querying will get attribute values without backslashes
					if(_cp.matchFor){
						_cp.matchFor = _cp.matchFor.replace(/\\/g, "");
					}

					// end the attribute by adding it to the list of attributes.
					currentPart.attrs.push(_cp);
					_cp = null; // necessary?
					inBrackets = inMatchFor = -1;
				}else if(cc == "="){
					// if the last char was an operator prefix, make sure we
					// record it along with the "=" operator.
					var addToCc = ("|~^$*".indexOf(lc) >=0 ) ? lc : "";
					_cp.type = addToCc+cc;
					_cp.attr = ts(inBrackets+1, x-addToCc.length);
					inMatchFor = x+1;
				}
				// now look for other clause parts
			}else if(inParens >= 0){
				// if we're in a parenthetical expression, we need to figure
				// out if it's attached to a pseudo-selector rule like
				// :nth-child(1)
				if(cc == ")"){
					if(inPseudo >= 0){
						_cp.value = ts(inParens+1, x);
					}
					inPseudo = inParens = -1;
				}
			}else if(cc == "#"){
				// start of an ID match
				endAll();
				inId = x+1;
			}else if(cc == "."){
				// start of a class match
				endAll();
				inClass = x;
			}else if(cc == ":"){
				// start of a pseudo-selector match
				endAll();
				inPseudo = x;
			}else if(cc == "["){
				// start of an attribute match.
				endAll();
				inBrackets = x;
				// provide a new structure for the attribute match to fill-in
				_cp = {
					/*=====
					attr: null, type: null, matchFor: null
					=====*/
				};
			}else if(cc == "("){
				// we really only care if we've entered a parenthetical
				// expression if we're already inside a pseudo-selector match
				if(inPseudo >= 0){
					// provide a new structure for the pseudo match to fill-in
					_cp = {
						name: ts(inPseudo+1, x),
						value: null
					};
					currentPart.pseudos.push(_cp);
				}
				inParens = x;
			}else if(
				(cc == " ") &&
				// if it's a space char and the last char is too, consume the
				// current one without doing more work
				(lc != cc)
			){
				endPart();
			}
		}
		return queryParts;
	};


	////////////////////////////////////////////////////////////////////////
	// DOM query infrastructure
	////////////////////////////////////////////////////////////////////////

	var agree = function(first, second){
		// the basic building block of the yes/no chaining system. agree(f1,
		// f2) generates a new function which returns the boolean results of
		// both of the passed functions to a single logical-anded result. If
		// either are not passed, the other is used exclusively.
		if(!first){ return second; }
		if(!second){ return first; }

		return function(){
			return first.apply(window, arguments) && second.apply(window, arguments);
		};
	};

	var getArr = function(i, arr){
		// helps us avoid array alloc when we don't need it
		var r = arr||[]; // FIXME: should this be 'new d._NodeListCtor()' ?
		if(i){ r.push(i); }
		return r;
	};

	var _isElement = function(n){ return (1 == n.nodeType); };

	// FIXME: need to coalesce _getAttr with defaultGetter
	var blank = "";
	var _getAttr = function(elem, attr){
		if(!elem){ return blank; }
		if(attr == "class"){
			return elem.className || blank;
		}
		if(attr == "for"){
			return elem.htmlFor || blank;
		}
		if(attr == "style"){
			return elem.style.cssText || blank;
		}
		return (caseSensitive ? elem.getAttribute(attr) : elem.getAttribute(attr, 2)) || blank;
	};

	var attrs = {
		"*=": function(attr, value){
			return function(elem){
				// E[foo*="bar"]
				//		an E element whose "foo" attribute value contains
				//		the substring "bar"
				return (_getAttr(elem, attr).indexOf(value)>=0);
			};
		},
		"^=": function(attr, value){
			// E[foo^="bar"]
			//		an E element whose "foo" attribute value begins exactly
			//		with the string "bar"
			return function(elem){
				return (_getAttr(elem, attr).indexOf(value)==0);
			};
		},
		"$=": function(attr, value){
			// E[foo$="bar"]
			//		an E element whose "foo" attribute value ends exactly
			//		with the string "bar"
			return function(elem){
				var ea = " "+_getAttr(elem, attr);
				var lastIndex = ea.lastIndexOf(value);
				return lastIndex > -1 && (lastIndex==(ea.length-value.length));
			};
		},
		"~=": function(attr, value){
			// E[foo~="bar"]
			//		an E element whose "foo" attribute value is a list of
			//		space-separated values, one of which is exactly equal
			//		to "bar"

			// return "[contains(concat(' ',@"+attr+",' '), ' "+ value +" ')]";
			var tval = " "+value+" ";
			return function(elem){
				var ea = " "+_getAttr(elem, attr)+" ";
				return (ea.indexOf(tval)>=0);
			};
		},
		"|=": function(attr, value){
			// E[hreflang|="en"]
			//		an E element whose "hreflang" attribute has a
			//		hyphen-separated list of values beginning (from the
			//		left) with "en"
			var valueDash = value+"-";
			return function(elem){
				var ea = _getAttr(elem, attr);
				return (
					(ea == value) ||
					(ea.indexOf(valueDash)==0)
				);
			};
		},
		"=": function(attr, value){
			return function(elem){
				return (_getAttr(elem, attr) == value);
			};
		}
	};

	// avoid testing for node type if we can. Defining this in the negative
	// here to avoid negation in the fast path.
	var _noNES = (typeof getDoc().firstChild.nextElementSibling == "undefined");
	var _ns = !_noNES ? "nextElementSibling" : "nextSibling";
	var _ps = !_noNES ? "previousElementSibling" : "previousSibling";
	var _simpleNodeTest = (_noNES ? _isElement : yesman);

	var _lookLeft = function(node){
		// look left
		while(node = node[_ps]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var _lookRight = function(node){
		// look right
		while(node = node[_ns]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var getNodeIndex = function(node){
		var root = node.parentNode;
		root = root.nodeType != 7 ? root : root.nextSibling; // PROCESSING_INSTRUCTION_NODE
		var i = 0,
			tret = root.children || root.childNodes,
			ci = (node["_i"]||node.getAttribute("_i")||-1),
			cl = (root["_l"]|| (typeof root.getAttribute !== "undefined" ? root.getAttribute("_l") : -1));

		if(!tret){ return -1; }
		var l = tret.length;

		// we calculate the parent length as a cheap way to invalidate the
		// cache. It's not 100% accurate, but it's much more honest than what
		// other libraries do
		if( cl == l && ci >= 0 && cl >= 0 ){
			// if it's legit, tag and release
			return ci;
		}

		// else re-key things
		if(has("ie") && typeof root.setAttribute !== "undefined"){
			root.setAttribute("_l", l);
		}else{
			root["_l"] = l;
		}
		ci = -1;
		for(var te = root["firstElementChild"]||root["firstChild"]; te; te = te[_ns]){
			if(_simpleNodeTest(te)){
				if(has("ie")){
					te.setAttribute("_i", ++i);
				}else{
					te["_i"] = ++i;
				}
				if(node === te){
					// NOTE:
					//	shortcutting the return at this step in indexing works
					//	very well for benchmarking but we avoid it here since
					//	it leads to potential O(n^2) behavior in sequential
					//	getNodexIndex operations on a previously un-indexed
					//	parent. We may revisit this at a later time, but for
					//	now we just want to get the right answer more often
					//	than not.
					ci = i;
				}
			}
		}
		return ci;
	};

	var isEven = function(elem){
		return !((getNodeIndex(elem)) % 2);
	};

	var isOdd = function(elem){
		return ((getNodeIndex(elem)) % 2);
	};

	var pseudos = {
		"checked": function(name, condition){
			return function(elem){
				return !!("checked" in elem ? elem.checked : elem.selected);
			};
		},
		"disabled": function(name, condition){
			return function(elem){
				return elem.disabled;
			};
		},
		"enabled": function(name, condition){
			return function(elem){
				return !elem.disabled;
			};
		},
		"first-child": function(){ return _lookLeft; },
		"last-child": function(){ return _lookRight; },
		"only-child": function(name, condition){
			return function(node){
				return _lookLeft(node) && _lookRight(node);
			};
		},
		"empty": function(name, condition){
			return function(elem){
				// DomQuery and jQuery get this wrong, oddly enough.
				// The CSS 3 selectors spec is pretty explicit about it, too.
				var cn = elem.childNodes;
				var cnl = elem.childNodes.length;
				// if(!cnl){ return true; }
				for(var x=cnl-1; x >= 0; x--){
					var nt = cn[x].nodeType;
					if((nt === 1)||(nt == 3)){ return false; }
				}
				return true;
			};
		},
		"contains": function(name, condition){
			var cz = condition.charAt(0);
			if( cz == '"' || cz == "'" ){ //remove quote
				condition = condition.slice(1, -1);
			}
			return function(elem){
				return (elem.innerHTML.indexOf(condition) >= 0);
			};
		},
		"not": function(name, condition){
			var p = getQueryParts(condition)[0];
			var ignores = { el: 1 };
			if(p.tag != "*"){
				ignores.tag = 1;
			}
			if(!p.classes.length){
				ignores.classes = 1;
			}
			var ntf = getSimpleFilterFunc(p, ignores);
			return function(elem){
				return (!ntf(elem));
			};
		},
		"nth-child": function(name, condition){
			var pi = parseInt;
			// avoid re-defining function objects if we can
			if(condition == "odd"){
				return isOdd;
			}else if(condition == "even"){
				return isEven;
			}
			// FIXME: can we shorten this?
			if(condition.indexOf("n") != -1){
				var tparts = condition.split("n", 2);
				var pred = tparts[0] ? ((tparts[0] == '-') ? -1 : pi(tparts[0])) : 1;
				var idx = tparts[1] ? pi(tparts[1]) : 0;
				var lb = 0, ub = -1;
				if(pred > 0){
					if(idx < 0){
						idx = (idx % pred) && (pred + (idx % pred));
					}else if(idx>0){
						if(idx >= pred){
							lb = idx - idx % pred;
						}
						idx = idx % pred;
					}
				}else if(pred<0){
					pred *= -1;
					// idx has to be greater than 0 when pred is negative;
					// shall we throw an error here?
					if(idx > 0){
						ub = idx;
						idx = idx % pred;
					}
				}
				if(pred > 0){
					return function(elem){
						var i = getNodeIndex(elem);
						return (i>=lb) && (ub<0 || i<=ub) && ((i % pred) == idx);
					};
				}else{
					condition = idx;
				}
			}
			var ncount = pi(condition);
			return function(elem){
				return (getNodeIndex(elem) == ncount);
			};
		}
	};

	var defaultGetter = (has("ie") < 9 || has("ie") == 9 && has("quirks")) ? function(cond){
		var clc = cond.toLowerCase();
		if(clc == "class"){ cond = "className"; }
		return function(elem){
			return (caseSensitive ? elem.getAttribute(cond) : elem[cond]||elem[clc]);
		};
	} : function(cond){
		return function(elem){
			return (elem && elem.getAttribute && elem.hasAttribute(cond));
		};
	};

	var getSimpleFilterFunc = function(query, ignores){
		// generates a node tester function based on the passed query part. The
		// query part is one of the structures generated by the query parser
		// when it creates the query AST. The "ignores" object specifies which
		// (if any) tests to skip, allowing the system to avoid duplicating
		// work where it may have already been taken into account by other
		// factors such as how the nodes to test were fetched in the first
		// place
		if(!query){ return yesman; }
		ignores = ignores||{};

		var ff = null;

		if(!("el" in ignores)){
			ff = agree(ff, _isElement);
		}

		if(!("tag" in ignores)){
			if(query.tag != "*"){
				ff = agree(ff, function(elem){
					return (elem && ((caseSensitive ? elem.tagName : elem.tagName.toUpperCase()) == query.getTag()));
				});
			}
		}

		if(!("classes" in ignores)){
			each(query.classes, function(cname, idx, arr){
				// get the class name
				/*
				var isWildcard = cname.charAt(cname.length-1) == "*";
				if(isWildcard){
					cname = cname.substr(0, cname.length-1);
				}
				// I dislike the regex thing, even if memoized in a cache, but it's VERY short
				var re = new RegExp("(?:^|\\s)" + cname + (isWildcard ? ".*" : "") + "(?:\\s|$)");
				*/
				var re = new RegExp("(?:^|\\s)" + cname + "(?:\\s|$)");
				ff = agree(ff, function(elem){
					return re.test(elem.className);
				});
				ff.count = idx;
			});
		}

		if(!("pseudos" in ignores)){
			each(query.pseudos, function(pseudo){
				var pn = pseudo.name;
				if(pseudos[pn]){
					ff = agree(ff, pseudos[pn](pn, pseudo.value));
				}
			});
		}

		if(!("attrs" in ignores)){
			each(query.attrs, function(attr){
				var matcher;
				var a = attr.attr;
				// type, attr, matchFor
				if(attr.type && attrs[attr.type]){
					matcher = attrs[attr.type](a, attr.matchFor);
				}else if(a.length){
					matcher = defaultGetter(a);
				}
				if(matcher){
					ff = agree(ff, matcher);
				}
			});
		}

		if(!("id" in ignores)){
			if(query.id){
				ff = agree(ff, function(elem){
					return (!!elem && (elem.id == query.id));
				});
			}
		}

		if(!ff){
			if(!("default" in ignores)){
				ff = yesman;
			}
		}
		return ff;
	};

	var _nextSibling = function(filterFunc){
		return function(node, ret, bag){
			while(node = node[_ns]){
				if(_noNES && (!_isElement(node))){ continue; }
				if(
					(!bag || _isUnique(node, bag)) &&
					filterFunc(node)
				){
					ret.push(node);
				}
				break;
			}
			return ret;
		};
	};

	var _nextSiblings = function(filterFunc){
		return function(root, ret, bag){
			var te = root[_ns];
			while(te){
				if(_simpleNodeTest(te)){
					if(bag && !_isUnique(te, bag)){
						break;
					}
					if(filterFunc(te)){
						ret.push(te);
					}
				}
				te = te[_ns];
			}
			return ret;
		};
	};

	// get an array of child *elements*, skipping text and comment nodes
	var _childElements = function(filterFunc){
		filterFunc = filterFunc||yesman;
		return function(root, ret, bag){
			// get an array of child elements, skipping text and comment nodes
			var te, x = 0, tret = root.children || root.childNodes;
			while(te = tret[x++]){
				if(
					_simpleNodeTest(te) &&
					(!bag || _isUnique(te, bag)) &&
					(filterFunc(te, x))
				){
					ret.push(te);
				}
			}
			return ret;
		};
	};

	// test to see if node is below root
	var _isDescendant = function(node, root){
		var pn = node.parentNode;
		while(pn){
			if(pn == root){
				break;
			}
			pn = pn.parentNode;
		}
		return !!pn;
	};

	var _getElementsFuncCache = {};

	var getElementsFunc = function(query){
		var retFunc = _getElementsFuncCache[query.query];
		// if we've got a cached dispatcher, just use that
		if(retFunc){ return retFunc; }
		// else, generate a new on

		// NOTE:
		//		this function returns a function that searches for nodes and
		//		filters them.  The search may be specialized by infix operators
		//		(">", "~", or "+") else it will default to searching all
		//		descendants (the " " selector). Once a group of children is
		//		found, a test function is applied to weed out the ones we
		//		don't want. Many common cases can be fast-pathed. We spend a
		//		lot of cycles to create a dispatcher that doesn't do more work
		//		than necessary at any point since, unlike this function, the
		//		dispatchers will be called every time. The logic of generating
		//		efficient dispatchers looks like this in pseudo code:
		//
		//		# if it's a purely descendant query (no ">", "+", or "~" modifiers)
		//		if infixOperator == " ":
		//			if only(id):
		//				return def(root):
		//					return d.byId(id, root);
		//
		//			elif id:
		//				return def(root):
		//					return filter(d.byId(id, root));
		//
		//			elif cssClass && getElementsByClassName:
		//				return def(root):
		//					return filter(root.getElementsByClassName(cssClass));
		//
		//			elif only(tag):
		//				return def(root):
		//					return root.getElementsByTagName(tagName);
		//
		//			else:
		//				# search by tag name, then filter
		//				return def(root):
		//					return filter(root.getElementsByTagName(tagName||"*"));
		//
		//		elif infixOperator == ">":
		//			# search direct children
		//			return def(root):
		//				return filter(root.children);
		//
		//		elif infixOperator == "+":
		//			# search next sibling
		//			return def(root):
		//				return filter(root.nextElementSibling);
		//
		//		elif infixOperator == "~":
		//			# search rightward siblings
		//			return def(root):
		//				return filter(nextSiblings(root));

		var io = query.infixOper;
		var oper = (io ? io.oper : "");
		// the default filter func which tests for all conditions in the query
		// part. This is potentially inefficient, so some optimized paths may
		// re-define it to test fewer things.
		var filterFunc = getSimpleFilterFunc(query, { el: 1 });
		var qt = query.tag;
		var wildcardTag = ("*" == qt);
		var ecs = getDoc()["getElementsByClassName"];

		if(!oper){
			// if there's no infix operator, then it's a descendant query. ID
			// and "elements by class name" variants can be accelerated so we
			// call them out explicitly:
			if(query.id){
				// testing shows that the overhead of yesman() is acceptable
				// and can save us some bytes vs. re-defining the function
				// everywhere.
				filterFunc = (!query.loops && wildcardTag) ?
					yesman :
					getSimpleFilterFunc(query, { el: 1, id: 1 });

				retFunc = function(root, arr){
					var te = dom.byId(query.id, (root.ownerDocument||root));
					if(!te || !filterFunc(te)){ return; }
					if(9 == root.nodeType){ // if root's a doc, we just return directly
						return getArr(te, arr);
					}else{ // otherwise check ancestry
						if(_isDescendant(te, root)){
							return getArr(te, arr);
						}
					}
				};
			}else if(
				ecs &&
				// isAlien check. Workaround for Prototype.js being totally evil/dumb.
				/\{\s*\[native code\]\s*\}/.test(String(ecs)) &&
				query.classes.length &&
				!cssCaseBug
			){
				// it's a class-based query and we've got a fast way to run it.

				// ignore class and ID filters since we will have handled both
				filterFunc = getSimpleFilterFunc(query, { el: 1, classes: 1, id: 1 });
				var classesString = query.classes.join(" ");
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tret = root.getElementsByClassName(classesString);
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};

			}else if(!wildcardTag && !query.loops){
				// it's tag only. Fast-path it.
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tag = query.getTag(),
						tret = tag ? root.getElementsByTagName(tag) : [];
					while((te = tret[x++])){
						if(_isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}else{
				// the common case:
				//		a descendant selector without a fast path. By now it's got
				//		to have a tag selector, even if it's just "*" so we query
				//		by that and filter
				filterFunc = getSimpleFilterFunc(query, { el: 1, tag: 1, id: 1 });
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					// we use getTag() to avoid case sensitivity issues
					var tag = query.getTag(),
						tret = tag ? root.getElementsByTagName(tag) : [];
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}
		}else{
			// the query is scoped in some way. Instead of querying by tag we
			// use some other collection to find candidate nodes
			var skipFilters = { el: 1 };
			if(wildcardTag){
				skipFilters.tag = 1;
			}
			filterFunc = getSimpleFilterFunc(query, skipFilters);
			if("+" == oper){
				retFunc = _nextSibling(filterFunc);
			}else if("~" == oper){
				retFunc = _nextSiblings(filterFunc);
			}else if(">" == oper){
				retFunc = _childElements(filterFunc);
			}
		}
		// cache it and return
		return _getElementsFuncCache[query.query] = retFunc;
	};

	var filterDown = function(root, queryParts){
		// NOTE:
		//		this is the guts of the DOM query system. It takes a list of
		//		parsed query parts and a root and finds children which match
		//		the selector represented by the parts
		var candidates = getArr(root), qp, x, te, qpl = queryParts.length, bag, ret;

		for(var i = 0; i < qpl; i++){
			ret = [];
			qp = queryParts[i];
			x = candidates.length - 1;
			if(x > 0){
				// if we have more than one root at this level, provide a new
				// hash to use for checking group membership but tell the
				// system not to post-filter us since we will already have been
				// guaranteed to be unique
				bag = {};
				ret.nozip = true;
			}
			var gef = getElementsFunc(qp);
			for(var j = 0; (te = candidates[j]); j++){
				// for every root, get the elements that match the descendant
				// selector, adding them to the "ret" array and filtering them
				// via membership in this level's bag. If there are more query
				// parts, then this level's return will be used as the next
				// level's candidates
				gef(te, ret, bag);
			}
			if(!ret.length){ break; }
			candidates = ret;
		}
		return ret;
	};

	////////////////////////////////////////////////////////////////////////
	// the query runner
	////////////////////////////////////////////////////////////////////////

	// these are the primary caches for full-query results. The query
	// dispatcher functions are generated then stored here for hash lookup in
	// the future
	var _queryFuncCacheDOM = {},
		_queryFuncCacheQSA = {};

	// this is the second level of splitting, from full-length queries (e.g.,
	// "div.foo .bar") into simple query expressions (e.g., ["div.foo",
	// ".bar"])
	var getStepQueryFunc = function(query){
		var qparts = getQueryParts(trim(query));

		// if it's trivial, avoid iteration and zipping costs
		if(qparts.length == 1){
			// we optimize this case here to prevent dispatch further down the
			// chain, potentially slowing things down. We could more elegantly
			// handle this in filterDown(), but it's slower for simple things
			// that need to be fast (e.g., "#someId").
			var tef = getElementsFunc(qparts[0]);
			return function(root){
				var r = tef(root, []);
				if(r){ r.nozip = true; }
				return r;
			};
		}

		// otherwise, break it up and return a runner that iterates over the parts recursively
		return function(root){
			return filterDown(root, qparts);
		};
	};

	// NOTES:
	//	* we can't trust QSA for anything but document-rooted queries, so
	//	  caching is split into DOM query evaluators and QSA query evaluators
	//	* caching query results is dirty and leak-prone (or, at a minimum,
	//	  prone to unbounded growth). Other toolkits may go this route, but
	//	  they totally destroy their own ability to manage their memory
	//	  footprint. If we implement it, it should only ever be with a fixed
	//	  total element reference # limit and an LRU-style algorithm since JS
	//	  has no weakref support. Caching compiled query evaluators is also
	//	  potentially problematic, but even on large documents the size of the
	//	  query evaluators is often < 100 function objects per evaluator (and
	//	  LRU can be applied if it's ever shown to be an issue).
	//	* since IE's QSA support is currently only for HTML documents and even
	//	  then only in IE 8's "standards mode", we have to detect our dispatch
	//	  route at query time and keep 2 separate caches. Ugg.

	// we need to determine if we think we can run a given query via
	// querySelectorAll or if we'll need to fall back on DOM queries to get
	// there. We need a lot of information about the environment and the query
	// to make the determination (e.g. does it support QSA, does the query in
	// question work in the native QSA impl, etc.).

	// IE QSA queries may incorrectly include comment nodes, so we throw the
	// zipping function into "remove" comments mode instead of the normal "skip
	// it" which every other QSA-clued browser enjoys
	var noZip = has("ie") ? "commentStrip" : "nozip";

	var qsa = "querySelectorAll";
	var qsaAvail = !!getDoc()[qsa];

	//Don't bother with n+3 type of matches, IE complains if we modify those.
	var infixSpaceRe = /\\[>~+]|n\+\d|([^ \\])?([>~+])([^ =])?/g;
	var infixSpaceFunc = function(match, pre, ch, post){
		return ch ? (pre ? pre + " " : "") + ch + (post ? " " + post : "") : /*n+3*/ match;
	};
	
	//Don't apply the infixSpaceRe to attribute value selectors
	var attRe = /([^[]*)([^\]]*])?/g;
	var attFunc = function(match, nonAtt, att){
		return nonAtt.replace(infixSpaceRe, infixSpaceFunc) + (att||"");
	};
	var getQueryFunc = function(query, forceDOM){
		//Normalize query. The CSS3 selectors spec allows for omitting spaces around
		//infix operators, >, ~ and +
		//Do the work here since detection for spaces is used as a simple "not use QSA"
		//test below.
		query = query.replace(attRe, attFunc);

		if(qsaAvail){
			// if we've got a cached variant and we think we can do it, run it!
			var qsaCached = _queryFuncCacheQSA[query];
			if(qsaCached && !forceDOM){ return qsaCached; }
		}

		// else if we've got a DOM cached variant, assume that we already know
		// all we need to and use it
		var domCached = _queryFuncCacheDOM[query];
		if(domCached){ return domCached; }

		// TODO:
		//		today we're caching DOM and QSA branches separately so we
		//		recalc useQSA every time. If we had a way to tag root+query
		//		efficiently, we'd be in good shape to do a global cache.

		var qcz = query.charAt(0);
		var nospace = (-1 == query.indexOf(" "));

		// byId searches are wicked fast compared to QSA, even when filtering
		// is required
		if( (query.indexOf("#") >= 0) && (nospace) ){
			forceDOM = true;
		}

		var useQSA = (
			qsaAvail && (!forceDOM) &&
			// as per CSS 3, we can't currently start w/ combinator:
			//		http://www.w3.org/TR/css3-selectors/#w3cselgrammar
			(specials.indexOf(qcz) == -1) &&
			// IE's QSA impl sucks on pseudos
			(!has("ie") || (query.indexOf(":") == -1)) &&

			(!(cssCaseBug && (query.indexOf(".") >= 0))) &&

			// FIXME:
			//		need to tighten up browser rules on ":contains" and "|=" to
			//		figure out which aren't good
			//		Latest webkit (around 531.21.8) does not seem to do well with :checked on option
			//		elements, even though according to spec, selected options should
			//		match :checked. So go nonQSA for it:
			//		http://bugs.dojotoolkit.org/ticket/5179
			(query.indexOf(":contains") == -1) && (query.indexOf(":checked") == -1) &&
			(query.indexOf("|=") == -1) // some browsers don't grok it
		);

		// TODO:
		//		if we've got a descendant query (e.g., "> .thinger" instead of
		//		just ".thinger") in a QSA-able doc, but are passed a child as a
		//		root, it should be possible to give the item a synthetic ID and
		//		trivially rewrite the query to the form "#synid > .thinger" to
		//		use the QSA branch


		if(useQSA){
			var tq = (specials.indexOf(query.charAt(query.length-1)) >= 0) ?
						(query + " *") : query;
			return _queryFuncCacheQSA[query] = function(root){
				try{
					// the QSA system contains an egregious spec bug which
					// limits us, effectively, to only running QSA queries over
					// entire documents.  See:
					//		http://ejohn.org/blog/thoughts-on-queryselectorall/
					//	despite this, we can also handle QSA runs on simple
					//	selectors, but we don't want detection to be expensive
					//	so we're just checking for the presence of a space char
					//	right now. Not elegant, but it's cheaper than running
					//	the query parser when we might not need to
					if(!((9 == root.nodeType) || nospace)){ throw ""; }
					var r = root[qsa](tq);
					// skip expensive duplication checks and just wrap in a NodeList
					r[noZip] = true;
					return r;
				}catch(e){
					// else run the DOM branch on this query, ensuring that we
					// default that way in the future
					return getQueryFunc(query, true)(root);
				}
			};
		}else{
			// DOM branch
			var parts = query.match(/([^\s,](?:"(?:\\.|[^"])+"|'(?:\\.|[^'])+'|[^,])*)/g);
			return _queryFuncCacheDOM[query] = ((parts.length < 2) ?
				// if not a compound query (e.g., ".foo, .bar"), cache and return a dispatcher
				getStepQueryFunc(query) :
				// if it *is* a complex query, break it up into its
				// constituent parts and return a dispatcher that will
				// merge the parts when run
				function(root){
					var pindex = 0, // avoid array alloc for every invocation
						ret = [],
						tp;
					while((tp = parts[pindex++])){
						ret = ret.concat(getStepQueryFunc(tp)(root));
					}
					return ret;
				}
			);
		}
	};

	var _zipIdx = 0;

	// NOTE:
	//		this function is Moo inspired, but our own impl to deal correctly
	//		with XML in IE
	var _nodeUID = has("ie") ? function(node){
		if(caseSensitive){
			// XML docs don't have uniqueID on their nodes
			return (node.getAttribute("_uid") || node.setAttribute("_uid", ++_zipIdx) || _zipIdx);

		}else{
			return node.uniqueID;
		}
	} :
	function(node){
		return (node._uid || (node._uid = ++_zipIdx));
	};

	// determine if a node in is unique in a "bag". In this case we don't want
	// to flatten a list of unique items, but rather just tell if the item in
	// question is already in the bag. Normally we'd just use hash lookup to do
	// this for us but IE's DOM is busted so we can't really count on that. On
	// the upside, it gives us a built in unique ID function.
	var _isUnique = function(node, bag){
		if(!bag){ return 1; }
		var id = _nodeUID(node);
		if(!bag[id]){ return bag[id] = 1; }
		return 0;
	};

	// attempt to efficiently determine if an item in a list is a dupe,
	// returning a list of "uniques", hopefully in document order
	var _zipIdxName = "_zipIdx";
	var _zip = function(arr){
		if(arr && arr.nozip){ return arr; }

		if(!arr || !arr.length){ return []; }
		if(arr.length < 2){ return [arr[0]]; }

		var ret = [];

		_zipIdx++;

		// we have to fork here for IE and XML docs because we can't set
		// expandos on their nodes (apparently). *sigh*
		var x, te;
		if(has("ie") && caseSensitive){
			var szidx = _zipIdx+"";
			for(x = 0; x < arr.length; x++){
				if((te = arr[x]) && te.getAttribute(_zipIdxName) != szidx){
					ret.push(te);
					te.setAttribute(_zipIdxName, szidx);
				}
			}
		}else if(has("ie") && arr.commentStrip){
			try{
				for(x = 0; x < arr.length; x++){
					if((te = arr[x]) && _isElement(te)){
						ret.push(te);
					}
				}
			}catch(e){ /* squelch */ }
		}else{
			for(x = 0; x < arr.length; x++){
				if((te = arr[x]) && te[_zipIdxName] != _zipIdx){
					ret.push(te);
					te[_zipIdxName] = _zipIdx;
				}
			}
		}
		return ret;
	};

	// the main executor
	var query = function(/*String*/ query, /*String|DOMNode?*/ root){
		// summary:
		//		Returns nodes which match the given CSS3 selector, searching the
		//		entire document by default but optionally taking a node to scope
		//		the search by. Returns an array.
		// description:
		//		dojo.query() is the swiss army knife of DOM node manipulation in
		//		Dojo. Much like Prototype's "$$" (bling-bling) function or JQuery's
		//		"$" function, dojo.query provides robust, high-performance
		//		CSS-based node selector support with the option of scoping searches
		//		to a particular sub-tree of a document.
		//
		//		Supported Selectors:
		//		--------------------
		//
		//		acme supports a rich set of CSS3 selectors, including:
		//
		//		- class selectors (e.g., `.foo`)
		//		- node type selectors like `span`
		//		- ` ` descendant selectors
		//		- `>` child element selectors
		//		- `#foo` style ID selectors
		//		- `*` universal selector
		//		- `~`, the preceded-by sibling selector
		//		- `+`, the immediately preceded-by sibling selector
		//		- attribute queries:
		//			- `[foo]` attribute presence selector
		//			- `[foo='bar']` attribute value exact match
		//			- `[foo~='bar']` attribute value list item match
		//			- `[foo^='bar']` attribute start match
		//			- `[foo$='bar']` attribute end match
		//			- `[foo*='bar']` attribute substring match
		//		- `:first-child`, `:last-child`, and `:only-child` positional selectors
		//		- `:empty` content emtpy selector
		//		- `:checked` pseudo selector
		//		- `:nth-child(n)`, `:nth-child(2n+1)` style positional calculations
		//		- `:nth-child(even)`, `:nth-child(odd)` positional selectors
		//		- `:not(...)` negation pseudo selectors
		//
		//		Any legal combination of these selectors will work with
		//		`dojo.query()`, including compound selectors ("," delimited).
		//		Very complex and useful searches can be constructed with this
		//		palette of selectors and when combined with functions for
		//		manipulation presented by dojo/NodeList, many types of DOM
		//		manipulation operations become very straightforward.
		//
		//		Unsupported Selectors:
		//		----------------------
		//
		//		While dojo.query handles many CSS3 selectors, some fall outside of
		//		what's reasonable for a programmatic node querying engine to
		//		handle. Currently unsupported selectors include:
		//
		//		- namespace-differentiated selectors of any form
		//		- all `::` pseduo-element selectors
		//		- certain pseudo-selectors which don't get a lot of day-to-day use:
		//			- `:root`, `:lang()`, `:target`, `:focus`
		//		- all visual and state selectors:
		//			- `:root`, `:active`, `:hover`, `:visited`, `:link`,
		//				  `:enabled`, `:disabled`
		//			- `:*-of-type` pseudo selectors
		//
		//		dojo.query and XML Documents:
		//		-----------------------------
		//
		//		`dojo.query` (as of dojo 1.2) supports searching XML documents
		//		in a case-sensitive manner. If an HTML document is served with
		//		a doctype that forces case-sensitivity (e.g., XHTML 1.1
		//		Strict), dojo.query() will detect this and "do the right
		//		thing". Case sensitivity is dependent upon the document being
		//		searched and not the query used. It is therefore possible to
		//		use case-sensitive queries on strict sub-documents (iframes,
		//		etc.) or XML documents while still assuming case-insensitivity
		//		for a host/root document.
		//
		//		Non-selector Queries:
		//		---------------------
		//
		//		If something other than a String is passed for the query,
		//		`dojo.query` will return a new `dojo/NodeList` instance
		//		constructed from that parameter alone and all further
		//		processing will stop. This means that if you have a reference
		//		to a node or NodeList, you can quickly construct a new NodeList
		//		from the original by calling `dojo.query(node)` or
		//		`dojo.query(list)`.
		//
		// query:
		//		The CSS3 expression to match against. For details on the syntax of
		//		CSS3 selectors, see <http://www.w3.org/TR/css3-selectors/#selectors>
		// root:
		//		A DOMNode (or node id) to scope the search from. Optional.
		// returns: Array
		// example:
		//		search the entire document for elements with the class "foo":
		//	|	dojo.query(".foo");
		//		these elements will match:
		//	|	<span class="foo"></span>
		//	|	<span class="foo bar"></span>
		//	|	<p class="thud foo"></p>
		// example:
		//		search the entire document for elements with the classes "foo" *and* "bar":
		//	|	dojo.query(".foo.bar");
		//		these elements will match:
		//	|	<span class="foo bar"></span>
		//		while these will not:
		//	|	<span class="foo"></span>
		//	|	<p class="thud foo"></p>
		// example:
		//		find `<span>` elements which are descendants of paragraphs and
		//		which have a "highlighted" class:
		//	|	dojo.query("p span.highlighted");
		//		the innermost span in this fragment matches:
		//	|	<p class="foo">
		//	|		<span>...
		//	|			<span class="highlighted foo bar">...</span>
		//	|		</span>
		//	|	</p>
		// example:
		//		set an "odd" class on all odd table rows inside of the table
		//		`#tabular_data`, using the `>` (direct child) selector to avoid
		//		affecting any nested tables:
		//	|	dojo.query("#tabular_data > tbody > tr:nth-child(odd)").addClass("odd");
		// example:
		//		remove all elements with the class "error" from the document
		//		and store them in a list:
		//	|	var errors = dojo.query(".error").orphan();
		// example:
		//		add an onclick handler to every submit button in the document
		//		which causes the form to be sent via Ajax instead:
		//	|	dojo.query("input[type='submit']").onclick(function(e){
		//	|		dojo.stopEvent(e); // prevent sending the form
		//	|		var btn = e.target;
		//	|		dojo.xhrPost({
		//	|			form: btn.form,
		//	|			load: function(data){
		//	|				// replace the form with the response
		//	|				var div = dojo.doc.createElement("div");
		//	|				dojo.place(div, btn.form, "after");
		//	|				div.innerHTML = data;
		//	|				dojo.style(btn.form, "display", "none");
		//	|			}
		//	|		});
		//	|	});

		root = root || getDoc();

		// throw the big case sensitivity switch
		var od = root.ownerDocument || root;	// root is either Document or a node inside the document
		caseSensitive = (od.createElement("div").tagName === "div");

		// NOTE:
		//		adding "true" as the 2nd argument to getQueryFunc is useful for
		//		testing the DOM branch without worrying about the
		//		behavior/performance of the QSA branch.
		var r = getQueryFunc(query)(root);

		// FIXME:
		//		need to investigate this branch WRT #8074 and #8075
		if(r && r.nozip){
			return r;
		}
		return _zip(r); // dojo/NodeList
	};
	query.filter = function(/*Node[]*/ nodeList, /*String*/ filter, /*String|DOMNode?*/ root){
		// summary:
		//		function for filtering a NodeList based on a selector, optimized for simple selectors
		var tmpNodeList = [],
			parts = getQueryParts(filter),
			filterFunc =
				(parts.length == 1 && !/[^\w#\.]/.test(filter)) ?
				getSimpleFilterFunc(parts[0]) :
				function(node){
					return array.indexOf(query(filter, dom.byId(root)), node) != -1;
				};
		for(var x = 0, te; te = nodeList[x]; x++){
			if(filterFunc(te)){ tmpNodeList.push(te); }
		}
		return tmpNodeList;
	};
	return query;
});

},
'xcitec/grid/HGridBottomPane':function(){
define([
	"dojo",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dijit/_Contained",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (dojo, declare, domClass, _Contained, _Container, _Templated, _Widget) {

	var HGridBottomPane = declare(
				[_Widget, _Templated, _Container],
				{
					templateString: '<div class="dijit" data-dojo-attach-point="containerNode"></div>'
				}
	)
	return HGridBottomPane;
});

},
'dijit/main':function(){
define([
	"dojo/_base/kernel"
], function(dojo){
	// module:
	//		dijit/main

/*=====
return {
	// summary:
	//		The dijit package main module.
	//		Deprecated.   Users should access individual modules (ex: dijit/registry) directly.
};
=====*/

	return dojo.dijit;
});

},
'xcitec/form/LinkButton':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/dom",
	"../_StateWidget"
], function (declare, windowModule, dom, _StateWidget) {

	var LinkButton = declare( _StateWidget, {
		text: "",
		caption: "",
		captionId: "",
		disabled: false,
		readOnly: false,
		baseClass: "linkButton",
		templateString: '<span class=${baseClass} data-dojo-attach-event="onclick:_onLinkClick, onmouseover:_onMouse, onmouseout:_onMouse, onmousedown:_onMouse, onmouseup:_onMouse" data-dojo-attach-point="containerNode"></span>',
	
		_fillContent: function(/*DomNode*/ source){
			this.caption = this.caption || this.text;
			if(this.caption){
				this.containerNode.appendChild(windowModule.doc.createTextNode(this.caption));
				return;
			}
			this.inherited(arguments);
		},
	
		postCreate: function(){
		dom.setSelectable(this.domNode, false);
			this.inherited(arguments);
		},
	
		onClick: function(/*Event*/ e){
			//Internal overridable function for when the button is clicked
		},
	
		_onLinkClick: function(/*Event*/ e){
			if(!this.readOnly && !this.disabled){
				this.onClick(e);
				e.preventDefault();
			}
		}
	});
	return LinkButton;
});

},
'dijit/layout/ContentPane':function(){
define([
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin lang.delegate lang.hitch lang.isFunction lang.isObject
	"../_Widget",
	"../_Container",
	"./_ContentPaneResizeMixin",
	"dojo/string", // string.substitute
	"dojo/html", // html._ContentSetter
	"dojo/i18n!../nls/loading",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.attr
	"dojo/dom-construct", // empty()
	"dojo/_base/xhr", // xhr.get
	"dojo/i18n", // i18n.getLocalization
	"dojo/when"
], function(kernel, lang, _Widget, _Container, _ContentPaneResizeMixin, string, html, nlsLoading, array, declare,
			Deferred, dom, domAttr, domConstruct, xhr, i18n, when){

	// module:
	//		dijit/layout/ContentPane

	return declare("dijit.layout.ContentPane", [_Widget, _Container, _ContentPaneResizeMixin], {
		// summary:
		//		A widget containing an HTML fragment, specified inline
		//		or by uri.  Fragment may include widgets.
		//
		// description:
		//		This widget embeds a document fragment in the page, specified
		//		either by uri, javascript generated markup or DOM reference.
		//		Any widgets within this content are instantiated and managed,
		//		but laid out according to the HTML structure.  Unlike IFRAME,
		//		ContentPane embeds a document fragment as would be found
		//		inside the BODY tag of a full HTML document.  It should not
		//		contain the HTML, HEAD, or BODY tags.
		//		For more advanced functionality with scripts and
		//		stylesheets, see dojox/layout/ContentPane.  This widget may be
		//		used stand alone or as a base class for other widgets.
		//		ContentPane is useful as a child of other layout containers
		//		such as BorderContainer or TabContainer, but note that those
		//		widgets can contain any widget as a child.
		//
		// example:
		//		Some quick samples:
		//		To change the innerHTML:
		// |		cp.set('content', '<b>new content</b>')`
		//		Or you can send it a NodeList:
		// |		cp.set('content', dojo.query('div [class=selected]', userSelection))
		//		To do an ajax update:
		// |		cp.set('href', url)

		// href: String
		//		The href of the content that displays now.
		//		Set this at construction if you want to load data externally when the
		//		pane is shown.  (Set preload=true to load it immediately.)
		//		Changing href after creation doesn't have any effect; Use set('href', ...);
		href: "",

		// content: String|DomNode|NodeList|dijit/_Widget
		//		The innerHTML of the ContentPane.
		//		Note that the initialization parameter / argument to set("content", ...)
		//		can be a String, DomNode, Nodelist, or _Widget.
		content: "",

		// extractContent: Boolean
		//		Extract visible content from inside of `<body> .... </body>`.
		//		I.e., strip `<html>` and `<head>` (and it's contents) from the href
		extractContent: false,

		// parseOnLoad: Boolean
		//		Parse content and create the widgets, if any.
		parseOnLoad: true,

		// parserScope: String
		//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
		//		will search for data-dojo-type (or dojoType).  For backwards compatibility
		//		reasons defaults to dojo._scopeName (which is "dojo" except when
		//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		parserScope: kernel._scopeName,

		// preventCache: Boolean
		//		Prevent caching of data from href's by appending a timestamp to the href.
		preventCache: false,

		// preload: Boolean
		//		Force load of data on initialization even if pane is hidden.
		preload: false,

		// refreshOnShow: Boolean
		//		Refresh (re-download) content when pane goes from hidden to shown
		refreshOnShow: false,

		// loadingMessage: String
		//		Message that shows while downloading
		loadingMessage: "<span class='dijitContentPaneLoading'><span class='dijitInline dijitIconLoading'></span>${loadingState}</span>",

		// errorMessage: String
		//		Message that shows if an error occurs
		errorMessage: "<span class='dijitContentPaneError'><span class='dijitInline dijitIconError'></span>${errorState}</span>",

		// isLoaded: [readonly] Boolean
		//		True if the ContentPane has data in it, either specified
		//		during initialization (via href or inline content), or set
		//		via set('content', ...) / set('href', ...)
		//
		//		False if it doesn't have any content, or if ContentPane is
		//		still in the process of downloading href.
		isLoaded: false,

		baseClass: "dijitContentPane",

		/*======
		 // ioMethod: dojo/_base/xhr.get|dojo._base/xhr.post
		 //		Function that should grab the content specified via href.
		 ioMethod: dojo.xhrGet,
		 ======*/

		// ioArgs: Object
		//		Parameters to pass to xhrGet() request, for example:
		// |	<div data-dojo-type="dijit/layout/ContentPane" data-dojo-props="href: './bar', ioArgs: {timeout: 500}">
		ioArgs: {},

		// onLoadDeferred: [readonly] dojo.Deferred
		//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
		//		Calling onLoadDeferred.then() registers your
		//		callback to be called only once, when the prior set('href', ...) call or
		//		the initial href parameter to the constructor finishes loading.
		//
		//		This is different than an onLoad() handler which gets called any time any href
		//		or content is loaded.
		onLoadDeferred: null,

		// Cancel _WidgetBase's _setTitleAttr because we don't want the title attribute (used to specify
		// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
		// entire pane.
		_setTitleAttr: null,

		// Flag to parser that I'll parse my contents, so it shouldn't.
		stopParser: true,

		// template: [private] Boolean
		//		Flag from the parser that this ContentPane is inside a template
		//		so the contents are pre-parsed.
		// TODO: this declaration can be commented out in 2.0
		template: false,

		markupFactory: function(params, node, ctor){
			var self = new ctor(params, node);

			// If a parse has started but is waiting for modules to load, then return a Promise for when the parser
			// finishes.  Don't return a promise though for the case when content hasn't started loading because the
			// ContentPane is hidden and it has an href (ex: hidden pane of a TabContainer).   In that case we consider
			// that initialization has already finished.
			return !self.href && self._contentSetter && self._contentSetter.parseDeferred && !self._contentSetter.parseDeferred.isFulfilled() ?
				self._contentSetter.parseDeferred.then(function(){
					return self;
				}) : self;
		},

		create: function(params, srcNodeRef){
			// Convert a srcNodeRef argument into a content parameter, so that the original contents are
			// processed in the same way as contents set via set("content", ...), calling the parser etc.
			// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
			if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
				srcNodeRef = dom.byId(srcNodeRef);
				var df = srcNodeRef.ownerDocument.createDocumentFragment();
				while(srcNodeRef.firstChild){
					df.appendChild(srcNodeRef.firstChild);
				}
				params = lang.delegate(params, {content: df});
			}
			this.inherited(arguments, [params, srcNodeRef]);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			var messages = i18n.getLocalization("dijit", "loading", this.lang);
			this.loadingMessage = string.substitute(this.loadingMessage, messages);
			this.errorMessage = string.substitute(this.errorMessage, messages);
		},

		buildRendering: function(){
			this.inherited(arguments);

			// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
			// For subclasses of ContentPane that do have a template, does nothing.
			if(!this.containerNode){
				this.containerNode = this.domNode;
			}

			// remove the title attribute so it doesn't show up when hovering
			// over a node  (TODO: remove in 2.0, no longer needed after #11490)
			this.domNode.removeAttribute("title");
		},

		startup: function(){
			// summary:
			//		Call startup() on all children including non _Widget ones like dojo/dnd/Source objects

			// This starts all the widgets
			this.inherited(arguments);

			// And this catches stuff like dojo/dnd/Source
			if(this._contentSetter){
				array.forEach(this._contentSetter.parseResults, function(obj){
					if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
						obj.startup();
						obj._started = true;
					}
				}, this);
			}
		},

		_startChildren: function(){
			// summary:
			//		Called when content is loaded.   Calls startup on each child widget.   Similar to ContentPane.startup()
			//		itself, but avoids marking the ContentPane itself as "restarted" (see #15581).

			// This starts all the widgets
			array.forEach(this.getChildren(), function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			});

			// And this catches stuff like dojo/dnd/Source
			if(this._contentSetter){
				array.forEach(this._contentSetter.parseResults, function(obj){
					if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
						obj.startup();
						obj._started = true;
					}
				}, this);
			}
		},

		setHref: function(/*String|Uri*/ href){
			// summary:
			//		Deprecated.   Use set('href', ...) instead.
			kernel.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
			return this.set("href", href);
		},
		_setHrefAttr: function(/*String|Uri*/ href){
			// summary:
			//		Hook so set("href", ...) works.
			// description:
			//		Reset the (external defined) content of this pane and replace with new url
			//		Note: It delays the download until widget is shown if preload is false.
			// href:
			//		url to the page you want to get, must be within the same domain as your mainpage

			// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
			this.cancel();

			this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));

			this._set("href", href);

			// _setHrefAttr() is called during creation and by the user, after creation.
			// Assuming preload == false, only in the second case do we actually load the URL;
			// otherwise it's done in startup(), and only if this widget is shown.
			if(this.preload || (this._created && this._isShown())){
				this._load();
			}else{
				// Set flag to indicate that href needs to be loaded the next time the
				// ContentPane is made visible
				this._hrefChanged = true;
			}

			return this.onLoadDeferred;		// Deferred
		},

		setContent: function(/*String|DomNode|Nodelist*/data){
			// summary:
			//		Deprecated.   Use set('content', ...) instead.
			kernel.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
			this.set("content", data);
		},
		_setContentAttr: function(/*String|DomNode|Nodelist*/data){
			// summary:
			//		Hook to make set("content", ...) work.
			//		Replaces old content with data content, include style classes from old content
			// data:
			//		the new Content may be String, DomNode or NodeList
			//
			//		if data is a NodeList (or an array of nodes) nodes are copied
			//		so you can import nodes from another document implicitly

			// clear href so we can't run refresh and clear content
			// refresh should only work if we downloaded the content
			this._set("href", "");

			// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
			this.cancel();

			// Even though user is just setting content directly, still need to define an onLoadDeferred
			// because the _onLoadHandler() handler is still getting called from setContent()
			this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
			if(this._created){
				// For back-compat reasons, call onLoad() for set('content', ...)
				// calls but not for content specified in srcNodeRef (ie: <div data-dojo-type=ContentPane>...</div>)
				// or as initialization parameter (ie: new ContentPane({content: ...})
				this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
			}

			this._setContent(data || "");

			this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

			return this.onLoadDeferred;	// Deferred
		},
		_getContentAttr: function(){
			// summary:
			//		Hook to make get("content") work
			return this.containerNode.innerHTML;
		},

		cancel: function(){
			// summary:
			//		Cancels an in-flight download of content
			if(this._xhrDfd && (this._xhrDfd.fired == -1)){
				this._xhrDfd.cancel();
			}
			delete this._xhrDfd; // garbage collect

			this.onLoadDeferred = null;
		},

		destroy: function(){
			this.cancel();
			this.inherited(arguments);
		},

		destroyRecursive: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy the ContentPane and its contents

			// if we have multiple controllers destroying us, bail after the first
			if(this._beingDestroyed){
				return;
			}
			this.inherited(arguments);
		},

		_onShow: function(){
			// summary:
			//		Called when the ContentPane is made visible
			// description:
			//		For a plain ContentPane, this is called on initialization, from startup().
			//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
			//		called whenever the pane is made visible.
			//
			//		Does necessary processing, including href download and layout/resize of
			//		child widget(s)

			this.inherited(arguments);

			if(this.href){
				if(!this._xhrDfd && // if there's an href that isn't already being loaded
					(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
					){
					return this.refresh();	// If child has an href, promise that fires when the load is complete
				}
			}
		},

		refresh: function(){
			// summary:
			//		[Re]download contents of href and display
			// description:
			//		1. cancels any currently in-flight requests
			//		2. posts "loading..." message
			//		3. sends XHR to download new data

			// Cancel possible prior in-flight request
			this.cancel();

			this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
			this._load();
			return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
		},

		_load: function(){
			// summary:
			//		Load/reload the href specified in this.href

			// display loading message
			this._setContent(this.onDownloadStart(), true);

			var self = this;
			var getArgs = {
				preventCache: (this.preventCache || this.refreshOnShow),
				url: this.href,
				handleAs: "text"
			};
			if(lang.isObject(this.ioArgs)){
				lang.mixin(getArgs, this.ioArgs);
			}

			var hand = (this._xhrDfd = (this.ioMethod || xhr.get)(getArgs)),
				returnedHtml;

			hand.then(
				function(html){
					returnedHtml = html;
					try{
						self._isDownloaded = true;
						return self._setContent(html, false);
					}catch(err){
						self._onError('Content', err); // onContentError
					}
				},
				function(err){
					if(!hand.canceled){
						// show error message in the pane
						self._onError('Download', err); // onDownloadError
					}
					delete self._xhrDfd;
					return err;
				}
			).then(function(){
					self.onDownloadEnd();
					delete self._xhrDfd;
					return returnedHtml;
				});

			// Remove flag saying that a load is needed
			delete this._hrefChanged;
		},

		_onLoadHandler: function(data){
			// summary:
			//		This is called whenever new content is being loaded
			this._set("isLoaded", true);
			try{
				this.onLoadDeferred.resolve(data);
			}catch(e){
				console.error('Error ' + this.widgetId + ' running custom onLoad code: ' + e.message);
			}
		},

		_onUnloadHandler: function(){
			// summary:
			//		This is called whenever the content is being unloaded
			this._set("isLoaded", false);
			try{
				this.onUnload();
			}catch(e){
				console.error('Error ' + this.widgetId + ' running custom onUnload code: ' + e.message);
			}
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy all the widgets inside the ContentPane and empty containerNode

			// Make sure we call onUnload (but only when the ContentPane has real content)
			if(this.isLoaded){
				this._onUnloadHandler();
			}

			// Even if this.isLoaded == false there might still be a "Loading..." message
			// to erase, so continue...

			// For historical reasons we need to delete all widgets under this.containerNode,
			// even ones that the user has created manually.
			var setter = this._contentSetter;
			array.forEach(this.getChildren(), function(widget){
				if(widget.destroyRecursive){
					// All widgets will hit this branch
					widget.destroyRecursive(preserveDom);
				}else if(widget.destroy){
					// Things like dojo/dnd/Source have destroy(), not destroyRecursive()
					widget.destroy(preserveDom);
				}
				widget._destroyed = true;
			});
			if(setter){
				// Most of the widgets in setter.parseResults have already been destroyed, but
				// things like Menu that have been moved to <body> haven't yet
				array.forEach(setter.parseResults, function(widget){
					if(!widget._destroyed){
						if(widget.destroyRecursive){
							// All widgets will hit this branch
							widget.destroyRecursive(preserveDom);
						}else if(widget.destroy){
							// Things like dojo/dnd/Source have destroy(), not destroyRecursive()
							widget.destroy(preserveDom);
						}
						widget._destroyed = true;
					}
				});
				delete setter.parseResults;
			}

			// And then clear away all the DOM nodes
			if(!preserveDom){
				domConstruct.empty(this.containerNode);
			}

			// Delete any state information we have about current contents
			delete this._singleChild;
		},

		_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
			// summary:
			//		Insert the content into the container node
			// returns:
			//		Returns a Deferred promise that is resolved when the content is parsed.

			// first get rid of child widgets
			this.destroyDescendants();

			// html.set will take care of the rest of the details
			// we provide an override for the error handling to ensure the widget gets the errors
			// configure the setter instance with only the relevant widget instance properties
			// NOTE: unless we hook into attr, or provide property setters for each property,
			// we need to re-configure the ContentSetter with each use
			var setter = this._contentSetter;
			if(!(setter && setter instanceof html._ContentSetter)){
				setter = this._contentSetter = new html._ContentSetter({
					node: this.containerNode,
					_onError: lang.hitch(this, this._onError),
					onContentError: lang.hitch(this, function(e){
						// fires if a domfault occurs when we are appending this.errorMessage
						// like for instance if domNode is a UL and we try append a DIV
						var errMess = this.onContentError(e);
						try{
							this.containerNode.innerHTML = errMess;
						}catch(e){
							console.error('Fatal ' + this.id + ' could not change content due to ' + e.message, e);
						}
					})/*,
					 _onError */
				});
			}

			var setterParams = lang.mixin({
				cleanContent: this.cleanContent,
				extractContent: this.extractContent,
				parseContent: !cont.domNode && this.parseOnLoad,
				parserScope: this.parserScope,
				startup: false,
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir
			}, this._contentSetterParams || {});

			var p = setter.set((lang.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams);

			// dojox/layout/html/_base::_ContentSetter.set() returns a Promise that indicates when everything is completed.
			// dojo/html::_ContentSetter.set() currently returns the DOMNode, but that will be changed for 2.0.
			// So, if set() returns a promise then use it, otherwise fallback to waiting on setter.parseDeferred
			var self = this;
			return when(p && p.then ? p : setter.parseDeferred, function(){
				// setter params must be pulled afresh from the ContentPane each time
				delete self._contentSetterParams;

				if(!isFakeContent){
					if(self._started){
						// Startup each top level child widget (and they will start their children, recursively)
						self._startChildren();

						// Call resize() on each of my child layout widgets,
						// or resize() on my single child layout widget...
						// either now (if I'm currently visible) or when I become visible
						self._scheduleLayout();
					}
					self._onLoadHandler(cont);
				}
			});
		},

		_onError: function(type, err, consoleText){
			this.onLoadDeferred.reject(err);

			// shows user the string that is returned by on[type]Error
			// override on[type]Error and return your own string to customize
			var errText = this['on' + type + 'Error'].call(this, err);
			if(consoleText){
				console.error(consoleText, err);
			}else if(errText){// a empty string won't change current content
				this._setContent(errText, true);
			}
		},

		// EVENT's, should be overide-able
		onLoad: function(/*===== data =====*/){
			// summary:
			//		Event hook, is called after everything is loaded and widgetified
			// tags:
			//		callback
		},

		onUnload: function(){
			// summary:
			//		Event hook, is called before old content is cleared
			// tags:
			//		callback
		},

		onDownloadStart: function(){
			// summary:
			//		Called before download starts.
			// description:
			//		The string returned by this function will be the html
			//		that tells the user we are loading something.
			//		Override with your own function if you want to change text.
			// tags:
			//		extension
			return this.loadingMessage;
		},

		onContentError: function(/*Error*/ /*===== error =====*/){
			// summary:
			//		Called on DOM faults, require faults etc. in content.
			//
			//		In order to display an error message in the pane, return
			//		the error message from this method, as an HTML string.
			//
			//		By default (if this method is not overriden), it returns
			//		nothing, so the error message is just printed to the console.
			// tags:
			//		extension
		},

		onDownloadError: function(/*Error*/ /*===== error =====*/){
			// summary:
			//		Called when download error occurs.
			//
			//		In order to display an error message in the pane, return
			//		the error message from this method, as an HTML string.
			//
			//		Default behavior (if this method is not overriden) is to display
			//		the error message inside the pane.
			// tags:
			//		extension
			return this.errorMessage;
		},

		onDownloadEnd: function(){
			// summary:
			//		Called when download is finished.
			// tags:
			//		callback
		}
	});
});

},
'xcitec/wicket/_ajax':function(){
define(["../xcitecUtil", "dijit/registry"], function (xcitecUtil, registry) {

	function dojoWicketAjaxGet(url, successHandler, failureHandler, precondition, channel) {
	
	    var call = new Wicket.Ajax.Call(url, successHandler, failureHandler, channel);
	    call.processComponent = function(steps, node) {
	        steps.push(function(notify) {
	            // get the component id
	            var compId = node.getAttribute("id");
	            var text="";
	
	            // get the new component body
	            if (node.hasChildNodes()) {
	                text = node.firstChild.nodeValue;
	            }
	
	            // if the text was escaped, unascape it
	            // (escaping is done when the component body contains a CDATA section)
	            var encoding = node.getAttribute("encoding");
	            if (encoding != null && encoding!="") {
	                text = Wicket.decode(encoding, text);
	            }
	
	            var match = text.match(/<div[^>]*>\s*([\s\S]+)\s*<\/div>/im);
	            if(match){ text = match[1]; }
	
	
				// get existing component
	            var element = registry.byId(compId);
	
				if (element == null || typeof(element) == "undefined") {
					Wicket.Log.error("Component with id [["+compId+"]] a was not found while trying to perform markup update. Make sure you called component.setOutputMarkupId(true) on the component whose markup you are trying to update.");
				} else {
					// replace the component
	                element.set("content", text);
	                //element._setContent(text, false);
	
				}
	            // continue to next step
	            notify();
	        });
	    };
	
	    //alert("XCall");
	    if (typeof(precondition) != "undefined" && precondition != null) {
	        call.request.precondition = precondition;
	    }
	    return call.call();
	};
	
	xcitecUtil.dojoWicketAjaxGet = dojoWicketAjaxGet;
});

},
'dijit/_WidgetsInTemplateMixin':function(){
define([
	"dojo/_base/array", // forEach()
	"dojo/aspect", // after()
	"dojo/_base/declare", // declare()
	"dojo/_base/lang",	// hitch()
	"dojo/parser" // parse()
], function(array, aspect, declare, lang, parser){

	// module:
	//		dijit/_WidgetsInTemplateMixin

	return declare("dijit._WidgetsInTemplateMixin", null, {
		// summary:
		//		Mixin to supplement _TemplatedMixin when template contains widgets

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  (Remove for 2.0 and assume true)
		widgetsInTemplate: true,

		// contextRequire: Function
		//		Used to provide a context require to the dojo/parser in order to be
		//		able to use relative MIDs (e.g. `./Widget`) in the widget's template.
		contextRequire: null,

		_beforeFillContent: function(){
			if(this.widgetsInTemplate){
				// Before copying over content, instantiate widgets in template
				var node = this.domNode;

				if(this.containerNode && !this.searchContainerNode){
					// Tell parse call below not to look for widgets inside of this.containerNode
					this.containerNode.stopParser = true;
				}

				parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang, textDir: this.textDir},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					contextRequire: this.contextRequire,
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}).then(lang.hitch(this, function(widgets){
					this._startupWidgets = widgets;

					// _WidgetBase::destroy() will destroy any supporting widgets under this.domNode.
					// If we wanted to, we could call this.own() on anything in this._startupWidgets that was moved outside
					// of this.domNode (like Dialog, which is moved to <body>).

					// Hook up attach points and events for nodes that were converted to widgets
					for(var i = 0; i < widgets.length; i++){
						this._processTemplateNode(widgets[i], function(n,p){
							// callback to get a property of a widget
							return n[p];
						}, function(widget, type, callback){
							// callback to do data-dojo-attach-event to a widget
							if(type in widget){
								// back-compat, remove for 2.0
								return widget.connect(widget, type, callback);
							}else{
								// 1.x may never hit this branch, but it's the default for 2.0
								return widget.on(type, callback, true);
							}
						});
					}

					// Cleanup flag set above, just in case
					if(this.containerNode && this.containerNode.stopParser){
						delete this.containerNode.stopParser;
					}
				}));

				if(!this._startupWidgets){
					throw new Error(this.declaredClass + ": parser returned unfilled promise (probably waiting for module auto-load), " +
						"unsupported by _WidgetsInTemplateMixin.   Must pre-load all supporting widgets before instantiation.");
				}
			}
		},

		_processTemplateNode: function(/*DOMNode|Widget*/ baseNode, getAttrFunc, attachFunc){
			// Override _AttachMixin._processNode to skip DOMNodes with data-dojo-type set.   They are handled separately
			// in the _beforeFillContent() code above.

			if(getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type")){
				return true;
			}

			return this.inherited(arguments);
		},

		startup: function(){
			array.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this._startupWidgets = null;
			this.inherited(arguments);
		}
	});
});

},
'slmui/layout/ChapterAdvanced':function(){
define([
	'dojo/_base/declare',
	'./_ChapterBase',
	'./_ChapterAdvancedItem',
	'dojo/text!./templates/ChapterAdvanced.html'
], function(declare, _ChapterBase, _ChapterAdvancedItem, template){

	// summary:
	//		Chapter layout with advanced view. Allowed next modes of controls rendering:
	//		mode1 (auto): control take default place label left
	//		mode2: control take lable+default place, label top
	//		mode3: control take default+right_part place, label left
	//		mode4: control take lable+default+right_part place, label top
	//		mode5: control take lable+left_part place, label right

	return declare([_ChapterBase], {
		templateString: template,

		buildControlHolder: function(controlNode){
			// summary:
			//		Deprecated. Build control's wrapper and label.
			// tags:
			//		deprecated protected extension
			return new _ChapterAdvancedItem({controlNode: controlNode});
		}
	});
});
},
'slmui/layout/CollapsiblePane':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/dom-attr',
    'dojo/dom-construct',
    'dojo/parser',
	'dijit/TitlePane',
	'dijit/_TemplatedMixin',
	'xcitec/xcitecUtil',
	'dojo/text!./templates/ColapsiblePane.html'
], function(declare, array, domAttr, domConstruct, parser, TitlePane, _TemplatedMixin, xcitecUtil, template){
	return declare([TitlePane, _TemplatedMixin], {

		templateString: template,
		baseClass: "slmCollapsiblePane",
        headerelemNode: null,

		buildRendering: function() {
			var _this = this;
			array.forEach(this.content.childNodes, function(node) {
				switch (domAttr.get(node, "xcitecRole")) {
					case "cpstateholder":
						_this.stateHolder = node;
						_this.open = xcitecUtil.str2bool(domAttr.get(_this.stateHolder, "value"));
						break;
                    case "headerelem":
                        _this.headerelemNode = node;
                        break;
					default:
						break;
				}
			});
			this.inherited(arguments);
			this._setOpenAttr(this.open);
		},

        _fillContent: function() {
            if(this.headerelemNode){
                domConstruct.place(this.headerelemNode, this.headerelem, "last");
            }
            this.inherited(arguments);
        },

        postCreate: function() {
            parser.parse(this.headerelem);
            this.inherited(arguments);
        },

		_setOpenAttr: function(/*Boolean*/ open, /*Boolean*/ animate){
			this.inherited(arguments);

			if (this.stateHolder) {
				domAttr.set(this.stateHolder, "value", open);
			}
		}
	});
});
},
'dijit/_HasDropDown':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred",
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/has", // has("touch")
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER keys.ESCAPE
	"dojo/_base/lang", // lang.hitch lang.isFunction
	"dojo/on",
	"dojo/touch",
	"./registry", // registry.byNode()
	"./focus",
	"./popup",
	"./_FocusMixin"
], function(declare, Deferred, dom, domAttr, domClass, domGeometry, domStyle, has, keys, lang, on, touch,
			registry, focus, popup, _FocusMixin){


	// module:
	//		dijit/_HasDropDown

	return declare("dijit._HasDropDown", _FocusMixin, {
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the aria-expanded class on.
		//		Also sets popupActive class but that will be removed in 2.0.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width.
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.
		//		Any dropdown taller than this will have scrollbars.
		//		Set to 0 for no max height, or -1 to limit height to available space in viewport
		maxHeight: -1,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//		- before: places drop down to the left of the target node/widget, or to the right in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- after: places drop down to the right of the target node/widget, or to the left in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- above: drop down goes above target node
		//		- below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below", "above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subclass
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown/touchstart on the arrow icon.

			if(this.disabled || this.readOnly){
				return;
			}

			// Prevent default to stop things like text selection, but don't stop propagation, so that:
			//		1. TimeTextBox etc. can focus the <input> on mousedown
			//		2. dropDownButtonActive class applied by _CssStateMixin (on button depress)
			//		3. user defined onMouseDown handler fires
			//
			// Also, don't call preventDefault() on MSPointerDown event (on IE10) because that prevents the button
			// from getting focus, and then the focus manager doesn't know what's going on (#17262)
			if(e.type != "MSPointerDown" && e.type != "pointerdown"){
				e.preventDefault();
			}

			this.own(on.once(this.ownerDocument, touch.release, lang.hitch(this, "_onDropDownMouseUp")));

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback on mouseup/touchend after mousedown/touchstart on the arrow icon.
			//		Note that this function is called regardless of what node the event occurred on (but only after
			//		a mousedown/touchstart on the arrow).
			//
			//		If the drop down is a simple menu and the cursor is over the menu, we execute it, otherwise, we focus our
			//		drop down widget.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser `<select>` nodes:
			//
			//		1. mouse down on the select node (probably on the arrow)
			//		2. move mouse to a menu item while holding down the mouse button
			//		3. mouse up.  this selects the menu item as though the user had clicked it.

			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = domGeometry.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) || !(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(domClass.contains(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = registry.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened){
				// Focus the dropdown widget unless it's a menu (in which case autoFocus is set to false).
				// Even if it's a menu, we need to focus it if this is a fake mouse event caused by the user typing
				// SPACE/ENTER while using JAWS.  Jaws converts the SPACE/ENTER key into mousedown/mouseup events.
				// If this.hovering is false then it's presumably actually a keyboard event.
				if(dropDown.focus && (dropDown.autoFocus !== false || (e.type == "mouseup" && !this.hovering))){
					// Do it on a delay so that we don't steal back focus from the dropdown.
					this._focusDropDownTimer = this.defer(function(){
						dropDown.focus();
						delete this._focusDropDownTimer;
					});
				}
			}else{
				// The drop down arrow icon probably can't receive focus, but widget itself should get focus.
				// defer() needed to make it work on IE (test DateTextBox)
				if(this.focus){
					this.defer("focus");
				}
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			// The drop down was already opened on mousedown/keydown; just need to stop the event
			if(this._stopClickEvents){
				e.stopPropagation();
				e.preventDefault();
			}
		},

		buildRendering: function(){
			this.inherited(arguments);

			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
				"after": this.isLeftToRight() ? "Right" : "Left",
				"before": this.isLeftToRight() ? "Left" : "Right",
				"above": "Up",
				"below": "Down",
				"left": "Left",
				"right": "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			domClass.add(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			// summary:
			//		set up nodes and connect our mouse and keyboard events

			this.inherited(arguments);

			var keyboardEventNode = this.focusNode || this.domNode;
			this.own(
				on(this._buttonNode, touch.press, lang.hitch(this, "_onDropDownMouseDown")),
				on(this._buttonNode, "click", lang.hitch(this, "_onDropDownClick")),
				on(keyboardEventNode, "keydown", lang.hitch(this, "_onKey")),
				on(keyboardEventNode, "keyup", lang.hitch(this, "_onKeyUp"))
			);
		},

		destroy: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){
				return;
			}
			var d = this.dropDown, target = e.target;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){
					/* false return code means that the drop down handled the key */
					e.stopPropagation();
					e.preventDefault();
					return;
				}
			}
			if(d && this._opened && e.keyCode == keys.ESCAPE){
				this.closeDropDown();
				e.stopPropagation();
				e.preventDefault();
			}else if(!this._opened &&
				(e.keyCode == keys.DOWN_ARROW ||
					// ignore unmodified SPACE if _KeyNavMixin has active searching in progress
					( (e.keyCode == keys.ENTER || (e.keyCode == keys.SPACE && (!this._searchTimer || (e.ctrlKey || e.altKey || e.metaKey)))) &&
						//ignore enter and space if the event is for a text input
						((target.tagName || "").toLowerCase() !== 'input' ||
							(target.type && target.type.toLowerCase() !== 'text'))))){
				// Toggle the drop down, but wait until keyup so that the drop down doesn't
				// get a stray keyup event, or in the case of key-repeat (because user held
				// down key for too long), stray keydown events
				this._toggleOnKeyUp = true;
				e.stopPropagation();
				e.preventDefault();
			}
		},

		_onKeyUp: function(){
			if(this._toggleOnKeyUp){
				delete this._toggleOnKeyUp;
				this.toggleDropDown();
				var d = this.dropDown;	// drop down may not exist until toggleDropDown() call
				if(d && d.focus){
					this.defer(lang.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			// Close dropdown but don't focus my <input>.  User may have focused somewhere else (ex: clicked another
			// input), and even if they just clicked a blank area of the screen, focusing my <input> will unwantedly
			// popup the keyboard on mobile.
			this.closeDropDown(false);

			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns true if the dropdown exists and it's data is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/*Function*/ loadCallback){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and then calls
			//		the given callback.
			// tags:
			//		protected

			// TODO: for 2.0, change API to return a Deferred, instead of calling loadCallback?
			loadCallback();
		},

		loadAndOpenDropDown: function(){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and
			//		then opens the drop down.  This is basically a callback when the
			//		user presses the down arrow button to open the drop down.
			// returns: Deferred
			//		Deferred for the drop down widget that
			//		fires when drop down is created and loaded
			// tags:
			//		protected
			var d = new Deferred(),
				afterLoad = lang.hitch(this, function(){
					this.openDropDown();
					d.resolve(this.dropDown);
				});
			if(!this.isLoaded()){
				this.loadDropDown(afterLoad);
			}else{
				afterLoad();
			}
			return d;
		},

		toggleDropDown: function(){
			// summary:
			//		Callback when the user presses the down arrow button or presses
			//		the down arrow key to open/close the drop down.
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){
				return;
			}
			if(!this._opened){
				this.loadAndOpenDropDown();
			}else{
				this.closeDropDown(true);	// refocus button to avoid hiding node w/focus
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit/popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			var retVal = popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: this.dropDownPosition,
				maxHeight: this.maxHeight,
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					domAttr.set(self._popupStateNode, "popupActive", false);
					domClass.remove(self._popupStateNode, "dijitHasDropDownOpen");
					self._set("_opened", false);	// use set() because _CssStateMixin is watching
				}
			});

			// Set width of drop down if necessary, so that dropdown width + width of scrollbar (from popup wrapper)
			// matches width of aroundNode
			if(this.forceWidth || (this.autoWidth && aroundNode.offsetWidth > dropDown._popupWrapper.offsetWidth)){
				var widthAdjust = aroundNode.offsetWidth - dropDown._popupWrapper.offsetWidth;
				var resizeArgs = {
					w: dropDown.domNode.offsetWidth + widthAdjust
				};
				if(lang.isFunction(dropDown.resize)){
					dropDown.resize(resizeArgs);
				}else{
					domGeometry.setMarginBox(ddNode, resizeArgs);
				}

				// If dropdown is right-aligned then compensate for width change by changing horizontal position
				if(retVal.corner[1] == "R"){
					dropDown._popupWrapper.style.left =
						(dropDown._popupWrapper.style.left.replace("px", "") - widthAdjust) + "px";
				}
			}

			domAttr.set(this._popupStateNode, "popupActive", "true");
			domClass.add(this._popupStateNode, "dijitHasDropDownOpen");
			this._set("_opened", true);	// use set() because _CssStateMixin is watching

			this._popupStateNode.setAttribute("aria-expanded", "true");
			this._popupStateNode.setAttribute("aria-owns", dropDown.id);

			// Set aria-labelledby on dropdown if it's not already set to something more meaningful
			if(ddNode.getAttribute("role") !== "presentation" && !ddNode.getAttribute("aria-labelledby")){
				ddNode.setAttribute("aria-labelledby", this.id);
			}

			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// focus:
			//		If true, refocuses the button widget
			// tags:
			//		protected

			if(this._focusDropDownTimer){
				this._focusDropDownTimer.remove();
				delete this._focusDropDownTimer;
			}

			if(this._opened){
				this._popupStateNode.setAttribute("aria-expanded", "false");
				if(focus){
					this.focus();
				}
				popup.close(this.dropDown);
				this._opened = false;
			}
		}

	});
});

},
'xcitec/_base/Line':function(){
define([
 "dojo/_base/declare",
 "dojo/_base/array",
 "dojo/_base/lang"
], function(declare, array, lang){
  	return declare(null, {

      // summary:
      //      a custom _Line to accomodate multi-dimensional values
      //
      // description:
      //	a normal dojo._Line is the curve, and does Line(start,end)
      //	for propertyAnimation. as we make more complicatied animations, we realize
      //	some properties can have 2, or 4 values relevant (x,y) or (t,l,r,b) for example
      //
      // 	this function provides support for those Lines, and is ported directly from 0.4
      //	this is a lot of extra code for something so seldom used, so we'll put it here as
      //	and optional core addition. you can create a new line, and use it during onAnimate
      //	as you see fit.
      //
      // start: Integer|Array
      //	An Integer (or an Array of integers) to use as a starting point
      // end: Integer|Array
      //	An Integer (or an Array of integers) to use as an ending point
      //
      // example:
      //      see dojox.fx.smoothScroll
      //
      // example:
      // |	// this is 10 .. 100 and 50 .. 500
      // |	var curve = new dojox.fx._Line([10,50],[100,500]);
      // |	// dojo.Animation.onAnimate is called at every step of the animation
      // |	// to define current values. this _Line returns an array
      // | 	// at each step. arguments[0] and [1] in this example.
      //

      constructor : function(start, end){
        this.start = start;
        this.end = end;
      },

      getValue : function(/*float*/ n){
        var isArray = lang.isArray(this.start),
            d = (isArray ? [] : this.end - this.start);

        if(isArray){
          array.forEach(this.start, function(s, i){
            d[i] = this.end[i] - s;
          }, this);

          var res = [];
          array.forEach(this.start, function(s, i){
            res[i] = (d[i] * n) + s;
          }, this);
          return res; // Array

        }else{
        // single value branch, document here for both branches:

				// summary:
				//      Returns the point on the line, or an array of points
				// n: a floating point number greater than 0 and less than 1
				// returns: Mixed
				return (d * n) + this.start; // Decimal
      }
		}
	});
});
},
'dojo/_base/event':function(){
define(["./kernel", "../on", "../has", "../dom-geometry"], function(dojo, on, has, dom){
	// module:
	//		dojo/_base/event

	if(on._fixEvent){
		var fixEvent = on._fixEvent;
		on._fixEvent = function(evt, se){
			// add some additional normalization for back-compat, this isn't in on.js because it is somewhat more expensive
			evt = fixEvent(evt, se);
			if(evt){
				dom.normalizeEvent(evt);
			}
			return evt;
		};		
	}
	
	var ret = {
		// summary:
		//		This module defines dojo DOM event API.   Usually you should use dojo/on, and evt.stopPropagation() +
		//		evt.preventDefault(), rather than this module.

		fix: function(/*Event*/ evt, /*DOMNode*/ sender){
			// summary:
			//		normalizes properties on the event object including event
			//		bubbling methods, keystroke normalization, and x/y positions
			// evt: Event
			//		native event object
			// sender: DOMNode
			//		node to treat as "currentTarget"
			if(on._fixEvent){
				return on._fixEvent(evt, sender);
			}
			return evt;	// Event
		},
	
		stop: function(/*Event*/ evt){
			// summary:
			//		prevents propagation and clobbers the default action of the
			//		passed event
			// evt: Event
			//		The event object. If omitted, window.event is used on IE.
			if(has("dom-addeventlistener") || (evt && evt.preventDefault)){
				evt.preventDefault();
				evt.stopPropagation();
			}else{
				evt = evt || window.event;
				evt.cancelBubble = true;
				on._preventDefault.call(evt);
			}
		}
	};

	if( 1 ){
		dojo.fixEvent = ret.fix;
		dojo.stopEvent = ret.stop;
	}

	return ret;
});

},
'xcitec/form/InlineDropDownButton':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/text!./templates/InlineDropDownButton.html",
	"./DropDownButton"
], function (declare, domClass, template, DropDownButton) {

	/*
	Actually used for Box - as layoutbutton
	and inside UserInfoBox
	 */
	var InlineDropDownButton = declare( DropDownButton, {
	    templateString: template,
	
		postCreate: function(){
			this.inherited(arguments);
			if(!this.iconClass){
				domClass.add(this.iconNode, 'dijitNoIcon');
			}
		}
	});
	return InlineDropDownButton;
});

},
'dojo/_base/html':function(){
define(["./kernel", "../dom", "../dom-style", "../dom-attr", "../dom-prop", "../dom-class", "../dom-construct", "../dom-geometry"], function(dojo, dom, style, attr, prop, cls, ctr, geom){
	// module:
	//		dojo/dom

	/*=====
	return {
		// summary:
		//		This module is a stub for the core dojo DOM API.
	};
	=====*/

	// mix-in dom
	dojo.byId = dom.byId;
	dojo.isDescendant = dom.isDescendant;
	dojo.setSelectable = dom.setSelectable;

	// mix-in dom-attr
	dojo.getAttr = attr.get;
	dojo.setAttr = attr.set;
	dojo.hasAttr = attr.has;
	dojo.removeAttr = attr.remove;
	dojo.getNodeProp = attr.getNodeProp;

	dojo.attr = function(node, name, value){
		// summary:
		//		Gets or sets an attribute on an HTML element.
		// description:
		//		Handles normalized getting and setting of attributes on DOM
		//		Nodes. If 2 arguments are passed, and a the second argument is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to get or set the attribute on
		// name: String|Object
		//		the name of the attribute to get or set.
		// value: String?
		//		The value to set for the attribute
		// returns:
		//		when used as a getter, the value of the requested attribute
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		// example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	dojo.attr(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.attr("nodeId", "foo");
		//
		// example:
		//	|	// use attr() to set the tab index
		//	|	dojo.attr("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.attr("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.attr("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.attr("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		if(arguments.length == 2){
			return attr[typeof name == "string" ? "get" : "set"](node, name);
		}
		return attr.set(node, name, value);
	};

	// mix-in dom-class
	dojo.hasClass = cls.contains;
	dojo.addClass = cls.add;
	dojo.removeClass = cls.remove;
	dojo.toggleClass = cls.toggle;
	dojo.replaceClass = cls.replace;

	// mix-in dom-construct
	dojo._toDom = dojo.toDom = ctr.toDom;
	dojo.place = ctr.place;
	dojo.create = ctr.create;
	dojo.empty = function(node){ ctr.empty(node); };
	dojo._destroyElement = dojo.destroy = function(node){ ctr.destroy(node); };

	// mix-in dom-geometry
	dojo._getPadExtents = dojo.getPadExtents = geom.getPadExtents;
	dojo._getBorderExtents = dojo.getBorderExtents = geom.getBorderExtents;
	dojo._getPadBorderExtents = dojo.getPadBorderExtents = geom.getPadBorderExtents;
	dojo._getMarginExtents = dojo.getMarginExtents = geom.getMarginExtents;
	dojo._getMarginSize = dojo.getMarginSize = geom.getMarginSize;
	dojo._getMarginBox = dojo.getMarginBox = geom.getMarginBox;
	dojo.setMarginBox = geom.setMarginBox;
	dojo._getContentBox = dojo.getContentBox = geom.getContentBox;
	dojo.setContentSize = geom.setContentSize;
	dojo._isBodyLtr = dojo.isBodyLtr = geom.isBodyLtr;
	dojo._docScroll = dojo.docScroll = geom.docScroll;
	dojo._getIeDocumentElementOffset = dojo.getIeDocumentElementOffset = geom.getIeDocumentElementOffset;
	dojo._fixIeBiDiScrollLeft = dojo.fixIeBiDiScrollLeft = geom.fixIeBiDiScrollLeft;
	dojo.position = geom.position;

	dojo.marginBox = function marginBox(/*DomNode|String*/node, /*Object?*/box){
		// summary:
		//		Getter/setter for the margin-box of node.
		// description:
		//		Getter/setter for the margin-box of node.
		//		Returns an object in the expected format of box (regardless
		//		if box is passed). The object might look like:
		//		`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a margin width of 300px and a margin-height of
		//		150px.
		// node:
		//		id or reference to DOM Node to get/set box for
		// box:
		//		If passed, denotes that dojo.marginBox() should
		//		update/set the margin box for node. Box is an object in the
		//		above format. All properties are optional if passed.
		// example:
		//		Retrieve the margin box of a passed node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	console.dir(box);
		//
		// example:
		//		Set a node's margin box to the size of another node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	dojo.marginBox("someOtherNode", box);
		return box ? geom.setMarginBox(node, box) : geom.getMarginBox(node); // Object
	};

	dojo.contentBox = function contentBox(/*DomNode|String*/node, /*Object?*/box){
		// summary:
		//		Getter/setter for the content-box of node.
		// description:
		//		Returns an object in the expected format of box (regardless if box is passed).
		//		The object might look like:
		//		`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a content width of 300px and a content-height of
		//		150px. Note that the content box may have a much larger border
		//		or margin box, depending on the box model currently in use and
		//		CSS values set/inherited for node.
		//		While the getter will return top and left values, the
		//		setter only accepts setting the width and height.
		// node:
		//		id or reference to DOM Node to get/set box for
		// box:
		//		If passed, denotes that dojo.contentBox() should
		//		update/set the content box for node. Box is an object in the
		//		above format, but only w (width) and h (height) are supported.
		//		All properties are optional if passed.
		return box ? geom.setContentSize(node, box) : geom.getContentBox(node); // Object
	};

	dojo.coords = function(/*DomNode|String*/node, /*Boolean?*/includeScroll){
		// summary:
		//		Deprecated: Use position() for border-box x/y/w/h
		//		or marginBox() for margin-box w/h/l/t.
		//
		//		Returns an object that measures margin-box (w)idth/(h)eight
		//		and absolute position x/y of the border-box. Also returned
		//		is computed (l)eft and (t)op values in pixels from the
		//		node's offsetParent as returned from marginBox().
		//		Return value will be in the form:
		//|			{ l: 50, t: 200, w: 300: h: 150, x: 100, y: 300 }
		//		Does not act as a setter. If includeScroll is passed, the x and
		//		y params are affected as one would expect in dojo.position().
		dojo.deprecated("dojo.coords()", "Use dojo.position() or dojo.marginBox().");
		node = dom.byId(node);
		var s = style.getComputedStyle(node), mb = geom.getMarginBox(node, s);
		var abs = geom.position(node, includeScroll);
		mb.x = abs.x;
		mb.y = abs.y;
		return mb;	// Object
	};

	// mix-in dom-prop
	dojo.getProp = prop.get;
	dojo.setProp = prop.set;

	dojo.prop = function(/*DomNode|String*/node, /*String|Object*/name, /*String?*/value){
		// summary:
		//		Gets or sets a property on an HTML element.
		// description:
		//		Handles normalized getting and setting of properties on DOM
		//		Nodes. If 2 arguments are passed, and a the second argument is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node:
		//		id or reference to the element to get or set the property on
		// name:
		//		the name of the property to get or set.
		// value:
		//		The value to set for the property
		// returns:
		//		when used as a getter, the value of the requested property
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		// example:
		//	|	// get the current value of the "foo" property on a node
		//	|	dojo.prop(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.prop("nodeId", "foo");
		//
		// example:
		//	|	// use prop() to set the tab index
		//	|	dojo.prop("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.prop("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//		Style is s special case: Only set with an object hash of styles
		//	|	dojo.prop("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//		Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.prop("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		if(arguments.length == 2){
			return prop[typeof name == "string" ? "get" : "set"](node, name);
		}
		// setter
		return prop.set(node, name, value);
	};

	// mix-in dom-style
	dojo.getStyle = style.get;
	dojo.setStyle = style.set;
	dojo.getComputedStyle = style.getComputedStyle;
	dojo.__toPixelValue = dojo.toPixelValue = style.toPixelValue;

	dojo.style = function(node, name, value){
		// summary:
		//		Accesses styles on a node. If 2 arguments are
		//		passed, acts as a getter. If 3 arguments are passed, acts
		//		as a setter.
		// description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo.marginBox()`,
		//		`dojo.contentBox()` or `dojo.position()`.
		// node: DOMNode|String
		//		id or reference to node to get/set style for
		// name: String|Object?
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		// value: String?
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		// returns:
		//		when used as a getter, return the computed style of the node if passing in an ID or node,
		//		or return the normalized, computed value for the property when passing in a node and a style property
		// example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	dojo.style("thinger");
		// example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	dojo.style("thinger", "opacity"); // 1 by default
		//
		// example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	dojo.style("thinger", "opacity", 0.5); // == 0.5
		//
		// example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	dojo.style("thinger", {
		//	|		"opacity": 0.5,
		//	|		"border": "3px solid black",
		//	|		"height": "300px"
		//	|	});
		//
		// example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	dojo.style("thinger",{
		//	|		fontSize:"14pt",
		//	|		letterSpacing:"1.2em"
		//	|	});
		//
		// example:
		//		dojo/NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo.style() on every element of the list. See: `dojo/query` and `dojo/NodeList`
		//	|	dojo.query(".someClassName").style("visibility","hidden");
		//	|	// or
		//	|	dojo.query("#baz > div").style({
		//	|		opacity:0.75,
		//	|		fontSize:"13pt"
		//	|	});

		switch(arguments.length){
			case 1:
				return style.get(node);
			case 2:
				return style[typeof name == "string" ? "get" : "set"](node, name);
		}
		// setter
		return style.set(node, name, value);
	};

	return dojo;
});

},
'xcitec/page/BoxPanelV2':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/query",
	"dojo/text!xcitec/page/templates/BoxPanelV2.html",
	"dijit/registry",
	"../layout/XLayout",
  "../MessagePane"
], function (dojo, array, declare, domAttr, domConstruct, query, template, registry, XLayout, MessagePane) {

	/*
	 * BoxPanelV2 control holds the content of new GUI template.
	 */
	var BoxPanelV2 = declare( XLayout, {
	    templateString: template,
	
	    // box nodes and widgets (Array)
	    allNodes: null,
	    boxWidgets: null,
		
	    messagepane: null,
	
		postMixInProperties: function() {
	        this.allNodes = [];
	        this.boxWidgets = [];
	        this.inherited(arguments);
	    },
	
	    postCreate: function() {
	        this.inherited(arguments);
	
	        // check for toolbar existance
	        var isToolbarExists = false;
	        array.forEach(this.xcitecRoles, function(node) {
	            if (domAttr.get(node, "xcitecRole") == "toolbar") {
	                isToolbarExists = true;
	            }
	        });
	        if (!isToolbarExists) {
	            domConstruct.destroy(this.toolbarwrap);
	        }
	    },
	
	    startup: function() {
	        this.boxWidgets = [];
	        var _this = this;
	
	        this.messagepane = query("> [xcitecRole=messagepane]", this.containerNode)[0];
	
	        // seraching all Widget's nodes
	        this.allNodes = query("> [widgetId]", this.containerNode);
	        array.forEach(this.allNodes, function(node) {
	            if (registry.byNode(node) instanceof MessagePane ) {
	                _this.messagepane = node;
	            }
	        });
	        array.forEach(this.allNodes, function(node) {
	            // find all boxes
	            if (("componentType" in registry.byNode(node)) && (registry.byNode(node).componentType == "box")) {
	                // find box in edit mode
	                if (registry.byNode(node).editMode) {
	                    _this.boxWidgets.push(registry.byNode(node));
	                }
	            }
	        });
	        if (this.messagepane) {
	            if (this.boxWidgets.length > 0) {
	                domConstruct.place(this.messagepane, this.boxWidgets[this.boxWidgets.length - 1].containerNode, "first")
	            }
	        }
	    }
	});
	return BoxPanelV2;
});

},
'xcitec/form/_XButtonRight':function(){
define([
  "dojo/_base/declare",
	"./_XButton"
], function (declare, _XButton) {
  return declare(_XButton,	{
	    baseClass: "dijitXcitecBtn dijitXcitecBtnRight"
	});
});
},
'xcitec/page/MainMenuV1':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom",
	"dojo/dom-geometry",
	"dojo/text!./templates/MainMenuV1.html",
	"dojo/topic",
	"dijit/_Container",
	"dijit/_TemplatedMixin",
  "dijit/_WidgetsInTemplateMixin",
	"dijit/_WidgetBase",
  "dijit/popup",
	"../Label"
], function (declare, domClass, dom, geometry, template, topic, _Container, _Templated, _WidgetsInTemplateMixin, _Widget, popup) {

	var MainMenuV1 = declare(
	  
	  [_Widget, _Templated, _WidgetsInTemplateMixin, _Container],
	  {
	    //templateString : '',
	    templateString: template,
	
	    //baseClass: "MainMenu",
	    widgetsInTemplate: true,
	
	    // label: String
	    //  Text to display in button.
	  	label: '',
	
	    // labelId: String
	    //  id Of Label for MultyLanguage support.
	  	labelId: '',
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        this.isBodyClickable = ("onClick" in this.params);
	    },
	
	    //--- BEGIN --- functions of The _Widget Lifecycle  (in order of call) --
			postCreate: function(){
				this.inherited(arguments);
				dom.setSelectable(this.domNode, false);
	      if (this.isBodyClickable) {
	        domClass.add(this.titleNode, 'MainMenuContentClickable');
	      }
	    },
	
	    startup: function(){
	      if(this._started){ return; }
	
	      // the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
	      // make it invisible, and store a reference to pass to the popup code.
	      if(!this.dropDown){
	        //var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
	        //this.dropDown = dijit.byNode(dropDownNode);
	        //delete this.dropDownContainer;
	        this.dropDown = this.getChildren()[1]; // 0 - label; 1 - popup item
	      }
	      if(this.dropDown){
          popup.moveOffScreen(this.dropDown);
	        domClass.add(this.popupStateNode, 'MainMenuDownArrowClickable');
	      }
	
	      this.inherited(arguments);
	    },
	
	    destroyDescendants: function(){
	      if(this.dropDown){
	        this.dropDown.destroyRecursive();
	        delete this.dropDown;
	      }
	      this.inherited(arguments);
	    },
	    //--- END --- functions of The _Widget Lifecycle  (in order of call) --
	
	    _onBlur: function(){
	      // summary: called magically when focus has shifted away from this widget and it's dropdown
	      this._closeDropDown();
	      // don't focus on button.  the user has explicitly focused on something else.
	      this.inherited(arguments);
	    },
	
	    _onDownArrowClick:   function(/*Event*/ e) {
	      e.preventDefault(), e.stopPropagation();
	      if (!this.dropDown) {
	        console.debug("No DropDown");
	        return;
	      }
	      if(!this._opened){
	        this._openDropDown();
	  		}else{
	  			this._closeDropDown();
	  		}
	  	},
	
	    _openDropDown: function(){
	      var dropDown = this.dropDown;
	      var oldWidth=dropDown.domNode.style.width;
	      var self = this;
	
	      dijit.popup.open({
	        parent: this,
	        popup: dropDown,
	        around: this.domNode,
	        orient:
	          // TODO: add user-defined positioning option, like in Tooltip.js
	          this.isLeftToRight() ? {'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} : {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
	        onExecute: function(){
	          self._closeDropDown(true);
	        },
	        onCancel: function(){
	          self._closeDropDown(true);
	        },
	        onClose: function(){
	          dropDown.domNode.style.width = oldWidth;
	          self.popupStateNode.removeAttribute("popupActive");
	          self._opened = false;
	        }
	      });
	      if(this.domNode.offsetWidth > dropDown.domNode.offsetWidth){
	        var adjustNode = null;
	        if(!this.isLeftToRight()){
	          adjustNode = dropDown.domNode.parentNode;
	          var oldRight = adjustNode.offsetLeft + adjustNode.offsetWidth;
	        }
	        // make menu at least as wide as the button
	        geometry.setMarginBox(dropDown.domNode, {w: this.domNode.offsetWidth});
	        if(adjustNode){
	          adjustNode.style.left = oldRight - this.domNode.offsetWidth + "px";
	        }
	      }
	      this.popupStateNode.setAttribute("popupActive", "true");
	      this._opened=true;
	      domClass.add(this.popupStateNode, 'MainMenuDownArrowActive');
	    },
	
	/*
	    // _openDropDown inherited from DropDownButton
	    _openDropDown: function(){
	  		//this.inherited(arguments);
	    },
	*/
	
	    // _closeDropDown inherited from DropDownButton
	    _closeDropDown: function(){
	      if(this._opened){
          popup.close(this.dropDown);
	        this._opened = false;
	        domClass.remove(this.popupStateNode, 'MainMenuDownArrowActive');
	      }
	    },
	
	
	    log: function( str ) {
	      //if (!this._logCounter) this._logCounter=0;
	      //this._logCounter++;
	      //console.log( "{"+ this._logCounter +"} "+ str) 
	      // dummy function
	    }
	  }
	);
	
	
	return MainMenuV1;
});

},
'xcitec/page/MainMenuV2':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom",
	"dojo/query",
	"dojo/topic",
  "dojo/keys",
	"xcitec/_StateWidget"
], function (dojo, array, declare, lang, windowModule, domAttr, domClass, dom, query, topic, keys, _StateWidget) {

	return declare( _StateWidget, {
		templateString: "<div data-dojo-attach-point='containerNode' role='navigation'><div class='menu_header' data-dojo-attach-point='stateNode,labelNode' data-dojo-attach-event='onclick:_onLabelClick, onkeypress:_onLabelKeyPress' tabindex='0'></div></div>",
		label: "",
		labelId: "",
		level: -1,
	
		buildRendering: function(){
			this.parseStructure(this.srcNodeRef, this.level);
			this.inherited(arguments);
			this.labelNode.appendChild(windowModule.doc.createTextNode(this.label));
		},
	
		postCreate: function(){
			dom.setSelectable(this.domNode, false);
			domClass.add(query(">", this.domNode)[1], "menu");
			this.saveOnClick = this.onClick;
			this.onClick = lang.hitch(this, "_onClick");
			this.inherited(arguments);
		},
	
		startup: function(){
			this.inherited(arguments);
			domAttr.set(query(">", this.domNode)[1], 'role', 'menubar');
		},
	
		/**
		 * Parses srcNodeRef structure, sets appropriate level,
		 * itemIndex
		 * @param node - node to parse
		 * @param level - level to set for node being parsed
		 */
		parseStructure: function(node, level){
			var _this = this;
			var itemIndex = 0;
			array.forEach(node.childNodes, function(e){
				if(e.nodeType == 1){
					domAttr.set(node, "level", level);
					domAttr.set(node, "itemIndex", itemIndex++);
					if(e.childNodes.length > 0){
						_this.parseStructure(e, level + 1);
					}else if(level == 0){
						domAttr.set(e, "level", 1);
					}
				}
			});
		},

    onClick: function(evt){
    },
	
		_onClick: function(e){
			// block function for this.domNode onclick action
		},
	
		_onLabelClick: function(e){
			this.saveOnClick();
		},
	
		_onLabelKeyPress: function(/*Event*/ e){
			switch(e.keyCode){
				case keys.ENTER:
					this._onLabelClick(e);
					break;
				case keys.DOWN_ARROW:
					e.preventDefault(), e.stopPropagation();
					query('> ul > li > span', this.domNode)[0].focus();
					break;
			}
		}
	});
});
},
'dijit/form/_CheckBoxMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr" // domAttr.set
], function(declare, domAttr){

	// module:
	//		dijit/form/_CheckBoxMixin

	return declare("dijit.form._CheckBoxMixin", null, {
		// summary:
		//		Mixin to provide widget functionality corresponding to an HTML checkbox
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//

		// type: [private] String
		//		type attribute on `<input>` node.
		//		Overrides `dijit/form/Button.type`.  Users should not change this value.
		type: "checkbox",

		// value: String
		//		As an initialization parameter, equivalent to value field on normal checkbox
		//		(if checked, the value is passed as the value when form is submitted).
		value: "on",

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,

		// aria-pressed for toggle buttons, and aria-checked for checkboxes
		_aria_attr: "aria-checked",

		_setReadOnlyAttr: function(/*Boolean*/ value){
			this._set("readOnly", value);
			domAttr.set(this.focusNode, 'readOnly', value);
		},

		// Override dijit/form/Button._setLabelAttr() since we don't even have a containerNode.
		// Normally users won't try to set label, except when CheckBox or RadioButton is the child of a dojox/layout/TabContainer
		_setLabelAttr: undefined,

		_getSubmitValue: function(/*String*/ value){
			return (value == null || value === "") ? "on" : value;
		},

		_setValueAttr: function(newValue){
			newValue = this._getSubmitValue(newValue);	// "on" to match browser native behavior when value unspecified
			this._set("value", newValue);
			domAttr.set(this.focusNode, "value", newValue);
		},

		reset: function(){
			this.inherited(arguments);
			// Handle unlikely event that the <input type=checkbox> value attribute has changed
			this._set("value", this._getSubmitValue(this.params.value));
			domAttr.set(this.focusNode, 'value', this.value);
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions - need to check
			//		readOnly, since button no longer does that check.
			if(this.readOnly){
				e.stopPropagation();
				e.preventDefault();
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'xcitec/Source':function(){
define([
 "dojo/_base/declare",
	'dojo/dom-geometry',
	'dojo/dom-class',
	'dojo/dnd/common',
 "dojo/dnd/Source",
 "dojo/dnd/Manager"
], function(declare, domGeometry, domClass, dnd, Source, Manager){
  return declare(Source, {
    onMouseMove: function(e) {
        // summary:
        //		event processor for onmousemove
        // e: Event
        //		mouse event
        if (this.isDragging && this.targetState == "Disabled") {
            return;
        }
        Source.superclass.onMouseMove.call(this, e);
		var manager = Manager.manager();
        if (this.isDragging) {
            // calculate before/after
            var before, over = false;
            if (this.current) {
                if (!this.targetBox || this.targetAnchor != this.current) {
                    this.targetBox = domGeometry.position(this.current, true);
                }
                if (this.horizontal) {
                    before = (e.pageX - this.targetBox.x) < (this.targetBox.w / 2);
                } else {
                    before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
                    // detect if cursor is over the node
                    over = (((e.pageY - this.targetBox.y) < this.targetBox.h - 6) && ((e.pageY - this.targetBox.y) > 6));
                    if (over) {
                        domClass.add(this.current, 'dojoDndItemXcitecOver');
                    } else {
                        domClass.remove(this.current, 'dojoDndItemXcitecOver');
                    }
                }
            }

            if (this.current != this.targetAnchor || before != this.before) {
                this._markTargetAnchor(before, over);
                manager.canDrop(!this.current || manager.source != this || !(this.current.id in this.selection));
            }
        } else {
            if (this.mouseDown && this.isSource &&
                    (Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)) {
                var nodes = this.getSelectedNodes();
                if (nodes.length) {
                    manager.startDrag(this, nodes, this.copyState(dnd.getCopyKeyState(e), true));
                }
            }
        }
    }
  });
});
},
'dojo/html':function(){
define(["./_base/kernel", "./_base/lang", "./_base/array", "./_base/declare", "./dom", "./dom-construct", "./parser"],
	function(kernel, lang, darray, declare, dom, domConstruct, parser){
	// module:
	//		dojo/html

	// the parser might be needed..

	// idCounter is incremented with each instantiation to allow assignment of a unique id for tracking, logging purposes
	var idCounter = 0;

	var html = {
		// summary:
		//		TODOC

		_secureForInnerHtml: function(/*String*/ cont){
			// summary:
			//		removes !DOCTYPE and title elements from the html string.
			//
			//		khtml is picky about dom faults, you can't attach a style or `<title>` node as child of body
			//		must go into head, so we need to cut out those tags
			// cont:
			//		An html string for insertion into the dom
			//
			return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
		},

		// Deprecated, should use dojo/dom-constuct.empty() directly, remove in 2.0.
		_emptyNode: domConstruct.empty,

		_setNodeContent: function(/*DomNode*/ node, /*String|DomNode|NodeList*/ cont){
			// summary:
			//		inserts the given content into the given node
			// node:
			//		the parent element
			// content:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo/NodeList, Array or other enumerable list of nodes

			// always empty
			domConstruct.empty(node);

			if(cont){
				if(typeof cont == "string"){
					cont = domConstruct.toDom(cont, node.ownerDocument);
				}
				if(!cont.nodeType && lang.isArrayLike(cont)){
					// handle as enumerable, but it may shrink as we enumerate it
					for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0){
						domConstruct.place( cont[i], node, "last");
					}
				}else{
					// pass nodes, documentFragments and unknowns through to dojo.place
					domConstruct.place(cont, node, "last");
				}
			}

			// return DomNode
			return node;
		},

		// we wrap up the content-setting operation in a object
		_ContentSetter: declare("dojo.html._ContentSetter", null, {
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",

			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,

			// extractContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of `<html> <body>` wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.	Root for attribute names to search for.	  If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: kernel._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.	  Only obeyed if parseContent is true.
			startup: true,

			// lifecycle methods
			constructor: function(/*Object*/ params, /*String|DomNode*/ node){
				// summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..

				// the original params are mixed directly into the instance "this"
				lang.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dom.byId( this.node || node );

				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /*Object?*/ params){
				// summary:
				//		front-end to the set-content sequence
				// cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();

				var ret = this.onEnd();

				if(ret && ret.then){
					// Make dojox/html/_ContentSetter.set() return a Promise that resolves when load and parse complete.
					return ret;
				}else{
					// Vanilla dojo/html._ContentSetter.set() returns a DOMNode for back compat.   For 2.0, switch it to
					// return a Deferred like above.
					return this.node;
				}
			},

			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node){
					// can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV

					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},

			empty: function(){
				// summary:
				//		cleanly empty out existing content

				// If there is a parse in progress, cancel it.
				if(this.parseDeferred){
					if(!this.parseDeferred.isResolved()){
						this.parseDeferred.cancel();
					}
					delete this.parseDeferred;
				}

				// destroy any widgets from a previous run
				// NOTE: if you don't want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happening
				if(this.parseResults && this.parseResults.length){
					darray.forEach(this.parseResults, function(w){
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				domConstruct.empty(this.node);
			},

			onBegin: function(){
				// summary:
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties -
				//		including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;

				if(lang.isString(cont)){
					if(this.cleanContent){
						cont = html._secureForInnerHtml(cont);
					}

					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();

				this.content = cont;
				return this.node; // DomNode
			},

			onEnd: function(){
				// summary:
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults and this.parseDeferred if you need those..
					this._parse();
				}
				return this.node; // DomNode
				// TODO: for 2.0 return a Promise indicating that the parse completed.
			},

			tearDown: function(){
				// summary:
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description:
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.parseDeferred;
				delete this.node;
				delete this.content;
			},

			onContentError: function(err){
				return "Error occurred setting content: " + err;
			},

			onExecError: function(err){
				return "Error occurred executing scripts: " + err;
			},

			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we don't consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		and the parse promise in this.parseDeferred
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					darray.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					var self = this;
					this.parseDeferred = parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					}).then(function(results){
							return self.parseResults = results;
						}, function(e){
							self._onError('Content', e, "Error parsing in _ContentSetter#" + this.id);
						});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#" + this.id);
				}
			},

			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		override/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					html._setNodeContent(this.node, errText, true);
				}
			}
		}), // end declare()

		set: function(/*DomNode*/ node, /*String|DomNode|NodeList*/ cont, /*Object?*/ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo/dom-construct.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo/dom-construct.place(cont, node, "only"). dojo/dom-construct..place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo/dom-construct..place does not handle NodeList insertions
			//		dojo/dom-construct.place(cont, node, "only"). dojo/dom-construct.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo/dom-construct.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			// node:
			//		the parent element that will receive the content
			// cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo/NodeList, Array or other enumerable list of nodes
			// params:
			//		Optional flags/properties to configure the content-setting. See dojo/html/_ContentSetter
			// example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//	|	html.set(node, "some string");
			//	|	html.set(node, contentNode, {options});
			//	|	html.set(node, myNode.childNodes, {options});
			if(undefined == cont){
				console.warn("dojo.html.set: no cont argument provided, using empty string");
				cont = "";
			}
			if(!params){
				// simple and fast
				return html._setNodeContent(node, cont, true);
			}else{
				// more options but slower
				// note the arguments are reversed in order, to match the convention for instantiation via the parser
				var op = new html._ContentSetter(lang.mixin(
					params,
					{ content: cont, node: node }
				));
				return op.set();
			}
		}
	};
	lang.setObject("dojo.html", html);

	return html;
});

},
'slmui/form/MultiSelect':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/_base/lang',
	'dojo/aspect',
	'dojo/query',
	'dojo/json',
	'dojo/dom-attr',
	'dojo/dom-class',
	'dojo/dom-construct',
	'dojo/keys',
	'dojo/store/Memory',
	'dojo/store/Observable',
	'dijit/registry',
	'dijit/_Container',
	'dijit/Tree',
	'dijit/tree/ObjectStoreModel'
], function(declare, array, lang, aspect, query, json, domAttr, domClass, domConstruct, keys, Memory, Observable, registry, _Container, Tree, ObjectStoreModel){

	// override dijit/Tree._TreeNode to hide leafs for root children that don't have children
	lang.extend(Tree._TreeNode, {
		_setExpando: function(/*Boolean*/ processing){
			// summary:
			//		Set the right image for the expando node
			// tags:
			//		private

			var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
					"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"],
				_a11yStates = ["*", "-", "+", "*"],
				idx = processing ? 0 : (this.isExpandable ? (this.isExpanded ? 1 : 2) : 3);

			// apply the appropriate class to the expando node
			// if node is root child and not parent we don't need this
			if(this.item.parent != 'root' || idx != 3){
				domClass.replace(this.expandoNode, styles[idx], styles);
			}else{
				domClass.remove(this.expandoNode, styles);
			}

			// provide a non-image based indicator for images-off mode
			this.expandoNodeText.innerHTML = _a11yStates[idx];

		}
	});

	return declare([Tree, _Container], {

		showRoot: false,

		buildRendering: function(){
			this._parseSource();
			this.inherited(arguments);
		},

		_parseSource: function(){
			var jsonObject = [
				{name: this.id, id: 'root'}
			];

			this._openedItems = {};

			query('> div', this.srcNodeRef).forEach(function(node, index){
				var id = domAttr.get(node, 'value');
				jsonObject[index + 1] = {
					name: node.innerHTML,
					id: id,
					parent: domAttr.get(node, "parent") ? domAttr.get(node, 'parent') : 'root',
					fixedPos: domAttr.get(node, 'fixedPos') == 'true',
					notSortable: domAttr.get(node, 'notSortable') == 'true'
				};

				this._openedItems[domAttr.get(node, 'value')] = domAttr.get(node, 'expanded') == 'true';
			}, this);

			domConstruct.empty(this.srcNodeRef);

			var store = new Memory({
				data: jsonObject,
				getChildren: function(obj){
					return this.query({parent: obj.id});
				}
			});

			this.store = new Observable(store);
		},

		_store2model: function(){
			this.model = new ObjectStoreModel({
				store: this.store,
				query: {id: 'root'},
				mayHaveChildren: function(item){
					if(this.store.getChildren(item).length > 0){
						return true;
					}
					return false;
				},

				pasteItemToPos: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem,
									/*Boolean*/ bCopy, /*String*/ position, /*Item*/ target){

					var newParentChildren = [].concat(this.childrenCache[this.getIdentity(newParentItem)]);

					if(!bCopy){
						var oldParentChildren = [].concat(this.childrenCache[this.getIdentity(oldParentItem)]),
							index = array.indexOf(oldParentChildren, childItem);
						oldParentChildren.splice(index, 1);
						this.onChildrenChange(oldParentItem, oldParentChildren);
					}

					if(newParentItem == oldParentItem){
						newParentChildren = oldParentChildren;
					}

					var insertIndex = array.indexOf(newParentChildren, target),
						putOptions = {
							overwrite: true,
							parent: newParentItem,
							after: target
						};

					if(position == 'before'){
						newParentChildren.splice(insertIndex, 0, childItem);
						delete putOptions.after;
						putOptions.before = target;
					}else{
						newParentChildren.splice(insertIndex + 1, 0, childItem);
					}

					this.onChildrenChange(newParentItem, newParentChildren);

					var childrenCache = this.childrenCache[this.getIdentity(newParentItem)];
					array.forEach(newParentChildren, function(item, index){
						childrenCache[index] = newParentChildren[index];
					});

					return this.store.put(childItem, putOptions);
				}
			});
		},

		postCreate: function(){
			this.inherited(arguments);

			var store = this.store;

			aspect.around(store, "put", function(originalPut){
				return function(obj, options){
					if(options && options.parent){
						obj.parent = options.parent.id;
					}
					return originalPut.call(store, obj, options);
				}
			});
		},

		addSelected: function(/*dijit/form/MultiSelect*/ select){
			// summary:
			//		Move the selected nodes of a passed Select widget
			//		instance to this Select widget.
			//
			// example:
			// |	// move all the selected values from "bar" to "foo"
			// |	dijit.byId("foo").addSelected(dijit.byId("bar"));

			var selectedItems = select.getSelected();
			array.forEach(selectedItems, function(item){
				var existingItem = this.store.get(item.id);
				if(!existingItem){
					var it = item,
						root = this.store.get('root');
					while(it.id != 'root'){
						it = it.parentCache;
					}
					if(it.name == root.name){
						var parent = this.store.get(item.parentCache.id);

						if(!parent){
							it = item.parentCache;

							do{
								while(it.id != 'root'){
									parent = this.store.get(it.parentCache.id);

									if(parent){
										it.moved = true;
										this.store.put(it, {overwrite: true, parent: parent});
										break;
									}
									it = it.parentCache;
								}
								parent = it;
							}while(parent.id != item.parentCache.id);
						}

						if(parent){
							this.store.put(item, {overwrite: true, parent: parent});
						}
					}else{
						this.store.put(item, {overwrite: true, parent: root});
					}
				}else{
					existingItem.moved = undefined;
				}

				this.setSelectedItem(this.getNodesByItem(item)[0], true, false);
			}, this);
		},

		getSelected: function(){
			// summary:
			//		Access the NodeList of the selected options directly
			var selectedArray = [],
				model = this.model,
				store = this.store;
			array.forEach(this.get("selectedItems"), function(item){
				if(!item.parentCache){
					var it = item;
					while(it.parent != 'root'){
						var itParent = this.store.get(it.parent);
						it.parentCache = itParent;
						it = itParent;
					}

					it.parentCache = this.store.get('root');
				}

				var itemClone = {};
				for(var attr in item){
					if(attr == 'moved'){
						itemClone[attr] = undefined;
					}
					itemClone[attr] = item[attr];
				}

				selectedArray.push(itemClone);
				this.setSelectedItem(this.getNodesByItem(item)[0], true, false);
				if(store.getChildren(item).length == 0){
					store.remove(store.getIdentity(item));
				}else{
					item.moved = true;
				}


				var parent = store.query({id: itemClone.parent})[0];
				while(store.getChildren(parent).length == 0 && parent.moved){
					var parentClone = {};
					for(var attr in parent){
						if(attr == 'moved'){
							parentClone[attr] = undefined;
						}
						parentClone[attr] = parent[attr];
					}
					selectedArray.push(parentClone);
					store.remove(store.getIdentity(parent));
					parent = store.get(parentClone.parent)
				}
			}, this);
			return selectedArray;
		},

		moveSelectedUp: function(){
			// summary:
			//		change the order inside.
			//    move selected items one step up.
			var items = this.selectedItems;
			array.forEach(items, function(item){
				item = this.getNodesByItem(item)[0];
				if(item){
					var node = item.domNode,
						prevNode = node.previousSibling;
					if(!prevNode) return;
					var prevItem = registry.byNode(prevNode).item;
					if(item.item.fixedPos || prevItem.fixedPos) return;
					if(query('.dijitTreeRowSelected', prevNode).length) return;

					this.model.pasteItemToPos(item.item, this.model.root, this.model.root, false, 'before', prevItem);
				}
			}, this);
		},

		moveSelectedDown: function(){
			// summary:
			//		change the order inside.
			//    move selected items one step down.
			var items = this.selectedItems;
			array.forEach(items, function(item){
				item = this.getNodesByItem(item)[0];
				if(item){
					var node = item.domNode,
						nextNode = node.nextSibling;
					if(!nextNode) return;
					var nextItem = registry.byNode(nextNode).item;
					if(item.item.fixedPos || nextItem.fixedPos) return;
					if(query('.dijitTreeRowSelected', nextNode).length) return;

					this.model.pasteItemToPos(item.item, this.model.root, this.model.root, false, 'after', nextItem);
				}
			}, this);
		},


		/////////// Keyboard and Mouse handlers ////////////////////

		_onDownArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		down arrow pressed; get next visible node, set focus there

			var nextNode = this._getNext(node);
			if(nextNode && nextNode.isTreeNode){
				this.focusNode(nextNode);
				this.setSelectedItem(nextNode, false, evt.shiftKey);
			}
		},

		_onUpArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Up arrow pressed; move to previous visible node

			// if younger siblings
			var previousSibling = node.getPreviousSibling();
			if(previousSibling){
				node = previousSibling;
				// if the previous node is expanded, dive in deep
				while(node.isExpandable && node.isExpanded && node.hasChildren()){
					// move to the last child
					var children = node.getChildren();
					node = children[children.length - 1];
				}
			}else{
				// if this is the first child, return the parent
				// unless the parent is the root of a tree with a hidden root
				var parent = node.getParent();
				if(!(!this.showRoot && parent === this.rootNode)){
					node = parent;
				}
			}

			if(node && node.isTreeNode){
				this.focusNode(node);
				this.setSelectedItem(node, false, evt.shiftKey);
			}
		},

		_onRightArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Right arrow pressed; go to child node

			// if not expanded, expand, else move to 1st child
			if(node.isExpandable && !node.isExpanded){
				this._expandNode(node);
			}else if(node.hasChildren()){
				node = node.getChildren()[0];
				if(node && node.isTreeNode){
					this.focusNode(node);
					this.setSelectedItem(node, false, evt.shiftKey);
				}
			}
		},

		_onLeftArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Left arrow pressed.
			//		If not collapsed, collapse, else move to parent.

			if(node.isExpandable && node.isExpanded){
				this._collapseNode(node);
			}else{
				var parent = node.getParent();
				if(parent && parent.isTreeNode && !(!this.showRoot && parent === this.rootNode)){
					this.focusNode(parent);
					this.setSelectedItem(node, false, evt.shiftKey);
				}
			}
		},

		setSelectedItem: function(node, multi, range){
			this.dndController.userSelect(node, multi, range);
		},

		getItems: function(){
			return array.map(query('>', this.rootNode.containerNode), function(node){
				var childWidget = registry.byNode(node);
				if(childWidget){
					return childWidget.item;
				}
			});
		},

		expandNodes: function(){
			for(var item in this._openedItems){
				if(this._openedItems[item]){
					this.getNodesByItem(item)[0].expand();
				}
			}
		}
	});
});
},
'dojo/dom-style':function(){
define(["./sniff", "./dom"], function(has, dom){
	// module:
	//		dojo/dom-style

	// =============================
	// Style Functions
	// =============================

	// getComputedStyle drives most of the style code.
	// Wherever possible, reuse the returned object.
	//
	// API functions below that need to access computed styles accept an
	// optional computedStyle parameter.
	// If this parameter is omitted, the functions will call getComputedStyle themselves.
	// This way, calling code can access computedStyle once, and then pass the reference to
	// multiple API functions.

	// Although we normally eschew argument validation at this
	// level, here we test argument 'node' for (duck)type,
	// by testing nodeType, ecause 'document' is the 'parentNode' of 'body'
	// it is frequently sent to this function even
	// though it is not Element.
	var getComputedStyle, style = {
		// summary:
		//		This module defines the core dojo DOM style API.
	};
	if(has("webkit")){
		getComputedStyle = function(/*DomNode*/ node){
			var s;
			if(node.nodeType == 1){
				var dv = node.ownerDocument.defaultView;
				s = dv.getComputedStyle(node, null);
				if(!s && node.style){
					node.style.display = "";
					s = dv.getComputedStyle(node, null);
				}
			}
			return s || {};
		};
	}else if(has("ie") && (has("ie") < 9 || has("quirks"))){
		getComputedStyle = function(node){
			// IE (as of 7) doesn't expose Element like sane browsers
			// currentStyle can be null on IE8!
			return node.nodeType == 1 /* ELEMENT_NODE*/ && node.currentStyle ? node.currentStyle : {};
		};
	}else{
		getComputedStyle = function(node){
			return node.nodeType == 1 /* ELEMENT_NODE*/ ?
				node.ownerDocument.defaultView.getComputedStyle(node, null) : {};
		};
	}
	style.getComputedStyle = getComputedStyle;
	/*=====
	style.getComputedStyle = function(node){
		// summary:
		//		Returns a "computed style" object.
		//
		// description:
		//		Gets a "computed style" object which can be used to gather
		//		information about the current state of the rendered node.
		//
		//		Note that this may behave differently on different browsers.
		//		Values may have different formats and value encodings across
		//		browsers.
		//
		//		Note also that this method is expensive.  Wherever possible,
		//		reuse the returned object.
		//
		//		Use the dojo/dom-style.get() method for more consistent (pixelized)
		//		return values.
		//
		// node: DOMNode
		//		A reference to a DOM node. Does NOT support taking an
		//		ID string for speed reasons.
		// example:
		//	|	require(["dojo/dom-style", "dojo/dom"], function(domStyle, dom){
		//	|		domStyle.getComputedStyle(dom.byId('foo')).borderWidth;
		//	|	});
		//
		// example:
		//		Reusing the returned object, avoiding multiple lookups:
		//	|	require(["dojo/dom-style", "dojo/dom"], function(domStyle, dom){
		//	|		var cs = domStyle.getComputedStyle(dom.byId("someNode"));
		//	|		var w = cs.width, h = cs.height;
		//	|	});
		return; // CSS2Properties
	};
	=====*/

	var toPixel;
	if(!has("ie")){
		toPixel = function(element, value){
			// style values can be floats, client code may want
			// to round for integer pixels.
			return parseFloat(value) || 0;
		};
	}else{
		toPixel = function(element, avalue){
			if(!avalue){ return 0; }
			// on IE7, medium is usually 4 pixels
			if(avalue == "medium"){ return 4; }
			// style values can be floats, client code may
			// want to round this value for integer pixels.
			if(avalue.slice && avalue.slice(-2) == 'px'){ return parseFloat(avalue); }
			var s = element.style, rs = element.runtimeStyle, cs = element.currentStyle,
				sLeft = s.left, rsLeft = rs.left;
			rs.left = cs.left;
			try{
				// 'avalue' may be incompatible with style.left, which can cause IE to throw
				// this has been observed for border widths using "thin", "medium", "thick" constants
				// those particular constants could be trapped by a lookup
				// but perhaps there are more
				s.left = avalue;
				avalue = s.pixelLeft;
			}catch(e){
				avalue = 0;
			}
			s.left = sLeft;
			rs.left = rsLeft;
			return avalue;
		};
	}
	style.toPixelValue = toPixel;
	/*=====
	style.toPixelValue = function(node, value){
		// summary:
		//		converts style value to pixels on IE or return a numeric value.
		// node: DOMNode
		// value: String
		// returns: Number
	};
	=====*/

	// FIXME: there opacity quirks on FF that we haven't ported over. Hrm.

	var astr = "DXImageTransform.Microsoft.Alpha";
	var af = function(n, f){
		try{
			return n.filters.item(astr);
		}catch(e){
			return f ? {} : null;
		}
	};

	var _getOpacity =
		has("ie") < 9 || (has("ie") < 10 && has("quirks")) ? function(node){
			try{
				return af(node).Opacity / 100; // Number
			}catch(e){
				return 1; // Number
			}
		} :
		function(node){
			return getComputedStyle(node).opacity;
		};

	var _setOpacity =
		has("ie") < 9 || (has("ie") < 10 && has("quirks")) ? function(/*DomNode*/ node, /*Number*/ opacity){
			if(opacity === ""){ opacity = 1; }
			var ov = opacity * 100, fullyOpaque = opacity === 1;

			// on IE7 Alpha(Filter opacity=100) makes text look fuzzy so disable it altogether (bug #2661),
			// but still update the opacity value so we can get a correct reading if it is read later:
			// af(node, 1).Enabled = !fullyOpaque;

			if(fullyOpaque){
				node.style.zoom = "";
				if(af(node)){
					node.style.filter = node.style.filter.replace(
						new RegExp("\\s*progid:" + astr + "\\([^\\)]+?\\)", "i"), "");
				}
			}else{
				node.style.zoom = 1;
				if(af(node)){
					af(node, 1).Opacity = ov;
				}else{
					node.style.filter += " progid:" + astr + "(Opacity=" + ov + ")";
				}
				af(node, 1).Enabled = true;
			}

			if(node.tagName.toLowerCase() == "tr"){
				for(var td = node.firstChild; td; td = td.nextSibling){
					if(td.tagName.toLowerCase() == "td"){
						_setOpacity(td, opacity);
					}
				}
			}
			return opacity;
		} :
		function(node, opacity){
			return node.style.opacity = opacity;
		};

	var _pixelNamesCache = {
		left: true, top: true
	};
	var _pixelRegExp = /margin|padding|width|height|max|min|offset/; // |border
	function _toStyleValue(node, type, value){
		//TODO: should we really be doing string case conversion here? Should we cache it? Need to profile!
		type = type.toLowerCase();
		if(has("ie") || has("trident")){
			if(value == "auto"){
				if(type == "height"){ return node.offsetHeight; }
				if(type == "width"){ return node.offsetWidth; }
			}
			if(type == "fontweight"){
				switch(value){
					case 700: return "bold";
					case 400:
					default: return "normal";
				}
			}
		}
		if(!(type in _pixelNamesCache)){
			_pixelNamesCache[type] = _pixelRegExp.test(type);
		}
		return _pixelNamesCache[type] ? toPixel(node, value) : value;
	}

	var _floatAliases = {cssFloat: 1, styleFloat: 1, "float": 1};

	// public API

	style.get = function getStyle(/*DOMNode|String*/ node, /*String?*/ name){
		// summary:
		//		Accesses styles on a node.
		// description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo/dom-geometry.getMarginBox()`,
		//		`dojo/dom-geometry.getContentBox()` or `dojo/dom-geometry.getPosition()`.
		// node: DOMNode|String
		//		id or reference to node to get style for
		// name: String?
		//		the style property to get
		// example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	require(["dojo/dom-style", "dojo/dom"], function(domStyle, dom){
		//	|		domStyle.get("thinger");
		//	|	});
		// example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	require(["dojo/dom-style", "dojo/dom"], function(domStyle, dom){
		//	|		domStyle.get("thinger", "opacity"); // 1 by default
		//	|	});

		var n = dom.byId(node), l = arguments.length, op = (name == "opacity");
		if(l == 2 && op){
			return _getOpacity(n);
		}
		name = _floatAliases[name] ? "cssFloat" in n.style ? "cssFloat" : "styleFloat" : name;
		var s = style.getComputedStyle(n);
		return (l == 1) ? s : _toStyleValue(n, name, s[name] || n.style[name]); /* CSS2Properties||String||Number */
	};

	style.set = function setStyle(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		// summary:
		//		Sets styles on a node.
		// node: DOMNode|String
		//		id or reference to node to set style for
		// name: String|Object
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		// value: String?
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		//
		// example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	require(["dojo/dom-style"], function(domStyle){
		//	|		domStyle.set("thinger", "opacity", 0.5); // == 0.5
		//	|	});
		//
		// example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	require(["dojo/dom-style"], function(domStyle){
		//	|		domStyle.set("thinger", {
		//	|			"opacity": 0.5,
		//	|			"border": "3px solid black",
		//	|			"height": "300px"
		//	|		});
		//	|	});
		//
		// example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	require(["dojo/dom-style", "dojo/dom"], function(domStyle, dom){
		//	|		domStyle.set("thinger",{
		//	|			fontSize:"14pt",
		//	|			letterSpacing:"1.2em"
		//	|		});
		//	|	});
		//
		// example:
		//		dojo/NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo/dom-style.get() on every element of the list. See: `dojo/query` and `dojo/NodeList`
		//	|	require(["dojo/dom-style", "dojo/query", "dojo/NodeList-dom"],
		//	|	function(domStyle, query){
		//	|		query(".someClassName").style("visibility","hidden");
		//	|		// or
		//	|		query("#baz > div").style({
		//	|			opacity:0.75,
		//	|			fontSize:"13pt"
		//	|		});
		//	|	});

		var n = dom.byId(node), l = arguments.length, op = (name == "opacity");
		name = _floatAliases[name] ? "cssFloat" in n.style ? "cssFloat" : "styleFloat" : name;
		if(l == 3){
			return op ? _setOpacity(n, value) : n.style[name] = value; // Number
		}
		for(var x in name){
			style.set(node, x, name[x]);
		}
		return style.getComputedStyle(n);
	};

	return style;
});

},
'xcitec/_base/LanguageImageFamily':function(){
define([
         "dojo/_base/declare",
         "./ImageFamily"
       ], function (declare, ImageFamily) {
  return declare(ImageFamily, {
    familyId: "LANGUAGE",

    createIconCss: function(/*String*/ iconId){
      return "iconLanguage" + (!iconId ? "" : " iconLanguage" + iconId.toUpperCase());
    }
  });
});

},
'dojo/Stateful':function(){
define(["./_base/declare", "./_base/lang", "./_base/array", "./when"], function(declare, lang, array, when){
	// module:
	//		dojo/Stateful

return declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	//
	//		The class also provides the functionality to auto-magically manage getters
	//		and setters for object attributes/properties.
	//		
	//		Getters and Setters should follow the format of _xxxGetter or _xxxSetter where 
	//		the xxx is a name of the attribute to handle.  So an attribute of "foo" 
	//		would have a custom getter of _fooGetter and a custom setter of _fooSetter.
	//
	// example:
	//	|	require(["dojo/Stateful", function(Stateful) {
	//	|		var obj = new Stateful();
	//	|		obj.watch("foo", function(){
	//	|			console.log("foo changed to " + this.get("foo"));
	//	|		});
	//	|		obj.set("foo","bar");
	//	|	});

	// _attrPairNames: Hash
	//		Used across all instances a hash to cache attribute names and their getter 
	//		and setter names.
	_attrPairNames: {},

	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		return (apn[name] = {
			s: "_" + name + "Setter",
			g: "_" + name + "Getter"
		});
	},

	postscript: function(/*Object?*/ params){
		// Automatic setting of params during construction
		if (params){ this.set(params); }
	},

	_get: function(name, names){
		// summary:
		//		Private function that does a get based off a hash of names
		// names:
		//		Hash of names of custom attributes
		return typeof this[names.g] === "function" ? this[names.g]() : this[name];
	},
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		// name:
		//		The property to get.
		// returns:
		//		The property value on this Stateful instance.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		//		this just retrieves the object's property.
		// example:
		//	|	require(["dojo/Stateful", function(Stateful) {
		//	|		var stateful = new Stateful({foo: 3});
		//	|		stateful.get("foo") // returns 3
		//	|		stateful.foo // returns 3
		//	|	});

		return this._get(name, this._getAttrNames(name)); //Any
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		// name:
		//		The property to set.
		// value:
		//		The value to set in the property.
		// returns:
		//		The function returns this dojo.Stateful instance.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		//		the property. A programmatic setter may be defined in subclasses.
		// example:
		//	|	require(["dojo/Stateful", function(Stateful) {
		//	|		var stateful = new Stateful();
		//	|		stateful.watch(function(name, oldValue, value){
		//	|			// this will be called on the set below
		//	|		}
		//	|		stateful.set(foo, 5);
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|		stateful.set({
		//	|			foo: "Howdy",
		//	|			bar: 3
		//	|		});
		//	|	});
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		// If an object is used, iterate through object
		if(typeof name === "object"){
			for(var x in name){
				if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
					this.set(x, name[x]);
				}
			}
			return this;
		}

		var names = this._getAttrNames(name),
			oldValue = this._get(name, names),
			setter = this[names.s],
			result;
		if(typeof setter === "function"){
			// use the explicit setter
			result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// no setter so set attribute directly
			this[name] = value;
		}
		if(this._watchCallbacks){
			var self = this;
			// If setter returned a promise, wait for it to complete, otherwise call watches immediatly
			when(result, function(){
				self._watchCallbacks(name, oldValue, value);
			});
		}
		return this; // dojo/Stateful
	},
	_changeAttrValue: function(name, value){
		// summary:
		//		Internal helper for directly changing an attribute value.
		//
		// name: String
		//		The property to set.
		// value: Mixed
		//		The value to set in the property.
		//
		// description:
		//		Directly change the value of an attribute on an object, bypassing any 
		//		accessor setter.  Also handles the calling of watch and emitting events. 
		//		It is designed to be used by descendent class when there are two values 
		//		of attributes that are linked, but calling .set() is not appropriate.

		var oldValue = this.get(name);
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this; // dojo/Stateful
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		// name:
		//		Indicates the property to watch. This is optional (the callback may be the
		//		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		//		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		// callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		//		second argument as the old value and the third argument as the new value.

		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
						propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							propertyCallbacks[i].call(self, name, oldValue, value);
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);

		// TODO: Remove unwatch in 2.0
		var handle = {};
		handle.unwatch = handle.remove = function(){
			var index = array.indexOf(propertyCallbacks, callback);
			if(index > -1){
				propertyCallbacks.splice(index, 1);
			}
		};
		return handle; //Object
	}

});

});

},
'dojo/dnd/autoscroll':function(){
define(["../_base/lang", "../sniff", "../_base/window", "../dom-geometry", "../dom-style", "../window"],
	function(lang, has, win, domGeom, domStyle, winUtils){

// module:
//		dojo/dnd/autoscroll

var exports = {
	// summary:
	//		Used by dojo/dnd/Manager to scroll document or internal node when the user
	//		drags near the edge of the viewport or a scrollable node
};
lang.setObject("dojo.dnd.autoscroll", exports);

exports.getViewport = winUtils.getBox;

exports.V_TRIGGER_AUTOSCROLL = 32;
exports.H_TRIGGER_AUTOSCROLL = 32;

exports.V_AUTOSCROLL_VALUE = 16;
exports.H_AUTOSCROLL_VALUE = 16;

// These are set by autoScrollStart().
// Set to default values in case autoScrollStart() isn't called. (back-compat, remove for 2.0)
var viewport,
	doc = win.doc,
	maxScrollTop = Infinity,
	maxScrollLeft = Infinity;

exports.autoScrollStart = function(d){
	// summary:
	//		Called at the start of a drag.
	// d: Document
	//		The document of the node being dragged.

	doc = d;
	viewport = winUtils.getBox(doc);

	// Save height/width of document at start of drag, before it gets distorted by a user dragging an avatar past
	// the document's edge
	var html = win.body(doc).parentNode;
	maxScrollTop = Math.max(html.scrollHeight - viewport.h, 0);
	maxScrollLeft = Math.max(html.scrollWidth - viewport.w, 0);	// usually 0
};

exports.autoScroll = function(e){
	// summary:
	//		a handler for mousemove and touchmove events, which scrolls the window, if
	//		necessary
	// e: Event
	//		mousemove/touchmove event

	// FIXME: needs more docs!
	var v = viewport || winUtils.getBox(doc), // getBox() call for back-compat, in case autoScrollStart() wasn't called
		html = win.body(doc).parentNode,
		dx = 0, dy = 0;
	if(e.clientX < exports.H_TRIGGER_AUTOSCROLL){
		dx = -exports.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - exports.H_TRIGGER_AUTOSCROLL){
		dx = Math.min(exports.H_AUTOSCROLL_VALUE, maxScrollLeft - html.scrollLeft);	// don't scroll past edge of doc
	}
	if(e.clientY < exports.V_TRIGGER_AUTOSCROLL){
		dy = -exports.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - exports.V_TRIGGER_AUTOSCROLL){
		dy = Math.min(exports.V_AUTOSCROLL_VALUE, maxScrollTop - html.scrollTop);	// don't scroll past edge of doc
	}
	window.scrollBy(dx, dy);
};

exports._validNodes = {"div": 1, "p": 1, "td": 1};
exports._validOverflow = {"auto": 1, "scroll": 1};

exports.autoScrollNodes = function(e){
	// summary:
	//		a handler for mousemove and touchmove events, which scrolls the first available
	//		Dom element, it falls back to exports.autoScroll()
	// e: Event
	//		mousemove/touchmove event

	// FIXME: needs more docs!

	var b, t, w, h, rx, ry, dx = 0, dy = 0, oldLeft, oldTop;

	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in exports._validNodes)){
			var s = domStyle.getComputedStyle(n),
				overflow = (s.overflow.toLowerCase() in exports._validOverflow),
				overflowX = (s.overflowX.toLowerCase() in exports._validOverflow),
				overflowY = (s.overflowY.toLowerCase() in exports._validOverflow);
			if(overflow || overflowX || overflowY){
				b = domGeom.getContentBox(n, s);
				t = domGeom.position(n, true);
			}
			// overflow-x
			if(overflow || overflowX){
				w = Math.min(exports.H_TRIGGER_AUTOSCROLL, b.w / 2);
				rx = e.pageX - t.x;
				if(has("webkit") || has("opera")){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the domGeom.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += win.body().scrollLeft;
				}
				dx = 0;
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
					oldLeft = n.scrollLeft;
					n.scrollLeft = n.scrollLeft + dx;
				}
			}
			// overflow-y
			if(overflow || overflowY){
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				h = Math.min(exports.V_TRIGGER_AUTOSCROLL, b.h / 2);
				ry = e.pageY - t.y;
				if(has("webkit") || has("opera")){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the domGeom.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					ry += win.body().scrollTop;
				}
				dy = 0;
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
					oldTop = n.scrollTop;
					n.scrollTop  = n.scrollTop  + dy;
				}
			}
			if(dx || dy){ return; }
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	exports.autoScroll(e);
};

return exports;

});

},
'xcitec/Panel':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	'dojo/dom-attr',
	"dojo/dom-construct",
	"dojo/query",
	"dijit/_WidgetBase",
	"dijit/registry",
	"xcitec/Text"
], function (array, declare, domAttr, domConstruct, query, _Widget, registry, Text) {

	 
	var Panel = declare( _Widget, {
	  // caption: String
	  //  Chapter caption.
	  caption: '',
	
	  // captionId: String
	  //  id Of Label for MultyLanguage support.
	  captionId: '',
	
	  buildRendering: function(){
	    this.inherited(arguments);
	    if (this.caption) {
	      var captionWidget = new Text ({
	        baseClass: "xcitecTextBoxBold",
	         text:this.caption,
	         textId:this.captionId
	      }, null);
	      domConstruct.place(captionWidget.domNode, this.domNode, "first");
	    }
	  },
	
	    destroy: function() {
	        var widgets = query("> [widgetId]", this.domNode);
	        array.forEach(widgets, function(widget) {
	            if (registry.byNode(widget)) {
	                registry.byNode(widget).destroyDescendants();
	                registry.byNode(widget).destroy();
	            }
	        });
	        this.inherited(arguments);
	    },

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set('disabled', value);
			domAttr.remove(this.domNode, 'disabled');
		}
	
	//  // markup methods
	//  provide only attributes defined in base class
	//  markupFactory: function(params, node, clazz){
	//    console.log("markupFactory: params: ");
	//    console.log(params);
	//
	//    var layoutType = params[layoutType];
	//    console.log("markupFactory: layoutType: "+ layoutType);
	//    if (layoutType=="Question") {
	//      return new xcitec.xlayout.Question(params,node);
	//    }
	//    return new xcitec.Panel(params,node);
	//  }
	
	});
	
	return Panel;
});

},
'dijit/layout/utils':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/lang" // lang.mixin, lang.setObject
], function(array, domClass, domGeometry, domStyle, lang){

	// module:
	//		dijit/layout/utils

	function capitalize(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	}

	function size(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : domGeometry.setMarginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			lang.mixin(widget, newSize);
		}else{
			// otherwise, call getMarginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			lang.mixin(widget, domGeometry.getMarginBox(widget.domNode));
			lang.mixin(widget, dim);
		}
	}

	var utils = {
		// summary:
		//		Utility functions for doing layout

		marginBox2contentBox: function(/*DomNode*/ node, /*Object*/ mb){
			// summary:
			//		Given the margin-box size of a node, return its content box size.
			//		Functions like domGeometry.contentBox() but is more reliable since it doesn't have
			//		to wait for the browser to compute sizes.
			var cs = domStyle.getComputedStyle(node);
			var me = domGeometry.getMarginExtents(node, cs);
			var pb = domGeometry.getPadBorderExtents(node, cs);
			return {
				l: domStyle.toPixelValue(node, cs.paddingLeft),
				t: domStyle.toPixelValue(node, cs.paddingTop),
				w: mb.w - (me.w + pb.w),
				h: mb.h - (me.h + pb.h)
			};
		},


		layoutChildren: function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
				/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
			// summary:
			//		Layout a bunch of child dom nodes within a parent dom node
			// container:
			//		parent node
			// dim:
			//		{l, t, w, h} object specifying dimensions of container into which to place children
			// children:
			//		An array of Widgets or at least objects containing:
			//
			//		- domNode: pointer to DOM node to position
			//		- region or layoutAlign: position to place DOM node
			//		- resize(): (optional) method to set size of node
			//		- id: (optional) Id of widgets, referenced from resize object, below.
			//
			//		The widgets in this array should be ordered according to how they should be laid out
			//		(each element will be processed in order, and take up as much remaining space as needed),
			//		with the center widget last.
			// changedRegionId:
			//		If specified, the slider for the region with the specified id has been dragged, and thus
			//		the region's height or width should be adjusted according to changedRegionSize
			// changedRegionSize:
			//		See changedRegionId.

			// copy dim because we are going to modify it
			dim = lang.mixin({}, dim);

			domClass.add(container, "dijitLayoutContainer");

			// Move "client" elements to the end of the array for layout.  a11y dictates that the author
			// needs to be able to put them in the document in tab-order, but this algorithm requires that
			// client be last.    TODO: remove for 2.0, all dijit client code already sends children as last item.
			children = array.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
				.concat(array.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

			// set positions/sizes
			array.forEach(children, function(child){
				var elm = child.domNode,
					pos = (child.region || child.layoutAlign);
				if(!pos){
					throw new Error("No region setting for " + child.id)
				}

				// set elem to upper left corner of unused space; may move it later
				var elmStyle = elm.style;
				elmStyle.left = dim.l+"px";
				elmStyle.top = dim.t+"px";
				elmStyle.position = "absolute";

				domClass.add(elm, "dijitAlign" + capitalize(pos));

				// Size adjustments to make to this child widget
				var sizeSetting = {};

				// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
				// panes and width adjustment for left/right align panes.
				if(changedRegionId && changedRegionId == child.id){
					sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
				}

				if(pos == "leading"){
					pos = child.isLeftToRight() ? "left" : "right";
				}
				if(pos == "trailing"){
					pos = child.isLeftToRight() ? "right" : "left";
				}

				// set size && adjust record of remaining space.
				// note that setting the width of a <div> may affect its height.
				if(pos == "top" || pos == "bottom"){
					sizeSetting.w = dim.w;
					size(child, sizeSetting);
					dim.h -= child.h;
					if(pos == "top"){
						dim.t += child.h;
					}else{
						elmStyle.top = dim.t + dim.h + "px";
					}
				}else if(pos == "left" || pos == "right"){
					sizeSetting.h = dim.h;
					size(child, sizeSetting);
					dim.w -= child.w;
					if(pos == "left"){
						dim.l += child.w;
					}else{
						elmStyle.left = dim.l + dim.w + "px";
					}
				}else if(pos == "client" || pos == "center"){
					size(child, dim);
				}
			});
		}
	};

	lang.setObject("dijit.layout.utils", utils);	// remove for 2.0

	return utils;
});

},
'dijit/ProgressBar':function(){
define([
	"require", // require.toUrl
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/lang", // lang.mixin
	"dojo/number", // number.format
	"./_Widget",
	"./_TemplatedMixin",
	"dojo/text!./templates/ProgressBar.html"
], function(require, declare, domClass, lang, number, _Widget, _TemplatedMixin, template){

	// module:
	//		dijit/ProgressBar

	return declare("dijit.ProgressBar", [_Widget, _TemplatedMixin], {
		// summary:
		//		A progress indication widget, showing the amount completed
		//		(often the percentage completed) of a task.

		// progress: [const] String (Percentage or Number)
		//		Number or percentage indicating amount of task completed.
		//		Deprecated.   Use "value" instead.
		progress: "0",

		// value: String (Percentage or Number)
		//		Number or percentage indicating amount of task completed.
		//		With "%": percentage value, 0% <= progress <= 100%, or
		//		without "%": absolute value, 0 <= progress <= maximum.
		//		Infinity means that the progress bar is indeterminate.
		value: "",

		// maximum: [const] Float
		//		Max sample number
		maximum: 100,

		// places: [const] Number
		//		Number of places to show in values; 0 by default
		places: 0,

		// indeterminate: [const] Boolean
		//		If false: show progress value (number or percentage).
		//		If true: show that a process is underway but that the amount completed is unknown.
		//		Deprecated.   Use "value" instead.
		indeterminate: false,

		// label: String?
		//		HTML label on progress bar.   Defaults to percentage for determinate progress bar and
		//		blank for indeterminate progress bar.
		label: "",

		// name: String
		//		this is the field name (for a form) if set. This needs to be set if you want to use
		//		this widget in a dijit/form/Form widget (such as dijit/Dialog)
		name: '',

		templateString: template,

		// _indeterminateHighContrastImagePath: [private] URL
		//		URL to image to use for indeterminate progress bar when display is in high contrast mode
		_indeterminateHighContrastImagePath: require.toUrl("./themes/a11y/indeterminate_progress.gif"),

		postMixInProperties: function(){
			this.inherited(arguments);

			// Back-compat for when constructor specifies indeterminate or progress, rather than value.   Remove for 2.0.
			if(!(this.params && "value" in this.params)){
				this.value = this.indeterminate ? Infinity : this.progress;
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			this.indeterminateHighContrastImage.setAttribute("src",
				this._indeterminateHighContrastImagePath.toString());
			this.update();
		},

		_setDirAttr: function(val){
			// Normally _CssStateMixin takes care of this, but we aren't extending it
			domClass.toggle(this.domNode, "dijitProgressBarRtl", val == "rtl");
			this.inherited(arguments);
		},

		update: function(/*Object?*/attributes){
			// summary:
			//		Internal method to change attributes of ProgressBar, similar to set(hash).  Users should call
			//		set("value", ...) rather than calling this method directly.
			// attributes:
			//		May provide progress and/or maximum properties on this parameter;
			//		see attribute specs for details.
			// example:
			//	|	myProgressBar.update({'indeterminate': true});
			//	|	myProgressBar.update({'progress': 80});
			//	|	myProgressBar.update({'indeterminate': true, label:"Loading ..." })
			// tags:
			//		private

			// TODO: deprecate this method and use set() instead

			lang.mixin(this, attributes || {});
			var tip = this.internalProgress, ap = this.domNode;
			var percent = 1;
			if(this.indeterminate){
				ap.removeAttribute("aria-valuenow");
			}else{
				if(String(this.progress).indexOf("%") != -1){
					percent = Math.min(parseFloat(this.progress) / 100, 1);
					this.progress = percent * this.maximum;
				}else{
					this.progress = Math.min(this.progress, this.maximum);
					percent = this.maximum ? this.progress / this.maximum : 0;
				}
				ap.setAttribute("aria-valuenow", this.progress);
			}

			// Even indeterminate ProgressBars should have these attributes
			ap.setAttribute("aria-labelledby", this.labelNode.id);
			ap.setAttribute("aria-valuemin", 0);
			ap.setAttribute("aria-valuemax", this.maximum);

			this.labelNode.innerHTML = this.report(percent);

			domClass.toggle(this.domNode, "dijitProgressBarIndeterminate", this.indeterminate);
			tip.style.width = (percent * 100) + "%";
			this.onChange();
		},

		_setValueAttr: function(v){
			this._set("value", v);
			if(v == Infinity){
				this.update({indeterminate: true});
			}else{
				this.update({indeterminate: false, progress: v});
			}
		},

		_setLabelAttr: function(label){
			this._set("label", label);
			this.update();
		},

		_setIndeterminateAttr: function(indeterminate){
			// Deprecated, use set("value", ...) instead
			this._set("indeterminate", indeterminate);
			this.update();
		},

		report: function(/*float*/percent){
			// summary:
			//		Generates HTML message to show inside progress bar (normally indicating amount of task completed).
			//		May be overridden.
			// tags:
			//		extension

			return this.label ? this.label :
				(this.indeterminate ? "&#160;" : number.format(percent, { type: "percent", places: this.places, locale: this.lang }));
		},

		onChange: function(){
			// summary:
			//		Callback fired when progress updates.
			// tags:
			//		extension
		}
	});
});

},
'xcitec/form/ListBox':function(){
define([
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom",
	"dojo/query",
	"dojo/text!./templates/ListBox.html",
	"dijit/_WidgetBase",
	"dijit/form/_FormValueWidget",
	"xcitec/form/MultiSelectUtils",
  "./_ListBoxItem"
], function (dojo, array, declare, lang, domAttr, domClass, domConstruct, dom, query, template, _Widget, _FormValueWidget, MultiSelectUtils, _ListBoxItem) {

	return declare( [_FormValueWidget, MultiSelectUtils], {
	
	    value: "",
	    items: null, // array of items
	    type: "",
	    selItems: null,  // array of selected ListBoxItems separated by comma
	    selWidgets: null,  // array of selected ListBoxItem widgets
	    lastSelected: null,
	    tempSelWidgets: null, // array
	    multiple: "true",
	    baseClass: "xcitecListBox",
	    error: "",
	    required:false,
		icons: null, //array
	
	    templateString: template,
	
	    _parseSource: function() {
	        this.values = query(" > div", this.srcNodeRef).map(function(node) {
	            return domAttr.get(node, "value");
	        });
	        this.selectedRows = query(" > div", this.srcNodeRef).map(function(node) {
	            return domAttr.get(node, "selected");
	        });
		    this.icons = query(" > div", this.srcNodeRef).map(function(node) {
	            return domAttr.get(node, "icon");
	        });
	        /*this.labels = dojo.query(" > div", this.srcNodeRef).map(function(node) {
	            return String(node.innerHTML);
	        });*/
	        this.labels = query(" > div", this.srcNodeRef);
	        var _this = this;
	        array.forEach(this.labels, function(label, index) {
	            var itemWidget = new _ListBoxItem({
	                itemLabel:label,
	                itemIndex:index,
	                parentNode:_this,
	                value:_this.values[index],
	                isSelected:_this.selectedRows[index],
	                icon:_this.icons[index]
	            });
		        if(itemWidget.isSelected){
			        _this.lastSelected = itemWidget;
		        }
	            _this.itemsNode.appendChild(itemWidget.domNode);
	            _this.items = _this.items.concat(itemWidget);
	            dom.setSelectable(itemWidget.domNode, false);
	        });
	    },
	
	    postMixInProperties: function() {
	        //check usage:
	        this.items=[];
	        this.selItems=[];
	        this.selWidgets=[];
	        this.tempSelWidgets=[];
	        this.icons=[];
	
	        this.inherited(arguments);
	        this.value = this.value.split(",");
	        this.value = array.map(this.value, function(value){
	            return lang.trim(value);
	        });
	        if (this.multiple != "true") {
	            this.value = this.js2splice(this.value, 1);
	        }
	        if (this.error) {
	            this.state = "Error";
	        }
	        if (this.required) {
	            this.baseClass = this.baseClass + "Required";
	        }
	    },
	
		_fillContent: function() {
	        this._parseSource();
	        this.textbox.value = "";
	        this.reorderSelWidgets();
	        this.updateHiddenInput(true);
	    },
	
	    _onChange: function() {
	        if (this.readOnly || this.disabled) {
	            return;
	        }
	        // user defined function
	        this.onchange();
	    },
	
	    onchange: function() {
	        // custom event
	    },
	
	    selectRow: function(/* Widget */ node, /* Event */ evt) {
	        if (this.multiple == "true") {
	            this._multiSelect(node, evt);
	        } else {
	            if (!this.isNodeInList(this.selWidgets, node)) {
	                array.forEach(this.selWidgets, function(value) {
	                    value.setSelected(false);
	                });
	                this.selWidgets = this.clearList(this.selWidgets);
	                node.setSelected(true);
	                this.selWidgets = this.addToList(this.selWidgets, node);
	                this.updateHiddenInput();
	            } else {
	                if (evt.ctrlKey) {
	                    node.setSelected(false);
	                    this.selWidgets = this.clearList(this.selWidgets);
	                    this.updateHiddenInput();
	                }
	            }
	        }
	    },
	
	    _multiSelect: function(/* Widget */ node, /* Event */ evt) {
	        var index = node.itemIndex; // index of selected row
	        var sendValue = node.value ? node.value : node.itemLabel;
	        if (this.selWidgets.length >= 1) { // if there are selected rows
	            if (evt.ctrlKey) {
	                if (this.isNodeInList(this.selWidgets, node)) {
	                    this.selWidgets = this.delFromList(this.selWidgets, node);
	                    node.setSelected(false);
	                    this.updateHiddenInput();
	                } else {
	                    this.selWidgets = this.addToList(this.selWidgets, node);
	                    this.reorderSelWidgets();
	                    node.setSelected(true);
	                    this.lastSelected = node;
	                    this.updateHiddenInput();
	                }
	            } else if (evt.shiftKey) {
	                var _this = this;
	                if (index >= this.lastSelected.itemIndex) {
	                    array.forEach(_this.items, function(value) {
	                        if ((Number(value.itemIndex) >= Number(_this.lastSelected.itemIndex)) && (Number(value.itemIndex) <= Number(node.itemIndex))) {
	                            if (!(_this.isNodeInList(_this.selWidgets, value))) {
	                                _this.selWidgets = _this.addToList(_this.selWidgets, value);
	                                _this.reorderSelWidgets();
	                                value.setSelected(true);
	                            }
	                        }
	                    });
	                } else {
	                    array.forEach(_this.items, function(value) {
	                        if ((Number(value.itemIndex) >= Number(node.itemIndex)) && (Number(value.itemIndex) <= Number(_this.lastSelected.itemIndex))) {
	                            if (!(_this.isNodeInList(_this.selWidgets, value))) {
	                                _this.selWidgets = _this.addToList(_this.selWidgets, value);
	                                _this.reorderSelWidgets();
	                                value.setSelected(true);
	                            }
	                        }
	                    });
	                }
	                this.updateHiddenInput();
	            } else {
	                array.forEach(this.selWidgets, function(node) {
	                    node.setSelected(false);
	                });
	                this.selWidgets = this.clearList(this.selWidgets);
	                node.setSelected(true);
	                this.lastSelected = node;
	                this.selWidgets = this.addToList(this.selWidgets, node);
	                this.reorderSelWidgets();
	                this.updateHiddenInput();
	            }
	        } else { // if there aren't selected rows
	            if (!evt.shiftKey) {
	                this.selWidgets = this.addToList(this.selWidgets, node);
	                this.reorderSelWidgets();
	                node.setSelected(true);
	                this.lastSelected = node;
	                this.updateHiddenInput();
	            }
	        }
	    },
	
	    updateHiddenInput: function(/*Boolean*/ skipEvent) {
	        var res = array.map(this.selWidgets, function(item){
	            //var sendValue = item.value ? item.value : item.itemLabel;
	            return item.value;
	        });
	        this.textbox.value = res.join(",");
	        if(!skipEvent){
	            this._onChange();
	        }
	    },
	
	    reorderSelWidgets: function() {
	        var _this = this;
	        var numbers = [];
	        array.forEach(_this.selWidgets, function(widget) {
	            numbers = _this.addToList(numbers, widget.itemIndex);
	        });
	        numbers.sort(function(a, b) {
	            return a - b;
	        });
	        for (var i = 0; i <= numbers[numbers.length - 1]; i++) {
	            array.forEach(_this.selWidgets, function(widget) {
	                if (widget.itemIndex == i) {
	                    _this.tempSelWidgets = _this.addToList(_this.tempSelWidgets, widget);
	                }
	            });
	        }
	        this.selWidgets = this.clearList(this.selWidgets);
	        this.selWidgets = this.tempSelWidgets;
	        this.tempSelWidgets = this.clearList(this.tempSelWidgets);
	    }
	});
});

},
'xcitec/layout/_ContentAreaColumn':function(){
/*
 slmui.layout._ContentAreaColumn is internal holder for Label/Control
 */
define(["dojo/_base/declare",
        "dojo/dom-class",
        "./_LabelControlHolder"],
       function(declare, domClass, _LabelControlHolder){
  return declare(_LabelControlHolder, {

    templateString: '<div class="${baseClass}" ' +
                    '><div class="${baseClass}Inner${columnType}" data-dojo-attach-point="containerNode, controlHolderNode" ' +
                    '></div></div>',
    baseClass: "ContentAreaColumn",

    // Define which column type Left/Middle/Right
    columnType: "",

    setColumnType: function(/*Strin*/ type){
      /* Set which column it represent  Left/Middle/Right */
      if (this.columnType==type) return;
      if (this.columnType) {
        domClass.remove(this.containerNode, this.baseClass+"Inner"+this.columnType);
      }
      this.columnType = type
      if (this.columnType) {
        domClass.add(this.containerNode, this.baseClass+"Inner"+this.columnType);
      }
    }
  });
});

},
'xcitec/_base/wiky':function(){
define([], function () {

	/*	This work is licensed under Creative Commons GNU LGPL License.
	
	 License: http://creativecommons.org/licenses/LGPL/2.1/
	
	 Author:  Stefan Goessner/2005-06
	 Web:     http://goessner.net/
	 
	 kave: shrinksafe has a problem with \xXX values in strings, it replaces them into char as result script is broken
	       as workarround: \xXX replaced to \u00XX
	 */
	var Wiky = {
		version: 0.95,
		blocks: null,
		rules: {
			all: [
				"Wiky.rules.pre",
				"Wiky.rules.nonwikiblocks",
				"Wiky.rules.wikiblocks",
				"Wiky.rules.post"
			],
			pre: [
				{ rex:/(\r?\n)/g, tmplt:"\u00B6" }
				// replace line breaks with '�' ..
			],
			post: [
				{ rex:/(^\u00B6)|(\u00B6$)/g, tmplt:"" },
				// .. remove linebreaks at BOS and EOS ..
				{ rex:/@([0-9]+)@/g, tmplt:function($0, $1) {
					return Wiky.restore($1);
				} },
				// resolve blocks ..
				{ rex:/\u00B6/g, tmplt:"\n" } // replace '�' with line breaks ..
			],
			nonwikiblocks: [
				{ rex:/\\([%])/g, tmplt:function($0, $1) {
					return Wiky.store($1);
				} },
				{ rex:/\[(?:\{([^}]*)\})?(?:\(([^)]*)\))?%(.*?)%\]/g, tmplt:function($0, $1, $2, $3) {
					return ":p]" + Wiky.store("<pre" + ($2 ? (" lang=\"x-" + Wiky.attr($2) + "\"") : "") + Wiky.style($1) + ">" + Wiky.apply($3, $2 ? Wiky.rules.lang[Wiky.attr($2)] : Wiky.rules.code) + "</pre>") + "[p:";
				} } //programm code block
			],
			wikiblocks: [
				"Wiky.rules.nonwikiinlines",
				"Wiky.rules.escapes",
				"Wiky.rules.code",
				{ rex:/(?:^|\u00B6)(={1,6})(.*?)[=]*(?=\u00B6|$)/g, tmplt:function($0, $1, $2) {
					var h = $1.length;
					return ":p]\u00B6<h" + h + ">" + $2 + "</h" + h + ">\u00B6[p:";
				} }, // <h1> .. <h6>
				{ rex:/(?:^|\u00B6)[-]{4}(?:\u00B6|$)/g, tmplt:"\u00B6<hr/>\u00B6" },  // horizontal ruler ..
				{ rex:/\\\\([ \u00B6])/g, tmplt:"<br/>$1" },  // forced line break ..
				{ rex:/(^|\u00B6)([*01aAiIg]*[\.*])[ ]/g, tmplt:function($0, $1, $2) {
					var state = $2.replace(/([*])/g, "u").replace(/([\.])/, "");
					return ":" + state + "]" + $1 + "[" + state + ":";
				}},
				{ rex:/(?:^|\u00B6);[ ](.*?):[ ]/g, tmplt:"\u00B6:l][l:$1:d][d:"},  // ; term : definition
				{ rex:/\[(?:\{([^}]*)\})?(?:\(([^)]*)\))?\"/g, tmplt:function($0, $1, $2) {
					return ":p]<blockquote" + Wiky.attr($2, "cite", 0) + Wiky.attr($2, "title", 1) + Wiky.style($1) + ">[p:";
				} }, // block quotation start
				{ rex:/\"\]/g, tmplt:":p]</blockquote>[p:" }, // block quotation end
				{ rex:/\[(\{[^}]*\})?\|/g, tmplt:":t]$1[r:" },  // .. start table ..
				{ rex:/\|\]/g, tmplt:":r][t:" },  // .. end table ..
				{ rex:/\|\u00B6[ ]?\|/g, tmplt:":r]\u00B6[r:" },  // .. end/start table row ..
				{ rex:/\|/g, tmplt:":c][c:" },  // .. end/start table cell ..
				{ rex:/^(.*)$/g, tmplt:"[p:$1:p]" },  // start paragraph '[p:' at BOS .. end paragraph ':p]' at EOS ..
				{ rex:/(([\u00B6])([ \t\f\v\u00B6]*?)){2,}/g, tmplt:":p]$1[p:" },  // .. separate paragraphs at blank lines ..
				{ rex:/\[([01AIacdgilprtu]+)[:](.*?)[:]([01AIacdgilprtu]+)\]/g, tmplt:function($0, $1, $2, $3) {
					return Wiky.sectionRule($1 == undefined ? "" : $1, "", Wiky.apply($2, Wiky.rules.wikiinlines), !$3 ? "" : $3);
				} },
				{ rex:/\[[01AIacdgilprtu]+[:]|[:][01AIacdgilprtu]+\]/g, tmplt:"" },  // .. remove singular section delimiters (they frequently exist with incomplete documents while typing) ..
				{ rex:/<td>(?:([0-9]*)[>])?([ ]?)(.*?)([ ]?)<\/td>/g, tmplt:function($0, $1, $2, $3, $4) {
					return "<td" + ($1 ? " colspan=\"" + $1 + "\"" : "") + ($2 == " " ? (" style=\"text-align:" + ($2 == $4 ? "center" : "right") + ";\"") : ($4 == " " ? " style=\"text-align:left;\"" : "")) + ">" + $2 + $3 + $4 + "</td>";
				} },
				{ rex:/<(p|table)>(?:\u00B6)?(?:\{(.*?)\})/g, tmplt:function($0, $1, $2) {
					return "<" + $1 + Wiky.style($2) + ">";
				} },
				{ rex:/<p>([ \t\f\v\u00B6]*?)<\/p>/g, tmplt:"$1" },  // .. remove empty paragraphs ..
				"Wiky.rules.shortcuts"
			],
			nonwikiinlines: [
				{ rex:/%(?:\{([^}]*)\})?(?:\(([^)]*)\))?(.*?)%/g, tmplt:function($0, $1, $2, $3) {
					return Wiky.store("<code" + ($2 ? (" lang=\"x-" + Wiky.attr($2) + "\"") : "") + Wiky.style($1) + ">" + Wiky.apply($3, $2 ? Wiky.rules.lang[Wiky.attr($2)] : Wiky.rules.code) + "</code>");
				} },
				// inline code
				{ rex:/%(.*?)%/g, tmplt:function($0, $1) {
					return Wiky.store("<code>" + Wiky.apply($2, Wiky.rules.code) + "</code>");
				} }
			],
			wikiinlines: [
				{ rex:/\*([^*]+)\*/g, tmplt:"<strong>$1</strong>" },
				// .. strong ..
				{ rex:/_([^_]+)_/g, tmplt:"<em>$1</em>" },
				{ rex:/\^([^^]+)\^/g, tmplt:"<sup>$1</sup>" },
				{ rex:/~([^~]+)~/g, tmplt:"<sub>$1</sub>" },
				{ rex:/\(-(.+?)-\)/g, tmplt:"<del>$1</del>" },
				{ rex:/\?([^ \t\f\v\u00B6]+)\((.+)\)\?/g, tmplt:"<abbr title=\"$2\">$1</abbr>" },
				// .. abbreviation ..
				{ rex:/\[(?:\{([^}]*)\})?[Ii]ma?ge?\:([^ ,\]]*)(?:[, ]([^\]]*))?\]/g, tmplt:function($0, $1, $2, $3) {
					return Wiky.store("<img" + Wiky.style($1) + " src=\"" + $2 + "\" alt=\"" + ($3 ? $3 : $2) + "\" title=\"" + ($3 ? $3 : $2) + "\"/>");
				} },
				// wikimedia image style ..
				{ rex:/\[([^ ,]+)[, ]([^\]]*)\]/g, tmplt:function($0, $1, $2) {
					return Wiky.store("<a href=\"" + $1 + "\" target=\"_blank\">" + $2 + "</a>");
				}},
				// wiki block style uri's ..
				{ rex:/(((http(s?))\:\/\/)?[A-Za-z0-9\._\/~\-:]+\.(?:png|jpg|jpeg|gif|bmp))/g, tmplt:function($0, $1, $2) {
					return Wiky.store("<img src=\"" + $1 + "\" alt=\"" + $1 + "\"/>");
				} },
				// simple images ..
				{ rex:/((mailto\:|javascript\:|(news|file|(ht|f)tp(s?))\:\/\/)[A-Za-z0-9\.:_\/~%\-+&#?!=()@\x80-\xB5\xB7\xFF]+)/g, tmplt:"<a href=\"$1\" target=\"_blank\">$1</a>" }  // simple uri's ..
			],
			escapes: [
				{ rex:/\\([|*_~\^])/g, tmplt:function($0, $1) {
					return Wiky.store($1);
				} },
				{ rex:/\\&/g, tmplt:"&amp;" },
				{ rex:/\\>/g, tmplt:"&gt;" },
				{ rex:/\\</g, tmplt:"&lt;" }
			],
			shortcuts: [
				{ rex:/---/g, tmplt:"&#8212;" },
				// &mdash;
				{ rex:/--/g, tmplt:"&#8211;" },
				// &ndash;
				{ rex:/[\.]{3}/g, tmplt:"&#8230;"},
				// &hellip;
				{ rex:/<->/g, tmplt:"&#8596;"},
				// $harr;
				{ rex:/<-/g, tmplt:"&#8592;"},
				// &larr;
				{ rex:/->/g, tmplt:"&#8594;"}
				//&rarr;
			],
			code: [
				{ rex:/&/g, tmplt:"&amp;"},
				{ rex:/</g, tmplt:"&lt;"},
				{ rex:/>/g, tmplt:"&gt;"}
			],
			lang: {}
		},
	
		inverse: {
			all: [
				"Wiky.inverse.pre",
				"Wiky.inverse.nonwikiblocks",
				"Wiky.inverse.wikiblocks",
				"Wiky.inverse.post"
			],
			pre: [
				{ rex:/(\r?\n)/g, tmplt:"\u00B6" }  // replace line breaks with '�' ..
			],
			post: [
				{ rex:/@([0-9]+)@/g, tmplt:function($0, $1) {
					return Wiky.restore($1);
				} },
				// resolve blocks ..
				{ rex:/\u00B6/g, tmplt:"\n" }  // replace '�' with line breaks ..
			],
			nonwikiblocks: [
				{ rex:/<pre([^>]*)>(.*?)<\/pre>/mgi, tmplt:function($0, $1, $2) {
					return Wiky.store("[" + Wiky.invStyle($1) + Wiky.invAttr($1, ["lang"]).replace(/x\-/, "") + "%" + Wiky.apply($2, Wiky.hasAttr($1, "lang") ? Wiky.inverse.lang[Wiky.attrVal($1, "lang").substr(2)] : Wiky.inverse.code) + "%]");
				} } //code block
			],
			wikiblocks: [
				"Wiky.inverse.nonwikiinlines",
				"Wiky.inverse.escapes",
				"Wiky.inverse.wikiinlines",
				{ rex:/<h1>(.*?)<\/h1>/mgi, tmplt:"=$1=" },
				{ rex:/<h2>(.*?)<\/h2>/mgi, tmplt:"==$1==" },
				{ rex:/<h3>(.*?)<\/h3>/mgi, tmplt:"===$1===" },
				{ rex:/<h4>(.*?)<\/h4>/mgi, tmplt:"====$1====" },
				{ rex:/<h5>(.*?)<\/h5>/mgi, tmplt:"=====$1=====" },
				{ rex:/<h6>(.*?)<\/h6>/mgi, tmplt:"======$1======" },
				{ rex:/<(p|table)[^>]+(style=\"[^\"]*\")[^>]*>/mgi, tmplt:function($0, $1, $2) {
					return "<" + $1 + ">" + Wiky.invStyle($2);
				} },
				{ rex:/\u00B6{2}<li/mgi, tmplt:"\u00B6<li" },  // ie6 only ..
				{ rex:/<li class=\"?([^ >\"]*)\"?[^>]*?>([^<]*)/mgi, tmplt:function($0, $1, $2) {
					return $1.replace(/u/g, "*").replace(/([01aAiIg])$/, "$1.") + " " + $2;
				}},  // list items ..
				{ rex:/(^|\u00B6)<(u|o)l[^>]*?>\u00B6/mgi, tmplt:"$1" },  // only outer level list start at BOL ...
				{ rex:/(<\/(?:dl|ol|ul|p)>[ \u00B6]*<(?:p)>)/gi, tmplt:"\u00B6\u00B6" },
				{ rex:/<dt>(.*?)<\/dt>[ \f\n\r\t\v]*<dd>/mgi, tmplt:"; $1: " },
				{ rex:/<blockquote([^>]*)>/mgi, tmplt:function($0, $1) {
					return Wiky.store("[" + Wiky.invStyle($1) + Wiky.invAttr($1, ["cite","title"]) + "\"");
				} },
				{ rex:/<\/blockquote>/mgi, tmplt:"\"]" },
				{ rex:/<td class=\"?lft\"?>\u00B6*[ ]?|<\/tr>/mgi, tmplt:"|" },  // ie6 only ..
				{ rex:/\u00B6<tr(?:[^>]*?)>/mgi, tmplt:"\u00B6" },
				{ rex:/<td colspan=\"([0-9]+)\"(?:[^>]*?)>/mgi, tmplt:"|$1>" },
				{ rex:/<td(?:[^>]*?)>/mgi, tmplt:"|" },
				{ rex:/<table>/mgi, tmplt:"[" },
				{ rex:/<\/table>/mgi, tmplt:"]" },
				{ rex:/<tr(?:[^>]*?)>\u00B6*|<\/td>\u00B6*|<tbody>\u00B6*|<\/tbody>/mgi, tmplt:"" },
				{ rex:/<hr\/?>/mgi, tmplt:"----" },
				{ rex:/<br\/?>/mgi, tmplt:"\\\\" },
				{ rex:/(<p>|<(d|o|u)l[^>]*>|<\/(dl|ol|ul|p)>|<\/(li|dd)>)/mgi, tmplt:"" },
				"Wiky.inverse.shortcuts"
			],
			nonwikiinlines: [
				{ rex:/<code>(.*?)<\/code>/g, tmplt:function($0, $1) {
					return Wiky.store("%" + Wiky.apply($1, Wiky.inverse["code"]) + "%");
				} }
			],
			wikiinlines: [
				{ rex:/<strong[^>]*?>(.*?)<\/strong>/mgi, tmplt:"*$1*" },
				{ rex:/<b[^>]*?>(.*?)<\/b>/mgi, tmplt:"*$1*" },
				{ rex:/<em[^>]*?>(.*?)<\/em>/mgi, tmplt:"_$1_" },
				{ rex:/<i[^>]*?>(.*?)<\/i>/mgi, tmplt:"_$1_" },
				{ rex:/<sup[^>]*?>(.*?)<\/sup>/mgi, tmplt:"^$1^" },
				{ rex:/<sub[^>]*?>(.*?)<\/sub>/mgi, tmplt:"~$1~" },
				{ rex:/<del[^>]*?>(.*?)<\/del>/mgi, tmplt:"(-$1-)" },
				{ rex:/<abbr title=\"([^\"]*)\">(.*?)<\/abbr>/mgi, tmplt:"?$2($1)?" },
				{ rex:/<a href=\"([^\"]*)\"[^>]*?>(.*?)<\/a>/mgi, tmplt:function($0, $1, $2) {
					return $1 == $2 ? $1 : "[" + $1 + "," + $2 + "]";
				}},
				{ rex:/<img([^>]*)\/>/mgi, tmplt:function($0, $1) {
					var a = Wiky.attrVal($1, "alt"),h = Wiky.attrVal($1, "src"),t = Wiky.attrVal($1, "title"),s = Wiky.attrVal($1, "style");
					return s || (t && h != t) ? ("[" + Wiky.invStyle($1) + "img:" + h + (t && ("," + t)) + "]") : h;
				}}
			],
			escapes: [
				{ rex:/([|*_~%\^])/g, tmplt:"\\$1" },
				{ rex:/&amp;/g, tmplt:"\\&" },
				{ rex:/&gt;/g, tmplt:"\\>" },
				{ rex:/&lt;/g, tmplt:"\\<" }
			],
			shortcuts: [
				{ rex:/&#8211;|\u2013/g, tmplt:"--"},
				{ rex:/&#8212;|\u2014/g, tmplt:"---"},
				{ rex:/&#8230;|\u2026/g, tmplt:"..."},
				{ rex:/&#8596;|\u2194/g, tmplt:"<->"},
				{ rex:/&#8592;|\u2190/g, tmplt:"<-"},
				{ rex:/&#8594;|\u2192/g, tmplt:"->"}
			],
			code: [
				{ rex:/&amp;/g, tmplt:"&"},
				{ rex:/&lt;/g, tmplt:"<"},
				{ rex:/&gt;/g, tmplt:">"}
			],
			lang: {}
		},
	
		toHtml: function(str) {
			Wiky.blocks = [];
			return Wiky.apply(str, Wiky.rules.all);
		},
	
		toWiki: function(str) {
			Wiky.blocks = [];
			return Wiky.apply(str, Wiky.inverse.all);
		},
	
		apply: function(str, rules) {
			if (str && rules)
				for (var i in rules) {
					if (typeof(rules[i]) == "string")
						str = Wiky.apply(str, eval(rules[i]));
					else
						str = str.replace(rules[i].rex, rules[i].tmplt);
				}
			return str;
		},
		store: function(str, unresolved) {
			return unresolved ? "@" + (Wiky.blocks.push(str) - 1) + "@"
					: "@" + (Wiky.blocks.push(str.replace(/@([0-9]+)@/g, function($0, $1) {
				return Wiky.restore($1);
			})) - 1) + "@";
		},
		restore: function(idx) {
			return Wiky.blocks[idx];
		},
		attr: function(str, name, idx) {
			var a = str && str.split(",")[idx || 0];
			return a ? (name ? (" " + name + "=\"" + a + "\"") : a) : "";
		},
		hasAttr: function(str, name) {
			return new RegExp(name + "=").test(str);
		},
		attrVal: function(str, name) {
			return str.replace(new RegExp("^.*?" + name + "=\"(.*?)\".*?$"), "$1");
		},
		invAttr: function(str, names) {
			var a = [], x;
			for (var i in names)
				if (str.indexOf(names[i] + "=") >= 0)
					a.push(str.replace(new RegExp("^.*?" + names[i] + "=\"(.*?)\".*?$"), "$1"));
			return a.length ? ("(" + a.join(",") + ")") : "";
		},
		style: function(str) {
			var s = str && str.split(/,|;/), p, style = "";
			for (var i in s) {
				p = s[i].split(":");
				if (p[0] == ">")       style += "margin-left:4em;";
				else if (p[0] == "<")  style += "margin-right:4em;";
				else if (p[0] == ">>") style += "float:right;";
				else if (p[0] == "<<") style += "float:left;";
				else if (p[0] == "=") style += "display:block;margin:0 auto;";
				else if (p[0] == "_")  style += "text-decoration:underline;";
				else if (p[0] == "b")  style += "border:solid 1px;";
				else if (p[0] == "c")  style += "color:" + p[1] + ";";
				else if (p[0] == "C")  style += "background:" + p[1] + ";";
				else if (p[0] == "w")  style += "width:" + p[1] + ";";
				else                   style += p[0] + ":" + p[1] + ";";
			}
			return style ? " style=\"" + style + "\"" : "";
		},
		invStyle: function(str) {
			var s = /style=/.test(str) ? str.replace(/^.*?style=\"(.*?)\".*?$/, "$1") : "",
					p = s && s.split(";"), pi, prop = [];
			for (var i in p) {
				pi = p[i].split(":");
				if (pi[0] == "margin-left" && pi[1] == "4em") prop.push(">");
				else if (pi[0] == "margin-right" && pi[1] == "4em") prop.push("<");
				else if (pi[0] == "float" && pi[1] == "right") prop.push(">>");
				else if (pi[0] == "float" && pi[1] == "left") prop.push("<<");
				else if (pi[0] == "margin" && pi[1] == "0 auto") prop.push("=");
				else if (pi[0] == "display" && pi[1] == "block") ;
				else if (pi[0] == "text-decoration" && pi[1] == "underline") prop.push("_");
				else if (pi[0] == "border" && pi[1] == "solid 1px") prop.push("b");
				else if (pi[0] == "color") prop.push("c:" + pi[1]);
				else if (pi[0] == "background") prop.push("C:" + pi[1]);
				else if (pi[0] == "width") prop.push("w:" + pi[1]);
				else if (pi[0]) prop.push(pi[0] + ":" + pi[1]);
			}
			return prop.length ? ("{" + prop.join(",") + "}") : "";
		},
		sectionRule: function(fromLevel, style, content, toLevel) {
			var trf = { p_p: "<p>$1</p>",
				p_u: "<p>$1</p><ul$3>",
				p_o: "<p>$1</p><ol$3>",
				// p - ul
				// ul - p
				u_p: "<li$2>$1</li></ul>",
				u_c: "<li$2>$1</li></ul></td>",
				u_r: "<li$2>$1</li></ul></td></tr>",
				uu_p: "<li$2>$1</li></ul></li></ul>",
				uo_p: "<li$2>$1</li></ol></li></ul>",
				uuu_p: "<li$2>$1</li></ul></li></ul></li></ul>",
				uou_p: "<li$2>$1</li></ul></li></ol></li></ul>",
				uuo_p: "<li$2>$1</li></ol></li></ul></li></ul>",
				uoo_p: "<li$2>$1</li></ol></li></ol></li></ul>",
				// ul - ul
				u_u: "<li$2>$1</li>",
				uu_u: "<li$2>$1</li></ul></li>",
				uo_u: "<li$2>$1</li></ol></li>",
				uuu_u: "<li$2>$1</li></ul></li></ul></li>",
				uou_u: "<li$2>$1</li></ul></li></ol></li>",
				uuo_u: "<li$2>$1</li></ol></li></ul></li>",
				uoo_u: "<li$2>$1</li></ol></li></ol></li>",
				u_uu: "<li$2>$1<ul$3>",
				// ul - ol
				u_o: "<li$2>$1</li></ul><ol$3>",
				uu_o: "<li$2>$1</li></ul></li></ul><ol$3>",
				uo_o: "<li$2>$1</li></ol></li></ul><ol$3>",
				uuu_o: "<li$2>$1</li></ul></li></ul></li></ul><ol$3>",
				uou_o: "<li$2>$1</li></ul></li></ol></li></ul><ol$3>",
				uuo_o: "<li$2>$1</li></ol></li></ul></li></ul><ol$3>",
				uoo_o: "<li$2>$1</li></ol></li></ol></li></ul><ol$3>",
				u_uo: "<li$2>$1<ol$3>",
				// ol - p
				o_p: "<li$2>$1</li></ol>",
				oo_p: "<li$2>$1</li></ol></li></ol>",
				ou_p: "<li$2>$1</li></ul></li></ol>",
				ooo_p: "<li$2>$1</li></ol></li></ol>",
				ouo_p: "<li$2>$1</li></ol></li></ul></li></ol>",
				oou_p: "<li$2>$1</li></ul></li></ol></li></ol>",
				ouu_p: "<li$2>$1</li></ul></li></ul></li></ol>",
				// ol - ul
				o_u: "<li$2>$1</li></ol><ul$3>",
				oo_u: "<li$2>$1</li></ol></li></ol><ul$3>",
				ou_u: "<li$2>$1</li></ul></li></ol><ul$3>",
				ooo_u: "<li$2>$1</li></ol></li></ol></li></ol><ul$3>",
				ouo_u: "<li$2>$1</li></ol></li></ul></li></ol><ul$3>",
				oou_u: "<li$2>$1</li></ul></li></ol></li></ol><ul$3>",
				ouu_u: "<li$2>$1</li></ul></li></ul></li></ol><ul$3>",
				o_ou: "<li$2>$1<ul$3>",
				// -- ol - ol --
				o_o: "<li$2>$1</li>",
				oo_o: "<li$2>$1</li></ol></li>",
				ou_o: "<li$2>$1</li></ul></li>",
				ooo_o: "<li$2>$1</li></ol></li></ol></li>",
				ouo_o: "<li$2>$1</li></ol></li></ul></li>",
				oou_o: "<li$2>$1</li></ul></li></ol></li>",
				ouu_o: "<li$2>$1</li></ul></li></ul></li>",
				o_oo: "<li$2>$1<ol$3>",
				// -- dl --
				l_d: "<dt>$1</dt>",
				d_l: "<dd>$1</dd>",
				d_u: "<dd>$1</dd></dl><ul>",
				d_o: "<dd>$1</dd></dl><ol>",
				p_l: "<p>$1</p><dl>",
				u_l: "<li$2>$1</li></ul><dl>",
				o_l: "<li$2>$1</li></ol><dl>",
				uu_l: "<li$2>$1</li></ul></li></ul><dl>",
				uo_l: "<li$2>$1</li></ol></li></ul><dl>",
				ou_l: "<li$2>$1</li></ul></li></ol><dl>",
				oo_l: "<li$2>$1</li></ol></li></ol><dl>",
				d_p: "<dd>$1</dd></dl>",
				// -- table --
				p_t: "<p>$1</p><table>",
				p_r: "<p>$1</p></td></tr>",
				p_c: "<p>$1</p></td>",
				t_p: "</table><p>$1</p>",
				r_r: "<tr><td>$1</td></tr>",
				r_p: "<tr><td><p>$1</p>",
				r_c: "<tr><td>$1</td>",
				r_u: "<tr><td>$1<ul>",
				c_p: "<td><p>$1</p>",
				c_r: "<td>$1</td></tr>",
				c_c: "<td>$1</td>",
				//                  c_u: "<td>$1<ul>",
				u_t: "<li$2>$1</li></ul><table>",
				o_t: "<li$2>$1</li></ol><table>",
				d_t: "<dd>$1</dd></dl><table>",
				t_u: "</table><p>$1</p><ul>",
				t_o: "</table><p>$1</p><ol>",
				t_l: "</table><p>$1</p><dl>"
			};
			var type = { "0": "decimal-leading-zero",
				"1": "decimal",
				"a": "lower-alpha",
				"A": "upper-alpha",
				"i": "lower-roman",
				"I": "upper-roman",
				"g": "lower-greek" };
	
			var from = "", to = "", maxlen = Math.max(fromLevel.length, toLevel.length), sync = true, sectiontype = type[toLevel.charAt(toLevel.length - 1)], transition;
	
			for (var i = 0; i < maxlen; i++)
				if (fromLevel.charAt(i + 1) != toLevel.charAt(i + 1) || !sync || i == maxlen - 1) {
					from += fromLevel.charAt(i) == undefined ? " " : fromLevel.charAt(i);
					to += toLevel.charAt(i) == undefined ? " " : toLevel.charAt(i);
					sync = false;
				}
			transition = (from + "_" + to).replace(/([01AIagi])/g, "o");
			return !trf[transition] ? ("?(" + transition + ")")  // error string !
					: trf[transition].replace(/\$2/, " class=\"" + fromLevel + "\"")
					.replace(/\$3/, !sectiontype ? "" : (" style=\"list-style-type:" + sectiontype + ";\""))
					.replace(/\$1/, content)
					.replace(/<p><\/p>/, "");
		}
	}
	
	return Wiky;
});

},
'dijit/_FocusMixin':function(){
define([
	"./focus",
	"./_WidgetBase",
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.extend
], function(focus, _WidgetBase, declare, lang){

	// module:
	//		dijit/_FocusMixin

	// We don't know where _FocusMixin will occur in the inheritance chain, but we need the _onFocus()/_onBlur() below
	// to be last in the inheritance chain, so mixin to _WidgetBase.
	lang.extend(_WidgetBase, {
		// focused: [readonly] Boolean
		//		This widget or a widget it contains has focus, or is "active" because
		//		it was recently clicked.
		focused: false,

		onFocus: function(){
			// summary:
			//		Called when the widget becomes "active" because
			//		it or a widget inside of it either has focus, or has recently
			//		been clicked.
			// tags:
			//		callback
		},

		onBlur: function(){
			// summary:
			//		Called when the widget stops being "active" because
			//		focus moved to something outside of it, or the user
			//		clicked somewhere outside of it, or the widget was
			//		hidden.
			// tags:
			//		callback
		},

		_onFocus: function(){
			// summary:
			//		This is where widgets do processing for when they are active,
			//		such as changing CSS classes.  See onFocus() for more details.
			// tags:
			//		protected
			this.onFocus();
		},

		_onBlur: function(){
			// summary:
			//		This is where widgets do processing for when they stop being active,
			//		such as changing CSS classes.  See onBlur() for more details.
			// tags:
			//		protected
			this.onBlur();
		}
	});

	return declare("dijit._FocusMixin", null, {
		// summary:
		//		Mixin to widget to provide _onFocus() and _onBlur() methods that
		//		fire when a widget or its descendants get/lose focus

		// flag that I want _onFocus()/_onBlur() notifications from focus manager
		_focusManager: focus
	});

});

},
'slmui/layout/_ChapterBase':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/lang',
	'dojo/_base/array',
	'dojo/dom-style',
	'dijit/layout/_LayoutWidget',
	'dijit/_TemplatedMixin',
	'xcitec/xcitecUtil',
	'./_XcitecCompatibleLayoutMixin'
], function(declare, lang, array, domStyle, _LayoutWidget, _TemplatedMixin, xcitecUtil, _XcitecCompatibleLayoutMixin){

	// summary:
	//		Base class for Chapter layouts.

	return declare([_LayoutWidget, _TemplatedMixin, _XcitecCompatibleLayoutMixin], {

		baseClass: 'fieldsetChapter',

		// caption: String
		//		Chapter's title.
		caption: '',

		// captionId: String
		//		Caption id for i18n support.
		captionId: '',

		// frameBox: String
		//		Background color
		frameBox: '',

		buildRendering: function(){
			this.inherited(arguments);

			if(!this.caption){
				domStyle.set(this.captionContainerNode, "display", "none");
			}
		},

		layout: function(){
			this.inherited(arguments);

			array.forEach(this.getChildren(), function(child){
				if(lang.isFunction(child.resize)){
					child.resize();
				}
			});
		},

		_setCaptionAttr: function(value){
			// summary:
			//		Setter for caption.

			if(value){
				this.caption = xcitecUtil.escapeXml(value);

				if(this.captionNode){
					this.captionNode.innerHTML = this.caption;
				}
			}
		},

		_setFrameBoxAttr: function(value){
			// summary:
			//		Setter for frameBox.

			if(value){
				this.frameBox = value;

				var fb = value.toLowerCase().replace(/-(.)/g, function(match){
					return match.toUpperCase();
				});

				this.baseClass += ' frameBox frameBox' + fb.charAt(0).toUpperCase() + fb.slice(1);
			}
		}
	});
});
},
'idx/util':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["dojo/_base/lang",
        "idx/main",
        "dojo/_base/kernel",
        "dojo/has",
		"dojo/aspect",
        "dojo/_base/xhr",
        "dojo/_base/window",
        "dojo/_base/url",
        "dojo/date/stamp",
        "dojo/json",
        "dojo/string",
        "dojo/dom-class",
        "dojo/dom-style",
        "dojo/dom-attr",
        "dojo/dom-construct",
        "dojo/dom-geometry",
        "dojo/io-query",
        "dojo/query",
        "dojo/NodeList-dom",
		"dojo/Stateful",
        "dijit/registry",
        "dijit/form/_FormWidget",
        "dijit/_WidgetBase",
        "dojo/_base/sniff",
        "dijit/_base/manager"], // temporarily resolve dijit.byid() uncaught exception issue in aprser 
		function(dLang,				// dojo/_base/lang
				 iMain,				// idx
		         dKernel, 			// dojo/_base/kernel
		     	 dHas,				// dojo/has
				 dAspect,			// dojo/aspect
		     	 dXhr,				// dojo/_base/xhr
   		    	 dWindow,			// dojo/_base/window
   		     	 dURL,				// dojo/_base/url
   		     	 dDateStamp,		// dojo/date/stamp
	   		     dJson,				// dojo/json
	   		     dString,			// dojo/string
				 dDomClass,			// dojo/dom-class (for dDomClass.add)
				 dDomStyle,			// dojo/dom-style (for dDomStyle.getComputedStyle/set)
				 dDomAttr,			// dojo/dom-attr
				 dDomConstruct,		// dojo/dom-construct
				 dDomGeo,			// dojo/dom-geometry (for dDomGeo.getMarginBox)
				 dIOQuery,			// dojo/io-query
				 dQuery,			// dojo/query
				 dNodeList,			// dojo/NodeList-dom
				 dStateful,			// dojo/Stateful
				 dRegistry,			// dijit/registry
				 dFormWidget,		// dijit/form/_FormWidget
				 dWidget)			// dijit/_WidgetBase
{
    /**
 	 * @name idx.util
 	 * @namespace Provides Javascript utility methods in addition to those provided by Dojo.
 	 */
	var iUtil = dLang.getObject("util", true, iMain);
	
	/**
  	 * @public
  	 * @function
 	 * @name idx.util.getVersion
 	 * @description Returns the IDX toolkit version string from the "version.txt" file embedded in the toolkit.
 	 * @param {Boolean} full A boolean value indicating if the full version or partial version is desired.
 	 */
	iUtil.getVersion = function(full) {
		var params = {
			url: dKernel.moduleUrl("idx", "version.txt"),
			showProgress: false,
			handleAs: "json",
			load: function(response, ioArgs) {
				var msg = response.version;
				if(full) {
					msg += "-";
					msg += response.revision;
				}
				console.debug(msg);
			},
			error: function(response, ioArgs) {
				console.debug(response);
				return;
			}
		};
		dXhr.xhrGet(params);
	};
	
	/**
  	 * @public
  	 * @function
 	 * @name idx.util.getOffsetPosition
 	 * @description Returns the pixel offset from top-left for a given node relative to a "root" node.
 	 * @param {Node} node The node for which the offset is desired.
 	 * @param {Node} root The optional ancestor node of the first specified node.  If not specified
 	 *                    then the Window body tag is used.
 	 * @return {Object} The returned value has two fields: "l" and "t" representing "left offset" and 
 	 *                  "top offset", respectively.  Results are undefined if the "root" node is not an
 	 *                  ancestor of the first specified node.
 	 */
	iUtil.getOffsetPosition = function(node, root) {
		var body = dWindow.body();
		root = root || body;
		var n = node;
		
		var l = 0;
		var t = 0;
		
		while (n !== root) {
			// avoid infinite loop if root is not ancestor of node
			if (n === body) throw "idx.util.getOffsetPosition: specified root is not ancestor of specified node";
			
			// otherwise accumulate the offsets and move on to the parent
			l += n.offsetLeft;
			t += n.offsetTop;
			n = n.offsetParent;
		}
		return {l: l, t: t};
	};

	/**
  	 * @public
  	 * @function
 	 * @name idx.util.typeOfObject
 	 * @description Provides a type for the specified object based on the locally-scoped
 	 *              "val2type" function from "dojo/parser" module.
 	 * @param {Any} value The value for which the type is desired.
 	 * @return {String} Possible return values include: "string", "undefined", "number", "boolean", 
 	 *                  "function", "array", "date", "url" and "object".
 	 */
	iUtil.typeOfObject = function(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(dLang.isString(value)){ return "string"; }
		if(typeof value == "undefined") { return "undefined"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(dLang.isFunction(value)){ return "function"; }
		if(dLang.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof dURL){ return "url"; }
		return "object";
	};

	/**
  	 * @public
  	 * @function
 	 * @name idx.util.typeOfObject
 	 * @description Provides conversion of one object (or string) to another type based on
 	 *              the locally-scoped "str2obj" function from the "dojo/parser" module.
 	 * @param {String|Any} value The value to be converted.
 	 * @param {String} type The Type to convert the object to ("string", "number", "boolean", 
 	 *                      "function", "array", "date", or "url").  If not provided then 
 	 *                      string values are parsed as JSON and non-string value are returned as-is.
 	 * 
 	 * @return {Object} The converted value of the object according to the specified "type".
 	 */
	iUtil.parseObject = function(/*Object*/ value, /*String*/ type){
		var lastIndex = 0;
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "regex": 
				value = "" + value;
				lastIndex = value.lastIndexOf('/');
				if ((value.length>2) && (value.charAt(0) == '/') && (lastIndex > 0)) {
					return new RegExp(value.substring(1,lastIndex), 
									  ((lastIndex==value.length-1)?undefined:value.substring(lastIndex+1)));
				} else {
					return new RegExp(value);
				}
				break;
			case "null":
				return null;
			case "undefined":
				return undefined;
			case "string":
				return "" + value;
			case "number":
				if (typeof value == "number") return value;
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".  interpret as true.
				return (typeof value == "boolean") ? value : !(value.toLowerCase()=="false");
			case "function":
				if(dLang.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }"). 
					//  Therefore, convert to string
					value=value.toString();
					value=dLang.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return dLang.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				if (dLang.isArray(value)) return value;
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				if (value instanceof Date) return value;
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: {
						return dDateStamp.fromISOString(value);
					}
				}
			case "url":
				if(value instanceof dURL){ return value; }
				return dKernel.baseUrl + value;
			default:
				if (iUtil.typeOfObject(value) == "string") {
					return dJson.parse(value);
				} else {
					return value;
				}
		}
	};

	/**
  	 * @public
  	 * @function
 	 * @name idx.util.getCSSOptions
 	 * @description Creates a temporary div as a child of the optional parent (otherwise the body node),
 	 *              applies the specified CSS class to the div and extracts the query-string from the 
 	 *              background image of the created div to determine the CSS options.  The parameters in
 	 *              query string are converted to specific object types according to the optionally specified
 	 *              "guide" object which is checked for attributes with names in common with the provided 
 	 *              parameters (see idx.util.mixin).  This conversion is done in much the same way Dojo 1.6 
 	 *              parser converted string attribute values to the proper type for the created widget's attributes.
 	 * @param {String} className The CSS class name to use for getting the options.
 	 * @param {Node} parentNode The optional (but recommended) parent node parameter for the scope in which
 	 *                          to create the temporary node.
 	 * @param {Object} guide The optional guide to help in converting the CSS options to objects. 
 	 * 
 	 * @return {Object} The Object containing the CSS options with the attribute types matching the
 	 *                  specified guide where applicable.
 	 */
    iUtil.getCSSOptions = function(/*String*/  className,
                                   /*Node?*/   parentNode,
                                   /*Object?*/ guide,
                                   /*Object?*/ fallback) {
            var body = dWindow.body();
            if ((! parentNode) || (("canHaveHTML" in parentNode) && (! parentNode.canHaveHTML))) {
                parentNode = body;
            }
            
            // determine if the parent node is rooted in the body
            var rooted = false;
            var trav = parentNode;
            while (trav && (!rooted)) {
            	if (trav === body) rooted = true;
            	trav = trav.parentNode;
            }
            var root = null;
			if (!rooted) {
				trav = parentNode;
				var classes = [];
				while (trav) {
					classes.push(dDomAttr.get(trav, "class"));
					trav = trav.parentNode;
				}
				classes.reverse();
				var root = dDomConstruct.create("div",{style: "visibility: hidden; display: none;"},body,"last");
				var trav = root; 
				for (var index = 0; index < classes.length; index++) {
					var classAttr = classes[index];
					if (classAttr) classAttr = dString.trim(classAttr);
					if (classAttr && classAttr.length == 0) classAttr = null;
					var attrs = (classAttr) ? {"class": classAttr} : null;
					trav = dDomConstruct.create("div", attrs, trav, "last");
				}					
				parentNode = trav;
			}
			            
            var optionElem = dDomConstruct.create("div", null, parentNode);
        	dDomClass.add(optionElem, className);
        	var myStyle = dDomStyle.getComputedStyle(optionElem);
        	var bgImage = null;
        	if (myStyle) {
        		bgImage = "" + myStyle.backgroundImage;
        	}
        	dDomConstruct.destroy(optionElem);
            if (root) dDomConstruct.destroy(root);
            
            var noURL = ((! bgImage)
            			  || (bgImage.length < 5) 
            			  || (bgImage.toLowerCase().substring(0, 4) != "url(")
            			  || (bgImage.charAt(bgImage.length - 1) != ")"));
            			  
            var options = null;
            if (noURL && (bgImage == null || bgImage == "none") && fallback && (!dLang.isString(fallback))) {
            	options = fallback;
            } 
            
            if (! options) {
            	var cssOpts = null;
	            if (noURL && (bgImage == null || bgImage == "none") && fallback && dLang.isString(fallback)) {
            		cssOpts = fallback;
            		
            	} else if (!noURL) {
		            // remove the "url(" prefix and ")" suffix
    		        bgImage = bgImage.substring(4, bgImage.length - 1);
            
        			// check if our URL is quoted
	        	    if (bgImage.charAt(0) == "\"") {
    	        	    // if not properly quoted then we don't parse it
        	        	if (bgImage.length < 2) return null;
            	    	if (bgImage.charAt(bgImage.length - 1) != "\"") return null;
                
	                	// otherwise remove the quotes
    	            	bgImage = bgImage.substring(1, bgImage.length - 1);
            		}
            
	        		// find the query string
    	        	var queryIdx = bgImage.lastIndexOf("?");
            		var slashIdx = bgImage.lastIndexOf("/");
            		if (queryIdx < 0) return null;
            		if (queryIdx < slashIdx) return null;
            
        			// get just the query string from the URL
            		cssOpts = bgImage.substring(queryIdx + 1, bgImage.length);
            	}         
            	if (cssOpts == null) return null;
            	if (cssOpts.length == 0) return null;
            
        		// parse the query string and return the result
            	options = dIOQuery.queryToObject(cssOpts);
            }
            return (guide) ? iUtil.mixin({}, options, guide) : options;
        };
        
	/**
  	 * @public
  	 * @function
 	 * @name idx.util.mixinCSSDefaults
 	 * @description Obtains the "CSS Options" using idx.util.getCSSOptions via the specified CSS class name
 	 *              and optional parent node using the specified target object as the "guide" and mixing the
 	 *              CSS options directly into the target object via idx.util.mixin.  Any CSS options not 
 	 *              matching an attribute of the target object are ignored.
	 * 	 
 	 * @param {Object} target The target object to mix the CSS defaults into.
 	 * @param {String} className The CSS class name passed to idx.util.getCSSOptions.
 	 * @param {Node} parentNode The optional (but recommended) parent node passed to idx.util.getCSSOptions.
 	 * 
 	 * @return {Object} The Object containing the CSS options with the attribute types matching the
 	 *                  specified guide where applicable.
 	 */
    iUtil.mixinCSSDefaults = function(/*Object*/ target,
                                      /*String*/ className,
                                      /*Node?*/  parentNode) {
            if (!target) return null;
            var opts = iUtil.getCSSOptions(className, parentNode);

            if (!opts) return null;
            
            iUtil.mixin(target, opts);
            
            return opts;
        };

	/**
  	 * @public
  	 * @function
 	 * @name idx.util.mixin
 	 * @description Provides a restricted version of dojo.lang.mixin.  This function Mixes the attributes of the 
 	 *              specified "source" into the specified "target" but only if the specified target object (or
 	 *              optionally specified "guide" object) already has the attribute from the source object.  Further,
 	 *              the type of each attribute that is mixed in is interpretted to match the type of the same 
 	 *              attribute in the target (or rather the "guide" object if provided).  If no guide is specified, 
 	 *              then the target object is used as a guide.  If a guide is specified the the target object simply
 	 *              becomes the landing zone for the mixed-in attributes.
	 * 	 
 	 * @param {Object} target The target object to mix the attributes into.
 	 * @param {Object} source The source object to pull the attributes from.
 	 * @param {Object} guide The optional object whose attributes and types of those attributes will be used 
 	 *                       as a guide for converting the source attributes to the target attribute.
 	 * @return The specified target object is returned.
 	 */
     iUtil.mixin = function(/*Object*/  target,
                            /*Object*/  source,
                            /*Object?*/ guide) {
     	if (!target) return null; 	// cannot mixin to null
        if (!source) return target;	// if nothing to mixin then do nothing
        if (!guide) guide = target;	// if no guide is specified then use the target as the guide
        
        var src = { };
        // if we have the class info, then parse the fields of the options
        for (var field in source) {
	       	if (! (field in guide)) continue;
    	        var attrType = iUtil.typeOfObject(guide[field]);
                src[field] = iUtil.parseObject(source[field], attrType);
        }
            
         // mixin the options
         dLang.mixin(target, src);
         return target;
     };
        
    /**
  	 * @public
  	 * @function
 	 * @name idx.util.recursiveMixin
 	 * @description Recursively mixes in the second specified object into the first, optionally using
 	 *              the specifed options.  Recursion occurs when the attribute is contained in both
 	 *              the first and second object and is of type "object" in both cases.  Recursion can
 	 *              be made optional via the "options" parameter by specifying the name of a "controlField"
 	 *              and "controlValue".  In such cases the first object is checked for the presence of the
 	 *              "controlField" and if it exists and the value is equal to the specified "controlValue"
 	 *              then recusion occurs, otherwise it does not.  When recursion does not occur an an object
 	 *              value from the second object is copied it may optionally be cloned by setting "options.clone"
 	 *              to true.
     * @param {Object} first The object the attributes will be mixed into
     * @param {Object} second The object that holds the attributes to mixin
     * @param {Objet} options (includes "clone", "controlField" and "controlValue").  If "clone" is 
     *                        specified then attributes whose values are of type object in the second 
     *                        object are cloned before being set in the first object.  The "controlField"
     *                        and "controlValue" options are used to determine if an object in the first
     *                        object should be recursively mixed in.  If "controlField" is provided, but
     *                        not "controlValue" then "controlValue" is defaulted to true.
     */
     iUtil.recursiveMixin = function(first, second, options) {
        	var clone = null;
        	var controlField = null;
        	var controlValue = null;
        	if (options) {
        		clone = options.clone;
        		controlField = options.controlField;
        		if ("controlValue" in options) {
        			controlValue = options.controlValue;
        		} else {
        			controlValue = true;
        		}
        	}
        	
        	for (field in second) {
        		if (field in first) {
        			// get the field values
        			var firstValue = first[field];
        			var secondValue = second[field];
        			
        			// get the types for the values
        			var firstType = iUtil.typeOfObject(firstValue);
        			var secondType = iUtil.typeOfObject(secondValue);

        			// check if they are not the same type
        			if ((firstType == secondType) && (firstType == "object")
        				&& ((!controlField) || (firstValue[controlField] == controlValue))) {
        				// if both are objects then mix the second into the first
        				iUtil.recursiveMixin(firstValue, secondValue, options);
        				
        			} else {
        				// otherwise overwrite the first with the second
        				first[field] = (clone) ? dLang.clone(secondValue) : secondValue;
        			}
        		} else {
        			first[field] = (clone) ? dLang.clone(second[field]) : second[field];
        		}
        	}
        };
        
        
    /**
  	 * @public
  	 * @function
 	 * @name idx.util.nullify
 	 * @description Accepts a target object, an object that represents arguments passed
     *              to construct the target object (usually via mixin). and an
     *              and an array of property names for which the value should be null if
     *              not otherwise specified.  For each of the specified properties, if 
     *              the construction arguments does not specify a value for that property, 
     *              then the same property is set to null on the target object.  If a 
     *              property name is found not to exist in the target object then it is 
     *              ignored.
     *
     * @param {Object} target Usually the object being constructed.
     * 
     * @param {Object} ctorArgs The objects that would specify attributes on the target.
     * 
     * @param {Object} props The array of property names for properties to be set to null
     *                       if none of the objects in the argsArray specify them.
     */
    iUtil.nullify = function(target,ctorArgs,props) {
        var index = 0;
        for (index = 0; index < props.length; index++) {
            var prop = props[index];
            if (! (prop in target)) continue;
            if ((ctorArgs) && (prop in ctorArgs)) continue;
            target[prop] = null;
        }
    };
        
    /**
  	 * @private
  	 * @function
 	 * @name idx.util._getNodeStyle
     * @description Internal method to get the node's style as an object.  This method does not
     *              normalize the style fields so it will need to be extended to make it more
     *              robust for the general case.  idx.util only needs this for things like "position",
     *              "width" and "height" in order to reset values on a node after changing them.
     */
    iUtil._getNodeStyle = function(node) {
        	var nodeStyle = dDomAttr.get(node, "style");
        	if (!nodeStyle) return null;
        	var result = null;
       		if (iUtil.typeOfObject(nodeStyle) == "string") {
       			result = {};
       			var tokens = nodeStyle.split(";");
       			for (var index = 0; index < tokens.length; index++) {
       				var token = tokens[index];
       				var colonIndex = token.indexOf(":");
       				if (colonIndex < 0) continue;
       				var field = token.substring(0, colonIndex);
       				var value = "";
       				if (colonIndex < token.length - 1) {
       					value = token.substring(colonIndex+1);
       				}
       				result[field] = value;
       			}
     		} else {
     			result = nodeStyle;
     		}
       		return result;
        };

    /**
  	 * @private
  	 * @function
 	 * @name idx.util._getNodePosition
     * @description Internal method to get the node's specific position and detect when none is specifically
     *              assigned to the node.
     */
    iUtil._getNodePosition = function(node) {
        	var style = iUtil._getNodeStyle(node);
        	if (! style) return "";
        	if (! style.position) return "";
        	return style.position;
    };
        
    /**
  	 * @public
  	 * @function
 	 * @name idx.util.fitToWidth
     * @description Sizes a parent to fit the child node as if the child node's positioning was
     *              NOT absolute.  Absolutely positioned elements due not "reserve" space, so this
     *              method will temporarily position the element as "static", then determine the
     *              result size of the parent, set the parent's width explicitly, and then return the
     *              child to the default previously set positioning.  This is especially handy in that
     *              it allows the parent to define padding which will be respected.
     *
     * @param {Node} parent The parent node -- no checking is done to ensure this node is actually 
     *                      a parent of the specified child.
     * @param {Node} child The child node -- no checking is done to ensure this node is actually a
     *                     child of the specified parent.
     */
    iUtil.fitToWidth = function(/*Node*/ parent, /*Node*/ child) {
        	var pos = iUtil._getNodePosition(child);
            dDomStyle.set(parent, {width: "auto"});
            dDomStyle.set(child, {position: "static"});
            var dim = dDomGeo.getMarginBox(parent);
            dDomStyle.set(parent, {width: dim.w + "px"});
            dDomStyle.set(child, {position: pos});  
            return dim;
    };
        
    /**
  	 * @public
  	 * @function
 	 * @name idx.util.fitToHeight
 	 * @description Sizes a parent to fit the child node as if the child node's positioning was
 	 *              NOT absolute.  Absolutely positioned elements due not "reserve" space, so 
 	 *              this method will temporarily position the element as "static", then determine
 	 *              the result size of the parent, set the parent's height explicitly, and then 
 	 *              return the child to the default previously set positioning.  This is especially
 	 *              handy in that it allows the parent to define padding which will be respected.
     *
     * @param {Node} parent The parent node -- no checking is done to ensure this node is actually 
     *                      a parent of the specified child.
	 * @param {Node} child The child node -- no checking is done to ensure this node is actually a
	 *                     child of the specified parent.
 	 */
    iUtil.fitToHeight = function(/*Node*/ parent, /*Node*/ child) {
        	var pos = iUtil._getNodePosition(child);
            dDomStyle.set(parent, {height: "auto"});
            dDomStyle.set(child, {position: "static"});
            var dim = dDomGeo.getMarginBox(parent);
            dDomStyle.set(parent, {height: dim.h + "px"});
            dDomStyle.set(child, {position: pos});  
            return dim;
    };
       
    /**
  	 * @public
  	 * @function
 	 * @name idx.util.fitToSize
 	 * @description Sizes a parent to fit the child node as if the child node's positioning was 
 	 *              NOT absolute.  Absolutely positioned elements due not "reserve" space, so this
 	 *              method will temporarily position the element as "static", then determine the 
 	 *              result size of the parent, set the parent's size explicitly, and then return the
	 *              child to the default previously set positioning.  This is especially handy in 
	 *              that it allows the parent to define padding which will be respected.
	 * 
     * @param {Node} parent The parent node -- no checking is done to ensure this node is actually 
     *                      a parent of the specified child.
     *
     * @param {Node} child The child node -- no checking is done to ensure this node is actually a 
     *                     child of the specified parent.
     */
    iUtil.fitToSize = function(/*Node*/ parent, /*Node*/ child) {
        	var pos = iUtil._getNodePosition(child);
            dDomStyle.set(parent, {width: "auto", height: "auto"});
            dDomStyle.set(child, {position: "static"});
            var dim = dDomGeo.getMarginBox(parent);
            dDomStyle.set(parent, {width: dim.w + "px", height: dim.h + "px"});
            dDomStyle.set(child, {position: pos});
            return dim;
        };
     
     /**
      * @public
  	  * @function
 	  * @name idx.util.getStaticSize
 	  * @description Determines the  dimensions of the specified node if it were to use static positioning.
      *
      * @param {Node} node The node to work with.
      */  
     iUtil.getStaticSize = function(/*Node*/ node) {
        	var style = iUtil._getNodeStyle(node);
        	var pos = (style && style.position) ? style.position: "";
            var width  = (style && style.width) ? style.width : "";
            var height = (style && style.height) ? style.height : "";
            dDomStyle.set(node, {position: "static", width: "auto", height: "auto"});
            var dim = dDomGeo.getMarginBox(node);
            dDomStyle.set(node, {position: pos, width: width, height: height});
            return dim;
        };
        
     /**
      * @public
  	  * @function
 	  * @name idx.util.reposition
 	  * @description Determines the  dimensions of the specified node if it were to use static positioning.
      *
      * @param {Node} node The node to work with.
      * @param {String} position The CSS position to use.
      */  
     iUtil.reposition = function(/*Node*/ node, /*String*/ position) {
        	var oldpos = iUtil._getNodePosition(node);
            dDomStyle.set(node, {position: position});
            return oldpos;
     };

		        
     /**
      * @public
  	  * @function
 	  * @name idx.util.getParentWidget
 	  * @description Determines the widget that is the parent of the specified widget or node.  This 
 	  *              is determined by obtaining the parent node for the specified node or "widget.domNode"
 	  *              and then calling dijit.getEnclosingWidget().  This method may return null if no widget
 	  *              parent exists.
      *
      * @param {Node|Widget} child The node or dijit._WidgetBase child for which the parent is being requested.
      * @param {Type} widgetType The optional type of widget for the parent.  The method recursively looks for 
      *                          the first ancestor of this type until found or we run out of ancestors.
      *
      * @return {Widget} The parent or ancestor widget.
      */  
     iUtil.getParentWidget = function(/*Node|Widget*/ child,
     								  /*Type*/        widgetType) {
            // get the widget node
            var childNode = (child instanceof dWidget) ? child.domNode : child;
            
            // get the parent node of the DOM node
            var parentNode = childNode.parentNode;
            
            // check the parent node
            if (parentNode == null) return null;
            
            // get the widget for the node
            var parent = dRegistry.getEnclosingWidget(parentNode);
            
            // check if looking for a specific widget type
            while ((widgetType) && (parent) && (! (parent instanceof widgetType))) {
            	parentNode = parent.domNode.parentNode;
            	parent = null;
            	if (parentNode) {
            		parent = dRegistry.getEnclosingWidget(parentNode);
            	}
            } 
            
            // return the parent
            return parent;
    };

     /**
      * @public
  	  * @function
 	  * @name idx.util.getSiblingWidget
 	  * @description Determines the widget that is the first next or previous sibling of the 
 	  *              specified widget or node that is a widget (optionally of a specific type).  This
 	  *              is determined by obtaining the parent node for the specified node or "widget.domNode",
 	  *              finding the first next or previous sibling node that is the domNode for a widget, and
 	  *              if not moving on to the next.  This method may return null if no widget sibling exists.
      *
      * @param {Node|Widget} target The node or dijit._WidgetBase for which the sibling is being requested.
      * @param {Boolean} previous Optional parameter for indicating which sibling.  Specify true if the 
      *                           previous sibling is desired and false if the next sibling.  If omitted then
      *                           false is the default.
      * @param {Type} widgetType The optional type of widget for the sibling.  If specified, this method ignores
      *                          any sibling that is not of the specified type.
      *
      * @return {Widget} The sibling widget or null if no widget matching the criteria is found..
      */  
    iUtil.getSiblingWidget = function(/*Node|Widget*/ target, 
        	 						  /*Boolean*/     previous,
        							  /*Type*/        widgetType) {        	
            // get the widget node
            var widgetNode = (target instanceof dWidget) ? target.domNode : target;
            
            // get the parent node of the DOM node
            var parentNode = widgetNode.parentNode;
            
            // check the parent node
            if (parentNode == null) return null;
            
            // get the children of the parent
            var children = parentNode.childNodes;
            if (! children) return null;
            
            // find the index for the child
            var index = 0;
            for (index = 0; index < children.length; index++) {
            	if (children[index] == widgetNode) break;
            }
            
            if (index == children.length) return null;
            
            // work forward are backward from the index
            var step = (previous) ? -1: 1;
            var limit = (previous) ? -1 : children.length;
            var sibindex = 0;
            var sibling  = null;
            for (sibindex = (index + step); sibindex != limit; (sibindex += step)) {
            	var sibnode = children[sibindex];
            	
            	// get the widget for the node
                var sibwidget = dRegistry.getEnclosingWidget(sibnode);
                if (! sibwidget) continue;
                if (sibwidget.domNode == sibnode) {
                	if ((!widgetType) || (sibwidget instanceof widgetType)) {
                		sibling = sibwidget;
                		break;
                	}
                }
            }
            
            // return the sibling
            return sibling;
    };
        

     /**
      * @public
  	  * @function
 	  * @name idx.util.getChildWidget
 	  * @description Determines the widget that is the first or last child of the specified widget 
 	  *              or node that is a widget (optionally of a specified type).  This is determined
 	  *              by obtaining the widget for the specified parent, obtaining the children widgets
 	  *              and returning either the first or last that is optionally of a specified type.
      *              This method may return null if no widget child exists.
      *
      * @param {Node|Widget} parent The node or dijit._WidgetBase for which the child is being requested.
      * @param {Boolean} last Optional parameter for indicating which child.  Specify true if the 
      *                       last child is desired and false if the first child.  If omitted then
      *                       false is the default.
      * @param {Type} widgetType The optional type of widget for the child.  If specified, this method ignores
      *                          any child that is not of the specified type.
      *
      * @return {Widget} The child widget or null if no widget matching the criteria is found.
      */  
     iUtil.getChildWidget = function(/*Node|Widget*/ parent, 
        								   /*Boolean*/     last,
        								   /*Type*/        widgetType) {
            // get the widget node
        	if (! (parent instanceof dWidget)) {
            	var widget = dRegistry.getEnclosingWidget(parent);
            	if (widget) parent = widget;
            }

            var children = null;
            if (parent instanceof dWidget) {
            	children = parent.getChildren();
            } else {
                children = parent.childNodes;
            }
            
            // check the children
            if (! children) return null;
            if (children.length == 0) return null;

            // setup the looping variables
            var start = (last) ? (children.length - 1) : 0;
            var step  = (last) ? -1 : 1;
            var limit = (last) ? -1 : children.length;

            // work forward are backward from the index
            var childIndex = 0;
            var child      = null;
            for (childIndex = start; childIndex != limit; (childIndex += step)) {
            	var widget = children[childIndex];
            	if (! (widget instanceof dWidget)) {
                	// get the widget for the node
                    var node = widget;
                    widget = dRegistry.getEnclosingWidget(node);
                    if (! widget) continue;
                    if (widget.domNode != node) continue;
            	}
            	if ((!widgetType) || (widget instanceof widgetType)) {
            		child = widget;
            		break;
            	}
            }
            
            // return the child
            return child;
        };

     /**
      * @public
  	  * @function
 	  * @name idx.util.getFormWidget
 	  * @description Determines the widget (derived from dijit.form._FormWidget) that is a child of the 
 	  *              specified node or widget (usually a dijit.form.Form) that has the specified form 
 	  *              field name.  If the found form field is found to be an instance of dijit.form._FormWidget
 	  *              then it is returned, otherwise null is returned.
      *
      * @param {String} formFieldName The name of the form field widget that is being requested.
      * @param {Node|Widget} parent Optional root node or root widget under which to look for the form 
      *                             widgets. If not specified then the entire document body is searched.
      * @return The form field widget with the specified name or null if not found.
      */  
     iUtil.getFormWidget = function(/*String*/ 			formFieldName,
        							/*Node|Widget?*/	parent) {
        	// get the widget node
        	var rootNode = null;
        	if (!parent) {
        		parent = dWindow.body();
        	} else if (parent instanceof dWidget) {
        		rootNode = parent.domNode;
        	} else {
        		rootNode = form;
        	}
        			
        	var formWidget = null;
        	var nodeList = dQuery("[name='" + formFieldName + "']", rootNode);
        	for (var index = 0; (!formWidget) && (index < nodeList.length);index++) {
        		var node = nodeList[index];
        		var widget = dRegistry.getEnclosingWidget(node);
        		if (!widget) continue;
        		if (! (widget instanceof dFormWidget)) {
        			continue;
        		}
        		var name = widget.get("name");
        		if (name != formFieldName) {
        			continue;
        		}
        		formWidget = widget;
        	}
        	return formWidget;
   };
        
        
     /**
      * @public
  	  * @function
 	  * @name idx.util.isNodeOrElement
 	  * @description Attempts to determine if the specified object is a DOM node.  This is needed since IE does
 	  *              not recognize the "Node" type, and only IE-8 recognizes the "Element" type.
      *
      * @param {Object|Node|Element} obj The object to check.
      * @return The result is true if the specified object is a node or element, otherwise false.
      */  
     iUtil.isNodeOrElement = function(/*Object*/ obj) {
         return ((obj.parentNode) && (obj.childNodes) && (obj.attributes)) ? true : false;
     };

     /**
      * @public
  	  * @function
 	  * @name idx.util.debugObject
 	  * @description Generates a diagnostic string describing the specified object.
      *
      * @param {Object} obj The object to debug
      * @return The debug string for the object.
      */  
     iUtil.debugObject = function(/*Object*/ obj) {
        	return iUtil._debugObject(obj, "/", [ ]);
     };
        
     /**
      * @private
  	  * @function
 	  * @name idx.util._debugObject
 	  * @description Recursively generates a diagnostic string describing the specified object
 	  *              and tries to detect circular references.
      *
      * @param {Object} obj The object to debug
      * @param {String} path The current attribute path within the object.
      * @param {Object} seen The associative array of paths to boolean values to check if already seen in 
      *                      order to avoid circular recursion.
      * @return The debug string for the object.
      */  
     iUtil._debugObject = function(/*Object*/ obj, /*String*/ path, /*Array*/ seen) {
           if (obj === null) return "null";
           var objType = iUtil.typeOfObject(obj);
           switch (objType) {
           case 'object':
        	   for (var index = 0; index < seen.length; index++) {
        		   if (seen[index].obj == obj) {
        			   return "CIRCULAR_REFERENCE[ " + seen[index].path + " ]";
        		   }
        	   }
        	   seen.push({obj: obj, path: path});
        	   var result = "{ ";
        	   var prefix = "";
        	   for (field in obj) {
        	     result = (result + prefix + '"' + field + '": ' 
        	    		   + iUtil._debugObject(obj[field], 
        	    				   				   path + '/"' + field + '"', 
        	    				   				   seen));
        	     prefix = ", ";
        	   }
        	   result = result + " }";
        	   return result;
           case 'date':
        	   return "DATE[ " + dDateStamp.toISOString(obj) + " ]";
           default:
        	   return dJson.stringify(obj);
           }
        };
     
     /**
      * @public
  	  * @field
 	  * @name idx.util.isBrowser
 	  * 
 	  * @description Equivalent to "dojo.isBrowser" in Dojo 1.6 and dojo/ 1  in Dojo 1.7+
 	  * @deprecated Use dojo/ 1  instead.
 	  */
      iUtil.isBrowser = dHas("host-browser");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isIE
 	  * 
 	  * @description Equivalent to "dojo.isIE" in Dojo 1.6 and dojo/has("ie") in Dojo 1.7+
 	  * @deprecated Use dojo/has("ie") instead.
 	  */
      iUtil.isIE = dHas("ie");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isFF
 	  * 
 	  * @description Equivalent to "dojo.isFF" in Dojo 1.6 and dojo/has("ff") in Dojo 1.7+
 	  * @deprecated Use dojo/has("ff") instead.
 	  */
      iUtil.isFF = dHas("ff");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isSafari
 	  * 
 	  * @description Equivalent to "dojo.isSafari" in Dojo 1.6 and dojo/has("safari") in Dojo 1.7+
 	  * @deprecated Use dojo/has("safari") instead.
 	  */
      iUtil.isSafari = dHas("safari");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isChrome
 	  * 
 	  * @description Equivalent to "dojo.isChrome" in Dojo 1.6 and dojo/has("chrome") in Dojo 1.7+
 	  * @deprecated Use dojo/has("chrome") instead.
 	  */
      iUtil.isChrome = dHas("chrome");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isMozilla
 	  * 
 	  * @description Equivalent to "dojo.isMozilla" in Dojo 1.6 and dojo/has("mozilla") in Dojo 1.7+
 	  * @deprecated Use dojo/has("mozilla") instead.
 	  */
      iUtil.isMozilla = dHas("mozilla");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isOpera
 	  * 
 	  * @description Equivalent to "dojo.isOpera" in Dojo 1.6 and dojo/has("opera") in Dojo 1.7+
 	  * @deprecated Use dojo/has("opera") instead.
 	  */
      iUtil.isOpera = dHas("opera");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isKhtml
 	  * 
 	  * @description Equivalent to "dojo.isKhtml" in Dojo 1.6 and dojo/has("khtml") in Dojo 1.7+
 	  * @deprecated Use dojo/has("khtml") instead.
 	  */
      iUtil.isKhtml	= dHas("khtml");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isAIR
 	  * 
 	  * @description Equivalent to "dojo.isAIR" in Dojo 1.6 and dojo/has("air") in Dojo 1.7+
 	  * @deprecated Use dojo/has("air") instead.
 	  */
      iUtil.isAIR = dHas("air");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isQuirks
 	  * 
 	  * @description Equivalent to "dojo.isQuirks" in Dojo 1.6 and dojo/has("quirks") in Dojo 1.7+
 	  * @deprecated Use dojo/has("quirks") instead.
 	  */
      iUtil.isQuirks = dHas("quirks");
      
     /**
      * @public
  	  * @field
 	  * @name idx.util.isWebkit
 	  * 
 	  * @description Equivalent to "dojo.isWebkit" in Dojo 1.6 and dojo/has("webkit") in Dojo 1.7+
 	  * @deprecated Use dojo/has("webkit") instead.
 	  */
      iUtil.isWebKit = dHas("webkit");
        

        // idx.util.fromJson is copied from IBM Dojo Toolkit 1.7.x implementation of dojox.secure.fromJson
        // because that module lacks AMD support in Dojo 1.7 and early releases of Dojo 1.8.  The original 
        // implementation of dojox.secure.fromJson includes the following notice:
        //
        // Used with permission from Mike Samuel of Google (has CCLA), from the json-sans-eval project:
        // http://code.google.com/p/json-sans-eval/
        //	Mike Samuel <mikesamuel_at_gmail_dot_com>

     /**
      * @public
  	  * @function
 	  * @name idx.util.fromJson
 	  * @description Generates a diagnostic string describing the specified object.  Parses a string 
 	  *              of well-formed JSON text without exposing the application to cross-site request
 	  *              forgery attacks.  Use this in place of dojo.fromJson().  The code is copied from 
 	  *              IBM Dojo Toolkit 1.7.x implementation of dojox.secure.fromJson module which lacks 
 	  *              support for AMD as of Dojo 1.7 & Dojo 1.8 RC1.
      * 
      *              Parses a string of well-formed JSON text. If the input is not well-formed, then behavior
      *              is undefined, but it is deterministic and is guaranteed not to modify any object other 
      *              than its return value.
      *
	  * 			 This does not use `eval` so is less likely to have obscure security bugs than json2.js.
	  *              It is optimized for speed, so is much faster than json_parse.js.
	  *
	  *              This library should be used whenever security is a concern (when JSON may
	  *              come from an untrusted source), speed is a concern, and erroring on malformed
      *              JSON is *not* a concern.
	  *	
	  *              json2.js is very fast, but potentially insecure since it calls `eval` to
	  *              parse JSON data, so an attacker might be able to supply strange JS that
      *              looks like JSON, but that executes arbitrary javascript.
      * 
	  * @param {String} json JSON text per RFC 4627
	  * 
	  * @param {Function (this:Object, string, *)} optReviver Optional function that reworks JSON objects post-parse
	  *                                                       per Chapter 15.12 of EcmaScript3.1.  If supplied, the
	  *                                                       function is called with a string key, and a value.
	  * 													  The value is the property of 'this'.	The reviver should
	  *                                                       return the value to use in its place.	So if dates were
	  *                                                       serialized as {@code { "type": "Date", "time": 1234 }},
	  *                                                       then a reviver might look like 
	  *	  {@code 
	  *			function (key, value) {
      * 			if (value && typeof value === 'object' && 'Date' === value.type) {
	  *				return new Date(value.time);
	  *			} else {
	  * 				return value;
	  *			}
	  * 		}}.
	  * 		If the reviver returns {@code undefined} then the property named by key
	  *		will be deleted from its container.
	  *		{@code this} is bound to the object containing the specified property.
	  * 
	  * @return {Object|Array} representing the parsed object.
      */
     iUtil.fromJson = typeof JSON != "undefined" ? JSON.parse :
        	(function () {
        		var number
        			= '(?:-?\\b(?:0|[1-9][0-9]*)(?:\\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\\b)';
        		var oneChar = '(?:[^\\0-\\x08\\x0a-\\x1f\"\\\\]'
        			+ '|\\\\(?:[\"/\\\\bfnrt]|u[0-9A-Fa-f]{4}))';
        		var string = '(?:\"' + oneChar + '*\")';

        		// Will match a value in a well-formed JSON file.
        		// If the input is not well-formed, may match strangely, but not in an unsafe
        		// way.
        		// Since this only matches value tokens, it does not match whitespace, colons,
        		// or commas.
        		var jsonToken = new RegExp(
        				'(?:false|true|null|[\\{\\}\\[\\]]'
        				+ '|' + number
        				+ '|' + string
        				+ ')', 'g');

        		// Matches escape sequences in a string literal
        		var escapeSequence = new RegExp('\\\\(?:([^u])|u(.{4}))', 'g');

        		// Decodes escape sequences in object literals
        		var escapes = {
        				'"': '"',
        				'/': '/',
        				'\\': '\\',
        				'b': '\b',
        				'f': '\f',
        				'n': '\n',
        				'r': '\r',
        				't': '\t'
        		};
        		function unescapeOne(_, ch, hex) {
        			return ch ? escapes[ch] : String.fromCharCode(parseInt(hex, 16));
        		}

        		// A non-falsy value that coerces to the empty string when used as a key.
        		var EMPTY_STRING = new String('');
        		var SLASH = '\\';

        		// Constructor to use based on an open token.
        		var firstTokenCtors = { '{': Object, '[': Array };

        		var hop = Object.hasOwnProperty;

        		return function (json, opt_reviver) {
        			// Split into tokens
        			var toks = json.match(jsonToken);
        			// Construct the object to return
        			var result;
        			var tok = toks[0];
        			var topLevelPrimitive = false;
        			if ('{' === tok) {
        				result = {};
        			} else if ('[' === tok) {
        				result = [];
        			} else {
        				// The RFC only allows arrays or objects at the top level, but the JSON.parse
        				// defined by the EcmaScript 5 draft does allow strings, booleans, numbers, and null
        				// at the top level.
        				result = [];
        				topLevelPrimitive = true;
        			}

        			// If undefined, the key in an object key/value record to use for the next
        			// value parsed.
        			var key;
        			// Loop over remaining tokens maintaining a stack of uncompleted objects and
        			// arrays.
        			var stack = [result];
        			for (var i = 1 - topLevelPrimitive, n = toks.length; i < n; ++i) {
        				tok = toks[i];

        			var cont;
        			switch (tok.charCodeAt(0)) {
        			default:	// sign or digit
        				cont = stack[0];
        				cont[key || cont.length] = +(tok);
        				key = void 0;
        				break;
        			case 0x22:	// '"'
        				tok = tok.substring(1, tok.length - 1);
        				if (tok.indexOf(SLASH) !== -1) {
        					tok = tok.replace(escapeSequence, unescapeOne);
        				}
        				cont = stack[0];
        				if (!key) {
        					if (cont instanceof Array) {
        						key = cont.length;
        					} else {
        						key = tok || EMPTY_STRING;	// Use as key for next value seen.
        						break;
        					}
        				}
        				cont[key] = tok;
        				key = void 0;
        				break;
        			case 0x5b:	// '['
        				cont = stack[0];
        				stack.unshift(cont[key || cont.length] = []);
        				key = void 0;
        				break;
        			case 0x5d:	// ']'
        				stack.shift();
        				break;
        			case 0x66:	// 'f'
        				cont = stack[0];
        				cont[key || cont.length] = false;
        				key = void 0;
        				break;
        			case 0x6e:	// 'n'
        				cont = stack[0];
        				cont[key || cont.length] = null;
        				key = void 0;
        				break;
        			case 0x74:	// 't'
        				cont = stack[0];
        				cont[key || cont.length] = true;
        				key = void 0;
        				break;
        			case 0x7b:	// '{'
        				cont = stack[0];
        				stack.unshift(cont[key || cont.length] = {});
        				key = void 0;
        				break;
        			case 0x7d:	// '}'
        				stack.shift();
        				break;
        			}
        		}
        		// Fail if we've got an uncompleted object.
        		if (topLevelPrimitive) {
        			if (stack.length !== 1) { throw new Error(); }
        			result = result[0];
        		} else {
        			if (stack.length) { throw new Error(); }
        		}

        		if (opt_reviver) {
        			// Based on walk as implemented in http://www.json.org/json2.js
        			var walk = function (holder, key) {
        				var value = holder[key];
        				if (value && typeof value === 'object') {
        					var toDelete = null;
        					for (var k in value) {
        						if (hop.call(value, k) && value !== holder) {
        							// Recurse to properties first.	This has the effect of causing
        							// the reviver to be called on the object graph depth-first.

        							// Since 'this' is bound to the holder of the property, the
        							// reviver can access sibling properties of k including ones
        							// that have not yet been revived.

        							// The value returned by the reviver is used in place of the
        							// current value of property k.
        							// If it returns undefined then the property is deleted.
        							var v = walk(value, k);
        							if (v !== void 0) {
        								value[k] = v;
        							} else {
        								// Deleting properties inside the loop has vaguely defined
        								// semantics in ES3 and ES3.1.
        								if (!toDelete) { toDelete = []; }
        								toDelete.push(k);
        							}
        						}
        					}
        					if (toDelete) {
        						for (var i = toDelete.length; --i >= 0;) {
        							delete value[toDelete[i]];
        						}
        					}
        				}
        				return opt_reviver.call(holder, key, value);
        			};
        			result = walk({ '': result }, '');
        		}

        		return result;
        		};
        	})();
        
     /**
      * @private
  	  * @function
 	  * @name idx.util._getFontMeasurements
 	  * @description Internal function for computing the measurements of on-screen fonts.
 	  */ 
     function _getFontMeasurements(measurements,fontSize){
    		// initialize heights to the specified fontMeasurements parameter
    		var heights = measurements;
    		
    		// if it does not yet exist then create it
    		if (!heights) {
    			heights = {
    				'1em': -1, '1ex': -1, '100%': -1, '12pt': -1, '16px': -1, 'xx-small': -1,
	    			'x-small': -1, 'small': 1, 'medium': -1, 'large': -1, 'x-large': -1,
    				'xx-large': -1
    			};
    		}
    		
    		// check if a specific font size was requested in addition to defaults
    		if (fontSize && (! (fontSize in heights))) {
    			heights[fontSize] = -1;
    		}
    		
    		var p;
    		if(dHas("ie")){
    			dWindow.doc.documentElement.style.fontSize="100%";
    		}
    		var div = dDomConstruct.create("div", {style: {
    				position: "absolute",
    				left: "0",
    				top: "-100000px",
    				width: "30px",
    				height: "1000em",
    				borderWidth: "0",
    				margin: "0",
    				padding: "0",
    				outline: "none",
    				lineHeight: "1",
    				overflow: "hidden"
    			}}, dWindow.body());
    		for(p in heights){
    			// check if the given font size was already calculated and skip it
    			if (heights[p] >= 0) {
    				continue;
    			}
    			
    			// set the font size
    			dDomStyle.set(div, "fontSize", p);
    			
    			// get the measurement
    			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
    		}

    		dWindow.body().removeChild(div);
    		return heights; //object
    };
    
     /**
      * @private
      * @function
      * @name idx.util._toFontSize
      * @description Internal function to convert a node to a font size string.
      * @param fontSizeOrNode The font size as a string or a node.
      */
     function _toFontSize(fontSize) {
     		var fontSizeType = iUtil.typeOfObject(fontSize);
     		
     		if ((fontSizeType == "object")&&(iUtil.isNodeOrElement(fontSize))) {
    			var compStyle = dDomStyle.getComputedStyle(fontSize);
     			fontSize = compStyle["fontSize"];
     		} else if (fontSizeType != "string") {
     			fontSize = "" + fontSize;
     		}
     		return fontSize;
     };
     
     /**
      * @private
  	  * @function
 	  * @name idx.util._getCachedFontMeasurements
 	  * @description Internal function for computing and cacheing the measurements of on-screen fonts.
 	  */ 
	 var fontMeasurements = null;
     function _getCachedFontMeasurements(fontSize,recalculate){
     		var fontSizeType = iUtil.typeOfObject(fontSize);
     		
     		if(fontSizeType == "boolean"){ 
     			recalculate = fontSize;
     			fontSize = null;
     			
     		} else {
     			fontSize = _toFontSize(fontSize);
     		}
     		
     		// if recalculating then clear out old calculations
     		if (fontMeasurements && recalculate) {
     			if (fontSize) {
     				if (fontMeasurements[fontSize]) delete fontMeasurements[fontSize];
     			} else {
     				fontMeasurements = null;
     			}
     		}
     		
    		if(recalculate || !fontMeasurements || (fontSize && !fontMeasurements[fontSize])){
    			fontMeasurements = _getFontMeasurements(fontMeasurements, fontSize);
    		}
    		return fontMeasurements;
	};
   
   		 	
     /**
      * @public
  	  * @function
 	  * @name idx.util.normalizedLength
 	  * @description Converts non-percentage CSS widths from various units to pixels.
 	  * @param len The CSS length with optional units.
 	  * @param fontSizeOrNode The optional font size or node for which the font size is computed (defaults to "12pt").
 	  * @return The length/width in pixels.
 	  */ 
     iUtil.normalizedLength = function(len,fontSize) {
    		if(len.length === 0){ return 0; }
    		if(!fontSize) fontSize = "12pt";
    		else fontSize = _toFontSize(fontSize);
    		if(len.length > 2){
				// we don't want to use the fontSize parameter if the
				// specified length is not font-based units
				var units = len.slice(-2);
				var fontUnits = (units == "em" || units == "ex");
				if (! fontUnits) fontSize = "12pt";     		
    			var fm = _getCachedFontMeasurements(fontSize);
    			var px_in_pt = fm["12pt"] / 12;
    			var px_in_em = fm[fontSize];
    			var px_in_ex = px_in_em / 2;
    			var val = parseFloat(len);
    			switch(len.slice(-2)){
    				case "px": return val;
    				case "em": return val * px_in_em;
    				case "ex": return val * px_in_ex;
    				case "pt": return val * px_in_pt;
    				case "in": return val * 72 * px_in_pt;
    				case "pc": return val * 12 * px_in_pt;
    				case "mm": return val * g.mm_in_pt * px_in_pt;
    				case "cm": return val * g.cm_in_pt * px_in_pt;
    			}
    		}
    		return parseFloat(len);	// Number
    };
	/**
	 * @public
	 * @function
	 * @name idx.util.isPercentage
	 * @description Attempts to determine if the value is a percentage 
	 * @param value to be check 
	 * @return return true if value is a percentage
	 */
	iUtil.isPercentage = function(value){
		return  /^\d+%$/.test(value);
	};
	/**
	 * @public
	 * @function
	 * @name idx.util.includeValidCSSWidth
	 * @description Attempts to determine if the value includes a valid CSS width
	 * @param value to be check
	 * @return return ture if the value includes a valid CSS width
	 */
	iUtil.includeValidCSSWidth = function(value){
		return /width:\s*\d+(%|px|pt|in|pc|mm|cm)/.test(value)
	};
	iUtil.getValidCSSWidth = function(style){
		var styleType = iUtil.typeOfObject(style);
		if (styleType != "string") {
			if ("width" in style) {
				style = "width: " + style.width + ";";
			} else {
				style = "width: " + style + ";";
			}
		}
		return /width:\s*\d+(%|px|pt|in|pc|mm|cm|em|ex)/.test(style) ? 
			style.match(/width:\s*(\d+(%|px|pt|in|pc|mm|cm|em|ex))/)[1] : "";
	};

	/**
	 * Compares to widgets to see if they are identical or alternatively compares
	 * strings to strings or strings to widgets, treating the strings as the ID of
	 * a widget.  The widget for the specified ID need not exist yet in the registry
	 * for this function to compare since it relies on the "id" field of any specified
	 * widget to perform a string compare.
	 *
	 * @param {String|Widget} w1 The first widget or the ID for the first widget.
	 * @param {String|Widget| w2 The second widget or the ID for the second widget.
	 *
	 * @return {boolean} Returns true if equal, otherwise false.
	 */
	iUtil.widgetEquals = function(w1,w2) {
		if (w1 === w2) return true;
		if (!w1 && w2) return false;
		if (w1 && !w2) return false;
		if ((!dLang.isString(w1)) && ("id" in w1)) w1 = w1.id;
		if ((!dLang.isString(w2)) && ("id" in w2)) w2 = w2.id;
		return (w1 == w2);
	};
  
	/**
	 * Provides a way to watch an attribute on an object whether it be an instance of
	 * dojo/Stateful or dijit/_WidgetBase.  The "watch()" method does not work on 
	 * dijit/_WidgetBase instances that implement custom setters.
	 *
	 * @param {Stateful|Widget} obj The object that owns the attribute to watch.
	 * @param {String} attr The attribute name to watch.
	 *
	 * @return {Handle} Returns the handle to remove the watch or null if the attribute
	 *         cannot be watched on the specified object.
	 */
	iUtil.watch = function(obj, attr, func) {
		if (!obj) return null;
		if (!attr) return null;
		if  ((! ("watch" in obj)) || (!dLang.isFunction(obj.watch))) {
			// object is not an instance of dojo/Stateful
			return null;
		}
		attr = dString.trim(attr);
		if (attr.length == 0) return null;
		var uc = attr.charAt(0).toUpperCase();
		if (attr.length > 1) {
			uc = uc + attr.substring(1);
		}
		var funcName = "_set" + uc + "Attr";
		if (funcName in obj) {
			return dAspect.around(obj, funcName, function(origFunc) {
				return function(value) {
					var oldValue = obj.get(attr);
					origFunc.apply(obj, arguments);
					var newValue = obj.get(attr);
					if (oldValue != newValue) {
						func.call(undefined, attr, oldValue, newValue);
					}
				};
			});
		} else {
			return obj.watch(attr, func);
		}
	};
  	
    return iUtil;
});
},
'xcitec/form/TextBoxButton':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
    "dojo/text!./templates/TextBoxButton.html",
    "dijit/_WidgetsInTemplateMixin",
	"./TextBox",
    "./_XButtonRight",
    "./_XButtonSingleRight"
], function (declare, domClass, template, _WidgetsInTemplateMixin, TextBox) {

	return declare( [TextBox, _WidgetsInTemplateMixin], {
	    xbtnClass: "xcitec/form/_XButtonRight",
	    disabledAttrSetting: "",
	    actionIconType: "Info",
	    actionIconText: "",
	    buttonEnabled: false,
        widgetsInTemplate: true,
	
	    templateString: template,
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
		    this.buttonEnabled = this.buttonEnabled || ("onActionClick" in this.params);
	
	        if (this.disabled) {
	            this.required = false;
	            this.buttonEnabled = false;
	            this.disabledAttrSetting = "disabled";
	        }
	        if (this.readOnly) {
	            // kave: in readOnly action is allowed to
	            //this.buttonEnabled = false;
	            this.xbtnClass = "xcitec/form/_XButtonSingleRight";
	            this.required = false;
	        }
	    },
	
	    postCreate: function() {
	        this.inherited(arguments);
	        if (!this.buttonEnabled) {
	            // if action for buton is not allowed button has to be deactivated for user
	            // XButton has different view as readOnly and disabled
	            // to feet to style:
	            // - if widget is active -> mark action button as readOnly
	            // - if widget is readOnly -> mark action button as disabled
	            // - if widget is disabled -> action button is not visible in any case
	            if (this.readOnly) {
	                this.button.set("disabled", true);
	            } else {
	                this.button.set("readOnly", true);
	            }
	            this.buttonEnabled = false;
	        }
	        if (this.required) {
	          domClass.add(this.domNode, "dijitXcitecRequired");
	        }
	    },
	
	    _onClick: function() {
	        if (!this.buttonEnabled) {
	            return;
	        }
	        this.onActionClick();
	    },

	    onActionClick: function() {
	        var targetValue = this.get('value');
	        if (!targetValue) return;
	        this._doExecuteAction(targetValue);
	    },
	
	    _doExecuteAction: function(/*String*/ value) {
	        //have to be ovveriden
	    }
	});
});

},
'dijit/form/ValidationTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/i18n", // i18n.getLocalization
	"./TextBox",
	"../Tooltip",
	"dojo/text!./templates/ValidationTextBox.html",
	"dojo/i18n!./nls/validate"
], function(declare, kernel, i18n, TextBox, Tooltip, template){

	// module:
	//		dijit/form/ValidationTextBox


	/*=====
	var __Constraints = {
		// locale: String
		//		locale used for validation, picks up value from this widget's lang attribute
		// _flags_: anything
		//		various flags passed to pattern function
	};
	=====*/

	var ValidationTextBox;
	return ValidationTextBox = declare("dijit.form.ValidationTextBox", TextBox, {
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.

		templateString: template,

		// required: Boolean
		//		User is required to enter data into this field.
		required: false,

		// promptMessage: String
		//		If defined, display this hint string immediately on focus to the textbox, if empty.
		//		Also displays if the textbox value is Incomplete (not yet valid but will be with additional input).
		//		Think of this like a tooltip that tells the user what to do, not an error message
		//		that tells the user what they've done wrong.
		//
		//		Message disappears when user starts typing.
		promptMessage: "",

		// invalidMessage: String
		//		The message to display if value is invalid.
		//		The translated string value is read from the message file by default.
		//		Set to "" to use the promptMessage instead.
		invalidMessage: "$_unset_$",

		// missingMessage: String
		//		The message to display if value is empty and the field is required.
		//		The translated string value is read from the message file by default.
		//		Set to "" to use the invalidMessage instead.
		missingMessage: "$_unset_$",

		// message: String
		//		Currently error/prompt message.
		//		When using the default tooltip implementation, this will only be
		//		displayed when the field is focused.
		message: "",

		// constraints: __Constraints
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},

		// pattern: [extension protected] String|Function(constraints) returning a string.
		//		This defines the regular expression used to validate the input.
		//		Do not add leading ^ or $ characters since the widget adds these.
		//		A function may be used to generate a valid pattern when dependent on constraints or other runtime factors.
		//		set('pattern', String|Function).
		pattern: ".*",

		// regExp: Deprecated [extension protected] String.  Use "pattern" instead.
		regExp: "",

		regExpGen: function(/*__Constraints*/ /*===== constraints =====*/){
			// summary:
			//		Deprecated.  Use set('pattern', Function) instead.
		},

		// state: [readonly] String
		//		Shows current state (ie, validation result) of input (""=Normal, Incomplete, or Error)
		state: "",

		// tooltipPosition: String[]
		//		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
		tooltipPosition: [],

		_deprecateRegExp: function(attr, value){
			if(value != ValidationTextBox.prototype[attr]){
				kernel.deprecated("ValidationTextBox id="+this.id+", set('" + attr + "', ...) is deprecated.  Use set('pattern', ...) instead.", "", "2.0");
				this.set('pattern', value);
			}
		},
		_setRegExpGenAttr: function(/*Function*/ newFcn){
			this._deprecateRegExp("regExpGen", newFcn);
			this._set("regExpGen", this._computeRegexp); // backward compat with this.regExpGen(this.constraints)
		},
		_setRegExpAttr: function(/*String*/ value){
			this._deprecateRegExp("regExp", value);
		},

		_setValueAttr: function(){
			// summary:
			//		Hook so set('value', ...) works.
			this.inherited(arguments);
			this._refreshState();
		},

		validator: function(/*anything*/ value, /*__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the text input against the regular expression.
			// tags:
			//		protected
			return (new RegExp("^(?:" + this._computeRegexp(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
		},

		_isValidSubset: function(){
			// summary:
			//		Returns true if the value is either already valid or could be made valid by appending characters.
			//		This is used for validation while the user [may be] still typing.
			return this.textbox.value.search(this._partialre) == 0;
		},

		isValid: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if value is valid.
			//		Can override with your own routine in a subclass.
			// tags:
			//		protected
			return this.validator(this.textbox.value, this.get('constraints'));
		},

		_isEmpty: function(value){
			// summary:
			//		Checks for whitespace
			return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return an error message to show if appropriate
			// tags:
			//		protected
			var invalid = this.invalidMessage == "$_unset_$" ? this.messages.invalidMessage :
				!this.invalidMessage ? this.promptMessage : this.invalidMessage;
			var missing = this.missingMessage == "$_unset_$" ? this.messages.missingMessage :
				!this.missingMessage ? invalid : this.missingMessage;
			return (this.required && this._isEmpty(this.textbox.value)) ? missing : invalid; // String
		},

		getPromptMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return a hint message to show when widget is first focused
			// tags:
			//		protected
			return this.promptMessage; // String
		},

		_maskValidSubsetError: true,
		validate: function(/*Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			// tags:
			//		protected
			var message = "";
			var isValid = this.disabled || this.isValid(isFocused);
			if(isValid){ this._maskValidSubsetError = true; }
			var isEmpty = this._isEmpty(this.textbox.value);
			var isValidSubset = !isValid && isFocused && this._isValidSubset();
			this._set("state", isValid ? "" : (((((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && (this._maskValidSubsetError || (isValidSubset && !this._hasBeenBlurred && isFocused))) ? "Incomplete" : "Error"));
			this.focusNode.setAttribute("aria-invalid", this.state == "Error" ? "true" : "false");

			if(this.state == "Error"){
				this._maskValidSubsetError = isFocused && isValidSubset; // we want the error to show up after a blur and refocus
				message = this.getErrorMessage(isFocused);
			}else if(this.state == "Incomplete"){
				message = this.getPromptMessage(isFocused); // show the prompt whenever the value is not yet complete
				this._maskValidSubsetError = !this._hasBeenBlurred || isFocused; // no Incomplete warnings while focused
			}else if(isEmpty){
				message = this.getPromptMessage(isFocused); // show the prompt whenever there's no error and no text
			}
			this.set("message", message);

			return isValid;
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		Overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			// tags:
			//		extension
			if(message && this.focused){
				Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}else{
				Tooltip.hide(this.domNode);
			}
		},

		_refreshState: function(){
			// Overrides TextBox._refreshState()
			if(this._created){ // should instead be this._started but that would require all programmatic ValidationTextBox instantiations to call startup()
				this.validate(this.focused);
			}
			this.inherited(arguments);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

			this.constraints = {};
			this.baseClass += ' dijitValidationTextBox';
		},

		startup: function(){
			this.inherited(arguments);
			this._refreshState(); // after all _set* methods have run
		},

		_setConstraintsAttr: function(/*__Constraints*/ constraints){
			if(!constraints.locale && this.lang){
				constraints.locale = this.lang;
			}
			this._set("constraints", constraints);
			this._refreshState();
		},

		_setPatternAttr: function(/*String|Function*/ pattern){
			this._set("pattern", pattern); // don't set on INPUT to avoid native HTML5 validation
			this._refreshState();
		},

		_computeRegexp: function(/*__Constraints*/ constraints){
			// summary:
			//		Hook to get the current regExp and to compute the partial validation RE.

			var p = this.pattern;
			if(typeof p == "function"){
				p = p.call(this, constraints);
			}
			if(p != this._lastRegExp){
				var partialre = "";
				this._lastRegExp = p;
				// parse the regexp and produce a new regexp that matches valid subsets
				// if the regexp is .* then there's no use in matching subsets since everything is valid
				if(p != ".*"){
					p.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
					function(re){
						switch(re.charAt(0)){
							case '{':
							case '+':
							case '?':
							case '*':
							case '^':
							case '$':
							case '|':
							case '(':
								partialre += re;
								break;
							case ")":
								partialre += "|$)";
								break;
							 default:
								partialre += "(?:"+re+"|$)";
								break;
						}
					});
				}
				try{ // this is needed for now since the above regexp parsing needs more test verification
					"".search(partialre);
				}catch(e){ // should never be here unless the original RE is bad or the parsing is bad
					partialre = this.pattern;
					console.warn('RegExp error in ' + this.declaredClass + ': ' + this.pattern);
				} // should never be here unless the original RE is bad or the parsing is bad
				this._partialre = "^(?:" + partialre + ")$";
			}
			return p;
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
			this._refreshState();
		},

		_setRequiredAttr: function(/*Boolean*/ value){
			this._set("required", value);
			this.focusNode.setAttribute("aria-required", value);
			this._refreshState();
		},

		_setMessageAttr: function(/*String*/ message){
			this._set("message", message);
			this.displayMessage(message);
		},

		reset:function(){
			// Overrides dijit/form/TextBox.reset() by also
			// hiding errors about partial matches
			this._maskValidSubsetError = true;
			this.inherited(arguments);
		},

		_onBlur: function(){
			// the message still exists but for back-compat, and to erase the tooltip
			// (if the message is being displayed as a tooltip), call displayMessage('')
			this.displayMessage('');

			this.inherited(arguments);
		}
	});
});

},
'xcitec/MenuItem':function(){
define([
	"dojo/_base/declare",
	"dojo/text!xcitec/templates/MenuItem.html",
	"dijit/MenuItem",
  "./xcitecUtil",
  "./_base/ImageFactory"
], function (declare, template, MenuItem, xcitecUtil, ImageFactory) {

	var MenuItem = declare( MenuItem, {
	    templateString: template,
	  
	    postMixInProperties: function() {
		    this.label = xcitecUtil.escapeXml(this.label);
	        if (this.iconClass) {
	            this.iconClass = ImageFactory.createIconCss(this.iconClass);
	        }
	        this.inherited(arguments);
	    }
	});
	return MenuItem;
});

},
'xcitec/BoldText':function(){
define([
         "dojo/_base/declare",
         "dojo/dom-class",
         "./Text"
       ], function (declare, domClass, Text) {
    return declare(Text, {
      postCreate: function(){
        this.inherited(arguments);
        domClass.add(this.domNode, "xcitecBoldText")
      }
    });
});

},
'dijit/form/NumberTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch lang.mixin
	"dojo/number", // number._realNumberRegexp number.format number.parse number.regexp
	"./RangeBoundTextBox"
], function(declare, lang, number, RangeBoundTextBox){

	// module:
	//		dijit/form/NumberTextBox


	var NumberTextBoxMixin = declare("dijit.form.NumberTextBoxMixin", null, {
		// summary:
		//		A mixin for all number textboxes
		// tags:
		//		protected

		// Override ValidationTextBox.pattern.... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale (plus formatting options too?)
		pattern: function(constraints){
			// if focused, accept either currency data or NumberTextBox format
			return '(' + (this.focused && this.editOptions ? this._regExpGenerator(lang.delegate(constraints, this.editOptions)) + '|' : '')
				+ this._regExpGenerator(constraints) + ')';
		},

		/*=====
		// constraints: NumberTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including minimum/maximum allowed values) as well as
		//		formatting options like places (the number of digits to display after
		//		the decimal point).
		constraints: {},
		======*/

		// value: Number
		//		The value of this NumberTextBox as a Javascript Number (i.e., not a String).
		//		If the displayed value is blank, the value is NaN, and if the user types in
		//		an gibberish value (like "hello world"), the value is undefined
		//		(i.e. get('value') returns undefined).
		//
		//		Symmetrically, set('value', NaN) will clear the displayed value,
		//		whereas set('value', undefined) will have no effect.
		value: NaN,

		// editOptions: [protected] Object
		//		Properties to mix into constraints when the value is being edited.
		//		This is here because we edit the number in the format "12345", which is
		//		different than the display value (ex: "12,345")
		editOptions: { pattern: '#.######' },

		/*=====
		_formatter: function(value, options){
			// summary:
			//		_formatter() is called by format().  It's the base routine for formatting a number,
			//		as a string, for example converting 12345 into "12,345".
			// value: Number
			//		The number to be converted into a string.
			// options: number.__FormatOptions?
			//		Formatting options
			// tags:
			//		protected extension

			return "12345";		// String
		},
		 =====*/
		_formatter: number.format,

		/*=====
		_regExpGenerator: function(constraints){
			// summary:
			//		Generate a localized regular expression as a string, according to constraints.
			// constraints: number.__ParseOptions
			//		Formatting options
			// tags:
			//		protected

			return "(\d*).(\d*)";	// string
		},
		=====*/
		_regExpGenerator: number.regexp,

		postMixInProperties: function(){
			this.inherited(arguments);
			this._set("type", "text"); // in case type="number" was specified which messes up parse/format
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			var places = typeof constraints.places == "number"? constraints.places : 0;
			if(places){ places++; } // decimal rounding errors take away another digit of precision
			if(typeof constraints.max != "number"){
				constraints.max = 9 * Math.pow(10, 15-places);
			}
			if(typeof constraints.min != "number"){
				constraints.min = -9 * Math.pow(10, 15-places);
			}
			this.inherited(arguments, [ constraints ]);
			if(this.focusNode && this.focusNode.value && !isNaN(this.value)){
				this.set('value', this.value);
			}
		},

		_onFocus: function(){
			if(this.disabled || this.readOnly){ return; }
			var val = this.get('value');
			if(typeof val == "number" && !isNaN(val)){
				var formattedValue = this.format(val, this.constraints);
				if(formattedValue !== undefined){
					this.textbox.value = formattedValue;
				}
			}
			this.inherited(arguments);
		},

		format: function(/*Number*/ value, /*number.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Number, according to constraints.
			// tags:
			//		protected

			var formattedValue = String(value);
			if(typeof value != "number"){ return formattedValue; }
			if(isNaN(value)){ return ""; }
			// check for exponential notation that dojo/number.format() chokes on
			if(!("rangeCheck" in this && this.rangeCheck(value, constraints)) && constraints.exponent !== false && /\de[-+]?\d/i.test(formattedValue)){
				return formattedValue;
			}
			if(this.editOptions && this.focused){
				constraints = lang.mixin({}, constraints, this.editOptions);
			}
			return this._formatter(value, constraints);
		},

		/*=====
		_parser: function(value, constraints){
			// summary:
			//		Parses the string value as a Number, according to constraints.
			// value: String
			//		String representing a number
			// constraints: number.__ParseOptions
			//		Formatting options
			// tags:
			//		protected

			return 123.45;		// Number
		},
		=====*/
		_parser: number.parse,

		parse: function(/*String*/ value, /*number.__FormatOptions*/ constraints){
			// summary:
			//		Replaceable function to convert a formatted string to a number value
			// tags:
			//		protected extension

			var v = this._parser(value, lang.mixin({}, constraints, (this.editOptions && this.focused) ? this.editOptions : {}));
			if(this.editOptions && this.focused && isNaN(v)){
				v = this._parser(value, constraints); // parse w/o editOptions: not technically needed but is nice for the user
			}
			return v;
		},

		_getDisplayedValueAttr: function(){
			var v = this.inherited(arguments);
			return isNaN(v) ? this.textbox.value : v;
		},

		filter: function(/*Number*/ value){
			// summary:
			//		This is called with both the display value (string), and the actual value (a number).
			//		When called with the actual value it does corrections so that '' etc. are represented as NaN.
			//		Otherwise it dispatches to the superclass's filter() method.
			//
			//		See `dijit/form/TextBox.filter()` for more details.
			return (value == null /* or undefined */ || value === '') ? NaN : this.inherited(arguments); // set('value', null||''||undefined) should fire onChange(NaN)
		},

		serialize: function(/*Number*/ value, /*Object?*/ options){
			// summary:
			//		Convert value (a Number) into a canonical string (ie, how the number literal is written in javascript/java/C/etc.)
			// tags:
			//		protected
			return (typeof value != "number" || isNaN(value)) ? '' : this.inherited(arguments);
		},

		_setBlurValue: function(){
			var val = lang.hitch(lang.delegate(this, { focused: true }), "get")('value'); // parse with editOptions
			this._setValueAttr(val, true);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so set('value', ...) works.
			if(value !== undefined && formattedValue === undefined){
				formattedValue = String(value);
				if(typeof value == "number"){
					if(isNaN(value)){ formattedValue = '' }
					// check for exponential notation that number.format chokes on
					else if(("rangeCheck" in this && this.rangeCheck(value, this.constraints)) || this.constraints.exponent === false || !/\de[-+]?\d/i.test(formattedValue)){
						formattedValue = undefined; // lets format compute a real string value
					}
				}else if(!value){ // 0 processed in if branch above, ''|null|undefined flows through here
					formattedValue = '';
					value = NaN;
				}else{ // non-numeric values
					value = undefined;
				}
			}
			this.inherited(arguments, [value, priorityChange, formattedValue]);
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			//		Returns Number, NaN for '', or undefined for unparseable text
			var v = this.inherited(arguments); // returns Number for all values accepted by parse() or NaN for all other displayed values

			// If the displayed value of the textbox is gibberish (ex: "hello world"), this.inherited() above
			// returns NaN; this if() branch converts the return value to undefined.
			// Returning undefined prevents user text from being overwritten when doing _setValueAttr(_getValueAttr()).
			// A blank displayed value is still returned as NaN.
			if(isNaN(v) && this.textbox.value !== ''){
				if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value) && (new RegExp("^"+number._realNumberRegexp(lang.delegate(this.constraints))+"$").test(this.textbox.value))){	// check for exponential notation that parse() rejected (erroneously?)
					var n = Number(this.textbox.value);
					return isNaN(n) ? undefined : n; // return exponential Number or undefined for random text (may not be possible to do with the above RegExp check)
				}else{
					return undefined; // gibberish
				}
			}else{
				return v; // Number or NaN for ''
			}
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/RangeBoundTextBox.isValid() to check that the editing-mode value is valid since
			// it may not be formatted according to the regExp validation rules
			if(!this.focused || this._isEmpty(this.textbox.value)){
				return this.inherited(arguments);
			}else{
				var v = this.get('value');
				if(!isNaN(v) && this.rangeCheck(v, this.constraints)){
					if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value)){ // exponential, parse doesn't like it
						return true; // valid exponential number in range
					}else{
						return this.inherited(arguments);
					}
				}else{
					return false;
				}
			}
		}
	});

	var NumberTextBox = declare("dijit.form.NumberTextBox", [RangeBoundTextBox, NumberTextBoxMixin], {
		// summary:
		//		A TextBox for entering numbers, with formatting and range checking
		// description:
		//		NumberTextBox is a textbox for entering and displaying numbers, supporting
		//		the following main features:
		//
		//		1. Enforce minimum/maximum allowed values (as well as enforcing that the user types
		//			a number rather than a random string)
		//		2. NLS support (altering roles of comma and dot as "thousands-separator" and "decimal-point"
		//			depending on locale).
		//		3. Separate modes for editing the value and displaying it, specifically that
		//			the thousands separator character (typically comma) disappears when editing
		//			but reappears after the field is blurred.
		//		4. Formatting and constraints regarding the number of places (digits after the decimal point)
		//			allowed on input, and number of places displayed when blurred (see `constraints` parameter).

		baseClass: "dijitTextBox dijitNumberTextBox"
	});

	NumberTextBox.Mixin = NumberTextBoxMixin;	// for monkey patching

	/*=====
	 NumberTextBox.__Constraints = declare([RangeBoundTextBox.__Constraints, number.__FormatOptions, number.__ParseOptions], {
		 // summary:
		 //		Specifies both the rules on valid/invalid values (minimum, maximum,
		 //		number of required decimal places), and also formatting options for
		 //		displaying the value when the field is not focused.
		 // example:
		 //		Minimum/maximum:
		 //		To specify a field between 0 and 120:
		 //	|		{min:0,max:120}
		 //		To specify a field that must be an integer:
		 //	|		{fractional:false}
		 //		To specify a field where 0 to 3 decimal places are allowed on input:
		 //	|		{places:'0,3'}
	 });
	 =====*/

	return NumberTextBox;
});

},
'dojo/_base/url':function(){
define(["./kernel"], function(dojo){
	// module:
	//		dojo/url

	var
		ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),
		ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),
		_Url = function(){
			var n = null,
				_a = arguments,
				uri = [_a[0]];
			// resolve uri components relative to each other
			for(var i = 1; i<_a.length; i++){
				if(!_a[i]){ continue; }

				// Safari doesn't support this.constructor so we have to be explicit
				// FIXME: Tracked (and fixed) in Webkit bug 3537.
				//		http://bugs.webkit.org/show_bug.cgi?id=3537
				var relobj = new _Url(_a[i]+""),
					uriobj = new _Url(uri[0]+"");

				if(
					relobj.path == "" &&
					!relobj.scheme &&
					!relobj.authority &&
					!relobj.query
				){
					if(relobj.fragment != n){
						uriobj.fragment = relobj.fragment;
					}
					relobj = uriobj;
				}else if(!relobj.scheme){
					relobj.scheme = uriobj.scheme;

					if(!relobj.authority){
						relobj.authority = uriobj.authority;

						if(relobj.path.charAt(0) != "/"){
							var path = uriobj.path.substring(0,
								uriobj.path.lastIndexOf("/") + 1) + relobj.path;

							var segs = path.split("/");
							for(var j = 0; j < segs.length; j++){
								if(segs[j] == "."){
									// flatten "./" references
									if(j == segs.length - 1){
										segs[j] = "";
									}else{
										segs.splice(j, 1);
										j--;
									}
								}else if(j > 0 && !(j == 1 && segs[0] == "") &&
									segs[j] == ".." && segs[j-1] != ".."){
									// flatten "../" references
									if(j == (segs.length - 1)){
										segs.splice(j, 1);
										segs[j - 1] = "";
									}else{
										segs.splice(j - 1, 2);
										j -= 2;
									}
								}
							}
							relobj.path = segs.join("/");
						}
					}
				}

				uri = [];
				if(relobj.scheme){
					uri.push(relobj.scheme, ":");
				}
				if(relobj.authority){
					uri.push("//", relobj.authority);
				}
				uri.push(relobj.path);
				if(relobj.query){
					uri.push("?", relobj.query);
				}
				if(relobj.fragment){
					uri.push("#", relobj.fragment);
				}
			}

			this.uri = uri.join("");

			// break the uri into its main components
			var r = this.uri.match(ore);

			this.scheme = r[2] || (r[1] ? "" : n);
			this.authority = r[4] || (r[3] ? "" : n);
			this.path = r[5]; // can never be undefined
			this.query = r[7] || (r[6] ? "" : n);
			this.fragment	 = r[9] || (r[8] ? "" : n);

			if(this.authority != n){
				// server based naming authority
				r = this.authority.match(ire);

				this.user = r[3] || n;
				this.password = r[4] || n;
				this.host = r[6] || r[7]; // ipv6 || ipv4
				this.port = r[9] || n;
			}
		};
	_Url.prototype.toString = function(){ return this.uri; };

	return dojo._Url = _Url;
});

},
'dijit/Tree':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/aspect",
	"dojo/_base/connect", // connect.isCopyKey()
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/Deferred", // Deferred
	"dojo/promise/all",
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.remove domClass.replace domClass.toggle
	"dojo/dom-geometry", // domGeometry.setMarginBox domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/errors/create", // createError
	"dojo/fx", // fxUtils.wipeIn fxUtils.wipeOut
	"dojo/has",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // arrows etc.
	"dojo/_base/lang", // lang.getObject lang.mixin lang.hitch
	"dojo/on", // on(), on.selector()
	"dojo/topic",
	"dojo/touch",
	"dojo/when",
	"./a11yclick",
	"./focus",
	"./registry", // registry.byNode(), registry.getEnclosingWidget()
	"./_base/manager", // manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./_Contained",
	"./_CssStateMixin",
	"./_KeyNavMixin",
	"dojo/text!./templates/TreeNode.html",
	"dojo/text!./templates/Tree.html",
	"./tree/TreeStoreModel",
	"./tree/ForestStoreModel",
	"./tree/_dndSelector",
	"dojo/query!css2"	// needed when on.selector() used with a string for the selector
], function(array, aspect, connect, cookie, declare, Deferred, all,
			dom, domClass, domGeometry, domStyle, createError, fxUtils, has, kernel, keys, lang, on, topic, touch, when,
			a11yclick, focus, registry, manager, _Widget, _TemplatedMixin, _Container, _Contained, _CssStateMixin, _KeyNavMixin,
			treeNodeTemplate, treeTemplate, TreeStoreModel, ForestStoreModel, _dndSelector){

	// module:
	//		dijit/Tree

	function shimmedPromise(/*Deferred|Promise*/ d){
		// summary:
		//		Return a Promise based on given Deferred or Promise, with back-compat addCallback() and addErrback() shims
		//		added (TODO: remove those back-compat shims, and this method, for 2.0)

		return lang.delegate(d.promise || d, {
			addCallback: function(callback){
				this.then(callback);
			},
			addErrback: function(errback){
				this.otherwise(errback);
			}
		});
	}

	var TreeNode = declare("dijit._TreeNode", [_Widget, _TemplatedMixin, _Container, _Contained, _CssStateMixin], {
		// summary:
		//		Single node within a tree.   This class is used internally
		//		by Tree and should not be accessed directly.
		// tags:
		//		private

		// item: [const] Item
		//		the dojo.data entry this tree represents
		item: null,

		// isTreeNode: [protected] Boolean
		//		Indicates that this is a TreeNode.   Used by `dijit.Tree` only,
		//		should not be accessed directly.
		isTreeNode: true,

		// label: String
		//		Text of this tree node
		label: "",
		_setLabelAttr: function(val){
			this.labelNode[this.labelType == "html" ? "innerHTML" : "innerText" in this.labelNode ?
				"innerText" : "textContent"] = val;
			this._set("label", val);
		},

		// labelType: [const] String
		//		Specifies how to interpret the label.  Can be "html" or "text".
		labelType: "text",

		// isExpandable: [private] Boolean
		//		This node has children, so show the expando node (+ sign)
		isExpandable: null,

		// isExpanded: [readonly] Boolean
		//		This node is currently expanded (ie, opened)
		isExpanded: false,

		// state: [private] String
		//		Dynamic loading-related stuff.
		//		When an empty folder node appears, it is "NotLoaded" first,
		//		then after dojo.data query it becomes "Loading" and, finally "Loaded"
		state: "NotLoaded",

		templateString: treeNodeTemplate,

		baseClass: "dijitTreeNode",

		// For hover effect for tree node, and focus effect for label
		cssStateNodes: {
			rowNode: "dijitTreeRow"
		},

		// Tooltip is defined in _WidgetBase but we need to handle the mapping to DOM here
		_setTooltipAttr: {node: "rowNode", type: "attribute", attribute: "title"},

		buildRendering: function(){
			this.inherited(arguments);

			// set expand icon for leaf
			this._setExpando();

			// set icon and label class based on item
			this._updateItemClasses(this.item);

			if(this.isExpandable){
				this.labelNode.setAttribute("aria-expanded", this.isExpanded);
			}

			//aria-selected should be false on all selectable elements.
			this.setSelected(false);
		},

		_setIndentAttr: function(indent){
			// summary:
			//		Tell this node how many levels it should be indented
			// description:
			//		0 for top level nodes, 1 for their children, 2 for their
			//		grandchildren, etc.

			// Math.max() is to prevent negative padding on hidden root node (when indent == -1)
			var pixels = (Math.max(indent, 0) * this.tree._nodePixelIndent) + "px";

			domStyle.set(this.domNode, "backgroundPosition", pixels + " 0px");	// TODOC: what is this for???
			domStyle.set(this.rowNode, this.isLeftToRight() ? "paddingLeft" : "paddingRight", pixels);

			array.forEach(this.getChildren(), function(child){
				child.set("indent", indent + 1);
			});

			this._set("indent", indent);
		},

		markProcessing: function(){
			// summary:
			//		Visually denote that tree is loading data, etc.
			// tags:
			//		private
			this.state = "Loading";
			this._setExpando(true);
		},

		unmarkProcessing: function(){
			// summary:
			//		Clear markup from markProcessing() call
			// tags:
			//		private
			this._setExpando(false);
		},

		_updateItemClasses: function(item){
			// summary:
			//		Set appropriate CSS classes for icon and label dom node
			//		(used to allow for item updates to change respective CSS)
			// tags:
			//		private
			var tree = this.tree, model = tree.model;
			if(tree._v10Compat && item === model.root){
				// For back-compat with 1.0, need to use null to specify root item (TODO: remove in 2.0)
				item = null;
			}
			this._applyClassAndStyle(item, "icon", "Icon");
			this._applyClassAndStyle(item, "label", "Label");
			this._applyClassAndStyle(item, "row", "Row");

			this.tree._startPaint(true);		// signifies paint started and finished (synchronously)
		},

		_applyClassAndStyle: function(item, lower, upper){
			// summary:
			//		Set the appropriate CSS classes and styles for labels, icons and rows.
			//
			// item:
			//		The data item.
			//
			// lower:
			//		The lower case attribute to use, e.g. 'icon', 'label' or 'row'.
			//
			// upper:
			//		The upper case attribute to use, e.g. 'Icon', 'Label' or 'Row'.
			//
			// tags:
			//		private

			var clsName = "_" + lower + "Class";
			var nodeName = lower + "Node";
			var oldCls = this[clsName];

			this[clsName] = this.tree["get" + upper + "Class"](item, this.isExpanded);
			domClass.replace(this[nodeName], this[clsName] || "", oldCls || "");

			domStyle.set(this[nodeName], this.tree["get" + upper + "Style"](item, this.isExpanded) || {});
		},

		_updateLayout: function(){
			// summary:
			//		Set appropriate CSS classes for this.domNode
			// tags:
			//		private
			var parent = this.getParent();
			if(!parent || !parent.rowNode || parent.rowNode.style.display == "none"){
				/* if we are hiding the root node then make every first level child look like a root node */
				domClass.add(this.domNode, "dijitTreeIsRoot");
			}else{
				domClass.toggle(this.domNode, "dijitTreeIsLast", !this.getNextSibling());
			}
		},

		_setExpando: function(/*Boolean*/ processing){
			// summary:
			//		Set the right image for the expando node
			// tags:
			//		private

			var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
					"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"],
				_a11yStates = ["*", "-", "+", "*"],
				idx = processing ? 0 : (this.isExpandable ? (this.isExpanded ? 1 : 2) : 3);

			// apply the appropriate class to the expando node
			domClass.replace(this.expandoNode, styles[idx], styles);

			// provide a non-image based indicator for images-off mode
			this.expandoNodeText.innerHTML = _a11yStates[idx];

		},

		expand: function(){
			// summary:
			//		Show my children
			// returns:
			//		Promise that resolves when expansion is complete

			// If there's already an expand in progress or we are already expanded, just return
			if(this._expandDeferred){
				return shimmedPromise(this._expandDeferred);		// dojo/promise/Promise
			}

			// cancel in progress collapse operation
			if(this._collapseDeferred){
				this._collapseDeferred.cancel();
				delete this._collapseDeferred;
			}

			// All the state information for when a node is expanded, maybe this should be
			// set when the animation completes instead
			this.isExpanded = true;
			this.labelNode.setAttribute("aria-expanded", "true");
			if(this.tree.showRoot || this !== this.tree.rootNode){
				this.containerNode.setAttribute("role", "group");
			}
			domClass.add(this.contentNode, 'dijitTreeContentExpanded');
			this._setExpando();
			this._updateItemClasses(this.item);

			if(this == this.tree.rootNode && this.tree.showRoot){
				this.tree.domNode.setAttribute("aria-expanded", "true");
			}

			var wipeIn = fxUtils.wipeIn({
				node: this.containerNode,
				duration: manager.defaultDuration
			});

			// Deferred that fires when expand is complete
			var def = (this._expandDeferred = new Deferred(function(){
				// Canceller
				wipeIn.stop();
			}));

			aspect.after(wipeIn, "onEnd", function(){
				def.resolve(true);
			}, true);

			wipeIn.play();

			return shimmedPromise(def);		// dojo/promise/Promise
		},

		collapse: function(){
			// summary:
			//		Collapse this node (if it's expanded)
			// returns:
			//		Promise that resolves when collapse is complete

			if(this._collapseDeferred){
				// Node is already collapsed, or there's a collapse in progress, just return that Deferred
				return shimmedPromise(this._collapseDeferred);
			}

			// cancel in progress expand operation
			if(this._expandDeferred){
				this._expandDeferred.cancel();
				delete this._expandDeferred;
			}

			this.isExpanded = false;
			this.labelNode.setAttribute("aria-expanded", "false");
			if(this == this.tree.rootNode && this.tree.showRoot){
				this.tree.domNode.setAttribute("aria-expanded", "false");
			}
			domClass.remove(this.contentNode, 'dijitTreeContentExpanded');
			this._setExpando();
			this._updateItemClasses(this.item);

			var wipeOut = fxUtils.wipeOut({
				node: this.containerNode,
				duration: manager.defaultDuration
			});

			// Deferred that fires when expand is complete
			var def = (this._collapseDeferred = new Deferred(function(){
				// Canceller
				wipeOut.stop();
			}));

			aspect.after(wipeOut, "onEnd", function(){
				def.resolve(true);
			}, true);

			wipeOut.play();

			return shimmedPromise(def);		// dojo/promise/Promise
		},

		// indent: Integer
		//		Levels from this node to the root node
		indent: 0,

		setChildItems: function(/* Object[] */ items){
			// summary:
			//		Sets the child items of this node, removing/adding nodes
			//		from current children to match specified items[] array.
			//		Also, if this.persist == true, expands any children that were previously
			//		opened.
			// returns:
			//		Promise that resolves after all previously opened children
			//		have been expanded again (or fires instantly if there are no such children).

			var tree = this.tree,
				model = tree.model,
				defs = [];	// list of deferreds that need to fire before I am complete


			// Orphan all my existing children.
			// If items contains some of the same items as before then we will reattach them.
			// Don't call this.removeChild() because that will collapse the tree etc.
			var oldChildren = this.getChildren();
			array.forEach(oldChildren, function(child){
				_Container.prototype.removeChild.call(this, child);
			}, this);

			// All the old children of this TreeNode are subject for destruction if
			//		1) they aren't listed in the new children array (items)
			//		2) they aren't immediately adopted by another node (DnD)
			this.defer(function(){
				array.forEach(oldChildren, function(node){
					if(!node._destroyed && !node.getParent()){
						// If node is in selection then remove it.
						tree.dndController.removeTreeNode(node);

						// Deregister mapping from item id --> this node and its descendants
						function remove(node){
							var id = model.getIdentity(node.item),
								ary = tree._itemNodesMap[id];
							if(ary.length == 1){
								delete tree._itemNodesMap[id];
							}else{
								var index = array.indexOf(ary, node);
								if(index != -1){
									ary.splice(index, 1);
								}
							}
							array.forEach(node.getChildren(), remove);
						}

						remove(node);

						// Remove any entries involving this node from cookie tracking expanded nodes
						if(tree.persist){
							var destroyedPath = array.map(node.getTreePath(),function(item){
								return tree.model.getIdentity(item);
							}).join("/");
							for(var path in tree._openedNodes){
								if(path.substr(0, destroyedPath.length) == destroyedPath){
									delete tree._openedNodes[path];
								}
							}
							tree._saveExpandedNodes();
						}

						// And finally we can destroy the node
						node.destroyRecursive();
					}
				});
			});

			this.state = "Loaded";

			if(items && items.length > 0){
				this.isExpandable = true;

				// Create _TreeNode widget for each specified tree node, unless one already
				// exists and isn't being used (presumably it's from a DnD move and was recently
				// released
				array.forEach(items, function(item){    // MARKER: REUSE NODE
					var id = model.getIdentity(item),
						existingNodes = tree._itemNodesMap[id],
						node;
					if(existingNodes){
						for(var i = 0; i < existingNodes.length; i++){
							if(existingNodes[i] && !existingNodes[i].getParent()){
								node = existingNodes[i];
								node.set('indent', this.indent + 1);
								break;
							}
						}
					}
					if(!node){
						node = this.tree._createTreeNode({
							item: item,
							tree: tree,
							isExpandable: model.mayHaveChildren(item),
							label: tree.getLabel(item),
							labelType: (tree.model && tree.model.labelType) || "text",
							tooltip: tree.getTooltip(item),
							ownerDocument: tree.ownerDocument,
							dir: tree.dir,
							lang: tree.lang,
							textDir: tree.textDir,
							indent: this.indent + 1
						});
						if(existingNodes){
							existingNodes.push(node);
						}else{
							tree._itemNodesMap[id] = [node];
						}
					}
					this.addChild(node);

					// If node was previously opened then open it again now (this may trigger
					// more data store accesses, recursively)
					if(this.tree.autoExpand || this.tree._state(node)){
						defs.push(tree._expandNode(node));
					}
				}, this);

				// note that updateLayout() needs to be called on each child after
				// _all_ the children exist
				array.forEach(this.getChildren(), function(child){
					child._updateLayout();
				});
			}else{
				this.isExpandable = false;
			}

			if(this._setExpando){
				// change expando to/from dot or + icon, as appropriate
				this._setExpando(false);
			}

			// Set leaf icon or folder icon, as appropriate
			this._updateItemClasses(this.item);

			var def = all(defs);
			this.tree._startPaint(def);		// to reset TreeNode widths after an item is added/removed from the Tree
			return shimmedPromise(def);		// dojo/promise/Promise
		},

		getTreePath: function(){
			var node = this;
			var path = [];
			while(node && node !== this.tree.rootNode){
				path.unshift(node.item);
				node = node.getParent();
			}
			path.unshift(this.tree.rootNode.item);

			return path;
		},

		getIdentity: function(){
			return this.tree.model.getIdentity(this.item);
		},

		removeChild: function(/* treeNode */ node){
			this.inherited(arguments);

			var children = this.getChildren();
			if(children.length == 0){
				this.isExpandable = false;
				this.collapse();
			}

			array.forEach(children, function(child){
				child._updateLayout();
			});
		},

		makeExpandable: function(){
			// summary:
			//		if this node wasn't already showing the expando node,
			//		turn it into one and call _setExpando()

			// TODO: hmm this isn't called from anywhere, maybe should remove it for 2.0

			this.isExpandable = true;
			this._setExpando(false);
		},

		setSelected: function(/*Boolean*/ selected){
			// summary:
			//		A Tree has a (single) currently selected node.
			//		Mark that this node is/isn't that currently selected node.
			// description:
			//		In particular, setting a node as selected involves setting tabIndex
			//		so that when user tabs to the tree, focus will go to that node (only).
			this.labelNode.setAttribute("aria-selected", selected ? "true" : "false");
			domClass.toggle(this.rowNode, "dijitTreeRowSelected", selected);
		},

		focus: function(){
			focus.focus(this.focusNode);
		}
	});

	if(has("dojo-bidi")){
		TreeNode.extend({
			_setTextDirAttr: function(textDir){
				if(textDir && ((this.textDir != textDir) || !this._created)){
					this._set("textDir", textDir);
					this.applyTextDir(this.labelNode);
					array.forEach(this.getChildren(), function(childNode){
						childNode.set("textDir", textDir);
					}, this);
				}
			}
		});
	}

	var Tree = declare("dijit.Tree", [_Widget, _KeyNavMixin, _TemplatedMixin, _CssStateMixin], {
		// summary:
		//		This widget displays hierarchical data from a store.

		baseClass: "dijitTree",

		// store: [deprecated] String|dojo/data/Store
		//		Deprecated.  Use "model" parameter instead.
		//		The store to get data to display in the tree.
		store: null,

		// model: [const] dijit/tree/model
		//		Interface to read tree data, get notifications of changes to tree data,
		//		and for handling drop operations (i.e drag and drop onto the tree)
		model: null,

		// query: [deprecated] anything
		//		Deprecated.  User should specify query to the model directly instead.
		//		Specifies datastore query to return the root item or top items for the tree.
		query: null,

		// label: [deprecated] String
		//		Deprecated.  Use dijit/tree/ForestStoreModel directly instead.
		//		Used in conjunction with query parameter.
		//		If a query is specified (rather than a root node id), and a label is also specified,
		//		then a fake root node is created and displayed, with this label.
		label: "",

		// showRoot: [const] Boolean
		//		Should the root node be displayed, or hidden?
		showRoot: true,

		// childrenAttr: [deprecated] String[]
		//		Deprecated.   This information should be specified in the model.
		//		One ore more attributes that holds children of a tree node
		childrenAttr: ["children"],

		// paths: String[][] or Item[][]
		//		Full paths from rootNode to selected nodes expressed as array of items or array of ids.
		//		Since setting the paths may be asynchronous (because of waiting on dojo.data), set("paths", ...)
		//		returns a Promise to indicate when the set is complete.
		paths: [],

		// path: String[] or Item[]
		//		Backward compatible singular variant of paths.
		path: [],

		// selectedItems: [readonly] Item[]
		//		The currently selected items in this tree.
		//		This property can only be set (via set('selectedItems', ...)) when that item is already
		//		visible in the tree.   (I.e. the tree has already been expanded to show that node.)
		//		Should generally use `paths` attribute to set the selected items instead.
		selectedItems: null,

		// selectedItem: [readonly] Item
		//		Backward compatible singular variant of selectedItems.
		selectedItem: null,

		// openOnClick: Boolean
		//		If true, clicking a folder node's label will open it, rather than calling onClick()
		openOnClick: false,

		// openOnDblClick: Boolean
		//		If true, double-clicking a folder node's label will open it, rather than calling onDblClick()
		openOnDblClick: false,

		templateString: treeTemplate,

		// persist: Boolean
		//		Enables/disables use of cookies for state saving.
		persist: false,

		// autoExpand: Boolean
		//		Fully expand the tree on load.   Overrides `persist`.
		autoExpand: false,

		// dndController: [protected] Function|String
		//		Class to use as as the dnd controller.  Specifying this class enables DnD.
		//		Generally you should specify this as dijit/tree/dndSource.
		//		Setting of dijit/tree/_dndSelector handles selection only (no actual DnD).
		dndController: _dndSelector,

		// parameters to pull off of the tree and pass on to the dndController as its params
		dndParams: ["onDndDrop", "itemCreator", "onDndCancel", "checkAcceptance", "checkItemAcceptance", "dragThreshold", "betweenThreshold"],

		//declare the above items so they can be pulled from the tree's markup

		// onDndDrop: [protected] Function
		//		Parameter to dndController, see `dijit/tree/dndSource.onDndDrop()`.
		//		Generally this doesn't need to be set.
		onDndDrop: null,

		itemCreator: null,
		/*=====
		itemCreator: function(nodes, target, source){
			// summary:
			//		Returns objects passed to `Tree.model.newItem()` based on DnD nodes
			//		dropped onto the tree.   Developer must override this method to enable
			//		dropping from external sources onto this Tree, unless the Tree.model's items
			//		happen to look like {id: 123, name: "Apple" } with no other attributes.
			//
			//		For each node in nodes[], which came from source, create a hash of name/value
			//		pairs to be passed to Tree.model.newItem().  Returns array of those hashes.
			// nodes: DomNode[]
			//		The DOMNodes dragged from the source container
			// target: DomNode
			//		The target TreeNode.rowNode
			// source: dojo/dnd/Source
			//		The source container the nodes were dragged from, perhaps another Tree or a plain dojo/dnd/Source
			// returns: Object[]
			//		Array of name/value hashes for each new item to be added to the Tree, like:
			// |	[
			// |		{ id: 123, label: "apple", foo: "bar" },
			// |		{ id: 456, label: "pear", zaz: "bam" }
			// |	]
			// tags:
			//		extension
			return [{}];
		},
		=====*/

		// onDndCancel: [protected] Function
		//		Parameter to dndController, see `dijit/tree/dndSource.onDndCancel()`.
		//		Generally this doesn't need to be set.
		onDndCancel: null,

		/*=====
		checkAcceptance: function(source, nodes){
			// summary:
			//		Checks if the Tree itself can accept nodes from this source
			// source: dijit/tree/dndSource
			//		The source which provides items
			// nodes: DOMNode[]
			//		Array of DOM nodes corresponding to nodes being dropped, dijitTreeRow nodes if
			//		source is a dijit/Tree.
			// tags:
			//		extension
			return true;	// Boolean
		},
		=====*/
		checkAcceptance: null,

		/*=====
		checkItemAcceptance: function(target, source, position){
			// summary:
			//		Stub function to be overridden if one wants to check for the ability to drop at the node/item level
			// description:
			//		In the base case, this is called to check if target can become a child of source.
			//		When betweenThreshold is set, position="before" or "after" means that we
			//		are asking if the source node can be dropped before/after the target node.
			// target: DOMNode
			//		The dijitTreeRoot DOM node inside of the TreeNode that we are dropping on to
			//		Use registry.getEnclosingWidget(target) to get the TreeNode.
			// source: dijit/tree/dndSource
			//		The (set of) nodes we are dropping
			// position: String
			//		"over", "before", or "after"
			// tags:
			//		extension
			return true;	// Boolean
		},
		=====*/
		checkItemAcceptance: null,

		// dragThreshold: Integer
		//		Number of pixels mouse moves before it's considered the start of a drag operation
		dragThreshold: 5,

		// betweenThreshold: Integer
		//		Set to a positive value to allow drag and drop "between" nodes.
		//
		//		If during DnD mouse is over a (target) node but less than betweenThreshold
		//		pixels from the bottom edge, dropping the the dragged node will make it
		//		the next sibling of the target node, rather than the child.
		//
		//		Similarly, if mouse is over a target node but less that betweenThreshold
		//		pixels from the top edge, dropping the dragged node will make it
		//		the target node's previous sibling rather than the target node's child.
		betweenThreshold: 0,

		// _nodePixelIndent: Integer
		//		Number of pixels to indent tree nodes (relative to parent node).
		//		Default is 19 but can be overridden by setting CSS class dijitTreeIndent
		//		and calling resize() or startup() on tree after it's in the DOM.
		_nodePixelIndent: 19,

		_publish: function(/*String*/ topicName, /*Object*/ message){
			// summary:
			//		Publish a message for this widget/topic
			topic.publish(this.id, lang.mixin({tree: this, event: topicName}, message || {}));	// publish
		},

		postMixInProperties: function(){
			this.tree = this;

			if(this.autoExpand){
				// There's little point in saving opened/closed state of nodes for a Tree
				// that initially opens all it's nodes.
				this.persist = false;
			}

			this._itemNodesMap = {};

			if(!this.cookieName && this.id){
				this.cookieName = this.id + "SaveStateCookie";
			}

			// Deferred that resolves when all the children have loaded.
			this.expandChildrenDeferred = new Deferred();

			// Promise that resolves when all pending operations complete.
			this.pendingCommandsPromise = this.expandChildrenDeferred.promise;

			this.inherited(arguments);
		},

		postCreate: function(){
			this._initState();

			// Catch events on TreeNodes
			var self = this;
			this.own(
				on(this.containerNode, on.selector(".dijitTreeNode", touch.enter), function(evt){
					self._onNodeMouseEnter(registry.byNode(this), evt);
				}),
				on(this.containerNode, on.selector(".dijitTreeNode", touch.leave), function(evt){
					self._onNodeMouseLeave(registry.byNode(this), evt);
				}),
				on(this.containerNode, on.selector(".dijitTreeRow", a11yclick.press), function(evt){
					self._onNodePress(registry.getEnclosingWidget(this), evt);
				}),
				on(this.containerNode, on.selector(".dijitTreeRow", a11yclick), function(evt){
					self._onClick(registry.getEnclosingWidget(this), evt);
				}),
				on(this.containerNode, on.selector(".dijitTreeRow", "dblclick"), function(evt){
					self._onDblClick(registry.getEnclosingWidget(this), evt);
				})
			);

			// Create glue between store and Tree, if not specified directly by user
			if(!this.model){
				this._store2model();
			}

			// monitor changes to items
			this.own(
				aspect.after(this.model, "onChange", lang.hitch(this, "_onItemChange"), true),
				aspect.after(this.model, "onChildrenChange", lang.hitch(this, "_onItemChildrenChange"), true),
				aspect.after(this.model, "onDelete", lang.hitch(this, "_onItemDelete"), true)
			);

			this.inherited(arguments);

			if(this.dndController){
				// TODO: remove string support in 2.0.
				if(lang.isString(this.dndController)){
					this.dndController = lang.getObject(this.dndController);
				}
				var params = {};
				for(var i = 0; i < this.dndParams.length; i++){
					if(this[this.dndParams[i]]){
						params[this.dndParams[i]] = this[this.dndParams[i]];
					}
				}
				this.dndController = new this.dndController(this, params);
			}

			this._load();

			// onLoadDeferred should fire when all commands that are part of initialization have completed.
			// It will include all the set("paths", ...) commands that happen during initialization.
			this.onLoadDeferred = shimmedPromise(this.pendingCommandsPromise);

			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
		},

		_store2model: function(){
			// summary:
			//		User specified a store&query rather than model, so create model from store/query
			this._v10Compat = true;
			kernel.deprecated("Tree: from version 2.0, should specify a model object rather than a store/query");

			var modelParams = {
				id: this.id + "_ForestStoreModel",
				store: this.store,
				query: this.query,
				childrenAttrs: this.childrenAttr
			};

			// Only override the model's mayHaveChildren() method if the user has specified an override
			if(this.params.mayHaveChildren){
				modelParams.mayHaveChildren = lang.hitch(this, "mayHaveChildren");
			}

			if(this.params.getItemChildren){
				modelParams.getChildren = lang.hitch(this, function(item, onComplete, onError){
					this.getItemChildren((this._v10Compat && item === this.model.root) ? null : item, onComplete, onError);
				});
			}
			this.model = new ForestStoreModel(modelParams);

			// For backwards compatibility, the visibility of the root node is controlled by
			// whether or not the user has specified a label
			this.showRoot = Boolean(this.label);
		},

		onLoad: function(){
			// summary:
			//		Called when tree finishes loading and expanding.
			// description:
			//		If persist == true the loading may encompass many levels of fetches
			//		from the data store, each asynchronous.   Waits for all to finish.
			// tags:
			//		callback
		},

		_load: function(){
			// summary:
			//		Initial load of the tree.
			//		Load root node (possibly hidden) and it's children.
			this.model.getRoot(
				lang.hitch(this, function(item){
					var rn = (this.rootNode = this.tree._createTreeNode({
						item: item,
						tree: this,
						isExpandable: true,
						label: this.label || this.getLabel(item),
						labelType: this.model.labelType || "text",
						textDir: this.textDir,
						indent: this.showRoot ? 0 : -1
					}));

					if(!this.showRoot){
						rn.rowNode.style.display = "none";
						// if root is not visible, move tree role to the invisible
						// root node's containerNode, see #12135
						this.domNode.setAttribute("role", "presentation");
						this.domNode.removeAttribute("aria-expanded");
						this.domNode.removeAttribute("aria-multiselectable");

						// move the aria-label or aria-labelledby to the element with the role
						if(this["aria-label"]){
							rn.containerNode.setAttribute("aria-label", this["aria-label"]);
							this.domNode.removeAttribute("aria-label");
						}else if(this["aria-labelledby"]){
							rn.containerNode.setAttribute("aria-labelledby", this["aria-labelledby"]);
							this.domNode.removeAttribute("aria-labelledby");
						}
						rn.labelNode.setAttribute("role", "presentation");
						rn.containerNode.setAttribute("role", "tree");
						rn.containerNode.setAttribute("aria-expanded", "true");
						rn.containerNode.setAttribute("aria-multiselectable", !this.dndController.singular);
					}else{
						this.domNode.setAttribute("aria-multiselectable", !this.dndController.singular);
						this.rootLoadingIndicator.style.display = "none";
					}

					this.containerNode.appendChild(rn.domNode);
					var identity = this.model.getIdentity(item);
					if(this._itemNodesMap[identity]){
						this._itemNodesMap[identity].push(rn);
					}else{
						this._itemNodesMap[identity] = [rn];
					}

					rn._updateLayout();		// sets "dijitTreeIsRoot" CSS classname

					// Load top level children, and if persist==true, all nodes that were previously opened
					this._expandNode(rn).then(lang.hitch(this, function(){
						// Then, select the nodes specified by params.paths[].

						this.rootLoadingIndicator.style.display = "none";
						this.expandChildrenDeferred.resolve(true);
					}));
				}),
				lang.hitch(this, function(err){
					console.error(this, ": error loading root: ", err);
				})
			);
		},

		getNodesByItem: function(/*Item or id*/ item){
			// summary:
			//		Returns all tree nodes that refer to an item
			// returns:
			//		Array of tree nodes that refer to passed item

			if(!item){
				return [];
			}
			var identity = lang.isString(item) ? item : this.model.getIdentity(item);
			// return a copy so widget don't get messed up by changes to returned array
			return [].concat(this._itemNodesMap[identity]);
		},

		_setSelectedItemAttr: function(/*Item or id*/ item){
			this.set('selectedItems', [item]);
		},

		_setSelectedItemsAttr: function(/*Items or ids*/ items){
			// summary:
			//		Select tree nodes related to passed items.
			//		WARNING: if model use multi-parented items or desired tree node isn't already loaded
			//		behavior is undefined. Use set('paths', ...) instead.
			var tree = this;
			return this.pendingCommandsPromise = this.pendingCommandsPromise.always(lang.hitch(this, function(){
				var identities = array.map(items, function(item){
					return (!item || lang.isString(item)) ? item : tree.model.getIdentity(item);
				});
				var nodes = [];
				array.forEach(identities, function(id){
					nodes = nodes.concat(tree._itemNodesMap[id] || []);
				});
				this.set('selectedNodes', nodes);
			}));
		},

		_setPathAttr: function(/*Item[]|String[]*/ path){
			// summary:
			//		Singular variant of _setPathsAttr
			if(path.length){
				return shimmedPromise(this.set("paths", [path]).then(function(paths){ return paths[0]; }));
			}else{
				// Empty list is interpreted as "select nothing"
				return shimmedPromise(this.set("paths", []).then(function(paths){ return paths[0]; }));
			}
		},

		_setPathsAttr: function(/*Item[][]|String[][]*/ paths){
			// summary:
			//		Select the tree nodes identified by passed paths.
			// paths:
			//		Array of arrays of items or item id's
			// returns:
			//		Promise to indicate when the set is complete

			var tree = this;

			function selectPath(path, nodes){
				// Traverse path, returning Promise for node at the end of the path.
				// The next path component should be among "nodes".
				var nextPath = path.shift();
				var nextNode = array.filter(nodes, function(node){
					return node.getIdentity() == nextPath;
				})[0];
				if(!!nextNode){
					if(path.length){
						return tree._expandNode(nextNode).then(function(){
							return selectPath(path, nextNode.getChildren());
						});
					}else{
						// Successfully reached the end of this path
						return nextNode;
					}
				}else{
					throw new Tree.PathError("Could not expand path at " + nextPath);
				}
			}

			// Let any previous set("path", ...) commands complete before this one starts.
			// TODO for 2.0: make the user do this wait themselves?
			return shimmedPromise(this.pendingCommandsPromise = this.pendingCommandsPromise.always(function(){
				// We may need to wait for some nodes to expand, so setting
				// each path will involve a Deferred. We bring those deferreds
				// together with a dojo/promise/all.
				return all(array.map(paths, function(path){
					// normalize path to use identity
					path = array.map(path, function(item){
						return lang.isString(item) ? item : tree.model.getIdentity(item);
					});

					if(path.length){
						return selectPath(path, [tree.rootNode]);
					}else{
						throw new Tree.PathError("Empty path");
					}
				}));
			}).then(function setNodes(newNodes){
				// After all expansion is finished, set the selection to last element from each path
				tree.set("selectedNodes", newNodes);
				return tree.paths;
			}));
		},

		_setSelectedNodeAttr: function(node){
			this.set('selectedNodes', [node]);
		},
		_setSelectedNodesAttr: function(nodes){
			// summary:
			//		Marks the specified TreeNodes as selected.
			// nodes: TreeNode[]
			//		TreeNodes to mark.
			this.dndController.setSelection(nodes);
		},


		expandAll: function(){
			// summary:
			//		Expand all nodes in the tree
			// returns:
			//		Promise that resolves when all nodes have expanded

			var _this = this;

			function expand(node){
				// Expand the node
				return _this._expandNode(node).then(function(){
					// When node has expanded, call expand() recursively on each non-leaf child
					var childBranches = array.filter(node.getChildren() || [], function(node){
						return node.isExpandable;
					});

					// And when all those recursive calls finish, signal that I'm finished
					return all(array.map(childBranches, expand));
				});
			}

			return shimmedPromise(expand(this.rootNode));
		},

		collapseAll: function(){
			// summary:
			//		Collapse all nodes in the tree
			// returns:
			//		Promise that resolves when all nodes have collapsed

			var _this = this;

			function collapse(node){
				// Collapse children first
				var childBranches = array.filter(node.getChildren() || [], function(node){
						return node.isExpandable;
					}),
					defs = all(array.map(childBranches, collapse));

				// And when all those recursive calls finish, collapse myself, unless I'm the invisible root node,
				// in which case collapseAll() is finished
				if(!node.isExpanded || (node == _this.rootNode && !_this.showRoot)){
					return defs;
				}else{
					// When node has collapsed, signal that call is finished
					return defs.then(function(){
						return _this._collapseNode(node);
					});
				}
			}

			return shimmedPromise(collapse(this.rootNode));
		},

		////////////// Data store related functions //////////////////////
		// These just get passed to the model; they are here for back-compat

		mayHaveChildren: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Deprecated.   This should be specified on the model itself.
			//
			//		Overridable function to tell if an item has or may have children.
			//		Controls whether or not +/- expando icon is shown.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node)
			// tags:
			//		deprecated
		},

		getItemChildren: function(/*===== parentItem, onComplete =====*/){
			// summary:
			//		Deprecated.   This should be specified on the model itself.
			//
			//		Overridable function that return array of child items of given parent item,
			//		or if parentItem==null then return top items in tree
			// tags:
			//		deprecated
		},

		///////////////////////////////////////////////////////
		// Functions for converting an item to a TreeNode
		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Overridable function to get the label for a tree node (given the item)
			// tags:
			//		extension
			return this.model.getLabel(item);	// String
		},

		getIconClass: function(/*dojo/data/Item*/ item, /*Boolean*/ opened){
			// summary:
			//		Overridable function to return CSS class name to display icon
			// tags:
			//		extension
			return (!item || this.model.mayHaveChildren(item)) ? (opened ? "dijitFolderOpened" : "dijitFolderClosed") : "dijitLeaf"
		},

		getLabelClass: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS class name to display label
			// item: dojo/data/Item
			// opened: Boolean
			// returns: String
			//		CSS class name
			// tags:
			//		extension
		},

		getRowClass: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS class name to display row
			// item: dojo/data/Item
			// opened: Boolean
			// returns: String
			//		CSS class name
			// tags:
			//		extension
		},

		getIconStyle: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS styles to display icon
			// item: dojo/data/Item
			// opened: Boolean
			// returns: Object
			//		Object suitable for input to dojo.style() like {backgroundImage: "url(...)"}
			// tags:
			//		extension
		},

		getLabelStyle: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS styles to display label
			// item: dojo/data/Item
			// opened: Boolean
			// returns:
			//		Object suitable for input to dojo.style() like {color: "red", background: "green"}
			// tags:
			//		extension
		},

		getRowStyle: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS styles to display row
			// item: dojo/data/Item
			// opened: Boolean
			// returns:
			//		Object suitable for input to dojo.style() like {background-color: "#bbb"}
			// tags:
			//		extension
		},

		getTooltip: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Overridable function to get the tooltip for a tree node (given the item)
			// tags:
			//		extension
			return "";	// String
		},

		/////////// Keyboard and Mouse handlers ////////////////////


		_onDownArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		down arrow pressed; get next visible node, set focus there

			var nextNode = this._getNext(node);
			if(nextNode && nextNode.isTreeNode){
				this.focusNode(nextNode);
			}
		},

		_onUpArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Up arrow pressed; move to previous visible node

			// if younger siblings
			var previousSibling = node.getPreviousSibling();
			if(previousSibling){
				node = previousSibling;
				// if the previous node is expanded, dive in deep
				while(node.isExpandable && node.isExpanded && node.hasChildren()){
					// move to the last child
					var children = node.getChildren();
					node = children[children.length - 1];
				}
			}else{
				// if this is the first child, return the parent
				// unless the parent is the root of a tree with a hidden root
				var parent = node.getParent();
				if(!(!this.showRoot && parent === this.rootNode)){
					node = parent;
				}
			}

			if(node && node.isTreeNode){
				this.focusNode(node);
			}
		},

		_onRightArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Right arrow pressed; go to child node

			// if not expanded, expand, else move to 1st child
			if(node.isExpandable && !node.isExpanded){
				this._expandNode(node);
			}else if(node.hasChildren()){
				node = node.getChildren()[0];
				if(node && node.isTreeNode){
					this.focusNode(node);
				}
			}
		},

		_onLeftArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Left arrow pressed.
			//		If not collapsed, collapse, else move to parent.

			if(node.isExpandable && node.isExpanded){
				this._collapseNode(node);
			}else{
				var parent = node.getParent();
				if(parent && parent.isTreeNode && !(!this.showRoot && parent === this.rootNode)){
					this.focusNode(parent);
				}
			}
		},

		focusLastChild: function(){
			// summary:
			//		End key pressed; go to last visible node.

			var node = this._getLast();
			if(node && node.isTreeNode){
				this.focusNode(node);
			}
		},

		_getFirst: function(){
			// summary:
			//		Returns the first child.
			// tags:
			//		abstract extension
			return this.showRoot ? this.rootNode : this.rootNode.getChildren()[0];
		},

		_getLast: function(){
			// summary:
			//		Returns the last descendant.
			// tags:
			//		abstract extension
			var node = this.rootNode;
			while(node.isExpanded){
				var c = node.getChildren();
				if(!c.length){
					break;
				}
				node = c[c.length - 1];
			}
			return node;
		},

		// Tree only searches forward so dir parameter is unused
		_getNext: function(node){
			// summary:
			//		Returns the next descendant, compared to "child".
			// node: Widget
			//		The current widget
			// tags:
			//		abstract extension

			if(node.isExpandable && node.isExpanded && node.hasChildren()){
				// if this is an expanded node, get the first child
				return node.getChildren()[0];		// TreeNode
			}else{
				// find a parent node with a sibling
				while(node && node.isTreeNode){
					var returnNode = node.getNextSibling();
					if(returnNode){
						return returnNode;		// TreeNode
					}
					node = node.getParent();
				}
				return null;
			}
		},

		// Implement _KeyNavContainer.childSelector, to identify which nodes are navigable
		childSelector: ".dijitTreeRow",

		isExpandoNode: function(node, widget){
			// summary:
			//		check whether a dom node is the expandoNode for a particular TreeNode widget
			return dom.isDescendant(node, widget.expandoNode) || dom.isDescendant(node, widget.expandoNodeText);
		},

		_onNodePress: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
			// Touching a node should focus it, even if you touch the expando node or the edges rather than the label.
			// Especially important to avoid _KeyNavMixin._onContainerFocus() causing the previously focused TreeNode
			// to get focus
			this.focusNode(nodeWidget);
		},

		__click: function(/*TreeNode*/ nodeWidget, /*Event*/ e, /*Boolean*/doOpen, /*String*/func){
			var domElement = e.target,
				isExpandoClick = this.isExpandoNode(domElement, nodeWidget);

			if(nodeWidget.isExpandable && (doOpen || isExpandoClick)){
				// expando node was clicked, or label of a folder node was clicked; open it
				this._onExpandoClick({node: nodeWidget});
			}else{
				this._publish("execute", { item: nodeWidget.item, node: nodeWidget, evt: e });
				this[func](nodeWidget.item, nodeWidget, e);
				this.focusNode(nodeWidget);
			}
			e.stopPropagation();
			e.preventDefault();
		},
		_onClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
			// summary:
			//		Translates click events into commands for the controller to process
			this.__click(nodeWidget, e, this.openOnClick, 'onClick');
		},
		_onDblClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
			// summary:
			//		Translates double-click events into commands for the controller to process
			this.__click(nodeWidget, e, this.openOnDblClick, 'onDblClick');
		},

		_onExpandoClick: function(/*Object*/ message){
			// summary:
			//		User clicked the +/- icon; expand or collapse my children.
			var node = message.node;

			// If we are collapsing, we might be hiding the currently focused node.
			// Also, clicking the expando node might have erased focus from the current node.
			// For simplicity's sake just focus on the node with the expando.
			this.focusNode(node);

			if(node.isExpanded){
				this._collapseNode(node);
			}else{
				this._expandNode(node);
			}
		},

		onClick: function(/*===== item, node, evt =====*/){
			// summary:
			//		Callback when a tree node is clicked
			// item: Object
			//		Object from the dojo/store corresponding to this TreeNode
			// node: TreeNode
			//		The TreeNode itself
			// evt: Event
			//		The event
			// tags:
			//		callback
		},
		onDblClick: function(/*===== item, node, evt =====*/){
			// summary:
			//		Callback when a tree node is double-clicked
			// item: Object
			//		Object from the dojo/store corresponding to this TreeNode
			// node: TreeNode
			//		The TreeNode itself
			// evt: Event
			//		The event
			// tags:
			//		callback
		},
		onOpen: function(/*===== item, node =====*/){
			// summary:
			//		Callback when a node is opened
			// item: dojo/data/Item
			// node: TreeNode
			// tags:
			//		callback
		},
		onClose: function(/*===== item, node =====*/){
			// summary:
			//		Callback when a node is closed
			// item: Object
			//		Object from the dojo/store corresponding to this TreeNode
			// node: TreeNode
			//		The TreeNode itself
			// tags:
			//		callback
		},

		_getNextNode: function(node){
			// summary:
			//		Get next visible node

			kernel.deprecated(this.declaredClass + "::_getNextNode(node) is deprecated. Use _getNext(node) instead.", "", "2.0");
			return this._getNext(node);
		},

		_getRootOrFirstNode: function(){
			// summary:
			//		Get first visible node
			kernel.deprecated(this.declaredClass + "::_getRootOrFirstNode() is deprecated. Use _getFirst() instead.", "", "2.0");
			return this._getFirst();
		},

		_collapseNode: function(/*TreeNode*/ node){
			// summary:
			//		Called when the user has requested to collapse the node
			// returns:
			//		Promise that resolves when the node has finished closing

			if(node._expandNodeDeferred){
				delete node._expandNodeDeferred;
			}

			if(node.state == "Loading"){
				// ignore clicks while we are in the process of loading data
				return;
			}

			if(node.isExpanded){
				var ret = node.collapse();

				this.onClose(node.item, node);
				this._state(node, false);

				this._startPaint(ret);	// after this finishes, need to reset widths of TreeNodes

				return ret;
			}
		},

		_expandNode: function(/*TreeNode*/ node){
			// summary:
			//		Called when the user has requested to expand the node
			// returns:
			//		Promise that resolves when the node is loaded and opened and (if persist=true) all it's descendants
			//		that were previously opened too

			if(node._expandNodeDeferred){
				// there's already an expand in progress, or completed, so just return
				return node._expandNodeDeferred;	// dojo/Deferred
			}

			var model = this.model,
				item = node.item,
				_this = this;

			// Load data if it's not already loaded
			if(!node._loadDeferred){
				// need to load all the children before expanding
				node.markProcessing();

				// Setup deferred to signal when the load and expand are finished.
				// Save that deferred in this._expandDeferred as a flag that operation is in progress.
				node._loadDeferred = new Deferred();

				// Get the children
				model.getChildren(
					item,
					function(items){
						node.unmarkProcessing();

						// Display the children and also start expanding any children that were previously expanded
						// (if this.persist == true).   The returned Deferred will fire when those expansions finish.
						node.setChildItems(items).then(function(){
							node._loadDeferred.resolve(items);
						});
					},
					function(err){
						console.error(_this, ": error loading " + node.label + " children: ", err);
						node._loadDeferred.reject(err);
					}
				);
			}

			// Expand the node after data has loaded
			var def = node._loadDeferred.then(lang.hitch(this, function(){
				var def2 = node.expand();

				// seems like these should delayed until node.expand() completes, but left here for back-compat about
				// when this.isOpen flag gets set (ie, at the beginning of the animation)
				this.onOpen(node.item, node);
				this._state(node, true);

				return def2;
			}));

			this._startPaint(def);	// after this finishes, need to reset widths of TreeNodes

			return def;	// dojo/promise/Promise
		},

		////////////////// Miscellaneous functions ////////////////

		focusNode: function(/* _tree.Node */ node){
			// summary:
			//		Focus on the specified node (which must be visible)
			// tags:
			//		protected

			var scrollLeft = this.domNode.scrollLeft;
			this.focusChild(node);
			this.domNode.scrollLeft = scrollLeft;
		},

		_onNodeMouseEnter: function(/*dijit/_WidgetBase*/ /*===== node =====*/){
			// summary:
			//		Called when mouse is over a node (onmouseenter event),
			//		this is monitored by the DND code
		},

		_onNodeMouseLeave: function(/*dijit/_WidgetBase*/ /*===== node =====*/){
			// summary:
			//		Called when mouse leaves a node (onmouseleave event),
			//		this is monitored by the DND code
		},

		//////////////// Events from the model //////////////////////////

		_onItemChange: function(/*Item*/ item){
			// summary:
			//		Processes notification of a change to an item's scalar values like label
			var model = this.model,
				identity = model.getIdentity(item),
				nodes = this._itemNodesMap[identity];

			if(nodes){
				var label = this.getLabel(item),
					tooltip = this.getTooltip(item);
				array.forEach(nodes, function(node){
					node.set({
						item: item, // theoretically could be new JS Object representing same item
						label: label,
						tooltip: tooltip
					});
					node._updateItemClasses(item);
				});
			}
		},

		_onItemChildrenChange: function(/*dojo/data/Item*/ parent, /*dojo/data/Item[]*/ newChildrenList){
			// summary:
			//		Processes notification of a change to an item's children
			var model = this.model,
				identity = model.getIdentity(parent),
				parentNodes = this._itemNodesMap[identity];

			if(parentNodes){
				array.forEach(parentNodes, function(parentNode){
					parentNode.setChildItems(newChildrenList);
				});
			}
		},

		_onItemDelete: function(/*Item*/ item){
			// summary:
			//		Processes notification of a deletion of an item.
			//		Not called from new dojo.store interface but there's cleanup code in setChildItems() instead.

			var model = this.model,
				identity = model.getIdentity(item),
				nodes = this._itemNodesMap[identity];

			if(nodes){
				array.forEach(nodes, function(node){
					// Remove node from set of selected nodes (if it's selected)
					this.dndController.removeTreeNode(node);

					var parent = node.getParent();
					if(parent){
						// if node has not already been orphaned from a _onSetItem(parent, "children", ..) call...
						parent.removeChild(node);
					}
					node.destroyRecursive();
				}, this);
				delete this._itemNodesMap[identity];
			}
		},

		/////////////// Miscellaneous funcs

		_initState: function(){
			// summary:
			//		Load in which nodes should be opened automatically
			this._openedNodes = {};
			if(this.persist && this.cookieName){
				var oreo = cookie(this.cookieName);
				if(oreo){
					array.forEach(oreo.split(','), function(item){
						this._openedNodes[item] = true;
					}, this);
				}
			}
		},

		_state: function(node, expanded){
			// summary:
			//		Query or set expanded state for an node
			if(!this.persist){
				return false;
			}
			var path = array.map(node.getTreePath(),function(item){
				return this.model.getIdentity(item);
			}, this).join("/");
			if(arguments.length === 1){
				return this._openedNodes[path];
			}else{
				if(expanded){
					this._openedNodes[path] = true;
				}else{
					delete this._openedNodes[path];
				}
				this._saveExpandedNodes();
			}
		},

		_saveExpandedNodes: function(){
			if(this.persist && this.cookieName){
				var ary = [];
				for(var id in this._openedNodes){
					ary.push(id);
				}
				cookie(this.cookieName, ary.join(","), {expires: 365});
			}
		},

		destroy: function(){
			if(this._curSearch){
				this._curSearch.timer.remove();
				delete this._curSearch;
			}
			if(this.rootNode){
				this.rootNode.destroyRecursive();
			}
			if(this.dndController && !lang.isString(this.dndController)){
				this.dndController.destroy();
			}
			this.rootNode = null;
			this.inherited(arguments);
		},

		destroyRecursive: function(){
			// A tree is treated as a leaf, not as a node with children (like a grid),
			// but defining destroyRecursive for back-compat.
			this.destroy();
		},

		resize: function(changeSize){
			if(changeSize){
				domGeometry.setMarginBox(this.domNode, changeSize);
			}

			// The main JS sizing involved w/tree is the indentation, which is specified
			// in CSS and read in through this dummy indentDetector node (tree must be
			// visible and attached to the DOM to read this).
			// If the Tree is hidden domGeometry.position(this.tree.indentDetector).w will return 0, in which case just
			// keep the default value.
			this._nodePixelIndent = domGeometry.position(this.tree.indentDetector).w || this._nodePixelIndent;

			// resize() may be called before this.rootNode is created, so wait until it's available
			this.expandChildrenDeferred.then(lang.hitch(this, function(){
				// If tree has already loaded, then reset indent for all the nodes
				this.rootNode.set('indent', this.showRoot ? 0 : -1);

				// Also, adjust widths of all rows to match width of Tree
				this._adjustWidths();
			}));
		},

		_outstandingPaintOperations: 0,
		_startPaint: function(/*Promise|Boolean*/ p){
			// summary:
			//		Called at the start of an operation that will change what's displayed.
			// p:
			//		Promise that tells when the operation will complete.  Alternately, if it's just a Boolean, it signifies
			//		that the operation was synchronous, and already completed.

			this._outstandingPaintOperations++;
			if(this._adjustWidthsTimer){
				this._adjustWidthsTimer.remove();
				delete this._adjustWidthsTimer;
			}

			var oc = lang.hitch(this, function(){
				this._outstandingPaintOperations--;

				if(this._outstandingPaintOperations <= 0 && !this._adjustWidthsTimer && this._started){
					// Use defer() to avoid a width adjustment when another operation will immediately follow,
					// such as a sequence of opening a node, then it's children, then it's grandchildren, etc.
					this._adjustWidthsTimer = this.defer("_adjustWidths");
				}
			});
			when(p, oc, oc);
		},

		_adjustWidths: function(){
			// summary:
			//		Size container to match widest TreeNode, so that highlighting with scrolling works (#13141, #16132)

			if(this._adjustWidthsTimer){
				this._adjustWidthsTimer.remove();
				delete this._adjustWidthsTimer;
			}

			this.containerNode.style.width = "auto";
			this.containerNode.style.width = this.domNode.scrollWidth > this.domNode.offsetWidth ? "auto" : "100%";
		},

		_createTreeNode: function(/*Object*/ args){
			// summary:
			//		creates a TreeNode
			// description:
			//		Developers can override this method to define their own TreeNode class;
			//		However it will probably be removed in a future release in favor of a way
			//		of just specifying a widget for the label, rather than one that contains
			//		the children too.
			return new TreeNode(args);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the previously focused child, or first child.
			//		Some applications may want to change this method to focus the [first] selected child.

			if(this.lastFocusedChild){
				this.focusNode(this.lastFocusedChild);
			}else{
				this.focusFirstChild();
			}
		}
	});

	if(has("dojo-bidi")){
		Tree.extend({
			_setTextDirAttr: function(textDir){
				if(textDir && this.textDir != textDir){
					this._set("textDir", textDir);
					this.rootNode.set("textDir", textDir);
				}
			}
		});
	}

	Tree.PathError = createError("TreePathError");
	Tree._TreeNode = TreeNode;	// for monkey patching or creating subclasses of TreeNode

	return Tree;
});

},
'dijit/form/Button':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"dojo/has", // has("dijit-legacy-requires")
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.trim
	"dojo/ready",
	"./_FormWidget",
	"./_ButtonMixin",
	"dojo/text!./templates/Button.html"
], function(require, declare, domClass, has, kernel, lang, ready, _FormWidget, _ButtonMixin, template){

	// module:
	//		dijit/form/Button

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/DropDownButton", "dijit/form/ComboButton", "dijit/form/ToggleButton"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	var Button = declare("dijit.form.Button" + (has("dojo-bidi") ? "_NoBidi" : ""), [_FormWidget, _ButtonMixin], {
		// summary:
		//		Basically the same thing as a normal HTML button, but with special styling.
		// description:
		//		Buttons can display a label, an icon, or both.
		//		A label should always be specified (through innerHTML) or the label
		//		attribute.  It can be hidden via showLabel=false.
		// example:
		// |	<button data-dojo-type="dijit/form/Button" onClick="...">Hello world</button>
		//
		// example:
		// |	var button1 = new Button({label: "hello world", onClick: foo});
		// |	dojo.body().appendChild(button1.domNode);

		// showLabel: Boolean
		//		Set this to true to hide the label text and display only the icon.
		//		(If showLabel=false then iconClass must be specified.)
		//		Especially useful for toolbars.
		//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
		//
		//		The exception case is for computers in high-contrast mode, where the label
		//		will still be displayed, since the icon doesn't appear.
		showLabel: true,

		// iconClass: String
		//		Class to apply to DOMNode in button to make it display an icon
		iconClass: "dijitNoIcon",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		baseClass: "dijitButton",

		templateString: template,

		// Map widget attributes to DOMNode attributes.
		_setValueAttr: "valueNode",
		_setNameAttr: function(name){
			// avoid breaking existing subclasses where valueNode undefined.  Perhaps in 2.0 require it to be defined?
			if(this.valueNode){
				this.valueNode.setAttribute("name", name);
			}
		},

		_fillContent: function(/*DomNode*/ source){
			// Overrides _Templated._fillContent().
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			// TODO: remove the method in 2.0, parser will do it all for me
			if(source && (!this.params || !("label" in this.params))){
				var sourceLabel = lang.trim(source.innerHTML);
				if(sourceLabel){
					this.label = sourceLabel; // _applyAttributes will be called after buildRendering completes to update the DOM
				}
			}
		},

		_setShowLabelAttr: function(val){
			if(this.containerNode){
				domClass.toggle(this.containerNode, "dijitDisplayNone", !val);
			}
			this._set("showLabel", val);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.  Use set('label', ...) instead.
			kernel.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		_setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for set('label', ...) to work.
			// description:
			//		Set the label (text) of the button; takes an HTML string.
			//		If the label is hidden (showLabel=false) then and no title has
			//		been specified, then label is also set as title attribute of icon.
			this.inherited(arguments);
			if(!this.showLabel && !("title" in this.params)){
				this.titleNode.title = lang.trim(this.containerNode.innerText || this.containerNode.textContent || '');
			}
		}
	});

	if(has("dojo-bidi")){
		Button = declare("dijit.form.Button", Button, {
			_setLabelAttr: function(/*String*/ content){
				this.inherited(arguments);
				if(this.titleNode.title){
					this.applyTextDir(this.titleNode, this.titleNode.title);
				}
			},

			_setTextDirAttr: function(/*String*/ textDir){
				if(this._created && this.textDir != textDir){
					this._set("textDir", textDir);
					this._setLabelAttr(this.label); // call applyTextDir on both focusNode and titleNode
				}
			}
		});
	}

	return Button;
});

},
'dojo/_base/unload':function(){
define(["./kernel", "./lang", "../on"], function(dojo, lang, on){

// module:
//		dojo/unload

var win = window;

var unload = {
	// summary:
	//		This module contains the document and window unload detection API.

	addOnWindowUnload: function(/*Object|Function?*/ obj, /*String|Function?*/ functionName){
		// summary:
		//		registers a function to be triggered when window.onunload
		//		fires.
		// description:
		//		The first time that addOnWindowUnload is called Dojo
		//		will register a page listener to trigger your unload
		//		handler with. Note that registering these handlers may
		//		destroy "fastback" page caching in browsers that support
		//		it. Be careful trying to modify the DOM or access
		//		JavaScript properties during this phase of page unloading:
		//		they may not always be available. Consider
		//		addOnUnload() if you need to modify the DOM or do
		//		heavy JavaScript work since it fires at the equivalent of
		//		the page's "onbeforeunload" event.
		// example:
		//	|	unload.addOnWindowUnload(functionPointer)
		//	|	unload.addOnWindowUnload(object, "functionName");
		//	|	unload.addOnWindowUnload(object, function(){ /* ... */});

		if (!dojo.windowUnloaded){
			on(win, "unload", (dojo.windowUnloaded = function(){
				// summary:
				//		signal fired by impending window destruction. You may use
				//		dojo.addOnWindowUnload() to register a listener for this
				//		event. NOTE: if you wish to dojo.connect() to this method
				//		to perform page/application cleanup, be aware that this
				//		event WILL NOT fire if no handler has been registered with
				//		addOnWindowUnload(). This behavior started in Dojo 1.3.
				//		Previous versions always triggered windowUnloaded(). See
				//		addOnWindowUnload for more info.
			}));
		}
		on(win, "unload", lang.hitch(obj, functionName));
	},

	addOnUnload: function(/*Object?|Function?*/ obj, /*String|Function?*/ functionName){
		// summary:
		//		registers a function to be triggered when the page unloads.
		// description:
		//		The first time that addOnUnload is called Dojo will
		//		register a page listener to trigger your unload handler
		//		with.
		//
		//		In a browser environment, the functions will be triggered
		//		during the window.onbeforeunload event. Be careful of doing
		//		too much work in an unload handler. onbeforeunload can be
		//		triggered if a link to download a file is clicked, or if
		//		the link is a javascript: link. In these cases, the
		//		onbeforeunload event fires, but the document is not
		//		actually destroyed. So be careful about doing destructive
		//		operations in a dojo.addOnUnload callback.
		//
		//		Further note that calling dojo.addOnUnload will prevent
		//		browsers from using a "fast back" cache to make page
		//		loading via back button instantaneous.
		// example:
		//	|	var afunc = function() {console.log("global function");};
		//	|	require(["dojo/_base/unload"], function(unload) {
		//	|		var foo = {unload: function(){ console.log("unloading...");}, data: "mydata"};
		//	|		unload.addOnUnload(afunc);
		//	|		unload.addOnUnload(foo, "unload");
		//	|		unload.addOnUnload(foo, function(){console.log("", this.data);});
		//	|	});

		on(win, "beforeunload", lang.hitch(obj, functionName));
	}
};

dojo.addOnWindowUnload = unload.addOnWindowUnload;
dojo.addOnUnload = unload.addOnUnload;

return unload;

});

},
'xcitec/LabelMultiline':function(){
// Multiline Label
define(["dojo/_base/declare", "./Label"], function(declare, Label){
  return declare(Label, {
    allowWrap: true
  });
});

},
'xcitec/page/ReportMenuItem':function(){
define([
         "dojo/dom-attr",
         "dojo/_base/declare",
         "dojo/_base/lang",
         "dojo/_base/window",
         "dojo/dom-construct",
         "dojo/dom-style",
         "dojo/fx",
         "dojo/query",
         "dijit/_Container",
         "../_StateWidget",
         "../xcitecUtil",
         "../_base/ImageFactory"
       ], function (domAttr, declare, lang, windowModule, domConstruct, domStyle, fx, query, _Container, _StateWidget, xcitecUtil, ImageFactory) {
  return declare([_StateWidget, _Container], {
    templateString: '<div class="${baseClass}">' +
                    '<div class="reportMenuItemHead" data-dojo-attach-event="onclick: _onClick">' +
                    '<img height="1" width="1" alt="" src="${_blankGif}" class="reportMenuItemIcon ${iconClass}">' +
                    '<strong data-dojo-attach-point="labelNode"></strong>' +
                    '</div>' +
                    '<div class="reportMenuItemBody" data-dojo-attach-point="animationNode">' +
                    '<div data-dojo-attach-point="containerNode" class="reportMenuItemWrap">' +
                    '</div>' +
                    '</div>' +
                    '</div>',

    baseClass: "reportMenuItem",
    selected: false,
    label: "",
    iconClass: "",

    postMixInProperties: function(){
      this.iconClass = ImageFactory.createIconCss(this.iconClass);
      this.stateHolder = query(">*[xcitecRole$=repstateholder]", this.srcNodeRef)[0];
      if(this.stateHolder){
        this.selected = xcitecUtil.str2bool(domAttr.get(this.stateHolder, "value"));
      }
      this.inherited(arguments);
    },

    _fillContent: function(){
      if(this.label != ""){
        this.labelNode.appendChild(windowModule.doc.createTextNode(this.label));
      }
      this.inherited(arguments);
    },

    postCreate: function(){
      this._setStateClass();
      if(this.stateHolder){
        domConstruct.place(this.stateHolder, this.domNode, "first");
      }
      this.inherited(arguments);
    },

    _onClick: function(){
      this.selected ? this.collapse() : this.expand();
    },

    expand: function(){
      this.selected = true;
      if(this.stateHolder){
        domAttr.set(this.stateHolder, "value", "true");
      }
      domStyle.set(this.animationNode, "display", "none");
      fx.wipeIn({ node: this.animationNode, beforeBegin: lang.hitch(this, "_setStateClass") }).play();
    },

    collapse: function(){
      this.selected = false;
      if(this.stateHolder){
        domAttr.set(this.stateHolder, "value", "false");
      }
      fx.wipeOut({ node: this.animationNode, onEnd: lang.hitch(this, "_setStateClass") }).play();
    },

    hasInnerCombo: function(){
      return false;
    }
  });
});

},
'dijit/CalendarLite':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/cldr/supplemental", // cldrSupplemental.getFirstDayOfWeek
	"dojo/date", // date
	"dojo/date/locale",
	"dojo/date/stamp", // stamp.fromISOString
	"dojo/dom", // dom.setSelectable
	"dojo/dom-class", // domClass.contains
	"dojo/_base/lang", // lang.getObject, lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie") has("webkit")
	"dojo/string", // string.substitute
	"./_WidgetBase",
	"./_TemplatedMixin",
	"dojo/text!./templates/Calendar.html",
	"./a11yclick",	// not used directly, but template has ondijitclick in it
	"./hccss"    // not used directly, but sets CSS class on <body>
], function(array, declare, cldrSupplemental, date, locale, stamp, dom, domClass, lang, on, has, string, _WidgetBase, _TemplatedMixin, template){


	// module:
	//		dijit/CalendarLite

	var CalendarLite = declare("dijit.CalendarLite", [_WidgetBase, _TemplatedMixin], {
		// summary:
		//		Lightweight version of Calendar widget aimed towards mobile use
		//
		// description:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//		This widget can't be used in a form because it doesn't serialize the date to an
		//		`<input>` field.  For a form element, use dijit/form/DateTextBox instead.
		//
		//		Note that the parser takes all dates attributes passed in the
		//		[RFC 3339 format](http://www.faqs.org/rfcs/rfc3339.html), e.g. `2005-06-30T08:05:00-07:00`
		//		so that they are serializable and locale-independent.
		//
		//		Also note that this widget isn't keyboard accessible; use dijit.Calendar for that
		// example:
		//	|	var calendar = new dijit.CalendarLite({}, dojo.byId("calendarNode"));
		//
		// example:
		//	|	<div data-dojo-type="dijit/CalendarLite"></div>

		// Template for main calendar
		templateString: template,

		// Template for cell for a day of the week (ex: M)
		dowTemplateString: '<th class="dijitReset dijitCalendarDayLabelTemplate" role="columnheader" scope="col"><span class="dijitCalendarDayLabel">${d}</span></th>',

		// Templates for a single date (ex: 13), and for a row for a week (ex: 20 21 22 23 24 25 26)
		dateTemplateString: '<td class="dijitReset" role="gridcell" data-dojo-attach-point="dateCells"><span class="dijitCalendarDateLabel" data-dojo-attach-point="dateLabels"></span></td>',
		weekTemplateString: '<tr class="dijitReset dijitCalendarWeekTemplate" role="row">${d}${d}${d}${d}${d}${d}${d}</tr>',

		// value: Date
		//		The currently selected Date, initially set to invalid date to indicate no selection.
		value: new Date(""),
		// TODO: for 2.0 make this a string (ISO format) rather than a Date

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 If unspecified, uses Gregorian calendar routines
		//		at dojo/date and dojo/date/locale.
		datePackage: "",
		//		TODO: for 2.0, replace datePackage with dateModule and dateLocalModule attributes specifying MIDs,
		//		or alternately just get rid of this completely and tell user to use module ID remapping
		//		via require

		// dayWidth: String
		//		How to represent the days of the week in the calendar header. See locale
		dayWidth: "narrow",

		// tabIndex: String
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",

		// currentFocus: Date
		//		Date object containing the currently focused date, or the date which would be focused
		//		if the calendar itself was focused.   Also indicates which year and month to display,
		//		i.e. the current "page" the calendar is on.
		currentFocus: new Date(),

		// Put the summary to the node with role=grid
		_setSummaryAttr: "gridNode",

		baseClass: "dijitCalendar",

		_isValidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking that it's a valid date, rather
			//		than blank or NaN.
			// tags:
			//		private
			return value && !isNaN(value) && typeof value == "object" &&
				value.toString() != this.constructor.prototype.value.toString();
		},

		_getValueAttr: function(){
			// summary:
			//		Support get('value')

			// this.value is set to 1AM, but return midnight, local time for back-compat
			var storedVal = this._get("value");
			if(storedVal && !isNaN(storedVal)){
				var value = new this.dateClassObj(storedVal);
				value.setHours(0, 0, 0, 0);

				// If daylight savings pushes midnight to the previous date, fix the Date
				// object to point at 1am so it will represent the correct day. See #9366
				if(value.getDate() < storedVal.getDate()){
					value = this.dateModule.add(value, "hour", 1);
				}
				return value;
			}else{
				return null;
			}
		},

		_setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
			// summary:
			//		Support set("value", ...)
			// description:
			//		Set the current date and update the UI.  If the date is disabled, the value will
			//		not change, but the display will change to the corresponding month.
			// value:
			//		Either a Date or the number of seconds since 1970.
			// tags:
			//		protected
			if(typeof value == "string"){
				value = stamp.fromISOString(value);
			}
			value = this._patchDate(value);

			if(this._isValidDate(value) && !this.isDisabledDate(value, this.lang)){
				this._set("value", value);

				// Set focus cell to the new value.   Arguably this should only happen when there isn't a current
				// focus point.   This will also repopulate the grid to new month/year if necessary.
				this.set("currentFocus", value);

				// Mark the selected date
				this._markSelectedDates([value]);

				if(this._created && (priorityChange || typeof priorityChange == "undefined")){
					this.onChange(this.get('value'));
				}
			}else{
				// clear value, and mark all dates as unselected
				this._set("value", null);
				this._markSelectedDates([]);
			}
		},

		_patchDate: function(/*Date|Number*/ value){
			// summary:
			//		Convert Number into Date, or copy Date object.   Then, round to nearest day,
			//		setting to 1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)
			if(value){
				value = new this.dateClassObj(value);
				value.setHours(1, 0, 0, 0);
			}
			return value;
		},

		_setText: function(node, text){
			// summary:
			//		This just sets the content of node to the specified text.
			//		Can't do "node.innerHTML=text" because of an IE bug w/tables, see #3434.
			// tags:
			//		private
			while(node.firstChild){
				node.removeChild(node.firstChild);
			}
			node.appendChild(node.ownerDocument.createTextNode(text));
		},

		_populateGrid: function(){
			// summary:
			//		Fills in the calendar grid with each day (1-31).
			//		Call this on creation, when moving to a new month.
			// tags:
			//		private

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			var firstDay = month.getDay(),
				daysInMonth = this.dateModule.getDaysInMonth(month),
				daysInPreviousMonth = this.dateModule.getDaysInMonth(this.dateModule.add(month, "month", -1)),
				today = new this.dateClassObj(),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			if(dayOffset > firstDay){
				dayOffset -= 7;
			}

			// If they didn't provide a summary, change the default summary to match with the new month
			if(!this.summary){
				var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month)
				this.gridNode.setAttribute("summary", monthNames[month.getMonth()]);
			}

			// Mapping from date (as specified by number returned from Date.valueOf()) to corresponding <td>
			this._date2cell = {};

			// Iterate through dates in the calendar and fill in date numbers and style info
			array.forEach(this.dateCells, function(template, idx){
				var i = idx + dayOffset;
				var date = new this.dateClassObj(month),
					number, clazz = "dijitCalendar", adj = 0;

				if(i < firstDay){
					number = daysInPreviousMonth - firstDay + i + 1;
					adj = -1;
					clazz += "Previous";
				}else if(i >= (firstDay + daysInMonth)){
					number = i - firstDay - daysInMonth + 1;
					adj = 1;
					clazz += "Next";
				}else{
					number = i - firstDay + 1;
					clazz += "Current";
				}

				if(adj){
					date = this.dateModule.add(date, "month", adj);
				}
				date.setDate(number);

				if(!this.dateModule.compare(date, today, "date")){
					clazz = "dijitCalendarCurrentDate " + clazz;
				}

				if(this.isDisabledDate(date, this.lang)){
					clazz = "dijitCalendarDisabledDate " + clazz;
					template.setAttribute("aria-disabled", "true");
				}else{
					clazz = "dijitCalendarEnabledDate " + clazz;
					template.removeAttribute("aria-disabled");
					template.setAttribute("aria-selected", "false");
				}

				var clazz2 = this.getClassForDate(date, this.lang);
				if(clazz2){
					clazz = clazz2 + " " + clazz;
				}

				template.className = clazz + "Month dijitCalendarDateTemplate";

				// Each cell has an associated integer value representing it's date
				var dateVal = date.valueOf();
				this._date2cell[dateVal] = template;
				template.dijitDateValue = dateVal;

				// Set Date string (ex: "13").
				this._setText(this.dateLabels[idx], date.getDateLocalized ? date.getDateLocalized(this.lang) : date.getDate());
			}, this);
		},

		_populateControls: function(){
			// summary:
			//		Fill in localized month, and prev/current/next years
			// tags:
			//		protected

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			// set name of this month
			this.monthWidget.set("month", month);

			var y = month.getFullYear() - 1;
			var d = new this.dateClassObj();
			array.forEach(["previous", "current", "next"], function(name){
				d.setFullYear(y++);
				this._setText(this[name + "YearLabelNode"],
					this.dateLocaleModule.format(d, {selector: 'year', locale: this.lang}));
			}, this);
		},

		goToToday: function(){
			// summary:
			//		Sets calendar's value to today's date
			this.set('value', new this.dateClassObj());
		},

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.dateModule = params.datePackage ? lang.getObject(params.datePackage, false) : date;
			this.dateClassObj = this.dateModule.Date || Date;
			this.dateLocaleModule = params.datePackage ? lang.getObject(params.datePackage + ".locale", false) : locale;
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return CalendarLite._MonthWidget({
				id: this.id + "_mddb",
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		buildRendering: function(){
			// Markup for days of the week (referenced from template)
			var d = this.dowTemplateString,
				dayNames = this.dateLocaleModule.getNames('days', this.dayWidth, 'standAlone', this.lang),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			this.dayCellsHtml = string.substitute([d, d, d, d, d, d, d].join(""), {d: ""}, function(){
				return dayNames[dayOffset++ % 7];
			});

			// Markup for dates of the month (referenced from template), but without numbers filled in
			var r = string.substitute(this.weekTemplateString, {d: this.dateTemplateString});
			this.dateRowsHtml = [r, r, r, r, r, r].join("");

			// Instantiate from template.
			// dateCells and dateLabels arrays filled when _Templated parses my template.
			this.dateCells = [];
			this.dateLabels = [];
			this.inherited(arguments);

			dom.setSelectable(this.domNode, false);

			var dateObj = new this.dateClassObj(this.currentFocus);

			this.monthWidget = this._createMonthWidget();

			this.set('currentFocus', dateObj, false);	// draw the grid to the month specified by currentFocus
		},

		postCreate: function(){
			this.inherited(arguments);
			this._connectControls();
		},

		_connectControls: function(){
			// summary:
			//		Set up connects for increment/decrement of months/years
			// tags:
			//		protected

			var connect = lang.hitch(this, function(nodeProp, part, amount){
				return on(this[nodeProp], "click", lang.hitch(this, function(){
					this._setCurrentFocusAttr(this.dateModule.add(this.currentFocus, part, amount));
				}));
			});

			this.own(
				connect("incrementMonth", "month", 1),
				connect("decrementMonth", "month", -1),
				connect("nextYearLabelNode", "year", 1),
				connect("previousYearLabelNode", "year", -1)
			);
		},

		_setCurrentFocusAttr: function(/*Date*/ date, /*Boolean*/ forceFocus){
			// summary:
			//		If the calendar currently has focus, then focuses specified date,
			//		changing the currently displayed month/year if necessary.
			//		If the calendar doesn't have focus, updates currently
			//		displayed month/year, and sets the cell that will get focus
			//		when Calendar is focused.
			// forceFocus:
			//		If true, will focus() the cell even if calendar itself doesn't have focus

			var oldFocus = this.currentFocus,
				oldCell = this._getNodeByDate(oldFocus);
			date = this._patchDate(date);

			this._set("currentFocus", date);

			// If the focus is on a different month than the current calendar month, switch the displayed month.
			// Also will populate the grid initially, on Calendar creation.
			if(!this._date2cell || this.dateModule.difference(oldFocus, date, "month") != 0){
				this._populateGrid();
				this._populateControls();
				this._markSelectedDates([this.value]);
			}

			// set tabIndex=0 on new cell, and focus it (but only if Calendar itself is focused)
			var newCell = this._getNodeByDate(date);
			newCell.setAttribute("tabIndex", this.tabIndex);
			if(this.focused || forceFocus){
				newCell.focus();
			}

			// set tabIndex=-1 on old focusable cell
			if(oldCell && oldCell != newCell){
				if(has("webkit")){    // see #11064 about webkit bug
					oldCell.setAttribute("tabIndex", "-1");
				}else{
					oldCell.removeAttribute("tabIndex");
				}
			}
		},

		focus: function(){
			// summary:
			//		Focus the calendar by focusing one of the calendar cells
			this._setCurrentFocusAttr(this.currentFocus, true);
		},

		_onDayClick: function(/*Event*/ evt){
			// summary:
			//		Handler for day clicks, selects the date if appropriate
			// tags:
			//		protected
			evt.stopPropagation();
			evt.preventDefault();
			for(var node = evt.target; node && !node.dijitDateValue; node = node.parentNode){
				;
			}
			if(node && !domClass.contains(node, "dijitCalendarDisabledDate")){
				this.set('value', node.dijitDateValue);
			}
		},

		_getNodeByDate: function(/*Date*/ value){
			// summary:
			//		Returns the cell corresponding to the date, or null if the date is not within the currently
			//		displayed month.
			value = this._patchDate(value);
			return value && this._date2cell ? this._date2cell[value.valueOf()] : null;
		},

		_markSelectedDates: function(/*Date[]*/ dates){
			// summary:
			//		Marks the specified cells as selected, and clears cells previously marked as selected.
			//		For CalendarLite at most one cell is selected at any point, but this allows an array
			//		for easy subclassing.

			// Function to mark a cell as selected or unselected
			function mark(/*Boolean*/ selected, /*DomNode*/ cell){
				domClass.toggle(cell, "dijitCalendarSelectedDate", selected);
				cell.setAttribute("aria-selected", selected ? "true" : "false");
			}

			// Clear previously selected cells.
			array.forEach(this._selectedCells || [], lang.partial(mark, false));

			// Mark newly selected cells.  Ignore dates outside the currently displayed month.
			this._selectedCells = array.filter(array.map(dates, this._getNodeByDate, this), function(n){
				return n;
			});
			array.forEach(this._selectedCells, lang.partial(mark, true));
		},

		onChange: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Called only when the selected date has changed
		},

		isDisabledDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to disable certain dates in the calendar e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension
			/*=====
			 return false; // Boolean
			 =====*/
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension

			/*=====
			 return ""; // String
			 =====*/
		}
	});

	CalendarLite._MonthWidget = declare("dijit.CalendarLite._MonthWidget", _WidgetBase, {
		// summary:
		//		Displays name of current month padded to the width of the month
		//		w/the longest name, so that changing months doesn't change width.
		//
		//		Create as:
		// |	new Calendar._MonthWidget({
		// |			lang: ...,
		// |			dateLocaleModule: ...
		// |		})

		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month),
				spacer =
					(has("ie") == 6 ? "" : "<div class='dijitSpacer'>" +
						array.map(monthNames,function(s){
							return "<div>" + s + "</div>";
						}).join("") + "</div>");

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.domNode.innerHTML =
				spacer +
					"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +
					monthNames[month.getMonth()] + "</div>";
		}
	});

	return CalendarLite;
});

},
'dojo/parser':function(){
define([
	"require", "./_base/kernel", "./_base/lang", "./_base/array", "./_base/config", "./dom", "./_base/window",
		"./_base/url", "./aspect", "./promise/all", "./date/stamp", "./Deferred", "./has", "./query", "./on", "./ready"
], function(require, dojo, dlang, darray, config, dom, dwindow, _Url, aspect, all, dates, Deferred, has, query, don, ready){

	// module:
	//		dojo/parser

	new Date("X"); // workaround for #11279, new Date("") == NaN

	// data-dojo-props etc. is not restricted to JSON, it can be any javascript
	function myEval(text){
		return eval("(" + text + ")");
	}

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	var extendCnt = 0;
	aspect.after(dlang, "extend", function(){
		extendCnt++;
	}, true);

	function getNameMap(ctor){
		// summary:
		//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
		var map = ctor._nameCaseMap, proto = ctor.prototype;

		// Create the map if it's undefined.
		// Refresh the map if a superclass was possibly extended with new methods since the map was created.
		if(!map || map._extendCnt < extendCnt){
			map = ctor._nameCaseMap = {};
			for(var name in proto){
				if(name.charAt(0) === "_"){
					continue;
				}	// skip internal properties
				map[name.toLowerCase()] = name;
			}
			map._extendCnt = extendCnt;
		}
		return map;
	}

	// Map from widget name or list of widget names(ex: "dijit/form/Button,acme/MyMixin") to a constructor.
	var _ctorMap = {};

	function getCtor(/*String[]*/ types, /*Function?*/ contextRequire){
		// summary:
		//		Retrieves a constructor.  If the types array contains more than one class/MID then the
		//		subsequent classes will be mixed into the first class and a unique constructor will be
		//		returned for that array.

		var ts = types.join();
		if(!_ctorMap[ts]){
			var mixins = [];
			for(var i = 0, l = types.length; i < l; i++){
				var t = types[i];
				// TODO: Consider swapping getObject and require in the future
				mixins[mixins.length] = (_ctorMap[t] = _ctorMap[t] || (dlang.getObject(t) || (~t.indexOf('/') &&
					(contextRequire ? contextRequire(t) : require(t)))));
			}
			var ctor = mixins.shift();
			_ctorMap[ts] = mixins.length ? (ctor.createSubclass ? ctor.createSubclass(mixins) : ctor.extend.apply(ctor, mixins)) : ctor;
		}

		return _ctorMap[ts];
	}

	var parser = {
		// summary:
		//		The Dom/Widget parsing package

		_clearCache: function(){
			// summary:
			//		Clear cached data.   Used mainly for benchmarking.
			extendCnt++;
			_ctorMap = {};
		},

		_functionFromScript: function(script, attrData){
			// summary:
			//		Convert a `<script type="dojo/method" args="a, b, c"> ... </script>`
			//		into a function
			// script: DOMNode
			//		The `<script>` DOMNode
			// attrData: String
			//		For HTML5 compliance, searches for attrData + "args" (typically
			//		"data-dojo-args") instead of "args"
			var preamble = "",
				suffix = "",
				argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args")),
				withStr = script.getAttribute("with");

			// Convert any arguments supplied in script tag into an array to be passed to the
			var fnArgs = (argsStr || "").split(/\s*,\s*/);

			if(withStr && withStr.length){
				darray.forEach(withStr.split(/\s*,\s*/), function(part){
					preamble += "with(" + part + "){";
					suffix += "}";
				});
			}

			return new Function(fnArgs, preamble + script.innerHTML + suffix);
		},

		instantiate: function(nodes, mixin, options){
			// summary:
			//		Takes array of nodes, and turns them into class instances and
			//		potentially calls a startup method to allow them to connect with
			//		any children.
			// nodes: Array
			//		Array of DOM nodes
			// mixin: Object?
			//		An object that will be mixed in with each node in the array.
			//		Values in the mixin will override values in the node, if they
			//		exist.
			// options: Object?
			//		An object used to hold kwArgs for instantiation.
			//		See parse.options argument for details.
			// returns:
			//		Array of instances.

			mixin = mixin || {};
			options = options || {};

			var dojoType = (options.scope || dojo._scopeName) + "Type", // typically "dojoType"
				attrData = "data-" + (options.scope || dojo._scopeName) + "-", // typically "data-dojo-"
				dataDojoType = attrData + "type", // typically "data-dojo-type"
				dataDojoMixins = attrData + "mixins";					// typically "data-dojo-mixins"

			var list = [];
			darray.forEach(nodes, function(node){
				var type = dojoType in mixin ? mixin[dojoType] : node.getAttribute(dataDojoType) || node.getAttribute(dojoType);
				if(type){
					var mixinsValue = node.getAttribute(dataDojoMixins),
						types = mixinsValue ? [type].concat(mixinsValue.split(/\s*,\s*/)) : [type];

					list.push({
						node: node,
						types: types
					});
				}
			});

			// Instantiate the nodes and return the list of instances.
			return this._instantiate(list, mixin, options);
		},

		_instantiate: function(nodes, mixin, options, returnPromise){
			// summary:
			//		Takes array of objects representing nodes, and turns them into class instances and
			//		potentially calls a startup method to allow them to connect with
			//		any children.
			// nodes: Array
			//		Array of objects like
			//	|		{
			//	|			ctor: Function (may be null)
			//	|			types: ["dijit/form/Button", "acme/MyMixin"] (used if ctor not specified)
			//	|			node: DOMNode,
			//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
			//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
			//	|		}
			// mixin: Object
			//		An object that will be mixed in with each node in the array.
			//		Values in the mixin will override values in the node, if they
			//		exist.
			// options: Object
			//		An options object used to hold kwArgs for instantiation.
			//		See parse.options argument for details.
			// returnPromise: Boolean
			//		Return a Promise rather than the instance; supports asynchronous widget creation.
			// returns:
			//		Array of instances, or if returnPromise is true, a promise for array of instances
			//		that resolves when instances have finished initializing.

			// Call widget constructors.   Some may be asynchronous and return promises.
			var thelist = darray.map(nodes, function(obj){
				var ctor = obj.ctor || getCtor(obj.types, options.contextRequire);
				// If we still haven't resolved a ctor, it is fatal now
				if(!ctor){
					throw new Error("Unable to resolve constructor for: '" + obj.types.join() + "'");
				}
				return this.construct(ctor, obj.node, mixin, options, obj.scripts, obj.inherited);
			}, this);

			// After all widget construction finishes, call startup on each top level instance if it makes sense (as for
			// widgets).  Parent widgets will recursively call startup on their (non-top level) children
			function onConstruct(thelist){
				if(!mixin._started && !options.noStart){
					darray.forEach(thelist, function(instance){
						if(typeof instance.startup === "function" && !instance._started){
							instance.startup();
						}
					});
				}

				return thelist;
			}

			if(returnPromise){
				return all(thelist).then(onConstruct);
			}else{
				// Back-compat path, remove for 2.0
				return onConstruct(thelist);
			}
		},

		construct: function(ctor, node, mixin, options, scripts, inherited){
			// summary:
			//		Calls new ctor(params, node), where params is the hash of parameters specified on the node,
			//		excluding data-dojo-type and data-dojo-mixins.   Does not call startup().
			// ctor: Function
			//		Widget constructor.
			// node: DOMNode
			//		This node will be replaced/attached to by the widget.  It also specifies the arguments to pass to ctor.
			// mixin: Object?
			//		Attributes in this object will be passed as parameters to ctor,
			//		overriding attributes specified on the node.
			// options: Object?
			//		An options object used to hold kwArgs for instantiation.   See parse.options argument for details.
			// scripts: DomNode[]?
			//		Array of `<script type="dojo/*">` DOMNodes.  If not specified, will search for `<script>` tags inside node.
			// inherited: Object?
			//		Settings from dir=rtl or lang=... on a node above this node.   Overrides options.inherited.
			// returns:
			//		Instance or Promise for the instance, if markupFactory() itself returned a promise

			var proto = ctor && ctor.prototype;
			options = options || {};

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};

			if(options.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dlang.mixin(params, options.defaults);
			}
			if(inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dlang.mixin(params, inherited);
			}

			// Get list of attributes explicitly listed in the markup
			var attributes;
			if(has("dom-attributes-explicit")){
				// Standard path to get list of user specified attributes
				attributes = node.attributes;
			}else if(has("dom-attributes-specified-flag")){
				// Special processing needed for IE8, to skip a few faux values in attributes[]
				attributes = darray.filter(node.attributes, function(a){
					return a.specified;
				});
			}else{
				// Special path for IE6-7, avoid (sometimes >100) bogus entries in node.attributes
				var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
					attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*\s*/, "").replace(/\s*>.*$/, "");

				attributes = darray.map(attrs.split(/\s+/), function(name){
					var lcName = name.toLowerCase();
					return {
						name: name,
						// getAttribute() doesn't work for button.value, returns innerHTML of button.
						// but getAttributeNode().value doesn't work for the form.encType or li.value
						value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
							node.getAttribute(lcName) : node.getAttributeNode(lcName).value
					};
				});
			}

			// Hash to convert scoped attribute name (ex: data-dojo17-params) to something friendly (ex: data-dojo-params)
			// TODO: remove scope for 2.0
			var scope = options.scope || dojo._scopeName,
				attrData = "data-" + scope + "-", // typically "data-dojo-"
				hash = {};
			if(scope !== "dojo"){
				hash[attrData + "props"] = "data-dojo-props";
				hash[attrData + "type"] = "data-dojo-type";
				hash[attrData + "mixins"] = "data-dojo-mixins";
				hash[scope + "type"] = "dojoType";
				hash[attrData + "id"] = "data-dojo-id";
			}

			// Read in attributes and process them, including data-dojo-props, data-dojo-type,
			// dojoAttachPoint, etc., as well as normal foo=bar attributes.
			var i = 0, item, funcAttrs = [], jsname, extra;
			while(item = attributes[i++]){
				var name = item.name,
					lcName = name.toLowerCase(),
					value = item.value;

				switch(hash[lcName] || lcName){
				// Already processed, just ignore
				case "data-dojo-type":
				case "dojotype":
				case "data-dojo-mixins":
					break;

				// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
				case "data-dojo-props":
					extra = value;
					break;

				// data-dojo-id or jsId. TODO: drop jsId in 2.0
				case "data-dojo-id":
				case "jsid":
					jsname = value;
					break;

				// For the benefit of _Templated
				case "data-dojo-attach-point":
				case "dojoattachpoint":
					params.dojoAttachPoint = value;
					break;
				case "data-dojo-attach-event":
				case "dojoattachevent":
					params.dojoAttachEvent = value;
					break;

				// Special parameter handling needed for IE
				case "class":
					params["class"] = node.className;
					break;
				case "style":
					params["style"] = node.style && node.style.cssText;
					break;
				default:
					// Normal attribute, ex: value="123"

					// Find attribute in widget corresponding to specified name.
					// May involve case conversion, ex: onclick --> onClick
					if(!(name in proto)){
						var map = getNameMap(ctor);
						name = map[lcName] || name;
					}

					// Set params[name] to value, doing type conversion
					if(name in proto){
						switch(typeof proto[name]){
						case "string":
							params[name] = value;
							break;
						case "number":
							params[name] = value.length ? Number(value) : NaN;
							break;
						case "boolean":
							// for checked/disabled value might be "" or "checked".	 interpret as true.
							params[name] = value.toLowerCase() != "false";
							break;
						case "function":
							if(value === "" || value.search(/[^\w\.]+/i) != -1){
								// The user has specified some text for a function like "return x+5"
								params[name] = new Function(value);
							}else{
								// The user has specified the name of a global function like "myOnClick"
								// or a single word function "return"
								params[name] = dlang.getObject(value, false) || new Function(value);
							}
							funcAttrs.push(name);	// prevent "double connect", see #15026
							break;
						default:
							var pVal = proto[name];
							params[name] =
								(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
									(pVal instanceof Date) ?
										(value == "" ? new Date("") :	// the NaN of dates
										value == "now" ? new Date() :	// current date
										dates.fromISOString(value)) :
								(pVal instanceof _Url) ? (dojo.baseUrl + value) :
								myEval(value);
						}
					}else{
						params[name] = value;
					}
				}
			}

			// Remove function attributes from DOMNode to prevent "double connect" problem, see #15026.
			// Do this as a separate loop since attributes[] is often a live collection (depends on the browser though).
			for(var j = 0; j < funcAttrs.length; j++){
				var lcfname = funcAttrs[j].toLowerCase();
				node.removeAttribute(lcfname);
				node[lcfname] = null;
			}

			// Mix things found in data-dojo-props into the params, overriding any direct settings
			if(extra){
				try{
					extra = myEval.call(options.propsThis, "{" + extra + "}");
					dlang.mixin(params, extra);
				}catch(e){
					// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
					throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
				}
			}

			// Any parameters specified in "mixin" override everything else.
			dlang.mixin(params, mixin);

			// Get <script> nodes associated with this widget, if they weren't specified explicitly
			if(!scripts){
				scripts = (ctor && (ctor._noScript || proto._noScript) ? [] : query("> script[type^='dojo/']", node));
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" data-dojo-event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" data-dojo-event="foo"> tags are dojo.connected after instantiation,
			// and likewise with <script type="dojo/aspect" data-dojo-method="foo">
			// <script type="dojo/watch" data-dojo-prop="foo"> tags are dojo.watch after instantiation
			// <script type="dojo/on" data-dojo-event="foo"> tags are dojo.on after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var aspects = [],	// aspects to connect after instantiation
				calls = [],		// functions to call after instantiation
				watches = [],  // functions to watch after instantiation
				ons = []; // functions to on after instantiation

			if(scripts){
				for(i = 0; i < scripts.length; i++){
					var script = scripts[i];
					node.removeChild(script);
					// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
					var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
						prop = script.getAttribute(attrData + "prop"),
						method = script.getAttribute(attrData + "method"),
						advice = script.getAttribute(attrData + "advice"),
						scriptType = script.getAttribute("type"),
						nf = this._functionFromScript(script, attrData);
					if(event){
						if(scriptType == "dojo/connect"){
							aspects.push({ method: event, func: nf });
						}else if(scriptType == "dojo/on"){
							ons.push({ event: event, func: nf });
						}else{
							// <script type="dojo/method" data-dojo-event="foo">
							// TODO for 2.0: use data-dojo-method="foo" instead (also affects dijit/Declaration)
							params[event] = nf;
						}
					}else if(scriptType == "dojo/aspect"){
						aspects.push({ method: method, advice: advice, func: nf });
					}else if(scriptType == "dojo/watch"){
						watches.push({ prop: prop, func: nf });
					}else{
						calls.push(nf);
					}
				}
			}

			// create the instance
			var markupFactory = ctor.markupFactory || proto.markupFactory;
			var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);

			function onInstantiate(instance){
				// map it to the JS namespace if that makes sense
				if(jsname){
					dlang.setObject(jsname, instance);
				}

				// process connections and startup functions
				for(i = 0; i < aspects.length; i++){
					aspect[aspects[i].advice || "after"](instance, aspects[i].method, dlang.hitch(instance, aspects[i].func), true);
				}
				for(i = 0; i < calls.length; i++){
					calls[i].call(instance);
				}
				for(i = 0; i < watches.length; i++){
					instance.watch(watches[i].prop, watches[i].func);
				}
				for(i = 0; i < ons.length; i++){
					don(instance, ons[i].event, ons[i].func);
				}

				return instance;
			}

			if(instance.then){
				return instance.then(onInstantiate);
			}else{
				return onInstantiate(instance);
			}
		},

		scan: function(root, options){
			// summary:
			//		Scan a DOM tree and return an array of objects representing the DOMNodes
			//		that need to be turned into widgets.
			// description:
			//		Search specified node (or document root node) recursively for class instances
			//		and return an array of objects that represent potential widgets to be
			//		instantiated. Searches for either data-dojo-type="MID" or dojoType="MID" where
			//		"MID" is a module ID like "dijit/form/Button" or a fully qualified Class name
			//		like "dijit/form/Button".  If the MID is not currently available, scan will
			//		attempt to require() in the module.
			//
			//		See parser.parse() for details of markup.
			// root: DomNode?
			//		A default starting root node from which to start the parsing. Can be
			//		omitted, defaulting to the entire document. If omitted, the `options`
			//		object can be passed in this place. If the `options` object has a
			//		`rootNode` member, that is used.
			// options: Object
			//		a kwArgs options object, see parse() for details
			//
			// returns: Promise
			//		A promise that is resolved with the nodes that have been parsed.

			var list = [], // Output List
				mids = [], // An array of modules that are not yet loaded
				midsHash = {}; // Used to keep the mids array unique

			var dojoType = (options.scope || dojo._scopeName) + "Type", // typically "dojoType"
				attrData = "data-" + (options.scope || dojo._scopeName) + "-", // typically "data-dojo-"
				dataDojoType = attrData + "type", // typically "data-dojo-type"
				dataDojoTextDir = attrData + "textdir", // typically "data-dojo-textdir"
				dataDojoMixins = attrData + "mixins";					// typically "data-dojo-mixins"

			// Info on DOMNode currently being processed
			var node = root.firstChild;

			// Info on parent of DOMNode currently being processed
			//	- inherited: dir, lang, and textDir setting of parent, or inherited by parent
			//	- parent: pointer to identical structure for my parent (or null if no parent)
			//	- scripts: if specified, collects <script type="dojo/..."> type nodes from children
			var inherited = options.inherited;
			if(!inherited){
				function findAncestorAttr(node, attr){
					return (node.getAttribute && node.getAttribute(attr)) ||
						(node.parentNode && findAncestorAttr(node.parentNode, attr));
				}

				inherited = {
					dir: findAncestorAttr(root, "dir"),
					lang: findAncestorAttr(root, "lang"),
					textDir: findAncestorAttr(root, dataDojoTextDir)
				};
				for(var key in inherited){
					if(!inherited[key]){
						delete inherited[key];
					}
				}
			}

			// Metadata about parent node
			var parent = {
				inherited: inherited
			};

			// For collecting <script type="dojo/..."> type nodes (when null, we don't need to collect)
			var scripts;

			// when true, only look for <script type="dojo/..."> tags, and don't recurse to children
			var scriptsOnly;

			function getEffective(parent){
				// summary:
				//		Get effective dir, lang, textDir settings for specified obj
				//		(matching "parent" object structure above), and do caching.
				//		Take care not to return null entries.
				if(!parent.inherited){
					parent.inherited = {};
					var node = parent.node,
						grandparent = getEffective(parent.parent);
					var inherited = {
						dir: node.getAttribute("dir") || grandparent.dir,
						lang: node.getAttribute("lang") || grandparent.lang,
						textDir: node.getAttribute(dataDojoTextDir) || grandparent.textDir
					};
					for(var key in inherited){
						if(inherited[key]){
							parent.inherited[key] = inherited[key];
						}
					}
				}
				return parent.inherited;
			}

			// DFS on DOM tree, collecting nodes with data-dojo-type specified.
			while(true){
				if(!node){
					// Finished this level, continue to parent's next sibling
					if(!parent || !parent.node){
						break;
					}
					node = parent.node.nextSibling;
					scriptsOnly = false;
					parent = parent.parent;
					scripts = parent.scripts;
					continue;
				}

				if(node.nodeType != 1){
					// Text or comment node, skip to next sibling
					node = node.nextSibling;
					continue;
				}

				if(scripts && node.nodeName.toLowerCase() == "script"){
					// Save <script type="dojo/..."> for parent, then continue to next sibling
					type = node.getAttribute("type");
					if(type && /^dojo\/\w/i.test(type)){
						scripts.push(node);
					}
					node = node.nextSibling;
					continue;
				}
				if(scriptsOnly){
					// scriptsOnly flag is set, we have already collected scripts if the parent wants them, so now we shouldn't
					// continue further analysis of the node and will continue to the next sibling
					node = node.nextSibling;
					continue;
				}

				// Check for data-dojo-type attribute, fallback to backward compatible dojoType
				// TODO: Remove dojoType in 2.0
				var type = node.getAttribute(dataDojoType) || node.getAttribute(dojoType);

				// Short circuit for leaf nodes containing nothing [but text]
				var firstChild = node.firstChild;
				if(!type && (!firstChild || (firstChild.nodeType == 3 && !firstChild.nextSibling))){
					node = node.nextSibling;
					continue;
				}

				// Meta data about current node
				var current;

				var ctor = null;
				if(type){
					// If dojoType/data-dojo-type specified, add to output array of nodes to instantiate.
					var mixinsValue = node.getAttribute(dataDojoMixins),
						types = mixinsValue ? [type].concat(mixinsValue.split(/\s*,\s*/)) : [type];

					// Note: won't find classes declared via dojo/Declaration or any modules that haven't been
					// loaded yet so use try/catch to avoid throw from require()
					try{
						ctor = getCtor(types, options.contextRequire);
					}catch(e){}

					// If the constructor was not found, check to see if it has modules that can be loaded
					if(!ctor){
						darray.forEach(types, function(t){
							if(~t.indexOf('/') && !midsHash[t]){
								// If the type looks like a MID and it currently isn't in the array of MIDs to load, add it.
								midsHash[t] = true;
								mids[mids.length] = t;
							}
						});
					}

					var childScripts = ctor && !ctor.prototype._noScript ? [] : null; // <script> nodes that are parent's children

					// Setup meta data about this widget node, and save it to list of nodes to instantiate
					current = {
						types: types,
						ctor: ctor,
						parent: parent,
						node: node,
						scripts: childScripts
					};
					current.inherited = getEffective(current); // dir & lang settings for current node, explicit or inherited
					list.push(current);
				}else{
					// Meta data about this non-widget node
					current = {
						node: node,
						scripts: scripts,
						parent: parent
					};
				}

				// Recurse, collecting <script type="dojo/..."> children, and also looking for
				// descendant nodes with dojoType specified (unless the widget has the stopParser flag).
				// When finished with children, go to my next sibling.
				scripts = childScripts;
				scriptsOnly = node.stopParser || (ctor && ctor.prototype.stopParser && !(options.template));
				parent = current;
				node = firstChild;
			}

			var d = new Deferred();

			// If there are modules to load then require them in
			if(mids.length){
				// Warn that there are modules being auto-required
				if(has("dojo-debug-messages")){
					console.warn("WARNING: Modules being Auto-Required: " + mids.join(", "));
				}
				var r = options.contextRequire || require;
				r(mids, function(){
					// Go through list of widget nodes, filling in missing constructors, and filtering out nodes that shouldn't
					// be instantiated due to a stopParser flag on an ancestor that we belatedly learned about due to
					// auto-require of a module like ContentPane.   Assumes list is in DFS order.
					d.resolve(darray.filter(list, function(widget){
						if(!widget.ctor){
							// Attempt to find the constructor again.   Still won't find classes defined via
							// dijit/Declaration so need to try/catch.
							try{
								widget.ctor = getCtor(widget.types, options.contextRequire);
							}catch(e){}
						}

						// Get the parent widget
						var parent = widget.parent;
						while(parent && !parent.types){
							parent = parent.parent;
						}

						// Return false if this node should be skipped due to stopParser on an ancestor.
						// Since list[] is in DFS order, this loop will always set parent.instantiateChildren before
						// trying to compute widget.instantiate.
						var proto = widget.ctor && widget.ctor.prototype;
						widget.instantiateChildren = !(proto && proto.stopParser && !(options.template));
						widget.instantiate = !parent || (parent.instantiate && parent.instantiateChildren);
						return widget.instantiate;
					}));
				});
			}else{
				// There were no modules to load, so just resolve with the parsed nodes.   This separate code path is for
				// efficiency, to avoid running the require() and the callback code above.
				d.resolve(list);
			}

			// Return the promise
			return d.promise;
		},

		_require: function(/*DOMNode*/ script, /*Object?*/ options){
			// summary:
			//		Helper for _scanAMD().  Takes a `<script type=dojo/require>bar: "acme/bar", ...</script>` node,
			//		calls require() to load the specified modules and (asynchronously) assign them to the specified global
			//		variables, and returns a Promise for when that operation completes.
			//
			//		In the example above, it is effectively doing a require(["acme/bar", ...], function(a){ bar = a; }).

			var hash = myEval("{" + script.innerHTML + "}"), // can't use dojo/json::parse() because maybe no quotes
				vars = [],
				mids = [],
				d = new Deferred();

			var contextRequire = (options && options.contextRequire) || require;

			for(var name in hash){
				vars.push(name);
				mids.push(hash[name]);
			}

			contextRequire(mids, function(){
				for(var i = 0; i < vars.length; i++){
					dlang.setObject(vars[i], arguments[i]);
				}
				d.resolve(arguments);
			});

			return d.promise;
		},

		_scanAmd: function(root, options){
			// summary:
			//		Scans the DOM for any declarative requires and returns their values.
			// description:
			//		Looks for `<script type=dojo/require>bar: "acme/bar", ...</script>` node, calls require() to load the
			//		specified modules and (asynchronously) assign them to the specified global variables,
			//		and returns a Promise for when those operations complete.
			// root: DomNode
			//		The node to base the scan from.
			// options: Object?
			//		a kwArgs options object, see parse() for details

			// Promise that resolves when all the <script type=dojo/require> nodes have finished loading.
			var deferred = new Deferred(),
				promise = deferred.promise;
			deferred.resolve(true);

			var self = this;
			query("script[type='dojo/require']", root).forEach(function(node){
				// Fire off require() call for specified modules.  Chain this require to fire after
				// any previous requires complete, so that layers can be loaded before individual module require()'s fire.
				promise = promise.then(function(){
					return self._require(node, options);
				});

				// Remove from DOM so it isn't seen again
				node.parentNode.removeChild(node);
			});

			return promise;
		},

		parse: function(rootNode, options){
			// summary:
			//		Scan the DOM for class instances, and instantiate them.
			// description:
			//		Search specified node (or root node) recursively for class instances,
			//		and instantiate them. Searches for either data-dojo-type="Class" or
			//		dojoType="Class" where "Class" is a a fully qualified class name,
			//		like `dijit/form/Button`
			//
			//		Using `data-dojo-type`:
			//		Attributes using can be mixed into the parameters used to instantiate the
			//		Class by using a `data-dojo-props` attribute on the node being converted.
			//		`data-dojo-props` should be a string attribute to be converted from JSON.
			//
			//		Using `dojoType`:
			//		Attributes are read from the original domNode and converted to appropriate
			//		types by looking up the Class prototype values. This is the default behavior
			//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
			//		go away in Dojo 2.0.
			// rootNode: DomNode?
			//		A default starting root node from which to start the parsing. Can be
			//		omitted, defaulting to the entire document. If omitted, the `options`
			//		object can be passed in this place. If the `options` object has a
			//		`rootNode` member, that is used.
			// options: Object?
			//		A hash of options.
			//
			//		- noStart: Boolean?:
			//			when set will prevent the parser from calling .startup()
			//			when locating the nodes.
			//		- rootNode: DomNode?:
			//			identical to the function's `rootNode` argument, though
			//			allowed to be passed in via this `options object.
			//		- template: Boolean:
			//			If true, ignores ContentPane's stopParser flag and parses contents inside of
			//			a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
			//			nested inside the ContentPane to work.
			//		- inherited: Object:
			//			Hash possibly containing dir and lang settings to be applied to
			//			parsed widgets, unless there's another setting on a sub-node that overrides
			//		- scope: String:
			//			Root for attribute names to search for.   If scopeName is dojo,
			//			will search for data-dojo-type (or dojoType).   For backwards compatibility
			//			reasons defaults to dojo._scopeName (which is "dojo" except when
			//			multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			//		- propsThis: Object:
			//			If specified, "this" referenced from data-dojo-props will refer to propsThis.
			//			Intended for use from the widgets-in-template feature of `dijit._WidgetsInTemplateMixin`
			//		- contextRequire: Function:
			//			If specified, this require is utilised for looking resolving modules instead of the
			//			`dojo/parser` context `require()`.  Intended for use from the widgets-in-template feature of
			//			`dijit._WidgetsInTemplateMixin`.
			// returns: Mixed
			//		Returns a blended object that is an array of the instantiated objects, but also can include
			//		a promise that is resolved with the instantiated objects.  This is done for backwards
			//		compatibility.  If the parser auto-requires modules, it will always behave in a promise
			//		fashion and `parser.parse().then(function(instances){...})` should be used.
			// example:
			//		Parse all widgets on a page:
			//	|		parser.parse();
			// example:
			//		Parse all classes within the node with id="foo"
			//	|		parser.parse(dojo.byId('foo'));
			// example:
			//		Parse all classes in a page, but do not call .startup() on any
			//		child
			//	|		parser.parse({ noStart: true })
			// example:
			//		Parse all classes in a node, but do not call .startup()
			//	|		parser.parse(someNode, { noStart:true });
			//	|		// or
			//	|		parser.parse({ noStart:true, rootNode: someNode });

			// determine the root node and options based on the passed arguments.
			var root;
			if(!options && rootNode && rootNode.rootNode){
				options = rootNode;
				root = options.rootNode;
			}else if(rootNode && dlang.isObject(rootNode) && !("nodeType" in rootNode)){
				options = rootNode;
			}else{
				root = rootNode;
			}
			root = root ? dom.byId(root) : dwindow.body();

			options = options || {};

			var mixin = options.template ? { template: true } : {},
				instances = [],
				self = this;

			// First scan for any <script type=dojo/require> nodes, and execute.
			// Then scan for all nodes with data-dojo-type, and load any unloaded modules.
			// Then build the object instances.  Add instances to already existing (but empty) instances[] array,
			// which may already have been returned to caller.  Also, use otherwise to collect and throw any errors
			// that occur during the parse().
			var p =
				this._scanAmd(root, options).then(function(){
					return self.scan(root, options);
				}).then(function(parsedNodes){
					return self._instantiate(parsedNodes, mixin, options, true);
				}).then(function(_instances){
					// Copy the instances into the instances[] array we declared above, and are accessing as
					// our return value.
					return instances = instances.concat(_instances);
				}).otherwise(function(e){
					// TODO Modify to follow better pattern for promise error management when available
					console.error("dojo/parser::parse() error", e);
					throw e;
				});

			// Blend the array with the promise
			dlang.mixin(instances, p);
			return instances;
		}
	};

	if( 1 ){
		dojo.parser = parser;
	}

	// Register the parser callback. It should be the first callback
	// after the a11y test.
	if(config.parseOnLoad){
		ready(100, parser, "parse");
	}

	return parser;
});

},
'xcitec/layout/ContentAreaPane':function(){
define(["dojo/_base/declare",
        "dojo/_base/window",
        "dojo/dom-class",
        "./_LayoutWidget",
        "./_ContentAreaPaneItem"], function(declare, windowModule, domClass, _LayoutWidget, _ContentAreaPaneItem){
  return declare(_LayoutWidget, {
    baseClass: "ContentAreaPane",
    onlyWidgetsAllowed: false,

    buildControlHolderNode: function(/*WidgetDomNode*/ controlNode, /*LabelDomNode*/ labelNode){
      var controlContainer = new _ContentAreaPaneItem({controlNode:controlNode, labelNode:labelNode});
      return controlContainer.domNode;
    },

    buildDelimiterNode: function(){
      var hr = windowModule.doc.createElement('div');
      domClass.add(hr, "ContentAreaHR");
      return hr;
    }
  });
});

},
'xcitec/_StateWidget':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/window",
  "dijit/focus",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetBase"
], function (array, declare, windowModule, focusUtil, _Templated, _Widget) {

	var _StateWidget = declare( [_Widget, _Templated], {
	  // stripped from dijit.form._FormWidget
	
	  // baseClass: [protected] String
	  //		Root CSS class of the widget (ex: dijitTextBox), used to add CSS classes of widget
	  //		(ex: "dijitTextBox dijitTextBoxInvalid dijitTextBoxFocused dijitTextBoxInvalidFocused")
	  //		See _setStateClass().
	  baseClass: "",
	
	  // disabled: Boolean
	  //		Should this widget respond to user input?
	  //		In markup, this is specified as "disabled='disabled'", or just "disabled".
	  disabled: false,
	
	  // readOnly: Boolean
	  //		Should this widget respond to user input?
	  //		In markup, this is specified as "readOnly".
	  //		Similar to disabled except readOnly form values are submitted.
	  readOnly: false,
	
	  _setDisabledAttr: function(/*Boolean*/ value){
	    this.disabled = value;
	
	    if(value){
	      //reset those, because after the domNode is disabled, we can no longer receive
	      //mouse related events
	      this._hovering = false;
	      this._active = false;
	    }
	    this._setStateClass();
	  },
	
	  _setReadOnlyAttr: function(/*Boolean*/ value){
	    this.readOnly = value;
	    if(value){
	      this._hovering = false;
	      this._active = false;
	    }
	    this._setStateClass();
	  },
	
	  _onMouse : function(/*Event*/ event){
	    // summary:
	    //	Sets _hovering, _active, and stateModifier properties depending on mouse state,
	    //	then calls setStateClass() to set appropriate CSS classes for this.domNode.
	    //
	    //	To get a different CSS class for hover, send onmouseover and onmouseout events to this method.
	    //	To get a different CSS class while mouse button is depressed, send onmousedown to this method.
	
	    var mouseNode = event.currentTarget;
	    if(mouseNode && mouseNode.getAttribute){
	      this.stateModifier = mouseNode.getAttribute("stateModifier") || "";
	    }
	
	    if(!this.disabled){
	      switch(event.type){
	        case "mouseenter":
	        case "mouseover":
	          this._hovering = true;
	          this._active = this._mouseDown;
	          break;
	
	        case "mouseout":
	        case "mouseleave":
	          this._hovering = false;
	          this._active = false;
	          break;
	
	        case "mousedown" :
	          this._active = true;
	          this._mouseDown = true;
	          // set a global event to handle mouseup, so it fires properly
	          //	even if the cursor leaves the button
	          var mouseUpConnector = this.connect(windowModule.body(), "onmouseup", function(){
	            //if user clicks on the button, even if the mouse is released outside of it,
	            //this button should get focus (which mimics native browser buttons)
	            if(this._mouseDown && this.isFocusable()){
	              this.focus();
	            }
	            this._active = false;
	            this._mouseDown = false;
	            this._setStateClass();
	            this.disconnect(mouseUpConnector);
	          });
	          break;
	      }
	      this._setStateClass();
	    }
	  },
	
		_onFocus: function(){
			this._setStateClass();
		},
	
		setFocus: function(){
      focusUtil.focus(this.stateNode);
		},
	
	  _setStateClass: function(){
	    // summary:
	    //		Update the visual state of the widget by setting the css classes on this.domNode
	    //		(or this.stateNode if defined) by combining this.baseClass with
	    //		various suffixes that represent the current widget state(s).
	    //
	    // description:
	    //		In the case where a widget has multiple
	    //		states, it sets the class based on all possible
	    //	 	combinations.  For example, an invalid form widget that is being hovered
	    //		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
	    //
	    //		For complex widgets with multiple regions, there can be various hover/active states,
	    //		such as "Hover" or "CloseButtonHover" (for tab buttons).
	    //		This is controlled by a stateModifier="CloseButton" attribute on the close button node.
	    //
	    //		The widget may have one or more of the following states, determined
	    //		by this.state, this.checked, this.valid, and this.selected:
	    //			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
	    //			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
	    //			- Selected - ex: currently selected tab will have this.selected==true
	    //
	    //		In addition, it may have one or more of the following states,
	    //		based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
	    //			- Disabled	- if the widget is disabled
	    //			- Active		- if the mouse (or space/enter key?) is being pressed down
	    //			- Focused		- if the widget has focus
	    //			- Hover		- if the mouse is over the widget
	
	    // Compute new set of classes
	    var newStateClasses = this.baseClass.split(" ");
	
	    function multiply(modifier){
	//      newStateClasses = newStateClasses.concat(dojo.map(newStateClasses, function(c){ return c+modifier; }));
	      newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
	    }
	
	    if(this.checked){
	      multiply("Checked");
	    }
	    if(this.state){
	      multiply(this.state);
	    }
	    if(this.selected){
	      multiply("Selected");
	    }
	
	    if(this.disabled){
	      multiply("Disabled");
	    }else if(this.readOnly){
	      multiply("ReadOnly");
	    }else if(this._active){
	      multiply(this.stateModifier+"Active");
	    }else{
	      if(this._focused){
	        multiply("Focused");
	      }
	      if(this._hovering){
	        multiply(this.stateModifier+"Hover");
	      }
	    }
	
	    // Remove old state classes and add new ones.
	    // For performance concerns we only write into domNode.className once.
	    var tn = this.stateNode || this.domNode,
	      classHash = {};	// set of all classes (state and otherwise) for node
	
	    array.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });
	
	    if("_stateClasses" in this){
	      array.forEach(this._stateClasses, function(c){ delete classHash[c]; });
	    }
	
	    array.forEach(newStateClasses, function(c){ classHash[c] = true; });
	
	    var newClasses = [];
	    for(var c in classHash){
	      newClasses.push(c);
	    }
	    tn.className = newClasses.join(" ");
	
	    this._stateClasses = newStateClasses;
	  },
	
	  create: function(){
	    // Overrides _Widget.create()
	    this.inherited(arguments);
	    this._setStateClass();
	  }
	});
	
	
	return _StateWidget;
});

},
'dijit/Toolbar':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/keys", // keys.LEFT_ARROW keys.RIGHT_ARROW
	"dojo/ready",
	"./_Widget",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(require, declare, has, keys, ready, _Widget, _KeyNavContainer, _TemplatedMixin){

	// module:
	//		dijit/Toolbar


	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/ToolbarSeparator"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.Toolbar", [_Widget, _TemplatedMixin, _KeyNavContainer], {
		// summary:
		//		A Toolbar widget, used to hold things like `dijit/Editor` buttons

		templateString:
			'<div class="dijit" role="toolbar" tabIndex="${tabIndex}" data-dojo-attach-point="containerNode">' +
			'</div>',

		baseClass: "dijitToolbar",

		_onLeftArrow: function(){
			this.focusPrev();
		},

		_onRightArrow: function(){
			this.focusNext();
		}
	});
});

},
'dojo/promise/all':function(){
define([
	"../_base/array",
	"../Deferred",
	"../when"
], function(array, Deferred, when){
	"use strict";

	// module:
	//		dojo/promise/all

	var some = array.some;

	return function all(objectOrArray){
		// summary:
		//		Takes multiple promises and returns a new promise that is fulfilled
		//		when all promises have been fulfilled.
		// description:
		//		Takes multiple promises and returns a new promise that is fulfilled
		//		when all promises have been fulfilled. If one of the promises is rejected,
		//		the returned promise is also rejected. Canceling the returned promise will
		//		*not* cancel any passed promises.
		// objectOrArray: Object|Array?
		//		The promise will be fulfilled with a list of results if invoked with an
		//		array, or an object of results when passed an object (using the same
		//		keys). If passed neither an object or array it is resolved with an
		//		undefined value.
		// returns: dojo/promise/Promise

		var object, array;
		if(objectOrArray instanceof Array){
			array = objectOrArray;
		}else if(objectOrArray && typeof objectOrArray === "object"){
			object = objectOrArray;
		}

		var results;
		var keyLookup = [];
		if(object){
			array = [];
			for(var key in object){
				if(Object.hasOwnProperty.call(object, key)){
					keyLookup.push(key);
					array.push(object[key]);
				}
			}
			results = {};
		}else if(array){
			results = [];
		}

		if(!array || !array.length){
			return new Deferred().resolve(results);
		}

		var deferred = new Deferred();
		deferred.promise.always(function(){
			results = keyLookup = null;
		});
		var waiting = array.length;
		some(array, function(valueOrPromise, index){
			if(!object){
				keyLookup.push(index);
			}
			when(valueOrPromise, function(value){
				if(!deferred.isFulfilled()){
					results[keyLookup[index]] = value;
					if(--waiting === 0){
						deferred.resolve(results);
					}
				}
			}, deferred.reject);
			return deferred.isFulfilled();
		});
		return deferred.promise;	// dojo/promise/Promise
	};
});

},
'xcitec/ToolbarSeparator':function(){
define([
         "dojo/_base/declare",
         "dijit/ToolbarSeparator"
       ], function (declare, ToolbarSeparator) {
  return declare(ToolbarSeparator,	{
  });
});

},
'xcitec/form/RadioButton':function(){
define([
	"dojo/_base/declare",
	"dojo/text!xcitec/form/templates/RadioButton.html",
  "dijit/form/RadioButton",
	"../Label",
	"./_FormWidget",
  "../_base/ImageFactory",
  "dijit/_WidgetsInTemplateMixin",
	'../xcitecUtil'
], function (declare, template, RadioButton, Label, _FormWidget, ImageFactory, _WidgetsInTemplateMixin, xcitecUtil) {

	var RadioButton = declare( [RadioButton, _FormWidget, _WidgetsInTemplateMixin], {
	    labelTypeAttrSetting: "xcitec/LabelMultiline",
	    baseClass: "dijitXcitecRadio",
	
	    caption: "",
	    captionId: "",
	    parent: null,
	    index: -1,
	    icon: "",
	    required:false,
        widgetsInTemplate: false,
	
	    templateString: template,
	
	    postMixInProperties: function() {
	        this.inherited(arguments);
	        if (this.icon) {
	            this.icon = ImageFactory.createIconCss(this.icon);
	        }
	        if (this.fitToContainer) {
	            this.labelTypeAttrSetting = "xcitec/Label";
	        }
	        if (this.caption) {
				this.caption = xcitecUtil.escapeXml(this.caption);
	            this.widgetsInTemplate = true;
	        }
	        if (this.required) {
	            this.baseClass = this.baseClass + " dijitXcitecRadioboxRequired";
	        }
	    },
	
	    _setLabelAttr: function(/*String*/ content) {
	        // overriding of Button.js _setLabelAttr method
	    }
	});
	return RadioButton;
});

},
'url:slmui/layout/templates/Multicolumnlayout.html':"<div>\r\n\t<table class=\"MulticolumnLayoutTable\">\r\n\t\t<tr data-dojo-attach-point=\"containerNode\"></tr>\r\n\t</table>\r\n</div>",
'url:dijit/templates/Tree.html':"<div role=\"tree\">\n\t<div class=\"dijitInline dijitTreeIndent\" style=\"position: absolute; top: -9999px\" data-dojo-attach-point=\"indentDetector\"></div>\n\t<div class=\"dijitTreeExpando dijitTreeExpandoLoading\" data-dojo-attach-point=\"rootLoadingIndicator\"></div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\">\n\t</div>\n</div>\n",
'url:xcitec/form/templates/EntitySelect.html':"<div class=\"dijit dijitReset dijitInlineTable dijitLeft\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow:hidden;\">\r\n    <div class=\"dijitReset dijitXcitecClose\" data-dojo-attach-event=\"onclick: onDelete\"></div>\r\n    <img class=\"${frontIconClass}\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"frontIconNode\">\r\n\r\n    <div class=\"dijitReset dijitXcitecRight\">\r\n      <div data-dojo-attach-point=\"infoButton\" data-dojo-attach-event=\"onDoAction:_onClickInfoButton\" data-dojo-type=\"${!xbtnInfoClass}\"\r\n           data-dojo-props='\"iconClass\":\"${infoIconClass}\"' ${disabledInfoAttrSetting} ${readOnlyInfoAttrSetting}></div><div data-dojo-attach-point=\"chooseButton\" data-dojo-attach-event=\"onDoAction:_onClickChooseButton\" data-dojo-type=\"xcitec/form/_XButtonRight\"\r\n           data-dojo-props='\"iconType\":\"Choose\"' ${disabledChooseAttrSetting}></div>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecMiddle\">\r\n      <div class=\"dijitReset\" data-dojo-attach-point=\"stateNode\">\r\n        <div class=\"dijitReset dijitXcitecInputField\">\r\n          <div class=\"dijitReset dijitXcitecInputFieldContent\" data-dojo-attach-point=\"textbox,focusNode\">\r\n            <img class=\"iconWarning\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\">\r\n            <img class=\"${iconClass}\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"imageNode\">\r\n            <span data-dojo-attach-point=\"textNode\"></span>\r\n          </div>\r\n          <input type=\"text\" style=\"display:none;\" autocomplete=\"off\" data-dojo-attach-point=\"valueNode\" ${!nameAttrSetting}>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>\r\n",
'url:dijit/form/templates/Button.html':"<span class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdata-dojo-attach-event=\"ondijitclick:__onClick\" role=\"presentation\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\tdata-dojo-attach-event=\"onclick:_onClick\"\n\t\ttabIndex=\"-1\" role=\"presentation\" aria-hidden=\"true\" data-dojo-attach-point=\"valueNode\"\n/></span>\n",
'url:xcitec/formx/templates/STICriterion.html':"<div class=\"stiComboExpendable stiComboExpendableBgInner\" data-dojo-attach-point=\"containerNode\">\r\n  <div class=\"stiComboExpendableWrapper\">\r\n    <div class=\"${baseClass}\" data-dojo-attach-point=\"stateNode\">\r\n      <div class=\"stiComboExpendableColumn stiComboExpendableColumnLeft clearFix\">\r\n\r\n        <div class=\"stiComboExpendableRight\">\r\n          <div class=\"stiComboExpendablstilue\"></div>\r\n          <div class=\"stiComboWeight\" data-dojo-attach-point=\"weight\"></div>\r\n        </div>\r\n        <div class=\"stiComboExpendableLeft\" data-dojo-attach-point=\"hierarchyNode\">\r\n          <div class=\"stiComboExpendableTitle\">\r\n            <img src=\"${_blankGif}\" class=\"stiComboExpendableTitleIcon\" alt=\"\" data-dojo-attach-point=\"icon\" data-dojo-attach-event=\"onclick:_onClick\">\r\n            <img src=\"${_blankGif}\" class=\"${iconClass}\" alt=\"\" data-dojo-attach-point=\"statusIcon\">\r\n            <span data-dojo-attach-point=\"captionNode\"></span>\r\n          </div>\r\n        </div>\r\n\r\n\r\n      </div>\r\n      <div class=\"stiComboExpendableColumn stiComboExpendableColumnRight\">\r\n        <div class=\"stiComboExpendableColumnWrapper clearFix\">\r\n\r\n\r\n          <div class=\"stiComboExpendableHeadColumn\" style=\"float:left; width: 36%; overflow:hidden;\">\r\n\r\n            <div class=\"stiComboExpendableRight fieldsetChapterLabelCell\" data-dojo-attach-point=\"ratinglabel\">\r\n\r\n            </div>\r\n\r\n\r\n          </div>\r\n          <div class=\"stiComboExpendableHeadColumn\" style=\" overflow: hidden; zoom:1\">\r\n\r\n            <div class=\"stiComboExpendableRight\" data-dojo-attach-point=\"na\">\r\n              <img src=\"${_blankGif}\" class=\"stiComboExpendableCommentIcon\" alt=\"\" data-dojo-attach-point=\"commentIcon\">\r\n            </div>\r\n            <div class=\"stiComboExpendableLeft\">\r\n              <div class=\"stiComboExpendableRating\" data-dojo-attach-point=\"rating\"></div>\r\n            </div>\r\n\r\n          </div>\r\n\r\n\r\n        </div>\r\n      </div>\r\n    </div>\r\n    <div class=\"stiComboExpendableBody\" data-dojo-attach-point=\"comboBody\">\r\n      <div class=\"stiComboExpendableBodyWrap\">\r\n        <div class=\"stiComboExpendableColumn stiComboExpendableColumnLeft\" data-dojo-attach-point=\"leftColumn\">\r\n\r\n          <div class=\"stiComboExpendableInfo\" data-dojo-attach-point=\"description\"></div>\r\n\r\n\r\n        </div>\r\n        <div class=\"stiComboExpendableColumn stiComboExpendableColumnRight\" data-dojo-attach-point=\"rightColumn\">\r\n          <div class=\"stiComboExpendableColumnWrapper\">\r\n\r\n            <div class=\"stiComboExpendableBodyWrapper\">\r\n\r\n              <div class=\"stiComboExpendableBodyColumn\">\r\n                <div class=\"stiComboExpendableBodyColumnWrapper\">\r\n                  <div class=\"stiComboExpendableBodyColumnInner\" data-dojo-attach-point=\"columnInner\">\r\n                    <div class=\"stiComboExpendableCommentField\" data-dojo-attach-point=\"comment\"></div>\r\n                  </div>\r\n                </div>\r\n              </div>\r\n            </div>\r\n\r\n          </div>\r\n        </div>\r\n      </div>\r\n\r\n    </div>\r\n  </div>\r\n  <div class=\"stiComboCorner stiComboCornerLT\"></div>\r\n  <div class=\"stiComboCorner stiComboCornerRT\"></div>\r\n  <div class=\"stiComboCorner stiComboCornerLB\"></div>\r\n  <div class=\"stiComboCorner stiComboCornerRB\"></div>\r\n</div>",
'url:xcitec/grid/templates/HGridFC.html':"<div class=\"dataTable\">\r\n  <div class=\"dataTableHeadBg\">\r\n    <div class=\"dataTableHeadBgLeft\"></div>\r\n    <div class=\"dataTableHeadBgRight\"></div>\r\n  </div>\r\n  <div class=\"colWrapper\">\r\n    <!--<div class=\"colFrozen\"></div>-->\r\n    <div class=\"colFrozen\" data-dojo-attach-point=\"nodeFrozenPartContainer\">\r\n      <div class=\"dataTableBody\" data-dojo-attach-point=\"frozenDataTableBody\">\r\n        <table width=\"100%\" class=\"dataTableMinWidth\">\r\n          <tr>\r\n            <td>\r\n              <table width=\"100%\" class=\"body_content\" data-dojo-attach-point=\"frozenBody\">\r\n              </table>\r\n              <img src=\"${_blankGif}\" class=\"lineMinWidth\" height=\"1\" alt=\"\" data-dojo-attach-point=\"frozenBodyMinWidth\">\r\n            </td>\r\n          </tr>\r\n        </table>\r\n      </div>\r\n    </div>\r\n\r\n    <div class=\"colDinamic\" data-dojo-attach-point=\"nodeTableContainer\">\r\n      <div class=\"dataTableBody\" data-dojo-attach-point=\"dataTableBody\">\r\n        <table width=\"100%\" class=\"dataTableMinWidth\">\r\n          <tr>\r\n            <td>\r\n              <table width=\"100%\" class=\"body_content\" data-dojo-attach-point=\"body\">\r\n\r\n              </table>\r\n              <img src=\"${_blankGif}\" class=\"lineMinWidth\" height=\"1\" alt=\"\" data-dojo-attach-point=\"bodyMinWidth\">\r\n            </td>\r\n          </tr>\r\n        </table>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <div class=\"dataTableFooter\" data-dojo-attach-point=\"footer\">\r\n    <div class=\"dataTableFooterLeft\"></div>\r\n    <div class=\"dataTableFooterRight\"></div>\r\n  </div>\r\n</div>",
'url:dijit/templates/TitlePane.html':"<div>\n\t<div data-dojo-attach-event=\"ondijitclick:_onTitleClick, onkeydown:_onTitleKey\"\n\t\t\tclass=\"dijitTitlePaneTitle\" data-dojo-attach-point=\"titleBarNode\" id=\"${id}_titleBarNode\">\n\t\t<div class=\"dijitTitlePaneTitleFocus\" data-dojo-attach-point=\"focusNode\">\n\t\t\t<span data-dojo-attach-point=\"arrowNode\" class=\"dijitInline dijitArrowNode\" role=\"presentation\"></span\n\t\t\t><span data-dojo-attach-point=\"arrowNodeInner\" class=\"dijitArrowNodeInner\"></span\n\t\t\t><span data-dojo-attach-point=\"titleNode\" class=\"dijitTitlePaneTextNode\"></span>\n\t\t</div>\n\t</div>\n\t<div class=\"dijitTitlePaneContentOuter\" data-dojo-attach-point=\"hideNode\" role=\"presentation\">\n\t\t<div class=\"dijitReset\" data-dojo-attach-point=\"wipeNode\" role=\"presentation\">\n\t\t\t<div class=\"dijitTitlePaneContentInner\" data-dojo-attach-point=\"containerNode\" role=\"region\" id=\"${id}_pane\" aria-labelledby=\"${id}_titleBarNode\">\n\t\t\t\t<!-- nested divs because wipeIn()/wipeOut() doesn't work right on node w/padding etc.  Put padding on inner div. -->\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n",
'url:dijit/templates/Dialog.html':"<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n\t<div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t\t<span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"\n\t\t\t\trole=\"heading\" level=\"1\"></span>\n\t\t<span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${buttonCancel}\" role=\"button\" tabindex=\"0\">\n\t\t\t<span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t\t</span>\n\t</div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n",
'url:slmui/layout/templates/ChapterAdvanced.html':"<table class=\"dijitReset fieldsetChapterTable\" cellSpacing=\"0\" cellPadding=\"0\">\r\n\t<thead class=\"dijitReset\">\r\n\t<tr class=\"dijitReset\" style=\"height:0px;\">\r\n\t\t<td class=\"dijitReset fieldsetChapter2CLabelColumn\" data-dojo-attach-point=\"columnLabel\"></td>\r\n\t\t<td class=\"dijitReset fieldsetChapter2CControlColumn\" data-dojo-attach-point=\"columnWidget\"></td>\r\n\t\t<td class=\"dijitReset fieldsetChapter2CDelimColumn\"></td>\r\n\t\t<td class=\"dijitReset fieldsetChapter2CLabelColumn\"></td>\r\n\t\t<td class=\"dijitReset fieldsetChapter2CControlColumn\"></td>\r\n\t</tr>\r\n\t<tr class=\"dijitReset\">\r\n\t\t<td class=\"dijitReset fieldsetChapterCaption\" colspan=\"5\"\r\n\t\t\tdata-dojo-attach-point=\"captionContainerNode\">\r\n\t\t\t<label class=\"dijitReset fieldsetChapterCaption\" data-dojo-attach-point=\"captionNode\"></label>\r\n\t\t</td>\r\n\t</tr>\r\n\t</thead>\r\n\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\r\n</table>",
'url:xcitec/formx/templates/EVACriterionSingleColumn.html':"<div class=\"evacriterion clearFix\" data-dojo-attach-point=\"containerNode\">\r\n  <!-- evacriterion level -->\r\n  <div class=\"evacriterionLevel\"> </div>\r\n  <!-- evacriterion wrapper -->\r\n  <div class=\"evacriterionInner\">\r\n    <!-- header -->\r\n    <div class=\"clearFix evacriterionHeader\" data-dojo-attach-point=\"stateNode,comboHeader\">\r\n      <!-- header right col -->\r\n      <div class=\"evacriterionHeaderRightcolWrapper\" data-dojo-attach-point=\"headerRightcolWrapperNode\">\r\n        <div class=\"evacriterionHeaderRightcol clearFix\" data-dojo-attach-point=\"headerRightcolNode\">\r\n          <div class=\"evacriterionHeaderRightcolRightitem clearFix\">\r\n            <!-- Rightcol_Rightitem -->\r\n            <div class=\"evacriterionAttachmentIcon\" data-dojo-attach-point=\"attachmentIcon\"></div>\r\n            <div class=\"evacriterionCommentIcon\" data-dojo-attach-point=\"commentIcon\"></div>\r\n            <div class=\"evacriterionRating evacriterionLine\" data-dojo-attach-point=\"rating\"></div>\r\n          </div>\r\n          <div class=\"evacriterionHeaderRightcolLeftitem\">\r\n            <!-- Rightcol_Lefttitem -->\r\n            <div class=\"evacriterionRatingLabel\" data-dojo-attach-point=\"ratinglabel\"></div>\r\n          </div>\r\n          <div class=\"evacriterionHeaderRightcolCenteritem evacriterionLine\" data-dojo-attach-point=\"lineNode2\">\r\n            <!-- Rightcol_Centeritem -->\r\n          </div>\r\n        </div>\r\n      </div>\r\n      <!-- header left col -->\r\n      <div class=\"evacriterionHeaderLeftcolWrapper\">\r\n        <div class=\"evacriterionHeaderLeftcol clearFix\">\r\n          <div class=\"evacriterionHeaderLeftcolRightitem\">\r\n            <!--Leftcol_Rightitem -->\r\n            <div data-dojo-attach-point=\"weight\"></div>\r\n          </div>\r\n          <div class=\"evacriterionHeaderLeftcolLeftCenterWrap clearFix\">\r\n            <div class=\"evacriterionHeaderLeftcolLeftitem\">\r\n              <!-- Leftcol_Leftitem -->\r\n              <img src=\"${_blankGif}\" class=\"evacriterionTitleIcon\" alt=\"\" data-dojo-attach-point=\"icon\" data-dojo-attach-event=\"onclick:_onClick\">\r\n              <img src=\"${_blankGif}\" class=\"${iconClass}\" alt=\"\" data-dojo-attach-point=\"statusIcon\">\r\n              <span class=\"evacriterionCaption\" data-dojo-attach-point=\"captionNode\"></span>\r\n            </div>\r\n            <div class=\"evacriterionHeaderLeftcolCenteritemWrap\">\r\n              <!--Leftcol_Centeritem -->\r\n              <div class=\"evacriterionHeaderLeftcolCenteritem evacriterionLine\" data-dojo-attach-point=\"lineNode1\">\r\n                <div class=\"evacriterionEvaluator\" data-dojo-attach-point=\"evaluation\"></div>\r\n              </div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n      </div>\r\n    </div>\r\n    <!-- body -->\r\n    <div class=\"evacriterionBodyWrapper\" data-dojo-attach-point=\"comboBody\">\r\n      <div class=\"evacriterionBody\">\r\n        <div class=\"clearFix\" data-dojo-attach-point=\"bodyNode\">\r\n          <!-- body right col -->\r\n          <div class=\"evacriterionBodyRightcolWrapper\" data-dojo-attach-point=\"bodyRightcolWrapperNode\">\r\n            <div class=\"evacriterionBodyRightcol\" data-dojo-attach-point=\"bodyRightcolNode\">\r\n              <!-- Rightcol -->\r\n              <div data-dojo-attach-point=\"description\"></div>\r\n              <div data-dojo-attach-point=\"combofieldNode\"></div>\r\n            </div>\r\n          </div>\r\n          <!-- body left col -->\r\n          <div class=\"evacriterionBodyLeftcolWrapper\">\r\n            <div class=\"evacriterionBodyLeftcol\">\r\n              <!-- Leftcol -->\r\n              <div data-dojo-attach-point=\"comment\"></div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n        <!-- evacriterion -->\r\n        <div data-dojo-attach-point=\"evacriterion\"></div>\r\n      </div>\r\n    </div>\r\n    <div class=\"evacriterionCorner evacriterionCornerLT\"></div>\r\n    <div class=\"evacriterionCorner evacriterionCornerRT\"></div>\r\n    <div class=\"evacriterionCorner evacriterionCornerLB\"></div>\r\n    <div class=\"evacriterionCorner evacriterionCornerRB\"></div>\r\n  </div>\r\n</div>",
'url:dijit/form/templates/CheckBox.html':"<div class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><input\n\t \t${!nameAttrSetting} type=\"${type}\" role=\"${type}\" aria-checked=\"false\" ${checkedAttrSetting}\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\n\t\tdata-dojo-attach-point=\"focusNode\"\n\t \tdata-dojo-attach-event=\"ondijitclick:_onClick\"\n/></div>\n",
'url:xcitec/templates/MultiSelectDialog.html':"<table class=\"dijitDialog\" tabindex=\"-1\" wairole=\"dialog\" waistate=\"labelledby-${id}_title\">\r\n  <tr>\r\n    <td class=\"cell titleBarWrap\">\r\n      <div class=\"dialogBg titleBarLeft\"></div>\r\n      <div class=\"dialogBg titleBarRight\"></div>\r\n      <div class=\"dialogBg dijitDialogTitleBar\" data-dojo-attach-point=\"titleBar\">\r\n        <span class=\"dijitDialogTitle\" id=\"${id}_title\" data-dojo-attach-point=\"titleNode\"></span>\r\n        <span class=\"dijitDialogCloseIcon\" title=\"${buttonCancel}\" data-dojo-attach-point=\"closeButtonNode\" data-dojo-attach-event=\"onclick: onCancel\">\r\n          <span class=\"closeText\" title=\"${buttonCancel}\" data-dojo-attach-point=\"closeText\"></span>\r\n        </span>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"cell\">\r\n      <div class=\"dijitDialogPaneContent\" data-dojo-attach-point=\"containerNode\">\r\n        <div class=\"minWidthRestriction\"></div>\r\n        <div data-dojo-attach-point=\"ms_description\"></div>\r\n        <br><br>\r\n        <div data-dojo-attach-point=\"ms_filterbox\"></div>\r\n        <table class=\"dijitReset\" cellspacing=\"0\" cellpadding=\"0\" style=\"width: 100%; table-layout:fixed; min-width:450px\">\r\n          <thead class=\"dijitReset\">\r\n          <tr class=\"dijitReset\" style=\"height:0px;\">\r\n            <td class=\"dijitReset\"></td>\r\n            <td class=\"dijitReset\" style=\"width: 2px;\"></td>\r\n            <td class=\"dijitReset\" style=\"width: 40px;\"></td>\r\n            <td class=\"dijitReset\" style=\"width: 8px;\"></td>\r\n            <td class=\"dijitReset\"></td>\r\n          </tr>\r\n          </thead>\r\n          <tbody class=\"dijitReset\">\r\n          <tr class=\"dijitReset\">\r\n            <td class=\"dijitReset\">\r\n              <div data-dojo-attach-point=\"ms_searchbox\"></div>\r\n            </td>\r\n            <td class=\"dijitReset\"></td>\r\n            <td class=\"dijitReset\" align=\"center\">\r\n              <div data-dojo-attach-point=\"select_all\"></div>\r\n              <br><br>\r\n              <div data-dojo-attach-point=\"select_with_children\"></div>\r\n              <br><br>\r\n              <div data-dojo-attach-point=\"select\"></div>\r\n              <br><br><br><br>\r\n              <div data-dojo-attach-point=\"deselect\"></div>\r\n              <br><br>\r\n              <div data-dojo-attach-point=\"deselect_all\"></div>\r\n            </td>\r\n            <td class=\"dijitReset\"></td>\r\n            <td class=\"dijitReset\">\r\n              <div data-dojo-attach-point=\"ms_selectbox\"></div>\r\n            </td>\r\n          </tr>\r\n          <tr class=\"dijitReset\">\r\n            <td class=\"dijitReset\"></td>\r\n            <td class=\"dijitReset\"></td>\r\n            <td class=\"dijitReset\"></td>\r\n          </tr>\r\n          </tbody>\r\n        </table>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"cell\">\r\n      <div class=\"dialogBg buttonBarLeft\"></div>\r\n      <div class=\"dialogBg buttonBarRight\"></div>\r\n      <div class=\"dialogBg xcitecDailogButtonsBar\" data-dojo-attach-point=\"buttonsBarNode\">\r\n        <div class=\"xcitecDailogButtonsBarLine\"></div>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n</table>",
'url:xcitec/templates/DialogPane.html':"<div class=\"dijitReset xcitecDailogPane\">\r\n  <div class=\"xcitecDailogPaneBar xcitecDailogPaneTitleBar\" data-dojo-attach-point=\"titleBarNode\">\r\n    <span class=\"xcitecDailogPaneTitle\" data-dojo-attach-point=\"captionNode\"></span>\r\n  </div>\r\n  <div class=\"xcitecDailogPaneContent\" data-dojo-attach-point=\"containerNode\"></div>\r\n  <div class=\"xcitecDailogPaneBar xcitecDailogPaneButtonsBar\" data-dojo-attach-point=\"buttonsBarNode\"></div>\r\n</div>",
'url:xcitec/grid/templates/HGrid.html':"<div class=\"dataTable\" data-dojo-attach-point=\"dataTable\">\r\n  <div class=\"dataTableHeadBg\">\r\n    <div class=\"dataTableHeadBgLeft\"></div>\r\n    <div class=\"dataTableHeadBgRight\"></div>\r\n  </div>\r\n\r\n  <div class=\"colDinamic\" data-dojo-attach-point=\"nodeTableContainer\">\r\n    <div class=\"dataTableBody\" data-dojo-attach-point=\"dataTableBody\">\r\n      <table width=\"100%\" class=\"dataTableMinWidth\">\r\n        <tr>\r\n          <td>\r\n            <table width=\"100%\" class=\"body_content\" data-dojo-attach-point=\"body\">\r\n\r\n            </table>\r\n            <img src=\"${_blankGif}\" class=\"lineMinWidth\" height=\"1\" alt=\"\" data-dojo-attach-point=\"bodyMinWidth\">\r\n          </td>\r\n        </tr>\r\n      </table>\r\n    </div>\r\n  </div>\r\n  <div class=\"dataTableFooter\" data-dojo-attach-point=\"footer\">\r\n    <div class=\"dataTableFooterLeft\"></div>\r\n    <div class=\"dataTableFooterRight\"></div>\r\n  </div>\r\n</div>",
'url:xcitec/layout/templates/DoubleField.html':"<table class=\"dijitReset fieldsetDoubleField\" cellpadding=\"0\" cellspacing=\"0\">\r\n    <thead class=\"dijitReset\">\r\n    <tr class=\"dijitReset fieldsetDoubleFieldColumns\" style=\"height: 0px;\" data-dojo-attach-point=\"columnsDefinition\">\r\n        <td class=\"dijitReset fieldsetDoubleFieldColumnLeft\" data-dojo-attach-point=\"columnLeft\"></td>\r\n        <td class=\"dijitReset\" style=\"width: 2px;\"></td>\r\n        <td class=\"dijitReset fieldsetDoubleFieldColumnMiddle\" data-dojo-attach-point=\"columnMiddle\"></td>\r\n        <td class=\"dijitReset\" style=\"width: 2px;\"></td>\r\n        <td class=\"dijitReset fieldsetDoubleFieldColumnRight\" data-dojo-attach-point=\"columnRight\"></td>\r\n    </tr>\r\n    </thead>\r\n    <tbody class=\"dijitReset\">\r\n    <tr class=\"dijitReset\">\r\n        <td class=\"dijitReset\" data-dojo-attach-point=\"leftFieldNode\"></td>\r\n        <td></td>\r\n        <td valign=\"top\" align=\"center\" data-dojo-attach-point=\"middleFieldNode\"><label\r\n                data-dojo-attach-point=\"middleLableWidget\" data-dojo-type=\"xcitec/DividingLabel\"\r\n                data-dojo-props='\"labelId\":\"${dividingTextId}\"'></label></td>\r\n        <td></td>\r\n        <td class=\"dijitReset\" data-dojo-attach-point=\"rightFieldNode\"></td>\r\n    </tr>\r\n    </tbody>\r\n</table>\r\n",
'url:slmui/form/templates/RadioButton.html':"<div class=\"dijitReset\" wairole=\"presentation\" data-dojo-attach-event=\"onclick:_onClick\" style=\"display: inline-block;\">\r\n    <div class=\"dijitXcitecRadioWrap\">\r\n        <div data-dojo-attach-point=\"borderNode\" class=\"radioButtonContainer\" title=\"${title}\">\r\n            <div class=\"dijitReset dijitXcitecRadioIcon centeredRadioButtonIcon\">\r\n                <input type=\"${type}\" autocomplete=\"off\" class=\"dijitReset dijitCheckBoxInput\" data-dojo-attach-point=\"focusNode\" ${!nameAttrSetting} ${checkedAttrSetting}>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>\r\n",
'url:xcitec/form/templates/Button.html':"<div class=\"dijit dijitReset dijitInline\" data-dojo-attach-point=\"titleNode,focusNode\" data-dojo-attach-event=\"ondijitclick:_onClick\">\r\n  <div class=\"dijitBorder\" style=\"overflow: hidden;\">\r\n    <div class=\"dijitLeft\">\r\n\r\n    </div>\r\n    <div class=\"dijitReset dijitStretch\">\r\n      <span class=\"dijitReset dijitInline\" data-dojo-attach-point=\"iconNode\"></span>\r\n      <span xclass=\"dijitReset dijitInline dijitButtonText\" data-dojo-attach-point=\"containerNode\"></span>\r\n    </div>\r\n    <div class=\"dijitRight\">\r\n\r\n    </div>\r\n  </div>\r\n</div>\r\n",
'url:xcitec/formx/templates/EVACriterion.html':"<div class=\"clearFix\" data-dojo-attach-point=\"containerNode\">\r\n  <!-- evacriterion level -->\r\n  <div class=\"evacriterionLevel\"> </div>\r\n  <!-- evacriterion wrapper -->\r\n  <div class=\"evacriterionInner\">\r\n    <!-- header -->\r\n    <div class=\"clearFix evacriterionHeader\" data-dojo-attach-point=\"stateNode,comboHeader\">\r\n      <!-- header right col -->\r\n      <div class=\"evacriterionHeaderRightcolWrapper\" data-dojo-attach-point=\"headerRightcolWrapperNode\">\r\n        <div class=\"evacriterionHeaderRightcol clearFix\" data-dojo-attach-point=\"headerRightcolNode\">\r\n          <div class=\"evacriterionHeaderRightcolRightitem clearFix\">\r\n            <!-- Rightcol_Rightitem -->\r\n            <div class=\"evacriterionAttachmentIcon\" data-dojo-attach-point=\"attachmentIcon\"></div>\r\n            <div class=\"evacriterionCommentIcon\" data-dojo-attach-point=\"commentIcon\"></div>\r\n            <div class=\"evacriterionRating evacriterionLine\" data-dojo-attach-point=\"rating\"></div>\r\n          </div>\r\n          <div class=\"clearFix evacriterionHeaderRightcolLeftCenterWrap\">\r\n            <div class=\"evacriterionHeaderRightcolLeftitem\">\r\n              <!-- Rightcol_Lefttitem -->\r\n              <div class=\"evacriterionRatingLabel\" data-dojo-attach-point=\"ratinglabel\"></div>\r\n            </div>\r\n            <div class=\"evacriterionHeaderRightcolCenteritem evacriterionLine\" data-dojo-attach-point=\"lineNode2\">\r\n              <!-- Rightcol_Centeritem -->\r\n            </div>\r\n          </div>\r\n        </div>\r\n      </div>\r\n      <!-- header left col -->\r\n      <div class=\"evacriterionHeaderLeftcolWrapper\">\r\n        <div class=\"evacriterionHeaderLeftcol clearFix\">\r\n          <div class=\"evacriterionHeaderLeftcolRightitem\">\r\n            <!--Leftcol_Rightitem -->\r\n            <div data-dojo-attach-point=\"weight\"></div>\r\n          </div>\r\n          <div class=\"evacriterionHeaderLeftcolLeftCenterWrap clearFix\">\r\n            <div class=\"evacriterionHeaderLeftcolLeftitem\">\r\n              <!-- Leftcol_Leftitem -->\r\n              <img src=\"${_blankGif}\" class=\"evacriterionTitleIcon\" alt=\"\" data-dojo-attach-point=\"icon\" data-dojo-attach-event=\"onclick:_onClick\">\r\n              <img src=\"${_blankGif}\" class=\"${iconClass}\" alt=\"\" data-dojo-attach-point=\"statusIcon\">\r\n              <span class=\"evacriterionCaption\" data-dojo-attach-point=\"captionNode\"></span>\r\n            </div>\r\n            <div class=\"evacriterionHeaderLeftcolCenteritemWrap\">\r\n              <!--Leftcol_Centeritem -->\r\n              <div class=\"evacriterionHeaderLeftcolCenteritem evacriterionLine\" data-dojo-attach-point=\"lineNode1\">\r\n                <div class=\"evacriterionEvaluator\" data-dojo-attach-point=\"evaluation\"></div>\r\n              </div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n      </div>\r\n    </div>\r\n    <!-- body -->\r\n    <div class=\"evacriterionBodyWrapper\" data-dojo-attach-point=\"comboBody\">\r\n      <div class=\"evacriterionBody\">\r\n        <div class=\"clearFix\" data-dojo-attach-point=\"bodyNode\">\r\n          <!-- body right col -->\r\n          <div class=\"evacriterionBodyRightcolWrapper\" data-dojo-attach-point=\"bodyRightcolWrapperNode\">\r\n            <div class=\"evacriterionBodyRightcol\" data-dojo-attach-point=\"bodyRightcolNode\">\r\n              <!-- Rightcol -->\r\n              <div data-dojo-attach-point=\"comment\"></div>\r\n            </div>\r\n          </div>\r\n          <!-- body left col -->\r\n          <div class=\"evacriterionBodyLeftcolWrapper\">\r\n            <div class=\"evacriterionBodyLeftcol\">\r\n              <!-- Leftcol -->\r\n              <div data-dojo-attach-point=\"description\"></div>\r\n              <div data-dojo-attach-point=\"combofieldNode\"></div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n        <!-- evacriterion -->\r\n        <div data-dojo-attach-point=\"evacriterion\"></div>\r\n      </div>\r\n    </div>\r\n    <div class=\"evacriterionCorner evacriterionCornerLT\"></div>\r\n    <div class=\"evacriterionCorner evacriterionCornerRT\"></div>\r\n    <div class=\"evacriterionCorner evacriterionCornerLB\"></div>\r\n    <div class=\"evacriterionCorner evacriterionCornerRB\"></div>\r\n  </div>\r\n</div>",
'url:dijit/templates/Calendar.html':"<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" aria-labelledby=\"${id}_mddb ${id}_year\" data-dojo-attach-point=\"gridNode\">\n\t<thead>\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\n\t\t\t<th class='dijitReset dijitCalendarArrow' data-dojo-attach-point=\"decrementMonth\" scope=\"col\">\n\t\t\t\t<span class=\"dijitInline dijitCalendarIncrementControl dijitCalendarDecrease\" role=\"presentation\"></span>\n\t\t\t\t<span data-dojo-attach-point=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\n\t\t\t</th>\n\t\t\t<th class='dijitReset' colspan=\"5\" scope=\"col\">\n\t\t\t\t<div data-dojo-attach-point=\"monthNode\">\n\t\t\t\t</div>\n\t\t\t</th>\n\t\t\t<th class='dijitReset dijitCalendarArrow' scope=\"col\" data-dojo-attach-point=\"incrementMonth\">\n\t\t\t\t<span class=\"dijitInline dijitCalendarIncrementControl dijitCalendarIncrease\" role=\"presentation\"></span>\n\t\t\t\t<span data-dojo-attach-point=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\n\t\t\t</th>\n\t\t</tr>\n\t\t<tr role=\"row\">\n\t\t\t${!dayCellsHtml}\n\t\t</tr>\n\t</thead>\n\t<tbody data-dojo-attach-point=\"dateRowsNode\" data-dojo-attach-event=\"ondijitclick: _onDayClick\" class=\"dijitReset dijitCalendarBodyContainer\">\n\t\t\t${!dateRowsHtml}\n\t</tbody>\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\n\t\t<tr>\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\" role=\"presentation\">\n\t\t\t\t<div class=\"dijitCalendarYearLabel\">\n\t\t\t\t\t<span data-dojo-attach-point=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\" role=\"button\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\" role=\"button\" id=\"${id}_year\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\" role=\"button\"></span>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t</tr>\n\t</tfoot>\n</table>\n",
'url:xcitec/layout/templates/FacetFilter.html':"<div dojoAttachPoint=\"containerNode\">\r\n\t<div dojoAttachPoint=\"buttonsContainer\" class=\"buttonsContainer\"></div>\r\n</div>",
'url:xcitec/form/templates/TaskButton.html':"<div class=\"dijit dijitReset dijitInline taskButtonWrap\" data-dojo-attach-point=\"titleNode,focusNode\" data-dojo-attach-event=\"ondijitclick:_onClick\">\r\n  <div style=\"overflow: hidden; zoom:1;\">\r\n    <div class=\"dijitLeft\"></div>\r\n    <div class=\"dijitReset dijitStretch\">\r\n      <span class=\"dijitReset dijitInline\" data-dojo-attach-point=\"iconNode\"></span>\r\n      <span class=\"dijitButtonText\">\r\n        <span class=\"dijitButtonTextInner\" data-dojo-attach-point=\"containerNode\"></span>\r\n      </span>\r\n    </div>\r\n    <div class=\"dijitRight\"></div>\r\n  </div>\r\n</div>\r\n",
'url:xcitec/widgets/templates/SyncSupplierWidget.html':"<iframe class=\"boxExternalSynchronizedSupplier\" frameborder=\"no\" allowtransparency=\"true\" title=\"\" data-dojo-attach-point=\"SynchronizedSupplierFrame\"></iframe>",
'url:idx/form/templates/HorizontalSlider.html':"<div id=\"widget_${id}\" class=\"dijitReset dijitLeft idxComposite dijitInline\"\r\n\t><div class=\"idxLabel dijitInline dijitHidden\" dojoAttachPoint=\"labelWrap\"><label id=\"${id}_label\" dojoAttachPoint=\"compLabelNode\"></label></div\r\n\t><div class=\"dijitInline\"\r\n\t><table class=\"dijit dijitReset dijitSlider dijitSliderH\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" dojoAttachPoint='stateNode,oneuiBaseNode' dojoAttachEvent=\"onkeydown:_onKeyDown,onkeyup:_onKeyUp\"\r\n\trole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\r\n\t\t><td dojoAttachPoint=\"topDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationT dijitSliderDecorationH\"></td\r\n\t\t><td class=\"dijitReset\" colspan=\"3\"></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\r\n\t\t\t><div class=\"dijitSliderDecrementIconH\" style=\"display:none\" dojoAttachPoint=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper\" dojoAttachEvent=\"onmousedown:_onClkDecBumper\"></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><input dojoAttachPoint=\"valueNode\" type=\"hidden\" ${!nameAttrSetting}\r\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" role=\"presentation\" dojoAttachPoint=\"sliderBarContainer\"\r\n\t\t\t\t><div role=\"presentation\" dojoAttachPoint=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" dojoAttachEvent=\"onmousedown:_onBarClick\"\r\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\"\r\n\t\t\t\t\t\t><div dojoAttachPoint=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" dojoAttachEvent=\"onmousedown:_onHandleClick\" role=\"slider\" valuemin=\"${minimum}\" valuemax=\"${maximum}\"></div\r\n\t\t\t\t\t></div\r\n\t\t\t\t></div\r\n\t\t\t\t><div role=\"presentation\" dojoAttachPoint=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" dojoAttachEvent=\"onmousedown:_onBarClick\"></div\r\n\t\t\t></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper\" dojoAttachEvent=\"onmousedown:_onClkIncBumper\"></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\r\n\t\t\t><div class=\"dijitSliderIncrementIconH\" style=\"display:none\" dojoAttachPoint=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\r\n\t\t></td\r\n\t><td class='dijitReset'>\r\n\t\t<div class='dijitValidationContainer' dojoAttachPoint=\"iconNode\"\r\n\t\t\t><div class=\"dijitValidationIcon\"><input class=\"dijitReset dijitInputField dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\r\n\t\t/></div></div\r\n\t></td>\r\n\t</tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\r\n\t\t><td dojoAttachPoint=\"containerNode,bottomDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationB dijitSliderDecorationH\"></td\r\n\t\t><td class=\"dijitReset\" colspan=\"3\"></td\r\n\t></tr></table\r\n\t></div\r\n></div>",
'url:xcitec/form/templates/DropDownSelect.html':"<div class=\"dijit dijitReset dijitInlineTable dijitXcitec\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow:hidden;\">\r\n    <div class=\"dijitReset dijitXcitecRight\">\r\n      <div data-dojo-attach-point=\"button\" data-dojo-attach-event=\"onDoAction:_onClick\" data-dojo-type=\"xcitec/form/_XButtonRight\"\r\n           data-dojo-props='\"iconType\":\"DropDown\"'></div>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecInputField\">\r\n      <div class=\"dijitReset dijitXcitecInputFieldContent\" data-dojo-attach-point=\"textbox,focusNode\"\r\n           data-dojo-attach-event=\"onkeypress:_onKeyPress, onclick:_onClick\">\r\n      </div>\r\n      <input type=\"text\" style=\"display:none;\" autocomplete=\"off\" data-dojo-attach-point=\"valueNode\" ${!nameAttrSetting}></div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>",
'url:dijit/templates/MenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<span role=\"presentation\" class=\"dijitInline dijitIcon dijitMenuItemIcon\" data-dojo-attach-point=\"iconNode\"></span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode,textDirNode\"\n\t\trole=\"presentation\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">\n\t\t<span data-dojo-attach-point=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<span class=\"dijitInline dijitIcon dijitMenuExpand\"></span>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</span>\n\t</td>\n</tr>\n",
'url:idx/widget/templates/HoverHelpTooltip.html':"<div class=\"idxOneuiHoverHelpTooltip idxOneuiHoverHelpTooltipLeft\" role=\"alert\"\r\n\t><div\r\n\t\t><span data-dojo-attach-point=\"closeButtonNode\" class=\"idxOneuiHoverHelpTooltipCloseIcon\" \r\n\t\t\tdata-dojo-attach-event=\"ondijitclick: hideOnClickClose\" role=\"button\" aria-label=\"${buttonClose}\"\r\n\t\t\t><span class=\"idxHoverHelpCloseText\">x</span\r\n\t\t></span\r\n\t></div\r\n\t><div data-dojo-attach-point=\"outerContainerNode\" class=\"idxOneuiHoverHelpTooltipContainer idxOneuiHoverHelpTooltipContents\"\r\n\t\t><div data-dojo-attach-point=\"containerNode\" role=\"presentation\"></div\r\n\t\t><a target=\"_blank\" href=\"#updateme\" role=\"link\" class=\"idxOneuiHoverHelpTooltipLearnLink\" data-dojo-attach-point=\"learnMoreNode\"><span>${learnMoreLabel}</span></a\r\n\t></div\r\n\t><div class=\"idxOneuiHoverHelpTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\r\n></div>",
'url:dijit/templates/Tooltip.html':"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\n\t><div class=\"dijitTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" data-dojo-attach-point=\"containerNode\" role='alert'></div\n></div>\n",
'url:xcitec/page/templates/MainFrameV2.html':"<div>\r\n  <!-- LANDMARK banner -->\r\n  <div role=\"banner\">\r\n    <!-- suitenav -->\r\n    <div data-dojo-attach-point=\"suitenav\"></div>\r\n    <!-- [Header Area] -->\r\n    <div class=\"header\">\r\n      <div class=\"header_l\">\r\n        <div class=\"header_r\">\r\n          <!-- userinfobox -->\r\n          <div data-dojo-attach-point=\"userinfobox\"></div>\r\n          <!-- tabbar -->\r\n          <div class=\"xcitec_dynTabs\">\r\n            <div class=\"xcitec_dynTabs_wrap\">\r\n              <div data-dojo-attach-point=\"tabbar\"></div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <!-- [Content Area] -->\r\n  <div class=\"content\">\r\n    <!-- LANDMARK complementary -->\r\n    <!-- menu -->\r\n    <div class=\"menu_wrap\" role=\"complementary\">\r\n      <div data-dojo-attach-point=\"menu\"></div>\r\n      <!-- sidebar -->\r\n      <div class=\"xcitec_login\">\r\n        <div data-dojo-attach-point=\"sidebar\"></div>\r\n      </div>\r\n    </div>\r\n    <!-- LANDMARK main -->\r\n    <!-- data -->\r\n    <div class=\"dataContent\" role=\"main\" data-dojo-attach-event=\"onkeypress:_onKeyPress\">\r\n      <div data-dojo-attach-point=\"content\"></div>\r\n    </div>\r\n  </div>\r\n  <div class=\"footer_empty\"> </div>\r\n  <div data-dojo-attach-point=\"modal_dialog\"></div>\r\n  <div data-dojo-attach-point=\"feedback_dialog\"></div>\r\n</div>",
'url:dijit/form/templates/HorizontalSlider.html':"<table class=\"dijit dijitReset dijitSlider dijitSliderH\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" data-dojo-attach-event=\"onkeydown:_onKeyDown, onkeyup:_onKeyUp\"\n\trole=\"presentation\"\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"topDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationT dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderDecrementIconH\" style=\"display:none\" data-dojo-attach-point=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper\" data-dojo-attach-event=\"press:_onClkDecBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><input data-dojo-attach-point=\"valueNode\" type=\"hidden\" ${!nameAttrSetting}\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" role=\"presentation\" data-dojo-attach-point=\"sliderBarContainer\"\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" data-dojo-attach-event=\"press:_onBarClick\"\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\"\n\t\t\t\t\t\t><div data-dojo-attach-point=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" data-dojo-attach-event=\"press:_onHandleClick\" role=\"slider\"></div\n\t\t\t\t\t></div\n\t\t\t\t></div\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" data-dojo-attach-event=\"press:_onBarClick\"></div\n\t\t\t></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper\" data-dojo-attach-event=\"press:_onClkIncBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderIncrementIconH\" style=\"display:none\" data-dojo-attach-point=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\n\t\t></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"containerNode,bottomDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationB dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n></table>\n",
'url:dijit/templates/TreeNode.html':"<div class=\"dijitTreeNode\" role=\"presentation\"\n\t><div data-dojo-attach-point=\"rowNode\" class=\"dijitTreeRow\" role=\"presentation\"\n\t\t><span data-dojo-attach-point=\"expandoNode\" class=\"dijitInline dijitTreeExpando\" role=\"presentation\"></span\n\t\t><span data-dojo-attach-point=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"></span\n\t\t><span data-dojo-attach-point=\"contentNode\"\n\t\t\tclass=\"dijitTreeContent\" role=\"presentation\">\n\t\t\t<span role=\"presentation\" class=\"dijitInline dijitIcon dijitTreeIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span data-dojo-attach-point=\"labelNode,focusNode\" class=\"dijitTreeLabel\" role=\"treeitem\"\n\t\t\t\t   tabindex=\"-1\" aria-selected=\"false\" id=\"${id}_label\"></span>\n\t\t</span\n\t></div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeNodeContainer\" role=\"presentation\"\n\t\t style=\"display: none;\" aria-labelledby=\"${id}_label\"></div>\n</div>\n",
'url:xcitec/templates/Box.html':"<div class=\"${baseClass}\">\r\n  <div class=\"box_header\" data-dojo-attach-point=\"header\">\r\n    <div class=\"right\">\r\n      <div class=\"btn\" data-dojo-attach-point=\"headerRight\">\r\n\r\n      </div>\r\n    </div>\r\n    <div class=\"left\" data-dojo-attach-point=\"headerLeft\">\r\n      <span class=\"wrap\">\r\n        <img src=\"${_blankGif}\" class=\"icon\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"icon\" data-dojo-attach-event=\"onclick:_onClick\">\r\n        <img src=\"${_blankGif}\" class=\"${iconClass}\" width=\"1\" height=\"1\" alt=\"\">\r\n        <span class=\"title\">\r\n          <h2 data-dojo-attach-point=\"captionNode\"></h2> <span data-dojo-attach-point=\"comment\"></span>\r\n        </span>\r\n      </span>\r\n      <span class=\"btn\" data-dojo-attach-point=\"editNode\"></span>\r\n    </div>\r\n    <img src=\"${_blankGif}\" class=\"border_left\" width=\"1\" height=\"1\" alt=\"\">\r\n  </div>\r\n  <div class=\"box_content\" style=\"display: none;\" data-dojo-attach-point=\"animationNode\">\r\n    <div class=\"box_content_wrap\">\r\n      <div class=\"box_content_shadow\" data-dojo-attach-point=\"containerNode\">\r\n        <div class=\"toolbar_wrap\" data-dojo-attach-point=\"toolbar\"></div>\r\n        <div class=\"bottomButtons\" data-dojo-attach-point=\"bottomButtons\"></div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>",
'url:xcitec/layout/templates/MultiSelectLayout.html':"<div class=\"multiSelect\">\r\n  <div data-dojo-attach-point=\"filtersNode\">\r\n  </div>\r\n  <table class=\"multiSelectField\">\r\n    <tbody>\r\n    <tr>\r\n      <td class=\"multiSelectSearch\" data-dojo-attach-point=\"searchResultNode\"></td>\r\n      <td class=\"multiSelectNavigation\" data-dojo-attach-point=\"navigationNode\"></td>\r\n      <td class=\"multiSelectSelection\" data-dojo-attach-point=\"selectionResultNode\"></td>\r\n    </tr>\r\n    </tbody>\r\n  </table>\r\n</div>",
'url:xcitec/form/templates/Slider.html':"<table class=\"dijit dijitReset dijitSlider\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" data-dojo-attach-event=\"onkeypress:_onKeyPress\">\r\n  <tr class=\"dijitReset\">\r\n    <td class=\"dijitReset\" colspan=\"2\"></td>\r\n    <td class=\"dijitReset\" style=\"text-align:center;width:100%;\" data-dojo-attach-point=\"containerNode,topDecoration\"></td>\r\n    <td class=\"dijitReset\" colspan=\"2\"></td>\r\n  </tr>\r\n  <tr class=\"dijitReset\">\r\n    <td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\">\r\n      <div class=\"dijitSliderDecrementIconH\" tabindex=\"-1\" style=\"display:none\" data-dojo-attach-point=\"decrementButton\"><span\r\n          class=\"dijitSliderButtonInner\">-</span></div>\r\n    </td>\r\n    <td class=\"dijitReset\">\r\n      <div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper dijitSliderLeftBumper\"\r\n           data-dojo-attach-event=\"onmousedown:_onClkDecBumper\"></div>\r\n    </td>\r\n    <td class=\"dijitReset\"><input type=\"hidden\" data-dojo-attach-point=\"valueNode\" ${!nameAttrSetting}>\r\n\r\n      <div class=\"dijitReset dijitSliderBarContainerH\" wairole=\"presentation\" data-dojo-attach-point=\"sliderBarContainer\">\r\n        <div wairole=\"presentation\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" data-dojo-attach-point=\"progressBar\"\r\n             data-dojo-attach-event=\"onmousedown:_onBarClick\">\r\n          <div class=\"dijitSliderMoveable dijitSliderMoveableH\">\r\n            <div class=\"dijitSliderImageHandle dijitSliderImageHandleH\" wairole=\"slider\" valuemin=\"${minimum}\" valuemax=\"${maximum}\"\r\n                 data-dojo-attach-point=\"sliderHandle,focusNode\" data-dojo-attach-event=\"onmousedown:_onHandleClick\"></div>\r\n          </div>\r\n        </div>\r\n        <div wairole=\"presentation\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\"\r\n             data-dojo-attach-point=\"remainingBar\" data-dojo-attach-event=\"onmousedown:_onBarClick\"></div>\r\n      </div>\r\n    </td>\r\n    <td class=\"dijitReset\">\r\n      <div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper dijitSliderRightBumper\"\r\n           data-dojo-attach-event=\"onmousedown:_onClkIncBumper\"></div>\r\n    </td>\r\n    <td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\" style=\"right:0px;\">\r\n      <div class=\"dijitSliderIncrementIconH\" tabindex=\"-1\" style=\"display:none\" data-dojo-attach-point=\"incrementButton\"><span\r\n          class=\"dijitSliderButtonInner\">+</span></div>\r\n    </td>\r\n  </tr>\r\n  <tr class=\"dijitReset\">\r\n    <td class=\"dijitReset\" colspan=\"2\"></td>\r\n    <td class=\"dijitReset\" style=\"text-align:center;\" data-dojo-attach-point=\"containerNode,bottomDecoration\"></td>\r\n    <td class=\"dijitReset\" colspan=\"2\"></td>\r\n  </tr>\r\n</table>",
'url:xcitec/widgets/templates/WidgetBox.html':"<div class=\"xwidget\">\r\n  <div class=\"xwidget_header\">\r\n    <div class=\"xwidget_header_l\">\r\n      <div class=\"xwidget_header_r\">\r\n        <div class=\"icon_settings\" data-dojo-attach-event=\"onclick:_onIconClick\"></div>\r\n        <img height=\"1\" width=\"1\" alt=\"\" src=\"${_blankGif}\" class=\"${iconClass}\">\r\n        <div class=\"title\" data-dojo-attach-point=\"text\"></div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <div class=\"xwidget_content\">\r\n    <div class=\"xwidget_content_inner\" data-dojo-attach-point=\"containerNode\">\r\n    </div>\r\n  </div>\r\n  <div class=\"xwidget_footer_corner xwidget_footer_l\"></div>\r\n  <div class=\"xwidget_footer_corner xwidget_footer_r\"></div>\r\n</div>",
'url:xcitec/form/templates/NumberTextBox.html':"<div class=\"dijit dijitReset dijitInlineTable dijitLeft\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow:hidden;\">\r\n    <div class=\"dijitReset dijitXcitecInputField\">\r\n      <input class=\"dijitReset\" type=\"${type}\" autocomplete=\"off\" data-dojo-attach-point=\"textbox,focusNode\" ${!nameAttrSetting}>\r\n    </div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>",
'url:xcitec/templates/xcitecDialog.html':"<table class=\"dijitDialog\" tabindex=\"0\" wairole=\"dialog\" waistate=\"labelledby-${id}_title\" data-dojo-attach-point=\"dialogNode\">\r\n  <tr>\r\n    <td class=\"cell titleBarWrap\">\r\n      <div class=\"dialogBg titleBarLeft\"></div>\r\n      <div class=\"dialogBg titleBarRight\"></div>\r\n      <div class=\"dialogBg dijitDialogTitleBar\" data-dojo-attach-point=\"titleBar\">\r\n        <span class=\"dijitDialogTitle\" id=\"${id}_title\" data-dojo-attach-point=\"titleNode\"></span>\r\n        <span class=\"dijitDialogCloseIcon\" title=\"${buttonCancel}\" data-dojo-attach-point=\"closeButtonNode\" data-dojo-attach-event=\"onclick: onCancel\">\r\n          <span class=\"closeText\" title=\"${buttonCancel}\" data-dojo-attach-point=\"closeText\"></span>\r\n        </span>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"cellMain\">\r\n      <img class=\"dialogIcon\" height=\"1\" width=\"1\" alt=\"\" src=\"${_blankGif}\">\r\n      <div class=\"cellContent\" data-dojo-attach-point=\"containerNode\">\r\n        <div class=\"minWidthRestriction\"></div>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"cellFooter\">\r\n      <div class=\"dialogBg buttonBarLeft\"></div>\r\n      <div class=\"dialogBg buttonBarRight\"></div>\r\n      <div class=\"dialogBg xcitecDailogButtonsBar\" data-dojo-attach-point=\"buttonsBarNode\">\r\n        <div class=\"xcitecDailogButtonsBarLine\"></div>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n</table>",
'url:dijit/form/templates/ValidationTextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'url:xcitec/templates/TabBar.html':"<div class=\"dijitReset TabBar\">\r\n  <!-- tabbar scroll left -->\r\n  <div class=\"TabBarButtonLeft\" data-dojo-attach-point=\"scrollArrowLeftNode\"\r\n       data-dojo-attach-event=\"onmouseenter:_onScrollArrowLeftHover,onmouseleave:_onScrollArrowLeftUnhover,ondijitclick:_onScrollArrowLeftClick\"></div>\r\n  <div class=\"TabBarRightWrap\" data-dojo-attach-point=\"rightWrapNode\">\r\n    <!-- tabbar scroll right -->\r\n    <div class=\"TabBarButtonRight\" data-dojo-attach-point=\"scrollArrowRight\"\r\n         data-dojo-attach-event=\"onmouseenter:_onScrollArrowRightHover,onmouseleave:_onScrollArrowRightUnhover,ondijitclick:_onScrollArrowRightClick\"></div>\r\n    <!-- tabbar list button -->\r\n    <div class=\"TabBarListButton\" data-dojo-attach-point=\"downArrowNode,popupStateNode\"\r\n         data-dojo-attach-event=\"onmouseenter:_onDownArrowHover,onmouseleave:_onDownArrowUnhover,ondijitclick:_onDownArrowClick\"></div>\r\n  </div>\r\n  <!-- tabs -->\r\n  <div class=\"TabBarItemScrollWrap\" role=\"tablist\">\r\n    <div class=\"dijitReset TabBarItemWrap\" data-dojo-attach-point=\"containerNode\"></div>\r\n  </div>\r\n</div>",
'url:xcitec/templates/TabBarItem.html':"<div class=\"dijitReset ${baseClass}\" role=\"tab\" tabindex=\"-1\" data-dojo-attach-point=\"stateNode\"\r\n     data-dojo-attach-event=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick, onkeypress:_onKeyPress\">\r\n  <div class=\"dijitReset TabBarItemLeft\">\r\n    <span class=\"pngFixIe6 ${iconClass}\" data-dojo-attach-point=\"iconNode\">\r\n    </span>\r\n  </div>\r\n  <div class=\"dijitReset TabBarItemRight\">\r\n    <img class=\"TabBarCloseButton\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"closeButton\"\r\n         data-dojo-attach-event=\"onmouseenter:_onCloseButtonHover,onmouseleave:_onCloseButtonUnhover,ondijitclick:_onCloseButtonClick\">\r\n  </div>\r\n  <div class=\"dijitReset TabBarItemTitle\" title=\"${label}\">\r\n    <div label=\"${label}\" title=\"${label}\" data-dojo-type=\"xcitec/Label\" data-dojo-props='\"labelId\":\"${labelId}\"'></div>\r\n  </div>\r\n</div>",
'url:xcitec/form/templates/CheckBox.html':"<div class=\"dijitReset dijitInline\" wairole=\"presentation\">\r\n  <div class=\"dijitXcitecCheckBoxWrap\">\r\n    <div class=\"dijitReset dijitXcitecCheckBoxIcon\">\r\n      <input type=\"${type}\" autocomplete=\"off\" class=\"dijitReset dijitCheckBoxInput\"\r\n             data-dojo-attach-point=\"focusNode\" data-dojo-attach-event=\"onclick:_onClick\" ${!nameAttrSetting} ${checkedAttrSetting}/>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecCheckBoxText\">\r\n      <label for=\"${id}\" data-dojo-attach-point=\"labelNode\" data-dojo-type=\"${labelTypeAttrSetting}\"\r\n             data-dojo-props='\"labelId\":\"${captionId}\"'></label>\r\n    </div>\r\n  </div>\r\n  <div class=\"required\"></div>\r\n</div>\r\n\r\n\r\n\r\n\r\n",
'url:xcitec/page/templates/SearchPanelV1.html':"<div>\r\n  <div class=\"xcitec_toolbar\">\r\n    <div class=\"xcitec_toolbar_right\">\r\n      <div data-dojo-attach-point=\"tablelayoutbutton\"></div>\r\n    </div>\r\n    <div class=\"xcitec_toolbar_wrap\">\r\n      <div data-dojo-attach-point=\"xcitectitle\"></div>\r\n      <div class=\"xcitec_actions\">\r\n        <div data-dojo-attach-point=\"toolbar\"></div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <div class=\"xcitec_dataContent\">\r\n    <div class=\"xcitec_left\">\r\n      <div class=\"xcitec_GridFilterArea\">\r\n        <div data-dojo-attach-point=\"filterlayoutbutton\"></div>\r\n        <div data-dojo-attach-point=\"filters\"></div>\r\n        <div data-dojo-attach-point=\"buttons\"></div>\r\n      </div>\r\n    </div>\r\n    <div class=\"xcitec_dataTable\">\r\n      <div data-dojo-attach-point=\"messagepane\"></div>\r\n      <div data-dojo-attach-point=\"grid\"></div>\r\n    </div>\r\n  </div>\r\n</div>",
'url:dijit/templates/MenuSeparator.html':"<tr class=\"dijitMenuSeparator\" role=\"separator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>\n",
'url:xcitec/form/templates/EntityDropDownSelect.html':"<div class=\"dijit dijitReset dijitInlineTable dijitLeft\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow:hidden;\">\r\n    <div class=\"dijitReset dijitXcitecClose\"></div>\r\n    <div class=\"dijitReset dijitXcitecRight\">\r\n      <div data-dojo-attach-point=\"infoButton\" data-dojo-attach-event=\"onDoAction:_onClickInfoButton\" data-dojo-type=\"${!xbtnInfoClass}\"\r\n           data-dojo-props='\"iconClass\":\"${infoIconClass}\"' ${disabledInfoAttrSetting} ${readOnlyInfoAttrSetting}></div><div data-dojo-attach-point=\"chooseButton\" data-dojo-attach-event=\"onDoAction:_onClickChooseButton\" data-dojo-type=\"xcitec/form/_XButtonRight\"\r\n           data-dojo-props='\"iconType\":\"DropDown\"' ${disabledChooseAttrSetting}></div>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecMiddle\">\r\n      <div class=\"dijitReset\" data-dojo-attach-point=\"stateNode\">\r\n        <div class=\"dijitReset dijitXcitecInputField\">\r\n          <div class=\"dijitReset dijitXcitecInputFieldContent\" data-dojo-attach-point=\"textbox,focusNode\"\r\n               data-dojo-attach-event=\"onkeypress:_onKeyPress, onclick:_onClick\">\r\n          </div>\r\n          <input type=\"text\" style=\"display:none;\" autocomplete=\"off\" data-dojo-attach-point=\"valueNode\" ${!nameAttrSetting}>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>\r\n",
'url:xcitec/formx/templates/RISIndicator.html':"<div class=\"stiComboExpendable risIndicatorBg\">\r\n  <div class=\"stiComboExpendableWrapper\">\r\n    <div class=\"${baseClass}\" data-dojo-attach-point=\"stateNode\">\r\n      <div class=\"stiComboExpendableColumn stiComboExpendableColumnLeft clearFix\">\r\n\r\n        <div class=\"stiComboExpendableRight\">\r\n          <div class=\"stiComboExpendablstilue\"></div>\r\n          <div class=\"stiComboWeight\" data-dojo-attach-point=\"weight\"></div>\r\n        </div>\r\n        <div class=\"stiComboExpendableLeft\" data-dojo-attach-point=\"hierarchyNode\">\r\n          <div class=\"stiComboExpendableTitle\">\r\n            <img src=\"${_blankGif}\" class=\"stiComboExpendableTitleIcon\" alt=\"\" data-dojo-attach-point=\"icon\" data-dojo-attach-event=\"onclick:_onClick\">\r\n            <img src=\"${_blankGif}\" class=\"${iconClass}\" alt=\"\" data-dojo-attach-point=\"statusIcon\">\r\n            <span data-dojo-attach-point=\"captionNode\"></span>\r\n          </div>\r\n        </div>\r\n\r\n\r\n      </div>\r\n      <div class=\"stiComboExpendableColumn stiComboExpendableColumnRight\">\r\n        <div class=\"stiComboExpendableColumnWrapper clearFix\">\r\n\r\n\r\n          <div class=\"stiComboExpendableHeadColumn\" style=\"float:left; width: 36%;\">\r\n\r\n            <div class=\"stiComboExpendableRight fieldsetChapterLabelCell\" data-dojo-attach-point=\"ratinglabel\">\r\n\r\n            </div>\r\n\r\n\r\n          </div>\r\n          <div class=\"stiComboExpendableHeadColumn\" style=\" overflow: hidden; zoom:1\">\r\n\r\n            <div class=\"stiComboExpendableRight\" data-dojo-attach-point=\"na\">\r\n              <img src=\"${_blankGif}\" class=\"stiComboExpendableCommentIcon\" alt=\"\" data-dojo-attach-point=\"commentIcon\">\r\n            </div>\r\n            <div class=\"stiComboExpendableLeft\">\r\n              <div class=\"stiComboExpendableRating\" data-dojo-attach-point=\"resultHolderNode\"></div>\r\n            </div>\r\n\r\n          </div>\r\n\r\n\r\n        </div>\r\n      </div>\r\n    </div>\r\n    <div class=\"stiComboExpendableBody\" data-dojo-attach-point=\"comboBody\">\r\n      <div class=\"stiComboExpendableBodyWrap\">\r\n        <div class=\"stiComboExpendableColumn stiComboExpendableColumnLeft\" data-dojo-attach-point=\"leftColumn\">\r\n\r\n          <div class=\"stiComboExpendableInfo\" data-dojo-attach-point=\"body1HolderNode\"></div>\r\n\r\n\r\n        </div>\r\n        <div class=\"stiComboExpendableColumn stiComboExpendableColumnRight\" data-dojo-attach-point=\"rightColumn\">\r\n          <div class=\"stiComboExpendableColumnWrapper\">\r\n\r\n            <div class=\"stiComboExpendableBodyWrapper\">\r\n\r\n              <div class=\"stiComboExpendableBodyColumn\">\r\n                <div class=\"stiComboExpendableBodyColumnWrapper\">\r\n                  <div class=\"stiComboExpendableBodyColumnInner\" data-dojo-attach-point=\"columnInner\">\r\n                    <div class=\"stiComboExpendableCommentField\" data-dojo-attach-point=\"body2HolderNode\"></div>\r\n                  </div>\r\n                </div>\r\n              </div>\r\n            </div>\r\n\r\n          </div>\r\n        </div>\r\n      </div>\r\n\r\n    </div>\r\n  </div>\r\n  <div class=\"stiComboCorner stiComboCornerLT\"></div>\r\n  <div class=\"stiComboCorner stiComboCornerRT\"></div>\r\n  <div class=\"stiComboCorner stiComboCornerLB\"></div>\r\n  <div class=\"stiComboCorner stiComboCornerRB\"></div>\r\n</div>",
'url:xcitec/form/templates/AutoCompleteEntitySelect.html':"<div class=\"dijit dijitReset dijitInlineTable dijitLeft\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow:hidden;\">\r\n    <div class=\"dijitReset dijitXcitecClose\" data-dojo-attach-event=\"onclick: onDelete\"></div>\r\n    <img class=\"${frontIconClass}\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"frontIconNode\">\r\n\r\n    <div class=\"dijitReset dijitXcitecRight\">\r\n      <div data-dojo-attach-point=\"infoButton\" data-dojo-attach-event=\"onDoAction:_onClickInfoButton\" data-dojo-type=\"${!xbtnInfoClass}\"\r\n           data-dojo-props='\"iconClass\":\"${infoIconClass}\"' ${disabledInfoAttrSetting} ${readOnlyInfoAttrSetting}></div><div data-dojo-attach-point=\"chooseButton\" data-dojo-attach-event=\"onDoAction:_onClickChooseButton\" data-dojo-type=\"xcitec/form/_XButtonRight\"\r\n           data-dojo-props='\"iconType\":\"Choose\"' ${disabledChooseAttrSetting}></div>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecMiddle\">\r\n      <div class=\"dijitReset\" data-dojo-attach-point=\"stateNode\">\r\n        <div class=\"dijitReset dijitXcitecInputField\">\r\n          <div class=\"dijitReset dijitXcitecInputFieldContent\" data-dojo-attach-point=\"textbox\" data-dojo-attach-event=\"onclick:_onClick\">\r\n            <img class=\"iconWarning\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\">\r\n            <img style=\"float: left; margin:0 5px 0 0; display: inline\" class=\"${iconClass}\" src=\"${_blankGif}\" width=\"1\" height=\"1\" alt=\"\"\r\n                 data-dojo-attach-point=\"imageNode\">\r\n\r\n            <div style=\"overflow: hidden; zoom:1\"><input type=\"text\" autocomplete=\"off\" data-dojo-attach-point=\"textNode,focusNode\"></div>\r\n          </div>\r\n          <input type=\"text\" style=\"display:none;\" autocomplete=\"off\" data-dojo-attach-point=\"valueNode\" ${!nameAttrSetting}>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>\r\n",
'url:dijit/templates/Menu.html':"<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\"\n\t   cellspacing=\"0\">\n\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\n</table>\n",
'url:xcitec/page/templates/MainFrameV1.html':"<div>\r\n  <!-- suitenav -->\r\n  <div data-dojo-attach-point=\"suitenav\"></div>\r\n  <!-- [Header Area] -->\r\n  <div class=\"xcitec_header\">\r\n    <div class=\"wrap\">\r\n      <div class=\"xcitec_logo\"></div>\r\n      <div class=\"xcitec_userInfoBox\">\r\n        <!-- userinfobox -->\r\n        <div data-dojo-attach-point=\"userinfobox\"></div>\r\n      </div>\r\n      <div class=\"xcitec_tabBox\">\r\n        <div class=\"xcitec_mainTab\">\r\n          <!-- menu -->\r\n          <div data-dojo-attach-point=\"menu\"></div>\r\n        </div>\r\n        <div class=\"xcitec_dynTabs\">\r\n          <!-- tabbar -->\r\n          <div data-dojo-attach-point=\"tabbar\"></div>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <!-- [Content Area] -->\r\n  <div class=\"xcitec_content\">\r\n    <!-- sidebar -->\r\n    <div class=\"xcitec_left inner\">\r\n      <div data-dojo-attach-point=\"sidebar\"></div>\r\n    </div>\r\n    <!-- content -->\r\n    <div data-dojo-attach-point=\"content\"></div>\r\n  </div>\r\n  <div class=\"footer_empty\"> </div>\r\n</div>",
'url:xcitec/page/templates/BoxPanelV2.html':"<div>\r\n  <div class=\"title_wrap\">\r\n    <div class=\"right\">\r\n      <div data-dojo-attach-point=\"nextprevious\"></div>\r\n    </div>\r\n    <div class=\"left\">\r\n      <div data-dojo-attach-point=\"xcitectitle\"></div>\r\n    </div>\r\n  </div>\r\n  <div class=\"toolbar_wrap\" data-dojo-attach-point=\"toolbarwrap\">\r\n    <div class=\"left\">\r\n      <div data-dojo-attach-point=\"toolbar\"></div>\r\n    </div>\r\n    <div class=\"right\">\r\n      <div data-dojo-attach-point=\"opencollapse\"></div>\r\n    </div>\r\n  </div>\r\n  <div data-dojo-attach-point=\"containerNode\"></div>\r\n</div>",
'url:xcitec/layout/templates/DeletableBlock.html':"<div class=\"ContentAreaEditBlock\">\r\n  <div class=\"ContentAreaEditWrapper\" data-dojo-attach-point=\"containerNode\">\r\n    <div class=\"ContentAreaEditCorners ContentAreaEditCornersTL\"></div>\r\n    <div class=\"ContentAreaEditCorners ContentAreaEditCornersTR\"></div>\r\n    <div class=\"ContentAreaEditCorners ContentAreaEditCornersBL\"></div>\r\n    <div class=\"ContentAreaEditCorners ContentAreaEditCornersBR\"></div>\r\n    <div class=\"${xbtnClass}\" data-dojo-attach-event=\"onclick: _onClick\"></div>\r\n  </div>\r\n</div>",
'url:xcitec/form/templates/RadioButton.html':"<div class=\"dijitReset dijitInline\" wairole=\"presentation\" data-dojo-attach-event=\"onclick:_onClick\">\r\n  <div class=\"dijitXcitecRadioWrap\">\r\n    <div class=\"dijitReset dijitXcitecRadioIcon\">\r\n      <input type=\"${type}\" autocomplete=\"off\" class=\"dijitReset dijitCheckBoxInput\" data-dojo-attach-point=\"focusNode\" ${!nameAttrSetting} ${checkedAttrSetting}>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecRadioText ${icon}\">\r\n      <label for=\"${id}\" data-dojo-type=\"${labelTypeAttrSetting}\" data-dojo-props='\"labelId\":\"${captionId}\"'>${caption}</label>\r\n    </div>\r\n  </div>\r\n  <div class=\"required\"></div>\r\n</div>",
'url:xcitec/page/templates/SubTabPanelV1.html':"<div>\r\n  <div class=\"xcitec_toolbar\">\r\n    <div class=\"xcitec_toolbar_right\"></div>\r\n    <div class=\"xcitec_toolbar_wrap\">\r\n      <div data-dojo-attach-point=\"xcitectitle\"></div>\r\n      <div class=\"xcitec_actions\">\r\n        <div data-dojo-attach-point=\"toolbar\"></div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <div class=\"xcitec_dataContent\">\r\n    <div class=\"xcitec_left\">\r\n      <div data-dojo-attach-point=\"tabcontainer\"></div>\r\n    </div>\r\n    <div class=\"xcitec_dataTable\">\r\n      <div data-dojo-attach-point=\"messagepane\"></div>\r\n      <div data-dojo-attach-point=\"data\"></div>\r\n    </div>\r\n  </div>\r\n</div>",
'url:slmui/layout/templates/ColapsiblePane.html':"<div>\r\n\t<div data-dojo-attach-event=\"onkeydown:_onTitleKey\"\r\n\t\t class=\"dijitTitlePaneTitle\" data-dojo-attach-point=\"titleBarNode\" id=\"${id}_titleBarNode\">\r\n\t\t<div class=\"dijitTitlePaneTitleFocus\" data-dojo-attach-point=\"focusNode\">\r\n\t\t\t<span data-dojo-attach-event=\"ondijitclick:_onTitleClick\" data-dojo-attach-point=\"arrowNode\" class=\"dijitInline dijitArrowNode\" role=\"presentation\"></span>\r\n\t\t\t<span data-dojo-attach-point=\"arrowNodeInner\" class=\"dijitArrowNodeInner\"></span>\r\n\t\t\t<span data-dojo-attach-point=\"titleNode\" class=\"dijitTitlePaneTextNode\"></span>\r\n\t\t</div>\r\n        <div class=\"slmCollapsiblePaneHeaderRight\" data-dojo-attach-point=\"headerelem\"></div>\r\n\t</div>\r\n\t<div class=\"dijitTitlePaneContentOuter\" data-dojo-attach-point=\"hideNode\" role=\"presentation\">\r\n\t\t<div class=\"dijitReset\" data-dojo-attach-point=\"wipeNode\" role=\"presentation\">\r\n\t\t\t<div class=\"dijitTitlePaneContentInner\" data-dojo-attach-point=\"containerNode\" role=\"region\" id=\"${id}_pane\" aria-labelledby=\"${id}_titleBarNode\">\r\n\t\t\t\t<!-- nested divs because wipeIn()/wipeOut() doesn't work right on node w/padding etc.  Put padding on inner div. -->\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n",
'url:xcitec/form/templates/InlineDropDownButton.html':"<div class=\"dijit dijitReset dijitInline\" data-dojo-attach-point=\"_buttonNode,titleNode,focusNode\">\r\n    <span class=\"button_inner\">\r\n      <img src=\"${_blankGif}\" class=\"icon_settings\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"iconNode\">\r\n      <strong data-dojo-attach-point=\"containerNode,_popupStateNode\"></strong>\r\n      <img src=\"${_blankGif}\" class=\"arrow\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"arrowNode\">\r\n    </span>\r\n</div>",
'url:xcitec/templates/MenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" wairole=\"menuitem\" tabindex=\"-1\" data-dojo-attach-point=\"focusNode\" >\r\n  <td class=\"dijitReset\" wairole=\"presentation\">\r\n    <img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon\" data-dojo-attach-point=\"iconNode\">\r\n  </td>\r\n  <td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\">\r\n    <div class=\"dijitMenuItemWrapperLabel\" data-dojo-attach-point=\"containerNode\"></div>\r\n  </td>\r\n  <td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\r\n  <td class=\"dijitReset dijitMenuArrowCell\" wairole=\"presentation\">\r\n    <div style=\"visibility: hidden\" data-dojo-attach-point=\"arrowWrapper\">\r\n      <img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\">\r\n      <span class=\"dijitMenuExpandA11y\">+</span>\r\n    </div>\r\n  </td>\r\n</tr>\r\n",
'url:idx/form/templates/TriStateCheckBox.html':"<div id=\"widget_${id}\" class=\"dijitReset dijitInline idxComposite\"\r\n\t><div class=\"dijit dijitReset dijitInline\" role=\"presentation\" dojoAttachPoint=\"stateNode,oneuiBaseNode\"\r\n\t\t><div class=\"idxOneuiTriStateCheckBoxInner\" dojoAttachPoint=\"stateLabelNode\"></div\r\n\t\t><input ${!nameAttrSetting} type=\"checkbox\" dojoAttachPoint=\"focusNode\" role=\"checkbox\"\r\n\t\tclass=\"dijitReset idxOneuiTriStateCheckBoxInput\" dojoAttachEvent=\"onclick:_onClick\"\r\n\t/></div><div class=\"idxLabel dijitInline dijitHidden\" dojoAttachPoint=\"labelWrap\"\r\n\t><label dojoAttachPoint=\"compLabelNode\"></label\r\n\t></div\r\n\t><div class='dijitReset dijitValidationContainer dijitInline' dojoAttachPoint=\"iconNode\"\r\n\t\t><div class=\"dijitValidationIcon\"><input class=\"dijitReset dijitInputField dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"/\r\n\t></div></div\r\n></div>",
'url:xcitec/templates/AboutDialog.html':"<div class=\"dijitDialog xcitecAboutDialog\" tabindex=\"-1\" wairole=\"dialog\" waistate=\"labelledby-${id}_title\" data-dojo-attach-point=\"dialogNode\"\r\n     data-dojo-attach-event=\"onclick: _onClick\">\r\n  <div class=\"aboutDialogContainer\" data-dojo-attach-point=\"containerNode\">\r\n    <span class=\"dijitDialogCloseIcon\" data-dojo-attach-point=\"closeButtonNode\"></span>\r\n    <span class=\"versionLabel\">${version}</span>\r\n    <span class=\"licenseInfo\">${licenseInfo}</span>\r\n    <span class=\"fullVersionLabel\">${fullVersion}</span>\r\n  </div>\r\n</div>",
'url:slmui/form/templates/TriStateCheckBox.html':"<div id=\"widget_${id}\" class=\"dijitReset dijitInline idxComposite\"\r\n\t><div class=\"required\"></div><div class=\"dijit dijitReset dijitInline\" role=\"presentation\" dojoAttachPoint=\"stateNode,oneuiBaseNode\"\r\n\t\t><div class=\"idxOneuiTriStateCheckBoxInner\" dojoAttachPoint=\"stateLabelNode\"></div\r\n\t\t><input ${!nameAttrSetting} type=\"checkbox\" dojoAttachPoint=\"focusNode\" role=\"checkbox\"\r\n\t\tclass=\"dijitReset idxOneuiTriStateCheckBoxInput\" dojoAttachEvent=\"onclick:_onClick\"\r\n\t/></div><div class=\"idxLabel dijitInline dijitHidden\" dojoAttachPoint=\"labelWrap\"\r\n\t><label for=\"widget_${id}\" dojoAttachPoint=\"compLabelNode\" dojoAttachEvent=\"onclick:_onClickCaption\"></label\r\n\t></div\r\n\t></div>",
'url:xcitec/form/templates/TextBoxButton.html':"<div class=\"dijit dijitReset dijitInlineTable\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow:hidden;\">\r\n    <div class=\"dijitReset dijitXcitecRight\">\r\n      <div data-dojo-attach-point=\"button\" data-dojo-attach-event=\"onDoAction:_onClick\" data-dojo-type=\"${!xbtnClass}\"\r\n           data-dojo-props='\"iconType\":\"${actionIconType}\",\"iconText\":\"${actionIconText}\"' ${disabledAttrSetting}>\r\n      </div>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecMiddle\">\r\n      <div class=\"dijitReset\" data-dojo-attach-point=\"stateNode\">\r\n        <div class=\"dijitReset dijitXcitecInputField\">\r\n          <input class=\"dijitReset\" type=\"${type}\" autocomplete=\"off\" data-dojo-attach-point=\"textbox,focusNode\" ${!nameAttrSetting}>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>\r\n",
'url:xcitec/templates/Subsegment.html':"<div class=\"dojoDndItem ${baseClass}\">\r\n  <div class=\"subsegmentWrapper\" data-dojo-attach-point=\"containerNode\">\r\n    <div class=\"clearfix subsegmentHeadWrapper\" data-dojo-attach-point=\"subsegmentHead\">\r\n      <ul class=\"subsegmentEditChapters\" data-dojo-attach-point=\"editButtons\">\r\n        <li class=\"editChaptersIcon editChaptersIconSettings\" data-dojo-attach-event=\"onclick: _onSettingsClick\">\r\n        <li class=\"editChaptersIcon editChaptersIconDelete\" data-dojo-attach-event=\"onclick: _onDeleteClick\">\r\n      </ul>\r\n      <div class=\"subsegmentHead\" data-dojo-attach-point=\"left\">\r\n        <span class=\"subsegmentTitle\" data-dojo-attach-point=\"titleNode\">\r\n          <img src=\"${_blankGif}\" class=\"subsegmentTitleIcon\" alt=\"\" data-dojo-attach-event=\"onclick: _onClick\">\r\n          <img src=\"${_blankGif}\" class=\"iconMarking\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"iconMarkingKo\">\r\n          <img src=\"${_blankGif}\" class=\"iconMarking\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"iconMarkingCritical\">\r\n          <img src=\"${_blankGif}\" class=\"iconMarking\" title=\"\" width=\"1\" height=\"1\" alt=\"\" data-dojo-attach-point=\"iconMarkingScore\">\r\n          </span>\r\n        <span class=\"btn\" data-dojo-attach-point=\"evaluate\"></span>\r\n      </div>\r\n    </div>\r\n    <div class=\"subsegmentBody\" data-dojo-attach-point=\"subsegmentBody\">\r\n      <div class=\"subsegmentBodyWrapper clearfix\" data-dojo-attach-point=\"containerNode\">\r\n        <div class=\"bottomButtons\" data-dojo-attach-point=\"bottomButtons\"></div>\r\n      </div>\r\n    </div>\r\n    <div class=\"subsegmentCorner subsegmentCornerLT\"></div>\r\n    <div class=\"subsegmentCorner subsegmentCornerRT\"></div>\r\n    <div class=\"subsegmentCorner subsegmentCornerLB\"></div>\r\n    <div class=\"subsegmentCorner subsegmentCornerRB\"></div>\r\n  </div>\r\n</div>",
'url:xcitec/page/templates/LoginFrame.html':"<div>\r\n  <!-- header -->\r\n  <div class=\"header\">\r\n    <div class=\"header_l\">\r\n      <div class=\"header_r\">\r\n\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <!-- wrapper -->\r\n  <div class=\"loginContent\" data-dojo-attach-point=\"loginContent\">\r\n    <!-- feedback area -->\r\n    <div data-dojo-attach-point=\"messagepane\"></div>\r\n    <!-- sidebar area -->\r\n    <div class=\"loginSidebar\">\r\n      <div data-dojo-attach-point=\"sidebar\"></div>\r\n    </div>\r\n    <!-- content area -->\r\n    <div class=\"marketingArea\"></div>\r\n    <!-- help area -->\r\n    <!-- content area -->\r\n  </div>\r\n</div>",
'url:xcitec/formx/templates/EVAComboField.html':"<div class=\"evacriterion evacriterionComboField clearFix\" data-dojo-attach-point=\"containerNode\">\r\n  <!-- evacriterion wrapper -->\r\n  <div class=\"evacriterionInner\">\r\n    <!-- header -->\r\n    <div class=\"${baseClass} clearFix\" data-dojo-attach-point=\"stateNode\">\r\n      <!-- header right col -->\r\n      <div class=\"evacriterionHeaderRightcolWrapper\" data-dojo-attach-point=\"headerRightcolWrapperNode\">\r\n        <div class=\"evacriterionHeaderRightcol clearFix\" data-dojo-attach-point=\"headerRightcolNode\">\r\n          <div class=\"evacriterionHeaderRightcolRightitem clearFix\">\r\n            <!-- Rightcol_Rightitem -->\r\n            <div data-dojo-attach-point=\"evaluator\"></div>\r\n          </div>\r\n          <div class=\"evacriterionHeaderRightcolLeftitem\">\r\n            <!-- Rightcol_Lefttitem -->\r\n            <div data-dojo-attach-point=\"criteriaResult\"></div>\r\n          </div>\r\n          <div class=\"evacriterionHeaderRightcolCenteritem\">\r\n            <!-- Rightcol_Centeritem -->\r\n            <div data-dojo-attach-point=\"hierarchyNode\">\r\n              <img src=\"${_blankGif}\" class=\"evacriterionTitleIcon\" alt=\"\" data-dojo-attach-point=\"icon\" data-dojo-attach-event=\"onclick:_onClick\">\r\n            </div>\r\n          </div>\r\n        </div>\r\n      </div>\r\n    </div>\r\n    <!-- body -->\r\n    <div class=\"evacriterionBodyWrapper\" data-dojo-attach-point=\"comboBody\">\r\n      <div class=\"evacriterionBody\">\r\n        <div class=\"clearFix\" data-dojo-attach-point=\"containerNode\">\r\n\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>",
'url:xcitec/form/templates/UploadDialog.html':"<table class=\"dijitDialog\" tabindex=\"-1\" wairole=\"dialog\" waistate=\"labelledby-${id}_title\">\r\n  <tr>\r\n    <td class=\"cell titleBarWrap\">\r\n      <div class=\"dialogBg titleBarLeft\"></div>\r\n      <div class=\"dialogBg titleBarRight\"></div>\r\n      <div class=\"dialogBg dijitDialogTitleBar\" data-dojo-attach-point=\"titleBar\">\r\n        <span class=\"dijitDialogTitle\" id=\"${id}_title\" data-dojo-attach-point=\"titleNode\"></span>\r\n            \t<span class=\"dijitDialogCloseIcon\" title=\"${buttonCancel}\" data-dojo-attach-point=\"closeButtonNode\" data-dojo-attach-event=\"onclick: onCancel\">\r\n            \t\t<span class=\"closeText\" title=\"${buttonCancel}\" data-dojo-attach-point=\"closeText\"></span>\r\n            \t</span>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"cell\">\r\n      <div class=\"dijitDialogPaneContent\" data-dojo-attach-point=\"containerNode\">\r\n        <div class=\"minWidthRestriction\"></div>\r\n        <iframe id=\"uploadFrameID\" name=\"uploadFrame\" height=\"0\" width=\"0\" frameborder=\"0\" scrolling=\"yes\"></iframe>\r\n        <form id=\"myForm\" enctype=\"multipart/form-data\" method=\"post\" target=\"uploadFrame\" action=\"../servlet/FileUploadServlet\"\r\n              data-dojo-attach-event=\"onsubmit:shiftFunction\">\r\n          <input type=\"file\" name=\"uploadedFile\" id=\"uploadedFile\" data-dojo-attach-point=\"file\">\r\n          <input type=\"submit\" id=\"submitID\" name=\"submit\" value=\"Upload\">\r\n        </form>\r\n        <!-- Add hidden DIVs for updating and the progress bar (just a table) below the form -->\r\n        <br>\r\n\r\n        <div style=\"width:400px\" data-dojo-attach-point=\"bar\" data-dojo-type=\"dijit/ProgressBar\" data-dojo-props='\"maximum\":100,\"progress\":0'>\r\n        </div>\r\n        <br>\r\n\r\n        <div id=\"progressBarTable\" xstyle=\"position: absolute; top: 100px;\">\r\n          <table width=\"100%\" style=\"background-color: white; color: black;\">\r\n            <tr>\r\n              <td align=\"center\" nowrap>\r\n                <span id=\"bytesRead\" style=\"font-weight: bold;\" data-dojo-attach-point=\"bytesRead\"> </span>\r\n              </td>\r\n            </tr>\r\n          </table>\r\n        </div>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"cell\">\r\n      <div class=\"dialogBg buttonBarLeft\"></div>\r\n      <div class=\"dialogBg buttonBarRight\"></div>\r\n      <div class=\"dialogBg xcitecDailogButtonsBar\" data-dojo-attach-point=\"buttonsBarNode\">\r\n        <div class=\"xcitecDailogButtonsBarLine\"></div>\r\n      </div>\r\n    </td>\r\n  </tr>\r\n</table>",
'url:xcitec/page/templates/ClassicVersionFrame.html':"<iframe class=\"boxExternalClassicVersion\" frameborder=\"no\" allowtransparency=\"true\" title=\"\" data-dojo-attach-point=\"ClassicVersionFrame\"></iframe>",
'url:dijit/form/templates/DropDownButton.html':"<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdata-dojo-attach-event=\"ondijitclick:__onClick\" data-dojo-attach-point=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"focusNode,titleNode,_arrowWrapperNode,_popupStateNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdata-dojo-attach-point=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-event=\"onclick:_onClick\"\n\t\tdata-dojo-attach-point=\"valueNode\" role=\"presentation\" aria-hidden=\"true\"\n/></span>\n",
'url:slmui/layout/templates/Chapter.html':"<div>\r\n\t<div class=\"fieldsetChapterFrame\">\r\n\t\t<div class=\"fieldsetChapterFrameH\">\r\n\t\t\t<div class=\"fieldsetChapterFrameV\">\r\n\t\t\t\t<table class=\"dijitReset fieldsetChapterTable\" cellSpacing=\"0\" cellPadding=\"0\">\r\n\t\t\t\t\t<thead class=\"dijitReset\">\r\n\t\t\t\t\t<tr class=\"dijitReset\" style=\"height:0px;\">\r\n\t\t\t\t\t\t<td class=\"dijitReset fieldsetChapterLabelColumn\" data-dojo-attach-point=\"columnLabel\"></td>\r\n\t\t\t\t\t\t<td class=\"dijitReset fieldsetChapterControlColumn\" data-dojo-attach-point=\"columnWidget\"></td>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t\t<tr class=\"dijitReset\">\r\n\t\t\t\t\t\t<td class=\"dijitReset fieldsetChapterCaption\" colspan=\"2\"\r\n\t\t\t\t\t\t\tdata-dojo-attach-point=\"captionContainerNode\">\r\n\t\t\t\t\t\t\t<label class=\"dijitReset fieldsetChapterCaption\"\r\n\t\t\t\t\t\t\t\t   data-dojo-attach-point=\"captionNode\"></label>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t\t</thead>\r\n\t\t\t\t\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\r\n\t\t\t\t</table>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t<div class=\"fieldsetChapterCorner fieldsetChapterCornerLT\"></div>\r\n\t\t<div class=\"fieldsetChapterCorner fieldsetChapterCornerRT\"></div>\r\n\t\t<div class=\"fieldsetChapterCorner fieldsetChapterCornerLB\"></div>\r\n\t\t<div class=\"fieldsetChapterCorner fieldsetChapterCornerRB\"></div>\r\n\t</div>\r\n\t<div class=\"requiredRound\"></div>\r\n</div>",
'url:xcitec/form/templates/ListBox.html':"<div class=\"xcitecListBox\" style=\"width: 200px; height: 93px\">\r\n  <input class=\"dijitReset\" type=\"hidden\" autocomplete=\"off\" data-dojo-attach-point=\"textbox,focusNode\" ${!nameAttrSetting}>\r\n  <table class=\"xcitecListBoxItems\">\r\n    <tbody data-dojo-attach-point=\"itemsNode, containerNode\">\r\n    </tbody>\r\n  </table>\r\n  <div class=\"required\"></div>\r\n</div>",
'url:dijit/templates/ProgressBar.html':"<div class=\"dijitProgressBar dijitProgressBarEmpty\" role=\"progressbar\"\n\t><div  data-dojo-attach-point=\"internalProgress\" class=\"dijitProgressBarFull\"\n\t\t><div class=\"dijitProgressBarTile\" role=\"presentation\"></div\n\t\t><span style=\"visibility:hidden\">&#160;</span\n\t></div\n\t><div data-dojo-attach-point=\"labelNode\" class=\"dijitProgressBarLabel\" id=\"${id}_label\"></div\n\t><span data-dojo-attach-point=\"indeterminateHighContrastImage\"\n\t\t   class=\"dijitInline dijitProgressBarIndeterminateHighContrastImage\"></span\n></div>\n",
'url:xcitec/form/templates/ListBoxV2Node.html':"<div class=\"dijitTreeNode\" wairole=\"presentation\">\r\n  <div class=\"dijitTreeRow\" wairole=\"presentation\" data-dojo-attach-point=\"rowNode\"\r\n       data-dojo-attach-event=\"onmouseenter:_onMouseEnter, onmouseleave:_onMouseLeave\">\r\n    <span class=\"dijitTreeRowInner\">\r\n      <img src=\"${_blankGif}\" alt=\"\" class=\"dijitTreeExpando\" wairole=\"presentation\" data-dojo-attach-point=\"expandoNode\">\r\n      <span class=\"dijitExpandoText\" wairole=\"presentation\" data-dojo-attach-point=\"expandoNodeText\"></span>\r\n      <span class=\"dijitTreeContent\" wairole=\"presentation\" data-dojo-attach-point=\"contentNode\">\r\n            <!--<img src=\"${_blankGif}\" alt=\"\" data-dojo-attach-point=\"iconNode\" class=\"dijitTreeIcon\" waiRole=\"presentation\">-->\r\n        <span class=\"dijitTreeLabel\" wairole=\"treeitem\" tabindex=\"-1\" waistate=\"selected-false\"\r\n              xdojoattachevent=\"onfocus:_onLabelFocus, onblur:_onLabelBlur\" data-dojo-attach-point=\"labelNode\"></span>\r\n      </span>\r\n    </span>\r\n  </div>\r\n  <div class=\"dijitTreeContainer\" wairole=\"presentation\" style=\"display: none;float:none;\" data-dojo-attach-point=\"containerNode\"></div>\r\n</div>",
'url:xcitec/page/templates/MainMenuV1.html':"<table class=\"dijit dijitReset dijitLeft MainMenu\" cellspacing=\"0\" cellpadding=\"0\" wairole=\"presentation\">\r\n  <tbody wairole=\"presentation\">\r\n  <tr wairole=\"presentation\">\r\n    <td class=\"dijitReset dijitStretch MainMenuContent\"\r\n        vk_data-dojo-attach-event=\"ondijitclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n        data-dojo-attach-point=\"titleNode,containerNode\">\r\n      <div data-dojo-type=\"xcitec/Label\" data-dojo-props='\"labelId\":\"${labelId}\"'>${label}</div>\r\n    </td>\r\n    <td class=\"dijitReset dijitRight MainMenuDownArrow\" statemodifier=\"DownArrow\" data-dojo-attach-point=\"popupStateNode,focusNode\"\r\n        data-dojo-attach-event=\"ondijitclick:_onDownArrowClick\"></td>\r\n  </tr>\r\n  </tbody>\r\n</table>",
'url:dijit/form/templates/TextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'url:xcitec/form/templates/ListBoxV2.html':"<div class=\"xcitecListBoxV2\" wairole=\"tree\" style=\"width: 200px; height: 93px\" data-dojo-attach-event=\"onkeypress:_onKeyPress,ondblclick:_onDblClick\">\r\n  <div class=\"dijitTreeBg\" data-dojo-attach-point=\"attach\"></div>\r\n  <input class=\"dijitReset\" type=\"hidden\" autocomplete=\"off\" data-dojo-attach-point=\"textbox\">\r\n  <div class=\"required\"></div>\r\n</div>",
'url:xcitec/widgets/templates/GoToWidget.html':"<div class=\"${baseClass}\">\r\n  <div class=\"c_right\" data-dojo-attach-point=\"rightCol\"></div>\r\n  <div class=\"c_left\" data-dojo-attach-point=\"leftCol\"></div>\r\n</div>",
'url:slmui/form/templates/RadioButtonSet.html':"<div class=\"dijit dijitReset dijitInline\" data-dojo-attach-point=\"focusNode,overflowContainer\" tabIndex=\"-1\" style=\"overflow-x: auto;\">\r\n        <div style=\"white-space: nowrap; display: inline-block; z-index: 999;\" data-dojo-attach-point=\"radioNode\"></div>\r\n        <div data-dojo-attach-point=\"tickMarks\" style=\"height: 15px; position: relative; white-space: nowrap;\"></div>\r\n</div>",
'url:xcitec/form/templates/DropDownButton.html':"<div class=\"dijit dijitReset dijitInline\" data-dojo-attach-point=\"_buttonNode,titleNode,focusNode\">\r\n  <div class=\"dijitBorder\" style=\"overflow: hidden;\">\r\n    <div class=\"dijitLeft\"></div>\r\n    <div class=\"dijitReset dijitStretch\">\r\n      <span class=\"dijitReset dijitInline\" data-dojo-attach-point=\"iconNode\"></span>\r\n      <span xclass=\"dijitReset dijitInline dijitButtonText\" data-dojo-attach-point=\"containerNode,_popupStateNode\"></span>\r\n    </div>\r\n    <div class=\"dijitRight\"><span class=\"dijitReset dijitInline dijitArrowButtonIcon\"></span></div>\r\n  </div>\r\n</div>",
'url:xcitec/templates/Attachment.html':"<div class=\"attachmentField\">\r\n  <div class=\"l\">\r\n    <span class=\"attach\" data-dojo-attach-event=\"onclick: _onClick\">\r\n      <span class=\"icon\"><img height=\"1\" width=\"1\" alt=\"\" src=\"${_blankGif}\" class=\"${iconClass}\" data-dojo-attach-point=\"documentIcon\"></span>\r\n      <span class=\"link\" data-dojo-attach-point=\"downloadLink\">download</span>\r\n    </span>\r\n  </div>\r\n  <div class=\"r\">\r\n    <p data-dojo-attach-point=\"docTitle\"><strong>${docTitle}</strong></p>\r\n    <p data-dojo-attach-point=\"containerNode\"></p>\r\n    <p data-dojo-attach-point=\"fileProperties\"><strong>${fileName}</strong> ${uploadedOn} <strong>${insertionDate}</strong> ${uploadedAt} <strong>${insertionHour}</strong>\r\n      ${uploadedBy} <strong>${person}</strong></p>\r\n    <div data-dojo-attach-point=\"bottomButtons\"></div>\r\n  </div>\r\n</div>",
'url:xcitec/form/templates/TextBox.html':"<div class=\"dijit dijitReset dijitInlineTable dijitXcitec\" id=\"widget_${id}\" wairole=\"presentation\">\r\n  <div style=\"overflow: hidden;\" class=\"dijitXcitecInner\">\r\n    <div class=\"dijitReset dijitXcitecRight\" data-dojo-attach-point=\"rightOutNode\">\r\n      <label data-dojo-attach-point=\"trailingTextWidget\" data-dojo-type=\"xcitec/TrailingLabel\" data-dojo-props='\"labelId\":\"${trailingTextId}\"'>\r\n      </label>\r\n    </div>\r\n    <div class=\"dijitReset dijitXcitecMiddle\">\r\n      <div class=\"dijitReset dijitXcitecInputField\">\r\n        <input class=\"dijitReset\" type=\"${type}\" ${!nameAttrSetting} autocomplete=\"off\" data-dojo-attach-point=\"textbox,focusNode\">\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <img src=\"${_blankGif}\" alt=\"\" class=\"required\">\r\n</div>\r\n",
'*now':function(r){r(['dojo/i18n!*preload*xcitec/nls/xcitec-all*["ar","ca","cs","da","de","el","en-gb","en-us","es-es","fi-fi","fr-fr","he-il","hu","it-it","ja-jp","ko-kr","nl-nl","nb","pl","pt-br","pt-pt","ru","sk","sl","sv","th","tr","zh-tw","zh-cn","ROOT"]']);}
}});
define("xcitec/xcitec-all", [
  "./main",

	//external items
  "dijit/ProgressBar",

  // base functionality (utils, ...)
	"xcitec/_StateWidget",

  // General widgets controls
	"xcitec/Label",
  "xcitec/BoldLabel",
  "xcitec/DividingLabel",
  "xcitec/LabelMultiline",
  "xcitec/TrailingLabel",
	"xcitec/Headline",
  "xcitec/page/MainMenuV1",
  "xcitec/Menu",
	"xcitec/MenuSeparator",
  "xcitec/PopupMenuItem",
	"xcitec/MenuItem",
	"xcitec/TabBar",
  "xcitec/TabBarItem",
	"xcitec/TabContainer",
  "xcitec/TabItem",
	"xcitec/Toolbar",
  "xcitec/ToolbarButton",
  "xcitec/ToolbarDropDownButton",
  "xcitec/ToolbarSeparator",
	"xcitec/MessagePane",
	"xcitec/Dialog",
  "xcitec/xcitecDialog",
  "xcitec/DataFormDialog",
  "xcitec/MultiSelectDialog",
	"xcitec/AboutDialog",
	"xcitec/Panel",
	"xcitec/DialogPane",
	"xcitec/Line",
	"xcitec/Text",
  "xcitec/BoldText",
  "xcitec/Title",
	"xcitec/WikiText",
	"xcitec/StatusText",
	"xcitec/Attachment",
	"xcitec/AttachmentContainer",
  "xcitec/Image",
	"xcitec/BlankSeparator",
	"xcitec/SuiteNav",

  // Layout widgets controls
  "xcitec/layout/_LabelControlHolder",
	"xcitec/layout/_LayoutWidget",
  "xcitec/layout/ContentArea",
  "xcitec/layout/ContentAreaPane",
  "xcitec/layout/_ContentAreaColumn",
  "xcitec/layout/_ContentAreaPaneItem",
	"xcitec/layout/Chapter",
  "xcitec/layout/_ChapterItem",
  "xcitec/layout/Chapter2C",
  "xcitec/layout/_Chapter2CItem",
  "xcitec/layout/Chapter2Columns",
  "xcitec/layout/DoubleField",
	'xcitec/layout/FacetFilter',
  "xcitec/layout/FilterField",
  "xcitec/layout/Filters",
  "xcitec/layout/_FiltersItem",
  "xcitec/layout/FormPanel",
  "xcitec/layout/TaskPanel",
  "xcitec/layout/MultiLanguageContainer",
  "xcitec/layout/DeletableBlock",
  "xcitec/layout/ExtendedLayout",
  "xcitec/layout/MultiSelectLayout",

  // Forms widgets controls
  "xcitec/form/_XButton",
  "xcitec/form/_XButtonMiddle",
  "xcitec/form/_XButtonRight",
  "xcitec/form/_XButtonSingleRight",
  "xcitec/form/TextBox",
	"xcitec/form/TextBoxButton",
  "xcitec/form/EmailTextBox",
  "xcitec/form/UrlTextBox",
  "xcitec/form/CheckBox",
	"xcitec/form/CheckBoxGroup",
	"xcitec/form/DropDownSelect",
	"xcitec/form/_DropDownMenu",
	"xcitec/form/EntityDropDownSelect",
  "xcitec/form/EntitySelect",
	"xcitec/form/AutoCompleteEntitySelect",
	"xcitec/form/NumberTextBox",
	"xcitec/form/DateTextBox",
  "xcitec/form/_Calendar",
  "xcitec/form/Textarea",
  "xcitec/form/Button",
  "xcitec/form/LinkButton",
  "xcitec/form/DropDownButton",
  "xcitec/form/RadioButton",
  "xcitec/form/MultiSelect",
  "xcitec/form/RadioGroup",
	"xcitec/form/MultiLanguageTextarea",
  "xcitec/form/Slider",
	"xcitec/form/ListBox",
	"xcitec/form/_ListBoxItem",
  "xcitec/form/ListBoxV2",
  "xcitec/form/FlatButton",
  "xcitec/form/TaskButton",
  "xcitec/form/Upload",

  // Grid widget control
  "xcitec/grid/HGrid",
	"xcitec/grid/HGridBottomPane",
	"xcitec/grid/GridControl",
	"xcitec/grid/LinkToolbar",
	"xcitec/grid/Actions",
  "xcitec/grid/Action",
	"xcitec/grid/HGridFC",

  //XForms
  "xcitec/formx/TableLayout",

  //XLayouts
  "xcitec/xlayout/Question",
	"xcitec/xlayout/QuestionV2",
  "xcitec/xlayout/_QuestionV2Item",
	"xcitec/xlayout/QuestionV2SingleColumn",
	"xcitec/xlayout/QualificationAvatarSingleColumn",
	"xcitec/xlayout/_ContentAreaPaneItemV2SingleColumn",
	"xcitec/xlayout/_QuestionV2ItemSingleColumn",
  "xcitec/xlayout/ComplexQuestionV2",
  "xcitec/xlayout/StyleElement",
  "xcitec/xlayout/QualificationLayout",
  "xcitec/xlayout/QualificationAvatar",
	"xcitec/xlayout/QualificationLayoutBase",
	"xcitec/xlayout/QualificationLayoutSingleColumn",
  "xcitec/xlayout/ContentAreaPaneV2",
  "xcitec/xlayout/_ContentAreaPaneItemV2",
	"xcitec/xlayout/ContentAreaPaneV2SingleColumn",

  // new GUI widgets
  "xcitec/page/MainMenuV2",
  "xcitec/page/MenuV2",
	"xcitec/page/MenuItemV2",
	"xcitec/page/PopupMenuItemV2",
  "xcitec/Box",
  "xcitec/FilterBox",
  "xcitec/ReportBox",
  "xcitec/SearchResultBox",
	"xcitec/form/InlineDropDownButton",
  "xcitec/form/InlineLinkButton",
  "xcitec/page/MainFrameV1",
	"xcitec/page/MainFrameV2",
  "xcitec/page/BoxPanelV2",
	"xcitec/page/SearchPanelV1",
	"xcitec/page/SubTabPanelV1",
	"xcitec/page/PageSeparator",
	"xcitec/page/ObjectDivider",
	"xcitec/page/ReportMenu",
  "xcitec/page/ReportMenuItem",
  "xcitec/page/ReportMenuSubItem",
	"xcitec/page/LoginFrame",
	"xcitec/page/ClassicVersionFrame",
	"xcitec/page/Form",
  "xcitec/UserInfoBoxV2",
  "xcitec/page/UserInfoBox",
  "xcitec/form/Rating",
  "xcitec/formx/STICriterion",
  "xcitec/formx/EVACriterion",
  "xcitec/formx/EVACriterionSingleColumn",
  "xcitec/formx/EVAComboField",
  "xcitec/formx/EVAComboFieldContainer",
  "xcitec/formx/RISIndicator",
  "xcitec/Subsegment",
  "xcitec/SubsegmentContainer",
  "xcitec/Source",
  "xcitec/SubsegmentAvatar",
  "xcitec/form/AnswerBox",
	"xcitec/ComboContainer",

  /* xcitec widgets */
  "xcitec/widgets/WidgetBox",
  "xcitec/widgets/GoToWidget",
  "xcitec/widgets/WidgetsContainer",
  "xcitec/widgets/SyncSupplierWidget",
  "xcitec/widgets/ThumbnailWidget",
  "xcitec/widgets/HtmlWidget",

  /* xcitec wicket & ajax */
  "xcitec/wicket/_ajax",
	"xcitec/wicket/AjaxLazyPane",

  /* additional from XcitecV2.js */
  "xcitec/page/MainFrame",
	"xcitec/grid/HGridV2",
	"xcitec/page/ContentPanel",
	"xcitec/page/ListTabPanel",
	"xcitec/page/DataTabPanel",

	/* slmui widgets */
	"slmui/main",
	"slmui/layout/_ChapterAdvancedItem",
	"slmui/layout/_ChapterBase",
	"slmui/layout/_ChapterItem",
	"slmui/layout/Chapter",
	"slmui/layout/ChapterAdvanced",
	"slmui/layout/MulticolumnLayout",
	"slmui/layout/FilterLayout",
	"slmui/layout/CollapsiblePane",
	"slmui/form/RadioButtonSet",
	"slmui/form/Slider",
	"slmui/form/MultiSelect",
	"slmui/form/TriStateCheckBox"

], function(xcitec){

	// module:
	//		xcitec/xcitec-all

	/*=====
	return {
		// summary:
		//		A rollup that includes every dijit.
	};
	=====*/
	return xcitec;
});
